<?php

namespace App;

use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\SiteSettingController;
use App\ModelsConnect\ExternalAccount;
use App\ModelsEpos\EposAddon;
use App\ModelsEpos\EposCategory;
use App\ModelsEpos\EposDeliveryCharge;
use App\ModelsEpos\EposFloor;
use App\ModelsEpos\EposIngredient;
use App\ModelsEpos\EposPaymentMethod;
use App\ModelsEpos\EposPrepLocation;
use App\ModelsEpos\EposProduct;
use App\ModelsEpos\EposProductAddon;
use App\ModelsEpos\EposProductIngredient;
use App\ModelsEpos\EposTable;
use App\ModelsEpos\EposUser;
use App\ModelsTiffintom\TiffintomCategory;
use App\ModelsTiffintom\TiffintomMainAddon;
use App\ModelsTiffintom\TiffintomMenu;
use App\ModelsTiffintom\TiffintomMenuAddons;
use App\ModelsTiffintom\TiffintomOffer;
use App\ModelsTiffintom\TiffintomOfferDay;
use App\ModelsTiffintom\TiffintomOfferDayPrice;
use App\ModelsTiffintom\TiffintomOfferDayProduct;
use App\ModelsTiffintom\TiffintomOfferMultiplePrice;
use App\ModelsTiffintom\TiffintomOfferPrice;
use App\ModelsTiffintom\TiffintomOfferProduct;
use App\ModelsTiffintom\TiffintomRestaurantVoucher;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class Business extends Model
{
    protected $fillable = [
        'name', 'trading_name', 'email', 'contact_numbers',
        'door_no',
        'address',
        'city',
        'state',
        'postcode',
        'company_number',
        'country_id', 'joining_date', 'domain', 'domain_valid_till', 'type', 'statement_description', 'statement_descriptor', 'online_statement_description', 'online_statement_descriptor', 'restaurant_id', 'restaurant_name', 'online_paypal_description',
        'payout_mode',
        'payout_method',
        'archived_payout_mode',
        'archived_payout_method',
        'supervisor_password',
        'admin_password',
        'payout_comment',
        'paypal_mode',
        'paypal_public_key_test',
        'paypal_private_key_test',
        'paypal_public_key_live',
        'paypal_private_key_live',
        'stripe_mode',
        'stripe_public_key_test',
        'stripe_private_key_test',
        'stripe_public_key_live',
        'stripe_private_key_live',
        'online_service',
        'merchant_service',
        'epos_service',
        'auto_archive',
        'db_name',
        'database_server_id',
        'connect_service',
        's_account_id',
        'connect_errors',
        'connect_status',
        'footer_a',
        'footer_b',
        'header_a',
        'header_b',
        'header_bill_a',
        'footer_bill_a',
        'epos_device_allow',
        'merchant_device_allow',
        'android_restaurant_app_version',
        'ios_restaurant_app_version',
        'android_partner_app_version',
        'ios_partner_app_version',
        'service_charge_type',
        'delivery_number_of_print',
        'collection_number_of_print',
        'online_order_number_of_print',
        'dinein_number_of_print',
        'day_of_transaction_limit',
        'statement_transaction_view',
        'online_order_limit',
        'auto_accept_order',
        'auto_accept_reservation',
        'auto_accept_fields',
        'transfer_connect',
        'extension_account_id',
        'access_number',
        'pouch_service',
        'pouch_weight',
        'load_wallet',
        'load_bank',
        'qr_permission',
        'auto_merchant_print',
        'auto_customer_print',
        'is_qr_print_on_bill',
    ];
    protected $hidden = ['updated_at', 'connect_errors', 'connect_status'];
    protected $appends = ['user_counts', 'business_commissions', 'country', 'balance', 'finance_user_id',  'finance_user_phone_number', 'image_url', 'thumb_url'];
    protected $casts = [
        'disabled' => 'bool',
        'online_service' => 'bool',
        'merchant_service' => 'bool',
        'epos_service' => 'bool',
        'connect_service' => 'bool',
        'issuing_service' => 'bool',
        'extension_service' => 'bool',
        'auto_archive' => 'bool',
        'tip' => 'bool',
        'print_logo' => 'bool',
        'gallery_video_visible' => 'bool',
        'send_receipt' => 'bool',
        'gallery_image_visible' => 'bool',
        'pouch_service' => 'bool',
        'pouch_weight' => 'double',
        'load_wallet' => 'bool',
        'load_bank' => 'bool',
        'qr_permission' => 'bool',
        'auto_merchant_print' => 'bool',
        'auto_customer_print' => 'bool',
        'is_qr_print_on_bill' => 'bool',
        'invoice_interval' => 'integer',
        'delivery_number_of_print' => 'integer',
        'collection_number_of_print' => 'integer',
        'online_order_number_of_print' => 'integer',
        'dinein_number_of_print' => 'integer',
        'day_of_transaction_limit' => 'integer',
        'auto_accept_fields' => 'json'
    ];

    // Relationships
    public function caxtonCards()
    {
        return $this->belongsTo(CaxtonCard::class);
    }

    // Attributes
    //
    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_BUSINESS, $this->image);
    }
    public function getThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_BUSINESS, $this->image);
    }
    public function getUserCountsAttribute()
    {
        return $this->usersCounts();
    }
    public function getBusinessCommissionsAttribute()
    {
        return $this->business_comissions();
    }
    public function getCountryAttribute()
    {
        return $this->country();
    }
    public function getDefaultBankDetailAttribute()
    {
        return $this->bank_detail();
    }
    public function getAllBankDetailAttribute()
    {
        return $this->all_bank_detail();
    }
    public function getDefaultBankAttribute()
    {
        return $this->bank();
    }
    public function getDefaultPaypalDetailAttribute()
    {
        return $this->paypal_detail();
    }
    public function getBalanceAttribute()
    {
        return Passbook::where('business_id', $this->id)->sum('amount');
    }
    public function getSupervisorPermissionsAttribute()
    {
        return $this->supervisor_permissions();
    }
    public function getAdminPermissionsAttribute()
    {
        return $this->admin_permissions();
    }
    public function getDatabaseServerAttribute()
    {
        return $this->database_server()->first();
    }
    public function getFinanceUserIdAttribute()
    {
        $user = CaxtonUser::where('business_id', $this->id)->first();
        if ($user) {
            return $user->id;
        } else {
            return null;
        }
    }
    public function getFinanceUserPhoneNumberAttribute()
    {
        $user = CaxtonUser::where('business_id', $this->id)->first();
        if ($user) {
            return $user->phone_number;
        } else {
            return null;
        }
    }
    public function getCardReadersAttribute()
    {
        return $this->card_readers();
    }
    public function getPrintersAttribute()
    {
        return $this->printers();
    }
    public function getPlansAttribute()
    {
        return $this->plans();
    }
    public function getUserAttribute()
    {
        return $this->user();
    }
    // Attributes
    //
    public function usersCounts()
    {
        return UserBusiness::where('business_id', $this->id)->count();
    }
    public function user()
    {
        $user_business = UserBusiness::where('business_id', $this->id)->first();
        return User::find($user_business->user_id);
    }
    public function business_comissions()
    {
        return BusinessCommission::where('business_id', $this->id)->first();
    }
    public function country()
    {
        return Country::where('id', $this->country_id)->first();
    }
    public function bank_detail()
    {
        return BankDetail::where('id', $this->bank_detail_id)->first();
    }
    public function all_bank_detail()
    {
        return ExternalAccount::where('business_id', $this->id)->orderBy('default', 'DESC')->get();
    }
    public function bank()
    {
        $externalAccount = ExternalAccount::where('business_id', $this->id)->where('default', 1)->orderByRaw('id DESC')->first();
        if (!empty($externalAccount)) {
            // $stringValue = 'UBSIDI LIMITED,Tiffin Tom,TIFFINTOMUK LTD,Tiffin Tom Limited,Ubsidi Ltd,TIFFINTOM LIMITED,TiffinTomuk Ltd';
            $stringValue = (new SiteSettingController())->findByKey('bank_string')->value;
            if ($externalAccount->account_holder_name == '' || stripos($stringValue, $externalAccount->account_holder_name) !== false) {
                $externalAccount->payable_amount = 1;
            } else {
                $externalAccount->payable_amount = 0;
            }
        }
        return $externalAccount;
    }
    public function paypal_detail()
    {
        return PaypalDetail::where('id', $this->paypal_detail_id)->first();
    }
    public function supervisor_permissions()
    {
        return BusinessPermission::where('business_id', $this->id)->where('role', 'supervisor')->first();
    }
    public function admin_permissions()
    {
        return BusinessPermission::where('business_id', $this->id)->where('role', 'admin')->first();
    }
    public function database_server()
    {
        return $this->belongsTo('App\DatabaseServer', 'database_server_id');
    }
    public function card_readers()
    {
        return BusinessCardReader::where('business_id', $this->id)->get();
    }
    public function printers()
    {
        return BusinessPrinter::selectRaw('business_printers.*,printers.name as printer_model_name')->leftJoin('printers', 'printers.id', '=', 'printer_id')->where('business_id', $this->id)->get();
    }
    public function plans()
    {
        $epos_plan = BusinessPlan::where('business_id', $this->id)->where('service', 'Epos')->first();
        $merchant_plan = BusinessPlan::where('business_id', $this->id)->where('service', 'Merchant')->first();
        $online_plan = BusinessPlan::where('business_id', $this->id)->where('service', 'Online')->first();
        $card_plan = BusinessPlan::where('business_id', $this->id)->where('service', 'Card')->first();
        $connect_plan = BusinessPlan::where('business_id', $this->id)->where('service', 'Connect')->first();

        $business = Business::find($this->id);
        if (empty($business)) Utils::error('Business not found!', 404);

        if (!empty($epos_plan)) {
            if (Carbon::parse($epos_plan->to_date)->timestamp < Carbon::today()->timestamp) {
                // $epos_plan = 'Expire';
                $epos_plan_expire = Plan::where('service', 'Epos')->where('name', 'Expire After Epos')->first();
                if (!empty($epos_plan_expire)) {
                    $permissions = $epos_plan_expire->permissions;
                    //Establish Connect On Epos Database
                    $this->establishEposConnection($business);

                    //Product Disable On Expire
                    $product = EposProduct::select('id')->skip($permissions['product']['allocate'])->take(PHP_INT_MAX)->get();
                    $product->makeHidden('image_url');
                    $product->makeHidden('thumb_url');
                    EposProduct::whereIn('id', $product)->update(['disabled' => 1]);

                    //Category Disable On Expire
                    $category = EposCategory::select('id')->skip($permissions['product']['allocate'])->take(PHP_INT_MAX)->get();
                    $category->makeHidden('title');
                    $category->makeHidden('key');
                    $category->makeHidden('children');
                    $category->makeHidden('image_url');
                    $category->makeHidden('thumb_url');
                    EposCategory::whereIn('id', $category)->update(['disabled' => 1]);

                    // Addon Disable On Expire
                    $addon = EposAddon::select('id')->skip($permissions['product']['allocate'])->take(PHP_INT_MAX)->get();
                    $addon->makeHidden('title');
                    $addon->makeHidden('key');
                    $addon->makeHidden('children');
                    $addon->makeHidden('image_url');
                    $addon->makeHidden('thumb_url');
                    EposAddon::whereIn('id', $addon)->update(['disabled' => 1]);
                    $product_addon = EposProductAddon::select('id')->skip($permissions['product']['allocate'])->take(PHP_INT_MAX)->get();
                    EposProductAddon::whereIn('id', $product_addon)->update(['disabled' => 1]);

                    // Addon Disable On Expire
                    $ingredient = EposIngredient::select('id')->skip($permissions['product']['allocate'])->take(PHP_INT_MAX)->get();
                    EposIngredient::whereIn('id', $ingredient)->update(['disabled' => 1]);
                    $product_ingredient = EposProductIngredient::select('id')->skip($permissions['product']['allocate'])->take(PHP_INT_MAX)->get();
                    EposProductIngredient::whereIn('id', $product_ingredient)->update(['disabled' => 1]);

                    // Floor Disable On Expire
                    // $floor = EposFloor::select('id')->skip($permissions['floor']['allocate'])->take(PHP_INT_MAX)->get();
                    // EposFloor::whereIn('id', $floor)->update(['disabled' => 1]);

                    // Table Disable On Expire
                    $table = EposTable::select('id')->skip($permissions['table']['allocate'])->take(PHP_INT_MAX)->get();
                    $table->makeHidden('table_status');
                    $table->makeHidden('last_order_id');
                    $table->makeHidden('last_order_total');
                    $table->makeHidden('last_order_created_at');
                    EposTable::whereIn('id', $table)->update(['disabled' => 1]);

                    // Payment Method Disable On Expire
                    $payment_method = EposPaymentMethod::select('id')->skip($permissions['payment_method']['allocate'])->take(PHP_INT_MAX)->get();
                    EposPaymentMethod::whereIn('id', $payment_method)->update(['disabled' => 1]);

                    // Delivery Charge Disable On Expire
                    $delivery_charge = EposDeliveryCharge::select('id')->skip($permissions['delivery_charge']['allocate'])->take(PHP_INT_MAX)->get();
                    EposDeliveryCharge::whereIn('id', $delivery_charge)->update(['disabled' => 1]);

                    // Device Disable On Expire
                    // $device = DeviceRegistration::select('id')->where('business_id', $this->id)->skip($permissions['device']['allocate'])->take(PHP_INT_MAX)->get();
                    // DeviceRegistration::whereIn('id', $device)->update(['disabled' => 1]);

                    // Preplocation On Expire
                    $preplocation = EposPrepLocation::select('id')->skip($permissions['preplocation']['allocate'])->take(PHP_INT_MAX)->get();
                    EposPrepLocation::whereIn('id', $preplocation)->update(['disabled' => 1]);

                    // User On Expire
                    $user = EposUser::select('id')->skip($permissions['user']['allocate'])->take(PHP_INT_MAX)->get();
                    $user->makeHidden('image_url');
                    $user->makeHidden('thumb_url');
                    EposUser::whereIn('id', $user)->update(['disabled' => 1]);

                    $epos_plan->plan_id = $epos_plan_expire->id;
                    $epos_plan->from_date = Carbon::today()->format('Y-m-d');
                    $epos_plan->to_date = Carbon::today()->addYears(10)->format('Y-m-d');
                    $epos_plan->save();
                }
            }
        } else {
            $epos_plan = '';
        }
        if (!empty($merchant_plan)) {
            if (Carbon::parse($merchant_plan->to_date)->timestamp < Carbon::today()->timestamp) {
                // $merchant_plan = 'Expire';
                $merchant_plan_expire = Plan::where('service', 'Merchant')->where('name', 'Expire After Merchant')->first();
                if (!empty($merchant_plan_expire)) {
                    $merchant_plan->plan_id = $merchant_plan_expire->id;
                    $merchant_plan->from_date = Carbon::today()->format('Y-m-d');
                    $merchant_plan->to_date = Carbon::today()->addYears(10)->format('Y-m-d');
                    $merchant_plan->save();
                }
            }
        } else {
            $merchant_plan = '';
        }
        if (!empty($online_plan)) {
            if (Carbon::parse($online_plan->to_date)->timestamp < Carbon::today()->timestamp) {
                // $online_plan = 'Expire';
                $online_plan_expire = Plan::where('service', 'Online')->where('name', 'Expire After Online')->first();
                if (!empty($online_plan_expire)) {

                    $onlinePermissions = $online_plan_expire->permissions;

                    //Categories Disable On Expire
                    $online_categories = TiffintomCategory::select('id')->where('restaurant_id', $this->restaurant_id)->skip($onlinePermissions['online_categories']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomCategory::whereIn('id', $online_categories)->update(['status' => '0']);

                    //Menu Disable On Expire
                    $online_menu = TiffintomMenu::select('id')->where('restaurant_id', $this->restaurant_id)->skip($onlinePermissions['online_menu']['allocate'])->take(PHP_INT_MAX)->get();
                    $online_menu->makeHidden('image_url');
                    $online_menu->makeHidden('thumb_url');
                    TiffintomMenu::whereIn('id', $online_menu)->update(['status' => '0']);

                    //Addon Disable On Expire
                    $online_addon = TiffintomMainAddon::select('id')->where('restaurant_id', $this->restaurant_id)->skip($onlinePermissions['online_addon']['allocate'])->take(PHP_INT_MAX)->get();
                    $online_addon->makeHidden('sub_addonses');
                    TiffintomMainAddon::whereIn('id', $online_addon)->update(['status' => '0']);

                    //Offer Disable On Expire
                    $offer = TiffintomOffer::select('id')->where('resid', $this->restaurant_id)->skip($onlinePermissions['offer']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomOffer::whereIn('id', $offer)->update(['status' => '0']);

                    //Product Offer Disable On Expire
                    $product_offer = TiffintomOfferProduct::select('id')->where('resid', $this->restaurant_id)->skip($onlinePermissions['product_offer']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomOfferProduct::whereIn('id', $product_offer)->update(['status' => '0']);

                    //Price Offer Disable On Expire
                    $price_offer = TiffintomOfferPrice::select('id')->where('resid', $this->restaurant_id)->skip($onlinePermissions['price_offer']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomOfferPrice::whereIn('id', $price_offer)->update(['status' => '0']);

                    //Day Offer Disable On Expire
                    $day_offer = TiffintomOfferDay::select('id')->where('resid', $this->restaurant_id)->skip($onlinePermissions['day_offer']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomOfferDay::whereIn('id', $day_offer)->update(['status' => '0']);

                    //Day Product Offer Disable On Expire
                    $day_product_offer = TiffintomOfferDayProduct::select('id')->where('resid', $this->restaurant_id)->skip($onlinePermissions['day_product_offer']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomOfferDayProduct::whereIn('id', $day_product_offer)->update(['status' => '0']);

                    //Day Price Offer Disable On Expire
                    $day_price_offer = TiffintomOfferDayPrice::select('id')->where('resid', $this->restaurant_id)->skip($onlinePermissions['day_price_offer']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomOfferDayPrice::whereIn('id', $day_price_offer)->update(['status' => '0']);

                    //Multiple Price Offer Disable On Expire
                    $multiple_price_offer = TiffintomOfferMultiplePrice::select('id')->where('resid', $this->restaurant_id)->skip($onlinePermissions['multiple_price_offer']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomOfferMultiplePrice::whereIn('id', $multiple_price_offer)->update(['status' => '0']);

                    //Voucher Disable On Expire
                    $online_voucher = TiffintomRestaurantVoucher::select('id')->where('restaurant_id', $this->restaurant_id)->skip($onlinePermissions['online_voucher']['allocate'])->take(PHP_INT_MAX)->get();
                    TiffintomRestaurantVoucher::whereIn('id', $online_voucher)->update(['status' => '0']);

                    $online_plan->plan_id = $online_plan_expire->id;
                    $online_plan->from_date = Carbon::today()->format('Y-m-d');
                    $online_plan->to_date = Carbon::today()->addYears(10)->format('Y-m-d');
                    $online_plan->save();
                }
            }
        } else {
            $online_plan = '';
        }

        $response = [
            'epos_plan' => $epos_plan,
            'merchant_plan' => $merchant_plan,
            'online_plan' => $online_plan,
            'card_plan' => $card_plan,
            'connect_plan' => $connect_plan
        ];

        return $response;
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);

        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }
}
