<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;
use Illuminate\Support\Facades\Log;

class CaxtonWallet extends Model
{
    protected $fillable = [
        'name',
        'business_id',
        'group_id',
        'archive',
        'bank',
    ];
    protected $hidden = ['updated_at'];
    protected $appends = ['business', 'wallet_group', 'use_limit', 'annual_balance', 'reccuring_balance'];
    protected $casts = [
        'archive' => 'bool',
        'bank' => 'bool',
    ];

    // Attributes
    //
    public function getBusinessAttribute()
    {
        $business = Business::select('id', 'name', 'connect_service')->find($this->business_id);
        if (!empty($business)) {
            $business->makeHidden('business_commissions');
            $business->makeHidden('country');
            $business->makeHidden('balance');
            $business->makeHidden('user_counts');
            $business->makeHidden('finance_user_id');
            $business->makeHidden('finance_user_phone_number');
            return $business;
        } else {
            return;
        }
    }
    public function getWalletGroupAttribute()
    {
        return CaxtonWalletGroup::select('id', 'name')->find($this->group_id);
    }
    public function getUseLimitAttribute()
    {
        return CaxtonWalletTransaction::where('wallet_id', $this->wallet_id)->sum('credit_amount');
    }
    public function getAnnualBalanceAttribute()
    {
        $from_date = Carbon::today()->subYear()->format('Y-m-d') .  " 00:00:00";
        $to_date = Carbon::today()->format('Y-m-d') . " 23:59:59";
        return CaxtonWalletTransaction::where('wallet_id', $this->wallet_id)->where('transaction_date', '>=', $from_date)->where('transaction_date', '<=', $to_date)->sum('credit_amount');
    }
    public function getReccuringBalanceAttribute()
    {
        $from_date = Carbon::parse($this->created_at)->format('Y-m-d') .  " 00:00:00";
        $to_date = Carbon::parse($this->created_at)->addYear()->format('Y-m-d') . " 23:59:59";
        return CaxtonWalletTransaction::where('wallet_id', $this->wallet_id)->where('transaction_date', '>=', $from_date)->where('transaction_date', '<=', $to_date)->sum('credit_amount');
    }
}
