<?php

namespace App\Console\Commands;

use App\Business;
use Illuminate\Console\Command;
use App\Jobs\ReportCreatorJob;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Queue;

class ReportCreator extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'ReportCreator:create {from_date?} {to_date?}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Cron for creating invoice';

    public static $process_busy = false;

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        // $carbon = Carbon::today();
        $carbon = Carbon::today()->subDays(7);

        $currentDayOfWeek = $carbon->dayOfWeek;
        if ($currentDayOfWeek <= 1) {
            $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
            $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
        } else {
            $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
            $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
        }
        if (!empty($this->argument('from_date'))) {
            $from_date = $this->argument('from_date');
        }
        if (!empty($this->argument('to_date'))) {
            $to_date = $this->argument('to_date');
        }
        // echo $currentDayOfWeek . " ";
        // echo $from_date . " ";
        // echo $to_date . " ";
        // return;
        $req = Request::create("");
        $req->merge(['from_date' => $from_date]);
        $req->merge(['to_date' => $to_date]);
        $req->merge(['save' => "true"]);
        $req->merge(['is_job' => "true"]);
        $businesses = Business::where('disabled', '0')->get();
        $businesses->each(function ($business) use ($req) {
            Queue::later(Carbon::now(), new ReportCreatorJob($req, $business->id));
        });
    }
}
