<?php

namespace App\Console\Commands;

use App\Business;
use App\Http\Controllers\ExtraController;
use App\Http\Controllers\SiteSettingController;
use Illuminate\Console\Command;
use App\Http\Controllers\StripeTransactionController;
use App\Jobs\StripeFetchJob;
use App\SiteSetting;
use Carbon\Carbon;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Queue;

class StripeTransactionFetcher extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'StripeTransactionFetcher:fetch';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Cron for fetching stripe transactions';

    public static $process_busy = false;

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $testStripeKey = "";
        $liveStripeKey = "";
        $connectLiveStripeKey = "";
        $connectTestStripeKey = "";

        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;
        $connectStripeMode = SiteSettingController::findByKey('connect_stripe_mode')->value;

        $testKeySetting = SiteSettingController::findByKey('stripe_private_key_test');
        $liveKeySetting = SiteSettingController::findByKey('stripe_private_key_live');

        $connectTestKeySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
        $connectLiveKeySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');

        if (!empty($testKeySetting)) {
            $testStripeKey = trim($testKeySetting->value);
        }
        if (!empty($liveKeySetting)) {
            $liveStripeKey = trim($liveKeySetting->value);
        }
        if (!empty($connectTestKeySetting)) {
            $connectTestStripeKey = trim($connectTestKeySetting->value);
        }
        if (!empty($connectLiveKeySetting)) {
            $connectLiveStripeKey = trim($connectLiveKeySetting->value);
        }

        $testKeyBusinesses = Business::where('disabled', 0)->where('stripe_private_key_test', '!=', $testStripeKey)->whereNotNull('stripe_private_key_test')->whereNull('s_account_id')->groupBy('stripe_private_key_test')->get();
        $liveKeyBusinesses = Business::where('disabled', 0)->where('stripe_private_key_live', '!=', $liveStripeKey)->whereNotNull('stripe_private_key_live')->whereNull('s_account_id')->groupBy('stripe_private_key_live')->get();

        if (!$testKeyBusinesses->isEmpty()) {
            $testKeyBusinesses->each(function ($business) {
                Queue::later(Carbon::now(), new StripeFetchJob($business, $business['stripe_private_key_test'], 'test'));
            });
        }
        if (!$liveKeyBusinesses->isEmpty()) {
            $liveKeyBusinesses->each(function ($business) {
                Queue::later(Carbon::now(), new StripeFetchJob($business, $business['stripe_private_key_live'], 'live'));
            });
        }
        if ($stripeMode == 'test') {
            Queue::later(Carbon::now(), new StripeFetchJob(null, $testStripeKey, 'test', 'normal'));
        }
        if ($stripeMode == 'live') {
            Queue::later(Carbon::now(), new StripeFetchJob(null, $liveStripeKey, 'live', 'normal'));
        }
        if ($connectStripeMode == 'test') {
            Queue::later(Carbon::now(), new StripeFetchJob(null, $connectTestStripeKey, 'test', 'connect'));
        }
        if ($connectStripeMode == 'live') {
            Queue::later(Carbon::now(), new StripeFetchJob(null, $connectLiveStripeKey, 'live', 'connect'));
        }
    }
}
