<?php

namespace App\Console;

use App\Business;
use App\Http\Controllers\ActionLogController;
use App\Http\Controllers\CaxtonfxController;
use App\Http\Controllers\Connect\ConnectAccountController;
use App\Http\Controllers\SiteSettingController;
use App\Http\Controllers\SMSLogController;
use App\Http\Controllers\StripeTransactionController;
use App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController;
use Carbon\Carbon;
use Illuminate\Console\Scheduling\Schedule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Laravel\Lumen\Console\Kernel as ConsoleKernel;
use Stripe\StripeClient;

class Kernel extends ConsoleKernel
{
    /**
     * The Artisan commands provided by your application.
     *
     * @var array
     */
    protected $commands = [
        '\App\Console\Commands\OnlineOrderFetcher',
        '\App\Console\Commands\StripeTransactionFetcher',
        '\App\Console\Commands\TransferTransactionFetcher',
        '\App\Console\Commands\PaypalTransactionsFetcher',
        '\App\Console\Commands\ReportCreator',
        '\App\Console\Commands\StripeTransactionTransferrer',
    ];

    /**
     * Define the application's command schedule.
     *
     * @param  \Illuminate\Console\Scheduling\Schedule  $schedule
     * @return void
     */
    protected function schedule(Schedule $schedule)
    {
        $schedule->command('ReportCreator:create')->twiceDaily()->withoutOverlapping()->runInBackground(); //->appendOutputTo('/storage/logs/cron_output.log');
        $schedule->command('StripeTransactionFetcher:fetch')->everyThreeMinutes()->withoutOverlapping()->runInBackground(); // ->onOneServer()
        $schedule->command('TransferTransactionFetcher:fetch')->everyThreeMinutes()->withoutOverlapping()->runInBackground(); // ->onOneServer()
        $schedule->command('InvoiceRecurring:fetch')->dailyAt('12:02')->withoutOverlapping()->runInBackground(); // ->onOneServer()
        $schedule->command('ClearOldLogs:clear')->daily()->withoutOverlapping()->runInBackground(); // ->onOneServer()
        // $schedule->command('PaypalTransactionsFetcher:fetch')->everyFiveMinutes()->withoutOverlapping()->runInBackground(); //->appendOutputTo('/storage/logs/cron_output.log');
        $schedule->command('OnlineOrderFetcher:fetch')->everyFiveMinutes()->withoutOverlapping()->runInBackground(); //->appendOutputTo('/storage/logs/cron_output.log');

        // Monthly / weekly recurring fees debit Only Save Databse
        $schedule->call(function () {
            $req = Request::create("");
            $req->merge(['job' => true]);
            $req->merge(['save' => true]);
            echo json_encode((new ConnectAccountController())->debit_recurring_fees($req));
        })->dailyAt('14:30')->runInBackground();

        // Monthly / weekly recurring fees debit charge
        $schedule->call(function () {
            $req = Request::create("");
            $req->merge(['job' => true]);
            echo json_encode((new ConnectAccountController())->debit_recurring_fees($req));
        })->dailyAt('14:32')->runInBackground();

        // Stripe Country and Card brand Update
        $schedule->call(function () {
            echo json_encode((new StripeTransactionController())->saveBlankChargeDetail());
        })->everySixHours()->runInBackground(); // daily //everySixHours

        // Monthly weekly recurring fees debit
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new StripeTransactionController())->updateDisputeTransaction($req));
        })->everyFiveMinutes()->runInBackground(); // dailyAt

        // Miss Stripe Transaction Add 
        $schedule->call(function () {
            echo json_encode((new StripeTransactionController())->missTransactionDetail());
        })->everyThreeMinutes()->runInBackground(); // daily //everySixHours

        // Reverse Commission Calculate Stripe Transaction Update 
        $schedule->call(function () {
            echo json_encode((new StripeTransactionController())->updateTransactionDetails());
        })->dailyAt('05:00')->runInBackground(); // daily 5'O Clock

        // Payout Only Save Databse
        $schedule->call(function () {
            echo json_encode((new StripeTransactionController())->payoutTransactionDetail());
        })->everyThreeMinutes()->runInBackground();

        // Caxton Sync wallet transactions
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new CaxtonfxController())->fetch_all_wallet_transaction($req));
        })->everyThreeMinutes()->runInBackground();

        // Caxton Sync Refund wallet transactions
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new CaxtonfxController())->fetch_all_refund_wallet_transaction($req));
        })->everyFiveMinutes()->runInBackground();

        // Caxton Sync transactions
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new CaxtonfxController())->fetch_all_transactions($req));
        })->everyThreeMinutes()->runInBackground();

        // Caxton Sync transfer transactions
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new CaxtonfxController())->fetch_all_transfer_transactions($req));
        })->everyThreeMinutes()->runInBackground();

        // Transfer Status Get
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new CaxtonfxController())->fetch_pending_transfers($req));
        })->everyMinute()->runInBackground();

        // Wallet Transaction Status Get
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new CaxtonfxController())->fetch_wallet_transaction_statuses($req));
        })->everyMinute()->runInBackground();

        // Pay By link delete previuos
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new TiffintomShortlnUrlController())->delete_previous($req));
        })->everySixHours()->runInBackground();

        // Action Logs delete previuos
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new ActionLogController())->delete_previous($req));
        })->everySixHours()->runInBackground();

        // Sms Logs delete previuos
        $schedule->call(function () {
            $req = Request::create("");
            echo json_encode((new SMSLogController())->delete_previous($req));
        })->everySixHours()->runInBackground();

        $schedule->command('queue:work --stop-when-empty')->everyMinute(); //->withoutOverlapping();
        $schedule->command('queue:work --queue=wallet_transfer --stop-when-empty')->everyMinute(); //->withoutOverlapping();
    }
}
