<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class DeviceRegistration extends Model
{
    protected $fillable = [
        'device_id',
        'device_token',
        'serial_number',
        'device_type',
        'service_type', // Epos,Merchant
        'web_order',
        'caller_id',
        'sunmi_printer_enable',
        'is_tap_to_pay',
        'name',
        'status',
        'business_id',
    ];
    protected $hidden = ['updated_at'];
    protected $appends = ['merchant_device_allow', 'epos_device_allow', 'approved_devices', 'epos_approved_devices'];
    protected $casts = ['web_order' => 'bool', 'caller_id' => 'bool', 'sunmi_printer_enable' => 'bool', 'is_tap_to_pay' => 'bool'];

    // Attributes
    //
    public function getMerchantDeviceAllowAttribute()
    {
        $business = Business::select('merchant_device_allow', 'id')->where('id', $this->business_id)->first();
        if (!empty($business)) {
            $business->makeHidden(['user_counts', 'business_commissions', 'country', 'balance']);
            return $business->merchant_device_allow;
        } else return null;
    }
    public function getApprovedDevicesAttribute()
    {
        $device_approved = Business::select('approved_devices')->leftJoin('device_registrations', 'device_registrations.business_id', '=', 'businesses.id')->where('businesses.id', $this->business_id)->where('device_registrations.service_type', 'Merchant')->where('device_registrations.status', 'Approved')->count();
        if (!empty($device_approved)) {
            return $device_approved;
        } else return 0;
    }
    public function getEposDeviceAllowAttribute()
    {
        $business = Business::select('epos_device_allow', 'id')->where('id', $this->business_id)->first();
        if (!empty($business)) {
            $business->makeHidden(['user_counts', 'business_commissions', 'country', 'balance']);
            return $business->epos_device_allow;
        } else return null;
    }
    public function getEposApprovedDevicesAttribute()
    {
        $epos_device_approved = Business::select('approved_devices')->leftJoin('device_registrations', 'device_registrations.business_id', '=', 'businesses.id')->where('businesses.id', $this->business_id)->where('device_registrations.service_type', 'Epos')->where('device_registrations.status', 'Approved')->count();
        if (!empty($epos_device_approved)) {
            return $epos_device_approved;
        } else return 0;
    }
}
