<?php

namespace App\Exceptions;

use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Database\QueryException;
use Illuminate\Validation\ValidationException;
use Laravel\Lumen\Exceptions\Handler as ExceptionHandler;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\Exception\MethodNotAllowedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Throwable;

class Handler extends ExceptionHandler
{
    /**
     * A list of the exception types that should not be reported.
     *
     * @var array
     */
    protected $dontReport = [
        AuthorizationException::class,
        HttpException::class,
        ModelNotFoundException::class,
        ValidationException::class,
        BaseException::class,
    ];

    /**
     * Report or log an exception.
     *
     * This is a great spot to send exceptions to Sentry, Bugsnag, etc.
     *
     * @param  \Throwable  $exception
     * @return void
     *
     * @throws \Exception
     */
    public function report(Throwable $exception)
    {
        parent::report($exception);
    }

    /**
     * Render an exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Throwable  $exception
     * @return \Illuminate\Http\Response|\Illuminate\Http\JsonResponse
     *
     * @throws \Throwable
     */
    public function render($request, Throwable $exception)
    {
        if ($request->is('api/*') || $request->is('connect/*') || $request->is('merchant/*') || $request->is('pg/*')  || $request->is('reseller/*') || $request->is('epos/*') || $request->is('tiffintom/*') || $request->is('online-desk-integration/*') || $request->wantsJson()) {
            $code = 400;
            $message = $exception->getMessage();
            if (!$request->is('epos/*') && $exception instanceof QueryException) {
                $code = 400;
                $message = "Request could not processed!";
            }
            if ($exception instanceof MethodNotAllowedHttpException) {
                $code = 400;
                $message = "Invalid request method";
            }
            if ($exception instanceof NotFoundHttpException) {
                $code = 404;
                $message = 'Path not found';
            }
            //
            else if ($exception instanceof HttpException) {
                $code = $exception->getStatusCode();
            }
            //
            else if ($exception instanceof BaseException) {
                return response()->json($exception->getErrorArray(), $exception->getCode());
            }
            //
            else if ($exception instanceof ValidationException) {
                $errors = $exception->errors();
                $message = reset($errors)[0];
            }
            return response()->json(['code' => $code, 'message' => $message], $code);
        }
        return parent::render($request, $exception);
    }
}
