<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class AdminMiscReportExport implements FromCollection, ShouldAutoSize, WithTitle, WithEvents
{
    use Exportable;
    public $report;
    public $fullyMergableRowNo = array();
    public $borderableRowNo = array();
    public $netAmountRows = array();
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function title(): string
    {
        $fromDate = Carbon::parse($this->report['from_date']);
        $toDate = Carbon::parse($this->report['to_date']);
        return $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $report = $this->report;
        $reportPeriod = Carbon::parse($report['from_date'])->format('d M Y') . " to " . Carbon::parse($report['to_date'])->format('d M Y');
        $normal_payout_report = $report['total__summary'];
        $summaryHeaders[] = ['Date Period', '', 'Total'];
        $summaryHeaders[] = [$reportPeriod, '',  $normal_payout_report['total'] ? $normal_payout_report['total'] : '0'];
        $summaryHeadersEmptyFooter[] = [' '];
        $mergableRowNoStart = 3;
        $data = [];
        if (!$report['misc_report']->isEmpty()) {
            $data[] = [''];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Business Name', 'Misc Charge', 'Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['misc_report']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:C$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";
            foreach ($report['misc_report'] as $invoice) {
                $data[] = [$invoice['name'], $invoice['misc_charge'] ? $invoice['misc_charge'] : '0',  $invoice['comment']];
            }
        }

        return new Collection([
            $summaryHeaders,
            $summaryHeadersEmptyFooter,
            $data
        ]);
        // return $this->report;
    }
    public function registerEvents(): array
    {

        return [
            AfterSheet::class    => function (AfterSheet $event) {

                $event->sheet->getStyle("A1:C2")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana',
                        'color' => ['argb' => '00ffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '000000']
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00ffffff'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);
                foreach ($this->netAmountRows as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                            'color' => ['argb' => '00ffffff'],
                        ], 'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '000000']
                        ],
                    ]);
                }
                foreach ($this->fullyMergableRowNo as $rowNo) {
                    // $event->sheet->getDelegate()->mergeCells("A$rowNo:C$rowNo");
                    $event->sheet->getStyle("A$rowNo:C$rowNo")->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                    $headerRowNo = $rowNo + 1;
                    $event->sheet->getStyle("A$headerRowNo:C$headerRowNo")->applyFromArray([
                        'font' => [
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ]
                    ]);
                }
                foreach ($this->borderableRowNo as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                }
            },
        ];
    }
}
