<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class AdminResellerPayoutReportExport implements FromCollection, ShouldAutoSize, WithTitle, WithEvents
{
    use Exportable;
    public $report;
    public $fullyMergableRowNo = array();
    public $borderableRowNo = array();
    public $netAmountRows = array();
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function title(): string
    {
        $fromDate = Carbon::parse($this->report['from_date']);
        $toDate = Carbon::parse($this->report['to_date']);
        return $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $report = $this->report;
        $reportPeriod = Carbon::parse($report['from_date'])->format('d M Y') . " to " . Carbon::parse($report['to_date'])->format('d M Y');
        $normal_payout_report = $report['normal_payout_summary'];
        $summaryHeaders[] = ['Date Period', 'Bank Payout', 'Archive Payout', 'Paypal Payout', 'In Account', 'Total'];
        $summaryHeaders[] = [$reportPeriod, $normal_payout_report['bank_payout'] ? $normal_payout_report['bank_payout'] : '0', $normal_payout_report['cash_payout'] ? $normal_payout_report['cash_payout'] : '0', $normal_payout_report['paypal_payout'] ? $normal_payout_report['paypal_payout'] : '0', $normal_payout_report['manual_payout'] ? $normal_payout_report['manual_payout'] : '0', $normal_payout_report['total'] ? $normal_payout_report['total'] : '0'];
        $summaryHeadersEmptyFooter[] = [' '];
        $mergableRowNoStart = 3;
        $data = [];
        if (!$report['normal_auto_bank_payouts']->isEmpty()) {
            $data[] = [''];
            $data[] = ['Auto Payout Bank'];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Name', 'Net', 'Mode', 'Account', '', 'Payout Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['normal_auto_bank_payouts']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";
            foreach ($report['normal_auto_bank_payouts'] as $invoice) {
                $data[] = [$invoice['first_name'] . " " . $invoice['last_name'], $invoice['net_fee'] ? $invoice['net_fee'] : '0', 'Bank', isset($invoice['payout_account']['account_number']) ? $invoice['payout_account']['account_number'] : ' ',   "", $invoice['payout_comment']];
            }
        }
        if (!$report['normal_auto_paypal_payouts']->isEmpty()) {

            $data[] = [''];
            $data[] = ['Auto Payout Paypal'];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Name', 'Net', 'Mode', 'Account', '', 'Payout Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['normal_auto_paypal_payouts']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";

            foreach ($report['normal_auto_paypal_payouts'] as $invoice) {
                $data[] = [$invoice['first_name'] . " " . $invoice['last_name'], $invoice['net_fee'] ? $invoice['net_fee'] : '0', 'Paypal', isset($invoice['payout_account']['account_number']) ? $invoice['payout_account']['account_number'] : ' ',  "", $invoice['payout_comment']];
            }
        }
        if (isset($report['normal_auto_cash_payouts']) && !$report['normal_auto_cash_payouts']->isEmpty()) {

            $data[] = [''];
            $data[] = ['Auto Payout Cash'];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Name', 'Net', 'Mode', 'Account', '', 'Payout Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['normal_auto_cash_payouts']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";
            foreach ($report['normal_auto_cash_payouts'] as $invoice) {
                $data[] = [$invoice['first_name'] . " " . $invoice['last_name'], $invoice['net_fee'] ? $invoice['net_fee'] : '0', 'Cash', "",  "", $invoice['payout_comment']];
            }
        }
        if (!$report['archived_auto_bank_payouts']->isEmpty()) {

            $data[] = [''];
            $data[] = ['Archived Auto Bank'];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Name', 'Net', 'Mode', 'Account', '', 'Payout Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['archived_auto_bank_payouts']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";

            foreach ($report['archived_auto_bank_payouts'] as $invoice) {
                $data[] = [$invoice['first_name'] . " " . $invoice['last_name'], $invoice['net_fee'] ? $invoice['net_fee'] : '0', 'Bank', isset($invoice['payout_account']['account_number']) ? $invoice['payout_account']['account_number'] : ' ', "",   $invoice['payout_comment']];
            }
        }
        if (!$report['archived_auto_paypal_payouts']->isEmpty()) {

            $data[] = [''];
            $data[] = ['Archived Auto Paypal'];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Name', 'Net', 'Mode', 'Account', '', 'Payout Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['archived_auto_paypal_payouts']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";

            foreach ($report['archived_auto_paypal_payouts'] as $invoice) {
                $data[] = [$invoice['first_name'] . " " . $invoice['last_name'], $invoice['net_fee'] ? $invoice['net_fee'] : '0', 'Paypal', isset($invoice['payout_account']['account_number']) ? $invoice['payout_account']['account_number'] : ' ', " ",  $invoice['payout_comment']];
            }
        }
        if (isset($report['archived_auto_cash_payouts']) && !$report['archived_auto_cash_payouts']->isEmpty()) {


            $data[] = [''];
            $data[] = ['Archived Auto Cash'];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Name', 'Net', 'Mode', 'Account', '', 'Payout Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['archived_auto_cash_payouts']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";

            foreach ($report['archived_auto_cash_payouts'] as $invoice) {
                $data[] = [$invoice['first_name'] . " " . $invoice['last_name'], $invoice['net_fee'] ? $invoice['net_fee'] : '0', 'Cash', "", "",   $invoice['payout_comment']];
            }
        }
        if (!$report['manual_payouts']->isEmpty()) {


            $data[] = [''];
            $data[] = ['Held Account Balance'];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Name', 'Net', 'Mode', 'Account', '', 'Payout Comment'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['manual_payouts']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";

            foreach ($report['manual_payouts'] as $invoice) {
                $data[] = [$invoice['first_name'] . " " . $invoice['last_name'], $invoice['net_fee'] ? $invoice['net_fee'] : '0', 'Balance', "",   "", $invoice['payout_comment']];
            }
        }

        return new Collection([
            $summaryHeaders,
            $summaryHeadersEmptyFooter,
            $data
        ]);
        // return $this->report;
    }
    public function registerEvents(): array
    {

        return [
            AfterSheet::class    => function (AfterSheet $event) {

                $event->sheet->getStyle("A1:F2")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana',
                        'color' => ['argb' => '00ffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '000000']
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00ffffff'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);
                foreach ($this->netAmountRows as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                            'color' => ['argb' => '00ffffff'],
                        ], 'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '000000']
                        ],
                    ]);
                }
                foreach ($this->fullyMergableRowNo as $rowNo) {
                    $nextRow = $rowNo + 1;
                    $event->sheet->getDelegate()->mergeCells("A$rowNo:F$rowNo");
                    $event->sheet->getDelegate()->mergeCells("D$nextRow:E$nextRow");
                    $event->sheet->getStyle("A$rowNo:F$rowNo")->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                    $headerRowNo = $rowNo + 1;
                    $event->sheet->getStyle("A$headerRowNo:G$headerRowNo")->applyFromArray([
                        'font' => [
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ]
                    ]);
                }
                foreach ($this->borderableRowNo as $rowNo) {
                    $range = explode(":", $rowNo);
                    $startingRow = str_replace("A", "", $range[0]) + 2;
                    $endingRow = str_replace("F", "", $range[1]) + 2;
                    $event->sheet->getDelegate()->mergeCells("D$startingRow:E$startingRow");
                    $event->sheet->getDelegate()->mergeCells("D$endingRow:E$endingRow");
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                }
            },
        ];
    }
}
