<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class FinancialReportExport implements FromCollection, ShouldAutoSize, WithTitle, WithEvents
{
    use Exportable;
    public $report;
    public $fullyMergableRowNo = array();
    public $borderableRowNo = array();
    public $netAmountRows = array();
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function title(): string
    {
        return  "Financial Report";
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $report = $this->report;
        $summaryHeaders[] = [' '];
        $summaryHeaders[] = [
            'Total Transaction',
            'Gross Total',
            'Gross Fee',
            'Gross Provides Fee',
            'Gross Reseller Fee',
            'Gross Refund',
            'Net Total',
            'Cash Payout',
            'Bank Payout',
            'Earning',
        ];

        $summaryHeaders[] = [
            sizeof($report['transactions']),
            $report['gross_total'],
            $report['gross_business_fee'],
            $report['gross_provider_fee'],
            $report['gross_reseller_fee'],
            $report['gross_refund'],
            $report['net_total'],
            $report['cash_payout'],
            $report['bank_payout'],
            $report['total_earning'],
        ];

        $summaryHeaders[] = [' '];
        $summaryHeadersEmptyFooter[] = [' '];
        $mergableRowNoStart = 2;
        $data = [];
        if (!empty($report['transactions'])) {
            $data[] = [''];

            $mergableRowNoStart = $mergableRowNoStart + 5;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = $mergableRowNoStart;
            $data[] = ['Id', 'Business', 'Type', 'Status', 'Amount', 'Business Fee', 'Reseller Fee', 'Provider Fee', 'Total Fee', 'Net Total'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report['transactions']);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:J$borderableRowEnd";
            $this->netAmountRows[] = "B" . ($borderableRowStart + 2) . ":B$borderableRowEnd";
            foreach ($report['transactions'] as $datas) {
                $data[] = [
                    $datas['payment_method'] != 'cod' ? $datas['txn_id'] : $datas['order_id'],
                    $datas['business_name'] . ($datas['order_archived'] == '1' ? ' : Archive' : ''), // . ' : ' . $datas['order_archived'] == '1' ? 'Archive' : '',
                    $datas['order_type'],
                    $datas['order_status'],
                    $datas['order_grand_total'],
                    $datas['business_fee'],
                    $datas['reseller_fee'] ? $datas['reseller_fee'] : 0,
                    $datas['provider_fee'],
                    $datas['total_fee'],
                    $datas['net_total'],
                ];
            }
        }

        return new Collection([
            $summaryHeaders,
            $summaryHeadersEmptyFooter,
            $data
        ]);
        // return $this->report;
    }
    public function registerEvents(): array
    {

        return [
            AfterSheet::class    => function (AfterSheet $event) {

                $event->sheet->getStyle("A2:J3")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana',
                        'bold' => true,
                        'color' => ['argb' => '00000000'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            // 'color' => ['argb' => '00000000'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);

                foreach ($this->fullyMergableRowNo as $rowNo) {
                    // $event->sheet->getDelegate()->mergeCells("A$rowNo:J$rowNo");
                    $event->sheet->getStyle("A$rowNo:J$rowNo")->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                        'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '00D9D9D9']
                        ],
                    ]);
                    $headerRowNo = $rowNo + 1;
                    $event->sheet->getStyle("A$headerRowNo:J$headerRowNo")->applyFromArray([
                        'font' => [
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ]
                    ]);
                }
                foreach ($this->borderableRowNo as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                }
            },
        ];
    }
}
