<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class MiscInvoiceExport implements FromCollection, ShouldAutoSize, WithEvents, WithTitle
{
    use Exportable;
    protected $miscpayoutInvoice;
    public function __construct($invoice)
    {
        $this->miscpayoutInvoice = $invoice;
    }

    public function title(): string
    {

        return $this->miscpayoutInvoice['business']['name'];
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $business = $this->miscpayoutInvoice['business'];
        $topHeader[] = [' '];
        $topHeader[] = ['Statement'];
        $topHeader[] = [' '];
        $topHeader[] = ['Simple Merchant Payment Service'];
        $topHeader[] = [' '];
        $topHeader[] = ['66 Little caldmore', '', '', '', '', '', '', 'Date: ' . Carbon::parse($this->miscpayoutInvoice['date'])->format('M d, Y')];
        $topHeader[] = ['Walsall', '', '', '', '', '', '', $business['address']];
        $topHeader[] = ['WS1 3RA', '', '', '', '', '',  '', $business['country']['name']];
        $topHeader[] = ['Email: admin@smartmps.co.uk', '', '', '', '', '', '', 'Email: ' . $business['email']];
        $topHeader[] = ['Tel: 08712884044'];
        $emptySpace[] = [' '];
        $tableHeader[] = ['Sr No.', 'Description', '', '', '', 'Rate', 'Quantity', 'Total'];
        $items = $this->miscpayoutInvoice['misc_invoice_items'];
        $srNo = 1;
        foreach ($items as $item) {
            $tableContent[] = [$srNo, $item['description'], '', '', '', $item['rate'], $item['quantity'], $item['total']];
            $srNo++;
        }
        return new Collection([
            $topHeader,
            $emptySpace,
            $tableHeader,
            $tableContent
        ]);
        // return $this->report;
    }
    public function registerEvents(): array
    {
        $tableHeaderRowNo = 12;

        return [
            AfterSheet::class    => function (AfterSheet $event) use ($tableHeaderRowNo) {
                $event->sheet->getDelegate()->mergeCells("A2:H2");
                $event->sheet->getStyle("A2:I2")->applyFromArray(
                    [
                        'font' => [
                            'size' => 18,
                            'bold' => true,
                            'fontFamily' => 'Verdana'
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_LEFT,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ]
                    ]
                );
                $event->sheet->getDelegate()->mergeCells("A4:C4");
                $event->sheet->getDelegate()->mergeCells("A5:C5");
                $event->sheet->getDelegate()->mergeCells("A6:C6");
                $event->sheet->getDelegate()->mergeCells("A7:C7");
                $event->sheet->getDelegate()->mergeCells("A8:C8");
                $event->sheet->getDelegate()->mergeCells("A9:C9");
                $event->sheet->getDelegate()->mergeCells("A10:C10");
                $event->sheet->getDelegate()->mergeCells("A11:C11");


                $event->sheet->getDelegate()->mergeCells("B$tableHeaderRowNo:E$tableHeaderRowNo");
                $tableHeaderRange = "A$tableHeaderRowNo:H$tableHeaderRowNo";
                $event->sheet->getStyle($tableHeaderRange)->applyFromArray(
                    [
                        'font' => [
                            'size' => 12,
                            'bold' => true,
                            'fontFamily' => 'Verdana',
                            'color' => ['argb' => '00ffffff'],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ], 'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '00138D75']
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                                'borderColor' => ['argb' => '0073C6B6']
                            ],
                        ],
                    ]
                );

                $tableContentStart = $tableHeaderRowNo;
                $tableContentEnd = $tableContentStart + sizeof($this->miscpayoutInvoice['misc_invoice_items']);
                $tableContentRange = "A$tableContentStart:H$tableContentEnd";

                $event->sheet->getStyle($tableContentRange)->applyFromArray(
                    [
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana'
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                                'borderColor' => ['argb' => '0073C6B6']
                            ],
                        ],
                    ]
                );
                $srNo = 0;
                $srNo = $srNo + $tableContentStart;
                foreach ($this->miscpayoutInvoice['misc_invoice_items'] as $item) {
                    $event->sheet->getDelegate()->mergeCells("B$srNo:E$srNo");
                    $srNo++;
                }
            },
        ];
    }
}
