<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class PayoutReportExport implements FromCollection, ShouldAutoSize, WithEvents, WithTitle
{
    use Exportable;
    public $report;
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function array(): array
    {
        return [
            ["From Date", "To Date"],
            [$this->report['from_date'], $this->report['to_date']]
        ];
    }
    public function title(): string
    {
        $fromDate = Carbon::parse($this->report['from_date']);
        $toDate = Carbon::parse($this->report['to_date']);
        $type = "";
        if (strtolower($this->report['transaction_type']) == 'archived') {
            $type = "Cash ";
        }
        return   $type . $this->report['business']['name'];
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $reportHeaders = [];
        $sheetHeader = [];
        $codReport = [];
        $fromDate = Carbon::parse($this->report['from_date']);
        $toDate = Carbon::parse($this->report['to_date']);
        if (isset($this->report['transactions'])) {
            $sheetHeader[] = [' '];
            if ($this->report['transaction_type'] != 'Normal') {
                $sheetHeader[] = [$this->report['transaction_type'] . " " . $this->report['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y')];
            } else {
                $sheetHeader[] = [$this->report['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y')];
            }
            $sheetHeader[] = [$fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y')];
            $sheetHeader[] = [' '];

            $codReport[] = ['Total Transactions', 'Gross Total', 'Gross Fee', 'Service Charge', 'Total Fee', 'Net Total', '', ''];
            $codReport[] = [$this->report['transactions'] ? sizeof($this->report['transactions']) : '0', $this->report['gross_total'] ? $this->report['gross_total'] : '0', $this->report['gross_fee'] ? $this->report['gross_fee'] : '0', $this->report['total_service_charge'] ? $this->report['total_service_charge'] : '0', $this->report['net_fee'] ? $this->report['net_fee'] : '0', $this->report['net_total'] && ($this->report['transaction_type'] == 'Archived' || $this->report['payable_tot_amount'] > 0) ? $this->report['net_total'] : '0(' . $this->report['net_total'] . ')', '', ''];
            $codReport[] = ['Bank Detail', 'Bank Name', 'IFSC Code', 'Route', 'Sort Code', 'Account No', '', ''];
            if (isset($this->report['business']['default_bank_detail'])) {
                $bankDetails = $this->report['business']['default_bank_detail'];
                $codReport[] = [$this->report['business']['name'], $bankDetails['bank_name'], '', '', $bankDetails['sort_code'], $bankDetails['account_number'], '', ''];
            } else {
                $codReport[] = [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '];
            }
            $codReport[] = [' '];
            $codReport[] = [' '];
            if (sizeof($this->report['transactions']) > 0)
                $codReport[] = ['ID', 'Date', 'TR Type', 'Gross', 'Fee', 'Net', 'Card - Country', 'Status'];
            foreach ($this->report['transactions'] as $transaction) {

                if (strtolower($transaction['order_type']) == 'merchant') {
                    $codReport[] = [$transaction['card_brand'] != null ? $transaction['card_brand'] : $transaction['txn_id'], $transaction['order_date'], $transaction['order_type'], $transaction['order_grand_total'], $transaction['total_fee'] ? $transaction['total_fee'] : '0', $transaction['net_total'], $transaction['card_brand'] . '-' . $transaction['country'], $transaction['order_status']];
                } else {
                    if (strtolower($transaction['payment_method']) == 'cod' || strtolower($transaction['payment_method']) == 'wallet') {
                        $codReport[] = [$transaction['card_brand'] != null ? $transaction['card_brand'] : $transaction['order_id'], $transaction['order_date'], $transaction['payment_method'] == 'Connect' ? 'Merchant Service' :ucwords($transaction['payment_method']), $transaction['order_grand_total'], $transaction['total_fee'] ? $transaction['total_fee'] : '0', $transaction['net_total'], $transaction['card_brand'] . '-' . $transaction['country'], $transaction['order_status']];
                    } else
                        $codReport[] = [$transaction['card_brand'] != null ? $transaction['card_brand'] : $transaction['txn_id'], $transaction['order_date'], $transaction['payment_method'] == 'Connect' ? 'Merchant Service' : ucwords($transaction['payment_method']), $transaction['order_grand_total'], $transaction['total_fee'] ? $transaction['total_fee'] : '0', $transaction['net_total'], $transaction['card_brand'] . '-' . $transaction['country'], $transaction['order_status']];
                }
                // $codReport[] = [$transaction['order_date'], $transaction['order_grand_total'], $transaction['total_fee'], $transaction['net_total'], $transaction['order_status']];
            }
            $footer[] = [' '];
            $footer[] = [' '];
            $footer[] = ['Charges Summary'];
            if (strtolower($this->report['transaction_type']) == 'normal') {
                $footer[] = ['', '', 'Payout Charges', 'Charge Back Count', 'Refunds', 'Non Domestic Card',  'Payout', ''];
                $footer[] = ['', '', $this->report['payout_charge'] ? $this->report['payout_charge'] : '0', $this->report['total_charge_back_transactions'] ? $this->report['total_charge_back_transactions'] : '0', $this->report['total_refund_transactions'] ? $this->report['total_refund_transactions'] : '0', $this->report['total_non_domestic_transactions'] ? $this->report['total_non_domestic_transactions'] : '0', $this->report['payout_mode'], ''];
            } else {
                $footer[] = ['Management Fee', 'Handling Fee', 'Payout Charges', 'Charge Back Count', 'Refunds', 'Non Domestic Card', 'Payout', ''];
                $footer[] = [$this->report['management_fee'] ? $this->report['management_fee'] : '0', $this->report['handling_fee'] ? $this->report['handling_fee'] : '0', $this->report['payout_charge'] ? $this->report['payout_charge'] : '0',  $this->report['total_charge_back_transactions'] ? $this->report['total_charge_back_transactions'] : '0', $this->report['total_refund_transactions'] ? $this->report['total_refund_transactions'] : '0', $this->report['total_non_domestic_transactions'] ? $this->report['total_non_domestic_transactions'] : '0',  $this->report['payout_mode'], ''];
            }

            $footer[] = ['Non Domestic Fees', 'Weekly Fee', 'Monthly Fee', 'Misc Fee', 'Charge Back Fees', 'Payout Method', $this->report['payout_method'], ''];

            $surReport = [];
            $invReport = [];

            if (sizeof($this->report['surcharges']) > 0) {
                $surReport[] = [' '];
                $surReport[] = [' '];
                if (sizeof($this->report['surcharges']) > 0) {
                    $surReport[] = ['Surcharges'];
                    $surReport[] = ['Name', 'Charge'];
                }
                foreach ($this->report['surcharges'] as $surcharge) {
                    $surReport[] = [$surcharge['name'], $surcharge['surcharge']];
                }
            }

            if (sizeof($this->report['invoices']) > 0) {
                $invReport[] = [' '];
                $invReport[] = [' '];
                if (sizeof($this->report['invoices']) > 0) {
                    $invReport[] = ['Invoices'];
                    $invReport[] = ['Number', 'Total'];
                }
                foreach ($this->report['invoices'] as $surcharge) {
                    $invReport[] = ["INV000" . $surcharge['id'], $surcharge['amount']];
                }
            }

            $payoutId = "";
            if (strtolower($this->report['payout_method']) == 'paypal') {
                if (isset($this->report['payout_account']) && $this->report['payout_account'] != null)
                    $payoutId = $this->report['payout_account']['email'];
            } else if (strtolower($this->report['payout_method']) == 'bank') {
                if (isset($report['payout_account']) && $report['payout_account'] != null)
                    $payoutId = $this->report['payout_account']['account_number'];
            }
            $footer[] = [$this->report['total_non_domestic_fee'] ? $this->report['total_non_domestic_fee'] : '0', $this->report['weekly_fee'] ? $this->report['weekly_fee'] : '0', $this->report['monthly_fee'] ? $this->report['monthly_fee'] : '0', $this->report['misc_charge'] ? $this->report['misc_charge'] : '0',  $this->report['total_charge_back_fee'] ? $this->report['total_charge_back_fee'] : '0', 'Payout ID', $payoutId];
        }
        return new Collection([
            $sheetHeader,
            $codReport,
            $footer,
            $surReport,
            $invReport,
        ]);
        // return $this->report;
    }
    public function registerEvents(): array
    {
        $topHeaders = 4;
        $topSummaryHeaders = 2;
        $bankHeaders = 0;
        if (sizeof($this->report['transactions']) > 0)
            $tableHeaders = 1;
        else $tableHeaders = 0;
        $topSpaceRows = 2;
        $bottomSpaceRows = 2;

        $bankHeaders = 2;
        $boldableRows = $topHeaders + $topSummaryHeaders + $bankHeaders + $topSpaceRows + $tableHeaders;
        $tilTransactionsLastRow = sizeof($this->report['transactions']) + $boldableRows;
        $footerRows = 5;
        $surchargeRows = 0;
        $invoiceRows = 0;

        if (sizeof($this->report['surcharges']) > 0) {
            $surchargeRows = sizeof($this->report['surcharges']) + 4;
        }
        if (sizeof($this->report['invoices']) > 0) {
            $invoiceRows = sizeof($this->report['invoices']) + 4;
        }
        $fullFooterRows = $invoiceRows + $surchargeRows + $footerRows + $bottomSpaceRows;
        $lastRow = $tilTransactionsLastRow + $fullFooterRows;
        return [
            AfterSheet::class    => function (AfterSheet $event) use ($topHeaders, $tilTransactionsLastRow, $topSpaceRows, $bottomSpaceRows, $lastRow, $boldableRows, $topSummaryHeaders, $bankHeaders) {
                $surchargeRows = 0;
                $invoiceRows = 0;
                if (sizeof($this->report['surcharges']) > 0) {
                    $surchargeRows = sizeof($this->report['surcharges']);
                }
                if (sizeof($this->report['invoices']) > 0) {
                    $invoiceRows = sizeof($this->report['invoices']);
                }
                $chargesHeaderRow = $tilTransactionsLastRow + 3;
                if ($surchargeRows > 0) {
                    $surChargesHeaderRow = $tilTransactionsLastRow + 10;
                }

                if ($invoiceRows > 0) {
                    $invoicesHeaderRow = $lastRow - 1 - $invoiceRows;
                }
                //  else {
                //     $invoicesHeaderRow = $lastRow - 4;
                // }
                // $surChargesHeaderRow = $lastRow - 1 - $surchargeRows;
                // $invoicesHeaderRow = $lastRow - 1 - $invoiceRows;

                $event->sheet->getDelegate()->mergeCells("A$chargesHeaderRow:H$chargesHeaderRow");
                if ($surchargeRows > 0) {
                    $event->sheet->getDelegate()->mergeCells("A$surChargesHeaderRow:H$surChargesHeaderRow");
                }
                if ($invoiceRows > 0) {
                    $event->sheet->getDelegate()->mergeCells("A$invoicesHeaderRow:H$invoicesHeaderRow");
                }
                $event->sheet->getDelegate()->mergeCells("A2:H3");

                // Sheet Details
                $event->sheet->getStyle("A2:H3")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'bold' => true,
                        'fontFamily' => 'Verdana'
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                        ],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                ]);

                // Summary Header
                $topSummaryHeaderStart = $topHeaders + 1;
                $topSummaryHeaderEnd = $topHeaders + $topSummaryHeaders + $bankHeaders;
                $event->sheet->getStyle("A$topSummaryHeaderStart:H$topSummaryHeaderEnd")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'bold' => true,
                        'fontFamily' => 'Verdana'
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                        ],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                ]);
                if ($this->report['payable_tot_amount'] == 0 && $this->report['transaction_type'] == 'Normal') {
                    $event->sheet->getStyle("F6")->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'bold' => true,
                            'fontFamily' => 'Verdana',
                            'strikethrough' => true
                        ],
                        'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ],
                        'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '00D9D9D9']
                        ],
                    ]);
                }

                // Transactions table 
                $tableStarts = $topSummaryHeaderEnd + $topSpaceRows + 1;
                $tableEnds = $tilTransactionsLastRow;
                $event->sheet->getStyle("A$tableStarts:H$tableEnds")->applyFromArray([

                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                        ],
                    ],

                ]);

                // Top headers , bank headers and transactions header row
                $event->sheet->getStyle("A1:H$boldableRows")->applyFromArray(
                    [
                        'font' => [
                            'bold' => true,
                            'size' => 12
                        ],
                    ]
                );

                // Charges row
                $chargesSummaryStarts = $tableEnds + $bottomSpaceRows + 1;
                $chargesSummaryEnds = $lastRow;
                $event->sheet->getStyle("A$chargesSummaryStarts:H$chargesSummaryEnds")->applyFromArray(
                    [
                        'font' => [
                            'bold' => true,
                            'size' => 12
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '00D9D9D9']
                        ],
                    ]
                );
                // Whole sheet
                $event->sheet->getStyle("A1:G$lastRow")->applyFromArray([
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ]

                ]);
            },
        ];
    }
}
