<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class PayoutRequestReportExport implements FromCollection, ShouldAutoSize, WithTitle, WithEvents
{
    use Exportable;
    public $report;
    public $fullyMergableRowNo = array();
    public $borderableRowNo = array();
    public $netAmountRows = array();
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function title(): string
    {
        return  "Payout Requests";
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $report = $this->report;
        $summaryHeaders[] = [' '];
        $summaryHeadersEmptyFooter[] = [' '];
        $mergableRowNoStart = 0;
        $data = [];
        if (!empty($report)) {
            $data[] = [''];

            $mergableRowNoStart = $mergableRowNoStart + 1;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = 0;
            $data[] = ['Initiated Date', 'Est. Arrival Date', 'Business Name', 'Amount', 'Bank', 'Note', 'Acc ID',  'Account Number',  'Payout Status',  'Status',];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:J$borderableRowEnd";
            foreach ($report as $datas) {
                $data[] = [
                    $datas['created'] != '' ? date('Y-m-d', $datas['created']) : '',
                    $datas['arrival_date'] != '' ? date('Y-m-d', $datas['arrival_date']) : '',
                    $datas['business_name'] != '' ? $datas['business_name'] : '',
                    $datas['amount'] != '' ? $datas['amount'] : '',
                    $datas['external_account'] != null ? $datas['external_account']['account_holder_name'] : '',
                    $datas['note'] != '' ? $datas['note'] : '',
                    $datas['external_account'] != null ? $datas['external_account']['s_account_id'] : '',
                    $datas['external_account'] != null ? $datas['external_account']['account_number'] : '',
                    $datas['payout_status'] != null ? $datas['payout_status'] : '',
                    $datas['status'] != '' ? $datas['status'] : '',
                ];
            }
        }
        return new Collection([
            $data
        ]);
    }
    public function registerEvents(): array
    {

        return [
            AfterSheet::class    => function (AfterSheet $event) {

                $event->sheet->getStyle("A2:J2")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana',
                        'bold' => true,
                        'color' => ['argb' => '00000000'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            // 'color' => ['argb' => '00000000'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);

                foreach ($this->borderableRowNo as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                }
            },
        ];
    }
}
