<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class ResellerPayoutReportExport implements FromCollection, ShouldAutoSize, WithEvents, WithTitle
{
    use Exportable;
    public $report;
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function array(): array
    {
        return [
            ["From Date", "To Date"],
            [$this->report['from_date'], $this->report['to_date']]
        ];
    }
    public function title(): string
    {
        $fromDate = Carbon::parse($this->report['from_date']);
        $toDate = Carbon::parse($this->report['to_date']);
        return $this->report['transaction_type'] . " " . $this->report['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
    }
    public function collection()
    {
        $reportHeaders = [];
        $sheetHeader = [];
        $codReport = [];
        $fromDate = Carbon::parse($this->report['from_date']);
        $toDate = Carbon::parse($this->report['to_date']);
        if (isset($this->report['transactions'])) {
            $sheetHeader[] = [' '];
            $sheetHeader[] = [$this->report['reseller']['first_name'] . " " . $this->report['reseller']['last_name']];
            $sheetHeader[] = [$this->report['transaction_type'] . " " . $this->report['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y')];
            $sheetHeader[] = [' '];

            $codReport[] = ['Total Transactions', 'Gross Total', '', '', 'Total Fee', 'Net Total', ''];
            $codReport[] = [sizeof($this->report['transactions']), $this->report['gross_total'] ? $this->report['gross_total'] : '0', '', '', $this->report['net_fee'] ? $this->report['net_fee'] : '0', $this->report['net_total'] ? $this->report['net_total'] : '0', ''];
            if (isset($this->report['reseller']['default_bank_detail'])) {
                $bankDetails = $this->report['reseller']['default_bank_detail'];
                $codReport[] = ['Bank Detail', 'Bank Name', 'IFSC Code', 'Route', 'Sort Code', 'Account No', ''];

                $codReport[] = [$this->report['reseller']['first_name'] . " " . $this->report['reseller']['last_name'], $bankDetails['bank_name'], '', '', $bankDetails['sort_code'], $bankDetails['account_number'], ''];
            }
            $codReport[] = [' '];
            $codReport[] = [' '];
            if (sizeof($this->report['transactions']) > 0)
                $codReport[] = ['ID', 'Date', 'TR Type', 'Gross', 'Fee', 'Net', 'Status'];
            foreach ($this->report['transactions'] as $transaction) {

                if (strtolower($transaction['order_type']) == 'Merchant') {
                    $codReport[] = [$transaction['txn_id'], $transaction['order_date'], $transaction['order_type'], $transaction['order_grand_total'], $transaction['total_fee'] ? $transaction['total_fee'] : '0', $transaction['net_total'], $transaction['order_status']];
                } else {
                    if (strtolower($transaction['payment_method']) == 'cod' || strtolower($transaction['payment_method']) == 'wallet') {
                        $codReport[] = [$transaction['order_id'], $transaction['order_date'], $transaction['payment_method'] == 'Connect' ? 'Merchant Service' : ucwords($transaction['payment_method']), $transaction['order_grand_total'], $transaction['total_fee'] ? $transaction['total_fee'] : '0', $transaction['net_total'], $transaction['order_status']];
                    } else
                        $codReport[] = [$transaction['txn_id'], $transaction['order_date'], $transaction['payment_method'] == 'Connect' ? 'Merchant Service' : ucwords($transaction['payment_method']), $transaction['order_grand_total'], $transaction['total_fee'] ? $transaction['total_fee'] : '0', $transaction['net_total'], $transaction['order_status']];
                }
                // $codReport[] = [$transaction['order_date'], $transaction['order_grand_total'], $transaction['total_fee'], $transaction['net_total'], $transaction['order_status']];
            }
            $footer[] = [' '];
            $footer[] = [' '];
            $footer[] = ['Charges Summary'];
            $footer[] = ['Management Fee', 'Handling Fee',  'Charge Back Fees',  'Charge Back Count', 'Non Domestic Card', '', 'Payout'];
            $footer[] = [$this->report['management_fee'] ? $this->report['management_fee'] : '0',  $this->report['handling_fee'] ? $this->report['handling_fee'] : '0', $this->report['total_charge_back_fee'] ? $this->report['total_charge_back_fee'] : '0', $this->report['total_charge_back_transactions'] ? $this->report['total_charge_back_transactions'] : '0',$this->report['total_non_domestic_transactions'] ? $this->report['total_non_domestic_transactions'] : '0', '', $this->report['payout_mode']];
            $footer[] = ['Non Domestic Fees', 'Monthly Fee','Weekly Fee', 'Misc Fees',  'Refunds', 'Payout Method', $this->report['payout_method']];
            $payoutId = "";
            if (strtolower($this->report['payout_method']) == 'paypal') {
                if (isset($this->report['payout_account']) && $this->report['payout_account'] != null)
                    $payoutId = $this->report['payout_account']['email'];
            } else if (strtolower($this->report['payout_method']) == 'bank') {
                if (isset($report['payout_account']) && $report['payout_account'] != null)
                    $payoutId = $this->report['payout_account']['account_number'];
            }
            $footer[] = [$this->report['total_non_domestic_fee'] ? $this->report['total_non_domestic_fee'] : '0', $this->report['monthly_fee'] ? $this->report['monthly_fee'] : '0',$this->report['weekly_fee'] ? $this->report['weekly_fee'] : '0', $this->report['misc_charge'] ? $this->report['misc_charge'] : '0',  $this->report['total_refund_transactions'] ? $this->report['total_refund_transactions'] : '0', 'Payout ID', $payoutId];
        }
        return new Collection([
            $sheetHeader,
            $codReport,
            $footer
        ]);
        // return $this->report;
    }
    public function registerEvents(): array
    {

        $topHeaders = 4;
        $topSummaryHeaders = 2;
        $bankHeaders = 0;
        if (sizeof($this->report['transactions']) > 0)
            $tableHeaders = 1;
        else $tableHeaders = 0;
        $topSpaceRows = 2;
        $bottomSpaceRows = 2;
        if ($this->report['reseller']['default_bank_detail'] != null) {
            $bankHeaders = 2;
        }
        $boldableRows = $topHeaders + $topSummaryHeaders + $bankHeaders + $topSpaceRows + $tableHeaders;
        $tilTransactionsLastRow = sizeof($this->report['transactions']) + $boldableRows;
        $footerRows = 5;
        $fullFooterRows = $footerRows + $bottomSpaceRows;
        $lastRow = $tilTransactionsLastRow + $fullFooterRows;
        return [
            AfterSheet::class    => function (AfterSheet $event) use ($topHeaders, $tilTransactionsLastRow, $topSpaceRows, $bottomSpaceRows, $lastRow, $boldableRows, $topSummaryHeaders, $bankHeaders) {
                $event->sheet->getDelegate()->mergeCells("B1:D1");
                $event->sheet->getDelegate()->mergeCells("B2:D2");
                $chargesHeaderRow = $lastRow - 4;
                $event->sheet->getDelegate()->mergeCells("A$chargesHeaderRow:G$chargesHeaderRow");

                $event->sheet->getDelegate()->mergeCells("A2:G2");
                $event->sheet->getDelegate()->mergeCells("A3:G3");

                $event->sheet->getDelegate()->mergeCells("B5:D5");
                $event->sheet->getDelegate()->mergeCells("B6:D6");

                // Sheet Details
                $event->sheet->getStyle("A2:G3")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'bold' => true,
                        'fontFamily' => 'Verdana'
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                        ],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                ]);

                // Summary Header
                $topSummaryHeaderStart = $topHeaders + 1;
                $topSummaryHeaderEnd = $topHeaders + $topSummaryHeaders + $bankHeaders;
                $event->sheet->getStyle("A$topSummaryHeaderStart:G$topSummaryHeaderEnd")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'bold' => true,
                        'fontFamily' => 'Verdana'
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                        ],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                ]);

                // Transactions table 
                $tableStarts = $topSummaryHeaderEnd + $topSpaceRows + 1;
                $tableEnds = $tilTransactionsLastRow;
                $event->sheet->getStyle("A$tableStarts:G$tableEnds")->applyFromArray([

                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                        ],
                    ],

                ]);

                // Top headers , bank headers and transactions header row
                $event->sheet->getStyle("A1:G$boldableRows")->applyFromArray(
                    [
                        'font' => [
                            'bold' => true,
                            'size' => 12
                        ],
                    ]
                );

                // Charges row
                $chargesSummaryStarts = $tableEnds + $bottomSpaceRows + 1;
                $chargesSummaryEnds = $lastRow;

                $event->sheet->getStyle("A$chargesSummaryStarts:G$chargesSummaryEnds")->applyFromArray(
                    [
                        'font' => [
                            'bold' => true,
                            'size' => 12
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '00D9D9D9']
                        ],
                    ]
                );
                // Whole sheet
                $event->sheet->getStyle("A1:G$lastRow")->applyFromArray([
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ]

                ]);
            },
        ];
    }
}