<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class ResellerPayoutRequestExport implements FromCollection, ShouldAutoSize, WithTitle, WithEvents
{
    use Exportable;
    public $report;
    public $showUserName = false;
    public $fullyMergableRowNo = array();
    public $borderableRowNo = array();
    public $netAmountRows = array();
    public function __construct($report1, $showUserName)
    {
        $this->report = $report1;
        $this->showUserName = $showUserName;
    }

    public function title(): string
    {
        return  "Reseller Payout Requests";
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $report = $this->report;
        $summaryHeaders[] = [' '];
        $summaryHeadersEmptyFooter[] = [' '];
        $mergableRowNoStart = 0;
        $data = [];
        if (!empty($report)) {
            $data[] = [''];

            $mergableRowNoStart = $mergableRowNoStart + 1;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = 0;
            if ($this->showUserName) {
                $data[] = ['Reseller Name', 'Amount', 'Status', 'Note', 'Request Type', 'Account Number', 'Account Name', 'Rejection Note',];
            } else {
                $data[] = ['Amount', 'Status', 'Note', 'Request Type', 'Account Number', 'Account Name', 'Rejection Note',];
            }

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:G$borderableRowEnd";
            if ($this->showUserName) {
                $this->borderableRowNo[] = "A$borderableRowStart:H$borderableRowEnd";
            }
            foreach ($report as $datas) {
                $new_data = [];

                if ($this->showUserName) {
                    $new_data['user_name'] = $datas['user']['first_name'].' '.$datas['user']['last_name'];
                }

                $new_data['amount'] = $datas['amount'];
                $new_data['status'] = $datas['status'];
                $new_data['note'] = $datas['note'] ?? '';
                if ($datas['request_type'] == 1) {
                    $new_data['request_type'] = 'Cash';
                } elseif ($datas['request_type'] == 2) {
                    $new_data['request_type'] = 'Bank Transfer';
                } elseif ($datas['request_type'] == 3) {
                    $new_data['request_type'] = 'Bacs';
                } elseif ($datas['request_type'] == 4) {
                    $new_data['request_type'] = 'Direct Debit';
                }
                $new_data['account_number'] = $datas['bankDetails']['account_number'] ?? '';
                $new_data['account_name'] = $datas['bankDetails']['account_name'] ?? '';
                $new_data['rejection_note'] = $datas['rejection_note'] ?? '';
                $data[] = $new_data;
            }
        }
        return new Collection([
            $data
        ]);
    }
    public function registerEvents(): array
    {

        return [
            AfterSheet::class    => function (AfterSheet $event) {

                $cells = "A2:G2";
                if ($this->showUserName) {
                    $cells = "A2:H2";
                }

                $event->sheet->getStyle($cells)->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana',
                        'bold' => true,
                        'color' => ['argb' => '00000000'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            // 'color' => ['argb' => '00000000'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);

                foreach ($this->borderableRowNo as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                }
            },
        ];
    }
}
