<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class StripeTransactionExports implements FromCollection, ShouldAutoSize, WithEvents, WithTitle
{
    use Exportable;
    public $transactions;
    public function __construct($transactions)
    {
        $this->transactions = $transactions;
    }

    public function title(): string
    {
        return  "Credit Card Transactions";
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $sheetHeader[] = ['Business Name', 'Amount', 'Fee', 'Provider Fee', 'Type', 'Service', 'Date', 'Txn Id', 'Descriptor', 'Description', 'Archived'];
        $data = [];
        foreach ($this->transactions as $transaction) {
            $data[] = [
                $transaction->business_name,
                $transaction->amount / 100,
                ($transaction->calculated_fees != null) ? $transaction->calculated_fees['total_fee'] : '0.00',
                $transaction->fee / 100,
                $transaction->type,
                $transaction->service,
                Carbon::createFromTimestampMs($transaction->created . '100')->format('d-M-Y h:i:s a'),
                $transaction->txn_id,
                $transaction->descriptor,
                $transaction->description,
                $transaction->archived ? 'Yes' : 'No',
            ];
        }
        return new Collection([
            $sheetHeader,
            $data,
        ]);
        // return $this->report;
    }
    public function registerEvents(): array
    {
        $endRow = sizeof($this->transactions) + 1;
        $cell_range = "A1:K$endRow";
        return [
            AfterSheet::class    => function (AfterSheet $event) use ($cell_range) {

                $event->sheet->getStyle($cell_range)->applyFromArray([
                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana'
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                        ],
                    ]
                ]);
                $event->sheet->getStyle("A1:K1")->applyFromArray([
                    'font' => [
                        'size' => 12,
                        'bold' => true,
                        'fontFamily' => 'Verdana'
                    ]
                ]);
            }
        ];
    }
}
