<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class UbsidiWalletTransferTransactionsExport implements FromCollection, ShouldAutoSize, WithTitle, WithEvents
{
    use Exportable;
    public $report;
    public $fullyMergableRowNo = array();
    public $borderableRowNo = array();
    public $netAmountRows = array();
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function title(): string
    {
        return  "Wallet Transactions";
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $report = $this->report;
        $summaryHeaders[] = [' '];
        $summaryHeadersEmptyFooter[] = [' '];
        $mergableRowNoStart = 0;
        $data = [];
        if (!empty($report)) {
            $data[] = [''];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = 1;
            $data[] = ['Business', 'Category', 'Description', 'Credit Amount', 'Debit Amount', 'Total Balance', 'Type', 'Status', 'Date'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:H$borderableRowEnd";
            foreach ($report as $datas) {
                if ($datas['is_admin'] == '1') {
                    $user = 'Admin - ' . $datas['user_name'];
                } else {
                    $user = 'User - ' . $datas['user_name'];
                }
                $data[] = [
                    $datas['business_name'],
                    $datas['transaction_category'],
                    $datas['description'],
                    $datas['credit_amount'] > 0 ? $datas['credit_amount'] : null,
                    $datas['debit_amount'] > 0 ? $datas['debit_amount'] : null,
                    $datas['total_balance'] > 0 ? $datas['total_balance'] : $datas['total_balance'],
                    $user,
                    $datas['status'],
                    Carbon::parse($datas['created_at'])->format('Y-m-d h:i:s A'),
                ];
            }
        }

        return new Collection([
            $data
        ]);
    }
    public function registerEvents(): array
    {

        return [
            AfterSheet::class    => function (AfterSheet $event) {

                $event->sheet->getStyle("A2:H2")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana',
                        'bold' => true,
                        'color' => ['argb' => '00000000'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            // 'color' => ['argb' => '00000000'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);

                foreach ($this->fullyMergableRowNo as $rowNo) {
                    $event->sheet->getStyle("A$rowNo:H$rowNo")->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                        'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '00D9D9D9']
                        ],
                    ]);
                    $headerRowNo = $rowNo + 1;
                    $event->sheet->getStyle("A$headerRowNo:H$headerRowNo")->applyFromArray([
                        'font' => [
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ]
                    ]);
                }
                foreach ($this->borderableRowNo as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                }
            },
        ];
    }
}
