<?php

namespace App\Exports;

use App\Business;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Fill;

class WalletTransactionsExport implements FromCollection, ShouldAutoSize, WithTitle, WithEvents
{
    use Exportable;
    public $report;
    public $fullyMergableRowNo = array();
    public $borderableRowNo = array();
    public $netAmountRows = array();
    public function __construct($report1)
    {
        $this->report = $report1;
    }

    public function title(): string
    {
        return  "Wallet Transactions";
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $report = $this->report;
        $summaryHeaders[] = [' '];
        $summaryHeadersEmptyFooter[] = [' '];
        $mergableRowNoStart = 0;
        $data = [];
        if (!empty($report)) {
            $data[] = [''];

            $mergableRowNoStart = $mergableRowNoStart + 2;

            $borderableRowStart = $mergableRowNoStart;
            $borderableRowEnd = $mergableRowNoStart;

            $this->fullyMergableRowNo[] = 1;
            $data[] = ['Wallet Name', 'Description', 'Category', 'Credit Amount', 'Debit Amount', 'Created'];

            $mergableRowNoStart = $mergableRowNoStart + 1;
            $mergableRowNoStart = $mergableRowNoStart + sizeof($report);

            $borderableRowEnd = $mergableRowNoStart;
            $this->borderableRowNo[] = "A$borderableRowStart:F$borderableRowEnd";
            foreach ($report as $datas) {
                $card = '';
                if ($datas['card'] != '') {
                    $card = '| Card : ' . $datas['card']['card_holder_name'] . ' - ' . $datas['card']['card_number'];
                }

                $data[] = [
                    $datas['wallet'] != '' ? $datas['wallet']['name'] : '',
                    $datas['description'] . ($card != '' ? $card : ''),
                    $datas['transaction_category'],
                    $datas['credit_amount'] > 0 ? $datas['credit_amount'] : null,
                    $datas['debit_amount'] > 0 ? $datas['debit_amount'] : null,
                    $datas['transaction_date'],
                ];
            }
        }

        return new Collection([
            $data
        ]);
    }
    public function registerEvents(): array
    {

        return [
            AfterSheet::class    => function (AfterSheet $event) {

                $event->sheet->getStyle("A2:F2")->applyFromArray([

                    'font' => [
                        'size' => 12,
                        'fontFamily' => 'Verdana',
                        'bold' => true,
                        'color' => ['argb' => '00000000'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00D9D9D9']
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            // 'color' => ['argb' => '00000000'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);

                foreach ($this->fullyMergableRowNo as $rowNo) {
                    $event->sheet->getStyle("A$rowNo:F$rowNo")->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                        'fill' => [
                            'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                            'color' => ['argb' => '00D9D9D9']
                        ],
                    ]);
                    $headerRowNo = $rowNo + 1;
                    $event->sheet->getStyle("A$headerRowNo:F$headerRowNo")->applyFromArray([
                        'font' => [
                            'fontFamily' => 'Verdana',
                            'bold' => true
                        ]
                    ]);
                }
                foreach ($this->borderableRowNo as $rowNo) {
                    $event->sheet->getStyle($rowNo)->applyFromArray([
                        'font' => [
                            'size' => 12,
                            'fontFamily' => 'Verdana',
                        ], 'borders' => [
                            'allBorders' => [
                                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            ],
                        ], 'alignment' => [
                            'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                            'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                        ],
                    ]);
                }
            },
        ];
    }
}
