<?php

namespace App\Helpers;

use App\Admin;
use App\Business;
use App\ModelsEpos\EposUser;
use App\ModelsTiffintom\TiffintomUser;
use App\User;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Lcobucci\JWT\Configuration;
use Lcobucci\JWT\Signer\Hmac\Sha256;
use Lcobucci\JWT\Signer\Key\InMemory;
use Lcobucci\JWT\Validation\Constraint\SignedWith;

class Auth
{
    protected static $user = null;
    protected static $role = null;
    protected static $id = null;
    public static $business_id = null;
    public static $restaurant_id = null;
    public static $package_name = null;
    public static function setPackageNmae($package_name)
    {
        Auth::$package_name = $package_name;
    }
    public static function setUser($role, $userId)
    {
        Auth::$role = $role;
        Auth::$id = $userId;
        switch ($role) {

            case 'admin':
                Auth::$user = Admin::where('id', $userId)->first();
                break;

            case 'user':
                Auth::$user = User::where('id', $userId)->first();
                break;

            case 'tiffintom_user':
                Auth::$user = TiffintomUser::where('id', $userId)->first();
                break;

            case 'epos_admin':
                Auth::$user = User::where('id', $userId)->first();

                Auth::setConfig();
                Auth::setUserConfig();
                break;

            case 'epos_user':
                Auth::$user = EposUser::where('id', $userId)->first();
                break;
            default:
                Auth::$user = null;
                break;
        }
    }

    public static function user()
    {
        return Auth::$user;
    }
    public static function admin()
    {
        return Auth::$user;
    }

    public static function guest()
    {
        return Auth::$user == null;
    }

    public static function isAdmin()
    {
        if (Auth::guest()) return false;
        return Auth::$role === 'admin';
    }

    public static function isUser()
    {
        if (Auth::guest()) return false;
        return Auth::$role === 'user';
    }
    public static function isTiffintomUser()
    {
        if (Auth::guest()) return false;
        return Auth::$role === 'tiffintom_user';
    }
    public static function isTiffintomAdmin()
    {
        if (Auth::guest()) return false;
        return Auth::$role === 'tiffintom_admin';
    }
    public static function isEposUser()
    {
        if (Auth::guest()) return false;
        return Auth::$role === 'epos_user';
    }
    public static function isEposAdmin()
    {
        if (Auth::guest()) return false;
        return Auth::$role === 'epos_admin';
    }
    public static function id()
    {
        if (Auth::guest()) return null;
        return Auth::$id;
    }

    public static function role()
    {
        if (Auth::guest()) return null;
        return Auth::$role;
    }
    public static function setConfig()
    {
        $business = Auth::user()->selected_business;
        if (!empty($business)) {
            if (empty(Auth::$business_id)) {
                Auth::$business_id = $business->id;
            }
            $dbName = $business->db_name;
            $business->append('database_server');
            $databaseServer = $business->database_server;
            if (!empty($databaseServer)) {
                Config::set('database.connections.epos_connection', [
                    'driver' => 'mysql',
                    'host' => $databaseServer->local_ip,
                    'database' => $dbName,
                    'username' => $databaseServer->username,
                    'password' => $databaseServer->password,
                    'port' => $databaseServer->port,
                    'charset'   => 'utf8',
                    'collation' => 'utf8_unicode_ci',
                    'strict'    => false,
                    'migrations' => 'migrations'
                ]);
            }
        }
    }
    public static function setUserConfig()
    {
        if (!empty(Auth::$business_id)) {
            $business = Business::find(Auth::$business_id);
            if (empty($business)) {
                Utils::error("Invalid business", 401, 401);
            }
            if ($business->epos_service == 1) {
                $business->append('database_server');
                $dbName = $business->db_name;
                $databaseServer = $business->database_server;
                if (!empty($databaseServer)) {
                    Config::set('database.connections.epos_connection', [
                        'driver' => 'mysql',
                        'host' => $databaseServer->local_ip,
                        'database' => $dbName,
                        'username' => $databaseServer->username,
                        'password' => $databaseServer->password,
                        'port' => $databaseServer->port,
                        'charset'   => 'utf8',
                        'collation' => 'utf8_unicode_ci',
                        'strict'    => false,
                        'migrations' => 'migrations'
                    ]);
                }
            }
        }
    }

    public static function getConfig($key): Configuration
    {
        $config = Configuration::forSymmetricSigner(new Sha256(), InMemory::plainText($key));
        $config->setValidationConstraints(new SignedWith($config->signer(), $config->signingKey()));
        return $config;
    }

    public static function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
        DB::purge('epos_connection');
    }
}
