<?php

namespace App\Helpers;

use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Intervention\Image\Facades\Image;
use phpDocumentor\Reflection\Types\Self_;

class ImageHelper
{
    public static $TYPE_USER = 'user';
    public static $TYPE_PROOF = 'proof';
    public static $TYPE_CATEGORY = 'category';
    public static $TYPE_PRODUCT = 'product';
    public static $TYPE_BRAND = 'brand';
    public static $TYPE_STORE = 'store';
    public static $TYPE_BANNER = 'banner';
    public static $TYPE_BUSINESS = 'business';
    public static $TYPE_BUSINESS_GALLERY = 'gallery';
    public static $TYPE_PAYMENT_METHOD = 'payment-method';
    public static $TYPE_HARDWARE_PRODUCT = 'hardware';
    public static $TYPE_MISC_INVOICE = 'misc-invoice';
    public static $TYPE_CV = 'cv';
    public static $TYPE_EVIDENCE = 'business';
    public static $TYPE_INVOICE_LOGO = 'invoice-logo';

    // Epos Folders
    public static $TYPE_EPOS_CATEGORY = 'epos/category';
    public static $TYPE_EPOS_PRODUCT = 'epos/product';
    public static $TYPE_EPOS_ADDON = 'epos/addon';
    public static $TYPE_EPOS_USER = 'epos/user';
    public static $TYPE_EPOS_ORDER_LOG = 'epos/order-log';

    public static $TYPE_CONNECT = 'connect';
    public static $imageServerUrl = "http://cust.smartmps.co.uk/rms/images/";  //"https://storage.tiffintom.com/ubsidi/";

    // Tiffintom Folders
    public static $TYPE_TIFFINTOM_RESTAURANT = 'storeImages';
    public static $TYPE_TIFFINTOM_PROMOTION_BANNER = 'siteImages/promoBanner';
    public static $TYPE_TIFFINTOM_USER = 'customerImages';
    public static $TYPE_TIFFINTOM_MENU = 'menuimage';
    public static $TYPE_TIFFINTOM_ORDERHELP = 'orderHelp';
    public static $TYPE_TIFFINTOM_CUISINE = 'cuisine';
    public static $TYPE_TIFFINTOM_BECOME = 'Newregistration';
    public static $TYPE_TIFFINTOM_PARTNERMEDIA = 'partnerMedia';
    public static $TYPE_TIFFINTOM_PAYMENT_METHOD = 'payment';
    public static $TYPE_TIFFINTOM_SITE_LOGO = 'siteImages/siteLogo';
    public static $TYPE_TIFFINTOM_SITE_FAVICON = 'siteImages/siteFav';

    public static function saveImage(UploadedFile $file, $type = 'other')
    {
        $filename = $file->hashName();
        $dir = Storage::disk('public')->path("images/$type/");
        $thumbDir = Storage::disk('public')->path("images/$type/thumb/");

        if (!is_dir($dir)) mkdir($dir, 0755, true);
        if (!is_dir($thumbDir)) mkdir($thumbDir, 0755, true);

        // image
        $img = Image::make($file);
        if ($img->getWidth() > 1280) {
            $img->resize(1280, null, function ($constraint) {
                $constraint->aspectRatio();
            });
        }
        $img->save($dir . $filename);
        unlink($file->path());

        // thumbnail
        $thumb = Image::make($dir . $filename);
        $thumb->resize(360, null, function ($constraint) {
            $constraint->aspectRatio();
        });
        $thumb->save($thumbDir . $filename);

        return $filename;
    }

    public static function delete($type, $image)
    {
        $imageFile = Storage::disk('public')->path("images/$type/") . $image;
        $thumbFile = Storage::disk('public')->path("images/$type/thumb/") . $image;

        if (file_exists($imageFile)) unlink($imageFile);
        if (file_exists($thumbFile)) unlink($thumbFile);
    }

    public static function getImageUrl($type, $image)
    {
        if ($image == null) return null;
        return Storage::disk('public')->url("images/$type/$image");
    }

    public static function getThumbUrl($type, $image)
    {
        if ($image == null) return null;
        return Storage::disk('public')->url("images/$type/thumb/$image");
    }

    public static function saveImageToServer(UploadedFile $file, $type = "other", $disk = 'ubsidi')
    {
        $filename = $file->hashName();
        if ($disk == 'ubsidi') {
            $folder_type = "images/$type/";
        } else {
            $folder_type = "$type/";
        }
        $bool = Storage::disk($disk)->put($folder_type . $filename, fopen($file, 'rw+'));
        return $filename;
    }
    public static function getServerImageUrl($type, $image, $path = null)
    {
        if ($image == null) return null;
        self::$imageServerUrl = env('FTP_IMAGE_PATH') . "images/";
        if ($path != null) {
            self::$imageServerUrl = $path;
        }
        return self::$imageServerUrl  . $type . "/" . $image;
    }

    public static function getServerThumbUrl($type, $image, $path = null)
    {
        if ($image == null) return null;
        self::$imageServerUrl = env('FTP_IMAGE_PATH') . "images/";
        if ($path != null) {
            self::$imageServerUrl = $path;
        }
        return self::$imageServerUrl  . $type . "/" . $image;
    }
    public static function deleteServerImage($type, $image, $disk = 'ubsidi')
    {
        if ($disk == 'ubsidi') {
            return Storage::disk($disk)->delete("images/$type/" . $image);
        } else {
            return Storage::disk($disk)->delete("$type/" . $image);
        }
    }
}