<?php

namespace App\Helpers;

use App\Email;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;
use PHPMailer\PHPMailer\Exception;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use App\Jobs\SMTPMailJob;
use GuzzleHttp\Client;

class MailHelper
{
    public static function sendMail($email)
    {
        if ($email['mail_type'] == 'SMTP') {
            // Instantiation and passing `true` enables exceptions
            $mail = new PHPMailer(true);

            $data = [];
            $data['from_address'] = $email['from'] != null ? $email['from'] : env('MAIL_FROM_ADDRESS');
            $data['from_name'] = $email['from'] != null ?  null : env('MAIL_FROM_NAME');
            $data['to_address'] = $email['to'];
            $data['subject'] = $email['subject'];
            $data['body'] = $email['body'];

            $SettingsList = TiffintomSiteSetting::where('id', '1')->first();

            if ($SettingsList) {
                $mailHost = $SettingsList->smtp_host;
                $userName = $SettingsList->smtp_username;
                $password = $SettingsList->smtp_password;
                $port = $SettingsList->smtp_port;
            } else {
                $mailHost = env('MAIL_HOST');
                $userName = env('MAIL_USERNAME');
                $password = env('MAIL_PASSWORD');
                $port = env('MAIL_PORT');
            }

            // try {

            //     // Server settings
            //     $mail->SMTPDebug = SMTP::DEBUG_OFF;
            //     $mail->isSMTP();
            //     $mail->Host       = $mailHost;
            //     $mail->SMTPAuth   = true;
            //     $mail->Username   = $userName;
            //     $mail->Password   = $password;
            //     $mail->SMTPSecure = env('MAIL_ENCRYPTION');
            //     $mail->Port       = $port;

            //     // Recipients
            //     if ($email['from'] != null) {
            //         $mail->setFrom($email['from']);
            //     } else {
            //         $mail->setFrom(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME'));
            //     }
            //     $mail->addAddress($email['to']);

            //     // Content
            //     $mail->isHTML(true);
            //     $mail->Subject = $email['subject'];
            //     $mail->Body    = $email['body'];
            //     $mail->AltBody = $email['body'];
            //     $mail->send();
            //     return 'sent';
            // } catch (Exception $e) {
            //     return 'error';
            // }

            Config::set('mail.default', 'sendmail');
            Config::set('mail.smtp.host', $mailHost);
            Config::set('mail.smtp.port', $port);
            Config::set('mail.smtp.username', $userName);
            Config::set('mail.smtp.password', $password);

            dispatch(new SMTPMailJob($data));

            return 'sent';
        } elseif ($email['mail_type'] == 'AWS-SES') {
            $result = (new MailHelper())->sendSesMail($email);
            return 'sent';
        } else {

            try {
                $headers = "MIME-Version: 1.0" . "\r\n";
                $headers .= "Content-type:text/html;charset=UTF-8" . "\r\n";
                $headers .= 'From: <' . $email['from'] . '>' . "\r\n";
                if (isset($email['header_pdf'])) {
                    $headers .= $email['header_pdf'];
                }
                $result = mail($email['to'], $email['subject'], $email['body'], $headers);
                return 'sent';
            } catch (Exception $e) {
                return 'error';
            }
        }
    }

    public static function send(Email $email)
    {
        switch (env('MAIL_PLUGIN')) {
            case 'phpmailer':
                return self::sendViaPhpMailer($email);
                break;

            default:
                return null;
                break;
        }
    }

    private static function sendViaPhpMailer(Email $email)
    {
        // Instantiation and passing `true` enables exceptions
        $mail = new PHPMailer(true);

        try {
            // Server settings
            $mail->SMTPDebug = SMTP::DEBUG_OFF;
            $mail->isSMTP();
            $mail->Host       = env('MAIL_HOST');
            $mail->SMTPAuth   = true;
            $mail->Username   = env('MAIL_USERNAME');
            $mail->Password   = env('MAIL_PASSWORD');
            $mail->SMTPSecure = env('MAIL_ENCRYPTION');
            $mail->Port       = env('MAIL_PORT');

            // Recipients
            if ($email->from != null) {
                $mail->setFrom($email->from['address'], $email->from['name']);
            } else {
                $mail->setFrom(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME'));
            }
            $mail->addAddress($email->to['address'], $email->to['name']);

            // Content
            $mail->isHTML(true);
            $mail->Subject = $email->subject;
            $mail->Body    = $email->body;
            $mail->AltBody = $email->altBody ?? $email->body;

            $mail->send();
            return 'sent';
        } catch (Exception $e) {
            // Log::error($mail->ErrorInfo);
            return 'error';
        }
    }

    public function sendSesMail($email)
    {
        $apiURL = env('MAIL_API_URL') . 'send-mail';
        try {

            $apiURL = env('MAIL_API_URL') . 'send-mail';
            $postInput = [
                'to_addresses' => [$email['to']],
                'subject' => $email['subject'],
                'from_address' => $email['from'] ?? env('MAIL_FROM_ADDRESS'),
                'from_name' => $email['from'] ?? env('MAIL_FROM_NAME'),
                'body' => $email['body'],
                'body_type' => 'html'
            ];
            
            $client = new Client();
            $response = $client->request('POST', $apiURL, ['form_params' => $postInput]);
            $statusCode = $response->getStatusCode();
            $responseBody = json_decode($response->getBody(), true);

            if ($statusCode == 200 && isset($responseBody['success']) && $responseBody['success'] == true) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            Log::debug($e);
            return true;
        }
    }

    private static function sendViaSwiftMailer($email)
    {
        return null;
    }

    private static function sendViaLaravelMail($email)
    {
        return null;
    }
}
