<?php

namespace App\Http\Controllers;

use App\ActionLog;
use App\Passbook;
use Carbon\Carbon;
use App\Helpers\Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\SiteSetting;
use App\AamarpayTransaction;
use App\Business;
use App\ModelsTiffintom\TiffintomRestaurant;

class AamarPayController extends Controller
{
    /**
     * Aamarpay payment initiate
     * Post /aamarpay/payment
    */
    public function payment(Request $req)
    {
        $this->validate($req, [
            'amount' => 'required',
            'currency' => 'required',
            'restaurant_id' => 'required',
        ]);

        $currency     = $req->input('currency');
        $amount       = $req->input('amount'); //10 taka is the minimum amount for show card option in aamarPay payment gateway
        $payment_mode = SiteSetting::where('key','aamarpay_mode')->value('value');
        $restaurant   = TiffintomRestaurant::where('id', $req->input('restaurant_id'))->first();
        $business     = Business::where('restaurant_id', $req->input('restaurant_id'))->first();

        if($payment_mode == 'test') {
            $tran_id      = "test_".rand(1111111,9999999); //unique transection id for every transection 
            $store_id      = SiteSetting::where('key','aamarpay_sandbox_store_id')->value('value');  
            $signature_key = SiteSetting::where('key','aamarpay_sandbox_signature_key')->value('value');
            $url           = SiteSetting::where('key','aamarpay_sandbox_url')->value('value');
        } else {
            $tran_id      = "live_".rand(1111111,9999999); //unique transection id for every transection 
            $store_id      = SiteSetting::where('key','aamarpay_live_store_id')->value('value');  
            $signature_key = SiteSetting::where('key','aamarpay_live_signature_key')->value('value');
            $url           = SiteSetting::where('key','aamarpay_live_url')->value('value');
        }

        
        if($req->input('success_url')) {
            $success_url = $req->input('success_url');
        } else {
            $success_url = url('aamarpay/success');
        }

        if($req->input('failed_url')) {
            $fail_url = $req->input('failed_url');
        } else {
            $fail_url = url('aamarpay/fail');
        }

        if($business && isset($business->statement_description)) {
            $description = $business->statement_description;
        } else {
            $description = isset($restaurant['stripe_description']) ? $restaurant['stripe_description'] : '';
        }

        $customer = Auth::user();
        $name     = $customer->first_name." ".$customer->last_name;
        $url      = $url."/jsonpost.php";
        $params   = [
            'store_id' => $store_id,
            'tran_id' => $tran_id,
            'success_url' => $success_url,
            'fail_url' => $fail_url,
            'cancel_url' => $fail_url,
            'amount' => $amount,
            'currency' => $currency,
            'cus_name' => $name,
            'signature_key' => $signature_key,
            'cus_email' => $customer->username,
            'cus_phone' => $customer->phone_number,
            'type' => 'json',
            'desc' => $description
        ];

        $response    = $this->curl($url,$params,'POST');        
        $responseObj = json_decode($response);
        if(isset($responseObj->payment_url) && !empty($responseObj->payment_url)) {
            $paymentUrl = $responseObj->payment_url;
            return $response;
        }else{
            return $response;
        }
    }

    /**
     * Aamarpay payment success
     * Post /aamarpay/success
    */
    public function success(Request $request)
    {
        $request_id   = $request->mer_txnid;
        $payment_mode = SiteSetting::where('key','aamarpay_mode')->value('value');

        if($payment_mode == 'test') {
            $tran_id      = "test_".rand(1111111,9999999); //unique transection id for every transection 
            $store_id      = SiteSetting::where('key','aamarpay_sandbox_store_id')->value('value');  
            $signature_key = SiteSetting::where('key','aamarpay_sandbox_signature_key')->value('value');
            $url           = SiteSetting::where('key','aamarpay_sandbox_url')->value('value');
        } else {
            $tran_id      = "live_".rand(1111111,9999999); //unique transection id for every transection 
            $store_id      = SiteSetting::where('key','aamarpay_live_store_id')->value('value');  
            $signature_key = SiteSetting::where('key','aamarpay_live_signature_key')->value('value');
            $url           = SiteSetting::where('key','aamarpay_live_url')->value('value');
        }


        $url      = $url."/api/v1/trxcheck/request.php?request_id=".$request_id."&store_id=".$store_id."&signature_key=".$signature_key."&type=json";
        $response = $this->curl($url,[],'GET');

        $response = json_decode($response,true);


        $transactionData = [
            'txn_id' => ($response['pg_txnid']) ? $response['pg_txnid'] : '',
            'merchant_id' => ($response['merchant_id']) ? $response['merchant_id'] : '',
            'description' => ($response['desc']) ? $response['desc'] : '',
            'amount' => ($response['amount']) ? $response['amount'] : '',
            'currency' => ($response['currency']) ? $response['currency'] : '',
            'currency_merchant' => ($response['currency_merchant']) ? $response['currency_merchant'] : '',
            'status' => ($response['pay_status']) ? $response['pay_status'] : '',
            'type' => ($response['payment_type']) ? $response['payment_type'] : '',
            'card_type' => ($response['payment_type']) ? $response['payment_type'] : '',
            'created' => ($response['date_processed']) ? $response['date_processed'] : '',
            'merchant_card_processing_ratio' => ($response['processing_ratio']) ? $response['processing_ratio'] : '',
            'merchant_card_processing_fee' => ($response['processing_charge']) ? $response['processing_charge'] : '',
        ];

        $business   = Business::where('statement_description', $response['desc'])->first();

        if($business) {
            $transactionData['business_id'] = $business->id;
        }

        AamarpayTransaction::create($transactionData);

        echo "<pre>"; print_r($response);

    }

    /**
     * curl call
    */
    public function curl($url,$params, $method)
    {
        $curl = curl_init();

        if($method == 'POST') {
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => json_encode($params),
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: application/json'
                ),
            ));
        } else {
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'GET',
            ));
        }
        
        $response = curl_exec($curl);
        curl_close($curl);
        return $response;
    }

    /**
     * Aamarpay payment fail
     * Post /aamarpay/fail
    */
    public function fail(Request $request)
    {
        return $request;
    }

    /**
     * Aamarpay payment cancel
     * get /aamarpay/cancel
    */
    public function cancel()
    {
        return 'Canceled';
    }

    public function pay()
    {
        return view('aamarpay');
    }

    public function checkout(Request $req)
    {
        $currency     = 'BDT';
        $amount       = $req->input('amount'); //10 taka is the minimum amount for show card option in aamarPay payment gateway
        $payment_mode = SiteSetting::where('key','aamarpay_mode')->value('value');
        $restaurant   = TiffintomRestaurant::where('id', 9)->first();
        if($payment_mode == 'test') {
            $tran_id      = "test_".rand(1111111,9999999); //unique transection id for every transection 
            $store_id      = SiteSetting::where('key','aamarpay_sandbox_store_id')->value('value');  
            $signature_key = SiteSetting::where('key','aamarpay_sandbox_signature_key')->value('value');
            $url           = SiteSetting::where('key','aamarpay_sandbox_url')->value('value');
        } else {
            $tran_id      = "live_".rand(1111111,9999999); //unique transection id for every transection 
            $store_id      = SiteSetting::where('key','aamarpay_live_store_id')->value('value');  
            $signature_key = SiteSetting::where('key','aamarpay_live_signature_key')->value('value');
            $url           = SiteSetting::where('key','aamarpay_live_url')->value('value');
        }

       // $customer = Auth::user();
        $name     = 'test user';
        $url      = $url."/jsonpost.php";
        $params   = [
            'store_id' => $store_id,
            'tran_id' => $tran_id,
            'success_url' => url('aamarpay/success'),
            'fail_url' => url('aamarpay/fail'),
            'cancel_url' =>  url('aamarpay/cancel'),
            'amount' => $amount,
            'currency' => $currency,
            'cus_name' => $name,
            'signature_key' => $signature_key,
            'cus_email' => 'test@gmail.com',
            'cus_phone' => '12345678',
            'type' => 'json',
            'desc' => isset($restaurant['stripe_description']) ? $restaurant['stripe_description'] : '',
        ];

        $response    = $this->curl($url,$params,'POST');        
        $responseObj = json_decode($response);
        if(isset($responseObj->payment_url) && !empty($responseObj->payment_url)) {
            $paymentUrl = $responseObj->payment_url;
            return redirect($paymentUrl);
        }else{
            return $response;
        }
    }
}
