<?php

namespace App\Http\Controllers;

use App\Business;
use App\Exports\AdminMiscReportExport;
use App\Exports\AdminPayoutReportExport;
use App\Exports\MultiplePayoutReportExport;
use App\Exports\PayoutReportExport;
use App\ActionLog;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsConnect\ExternalAccount;
use App\Passbook;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;

class ActionLogController extends Controller
{
    /**
     * List action-log.
     * GET /api/action-log
     */
    public function index(Request $req)
    {
        $action_log = ActionLog::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $action_log->whereRaw('(prefix LIKE ? OR end_point LIKE ? OR full_path LIKE ? OR method LIKE ? OR ip LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($user_type = $req->query('user_type', false)) {
            $action_log->where('user_type', $user_type);
        }
        if ($user_id = $req->query('user_id', false)) {
            $action_log->where('user_id', $user_id);
        }
        if ($business_id = $req->query('business_id', false)) {
            $action_log->where('business_id', $business_id);
        }
        if ($req->query('nopaginate', false) == '1') {
            $action_log = $action_log->get();
        } else {
            $action_log = $action_log->paginate($req->query('per_page', 20));
        }

        if ($action_log->isEmpty()) Utils::error('No action log found!', 404);

        return ($action_log);
    }

    /**
     * Create ActionLog.
     * POST /api/action-log
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'user_type' => 'string',
            'user_id' => 'required',
            'prefix' => 'string',
            'end_point' => 'string',
            'full_path' => 'string',
            'method' => 'string',
            'request_raw' => 'string',
            'ip' => 'string',
            'device_id' => 'string',
        ]);

        // create ActionLog
        $action_log = new ActionLog;
        $action_log->fill($req->all());
        $action_log->save();

        return $action_log;
    }

    /**
     * Show ActionLog.
     * GET /api/action-log/{id}
     */
    public function show(Request $req, $id)
    {
        $action_log = ActionLog::find($id);
        if (empty($action_log)) Utils::error('action log not found!', 404);
        $action_log->append('admin');
        return $action_log;
    }


    /**
     * Update ActionLog.
     * POST /api/action-log/{id}
     */
    public function update(Request $req, $id)
    {
        $action_log = ActionLog::find($id);
        if (empty($action_log)) Utils::error('action log not found!', 404);

        $this->validate($req, [
            'user_type' => 'string',
            'user_id' => 'required',
            'prefix' => 'string',
            'end_point' => 'string',
            'full_path' => 'string',
            'method' => 'string',
            'request_raw' => 'string',
            'ip' => 'string',
            'device_id' => 'string',
        ]);

        // update ActionLog
        if ($req->has('user_id')) $action_log->user_id = $req->input('user_id');
        if ($req->has('prefix')) $action_log->prefix = $req->input('prefix');
        if ($req->has('end_point')) $action_log->end_point = $req->input('end_point');
        if ($req->has('full_path')) $action_log->full_path = $req->input('full_path');
        if ($req->has('method')) $action_log->method = $req->input('method');
        if ($req->has('request_raw')) $action_log->request_raw = $req->input('request_raw');

        $action_log->save();

        return $action_log;
    }


    /**
     * Delete ActionLog by id.
     * DELETE /api/action-log/{id}
     */
    public function delete(Request $req, $id)
    {
        $action_log = ActionLog::find($id);
        if (empty($action_log)) Utils::error('action log not found!', 404);
        $action_log->delete();

        return $action_log;
    }

    /**
     * Delete All ActionLog
     * GET All /api/action-log/delete-all/
     */
    public function delete_previous(Request $req)
    {
        $to_date = Carbon::today()->subDays(90)->startOfDay();
        ActionLog::where('created', '<=', $to_date)->delete();
        return ['message' => 'action logs rejected done!'];
    }


    /**
     * Delete ActionLog by id.
     * DELETE /api/action-log/bulk-delete
     */
    public function bulk_delete(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);

        $action_logs = ActionLog::whereIn('id', $req->input('ids'))->delete();

        return $req;
    }
}
