<?php

namespace App\Http\Controllers;

use App\Helpers\Auth;
use Lcobucci\JWT\Builder;
use Lcobucci\JWT\Signer\Hmac\Sha256;
use Lcobucci\JWT\Token;
use Illuminate\Support\Facades\Crypt;
use App\Admin;
use App\Helpers\OtpHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\SecureIp;
use DateTimeImmutable;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Lcobucci\JWT\Configuration;
use Lcobucci\JWT\Signer\Key\InMemory;
use Lcobucci\JWT\Validation\Constraint\SignedWith;

class AdminController extends Controller
{
    public function index(Request $req)
    {
        $admins = Admin::latest();
        $loginAdmin = Auth::admin();
        if ($query = $req->query('query', false)) {
            $admins->whereRaw('(name LIKE ? OR email LIKE ? OR mobile_number LIKE ? OR username LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($loginAdmin->role == 'super_admin') {
            $query = $req->query('disabled');
            if ($query != null) {
                $admins->where('disabled', $query == 0 ? '0' : '1');
            }
            if ($req->query('role', false)) {
                $admins->where('role', $query);
            }
        } else {
            $admins->where('role', '!=', 'super_admin');
            $admins->where('disabled', 0);
        }
        if ($req->query('nopaginate', false) == '1') {
            $admins = $admins->get();
        } else {
            $admins = $admins->paginate($req->query('per_page', 20));
        }

        if ($admins->isEmpty()) Utils::error('No admins found!', 404);
        return $admins;
    }
    /**
     * Create admin.
     * GET /api/admins/
     */
    public function create(Request $req)
    {
        if (!Auth::admin()->superman) {
            Utils::error('You cannot create admin', 400);
        }
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'username' => 'required|string|max:255|unique:admins,username',
            'role' => 'required',
            'password' => 'required|string|min:6',
            'permissions' => 'required',
        ]);

        // create admin
        $admin = new Admin;
        $admin->fill($req->all());
        $admin->password = Hash::make($admin->password);
        $admin->save();
        return $admin;
    }
    /**
     * Update admin.
     * POST /api/admins/{id}
     */
    public function update(Request $req, $id = 'me')
    {
        if ($id == 'me') $admin = Auth::admin();
        else $admin = Admin::find($id);

        if (empty($admin)) Utils::error('Admin not found!', 404);
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'name' => 'string|max:255',
            'username' => ['string', 'max:255', Rule::unique('admins')->ignore($admin->id)],
            'password' => 'string|min:6',
            'role' => 'required',
            'permissions' => 'required',
            'superman' => 'boolean'
        ]);

        // update admin
        if ($req->has('name')) $admin->name = $req->input('name');
        if ($req->has('username')) $admin->username = $req->input('username');
        if ($req->has('email')) $admin->email = $req->input('email');
        if ($req->has('mobile_number ')) $admin->mobile_number  = $req->input('mobile_number ');
        if ($req->has('role')) $admin->role = $req->input('role');
        if ($req->has('permissions')) $admin->permissions = $req->input('permissions');
        $admin->superman = $req->input('superman');
        if (Auth::admin()->superman)
            if ($req->has('password')) $admin->password = Hash::make($req->input('password'));


        $admin->save();

        return $admin;
    }
    /**
     * Show admin.
     * GET /api/admins/{id}
     */
    public function show(Request $req, $id = 'me')
    {
        if ($id == 'me') $admin = Auth::admin();
        else $admin = Admin::find($id);

        if (empty($admin)) Utils::error('Admin not found!', 404);
        if ($id == 'me')
            if ($admin->disabled == 1) Utils::error('Your account is disabled', 401, 401);

        // if (isset($_SERVER['HTTP_CLIENT_IP']))
        //     $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
        // else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
        //     $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
        // else if (isset($_SERVER['HTTP_X_FORWARDED']))
        //     $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
        // else if (isset($_SERVER['HTTP_FORWARDED_FOR']))
        //     $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
        // else if (isset($_SERVER['HTTP_FORWARDED']))
        //     $ipaddress = $_SERVER['HTTP_FORWARDED'];
        // else if (isset($_SERVER['REMOTE_ADDR']))
        //     $ipaddress = $_SERVER['REMOTE_ADDR'];
        // else
        //     $ipaddress = '';

        // $secureIps = SecureIp::where('ip', $ipaddress)->where('disabled', 0)->first();
        // $admin->ip = $ipaddress;
        // if (!empty($secureIps)) {
        //     $admin->secure = 'allow';
        // } else {
        //     $admin->secure = 'block';
        // }

        return $admin;
    }

    /**
     * Login admin.
     * POST /api/admins/login
     */
    public function login(Request $req)
    {
        $this->validate($req, [
            'username' => 'required|string|max:255|exists:admins',
            'password' => 'required|string|min:6',
        ]);

        $admin = Admin::where('username', $req->input('username'))->first();

        if ($admin->disabled == 1) Utils::error('Your account is disabled', 401, 401);
        if (!Hash::check($req->input('password'), $admin->password)) {
            return Utils::error('Invalid password.', 400);
        } else {

            $login_method = SiteSettingController::findByKey('admin_login_method')->value;
            if ($login_method == 'otp') {
                if ($admin->mobile_number == null || $admin->mobile_number == '') {
                    return Utils::error('Mobile number not found', 400, 400);
                }
                $otp = rand(100000, 999999);
                $message = 'Your Ubsidi OTP is ' . $otp . '.Note:Please do not share this otp anyone.';

                (new OtpHelper())->sendMessage($message, $admin->mobile_number, 'admin_login');
                $admin->login_otp = $otp;
                $admin->save();
            } elseif ($login_method == 'authenticator') {
                if (!$admin->email) Utils::error('email not found', 400, 400);
                if (!$admin->logged_in) {
                    $g = new \Google\Authenticator\GoogleAuthenticator();
                    $secret = $g->generateSecret();
                    $admin->auth_secret = $secret;
                    $admin->save();
                    $admin->image = $g->getURL($admin->username, $admin->email, $secret);
                }
            } elseif ($login_method == null || $login_method == 'none') {
                $tokens = $this->getNewTokens($admin->id);

                return response()->json($admin, 200, [
                    'x-token' => (string) $tokens['aToken'],
                    'x-refresh-token' => (string) $tokens['rToken'],
                ]);
            }
            $admin->login_method = $login_method;
            return $admin;
        }
    }

    /**
     * Confirm admin login otp.
     * POST /api/admins/confirm-otp
     */
    public function confirm_otp(Request $req)
    {
        $this->validate($req, [
            'username' => 'required|string|max:255|exists:admins',
            'login_otp' => 'required|string|digits:6',
        ]);

        $admin = Admin::where('username', $req->input('username'))->first();

        if ($admin->disabled == 0) {
            $login_method = SiteSettingController::findByKey('admin_login_method')->value;
            if ($login_method == 'otp') {
                if ($admin->login_otp != $req->input('login_otp')) {
                    return Utils::error('Please enter valid otp.', 400);
                } else {
                    $admin->login_otp = null;
                }
            } elseif ($login_method == 'authenticator') {
                $g = new \Google\Authenticator\GoogleAuthenticator();
                $secret = $admin->auth_secret;
                if ($g->checkCode($secret, $req->input('login_otp'))) {
                    $admin->logged_in = 1;
                } else {
                    return Utils::error('Please enter valid code.', 400);
                }
            }
            $admin->save();

            $tokens = $this->getNewTokens($admin->id);

            return response()->json($admin, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Update admin password.
     * POST /api/admins/me/password
     */
    public function password(Request $req)
    {
        $admin = Auth::user();
        if (empty($admin)) Utils::error('Admin not found!', 404);

        $this->validate($req, [
            'password' => 'required|string|min:6',
            'new_password' => 'required|string|min:6',
        ]);


        if (!Hash::check($req->input('password'), $admin->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $admin->password = Hash::make($req->input('new_password'));
            $admin->save();

            $tokens = $this->getNewTokens($admin->id);
            return response()->json($admin, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Enable admin.
     * POST /api/admins/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $admin = Admin::find($id);
        if (empty($admin)) Utils::error('Admin not found!', 404);
        $admin->disabled = 0;
        $admin->save();
        return $admin;
    }

    /**
     * Disable admin.
     * POST /api/admins/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $admin = Admin::find($id);
        if (empty($admin)) Utils::error('Admin not found!', 404);
        $admin->disabled = 1;
        $admin->save();
        return $admin;
    }
    /**
     * Delete admin.
     * DELETE /api/admins/{id}/
     */
    public function delete(Request $req, $id)
    {
        if ($id == Auth::id()) Utils::error("Invalid action", 400);
        $admin = Admin::find($id);
        if (empty($admin)) Utils::error('Admin not found!', 404);
        $admin->delete();
        return $admin;
    }
    /**
     * Token helper to get new tokens.
     */
    public static function getNewTokens($adminId)
    {
        $admin = Admin::find($adminId);
        if (empty($admin)) return null;
        if ($admin->disabled == 1) Utils::error('Your account is disabled', 401, 401);

        $now   = new DateTimeImmutable();
        $aTokenExp = $now->modify('+1 minute');
        $rTokenExp = (new DateTimeImmutable())->modify('+19 hours');

        $aTokenConfig = Auth::getConfig(env('X_TOKEN_KEY'));
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $admin->password);
        $aBuilder = $aTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($aTokenExp)
            ->withClaim('id', $admin->id)
            ->withClaim('role', 'admin');
        $aToken = $aBuilder->getToken($aTokenConfig->signer(), $aTokenConfig->signingKey())->toString();


        $rBuilder = $rTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($rTokenExp)
            ->withClaim('id', $admin->id)
            ->withClaim('role', 'admin');
        $rToken = $rBuilder->getToken($rTokenConfig->signer(), $rTokenConfig->signingKey())->toString();
        return array('aToken' => Crypt::encrypt((string) $aToken), 'rToken' => Crypt::encrypt((string) $rToken));
    }


    /**
     * Token helper to get verify tokens.
     */
    public static function verifyRefreshToken(Token $rToken)
    {
        $admin = Admin::find($rToken->claims()->get('id'));
        if (empty($admin)) return false;
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $admin->password);
        $constraints = $rTokenConfig->validationConstraints();
        $rTokenConfig->validator()->assert($rToken, ...$constraints);
        if (!$rTokenConfig->validator()->validate($rToken, ...$constraints)) {
            return false;
        } else {
            return true;
        }
    }
}
