<?php

namespace App\Http\Controllers;

use App\Business;
use App\BusinessCardReader;
use App\BusinessCommission;
use App\BusinessPrinter;
use App\CardBrand;
use App\CardBrandCommission;
use App\CardBrandCommissionHistory;
use App\Country;
use App\CountryCommission;
use App\CountryCommissionHistory;
use App\DeviceRegistration;
use App\Exports\PayoutReportExport;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\Helpers\Utils;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\OnlineOrder;
use App\PayoutInvoice;
use App\PaypalPayment;
use App\ResellerCommission;
use App\ResellerStripeCommission;
use App\StripeTransaction;
use App\User;
use App\UserBusiness;
use Carbon\Carbon;
use DateInterval;
use DatePeriod;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;
use PayPal\Api\Amount;
use PayPal\Api\Payment;
use PayPal\Api\Refund;
use PayPal\Api\Sale;
use PayPal\Auth\OAuthTokenCredential;
use PayPal\Exception\PayPalConnectionException;
use PayPal\Rest\ApiContext;
use Stripe\StripeClient;
use App\CashTransaction;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Collection;
use Illuminate\Pagination\LengthAwarePaginator;

class AppMerchantController extends Controller
{
    public function login(Request $req)
    {
        $this->validate($req, [
            'username' => 'required',
            'password' => 'required',
        ]);
        $user = User::where('login_email', $req->input('username'))->first();
        if (empty($user)) Utils::error("Invalid login details", 400);

        // if ($user->reseller) {
        //     Utils::error("Invalid merchant credentials", 400);
        // }
        if (!Hash::check($req->input('password'), $user->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $user->makeHidden(['business_counts', 'balance']);
            $tokens = (new UserController())->getNewTokens($user->id, 'epos_admin');
            $userBusinesses = Business::whereRaw('(merchant_service = ? OR online_service=? OR issuing_service=? OR pouch_service=?) AND (id IN (SELECT business_id from user_businesses where user_id = ?))', ['1', '1', '1', '1', $user->id])->orderByRaw('name ASC')->get();
            $user->businesses = $userBusinesses;
            $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;
            if (!empty($user->businesses)) {
                foreach ($user->businesses as $business) {
                    if ($business->s_account_id != null) {
                        if ($connectMode == 'test') {
                            $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                            $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
                        }
                        if ($connectMode == 'live') {
                            $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                            $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
                        }
                    }

                    if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {

                        $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
                        $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
                    }
                    if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {

                        $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
                        $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
                    }
                    $business->makeHidden(['business_commissions', 'balance', 'user_counts']);
                    $business->append('card_readers');
                    $business->append('printers');
                    $business->append('admin_permissions');
                    $business->append('supervisor_permissions');
                }
            }
            $user->append('selected_business');
            return response()->json($user, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }


    /**
     * List merchant businesses
     * GET merchant/businesses
     */
    public function businesses_index(Request $req)
    {
        $user = Auth::user();
        // if ($user->reseller)  Utils::error("You are not allowed to access", 400);
        if (empty($user))  Utils::error("You are not allowed to access", 400);
        $user->makeHidden('business_counts');
        $userBusinesses = Business::whereRaw('(merchant_service = ? OR online_service=? OR issuing_service=? OR pouch_service=?) AND (id IN (SELECT business_id from user_businesses where user_id = ?))', ['1', '1', '1', '1', $user->id])->orderByRaw('name ASC')->get();
        if (empty($userBusinesses)) {
            Utils::error("No businesses found", 400);
        }
        $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;

        foreach ($userBusinesses as $business) {
            if ($business->s_account_id != null) {
                if ($connectMode == 'test') {
                    $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                    $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
                }
                if ($connectMode == 'live') {
                    $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                    $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
                }
            }

            if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {

                $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
                $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
            }
            if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {

                $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
                $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
            }
            $business->makeHidden(['business_commissions', 'balance', 'user_counts']);
            $business->append('card_readers');
            $business->append('admin_permissions');
            $business->append('default_bank');
            $business->append('supervisor_permissions');
            $business->append('printers');
        }
        return $userBusinesses;
    }

    /**
     * Show merchant business by id
     * GET merchant/businesses/{business_id}
     */
    public function businesses_show(Request $req, $business_id)
    {
        $user = Auth::user();
        // if ($user->reseller) Utils::error("You are not allowed to access", 400);
        $business = Business::whereRaw('(merchant_service = ? OR online_service=? OR issuing_service=? OR pouch_service=?) AND (id IN (SELECT business_id from user_businesses where user_id = ?))', ['1', '1', '1', '1', $user->id])->orderByRaw('name ASC')->where('id', $business_id)->first();

        if (empty($business)) {
            Utils::error("No business found", 400);
        }
        $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;

        if ($business->s_account_id != null) {
            if ($connectMode == 'test') {
                $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
            }
            if ($connectMode == 'live') {
                $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
            }
        }
        if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {

            $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
            $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
        }
        if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {

            $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
            $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
        }
        $business->append('admin_permissions');
        $business->append('supervisor_permissions');
        $business->append('default_bank');
        // $business->makeHidden(['business_commissions']);
        $business->append('printers');
        $business->append('card_readers');
        return $business;
    }

    /**
     * Add and validate device
     * POST merchant/businesses/{business_id}/device-registrations
     */
    public function device_registrations(Request $req, $business_id)
    {
        $user = Auth::user();
        // if ($user->reseller) Utils::error("Invalid merchant credentials", 400);
        $req->merge(['business_id' => $business_id]);
        $this->validate($req, [
            'device_id' => 'required|string',
            'device_type' => 'required|in:Android,iOS',
            'business_id' => 'required|exists:businesses,id'
        ]);

        $dbDevice = DeviceRegistration::where('device_id', $req->input('device_id'))->where('business_id', $business_id)->where('service_type', 'Merchant')->orderBy('created_at', 'DESC')->first();
        if (empty($dbDevice)) {
            $dbDevice = new DeviceRegistration;
            $dbDevice->fill($req->all());
            $dbDevice->business_id = $business_id;
            if ($business_id == '9') $dbDevice->status = 'Approved';
            $dbDevice->service_type = 'Merchant';
            $dbDevice->save();
        }
        return $dbDevice;
    }

    /**
     * validate device.
     * GET merchant/businesses/{business_id}/device-registrations/validate/{device_id}
     */
    public function device_validate(Request $req, $business_id, $device_id)
    {
        $device = DeviceRegistration::where('device_id', $device_id)->where('business_id', $business_id)->where('service_type', 'Merchant')->orderBy('created_at', 'DESC')->first();
        if (empty($device)) Utils::error('Merchant Device not found!', 404);
        return $device;
    }

    /**
     * List Business Statements
     * GET merchant/businesees/{business_id}/statements
     */
    public function statements(Request $req, $business_id)
    {

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        $payoutInvoices = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->orderByRaw('to_date DESC, payout_invoices.created_at DESC');

        $business = Business::find(Auth::$business_id);
        if (empty($business)) $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $payoutInvoices->where('business_id', $business->id);

        if ($business->statement_transaction_view != 'Both') {
            $payoutInvoices->where('transaction_type', $business->statement_transaction_view);
        }
        if ($business->statement_transaction_view == 'None') {
            Utils::error('No payoutInvoices found!', 404);
        }
        $payoutInvoices->where('business_id', $business->id);

        if ($query = $req->query('from_date', false)) {
            $payoutInvoices->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $payoutInvoices->where('to_date', '<=', $query);
        }

        $carbon = Carbon::today()->subDays(15);
        $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
        $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
        $payoutInvoices->where('from_date', '<=', $from_date);
        $payoutInvoices->where('to_date', '<=', $to_date);

        if ($req->query('nopaginate', false) == '1') {
            $payoutInvoices = $payoutInvoices->get();
        } else {
            $payoutInvoices = $payoutInvoices->paginate($req->query('per_page', 20));
        }

        if ($payoutInvoices->isEmpty()) Utils::error('No payoutInvoices found!', 404);

        return ($payoutInvoices);
    }
    /**
     * Download Business Statements
     * GET merchant/businesees/{business_id}/statements/{statement_id}/download
     */
    public function statement_download(Request $req, $statement_id)
    {
        $payoutInvoice = PayoutInvoice::select("payout_invoices.*", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->find($statement_id);
        if (empty($payoutInvoice)) Utils::error('PayoutInvoice not found!', 404);
        $payoutInvoice->append('transactions');
        $payoutInvoice->append('business');
        $type = 'excel';
        if ($format = $req->query('format', false)) {
            $type = $format;
        }
        $fromDate = Carbon::parse($payoutInvoice['from_date']);
        $toDate = Carbon::parse($payoutInvoice['to_date']);
        $fileName = $payoutInvoice['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');

        return Excel::download(new PayoutReportExport($payoutInvoice), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
    }

    /**
     * Show merchant business report summary
     * GET merchant/businesses/{business_id}/report-summary
     */
    public function report_summary(Request $req, $business_id)
    {
        $this->validate($req, [
            'from_date' => 'date',
            'to_date' => 'date',
            'dynamic' => 'boolean'
        ]);
        $user = Auth::user();
        // if (Auth::role() == 'epos_admin' && $user->reseller) Utils::error("Invalid merchant credentials", 400);

        $business = Business::where('id', Auth::$business_id)->first();
        if (empty($business)) {
            Utils::error("No business found", 400);
        }

        $dates = [];
        if ($req->input('dynamic', 0)) {
            //Today
            $dates[] = [
                'title' => 'Today',
                'filter' => 'today',
                'from_date' => Carbon::today()->startOfDay()->format('Y-m-d H:i:s'),
                'to_date' => Carbon::today()->endOfDay()->format('Y-m-d H:i:s'),
            ];
            //Yesterday
            $dates[] = [
                'title' => 'Yesterday',
                'filter' => 'yesterday',
                'from_date' => Carbon::today()->subDay(1)->startOfDay()->format('Y-m-d H:i:s'),
                'to_date' => Carbon::today()->subDay(1)->endOfDay()->format('Y-m-d H:i:s'),
            ];
            //Last 3 days
            $dates[] = [
                'title' => 'Last 3 Days',
                'filter' => 'last_3_days',
                'from_date' => Carbon::today()->subDay(3)->startOfDay()->format('Y-m-d H:i:s'),
                'to_date' => Carbon::today()->endOfDay()->format('Y-m-d H:i:s'),
            ];
            $report = [];
            for ($d = 0; $d < sizeof($dates); $d++) {
                $from_date = strtotime($dates[$d]['from_date']);
                $to_date = strtotime($dates[$d]['to_date']);
                $paypal_from_date = $dates[$d]['from_date'];
                $paypal_to_date = $dates[$d]['to_date'];

                $totalTransactions = StripeTransaction::where('business_id', $business->id)->where('archived', '0')->whereIn('reporting_category', ['charge', 'refund'])->where('created', '>=', $from_date)->where('created', '<=', $to_date)->count();
                $totalRefundedTransactions = StripeTransaction::where('business_id', $business->id)->where('archived', '0')->where('reporting_category', 'refund')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->count();
                $totalRefundAmount = StripeTransaction::where('business_id', $business->id)->where('archived', '0')->where('reporting_category', 'refund')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->sum('amount');
                $totalSuccessFulTransactions = StripeTransaction::where('business_id', $business->id)->where('archived', '0')->where('reporting_category', 'charge')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->count();
                $totalSuccessAmount = StripeTransaction::where('business_id', $business->id)->where('archived', '0')->where('reporting_category', 'charge')->where('refunded', '0')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->sum('amount');

                $totalPaypalTransactions = PaypalPayment::where('business_id', $business->id)->where('archived', '0')->whereIn('sale_state', ['completed', 'partially_refunded', 'refunded'])->where('created_at', '>=', "$paypal_from_date")->where('created_at', '<=', "$paypal_to_date")->count();
                $totalPaypalRefundedTransactions = PaypalPayment::where('business_id', $business->id)->where('archived', '0')->where('sale_state', '=', 'refunded')->where('created_at', '>=', "$paypal_from_date")->where('created_at', '<=', "$paypal_to_date")->count();
                $totalPaypalRefundAmount = PaypalPayment::where('business_id', $business->id)->where('archived', '0')->where('sale_state', '=', 'refunded')->where('created_at', '>=', "$paypal_from_date")->where('created_at', '<=', "$paypal_to_date")->sum('refunded_amount');
                $totalPaypalSuccessFulTransactions = PaypalPayment::where('business_id', $business->id)->where('archived', '0')->whereIn('sale_state', ['completed', 'partially_refunded'])->where('created_at', '>=', "$paypal_from_date")->where('created_at', '<=', "$paypal_to_date")->count();
                $totalPaypalSuccessAmount = PaypalPayment::where('business_id', $business->id)->where('archived', '0')->whereIn('sale_state', ['completed', 'partially_refunded'])->where('created_at', '>=', "$paypal_from_date")->where('created_at', '<=', "$paypal_to_date")->sum('transaction_amount');

                $data = [
                    'total_transactions' => $totalTransactions + $totalPaypalTransactions,
                    'total_refund_transactions' => $totalRefundedTransactions + $totalPaypalRefundedTransactions,
                    'total_refund_amount' => $this->convertToDecimal(($totalRefundAmount / 100) + $totalPaypalRefundAmount),
                    'total_successful_transactions' => $totalSuccessFulTransactions + $totalPaypalSuccessFulTransactions,
                    'total_successful_amount' => $this->convertToDecimal(($totalSuccessAmount / 100) + $totalPaypalSuccessAmount),
                ];
                $report[] = [
                    'title' => $dates[$d]['title'],
                    'filter' => $dates[$d]['filter'],
                    'from_date' => $dates[$d]['from_date'],
                    'to_date' => $dates[$d]['to_date'],
                    'report' => $data
                ];
            }

            return $report;
        }

        $from_date = Carbon::today()->subDays(3)->startOfDay();
        $to_date = Carbon::today()->endOfDay();

        if ($req->has('from_date')) {
            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
        }
        if ($req->has('to_date')) {
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
        }
        $from_date = strtotime($from_date);
        $to_date = strtotime($to_date);

        $totalTransactions = StripeTransaction::where('business_id', $business->id)->where('created', '>=', $from_date)->where('created', '<=', $to_date)->count();
        $totalRefundedTransactions = StripeTransaction::where('business_id', $business->id)->where('reporting_category', 'refund')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->count();
        $totalRefundAmount = StripeTransaction::where('business_id', $business->id)->where('reporting_category', 'refund')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->sum('amount');
        $totalSuccessFulTransactions = StripeTransaction::where('business_id', $business->id)->where('reporting_category', 'charge')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->count();
        $totalSuccessAmount = StripeTransaction::where('business_id', $business->id)->where('reporting_category', 'charge')->where('refunded', '0')->where('created', '>=', $from_date)->where('created', '<=', $to_date)->sum('amount');

        $data = [
            'total_transactions' => $totalTransactions,
            'total_refund_transactions' => $totalRefundedTransactions,
            'total_refund_amount' => (float)$totalRefundAmount,
            'total_successful_transactions' => $totalSuccessFulTransactions,
            'total_successful_amount' => (float)$totalSuccessAmount,
        ];
        return $data;
    }

    /**
     * Show merchant business earning summary
     * GET merchant/businesses/{business_id}/earning-summary
     */
    public function earning_summary(Request $req, $business_id)
    {
        $this->validate($req, [
            'from_date' => 'date',
            'to_date' => 'date',
        ]);

        $user = Auth::user();
        $business = Business::where('id', Auth::$business_id)->first();
        if (empty($business)) $business = Business::where('id', $business_id)->first();
        if (empty($business)) {
            Utils::error("No business found", 400);
        }

        if ($req->input('day_type') == 'today') {
            $from_date = Carbon::today()->startOfDay();
            $to_date = Carbon::today()->endOfDay();
            // Previous Date
            $previous_from_date = $from_date->subDays(6)->startOfDay();
            $previous_to_date = $to_date->subDays(6)->endOfDay();
        } else if ($req->input('day_type') == 'yesterday') {
            $from_date = Carbon::today()->subDays(1)->startOfDay();
            $to_date = Carbon::today()->endOfDay();
            // Previous Date
            $previous_from_date = $from_date->subDays(6)->startOfDay();
            $previous_to_date = $to_date->subDays(6)->endOfDay();
        } else if ($req->input('day_type') == 'last_week') {
            $from_date = Carbon::today()->startOfWeek();
            $to_date = Carbon::today()->endOfWeek();
            // Previous Date
            $previous_from_date = Carbon::today()->startOfWeek()->subWeek();
            $previous_to_date = Carbon::today()->endOfWeek()->subWeek();
        } else if ($req->input('day_type') == 'last_month') {
            $from_date = Carbon::today()->firstOfMonth();
            $to_date = Carbon::today()->endOfMonth();
            // Previous Date
            $previous_from_date = Carbon::today()->startOfMonth()->subMonth();
            $previous_to_date = Carbon::today()->endOfMonth()->subMonth();
        } else if ($req->input('day_type') == 'last_year') {
            $from_date = Carbon::today()->firstOfYear();
            $to_date = Carbon::today()->endOfYear();
            // Previous Date
            $previous_from_date = Carbon::today()->firstOfYear()->subYear();
            $previous_to_date = Carbon::today()->endOfYear()->subYear();
        } else {
            $from_date = Carbon::today()->subDays(3)->startOfDay();
            $to_date = Carbon::today()->endOfDay();
        }

        $refund_from_date = strtotime($from_date);
        $refund_to_date = strtotime($to_date);
        // Previous Date
        $previous_refund_from_date = strtotime($previous_from_date);
        $previous_refund_to_date = strtotime($previous_to_date);

        $grossTransactions = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->sum('gross_total');
        $netFeeTransactions = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->sum('gross_fee');
        $totalRefundAmount = StripeTransaction::where('business_id', $business->id)->where('reporting_category', 'refund')->where('created', '>=', $refund_from_date)->where('created', '<=', $refund_to_date)->sum('amount');
        $totalTransactions = PayoutInvoice::where('business_id', $business->id);
        $totalTransactions->where('from_date', '>=', $from_date);
        $totalTransactions->where('to_date', '<=', $to_date);
        $totalTransactions = $totalTransactions->get();
        $totalTransactions->makeHidden(['payout_invoice_raw']);

        // Previous transactions
        $previousgrossTransactions = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->sum('gross_total');
        $previousnetFeeTransactions = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->sum('gross_fee');
        $previoustotalRefundAmount = StripeTransaction::where('business_id', $business->id)->where('reporting_category', 'refund')->where('created', '>=', $previous_refund_from_date)->where('created', '<=', $previous_refund_to_date)->sum('amount');

        $net_sales = (float)$grossTransactions - (float)$netFeeTransactions;
        $previous_net_sales = ((float)$previousgrossTransactions - (float)$previousnetFeeTransactions) - ((float)$grossTransactions - (float)$netFeeTransactions);

        $refund = (float)$totalRefundAmount / 100;
        $previous_refund = ((float)$previoustotalRefundAmount / 100) - ((float)$totalRefundAmount / 100);

        if ($previousgrossTransactions == 0 || $grossTransactions == 0) {
            $diffGrossSales = $previousgrossTransactions == $grossTransactions ? 0 : -100;
        } else {
            if ($previousgrossTransactions < $grossTransactions) {
                $diffGrossSales = ((float)$grossTransactions * 100) / (float)$previousgrossTransactions;
            } else {
                $diffGrossSales = - ((float)$grossTransactions * 100) / (float)$previousgrossTransactions;
            }
        }

        if ($previous_net_sales == 0 || $net_sales == 0) {
            $diffNetSales = $previous_net_sales == $net_sales ? 0 : -100;
        } else {
            if ($previous_net_sales < $net_sales) {
                $diffNetSales = ((float)$net_sales * 100) / (float)$previous_net_sales;
            } else {
                $diffNetSales = - ((float)$net_sales * 100) / (float)$previous_net_sales;
            }
        }

        if ($previous_refund == 0 || $refund == 0) {
            $diffRefund = $previous_refund == $refund ? 0 : -100;
        } else {
            if ($previous_refund < $refund) {
                $diffRefund = ((float)$refund / 100) / ((float)$previous_refund / 100);
            } else {
                $diffRefund = - ((float)$refund / 100) / ((float)$previous_refund / 100);
            }
        }

        $graphReport = array();
        $previousGraphReport = array();

        if ($req->input('day_type') == 'today') {
            $graphReport = PayoutInvoice::selectRaw("hour(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%hh') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->get();
            $previousGraphReport = PayoutInvoice::selectRaw("hour(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%hh') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->get();
        } else if ($req->input('day_type') == 'yesterday') {
            $graphReport = PayoutInvoice::selectRaw("hour(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%hh') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->get();
            $previousGraphReport = PayoutInvoice::selectRaw("hour(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%hh') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->get();
        } else if ($req->input('day_type') == 'last_week') {
            // $graphReport = PayoutInvoice::selectRaw("DAYNAME(from_date) as schedule_string, DATE_FORMAT(CAST(from_date as DATE),'%d') as schedule, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("DATE_FORMAT(CAST(from_date as DATE),'%d%m')")->where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->get();
            // $previousGraphReport = PayoutInvoice::selectRaw("DAYNAME(from_date) as schedule_string, DATE_FORMAT(CAST(from_date as DATE),'%d') as schedule, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("DATE_FORMAT(CAST(from_date as DATE),'%d%m')")->where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->get();
            $interval = DateInterval::createFromDateString('1 day');
            $period = new DatePeriod($from_date, $interval, $to_date);

            foreach ($period as $day) {
                $from_date = Carbon::parse($day)->format('Y-m-d');
                $to_date = Carbon::parse($day)->format('Y-m-d');
                $grandTotalSum = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->sum('gross_total');

                $graphReports['schedule'] = Carbon::parse($from_date)->format('D');
                $graphReports['schedule_string'] = Carbon::parse($from_date)->format('l');
                $graphReports['revenue'] = $grandTotalSum;
                $graphReport[] = $graphReports;

                $previous_from_date = Carbon::parse($day)->format('Y-m-d');
                $previous_to_date = Carbon::parse($day)->format('Y-m-d');
                $grandPreviousTotalSum = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->sum('gross_total');

                $previousGraphReports['schedule'] = Carbon::parse($previous_from_date)->format('D');
                $previousGraphReports['schedule_string'] = Carbon::parse($previous_from_date)->format('l');
                $previousGraphReports['revenue'] = $grandPreviousTotalSum;
                $previousGraphReport[] = $previousGraphReports;
            }
        } else if ($req->input('day_type') == 'last_month') {
            // $graphReport = PayoutInvoice::selectRaw("WEEK(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%d-m') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->get();
            // $previousGraphReport = PayoutInvoice::selectRaw("WEEK(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%d-m') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->get();

            $weekly[] = [
                'start' => '01',
                "end" => "07",
            ];
            $weekly[] = [
                'start' => '08',
                "end" => "14",
            ];
            $weekly[] = [
                'start' => '15',
                "end" => "21",
            ];
            $weekly[] = [
                'start' => '22',
                "end" => "28",
            ];
            $weekly[] = [
                'start' => '29',
                "end" => "31",
            ];

            foreach ($weekly as $week) {
                $from_date = Carbon::parse($week['start'] . '-' .  date('m', strtotime($from_date)) . '-' . date('Y', strtotime($from_date)))->format('Y-m-d');
                $to_date = Carbon::parse($week['end'] . '-' . date('m', strtotime($from_date)) . '-' . date('Y', strtotime($from_date)))->format('Y-m-d');
                $grandTotalSum = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->sum('gross_total');

                $graphReports['schedule'] = Carbon::parse($from_date)->format('d/m') . ' - ' . Carbon::parse($to_date)->format('d/m');
                $graphReports['schedule_string'] = Carbon::parse($from_date)->format('Y-m-d') . ' to ' . Carbon::parse($to_date)->format('Y-m-d');
                $graphReports['revenue'] = $grandTotalSum;
                $graphReport[] = $graphReports;

                $previous_from_date = Carbon::parse($week['start'] . '-' . date('m', strtotime($previous_from_date)) . '-' . date('Y', strtotime($previous_from_date)))->format('Y-m-d');
                $previous_to_date = Carbon::parse($week['end'] . '-' . date('m', strtotime($previous_to_date)) . '-' . date('Y', strtotime($previous_to_date)))->format('Y-m-d');
                $grandPreviousTotalSum = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->sum('gross_total');

                $previousGraphReports['schedule'] = Carbon::parse($from_date)->format('d/m') . ' - ' . Carbon::parse($to_date)->format('d/m');
                $previousGraphReports['schedule_string'] = Carbon::parse($from_date)->format('Y-m-d') . ' to ' . Carbon::parse($to_date)->format('Y-m-d');
                $previousGraphReports['revenue'] = $grandPreviousTotalSum;
                $previousGraphReport[] = $previousGraphReports;
            }
        } else if ($req->input('day_type') == 'last_year') {
            // $graphReport = PayoutInvoice::selectRaw("DATE_FORMAT(CAST(from_date as DATE),'%M %Y') as schedule_string, DATE_FORMAT(CAST(from_date as DATE),'%m') as schedule, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("DATE_FORMAT(CAST(from_date as DATE),'%Y%m')")->where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->get();
            // $previousGraphReport = PayoutInvoice::selectRaw("DATE_FORMAT(CAST(from_date as DATE),'%M %Y') as schedule_string, DATE_FORMAT(CAST(from_date as DATE),'%m') as schedule, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("DATE_FORMAT(CAST(from_date as DATE),'%Y%m')")->where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->get();
            $months = array('01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12');
            foreach ($months as $month) {
                $from_date = Carbon::parse('01-' . $month . '-' . date('Y'))->format('Y-m-d');
                $to_date = Carbon::parse('31-' . $month . '-' . date('Y'))->format('Y-m-d');
                $grandTotalSum = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->sum('gross_total');

                $graphReports['schedule'] = $month;
                $graphReports['schedule_string'] = date('M', strtotime($from_date)) . ' ' . date('Y');
                $graphReports['revenue'] = $grandTotalSum;
                $graphReport[] = $graphReports;

                $previous_from_date = Carbon::parse('01-' . $month . '-' . date('Y', strtotime($previous_from_date)))->format('Y-m-d');
                $previous_to_date = Carbon::parse('31-' . $month . '-' . date('Y', strtotime($previous_to_date)))->format('Y-m-d');
                $grandPreviousTotalSum = PayoutInvoice::where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->sum('gross_total');

                $previousGraphReports['schedule'] = $month;
                $previousGraphReports['schedule_string'] = date('M', strtotime($previous_from_date)) . ' ' . date('Y', strtotime($previous_from_date));
                $previousGraphReports['revenue'] = $grandPreviousTotalSum;
                $previousGraphReport[] = $previousGraphReports;
            }
        } else {
            $graphReport = PayoutInvoice::selectRaw("WEEK(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%d-m') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $from_date)->where('to_date', '<=', $to_date)->get();
            $previousGraphReport = PayoutInvoice::selectRaw("WEEK(from_date) as schedule, DATE_FORMAT(CAST(from_date as DATE),'%d-m') as schedule_string, ROUND(SUM(gross_total),2) as revenue")->groupByRaw("schedule")->where('business_id', $business->id)->where('from_date', '>=', $previous_from_date)->where('to_date', '<=', $previous_to_date)->get();
        }
        // $graphReport->makeHidden(['country', 'total_paid_amount']);
        // $previousGraphReport->makeHidden(['country', 'total_paid_amount']);

        $data = [
            'gross_sales' => (float)$grossTransactions,
            'net_sales' => $net_sales,
            'total_refund_amount' => $refund,
            'previous_gross_sales' => (float)$previousgrossTransactions - (float)$grossTransactions,
            'previous_net_sales' => $previous_net_sales,
            'previous_total_refund_amount' => $previous_refund,
            'difference_gross_sales' => $diffGrossSales,
            'difference_net_sales' => $diffNetSales,
            'difference_total_refund_amount' => $diffRefund,
            'graph_report' => $graphReport,
            'previous_graph_report' => $previousGraphReport,
            'transactions' => $totalTransactions,
        ];

        return $data;
    }

    /**
     * Show merchant business all transactions by id
     * GET merchant/businesses/{business_id}/all-transactions
     */
    public function businesses_all_transactions(Request $req, $business_id)
    {
        $this->validate($req, [
            'filter' => 'in:archived,refunded'
        ]);
        // $user = Auth::user();
        // if (Auth::role() == 'epos_admin' && $user->reseller) Utils::error("Invalid merchant credentials", 400);
        $business = Business::where('id', $business_id)->first();

        if (empty($business)) {
            Utils::error("No business found", 400);
        }

        $stripeTransactions = StripeTransaction::where('business_id', $business->id)->where('mode', $business->stripe_mode)->orderByRaw('id DESC');
        $paypalTransactions = PaypalPayment::where('business_id', $business->id)->orderByRaw('id DESC');
        // $onlineOrders = array();
        $onlineOrders = OnlineOrder::where('business_id', $business->id)->where('payment_method', 'cod')->where('order_status', '!=', 'Pending')->orderByRaw('id DESC');
        $isRefundedTransactions = false;
        if ($query = $req->query('filter', false)) {
            if ($query == 'archived') {
                $stripeTransactions->where('archived', 1);
                $stripeTransactions->whereIn('reporting_category', ['charge', 'refund']);
                $paypalTransactions->where('archived', '1');
                $paypalTransactions->whereIn('sale_state', ['completed', 'partially_refunded']);
                $onlineOrders->where('archived', 1);
            } else if ($query == 'refunded') {
                $isRefundedTransactions = true;
                $stripeTransactions->where('archived', '0');
                $stripeTransactions->where('reporting_category', 'refund');
                $paypalTransactions->where('archived', '0');
                $paypalTransactions->whereIn('sale_state', ['refunded']);
            } else {
                $stripeTransactions->where('archived', 0);
                $paypalTransactions->where('archived', '0');
                $stripeTransactions->where('reporting_category', $query);
                $onlineOrders->where('archived', 0);
            }
        } else {
            $stripeTransactions->where('archived', 0);
            $stripeTransactions->where('refunded', 0);
            $stripeTransactions->whereIn('reporting_category', ['charge']);

            $paypalTransactions->where('archived', '0');
            $paypalTransactions->whereIn('sale_state', ['completed', 'partially_refunded']);

            $onlineOrders->where('archived', 0);
        }


        // Show only last 3 days transactions
        if ($business->day_of_transaction_limit > 0) {
            $lastDay = Carbon::today()->subDays($business->day_of_transaction_limit)->startOfDay();
        } else {
            $lastDay = Carbon::today()->subDays(3)->startOfDay();
        }

        if ($req->input('from_date') != null && $req->input('to_date') != null) {
            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();

            $stripeTransactions->where('created', '>=', $from_date->getTimestamp());
            $stripeTransactions->where('created', '<=', $to_date->getTimestamp());

            $online_from_date = Carbon::parse($req->input('from_date'))->format('Y-m-d 00:00:00');
            $online_to_date = Carbon::parse($req->input('to_date'))->format('Y-m-d 23:59:59');
            $onlineOrders->where('order_date', '>=', $online_from_date);
            $onlineOrders->where('order_date', '<=', $online_to_date);

            // $paypalTransactions->where('sale_created_at', '>=', $from_date->format("Y-m-d H:i:s"));
            // $paypalTransactions->where('sale_created_at', '<=', $to_date->format("Y-m-d H:i:s"));
        } else {
            $stripeTransactions->where('created', '>', $lastDay->getTimestamp());
            $paypalTransactions->where('sale_created_at', '>', $lastDay->format("Y-m-d H:i:s"));
            $onlineOrders->where('order_date', '>', $lastDay->format("Y-m-d H:i:s"));
        }

        if ($req->query('nopaginate', false) == '1') {
            $stripeTransactions = $stripeTransactions->get();
            $stripeTransactionAmounts = $stripeTransactions->sum('amount');
            $paypalTransactions = $paypalTransactions->get();
            $onlineOrders = $onlineOrders->get();
            $onlineOrderAmounts = $onlineOrders->sum('grand_total');
        } else {
            $stripeTransactionAmounts = $stripeTransactions->sum('amount');
            $stripeTransactionAmounts = $stripeTransactions->sum('amount');
            $stripeTransactions = $stripeTransactions->paginate($req->query('per_page', 20));
            $paypalTransactions = $paypalTransactions->paginate($req->query('per_page', 20));
            $onlineOrderAmounts = $onlineOrders->sum('grand_total');
            $onlineOrders = $onlineOrders->paginate($req->query('per_page', 20));
        }
        $stripeTransactions->each(function ($transaction) {
            $transaction->append('calculated_fees');
        });
        $onlineOrders->each(function ($transaction) {
            $transaction->append('calculated_fees');
        });

        $totalAmount = $this->convertToDecimal(($stripeTransactionAmounts / 100)); // + $onlineOrderAmounts
        $data = [
            'card_transactions' => $stripeTransactions,
            'paypal_transactions' => $paypalTransactions,
            'total_amount' => $totalAmount,
        ];

        if (!$isRefundedTransactions) {
            $data['order_transactions'] = $onlineOrders;
        }

        return $data;
    }

    /**
     * Archive Bulk All Transactions.
     * POST merchant/businesses/{business_id}/all-transactions/archive
     */
    public function businesses_archive_all_transactions(Request $req, $business_id)
    {
        if ($req->has('stripe_ids') && !is_array($req->input('stripe_ids')))
            $req->merge(['stripe_ids' => json_decode($req->input('stripe_ids'))]);
        if ($req->has('paypal_ids') && !is_array($req->input('paypal_ids')))
            $req->merge(['paypal_ids' => json_decode($req->input('paypal_ids'))]);
        if ($req->has('order_ids') && !is_array($req->input('order_ids')))
            $req->merge(['order_ids' => json_decode($req->input('order_ids'))]);

        if (
            (!$req->has('stripe_ids') || empty($req->input('stripe_ids')) || sizeof($req->input('stripe_ids')) == 0) &&
            (!$req->has('paypal_ids') || empty($req->input('paypal_ids')) || sizeof($req->input('paypal_ids')) == 0) &&
            (!$req->has('order_ids') || empty($req->input('order_ids')) || sizeof($req->input('order_ids')) == 0)
        ) {
            Utils::error("No transactions selected", 400);
        }
        $data = [];
        if (!empty($req->input('stripe_ids')) || sizeof($req->input('stripe_ids')) > 0) {
            $transactions = StripeTransaction::whereIn('id', $req->input('stripe_ids'))->update(['archived' => '1']); // where('reporting_category', 'charge')->where('refunded_amount', '=', '0')->
            $stripeTxns = StripeTransaction::whereIn('id', $req->input('stripe_ids'))->get();
            foreach ($stripeTxns as $tran) {
                $this->refundAction($tran, 0);
            }
            $data['stripe_transactions'] = $stripeTxns;
        }
        if (!empty($req->input('paypal_ids')) || sizeof($req->input('paypal_ids')) > 0) {
            $transactions = PaypalPayment::whereIn('id', $req->input('paypal_ids'))->update(['archived' => '1']); // where('sale_state', 'completed')->where('refunded_amount', '=', '0')->
            $paypalTxns = PaypalPayment::whereIn('id', $req->input('paypal_ids'))->get();
            $data['paypal_transactions'] = $paypalTxns;
        }
        if (!empty($req->input('order_ids')) || sizeof($req->input('order_ids')) > 0) {
            $transactions = OnlineOrder::whereIn('id', $req->input('order_ids'))->where('payment_method', 'cod')->update(['archived' => '1']);
            $orderTxns = OnlineOrder::whereIn('id', $req->input('order_ids'))->get();
            $data['order_transactions'] = $orderTxns;
        }
        return $data;
    }

    /**
     * UnArchive Bulk All Transactions.
     * POST merchant/businesses/{business_id}/all-transactions/unarchive
     */
    public function businesses_unarchive_all_transactions(Request $req, $business_id)
    {
        if ($req->has('stripe_ids') && !is_array($req->input('stripe_ids')))
            $req->merge(['stripe_ids' => json_decode($req->input('stripe_ids'))]);
        if ($req->has('paypal_ids') && !is_array($req->input('paypal_ids')))
            $req->merge(['paypal_ids' => json_decode($req->input('paypal_ids'))]);
        if ($req->has('order_ids') && !is_array($req->input('order_ids')))
            $req->merge(['order_ids' => json_decode($req->input('order_ids'))]);

        if (
            (!$req->has('stripe_ids') || empty($req->input('stripe_ids')) || sizeof($req->input('stripe_ids')) == 0) &&
            (!$req->has('paypal_ids') || empty($req->input('paypal_ids')) || sizeof($req->input('paypal_ids')) == 0) &&
            (!$req->has('order_ids') || empty($req->input('order_ids')) || sizeof($req->input('order_ids')) == 0)
        ) {
            Utils::error("No transactions selected", 400);
        }
        $data = [];
        if (!empty($req->input('stripe_ids')) || sizeof($req->input('stripe_ids')) > 0) {
            $transactions = StripeTransaction::whereIn('id', $req->input('stripe_ids'))->update(['archived' => '0']); // where('reporting_category', 'charge')->where('refunded_amount', '=', '0')->
            $stripeTxns = StripeTransaction::whereIn('id', $req->input('stripe_ids'))->get();
            foreach ($stripeTxns as $tran) {
                $this->refundAction($tran, 0);
            }
            $data['stripe_transactions'] = $stripeTxns;
        }
        if (!empty($req->input('paypal_ids')) || sizeof($req->input('paypal_ids')) > 0) {
            $transactions = PaypalPayment::whereIn('id', $req->input('paypal_ids'))->update(['archived' => '0']); // where('sale_state', 'completed')->where('refunded_amount', '=', '0')->
            $paypalTxns = PaypalPayment::whereIn('id', $req->input('paypal_ids'))->get();
            $data['paypal_transactions'] = $paypalTxns;
        }
        if (!empty($req->input('order_ids')) || sizeof($req->input('order_ids')) > 0) {
            $transactions = OnlineOrder::whereIn('id', $req->input('order_ids'))->where('payment_method', 'cod')->update(['archived' => '0']);
            $orderTxns = OnlineOrder::whereIn('id', $req->input('order_ids'))->get();
            $data['order_transactions'] = $orderTxns;
        }
        return $data;
    }

    /**
     * Show merchant business transactions by id
     * GET merchant/businesses/{business_id}/stripe-transactions
     */
    public function businesses_stripe_transactions(Request $req, $business_id)
    {
        $this->validate($req, [
            'filter' => 'in:archived,refunded'
        ]);
        // $user = Auth::user();
        // if (Auth::role() == 'epos_admin' && $user->reseller) Utils::error("Invalid merchant credentials", 400);
        $business = Business::where('id', $business_id)->first();
        if (empty($business)) {
            Utils::error("No business found", 400);
        }

        $stripeTransactions = StripeTransaction::where('business_id', $business->id)->where('mode', $business->stripe_mode)->orderByRaw('id DESC');
        if ($query = $req->query('filter', false)) {
            if ($query == 'archived') {
                $stripeTransactions->where('archived', 1);
                $stripeTransactions->whereIn('reporting_category', ['charge', 'refund']);
            } else if ($query == 'refunded') {
                $stripeTransactions->where('reporting_category', 'refund');
            } else {
                $stripeTransactions->where('archived', 0);
                $stripeTransactions->where('reporting_category', $query);
            }
        } else {
            $stripeTransactions->where('archived', 0);
            $stripeTransactions->where('refunded', 0);
            $stripeTransactions->whereIn('reporting_category', ['charge']);
        }

        if ($service = $req->query('service', false)) {
            $stripeTransactions->where('service', $service);
        }
        if ($query = $req->query('type', false)) {
            $stripeTransactions->where('type', $query);
        }
        if ($query = $req->query('transaction_status', false)) {
            if ($query == 'archived') {
                $stripeTransactions->where('archived', '1');
            }
            if ($query == 'unarchived') {
                $stripeTransactions->where('archived', '0');
            }
        }
        if ($query = $req->query('status', false)) {
            $stripeTransactions->where('reporting_category', $query);
        }
        if ($amount_filter = $req->query('amount_filter_type', false)) {
            if ($req->has('amount')) {
                $amount = $req->query('amount');
                if ($amount_filter == 'greater_than') {
                    $stripeTransactions->where('amount', '>', $amount * 100);
                } else if ($amount_filter == 'less_than') {
                    $stripeTransactions->where('amount', '<', $amount * 100);
                } else if ($amount_filter == 'between') {
                    $amounts = explode(",", $amount);
                    $stripeTransactions->where('amount', '>',  $amounts[0] * 100);
                    $stripeTransactions->where('amount', '<',  $amounts[1] * 100);
                } else if ($amount_filter == 'equal') {
                    $stripeTransactions->where('amount', '=', $amount * 100);
                }
            } else {
                Utils::error("Amount field required with type", 400);
            }
        }
        // if ($date_filter = $req->query('date_filter_type', false)) {
        //     if ($date = $req->query('date', false)) {
        //         if ($date_filter == 'in_last') {
        //             $today = Carbon::today()->endOfDay();
        //             $lastDay = Carbon::today()->subDays($date)->startOfDay();
        //             $stripeTransactions->where('created', '>', $lastDay->getTimestamp());
        //             $stripeTransactions->where('created', '<', $today->getTimestamp());
        //         } else if ($date_filter == 'before') {
        //             $date = Carbon::parse($date)->startOfDay();
        //             $stripeTransactions->where('created', '<', $date->getTimestamp());
        //         } else if ($date_filter == 'after') {
        //             $date = Carbon::parse($date)->endOfDay();
        //             $stripeTransactions->where('created', '>', $date->getTimestamp());
        //         } else if ($date_filter == 'between') {
        //             $date = $req->query('date');
        //             $dates = explode(",", $date);
        //             $dates[0] = Carbon::parse($dates[0])->startOfDay();
        //             $dates[1] = Carbon::parse($dates[1])->endOfDay();
        //             $stripeTransactions->where('created', '>',  $dates[0]->getTimestamp());
        //             $stripeTransactions->where('created', '<',  $dates[1]->getTimestamp());
        //         } else if ($date_filter == 'equal') {
        //             $startTime = Carbon::parse($date)->startOfDay();
        //             $endTime = Carbon::parse($date)->endOfDay();
        //             $stripeTransactions->where('created', '>',  $startTime->getTimestamp());
        //             $stripeTransactions->where('created', '<',  $endTime->getTimestamp());
        //         }
        //     } else {
        //         Utils::error("Date field required with type", 400);
        //     }
        // }

        // $this->validate($req, [
        //     'from_date' => 'date_format:Y-m-d',
        //     'to_date' => 'required_with:from_date|date_format:Y-m-d',
        // ]);

        // Show only last 3 days transactions
        if ($business->day_of_transaction_limit > 0) {
            $lastDay = Carbon::today()->subDays($business->day_of_transaction_limit)->startOfDay();
        } else {
            $lastDay = Carbon::today()->subDays(3)->startOfDay();
        }
        // $lastDay = Carbon::today()->subDays(3)->startOfDay();
        $stripeTransactions->where('created', '>', $lastDay->getTimestamp());

        $sql = $stripeTransactions->toSql();
        $bindings = $stripeTransactions->getBindings();

        $sql_with_bindings = preg_replace_callback('/\?/', function ($match) use ($sql, &$bindings) {
            return "'" . array_shift($bindings) . "'";
        }, $sql);

        if ($req->query('nopaginate', false) == '1') {
            $stripeTransactions = $stripeTransactions->get();
        } else {
            $stripeTransactions = $stripeTransactions->paginate($req->query('per_page', 20));
        }
        $stripeTransactions->each(function ($transaction) {
            $transaction->append('calculated_fees');
        });

        return $stripeTransactions;
    }

    /**
     * Get Archived Transacitons.
     * GET merchant/businesses/{business_id}/stripe-transactions/archive
     */
    public function businesses_archived_transactions(Request $req, $business_id)
    {
        $transactions = StripeTransaction::where('business_id', $business_id)->where('archived', '1')->orderByRaw('id DESC')->get();
        if (empty($transactions)) Utils::error("No transactions found", 400);
        return $transactions;
    }


    /**
     * Get Unarchived Transacitons.
     * GET merchant/businesses/{business_id}/stripe-transactions/unarchive
     */
    public function businesses_unarchived_transactions(Request $req, $business_id)
    {
        $transactions = StripeTransaction::where('business_id', $business_id)->where('archived', '0')->orderByRaw('id DESC')->get();
        if (empty($transactions)) Utils::error("No transactions found", 400);
        return $transactions;
    }

    /**
     * Archive Bulk Transacitons.
     * POST merchant/businesses/{business_id}/stripe-transactions/archive
     */
    public function businesses_archive_stripe_transactions(Request $req, $business_id)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->update(['archived' => '1']); //where('reporting_category', 'charge')->where('refunded_amount', '=', '0')->
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Online')->update(['archived' => '1']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Merchant')->update(['archived' => '1']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Connect')->where('transfer_status', 'pending')->update(['archived' => '1']);
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->get();

            foreach ($transactions as $tran) {
                $this->refundAction($tran, 1);
            }
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }


    /**
     * UnArchive Bulk Transacitons.
     * POST merchant/businesses/{business_id}/stripe-transactions/unarchive
     */
    public function businesses_unarchive_stripe_transactions(Request $req, $business_id)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->update(['archived' => '0']);
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->get();
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Online')->update(['archived' => '0']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Merchant')->update(['archived' => '0']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Connect')->where('transfer_status', 'pending')->update(['archived' => '0']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->get();
            foreach ($transactions as $tran) {
                $this->refundAction($tran, 0);
            }
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }

    public function refundAction($tran, $archive)
    {
        // When refund transaction Archive / Unarchive then same charge Archive / Unarchive
        $account_type = $tran->account_type;
        $stripeMode = "test";
        $stripeMode = $tran->mode;
        $stripeKey = "";

        if ($stripeMode == 'test') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        if ($account_type == 'normal') {
            $business = Business::find($tran->business_id);
            if (!empty($business)) {
                $mode = $tran->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);

        // Archive Charge Transaction 
        if ($tran->refunded == 1) { // && $tran->refunded_amount == 0
            $chargeDetail = $stripe->charges->retrieve($tran->source, []);
            $chargetransaction = StripeTransaction::where('source', $chargeDetail->refunds->data[0]->id)->first();
            if (!empty($chargetransaction)) {
                $chargetransaction->archived = $archive;
                $chargetransaction->save();
            }
        }

        // Unarchive Refund Transaction 
        if ($tran->reporting_category == 'refund') { // refund
            $refundDetail = $stripe->refunds->retrieve($tran->source, []);

            $refundtransaction = StripeTransaction::where('source', $refundDetail->charge)->first();
            if (!empty($refundtransaction)) {
                $refundtransaction->archived = $archive;
                $refundtransaction->save();
            }
        }
    }

    /**
     *  Refund Card Transaction by id
     * POST merchant/businesses/{business_id}/card-transactions/{id}/refund
     */
    public function refund(Request $req, $id)
    {

        $stripeTransaction = StripeTransaction::find($id);
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);

        $this->validate($req, [
            'password' => 'required|string',
            'amount' => 'required|numeric|gt:0'
        ]);

        if ($req->input('amount') * 100 != $stripeTransaction->amount) {
            // Utils::error('This transaction does not support partial refund!', 404);
        }
        if (Auth::$business_id) {
            $business = Business::find($stripeTransaction->business_id);
            if ($req->input('password') != $business->admin_password) {
                if ($req->input('password') != $business->supervisor_password) {
                    Utils::error("Password is incorrect", 400);
                }
            }
        } else {
            Utils::error('sorry you have no rights!', 404);
        }

        if ($stripeTransaction->source == null) {
            Utils::error("This transaction has no source", 400);
        }
        // if ($stripeTransaction->archived == '1') {
        //     Utils::error("Archived transaction cannot be refund", 400);
        // }
        if ($stripeTransaction->reporting_category != 'charge') {
            Utils::error("This transaction cannot be refund", 400);
        }
        if ($stripeTransaction->refunded) {
            Utils::error("This transaction already refunded", 400);
        }
        if ($stripeTransaction->service == 'Connect' && $stripeTransaction->transfer_status != 'pending') {
            // Utils::error("Cannot refund this transaction as it may not available", 400);
        }

        $account_type = $stripeTransaction->account_type;
        $stripeMode = "test";
        $stripeMode = $stripeTransaction->mode;
        $stripeKey = "";
        if ($stripeMode == 'test') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        if ($account_type == 'normal') {
            $business = Business::find($stripeTransaction->business_id);
            if (!empty($business)) {
                $mode = $stripeTransaction->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);
        $chargeDetails = $stripe->charges->retrieve($stripeTransaction->source);
        $refundParams = [
            'charge' => $stripeTransaction->source,
            'amount' => $req->input('amount') * 100
        ];
        if ($chargeDetails->transfer_data != null) {
            $refundParams['reverse_transfer'] = true;
        }
        try {
            $stripe->refunds->create($refundParams);
        } catch (\Stripe\Exception\InvalidRequestException $e) {
            // Invalid parameters were supplied to Stripe's API
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\CardException $e) {
            // Since it's a decline, \Stripe\Exception\CardException will be caught
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\ApiErrorException $e) {
            // Display a very generic error to the user, and maybe send
            // yourself an email
            Utils::error($e->getError()->message, 400);
        }
        (new StripeTransactionController())->transaction($req);
        return $stripeTransaction;
    }

    /**
     *  Refund Card Transaction by id
     * POST merchant/businesses/{business_id}/card-transactions/{id}/send-receipt
     */
    public function send_receipt(Request $req, $id)
    {

        $stripeTransaction = StripeTransaction::find($id);
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);

        $this->validate($req, [
            'email' => 'string',
            'mobile_number' => 'string',
        ]);

        if ($req->has('email')) {
            $email = explode(',', $req->input('email'));
            foreach ($email as $email) {
                $this->receiptMail($id, $email);
            }
        }

        if ($req->has('mobile_number')) {
            $mobile_number = explode(',', $req->input('mobile_number'));
            foreach ($mobile_number as $mobile_number) {
                $message = 'This is your payment confirmation, click on the link to get the receipt. ' . env('APP_URL') . 'invoice-send-recepit/' . $stripeTransaction->txn_id;
                (new OtpHelper())->sendMessage($message, $mobile_number, 'payment_sms_link');
            }
        }

        return $stripeTransaction;
    }


    /**
     * Refund Paypal Transaction by id
     * POST merchant/businesses/{business_id}/paypal-transactions/{id}/refund
     */
    public function refund_paypal(Request $req, $business_id, $id)
    {
        $this->validate($req, [
            'amount' => 'gt:0|numeric|required',
            'password' => 'required', // Human amount
        ]);

        $paypalPayment = PaypalPayment::find($id);
        if (empty($paypalPayment)) Utils::error("Payment not found", 404);
        if (strtolower($paypalPayment->sale_state) == 'refunded') Utils::error("This payment is already refunded", 400);
        if ($req->input('amount') > $this->convertToDecimal($paypalPayment->transaction_amount - abs($paypalPayment->refunded_amount))) Utils::error("Cannot refund more than transaction amount", 400);

        if (Auth::$business_id) {
            $business = Business::find($paypalPayment->business_id);
            if ($req->input('password') != $business->admin_password) {
                if ($req->input('password') != $business->supervisor_password) {
                    Utils::error("Password is incorrect", 400);
                }
            }
        } else {
            Utils::error('sorry you have no rights!', 404);
        }

        $siteSettingMode = SiteSettingController::findByKey('paypal_mode');

        if ($siteSettingMode->value == 'live') {
            $siteSettingpublicKey = SiteSettingController::findByKey('paypal_public_key_live');
            $siteSettingprivateKey = SiteSettingController::findByKey('paypal_private_key_live');
        } else {
            $siteSettingpublicKey = SiteSettingController::findByKey('paypal_public_key_test');
            $siteSettingprivateKey = SiteSettingController::findByKey('paypal_private_key_test');
        }
        $publicKey = $siteSettingpublicKey->value;
        $privateKey = $siteSettingprivateKey->value;

        try {
            $apiContext = new ApiContext(new OAuthTokenCredential($publicKey, $privateKey));
            $singlePayment = Payment::get($paypalPayment->payment_id, $apiContext);

            $paypalPayment->sale_state = $singlePayment->transactions[0]->related_resources[0]->sale->state;
            $refunded_amount = 0;
            foreach ($singlePayment->transactions[0]->related_resources as $resource) {
                if (isset($resource->refund)) {
                    $paypalPayment->refund_created_at = $resource->refund->create_time;
                    $refunded_amount = $refunded_amount + $resource->refund->amount->total;
                    if (isset($resource->refund->capture_id)) {
                        $paypalPayment->capture_id = $resource->refund->capture_id;
                    }
                }
            }
            $paypalPayment->refunded_amount = $refunded_amount;
            $paypalPayment->save();


            if (strtolower($paypalPayment->sale_state) == 'refunded') Utils::error("Payment is already refunded", 400);
            $totalRefundableAmount = $paypalPayment->transaction_amount - $paypalPayment->refunded_amount;
            if ($req->input('amount') > $totalRefundableAmount) {
                Utils::error("Transaction cannot refund more than transaction amount", 400);
            }

            //Create refund
            $amt = new Amount();
            $amt->setTotal($req->input('amount'))->setCurrency($paypalPayment->currency);

            $refund = new Refund();
            $refund->setAmount($amt);
            $sale = new Sale();
            $sale->setId($paypalPayment->sale_id);

            //Execute Refund
            $sale->refundSale($refund, $apiContext);

            //Retrive single payment
            $singlePayment = Payment::get($paypalPayment->payment_id, $apiContext);

            $paypalPayment->sale_state = $singlePayment->transactions[0]->related_resources[0]->sale->state;
            $refunded_amount = 0;
            foreach ($singlePayment->transactions[0]->related_resources as $resource) {
                if (isset($resource->refund)) {
                    $paypalPayment->refund_created_at = $resource->refund->create_time;
                    $refunded_amount = $refunded_amount + $resource->refund->amount->total;
                    if (isset($resource->refund->capture_id)) {
                        $paypalPayment->capture_id = $resource->refund->capture_id;
                    }
                }
            }
            $paypalPayment->refunded_amount = $refunded_amount;

            $paypalPayment->save();
            return $paypalPayment;
        } catch (PayPalConnectionException $ex) {
            return $ex->getData();
        } catch (Exception $ex) {
            return $ex->getMessage();
        }
    }


    /**
     *  Dispute Accept Card Transaction by id
     * POST merchant/businesses/{business_id}/card-transactions/{id}/dispute-accept
     */
    public function dispute_accept(Request $req, $id)
    {

        $stripeTransaction = StripeTransaction::find($id);
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);

        if ($stripeTransaction->reporting_category != 'dispute') {
            Utils::error("This transaction cannot be dispute", 400);
        }
        if ($stripeTransaction->dispute_status != 'needs_response' || $stripeTransaction->dispute_status != 'under_review') {
            Utils::error("This transaction already dispute", 400);
        }

        $account_type = $stripeTransaction->account_type;
        $stripeMode = "test";
        $stripeMode = $stripeTransaction->mode;
        $stripeKey = "";
        if ($stripeMode == 'test') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        if ($account_type == 'normal') {
            $business = Business::find($stripeTransaction->business_id);
            if (!empty($business)) {
                $mode = $stripeTransaction->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);
        $disputeDetails = $stripe->disputes->retrieve(
            $stripeTransaction->source,
            []
        );

        try {
            $stripe->disputes->close(
                $stripeTransaction->source,
                []
            );
        } catch (\Stripe\Exception\InvalidRequestException $e) {
            // Invalid parameters were supplied to Stripe's API
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\CardException $e) {
            // Since it's a decline, \Stripe\Exception\CardException will be caught
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\ApiErrorException $e) {
            // Display a very generic error to the user, and maybe send
            // yourself an email
            Utils::error($e->getError()->message, 400);
        }
        return $stripeTransaction;
    }

    /**
     *  Dispute Accept Card Transaction by id
     * POST merchant/businesses/{business_id}/card-transactions/{id}/dispute-accept
     */
    public function dispute_counter(Request $req, $id)
    {
        $this->validate($req, [
            'access_activity_log' => 'string',
            'billing_address' => 'string',
            'cancellation_policy_disclosure' => 'string',
            'cancellation_rebuttal' => 'string',
            'customer_communication' => 'image', //FILE
            'customer_email_address' => 'string',
            'customer_name' => 'string',
            'customer_purchase_ip' => 'string',
            'customer_signature' => 'image', //FILE
            'duplicate_charge_documentation' => 'image', //FILE
            'duplicate_charge_explanation' => 'string',
            'duplicate_charge_id' => 'string',
            'product_description' => 'string',
            'receipt' => 'image', //FILE
            'refund_policy' => 'image', //FILE
            'refund_policy_disclosure' => 'string',
            'refund_refusal_explanation' => 'string',
            'service_date' => '12/5/2022',
            'service_documentation' => 'string',
            'shipping_address' => 'string',
            'shipping_carrier' => 'string',
            'shipping_date' => '12/5/2022',
            'shipping_documentation' => 'image', //FILE
            'shipping_tracking_number' => 'string',
            'uncategorized_file' => 'image', //FILE
            'uncategorized_text' => 'string',
        ]);

        $stripeTransaction = StripeTransaction::find($id);
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);

        if ($stripeTransaction->reporting_category != 'dispute') {
            Utils::error("This transaction cannot be dispute", 400);
        }
        if ($stripeTransaction->dispute_status != 'needs_response') {
            Utils::error("This transaction already dispute", 400);
        }

        $account_type = $stripeTransaction->account_type;
        $stripeMode = "test";
        $stripeMode = $stripeTransaction->mode;
        $stripeKey = "";
        if ($stripeMode == 'test') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        if ($account_type == 'normal') {
            $business = Business::find($stripeTransaction->business_id);
            if (!empty($business)) {
                $mode = $stripeTransaction->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);
        // $disputeDetails = $stripe->disputes->retrieve(
        //     $stripeTransaction->source,
        //     []
        // );

        try {
            $params = [];

            // if ($req->has('uncategorized_file')) {
            //     $fileName = ImageHelper::saveImageToServer($req->file('uncategorized_file'), ImageHelper::$TYPE_EVIDENCE);
            //     $path = ImageHelper::getServerImageUrl(ImageHelper::$TYPE_BUSINESS, $fileName);
            //     $uncategorized_file = fopen($path, 'r');
            // }

            // $file = $stripe->files->create(
            //     [
            //         'file' => $uncategorized_file,
            //         'purpose' => 'uncategorized_file',
            //     ],
            //     ['stripe_account' => 'acct_1J9qlCQTxUOYbQHJ']
            // );


            $params['evidence'] = [
                'access_activity_log' => $req->input('access_activity_log'),
                'billing_address' => $req->input('billing_address'),
                'cancellation_policy_disclosure' => $req->input('cancellation_policy_disclosure'),
                'cancellation_rebuttal' => $req->input('cancellation_rebuttal'),
                // 'customer_communication' => $req->file('customer_communication'), //FILE
                'customer_email_address' => $req->input('customer_email_address'),
                'customer_name' => $req->input('customer_name'),
                'customer_purchase_ip' => $req->input('customer_purchase_ip'),
                // 'customer_signature' => $req->file('customer_signature'), //FILE
                // 'duplicate_charge_documentation' => $req->file('duplicate_charge_documentation'), //FILE
                'duplicate_charge_explanation' => $req->input('duplicate_charge_explanation'),
                'duplicate_charge_id' => $req->input('duplicate_charge_id'),
                'product_description' => $req->input('product_description'),
                // 'receipt' => $req->file('receipt'), //FILE
                // 'refund_policy' => $req->file('refund_policy'), //FILE
                'refund_policy_disclosure' => $req->input('refund_policy_disclosure'),
                'refund_refusal_explanation' => $req->input('refund_refusal_explanation'),
                'service_date' => $req->input('service_date'),
                'service_documentation' => $req->input('service_documentation'),
                'shipping_address' => $req->input('shipping_address'),
                'shipping_carrier' => $req->input('shipping_carrier'),
                'shipping_date' => $req->input('shipping_date'),
                // 'shipping_documentation' => $req->file('shipping_documentation'), //FILE
                'shipping_tracking_number' => $req->input('shipping_tracking_number'),
                // 'uncategorized_file' => $req->input('uncategorized_file'), //FILE
                'uncategorized_text' => $req->input('uncategorized_text'),
            ];

            // $stripe->disputes->update(
            //     $stripeTransaction->source,
            //     [
            //         $params,
            //         'metadata' => $req->input('metadata'),
            //         'submit' => true,
            //     ]
            // );
        } catch (\Stripe\Exception\InvalidRequestException $e) {
            // Invalid parameters were supplied to Stripe's API
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\CardException $e) {
            // Since it's a decline, \Stripe\Exception\CardException will be caught
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\ApiErrorException $e) {
            // Display a very generic error to the user, and maybe send
            // yourself an email
            Utils::error($e->getError()->message, 400);
        }
        return $stripeTransaction;
    }

    /**
     * Show merchant business transactions by id
     * GET merchant/businesses/{business_id}/report-stripe-transactions
     */
    public function businesses_report_stripe_transactions(Request $req, $business_id)
    {
        $business = Business::where('id', $business_id)->first();
        if (empty($business)) Utils::error("No business found", 400);

        $stripeTransactions = StripeTransaction::where('business_id', $business->id)->orderByRaw('id DESC');
        $cashTransactions = CashTransaction::latest();

        if ($service = $req->query('service', false)) {
            $stripeTransactions->where('service', $service);
        }
        if ($query = $req->query('type', false)) {
            $stripeTransactions->where('type', $query);
        }

        if ($status = $req->query('status', false)) {
            $stripeTransactions->where('reporting_category', $status);
        } else {
            $stripeTransactions->whereIn('reporting_category', ['charge', 'refund']);
        }

        if ($amount_filter = $req->query('amount_filter_type', false)) {
            if ($req->has('amount')) {
                $amount = $req->query('amount');
                if ($amount_filter == 'greater_than') {
                    $stripeTransactions->where('amount', '>', $amount * 100);
                    $cashTransactions->where('amount', '>', $amount);
                } else if ($amount_filter == 'less_than') {
                    $stripeTransactions->where('amount', '<', $amount * 100);
                    $cashTransactions->where('amount', '<', $amount);
                } else if ($amount_filter == 'between') {
                    $amounts = explode(",", $amount);
                    $stripeTransactions->where('amount', '>',  $amounts[0] * 100);
                    $stripeTransactions->where('amount', '<',  $amounts[1] * 100);
                    $cashTransactions->where('amount', '>',  $amounts[0]);
                    $cashTransactions->where('amount', '<',  $amounts[1]);
                } else if ($amount_filter == 'equal') {
                    $stripeTransactions->where('amount', '=', $amount * 100);
                    $cashTransactions->where('amount', '=', $amount);
                }
            } else {
                Utils::error("Amount field required with type", 400);
            }
        }
        if ($date_filter = $req->query('date_filter_type', false)) {
            if ($date = $req->query('date', false)) {
                if ($date_filter == 'in_last') {
                    $today = Carbon::today()->endOfDay();
                    $lastDay = Carbon::today()->subDays($date)->startOfDay();
                    $stripeTransactions->where('created', '>', $lastDay->getTimestamp());
                    $stripeTransactions->where('created', '<', $today->getTimestamp());
                    $cashTransactions->where('created_at', '>', $lastDay->getTimestamp());
                    $cashTransactions->where('created_at', '<', $today->getTimestamp());
                } else if ($date_filter == 'before') {
                    $date = Carbon::parse($date)->startOfDay();
                    $stripeTransactions->where('created', '<', $date->getTimestamp());
                    $cashTransactions->where('created_at', '<', $date->getTimestamp());
                } else if ($date_filter == 'after') {
                    $date = Carbon::parse($date)->endOfDay();
                    $stripeTransactions->where('created', '>', $date->getTimestamp());
                    $cashTransactions->where('created_at', '>', $date->getTimestamp());
                } else if ($date_filter == 'between') {
                    $date = $req->query('date');
                    $dates = explode(",", $date);
                    $dates[0] = Carbon::parse($dates[0])->startOfDay();
                    $dates[1] = Carbon::parse($dates[1])->endOfDay();
                    $stripeTransactions->where('created', '>',  $dates[0]->getTimestamp());
                    $stripeTransactions->where('created', '<',  $dates[1]->getTimestamp());
                    $cashTransactions->where('created_at', '>',  $dates[0]->getTimestamp());
                    $cashTransactions->where('created_at', '<',  $dates[1]->getTimestamp());
                } else if ($date_filter == 'equal') {
                    $startTime = Carbon::parse($date)->startOfDay();
                    $endTime = Carbon::parse($date)->endOfDay();
                    $stripeTransactions->where('created', '>',  $startTime->getTimestamp());
                    $stripeTransactions->where('created', '<',  $endTime->getTimestamp());
                    $cashTransactions->where('created_at', '>',  $startTime->getTimestamp());
                    $cashTransactions->where('created_at', '<',  $endTime->getTimestamp());
                }
            } else {
                Utils::error("Date field required with type", 400);
            }
        }

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        if ($req->has('from_date')) {
            $from_date = Carbon::parse($req->query('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->query('to_date'))->endOfDay();
            $stripeTransactions->where('created', '>',  $from_date->getTimestamp());
            $stripeTransactions->where('created', '<',  $to_date->getTimestamp());
            $cashTransactions->where('created', '>',  $from_date->getTimestamp());
            $cashTransactions->where('created', '<',  $to_date->getTimestamp());
        }

        $sql = $stripeTransactions->toSql();
        $bindings = $stripeTransactions->getBindings();

        $sql_with_bindings = preg_replace_callback('/\?/', function ($match) use ($sql, &$bindings) {
            return "'" . array_shift($bindings) . "'";
        }, $sql);

        $stripeTransactions = $stripeTransactions->get();
        $stripeTransactions->each(function ($transaction) {
            $transaction->append('calculated_fees');
        });

        $cashTransactions = $cashTransactions->get()->toArray();
        $stripeTransactions = $stripeTransactions->toArray();

        $allData = collect(array_merge($stripeTransactions, $cashTransactions))->sortByDesc('created_at');

        if ($req->query('nopaginate', false) == '1') {
            $data = $allData;
        } else {
            $data = $this->paginate($allData, $req->query('per_page', 20), $req->page ?? null);
        }

        return $data;
    }

    public function paginate($items, $perPage = 5, $page = null, $options = [])
    {
        $page = $page ?: (Paginator::resolveCurrentPage() ?: 1);
        $items = $items instanceof Collection ? $items : Collection::make($items);
        return new LengthAwarePaginator($items->forPage($page, $perPage), $items->count(), $perPage, $page, $options);
    }

    /**
     * Show merchant business dispute transactions by id
     * GET merchant/businesses/{business_id}/dispute-all-transactions
     */
    public function businesses_dispute_all_transactions(Request $req, $business_id)
    {
        $business = Business::where('id', $business_id)->first();
        if (empty($business)) Utils::error("No business found", 400);

        $stripeTransactions = StripeTransaction::where('business_id', $business->id)->orderByRaw('id DESC');

        $stripeTransactions->whereRaw('(reporting_category LIKE ?)', ["%dispute%"]);
        if ($dispute_status = $req->query('dispute_status', false)) {
            $stripeTransactions->where('dispute_status', $dispute_status);
        }

        if ($amount_filter = $req->query('amount_filter_type', false)) {
            if ($req->has('amount')) {
                $amount = $req->query('amount');
                if ($amount_filter == 'greater_than') {
                    $stripeTransactions->where('amount', '>', $amount * 100);
                } else if ($amount_filter == 'less_than') {
                    $stripeTransactions->where('amount', '<', $amount * 100);
                } else if ($amount_filter == 'between') {
                    $amounts = explode(",", $amount);
                    $stripeTransactions->where('amount', '>',  $amounts[0] * 100);
                    $stripeTransactions->where('amount', '<',  $amounts[1] * 100);
                } else if ($amount_filter == 'equal') {
                    $stripeTransactions->where('amount', '=', $amount * 100);
                }
            } else {
                Utils::error("Amount field required with type", 400);
            }
        }
        if ($date_filter = $req->query('date_filter_type', false)) {
            if ($date = $req->query('date', false)) {
                if ($date_filter == 'in_last') {
                    $today = Carbon::today()->endOfDay();
                    $lastDay = Carbon::today()->subDays($date)->startOfDay();
                    $stripeTransactions->where('created', '>', $lastDay->getTimestamp());
                    $stripeTransactions->where('created', '<', $today->getTimestamp());
                } else if ($date_filter == 'before') {
                    $date = Carbon::parse($date)->startOfDay();
                    $stripeTransactions->where('created', '<', $date->getTimestamp());
                } else if ($date_filter == 'after') {
                    $date = Carbon::parse($date)->endOfDay();
                    $stripeTransactions->where('created', '>', $date->getTimestamp());
                } else if ($date_filter == 'between') {
                    $date = $req->query('date');
                    $dates = explode(",", $date);
                    $dates[0] = Carbon::parse($dates[0])->startOfDay();
                    $dates[1] = Carbon::parse($dates[1])->endOfDay();
                    $stripeTransactions->where('created', '>',  $dates[0]->getTimestamp());
                    $stripeTransactions->where('created', '<',  $dates[1]->getTimestamp());
                } else if ($date_filter == 'equal') {
                    $startTime = Carbon::parse($date)->startOfDay();
                    $endTime = Carbon::parse($date)->endOfDay();
                    $stripeTransactions->where('created', '>',  $startTime->getTimestamp());
                    $stripeTransactions->where('created', '<',  $endTime->getTimestamp());
                }
            } else {
                Utils::error("Date field required with type", 400);
            }
        }

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        if ($req->has('from_date')) {
            $from_date = Carbon::parse($req->query('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->query('to_date'))->endOfDay();
            $stripeTransactions->where('created', '>',  $from_date->getTimestamp());
            $stripeTransactions->where('created', '<',  $to_date->getTimestamp());
        }

        if ($req->query('nopaginate', false) == '1') {
            $stripeTransactions = $stripeTransactions->get();
        } else {
            $stripeTransactions = $stripeTransactions->paginate($req->query('per_page', 20));
        }
        $stripeTransactions->each(function ($transaction) {
            $transaction->append('calculated_fees');
            $transaction->append('direct_stripe_transaction');
        });

        return $stripeTransactions;
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }

    public function receiptMail($id, $email)
    {
        $stripeTransaction = StripeTransaction::find($id);
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table bgcolor="#f6f9fc" border="0" cellpadding="0" cellspacing="0" width="650px" style="border: 0; margin: 0; padding: 0;">
        <tbody>
            <tr>
                <td style="border: 0; margin: 0; padding: 0;">
                    <table class="st-Wrapper" align="center" bgcolor="#ffffff" border="0" cellpadding="0" cellspacing="0" width="100%" style="border-top-left-radius: 16px; border-top-right-radius: 16px; margin: 0 auto; width: 600px;">
                        <tbody>
                            <tr>
                                <td style="border: 0; margin: 0; padding: 0;">
                                    <table class="st-Preheader st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td align="center" height="0" style="border: 0; margin: 0; padding: 0; color: #ffffff; display: none !important; font-size: 1px; line-height: 1px; max-height: 0; max-width: 0; mso-hide: all !important; opacity: 0; overflow: hidden; visibility: hidden;">
                                                    <span class="st-Delink st-Delink--preheader" style="color: #ffffff; text-decoration: none;">
                                                        Receipt from ubsidi limited [#1910-4259] Amount paid £20.95 Date
                                                        paid May 22, 2023, 1:17:13 AM
                                                    </span>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                    <div style="background-color:#f6f9fc;">
                                        <table dir="ltr" class="Section Header" width="100%" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;background-color: #ffffff;">
                                            <tbody>
                                                <tr>
                                                    <td class="Header-left Target" style="background-color: #09c4a9;border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-size: 0;line-height: 0px;mso-line-height-rule: exactly;background-size: 100% 100%;border-top-left-radius: 5px;" align="right" valign="bottom">
                                                        <a href="https://www.ubsidi.com/" target="_blank" style="-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;outline: 0;text-decoration: none;">
                                                            <img alt="" src="' . env('APP_URL') . 'images/Left.png" style="display: block;border: 0;height: 130px;width: 100%;">
                                                        </a>
                                                    </td>
                                                    <td class="Header-icon Target" style="background-color: #09c4a9;border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-size: 0;line-height: 0px;mso-line-height-rule: exactly;background-size: 100% 100%; width: 96px !important;" align="center" valign="bottom">
                                                        <a href="https://www.ubsidi.com/" target="_blank" style="-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;outline: 0;text-decoration: none;">
                                                            <img alt="" src="' . env('APP_URL') . 'images/twelve_degree_icon@2x.png" style="display: block;border: 0;height:129px;width: 96px;">
                                                        </a>
                                                    </td>
                                                    <td class="Header-right Target" style="background-color: #09c4a9;border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-size: 0;line-height: 0px;mso-line-height-rule: exactly;background-size: 100% 100%;border-top-right-radius: 5px;" align="left" valign="bottom">
                                                        <a href="https://www.ubsidi.com/" target="_blank" style="-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;outline: 0;text-decoration: none;">
                                                            <img alt="" src="' . env('APP_URL') . 'images/Right.png" style="display: block;border: 0;height: 129px;width: 100%;">
                                                        </a>
                                                    </td>
                                                </tr>
                                            </tbody>
                                        </table>
                                    </div>
                                    <table class="st-Copy st-Copy--caption st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td class="Content Title-copy Font Font--title" align="center" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;width: 472px;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #32325d;font-size: 24px;line-height: 32px;">
                                                    Receipt from ubsidi limited
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="12" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
    
                                    <table class="st-Copy st-Copy--caption st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td class="Content Title-copy Font Font--title" align="center" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;width: 472px;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #8898aa;font-size: 15px;line-height: 18px;">
                                                    Receipt #' . $id . '
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="12" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
    
                                    <table class="st-Spacer st-Spacer--standalone st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%">
                                        <tbody>
                                            <tr>
                                                <td height="20" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
    
                                    <table class="st-Copy st-Copy--standalone st-Copy--caption" border="0" cellpadding="0" cellspacing="0" width="100%">
                                        <tbody>
                                            <tr>
                                                <td class="st-Font st-Font--caption" style="border: 0; margin: 0; padding: 0; color: #687385; font-family: -apple-system, SF Pro Display, SF Pro Text, Helvetica, sans-serif; font-size: 12px; font-weight: bold; line-height: 16px; text-transform: uppercase;">
    
    
                                                </td>
                                                <td width="64" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                </td>
                                                <td class="DataBlocks-item" valign="top" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;">
                                                    <table style="border: 0;border-collapse: collapse;margin: 0;padding: 0;">
                                                        <tbody>
                                                            <tr>
                                                                <td class="Font Font--caption Font--uppercase Font--mute Font--noWrap" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #8898aa;font-size: 12px;line-height: 16px;white-space: nowrap;font-weight: bold;text-transform: uppercase;">
                                                                    Amount paid
                                                                </td>
                                                            </tr>
                                                            <tr>
                                                                <td class="Font Font--body Font--noWrap" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #525f7f;font-size: 15px;line-height: 24px;white-space: nowrap;">
    
                                                                    £' . ($stripeTransaction->amount / 100) . '
    
                                                                </td>
                                                            </tr>
                                                        </tbody>
                                                    </table>
                                                </td>
                                                <td width="20" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                </td>
                                                <td class="DataBlocks-item" valign="top" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;">
                                                    <table style="border: 0;border-collapse: collapse;margin: 0;padding: 0;">
                                                        <tbody>
                                                            <tr>
                                                                <td class="Font Font--caption Font--uppercase Font--mute Font--noWrap" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #8898aa;font-size: 12px;line-height: 16px;white-space: nowrap;font-weight: bold;text-transform: uppercase;">
                                                                    Date paid
                                                                </td>
                                                            </tr>
                                                            <tr>
                                                                <td class="Font Font--body Font--noWrap" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #525f7f;font-size: 15px;line-height: 24px;white-space: nowrap;">
                                                                    ' . date('M d-Y h:i:s A', strtotime($stripeTransaction->created_at)) . '
                                                                </td>
                                                            </tr>
                                                        </tbody>
                                                    </table>
                                                </td>
                                                <td width="20" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                </td>
                                                <td class="DataBlocks-item" valign="top" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;">
                                                    <table style="border: 0;border-collapse: collapse;margin: 0;padding: 0;">
                                                        <tbody>
                                                            <tr>
                                                                <td class="Font Font--caption Font--uppercase Font--mute Font--noWrap" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #8898aa;font-size: 12px;line-height: 16px;white-space: nowrap;font-weight: bold;text-transform: uppercase;">
                                                                    Payment method
                                                                </td>
                                                            </tr>
                                                            <tr>
                                                                <td class="Font Font--body Font--noWrap" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #525f7f;font-size: 15px;line-height: 24px;white-space: nowrap;">
    
                                                                    <span>
                                                                    ' . strtoupper($stripeTransaction->card_brand) . '
                                                                    </span>
                                                                    <span>
                                                                        - ' . $stripeTransaction->card_number . '
                                                                    </span>
    
                                                                </td>
                                                            </tr>
                                                        </tbody>
                                                    </table>
                                                </td>
                                                <td width="64" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
    
                                    <table class="st-Spacer st-Spacer--standalone st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%">
                                        <tbody>
                                            <tr>
                                                <td height="32" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
    
                                    <table class="st-Copy st-Copy--caption st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="8" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                                <td class="st-Font st-Font--caption" style="border: 0; margin: 0; padding: 0; color: #687385; font-family: -apple-system, SF Pro Display, SF Pro Text, Helvetica, sans-serif; font-weight:400; font-size: 12px; line-height: 16px; text-transform: uppercase;">
                                                    <span class="st-Delink" style="border: 0; margin: 0; padding: 0; font-weight: bold;">
                                                        Summary
                                                    </span>
                                                </td>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="8" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                    <table class="st-Blocks st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="24" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--kill" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                                <td style="border: 0; margin: 0; padding: 0;">
                                                    <table class="st-Blocks-inner" bgcolor="#f6f9fc" border="0" cellpadding="0" cellspacing="0" style="border-radius: 8px;" width="100%">
                                                        <tbody>
    
                                                            <tr>
                                                                <td style="border: 0; margin: 0; padding: 0;">
                                                                    <table class="st-Blocks-item" border="0" cellpadding="0" cellspacing="0" width="100%">
                                                                        <tbody>
                                                                            <tr>
                                                                                <td class="st-Spacer st-Spacer--blocksItemEnds" colspan="3" height="12" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                                                    <div class="st-Spacer st-Spacer--filler">
                                                                                    </div>
                                                                                </td>
                                                                            </tr>
                                                                            <tr>
                                                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="16">
                                                                                    <div class="st-Spacer st-Spacer--filler">
                                                                                    </div>
                                                                                </td>
                                                                                <td class="st-Blocks-item-cell st-Font st-Font--body" style="border: 0; margin: 0; padding: 0; color: #414552; font-family: -apple-system, SF Pro Display, SF Pro Text, Helvetica, sans-serif; font-size: 16px; line-height: 24px;">
                                                                                    <table style="padding-left: 5px; padding-right:5px;" width="100%">
                                                                                        <tr>
                                                                                            <td>
                                                                                            </td>
                                                                                        </tr>
                                                                                        <tr>
                                                                                            <td class="Table-description Font Font--body" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #525f7f;font-size: 15px;line-height: 24px; width: 100%; ">
                                                                                                ' . $stripeTransaction->description . '
                                                                                            </td>
                                                                                            <td class="Spacer Table-gap" width="8" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                                                            </td>
                                                                                            <td class="Table-amount Font Font--body" align="right" valign="top" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #525f7f;font-size: 15px;line-height: 24px;">
                                                                                                £' . ($stripeTransaction->amount / 100) . '
                                                                                            </td>
                                                                                        </tr>
                                                                                        <tr>
                                                                                            <td class="Table-divider Spacer" colspan="3" height="6" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                                                            </td>
                                                                                        </tr>
                                                                                        <tr>
                                                                                            <td class="Table-divider Spacer" colspan="3" height="6" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                                                            </td>
                                                                                        </tr>
    
                                                                                        <tr>
                                                                                            <td class="Spacer" bgcolor="e6ebf1" colspan="3" height="1" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                                                            </td>
                                                                                        </tr>
    
                                                                                        <tr>
                                                                                            <td class="Table-divider Spacer" colspan="3" height="8" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                                                            </td>
                                                                                        </tr>
    
                                                                                        <tr>
                                                                                            <td class="Table-description Font Font--body" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #525f7f;font-size: 15px;line-height: 24px; width: 100%; ">
                                                                                                <strong>Amount
                                                                                                    charged</strong>
                                                                                            </td>
                                                                                            <td class="Spacer Table-gap" width="8" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                                                            </td>
                                                                                            <td class="Table-amount Font Font--body" align="right" valign="top" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Ubuntu, sans-serif;mso-line-height-rule: exactly;vertical-align: middle;color: #525f7f;font-size: 15px;line-height: 24px;">
                                                                                                <strong>£' . $stripeTransaction->amount / 100 . '</strong>
                                                                                            </td>
                                                                                        </tr>
                                                                                        <tr>
                                                                                            <td class="Table-divider Spacer" colspan="3" height="6" style="border: 0;border-collapse: collapse;margin: 0;padding: 0;-webkit-font-smoothing: antialiased;-moz-osx-font-smoothing: grayscale;color: #ffffff;font-size: 1px;line-height: 1px;mso-line-height-rule: exactly;">
                                                                                            </td>
                                                                                        </tr>
                                                                                    </table>
                                                                                </td>
                                                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="16">
                                                                                    <div class="st-Spacer st-Spacer--filler">
                                                                                    </div>
                                                                                </td>
                                                                            </tr>
                                                                            <tr>
                                                                                <td class="st-Spacer st-Spacer--blocksItemEnds" colspan="3" height="12" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                                                    <div class="st-Spacer st-Spacer--filler">
                                                                                    </div>
                                                                                </td>
                                                                            </tr>
                                                                        </tbody>
                                                                    </table>
                                                                </td>
                                                            </tr>
    
                                                        </tbody>
                                                    </table>
                                                </td>
                                                <td class="st-Spacer st-Spacer--kill" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="24" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
    
                                    <table class="st-Divider st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--divider" colspan="3" height="24" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                                <td bgcolor="#e6ebf1" height="1" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--divider" colspan="3" height="24" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
    
                                    <table class="st-Copy st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="8" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                                <td style="border: 0; margin: 0; padding: 0; color: #414552 !important; font-family: -apple-system, SF Pro Display, SF Pro Text, Helvetica, sans-serif; font-weight:400; font-size: 16px; line-height: 24px;">
    
                                                    If you have any questions, visit our support site at <a style="border: 0; margin: 0; padding: 0; color: #625afa !important; font-weight: bold; text-decoration: none;  " href="https://www.ubsidi.com">https://www.ubsidi.com</a>, contact us
                                                    at <a style="border: 0; margin: 0; padding: 0; color: #625afa !important; font-weight: bold; text-decoration: none;  " href="mailto:admin@ubsidi.com">admin@ubsidi.com</a>, or call at <a style="border: 0; margin: 0; padding: 0; color: #625afa !important; font-weight: bold; text-decoration: none;  " href="tel:+448712884044">+44 871 288 4044</a>.
    
                                                </td>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--stacked" colspan="3" height="8" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                    <table class="st-Divider st-Width st-Width--mobile" border="0" cellpadding="0" cellspacing="0" width="100%" style="width: 600px;">
                                        <tbody>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--divider" colspan="3" height="24" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                                <td bgcolor="#e6ebf1" height="1" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                                <td class="st-Spacer st-Spacer--gutter" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;" width="48">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td class="st-Spacer st-Spacer--divider" colspan="3" height="24" style="border: 0; margin: 0; padding: 0; font-size: 1px; line-height: 1px; max-height: 1px; mso-line-height-rule: exactly;">
                                                    <div class="st-Spacer st-Spacer--filler"> </div>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                </td>
            </tr>
        </tbody>
    </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Payment Reciept',
            'mail_type' => $siteSettings->mail_option,
            'to' => $email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }
}
