<?php

namespace App\Http\Controllers;

use App\BacsDirect;
use App\Business;
use App\Country;
use Illuminate\Http\Request;
use App\Helpers\Utils;
use Exception;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class BacsDirectController extends Controller
{
    /**
     * List bacs-debit.
     * GET /api/bacs-debit
     */
    public function index(Request $req)
    {
        $bacs_debits = BacsDirect::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $bacs_debits->whereRaw('(title LIKE ? OR sort_code LIKE ? OR account_number LIKE ? OR amount LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $bacs_debits = $bacs_debits->get();
        } else {
            $bacs_debits = $bacs_debits->paginate($req->query('per_page', 20));
        }

        return $bacs_debits;
    }

    /**
     * Create bacs-debit.
     * POST /api/bacs-debits
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'title' => 'required|string',
            'time_period' => 'required|string',
            'sort_code' => 'required|string',
            'account_number' => 'required|string',
            'start_date' => 'required|string',
            'end_date' => 'string',
            'amount' => 'required|numeric',
            'last_transaction_date' => 'string',
            'description' => 'required|string|max:22',
            'statement_descriptor' => 'required|string|max:22'
        ]);

        // create bacs_debit
        $bacs_debit = new BacsDirect;
        $bacs_debit->fill($req->all());
        $bacs_debit->save();

        return $bacs_debit;
    }

    /**
     * Show bacs-debit.
     * GET /api/bacs-debits/{id}
     */
    public function show(Request $req, $id)
    {
        $bacs_debit = BacsDirect::find($id);
        if (empty($bacs_debit)) Utils::error('Direct debit not found!', 404);
        return $bacs_debit;
    }

    /*`*
     * Update bacs-debit.
     * POST /api/bacs-debits/{id}
     */
    public function update(Request $req, $id)
    {
        $bacs_debit = BacsDirect::find($id);
        if (empty($bacs_debit)) Utils::error('Direct debit not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'title' => 'required|string',
            'time_period' => 'required|string',
            'sort_code' => 'required|string',
            'account_number' => 'required|string',
            'start_date' => 'required|string',
            'end_date' => 'string',
            'amount' => 'required|numeric',
            'last_transaction_date' => 'string',
            'description' => 'required|string|max:22',
            'statement_descriptor' => 'required|string|max:22'
        ]);

        // update bacs_debit
        if ($req->has('business_id')) $bacs_debit->business_id = $req->input('business_id');
        if ($req->has('title')) $bacs_debit->title = $req->input('title');
        if ($req->has('description')) $bacs_debit->description = $req->input('description');
        if ($req->has('statement_descriptor')) $bacs_debit->statement_descriptor = $req->input('statement_descriptor');
        if ($req->has('time_period')) $bacs_debit->time_period = $req->input('time_period');
        if ($req->has('sort_code')) $bacs_debit->sort_code = $req->input('sort_code');
        if ($req->has('account_number')) $bacs_debit->account_number = $req->input('account_number');
        if ($req->has('start_date')) $bacs_debit->start_date = $req->input('start_date');
        if ($req->has('end_date')) $bacs_debit->end_date = $req->input('end_date');
        if ($req->has('amount')) $bacs_debit->amount = $req->input('amount');
        if ($req->has('last_transaction_date')) $bacs_debit->last_transaction_date = $req->input('last_transaction_date');

        $bacs_debit->save();

        return $bacs_debit;
    }

    /**
     * Delete bacs-debit by id.
     * DELETE /api/bacs-debits/{id}
     */
    public function delete(Request $req, $id)
    {
        $bacs_debit = BacsDirect::find($id);
        if (empty($bacs_debit)) Utils::error('Direct debit not found!', 404);
        $bacs_debit->delete();
        return $bacs_debit;
    }

    public function bacs_debit_payment(Request $req)
    {
        $business = Business::find($req->input('business_id'));
        if (empty($business)) Utils::error('business not found!', 404);
        $business->append('user');

        $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
        if ($mode == 'test') {
            $stripeKeyColoumns = 'connect_stripe_private_key_test';
        }
        if ($mode == 'live') {
            $stripeKeyColoumns = 'connect_stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        $stripe = new StripeClient(["api_key" => $key]);

        // try {
            $customerParams['name'] = $business->user->first_name . ' ' . $business->user->last_name;
            $customerParams['email'] = $business->user->contact_emails;

            $iso = '';
            if (!empty($business->user->country->iso)) {
                $iso = $business->user->country->iso;
            } else {
                $country = Country::where('name', 'United Kingdom')->select('iso')->first();
                $iso = $country->iso;
            }

            $customer = $stripe->customers->create($customerParams);

            $paymentMethod = $stripe->paymentMethods->create([
                'type' => 'bacs_debit',
                'billing_details' => [
                    'address' => [
                        'city' => $business->user->city,
                        'country' => $iso,
                        'postal_code' => $business->user->postcode,
                        'line1' => $business->user->residential_address,
                        'state' => $business->user->state,
                    ],
                    'email' => $business->user->contact_emails,
                    'name' => $business->user->first_name . $business->user->last_name,
                    'phone' => $business->user->contact_numbers,
                ],
                'bacs_debit' => [
                    'account_number' => $req->input('account_number'),
                    'sort_code' => $req->input('sort_code'),
                ]
            ]);

            $stripe->paymentMethods->attach(
                $paymentMethod->id,
                ['customer' => $customer->id]
            );

            // $setupIntents = $stripe->setupIntents->create([
            //     'payment_method_types' => ['bacs_debit'],
            // ]);

            // $setupIntentsConfirm = $stripe->setupIntents->confirm(
            //     $setupIntents->id,
            //     [
            //         'payment_method' => 'bacs_debit',
            //         'mandate_data' => [
            //             'customer_acceptance' => [
            //                 'type' => 'offline',
            //                 'offline' => []
            //             ]
            //         ]
            //     ],
            // );

            $response = $stripe->paymentIntents->create(
                [
                    'amount' => (float)($req->input('amount') * 100),
                    'currency' => strtolower($business->user->country->currency_code),
                    'payment_method_types' => ['bacs_debit'],
                    // 'payment_method' => $paymentMethod->id,
                    // 'customer' => $customer->id,
                    // 'automatic_payment_methods' => [
                    //     'enabled' => true,
                    //     'allow_redirects' => 'never'
                    // ],
                    'description' => $req->description,
                    'statement_descriptor' => $req->statement_descriptor,
                    // 'confirm' => true,
                    // 'mandate_data' => [
                    //     'customer_acceptance' => [
                    //         'type' => 'online',
                    //         'accepted_at' => time(),    
                    //         'online' => [
                    //             'ip_address' => $req->ip(),
                    //             'user_agent' => $req->header('user-agent')
                    //         ]
                    //     ]
                    // ]
                ]
            );
            // $response = $stripe->setupIntents->create(
            //     [
            //         'payment_method_types' => ['bacs_debit'],
            //         'payment_method' => $paymentMethod->id,
            //         'customer' => $customer->id,
            //         'confirm' => true,
            //     ]
            // );
            dd($response);

            $confirm_response = $stripe->paymentIntents->confirm(
                $response->id,
                [
                    'payment_method' => $paymentMethod->id,
                    'return_url' => url('/')
                ],
            );
        // } catch (Exception $e) {
        //     Utils::error($e->getMessage(), 404);
        // }

        return [
            'message' => 'Payment send successfully!'
        ];
    }
}
