<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\BankDetail;
use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\User;
use Faker\Factory;
use Illuminate\Http\Request;

class BankDetailController extends Controller
{
    /**
     * List bankDetails.
     * GET /api/bank-details
     */
    public function index(Request $req)
    {
        $bankDetails = BankDetail::orderByRaw('title DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $bankDetails->whereRaw('(name LIKE ? OR title LIKE ? )', ["%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $bankDetails->where('business_id', $query);
        }
        if ($query = $req->query('user_id', false)) {
            $bankDetails->where('user_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $bankDetails = $bankDetails->get();
        } else {
            $bankDetails = $bankDetails->paginate($req->query('per_page', 20));
        }

        if ($bankDetails->isEmpty()) Utils::error('No bankDetails found!', 404);


        return ($bankDetails);
    }

    /**
     * Create bankDetail.
     * POST /api/bank-details
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'title' => 'string|max:255',
            'bank_name' => 'required|string|max:255',
            'account_name' => 'required|string',
            'account_type' => 'required|string',
            'account_number' => 'required|string',
            'sort_code' => 'required|string',
            'bank_url' => 'url',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',

        ]);

        // create bankDetail
        $bankDetail = new BankDetail;
        $bankDetail->fill($req->all());
        $bankDetail->save();

        $bankDetails = BankDetail::orderByRaw('title DESC, created_at DESC');
        if ($bankDetail->business_id != null) {
            $bankDetails->where('business_id', $bankDetail->business_id);
        }
        if ($bankDetail->user_id != null) {
            $bankDetails->where('user_id', $bankDetail->user_id);
        }
        $bankDetails = $bankDetails->get();
        if (sizeof($bankDetails) == 1) {
            if ($bankDetail->business_id != null) {
                $business = Business::find($bankDetail->business_id);
                $business->bank_detail_id = $bankDetail->id;
                $business->save();
            }
            if ($bankDetail->user_id != null) {
                $user = User::find($bankDetail->user_id);
                $user->bank_detail_id = $bankDetail->id;
                $user->save();
            }
        }

        return $bankDetail;
    }

    /**
     * Show bankDetail.
     * GET /api/bank-details/{id}
     */
    public function show(Request $req, $id)
    {
        $bankDetail = BankDetail::find($id);
        if (empty($bankDetail)) Utils::error('BankDetail not found!', 404);
        return $bankDetail;
    }

    /**
     * Update bankDetail.
     * POST /api/bank-details/{id}
     */
    public function update(Request $req, $id)
    {
        $bankDetail = BankDetail::find($id);
        if (empty($bankDetail)) Utils::error('BankDetail not found!', 404);

        $this->validate($req, [
            'title' => 'string|max:255',
            'bank_name' => 'string|max:255',
            'account_name' => 'string',
            'account_type' => 'string',
            'account_number' => 'string',
            'sort_code' => 'string',
            'bank_url' => 'url',
            'business_id' => 'exists:businesses,id',
            'user_id' => 'exists:users,id',
        ]);

        // update bankDetail
        if ($req->has('bank_name')) $bankDetail->bank_name = $req->input('bank_name');
        if ($req->has('title')) $bankDetail->title = $req->input('title');
        if ($req->has('account_name')) $bankDetail->account_name = $req->input('account_name');
        if ($req->has('account_type')) $bankDetail->account_type = $req->input('account_type');
        if ($req->has('account_number')) $bankDetail->account_number = $req->input('account_number');
        if ($req->has('sort_code')) $bankDetail->sort_code = $req->input('sort_code');
        if ($req->has('bank_url')) $bankDetail->bank_url = $req->input('bank_url');
        if ($req->has('business_id')) $bankDetail->business_id = $req->input('business_id');
        if ($req->has('user_id')) $bankDetail->user_id = $req->input('user_id');
        $bankDetail->save();

        return $bankDetail;
    }


    /**
     * Delete bankDetail by id.
     * DELETE /api/bank-details/{id}/default
     */
    public function default(Request $req, $id)
    {
        $bankDetail = BankDetail::find($id);
        if (empty($bankDetail)) Utils::error('BankDetail not found!', 404);
        if ($bankDetail->business_id != null) {

            $business = Business::find($bankDetail->business_id);
            if (empty($bankDetail)) Utils::error('Business not found!', 404);
            $business->bank_detail_id = $bankDetail->id;
            $business->save();
        } else if ($bankDetail->user_id != null) {
            $user = User::find($bankDetail->user_id);
            if (empty($bankDetail)) Utils::error('User not found!', 404);
            $user->bank_detail_id = $bankDetail->id;
            $user->save();
        }

        return $bankDetail;
    }

    /**
     * Delete bankDetail by id.
     * DELETE /api/bank-details/{id}
     */
    public function delete(Request $req, $id)
    {
        $bankDetail = BankDetail::find($id);
        if (empty($bankDetail)) Utils::error('BankDetail not found!', 404);
        $bankDetail->delete();
        if ($bankDetail->image != null) ImageHelper::delete(ImageHelper::$TYPE_BRAND, $bankDetail->image);
        return $bankDetail;
    }
}
