<?php

namespace App\Http\Controllers;

use App\BankingRequest;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class BankingRequestController extends Controller
{
    /**
     * List banking-requests.
     * GET /api/banking-requests
     */
    public function index(Request $req)
    {
        $banking_requests = BankingRequest::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $banking_requests->whereRaw('(name LIKE ? OR email LIKE ? OR  phone LIKE ? OR  interest LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($business_id = $req->query('business_id', false)) {
            $banking_requests->where('business_id', $business_id);
        }
        if ($status = $req->query('status', false)) {
            $banking_requests->where('status', $status);
        }
        if ($req->query('nopaginate', false) == '1') {
            $banking_requests = $banking_requests->get();
        } else {
            $banking_requests = $banking_requests->paginate($req->query('per_page', 20));
        }

        if ($banking_requests->isEmpty()) Utils::error('No banking requests found!', 404);

        return $banking_requests;
    }

    /**
     * Create banking-request.
     * POST /api/banking-requests
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'name' => 'required|string',
            'email' => 'required|email',
            'phone' => 'required|string',
            'interest' => 'required|in:bank,card,both',
            'message' => 'string',
            'status' => 'in:pending,accepted,rejected',
        ]);

        // create banking_request
        $banking_request = new BankingRequest;
        $banking_request->fill($req->all());
        $banking_request->status = 'pending';
        $banking_request->save();
        return $banking_request;
    }

    /**
     * Show banking-request.
     * GET /api/banking-requests/{id}
     */
    public function show(Request $req, $id)
    {
        $banking_request = BankingRequest::find($id);
        if (empty($banking_request)) Utils::error('Banking request not found!', 404);
        return $banking_request;
    }

    /**
     * Count banking-requests.
     * GET /api/banking-requests/count
     */
    public function count(Request $req)
    {
        $bankCount = BankingRequest::orderByRaw('id DESC');
        if ($business_id = $req->query('business_id', false)) {
            $bankCount->where('business_id', $business_id);
        }
        $bankCount->where('interest', 'bank');
        $bankCount = $bankCount->count();

        $cardCount = BankingRequest::orderByRaw('id DESC');
        if ($business_id = $req->query('business_id', false)) {
            $cardCount->where('business_id', $business_id);
        }
        $cardCount->where('interest', 'card');
        $cardCount = $cardCount->count();

        $bothCount = BankingRequest::orderByRaw('id DESC');
        if ($business_id = $req->query('business_id', false)) {
            $bothCount->where('business_id', $business_id);
        }
        $bothCount->where('interest', 'both');
        $bothCount = $bothCount->count();

        $data = [
            "bank_count" => $bankCount,
            "card_count" => $cardCount,
            "both_count" => $bothCount,
        ];
        return $data;
    }

    /**
     * Update banking-request.
     * POST /api/banking-requests/{id}
     */
    public function update(Request $req, $id)
    {
        $banking_request = BankingRequest::find($id);
        if (empty($banking_request)) Utils::error('BankingRequest not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'name' => 'required|string',
            'email' => 'required|email',
            'phone' => 'required|string',
            'interest' => 'required|in:bank,card,both',
            'message' => 'string',
            'status' => 'in:pending,accepted,rejected',
        ]);

        // update banking_request
        if ($req->has('business_id')) $banking_request->business_id = $req->input('business_id');
        if ($req->has('name')) $banking_request->name = $req->input('name');
        if ($req->has('email')) $banking_request->email = $req->input('email');
        if ($req->has('phone')) $banking_request->phone = $req->input('phone');
        if ($req->has('interest')) $banking_request->interest = $req->input('interest');
        if ($req->has('message')) $banking_request->message = $req->input('message');
        if ($req->has('status')) $banking_request->status = $req->input('status');
        $banking_request->save();

        return $banking_request;
    }

    /**
     * Delete banking-request by id.
     * DELETE /api/banking-requests/{id}
     */
    public function delete(Request $req, $id)
    {
        $banking_request = BankingRequest::find($id);
        if (empty($banking_request)) Utils::error('Banking request not found!', 404);
        $banking_request->delete();
        return $banking_request;
    }
}
