<?php

namespace App\Http\Controllers;

use App\BusinessCardWallet;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class BusinessCardWalletController extends Controller
{
    /**
     * List card-wallets.
     * GET /api/card-wallets
     */
    public function index(Request $req)
    {
        $card_wallets = BusinessCardWallet::selectRaw('business_card_wallets.*,card_brands.name as brand_name')->leftJoin('card_brands', 'card_brands.id', '=', 'brand_id')->orderByRaw('business_card_wallets.id ASC');

        if ($query = $req->query('query', false)) {
            $card_wallets->whereRaw('(business_card_wallets.holder_name LIKE ? OR  business_card_wallets.card_number LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($prefilled = $req->query('prefilled', false)) {
            $card_wallets->where('business_card_wallets.brand_id', $prefilled);
        }

        if ($query = $req->query('business_id', false)) {
            $card_wallets->where('business_card_wallets.business_id', $query);
        }

        if (!$disabled = $req->query('disabled', false)) {
            $card_wallets->where('business_card_wallets.disabled', $disabled);
        }

        if ($req->query('nopaginate', false) == '1') {
            $card_wallets = $card_wallets->get();
        } else {
            $card_wallets = $card_wallets->paginate($req->query('per_page', 20));
        }

        if ($card_wallets->isEmpty()) Utils::error('No wallets found!', 404);

        if (!$prefilled = $req->query('prefilled', false)) {
            $card_wallets->each(function ($items) {
                $items->makeHidden('children');
            });
        }

        return $card_wallets;
    }

    /**
     * Create card-wallet.
     * POST /api/card-wallets
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'brand_id' => 'required|exists:card_brands,id',
            'holder_name' => 'string',
            'card_number' => 'string',
            'exp_month' => 'string',
            'exp_year' => 'string',
            'disabled' => 'boolean',
        ]);

        // create card_wallet
        $card_wallet = new BusinessCardWallet;
        $card_wallet->fill($req->all());
        $card_wallet->save();

        return $card_wallet;
    }

    /**
     * Show card-wallet.
     * GET /api/card-wallets/{id}
     */
    public function show(Request $req, $id)
    {
        $card_wallet = BusinessCardWallet::find($id);
        if (empty($card_wallet)) Utils::error('Wallet not found!', 404);
        return $card_wallet;
    }

    /**
     * Update card_wallet.
     * POST /api/card-wallets/{id}
     */
    public function update(Request $req, $id)
    {
        $card_wallet = BusinessCardWallet::find($id);
        if (empty($card_wallet)) Utils::error('Wallet not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'brand_id' => 'required|exists:card_brands,id',
            'holder_name' => 'string',
            'card_number' => 'string',
            'exp_month' => 'string',
            'exp_year' => 'string',
            'disabled' => 'boolean',
        ]);

        // update card_wallet
        if ($req->has('business_id')) $card_wallet->business_id = $req->input('business_id');
        if ($req->has('brand_id')) $card_wallet->brand_id = $req->input('brand_id');
        if ($req->has('holder_name')) $card_wallet->holder_name = $req->input('holder_name');
        if ($req->has('card_number')) $card_wallet->card_number = $req->input('card_number');
        if ($req->has('exp_month')) $card_wallet->exp_month = $req->input('exp_month');
        if ($req->has('exp_year')) $card_wallet->exp_year = $req->input('exp_year');
        $card_wallet->disabled = $req->input('disabled');

        $card_wallet->save();

        return $card_wallet;
    }

    /**
     * Delete card_wallet by id.
     * DELETE /api/card-wallets/{id}
     */
    public function delete(Request $req, $id)
    {
        $card_wallet = BusinessCardWallet::find($id);
        if (empty($card_wallet)) Utils::error('Wallet not found!', 404);
        $card_wallet->delete();
        return $card_wallet;
    }

    /**
     * Enable card_wallet.
     * POST /epos/card-wallets/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $card_wallet = BusinessCardWallet::find($id);
        if (empty($card_wallet)) Utils::error('Wallet not found!', 404);
        $card_wallet->disabled = 0;
        $card_wallet->save();
        return $card_wallet;
    }

    /**
     * Disable card_wallet.
     * POST /epos/card-wallets/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $card_wallet = BusinessCardWallet::find($id);
        if (empty($card_wallet)) Utils::error('Wallet not found!', 404);
        $card_wallet->disabled = 1;
        $card_wallet->save();
        return $card_wallet;
    }
}
