<?php

namespace App\Http\Controllers;

use App\BusinessCommission;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class BusinessCommissionController extends Controller
{
    /**
     * List businessCommissions.
     * GET /api/business-commissions
     */
    public function index(Request $req)
    {
        $businessCommissions = BusinessCommission::orderByRaw('type DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $businessCommissions->whereRaw('(type LIKE ? OR id_number LIKE ? )', ["%$query%", "%$query%"]);
        }

        if (Auth::isUser()) {
            $businessCommissions->where('user_id', Auth::id());
        } else {

            if ($query = $req->query('user_id', false)) {
                $businessCommissions->where('user_id', $query);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $businessCommissions = $businessCommissions->get();
        } else {
            $businessCommissions = $businessCommissions->paginate($req->query('per_page', 20));
        }

        if ($businessCommissions->isEmpty()) Utils::error('No businessCommissions found!', 404);


        return ($businessCommissions);
    }

    /**
     * Create businessCommission.
     * POST /api/business-commissions
     */
    public function create(Request $req)
    {
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',

            'cash_handling_fee_type' => 'required|in:flat,percentage',
            'cash_handling_fee' => 'required|numeric|min:0',
            'cash_handling_per_amount' => 'required|numeric|min:0',
            'management_fee_type' => 'required|in:flat,percentage',
            'management_fee' => 'required|numeric|min:0',
            'merchant_card_processing_fee_type' => 'required|in:flat,percentage',
            'merchant_card_processing_fee' => 'required|numeric|min:0',
            'merchant_card_auth_fee_type' => 'required|in:flat,percentage',
            'merchant_card_auth_fee' => 'required|numeric|min:0',

            'connect_card_processing_fee_type' => 'required|in:flat,percentage',
            'connect_card_processing_fee' => 'required|numeric|min:0',
            'connect_card_auth_fee_type' => 'required|in:flat,percentage',
            'connect_card_auth_fee' => 'required|numeric|min:0',

            'online_commission_fee_type' => 'required|in:flat,percentage',
            'online_commission_fee' => 'required|numeric|min:0',
            'dinein_commission_fee_type' => 'required|in:flat,percentage',
            'dinein_commission_fee' => 'required|numeric|min:0',

            'online_cod_fee_type' => 'required|in:flat,percentage',
            'online_cod_fee' => 'required|numeric|min:0',
            'online_paypal_fee_type' => 'required|in:flat,percentage',
            'online_paypal_fee' => 'required|numeric|min:0',
            'online_card_fee_type' => 'required|in:flat,percentage',
            'online_card_fee' => 'required|numeric|min:0',

            'online_service_fee_status' => 'required|in:show,hide',
            'online_service_fee_type' => 'required|in:flat,percentage',
            'online_service_fee' => 'required|numeric|min:0',
            'dinein_service_fee_status' => 'required|in:show,hide',
            'dinein_service_fee_type' => 'required|in:flat,percentage',
            'dinein_service_fee' => 'required',

            'order_charge_type' => 'required|in:order_based,transaction_based',

            'per_order_charge_type' => 'required|in:flat,percentage',
            'per_order_charge' => 'required',

            'cod_charge_type' => 'required|in:flat,percentage',
            'cod_charge' => 'required',
            'paypal_charge_type' => 'required|in:flat,percentage',
            'paypal_charge' => 'required',
            'card_charge_type' => 'required|in:flat,percentage',
            'card_charge' => 'required',
            'charge_back_fee_type' => 'required|in:flat,percentage',
            'charge_back_fee' => 'required',

            'weekly_fee_type' => 'required|in:flat,percentage',
            'weekly_fee' => 'required',
            'monthly_fee_type' => 'required|in:flat,percentage',
            'monthly_fee' => 'required',

            // 'instant_transfer_fee_type' => 'required|in:flat,percentage',
            // 'instant_transfer_fee' => 'required',

            'charges_refundable' => 'in:no,yes',
            'auto_charge_weekly' => 'in:no,yes',
            'auto_fee' => 'in:no,yes',

            'transfer_charge_type' => 'in:flat,percentage',
            'transfer_charge' => 'numeric|min:0',
            'wallet_transfer_charge_type' => 'in:flat,percentage',
            'wallet_transfer_charge' => 'numeric|min:0',
            'minimum_transfer_charge' => 'numeric|min:0',
            'transfer_charge_back' => 'numeric|min:0',
        ]);

        // create businessCommission
        $businessCommission = new BusinessCommission;
        $businessCommission->fill($req->all());

        $businessCommission->save();

        return $businessCommission;
    }

    /**
     * Show businessCommission.
     * GET /api/business-commissions/{id}
     */
    public function show(Request $req, $id)
    {
        $businessCommission = BusinessCommission::find($id);
        if (empty($businessCommission)) Utils::error('BusinessCommission not found!', 404);
        return $businessCommission;
    }

    /**
     * Update businessCommission.
     * POST /api/business-commissions/{id}
     */
    public function update(Request $req, $id)
    {
        $businessCommission = BusinessCommission::find($id);
        if (empty($businessCommission)) Utils::error('BusinessCommission not found!', 404);
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'cash_handling_fee_type' => 'required|in:flat,percentage',
            'cash_handling_fee' => 'required|numeric|min:0',
            'cash_handling_per_amount' => 'required|numeric|min:0',

            'management_fee_type' => 'required|in:flat,percentage',
            'management_fee' => 'required|numeric|min:0',
            'merchant_card_processing_fee_type' => 'required|in:flat,percentage',
            'merchant_card_processing_fee' => 'required|numeric|min:0',
            'merchant_card_auth_fee_type' => 'required|in:flat,percentage',
            'merchant_card_auth_fee' => 'required|numeric|min:0',

            'connect_card_processing_fee_type' => 'required|in:flat,percentage',
            'connect_card_processing_fee' => 'required|numeric|min:0',
            'connect_card_auth_fee_type' => 'required|in:flat,percentage',
            'connect_card_auth_fee' => 'required|numeric|min:0',

            'online_commission_fee_type' => 'required|in:flat,percentage',
            'online_commission_fee' => 'required|numeric|min:0',
            'dinein_commission_fee_type' => 'required|in:flat,percentage',
            'dinein_commission_fee' => 'required|numeric|min:0',

            'online_cod_fee_type' => 'required|in:flat,percentage',
            'online_cod_fee' => 'required|numeric|min:0',
            'online_paypal_fee_type' => 'required|in:flat,percentage',
            'online_paypal_fee' => 'required|numeric|min:0',
            'online_card_fee_type' => 'required|in:flat,percentage',
            'online_card_fee' => 'required|numeric|min:0',

            'online_service_fee_status' => 'required|in:show,hide',
            'online_service_fee_type' => 'required|in:flat,percentage',
            'online_service_fee' => 'required|numeric|min:0',
            'dinein_service_fee_status' => 'required|in:show,hide',
            'dinein_service_fee_type' => 'required|in:flat,percentage',
            'dinein_service_fee' => 'required',

            'order_charge_type' => 'required|in:order_based,transaction_based',

            'per_order_charge_type' => 'required|in:flat,percentage',
            'per_order_charge' => 'required',

            'cod_charge_type' => 'required|in:flat,percentage',
            'cod_charge' => 'required',
            'paypal_charge_type' => 'required|in:flat,percentage',
            'paypal_charge' => 'required',
            'card_charge_type' => 'required|in:flat,percentage',
            'card_charge' => 'required',
            'charge_back_fee_type' => 'required|in:flat,percentage',
            'charge_back_fee' => 'required',


            'weekly_fee_type' => 'required|in:flat,percentage',
            'weekly_fee' => 'required',
            'monthly_fee_type' => 'required|in:flat,percentage',
            'monthly_fee' => 'required',

            // 'instant_transfer_fee_type' => 'required|in:flat,percentage',
            // 'instant_transfer_fee' => 'required',

            'charges_refundable' => 'in:no,yes',
            'auto_charge_weekly' => 'in:no,yes',
            'auto_fee' => 'in:no,yes',

            'transfer_charge_type' => 'required|in:flat,percentage',
            'transfer_charge' => 'numeric|min:0',
            'wallet_transfer_charge_type' => 'required|in:flat,percentage',
            'wallet_transfer_charge' => 'numeric|min:0',
            'minimum_transfer_charge' => 'numeric|min:0',
            'transfer_charge_back' => 'numeric|min:0',
        ]);

        // update businessCommission
        if ($req->has('cash_handling_fee_type')) $businessCommission->cash_handling_fee_type = $req->input('cash_handling_fee_type');
        $businessCommission->cash_handling_fee = $req->input('cash_handling_fee');
        $businessCommission->cash_handling_per_amount = $req->input('cash_handling_per_amount');
        if ($req->has('management_fee_type')) $businessCommission->management_fee_type = $req->input('management_fee_type');
        $businessCommission->management_fee = $req->input('management_fee');
        if ($req->has('merchant_card_processing_fee_type')) $businessCommission->merchant_card_processing_fee_type = $req->input('merchant_card_processing_fee_type');
        $businessCommission->merchant_card_processing_fee = $req->input('merchant_card_processing_fee');
        if ($req->has('merchant_card_auth_fee_type')) $businessCommission->merchant_card_auth_fee_type = $req->input('merchant_card_auth_fee_type');
        $businessCommission->merchant_card_auth_fee = $req->input('merchant_card_auth_fee');

        if ($req->has('connect_card_processing_fee_type')) $businessCommission->connect_card_processing_fee_type = $req->input('connect_card_processing_fee_type');
        $businessCommission->connect_card_processing_fee = $req->input('connect_card_processing_fee');
        if ($req->has('connect_card_auth_fee_type')) $businessCommission->connect_card_auth_fee_type = $req->input('connect_card_auth_fee_type');
        $businessCommission->connect_card_auth_fee = $req->input('connect_card_auth_fee');

        if ($req->has('online_commission_fee_type')) $businessCommission->online_commission_fee_type = $req->input('online_commission_fee_type');
        $businessCommission->online_commission_fee = $req->input('online_commission_fee');
        if ($req->has('dinein_commission_fee_type')) $businessCommission->dinein_commission_fee_type = $req->input('dinein_commission_fee_type');
        $businessCommission->dinein_commission_fee = $req->input('dinein_commission_fee');

        if ($req->has('online_cod_fee_type')) $businessCommission->online_cod_fee_type = $req->input('online_cod_fee_type');
        $businessCommission->online_cod_fee = $req->input('online_cod_fee');
        if ($req->has('online_paypal_fee_type')) $businessCommission->online_paypal_fee_type = $req->input('online_paypal_fee_type');
        $businessCommission->online_paypal_fee = $req->input('online_paypal_fee');
        if ($req->has('online_card_fee_type')) $businessCommission->online_card_fee_type = $req->input('online_card_fee_type');
        $businessCommission->online_card_fee = $req->input('online_card_fee');

        if ($req->has('online_service_fee_status')) $businessCommission->online_service_fee_status = $req->input('online_service_fee_status');
        if ($req->has('online_service_fee_type')) $businessCommission->online_service_fee_type = $req->input('online_service_fee_type');
        $businessCommission->online_service_fee = $req->input('online_service_fee');
        if ($req->has('dinein_service_fee_status')) $businessCommission->dinein_service_fee_status = $req->input('dinein_service_fee_status');
        if ($req->has('dinein_service_fee_type')) $businessCommission->dinein_service_fee_type = $req->input('dinein_service_fee_type');
        $businessCommission->dinein_service_fee = $req->input('dinein_service_fee');

        if ($req->has('order_charge_type')) $businessCommission->order_charge_type = $req->input('order_charge_type');

        if ($req->has('per_order_charge_type')) $businessCommission->per_order_charge_type = $req->input('per_order_charge_type');
        $businessCommission->per_order_charge = $req->input('per_order_charge');

        if ($req->has('cod_charge_type')) $businessCommission->cod_charge_type = $req->input('cod_charge_type');
        $businessCommission->cod_charge = $req->input('cod_charge');
        if ($req->has('paypal_charge_type')) $businessCommission->paypal_charge_type = $req->input('paypal_charge_type');
        $businessCommission->paypal_charge = $req->input('paypal_charge');
        if ($req->has('card_charge_type')) $businessCommission->card_charge_type = $req->input('card_charge_type');
        $businessCommission->card_charge = $req->input('card_charge');

        if ($req->has('charge_back_fee_type')) $businessCommission->charge_back_fee_type = $req->input('charge_back_fee_type');
        $businessCommission->charge_back_fee = $req->input('charge_back_fee');

        if ($req->has('weekly_fee_type')) $businessCommission->weekly_fee_type = $req->input('weekly_fee_type');
        $businessCommission->weekly_fee = $req->input('weekly_fee');
        if ($req->has('monthly_fee_type')) $businessCommission->monthly_fee_type = $req->input('monthly_fee_type');
        $businessCommission->monthly_fee = $req->input('monthly_fee');

        if ($req->has('instant_transfer_fee_type')) $businessCommission->instant_transfer_fee_type = $req->input('instant_transfer_fee_type');
        $businessCommission->instant_transfer_fee = $req->input('instant_transfer_fee');

        $businessCommission->connect_monthly_fee = $req->input('connect_monthly_fee');
        $businessCommission->connect_weekly_fee = $req->input('connect_weekly_fee');

        if ($req->has('charges_refundable')) $businessCommission->charges_refundable = $req->input('charges_refundable');
        if ($req->has('auto_charge_weekly')) $businessCommission->auto_charge_weekly = $req->input('auto_charge_weekly');
        if ($req->has('auto_fee')) $businessCommission->auto_fee = $req->input('auto_fee');

        if ($req->has('transfer_charge_type')) $businessCommission->transfer_charge_type = $req->input('transfer_charge_type');
        $businessCommission->transfer_charge = $req->input('transfer_charge');
        if ($req->has('wallet_transfer_charge_type')) $businessCommission->wallet_transfer_charge_type = $req->input('wallet_transfer_charge_type');
        $businessCommission->wallet_transfer_charge = $req->input('wallet_transfer_charge');
        $businessCommission->minimum_transfer_charge = $req->input('minimum_transfer_charge');
        $businessCommission->transfer_charge_back = $req->input('transfer_charge_back');


        $businessCommission->save();

        return $businessCommission;
    }


    /**
     * Delete businessCommission by id.
     * DELETE /api/business-commissions/{id}
     */
    public function delete(Request $req, $id)
    {
        $businessCommission = BusinessCommission::find($id);
        if (empty($businessCommission)) Utils::error('BusinessCommission not found!', 404);
        $businessCommission->delete();
        return $businessCommission;
    }
}