<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\Business;
use App\BusinessCommission;
use App\BusinessDbServer;
use App\BusinessPlan;
use App\BusinessPrinter;
use App\CardHolder;
use App\Country;
use App\CardBrand;
use App\CardBrandCommission;
use App\CountryCommission;
use App\DatabaseServer;
use App\DeviceRegistration;
use App\Helpers\Auth;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Epos\EposDeviceController;
use App\Http\Controllers\Epos\EposPrintSettingController;
use App\Http\Controllers\Epos\EposSettingController;
use App\Http\Controllers\Tiffintom\TiffintomRestaurantController;
use App\ModelsEpos\EposDevice;
use App\ModelsEpos\EposPrinter;
use App\ModelsEpos\EposPrinterModel;
use App\ModelsEpos\EposPrintSetting;
use App\ModelsEpos\EposSetting;
use App\Plan;
use App\Printer;
use App\UserBusiness;
use Carbon\Carbon;
use Exception;
use Faker\Factory;
use Illuminate\Http\Request;
use GuzzleHttp\Client;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;
use mysqli;
use PDO;

class BusinessController extends Controller
{
    /**
     * List businesses.
     * GET /api/businesses
     */
    public function index(Request $req)
    {
        if ($req->query('pouch_weight') != null && $req->query('pouch_weight') != 'null') {
            $pouch_weight = $req->query('pouch_weight');
            $businesses = Business::orderByRaw('pouch_weight ' . $pouch_weight);
        } else {
            $businesses = Business::orderByRaw('id DESC');
        }

        if ($query = $req->query('query', false)) {
            $businesses->whereRaw('(name LIKE ? OR trading_name LIKE ? OR email LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('service', false)) {
            if ($query == 'online_service') {
                $businesses->where('online_service', '1');
            }
            if ($query == 'epos_service') {
                $businesses->where('epos_service', '1');
            }
            if ($query == 'merchant_service') {
                $businesses->where('merchant_service', '1');
            }
            if ($query == 'connect_service') {
                $businesses->where('connect_service', '1');
            }
            if ($query == 'issuing_service') {
                $businesses->where('issuing_service', '1');
            }
            if ($query == 'extension_service') {
                $businesses->where('extension_service', '1');
            }
            if ($query == 'pouch_service') {
                $businesses->where('pouch_service', '1');
            }
        }
        if ($req->query('status') != null && $req->query('status') != 'null') {
            $status = $req->query('status');
            if ($status == 'not_activated') {
                $businesses->whereNull('s_account_id')->where('connect_service', '1');
            } else {
                if ($status != 'all') {
                    $businesses->where('disabled', $status == 0 ? '0' : '1');
                }
            }
        }
        if ($req->query('online_service', false) == '1') {
            $businesses->where('online_service', '1');
        }
        if ($req->query('merchant_service', false) == '1') {
            $businesses->where('merchant_service', '1');
        }
        if ($req->query('epos_service', false) == '1') {
            $businesses->where('epos_service', '1');
        }
        if ($req->query('connect_service', false) == '1') {
            $businesses->where('connect_service', '1');
        }
        if ($req->query('issuing_service', false) == '1') {
            $businesses->where('issuing_service', '1');
        }
        if ($req->query('extension_service', false) == '1') {
            $businesses->where('extension_service', '1');
        }
        if ($req->query('pouch_service', false) == '1') {
            $businesses->where('pouch_service', '1');
        }
        if (Auth::isUser() || Auth::guest()) {
            $businesses->where('disabled', 0);
        } else {
            $query = $req->query('disabled');
            if ($query != null) {
                $businesses->where('disabled', $query == 0 ? '0' : '1');
            }
        }
        if (Auth::isUser()) {
            if (Auth::user()->reseller) {
                $loggedInUser = Auth::user();
                $businesses->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$loggedInUser->id]);
            } else {
                Utils::error('You not authorised to access!', 401, 401);
            }
        }

        if ($req->query('nopaginate', false) == '1') {
            $businesses = $businesses->get();
        } else {
            $businesses = $businesses->paginate($req->query('per_page', 20));
        }

        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);


        return ($businesses);
    }

    /**
     * List businesses.
     * GET /api/businesses/mini
     */
    public function mini(Request $req)
    {
        $businesses = Business::orderByRaw('name ASC');
        $businesses->where('disabled', 0);
        if ($query = $req->query('query', false)) {
            $businesses->whereRaw('(name LIKE ? OR trading_name LIKE ? OR email LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }
        if ($req->has('user_id')) {
            $businesses->whereRaw('(id NOT IN (SELECT business_id from user_businesses where user_id = ?))', [$req->input('user_id')]);
        }
        if ($req->query('pouch_service', false) == '1') {
            $businesses->where('pouch_service', '1');
        }
        if ($req->query('issuing_service', false) == '1') {
            $businesses->where('issuing_service', '1');
        }
        $businesses = $businesses->get();
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);

        $businesses->each(function ($business) {
            $business->makeHidden(['business_commissions']);
        });
        return ($businesses);
    }

    /**
     * List businesses.
     * GET /api/businesses/company-number/{id}
     */
    public function fetch_company_detail(Request $req, $id)
    {
        try {
            $company = (new Client())->request(
                'GET',
                'https://api.company-information.service.gov.uk/company/' . $id,
                ['headers' => ['Authorization' => 'Basic Mjc4Njk4NGMtOGE0MC00Mzc1LTlmOWQtMjU2ZjE3OWNhMTdjOg==']]
            );
            if ($company->getStatusCode() == 200) {
                return $company->getBody();
            } else {
                Utils::error('Company Details not found!', 404);
            }
        } catch (Exception $e) {
            Utils::error('Company Details not found!', 404);
        }
    }

    /**
     * List businesses.
     * GET /api/businesses/extension-account/{id}
     */
    public function extension_account(Request $req, $id)
    {
        $business = Business::where('extension_account_id', $id)->first();
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->append('default_bank_detail');
        if (!empty($business->default_bank_detail)) {
            $business->default_bank_detail->makeHidden('is_default');
        }

        return $business;
    }

    /**
     * Create business.
     * POST /api/businesses
     */
    public function create(Request $req)
    {
        if (!is_array($req->auto_accept_fields)) {
            $req->merge(['auto_accept_fields' => (json_decode(stripslashes($req->auto_accept_fields), true))]);
        }
        if (!is_array($req->business_commissions))
            $req->merge(['business_commissions' => (json_decode($req->business_commissions, true))]);
        if (!is_array($req->admin_permissions))
            $req->merge(['admin_permissions' => (json_decode($req->admin_permissions, true))]);
        if (!is_array($req->supervisor_permissions))
            $req->merge(['supervisor_permissions' => (json_decode($req->supervisor_permissions, true))]);
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'trading_name' => 'required|string|max:255',
            'door_no' => 'string',
            'address' => 'required|string',
            'city' => 'string',
            'state' => 'string',
            'postcode' => 'string',
            'email' => 'required|email|unique:businesses,email',
            'contact_numbers' => 'required|string',
            'company_number' => 'string',
            'joining_date' => 'date',
            'domain' => 'string',
            'domain_valid_till' => 'date',
            'type' => 'required|in:Sole trader,Partnership,Limited company,Not registered comapny,Other',
            'online_service' => 'required|boolean',
            'merchant_service' => 'required|boolean',
            'statement_description' => 'required_if:merchant_service,1|string|min:5|max:22',
            'statement_descriptor' => 'required_if:merchant_service,1|string|min:5|max:22',
            'online_statement_description' => 'required_if:online_service,1|string|min:5|max:22',
            'online_statement_descriptor' => 'required_if:online_service,1|string|min:5|max:22',
            'restaurant_id' => 'string',
            'country_id' => 'required|exists:countries,id',
            'online_paypal_description' => 'string',
            'restaurant_name' => 'string',
            'online_order_limit' => 'numeric',
            'auto_accept_order' => 'in:yes,no',
            'auto_accept_reservation' => 'in:yes,no',
            'service_charge_type' => 'required_if:online_service,1|in:hide,show',

            'payout_mode' => 'in:Auto,Manual',
            'payout_method' => 'in:Bank,Paypal,Cash',
            'archived_payout_mode' => 'in:Auto,Manual',
            'archived_payout_method' => 'in:Bank,Paypal,Cash',
            'paypal_mode' => 'in:Test,Live',
            'stripe_mode' => 'in:Test,Live',

            'invoice_interval' => 'numeric',
            'invoice_day' => 'in:sunday,monday,tuesday,wednesday,thursday,friday,saturday',

            'footer_a' => 'string',
            'footer_b' => 'string',
            'header_a' => 'string',
            'header_b' => 'string',
            'header_bill_a' => 'string',
            'footer_bill_a' => 'string',
            'epos_device_allow' => 'numeric',
            'merchant_device_allow' => 'numeric',
            'connect_service' => 'boolean',
            'issuing_service' => 'boolean',
            'extension_service' => 'boolean',
            'pouch_service' => 'boolean',
            'load_wallet' => 'boolean',
            'load_bank' => 'boolean',
            'qr_permission' => 'boolean',
            'auto_merchant_print' => 'boolean',
            'auto_customer_print' => 'boolean',
            'send_receipt' => 'boolean',

            'business_commissions' => 'required|array',
            'admin_permissions' => 'required_if:merchant_service,1|array',
            'supervisor_permissions' => 'required_if:merchant_service,1|array',
            'epos_service' => 'required|boolean',
            'auto_archive' => 'boolean',
            'tip' => 'boolean',
            'print_logo' => 'boolean',
            'day_of_transaction_limit' => 'numeric',
            'delivery_number_of_print' => 'numeric',
            'collection_number_of_print' => 'numeric',
            'online_order_number_of_print' => 'numeric',
            'dinein_number_of_print' => 'numeric',
            'gallery_video_visible' => 'boolean',
            'gallery_image_visible' => 'boolean',
            'is_qr_print_on_bill' => 'boolean',
            'android_restaurant_app_version' => 'numeric',
            'ios_restaurant_app_version' => 'string',
            'android_partner_app_version' => 'string',
            'ios_partner_app_version' => 'string',
            'db_name' => 'string',
            'database_server_id' => 'string|required_if:epos_service,1|exists:database_servers,id',
            'image' => 'image',
            'statement_transaction_view' => 'in:Normal,Archived,Both,None',

            'transfer_connect' => 'in:yes,no',
            'extension_account_id' => 'string',
            'access_number' => 'string',
        ]);
        // if ($req->has('db_name')) {
        //     $dbNameCount = Business::where('db_name', $req->input('db_name'))->count();
        //     if ($dbNameCount > 0) Utils::error('Database with same name already exists', 400);
        // }
        // create business
        $business = new Business;
        $business->fill($req->all());
        if ($business->epos_service) $business->db_name = $this->checkAndGetRandomName();

        //Extension Id Generate
        if ($req->input('extension_service') == '1') {
            $business->extension_account_id = 'ubex_' . substr(str_shuffle('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'), 1, 9);
        }
        $business->issuing_service = $req->input('issuing_service');
        //Access Number Generate
        if ($req->input('issuing_service') == '1' && !$req->has('access_number')) {
            $business->access_number = substr(str_shuffle('0123456789'), 1, 4);
        }

        if ($req->has('online_statement_descriptor') && !empty(trim($req->input('online_statement_descriptor')))) {
            $businessWithStatement = Business::where('online_statement_descriptor', $business->online_statement_descriptor)->count();
            if ($businessWithStatement > 0) Utils::error($req->input('online_statement_descriptor') . " Online Descriptor Duplicate", 400);
        }
        if ($req->has('statement_descriptor') && !empty(trim($req->input('statement_descriptor')))) {
            $businessWithStatement = Business::where('statement_descriptor', $business->statement_descriptor)->count();
            if ($businessWithStatement > 0) Utils::error($req->input('statement_descriptor') . " Statement Descriptor Duplicate", 400);
        }
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_BUSINESS);
            $business->image = $fileName;
        }

        $business->save();

        if ($req->has('business_commissions')) {
            $req->merge(($req->business_commissions));
            $req->merge(['business_id' => $business->id]);
            (new BusinessCommissionController())->create($req);
        } else {
            $businessCommission = new BusinessCommission();
            $businessCommission->business_id = $business->id;
            $businessCommission->save();
        }
        $req->merge(['business_id' => $business->id]);
        if ($req->has('admin_permissions')) {
            $req = $req->merge($req->admin_permissions);
            $req->merge(['role' => "admin"]);
            $admin_permissions = (new BusinessPermissionController())->create_update($req);
        }
        if ($req->has('supervisor_permissions')) {
            $req = $req->merge($req->supervisor_permissions);
            $req->merge(['role' => "supervisor"]);
            $supervisor_permissions = (new BusinessPermissionController())->create_update($req);
        }

        if ($business->epos_service) {
            $plan = Plan::where('service', 'Epos')->orderByRaw('id ASC')->first();
            if (!empty($plan)) {
                $businessPlan = new BusinessPlan;
                $businessPlan->business_id = $business->id;
                $businessPlan->plan_id = $plan->id;
                $businessPlan->service = 'Epos';
                $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                $businessPlan->save();
            }
        }

        if ($business->online_service) {
            $plan = Plan::where('service', 'Online')->orderByRaw('id ASC')->first();
            if (!empty($plan)) {
                $businessPlan = new BusinessPlan;
                $businessPlan->business_id = $business->id;
                $businessPlan->plan_id = $plan->id;
                $businessPlan->service = 'Online';
                $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                $businessPlan->save();
            }
        }

        if ($business->merchant_service) {
            $plan = Plan::where('service', 'Merchant')->orderByRaw('id ASC')->first();
            if (!empty($plan)) {
                $businessPlan = new BusinessPlan;
                $businessPlan->business_id = $business->id;
                $businessPlan->plan_id = $plan->id;
                $businessPlan->service = 'Merchant';
                $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                $businessPlan->save();
            }
        }

        $business->append('database_server');
        $this->getRestaurantDetails($business);
        $business->admin_permissions = $admin_permissions;
        $business->supervisor_permissions = $supervisor_permissions;
        return $business;
    }


    /**
     * Create business.
     * POST /api/businesses
     */
    public function direct_create(Request $req)
    {
        if (!is_array($req->business_commissions))
            $req->merge(['business_commissions' => (json_decode($req->business_commissions, true))]);
        if (!is_array($req->admin_permissions))
            $req->merge(['admin_permissions' => (json_decode($req->admin_permissions, true))]);
        if (!is_array($req->supervisor_permissions))
            $req->merge(['supervisor_permissions' => (json_decode($req->supervisor_permissions, true))]);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'trading_name' => 'string|max:255',
            'door_no' => 'string',
            'address' => 'required|string',
            'city' => 'string',
            'state' => 'string',
            'postcode' => 'string',
            'email' => 'required|email|unique:businesses,email',
            'contact_numbers' => 'required|string',
            'company_number' => 'string',
            'joining_date' => 'date',
            'type' => 'required|in:Sole trader,Partnership,Limited company,Not registered comapny,Other',
            'country_id' => 'required|exists:countries,id',
        ]);

        // create business
        $business = new Business;
        $business->fill($req->all());

        // if ($req->has('image')) {
        //     $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_BUSINESS);
        //     $business->image = $fileName;
        // }

        if ($req->input('merchant_service') == 1 || $req->input('online_service') == 1) {
            $business->connect_service = 1;
        }
        $business->invoice_interval = 7;
        $business->day_of_transaction_limit = 3;
        $business->header_a = $req->input('name');
        if ($req->has('name') && $req->input('merchant_service') == 1) {
            $business->statement_description = substr($req->input('name'), 0, 16) . ' L';
            $business->statement_descriptor = substr($req->input('name'), 0, 16) . ' L';
        }
        if ($req->has('name') && $req->input('online_service') == 1) {
            $business->online_statement_description = substr($req->input('name'), 0, 16) . ' OL';
            $business->online_statement_descriptor = substr($req->input('name'), 0, 16) . ' OL';
        }
        $business->disabled = 1;
        $business->save();

        // Business wise commission add
        if ($req->has('business_commissions')) {
            $req->merge(($req->business_commissions));
            $req->merge(['business_id' => $business->id]);
            (new BusinessCommissionController())->create($req);

            // Counry wise commision add
            $countryCommission = new CountryCommission();
            $countryCommission->business_id = $business->id;
            $countryCommission->stripe_percentage = 3.5;
            $countryCommission->stripe_fixed = 0.25;
            $countryCommission->other = 1;
            $countryCommission->save();

            $countries = Country::where('currency_code', 'GBP')->first();
            if (!empty($countries)) {
                $countryCommission = new CountryCommission();
                $countryCommission->business_id = $business->id;
                $countryCommission->country_id = $countries->id;
                $countryCommission->stripe_percentage = 0;
                $countryCommission->stripe_fixed = 0;
                $countryCommission->other = 0;
                $countryCommission->save();
            }

            // Card wise commision add
            $cardBrand = CardBrand::where('name', 'AMEX')->where('code', '34')->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = new CardBrandCommission();
                $cardBrandCommission->business_id = $business->id;
                $cardBrandCommission->card_brand_id = $cardBrand->id;
                $cardBrandCommission->stripe_percentage = 1.55;
                $cardBrandCommission->stripe_fixed = 0.00;
                $cardBrandCommission->save();
            }

            $cardBrand = CardBrand::where('name', 'AMEX')->where('code', '37')->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = new CardBrandCommission();
                $cardBrandCommission->business_id = $business->id;
                $cardBrandCommission->card_brand_id = $cardBrand->id;
                $cardBrandCommission->stripe_percentage = 1.55;
                $cardBrandCommission->stripe_fixed = 0.00;
                $cardBrandCommission->save();
            }
        } else {
            $businessCommission = new BusinessCommission();
            $businessCommission->business_id = $business->id;
            $businessCommission->save();
        }

        // Admin Permission add
        $req->merge(['business_id' => $business->id]);
        if ($req->has('admin_permissions')) {
            $req = $req->merge($req->admin_permissions);
            $req->merge(['role' => "admin"]);
            $admin_permissions = (new BusinessPermissionController())->create_update($req);
        }

        // Supervisor permission add
        if ($req->has('supervisor_permissions')) {
            $req = $req->merge($req->supervisor_permissions);
            $req->merge(['role' => "supervisor"]);
            $supervisor_permissions = (new BusinessPermissionController())->create_update($req);
        }

        // Database name assign business
        if ($business->epos_service) {
            $dataServer = DatabaseServer::orderByRaw('id DESC')->first();
            if (!empty($dataServer)) {
                $business->db_name = $this->checkAndGetRandomName();
                $business->database_server_id = $dataServer->id;
            }
        }

        // $business->append('database_server');
        // $this->getRestaurantDetails($business);
        $business->admin_permissions = $admin_permissions;
        $business->supervisor_permissions = $supervisor_permissions;
        return $business;
    }


    /**
     * Show business.
     * GET /api/businesses/{id}
     */
    public function show(Request $req, $id)
    {
        $business = Business::find($id);
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->append('default_bank_detail');
        $business->append('default_paypal_detail');
        $business->append('admin_permissions');
        $business->append('supervisor_permissions');
        $business->append('database_server');
        if (!empty($business->default_bank_detail)) {
            $business->default_bank_detail->makeHidden('is_default');
        }
        if (!empty($business->default_paypal_detail)) {
            $business->default_paypal_detail->makeHidden('is_default');
        }
        return $business;
    }

    /**
     * Update business.
     * POST /api/businesses/{id}
     */
    public function update(Request $req, $id)
    {
        $business = Business::find($id);
        if (empty($business)) Utils::error('Business not found!', 404);

        if (!is_array($req->auto_accept_fields)) {
            $req->merge(['auto_accept_fields' => (json_decode(stripslashes($req->auto_accept_fields), true))]);
        }

        if (!is_array($req->business_commissions))
            $req->merge(['business_commissions' => (json_decode($req->business_commissions, true))]);

        if (!is_array($req->admin_permissions))
            $req->merge(['admin_permissions' => (json_decode($req->admin_permissions, true))]);
        if (!is_array($req->supervisor_permissions))
            $req->merge(['supervisor_permissions' => (json_decode($req->supervisor_permissions, true))]);

        $this->validate($req, [
            'name' => 'string|max:255',
            'trading_name' => 'string|max:255',
            'door_no' => 'string',
            'address' => 'string',
            'city' => 'string',
            'state' => 'string',
            'postcode' => 'string',
            'email' => ['email', Rule::unique('businesses')->ignore($business->id)],
            'contact_numbers' => 'string',
            'company_number' => 'string',
            'joining_date' => 'date',
            'domain' => 'string',
            'domain_valid_till' => 'date',
            'type' => 'in:Sole trader,Partnership,Limited company,Not registered comapny,Other',

            'online_service' => 'required|boolean',
            'merchant_service' => 'required|boolean',
            'statement_description' => 'required_if:merchant_service,1|string|max:22|min:5',
            'statement_descriptor' => 'required_if:merchant_service,1|string|max:22|min:5',
            'online_statement_description' => 'required_if:online_service,1|string|max:22|min:5',
            'online_statement_descriptor' => 'required_if:online_service,1|string|max:22|min:5',
            'service_charge_type' => 'required_if:online_service,1|in:hide,show',

            'restaurant_id' => 'string',
            'online_paypal_description' => 'string',
            'restaurant_name' => 'string',
            'online_order_limit' => 'numeric',
            'auto_accept_order' => 'in:yes,no',
            'auto_accept_reservation' => 'in:yes,no',
            'country_id' => 'exists:countries,id',

            'payout_mode' => 'in:Auto,Manual',
            'payout_method' => 'in:Bank,Paypal,Cash',
            'archived_payout_mode' => 'in:Auto,Manual',
            'archived_payout_method' => 'in:Bank,Paypal,Cash',

            'invoice_interval' => 'numeric',
            'invoice_day' => 'in:sunday,monday,tuesday,wednesday,thursday,friday,saturday',

            'paypal_mode' => 'in:Test,Live',
            'stripe_mode' => 'in:Test,Live',

            'footer_a' => 'string',
            'footer_b' => 'string',
            'header_a' => 'string',
            'header_b' => 'string',
            'header_bill_a' => 'string',
            'footer_bill_a' => 'string',
            'epos_device_allow' => 'numeric',
            'merchant_device_allow' => 'numeric',

            'business_commissions' => 'required|array',
            'admin_permissions' => 'required_if:merchant_service,1|array',
            'supervisor_permissions' => 'required_if:merchant_service,1|array',

            'connect_service' => 'boolean',
            'issuing_service' => 'boolean',
            'extension_service' => 'boolean',
            'pouch_service' => 'boolean',
            'load_wallet' => 'boolean',
            'load_bank' => 'boolean',
            'qr_permission' => 'boolean',
            'auto_merchant_print' => 'boolean',
            'auto_customer_print' => 'boolean',
            'send_receipt' => 'boolean',

            'epos_service' => 'boolean',
            'tip' => 'boolean',
            'print_logo' => 'boolean',
            'day_of_transaction_limit' => 'numeric',
            'delivery_number_of_print' => 'numeric',
            'collection_number_of_print' => 'numeric',
            'online_order_number_of_print' => 'numeric',
            'dinein_number_of_print' => 'numeric',
            'gallery_video_visible' => 'boolean',
            'gallery_image_visible' => 'boolean',
            'android_restaurant_app_version' => 'numeric',
            'ios_restaurant_app_version' => 'string',
            'android_partner_app_version' => 'string',
            'ios_partner_app_version' => 'string',
            'db_name' => 'string',
            'database_server_id' => 'string|required_if:epos_service,1|exists:database_servers,id',
            'image' => 'image',
            'statement_transaction_view' => 'in:Normal,Archived,Both,None',
            'transfer_connect' => 'in:yes,no',
            'extension_account_id' => 'string',
            'access_number' => 'string',
            'pouch_weight' => 'numeric',
            'is_qr_print_on_bill' => 'boolean',
        ]);
        // check if db name is already taken or not
        // if ($req->has('db_name')) {
        //     $dbNameCount = Business::where('db_name', $req->input('db_name'))->where('id', '!=', $id)->count();
        //     if ($dbNameCount > 0) Utils::error('Database with same name already exists', 400);
        // }

        if ($req->has('online_statement_descriptor') && !empty(trim($req->input('statement_descriptor')))) {
            $businessWithStatement = Business::where('online_statement_descriptor', $req->input('online_statement_descriptor'))->whereNotIn('id', [$id])->count();
            if (($businessWithStatement > 0)) Utils::error($req->input('online_statement_descriptor') . " Online Descriptor Duplicate", 400);
        }
        if ($req->has('statement_descriptor') && !empty(trim($req->input('statement_descriptor')))) {
            $businessWithStatement = Business::where('statement_descriptor', $req->input('statement_descriptor'))->whereNotIn('id', [$id])->count();
            if (($businessWithStatement > 0)) Utils::error($req->input('statement_descriptor') . " Statement Descriptor Duplicate", 400);
        }

        // update business
        if ($req->has('name')) $business->name = $req->input('name');
        if ($req->has('trading_name')) $business->trading_name = $req->input('trading_name');
        if ($req->has('door_no')) $business->door_no = $req->input('door_no');
        if ($req->has('address')) $business->address = $req->input('address');
        if ($req->has('city')) $business->city = $req->input('city');
        if ($req->has('state')) $business->state = $req->input('state');
        if ($req->has('postcode')) $business->postcode = $req->input('postcode');
        if ($req->has('email')) $business->email = $req->input('email');
        if ($req->has('contact_numbers')) $business->contact_numbers = $req->input('contact_numbers');
        if ($req->has('company_number')) $business->company_number = $req->input('company_number');
        if ($req->has('joining_date')) $business->joining_date = $req->input('joining_date');
        if ($req->has('domain')) $business->domain = $req->input('domain');
        if ($req->has('domain_valid_till')) $business->domain_valid_till = $req->input('domain_valid_till');
        if ($req->has('type')) $business->type = $req->input('type');
        if ($req->has('country_id')) $business->country_id = $req->input('country_id');

        if ($req->has('statement_description')) $business->statement_description = $req->input('statement_description');
        if ($req->has('statement_descriptor')) $business->statement_descriptor = $req->input('statement_descriptor');
        if ($req->has('online_statement_description')) $business->online_statement_description = $req->input('online_statement_description');
        if ($req->has('online_statement_descriptor')) $business->online_statement_descriptor = $req->input('online_statement_descriptor');
        if ($req->has('service_charge_type')) $business->service_charge_type = $req->input('service_charge_type');
        if ($req->has('online_order_limit')) $business->online_order_limit = $req->input('online_order_limit');
        if ($req->has('auto_accept_order')) $business->auto_accept_order = $req->input('auto_accept_order');
        if ($req->has('auto_accept_reservation')) $business->auto_accept_reservation = $req->input('auto_accept_reservation');
        if ($req->has('auto_accept_fields')) $business->auto_accept_fields = $req->input('auto_accept_fields');

        if ($req->has('restaurant_id')) $business->restaurant_id = $req->input('restaurant_id');
        if ($req->has('online_paypal_description')) $business->online_paypal_description = $req->input('online_paypal_description');
        if ($req->has('restaurant_name')) $business->restaurant_name = $req->input('restaurant_name');
        if ($req->has('payout_mode')) $business->payout_mode = $req->input('payout_mode');
        if ($req->has('payout_method')) $business->payout_method = $req->input('payout_method');
        if ($req->has('archived_payout_mode')) $business->archived_payout_mode = $req->input('archived_payout_mode');
        if ($req->has('archived_payout_method')) $business->archived_payout_method = $req->input('archived_payout_method');
        if ($req->has('payout_comment')) $business->payout_comment = $req->input('payout_comment');

        if ($req->has('invoice_interval')) $business->invoice_interval = $req->input('invoice_interval');
        if ($req->has('invoice_day')) $business->invoice_day = $req->input('invoice_day');

        if ($req->has('supervisor_password')) $business->supervisor_password = $req->input('supervisor_password');
        if ($req->has('admin_password')) $business->admin_password = $req->input('admin_password');

        if ($req->has('paypal_mode')) $business->paypal_mode = $req->input('paypal_mode');

        if ($req->has('delivery_number_of_print')) $business->delivery_number_of_print = $req->input('delivery_number_of_print');
        if ($req->has('collection_number_of_print')) $business->collection_number_of_print = $req->input('collection_number_of_print');
        if ($req->has('online_order_number_of_print')) $business->online_order_number_of_print = $req->input('online_order_number_of_print');
        if ($req->has('dinein_number_of_print')) $business->dinein_number_of_print = $req->input('dinein_number_of_print');
        if ($req->has('day_of_transaction_limit')) $business->day_of_transaction_limit = $req->input('day_of_transaction_limit');
        if ($req->has('statement_transaction_view')) $business->statement_transaction_view = $req->input('statement_transaction_view');
        if ($req->has('transfer_connect')) $business->transfer_connect = $req->input('transfer_connect');

        //Extension Id Generate
        if ($req->input('extension_service') == 1 && $business->extension_account_id == '') {
            $business->extension_account_id = 'ubex_' . substr(str_shuffle('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'), 1, 9);
        }
        //Access Number Generate
        if ($req->has('access_number')) {
            $business->access_number = $req->input('access_number');
        } else {
            $business->access_number = substr(str_shuffle('0123456789'), 1, 4);
        }

        $business->paypal_public_key_test = $req->input('paypal_public_key_test');
        $business->paypal_private_key_test = $req->input('paypal_private_key_test');
        $business->paypal_public_key_live = $req->input('paypal_public_key_live');
        $business->paypal_private_key_live = $req->input('paypal_private_key_live');

        if ($req->has('stripe_mode')) $business->stripe_mode = $req->input('stripe_mode');
        $business->stripe_public_key_test = $req->input('stripe_public_key_test');
        $business->stripe_private_key_test = $req->input('stripe_private_key_test');
        $business->stripe_public_key_live = $req->input('stripe_public_key_live');
        $business->stripe_private_key_live = $req->input('stripe_private_key_live');

        $business->online_service = $req->input('online_service');
        $business->merchant_service = $req->input('merchant_service');
        $business->connect_service = $req->input('connect_service');
        $business->issuing_service = $req->input('issuing_service');
        $business->extension_service = $req->input('extension_service');
        $business->pouch_service = $req->input('pouch_service');
        $business->load_wallet = $req->input('load_wallet');
        $business->load_bank = $req->input('load_bank');
        $business->qr_permission = $req->input('qr_permission');
        $business->auto_merchant_print = $req->input('auto_merchant_print');
        $business->auto_customer_print = $req->input('auto_customer_print');
        $business->is_qr_print_on_bill = $req->input('is_qr_print_on_bill');

        $business->send_receipt = $req->input('send_receipt');

        if ($req->input('online_service') == 0) {
            $business->restaurant_id = '';
            $business->restaurant_name = '';
            $business->online_statement_description = '';
            $business->online_statement_descriptor = '';
        }

        $business->auto_archive = $req->input('auto_archive');
        $business->tip = $req->input('tip');
        $business->print_logo = $req->input('print_logo');
        $business->gallery_video_visible = $req->input('gallery_video_visible');
        $business->gallery_image_visible = $req->input('gallery_image_visible');
        if ($req->has('android_restaurant_app_version')) $business->android_restaurant_app_version = $req->input('android_restaurant_app_version');
        if ($req->has('ios_restaurant_app_version')) $business->ios_restaurant_app_version = $req->input('ios_restaurant_app_version');
        if ($req->has('android_partner_app_version')) $business->android_partner_app_version = $req->input('android_partner_app_version');
        if ($req->has('ios_partner_app_version')) $business->ios_partner_app_version = $req->input('ios_partner_app_version');
        $business->epos_service = $req->input('epos_service');
        if ($business->epos_service && (empty($business->db_name) || $business->db_name == null || $business->db_name == '')) $business->db_name = $this->checkAndGetRandomName();
        if ($req->has('database_server_id')) $business->database_server_id = $req->input('database_server_id');
        if ($req->has('pouch_weight')) $business->pouch_weight = $req->input('pouch_weight');

        $business->footer_a = $req->input('footer_a');
        $business->footer_b = $req->input('footer_b');
        $business->header_a = $req->input('header_a');
        $business->header_b = $req->input('header_b');
        $business->header_bill_a = $req->input('header_bill_a');
        $business->footer_bill_a = $req->input('footer_bill_a');
        $business->epos_device_allow = $req->input('epos_device_allow');
        $business->merchant_device_allow = $req->input('merchant_device_allow');

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_BUSINESS);
            $business->image = $fileName;
        }
        $business->save();

        if ($req->has('business_commissions')) {
            $req = $req->merge(($req->business_commissions));
            $dbBusinessCommission = BusinessCommission::where('business_id', $business->id)->first();
            if (empty($dbBusinessCommission)) {
                (new BusinessCommissionController())->create($req);
            } else (new BusinessCommissionController())->update($req, $dbBusinessCommission->id);
        }
        $req->merge(['business_id' => $business->id]);
        if ($req->has('admin_permissions')) {
            $req = $req->merge($req->admin_permissions);
            $req->merge(['role' => "admin"]);
            $req->merge(['business_id' => $business->id]);
            $admin_permissions = (new BusinessPermissionController())->create_update($req);
        }
        if ($req->has('supervisor_permissions')) {
            $req = $req->merge($req->supervisor_permissions);
            $req->merge(['role' => "supervisor"]);
            $req->merge(['business_id' => $business->id]);
            $supervisor_permissions = (new BusinessPermissionController())->create_update($req);
        }
        if ($business->epos_service) {
            $this->establishEposConnection($business);
            EposSetting::where('key', 'device_permission_allow')->update(['value' => $business->epos_device_allow]);
            if ($req->has('image')) {
                EposSetting::where('key', 'logo')->update(['value' => env('FTP_IMAGE_PATH') . 'images/business/' . $business->image]);
            }
        }

        // Plan Default Asssign
        $businessEposPlan = BusinessPlan::where('business_id', $business->id)->where('service', 'Epos')->orderByRaw('id ASC')->first();
        if ($business->epos_service && empty($businessEposPlan)) {
            $plan = Plan::where('service', 'Epos')->orderByRaw('id ASC')->first();
            if (!empty($plan)) {
                $businessPlan = new BusinessPlan;
                $businessPlan->business_id = $business->id;
                $businessPlan->plan_id = $plan->id;
                $businessPlan->service = 'Epos';
                $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                $businessPlan->save();
            }
        }

        $businessOnlinePlan = BusinessPlan::where('business_id', $business->id)->where('service', 'Online')->orderByRaw('id ASC')->first();
        if ($business->online_service && empty($businessOnlinePlan)) {
            $plan = Plan::where('service', 'Online')->orderByRaw('id ASC')->first();
            if (!empty($plan)) {
                $businessPlan = new BusinessPlan;
                $businessPlan->business_id = $business->id;
                $businessPlan->plan_id = $plan->id;
                $businessPlan->service = 'Online';
                $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                $businessPlan->save();
            }
        }

        $businessMerchantPlan = BusinessPlan::where('business_id', $business->id)->where('service', 'Merchant')->orderByRaw('id ASC')->first();
        if ($business->merchant_service && empty($businessMerchantPlan)) {
            $plan = Plan::where('service', 'Merchant')->orderByRaw('id ASC')->first();
            if (!empty($plan)) {
                $businessPlan = new BusinessPlan;
                $businessPlan->business_id = $business->id;
                $businessPlan->plan_id = $plan->id;
                $businessPlan->service = 'Merchant';
                $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                $businessPlan->save();
            }
        }

        $business->append('database_server');
        $this->getRestaurantDetails($business);
        return $business;
    }

    /**
     * Update business.
     * POST /api/businesses/direct-update/{id}
     */
    public function direct_update(Request $req, $id)
    {
        $business = Business::find($id);
        if (empty($business)) Utils::error('Business not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'trading_name' => 'string|max:255',
            'door_no' => 'string',
            'address' => 'string',
            'city' => 'string',
            'state' => 'string',
            'postcode' => 'string',
            'email' => ['email', Rule::unique('businesses')->ignore($business->id)],
            'contact_numbers' => 'string',
            'company_number' => 'string',
            'joining_date' => 'date',
            'type' => 'in:Sole trader,Partnership,Limited company,Not registered comapny,Other',
            'country_id' => 'exists:countries,id',
            'payout_mode' => 'in:Auto,Manual',
            'payout_method' => 'in:Bank,Paypal,Cash',
            'archived_payout_mode' => 'in:Auto,Manual',
            'archived_payout_method' => 'in:Bank,Paypal,Cash',
            'paypal_mode' => 'in:Test,Live',
            'stripe_mode' => 'in:Test,Live',
            'online_service' => 'boolean',
            'merchant_service' => 'boolean',
            'connect_service' => 'boolean',
            'issuing_service' => 'boolean',
            'extension_service' => 'boolean',
            'epos_service' => 'boolean',
        ]);
        // check if db name is already taken or not
        // if ($req->has('db_name')) {
        //     $dbNameCount = Business::where('db_name', $req->input('db_name'))->where('id', '!=', $id)->count();
        //     if ($dbNameCount > 0) Utils::error('Database with same name already exists', 400);
        // }

        // update business
        if ($req->has('name')) $business->name = $req->input('name');
        if ($req->has('trading_name')) $business->trading_name = $req->input('trading_name');
        if ($req->has('door_no')) $business->door_no = $req->input('door_no');
        if ($req->has('address')) $business->address = $req->input('address');
        if ($req->has('city')) $business->city = $req->input('city');
        if ($req->has('state')) $business->state = $req->input('state');
        if ($req->has('postcode')) $business->postcode = $req->input('postcode');
        if ($req->has('email')) $business->email = $req->input('email');
        if ($req->has('contact_numbers')) $business->contact_numbers = $req->input('contact_numbers');
        if ($req->has('company_number')) $business->company_number = $req->input('company_number');
        if ($req->has('joining_date')) $business->joining_date = $req->input('joining_date');
        if ($req->has('type')) $business->type = $req->input('type');
        if ($req->has('country_id')) $business->country_id = $req->input('country_id');
        if ($req->has('payout_mode')) $business->payout_mode = $req->input('payout_mode');
        if ($req->has('payout_method')) $business->payout_method = $req->input('payout_method');
        if ($req->has('archived_payout_mode')) $business->archived_payout_mode = $req->input('archived_payout_mode');
        if ($req->has('archived_payout_method')) $business->archived_payout_method = $req->input('archived_payout_method');
        if ($req->has('payout_comment')) $business->payout_comment = $req->input('payout_comment');
        if ($req->has('paypal_mode')) $business->paypal_mode = $req->input('paypal_mode');
        $business->online_service = $req->input('online_service');
        $business->merchant_service = $req->input('merchant_service');
        $business->connect_service = $req->input('connect_service');
        $business->issuing_service = $req->input('issuing_service');
        $business->extension_service = $req->input('extension_service');
        $business->epos_service = $req->input('epos_service');

        if ($business->online_service || $business->merchant_service) {
            $business->connect_service = 1;
        }

        if ($business->epos_service && $business->db_name == '') {
            $dataServer = DatabaseServer::orderByRaw('id DESC')->first();
            if (!empty($dataServer)) {
                $business->db_name = $this->checkAndGetRandomName();
                $business->database_server_id = $dataServer->id;
            }
        }
        $business->save();

        // Epos load database on default service
        if ($business->epos_service && $business->db_name == '') {
            $business->append('database_server');
            $databaseServer = $business->database_server;
            $conn = new mysqli($databaseServer->local_ip . ":" . $databaseServer->port, $databaseServer->username, $databaseServer->password);
            // Check connection
            if ($conn->connect_error) {
                return ("Connection failed: " . $conn->connect_error);
            }
            $sql = "CREATE DATABASE IF NOT EXISTS $business->db_name";
            if ($conn->query($sql) === TRUE) {
                Log::debug("$business->db_name Database created successfully");
            } else {
                return "Error creating database: " . $conn->error;
            }

            $path = "database/epos";
            $config = app()->make('config');
            $config->set('database.connections.epos', [
                'driver'   => 'mysql',
                'host' => $databaseServer->local_ip,
                'database' => $business->db_name,
                'username' => $databaseServer->username,
                'password' => $databaseServer->password,
                'port' => $databaseServer->port,
                'charset'   => 'utf8',
                'collation' => 'utf8_unicode_ci',
                'strict'    => false,
                'migrations' => 'migrations'
            ]);
            Artisan::call('migrate', [
                '--path' => $path,
                '--force' => true,
                '--database' => 'epos'
            ]);
        }

        // Plan Default Asssign
        $business_plan = BusinessPlan::where('business_id', $business->id)->orderByRaw('id ASC')->first();
        if (empty($business_plan)) {
            if ($business->epos_service) {
                $plan = Plan::where('service', 'Epos')->orderByRaw('id ASC')->first();
                if (!empty($plan)) {
                    $businessPlan = new BusinessPlan;
                    $businessPlan->business_id = $business->id;
                    $businessPlan->plan_id = $plan->id;
                    $businessPlan->service = 'Epos';
                    $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                    $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                    $businessPlan->save();
                }
            }

            if ($business->online_service) {
                $plan = Plan::where('service', 'Online')->orderByRaw('id ASC')->first();
                if (!empty($plan)) {
                    $businessPlan = new BusinessPlan;
                    $businessPlan->business_id = $business->id;
                    $businessPlan->plan_id = $plan->id;
                    $businessPlan->service = 'Online';
                    $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                    $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                    $businessPlan->save();
                }
            }

            if ($business->merchant_service) {
                $plan = Plan::where('service', 'Merchant')->orderByRaw('id ASC')->first();
                if (!empty($plan)) {
                    $businessPlan = new BusinessPlan;
                    $businessPlan->business_id = $business->id;
                    $businessPlan->plan_id = $plan->id;
                    $businessPlan->service = 'Merchant';
                    $businessPlan->from_date = Carbon::today()->format('Y-m-d');
                    $businessPlan->to_date = Carbon::today()->addMonths(1)->format('Y-m-d');
                    $businessPlan->save();
                }
            }
        }

        // Country Wise check and empty to insert
        $countryCommission = CountryCommission::where('business_id', $business->id)->first();
        if (empty($countryCommission)) {
            $countryCommission = new CountryCommission();
            $countryCommission->business_id = $business->id;
            $countryCommission->stripe_percentage = 3.5;
            $countryCommission->stripe_fixed = 0.25;
            $countryCommission->other = 1;
            $countryCommission->save();

            $countries = Country::where('currency_code', 'GBP')->first();
            if (!empty($countries)) {
                $countryCommission = new CountryCommission();
                $countryCommission->business_id = $business->id;
                $countryCommission->country_id = $countries->id;
                $countryCommission->stripe_percentage = 0;
                $countryCommission->stripe_fixed = 0;
                $countryCommission->other = 0;
                $countryCommission->save();
            }
        }

        // Card Brand Wise check and empty to insert
        $cardBrandCommission = CardBrandCommission::where('business_id', $business->id)->first();
        if (empty($cardBrandCommission)) {
            $cardBrand = CardBrand::where('name', 'AMEX')->where('code', '34')->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = new CardBrandCommission();
                $cardBrandCommission->business_id = $business->id;
                $cardBrandCommission->card_brand_id = $cardBrand->id;
                $cardBrandCommission->stripe_percentage = 3.5;
                $cardBrandCommission->stripe_fixed = 0.25;
                $cardBrandCommission->save();
            }

            $cardBrand = CardBrand::where('name', 'AMEX')->where('code', '37')->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = new CardBrandCommission();
                $cardBrandCommission->business_id = $business->id;
                $cardBrandCommission->card_brand_id = $cardBrand->id;
                $cardBrandCommission->stripe_percentage = 3.5;
                $cardBrandCommission->stripe_fixed = 0.25;
                $cardBrandCommission->save();
            }
        }

        return $business;
    }

    /**
     * Generate Database schema
     * POST /api/businesses/{id}/schema
     */
    public function schema(Request $req, $id)
    {
        ini_set('max_execution_time', 240);
        $business = Business::find($id);
        if (empty($business)) Utils::error("Business not found", 400);
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);

        // Create connection
        $conn = new mysqli($databaseServer->local_ip . ":" . $databaseServer->port, $databaseServer->username, $databaseServer->password);
        // Check connection
        if ($conn->connect_error) {
            return ("Connection failed: " . $conn->connect_error);
        }
        $sql = "CREATE DATABASE IF NOT EXISTS $business->db_name";
        if ($conn->query($sql) === TRUE) {
            Log::debug("$business->db_name Database created successfully");
        } else {
            return "Error creating database: " . $conn->error;
        }

        $path = "database/epos";
        $config = app()->make('config');
        $config->set('database.connections.epos', [
            'driver'   => 'mysql',
            'host' => $databaseServer->local_ip,
            'database' => $business->db_name,
            'username' => $databaseServer->username,
            'password' => $databaseServer->password,
            'port' => $databaseServer->port,
            'charset'   => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'strict'    => false,
            'migrations' => 'migrations'
        ]);
        Artisan::call('migrate', [
            '--path' => $path,
            '--force' => true,
            '--database' => 'epos'
        ]);

        return ['message' => 'Schema generated'];
    }

    /**
     * Re-Generate Database schema
     * POST /api/businesses/{id}/reset-schema
     */
    public function reset_schema(Request $req, $id)
    {
        $business = Business::find($id);
        if (empty($business)) Utils::error("Business not found", 400);
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;
        if (empty($databaseServer)) Utils::error("Database server not found", 400);

        $path = "database/epos";
        $config = app()->make('config');
        $config->set('database.connections.epos', [
            'driver'   => 'mysql',
            'host' => $databaseServer->local_ip,
            'database' => $business->db_name,
            'username' => $databaseServer->username,
            'password' => $databaseServer->password,
            'port' => $databaseServer->port,
            'charset'   => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'strict'    => false,
            'migrations' => 'migrations'
        ]);

        DB::connection('epos')->statement("CREATE DATABASE IF NOT EXISTS $business->db_name");
        Artisan::call('migrate:fresh', [
            '--path' => $path,
            '--force' => true,
            '--database' => 'epos'
        ]);
        $tables = DB::connection('epos')->select("SHOW TABLES");

        return ['message' => 'Schema re-generated', 'tables' => $tables];
    }

    /**
     * Seed default data into database
     * POST /api/businesses/{id}/load-data
     */
    public function load_data(Request $req, $id)
    {

        $business = Business::find($id);
        if (empty($business)) Utils::error("Business not found", 400);
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
        try {
            Artisan::call('db:seed', [
                '--class' => 'EposSeeder',
                '--force' => true,
                '--database' => 'epos_connection'
            ]);

            $business->append('country');

            $printers = Printer::orderByRaw('created_at DESC')->get();
            foreach ($printers as $printer) {
                $printers = EposPrinterModel::find($printer->id);
                if (empty($printers)) {
                    $printers = new EposPrinterModel;
                    $printers->name = $printer->name;
                    $printers->id = $printer->id;
                    $printers->save();
                } else {
                    $printers->name = $printer->name;
                    $printers->save();
                }
            }

            // $business_printers = BusinessPrinter::where('business_id', $id)->orderByRaw('created_at DESC')->get();
            // foreach ($business_printers as $business_printer) {
            //     $business_printer = new EposPrinter($business_printer);
            //     $business_printer->save();
            // }

            EposSetting::where('key', 'restaurant_full_name')->update(['value' => $business->name]);
            EposSetting::where('key', 'restaurant_short_name')->update(['value' => $business->name]);
            EposSetting::where('key', 'restaurant_city')->update(['value' => $business->city]);
            EposSetting::where('key', 'restaurant_country')->update(['value' => $business->country->name]);
            EposSetting::where('key', 'restaurant_email')->update(['value' => $business->email]);
            EposSetting::where('key', 'restaurant_postcode')->update(['value' => $business->postcode]);
            EposSetting::where('key', 'restaurant_street')->update(['value' => $business->address]);
            EposSetting::where('key', 'restaurant_telephone')->update(['value' => $business->contact_numbers]);
            EposSetting::where('key', 'restaurant_town')->update(['value' => $business->state]);
            EposSetting::where('key', 'res_id')->update(['value' => $business->online_service == 1 ? $business->restaurant_id : '']);
            EposSetting::where('key', 'statement_description')->update(['value' => $business->statement_description]);
            EposSetting::where('key', 'statement_descriptor')->update(['value' => $business->statement_descriptor]);
            EposSetting::where('key', 'device_permission_allow')->update(['value' => $business->epos_device_allow, 'is_editable' => 0]);
            EposSetting::where('key', 'address')->update(['value' => $business->door_no . ',' . $business->address . ',' . $business->city . ',' . $business->state]);
            EposSetting::where('key', 'is_weborder')->update(['value' => $business->online_service == 1 ? 'true' : 'false', 'is_editable' => 0]);
            EposSetting::where('key', 'logo')->update(['value' => $business->image_url != '' ? $business->image_url : '', 'is_editable' => 0]);

            // $settings[] = [
            //     "name" => "Prep Ticket",
            //     "service" => "epos",
            //     "print_structure" => '{"header_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"header_customer_name":{"style":"default","size":"size1","alignment":"center","visibility":true},"title":{"style":"default","size":"size1","alignment":"center","visibility":true},"order_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"table_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"no_of_diners":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_date":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_time":{"style":"default","size":"size0","alignment":"left","visibility":true},"items_header":{"style":"default","size":"size0","alignment":"left","visibility":true},"items":{"style":"default","size":"size1","alignment":"left","visibility":true},"items_subaddon":{"style":"default","size":"size0","alignment":"left","visibility":true},"subtotal":{"style":"default","size":"size0","alignment":"right","visibility":true},"discount":{"style":"default","size":"size0","alignment":"right","visibility":true},"delivery_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"service_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"gratuity":{"style":"default","size":"size0","alignment":"right","visibility":true},"charity":{"style":"default","size":"size0","alignment":"right","visibility":true},"grand_total":{"style":"default","size":"size1","alignment":"right","visibility":true},"serve_by":{"style":"default","size":"size0","alignment":"left","visibility":true},"customer_detail":{"style":"bold","size":"size0","alignment":"left","visibility":true},"payment_status":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_comment":{"style":"default","size":"size0","alignment":"left","visibility":true},"footer_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_table_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_order_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer":{"style":"default","size":"size0","alignment":"center","visibility":true}}',
            // ];

            // $settings[] = [
            //     "name" => "Bill",
            //     "service" => "epos",
            //     "print_structure" => '{"header_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"header_customer_name":{"style":"default","size":"size1","alignment":"center","visibility":true},"title":{"style":"default","size":"size1","alignment":"center","visibility":true},"order_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"table_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"no_of_diners":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_date":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_time":{"style":"default","size":"size0","alignment":"left","visibility":true},"items_header":{"style":"default","size":"size0","alignment":"left","visibility":true},"items":{"style":"default","size":"size0","alignment":"left","visibility":true},"items_subaddon":{"style":"default","size":"size0","alignment":"left","visibility":true},"subtotal":{"style":"default","size":"size0","alignment":"right","visibility":true},"discount":{"style":"default","size":"size0","alignment":"right","visibility":true},"delivery_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"service_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"gratuity":{"style":"default","size":"size0","alignment":"right","visibility":true},"charity":{"style":"default","size":"size0","alignment":"right","visibility":true},"grand_total":{"style":"default","size":"size1","alignment":"right","visibility":true},"payment_detail_header":{"style":"default","size":"size0","alignment":"left","payment_detail":{"style":"default","size":"size0","alignment":"left","serve_by":{"style":"default","size":"size0","alignment":"left","visibility":true},"customer_detail":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_comment":{"style":"default","size":"size0","alignment":"left","visibility":true},"payment_status":{"style":"default","size":"size0","alignment":"left","visibility":true},"footer_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_table_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_order_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer":{"style":"default","size":"size0","alignment":"center","visibility":true}}',
            // ];

            // $settings[] = [
            //     "name" => "Online Bill",
            //     "service" => "online",
            //     "print_structure" => '{"header_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"header_customer_name":{"style":"default","size":"size1","alignment":"center","visibility":true},"title":{"style":"default","size":"size1","alignment":"center","visibility":true},"order_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"table_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"no_of_diners":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_date":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_time":{"style":"default","size":"size0","alignment":"left","visibility":true},"items_header":{"style":"default","size":"size0","alignment":"left","visibility":true},"items":{"style":"default","size":"size0","alignment":"left","visibility":true},"items_subaddon":{"style":"default","size":"size0","alignment":"left","visibility":true},"subtotal":{"style":"default","size":"size0","alignment":"right","visibility":true},"discount":{"style":"default","size":"size0","alignment":"right","visibility":true},"delivery_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"service_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"gratuity":{"style":"default","size":"size0","alignment":"right","visibility":true},"charity":{"style":"default","size":"size0","alignment":"right","visibility":true},"grand_total":{"style":"default","size":"size1","alignment":"right","visibility":true},"serve_by":{"style":"default","size":"size0","alignment":"left","visibility":true},"payment_status":{"style":"default","size":"size0","alignment":"left","visibility":true},"footer_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_table_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_order_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer":{"style":"default","size":"size0","alignment":"center","visibility":true},"online_title":{"style":"default","size":"size1","alignment":"center","visibility":true},"online_site":{"style":"default","size":"size0","alignment":"center","visibility":true},"online_order_type":{"style":"bold","size":"size1","alignment":"left","visibility":true},"online_order_no":{"style":"bold","size":"size1","alignment":"left","visibility":true},"online_order_date":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_order_time":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_items":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_sub_total":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_delivery_charge":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_service_charge":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_driver_tip":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_loyalty":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_offer":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_grand_total":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_comment":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_delivery_instruction":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_customer_details":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_delivery_address":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_requested_for":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_payment_status":{"style":"bold","size":"size1","alignment":"left","visibility":true},"online_payment_method":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_receipt_time":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_order_status":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_restaurant_address":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_failed_reason":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_footer":{"style":"bold","size":"size0","alignment":"left","visibility":true}}',
            // ];

            // $settings[] = [
            //     "name" => "Online Kitchen",
            //     "service" => "online",
            //     "print_structure" => '{"header_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"header_customer_name":{"style":"default","size":"size1","alignment":"center","visibility":true},"title":{"style":"default","size":"size1","alignment":"center","visibility":true},"order_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"table_no":{"style":"default","size":"size0","alignment":"left","visibility":true},"no_of_diners":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_date":{"style":"default","size":"size0","alignment":"left","visibility":true},"order_time":{"style":"default","size":"size0","alignment":"left","visibility":true},"items_header":{"style":"default","size":"size0","alignment":"left","visibility":true},"items":{"style":"default","size":"size1","alignment":"left","visibility":true},"items_subaddon":{"style":"default","size":"size0","alignment":"left","visibility":true},"subtotal":{"style":"default","size":"size0","alignment":"right","visibility":true},"discount":{"style":"default","size":"size0","alignment":"right","visibility":true},"delivery_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"service_charge":{"style":"default","size":"size0","alignment":"right","visibility":true},"gratuity":{"style":"default","size":"size0","alignment":"right","visibility":true},"charity":{"style":"default","size":"size0","alignment":"right","visibility":true},"grand_total":{"style":"default","size":"size1","alignment":"right","visibility":true},"serve_by":{"style":"default","size":"size0","alignment":"left","visibility":true},"payment_status":{"style":"default","size":"size0","alignment":"left","visibility":true},"footer_order_type":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_table_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer_order_number":{"style":"default","size":"size1","alignment":"center","visibility":true},"footer":{"style":"default","size":"size0","alignment":"center","visibility":true},"online_title":{"style":"default","size":"size1","alignment":"center","visibility":true},"online_site":{"style":"default","size":"size0","alignment":"center","visibility":true},"online_order_type":{"style":"bold","size":"size1","alignment":"left","visibility":true},"online_order_no":{"style":"bold","size":"size1","alignment":"left","visibility":true},"online_order_date":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_order_time":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_items":{"style":"bold","size":"size1","alignment":"left","visibility":true},"online_sub_total":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_delivery_charge":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_service_charge":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_driver_tip":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_loyalty":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_offer":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_grand_total":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_comment":{"style":"default","size":"size1","alignment":"left","visibility":true},"online_delivery_instruction":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_customer_details":{"style":"default","size":"size0","alignment":"left","visibility":true},"online_delivery_address":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_requested_for":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_payment_status":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_payment_method":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_receipt_time":{"style":"bold","size":"size1","alignment":"left","visibility":true},"online_order_status":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_restaurant_address":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_failed_reason":{"style":"bold","size":"size0","alignment":"left","visibility":true},"online_footer":{"style":"bold","size":"size0","alignment":"left","visibility":true}}',
            // ];

            // foreach ($settings as $setting) {

            //     $printSetting = EposPrintSetting::where('name', $setting['name'])->first();
            //     if (empty($printSetting)) {
            //         $printSetting = new EposPrintSetting($setting);
            //         $printSetting->print_structure = json_decode($setting['print_structure']);
            //         $printSetting->save();
            //     }
            // }
        } catch (Exception $e) {
            return ['message' => $e->getMessage()];
        }
        return ['message' => 'Seeding done!'];
    }

    /**
     * Get EPOS Service devices
     * GET /api/businesses/{id}/epos-devices
     */
    public function epos_devices(Request $req, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        return (new EposDeviceController())->index($req);
    }

    /**
     * Update EPOS Devices
     * GET /api/businesses/{id}/epos-devices/{id}/enable
     */
    public function epos_enable_device(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        $device = EposDevice::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        $device->disabled = 0;
        $device->save();
        return $device;
    }

    /**
     * Update EPOS Devices
     * GET /api/businesses/{id}/epos-devices/{id}/disable
     */
    public function epos_disable_device(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        $device = EposDevice::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        $device->disabled = 1;
        $device->save();
        return $device;
    }

    /**
     * Update EPOS Devices
     * DELETE /api/businesses/{id}/epos-devices/{id}/disable
     */
    public function epos_delete(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        $device = EposDevice::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        $device->delete();
        return $device;
    }

    /**
     * Update EPOS Devices
     * GET /api/businesses/{id}/epos-devices/{id}/enable-weborder
     */
    public function epos_enable_device_weborder(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        return (new EposDeviceController())->enable_weborder_admin($req, $id);
    }

    /**
     * Update EPOS Devices
     * GET /api/businesses/{id}/epos-devices/{id}/enable-callerid
     */
    public function epos_enable_device_callerid(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        if ($business->epos_service) {
            $this->establishEposConnection($business);
        }

        return (new EposDeviceController())->enable_callerid_admin($req, $id);
    }

    /**
     * Update EPOS Devices
     * GET /api/businesses/{id}/epos-devices/{id}/enable-sunmi-printer
     */
    public function epos_enable_device_sunmi_printer(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        return (new EposDeviceController())->enable_sunmi_printer_admin($req, $id);
    }

    /**
     * Update EPOS Devices
     * GET /api/businesses/{id}/epos-devices/{id}/tap-to-pay
     */
    public function epos_device_tap_pay(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);

        return (new EposDeviceController())->enable_tap_pay_admin($req, $id);
    }

    /**
     * Get EPOS Settings
     * GET /api/businesses/{id}/epos-settings
     */
    public function epos_settings(Request $req, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        return (new EposSettingController())->index($req);
    }

    /**
     * Update EPOS Settings
     * GET /api/businesses/{id}/epos-settings
     */
    public function epos_settings_update(Request $req, $business_id, $id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        return (new EposSettingController())->update($req, $id);
    }

    /**
     * Update EPOS Settings
     * GET /api/businesses/{id}/epos-settings
     */
    public function epos_settings_update_bulk(Request $req, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        return (new EposSettingController())->setting_bulk($req);
    }

    /**
     * Enable business.
     * POST /api/businesses/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $business = Business::find($id);
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->disabled = 0;
        $business->save();

        $deviceRegistartions = DeviceRegistration::where('business_id', $business->id)->where('service_type', 'Merchant')->orderBy('created_at', 'DESC')->get();
        if (!$deviceRegistartions->isEmpty()) {
            // Business Fcm Notification
            $message      = 'Your business enabled successfully.';
            $notification['title']          = "Business Enable";
            $notification['body']        = $message;
            $notificationdata['data']['title']          = "Business Enable";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['enable']        = true;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "order");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            foreach ($deviceRegistartions as $Device) {
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device->device_token, $notification, 'customer');
            }
        }

        return $business;
    }

    /**
     * Disable business.
     * POST /api/businesses/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $business = Business::find($id);
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->disabled = 1;
        $business->save();

        $deviceRegistartions = DeviceRegistration::where('business_id', $business->id)->where('service_type', 'Merchant')->orderBy('created_at', 'DESC')->get();
        if (!$deviceRegistartions->isEmpty()) {
            // Business Fcm Notification
            $message      = 'Your business disabled successfully.';
            $notification['title']          = "Business Enable";
            $notification['body']        = $message;
            $notificationdata['data']['title']          = "Business Enable";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['disable']        = true;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "order");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            foreach ($deviceRegistartions as $Device) {
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device->device_token, $notification, 'customer');
            }
        }

        return $business;
    }

    /**
     * Delete business by id.
     * DELETE /api/businesses/{id}
     */
    public function delete(Request $req, $id)
    {
        $business = Business::find($id);
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->delete();
        if ($business->image != null) ImageHelper::delete(ImageHelper::$TYPE_BRAND, $business->image);
        return $business;
    }


    /*
     |-------------------------------------------
     | Business bank details
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/bank-details
    public function bank_details_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BankDetailController())->index($req);
    }

    // POST /api/businesses/{business_id}/bank-details
    public function bank_details_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BankDetailController())->create($req);
    }

    // POST /api/businesses/{business_id}/bank-details/{id}
    public function bank_details_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BankDetailController())->update($req, $id);
    }

    // POST /api/businesses/{business_id}/bank-details/{id}/default
    public function bank_details_default(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BankDetailController())->default($req, $id);
    }
    // DELETE /api/businesses/{business_id}/bank-details/{id}
    public function bank_details_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BankDetailController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business paypal details
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/paypal-details
    public function paypal_details_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalDetailController())->index($req);
    }

    // POST /api/businesses/{business_id}/paypal-details
    public function paypal_details_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalDetailController())->create($req);
    }

    // POST /api/businesses/{business_id}/paypal-details/{id}
    public function paypal_details_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalDetailController())->update($req, $id);
    }

    // POST /api/businesses/{business_id}/paypal-details/{id}/default
    public function paypal_details_default(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalDetailController())->default($req, $id);
    }
    // DELETE /api/businesses/{business_id}/paypal-details/{id}
    public function paypal_details_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalDetailController())->delete($req, $id);
    }


    /*
     |-------------------------------------------
     | User businesses
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/users
    public function users_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new UserBusinessController())->index($req);
    }

    // POST /api/businesses/{business_id}/users
    public function users_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new UserBusinessController())->create($req);
    }

    // POST /api/businesses/{business_id}/users/{id}
    public function users_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new UserBusinessController())->update($req, $id);
    }

    // DELETE /api/businesses/{business_id}/users/{id}
    public function users_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new UserBusinessController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business stripe transactions
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/stripe-transactions
    public function stripe_transactions_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new StripeTransactionController())->index($req);
    }
    // GET /api/businesses/{business_id}/stripe-transactions
    public function stripe_transactions_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new StripeTransactionController())->create($req);
    }
    // POST /api/businesses/{business_id}/stripe-transactions/transfer
    public function stripe_transactions_transfer(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new StripeTransactionController())->transfer($req);
    }
    // GET /api/businesses/{business_id}/stripe-transactions/connect
    public function stripe_transactions_connect(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new StripeTransactionController())->connect($req);
    }
    // GET /api/businesses/{business_id}/stripe-transactions/{id}
    public function stripe_transactions_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new StripeTransactionController())->show($req, $id);
    }
    // POST /api/businesses/{business_id}/stripe-transactions/archive
    public function stripe_transactions_archive(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new StripeTransactionController())->archive($req);
    }
    // POST /api/businesses/{business_id}/stripe-transactions/unarchive
    public function stripe_transactions_unarchive(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new StripeTransactionController())->unarchive($req);
    }


    /*
     |-------------------------------------------
     | Business paypal transactions
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/paypal-transactions
    public function paypal_transactions_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalPaymentController())->index($req);
    }
    // GET /api/businesses/{business_id}/paypal-transactions/{id}
    public function paypal_transactions_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalPaymentController())->show($req, $id);
    }
    // POST /api/businesses/{business_id}/paypal-transactions/archive
    public function paypal_transactions_archive(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalPaymentController())->archive($req);
    }
    // POST /api/businesses/{business_id}/paypal-transactions/unarchive
    public function paypal_transactions_unarchive(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PaypalPaymentController())->unarchive($req);
    }

    /*
     |-------------------------------------------
     | Business online orders
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/online-orders
    public function online_orders_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new OnlineOrderController())->index($req);
    }
    // POST /api/businesses/{business_id}/online-orders/archive
    public function online_orders_archive(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new OnlineOrderController())->archive($req);
    }
    // POST /api/businesses/{business_id}/online-orders/unarchive
    public function online_orders_unarchive(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new OnlineOrderController())->unarchive($req);
    }
    // GET /api/businesses/{business_id}/online-orders/{id}
    public function online_orders_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new OnlineOrderController())->show($req, $id);
    }

    /*
     |-------------------------------------------
     | Business country commission
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/country-commissions
    public function country_commissions_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CountryCommissionController())->index($req);
    }

    // POST /api/businesses/{business_id}/country-commissions
    public function country_commissions_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CountryCommissionController())->create($req);
    }

    // POST /api/businesses/{business_id}/country-commissions/{id}
    public function country_commissions_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CountryCommissionController())->update($req, $id);
    }

    // DELETE /api/businesses/{business_id}/country-commissions/{id}
    public function country_commissions_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CountryCommissionController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business card brand commission
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/card-brand-commissions
    public function card_brand_commissions_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CardBrandCommissionController())->index($req);
    }

    // POST /api/businesses/{business_id}/card-brand-commissions
    public function card_brand_commissions_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CardBrandCommissionController())->create($req);
    }

    // POST /api/businesses/{business_id}/card-brand-commissions/{id}
    public function card_brand_commissions_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CardBrandCommissionController())->update($req, $id);
    }

    // DELETE /api/businesses/{business_id}/card-brand-commissions/{id}
    public function card_brand_commissions_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new CardBrandCommissionController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business payout-charges
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/payout-charges
    public function payout_charges_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutChargeController())->index($req);
    }

    // POST /api/businesses/{business_id}/payout-charges
    public function payout_charges_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutChargeController())->create($req);
    }

    // POST /api/businesses/{business_id}/payout-charges/{id}
    public function payout_charges_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutChargeController())->update($req, $id);
    }

    // DELETE /api/businesses/{business_id}/payout-charges/{id}
    public function payout_charges_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutChargeController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business misc charges
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/misc-charges
    public function misc_charges_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new MiscChargeController())->index($req);
    }

    // POST /api/businesses/{business_id}/misc-charges
    public function misc_charges_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new MiscChargeController())->create($req);
    }

    // POST /api/businesses/{business_id}/misc-charges/{id}
    public function misc_charges_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new MiscChargeController())->update($req, $id);
    }

    // DELETE /api/businesses/{business_id}/misc-charges/{id}
    public function misc_charges_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new MiscChargeController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business notes
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/notes
    public function notes_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new NoteController())->index($req);
    }

    // POST /api/businesses/{business_id}/notes
    public function notes_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new NoteController())->create($req);
    }

    // GET /api/businesses/{business_id}/notes/{id}
    public function notes_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new NoteController())->show($req, $id);
    }

    // POST /api/businesses/{business_id}/notes/{id}
    public function notes_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new NoteController())->update($req, $id);
    }


    // DELETE /api/businesses/{business_id}/notes/{id}
    public function notes_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new NoteController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business device registrations
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/device-registrations
    public function device_registrations_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new DeviceRegistrationController())->index($req);
    }

    // POST /api/businesses/{business_id}/device-registrations
    public function device_registrations_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new DeviceRegistrationController())->create($req);
    }

    // GET /api/businesses/{business_id}/device-registrations/{id}
    public function device_registrations_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new DeviceRegistrationController())->show($req, $id);
    }

    // POST /api/businesses/{business_id}/device-registrations/{id}
    public function device_registrations_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new DeviceRegistrationController())->update($req, $id);
    }


    // DELETE /api/businesses/{business_id}/device-registrations/{id}
    public function device_registrations_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new DeviceRegistrationController())->delete($req, $id);
    }


    /*
     |-------------------------------------------
     | Business payout invoices
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/payout-invoices
    public function payout_invoice_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutInvoiceController())->index($req);
    }

    // POST /api/businesses/{business_id}/payout-invoices
    public function payout_invoice_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutInvoiceController())->create($req);
    }

    // GET /api/businesses/{business_id}/payout-invoices/{id}
    public function payout_invoice_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutInvoiceController())->show($req, $id);
    }
    // GET /api/businesses/{business_id}/payout-invoices/{id}/download
    public function payout_invoice_download(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutInvoiceController())->download($req, $id);
    }
    // DELETE /api/businesses/{business_id}/payout-invoices/{id}
    public function payout_invoice_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutInvoiceController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business printers
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/printers
    public function printers_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPrinterController())->index($req);
    }

    // POST /api/businesses/{business_id}/printers
    public function printers_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPrinterController())->create($req);
    }
    // POST /api/businesses/{business_id}/printers/{id}
    public function printers_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPrinterController())->update($req, $id);
    }
    // POST /api/businesses/{business_id}/printers/{id}/enable
    public function printers_enable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPrinterController())->enable($req, $id);
    }
    // POST /api/businesses/{business_id}/printers/{id}/disable
    public function printers_disable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPrinterController())->disable($req, $id);
    }
    // DELETE /api/businesses/{business_id}/printers/{id}
    public function printers_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPrinterController())->delete($req, $id);
    }
    // DELETE /api/businesses/{business_id}/printers/{id}
    public function printer_select(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPrinterController())->select($req, $id);
    }
    /*
     |-------------------------------------------
     | Business payout_requests
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/payout-requests
    public function payout_requests_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutRequestController())->index($req);
    }
    // POST /api/businesses/{business_id}/payout-requests
    public function payout_requests_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutRequestController())->create($req);
    }
    // POST /api/businesses/{business_id}/payout-requests/{id}
    public function payout_requests_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutRequestController())->update($req, $id);
    }
    // DELETE /api/businesses/{business_id}/payout-requests/{id}
    public function payout_requests_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PayoutRequestController())->delete($req, $id);
    }
    /*
     |-------------------------------------------
     | Business passbook
     |-------------------------------------------
     */

    // GET /api/business/{business_id}/passbook
    public function passbook_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PassbookController())->index($req);
    }
    // GET /api/business/{business_id}/passbook/statement
    public function passbook_statement(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PassbookController())->statement($req);
    }
    // POST /api/business/{business_id}/passbook
    public function passbook_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PassbookController())->create($req);
    }
    // POST /api/business/{business_id}/passbook/{id}
    public function passbook_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PassbookController())->update($req, $id);
    }
    // DELETE /api/business/{business_id}/passbook/{id}
    public function passbook_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new PassbookController())->delete($req, $id);
    }
    /*
     |-------------------------------------------
     | Business card_readers
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/card-readers
    public function card_readers_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->index($req);
    }

    // POST /api/businesses/{business_id}/card-readers
    public function card_readers_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->create($req);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}
    public function card_readers_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->update($req, $id);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}/enable
    public function card_readers_enable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->enable($req, $id);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}/disable
    public function card_readers_disable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->disable($req, $id);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}/enable-usb
    public function card_readers_usb_enable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->enable_usb($req, $id);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}/disable-usb
    public function card_readers_usb_disable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->disable_usb($req, $id);
    }
    // DELETE /api/businesses/{business_id}/card-readers/{id}
    public function card_readers_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardReaderController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business surcharges
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/card-readers
    public function surcharges_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessSurchargeController())->index($req);
    }

    // POST /api/businesses/{business_id}/card-readers
    public function surcharges_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessSurchargeController())->create($req);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}
    public function surcharges_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessSurchargeController())->update($req, $id);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}/enable
    public function surcharges_enable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessSurchargeController())->enable($req, $id);
    }
    // POST /api/businesses/{business_id}/card-readers/{id}/disable
    public function surcharges_disable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessSurchargeController())->disable($req, $id);
    }
    // DELETE /api/businesses/{business_id}/card-readers/{id}
    public function surcharges_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessSurchargeController())->delete($req, $id);
    }

    // GET /api/businesses/{business_id}/galleries
    public function gallery_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessGalleryController())->index($req);
    }

    // POST /api/businesses/{business_id}/galleries
    public function gallery_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessGalleryController())->create($req);
    }
    // POST /api/businesses/{business_id}/galleries/{id}
    public function gallery_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessGalleryController())->update($req, $id);
    }
    // DELETE /api/businesses/{business_id}/galleries/{id}
    public function gallery_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessGalleryController())->delete($req, $id);
    }

    // GET /api/businesses/{business_id}/issuing-cards
    public function issuing_cards_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new IssuingCardController())->index($req);
    }

    // POST /api/businesses/{business_id}/issuing-cards/reveal/{id}
    public function issuing_cards_reveal(Request $req, $business_id, $card_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new IssuingCardController())->reveal($req, $card_id);
    }
    // POST /api/businesses/{business_id}/issuing-cards
    public function issuing_cards_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new IssuingCardController())->create($req);
    }
    // POST /api/businesses/{business_id}/issuing-cards/{id}
    public function issuing_cards_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new IssuingCardController())->update($req, $id);
    }
    // DELETE /api/businesses/{business_id}/issuing-cards/{id}
    public function issuing_cards_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new IssuingCardController())->delete($req, $id);
    }
    // POST /api/businesses/{business_id}/issuing-cards/{id}/enable
    public function issuing_cards_enable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new IssuingCardController())->enable($req, $id);
    }
    // POST /api/businesses/{business_id}/issuing-cards/{id}/disable
    public function issuing_cards_disable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new IssuingCardController())->disable($req, $id);
    }

    /*
     |-------------------------------------------
     | Business plans
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/plans
    public function plans_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPlanController())->index($req);
    }

    // POST /api/businesses/{business_id}/plans
    public function plans_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPlanController())->create($req);
    }

    // GET /api/businesses/{business_id}/plans/{id}
    public function plans_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPlanController())->show($req, $id);
    }

    // POST /api/businesses/{business_id}/plans/{id}
    public function plans_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPlanController())->update($req, $id);
    }


    // DELETE /api/businesses/{business_id}/plans/{id}
    public function plans_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessPlanController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business tips
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/tips
    public function tips_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessTipController())->index($req);
    }

    // POST /api/businesses/{business_id}/tips
    public function tips_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessTipController())->create($req);
    }

    // GET /api/businesses/{business_id}/tips/{id}
    public function tips_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessTipController())->show($req, $id);
    }

    // POST /api/businesses/{business_id}/tips/{id}
    public function tips_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessTipController())->update($req, $id);
    }


    // DELETE /api/businesses/{business_id}/tips/{id}
    public function tips_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessTipController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Business Card Wallets
     |-------------------------------------------
     */

    // GET /api/businesses/{business_id}/card-wallets
    public function card_wallets_index(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardWalletController())->index($req);
    }

    // POST /api/businesses/{business_id}/card-wallets
    public function card_wallets_create(Request $req, $business_id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardWalletController())->create($req);
    }

    // GET /api/businesses/{business_id}/card-wallets/{id}
    public function card_wallets_show(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardWalletController())->show($req, $id);
    }

    // POST /api/businesses/{business_id}/card-wallets/{id}
    public function card_wallets_update(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardWalletController())->update($req, $id);
    }
    // DELETE /api/businesses/{business_id}/card-wallets/{id}
    public function card_wallets_delete(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardWalletController())->delete($req, $id);
    }
    // POST /api/businesses/{business_id}/card-wallets/{id}/enable
    public function card_wallets_enable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardWalletController())->enable($req, $id);
    }
    // POST /api/businesses/{business_id}/card-wallets/{id}/disable
    public function card_wallets_disable(Request $req, $business_id, $id)
    {
        $req->merge(['business_id' => $business_id]);
        return (new BusinessCardWalletController())->disable($req, $id);
    }

    /*
    |----------------------------------------
    | Fetch restaurant details from other server and save it in this server
    |----------------------------------------
    */
    public function getRestaurantDetails(Business $business)
    {
        if ($business->restaurant_id == null) return $business;
        // $response = (new Client())->request('POST', env('FETCH_REST_DETAIL', 'https://tiffintom.net/v2/StoreMobileApi/requests'), [
        //     'form_params' => ['action' => 'StoreDetail', 'user_id' => $business->restaurant_id]
        // ]);
        // if ($response->getStatusCode() == 200) {
        //     $jsonBody = json_decode($response->getBody());
        //     if ($jsonBody->success == '1') {
        //         $business->restaurant_name = $jsonBody->name;
        //         $business->online_paypal_description = $jsonBody->name;
        //         $business->save();
        //         return $business;
        //     } else {
        //         Utils::error("Invalid restaurant ID", 400);
        //     }
        // } else {
        //     Utils::error("Something went wrong", 400);
        // }
        $req = Request::create("");
        $restaurant = (new TiffintomRestaurantController())->show($req, $business->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 400);
        $business->restaurant_name = $restaurant->restaurant_name;
        $business->online_paypal_description = $restaurant->restaurant_name;
        $business->save();
        return $business;
    }
    public function fetchRestaurants(Request $req)
    {
        try {
            // $response = (new Client())->request('POST', env('FETCH_REST_LIST', 'https://www.tiffintom.net/v2/Allrestaurants'), [
            //     // 'form_params' => ['action' => 'StoreDetail', 'user_id' => $business->restaurant_id]
            // ]);
            // if ($response->getStatusCode() == 200) {
            //     $jsonBody = json_decode($response->getBody());
            //     if ($jsonBody->status == 'OK' && isset($jsonBody->result) && $jsonBody->result->success == '1') {
            //         $data = array();
            //         foreach ($jsonBody->result->RestaurantList as $restaurant) {
            //             $restaurant->id = (string)$restaurant->id;
            //             $data[] = $restaurant;
            //         }
            //         return $data;
            //     } else {
            //         Utils::error("No restaurants found", 400);
            //     }
            // } else {
            //     Utils::error("Something went wrong", 400);
            // }
            $req->merge(['nopaginate' => 1]);
            $restaurants = (new TiffintomRestaurantController())->index($req);
            if ($restaurants->isEmpty()) Utils::error("No restaurants found", 400);
            return $restaurants;
        } catch (Exception $e) {
            Utils::error("Could not connect to server", 400);
        }
    }

    public function download_database(Request $req, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->append('database_server');

        $databaseServer = $business->database_server;
        if (empty($databaseServer)) Utils::error("Database server not found", 400);
        try {
            $config = app()->make('config');
            $config->set('database.connections.epos', [
                'driver'   => 'mysql',
                'host' => $databaseServer->local_ip,
                'database' => $business->db_name,
                'username' => $databaseServer->username,
                'password' => $databaseServer->password,
                'port' => $databaseServer->port,
                'charset'   => 'utf8',
                'collation' => 'utf8_unicode_ci',
                'strict'    => false,
                'migrations' => 'migrations'
            ]);
            $result = DB::connection('epos')->select("SHOW TABLES");
            $tables = [
                'user_types', 'user_groups', 'users', 'customers', 'categories', 'printer_models', 'printers', 'preparation_locations', 'products', 'addons', 'product_addons', 'ingredients', 'product_ingredients', 'floors', 'table_statuses', 'tables', 'table_visitors', 'order_statuses', 'order_types', 'order_actions', 'devices', 'orders', 'order_splits', 'order_items', 'order_item_addons', 'order_item_ingredients', 'reservation_types', 'reservation_statuses', 'reservation_confirmations', 'deposit_types', 'reservations', 'delivery_charges', 'settings', 'discounts', 'product_discounts', 'vouchers', 'payment_methods', 'loyalty_points', 'order_payments', 'call_logs', 'cashdraw_logs', 'migrations', 'print_blocks', 'print_block_assigns'
            ];
            $structure = '';
            $data = '';
            foreach ($tables as $table) {
                $show_table_query = "SHOW CREATE TABLE " . $table . "";
                $show_table_result = DB::connection('epos')->select(DB::raw($show_table_query));
                foreach ($show_table_result as $show_table_row) {
                    $show_table_row = (array)$show_table_row;
                    $structure .= "\n\n" . $show_table_row["Create Table"] . ";\n\n";
                }
                $select_query = "SELECT * FROM " . $table;
                $records = DB::connection('epos')->select(DB::raw($select_query));
                foreach ($records as $record) {
                    $record = (array)$record;
                    $table_column_array = array_keys($record);
                    foreach ($table_column_array as $key => $name) {
                        $table_column_array[$key] = '`' . $table_column_array[$key] . '`';
                    }
                    $table_value_array = array_values($record);
                    $data .= "\nINSERT INTO $table (";
                    $data .= "" . implode(", ", $table_column_array) . ") VALUES \n";
                    foreach ($table_value_array as $key => $record_column)
                        $table_value_array[$key] = addslashes($record_column);
                    $data .= "('" . implode("','", $table_value_array) . "');\n";
                }
            }
            $file_name = $business->db_name . '.sql';
            $file_handle = fopen($file_name, 'w+');
            $output = $structure . $data;
            fwrite($file_handle, $output);
            fclose($file_handle);
        } catch (Exception $e) {
            Utils::error($e->getMessage(), 404);
        }

        return ['message' => env('APP_URL') . $file_name];
    }

    public function upload_database(Request $req, $business_id)
    {
        ini_set('max_execution_time', 240);
        $business = Business::find($business_id);
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->append('database_server');

        $databaseServer = $business->database_server;
        if (empty($databaseServer)) Utils::error("Database server not found", 400);

        $this->validate($req, [
            'upload_database' => 'file',
        ]);

        try {
            // Create connection
            $path = "database/epos";
            $config = app()->make('config');
            $config->set('database.connections.epos', [
                'driver'   => 'mysql',
                'host' => $databaseServer->local_ip,
                'database' => $business->db_name,
                'username' => $databaseServer->username,
                'password' => $databaseServer->password,
                'port' => $databaseServer->port,
                'charset'   => 'utf8',
                'collation' => 'utf8_unicode_ci',
                'strict'    => false,
                'migrations' => 'migrations'
            ]);
            $conn = new mysqli($databaseServer->local_ip . ":" . $databaseServer->port, $databaseServer->username, $databaseServer->password);
            // Check connection
            if ($conn->connect_error) {
                return ("Connection failed: " . $conn->connect_error);
            }

            $dropSql = "DROP DATABASE IF EXISTS $business->db_name";
            $conn->query($dropSql);
            $sql = "CREATE DATABASE IF NOT EXISTS $business->db_name";
            if ($conn->query($sql) === TRUE) {
                Log::debug("$business->db_name Database created successfully");
            } else {
                return "Error creating database: " . $conn->error;
            }

            $sql = file_get_contents($req->file('upload_database'));
            $array = explode("\n", $sql);
            $query = "";
            foreach ($array as $sql_line) {
                $sql_line = trim($sql_line);
                $query .= $sql_line;
                if (substr(rtrim($query), -1) == ';') {
                    $result = DB::connection('epos')->statement('SET FOREIGN_KEY_CHECKS=0;');
                    $result1 = DB::connection('epos')->statement($query);
                    $query = "";
                }
            }
        } catch (Exception $e) {
            Utils::error($e->getMessage(), 404);
        }

        return ['message' => 'Database upload successfully'];
    }


    public function checkAndGetRandomName()
    {
        $dbName = $this->getRandomDbName();
        $businesses = Business::where('db_name', $dbName)->get();
        if ($businesses->isEmpty()) {
            return $dbName;
        } else {
            return $this->checkAndGetRandomName();
        }
    }
    public function getRandomDbName()
    {
        return substr(str_shuffle('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'), 1, 9);
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }

    // Direct Debit
    public function bacs_debit_index(Request $request, $business_id)
    {
        $request->merge(['business_id' => $business_id]);
        return (new BacsDirectController())->index($request);
    }
    public function bacs_debit_create(Request $request, $business_id)
    {
        $request->merge(['business_id' => $business_id]);
        return (new BacsDirectController())->create($request);
    }
    public function bacs_debit_payment(Request $request, $business_id)
    {
        $request->merge(['business_id' => $business_id]);
        return (new BacsDirectController())->bacs_debit_payment($request);
    }
    public function bacs_debit_show(Request $request, $business_id, $id)
    {
        $request->merge(['business_id' => $business_id]);
        return (new BacsDirectController())->show($request, $id);
    }
    public function bacs_debit_update(Request $request, $business_id, $id)
    {
        $request->merge(['business_id' => $business_id]);
        return (new BacsDirectController())->update($request, $id);
    }
    public function bacs_debit_delete(Request $request, $business_id, $id)
    {
        $request->merge(['business_id' => $business_id]);
        return (new BacsDirectController())->delete($request, $id);
    }

    public function testPayment(Request $request, $business_id)
    {
        $request->merge(['business_id' => $business_id]);
        return (new BacsDirectController())->bacs_debit_payment($request);
    }
}