<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\BusinessGallery;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class BusinessGalleryController extends Controller
{
    /**
     * List businessGallery.
     * GET /api/business-galleries
     */
    public function index(Request $req)
    {
        $businessGallery = BusinessGallery::orderByRaw('created_at DESC');

        if ($query = $req->query('business_id', false)) {
            $businessGallery->where('business_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $businessGallery = $businessGallery->get();
        } else {
            $businessGallery = $businessGallery->paginate($req->query('per_page', 20));
        }

        if ($businessGallery->isEmpty()) Utils::error('No businessGallery found!', 404);


        return ($businessGallery);
    }

    /**
     * Create businessGallery.
     * POST /api/business-galleries
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'type' => 'required_if:link,image',
            'link' => 'string',
            'image' => 'image',
        ]);

        // create businessGallery
        $businessGallery = new BusinessGallery;
        $businessGallery->fill($req->all());
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_BUSINESS_GALLERY);
            $businessGallery->image = $fileName;
        }
        $businessGallery->save();

        return $businessGallery;
    }

    /**
     * Show businessGallery.
     * GET /api/business-galleries/{id}
     */
    public function show(Request $req, $id)
    {
        $businessGallery = BusinessGallery::find($id);
        if (empty($businessGallery)) Utils::error('BusinessGallery not found!', 404);
        return $businessGallery;
    }

    /**
     * Update businessGallery.
     * POST /api/business-galleries/{id}
     */
    public function update(Request $req, $id)
    {
        $businessGallery = BusinessGallery::find($id);
        if (empty($businessGallery)) Utils::error('BusinessGallery not found!', 404);

        $this->validate($req, [
            'type' => 'required_if:link,image',
            'link' => 'string',
            'image' => 'image',
        ]);

        // update businessGallery
        if ($req->has('link')) $businessGallery->link = $req->input('link');
        if ($req->has('type')) $businessGallery->type = $req->input('type');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_BUSINESS_GALLERY);
            if ($businessGallery->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_BUSINESS_GALLERY, $businessGallery->image);
            $businessGallery->image = $fileName;
        }
        $businessGallery->save();

        return $businessGallery;
    }

    /**
     * Delete businessGallery by id.
     * DELETE /api/business-galleries/{id}
     */
    public function delete(Request $req, $id)
    {
        $businessGallery = BusinessGallery::find($id);
        if (empty($businessGallery)) Utils::error('BusinessGallery not found!', 404);
        $businessGallery->delete();
        return $businessGallery;
    }
}
