<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\ImageHelper;
use App\BusinessSurcharge;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Stripe\StripeClient;

class BusinessSurchargeController extends Controller
{
    /**
     * List businessSurcharges.
     * GET /api/business-surcharges
     */
    public function index(Request $req)
    {
        $businessSurcharges = BusinessSurcharge::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $businessSurcharges->whereRaw('(name LIKE ? OR type LIKE ?)', ["%$query%", "%$query%"]);
        }
        if ($query = $req->query('type', false)) {
            $businessSurcharges->where('type', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $businessSurcharges->where('business_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $businessSurcharges = $businessSurcharges->get();
        } else {
            $businessSurcharges = $businessSurcharges->paginate($req->query('per_page', 20));
        }

        if ($businessSurcharges->isEmpty()) Utils::error('No businessSurcharges found!', 404);


        return ($businessSurcharges);
    }

    /**
     * Create businessSurcharge.
     * POST /api/business-surcharges
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'name' => 'required|string',
            'interval' => 'required|in:Weekly,Monthly',
            'surcharge' => 'required|numeric|min:0.1',
            'type' => 'required|in:Normal,Archive',
        ]);

        // create businessSurcharge
        $businessSurcharge = new BusinessSurcharge;
        $businessSurcharge->fill($req->all());
        $businessSurcharge->save();

        return $businessSurcharge;
    }

    /**
     * Show businessSurcharge.
     * GET /api/business-surcharges/{id}
     */
    public function show(Request $req, $id)
    {
        $businessSurcharge = BusinessSurcharge::find($id);
        if (empty($businessSurcharge)) Utils::error('BusinessSurcharge not found!', 404);
        return $businessSurcharge;
    }

    /**
     * Update businessSurcharge.
     * POST /api/business-surcharges/{id}
     */
    public function update(Request $req, $id)
    {

        $dbBusinessSurcharge = BusinessSurcharge::find($id);
        $businessSurcharge = clone ($dbBusinessSurcharge);
        if (empty($businessSurcharge)) Utils::error('BusinessSurcharge not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'name' => 'required|string',
            'interval' => 'required|in:Weekly,Monthly',
            'surcharge' => 'required|numeric|min:0.1',
            'type' => 'required|in:Normal,Archive',
        ]);

        // update businessSurcharge
        if ($req->has('business_id')) $businessSurcharge->business_id = $req->input('business_id');
        if ($req->has('name')) $businessSurcharge->name = $req->input('name');
        if ($req->has('interval')) $businessSurcharge->interval = $req->input('interval');
        if ($req->has('surcharge')) $businessSurcharge->surcharge = $req->input('surcharge');
        if ($req->has('type')) $businessSurcharge->type = $req->input('type');
        $businessSurcharge->save();

        return $businessSurcharge;
    }

    /**
     * Enable businessSurcharge.
     * POST /api/business-surcharges/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $businessSurcharge = BusinessSurcharge::find($id);
        if (empty($businessSurcharge)) Utils::error('BusinessSurcharge not found!', 404);
        $businessSurcharge->disabled = 0;
        $businessSurcharge->save();
        return $businessSurcharge;
    }

    /**
     * Disable businessSurcharge.
     * POST /api/business-surcharges/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $businessSurcharge = BusinessSurcharge::find($id);
        if (empty($businessSurcharge)) Utils::error('BusinessSurcharge not found!', 404);
        $businessSurcharge->disabled = 1;
        $businessSurcharge->save();
        return $businessSurcharge;
    }
    /**
     * Delete businessSurcharge by id.
     * DELETE /api/business-surcharges/{id}
     */
    public function delete(Request $req, $id)
    {
        $businessSurcharge = BusinessSurcharge::find($id);
        if (empty($businessSurcharge)) Utils::error('BusinessSurcharge not found!', 404);
        $businessSurcharge->delete();
        return $businessSurcharge;
    }
}
