<?php

namespace App\Http\Controllers;

use App\BusinessTip;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class BusinessTipController extends Controller
{
    /**
     * List businessTips.
     * GET /api/business-tips
     */
    public function index(Request $req)
    {
        $businessTips = BusinessTip::orderByRaw('created_at DESC');

        if ($query = $req->query('business_id', false)) {
            $businessTips->where('business_id', $query);
        }

        $businessTips = $businessTips->get();

        if ($businessTips->isEmpty()) Utils::error('No tips found!', 404);

        return ($businessTips);
    }

    /**
     * Create businessTip.
     * POST /api/business-tips
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'tip' => 'required|numeric|gt:0',
            'business_id' =>  ['required', 'exists:businesses,id', Rule::unique('business_tips')->where(function ($query) use ($req) {
                return $query->where('business_id', $req->input('business_id'))->where('tip', $req->input('tip'));
            })],
        ], [
            'business_id.unique' => "Tip already added same business",
        ]);

        // create businessTip
        $businessTip = new BusinessTip;
        $businessTip->fill($req->all());
        $businessTip->save();

        return $businessTip;
    }

    /**
     * Show businessTip.
     * GET /api/business-tips/{id}
     */
    public function show(Request $req, $id)
    {
        $businessTip = BusinessTip::find($id);
        if (empty($businessTip)) Utils::error('Tip not found!', 404);
        return $businessTip;
    }

    /**
     * Update businessTip.
     * POST /api/business-tips/{id}
     */
    public function update(Request $req, $id)
    {
        $businessTip = BusinessTip::find($id);
        if (empty($businessTip)) Utils::error('Tip not found!', 404);

        $this->validate($req, [
            'tip' => 'required|numeric|gt:0',
            'business_id' =>  ['required', 'exists:businesses,id', Rule::unique('business_tips')->where(function ($query) use ($req) {
                return $query->where('business_id', $req->input('business_id'))->where('tip', $req->input('tip'));
            })],
        ], [
            'business_id.unique' => "Tip already added same business",
        ]);

        // update businessTip
        if ($req->has('business_id')) $businessTip->business_id = $req->input('business_id');
        if ($req->has('tip')) $businessTip->tip = $req->input('tip');
        $businessTip->save();

        return $businessTip;
    }

    /**
     * Delete businessTip by id.
     * DELETE /api/business-tips/{id}
     */
    public function delete(Request $req, $id)
    {
        $businessTip = BusinessTip::find($id);
        if (empty($businessTip)) Utils::error('Tip not found!', 404);
        $businessTip->delete();
        return $businessTip;
    }
}