<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\CardBrandCommission;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class CardBrandCommissionController extends Controller
{
    /**
     * List cardBrandCommissions.
     * GET /api/card-brand-commissions
     */
    public function index(Request $req)
    {
        $cardBrandCommissions = CardBrandCommission::orderByRaw("other ASC");

        if ($query = $req->query('business_id', false)) {
            $cardBrandCommissions->where('business_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $cardBrandCommissions = $cardBrandCommissions->get();
        } else {
            $cardBrandCommissions = $cardBrandCommissions->paginate($req->query('per_page', 20));
        }

        if ($cardBrandCommissions->isEmpty()) Utils::error('No cardBrandCommissions found!', 404);


        return ($cardBrandCommissions);
    }

    /**
     * Create cardBrandCommission.
     * POST /api/card-brand-commissions
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'card_brand_id' => 'required_if:other,0|exists:card_brands,id',
            'business_id' => 'required|exists:businesses,id',
            'stripe_percentage' => 'numeric',
            'stripe_fixed' => 'numeric',

        ]);

        // create cardBrandCommission
        $cardBrandCommission = new CardBrandCommission;
        $cardBrandCommission->fill($req->all());

        $cardBrandCommission->save();

        return $cardBrandCommission;
    }

    /**
     * Show cardBrandCommission.
     * GET /api/card-brand-commissions/{id}
     */
    public function show(Request $req, $id)
    {
        $cardBrandCommission = CardBrandCommission::find($id);
        if (empty($cardBrandCommission)) Utils::error('CardBrandCommission not found!', 404);
        return $cardBrandCommission;
    }

    /**
     * Update cardBrandCommission.
     * POST /api/card-brand-commissions/{id}
     */
    public function update(Request $req, $id)
    {
        $cardBrandCommission = CardBrandCommission::find($id);
        if (empty($cardBrandCommission)) Utils::error('CardBrandCommission not found!', 404);

        $this->validate($req, [
            'card_brand_id' => 'required_if:other,0|exists:card_brands,id',
            'business_id' => 'exists:businesses,id',
            'other' => 'required_if:card_brand_id,null',

            'stripe_percentage' => 'numeric',
            'stripe_fixed' => 'numeric',

        ]);

        // update cardBrandCommission
        if ($req->has('card_brand_id')) $cardBrandCommission->card_brand_id = $req->input('card_brand_id');
        if ($req->has('business_id')) $cardBrandCommission->business_id = $req->input('business_id');

        $cardBrandCommission->other = $req->input('other');

        $cardBrandCommission->stripe_percentage = $req->input('stripe_percentage');
        $cardBrandCommission->stripe_fixed = $req->input('stripe_fixed');


        $cardBrandCommission->save();

        return $cardBrandCommission;
    }


    /**
     * Delete cardBrandCommission by id.
     * DELETE /api/card-brand-commissions/{id}
     */
    public function delete(Request $req, $id)
    {
        $cardBrandCommission = CardBrandCommission::find($id);
        if (empty($cardBrandCommission)) Utils::error('CardBrandCommission not found!', 404);
        $cardBrandCommission->delete();
        return $cardBrandCommission;
    }
}
