<?php

namespace App\Http\Controllers;

use App\CardBrand;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class CardBrandController extends Controller
{
    /**
     * List cardBrands.
     * GET /api/card-brands
     */
    public function index(Request $req)
    {
        $cardBrands = CardBrand::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $cardBrands->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $cardBrands = $cardBrands->get();
        } else {
            $cardBrands = $cardBrands->paginate($req->query('per_page', 20));
        }

        if ($cardBrands->isEmpty()) Utils::error('No cardBrands found!', 404);


        return ($cardBrands);
    }

    /**
     * Create cardBrand.
     * POST /api/card-brands
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'code' => 'required|string|max:255|unique:card_brands,code',

        ]);

        // create cardBrand
        $cardBrand = new CardBrand;
        $cardBrand->fill($req->all());

        $cardBrand->save();

        return $cardBrand;
    }

    /**
     * Show cardBrand.
     * GET /api/card-brands/{id}
     */
    public function show(Request $req, $id)
    {
        $cardBrand = CardBrand::find($id);
        if (empty($cardBrand)) Utils::error('CardBrand not found!', 404);
        return $cardBrand;
    }

    /**
     * Update cardBrand.
     * POST /api/card-brands/{id}
     */
    public function update(Request $req, $id)
    {
        $cardBrand = CardBrand::find($id);
        if (empty($cardBrand)) Utils::error('CardBrand not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'code' => ['string', 'max:255', Rule::unique('card_brands')->ignore($cardBrand->id)],

        ]);

        // update cardBrand
        if ($req->has('name')) $cardBrand->name = $req->input('name');
        if ($req->has('code')) $cardBrand->code = $req->input('code');

        $cardBrand->save();

        return $cardBrand;
    }


    /**
     * Delete cardBrand by id.
     * DELETE /api/card-brands/{id}
     */
    public function delete(Request $req, $id)
    {
        $cardBrand = CardBrand::find($id);
        if (empty($cardBrand)) Utils::error('CardBrand not found!', 404);
        $cardBrand->delete();
        return $cardBrand;
    }
}
