<?php

namespace App\Http\Controllers;

use App\CardHolder;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class CardHolderController extends Controller
{
    /**
     * List cardHolders.
     * GET /api/card-holders
     */
    public function index(Request $req)
    {
        $cardHolders = CardHolder::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $cardHolders->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $cardHolders = $cardHolders->get();
        } else {
            $cardHolders = $cardHolders->paginate($req->query('per_page', 20));
        }

        if ($cardHolders->isEmpty()) Utils::error('No cardHolders found!', 404);


        return ($cardHolders);
    }

    /**
     * Create cardHolder.
     * POST /api/card-holders
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'holder_id' => 'required|string|max:255',
            'name' => 'required|string|max:255',
            'email' => 'string|max:255',
            'phone_number' => 'string|max:255',
            'city' => 'string|max:255',
            'country' => 'string|max:255',
            'line1' => 'string|max:255',
            'postal_code' => 'string|max:255',
            'line2' => 'string|max:255',
            'state' => 'string|max:255',
            'company' => 'string|max:255',
            'individual' => 'string|max:255',
            'livemode' => 'boolean',
            'type' => 'required|in:individual,company',
            'status' => 'required|in:active,inactive',
        ]);

        // create cardHolder
        $cardHolder = new CardHolder;
        $cardHolder->fill($req->all());

        $cardHolder->save();

        return $cardHolder;
    }

    /**
     * Show cardHolder.
     * GET /api/card-holders/{id}
     */
    public function show(Request $req, $id)
    {
        $cardHolder = CardHolder::find($id);
        if (empty($cardHolder)) Utils::error('CardHolder not found!', 404);
        return $cardHolder;
    }

    /**
     * Update cardHolder.
     * POST /api/card-holders/{id}
     */
    public function update(Request $req, $id)
    {
        $cardHolder = CardHolder::find($id);
        if (empty($cardHolder)) Utils::error('CardHolder not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'holder_id' => 'required|string|max:255',
            'name' => 'required|string|max:255',
            'email' => 'string|max:255',
            'phone_number' => 'string|max:255',
            'city' => 'string|max:255',
            'country' => 'string|max:255',
            'line1' => 'string|max:255',
            'postal_code' => 'string|max:255',
            'line2' => 'string|max:255',
            'state' => 'string|max:255',
            'company' => 'string|max:255',
            'individual' => 'string|max:255',
            'livemode' => 'boolean',
            'type' => 'required|in:individual,company',
            'status' => 'required|in:active,inactive',
        ]);

        // update cardHolder
        if ($req->has('business_id')) $cardHolder->business_id = $req->input('business_id');
        if ($req->has('holder_id')) $cardHolder->holder_id = $req->input('holder_id');
        if ($req->has('name')) $cardHolder->name = $req->input('name');
        if ($req->has('email')) $cardHolder->email = $req->input('email');
        if ($req->has('phone_number')) $cardHolder->phone_number = $req->input('phone_number');
        if ($req->has('city')) $cardHolder->city = $req->input('city');
        if ($req->has('country')) $cardHolder->country = $req->input('country');
        if ($req->has('line1')) $cardHolder->line1 = $req->input('line1');
        if ($req->has('postal_code')) $cardHolder->postal_code = $req->input('postal_code');
        if ($req->has('line2')) $cardHolder->line2 = $req->input('line2');
        if ($req->has('state')) $cardHolder->state = $req->input('state');
        if ($req->has('company')) $cardHolder->company = $req->input('company');
        if ($req->has('individual')) $cardHolder->individual = $req->input('individual');
        $cardHolder->livemode = $req->input('livemode');
        if ($req->has('type')) $cardHolder->type = $req->input('type');
        if ($req->has('status')) $cardHolder->status = $req->input('status');

        $cardHolder->save();

        return $cardHolder;
    }


    /**
     * Delete cardHolder by id.
     * DELETE /api/card-holders/{id}
     */
    public function delete(Request $req, $id)
    {
        $cardHolder = CardHolder::find($id);
        if (empty($cardHolder)) Utils::error('CardHolder not found!', 404);
        $cardHolder->delete();
        return $cardHolder;
    }
}
