<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\CardReader;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class CardReaderController extends Controller
{
    /**
     * List cardReaders.
     * GET /api/card-readers
     */
    public function index(Request $req)
    {
        $cardReaders = CardReader::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $cardReaders->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('connectivity', false)) {
            $cardReaders->where('connectivity', $query);
        }
        if ($query = $req->query('ip', false)) {
            $cardReaders->where('ip', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $cardReaders = $cardReaders->get();
        } else {
            $cardReaders = $cardReaders->paginate($req->query('per_page', 20));
        }

        if ($cardReaders->isEmpty()) Utils::error('No cardReaders found!', 404);


        return ($cardReaders);
    }

    /**
     * Create cardReader.
     * POST /api/card-readers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'connectivity' => 'required|in:Wifi,Bluetooth,Internet,StripeBluetooth,POSBluetooth,TAPTOPAY',

        ]);

        // create cardReader
        $cardReader = new CardReader;
        $cardReader->fill($req->all());

        $cardReader->save();

        return $cardReader;
    }

    /**
     * Show cardReader.
     * GET /api/card-readers/{id}
     */
    public function show(Request $req, $id)
    {
        $cardReader = CardReader::find($id);
        if (empty($cardReader)) Utils::error('CardReader not found!', 404);
        return $cardReader;
    }

    /**
     * Update cardReader.
     * POST /api/card-readers/{id}
     */
    public function update(Request $req, $id)
    {
        $cardReader = CardReader::find($id);
        if (empty($cardReader)) Utils::error('CardReader not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
            'connectivity' => 'required|in:Wifi,Bluetooth,Internet,StripeBluetooth,POSBluetooth,TAPTOPAY',

        ]);

        // update cardReader
        if ($req->has('name')) $cardReader->name = $req->input('name');
        if ($req->has('connectivity')) $cardReader->connectivity = $req->input('connectivity');

        $cardReader->save();

        return $cardReader;
    }

    /**
     * Enable cardReader.
     * POST /api/card-readers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $cardReader = CardReader::find($id);
        if (empty($cardReader)) Utils::error('CardReader not found!', 404);
        $cardReader->disabled = 0;
        $cardReader->save();
        return $cardReader;
    }

    /**
     * Disable cardReader.
     * POST /api/card-readers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $cardReader = CardReader::find($id);
        if (empty($cardReader)) Utils::error('CardReader not found!', 404);
        $cardReader->disabled = 1;
        $cardReader->save();
        return $cardReader;
    }
    /**
     * Delete cardReader by id.
     * DELETE /api/card-readers/{id}
     */
    public function delete(Request $req, $id)
    {
        $cardReader = CardReader::find($id);
        if (empty($cardReader)) Utils::error('CardReader not found!', 404);
        $cardReader->delete();
        return $cardReader;
    }
}
