<?php

namespace App\Http\Controllers;

use App\Career;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class CareerController extends Controller
{
    /**
     * List careers.
     * GET /api/careers
     */
    public function index(Request $req)
    {
        $careers = Career::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $careers->whereRaw('(first_name LIKE ? OR  last_name LIKE ? OR  email LIKE ? OR  phone LIKE ? OR  position LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $careers = $careers->get();
        } else {
            $careers = $careers->paginate($req->query('per_page', 20));
        }

        if ($careers->isEmpty()) Utils::error('No careers found!', 404);

        return $careers;
    }

    /**
     * Create career.
     * POST /api/careers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'first_name' => 'required|string',
            'last_name' => 'required|string',
            'profile_link' => 'string',
            'email' => 'required|email',
            'phone' => 'required|string',
            'attach_cv' => 'required|file',
            'position' => 'required',
            'start_date' => 'required|date',
        ]);

        // create career
        $career = new Career;
        $career->fill($req->all());
        if ($req->has('attach_cv')) {
            $careerCV = ImageHelper::saveImageToServer($req->file('attach_cv'), ImageHelper::$TYPE_CV);
            $career->attach_cv = $careerCV;
        }
        $career->save();

        $this->sendMail($career->id);

        return $career;
    }

    /**
     * Show career.
     * GET /api/careers/{id}
     */
    public function show(Request $req, $id)
    {
        $career = Career::find($id);
        if (empty($career)) Utils::error('Career not found!', 404);
        return $career;
    }

    /**
     * Update career.
     * POST /api/careers/{id}
     */
    public function update(Request $req, $id)
    {
        $career = Career::find($id);
        if (empty($career)) Utils::error('Career not found!', 404);

        $this->validate($req, [
            'first_name' => 'required|string',
            'last_name' => 'required|string',
            'profile_link' => 'string',
            'email' => 'required|email',
            'phone' => 'required|string',
            'attach_cv' => 'required|image',
            'position' => 'required',
            'start_date' => 'required|date',
        ]);

        // update career
        if ($req->has('first_name')) $career->first_name = $req->input('first_name');
        if ($req->has('last_name')) $career->last_name = $req->input('last_name');
        if ($req->has('profile_link')) $career->profile_link = $req->input('profile_link');
        if ($req->has('email')) $career->email = $req->input('email');
        if ($req->has('phone')) $career->phone = $req->input('phone');
        if ($req->has('position')) $career->position = $req->input('position');
        if ($req->has('start_date')) $career->start_date = $req->input('start_date');
        if ($req->has('attach_cv')) {
            $careerCV = ImageHelper::saveImageToServer($req->file('attach_cv'), ImageHelper::$TYPE_CV);
            if ($career->attach_cv != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_CV, $career->attach_cv);
            $career->attach_cv = $careerCV;
        }

        $career->save();

        return $career;
    }

    /**
     * Delete career by id.
     * DELETE /api/careers/{id}
     */
    public function delete(Request $req, $id)
    {
        $career = Career::find($id);
        if (empty($career)) Utils::error('Career not found!', 404);
        $career->delete();
        return $career;
    }



    public function sendMail($id)
    {
        $career = Career::find($id);
        if (empty($career)) Utils::error("No career found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Dear ' . $career->first_name . ',<br>
                             </p>

                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Thank you for submitting your application for the [Position Name] role at UBSIDI. We appreciate your interest in our company and are excited to learn more about your qualifications and experience.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Our team is currently reviewing your application, and we will contact you within the next 24-48 Hour to schedule an interview or discuss next steps. Please note that we receive a high volume of applications, and it may take us some time to review your submission thoroughly.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                We want to assure you that we take every application seriously and are committed to providing a fair and objective evaluation process. We appreciate your patience and understanding as we work to find the best candidate for the position.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                In the meantime, we encourage you to learn more about our company and culture by visiting our website and social media pages. If you have any questions or would like to follow up on your application, please do not hesitate to contact us by replying to this email.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Thank you again for your interest in UBSIDI. We look forward to getting to know you better!
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Thank you for applying to UBSIDI',
            'mail_type' => $siteSettings->mail_option,
            'to' => $career->email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }
}
