<?php

namespace App\Http\Controllers;

use App\CashTransaction;
use App\Helpers\Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Helpers\Utils;
use Carbon\Carbon;

class CashTransactionController extends Controller
{
    public function index(Request $req)
    {
        $cashTransactions = CashTransaction::latest();

        if ($query = $req->query('from_date', false)) {
            $cashTransactions->where('created_at', '>=', Carbon::parse($query)->format('Y-m-d'));
        }

        if ($query = $req->query('to_date', false)) {
            $cashTransactions->where('created_at', '<=', Carbon::parse($query)->format('Y-m-d'));
        }

        if ($query = $req->query('business_id', false)) {
            $cashTransactions->where('business_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $cashTransactions = $cashTransactions->get();
        } else {
            $cashTransactions = $cashTransactions->paginate($req->query('per_page', 20));
        }

        if ($cashTransactions->isEmpty()) Utils::error('No cash transactions found!', 404);

        return $cashTransactions;
    }

    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'amount' => 'required|numeric',
            'device_id' => 'required',
        ]);

        $cashTransaction = new CashTransaction();
        $cashTransaction->fill($req->all());
        $cashTransaction->updater_id =  Auth::id();
        $cashTransaction->save();

        return $cashTransaction;
    }

    public function show($id)
    {
        $cashTransaction = CashTransaction::find($id);
        if (empty($cashTransaction)) Utils::error('Cash transaction not found.', 404);
        return $cashTransaction;
    }

    public function edit(Request $req, $id)
    {
        $cashTransaction = CashTransaction::find($id);
        if (empty($cashTransaction)) Utils::error('Cash transaction not found.', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'amount' => 'required|numeric',
            'device_id' => 'required',
        ]);

        if ($req->has('business_id')) $cashTransaction->business_id = $req->input('business_id');
        if ($req->has('amount')) $cashTransaction->amount = $req->input('amount');
        if ($req->has('device_id')) $cashTransaction->device_id = $req->input('device_id');
        $cashTransaction->updater_id =  Auth::id();
        $cashTransaction->save();

        return $cashTransaction;
    }

    public function delete($id)
    {
        $cashTransaction = CashTransaction::find($id);
        if (empty($cashTransaction)) Utils::error('Cash transaction not found.', 404);
        $cashTransaction->delete();

        return $cashTransaction;
    }
}
