<?php

namespace App\Http\Controllers;

use App\Business;
use App\CaxtonTransfer;
use App\CaxtonUser;
use App\Exports\PayoutRequestReportExport;
use App\Helpers\ImageHelper;
use App\CaxtonPayoutRequest;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsConnect\ConnectAccount;
use App\ModelsConnect\ExternalAccount;
use App\Passbook;
use App\PayoutCharge;
use App\PayoutInvoice;
use App\User;
use Carbon\Carbon;
use Exception;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;
use Stripe\StripeClient;

class CaxtonPayoutRequestController extends Controller
{
    /**
     * List bankPayoutRequest.
     * GET /api/banking-payout-requests
     */
    public function index(Request $req)
    {
        $bankPayoutRequests = CaxtonPayoutRequest::select("caxton_payout_requests.*", "businesses.name as business_name", "caxton_users.first_name", "caxton_users.last_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->leftJoin('caxton_users', 'caxton_users.id', '=', 'user_id')->orderByRaw('created DESC');

        if ($query = $req->query('query', false)) {
            $bankPayoutRequests->whereRaw('(businesses.name LIKE ? OR first_name LIKE ? OR last_name LIKE ? OR amount LIKE ? OR note LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('status', false)) {
            $bankPayoutRequests->where('status', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $bankPayoutRequests->where('business_id', $query);
        }
        if ($query = $req->query('from_date', false)) {
            $this->validate($req, [
                'from_date' => 'date_format:Y-m-d',
                'to_date' => 'required_with:from_date|date_format:Y-m-d',
            ]);

            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();

            $bankPayoutRequests = $bankPayoutRequests->where('caxton_payout_requests.created_at', '>=', $from_date->getTimestamp());
            $bankPayoutRequests = $bankPayoutRequests->where('caxton_payout_requests.created_at', '<=', $to_date->getTimestamp());
        }

        if ($query = $req->query('user_id', false)) {
            $bankPayoutRequests->where('user_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $bankPayoutRequests = $bankPayoutRequests->get();
        } else {
            $bankPayoutRequests = $bankPayoutRequests->paginate($req->query('per_page', 20));
        }

        if ($bankPayoutRequests->isEmpty()) Utils::error('No bank payout requests found!', 404);

        return ($bankPayoutRequests);
    }


    /**
     * Create bankPayoutRequest.
     * POST /api/banking-payout-requests
     */
    public function create(Request $req)
    {

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required|exists:businesses,id',
            'user_id' => 'exists:users,id',
            'wallet_id' => 'string',
            'beneficiary_id' => 'string',
        ]);

        // create bankPayoutRequest
        $bankPayoutRequest = new CaxtonPayoutRequest;
        $bankPayoutRequest->fill($req->all());
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
        }
        if ($req->has('user_id')) {
            $user = CaxtonUser::find($req->input('user_id'));
        }

        try {
            if ($req->has('wallet_id')) {
                $apiKey = SiteSettingController::findByKey('caxton_token')->value;
                $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

                $endPoint = 'payments/currency-bank/card-load';
                $client = new \GuzzleHttp\Client();
                $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                    'body' => '{
                    "WalletId":"' . $bankPayoutRequest->wallet_id . '",
                    "Currency":"GBP",
                    "Amount":"' . $bankPayoutRequest->amount . '",
                }',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                    $data = json_decode($data->getBody()->getContents());
                    $shop_order_id  = $data->Content->Model->ShopOrderId;
                    $processEndPoint = 'payments/currency-bank/process';
                    $walletDetails = $client->request('POST', env('CAXTON_BASEURL') . $processEndPoint, [
                        'body' => '{
                        "ShopOrderId":"' . $shop_order_id . '",
                    }',
                        'headers' => [
                            'Authorization' => 'bearer ' . $apiKey,
                            'UserApiToken' => $user_apiKey,
                            'content-type' => 'application/json',
                        ],
                    ]);
                    if ($walletDetails->getStatusCode() >= 200 && $walletDetails->getStatusCode() <= 210) {
                        $walletDetails = json_decode($walletDetails->getBody()->getContents());
                    }
                }
            }
            if ($req->has('beneficiary_id')) {
                $apiKey = SiteSettingController::findByKey('caxton_token')->value;
                $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

                $endPoint = 'transfers/pending';

                if ($bankPayoutRequest->note) {
                    $PayAwayReference = str_replace(' ', '', $bankPayoutRequest->note);
                } else {
                    $PayAwayReference = 'UBSIDIPAYMENT';
                }

                $client = new \GuzzleHttp\Client();
                $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                    'body' => '{
                            "BeneficiaryId":"' . $bankPayoutRequest->beneficiary_id . '",
                            "BuyCurrency": "GBP",
                            "BuyAmount": "' . $bankPayoutRequest->amount . '",
                            "PayAwayDate": "' . date('Y-m-d') . '",
                            "PayAwayReference": "' . $PayAwayReference . '",
                            "FeeCurrency": "",
                            "Fee": 0,
                            "OpenBankingAuthId": 0,
                            "TransferType": "Sepa",
                            "TransferReason": "Emigration",
                        }',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                    $data = json_decode($data->getBody()->getContents());
                    $caxtonTransfer = new CaxtonTransfer();
                    $caxtonTransfer->pending_transfer_id = $data->Content->Model->Value;
                    $caxtonTransfer->notes = $bankPayoutRequest->note;
                    $caxtonTransfer->save();

                    $clientTransfer = new \GuzzleHttp\Client();
                    $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                    $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                        '',
                        'headers' => [
                            'Authorization' => 'bearer ' . $apiKey,
                            'UserApiToken' => $user_apiKey,
                            'content-type' => 'application/json',
                        ],
                    ]);
                    if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                        $dataTransfer = json_decode($dataTransfer->getBody()->getContents());
                        $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                        $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                        $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                        $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                        $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                        $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                        $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                        $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                        $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                        $caxtonTransfer->save();

                        $passbook = new Passbook;
                        $passbook->entity = "caxton_payout_request";
                        $passbook->entity_id = $bankPayoutRequest->id;
                        $passbook->business_id = $bankPayoutRequest->business_id;
                        $passbook->user_id = $bankPayoutRequest->user_id;
                        $passbook->date = Carbon::today()->toDateString();
                        $passbook->from_date = $bankPayoutRequest->created_at;
                        $passbook->to_date = $bankPayoutRequest->created_at;
                        $passbook->amount = -1 * $bankPayoutRequest->amount;
                        $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($bankPayoutRequest->created_at)->toDateString();
                        $passbook->save();
                    }
                }
            }
            $bankPayoutRequest->save();
        } catch (Exception $e) {
            Utils::error($e->getMessage(), 400);
        }

        return $bankPayoutRequest;
    }

    /**
     * Show bankPayoutRequest.
     * GET /api/banking-payout-requests/{id}
     */
    public function show(Request $req, $id)
    {
        $bankPayoutRequest = CaxtonPayoutRequest::find($id);
        if (empty($bankPayoutRequest)) Utils::error('bank payout request not found!', 404);
        return $bankPayoutRequest;
    }

    /**
     * Update bankPayoutRequest.
     * POST /api/banking-payout-requests/{id}
     */
    public function update(Request $req, $id)
    {
        $bankPayoutRequest = CaxtonPayoutRequest::find($id);
        $dbPayoutRequest = $bankPayoutRequest->clone();
        if (empty($bankPayoutRequest)) Utils::error('PayoutRequest not found!', 404);

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required|exists:businesses,id',
            'user_id' => 'exists:users,id',
            'wallet_id' => 'string',
            'beneficiary_id' => 'string',
        ]);

        // update bankPayoutRequest
        if ($req->has('amount')) $bankPayoutRequest->amount = $req->input('amount');
        if ($req->has('note')) $bankPayoutRequest->note = $req->input('note');
        if ($req->has('status')) $bankPayoutRequest->status = $req->input('status');
        if ($req->has('business_id')) $bankPayoutRequest->business_id = $req->input('business_id');
        if ($req->has('user_id')) $bankPayoutRequest->user_id = $req->input('user_id');
        if ($req->has('wallet_id')) $bankPayoutRequest->wallet_id = $req->input('wallet_id');
        if ($req->has('beneficiary_id')) $bankPayoutRequest->beneficiary_id = $req->input('beneficiary_id');
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
        }
        if ($req->has('user_id')) {
            $user = CaxtonUser::find($req->input('user_id'));
        }
        $passbook = Passbook::where('entity', 'caxton_payout_request')->where('entity_id', $bankPayoutRequest->id);
        if ($req->has('business_id')) {
            $passbook->where('business_id', $bankPayoutRequest->business_id);
        }
        $passbook = $passbook->first();
        if ($bankPayoutRequest->status == 'Accepted') {
            if (empty($passbook)) {
                $passbook = new Passbook;
            }
            $passbook->entity = "caxton_payout_request";
            $passbook->entity_id = $bankPayoutRequest->id;
            $passbook->business_id = $bankPayoutRequest->business_id;
            $passbook->user_id = $bankPayoutRequest->user_id;
            $passbook->date = Carbon::today()->toDateString();
            $passbook->from_date = $bankPayoutRequest->created_at;
            $passbook->to_date = $bankPayoutRequest->created_at;
            $passbook->amount = -1 * $bankPayoutRequest->amount;
            $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($bankPayoutRequest->created_at)->toDateString();
            $passbook->save();
        } else {
            if (!empty($passbook))
                $passbook->delete();
        }

        $bankPayoutRequest->save();

        return $bankPayoutRequest;
    }

    /**
     * Delete bankPayoutRequest by id.
     * DELETE /api/banking-payout-requests/{id}
     */
    public function delete(Request $req, $id)
    {
        $bankPayoutRequest = CaxtonPayoutRequest::find($id);
        if (empty($bankPayoutRequest)) Utils::error('PayoutRequest not found!', 404);
        $bankPayoutRequest->delete();
        return $bankPayoutRequest;
    }

    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}