<?php

namespace App\Http\Controllers;

use App\CaxtonBankDetails;
use App\CaxtonBeneficiary;
use App\CaxtonCard;
use App\CaxtonTransfer;
use App\CaxtonTransferTransaction;
use App\CaxtonTransaction;
use App\CaxtonUser;
use App\Business;
use App\CaxtonPayoutRequest;
use App\PayoutInvoice;
use App\CaxtonWallet;
use App\CaxtonWalletGroup;
use App\CaxtonWalletTransaction;
use App\Exports\TransactionsExport;
use App\Exports\TransferTransactionsExport;
use App\Exports\WalletTransactionsExport;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Jobs\WalletTransactionCreatorJob;
use App\Passbook;
use App\PouchHistory;
use App\StandingInstruction;
use Carbon\Carbon;
use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request as Psr7Request;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Maatwebsite\Excel\Facades\Excel;
use Mpdf\Mpdf;
use Illuminate\Support\Facades\Queue;
use function Psy\debug;

class CaxtonfxController extends Controller
{

    /**
     * Create client object.
     * POST /api/finance/users
     */
    public function index(Request $req)
    {
        $endPoint = 'account/balances';
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;

        try {
            $client = new Client([
                'base_uri' => env('CAXTON_BASEURL'),
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'accept' => 'application/json',
                    'UserApiToken' => 'DNDZUeiXYLweMkM44pHBsA==',
                ],
            ]);
            $response = $client->get($endPoint);
            $data = json_decode($response->getBody()->getContents(), true);
            return $data;
        } catch (\Exception $e) {
            throw new \Exception($e->getCode());
        }

        return $req;
    }



    /*
     |-------------------------------------------
     | Wallet Group Datails
     |-------------------------------------------
     */

    // GET /api/finance/wallet-groups
    public function wallet_group_index(Request $req)
    {
        $wallet_groups = CaxtonWalletGroup::orderByRaw('name ASC');

        if ($query = $req->query('query', false)) {
            $wallet_groups->whereRaw('(name LIKE ?)', ["%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $wallet_groups = $wallet_groups->get();
        } else {
            $wallet_groups = $wallet_groups->paginate($req->query('per_page', 20));
        }

        if ($wallet_groups->isEmpty()) Utils::error('No wallet groups found!', 404);

        return ($wallet_groups);
    }

    // POST /api/finance/wallet-groups
    public function wallet_group_create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
        ]);

        $wallet_group = new CaxtonWalletGroup;
        $wallet_group->fill($req->all());
        $wallet_group->save();

        return $wallet_group;
    }

    /**
     * Show wallet group object.
     * POST /api/finance/wallet-groups/id
     */
    public function wallet_group_show(Request $req, $id)
    {
        $wallet_group = CaxtonWalletGroup::find($id);
        if (empty($wallet_group)) Utils::error('wallet group not found!', 404);
        return $wallet_group;
    }

    // POST /api/finance/wallet-groups/{id}
    public function wallet_group_update(Request $req, $id)
    {
        $wallet_group = CaxtonWalletGroup::find($id);
        if (empty($wallet_group)) Utils::error('wallet group not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
        ]);

        if ($req->has('name')) $wallet_group->name  = $req->input('name');
        $wallet_group->save();

        return $wallet_group;
    }

    // DELETE /api/finance/wallet-groups/{id}
    public function wallet_group_delete(Request $req, $id)
    {
        $wallet_group = CaxtonWalletGroup::find($id);
        if (empty($wallet_group)) Utils::error('wallet group not found!', 404);
        $wallet_group->delete();
        return $wallet_group;
    }


    /*
     |-------------------------------------------
     | WALLET
     |-------------------------------------------
     */

    /**
     * List Wallet.
     * GET /api/finance/wallets-fetch
     */
    public function get_wallets(Request $req)
    {
        if ($sorting = $req->query('sorting', false)) {
            $sorting_type = $req->input('sorting_type');
            $wallets = CaxtonWallet::selectRaw('caxton_wallets.*,businesses.name as business_name')->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw($sorting_type . ' ' . $sorting);
        } else {
            $wallets = CaxtonWallet::selectRaw('caxton_wallets.*,businesses.name as business_name')->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('created_at DESC');
        }

        if ($archive = $req->query('archive', false)) {
            $wallets =  $wallets->where('archive', '1');
        } else {
            $wallets =  $wallets->where('archive', '0');
        }

        if ($bank = $req->query('bank', false)) {
            if ($bank == 'bank') {
                $wallets =  $wallets->where('bank', '1');
            }
            if ($bank == 'not_bank') {
                $wallets =  $wallets->where('bank', '0');
            }
        }

        if ($query = $req->query('query', false)) {
            $wallets->whereRaw('(caxton_wallets.name LIKE ? OR wallet_id LIKE ? OR businesses.name LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($group_id = $req->query('group_id', false)) {
            $wallets->where('group_id', $group_id);
        }

        if ($business_id = $req->query('business_id', false)) {
            $wallets->where('business_id', $business_id);
        }

        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";

            $wallets = $wallets->where('caxton_wallets.created_at', '>=', $from_date);
            $wallets = $wallets->where('caxton_wallets.created_at', '<=', $to_date);
        }

        if ($req->query('nopaginate', false) == '1') {
            $wallets = $wallets->get();
            if ($req->query('hidden_all', false)) {
                $wallets->makeHidden(['business', 'wallet_group', 'use_limit', 'annual_balance', 'reccuring_balance']);
            }
        } else {
            $wallets = $wallets->paginate($req->query('per_page', 20));
        }

        if ($wallets->isEmpty()) Utils::error('No wallets found!', 404);

        return $wallets;
    }

    /**
     * Show wallet object.
     * POST /api/finance/wallet/id
     */
    public function show_wallet(Request $req, $id)
    {
        $caxtonWallet = CaxtonWallet::find($id);
        if (empty($caxtonWallet)) Utils::error('wallet not found!', 404);
        return $caxtonWallet;
    }

    /**
     * Unsigned Business object.
     * POST /api/finance/unsigned-business/id
     */
    public function unsigned_business(Request $req, $id)
    {
        $caxtonWallet = CaxtonWallet::find($id);
        if (empty($caxtonWallet)) Utils::error('wallet not found!', 404);
        $caxtonWallet->business_id = null;
        $caxtonWallet->save();
        return $caxtonWallet;
    }

    /**
     * Unsigned group object.
     * POST /api/finance/unsigned-group/id
     */
    public function unsigned_group(Request $req, $id)
    {
        $caxtonWallet = CaxtonWallet::find($id);
        if (empty($caxtonWallet)) Utils::error('wallet not found!', 404);
        $caxtonWallet->group_id = null;
        $caxtonWallet->save();
        return $caxtonWallet;
    }

    /**
     * Fetch wallet object.
     * POST /api/finance/account-wallets
     */
    public function fetch_all_wallet(Request $req)
    {
        $endPoint = 'wallets';
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                '',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $wallets = $data->Content->Model->Wallets;
                foreach ($wallets as $nwallet) {
                    $dbWallet = CaxtonWallet::where('wallet_id', $nwallet->WalletId)->first();
                    if (empty($dbWallet)) {
                        $wallet = new CaxtonWallet;
                    }
                    $wallet->name  = $nwallet->Name;
                    $wallet->wallet_id  = $nwallet->WalletId;
                    $wallet->save();
                }
                return $wallets;
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $data;
        } catch (\Exception $e) {
        }

        return $req;
    }

    /**
     * Fetch card object.
     * POST /api/finance/account-cards
     */
    public function fetch_all_card(Request $req)
    {
        $wallets = CaxtonWallet::select('wallet_id')->where('archive', '0')->orderByRaw('name ASC, created_at DESC')->get();
        if ($wallets->isEmpty()) Utils::error('No wallets found!', 404);
        $wallets->makeHidden(['business', 'wallet_group', 'use_limit', 'annual_balance', 'reccuring_balance']);

        foreach ($wallets as $wallet) {
            $endPoint = 'wallets/wallet/' . $wallet->wallet_id . '/cards';

            $apiKey = SiteSettingController::findByKey('caxton_token')->value;
            $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

            try {
                $client = new \GuzzleHttp\Client();
                $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                    '',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                    $data = json_decode($data->getBody()->getContents());
                    $cards = $data->Content->Model->Cards;
                    foreach ($cards as $ncard) {
                        $card = CaxtonCard::where('card_id', $ncard->CardId)->first();
                        if (empty($card)) {
                            $card = new CaxtonCard();
                        }
                        $card->card_id  = $ncard->CardId;
                        $card->card_holder_name  = $ncard->CardHolderName;
                        $card->wallet_id  = $wallet->wallet_id;
                        $card->email  = $ncard->CardHolderEmail;
                        $card->expiry  = $ncard->Expiry;
                        $card->card_number  = $ncard->Last4Digits;
                        $card->mobile_number  = $ncard->MobileNumber;
                        $card->masked_pan  = $ncard->MaskedPan;
                        $card->sequence_number  = $ncard->CardSequenceNumber;
                        $card->is_renewed  = $ncard->IsRenewed;
                        $card->can_be_renewed  = $ncard->CanBeRenewed;
                        $card->reissue_blocked  = $ncard->ReissueBlocked;
                        $card->reissue_failure  = $ncard->ReissueFailure;
                        $card->is_expired  = $ncard->IsExpired;
                        $card->date_issued  = $ncard->DateIssued;
                        $card->is_expiring  = $ncard->IsExpiring;
                        $card->is_blocked_by_client  = $ncard->IsBlockedByClient;
                        $card->is_contactless  = $ncard->IsContactless;
                        $card->pan_type_code  = $ncard->PanTypeCode;
                        $card->base_pan_type_code  = $ncard->BasePanTypeCode;
                        $card->is_primary_pan  = $ncard->IsPrimaryPan;
                        $card->gps_token  = $ncard->GpsToken;
                        $card->card_state  = $ncard->CardState;
                        $card->save();
                    }
                }
            } catch (\Exception $e) {
                continue;
            }
        }

        return $wallets;
    }

    /**
     * Create wallet object.
     * POST /api/finance/wallet-create
     */
    public function wallet_create(Request $req)
    {

        $this->validate($req, [
            'name' => 'required|string|max:255|unique:caxton_wallets,name',
            'business_id' =>  ['required', 'exists:businesses,id', Rule::unique('caxton_wallets')->where(function ($query) use ($req) {
                return $query->where('business_id', $req->input('business_id'))->where('name', $req->input('name'));
            })],
            'group_id' => 'required|exists:caxton_wallet_groups,id',
            'bank' => 'boolean',
        ]);

        $endPoint = 'wallets';
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $name = $req->input('name');
            $business_id = $req->input('business_id');
            $group_id = $req->input('group_id');

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Name":"' . $name . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $wallet = new CaxtonWallet;
                $wallet->name  = $data->Content->Model->Name;
                $wallet->business_id  = $business_id;
                $wallet->group_id  = $group_id;
                $wallet->wallet_id  = $data->Content->Model->WalletId;
                $wallet->save();
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $wallet;
        } catch (\Exception $e) {
        }

        return $req;
    }

    /**
     * Update wallet object.
     * POST /api/finance/wallet-update/id
     */
    public function wallet_update(Request $req, $id)
    {
        $wallet = CaxtonWallet::find($id);
        if (empty($wallet)) Utils::error('Wallet not found!', 404);

        $this->validate($req, [
            'name' => ['required', 'string', 'max:255', Rule::unique('caxton_wallets')->ignore($wallet->id)],
            'business_id' =>  ['exists:businesses,id', Rule::unique('caxton_wallets')->where(function ($query) use ($req, $id) {
                return $query->where('business_id', $req->input('business_id'))->where('name', $req->input('name'))->whereNotIn('id', [$id]);
            })],
            'group_id' => 'required|exists:caxton_wallet_groups,id',
            'bank' => 'boolean',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $endPoint = 'wallets/wallet/' . $wallet->wallet_id . '/rename';
        $client = new \GuzzleHttp\Client();

        if ($req->has('name') && $wallet->name != $req->input('name')) {
            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                        "Name":"' . $req->input('name') . '",
                    }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                if ($req->has('name')) $wallet->name = $req->input('name');
                if ($req->has('business_id')) $wallet->business_id = $req->input('business_id');
                if ($req->has('group_id')) $wallet->group_id = $req->input('group_id');
                if ($req->has('wallet_id')) $wallet->wallet_id = $req->input('wallet_id');
                $wallet->bank = $req->input('bank');
                $wallet->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
        } else {
            if ($req->has('name')) $wallet->name = $req->input('name');
            if ($req->has('business_id')) $wallet->business_id = $req->input('business_id');
            if ($req->has('group_id')) $wallet->group_id = $req->input('group_id');
            if ($req->has('wallet_id')) $wallet->wallet_id = $req->input('wallet_id');
            $wallet->bank = $req->input('bank');
            $wallet->save();
        }

        return $wallet;
    }

    /**
     * Get wallet object.
     * POST /api/finance/wallet-archive/id
     */
    public function wallet_archived(Request $req, $id)
    {
        $wallet = CaxtonWallet::find($id);
        if (empty($wallet)) Utils::error('wallet not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;
        try {
            $cards =  CaxtonCard::where('wallet_id', $wallet->wallet_id)->where('card_state', 'Active')->get();
            foreach ($cards as $card) {
                $card = CaxtonCard::find($card->id);
                if (empty($card)) continue;

                $endPointBlock = 'wallets/wallet/' . $card->wallet_id . '/cards/' . $card->card_id . '/lost/' . $card->sequence_number;
                try {
                    $client = new \GuzzleHttp\Client();
                    $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPointBlock, [
                        'headers' => [
                            'Authorization' => 'bearer ' . $apiKey,
                            'UserApiToken' => $user_apiKey,
                            'content-type' => 'application/json',
                        ],
                    ]);
                    if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                        $card->card_state  = 'Lost';
                        $card->save();
                    } else {
                        continue;
                    }
                } catch (\Exception $e) {
                    continue;
                }
            }

            $client = new \GuzzleHttp\Client();
            $endPoint = 'wallets/wallet/' . $wallet->wallet_id . '/archive';

            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $wallet->archive  = '1';
                $wallet->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    $message = str_replace(
                        rtrim($e->getMessage()),
                        (string) $e->getResponse()->getBody(),
                        (string) $e
                    );
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $wallet;
    }

    /*
     |-------------------------------------------
     | CARD
     |-------------------------------------------
     */

    /**
     * List Card.
     * GET /api/finance/cards-fetch
     */
    public function get_cards(Request $req)
    {
        if ($sorting = $req->query('sorting', false)) {
            $sorting_type = $req->input('sorting_type');
            $cards = CaxtonCard::orderByRaw($sorting_type . ' ' . $sorting);
        } else {
            $cards = CaxtonCard::orderByRaw('created_at DESC');
        }

        if ($query = $req->query('query', false)) {
            $cards->whereRaw('(card_holder_name LIKE ? OR email LIKE ? OR card_number LIKE ? OR mobile_number LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($wallet_id = $req->query('wallet_id', false)) {
            $cards->where('wallet_id', $wallet_id);
        }


        if ($archive = $req->query('archive', false)) {
            if ($archive == 'archived') {
                $cards->whereRaw('(card_state LIKE ? OR card_state LIKE ?)', ["%Stolen%", "%Lost%"]);
            }
            if ($archive == 'unarchived') {
                $cards->whereRaw('(card_state LIKE ? OR card_state LIKE ? OR card_state LIKE ?)', ["%Inactive%", "%Active%", "%Blocked%"]);
            }
        }

        if ($not_status = $req->query('not_status', false)) {
            $cards->where('card_state', '!=', 'Inactive');
        }

        if ($status = $req->query('status', false)) {
            $cards->where('card_state', $status);
        }

        if ($req->has('from_date')) {

            $from_date = Carbon::parse($req->input('from_date'))->startOfDay()->format('Y-m-d H:i:s');
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay()->format('Y-m-d H:i:s');
            // $cards = $cards->whereRaw('(DATE_FORMAT(CAST(date_issued), "Y-m-d H:i:s") >= ? )', $from_date);
            $cards = $cards->whereDate('date_issued', '>=', $from_date);
            $cards = $cards->where('date_issued', '<=', $to_date);
        }

        if ($req->query('nopaginate', false) == '1') {
            $cards = $cards->get();
        } else {
            $cards = $cards->paginate($req->query('per_page', 20));
        }

        if ($cards->isEmpty()) Utils::error('No cards found!', 404);

        return ($cards);
    }

    /**
     * Create card object.
     * POST /api/finance/card-create
     */
    public function card_create(Request $req)
    {
        $this->validate($req, [
            'first_name' => 'required|string|max:255',
            'last_name' => 'required|string|max:255',
            'gender' => 'required|in:Male,Female,Other',
            'dob' => 'required|date_format:Y-m-d',
            'email' => 'email',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $first_name = $req->input('first_name');
            $last_name = $req->input('last_name');
            $gender = $req->input('gender');
            $dob = $req->input('dob');
            $email = $req->input('email');
            $wallet_id = $req->input('wallet_id');

            $endPoint = 'wallets/wallet/' . $wallet_id . '/cards/card';
            $client = new \GuzzleHttp\Client();

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "FirstName":"' . $first_name . '",
                    "LastName":"' . $last_name . '",
                    "Gender":"' . $gender . '",
                    "Email":"' . $email . '",
                    "DateOfBirth":"' . $dob . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'walletId' => $wallet_id,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $card = new CaxtonCard;
                $card->card_id  = $data->Content->Model->CardId;
                $card->card_holder_name  = $req->input('first_name') . ' ' . $req->input('last_name');
                $card->wallet_id  = $wallet_id;
                $card->email  = $req->input('email');
                $card->dob  = $req->input('dob');
                $card->gender  = $req->input('gender');
                $card->save();

                $cardEndPoint = 'wallets/wallet/' . $wallet_id . '/cards/card/' . $card->card_id;
                $cardDetail = $client->request('GET', env('CAXTON_BASEURL') . $cardEndPoint, [
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'walletId' => $wallet_id,
                        'cardId' => $card->card_id,
                        'content-type' => 'application/json',
                    ],
                ]);

                if ($cardDetail->getStatusCode() >= 200 && $cardDetail->getStatusCode() <= 210) {
                    $cardDetail = json_decode($cardDetail->getBody()->getContents());
                    $cards = $cardDetail->Content->Model->Cards;
                    foreach ($cards as $ncard) {
                        $card->expiry  = $ncard->Expiry;
                        $card->card_number  = $ncard->Last4Digits;
                        $card->mobile_number  = $ncard->MobileNumber;
                        $card->masked_pan  = $ncard->MaskedPan;
                        $card->sequence_number  = $ncard->CardSequenceNumber;
                        $card->is_renewed  = $ncard->IsRenewed;
                        $card->can_be_renewed  = $ncard->CanBeRenewed;
                        $card->reissue_blocked  = $ncard->ReissueBlocked;
                        $card->reissue_failure  = $ncard->ReissueFailure;
                        $card->is_expired  = $ncard->IsExpired;
                        $card->date_issued  = $ncard->DateIssued;
                        $card->is_expiring  = $ncard->IsExpiring;
                        $card->is_blocked_by_client  = $ncard->IsBlockedByClient;
                        $card->is_contactless  = $ncard->IsContactless;
                        $card->pan_type_code  = $ncard->PanTypeCode;
                        $card->base_pan_type_code  = $ncard->BasePanTypeCode;
                        $card->is_primary_pan  = $ncard->IsPrimaryPan;
                        $card->gps_token  = $ncard->GpsToken;
                        $card->card_state  = $ncard->CardState;
                        $card->save();
                    }
                }
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $card;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $message = str_replace(
                        rtrim($e->getMessage()),
                        (string) $e->getResponse()->getBody(),
                        (string) $e
                    );
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Generate Token card object.
     * POST /api/finance/card-generate-token
     */
    public function card_active_token_generate(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/card/' . $card->card_id . '/activate/v2/seq/' . $card->sequence_number;
            $client = new \GuzzleHttp\Client();

            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $card->activation_token  = $data->Content->Model->CardActivationToken;
                $card->index1  = $data->Content->Model->Index1;
                $card->index2  = $data->Content->Model->Index2;
                $card->index3  = $data->Content->Model->Index3;
                $card->index4  = $data->Content->Model->Index4;
                $card->index5  = $data->Content->Model->Index5;
                $card->index6  = $data->Content->Model->Index6;
                $card->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $card;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }


    /**
     * Active card object.
     * POST /api/finance/card-active
     */
    public function card_active(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $client = new \GuzzleHttp\Client();

            $Digit1 = $req->input('Digit1');
            $Digit2 = $req->input('Digit2');
            $Digit3 = $req->input('Digit3');
            $Digit4 = $req->input('Digit4');
            $Digit5 = $req->input('Digit5');
            $Digit6 = $req->input('Digit6');

            $activeEndPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/card/' . $card->card_id . '/activate/v2/seq/' . $card->sequence_number;
            $cardDetail = $client->request('POST', env('CAXTON_BASEURL') . $activeEndPoint, [
                'body' => '{
                        "CardActivationToken":"' . $card->activation_token . '",
                        "Digit1":' . $Digit1 . ',
                        "Digit2":' . $Digit2 . ',
                        "Digit3":' . $Digit3 . ',
                        "Digit4":' . $Digit4 . ',
                        "Digit5":' . $Digit5 . ',
                        "Digit6":' . $Digit6 . ',
                        "Index1":"' . $card->index1 . '",
                        "Index2":"' . $card->index2 . '",
                        "Index3":"' . $card->index3 . '",
                        "Index4":"' . $card->index4 . '",
                        "Index5":"' . $card->Index5 . '",
                        "Index6":"' . $card->Index6 . '",
                    }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($cardDetail->getStatusCode() >= 200 && $cardDetail->getStatusCode() <= 210) {
                $cardDetail = json_decode($cardDetail->getBody()->getContents());
                $card->card_state  = 'Active';
                $card->save();
            }

            return $card;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Move card object.
     * POST /api/finance/card-move
     */
    public function card_move(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;
        $user_id = SiteSettingController::findByKey('caxton_user_id')->value;

        try {
            $move_wallet_id = $req->input('move_wallet_id');
            $endPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/move';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "DestinationUserId":"' . $user_id . '",
                    "DestinationWalletId":' . $move_wallet_id . ',
                    "CardId":' . $card->card_id . ',
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $card->wallet_id  = $move_wallet_id;
                $card->save();
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $card;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Get pin object.
     * POST /api/finance/get-pin/id
     */
    public function get_pin(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $endPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/card/' . $card->card_id . '/pin';

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $card->pin_image = $data->Content->Model->Value;
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getCode());
        }

        return $card;
    }

    /**
     * Get Card object.
     * POST /api/finance/card-block/id
     */
    public function card_block(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $endPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/card/' . $card->card_id . '/block/seq/' . $card->sequence_number;

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $card->card_state  = 'Blocked';
                $card->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $card;
    }

    /**
     * Get Card object.
     * POST /api/finance/card-unblock/id
     */
    public function card_unblock(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $endPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/card/' . $card->card_id . '/unblock/seq/' . $card->sequence_number;

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $card->card_state  = 'Active';
                $card->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $card;
    }

    /**
     * Get Card object.
     * POST /api/finance/card-stolen/id
     */
    public function card_stolen(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $endPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/' . $card->card_id . '/stolen/' . $card->sequence_number;

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $card->card_state  = 'Stolen';
                $card->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $card;
    }

    /**
     * Get Card object.
     * POST /api/finance/card-lost/id
     */
    public function card_lost(Request $req, $id)
    {
        $card = CaxtonCard::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $endPoint = 'wallets/wallet/' . $card->wallet_id . '/cards/' . $card->card_id . '/lost/' . $card->sequence_number;

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $card->card_state  = 'Lost';
                $card->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $card;
    }

    /**
     * Get Admin balance object.
     * POST /api/finance/main-balance
     */
    public function admin_balance(Request $req)
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'account/balances';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $balances = $data->Content->Model->Balances;
                $balance_amount = 0;
                $balance_symbol = "£";

                foreach ($balances as $balance) {
                    $balance_amount = $balance->Balance;
                    $balance_symbol = $balance->Symbol;
                    break;
                }

                return [
                    'balance' => $balance_amount,
                    'symbol' => $balance_symbol
                ];
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Get balance object.
     * POST /api/finance/id/balance
     */
    public function get_balance(Request $req, $id)
    {
        $wallet = CaxtonWallet::find($id);
        if (empty($wallet)) Utils::error('wallet not found!', 404);


        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'wallets/wallet/' . $wallet->wallet_id . '/balances/balance/specific/ccy/GBP';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $balances = $data->Content->Model->Balances;
                $balance_amount = 0;
                $balance_symbol = "£";
                foreach ($balances as $balance) {
                    $balance_amount = $balance->Balance;
                    $balance_symbol = $balance->Symbol;
                }
                $wallet->balance  = $balance_amount;
                $wallet->symbol  = $balance_symbol;
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $wallet;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Load wallet object.
     * POST /api/finance/load-balance
     */
    public function load_balance(Request $req)
    {
        $this->validate($req, [
            'wallet_id' => 'required|string',
            'amount' => 'required|numeric|gt:10',
        ]);

        try {
            $wallet_id = $req->input('wallet_id');
            $amount = $req->input('amount');
            $time = Carbon::now();
            $time->addMinute();

            $caxtonWallet = CaxtonWallet::where('wallet_id', $wallet_id)->first();

            $bankPayoutRequest = new CaxtonPayoutRequest();
            $bankPayoutRequest->business_id = $caxtonWallet->business_id;
            $bankPayoutRequest->wallet_id = $caxtonWallet->id;
            $bankPayoutRequest->note = 'Payout for ' . $caxtonWallet->name;
            $bankPayoutRequest->amount = $amount;
            $bankPayoutRequest->status = 'Pending';
            $bankPayoutRequest->available_at = $time;
            $bankPayoutRequest->save();

            $payout = (object) array(
                'wallet_id' => $wallet_id,
                'amount' => $amount,
                'bank_request_id' => $bankPayoutRequest->id,
            );

            Queue::later($time, new WalletTransactionCreatorJob($payout, true), '', 'wallet_transfer');
        } catch (\Exception $e) {
            Utils::error('something want to wrong', 404);
        }

        // $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        // $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        // try {
        //     $endPoint = 'payments/currency-bank/card-load';
        //     $client = new \GuzzleHttp\Client();
        //     $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
        //         'body' => '{
        //             "WalletId":"' . $wallet_id . '",
        //             "Currency":"GBP",
        //             "Amount":"' . $amount . '",
        //         }',
        //         'headers' => [
        //             'Authorization' => 'bearer ' . $apiKey,
        //             'UserApiToken' => $user_apiKey,
        //             'content-type' => 'application/json',
        //         ],
        //     ]);
        //     if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
        //         $data = json_decode($data->getBody()->getContents());
        //         $shop_order_id  = $data->Content->Model->ShopOrderId;
        //         $processEndPoint = 'payments/currency-bank/process';
        //         $walletDetails = $client->request('POST', env('CAXTON_BASEURL') . $processEndPoint, [
        //             'body' => '{
        //                 "ShopOrderId":"' . $shop_order_id . '",
        //             }',
        //             'headers' => [
        //                 'Authorization' => 'bearer ' . $apiKey,
        //                 'UserApiToken' => $user_apiKey,
        //                 'content-type' => 'application/json',
        //             ],
        //         ]);

        //         if ($walletDetails->getStatusCode() >= 200 && $walletDetails->getStatusCode() <= 210) {
        //             $walletDetails = json_decode($walletDetails->getBody()->getContents());
        //         }
        //     } else {
        //         Utils::error('something want to wrong', 404);
        //     }

        //     return ['message' => 'wallet load successfully!'];
        // } catch (\Exception $e) {
        //     if ($e->getCode() == 401) {
        //         $this->setting_generate_token();
        //     } else {
        //         if ($e instanceof \GuzzleHttp\Exception\RequestException) {
        //             $data = json_decode($e->getResponse()->getBody());
        //             Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
        //         }
        //     }
        // }

        return $req;
    }

    /**
     * Redeem wallet object.
     * POST /api/finance/redeem-balance
     */
    public function redeem_balance(Request $req)
    {
        $this->validate($req, [
            'wallet_id' => 'required|string',
            'amount' => 'required|numeric|gt:0',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $wallet_id = $req->input('wallet_id');
            $amount = $req->input('amount');

            $endPoint = 'wallets/balances/redeem/' . $wallet_id;

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Currency":"GBP",
                    "Amount":"' . $amount . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                return ['message' => 'wallet load successfully!'];
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * List Transactions.
     * GET /api/finance/wallet-transaction-fetch
     */
    public function get_wallet_transactions(Request $req)
    {
        $transactions = CaxtonWalletTransaction::orderByRaw('transaction_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(description LIKE ? OR debit_amount LIKE ? OR credit_amount LIKE ? OR transaction_category LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($transaction_category = $req->query('transaction_category', false)) {
            if ($transaction_category != 'null') {
                $transactions->whereRaw('(transaction_category LIKE ? )', ["%$transaction_category%"]);
            }
        }
        if ($transaction_status = $req->query('transaction_status', false)) {
            if ($transaction_status == 'pending') {
                $transactions->where('is_authorisation', '1');
            }
            if ($transaction_status == 'success') {
                $transactions->where('is_transaction', '1');
            }
        }
        if ($wallet_id = $req->query('wallet_id', false)) {
            if ($wallet_id != 'null') {
                $transactions->where('wallet_id', $wallet_id);
            }
        }
        if ($business_id = $req->query('business_id', false)) {
            if ($business_id != 'null') {
                $wallets = CaxtonWallet::select('wallet_id')->where('business_id', $business_id)->where('archive', '0')->orderByRaw('id DESC')->get();
                if (!$wallets->isEmpty()) {
                    $wallets->makeHidden('business');
                    $wallets->makeHidden('wallet_group');
                    $wallets->makeHidden('use_limit');
                    $wallets->makeHidden('annual_balance');
                    $wallets->makeHidden('reccuring_balance');
                    $transactions->whereIn('wallet_id', $wallets);
                } else {
                    Utils::error('No wallet found!', 404);
                }
            }
        }
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = $req->input('from_date') . " 00:00:00";
                $to_date = $req->input('to_date') . " 23:59:59";

                $transactions = $transactions->where('transaction_date', '>=', $from_date);
                $transactions = $transactions->where('transaction_date', '<=', $to_date);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $transactions = $transactions->get();
        } else {
            $transactions = $transactions->paginate($req->query('per_page', 20));
        }

        if ($transactions->isEmpty()) Utils::error('No transactions found!', 404);

        return ($transactions);
    }

    /**
     * count Transactions.
     * GET /api/finance/wallet-transaction-fetch-count
     */
    public function get_wallet_transactions_count(Request $req)
    {
        $transactions = CaxtonWalletTransaction::orderByRaw('transaction_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(description LIKE ? OR debit_amount LIKE ? OR credit_amount LIKE ? OR transaction_category LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($description = $req->query('description', false)) {
            if ($description != 'null') {
                $transactions->whereRaw('(description LIKE ? )', ["%$description%"]);
            }
        }
        if ($wallet_id = $req->query('wallet_id', false)) {
            if ($wallet_id != 'null') {
                $transactions->where('wallet_id', $wallet_id);
            }
        }
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = $req->input('from_date') . " 00:00:00";
                $to_date = $req->input('to_date') . " 23:59:59";

                $transactions = $transactions->where('transaction_date', '>=', $from_date);
                $transactions = $transactions->where('transaction_date', '<=', $to_date);
            }
        }
        $transactions = $transactions->count();

        $data = [
            'no_of_transactions' => (int)$transactions,
        ];

        return $data;
    }

    /**
     * Fetch Transactions object.
     * POST /api/finance/account-wallet-transaction
     */
    public function fetch_all_wallet_transaction(Request $req)
    {
        $wallets = CaxtonWallet::select('wallet_id')->where('archive', '0')->orderByRaw('name ASC, created_at DESC')->get();
        if ($wallets->isEmpty()) Utils::error('No wallets found!', 404);
        $wallets->makeHidden(['business', 'wallet_group', 'use_limit', 'annual_balance', 'reccuring_balance']);

        foreach ($wallets as $wallet) {
            $endPoint = 'transactions/wallets/wallet/' . $wallet->wallet_id;

            $apiKey = SiteSettingController::findByKey('caxton_token')->value;
            $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

            try {
                $client = new \GuzzleHttp\Client();
                // "TransactionHistoryTypeFilter":"Both",
                $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                    'body' => '{
                        "DateFrom":"' . Carbon::today()->subDays(10)->format('Y-m-d') . '",
                        "DateTo":"' . Carbon::today()->format('Y-m-d') . '",
                        "CardIdFilter":"0",
                        "TransactionTypeFilter": [
                            0,1,2,3,4,5,6,7,8
                            ]
                    }',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                    $data = json_decode($data->getBody()->getContents());
                    $transactions = $data->Content->Model->Result;
                    foreach ($transactions as $ntransaction) {
                        $transaction = CaxtonWalletTransaction::where('valitor_transaction_id', $ntransaction->ValitorTransactionId)->first();
                        // where('authorization_number', $ntransaction->AuthorizationNumber)->
                        if (empty($transaction)) {
                            $transaction = new CaxtonWalletTransaction();
                        }
                        $transaction->wallet_id = $wallet->wallet_id;
                        $transaction->card_id = $ntransaction->CardId;
                        $transaction->description = $ntransaction->Description;
                        $transaction->debit_amount = $ntransaction->DebitAmount;
                        $transaction->debit_currency = $ntransaction->DebitCurrency;
                        $transaction->debit_currency_symbol = $ntransaction->DebitCurrencySymbol;
                        $transaction->credit_amount = $ntransaction->CreditAmount;
                        $transaction->credit_currency = $ntransaction->CreditCurrency;
                        $transaction->credit_currency_symbol = $ntransaction->CreditCurrencySymbol;
                        $transaction->transaction_category = $ntransaction->TransactionCategory;
                        $transaction->transaction_type = $ntransaction->TransactionType;
                        $transaction->transaction_type_desc = $ntransaction->TransactionTypeDesc;
                        $transaction->transaction_date = date('Y-m-d h:m:i', strtotime($ntransaction->TransactionDate));
                        $transaction->is_authorisation = $ntransaction->IsAuthorisation;
                        $transaction->is_transaction = $ntransaction->IsTransaction;
                        $transaction->is_balance_adjustment = $ntransaction->IsBalanceAdjustment;
                        $transaction->pan_reference_number = $ntransaction->PanReferenceNumber;
                        $transaction->exchange_rate = $ntransaction->ExchangeRate;
                        $transaction->transaction_amount = $ntransaction->TransactionAmount;
                        $transaction->transaction_currency = $ntransaction->TransactionCurrency;
                        $transaction->transaction_currency_symbol = $ntransaction->TransactionCurrencySymbol;
                        $transaction->valitor_transaction_id = $ntransaction->ValitorTransactionId;
                        $transaction->merchant_name = $ntransaction->MerchantName;
                        $transaction->merchant_city = $ntransaction->MerchantCity;
                        $transaction->merchant_country = $ntransaction->MerchantCountry;
                        $transaction->is_loyalty_point = $ntransaction->IsLoyaltyPoint;
                        $transaction->authorization_number = $ntransaction->AuthorizationNumber;
                        $transaction->card_token_id = $ntransaction->CardTokenId;
                        $transaction->is_fee = $ntransaction->IsFee;
                        $transaction->is_wallet_alignment = $ntransaction->IsWalletAlignment;
                        $transaction->save();
                    }
                } else {
                    continue;
                }
            } catch (\Exception $e) {
                continue;
            }
        }

        return $wallets;
    }

    /**
     * Fetch Transactions object.
     * POST /api/finance/account-refund-wallet-transaction
     */
    public function fetch_all_refund_wallet_transaction(Request $req)
    {
        $wallets = CaxtonWallet::select('wallet_id')->where('archive', '0')->orderByRaw('name ASC, created_at DESC')->get();
        if ($wallets->isEmpty()) Utils::error('No wallets found!', 404);
        $wallets->makeHidden(['business', 'wallet_group', 'use_limit', 'annual_balance', 'reccuring_balance']);

        foreach ($wallets as $wallet) {
            $endPoint = 'transactions/wallets/wallet/' . $wallet->wallet_id;

            $apiKey = SiteSettingController::findByKey('caxton_token')->value;
            $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

            try {
                $client = new \GuzzleHttp\Client();
                $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                    'body' => '{
                        "DateFrom":"' . Carbon::today()->subDays(365)->format('Y-m-d') . '",
                        "DateTo":"' . Carbon::today()->format('Y-m-d') . '",
                        "CardIdFilter":"0",
                        "TransactionTypeFilter": [
                            "5"
                        ]
                    }',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                    $data = json_decode($data->getBody()->getContents());
                    $transactions = $data->Content->Model->Result;
                    foreach ($transactions as $ntransaction) {
                        $transaction = CaxtonWalletTransaction::where('valitor_transaction_id', $ntransaction->ValitorTransactionId)->first();
                        if (empty($transaction)) {
                            $transaction = new CaxtonWalletTransaction();
                        }
                        $transaction->wallet_id = $wallet->wallet_id;
                        $transaction->card_id = $ntransaction->CardId;
                        $transaction->description = $ntransaction->Description;
                        $transaction->debit_amount = $ntransaction->DebitAmount;
                        $transaction->debit_currency = $ntransaction->DebitCurrency;
                        $transaction->debit_currency_symbol = $ntransaction->DebitCurrencySymbol;
                        $transaction->credit_amount = $ntransaction->CreditAmount;
                        $transaction->credit_currency = $ntransaction->CreditCurrency;
                        $transaction->credit_currency_symbol = $ntransaction->CreditCurrencySymbol;
                        $transaction->transaction_category = $ntransaction->TransactionCategory;
                        $transaction->transaction_type = $ntransaction->TransactionType;
                        $transaction->transaction_type_desc = $ntransaction->TransactionTypeDesc;
                        $transaction->transaction_date = date('Y-m-d h:m:i', strtotime($ntransaction->TransactionDate));
                        $transaction->is_authorisation = $ntransaction->IsAuthorisation;
                        $transaction->is_transaction = $ntransaction->IsTransaction;
                        $transaction->is_balance_adjustment = $ntransaction->IsBalanceAdjustment;
                        $transaction->pan_reference_number = $ntransaction->PanReferenceNumber;
                        $transaction->exchange_rate = $ntransaction->ExchangeRate;
                        $transaction->transaction_amount = $ntransaction->TransactionAmount;
                        $transaction->transaction_currency = $ntransaction->TransactionCurrency;
                        $transaction->transaction_currency_symbol = $ntransaction->TransactionCurrencySymbol;
                        $transaction->valitor_transaction_id = $ntransaction->ValitorTransactionId;
                        $transaction->merchant_name = $ntransaction->MerchantName;
                        $transaction->merchant_city = $ntransaction->MerchantCity;
                        $transaction->merchant_country = $ntransaction->MerchantCountry;
                        $transaction->is_loyalty_point = $ntransaction->IsLoyaltyPoint;
                        $transaction->authorization_number = $ntransaction->AuthorizationNumber;
                        $transaction->card_token_id = $ntransaction->CardTokenId;
                        $transaction->is_fee = $ntransaction->IsFee;
                        $transaction->is_wallet_alignment = $ntransaction->IsWalletAlignment;
                        $transaction->save();
                    }
                } else {
                    continue;
                }
            } catch (\Exception $e) {
                continue;
            }
        }

        return $wallets;
    }

    public function export_wallet_transaction(Request $req)
    {
        $reports = '';
        $req->merge(['nopaginate' => 1]);

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $fileName =  $from_date->format('d-m-Y') . " to " . $to_date->format('d-m-Y');
            } else {
                $fileName = "Wallet Transactions";
            }
        } else {
            $fileName = "Wallet Transactions";
        }

        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $reports = ($this->get_wallet_transactions($req));
            return Excel::download(new WalletTransactionsExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            $reports = ($this->get_wallet_transactions($req));
            return Excel::download(new WalletTransactionsExport($reports), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    /*
     |-------------------------------------------
     | BANK DETAILS
     |-------------------------------------------
     */

    /**
     * Get balance object.
     * POST /api/finance/id/balance
     */
    public function get_bank_details(Request $req)
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'reference/lookup/banks/GBP';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $bankDetails = $data->Content->Model->BankDetails;
                foreach ($bankDetails as $bankDetail) {
                    $bankdetail = CaxtonBankDetails::where('account_number', $bankDetail->AccountNumber)->first();
                    if (empty($bankdetail)) {
                        $bankdetail = new CaxtonBankDetails;
                    }
                    $bankdetail->account_name  = $bankDetail->AccountName;
                    $bankdetail->currency  = $bankDetail->Currency;
                    $bankdetail->bank_name  = $bankDetail->BankName;
                    $bankdetail->account_number  = $bankDetail->AccountNumber;
                    $bankdetail->iban_number  = $bankDetail->IbanNumber;
                    $bankdetail->swift_code  = $bankDetail->SwiftCode;
                    $bankdetail->sortcode  = $bankDetail->SortCode;
                    $bankdetail->address  = $bankDetail->BankAddress;
                    $bankdetail->address1  = $bankDetail->BankAddress1;
                    $bankdetail->address2  = $bankDetail->BankAddress2;
                    $bankdetail->country  = $bankDetail->Country;
                    $bankdetail->type  = $bankDetail->Type;
                    $bankdetail->pay_in_reference  = $bankDetail->PayInReference;
                    $bankdetail->save();
                }
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $bankdetail;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Get Admin Bank Detail object.
     * POST /api/finance/id/admin-bank-details
     */
    public function admin_bank_details(Request $req)
    {
        $bankDetails = CaxtonBankDetails::whereNull('user_id')->first();
        if (empty($bankDetails)) Utils::error('bank-details not found!', 404);

        return $bankDetails;
    }

    /*
     |-------------------------------------------
     | BENEFICIARIES
     |-------------------------------------------
     */


    /**
     * List Card.
     * GET /api/finance/beneficiary-fetch
     */
    public function get_beneficiary(Request $req)
    {
        $beneficiary = CaxtonBeneficiary::selectRaw('caxton_beneficiaries.*,caxton_users.name as user_name,businesses.name as business_name')->leftJoin('caxton_users', 'caxton_users.id', '=', 'caxton_beneficiaries.user_id')->leftJoin('businesses', 'businesses.id', '=', 'caxton_beneficiaries.business_id')->orderByRaw('caxton_beneficiaries.created_at DESC');

        if ($query = $req->query('query', false)) {
            $beneficiary->whereRaw('(caxton_beneficiaries.name LIKE ? OR  caxton_beneficiaries.email LIKE ? OR caxton_beneficiaries.phone LIKE ? OR businesses.name LIKE ? OR caxton_users.name LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($business_id = $req->query('business_id', false)) {
            $beneficiary->where('caxton_beneficiaries.business_id', $business_id);
        }

        if ($type = $req->query('type', false)) {
            if ($type == 'user') {
                $beneficiary->whereNotNull('caxton_beneficiaries.user_id');
            }
            if ($type == 'admin') {
                $beneficiary->whereNull('caxton_beneficiaries.user_id');
            }
        }

        $query = $req->query('archive');
        if ($query != null) {
            $beneficiary->where('archive', $query == 0 ? '0' : '1');
        }

        if ($req->query('nopaginate', false) == '1') {
            $beneficiary = $beneficiary->get();
        } else {
            $beneficiary = $beneficiary->paginate($req->query('per_page', 20));
        }

        if ($beneficiary->isEmpty()) Utils::error('No beneficiaries found!', 404);

        return ($beneficiary);
    }

    /**
     * Create beneficiary object.
     * POST /api/finance/beneficiary-create
     */
    public function beneficiary_create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'country' => 'required|string|max:255',
            'currency' => 'required|string|max:255',
            'iban' => 'string|max:255',
            'swift' => 'string|max:255',
            'sortCode' => 'required|string|max:255',
            'account_number' => 'required|string|max:255',
            'bank_name' => 'string|max:255',
            'reference' => 'string|max:255',
            'address1' => 'required|string|max:255',
            'address2' => 'required|string|max:255',
            'email' => 'email',
            'phone' => 'string|max:15',
            'beneficiary_type' => 'required|in:Entity,Individual',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $name = str_replace(' ', '', $req->input('name'));
            $bank_name = $req->input('bank_name');
            $address1 = $req->input('address1');
            $address2 = $req->input('address2');
            $email = $req->input('email');
            $phone = $req->input('phone');
            $country = $req->input('country');
            $currency = $req->input('currency');
            $iban = $req->input('iban');
            $swift = $req->input('swift');
            $sortCode = $req->input('sortCode');
            $account_number = $req->input('account_number');
            $notes = $req->input('notes');

            $endPoint = 'beneficiaries/add';
            $client = new \GuzzleHttp\Client();

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Name":"' . $name . '",
                    "Reference": "' . $notes . '",
                    "Currency": "' . $currency . '",
                    "BeneficiaryAddress1": "",
                    "BeneficiaryAddress2": "",
                    "Email":"' . $email . '",
                    "Phone":"' . $phone . '",
                    "ClientBeneficiaryReference": "",
                    "BankName":"' . $bank_name . '",
                    "AccountNumber": "' . $account_number . '",
                    "Iban": "' . $iban . '",
                    "Swift": "' . $swift . '",
                    "SortCode":"' . $sortCode . '",
                    "BankAddress1":"' . $address1 . '",
                    "BankAddress2":"' . $address2 . '",
                    "Country":"' . $country . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $beneficiary = new CaxtonBeneficiary;
                $beneficiary->fill($req->all());
                $beneficiary->beneficiary_id  = $data->Content->Model->Value;
                $beneficiary->beneficiary_type  = $req->input('beneficiary_type');
                $beneficiary->save();
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $beneficiary;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Lookup beneficiary object.
     * POST /api/finance/beneficiary-lookup
     */
    public function beneficiary_lookup(Request $req)
    {
        $this->validate($req, [
            'sortCode' => 'required|string|max:255',
            'account_number' => 'required|string|max:255',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $sortCode = $req->input('sortCode');
            $account_number = $req->input('account_number');

            $endPoint = 'beneficiaries/retrieve-bank-details';
            $client = new \GuzzleHttp\Client();

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Country":"GB",
                    "Currency": "GBP",
                    "Iban": "",
                    "Swift": "",
                    "AccountNumber": "' . $account_number . '",
                    "SortCode":"' . $sortCode . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $beneficiary = new CaxtonBeneficiary;
                $beneficiary->bank_name  = $data->Content->Model->BankDetails[0]->BankName;
                $beneficiary->account_number  = $data->Content->Model->BankDetails[0]->AccountNumber;
                $beneficiary->iban  = $data->Content->Model->BankDetails[0]->IbanNumber;
                $beneficiary->swift  = $data->Content->Model->BankDetails[0]->SwiftCode;
                $beneficiary->sortCode  = $data->Content->Model->BankDetails[0]->SortCode;
                $beneficiary->address1  = $data->Content->Model->BankDetails[0]->BankAddress1;
                $beneficiary->address2  = $data->Content->Model->BankDetails[0]->BankAddress2;
                $beneficiary->country  = $data->Content->Model->BankDetails[0]->Country;
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $beneficiary;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Update beneficiary object.
     * POST /api/finance/beneficiary-update/{id}
     */
    public function beneficiary_update(Request $req, $id)
    {
        $beneficiary = CaxtonBeneficiary::find($id);
        if (empty($beneficiary)) Utils::error('Beneficiary not found!', 404);

        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'country' => 'required|string|max:255',
            'currency' => 'required|string|max:255',
            'iban' => 'string|max:255',
            'swift' => 'string|max:255',
            'sortCode' => 'required|string|max:255',
            'account_number' => 'required|string|max:255',
            'bank_name' => 'string|max:255',
            'reference' => 'string|max:255',
            'address1' => 'required|string|max:255',
            'address2' => 'required|string|max:255',
            'email' => 'email',
            'phone' => 'string|max:15',
            'beneficiary_type' => 'required|in:Entity,Individual',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $name = $req->input('name');
            $bank_name = $req->input('bank_name');
            $address1 = $req->input('address1');
            $address2 = $req->input('address2');
            $email = $req->input('email');
            $phone = $req->input('phone');
            $beneficiary_type = $req->input('beneficiary_type');

            if ($req->has('business_id')) $beneficiary->business_id  = $req->input('business_id');
            $beneficiary->save();

            $endPoint = 'beneficiaries/' . $beneficiary->beneficiary_id;
            $client = new \GuzzleHttp\Client();

            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Name":"' . $name . '",
                    "BankName":"' . $bank_name . '",
                    "Salutation": "",
                    "Reference": "",
                    "BankAddress1":"' . $address1 . '",
                    "BankAddress2":"' . $address2 . '",
                    "Email":"' . $email . '",
                    "Phone":"' . $phone . '",
                    "BeneficiaryType":"' . $beneficiary_type . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                if ($req->has('name')) $beneficiary->name  = $name;
                if ($req->has('beneficiary_id')) $beneficiary->beneficiary_id = $req->input('beneficiary_id');
                if ($req->has('business_id')) $beneficiary->business_id = $req->input('business_id');
                if ($req->has('address1')) $beneficiary->address1 = $req->input('address1');
                if ($req->has('address2')) $beneficiary->address2 = $req->input('address2');
                if ($req->has('email')) $beneficiary->email = $req->input('email');
                if ($req->has('phone')) $beneficiary->phone = $req->input('phone');
                if ($req->has('country')) $beneficiary->country = $req->input('country');
                if ($req->has('currency')) $beneficiary->currency = $req->input('currency');
                if ($req->has('iban')) $beneficiary->iban = $req->input('iban');
                if ($req->has('swift')) $beneficiary->swift = $req->input('swift');
                if ($req->has('sortCode')) $beneficiary->sortCode = $req->input('sortCode');
                if ($req->has('account_number')) $beneficiary->account_number = $req->input('account_number');
                if ($req->has('beneficiary_type')) $beneficiary->beneficiary_type = $req->input('beneficiary_type');
                $beneficiary->save();
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $beneficiary;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Delete beneficiary by id.
     * DELETE /api/finance/beneficiary-delete/{id}
     */
    public function delete_beneficiary(Request $req, $id)
    {
        $beneficiary = CaxtonBeneficiary::find($id);
        if (empty($beneficiary)) Utils::error('Beneficiary not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'beneficiaries/' . $beneficiary->beneficiary_id;
            $client = new \GuzzleHttp\Client();

            $data = $client->request('DELETE', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $beneficiary->delete();
                return $beneficiary;
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $beneficiary;
    }


    /**
     * Enable Archive beneficiary by id.
     * POST /api/finance/{id}/beneficiary-enable-archive
     */
    public function enable_archive_beneficiary(Request $req, $id)
    {
        $beneficiary = CaxtonBeneficiary::find($id);
        if (empty($beneficiary)) Utils::error('Beneficiary not found!', 404);
        $beneficiary->archive = 0;
        $beneficiary->save();
        return $beneficiary;
    }

    /**
     * Disable Archive beneficiary by id.
     * POST /api/finance/{id}/beneficiary-disable-archive
     */
    public function disable_archive_beneficiary(Request $req, $id)
    {
        $beneficiary = CaxtonBeneficiary::find($id);
        if (empty($beneficiary)) Utils::error('Beneficiary not found!', 404);
        $beneficiary->archive = 1;
        $beneficiary->save();
        return $beneficiary;
    }



    /**
     * Fetch Beneficiaries object.
     * POST /api/finance/account-beneficiaries
     */
    public function fetch_all_beneficiaries(Request $req)
    {
        $endPoint = 'beneficiaries';

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $baneficiaries = $data->Content->Model->Beneficiary;
                foreach ($baneficiaries as $nbaneficiary) {
                    $baneficiary = CaxtonBeneficiary::where('beneficiary_id', $nbaneficiary->Id)->first();
                    if (empty($baneficiary)) {
                        $baneficiary = new CaxtonBeneficiary();
                        $baneficiary->beneficiary_id = $nbaneficiary->Id;
                    }
                    $baneficiary->name = $nbaneficiary->Name;
                    $baneficiary->bank_name = $nbaneficiary->Account->BankName;
                    $baneficiary->reference = $nbaneficiary->Reference;
                    $baneficiary->notes = $nbaneficiary->Reference;
                    $baneficiary->address1 = $nbaneficiary->Account->BankAddress1;
                    $baneficiary->address2 = $nbaneficiary->Account->BankAddress2;
                    $baneficiary->email = $nbaneficiary->UserName;
                    $baneficiary->phone = $nbaneficiary->Phone;
                    $baneficiary->country = $nbaneficiary->Account->Country;
                    $baneficiary->currency = $nbaneficiary->Account->Currency;
                    $baneficiary->iban = $nbaneficiary->Account->IbanNumber;
                    $baneficiary->swift = $nbaneficiary->Account->SwiftCode;
                    $baneficiary->sortCode = $nbaneficiary->Account->SortCode;
                    $baneficiary->account_number = $nbaneficiary->Account->AccountNumber;
                    $baneficiary->account_number = $nbaneficiary->Account->AccountNumber;
                    $baneficiary->save();
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $baneficiaries;
    }


    /*
     |-------------------------------------------
     | Standing Instruction Datails
     |-------------------------------------------
     */

    // GET /api/finance/standing-instructions
    public function standing_instruction_index(Request $req)
    {
        $standing_instructions = StandingInstruction::selectRaw('standing_instructions.*,caxton_beneficiaries.name as beneficiary_name')->leftJoin('caxton_beneficiaries', 'caxton_beneficiaries.id', '=', 'standing_instructions.beneficiary_id')->orderByRaw('standing_instructions.created_at DESC');

        if ($query = $req->query('query', false)) {
            $standing_instructions->whereRaw('(standing_instructions.amount LIKE ?)', ["%$query%"]);
        }
        $standing_instructions->whereNull('standing_instructions.user_id');

        if ($beneficiary_id = $req->query('beneficiary_id', false)) {
            $standing_instructions->where('standing_instructions.beneficiary_id', $beneficiary_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $standing_instructions = $standing_instructions->get();
        } else {
            $standing_instructions = $standing_instructions->paginate($req->query('per_page', 20));
        }

        if ($standing_instructions->isEmpty()) Utils::error('No standing instructions found!', 404);

        return ($standing_instructions);
    }

    // POST /api/finance/standing-instructions
    public function standing_instruction_create(Request $req)
    {
        $this->validate($req, [
            // 'user_id' => 'required|exists:caxton_users,id',
            'beneficiary_id' => 'required|exists:caxton_beneficiaries,id',
            'notes' => 'string',
            'amount' => 'required|numeric|gt:0',
            'standing_type' => 'required|in:weekly,monthly',
            'order_date' => 'required_if:standing_type,monthly|date',
            'expiry_date' => 'required|date',
            'days' => 'required_if:standing_type,weekly|string',
        ]);

        $standing_instruction = new StandingInstruction;
        $standing_instruction->fill($req->all());
        $standing_instruction->save();

        return $standing_instruction;
    }

    // POST /api/finance/standing-instructions/{id}
    public function standing_instruction_update(Request $req, $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);

        $this->validate($req, [
            // 'user_id' => 'required|exists:caxton_users,id',
            'beneficiary_id' => 'required|exists:caxton_beneficiaries,id',
            'notes' => 'string',
            'amount' => 'required|numeric|gt:0',
            'standing_type' => 'required|in:weekly,monthly',
            'order_date' => 'required_if:standing_type,monthly|date',
            'expiry_date' => 'required|date',
            'days' => 'required_if:standing_type,weekly|string',
        ]);

        // if ($req->has('user_id')) $standing_instruction->user_id = $req->input('user_id');
        if ($req->has('beneficiary_id')) $standing_instruction->beneficiary_id  = $req->input('beneficiary_id');
        if ($req->has('notes')) $standing_instruction->notes  = $req->input('notes');
        if ($req->has('amount')) $standing_instruction->amount  = $req->input('amount');
        if ($req->has('standing_type')) $standing_instruction->standing_type  = $req->input('standing_type');
        if ($req->has('expiry_date')) $standing_instruction->expiry_date  = $req->input('expiry_date');
        if ($req->has('days')) $standing_instruction->days  = $req->input('days');
        if ($req->has('order_date')) $standing_instruction->order_date  = $req->input('order_date');
        $standing_instruction->save();

        return $standing_instruction;
    }

    // DELETE /api/finance/standing-instructions/{id}
    public function standing_instruction_delete(Request $req, $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);
        $standing_instruction->delete();
        return $standing_instruction;
    }

    /**
     * Enable standing-instructions.
     * POST /api/finance/standing-instructions/{id}/enable
     */
    public function standing_instruction_enable(Request $req,  $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);
        $standing_instruction->disabled = 0;
        $standing_instruction->save();
        return $standing_instruction;
    }

    /**
     * Disable standing-instructions.
     * POST /api/finance/standing-instructions/{id}/disable
     */
    public function standing_instruction_disable(Request $req,  $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);
        $standing_instruction->disabled = 1;
        $standing_instruction->save();
        return $standing_instruction;
    }


    /*
     |-------------------------------------------
     | TRANSFER DETAILS
     |-------------------------------------------
     */

    /**
     * List Card.
     * GET /api/finance/transfer-fetch
     */
    public function get_transfer(Request $req)
    {
        $transfers = CaxtonTransfer::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $transfers->whereRaw('(buy_amount LIKE ? OR beneficiary_name LIKE ? )', ["%$query%", "%$query%"]);
        }

        if ($business_id = $req->query('business_id', false)) {
            if ($business_id != 'null') {
                $beneficiaries = CaxtonBeneficiary::select('beneficiary_id')->where('archive', 0)->where('business_id', $business_id)->orderByRaw('id DESC')->get();
                $transfers->whereIn('beneficiary_id', $beneficiaries);
            }
        }
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = $req->input('from_date') . " 00:00:00";
                $to_date = $req->input('to_date') . " 23:59:59";

                $transfers = $transfers->where('created_at', '>=', $from_date);
                $transfers = $transfers->where('created_at', '<=', $to_date);
            }
        }

        if ($req->query('nopaginate', false) == '1') {
            $transfers = $transfers->get();
        } else {
            $transfers = $transfers->paginate($req->query('per_page', 20));
        }

        if ($transfers->isEmpty()) Utils::error('No transfers found!', 404);

        return ($transfers);
    }

    /**
     * Pending transfer object.
     * POST /api/finance/id/transfer
     */
    public function fetch_pending_transfers(Request $req)
    {
        $caxton_user_id = SiteSettingController::findByKey('caxton_user_id')->value;

        $caxtonTransfers = CaxtonTransfer::where('user_id', $caxton_user_id)->orderByRaw('created_at DESC')->get();
        if ($caxtonTransfers->isEmpty()) Utils::error('No transfers found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        foreach ($caxtonTransfers as $caxtonTransfer) {
            try {
                $caxtonTransfer = CaxtonTransfer::find($caxtonTransfer->id);
                if ($caxtonTransfer->transfer_id == 0) {
                    $clientTransfer = new \GuzzleHttp\Client();
                    $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                    $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                        '',
                        'headers' => [
                            'Authorization' => 'bearer ' . $apiKey,
                            'UserApiToken' => $user_apiKey,
                            'content-type' => 'application/json',
                        ],
                    ]);
                    if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                        $dataTransfer = json_decode($dataTransfer->getBody()->getContents());
                        $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                        $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                        $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                        $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                        $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                        $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                        $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                        $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                        $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                        $caxtonTransfer->save();
                    } else {
                        continue;
                    }
                }

                $endPoint = 'transfers/' . $caxtonTransfer->transfer_id;

                $client = new \GuzzleHttp\Client();
                $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                    $dataTransfers = json_decode($data->getBody()->getContents());
                    $caxtonTransfer->buy_ccy_Code  =  $dataTransfers->Content->Model->MoneyTransfer->BuyCcyCode;
                    $caxtonTransfer->buy_amount  =  $dataTransfers->Content->Model->MoneyTransfer->BuyAmount;
                    $caxtonTransfer->pay_away  =  $dataTransfers->Content->Model->MoneyTransfer->PayAwayDate;
                    $caxtonTransfer->fee_currency  =  $dataTransfers->Content->Model->MoneyTransfer->FeeCurrency;
                    $caxtonTransfer->fee  =  $dataTransfers->Content->Model->MoneyTransfer->Fee;
                    $caxtonTransfer->account_transaction_id  =  $dataTransfers->Content->Model->MoneyTransfer->AccountTransactionId;
                    $caxtonTransfer->complete  =  $dataTransfers->Content->Model->MoneyTransfer->Complete;
                    $caxtonTransfer->automatic  =  $dataTransfers->Content->Model->MoneyTransfer->Automatic;
                    $caxtonTransfer->executed  =  $dataTransfers->Content->Model->MoneyTransfer->ExecutedDate;
                    $caxtonTransfer->pay_status  =  $dataTransfers->Content->Model->MoneyTransfer->PayStatus;
                    $caxtonTransfer->cash_available  =  $dataTransfers->Content->Model->MoneyTransfer->Cashavailable;
                    $caxtonTransfer->authorisation  =  $dataTransfers->Content->Model->MoneyTransfer->Authorisation;
                    $caxtonTransfer->authorisation_status  =  $dataTransfers->Content->Model->MoneyTransfer->AuthorisationStatus;
                    $caxtonTransfer->payment_accepted  =  $dataTransfers->Content->Model->MoneyTransfer->PaymentAccepted;
                    $caxtonTransfer->rejection_reason  =  $dataTransfers->Content->Model->MoneyTransfer->RejectionReason;
                    $caxtonTransfer->payment_reference  =  $dataTransfers->Content->Model->MoneyTransfer->PaymentReference;
                    $caxtonTransfer->is_pending  =  $dataTransfers->Content->Model->MoneyTransfer->IsPending;
                    $caxtonTransfer->created  =  $dataTransfers->Content->Model->MoneyTransfer->Id;
                    $caxtonTransfer->payment_provider  =  $dataTransfers->Content->Model->MoneyTransfer->PaymentProvider;
                    $caxtonTransfer->compliance_status  =  $dataTransfers->Content->Model->MoneyTransfer->ComplianceStatus;
                    $caxtonTransfer->eye_check_status  =  $dataTransfers->Content->Model->MoneyTransfer->EyeCheckStatus;
                    $caxtonTransfer->compliance_status_desc  =  $dataTransfers->Content->Model->MoneyTransfer->ComplianceStatusDesc;
                    $caxtonTransfer->is_high_risk_transfer  =  $dataTransfers->Content->Model->MoneyTransfer->IsHighRiskTransfer;
                    $caxtonTransfer->external_source_id  =  $dataTransfers->Content->Model->MoneyTransfer->ExternalSourceId;
                    $caxtonTransfer->beneficiary_name  =  $dataTransfers->Content->Model->MoneyTransfer->BeneficiaryName;
                    $caxtonTransfer->save();
                } else {
                    continue;
                }
            } catch (\Exception $e) {
                continue;
            }
        }

        return $caxtonTransfers;
    }

    /**
     * Pending Wallet Transaction Status Get object.
     */
    public function fetch_wallet_transaction_statuses(Request $req)
    {
        $caxtonPayoutRequests = CaxtonPayoutRequest::whereNotNull('shop_order_id')->where('status', 'Pending')->orderByRaw('id ASC')->get();
        if ($caxtonPayoutRequests->isEmpty()) Utils::error('Requestses not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        foreach ($caxtonPayoutRequests as $caxtonPayoutRequest) {
            try {
                $client = new \GuzzleHttp\Client();

                $statusEndPoint = 'payments/currency-bank/status?shopOrderId=' . $caxtonPayoutRequest->shop_order_id;
                $statusDetails = $client->request('GET', env('CAXTON_BASEURL') . $statusEndPoint, [
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($statusDetails->getStatusCode() >= 200 && $statusDetails->getStatusCode() <= 210) {
                    $statussDetails = json_decode($statusDetails->getBody()->getContents());
                    $status = $statussDetails->Content->Model->PaymentStatus;

                    $business = Business::find($caxtonPayoutRequest->business_id);
                    if (empty($business)) Utils::error('business not found!', 404);

                    $caxtonWallet = CaxtonWallet::find($caxtonPayoutRequest->wallet_id);
                    if (empty($caxtonWallet)) Utils::error('wallet not found!', 404);

                    if ($status == 'Success') {
                        // Update Payout Request
                        $bankPayoutRequest = CaxtonPayoutRequest::find($caxtonPayoutRequest->id);
                        $bankPayoutRequest->status = 'Accepted';
                        $bankPayoutRequest->save();

                        // Create pouch-history
                        $pouch_history = new PouchHistory();
                        $pouch_history->business_id = $business->id;
                        $pouch_history->debit_amount = $caxtonPayoutRequest->amount;
                        $pouch_history->transaction_category = 'debit';
                        $pouch_history->description = 'Wallet Debited';
                        $pouch_history->status_description = 'wallet-transfer';
                        $pouch_history->remark = $caxtonPayoutRequest->note;
                        $pouch_history->wallet_id = $caxtonWallet->wallet_id;
                        $pouch_history->status = 'Accepted';
                        $pouch_history->updater_id = Auth::id();
                        $pouch_history->save();

                        // Total Balance Update
                        $pouch_history->total_balance = $business->pouch_weight;
                        $pouch_history->save();
                    }

                    if ($status == 'Failed') {
                        // Update Payout Request
                        $bankPayoutRequest = CaxtonPayoutRequest::find($caxtonPayoutRequest->id);
                        $bankPayoutRequest->status = 'Rejected';
                        $bankPayoutRequest->save();

                        // Create pouch-history
                        $pouch_history = new PouchHistory();
                        $pouch_history->business_id = $business->id;
                        $pouch_history->credit_amount = $caxtonPayoutRequest->amount;
                        $pouch_history->transaction_category = 'credit';
                        $pouch_history->description = 'Wallet Credited';
                        $pouch_history->status_description = 'wallet-transfer';
                        $pouch_history->remark = 'redemption';
                        $pouch_history->wallet_id = $caxtonWallet->wallet_id;
                        $pouch_history->status = 'Rejected';
                        $pouch_history->save();

                        // Business Wallet Update
                        $business->pouch_weight = $business->pouch_weight + $caxtonPayoutRequest->amount;
                        $business->save();

                        // Total Balance Update
                        $pouch_history->total_balance = $business->pouch_weight;
                        $pouch_history->save();
                    }
                } else {
                    continue;
                }
            } catch (\Exception $e) {
                continue;
            }
        }

        return $caxtonPayoutRequests;
    }

    /**
     * Show transfer object.
     * POST /api/finance/id/transfer
     */
    public function show_transfer(Request $req, $id)
    {
        $caxtonTransfer = CaxtonTransfer::find($id);
        if (empty($caxtonTransfer)) Utils::error('transfer not found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            if ($caxtonTransfer->transfer_id == 0) {
                $clientTransfer = new \GuzzleHttp\Client();
                $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                    '',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                    $dataTransfer = json_decode($dataTransfer->getBody()->getContents());
                    $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                    $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                    $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                    $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                    $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                    $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                    $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                    $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                    $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                    $caxtonTransfer->save();

                    return $caxtonTransfer;
                } else {
                    Utils::error('something want to wrong', 404);
                }
            }

            $endPoint = 'transfers/' . $caxtonTransfer->transfer_id;

            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $dataTransfer = json_decode($data->getBody()->getContents());
                $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->MoneyTransfer->BuyCcyCode;
                $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->MoneyTransfer->BuyAmount;
                $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->MoneyTransfer->PayAwayDate;
                $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->MoneyTransfer->FeeCurrency;
                $caxtonTransfer->fee  =  $dataTransfer->Content->Model->MoneyTransfer->Fee;
                $caxtonTransfer->account_transaction_id  =  $dataTransfer->Content->Model->MoneyTransfer->AccountTransactionId;
                $caxtonTransfer->complete  =  $dataTransfer->Content->Model->MoneyTransfer->Complete;
                $caxtonTransfer->automatic  =  $dataTransfer->Content->Model->MoneyTransfer->Automatic;
                $caxtonTransfer->executed  =  $dataTransfer->Content->Model->MoneyTransfer->ExecutedDate;
                $caxtonTransfer->pay_status  =  $dataTransfer->Content->Model->MoneyTransfer->PayStatus;
                $caxtonTransfer->cash_available  =  $dataTransfer->Content->Model->MoneyTransfer->Cashavailable;
                $caxtonTransfer->authorisation  =  $dataTransfer->Content->Model->MoneyTransfer->Authorisation;
                $caxtonTransfer->authorisation_status  =  $dataTransfer->Content->Model->MoneyTransfer->AuthorisationStatus;
                $caxtonTransfer->payment_accepted  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentAccepted;
                $caxtonTransfer->rejection_reason  =  $dataTransfer->Content->Model->MoneyTransfer->RejectionReason;
                $caxtonTransfer->payment_reference  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentReference;
                $caxtonTransfer->is_pending  =  $dataTransfer->Content->Model->MoneyTransfer->IsPending;
                $caxtonTransfer->created  =  $dataTransfer->Content->Model->MoneyTransfer->Id;
                $caxtonTransfer->payment_provider  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentProvider;
                $caxtonTransfer->compliance_status  =  $dataTransfer->Content->Model->MoneyTransfer->ComplianceStatus;
                $caxtonTransfer->eye_check_status  =  $dataTransfer->Content->Model->MoneyTransfer->EyeCheckStatus;
                $caxtonTransfer->compliance_status_desc  =  $dataTransfer->Content->Model->MoneyTransfer->ComplianceStatusDesc;
                $caxtonTransfer->is_high_risk_transfer  =  $dataTransfer->Content->Model->MoneyTransfer->IsHighRiskTransfer;
                $caxtonTransfer->external_source_id  =  $dataTransfer->Content->Model->MoneyTransfer->ExternalSourceId;
                $caxtonTransfer->beneficiary_name  =  $dataTransfer->Content->Model->MoneyTransfer->BeneficiaryName;

                $caxtonTransfer->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $caxtonTransfer;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Get transfer object.
     * POST /api/finance/transfer
     */
    public function transfer(Request $req)
    {

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $this->validate($req, [
            'notes' => 'string',
            'amount' => 'required|numeric|gt:0',
        ]);

        try {
            $endPoint = 'transfers/pending';

            $beneficiary_id = $req->input('beneficiary_id');
            $amount = $req->input('amount');
            $notes = $req->input('notes');
            $PayAwayReference = str_replace(' ', '', $notes); //substr(str_shuffle('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890'), 0, 4);

            if ($req->has('payment_date')) {
                $date = date('Y-m-d', strtotime($req->input('payment_date')));
            } else {
                $date = date('Y-m-d');
            }

            if ($req->has('transaction_reason')) {
                $transaction_reason = $req->input('transaction_reason');
            } else {
                $transaction_reason = 'Emigration';
            }

            if ($notes != '') {
                $beneficiary = CaxtonBeneficiary::where('beneficiary_id', $beneficiary_id)->first();
                $beneficiary->notes = $notes;
                $beneficiary->save();
            }

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "BeneficiaryId":"' . $beneficiary_id . '",
                    "BuyCurrency": "GBP",
                    "BuyAmount": "' . $amount . '",
                    "PayAwayDate": "' . $date . '",
                    "PayAwayReference": "' . $PayAwayReference . '",
                    "FeeCurrency": "",
                    "Fee": 0,
                    "OpenBankingAuthId": 0,
                    "TransferType": "0",
                    "TransferReason":"' . $transaction_reason . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $caxtonTransfer = new CaxtonTransfer();
                $caxtonTransfer->pending_transfer_id = $data->Content->Model->Value;
                $caxtonTransfer->notes = $notes;
                $caxtonTransfer->save();

                $clientTransfer = new \GuzzleHttp\Client();
                $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                    '',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                    $dataTransfer = json_decode($dataTransfer->getBody()->getContents());

                    $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                    $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                    $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                    $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                    $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                    $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                    $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                    $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                    $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                    $caxtonTransfer->save();

                    return $caxtonTransfer;
                } else {
                    Utils::error('something want to wrong', 404);
                }
            } else {
                Utils::error('something want to wrong', 404);
            }
            // return $bankdetail;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $message = str_replace(
                        rtrim($e->getMessage()),
                        (string) $e->getResponse()->getBody(),
                        (string) $e
                    );
                    Log::debug($message);
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Pay Transfer object.
     * POST /api/finance/transfer-load
     */
    public function transfer_load(Request $req)
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $this->validate($req, [
            'user_id' => 'string',
            'buy_amount' => 'required|numeric|gt:0',
        ]);

        try {
            $endPoint = 'account/balances/third-party/transfer';

            $user_id = $req->input('user_id');
            $buy_amount = $req->input('buy_amount');

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "UserId":"' . $user_id . '",
                    "SendCurrency": "GBP",
                    "SendAmount": ' . (float)$buy_amount . ',
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
            } else {
                Utils::error('something want to wrong', 404);
            }
            // return $bankdetail;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }
        return $req;
    }

    /*
     |-------------------------------------------
     | TRANSFER TRANSACTIONS DETAILS
     |-------------------------------------------
     */

    /**
     * List Transfer Transaction.
     * GET /api/finance/transfer-transaction-fetch
     */
    public function get_transfer_transaction(Request $req)
    {
        $transactions = CaxtonTransferTransaction::orderByRaw('transaction_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(amount LIKE ? OR status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $transactions->whereNull('user_id');
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $transactions = $transactions->where('transaction_date', '>=', $from_date);
                $transactions = $transactions->where('transaction_date', '<=', $to_date);
            }
        }
        if ($status = $req->query('status', false)) {
            if ($status != 'null') {
                $transactions = $transactions->where('status', 'like', '%' . $status . '%');
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $transactions = $transactions->get();
        } else {
            $transactions = $transactions->paginate($req->query('per_page', 20));
        }

        if ($transactions->isEmpty()) Utils::error('No transactions found!', 404);

        return ($transactions);
    }

    /**
     * Count Transfer Transaction.
     * GET /api/finance/transfer-transaction-fetch-count
     */
    public function get_transfer_transaction_count(Request $req)
    {
        $transactions = CaxtonTransferTransaction::orderByRaw('transaction_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(amount LIKE ? OR status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $transactions->whereNull('user_id');
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $transactions = $transactions->where('transaction_date', '>=', $from_date);
                $transactions = $transactions->where('transaction_date', '<=', $to_date);
            }
        }
        $transactions = $transactions->get();

        $data = [
            'no_of_transactions' => (int)$transactions,
        ];

        return $data;
    }

    /**
     * Fetch transfer transactions list.
     * POST /api/finance/account-transfer-transaction
     */
    public function fetch_all_transfer_transactions(Request $req)
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'account/transactions/search';
            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "StartDate": "' . Carbon::today()->subDays(10)->format('Y-m-d 00:00:00') . '",
                    "EndDate": "' . Carbon::today()->format('Y-m-d 23:59:59') . '",
                    "Currency": "GBP",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $transactions = $data->Content->Model->AccountTransaction;
                foreach ($transactions as $ntransaction) {
                    $transaction = CaxtonTransferTransaction::where('account_transation_id', $ntransaction->Id)->first();
                    if (empty($transaction)) {
                        $transaction = new CaxtonTransferTransaction;
                    }
                    $transaction->account_transation_id  =  $ntransaction->Id;
                    $transaction->account_summary_id  =  $ntransaction->AccountSummaryId;
                    $transaction->amount  =  $ntransaction->Amount;
                    $transaction->status  =  $ntransaction->Status;
                    $transaction->total_balance  =  $ntransaction->TotalBalance;
                    $transaction->currency  =  $ntransaction->Currency;
                    $transaction->base_currency  =  $ntransaction->BaseCurrency;
                    $transaction->transaction_date  =  $ntransaction->TransactionDateTime;

                    $transaction->save();
                }
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $transactions;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }


    /*
     |-------------------------------------------
     | TRANSACTIONS DETAILS
     |-------------------------------------------
     */

    /**
     * List Transaction.
     * GET /api/finance/transaction-fetch
     */
    public function get_transaction(Request $req)
    {
        $transactions = CaxtonTransaction::orderByRaw('executed_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(buy_amount LIKE ? OR pay_status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $caxton_user_id = SiteSettingController::findByKey('caxton_user_id')->value;
        $transactions->where('user_id', $caxton_user_id);
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $transactions = $transactions->where('executed_date', '>=', $from_date);
                $transactions = $transactions->where('executed_date', '<=', $to_date);
            }
        }
        if ($status = $req->query('status', false)) {
            if ($status != 'null') {
                $transactions = $transactions->where('pay_status', 'like', '%' . $status . '%');
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $transactions = $transactions->get();
        } else {
            $transactions = $transactions->paginate($req->query('per_page', 20));
        }

        if ($transactions->isEmpty()) Utils::error('No transactions found!', 404);

        return ($transactions);
    }

    /**
     * Count Transaction.
     * GET /api/finance/transaction-fetch-count
     */
    public function get_transaction_count(Request $req)
    {
        $transactions = CaxtonTransaction::orderByRaw('executed_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(amount LIKE ? OR status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $caxton_user_id = SiteSettingController::findByKey('caxton_user_id')->value;
        $transactions->where('user_id', $caxton_user_id);

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $transactions = $transactions->where('executed_date', '>=', $from_date);
                $transactions = $transactions->where('executed_date', '<=', $to_date);
            }
        }
        $transactions = $transactions->get();

        $data = [
            'no_of_transactions' => (int)$transactions,
        ];

        return $data;
    }

    /**
     * Fetch transactions list.
     * POST /api/finance/account-transaction
     */
    public function fetch_all_transactions(Request $req)
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'transfers';
            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Currency": "GBP",
                    "DateFrom": "' . Carbon::today()->subDays(1000)->format('Y-m-d') . '",
                    "DateTo": "' . Carbon::today()->format('Y-m-d') . '",
                    "pageNumber": 1,
                    "pageSize": 1000,
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $transactions = $data->Content->Model->Records;
                foreach ($transactions as $ntransaction) {
                    $transaction = CaxtonTransaction::where('transactions_id', $ntransaction->Id)->first();
                    if (empty($transaction)) {
                        $transaction = new CaxtonTransaction;
                    }
                    $transaction->transactions_id  =  $ntransaction->Id;
                    $transaction->beneficiary_id  =  $ntransaction->BeneficiaryId;
                    $transaction->user_id  =  $ntransaction->UserId;
                    $transaction->buy_ccy_code  =  $ntransaction->BuyCcyCode;
                    $transaction->buy_amount  =  $ntransaction->BuyAmount;
                    $transaction->executed_date  =  $ntransaction->ExecutedDate;
                    $transaction->pay_away_date  =  $ntransaction->PayAwayDate;
                    $transaction->pay_status  =  $ntransaction->PayStatus;
                    $transaction->account_transation_id  =  $ntransaction->AccountTransactionId;
                    $transaction->payment_reference  =  $ntransaction->PaymentReference;
                    $transaction->created_date  =  $ntransaction->CreatedDate;
                    $transaction->payment_provider  =  $ntransaction->PaymentProvider;
                    $transaction->compliance_status_desc  =  $ntransaction->ComplianceStatusDesc;
                    $transaction->beneficiary_name  =  $ntransaction->BeneficiaryName;
                    $transaction->save();
                }
            }

            $this->fetch_all_deposits($req);
            return $transactions;
        } catch (\Exception $e) {
            $this->fetch_all_deposits($req);
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            }
        }

        return $req;
    }


    /**
     * Fetch deposits list.
     * POST /api/finance/account-deposit
     */
    public function fetch_all_deposits(Request $req)
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $client = new \GuzzleHttp\Client();
            $endPointDeposit = 'account/transactions/deposits';
            $dataDeposit = $client->request('POST', env('CAXTON_BASEURL') . $endPointDeposit, [
                'body' => '{
                    "Currency": "GBP",
                    "StartDate": "' . Carbon::today()->subDays(1000)->format('Y-m-d') . '",
                    "EndDate": "' . Carbon::today()->format('Y-m-d') . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($dataDeposit->getStatusCode() >= 200 && $dataDeposit->getStatusCode() <= 210) {
                $dataDeposit = json_decode($dataDeposit->getBody()->getContents());
                $transactions = $dataDeposit->Content->Model->AccountTransaction;
                foreach ($transactions as $ntransaction) {
                    $transaction = CaxtonTransaction::where('account_transation_id', $ntransaction->AccountTransactionId)->first();
                    if (empty($transaction)) {
                        $transaction = new CaxtonTransaction;
                    }
                    $transaction->user_id  =  $ntransaction->UserId;
                    $transaction->buy_ccy_code  =  $ntransaction->Currency;
                    $transaction->buy_amount  =  $ntransaction->Amount;
                    $transaction->executed_date  =  $ntransaction->DepositDate;
                    $transaction->pay_status  =  $ntransaction->Status;
                    $transaction->account_transation_id  =  $ntransaction->AccountTransactionId;
                    $transaction->payment_reference  =  $ntransaction->Reference;
                    $transaction->created_date  =  $ntransaction->DepositDate;
                    $transaction->save();
                }
            }

            return $transactions;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            }
        }

        return $req;
    }

    /*
     |-------------------------------------------
     | Wallet Bulk Payout
     |-------------------------------------------
     */
    /**
     * Fetch wallet payout.
     * POST /api/finance/wallet-payout
     */
    public function wallet_payout_fetch(Request $req)
    {
        $businesses = Business::selectRaw('caxton_wallets.*,businesses.name as business_name,caxton_wallet_groups.name as group_name')->leftJoin('caxton_wallets', 'caxton_wallets.business_id', '=', 'businesses.id')->leftJoin('caxton_wallet_groups', 'caxton_wallet_groups.id', '=', 'group_id')->where('businesses.disabled', '0')->where('businesses.pouch_service', '0')->orderByRaw('businesses.name ASC');
        $businesses->whereNotNull('caxton_wallets.id');
        if ($query = $req->query('query', false)) {
            $businesses->whereRaw('(businesses.name LIKE ?)', ["%$query%"]);
        }
        if ($group_id = $req->query('group_id', false)) {
            $businesses->where('group_id', $group_id);
        }
        if ($req->query('nopaginate', false) == '1') {
            $businesses = $businesses->get();
        } else {
            $businesses = $businesses->paginate($req->query('per_page', 20));
        }
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);

        //Business Fields Hidden
        $businesses->makeHidden('created_at');
        $businesses->makeHidden('user_counts');
        $businesses->makeHidden('business_commissions');
        $businesses->makeHidden('country');
        $businesses->makeHidden('balance');
        $businesses->makeHidden('finance_user_id');
        $businesses->makeHidden('finance_user_phone_number');
        $businesses->makeHidden('image_url');
        $businesses->makeHidden('thumb_url');

        foreach ($businesses as $business) {
            $business->payout_description = 'Payout for ' . $business->business_name;
            $business->payout_statement_descriptor = 'Payout for ' . $business->business_name;
            $business->select_checkbox = true;
            $business->use_limit = CaxtonWalletTransaction::where('wallet_id', $business->wallet_id)->sum('credit_amount');

            $carbon = Carbon::today()->subDays(7);
            $currentDayOfWeek = $carbon->dayOfWeek;
            if ($currentDayOfWeek <= 1) {
                $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
            } else {
                $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
            }
            $payoutInvoice = PayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'Archived')->where('business_id', $business->business_id)->first();
            if (!empty($payoutInvoice)) {
                if ($payoutInvoice->net_total > 0) {
                    $business->amount = $payoutInvoice->net_total;
                } else {
                    $business->amount = 0;
                }
            } else {
                $business->amount = 0;
            }
        }
        return $businesses;
    }

    /**
     * Create bulk wallet payout.
     * POST /api/finance/wallet-bulk-payout
     */
    public function wallet_bulk_payout(Request $req)
    {
        if ($req->has('payouts') && !is_array($req->input('payouts')))
            $req->merge(['payouts' => json_decode($req->input('payouts'))]);

        $this->validate($req, [
            'payouts' => 'array',
        ]);

        if ($req->has('payouts') && !empty($req->input('payouts')) || sizeof($req->input('payouts')) > 0) {
            // create payoutRequest
            $payouts = ($req->input('payouts'));
            $time = Carbon::now();
            foreach ($payouts as $payout) {
                if ($payout->amount > 0) {
                    try {
                        $time->addMinute();

                        $caxtonWallet = CaxtonWallet::where('archive', '0')->where('wallet_id', $payout->wallet_id)->first();

                        $bankPayoutRequest = new CaxtonPayoutRequest();
                        $bankPayoutRequest->business_id = $payout->business_id;
                        $bankPayoutRequest->wallet_id = $caxtonWallet->id;
                        $bankPayoutRequest->note = 'Payout for ' . $payout->name;
                        $bankPayoutRequest->amount = $payout->amount;
                        $bankPayoutRequest->status = 'Pending';
                        $bankPayoutRequest->available_at = $time;
                        $bankPayoutRequest->save();
                        $payout->bank_request_id = $bankPayoutRequest->id;
                        Queue::later($time, new WalletTransactionCreatorJob($payout, true));
                    } catch (\Exception $e) {
                        continue;
                    }
                }
            }
        }

        return $payouts;
    }

    /*
     |-------------------------------------------
     | Beneficiary Bulk Payout
     |-------------------------------------------
     */
    /**
     * Fetch beneficiary payout.
     * POST /api/finance/beneficiary-payout
     */
    public function beneficiary_payout_fetch(Request $req)
    {
        $businesses = Business::selectRaw('id,name')->where('disabled', '0')->orderByRaw('name ASC');
        if ($query = $req->query('query', false)) {
            $businesses->whereRaw('(name LIKE ?)', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $businesses = $businesses->get();
        } else {
            $businesses = $businesses->paginate($req->query('per_page', 20));
        }
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);

        //Business Fields Hidden
        $businesses->makeHidden('created_at');
        $businesses->makeHidden('user_counts');
        $businesses->makeHidden('business_commissions');
        $businesses->makeHidden('country');
        $businesses->makeHidden('balance');
        $businesses->makeHidden('finance_user_id');
        $businesses->makeHidden('finance_user_phone_number');
        $businesses->makeHidden('image_url');
        $businesses->makeHidden('thumb_url');

        foreach ($businesses as $business) {
            $business->payout_description = 'Payout for ' . $business->name;
            $business->payout_statement_descriptor = 'Payout for ' . $business->name;
            $business->select_checkbox = true;

            $carbon = Carbon::today()->subDays(7);
            $currentDayOfWeek = $carbon->dayOfWeek;
            if ($currentDayOfWeek <= 1) {
                $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
            } else {
                $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
            }

            $payoutInvoice = PayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'Normal')->where('business_id', $business->id)->first();
            if (!empty($payoutInvoice)) {
                if ($payoutInvoice->net_total > 0) {
                    $business->amount = $payoutInvoice->net_total;
                } else {
                    $business->amount = 0;
                }
            } else {
                $business->amount = 0;
            }

            $business->beneficiary_details = CaxtonBeneficiary::where('business_id', $business->id)->whereNull('user_id')->where('archive', 0)->get();
        }

        return $businesses;
    }

    /**
     * Create bulk beneficiary payout.
     * POST /api/finance/beneficiary-bulk-payout
     */
    public function beneficiary_bulk_payout(Request $req)
    {
        if ($req->has('payouts') && !is_array($req->input('payouts')))
            $req->merge(['payouts' => json_decode($req->input('payouts'))]);

        $this->validate($req, [
            'payouts' => 'array',
        ]);

        if ($req->has('payouts') && !empty($req->input('payouts')) || sizeof($req->input('payouts')) > 0) {
            // create payoutRequest
            $payouts = ($req->input('payouts'));
            foreach ($payouts as $payout) {
                if ($payout->beneficiary_id && $payout->amount > 0) {
                    try {
                        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
                        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

                        $endPoint = 'transfers/pending';

                        if ($payout->notes) {
                            $PayAwayReference = str_replace(' ', '', $payout->notes);
                        } else {
                            $PayAwayReference = str_replace(' ', '', $payout->name);
                        }
                        $PayAwayReference = substr($PayAwayReference, 0, 18);
                        $PayAwayReference = preg_replace('/[^a-zA-Z0-9\s]/', '', $PayAwayReference);

                        $client = new \GuzzleHttp\Client();
                        $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                            'body' => '{
                                    "BeneficiaryId":"' . $payout->beneficiary_id . '",
                                    "BuyCurrency": "GBP",
                                    "BuyAmount": "' . $payout->amount . '",
                                    "PayAwayDate": "' . date('Y-m-d') . '",
                                    "PayAwayReference": "' . $PayAwayReference . '",
                                    "FeeCurrency": "",
                                    "Fee": 0,
                                    "OpenBankingAuthId": 0,
                                    "TransferType": "0",
                                    "TransferReason": "Emigration",
                                }',
                            'headers' => [
                                'Authorization' => 'bearer ' . $apiKey,
                                'UserApiToken' => $user_apiKey,
                                'content-type' => 'application/json',
                            ],
                        ]);
                        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                            $data = json_decode($data->getBody()->getContents());
                            $caxtonTransfer = new CaxtonTransfer();
                            $caxtonTransfer->pending_transfer_id = $data->Content->Model->Value;
                            $caxtonTransfer->notes = $payout->notes;
                            $caxtonTransfer->save();

                            $clientTransfer = new \GuzzleHttp\Client();
                            $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                            $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                                '',
                                'headers' => [
                                    'Authorization' => 'bearer ' . $apiKey,
                                    'UserApiToken' => $user_apiKey,
                                    'content-type' => 'application/json',
                                ],
                            ]);
                            if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                                $dataTransfer = json_decode($dataTransfer->getBody()->getContents());
                                $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                                $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                                $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                                $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                                $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                                $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                                $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                                $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                                $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                                $caxtonTransfer->save();

                                $bankPayoutRequest = new CaxtonPayoutRequest;
                                $bankPayoutRequest->business_id = $payout->id;
                                $bankPayoutRequest->beneficiary_id = $payout->beneficiary_id;
                                $bankPayoutRequest->note = $payout->notes;
                                $bankPayoutRequest->amount = $payout->amount;
                                $bankPayoutRequest->status = 'Accepted';
                                $bankPayoutRequest->save();

                                $passbook = new Passbook;
                                $passbook->entity = "payout_request";
                                $passbook->entity_id = $bankPayoutRequest->id;
                                $passbook->business_id = $bankPayoutRequest->business_id;
                                $passbook->date = Carbon::today()->toDateString();
                                $passbook->from_date = $bankPayoutRequest->created_at;
                                $passbook->to_date = $bankPayoutRequest->created_at;
                                $passbook->amount = -1 * $bankPayoutRequest->amount;
                                $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($bankPayoutRequest->created_at)->toDateString();
                                $passbook->save();
                            }
                        }
                    } catch (\Exception $e) {
                        continue;
                    }
                }
            }
        }

        return $payouts;
    }

    public function export_transfer_transaction(Request $req)
    {
        $reports = '';
        $req->merge(['nopaginate' => 1]);

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $fileName =  $from_date->format('d-m-Y') . " to " . $to_date->format('d-m-Y');
            } else {
                $fileName = "Transfer Transactions";
            }
        } else {
            $fileName = "Transfer Transactions";
        }

        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $reports = ($this->get_transfer_transaction($req));
            return Excel::download(new TransactionsExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            $reports = ($this->get_transfer_transaction($req));
            return $this->printStatement($req, $reports);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function printStatement($req, $transactions)
    {
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->format('d-M-Y');
                $to_date = Carbon::parse($req->input('to_date'))->format('d-M-Y');
            }
        }

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            /*border-left: 1px solid #000000;
                border-top: 1px solid #000000;*/
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #c5c7c9;
            font-weight:600;
            background-color:#ffffff;
            color:#000;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #c5c7c9;
        }
        .items-table tr:nth-child(even){
            background-color:#f0faff   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $bankDetails = CaxtonBankDetails::whereNull('user_id')->first();

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h2 style = "text-align:center">Account Statement</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td><h3>Transactions</h3></td>';
        $html = $html . '<td>Account Name : ' . $bankDetails->account_name . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>UBSIDI Limited</td>';
        $html = $html . '<td>Account Number : ' . $bankDetails->account_number . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td>Sort Code : ' . $bankDetails->sortcode . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall,WS1 3RA</td>';
        $html = $html . '<td>' . Carbon::today()->format('jS F, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $html = $html . '<tr>';
                $html = $html . '<td>Showing : ' . $from_date . ' to ' . $to_date . '</td>';
                $html = $html . '<td></td>';
                $html = $html . '</tr>';
            }
        }
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Date</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Paid in</th>';
        $html = $html . '<th>Paid out</th>';
        $html = $html . '<th>Balance</th>';
        $html = $html . '</tr>';
        foreach ($transactions as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . Carbon::parse($item->transaction_date)->format('d M Y') . '</td>';
            $html = $html . '<td style="text-align:left;">' . $item->status . '</td>';
            if ($item->amount >= 0) {
                $html = $html . '<td>£' . $item->amount . '</td>';
            } else {
                $html = $html . '<td> - </td>';
            }
            if ($item->amount < 0) {
                $html = $html . '<td>£' . abs($item->amount) . '</td>';
            } else {
                $html = $html . '<td> - </td>';
            }
            $html = $html . '<td>£' . $item->total_balance . '</td>';

            $html = $html . '</tr>';
        }

        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';

        $mpdf = new Mpdf();
        $mpdf->setFooter('{PAGENO}');
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($html);
        $fileName = "Statement";
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $fileName = 'Statement - ' . $from_date . ' to ' . $to_date;
            }
        }
        return $mpdf->Output("$fileName.pdf", "D");
    }


    public function export_transaction(Request $req)
    {
        $reports = '';
        $req->merge(['nopaginate' => 1]);

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $fileName =  $from_date->format('d-m-Y') . " to " . $to_date->format('d-m-Y');
            } else {
                $fileName = "Transfer Transactions";
            }
        } else {
            $fileName = "Transfer Transactions";
        }

        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $reports = ($this->get_transaction($req));
            return Excel::download(new TransferTransactionsExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            $reports = ($this->get_transaction($req));
            return $this->printTransaction($req, $reports);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function printTransaction($req, $transactions)
    {
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->format('d-M-Y');
                $to_date = Carbon::parse($req->input('to_date'))->format('d-M-Y');
            }
        }

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            /*border-left: 1px solid #000000;
                border-top: 1px solid #000000;*/
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #c5c7c9;
            font-weight:600;
            background-color:#ffffff;
            color:#000;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #c5c7c9;
        }
        .items-table tr:nth-child(even){
            background-color:#f0faff   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $bankDetails = CaxtonBankDetails::whereNull('user_id')->first();

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h2 style = "text-align:center">Account Statement</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td><h3>Transactions</h3></td>';
        $html = $html . '<td>Account Name : ' . $bankDetails->account_name . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>UBSIDI Limited</td>';
        $html = $html . '<td>Account Number : ' . $bankDetails->account_number . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td>Sort Code : ' . $bankDetails->sortcode . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall,WS1 3RA</td>';
        $html = $html . '<td>' . Carbon::today()->format('jS F, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $html = $html . '<tr>';
                $html = $html . '<td>Showing : ' . $from_date . ' to ' . $to_date . '</td>';
                $html = $html . '<td></td>';
                $html = $html . '</tr>';
            }
        }
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Date</th>';
        $html = $html . '<th>Executed Date</th>';
        $html = $html . '<th>Status</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Amount</th>';
        $html = $html . '<th>Beneficiary Name</th>';
        $html = $html . '</tr>';
        foreach ($transactions as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . Carbon::parse($item->created_date)->format('d M Y') . '</td>';
            $html = $html . '<td>' . Carbon::parse($item->executed_date)->format('d M Y') . '</td>';
            $html = $html . '<td style="text-align:left;">' . $item->pay_status . '</td>';
            $html = $html . '<td>' . $item->payment_reference . '</td>';
            $html = $html . '<td>£' . $item->buy_amount . '</td>';
            if (!empty($item->beneficiary)) {
                $html = $html . '<td>' . $item->beneficiary->name . '</td>';
            } else {
                $html = $html . '<td></td>';
            }
            $html = $html . '</tr>';
        }

        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';

        $mpdf = new Mpdf();
        $mpdf->setFooter('{PAGENO}');
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($html);
        $fileName = "Statement";
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $fileName = 'Statement - ' . $from_date . ' to ' . $to_date;
            }
        }
        return $mpdf->Output("$fileName.pdf", "D");
    }

    /*
     |-------------------------------------------
     | Payout Request Datails
     |-------------------------------------------
     */

    // GET /api/finance/payout-requests
    public function payout_request_index(Request $req)
    {
        $payout_requests = CaxtonPayoutRequest::select("caxton_payout_requests.*", "businesses.name as business_name", "users.first_name", "users.last_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->leftJoin('users', 'users.id', '=', 'user_id')->orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $payout_requests->whereRaw('(amount LIKE ?)', ["%$query%"]);
        }

        if ($query = $req->query('status', false)) {
            $payout_requests->where('caxton_payout_requests.status', $query);
        }

        if ($query = $req->query('business_id', false)) {
            $payout_requests->where('caxton_payout_requests.business_id', $query);
        }

        if ($query = $req->query('wallet_transfer', false)) {
            $payout_requests->whereNotNull('caxton_payout_requests.shop_order_id')->where('caxton_payout_requests.status', 'Pending');
        }

        if ($query = $req->query('from_date', false)) {
            $this->validate($req, [
                'from_date' => 'date_format:Y-m-d',
                'to_date' => 'required_with:from_date|date_format:Y-m-d',
            ]);

            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();

            $payout_requests = $payout_requests->where('caxton_payout_requests.created_at', '>=', $from_date->getTimestamp());
            $payout_requests = $payout_requests->where('caxton_payout_requests.created_at', '<=', $to_date->getTimestamp());
        }

        if ($query = $req->query('user_id', false)) {
            $payout_requests->where('caxton_payout_requests.user_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $payout_requests = $payout_requests->get();
        } else {
            $payout_requests = $payout_requests->paginate($req->query('per_page', 20));
        }

        if ($payout_requests->isEmpty()) Utils::error('No payout requests found!', 404);

        return ($payout_requests);
    }

    /**
     * Export finance/payout-requests.
     * POST /api/finance/payout-requests/export
     */
    public function export(Request $req)
    {
        if ($req->input('from_date') == '' || $req->input('from_date') == 'null') {
            $req->request->remove('from_date');
            $req->request->remove('to_date');
        }
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $fileName = "PayoutRequest";
            $reports = ($this->payout_request_index($req));
            return;
            // return Excel::download(new CaxtonPayoutRequestReportExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }


    /**
     * Show wallet group object.
     * POST /api/finance/payout-requests/id
     */
    public function payout_request_show(Request $req, $id)
    {
        $payout_request = CaxtonPayoutRequest::find($id);
        if (empty($payout_request)) Utils::error('payout request not found!', 404);
        return $payout_request;
    }

    // POST /api/finance/payout-requests/{id}
    public function payout_request_update(Request $req, $id)
    {
        $payout_request = CaxtonPayoutRequest::find($id);
        if (empty($payout_request)) Utils::error('payout request not found!', 404);

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'rejection_note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        if ($req->has('amount')) $payout_request->amount = $req->input('amount');
        if ($req->has('note')) $payout_request->note = $req->input('note');
        if ($req->has('status')) $payout_request->status = $req->input('status');
        if ($req->has('business_id')) $payout_request->business_id = $req->input('business_id');
        if ($req->has('user_id')) $payout_request->user_id = $req->input('user_id');
        if ($req->has('rejection_note')) $payout_request->rejection_note = $req->input('rejection_note');

        $passbook = Passbook::where('entity', 'payout_request')->where('entity_id', $payout_request->id);
        if ($req->has('user_id')) {
            $passbook->where('user_id', $payout_request->user_id);
        }
        if ($req->has('business_id')) {
            $passbook->where('business_id', $payout_request->business_id);
        }
        $passbook = $passbook->first();
        if ($payout_request->status == 'Accepted') {
            if (empty($passbook)) {
                $passbook = new Passbook;
            }
            $passbook->entity = "caxton_payout_request";
            $passbook->entity_id = $payout_request->id;
            $passbook->business_id = $payout_request->business_id;
            $passbook->user_id = $payout_request->user_id;
            $passbook->date = Carbon::today()->toDateString();
            $passbook->from_date = $payout_request->created_at;
            $passbook->to_date = $payout_request->created_at;
            $passbook->amount = -1 * $payout_request->amount;
            $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($payout_request->created_at)->toDateString();
            $passbook->save();
        } else {
            if (!empty($passbook)) $passbook->delete();
        }
        $payout_request->save();

        return $payout_request;
    }

    /**
     * Pending Wallet Transaction Status Get object.
     */
    public function payout_request_status_fetch(Request $req, $id)
    {
        $caxtonPayoutRequest = CaxtonPayoutRequest::find($id);
        if (empty($caxtonPayoutRequest)) Utils::error('No request found!', 404);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $client = new \GuzzleHttp\Client();

            $statusEndPoint = 'payments/currency-bank/status?shopOrderId=' . $caxtonPayoutRequest->shop_order_id;
            $statusDetails = $client->request('GET', env('CAXTON_BASEURL') . $statusEndPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($statusDetails->getStatusCode() >= 200 && $statusDetails->getStatusCode() <= 210) {
                $statussDetails = json_decode($statusDetails->getBody()->getContents());
                $status = $statussDetails->Content->Model->PaymentStatus;

                $business = Business::find($caxtonPayoutRequest->business_id);
                if (empty($business)) Utils::error('business not found!', 404);

                $caxtonWallet = CaxtonWallet::find($caxtonPayoutRequest->wallet_id);
                if (empty($caxtonWallet)) Utils::error('wallet not found!', 404);

                if ($status == 'Success') {
                    // Update Payout Request
                    $bankPayoutRequest = CaxtonPayoutRequest::find($caxtonPayoutRequest->id);
                    $bankPayoutRequest->status = 'Accepted';
                    $bankPayoutRequest->save();

                    // Create pouch-history
                    $pouch_history = new PouchHistory();
                    $pouch_history->business_id = $business->id;
                    $pouch_history->debit_amount = $caxtonPayoutRequest->amount;
                    $pouch_history->transaction_category = 'debit';
                    $pouch_history->description = 'Wallet Debited';
                    $pouch_history->status_description = 'wallet-transfer';
                    $pouch_history->remark = $caxtonPayoutRequest->note;
                    $pouch_history->wallet_id = $caxtonWallet->wallet_id;
                    $pouch_history->status = 'Accepted';
                    $pouch_history->updater_id = Auth::id();
                    $pouch_history->save();

                    // Total Balance Update
                    $pouch_history->total_balance = $business->pouch_weight;
                    $pouch_history->save();
                }

                if ($status == 'Failed') {
                    // Update Payout Request
                    $bankPayoutRequest = CaxtonPayoutRequest::find($caxtonPayoutRequest->id);
                    $bankPayoutRequest->status = 'Rejected';
                    $bankPayoutRequest->save();

                    // Create pouch-history
                    $pouch_history = new PouchHistory();
                    $pouch_history->business_id = $business->id;
                    $pouch_history->credit_amount = $caxtonPayoutRequest->amount;
                    $pouch_history->transaction_category = 'credit';
                    $pouch_history->description = 'Wallet Credited';
                    $pouch_history->status_description = 'wallet-transfer';
                    $pouch_history->remark = 'redemption';
                    $pouch_history->wallet_id = $caxtonWallet->wallet_id;
                    $pouch_history->status = 'Rejected';
                    $pouch_history->save();

                    // Business Wallet Update
                    $business->pouch_weight = $business->pouch_weight + $caxtonPayoutRequest->amount;
                    $business->save();

                    // Total Balance Update
                    $pouch_history->total_balance = $business->pouch_weight;
                    $pouch_history->save();
                }
            }
        } catch (\Exception $e) {
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            }
        }

        return $caxtonPayoutRequest;
    }


    // DELETE /api/finance/payout-requests/{id}
    public function payout_request_delete(Request $req, $id)
    {
        $payout_request = CaxtonPayoutRequest::find($id);
        if (empty($payout_request)) Utils::error('payout request not found!', 404);
        $payout_request->delete();
        return $payout_request;
    }


    public function setting_generate_token()
    {
        try {

            $endPoint = 'token';
            $client = new \GuzzleHttp\Client();
            $caxton_username = SiteSettingController::findByKey('caxton_username')->value;

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'form_params' => [
                    'Grant_Type' => 'password',
                    'Username' => $caxton_username,
                    'Password' => env('CAXTON_CLIENT_TOKEN')
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                // update Sitesetting
                $caxton_usersettingname = SiteSettingController::findByKey('caxton_token');
                $caxton_usersettingname->value  = $data->access_token;
                $caxton_usersettingname->save();

                //Login and Update Password
                // $this->login();
            }
        } catch (\Exception $e) {
        }
        return 'something want to wrong';
    }

    public function login()
    {
        $caxton_email = SiteSettingController::findByKey('caxton_email')->value;
        $caxton_token = SiteSettingController::findByKey('caxton_token')->value;
        $caxton_public_key = SiteSettingController::findByKey('caxton_public_key')->value;

        try {
            $encrypted_data = '';
            $cipher = "aes-128-cbc";
            //Generate a 256-bit encryption key 
            $encryption_key = $caxton_public_key;
            // Generate an initialization vector 
            $iv_size = openssl_cipher_iv_length($cipher);
            // please generate 16 digit random string here instead of using it
            $iv = substr(str_shuffle('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'), 0, 16);
            //Data to encrypt 
            $data = "ubsidi";
            $encrypted_data = openssl_encrypt($data, $cipher, $encryption_key, 0, $iv);

            $endPoint = 'users/auth/login-step-one';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "UserName":"' . $caxton_email . '",
                    "Password":"' . env('CAXTON_LOGIN_TOKEN') . '",
                    "DeviceId": "' . $encrypted_data . '",
                    "Device": "",
                    "OperatingSystem": "",
                    "AppVersion": ""
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxton_token,
                    'ivKey' => $iv,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                // update caxton password
                $caxton_usersettingname = SiteSettingController::findByKey('caxton_password');
                $caxton_usersettingname->value  = $data->Content->Model->LoginToken;
                $caxton_usersettingname->save();
            }
        } catch (\Exception $e) {
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $message = str_replace(
                    rtrim($e->getMessage()),
                    (string) $e->getResponse()->getBody(),
                    (string) $e
                );
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            }
        }

        return 'something want to wrong!';
    }
}
