<?php

namespace App\Http\Controllers;

use App\CaxtonBankDetails;
use App\CaxtonBeneficiary;
use App\CaxtonCard;
use App\StandingInstruction;
use App\CaxtonTransfer;
use App\CaxtonTransferTransaction;
use App\CaxtonTransaction;
use App\CaxtonUser;
use App\CaxtonWallet;
use App\CaxtonWalletTransaction;
use App\Exports\TransferTransactionsExport;
use App\Helpers\Auth;
use App\Helpers\OtpHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\SiteSetting;
use Carbon\Carbon;
use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request as Psr7Request;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Maatwebsite\Excel\Facades\Excel;
use Mpdf\Mpdf;

use function Psy\debug;

class CaxtonfxUserController extends Controller
{
    protected $client;

    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    /**
     * List User.
     * GET /api/caxton-users
     */
    public function index(Request $req)
    {
        $users = CaxtonUser::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $users->whereRaw('(name LIKE ? OR email LIKE ? OR username LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($bussiness_id =  $req->query('bussiness_id', false)) {
            $users->where('bussiness_id', $bussiness_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $users = $users->get();
        } else {
            $users = $users->paginate($req->query('per_page', 20));
        }

        if ($users->isEmpty()) Utils::error('No users found!', 404);

        return ($users);
    }


    /**
     * Create caxtonUser.
     * POST /api/caxton-users
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => 'required|email',
            'phone_number' => 'required|string',
        ]);

        $endPoint = 'clients/subsidiary';
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;

        try {
            $name = $req->input('name');
            $email = $req->input('email');

            $endPoint = 'clients/subsidiary';
            $client = new \GuzzleHttp\Client();

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "CompanyName":"' . $name . '",
                    "Email":"' . $email . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());

                // Create caxtonUser
                $caxtonUser = new CaxtonUser;
                $caxtonUser->fill($req->all());
                $caxtonUser->username  = $data->Content->Model->Username;
                $caxtonUser->name  = $data->Content->Model->Name;
                $caxtonUser->phone_number  = $req->input('phone_number');
                $caxtonUser->save();
                $this->generate_password($req, $caxtonUser->id);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
            return 'something went to wrong!';
        }
        return $caxtonUser;
    }

    /**
     * Update caxtonUser.
     * POST /api/caxton-users/id
     */
    public function update(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => 'required|email',
            'phone_number' => 'required|string',
        ]);

        // update caxtonUser
        // if ($req->has('name')) $caxtonUser->name = $req->input('name');
        // if ($req->has('email')) $caxtonUser->email = $req->input('email');
        if ($req->has('phone_number')) $caxtonUser->phone_number = $req->input('phone_number');

        $caxtonUser->save();

        return $caxtonUser;
    }

    /**
     * Show caxtonUser.
     * GET /api/finance-users/{id}
     */
    public function show(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);
        return $caxtonUser;
    }

    /**
     * Send Otp caxtonUser.
     * GET /api/finance-users/{id}/send-otp
     */
    public function send_otp(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        if (!$caxtonUser->phone_number) Utils::error('please update your phone number!', 404);

        $otp = rand(100000, 999999);
        $message = 'Your Ubsidi OTP is ' . $otp . '.Note:Please do not share this otp anyone.';

        (new OtpHelper())->sendMessage($message, $caxtonUser->phone_number, 'otp_service');
        $caxtonUser->otp = $otp;
        $caxtonUser->save();

        return ['message' => 'otp send on your mobile number.'];
    }

    /**
     * Delete caxtonUser by id.
     * DELETE /api/finance-users/{id}
     */
    public function delete(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);
        $caxtonUser->delete();
        return $caxtonUser;
    }

    /**
     * Get balance object.
     * POST /api/finance-users/id/balance
     */
    public function get_balance(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $endPoint = 'account/balances';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $balances = $data->Content->Model->Balances;
                $balance_amount = 0;
                $balance_symbol = "£";

                foreach ($balances as $balance) {
                    $balance_amount = $balance->Balance;
                    $balance_symbol = $balance->Symbol;
                    break;
                }

                return [
                    'balance' => $balance_amount,
                    'symbol' => $balance_symbol
                ];
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    // Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                    return [
                        'balance' => 0,
                        'symbol' => "£"
                    ];
                }
            }
        }

        return $req;
    }

    /*
     |-------------------------------------------
     | Beneficary Datails
     |-------------------------------------------
     */

    // GET /api/finance-users/{user_id}/beneficiary
    public function beneficiary_index(Request $req, $user_id)
    {
        $beneficiary = CaxtonBeneficiary::selectRaw('caxton_beneficiaries.*,caxton_users.name as user_name')->leftJoin('caxton_users', 'caxton_users.id', '=', 'user_id')->orderByRaw('caxton_beneficiaries.created_at DESC');
        $beneficiary->where('archive', 0);

        if ($query = $req->query('query', false)) {
            $beneficiary->whereRaw('(caxton_beneficiaries.name LIKE ? OR caxton_beneficiaries.email LIKE ? OR caxton_beneficiaries.phone LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }
        $beneficiary->where('user_id', $user_id);
        if ($req->query('nopaginate', false) == '1') {
            $beneficiary = $beneficiary->get();
        } else {
            $beneficiary = $beneficiary->paginate($req->query('per_page', 20));
        }

        if ($beneficiary->isEmpty()) Utils::error('No beneficiaries found!', 404);

        return ($beneficiary);
    }

    // POST /api/finance-users/{user_id}/beneficiary
    public function beneficiary_create(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $this->validate($req, [
            'country' => 'required|string|max:255',
            'currency' => 'required|string|max:255',
            'iban' => 'string|max:255',
            'swift' => 'string|max:255',
            'sortCode' => 'required|string|max:255',
            'account_number' => 'required|string|max:255',
            'bank_name' => 'string|max:255',
            'reference' => 'string|max:255',
            'address1' => 'required|string|max:255',
            'address2' => 'required|string|max:255',
            'email' => 'email',
            'phone' => 'string|max:15',
            'beneficiary_type' => 'required|in:Entity,Individual',
        ]);

        try {
            $name = str_replace(' ', '', $req->input('name'));
            $bank_name = $req->input('bank_name');
            $address1 = $req->input('address1');
            $address2 = $req->input('address2');
            $email = $req->input('email');
            $phone = $req->input('phone');
            $country = $req->input('country');
            $currency = $req->input('currency');
            $iban = $req->input('iban');
            $swift = $req->input('swift');
            $sortCode = $req->input('sortCode');
            $account_number = $req->input('account_number');
            $notes = $req->input('notes');

            $endPoint = 'beneficiaries/add';
            $client = new \GuzzleHttp\Client();

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Name":"' . $name . '",
                    "Reference": "' . $notes . '",
                    "Currency": "' . $currency . '",
                    "BeneficiaryAddress1": "",
                    "BeneficiaryAddress2": "",
                    "Email":"' . $email . '",
                    "Phone":"' . $phone . '",
                    "ClientBeneficiaryReference": "",
                    "BankName":"' . $bank_name . '",
                    "AccountNumber": "' . $account_number . '",
                    "Iban": "' . $iban . '",
                    "Swift": "' . $swift . '",
                    "SortCode":"' . $sortCode . '",
                    "BankAddress1":"' . $address1 . '",
                    "BankAddress2":"' . $address2 . '",
                    "Country":"' . $country . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $beneficiary = new CaxtonBeneficiary;
                $beneficiary->fill($req->all());
                $beneficiary->beneficiary_id  = $data->Content->Model->Value;
                $beneficiary->user_id  = $user_id;
                $beneficiary->beneficiary_type  = $req->input('beneficiary_type');
                $beneficiary->save();
                return $beneficiary;
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Lookup beneficiary object.
     * POST /api/finance-users/{user_id}/beneficiary/lookup
     */
    public function beneficiary_lookup(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $this->validate($req, [
            'sortCode' => 'required|string|max:255',
            'account_number' => 'required|string|max:255',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $sortCode = $req->input('sortCode');
            $account_number = $req->input('account_number');

            $endPoint = 'beneficiaries/retrieve-bank-details';
            $client = new \GuzzleHttp\Client();

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Country":"GB",
                    "Currency": "GBP",
                    "Iban": "",
                    "Swift": "",
                    "AccountNumber": "' . $account_number . '",
                    "SortCode":"' . $sortCode . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $beneficiary = new CaxtonBeneficiary;
                $beneficiary->bank_name  = $data->Content->Model->BankDetails[0]->BankName;
                $beneficiary->account_number  = $data->Content->Model->BankDetails[0]->AccountNumber;
                $beneficiary->iban  = $data->Content->Model->BankDetails[0]->IbanNumber;
                $beneficiary->swift  = $data->Content->Model->BankDetails[0]->SwiftCode;
                $beneficiary->sortCode  = $data->Content->Model->BankDetails[0]->SortCode;
                $beneficiary->address1  = $data->Content->Model->BankDetails[0]->BankAddress1;
                $beneficiary->address2  = $data->Content->Model->BankDetails[0]->BankAddress2;
                $beneficiary->country  = $data->Content->Model->BankDetails[0]->Country;
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $beneficiary;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    // POST /api/finance-users/{user_id}/beneficiary/{id}
    public function beneficiary_update(Request $req, $user_id, $id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $beneficiary = CaxtonBeneficiary::find($id);
        if (empty($beneficiary)) Utils::error('Beneficiary not found!', 404);

        $this->validate($req, [
            'country' => 'required|string|max:255',
            'currency' => 'required|string|max:255',
            'iban' => 'string|max:255',
            'swift' => 'string|max:255',
            'sortCode' => 'required|string|max:255',
            'account_number' => 'required|string|max:255',
            'bank_name' => 'string|max:255',
            'reference' => 'string|max:255',
            'address1' => 'required|string|max:255',
            'address2' => 'required|string|max:255',
            'email' => 'email',
            'phone' => 'string|max:15',
            'beneficiary_type' => 'required|in:Entity,Individual',
        ]);

        try {
            $name = $req->input('name');
            $bank_name = $req->input('bank_name');
            $address1 = $req->input('address1');
            $address2 = $req->input('address2');
            $email = $req->input('email');
            $phone = $req->input('phone');
            $beneficiary_type = $req->input('beneficiary_type');

            $endPoint = 'beneficiaries/' . $beneficiary->beneficiary_id;
            $client = new \GuzzleHttp\Client();

            $data = $client->request('PUT', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Name":"' . $name . '",
                    "Salutation": "",
                    "BankName":"' . $bank_name . '",
                    "Reference": "",
                    "BankAddress1":"' . $address1 . '",
                    "BankAddress2":"' . $address2 . '",
                    "Email":"' . $email . '",
                    "Phone":"' . $phone . '",
                    "BeneficiaryType":"' . $beneficiary_type . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                if ($req->has('name')) $beneficiary->name  = $name;
                if ($req->has('user_id')) $beneficiary->user_id = $req->input('user_id');
                if ($req->has('beneficiary_id')) $beneficiary->beneficiary_id = $req->input('beneficiary_id');
                if ($req->has('address1')) $beneficiary->address1 = $req->input('address1');
                if ($req->has('address2')) $beneficiary->address2 = $req->input('address2');
                if ($req->has('email')) $beneficiary->email = $req->input('email');
                if ($req->has('phone')) $beneficiary->phone = $req->input('phone');
                if ($req->has('country')) $beneficiary->country = $req->input('country');
                if ($req->has('currency')) $beneficiary->currency = $req->input('currency');
                if ($req->has('iban')) $beneficiary->iban = $req->input('iban');
                if ($req->has('swift')) $beneficiary->swift = $req->input('swift');
                if ($req->has('sortCode')) $beneficiary->sortCode = $req->input('sortCode');
                if ($req->has('account_number')) $beneficiary->account_number = $req->input('account_number');
                if ($req->has('beneficiary_type')) $beneficiary->beneficiary_type = $req->input('beneficiary_type');
                $beneficiary->save();
            } else {
                Utils::error('something want to wrong', 404);
            }

            return $beneficiary;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    // GET /api/finance-users/{user_id}/beneficiary/sync
    public function beneficiary_sync(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $baneficiaries = CaxtonBeneficiary::orderByRaw('name ASC, created_at DESC')->get();

        if ($baneficiaries->isEmpty()) Utils::error('No baneficiaries found!', 404);

        $endPoint = 'beneficiaries';

        try {
            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $baneficiaries = $data->Content->Model->Beneficiary;
                foreach ($baneficiaries as $nbaneficiary) {
                    $baneficiary = CaxtonBeneficiary::where('beneficiary_id', $nbaneficiary->Id)->first();
                    if (empty($baneficiary)) {
                        $baneficiary = new CaxtonBeneficiary();
                        $baneficiary->beneficiary_id = $nbaneficiary->Id;
                    }
                    $baneficiary->name = $nbaneficiary->Name;
                    $baneficiary->reference = $nbaneficiary->Reference;
                    $baneficiary->address1 = $nbaneficiary->BankAddress1;
                    $baneficiary->address2 = $nbaneficiary->BankAddress2;
                    $baneficiary->email = $nbaneficiary->UserName;
                    $baneficiary->phone = $nbaneficiary->Phone;
                    $baneficiary->country = $nbaneficiary->Account->Country;
                    $baneficiary->currency = $nbaneficiary->Account->Currency;
                    $baneficiary->iban = $nbaneficiary->Account->IbanNumber;
                    $baneficiary->swift = $nbaneficiary->Account->SwiftCode;
                    $baneficiary->sortCode = $nbaneficiary->Account->SortCode;
                    $baneficiary->account_number = $nbaneficiary->Account->AccountNumber;
                    $baneficiary->save();
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $baneficiaries;
    }

    // DELETE /api/finance-users/{user_id}/beneficiary/{id}
    public function beneficiary_delete(Request $req, $user_id, $id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $beneficiary = CaxtonBeneficiary::find($id);
        if (empty($beneficiary)) Utils::error('Beneficiary not found!', 404);

        try {
            $endPoint = 'beneficiaries/' . $beneficiary->beneficiary_id;
            $client = new \GuzzleHttp\Client();

            $data = $client->request('DELETE', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $beneficiary->delete();
                return $beneficiary;
            } else {
                Utils::error('something want to wrong', 404);
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $beneficiary;
    }

    // GET /api/finance-users/{user_id}/standing-instructions/{id}
    public function beneficiary_show(Request $req, $user_id, $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($country)) Utils::error('standing instruction not found!', 404);
        return $standing_instruction;
    }

    /*
     |-------------------------------------------
     | Standing Instruction Datails
     |-------------------------------------------
     */

    // GET /api/finance-users/{user_id}/standing-instructions
    public function standing_instruction_index(Request $req, $user_id)
    {
        $standing_instructions = StandingInstruction::selectRaw('standing_instructions.*,caxton_beneficiaries.name as beneficiary_name')->leftJoin('caxton_beneficiaries', 'caxton_beneficiaries.id', '=', 'standing_instructions.beneficiary_id')->orderByRaw('standing_instructions.created_at DESC');

        if ($query = $req->query('query', false)) {
            $standing_instructions->whereRaw('(standing_instructions.amount LIKE ?)', ["%$query%"]);
        }
        $standing_instructions->where('standing_instructions.user_id', $user_id);

        if ($beneficiary_id = $req->query('beneficiary_id', false)) {
            $standing_instructions->where('standing_instructions.beneficiary_id', $beneficiary_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $standing_instructions = $standing_instructions->get();
        } else {
            $standing_instructions = $standing_instructions->paginate($req->query('per_page', 20));
        }

        if ($standing_instructions->isEmpty()) Utils::error('No standing instructions found!', 404);

        return ($standing_instructions);
    }

    // POST /api/finance-users/{user_id}/standing-instructions
    public function standing_instruction_create(Request $req, $user_id)
    {
        $this->validate($req, [
            'user_id' => 'required|exists:caxton_users,id',
            'beneficiary_id' => 'required|exists:caxton_beneficiaries,id',
            'notes' => 'string',
            'amount' => 'required|numeric|gt:0',
            'standing_type' => 'required|in:weekly,monthly',
            'order_date' => 'required_if:standing_type,monthly|date',
            'expiry_date' => 'required|date',
            'days' => 'required_if:standing_type,weekly|string',
        ]);

        $standing_instruction = new StandingInstruction;
        $standing_instruction->fill($req->all());
        $standing_instruction->save();

        $standing_instruction->append('beneficiary_name');

        return $standing_instruction;
    }

    // POST /api/finance-users/{user_id}/standing-instructions/{id}
    public function standing_instruction_update(Request $req, $user_id, $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);

        $this->validate($req, [
            'user_id' => 'required|exists:caxton_users,id',
            'beneficiary_id' => 'required|exists:caxton_beneficiaries,id',
            'notes' => 'string',
            'amount' => 'required|numeric|gt:0',
            'standing_type' => 'required|in:weekly,monthly',
            'order_date' => 'required_if:standing_type,monthly|date',
            'expiry_date' => 'required|date',
            'days' => 'required_if:standing_type,weekly|string',
        ]);

        if ($req->has('user_id')) $standing_instruction->user_id = $req->input('user_id');
        if ($req->has('beneficiary_id')) $standing_instruction->beneficiary_id  = $req->input('beneficiary_id');
        if ($req->has('notes')) $standing_instruction->notes  = $req->input('notes');
        if ($req->has('amount')) $standing_instruction->amount  = $req->input('amount');
        if ($req->has('standing_type')) $standing_instruction->standing_type  = $req->input('standing_type');
        if ($req->has('expiry_date')) $standing_instruction->expiry_date  = $req->input('expiry_date');
        if ($req->has('days')) $standing_instruction->days  = $req->input('days');
        if ($req->has('order_date')) $standing_instruction->order_date  = $req->input('order_date');
        $standing_instruction->save();

        $standing_instruction->append('beneficiary_name');

        return $standing_instruction;
    }

    // DELETE /api/finance-users/{user_id}/standing-instructions/{id}
    public function standing_instruction_delete(Request $req, $user_id, $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);
        $standing_instruction->delete();
        return $standing_instruction;
    }

    /**
     * Enable standing-instructions.
     * POST /api/finance-users/{user_id}/standing-instructions/{id}/enable
     */
    public function standing_instruction_enable(Request $req, $user_id,  $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);
        $standing_instruction->disabled = 0;
        $standing_instruction->save();
        return $standing_instruction;
    }

    /**
     * Disable standing-instructions.
     * POST /api/finance-users/{user_id}/standing-instructions/{id}/disable
     */
    public function standing_instruction_disable(Request $req, $user_id,  $id)
    {
        $standing_instruction = StandingInstruction::find($id);
        if (empty($standing_instruction)) Utils::error('standing instruction not found!', 404);
        $standing_instruction->disabled = 1;
        $standing_instruction->save();
        return $standing_instruction;
    }

    /*
     |-------------------------------------------
     | BANK DETAILS
     |-------------------------------------------
     */

    /**
     * Get bank details object.
     * POST /api/finance-users/{user_id}/bank-details
     */
    public function get_bank_details(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $endPoint = 'reference/lookup/banks/GBP';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $bankDetails = $data->Content->Model->BankDetails;
                foreach ($bankDetails as $bankDetail) {
                    $bankdetail = CaxtonBankDetails::where('user_id', $user_id)->first();
                    if (empty($bankdetail)) {
                        $bankdetail = new CaxtonBankDetails;
                    }
                    $bankdetail->user_id  = $user_id;
                    $bankdetail->account_name  = $bankDetail->AccountName;
                    $bankdetail->currency  = $bankDetail->Currency;
                    $bankdetail->bank_name  = $bankDetail->BankName;
                    $bankdetail->account_number  = $bankDetail->AccountNumber;
                    $bankdetail->iban_number  = $bankDetail->IbanNumber;
                    $bankdetail->swift_code  = $bankDetail->SwiftCode;
                    $bankdetail->sortcode  = $bankDetail->SortCode;
                    $bankdetail->address  = $bankDetail->BankAddress;
                    $bankdetail->address1  = $bankDetail->BankAddress1;
                    $bankdetail->address2  = $bankDetail->BankAddress2;
                    $bankdetail->country  = $bankDetail->Country;
                    $bankdetail->type  = $bankDetail->Type;
                    $bankdetail->pay_in_reference  = $bankDetail->PayInReference;
                    $bankdetail->save();
                }
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $bankdetail;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->setting_generate_token();
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }


    /*
     |-------------------------------------------
     | TRANSFER
     |-------------------------------------------
     */

    /**
     * List Card.
     * GET /api/finance-users/{user_id}/transfer
     */
    public function transfer_index(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $transfers = CaxtonTransfer::orderByRaw('created_at DESC');
        $transfers->where('user_id', $caxtonUser->account_id);

        if ($query = $req->query('query', false)) {
            $transfers->whereRaw('(buy_amount LIKE ? OR beneficiary_name LIKE ? )', ["%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $transfers = $transfers->get();
        } else {
            $transfers = $transfers->paginate($req->query('per_page', 20));
        }

        if ($transfers->isEmpty()) Utils::error('No transfers found!', 404);

        return ($transfers);
    }


    /**
     * Pending transfer object.
     * POST /api/finance/id/transfer
     */
    public function fetch_pending_transfers(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        // if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $caxtonTransfers = CaxtonTransfer::orderByRaw('created_at DESC');
        $caxtonTransfers->where('user_id', $caxtonUser->account_id);
        $caxtonTransfers->whereNull('transfer_id');
        $caxtonTransfers = $caxtonTransfers->get();
        // if ($caxtonTransfers->isEmpty()) Utils::error('No transfers found!', 404);

        foreach ($caxtonTransfers as $caxtonTransfer) {
            try {
                $clientTransfer = new \GuzzleHttp\Client();
                $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                    '',
                    'headers' => [
                        'Authorization' => 'bearer ' . $caxtonUser->token,
                        'UserApiToken' => $caxtonUser->human_password,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                    $dataTransfer = json_decode($dataTransfer->getBody()->getContents());
                    $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                    $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                    $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                    $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                    $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                    $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                    $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                    $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                    $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                    $caxtonTransfer->save();
                } else {
                    // Utils::error('something want to wrong', 404);
                }

                $endPoint = 'transfers/' . $caxtonTransfer->transfer_id;

                $client = new \GuzzleHttp\Client();
                $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                    'headers' => [
                        'Authorization' => 'bearer ' . $caxtonUser->token,
                        'UserApiToken' => $caxtonUser->human_password,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                    $dataTransfer = json_decode($data->getBody()->getContents());
                    $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->MoneyTransfer->BuyCcyCode;
                    $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->MoneyTransfer->BuyAmount;
                    $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->MoneyTransfer->PayAwayDate;
                    $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->MoneyTransfer->FeeCurrency;
                    $caxtonTransfer->fee  =  $dataTransfer->Content->Model->MoneyTransfer->Fee;
                    $caxtonTransfer->account_transaction_id  =  $dataTransfer->Content->Model->MoneyTransfer->AccountTransactionId;
                    $caxtonTransfer->complete  =  $dataTransfer->Content->Model->MoneyTransfer->Complete;
                    $caxtonTransfer->automatic  =  $dataTransfer->Content->Model->MoneyTransfer->Automatic;
                    $caxtonTransfer->executed  =  $dataTransfer->Content->Model->MoneyTransfer->ExecutedDate;
                    $caxtonTransfer->pay_status  =  $dataTransfer->Content->Model->MoneyTransfer->PayStatus;
                    $caxtonTransfer->cash_available  =  $dataTransfer->Content->Model->MoneyTransfer->Cashavailable;
                    $caxtonTransfer->authorisation  =  $dataTransfer->Content->Model->MoneyTransfer->Authorisation;
                    $caxtonTransfer->authorisation_status  =  $dataTransfer->Content->Model->MoneyTransfer->AuthorisationStatus;
                    $caxtonTransfer->payment_accepted  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentAccepted;
                    $caxtonTransfer->rejection_reason  =  $dataTransfer->Content->Model->MoneyTransfer->RejectionReason;
                    $caxtonTransfer->payment_reference  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentReference;
                    $caxtonTransfer->is_pending  =  $dataTransfer->Content->Model->MoneyTransfer->IsPending;
                    $caxtonTransfer->created  =  $dataTransfer->Content->Model->MoneyTransfer->Id;
                    $caxtonTransfer->payment_provider  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentProvider;
                    $caxtonTransfer->compliance_status  =  $dataTransfer->Content->Model->MoneyTransfer->ComplianceStatus;
                    $caxtonTransfer->eye_check_status  =  $dataTransfer->Content->Model->MoneyTransfer->EyeCheckStatus;
                    $caxtonTransfer->compliance_status_desc  =  $dataTransfer->Content->Model->MoneyTransfer->ComplianceStatusDesc;
                    $caxtonTransfer->is_high_risk_transfer  =  $dataTransfer->Content->Model->MoneyTransfer->IsHighRiskTransfer;
                    $caxtonTransfer->external_source_id  =  $dataTransfer->Content->Model->MoneyTransfer->ExternalSourceId;
                    $caxtonTransfer->beneficiary_name  =  $dataTransfer->Content->Model->MoneyTransfer->BeneficiaryName;
                    $caxtonTransfer->save();
                } else {
                    // Utils::error('something want to wrong', 404);
                }
            } catch (\Exception $e) {
                if ($e->getCode() == 401) {
                    $this->setting_generate_token();
                }
            }
        }

        return $caxtonTransfers;
    }

    /**
     * Show transfer object.
     * POST /api/finance-users/{user_id}/show-transfer/id
     */
    public function show_transfer(Request $req, $user_id, $id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $caxtonTransfer = CaxtonTransfer::find($id);
        if (empty($caxtonTransfer)) Utils::error('transfer not found!', 404);

        try {
            if ($caxtonTransfer->transfer_id == 0) {
                $clientTransfer = new \GuzzleHttp\Client();
                $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                    '',
                    'headers' => [
                        'Authorization' => 'bearer ' . $caxtonUser->token,
                        'UserApiToken' => $caxtonUser->human_password,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                    $dataTransfer = json_decode($dataTransfer->getBody()->getContents());
                    $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                    $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                    $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                    $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                    $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                    $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                    $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                    $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                    $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                    $caxtonTransfer->save();

                    return $caxtonTransfer;
                } else {
                    Utils::error('something want to wrong', 404);
                }
            }

            $endPoint = 'transfers/' . $caxtonTransfer->transfer_id;

            $client = new \GuzzleHttp\Client();
            $data = $client->request('GET', env('CAXTON_BASEURL') . $endPoint, [
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $dataTransfer = json_decode($data->getBody()->getContents());
                $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->MoneyTransfer->BuyCcyCode;
                $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->MoneyTransfer->BuyAmount;
                $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->MoneyTransfer->PayAwayDate;
                $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->MoneyTransfer->FeeCurrency;
                $caxtonTransfer->fee  =  $dataTransfer->Content->Model->MoneyTransfer->Fee;
                $caxtonTransfer->account_transaction_id  =  $dataTransfer->Content->Model->MoneyTransfer->AccountTransactionId;
                $caxtonTransfer->complete  =  $dataTransfer->Content->Model->MoneyTransfer->Complete;
                $caxtonTransfer->automatic  =  $dataTransfer->Content->Model->MoneyTransfer->Automatic;
                $caxtonTransfer->executed  =  $dataTransfer->Content->Model->MoneyTransfer->ExecutedDate;
                $caxtonTransfer->pay_status  =  $dataTransfer->Content->Model->MoneyTransfer->PayStatus;
                $caxtonTransfer->cash_available  =  $dataTransfer->Content->Model->MoneyTransfer->Cashavailable;
                $caxtonTransfer->authorisation  =  $dataTransfer->Content->Model->MoneyTransfer->Authorisation;
                $caxtonTransfer->authorisation_status  =  $dataTransfer->Content->Model->MoneyTransfer->AuthorisationStatus;
                $caxtonTransfer->payment_accepted  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentAccepted;
                $caxtonTransfer->rejection_reason  =  $dataTransfer->Content->Model->MoneyTransfer->RejectionReason;
                $caxtonTransfer->payment_reference  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentReference;
                $caxtonTransfer->is_pending  =  $dataTransfer->Content->Model->MoneyTransfer->IsPending;
                $caxtonTransfer->created  =  $dataTransfer->Content->Model->MoneyTransfer->Id;
                $caxtonTransfer->payment_provider  =  $dataTransfer->Content->Model->MoneyTransfer->PaymentProvider;
                $caxtonTransfer->compliance_status  =  $dataTransfer->Content->Model->MoneyTransfer->ComplianceStatus;
                $caxtonTransfer->eye_check_status  =  $dataTransfer->Content->Model->MoneyTransfer->EyeCheckStatus;
                $caxtonTransfer->compliance_status_desc  =  $dataTransfer->Content->Model->MoneyTransfer->ComplianceStatusDesc;
                $caxtonTransfer->is_high_risk_transfer  =  $dataTransfer->Content->Model->MoneyTransfer->IsHighRiskTransfer;
                $caxtonTransfer->external_source_id  =  $dataTransfer->Content->Model->MoneyTransfer->ExternalSourceId;
                $caxtonTransfer->beneficiary_name  =  $dataTransfer->Content->Model->MoneyTransfer->BeneficiaryName;

                $caxtonTransfer->save();
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $caxtonTransfer;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * POST transfer object.
     * POST /api/finance-users/{user_id}/transfer
     */
    public function transfer_create(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'notes' => 'string',
            'amount' => 'required|numeric|gt:0',
            'otp' => 'required|string|max:6',
        ]);

        if ($req->input('otp') !=  $caxtonUser->otp) {
            return Utils::error('otp not matched.please enter correct otp.', 400);
        }

        try {
            $endPoint = 'transfers/pending';

            $beneficiary_id = $req->input('beneficiary_id');
            $amount = $req->input('amount');
            $notes = $req->input('notes');

            if ($req->has('payment_date')) {
                $date = date('Y-m-d', strtotime($req->input('payment_date')));
            } else {
                $date = date('Y-m-d');
            }

            if ($req->has('transaction_reason')) {
                $transaction_reason = $req->input('transaction_reason');
            } else {
                $transaction_reason = 'Emigration';
            }

            $PayAwayReference = '';
            if ($notes != '') {
                $beneficiary = CaxtonBeneficiary::where('beneficiary_id', $beneficiary_id)->first();
                $beneficiary->notes = $notes;
                $beneficiary->save();
                $PayAwayReference = str_replace(' ', '', $notes); //substr(str_shuffle('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890'), 0, 4);
            }

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "BeneficiaryId":"' . $beneficiary_id . '",
                    "BuyCurrency": "GBP",
                    "BuyAmount": "' . $amount . '",
                    "PayAwayDate": "' . $date . '",
                    "PayAwayReference": "' . $PayAwayReference . '",
                    "FeeCurrency": "",
                    "Fee": 0,
                    "OpenBankingAuthId": 0,
                    "TransferType": "0",
                    "TransferReason":"' . $transaction_reason . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $caxtonTransfer = new CaxtonTransfer();
                $caxtonTransfer->pending_transfer_id = $data->Content->Model->Value;
                $caxtonTransfer->notes = $notes;
                $caxtonTransfer->save();

                $user_id = SiteSettingController::findByKey('caxton_user_id')->value;

                $fee = $req->input('fee');
                if ($fee > 0) {
                    $endPointFee = 'account/balances/third-party/transfer';
                    $clientFee = new \GuzzleHttp\Client();
                    $clientFee->request('POST', env('CAXTON_BASEURL') . $endPointFee, [
                        'body' => '{
                            "UserId":"' . $user_id . '",
                            "SendCurrency": "GBP",
                            "SendAmount": ' . (float)$fee . ',
                        }',
                        'headers' => [
                            'Authorization' => 'bearer ' . $caxtonUser->token,
                            'UserApiToken' => $caxtonUser->human_password,
                            'content-type' => 'application/json',
                        ],
                    ]);
                }

                $clientTransfer = new \GuzzleHttp\Client();
                $endPointTransfer = 'transfers/pending/' . $caxtonTransfer->pending_transfer_id;
                $dataTransfer = $clientTransfer->request('GET', env('CAXTON_BASEURL') . $endPointTransfer, [
                    '',
                    'headers' => [
                        'Authorization' => 'bearer ' . $caxtonUser->token,
                        'UserApiToken' => $caxtonUser->human_password,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($dataTransfer->getStatusCode() >= 200 && $dataTransfer->getStatusCode() <= 210) {
                    $dataTransfer = json_decode($dataTransfer->getBody()->getContents());

                    $caxtonTransfer->pending_transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->Id;
                    $caxtonTransfer->transfer_id  =  $dataTransfer->Content->Model->PendingTransfer->TransferId;
                    $caxtonTransfer->beneficiary_id  =  $dataTransfer->Content->Model->PendingTransfer->BeneficiaryId;
                    $caxtonTransfer->user_id  =  $dataTransfer->Content->Model->PendingTransfer->UserId;
                    $caxtonTransfer->buy_ccy_Code  =  $dataTransfer->Content->Model->PendingTransfer->BuyCurrency;
                    $caxtonTransfer->buy_amount  =  $dataTransfer->Content->Model->PendingTransfer->BuyAmount;
                    $caxtonTransfer->pay_away  =  $dataTransfer->Content->Model->PendingTransfer->PayAwayDate;
                    $caxtonTransfer->fee_currency  =  $dataTransfer->Content->Model->PendingTransfer->FeeCurrency;
                    $caxtonTransfer->fee  =  $dataTransfer->Content->Model->PendingTransfer->Fee;
                    $caxtonTransfer->save();

                    return $caxtonTransfer;
                } else {
                    Utils::error('something want to wrong', 404);
                }
            } else {
                Utils::error('something want to wrong', 404);
            }
            // return $bankdetail;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $caxtonUser->id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }

    /**
     * Pay Transfer object.
     * POST /api/finance-users/{user_id}/transfer-load
     */
    public function transfer_load(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $endPoint = 'account/balances/third-party/transfer';

            $user_id =  SiteSettingController::findByKey('caxton_user_id')->value;
            $buy_amount = $req->input('buy_amount');

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "UserId":"' . $user_id . '",
                    "SendCurrency": "GBP",
                    "SendAmount": ' . (float)$buy_amount . ',
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
            } else {
                Utils::error('something want to wrong', 404);
            }
            // return $bankdetail;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }
        return $req;
    }


    /*
     |-------------------------------------------
     | TRANSFER TRANSACTIONS DETAILS
     |-------------------------------------------
     */

    /**
     * List Transfer Transaction.
     * GET /api/caxton/transfer-transaction-fetch
     */
    public function transfer_transaction_index(Request $req, $user_id)
    {
        $transactions = CaxtonTransferTransaction::orderByRaw('transaction_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(amount LIKE ? OR status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $transactions->where('user_id', $user_id);
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $transactions = $transactions->where('transaction_date', '>=', $from_date);
                $transactions = $transactions->where('transaction_date', '<=', $to_date);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $transactions = $transactions->get();
        } else {
            $transactions = $transactions->paginate($req->query('per_page', 20));
        }

        if ($transactions->isEmpty()) Utils::error('No transactions found!', 404);

        return ($transactions);
    }

    /**
     * Count Transfer Transaction.
     * GET /api/finance-users/{user_id}/transfer-transaction-count
     */
    public function transfer_transaction_count(Request $req, $user_id)
    {
        $transactions = CaxtonTransferTransaction::orderByRaw('transaction_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(amount LIKE ? OR status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $transactions->where('user_id', $user_id);
        if ($req->has('from_date')) {
            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
            $transactions = $transactions->where('transaction_date', '>=', $from_date);
            $transactions = $transactions->where('transaction_date', '<=', $to_date);
        }
        $transactions = $transactions->count();

        $data = [
            'no_of_transactions' => (int)$transactions,
        ];

        return $data;
    }

    /**
     * Export Transfer Transaction.
     * GET /api/finance-users/{user_id}/transfer-transaction-export
     */
    public function transfer_transaction_export(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->format('d-m-Y');
                $to_date = Carbon::parse($req->input('to_date'))->format('d-m-Y');
                $fileName =  $from_date . " to " . $to_date;
            } else {
                $fileName = "Transfer Transactions";
            }
        } else {
            $fileName = "Transfer Transactions";
        }

        $reports = '';
        $req->merge(['nopaginate' => 1]);
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $reports = ($this->transfer_transaction_index($req, $user_id));
            return Excel::download(new TransferTransactionsExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            $reports = ($this->transfer_transaction_index($req, $user_id));
            return $this->printStatement($req, $reports, $caxtonUser);
            // return Excel::download(new TransferTransactionsExport($reports), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    /**
     * Fetch transfer transactions list.
     * POST /api/caxton/account-transfer-transaction
     */
    public function transfer_transaction_sync(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $endPoint = 'account/transactions/search';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "StartDate": "' . Carbon::today()->subDays(5)->format('Y-m-d 00:00:00') . '",
                    "EndDate": "' . Carbon::today()->format('Y-m-d 23:59:59') . '",
                    "Currency": "GBP",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $transactions = $data->Content->Model->AccountTransaction;
                foreach ($transactions as $ntransaction) {
                    $transaction = CaxtonTransferTransaction::where('account_transation_id', $ntransaction->Id)->first();
                    if (empty($transaction)) {
                        $transaction = new CaxtonTransferTransaction;
                    }
                    $transaction->user_id  =  $user_id;
                    $transaction->account_transation_id  =  $ntransaction->Id;
                    $transaction->account_summary_id  =  $ntransaction->AccountSummaryId;
                    $transaction->amount  =  $ntransaction->Amount;
                    $transaction->status  =  $ntransaction->Status;
                    $transaction->total_balance  =  $ntransaction->TotalBalance;
                    $transaction->currency  =  $ntransaction->Currency;
                    $transaction->base_currency  =  $ntransaction->BaseCurrency;
                    $transaction->transaction_date  =  $ntransaction->TransactionDateTime;
                    $transaction->save();
                }
            } else {
                Utils::error('something want to wrong', 404);
            }
            return $transactions;
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            } else {
                if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                    $data = json_decode($e->getResponse()->getBody());
                    Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
                }
            }
        }

        return $req;
    }


    /**
     * Fetch transfer transactions list.
     */
    public function fetch_transfer_transaction_sync(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) return $req;

        try {
            $endPoint = 'account/transactions/search';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "StartDate": "' . Carbon::today()->subDays(5)->format('Y-m-d 00:00:00') . '",
                    "EndDate": "' . Carbon::today()->format('Y-m-d 23:59:59') . '",
                    "Currency": "GBP",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $transactions = $data->Content->Model->AccountTransaction;
                foreach ($transactions as $ntransaction) {
                    $transaction = CaxtonTransferTransaction::where('account_transation_id', $ntransaction->Id)->first();
                    if (empty($transaction)) {
                        $transaction = new CaxtonTransferTransaction;
                    }
                    $transaction->user_id  =  $user_id;
                    $transaction->account_transation_id  =  $ntransaction->Id;
                    $transaction->account_summary_id  =  $ntransaction->AccountSummaryId;
                    $transaction->amount  =  $ntransaction->Amount;
                    $transaction->status  =  $ntransaction->Status;
                    $transaction->total_balance  =  $ntransaction->TotalBalance;
                    $transaction->currency  =  $ntransaction->Currency;
                    $transaction->base_currency  =  $ntransaction->BaseCurrency;
                    $transaction->transaction_date  =  $ntransaction->TransactionDateTime;
                    $transaction->save();
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            }
        }

        return $req;
    }

    public function printStatement($req, $transactions, $user)
    {
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->format('d-M-Y');
                $to_date = Carbon::parse($req->input('to_date'))->format('d-M-Y');
            }
        }

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            /*border-left: 1px solid #000000;
                border-top: 1px solid #000000;*/
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #c5c7c9;
            font-weight:600;
            background-color:#ffffff;
            color:#000;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #c5c7c9;
        }
        .items-table tr:nth-child(even){
            background-color:#f0faff   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $bankDetails = CaxtonBankDetails::where('user_id', $user->id)->first();

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h3 style = "text-align:center">Account Statement</h3>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td><h2>Transactions</h2></td>';
        $html = $html . '<td>Account Name : ' . $bankDetails->account_name . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>UBSIDI Limited</td>';
        $html = $html . '<td>Account Number : ' . $bankDetails->account_number . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td>Sort Code : ' . $bankDetails->sortcode . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall,WS1 3RA</td>';
        $html = $html . '<td>' . Carbon::today()->format('jS F, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $html = $html . '<tr>';
                $html = $html . '<td>Showing : ' . $from_date . ' to ' . $to_date . '</td>';
                $html = $html . '<td></td>';
                $html = $html . '</tr>';
            }
        }
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Date</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Paid in</th>';
        $html = $html . '<th>Paid out</th>';
        $html = $html . '<th>Balance</th>';
        $html = $html . '</tr>';
        foreach ($transactions as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . Carbon::parse($item->transaction_date)->format('d M Y') . '</td>';
            $html = $html . '<td style="text-align:left;">' . $item->status . '</td>';
            if ($item->amount >= 0) {
                $html = $html . '<td>£' . $item->amount . '</td>';
            } else {
                $html = $html . '<td> - </td>';
            }
            if ($item->amount < 0) {
                $html = $html . '<td>£' . abs($item->amount) . '</td>';
            } else {
                $html = $html . '<td> - </td>';
            }
            $html = $html . '<td>£' . $item->total_balance . '</td>';

            $html = $html . '</tr>';
        }

        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';

        $mpdf = new Mpdf();
        $mpdf->setFooter('{PAGENO}');
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($html);
        $fileName = "Statement";
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $fileName = 'Statement - ' . $from_date . ' to ' . $to_date;
            }
        }
        return $mpdf->Output("$fileName.pdf", "D");
    }

    /*
     |-------------------------------------------
     | TRANSFER TRANSACTIONS DETAILS
     |-------------------------------------------
     */

    /**
     * List Transfer Transaction.
     * GET /api/caxton/transaction-fetch
     */
    public function transaction_index(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $transactions = CaxtonTransaction::orderByRaw('executed_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(buy_amount LIKE ? OR pay_status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $transactions->where('user_id', $caxtonUser->account_id);
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $transactions = $transactions->where('executed_date', '>=', $from_date);
                $transactions = $transactions->where('executed_date', '<=', $to_date);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $transactions = $transactions->get();
        } else {
            $transactions = $transactions->paginate($req->query('per_page', 20));
        }

        if ($transactions->isEmpty()) Utils::error('No transactions found!', 404);

        return ($transactions);
    }

    /**
     * Count Transfer Transaction.
     * GET /api/finance-users/{user_id}/transaction-count
     */
    public function transaction_count(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        $transactions = CaxtonTransaction::orderByRaw('executed_date DESC');

        if ($query = $req->query('query', false)) {
            $transactions->whereRaw('(buy_amount LIKE ? OR pay_status LIKE ? )', ["%$query%", "%$query%"]);
        }
        $transactions->where('user_id', $caxtonUser->account_id);
        if ($req->has('from_date')) {
            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
            $transactions = $transactions->where('executed_date', '>=', $from_date);
            $transactions = $transactions->where('executed_date', '<=', $to_date);
        }
        $transactions = $transactions->count();

        $data = [
            'no_of_transactions' => (int)$transactions,
        ];

        return $data;
    }

    /**
     * Export Transfer Transaction.
     * GET /api/finance-users/{user_id}/transaction-export
     */
    public function transaction_export(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->format('d-m-Y');
                $to_date = Carbon::parse($req->input('to_date'))->format('d-m-Y');
                $fileName =  $from_date . " to " . $to_date;
            } else {
                $fileName = "Transfer Transactions";
            }
        } else {
            $fileName = "Transfer Transactions";
        }

        $reports = '';
        $req->merge(['nopaginate' => 1]);
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $reports = ($this->transaction_index($req, $user_id));
            return Excel::download(new TransferTransactionsExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            $reports = ($this->transaction_index($req, $user_id));
            return $this->printTransaction($req, $reports, $caxtonUser);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    /**
     * Fetch transfer transactions list.
     * POST /api/caxton/account-transaction
     */
    public function transaction_sync(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $endPoint = 'transfers';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Currency": "GBP",
                    "DateFrom": "' . Carbon::today()->subDays(1000)->format('Y-m-d') . '",
                    "DateTo": "' . Carbon::today()->format('Y-m-d') . '",
                    "pageNumber": 1,
                    "pageSize": 1000,
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $transactions = $data->Content->Model->Records;
                foreach ($transactions as $ntransaction) {
                    $transaction = CaxtonTransaction::where('transactions_id', $ntransaction->Id)->first();
                    if (empty($transaction)) {
                        $transaction = new CaxtonTransaction;
                    }
                    $transaction->transactions_id  =  $ntransaction->Id;
                    $transaction->beneficiary_id  =  $ntransaction->BeneficiaryId;
                    $transaction->user_id  =  $ntransaction->UserId;
                    $transaction->buy_ccy_code  =  $ntransaction->BuyCcyCode;
                    $transaction->buy_amount  =  $ntransaction->BuyAmount;
                    $transaction->executed_date  =  $ntransaction->ExecutedDate;
                    $transaction->pay_away_date  =  $ntransaction->PayAwayDate;
                    $transaction->pay_status  =  $ntransaction->PayStatus;
                    $transaction->account_transation_id  =  $ntransaction->AccountTransactionId;
                    $transaction->payment_reference  =  $ntransaction->PaymentReference;
                    $transaction->created_date  =  $ntransaction->CreatedDate;
                    $transaction->payment_provider  =  $ntransaction->PaymentProvider;
                    $transaction->compliance_status_desc  =  $ntransaction->ComplianceStatusDesc;
                    $transaction->beneficiary_name  =  $ntransaction->BeneficiaryName;
                    $transaction->save();
                }
            }
            $this->deposit_sync($req, $user_id);
            return $transactions;
        } catch (\Exception $e) {
            $this->deposit_sync($req, $user_id);
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            }
        }

        return $req;
    }

    /**
     * Fetch transfer deposits list.
     * POST /api/caxton/account-deposit
     */
    public function deposit_sync(Request $req, $user_id)
    {
        $caxtonUser = CaxtonUser::find($user_id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {

            $client = new \GuzzleHttp\Client();
            $endPointDeposit = 'account/transactions/deposits';
            $dataDeposit = $client->request('POST', env('CAXTON_BASEURL') . $endPointDeposit, [
                'body' => '{
                    "Currency": "GBP",
                    "StartDate": "' . Carbon::today()->subDays(1000)->format('Y-m-d') . '",
                    "EndDate": "' . Carbon::today()->format('Y-m-d') . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($dataDeposit->getStatusCode() >= 200 && $dataDeposit->getStatusCode() <= 210) {
                $dataDeposit = json_decode($dataDeposit->getBody()->getContents());
                $transactions = $dataDeposit->Content->Model->AccountTransaction;
                foreach ($transactions as $ntransaction) {
                    $transaction = CaxtonTransaction::where('account_transation_id', $ntransaction->AccountTransactionId)->first();
                    if (empty($transaction)) {
                        $transaction = new CaxtonTransaction;
                    }
                    $transaction->user_id  =  $ntransaction->UserId;
                    $transaction->buy_ccy_code  =  $ntransaction->Currency;
                    $transaction->buy_amount  =  $ntransaction->Amount;
                    $transaction->executed_date  =  $ntransaction->DepositDate;
                    $transaction->pay_status  =  $ntransaction->Status;
                    $transaction->account_transation_id  =  $ntransaction->AccountTransactionId;
                    $transaction->payment_reference  =  $ntransaction->Reference;
                    $transaction->created_date  =  $ntransaction->DepositDate;
                    $transaction->save();
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $this->generate_token($req, $user_id);
            }
        }
    }

    public function printTransaction($req, $transactions, $user)
    {
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->format('d-M-Y');
                $to_date = Carbon::parse($req->input('to_date'))->format('d-M-Y');
            }
        }

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            /*border-left: 1px solid #000000;
                border-top: 1px solid #000000;*/
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #c5c7c9;
            font-weight:600;
            background-color:#ffffff;
            color:#000;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #c5c7c9;
        }
        .items-table tr:nth-child(even){
            background-color:#f0faff   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $bankDetails = CaxtonBankDetails::where('user_id', $user->id)->first();

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h3 style = "text-align:center">Account Statement</h3>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td><h2>Transactions</h2></td>';
        $html = $html . '<td>Account Name : ' . $bankDetails->account_name . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>UBSIDI Limited</td>';
        $html = $html . '<td>Account Number : ' . $bankDetails->account_number . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td>Sort Code : ' . $bankDetails->sortcode . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall,WS1 3RA</td>';
        $html = $html . '<td>' . Carbon::today()->format('jS F, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $html = $html . '<tr>';
                $html = $html . '<td>Showing : ' . $from_date . ' to ' . $to_date . '</td>';
                $html = $html . '<td></td>';
                $html = $html . '</tr>';
            }
        }
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Date</th>';
        $html = $html . '<th>Executed Date</th>';
        $html = $html . '<th>Status</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Amount</th>';
        $html = $html . '<th>Beneficiary Name</th>';
        $html = $html . '</tr>';
        foreach ($transactions as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . Carbon::parse($item->created_date)->format('d M Y') . '</td>';
            $html = $html . '<td>' . Carbon::parse($item->executed_date)->format('d M Y') . '</td>';
            $html = $html . '<td style="text-align:left;">' . $item->pay_status . '</td>';
            $html = $html . '<td>' . $item->payment_reference . '</td>';
            $html = $html . '<td>£' . $item->buy_amount . '</td>';
            if (!empty($item->beneficiary)) {
                $html = $html . '<td>' . $item->beneficiary->name . '</td>';
            } else {
                $html = $html . '<td></td>';
            }
            $html = $html . '</tr>';
        }


        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';

        $mpdf = new Mpdf();
        $mpdf->setFooter('{PAGENO}');
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($html);
        $fileName = "Statement";
        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $fileName = 'Statement - ' . $from_date . ' to ' . $to_date;
            }
        }
        return $mpdf->Output("$fileName.pdf", "D");
    }

    /*
     |-------------------------------------------
     | LOGIN & GENERATE TOKEN & PASSWORD
     |-------------------------------------------
     */

    public function generate_token(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $endPoint = 'token';
            $client = new \GuzzleHttp\Client();

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'form_params' => [
                    'Grant_Type' => 'password',
                    'Username' => $caxtonUser->username,
                    'Password' => $caxtonUser->password
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                // update caxtonUser
                $caxtonUser->token  = $data->access_token;
                $caxtonUser->save();

                // Login and update password
                $this->login($req, $id);
            }
        } catch (\Exception $e) {
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data, 404);
            }
        }

        return $caxtonUser;
    }

    public function login(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $encrypted_data = '';
            if ($caxtonUser->public_key) {
                $cipher = "aes-128-cbc";
                //Generate a 256-bit encryption key 
                $encryption_key = $caxtonUser->public_key; //set it to the value that /clients/auth/register api call  gives
                // Generate an initialization vector 
                $iv_size = openssl_cipher_iv_length($cipher);
                // please generate 16 digit random string here instead of using it
                $iv = substr(str_shuffle('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'), 0, 16);
                //Data to encrypt 
                $data = "ubsidi";
                $encrypted_data = openssl_encrypt($data, $cipher, $encryption_key, 0, $iv);

                $caxtonUser->encrypt_key  = $iv;
                $caxtonUser->save();
            }

            $endPoint = 'users/auth/login-step-one';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "UserName":"' . $caxtonUser->email . '",
                    "Password":"' . env('CAXTON_LOGIN_TOKEN') . '",
                    "DeviceId": "' . $encrypted_data . '",
                    "Device": "",
                    "OperatingSystem": "",
                    "AppVersion": ""
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'ivKey' => $caxtonUser->encrypt_key,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                // update caxtonUser
                $caxtonUser->human_password  = $data->Content->Model->LoginToken;
                $caxtonUser->account_id  = $data->Content->Model->Details->Id;
                $caxtonUser->save();
            }
        } catch (\Exception $e) {
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            }
        }

        return $caxtonUser;
    }

    public function generate_password(Request $req, $id)
    {
        $caxtonUser = CaxtonUser::find($id);
        if (empty($caxtonUser)) Utils::error('user not found!', 404);

        try {
            $endPoint = 'clients/auth/register';
            $client = new \GuzzleHttp\Client();

            $apiKey = SiteSettingController::findByKey('caxton_token')->value;

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Clientname":"' . $caxtonUser->name . '",
                    "Username":"' . $caxtonUser->username . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                // update caxtonUser
                $caxtonUser->password  = $data->Content->Model->ClientPassword;
                $caxtonUser->public_key  = $data->Content->Model->PublicKey;
                $caxtonUser->save();
                // $this->generate_token($req, $caxtonUser->id);
                // $this->login($req, $caxtonUser->id);
            }
        } catch (\Exception $e) {
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data, 404);
            }
        }

        return $caxtonUser;
    }

    public function setting_generate_token()
    {
        try {
            $endPoint = 'token';
            $client = new \GuzzleHttp\Client();

            $caxton_username = SiteSettingController::findByKey('caxton_username')->value;
            $caxton_password = SiteSettingController::findByKey('caxton_password')->value;

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "Grant_Type":"password",
                    "Username":"' . $caxton_username . '",
                    "Password":"' . $caxton_password . '",
                }',
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());

                // update Sitesetting
                $caxton_usersettingname = SiteSettingController::findByKey('caxton_token');
                $caxton_usersettingname->value  = $data->access_token;
                $caxton_usersettingname->save();
            }
        } catch (\Exception $e) {
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            }
        }

        return $caxton_usersettingname;
    }
}