<?php

namespace App\Http\Controllers\Connect;

use App\ModelsConnect\Company;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class CompanyController extends Controller
{
    /**
     * List comapnies.
     * GET /connect/comapnies
     */
    public function index(Request $req)
    {
        $comapnies = Company::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $comapnies->whereRaw('(s_account_id LIKE ? OR name LIKE ? OR phone_number LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('connect_account_id', false)) {
            $comapnies->where('connect_account_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $comapnies = $comapnies->get();
        } else {
            $comapnies = $comapnies->paginate($req->query('per_page', 20));
        }

        if ($comapnies->isEmpty()) Utils::error('No comapnies found!', 404);


        return ($comapnies);
    }

    /**
     * Create company.
     * POST /connect/comapnies
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            's_account_id' => 'string',
            'connect_account_id' => 'exists:connect_accounts.id',
            'structure' => 'string',
            'name' => 'required|string',
            'email' => 'required|email',
            'mcc' => 'required',
            'phone_number' => 'required|string',
            'tax_id' => 'required_with:structure|string',
            'line1' => 'string|required',
            'line2' => 'string',
            'city' => 'string|required',
            'state' => 'string',
            'postal_code' => 'string|required',
        ]);

        // create company
        $company = new Company;
        $company->fill($req->all());

        $company->save();

        return $company;
    }

    /**
     * Show company.
     * GET /connect/comapnies/{id}
     */
    public function show(Request $req, $id)
    {
        $company = Company::find($id);
        if (empty($company)) Utils::error('Company not found!', 404);
        return $company;
    }

    /**
     * Update company.
     * POST /connect/comapnies/{id}
     */
    public function update(Request $req, $id)
    {
        $company = Company::find($id);
        if (empty($company)) Utils::error('Company not found!', 404);

        $this->validate($req, [
            's_account_id' => 'string',
            'structure' => 'string',
            'name' => 'required|string',
            'email' => 'required|email',
            'mcc' => 'required',
            'phone_number' => 'required|string',
            'tax_id' => 'required_with:structure|string',
            'line1' => 'string|required',
            'line2' => 'string',
            'city' => 'string|required',
            'state' => 'string',
            'postal_code' => 'string|required',

        ]);

        // update company
        if ($req->has('s_account_id')) $company->s_account_id = $req->input('s_account_id');
        if ($req->has('structure')) $company->structure = $req->input('structure');
        if ($req->has('name')) $company->name = $req->input('name');
        if ($req->has('email')) $company->email = $req->input('email');
        if ($req->has('mcc')) $company->mcc = $req->input('mcc');
        if ($req->has('phone_number')) $company->phone_number = $req->input('phone_number');
        if ($req->has('tax_id')) $company->tax_id = $req->input('tax_id');
        if ($req->has('line1')) $company->line1 = $req->input('line1');
        if ($req->has('line2')) $company->line2 = $req->input('line2');
        if ($req->has('city')) $company->city = $req->input('city');
        if ($req->has('state')) $company->state = $req->input('state');
        if ($req->has('postal_code')) $company->postal_code = $req->input('postal_code');

        $company->save();

        return $company;
    }


    /**
     * Delete company by id.
     * DELETE /connect/comapnies/{id}
     */
    public function delete(Request $req, $id)
    {
        $company = Company::find($id);
        if (empty($company)) Utils::error('Company not found!', 404);
        $company->delete();
        return $company;
    }
}
