<?php

namespace App\Http\Controllers\Connect;

use App\Business;
use App\BusinessSurcharge;
use App\CardBrandCommissionHistory;
use App\CountryCommissionHistory;
use App\Exports\MultiplePayoutReportExport;
use App\ModelsConnect\ConnectAccount;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ResellerPayoutReportController;
use App\Http\Controllers\SiteSettingController;
use App\InvoiceTransaction;
use App\MiscCharge;
use App\MiscInvoice;
use App\ModelsConnect\Company;
use App\ModelsConnect\ConnectFile;
use App\ModelsConnect\ExternalAccount;
use App\ModelsConnect\Person;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\OnlineOrder;
use App\Passbook;
use App\PayoutCharge;
use App\PayoutInvoice;
use App\StripeTransaction;
use App\User;
use App\UserBusiness;
use App\WeeklyCharge;
use Carbon\Carbon;
use DateTime;
use Exception;
use Hamcrest\Util;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Maatwebsite\Excel\Facades\Excel;
use Stripe\File;
use Stripe\Stripe;
use Stripe\StripeClient;

class ConnectAccountController extends Controller
{
    private static $stripe;
    private static $key;
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'connect_stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'connect_stripe_private_key_live';
        }
        self::$key =  SiteSettingController::findByKey($columns)->value;
        self::$stripe = new StripeClient(["api_key" => self::$key, "stripe_version" => "2020-08-27"]);
    }
    /**
     * List accounts.
     * GET /connect/accounts
     */
    public function index(Request $req)
    {
        $accounts = ConnectAccount::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $accounts->whereRaw('(s_account_id LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $accounts->where('business_id', $query);
        }
        if ($query = $req->query('business_type', false)) {
            $accounts->where('business_type', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $accounts = $accounts->get();
        } else {
            $accounts = $accounts->paginate($req->query('per_page', 20));
        }

        if ($accounts->isEmpty()) Utils::error('No accounts found!', 404);


        return ($accounts);
    }

    /**
     * Create account.
     * POST /connect/accounts
     */
    public function create(Request $req)
    {
        if (!is_array($req->company))
            $req->merge(['company' => (json_decode($req->company, true))]);
        if (!is_array($req->account_opener))
            $req->merge(['account_opener' => (json_decode($req->account_opener, true))]);
        if (!is_array($req->account_owner))
            $req->merge(['account_owner' => (json_decode($req->account_owner, true))]);
        if (!is_array($req->external_account))
            $req->merge(['external_account' => (json_decode($req->external_account, true))]);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'business_type' => 'required|in:company,individual',
            'country' => 'required|max:2',
            'file_account_opener_doc_front' => 'file|max:2048',
            'file_account_opener_doc_back' => 'file|max:2048',
            'file_account_opener_add_front' => 'file|max:2048',
            'file_account_owner_doc_front' => 'file|max:2048',
            'file_account_owner_doc_back' => 'file|max:2048',
            'file_account_owner_add_front' => 'file|max:2048',
            // 'comapny' => 'required|array',
            // 'account_opener' => 'required|array',
        ]);
        $this->validate($req, [
            'company.structure' => 'string',
            'company.name' => 'required|string',
            'company.email' => 'required|email',
            'company.mcc' => 'required',
            'company.phone_number' => 'required|string',
            'company.tax_id' => 'required_with:structure|string',
            'company.line1' => 'string|required',
            'company.line2' => 'string',
            'company.city' => 'string|required',
            'company.state' => 'string',
            'company.postal_code' => 'string|required',
        ]);
        $this->validate($req, [
            'account_opener.first_name' => 'string',
            'account_opener.last_name' => 'required|string',
            'account_opener.email' => 'required|email',
            'account_opener.phone_number' => 'required|string',
            'account_opener.title' => 'required|string',
            'account_opener.id_number' => 'required|string',
            'account_opener.dob' => 'required|date',

            'account_opener.line1' => 'string|required',
            'account_opener.line2' => 'string',
            'account_opener.city' => 'string|required',
            'account_opener.state' => 'string',
            'account_opener.postal_code' => 'string|required',

            'account_opener.owner' => 'boolean',
            'account_opener.director' => 'boolean',
            'account_opener.executive' => 'boolean',
            'account_opener.account_opener' => 'boolean',
            'account_opener.representative' => 'boolean',

        ]);
        $this->validate($req, [
            'external_account.account_number' => 'string|required',
            'external_account.routing_number' => 'string|required',
            'external_account.account_holder_name' => 'string|required',
            'external_account.account_holder_type' => 'string|required',
            'external_account.country' => 'string|required',
            'external_account.currency' => 'string|required',

        ]);
        if ($req->input('business_type') == 'company') {
            $this->validate($req, [
                'account_owner.first_name' => 'string',
                'account_owner.last_name' => 'required|string',
                'account_owner.email' => 'required|email',
                'account_owner.phone_number' => 'required|string',
                'account_owner.title' => 'required|string',
                'account_owner.id_number' => 'required|string',
                'account_owner.dob' => 'required|date',

                'account_owner.line1' => 'string|required',
                'account_owner.line2' => 'string',
                'account_owner.city' => 'string|required',
                'account_owner.state' => 'string',
                'account_owner.postal_code' => 'string|required',

                'account_owner.owner' => 'boolean',
                'account_owner.director' => 'boolean',
                'account_owner.executive' => 'boolean',
                'account_owner.account_opener' => 'boolean',
                'account_owner.representative' => 'boolean',

            ]);
        }

        // create account

        $dbAccount =  ConnectAccount::where('business_id', $req->input('business_id'))->first();
        if (!empty($dbAccount)) {
            $account = $dbAccount;
            $dbOpenerPerson = Person::where('connect_account_id', $dbAccount->id)->where('account_opener', '1')->first();
            if (empty($dbOpenerPerson)) {
                if (!$req->has('file_account_opener_doc_front')) {
                    Utils::error("Account opener front document required", 400);
                }
                if (!$req->has('file_account_opener_doc_back')) {
                    Utils::error("Account opener back document required", 400);
                }
                if (!$req->has('file_account_opener_add_front')) {
                    Utils::error("Account opener additional document required", 400);
                }
            } else {
                $dbAccountOpenerDocFront = ConnectFile::where('person_id', $dbOpenerPerson->id)->where('proof_type', 'document_front')->first();
                if (empty($dbAccountOpenerDocFront) && !$req->hasFile('file_account_opener_doc_front')) {
                    Utils::error("Account opener front document required", 400);
                }
                $dbAccountOpenerDocBack = ConnectFile::where('person_id', $dbOpenerPerson->id)->where('proof_type', 'document_back')->first();
                if (empty($dbAccountOpenerDocBack) && !$req->hasFile('file_account_opener_doc_back')) {
                    Utils::error("Account opener back document required", 400);
                }
                $dbAccountOpenerAddFront = ConnectFile::where('person_id', $dbOpenerPerson->id)->where('proof_type', 'additional_front')->first();
                if (empty($dbAccountOpenerAddFront) && !$req->hasFile('file_account_opener_add_front')) {
                    Utils::error("Account opener additional document required", 400);
                }
            }
            if ($account->business_type == 'company') {
                $dbOwnerPerson = Person::where('connect_account_id', $dbAccount->id)->where('owner', '1')->first();
                if (empty($dbOwnerPerson)) {
                    if (!$req->has('file_account_owner_doc_front')) {
                        Utils::error("Account owner front document required", 400);
                    }
                    if (!$req->has('file_account_owner_doc_back')) {
                        Utils::error("Account owner back document required", 400);
                    }
                    if (!$req->has('file_account_owner_add_front')) {
                        Utils::error("Account owner additional document required", 400);
                    }
                } else {
                    $dbAccountOwnerDocFront = ConnectFile::where('person_id', $dbOwnerPerson->id)->where('proof_type', 'document_front')->first();
                    if (empty($dbAccountOwnerDocFront) && !$req->hasFile('file_account_owner_doc_front')) {
                        Utils::error("Account owner front document required", 400);
                    }
                    $dbAccountOwnerDocBack = ConnectFile::where('person_id', $dbOwnerPerson->id)->where('proof_type', 'document_back')->first();
                    if (empty($dbAccountOwnerDocBack) && !$req->hasFile('file_account_owner_doc_back')) {
                        Utils::error("Account owner back document required", 400);
                    }
                    $dbAccountOwnerAddFront = ConnectFile::where('person_id', $dbOwnerPerson->id)->where('proof_type', 'additional_front')->first();
                    if (empty($dbAccountOwnerAddFront) && !$req->hasFile('file_account_owner_add_front')) {
                        Utils::error("Account owner additional document required", 400);
                    }
                }
            }
        } else {
            $account = new ConnectAccount;
            if (!$req->has('file_account_opener_doc_front')) {
                Utils::error("Account opener front document required", 400);
            }
            if (!$req->has('file_account_opener_doc_back')) {
                Utils::error("Account opener back document required", 400);
            }
            if (!$req->has('file_account_opener_add_front')) {
                Utils::error("Account opener additional document required", 400);
            }

            if ($req->input('business_type') == 'company') {
                if (!$req->has('file_account_owner_doc_front')) {
                    Utils::error("Account owner front document required", 400);
                }
                if (!$req->has('file_account_owner_doc_back')) {
                    Utils::error("Account owner back document required", 400);
                }
                if (!$req->has('file_account_owner_add_front')) {
                    Utils::error("Account owner additional document required", 400);
                }
            }
        }
        $account->fill($req->all());

        $account->tos_ip = $req->ip();
        $account->tos_date = Carbon::today()->unix();
        $account->schedule_interval = 'weekly';
        $account->schedule_weekly_anchor = 'monday';

        $account->save();

        $dbCompany = Company::where('connect_account_id', $account->id)->first();
        if (!empty($dbCompany)) {
            $company = $dbCompany;
        } else {
            $company = new Company();
        }
        $company->fill($req->input('company'));
        $company->connect_account_id = $account->id;
        $company->save();

        // create accountOpener
        $dbPerson = Person::where('connect_account_id', $account->id)->where('email', $req->input('account_opener.email'))->where('account_opener', 1)->first();
        if (!empty($dbPerson))
            $accountOpener = $dbPerson;
        else   $accountOpener = new Person();

        $accountOpener->fill($req->input('account_opener'));
        $accountOpener->company_id = $company->id;
        $accountOpener->connect_account_id = $account->id;
        $accountOpener->account_opener = 1;
        $accountOpener->save();

        $dbExternalAccount = ExternalAccount::where('connect_account_id', $account->id)->first();
        if (!empty($dbExternalAccount))
            $externalAccount = $dbExternalAccount;
        else   $externalAccount = new ExternalAccount();

        $externalAccount->fill($req->input('external_account'));
        $externalAccount->connect_account_id = $account->id;
        $externalAccount->default = 1;
        $externalAccount->save();

        $dbAccountOpenerDocFront = ConnectFile::where('person_id', $accountOpener->id)->where('proof_type', 'document_front')->first();
        if (empty($dbAccountOpenerDocFront) && !$req->hasFile('file_account_opener_doc_front')) {
            Utils::error("Account opener front document required", 400);
        }
        $dbAccountOpenerDocBack = ConnectFile::where('person_id', $accountOpener->id)->where('proof_type', 'document_back')->first();
        if (empty($dbAccountOpenerDocBack) && !$req->hasFile('file_account_opener_doc_back')) {
            Utils::error("Account opener back document required", 400);
        }
        $dbAccountOpenerAddFront = ConnectFile::where('person_id', $accountOpener->id)->where('proof_type', 'additional_front')->first();
        if (empty($dbAccountOpenerAddFront) && !$req->hasFile('file_account_opener_add_front')) {
            Utils::error("Account opener additional document required", 400);
        }

        if ($req->hasFile('file_account_opener_doc_front')) {
            $accountOpenerDocFront = new ConnectFile();
            $accountOpenerDocFrontUploaded = Storage::disk('public')->put('files/connect/', $req->file('file_account_opener_doc_front'));
            $accountOpenerDocFront->file = $req->file('file_account_opener_doc_front')->hashName();
            $accountOpenerDocFront->proof_type = 'document_front';
            $accountOpenerDocFront->purpose = 'identity_document';
            $accountOpenerDocFront->connect_account_id = $account->id;
            $accountOpenerDocFront->person_id = $accountOpener->id;
            $accountOpenerDocFront->save();
            if (!empty($dbAccountOpenerDocFront)) {
                $dbAccountOpenerDocFront->delete();
                $fileName = Storage::disk('public')->path('files/connect/' . $dbAccountOpenerDocFront->file);
                if (file_exists($fileName)) unlink($fileName);
            }
        }

        if ($req->hasFile('file_account_opener_doc_back')) {
            $accountOpenerDocBack = new ConnectFile();
            $accountOpenerDocBackUploaded = Storage::disk('public')->put('files/connect/', $req->file('file_account_opener_doc_back'));
            $accountOpenerDocBack->file = $req->file('file_account_opener_doc_back')->hashName();
            $accountOpenerDocBack->proof_type = 'document_back';
            $accountOpenerDocBack->purpose = 'identity_document';
            $accountOpenerDocBack->connect_account_id = $account->id;
            $accountOpenerDocBack->person_id = $accountOpener->id;
            $accountOpenerDocBack->save();
            if (!empty($dbAccountOpenerDocBack)) {
                $dbAccountOpenerDocBack->delete();
                $fileName = Storage::disk('public')->path('files/connect/' . $dbAccountOpenerDocBack->file);
                if (file_exists($fileName)) unlink($fileName);
            }
        }

        if ($req->hasFile('file_account_opener_add_front')) {
            $accountOpenerAddFront = new ConnectFile();
            $accountOpenerAddFrontUploaded = Storage::disk('public')->put('files/connect/', $req->file('file_account_opener_add_front'));
            $accountOpenerAddFront->file = $req->file('file_account_opener_add_front')->hashName();
            $accountOpenerAddFront->proof_type = 'additional_front';
            $accountOpenerAddFront->purpose = 'identity_document';
            $accountOpenerAddFront->connect_account_id = $account->id;
            $accountOpenerAddFront->person_id = $accountOpener->id;
            $accountOpenerAddFront->save();

            if (!empty($dbAccountOpenerAddFront)) {
                $dbAccountOpenerAddFront->delete();
                $fileName = Storage::disk('public')->path('files/connect/' . $dbAccountOpenerAddFront->file);
                if (file_exists($fileName)) unlink($fileName);
            }
        }

        // create owner
        if ($account->business_type == 'company') {
            $dbOwnerPerson = Person::where('connect_account_id', $account->id)->where('email', $req->input('account_owner.email'))->where('owner', 1)->first();
            if (!empty($dbOwnerPerson)) {
                $accountOwner = $dbOwnerPerson;
            } else {
                $accountOwner = new Person();
            }
            $accountOwner->fill($req->input('account_owner'));
            $accountOwner->company_id = $company->id;
            $accountOwner->connect_account_id = $account->id;
            $accountOwner->owner = 1;
            $accountOwner->save();

            $dbAccountOwnerDocFront = ConnectFile::where('person_id', $accountOwner->id)->where('proof_type', 'document_front')->first();
            if (empty($dbAccountOwnerDocFront) && !$req->hasFile('file_account_owner_doc_front')) {
                Utils::error("Account owner front document required", 400);
            }
            $dbAccountOwnerDocBack = ConnectFile::where('person_id', $accountOwner->id)->where('proof_type', 'document_back')->first();
            if (empty($dbAccountOwnerDocBack) && !$req->hasFile('file_account_owner_doc_back')) {
                Utils::error("Account owner back document required", 400);
            }
            $dbAccountOwnerAddFront = ConnectFile::where('person_id', $accountOwner->id)->where('proof_type', 'additional_front')->first();
            if (empty($dbAccountOwnerAddFront) && !$req->hasFile('file_account_owner_add_front')) {
                Utils::error("Account owner additional document required", 400);
            }

            if ($req->hasFile('file_account_owner_doc_front')) {
                $accountOwnerDocFront = new ConnectFile();
                $accountOwnerDocFrontUploaded = Storage::disk('public')->put('files/connect/', $req->file('file_account_owner_doc_front'));
                $accountOwnerDocFront->file = $req->file('file_account_owner_doc_front')->hashName();
                $accountOwnerDocFront->proof_type = 'document_front';
                $accountOwnerDocFront->purpose = 'identity_document';
                $accountOwnerDocFront->connect_account_id = $account->id;
                $accountOwnerDocFront->person_id = $accountOwner->id;
                $accountOwnerDocFront->save();
                if (!empty($dbAccountOwnerDocFront)) {
                    $dbAccountOwnerDocFront->delete();
                    $fileName = Storage::disk('public')->path('files/connect/' . $dbAccountOwnerDocFront->file);
                    if (file_exists($fileName)) unlink($fileName);
                }
            }

            if ($req->hasFile('file_account_owner_doc_back')) {
                $accountOwnerDocBack = new ConnectFile();
                $accountOwnerDocBackUploaded = Storage::disk('public')->put('files/connect/', $req->file('file_account_owner_doc_back'));
                $accountOwnerDocBack->file = $req->file('file_account_owner_doc_back')->hashName();
                $accountOwnerDocBack->proof_type = 'document_back';
                $accountOwnerDocBack->purpose = 'identity_document';
                $accountOwnerDocBack->connect_account_id = $account->id;
                $accountOwnerDocBack->person_id = $accountOwner->id;
                $accountOwnerDocBack->save();
                if (!empty($dbAccountOwnerDocBack)) {
                    $dbAccountOwnerDocBack->delete();
                    $fileName = Storage::disk('public')->path('files/connect/' . $dbAccountOwnerDocBack->file);
                    if (file_exists($fileName)) unlink($fileName);
                }
            }

            if ($req->hasFile('file_account_owner_add_front')) {
                $accountOwnerAddFront = new ConnectFile();
                $accountOwnerAddFrontUploaded = Storage::disk('public')->put('files/connect/', $req->file('file_account_owner_add_front'));
                $accountOwnerAddFront->file = $req->file('file_account_owner_add_front')->hashName();
                $accountOwnerAddFront->proof_type = 'additional_front';
                $accountOwnerAddFront->purpose = 'identity_document';
                $accountOwnerAddFront->connect_account_id = $account->id;
                $accountOwnerAddFront->person_id = $accountOwner->id;
                $accountOwnerAddFront->save();

                if (!empty($dbAccountOwnerAddFront)) {
                    $dbAccountOwnerAddFront->delete();
                    $fileName = Storage::disk('public')->path('files/connect/' . $dbAccountOwnerAddFront->file);
                    if (file_exists($fileName)) unlink($fileName);
                }
            }
        }



        return $account;
    }

    public function generate(Request $req, $id)
    {

        $stripe = self::$stripe;

        $account = ConnectAccount::find($id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $business = Business::find($account->business_id);

        $company = Company::where('connect_account_id', $account->id)->latest()->first();
        $account_opener = Person::where('connect_account_id', $account->id)->where('account_opener', 1)->latest()->first();
        $account_owner = Person::where('connect_account_id', $account->id)->where('owner', 1)->latest()->first();
        $external_account = ExternalAccount::where('connect_account_id', $account->id)->latest()->first();
        $account_opener_dob = Carbon::parse($account_opener->dob);
        $account_opener_doc_front = ConnectFile::where('person_id', $account_opener->id)->where('proof_type', 'document_front')->latest()->first();
        $account_opener_doc_back = ConnectFile::where('person_id', $account_opener->id)->where('proof_type', 'document_back')->latest()->first();
        $account_opener_additional_front = ConnectFile::where('person_id', $account_opener->id)->where('proof_type', 'additional_front')->latest()->first();
        $account_owner_doc_front = ConnectFile::where('person_id', $account_owner->id)->where('proof_type', 'document_front')->latest()->first();
        $account_owner_doc_back = ConnectFile::where('person_id', $account_owner->id)->where('proof_type', 'document_back')->latest()->first();
        $account_owner_additional_front = ConnectFile::where('person_id', $account_owner->id)->where('proof_type', 'additional_front')->latest()->first();
        $isFresh = ($account->s_account_id == null) ? true : false;

        $stripeAccountParams = [
            'email' => $company->email,
            'tos_acceptance' => [
                'ip' => $req->ip(),
                'date' => Carbon::today()->unix()
            ],
            'business_profile' => [
                'name' => $company->name,
                'url' => $company->url,
                'mcc' => $company->mcc,
                'support_phone' => $company->phone_number,
            ],
            'business_type' => $account->business_type,
            'company' => [
                'name' => $company->name,
                'phone' => $company->phone_number,
                'tax_id' => $company->tax_id,
                'address' => [
                    'line1' => $company->line1,
                    'city' => $company->city,
                    'state' => $company->state,
                    'postal_code' => $company->postal_code,
                ],
            ],

            'external_account' => [
                "object" => "bank_account",
                'country' => $external_account->country,
                'currency' => $external_account->currency,
                'account_holder_type' => $external_account->account_holder_type,
                'account_holder_name' => $external_account->account_holder_name,
                'account_number' => $external_account->account_number,
                'routing_number' => $external_account->routing_number,
            ]
        ];
        if ($account->s_account_id == null) {
            $stripeAccountParams['type'] = 'custom';
            $stripeAccountParams['country'] = $account->country;
            $stripeAccountParams['capabilities'] = [
                'card_payments' => [
                    'requested' => true,
                ],
                'transfers' => [
                    'requested' => true,
                ],
            ];
        };

        if (!$isFresh) {
            $stripe_account = $stripe->accounts->update($account->s_account_id, $stripeAccountParams);
        } else {
            $stripe_account = $stripe->accounts->create($stripeAccountParams);

            $external_account->s_account_id = $stripe_account->id;
            $external_account->s_bank_account_id = $stripe_account->external_accounts->data[0]->id;
            $external_account->save();

            $business->s_account_id = $stripe_account->id;
            $business->save();
        }

        if ($account->s_account_id == null) {
            $account->s_account_id = $stripe_account->id;
            if (isset($stripe_account->keys)) {
                $account->public_key = $stripe_account->keys->publishable;
                $account->private_key = $stripe_account->keys->secret;
            }
            $account->save();
        }
        // Re-init object
        $key = self::$key;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        Stripe::setMaxNetworkRetries(4);

        // Create Persons
        if ($account->business_type == 'individual') {
            $stripeAccountParams = [];
            $stripeAccountParams['individual'] = [
                'first_name' => $account_opener->first_name,
                'last_name' => $account_opener->last_name,
                'email' => $account_opener->email,
                'phone' => $account_opener->phone_number,
                'id_number' => $account_opener->id_number,
                'address' => [
                    'line1' => $account_opener->line1,
                    'city' => $account_opener->city,
                    'country' => $account->country,
                    'state' => $account_opener->state,
                    'postal_code' => $account_opener->postal_code,
                ],

                'dob' => [
                    'day' => $account_opener_dob->day,
                    'month' => $account_opener_dob->month,
                    'year' => $account_opener_dob->year,
                ],
            ];
            $stripe_account = $stripe->accounts->update($account->s_account_id, $stripeAccountParams);

            $account_opener->s_person_id = $stripe_account->individual->id;
            $account_opener->s_account_id = $stripe_account->id;
            $account_opener->save();

            $openerDocFrontFile = $this->uploadFile($stripe, $stripe_account->id, $account_opener_doc_front);
            $account_opener_doc_front->s_person_id = $stripe_account->individual->id;
            $account_opener_doc_front->s_file_id = $openerDocFrontFile->id;
            $account_opener_doc_front->s_account_id = $stripe_account->id;
            $account_opener_doc_front->save();

            $openerDocBackFile = $this->uploadFile($stripe, $stripe_account->id, $account_opener_doc_back);
            $account_opener_doc_back->s_person_id = $stripe_account->individual->id;
            $account_opener_doc_back->s_file_id = $openerDocBackFile->id;
            $account_opener_doc_back->s_account_id = $stripe_account->id;
            $account_opener_doc_back->save();

            $openerAdditionalFrontFile = $this->uploadFile($stripe, $stripe_account->id, $account_opener_additional_front);
            $account_opener_additional_front->s_person_id = $stripe_account->individual->id;
            $account_opener_additional_front->s_file_id = $openerAdditionalFrontFile->id;
            $account_opener_additional_front->s_account_id = $stripe_account->id;
            $account_opener_additional_front->save();

            $personOpenerParams = [];
            $personOpenerParams['verification']['additional_document']['front'] = $account_opener_additional_front->s_file_id;
            $personOpenerParams['verification']['document']['front'] = $account_opener_doc_front->s_file_id;
            $personOpenerParams['verification']['document']['back'] = $account_opener_doc_back->s_file_id;

            $personOwner = $stripe->accounts->updatePerson($stripe_account->id, $account_opener->s_person_id, $personOpenerParams);
        }
        if ($account->business_type == 'company') {

            // Create Representativve
            $accountOpenerParams = [];
            $accountOpenerParams = [
                'first_name' => $account_opener->first_name,
                'last_name' => $account_opener->last_name,
                'email' => $account_opener->email,
                'phone' => $account_opener->phone_number,
                'id_number' => $account_opener->id_number,
                'address' => [
                    'line1' => $account_opener->line1,
                    'city' => $account_opener->city,
                    'state' => $account_opener->state,
                    'country' => $account->country,
                    'postal_code' => $account_opener->postal_code,
                ],
                'relationship' => [
                    'representative' => true,
                    'executive' => true,
                    'title' => $account_opener->title,
                ],
                'dob' => [
                    'day' => $account_opener_dob->day,
                    'month' => $account_opener_dob->month,
                    'year' => $account_opener_dob->year,
                ],
            ];
            $personOpener = $stripe->accounts->createPerson($stripe_account->id, $accountOpenerParams);
            $account_opener->s_person_id = $personOpener->id;
            $account_opener->s_account_id = $stripe_account->id;
            $account_opener->save();

            $openerDocFrontFile = $this->uploadFile($stripe, $stripe_account->id, $account_opener_doc_front);
            $account_opener_doc_front->s_person_id = $personOpener->id;
            $account_opener_doc_front->s_file_id = $openerDocFrontFile->id;
            $account_opener_doc_front->s_account_id = $stripe_account->id;
            $account_opener_doc_front->save();

            $openerDocBackFile = $this->uploadFile($stripe, $stripe_account->id, $account_opener_doc_back);
            $account_opener_doc_back->s_person_id = $personOpener->id;
            $account_opener_doc_back->s_file_id = $openerDocBackFile->id;
            $account_opener_doc_back->s_account_id = $stripe_account->id;
            $account_opener_doc_back->save();

            $openerAdditionalFrontFile = $this->uploadFile($stripe, $stripe_account->id, $account_opener_additional_front);
            $account_opener_additional_front->s_person_id = $personOpener->id;
            $account_opener_additional_front->s_file_id = $openerAdditionalFrontFile->id;
            $account_opener_additional_front->s_account_id = $stripe_account->id;
            $account_opener_additional_front->save();

            $personOpenerParams = [];

            $personOpenerParams['verification']['additional_document']['front'] = $account_opener_additional_front->s_file_id;
            $personOpenerParams['verification']['document']['front'] = $account_opener_doc_front->s_file_id;
            $personOpenerParams['verification']['document']['back'] = $account_opener_doc_back->s_file_id;

            $personOpener = $stripe->accounts->updatePerson($stripe_account->id, $account_opener->s_person_id, $personOpenerParams);

            // Create Owner Person
            $ownerDOB = Carbon::parse($account_owner->dob);
            // Create Person without files
            $personOwnerParams = [
                'first_name' => $account_owner->first_name,
                'last_name' => $account_owner->last_name,
                'email' => $account_owner->email,
                'phone' => $account_owner->phone_number,
                'id_number' => $account_owner->id_number,
                'address' => [
                    'line1' => $account_owner->line1,
                    'city' => $account_owner->city,
                    'state' => $account_owner->state,
                    'country' => $account->country,
                    'postal_code' => $account_owner->postal_code,
                ],
                'dob' => [
                    'day' => $ownerDOB->day,
                    'month' => $ownerDOB->month,
                    'year' => $ownerDOB->year,
                ],
                'relationship' => [
                    'title' => $account_owner->title,
                    'owner' => true,
                    'executive' => true
                ]
            ];
            $personOwner = $stripe->accounts->createPerson($stripe_account->id, $personOwnerParams);
            $account_owner->s_person_id = $personOwner->id;
            $account_owner->s_account_id = $stripe_account->id;
            $account_owner->save();

            // Now upload files
            $ownerDocFrontFile = $this->uploadFile($stripe, $stripe_account->id, $account_owner_doc_front);
            $account_owner_doc_front->s_person_id = $personOwner->id;
            $account_owner_doc_front->s_file_id = $ownerDocFrontFile->id;
            $account_owner_doc_front->s_account_id = $stripe_account->id;
            $account_owner_doc_front->save();

            $ownerDocBackFile = $this->uploadFile($stripe, $stripe_account->id, $account_owner_doc_back);
            $account_owner_doc_back->s_person_id = $personOwner->id;
            $account_owner_doc_back->s_file_id = $ownerDocBackFile->id;
            $account_owner_doc_back->s_account_id = $stripe_account->id;
            $account_owner_doc_back->save();

            $ownerAdditionalFrontFile = $this->uploadFile($stripe, $stripe_account->id, $account_owner_additional_front);
            $account_owner_additional_front->s_person_id = $personOwner->id;
            $account_owner_additional_front->s_file_id = $ownerAdditionalFrontFile->id;
            $account_owner_additional_front->s_account_id = $stripe_account->id;
            $account_owner_additional_front->save();

            $personOwnerParams = [];
            $personOwnerParams['verification']['additional_document']['front'] = $account_owner_additional_front->s_file_id;
            $personOwnerParams['verification']['document']['front'] = $account_owner_doc_front->s_file_id;
            $personOwnerParams['verification']['document']['back'] = $account_owner_doc_back->s_file_id;

            $personOwner = $stripe->accounts->updatePerson($stripe_account->id, $account_owner->s_person_id, $personOwnerParams);

            $params = [];
            $params['company']['executives_provided'] = true;
            $params['company']['owners_provided'] = true;
            $params['company']['directors_provided'] = true;
            $stripe_account = $stripe->accounts->update($stripe_account->id, $params);
        }

        return $stripe_account;
    }
    public function uploadFile($stripe, $stripe_id, $uploadableFile)
    {
        $uploadedStripeFile = $stripe->files->create([
            'purpose' => 'identity_document',
            'file' => fopen(Storage::disk('public')->path('files/connect/' . $uploadableFile->file), 'r'),
        ], [$stripe_id]);
        return $uploadedStripeFile;
    }
    /**
     * Show account.
     * GET /connect/accounts/{id}
     */
    public function show(Request $req, $id)
    {
        $account = ConnectAccount::find($id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $account->company = Company::where('connect_account_id', $account->id)->first();
        $account->account_opener = Person::where('connect_account_id', $account->id)->where('account_opener', 1)->first();
        $account->account_owner = Person::where('connect_account_id', $account->id)->where('owner', 1)->first();
        $account->external_account = ExternalAccount::where('connect_account_id', $account->id)->first();
        return $account;
    }
    /**
     * GET /connect/accounts/business/{business_id}
     * */
    public function show_by_business(Request $req, $business_id)
    {
        $account = ConnectAccount::where('business_id', $business_id)->first();
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);

        $stripe = self::$stripe;

        try {
            if ($account->s_account_id != null) {
                $account->account_balace = $stripe->balance->retrieve([], ['stripe_account' => $account->s_account_id]);
                $stripe_account = $stripe->accounts->retrieve($account->s_account_id);
                $account->stripe_connect_account = $stripe_account;
                $exIds = [];
                foreach ($stripe_account->external_accounts->data as $bankAccount) {
                    $dbExternalAccount = ExternalAccount::where('s_bank_account_id', $bankAccount->id)->latest()->first();
                    // if (empty($dbExternalAccount)) {
                    //     $dbExternalAccount = ExternalAccount::where('s_account_id', $bankAccount->account)->whereRaw('account_number LIKE ?', ["%$bankAccount->last4%"])->where('account_holder_name', $bankAccount->account_holder_name)->latest()->first();
                    // }
                    if (empty($dbExternalAccount)) {
                        $external_account = new ExternalAccount();
                        $external_account->country = $bankAccount->country;
                        $external_account->currency = $bankAccount->currency;
                        $external_account->account_holder_name = $bankAccount->account_holder_name;
                        $external_account->account_holder_type = $bankAccount->account_holder_type;
                        $external_account->account_number = $bankAccount->last4;
                        $external_account->routing_number = $bankAccount->routing_number;
                    } else {
                        $external_account = clone ($dbExternalAccount);
                    }
                    $external_account->connect_account_id = $account->id;
                    $external_account->business_id = $business_id;

                    $external_account->s_account_id = $bankAccount->account;
                    $external_account->s_bank_account_id = $bankAccount->id;
                    $external_account->default = $bankAccount->default_for_currency;

                    $external_account->save();
                    $exIds[] = $external_account->id;
                }

                $payout_mode = $stripe_account->settings->payouts->schedule->interval;
                if ($payout_mode != 'manual') {
                    $payout_mode = 'Auto';
                } else {
                    $payout_mode = 'Manual';
                }
                $business = Business::find($business_id);
                if (!empty($business)) {
                    $business->payout_mode = $payout_mode;
                    $business->save();
                }

                if (!empty($exIds)) {
                    $deleteExternalAccounts = ExternalAccount::where('business_id', $business_id)->whereNotIn('id', $exIds)->delete();
                }
            }
        } catch (Exception $e) {
        }

        $account->company = Company::where('connect_account_id', $account->id)->first();
        $account->account_opener = Person::where('connect_account_id', $account->id)->where('account_opener', 1)->first();
        $account->account_owner = Person::where('connect_account_id', $account->id)->where('owner', 1)->first();
        $account->external_accounts = ExternalAccount::where('connect_account_id', $account->id)->orderBy('default', 'DESC')->get();
        return $account;
    }

    /**
     * Update account.
     * POST /connect/accounts/{id}
     */
    public function update(Request $req, $id)
    {
        $account = ConnectAccount::find($id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);

        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'business_type' => 'required|in:company,individual',
            'country' => 'required|max:2',
        ]);

        // update account
        if ($req->has('business_id')) $account->business_id = $req->input('business_id');
        if ($req->has('business_type')) $account->business_type = $req->input('business_type');

        $account->save();

        return $account;
    }

    /**
     * Enable Customer show external-account.
     * POST /connect/accounts/{id}/enable-external-customer
     */
    public function enable_external_customer(Request $req, $id)
    {
        $external_account = ExternalAccount::find($id);
        if (empty($external_account)) Utils::error('External account not found!', 404);
        $external_account->is_customer = 1;
        $external_account->save();
        return $external_account;
    }

    /**
     * Disable Customer show external-account.
     * POST /connect/accounts/{id}/disable-external-customer
     */
    public function disable_external_customer(Request $req, $id)
    {
        $external_account = ExternalAccount::find($id);
        if (empty($external_account)) Utils::error('External account not found!', 404);
        $external_account->is_customer = 0;
        $external_account->save();
        return $external_account;
    }

    /**
     * Create Bank Account
     * POST /connect/accounts/{account_id}/bank-account
     */
    public function create_bank_account(Request $req, $account_id)
    {
        $account = ConnectAccount::find($account_id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $this->validate($req, [
            'country' => 'required',
            'currency' => 'required',
            'account_holder_type' => 'required',
            'account_holder_name' => 'required',
            'account_number' => 'required',
            'routing_number' => 'required',
            'description' => 'string',
            'statement_descriptor' => 'string',
            'connect_account_id' => 'required',
        ]);
        $business = Business::find($account->business_id);
        if (empty($business)) Utils::error("Business not found", 404);

        $external_account_params['external_account'] = [
            'object' => 'bank_account',
            'country' => $req->input('country'),
            'currency' => $req->input('currency'),
            'account_holder_type' => $req->input('account_holder_type'),
            'account_holder_name' => $req->input('account_holder_name'),
            'account_number' => $req->input('account_number'),
            'routing_number' => $req->input('routing_number'),
        ];

        $stripe = self::$stripe;

        $external_account = $stripe->accounts->createExternalAccount($account->s_account_id, $external_account_params);

        $bankAccount = new ExternalAccount();
        $bankAccount->fill($req->all());
        $bankAccount->business_id = $business->id;
        $bankAccount->default = 0;
        $bankAccount->s_bank_account_id = $external_account->id;
        $bankAccount->s_account_id = $external_account->account;
        $bankAccount->save();
        return $account;
    }
    /**
     * Update Bank Account
     * POST /connect/accounts/{account_id}/bank-account/{id}
     */
    public function update_bank_account(Request $req, $account_id, $id)
    {
        $account = ConnectAccount::find($account_id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $this->validate($req, [
            'country' => 'required',
            'currency' => 'required',
            's_account_id' => 'required',
            's_bank_account_id' => 'required',
            'account_holder_type' => 'required',
            'account_holder_name' => 'required',
            'account_number' => 'required',
            'routing_number' => 'required',
            'description' => 'string',
            'statement_descriptor' => 'string',
            'connect_account_id' => 'required',
            'default' => 'required|boolean'
        ]);
        $business = Business::find($account->business_id);
        if (empty($business)) Utils::error("Business not found", 404);
        $bankAccount = ExternalAccount::find($id);
        if (empty($bankAccount)) Utils::error("Bank account not found", 404);
        if ($req->has('country')) $bankAccount->country = $req->input('country');
        if ($req->has('currency')) $bankAccount->currency = $req->input('currency');
        if ($req->has('account_holder_type')) $bankAccount->account_holder_type = $req->input('account_holder_type');
        if ($req->has('account_holder_name')) $bankAccount->account_holder_name = $req->input('account_holder_name');
        if ($req->has('account_number')) $bankAccount->account_number = $req->input('account_number');
        if ($req->has('routing_number')) $bankAccount->routing_number = $req->input('routing_number');
        if ($req->has('description')) $bankAccount->description = $req->input('description');
        if ($req->has('statement_descriptor')) $bankAccount->statement_descriptor = $req->input('statement_descriptor');
        $bankAccount->default = $req->input('default');
        $bankAccount->save();

        $external_account_params = [
            'account_holder_type' => $bankAccount->account_holder_type,
            'account_holder_name' => $bankAccount->account_holder_name,
            'default_for_currency' => $bankAccount->default ? "true" : "false"
        ];

        if ($bankAccount->default) {
            ExternalAccount::where('id', '!=', $bankAccount->id)->where('business_id', $account->business_id)->update(['default' => '0']); // where('s_account_id', $bankAccount->s_account_id)->where('connect_account_id', $account->id)->
        }

        $stripe = self::$stripe;

        $external_account = $stripe->accounts->updateExternalAccount($account->s_account_id, $bankAccount->s_bank_account_id, $external_account_params);
        $bankAccount->s_bank_account_id = $external_account->id;
        $bankAccount->save();
        return $account;
    }

    /**
     * Add Connect Account ID.
     * POST /connect/accounts/business/
     */
    public function add_connect_id(Request $req)
    {
        $this->validate($req, [
            's_account_id' => 'required|string',
            'business_id' => 'required|string'
        ]);
        $business = Business::find($req->input('business_id'));
        if (empty($business)) Utils::error("Business not found", 400);
        $account = ConnectAccount::where('business_id', $business->id)->first();
        if (!empty($account)) Utils::error('ConnectAccount already generated!', 404);

        $stripe = self::$stripe;
        $stripe_account = $stripe->accounts->retrieve($req->input('s_account_id'));

        $business->s_account_id = $req->input('s_account_id');
        $business->save();

        ConnectAccount::where('business_id', $business->id)->delete();

        $connectedAccount = new ConnectAccount();
        $connectedAccount->s_account_id = $req->input('s_account_id');
        $connectedAccount->business_id = $req->input('business_id');
        $connectedAccount->business_type = $stripe_account->business_type;
        $connectedAccount->tos_date = $stripe_account->tos_acceptance->date;
        $connectedAccount->tos_ip = $stripe_account->tos_acceptance->ip;
        $connectedAccount->country = $stripe_account->country;
        $connectedAccount->schedule_interval = 'weekly';
        $connectedAccount->schedule_weekly_anchor = 'monday';
        $connectedAccount->save();

        $company  = new Company();
        $company->s_account_id = $req->input('s_account_id');
        $company->connect_account_id = $connectedAccount->id;
        $company->name = $stripe_account->business_profile->name;
        $company->email = $stripe_account->email;
        $company->url = $stripe_account->business_profile->url;
        $company->mcc = ($stripe_account->business_profile->mcc != null) ? $stripe_account->business_profile->mcc : '5814';
        $company->phone_number = $stripe_account->company->phone;
        $company->line1 = $stripe_account->company->address->line1;
        $company->line2 = $stripe_account->company->address->line2;
        $company->postal_code = $stripe_account->company->address->postal_code;
        $company->city = $stripe_account->company->address->city;
        $company->state = $stripe_account->company->address->state;
        $company->save();

        foreach ($stripe_account->external_accounts->data as $bankAccount) {
            $external_account = new ExternalAccount();
            $external_account->connect_account_id = $connectedAccount->id;
            $external_account->s_account_id = $business->s_account_id;
            $external_account->business_id = $business->id;

            $external_account->default = $bankAccount->default_for_currency;
            $external_account->s_bank_account_id = $bankAccount->id;
            $external_account->country = $bankAccount->country;
            $external_account->currency = $bankAccount->currency;
            $external_account->account_holder_name = ($bankAccount->account_holder_name != null) ? $bankAccount->account_holder_name : $stripe_account->business_profile->name;
            $external_account->account_holder_type = ($bankAccount->account_holder_type != null) ? $bankAccount->account_holder_type : $stripe_account->business_type;
            $external_account->account_number = $bankAccount->last4;
            $external_account->routing_number = $bankAccount->routing_number;
            $external_account->save();
        }

        return $business;
    }

    /**
     * Change Payout Mode.
     * POST /connect/accounts/{account_id}/payout-settings
     */
    public function payout_settings(Request $req, $account_id)
    {
        $account = ConnectAccount::find($account_id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $business = Business::find($account->business_id);
        if (empty($business)) Utils::error("Business not found", 404);
        $this->validate($req, [
            'schedule_interval' => 'in:manual,daily,weekly,monthly',
            'schedule_delay_days' => 'gt:6'
        ]);
        $stripe = self::$stripe;

        $interval = $req->input('schedule_interval');
        $delay_days = $req->input('schedule_delay_days');
        $weekly_anchor = $req->input('schedule_weekly_anchor');
        $monthly_anchor = $req->input('schedule_monthly_anchor');
        $payoutParams['settings']['payouts']['schedule']['interval'] = $interval;

        if ($interval != 'manual') {
            $payoutParams['settings']['payouts']['schedule']['delay_days'] = $delay_days;
            if ($interval == 'monthly') {
                $payoutParams['settings']['payouts']['schedule']['monthly_anchor'] = $monthly_anchor;
            }
            if ($interval == 'weekly') {
                $payoutParams['settings']['payouts']['schedule']['weekly_anchor'] = $weekly_anchor;
            }
        }
        $stripe_account = $stripe->accounts->update($account->s_account_id, $payoutParams);
        $account->schedule_interval = $interval;
        $account->schedule_delay_days = $delay_days;
        $account->schedule_weekly_anchor = $weekly_anchor;
        $account->schedule_monthly_anchor = $monthly_anchor;
        $account->save();
        return $this->show_by_business($req, $business->id);
    }
    /**
     * Create Payout .
     * POST /connect/accounts/{account_id}/payout
     */
    public function payout(Request $req, $account_id)
    {
        $account = ConnectAccount::find($account_id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $business = Business::find($account->business_id);
        if (empty($business)) Utils::error("Business not found", 404);
        $this->validate($req, [
            'amount' => 'gt:0' // human amount
        ]);
        $amount = $req->input('amount') * 100;
        $stripe = self::$stripe;
        $payout = $stripe->payouts->create([
            'amount' => $amount,
            'method' => 'standard',
            'statement_descriptor' => 'Payout of ' . $business->name,
            'currency' => $business->country->currency_code
        ], [
            'stripe_account' => $account->s_account_id
        ]);
        return $this->show_by_business($req, $business->id);
    }
    /**
     * Create Instant Payout .
     * POST /connect/accounts/{account_id}/instant-payout
     */
    public function instant_payout(Request $req, $account_id)
    {
        $stripe = self::$stripe;

        $account = ConnectAccount::find($account_id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $business = Business::find($account->business_id);
        if (empty($business)) Utils::error("Business not found", 404);
        $this->validate($req, [
            'amount' => 'gt:0', // human amount
        ]);
        $amount = $req->input('amount');

        $calculatedFee = $this->calculateFee($amount, $business->business_commissions->instant_transfer_fee, $business->business_commissions->instant_transfer_fee_type);
        $transferrableAmount = $this->convertToDecimal($amount - $calculatedFee);

        $defaultBankAccount = ExternalAccount::where('business_id', $business->id)->where('default', '1')->latest()->first();
        if (empty($defaultBankAccount)) Utils::error("No payout method found or no default payout account!", 400);
        $stripeExternalAccount = $stripe->accounts->retrieveExternalAccount($account->s_account_id, $defaultBankAccount->s_bank_account_id);
        if (!isset($stripeExternalAccount->available_payout_methods) || !in_array('instant', $stripeExternalAccount->available_payout_methods)) {
            Utils::error("No method available to process instant payout.", 400);
        }
        // We are payout to business
        $payout = $stripe->payouts->create([
            'amount' => $transferrableAmount * 100,
            'method' => 'instant',
            'statement_descriptor' => 'Payout of ' . $business->name,
            'currency' => $business->country->currency_code
        ], [
            'stripe_account' => $account->s_account_id
        ]);

        if ($calculatedFee > 0.30) {
            //We will debit our fee for instant payout from bussiness connect account.
            $charge = $stripe->charges->create([
                'amount'   => $calculatedFee * 100,
                'currency' => 'gbp',
                'source' => $business->s_account_id,
                'description' => "Instant payout fee for " . $business->name,
                'statement_descriptor' => $business->s_account_id
            ]);
        }

        return $this->show_by_business($req, $business->id);
    }
    /**
     * Delete account by id.
     * DELETE /connect/accounts/{id}
     */
    public function delete(Request $req, $id)
    {
        $account = ConnectAccount::find($id);
        if (empty($account)) Utils::error('ConnectAccount not found!', 404);
        $account->delete();
        return $account;
    }

    /**
     * Delete external account by id.
     * DELETE /connect/accounts/{id}/external-accounts
     */
    public function delete_external(Request $req, $id)
    {
        $external_account = ExternalAccount::find($id);
        if (empty($external_account)) Utils::error('External account not found!', 404);
        $business = Business::find($external_account->business_id);
        if (empty($business)) Utils::error('Business not found!', 404);
        try {
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
            }

            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
            $stripe->accounts->deleteExternalAccount(
                $business->s_account_id,
                $external_account->s_bank_account_id,
                []
            );
        } catch (Exception $e) {
            // Utils::error($e, 404);
        }

        $external_account->delete();
        return $external_account;
    }

    public function debit_recurring_fees(Request $req)
    {
        $connectedAccountBusinesses = Business::where('connect_service', '1')->whereNotNull('s_account_id')->where('disabled', '0')->get();
        if ($connectedAccountBusinesses->isEmpty()) {
            if ($req->has('job')) {
                return null;
            } else {
                Utils::error("No connect businesses found", 400);
            }
        }
        $stripe = self::$stripe;

        $response = [];
        foreach ($connectedAccountBusinesses as $business) {
            try {
                $connectedAccount = $stripe->accounts->retrieve($business->s_account_id);
                if (!empty($connectedAccount)) {
                    $weeklyAmount = $business->business_commissions->connect_weekly_fee;
                    $monthlyAmount = $business->business_commissions->connect_monthly_fee;

                    $created = $connectedAccount->created;
                    $startDate = Carbon::parse($business->joining_date)->format('Ymd');
                    $monthlyOccuranceDate = $this->nextDate($startDate, 28, 'Ymd');
                    $weeklyOccuranceDate = $this->nextDate($startDate, 7, 'Ymd');
                    $today = Carbon::now()->format('Ymd');
                    $carbon = Carbon::today();
                    $normal = 0;
                    $archived = 0;

                    $currentDayOfWeek = $carbon->dayOfWeek;
                    $miscInvoices = MiscInvoice::where('payment_mode', 'percentage')->where('business_id', $business->id)->where('status', '!=', 'Paid')->get();
                    $miscInvoicesAmount = 0;
                    if (!$miscInvoices->isEmpty()) {
                        foreach ($miscInvoices as $invoice) {
                            $total_paid_amount = $invoice->total_paid_amount;
                            if (!empty($payoutInvoice))
                                $total_paid_amount = -1 * Passbook::where('business_id', $business->id)->where('entity_id', $invoice->id)->where('entity', 'misc_invoice')->where('ref_entity_id', '!=', $payoutInvoice->id)->where('ref_entity', '!=', 'payout_invoice')->sum('amount');
                            $payableAmount = $invoice->grand_total - $total_paid_amount;
                            if ($payableAmount > 0) {
                                $actualEmi = $this->calculateFee($invoice->grand_total, $invoice->percentage, 'percentage');
                                if ($actualEmi < $payableAmount) {
                                    $actualEmi = $this->calculateFee($payableAmount, $invoice->percentage, 'percentage');
                                } else {
                                    $actualEmi = $payableAmount;
                                }
                                $miscInvoicesAmount = $miscInvoicesAmount + $actualEmi;
                                $invoice['emi_amount'] = $actualEmi;
                            } else {
                                $invoice['emi_amount'] = 0;
                            }
                        };
                    }

                    if ($miscInvoicesAmount > 0 && $business->balance >= $miscInvoicesAmount) {

                        if (!empty($business)) {
                            $description = "Miscellaneous Invoice for $business->name";
                        } else {
                            $description = "Miscellaneous Invoice for $business->s_account_id";
                        }
                        $charge = $stripe->charges->create([
                            'amount'   => $miscInvoicesAmount * 100,
                            'currency' => 'gbp',
                            'source' => $business->s_account_id,
                            'description' => $description,
                            'statement_descriptor' => $business->s_account_id
                        ]);

                        foreach ($miscInvoices as $invoice) {
                            $passbook = new Passbook;
                            $passbook->business_id = $business->id;
                            $passbook->entity_id = $invoice->id;
                            $passbook->entity = "misc_invoice";
                            $passbook->amount =  $invoice->emi_amount;
                            $passbook->date = Carbon::today();
                            $passbook->from_date = Carbon::today();
                            $passbook->to_date = Carbon::today();
                            $passbook->description = "Debited for Miscellaneous invoice #$invoice->id dated on " . Carbon::today()->toDateString();
                            $passbook->save();
                        }
                    }

                    // if ($weeklyAmount == 0 && $monthlyAmount == 0) continue;

                    if ($weeklyAmount >= 0 && $business->balance >= $weeklyAmount && $business->business_commissions->auto_charge_weekly == 'yes') {
                        $carbon = Carbon::today(); // ->subDays(7)
                        $currentDayOfWeek = $carbon->dayOfWeek;
                        if ($currentDayOfWeek == 2) {
                            $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                            $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
                        }

                        // $from_date = Carbon::today()->subDays(1)->startOfDay()->format('Y-m-d');
                        // $to_date = Carbon::today()->endOfDay()->format('Y-m-d');

                        if ($from_date != '' && $to_date != '') {
                            $business->append('default_bank_detail');
                            $business->append('default_paypal_detail');
                            $business->makeHidden(['business_commissions']);
                            $req->merge(['from_date' => $from_date]);
                            $req->merge(['to_date' => $to_date]);
                            $data = $this->setupTransactions($req, $business, $business->id, $from_date, $to_date);

                            if (!empty($data['normal'])) {
                                $normal = $data['normal']['net_fee'];
                            }
                            if (!empty($data['archived'])) {
                                $archived = $data['archived']['net_fee'];
                            }
                            $weeklyAmount = $normal + $archived;
                            if (!empty($business)) {
                                $description = "Weekly fee for $business->name";
                            } else {
                                $description = "Weekly fee for $business->s_account_id";
                            }

                            if ($req->has('save') && $weeklyAmount > 0) {
                                $weekly_charge = new WeeklyCharge();
                                $weekly_charge->business_id = $business->id;
                                $weekly_charge->start_date = $from_date;
                                $weekly_charge->end_date = $to_date;
                                $weekly_charge->amount = $weeklyAmount;
                                $weekly_charge->debited = 0;
                                $weekly_charge->save();
                            } else {
                                if ($weeklyAmount > 0) {
                                    $weekly_charge = WeeklyCharge::where('start_date', $from_date)->where('end_date', $to_date)->where('business_id', $business->id)->first();
                                    $weekly_charge->debited = 1;
                                    $weekly_charge->save();

                                    $charge = $stripe->charges->create([
                                        'amount'   => $weeklyAmount * 100,
                                        'currency' => 'gbp',
                                        'source' => $business->s_account_id,
                                        'description' => $description,
                                        'statement_descriptor' => $description
                                    ]);
                                }
                            }
                        }
                    }

                    // if ($monthlyAmount >= 0 && $business->balance >= $monthlyAmount && $business->business_commissions->auto_charge_weekly == 'yes') {
                    //     // Create charge
                    //     $carbon = Carbon::today();
                    //     $currentDayOfMonth = $carbon->day;
                    //     if ($currentDayOfMonth == 2) {
                    //         $from_date = date('Y-m-d', strtotime('first day of last month', $carbon->getTimestamp()));
                    //         $to_date = date('Y-m-d', strtotime('last day of last month', $carbon->getTimestamp()));
                    //     }

                    //     // $from_date = Carbon::today()->subDays(1)->startOfDay()->format('Y-m-d');
                    //     // $to_date = Carbon::today()->endOfDay()->format('Y-m-d');

                    //     if ($from_date != '' && $to_date != '') {
                    //         $business->append('default_bank_detail');
                    //         $business->append('default_paypal_detail');
                    //         $business->makeHidden(['business_commissions']);

                    //         $req->merge(['from_date' => $from_date]);
                    //         $req->merge(['to_date' => $to_date]);
                    //         $data = $this->setupTransactions($req, $business, $business->id, $from_date, $to_date);

                    //         if (!empty($data['normal'])) {
                    //             $normal = $data['normal']['net_fee'];
                    //         }
                    //         if (!empty($data['archived'])) {
                    //             $archived = $data['archived']['net_fee'];
                    //         }
                    //         $monthlyAmount = $normal + $archived;

                    //         if (!empty($business)) {
                    //             $description = "Monthly fee for $business->name";
                    //         } else {
                    //             $description = "Monthly fee for $business->s_account_id";
                    //         }

                    //         if ($req->has('save') && $weeklyAmount > 0) {
                    //             $weekly_charge = new WeeklyCharge();
                    //             $weekly_charge->business_id = $business->id;
                    //             $weekly_charge->start_date = $from_date;
                    //             $weekly_charge->end_date = $to_date;
                    //             $weekly_charge->amount = $monthlyAmount;
                    //             $weekly_charge->debit = 0;
                    //             $weekly_charge->save();
                    //         } else {
                    //             if ($weeklyAmount > 0) {
                    //                 $weekly_charge = WeeklyCharge::where('start_date', $from_date)->where('end_date', $to_date)->where('business_id', $business->id)->first();
                    //                 $weekly_charge->debit = 1;
                    //                 $weekly_charge->save();

                    //                 $charge = $stripe->charges->create([
                    //                     'amount'   => $monthlyAmount * 100,
                    //                     'currency' => 'gbp',
                    //                     'source' => $business->s_account_id,
                    //                     'description' => $description,
                    //                     'statement_descriptor' => $description  // $business->s_account_id
                    //                 ]);
                    //             }
                    //         }
                    //     }
                    // }
                    $response[] = ['id' => $business->id, 'name' => $business->name, 'weekly' => $weeklyAmount, 'monthly' => $monthlyAmount, 'weekly_date' => $weeklyOccuranceDate, 'monthly_date' => $monthlyOccuranceDate, 'today' => $today];
                }
            } catch (Exception $e) {
            }
        }
        return $response;
    }

    public function debit_single_amount(Request $req, $account_id)
    {
        $this->validate($req, [
            'business_id' => 'required',
            'amount' => 'gt:0|numeric',
        ]);

        $business_id = $req->input('business_id');
        $amount = $req->input('amount'); //human amount

        $business = Business::where('id', $business_id)->where('connect_service', '1')->whereNotNull('s_account_id')->where('disabled', '0')->first();
        if (empty($business)) Utils::error("No connect businesses found", 400);

        $stripe = self::$stripe;
        $response = [];

        $connectedAccount = $stripe->accounts->retrieve($business->s_account_id);
        if (!empty($connectedAccount)) {
            // Create charge

            // $description = $business->statement_description;
            $siteSetting =  TiffintomSiteSetting::find('1');
            $description = $siteSetting->stripe_description;
            $descriptor = $siteSetting->stripe_descriptor;

            $charge = $stripe->charges->create([
                'amount'   => $amount * 100,
                'currency' => 'gbp',
                'source' => $business->s_account_id,
                'description' => $description,
                'statement_descriptor' => $descriptor //$business->s_account_id
            ]);

            $response[] = ['id' => $business->id, 'name' => $business->name, 'charge' => $charge];
        }
        return $response;
    }

    public function setupTransactions($req, $business, $business_id, $from_date, $to_date)
    {
        $isConnectAccount = $business->connect_service;
        $accountType = $isConnectAccount ? 'Connect' : 'Normal';
        $chargeDescription = $business->statement_desciptor;

        $data = $this->transactionQueries($from_date, $to_date, 0, $business_id, $chargeDescription, $isConnectAccount);
        $data['from_date'] = $from_date;
        $data['to_date'] = $to_date;
        $data['business'] = $business;

        $data['misc_charge'] = 0;
        $data['weekly_fee'] = 0;
        $data['monthly_fee'] = 0;
        $data['transaction_type'] = 'Normal';

        $data['payout_mode'] = $business->payout_mode;
        $data['payout_method'] = $business->payout_method;
        $data['payout_comment'] = $business->payout_comment;
        $data['account_type'] = $isConnectAccount ? 'Connect' : 'Normal';

        if (strtolower($business->payout_method) == 'paypal') {
            if ($business->default_paypal_detail != null) {
                $data['payout_account'] = $business->default_paypal_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else if (strtolower($business->payout_method) == 'bank') {
            if ($business->default_bank_detail != null) {
                $data['payout_account'] = $business->default_bank_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else {
            $data['payout_account'] = "";
        }

        $mergedData =  $this->mergeAllTransactionsAndSort($data);

        $archivedData = $this->transactionQueries($from_date, $to_date, 1, $business_id, $chargeDescription, $isConnectAccount);
        $archivedData['from_date'] = $from_date;
        $archivedData['to_date'] = $to_date;
        $archivedData['business'] = $business;
        $archivedData['misc_charge'] = 0;
        $archivedData['weekly_fee'] = 0;
        $archivedData['monthly_fee'] = 0;
        $archivedData['transaction_type'] = 'Archived';

        $archivedData['payout_mode'] = $business->archived_payout_mode;
        $archivedData['payout_method'] = $business->archived_payout_method;
        $archivedData['payout_comment'] = $business->payout_comment;
        $archivedData['account_type'] = $isConnectAccount ? 'Connect' : 'Normal';

        if (strtolower($business->archived_payout_method) == 'paypal') {
            if ($business->default_paypal_detail != null) {
                $archivedData['payout_account'] = $business->default_paypal_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else if (strtolower($business->archived_payout_method) == 'bank') {
            if ($business->default_bank_detail != null) {
                $archivedData['payout_account'] = $business->default_bank_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else {
            $archivedData['payout_account'] = "";
        }

        $mergedArchivedData =  $this->mergeAllTransactionsAndSort($archivedData);

        // Create emi for misc invoice amount
        $payoutInvoice = PayoutInvoice::where('from_date', explode(" ", $mergedData['from_date'])[0])->where('to_date', explode(" ", $mergedData['to_date'])[0])->where('transaction_type', $mergedData['transaction_type'])->where('business_id', $mergedData['business']['id'])->first();
        $miscInvoices = MiscInvoice::where('payment_mode', 'percentage')->where('business_id', $business->id)->where('status', '!=', 'Paid')->get();
        $miscInvoicesAmount = 0;
        if (!$miscInvoices->isEmpty()) {
            foreach ($miscInvoices as $invoice) {
                $total_paid_amount = $invoice->total_paid_amount;
                if (!empty($payoutInvoice))
                    $total_paid_amount = -1 * Passbook::where('business_id', $business->id)->where('entity_id', $invoice->id)->where('entity', 'misc_invoice')->where('ref_entity_id', '!=', $payoutInvoice->id)->where('ref_entity', '!=', 'payout_invoice')->sum('amount');
                $payableAmount = $invoice->grand_total - $total_paid_amount;
                if ($payableAmount > 0) {
                    $actualEmi = $this->calculateFee($invoice->grand_total, $invoice->percentage, 'percentage');
                    if ($actualEmi < $payableAmount) {
                        $actualEmi = $this->calculateFee($payableAmount, $invoice->percentage, 'percentage');
                    } else {
                        $actualEmi = $payableAmount;
                    }
                    $miscInvoicesAmount = $miscInvoicesAmount + $actualEmi;
                }
            };
        }

        $miscCharges = MiscCharge::where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('archived', '0')->where('business_id', $business->id)->sum('amount');
        $miscChargesArchive = MiscCharge::where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('archived', '1')->where('business_id', $business->id)->sum('amount');
        $weeklyFee = $business->business_commissions->weekly_fee;

        $mergedData['total_normal_surcharges'] = 0;
        $all_normal_surcharges = [];
        $weeklySurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Normal')->where('business_id', $business->id)->get();
        if (!$weeklySurcharges->isEmpty()) {
            foreach ($weeklySurcharges as $value) {
                $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
            }
            $all_normal_surcharges = array_merge($all_normal_surcharges, $weeklySurcharges->toArray());
        }
        $monthlyFee = 0;
        $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $from_date")->format('Y-m-d');
        $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
        if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
            $monthlyFee = $business->business_commissions->monthly_fee;

            $monthlySurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Normal')->where('business_id', $business->id)->get();
            if (!$monthlySurcharges->isEmpty()) {
                foreach ($monthlySurcharges as $value) {
                    $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
                }
                $all_normal_surcharges = array_merge($all_normal_surcharges, $monthlySurcharges->toArray());
            }
        }

        $mergedData['surcharges'] = $all_normal_surcharges;

        $thisMonthInvoices = PayoutInvoice::whereRaw('(DATE_FORMAT(CAST(to_date as DATE), "%m/%Y") LIKE ? )', Carbon::today()->format('m/Y'))->where('business_id', $business_id)->count();
        if ($thisMonthInvoices > 0) {
            $monthlyFee = 0;
        }

        $normalTransactionsFound = false;
        if (sizeof($mergedData['transactions']) > 0 || (($weeklyFee > 0 || $monthlyFee > 0 || $miscCharges > 0) && sizeof($mergedArchivedData['transactions']) == 0)) {
            $normalTransactionsFound = true;

            $mergedArchivedData['total_archive_surcharges'] = 0;
            $all_archive_surcharges = [];
            $weeklyArchiveSurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Archive')->where('business_id', $business->id)->get();
            if (!$weeklyArchiveSurcharges->isEmpty()) {
                foreach ($weeklyArchiveSurcharges as $value) {
                    $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                }
                $all_archive_surcharges = array_merge($all_archive_surcharges, $weeklyArchiveSurcharges->toArray());
            }

            $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $from_date")->format('Y-m-d');
            $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
            if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
                $monthlyFee = $business->business_commissions->monthly_fee;
                $monthlyArchiveSurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Archive')->where('business_id', $business->id)->get();
                if (!$monthlyArchiveSurcharges->isEmpty()) {
                    foreach ($monthlyArchiveSurcharges as $value) {
                        $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                    }
                    $all_archive_surcharges = array_merge($all_archive_surcharges, $monthlyArchiveSurcharges->toArray());
                }
            }

            $mergedData['surcharges'] = array_merge($all_normal_surcharges, $all_archive_surcharges);

            $mergedData['misc_charge'] = $miscCharges;
            $mergedData['weekly_fee'] = $weeklyFee;
            $mergedData['monthly_fee'] = $monthlyFee;

            $mergedData['management_fee'] =  0;
            $mergedData['handling_fee'] =  0;

            $mergedData['misc_invoices'] = $miscInvoices;
            $mergedData['misc_invoice_amount'] = $miscInvoicesAmount;
            $mergedData['misc_charge'] = $mergedData['misc_charge'] +  $mergedData['misc_invoice_amount'];

            $mergedData['net_fee'] = $mergedData['net_fee'] +    $mergedData['monthly_fee'] + $mergedData['weekly_fee'] + $mergedData['total_normal_surcharges']; // $mergedData['handling_fee'] +  $mergedData['management_fee'] +  $mergedData['misc_charge'] +
            $mergedData['net_total'] = $mergedData['net_total']  - ($mergedData['handling_fee'] +  $mergedData['management_fee'] +  $mergedData['misc_charge'] + $mergedData['monthly_fee'] + $mergedData['weekly_fee'] + $mergedData['total_normal_surcharges']);

            // Add payout charges
            $payoutCharge = PayoutCharge::whereRaw('(amount>= ? AND (SELECT pc.amount FROM payout_charges pc WHERE pc.business_id = ? ORDER BY pc.amount DESC LIMIT 1))', [$mergedData['net_total'], $business_id])->where('business_id', $business_id)->orderByRaw('amount ASC')->first();
            $payoutChargeAmount = 0;
            if (!empty($payoutCharge)) {
                $payoutChargeAmount = $payoutCharge->charge;
            }
            $mergedData['payout_charge'] =  $payoutChargeAmount;
            $mergedData['net_fee'] = $mergedData['net_fee'] + $mergedData['payout_charge'];
            $mergedData['net_total'] = $mergedData['net_total'] - $mergedData['payout_charge'];
        } else {
            $mergedData = null;
        }

        if (sizeof($mergedArchivedData['transactions']) > 0 || $miscChargesArchive > 0) {
            $mergedArchivedData['total_archive_surcharges'] = 0;
            $all_archive_surcharges = [];
            $weeklyArchiveSurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Archive')->where('business_id', $business->id)->get();
            if (!$weeklyArchiveSurcharges->isEmpty()) {
                foreach ($weeklyArchiveSurcharges as $value) {
                    $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                }
                $all_archive_surcharges = array_merge($all_archive_surcharges, $weeklyArchiveSurcharges->toArray());
            }

            $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $from_date")->format('Y-m-d');
            $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
            if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
                $monthlyFee = $business->business_commissions->monthly_fee;
                $monthlyArchiveSurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Archive')->where('business_id', $business->id)->get();
                if (!$monthlyArchiveSurcharges->isEmpty()) {
                    foreach ($monthlyArchiveSurcharges as $value) {
                        $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                    }
                    $all_archive_surcharges = array_merge($all_archive_surcharges, $monthlyArchiveSurcharges->toArray());
                }
            }
            $mergedArchivedData['surcharges'] = $all_archive_surcharges;

            $cashFee = $business->business_commissions->cash_handling_fee;
            $managementFee = $business->business_commissions->management_fee;
            $cashFeeOnEveryAmount = $business->business_commissions->cash_handling_per_amount;
            $handlingFee = 0;
            if ($cashFeeOnEveryAmount > 0) {
                $handlingFee = $cashFee * ceil($mergedArchivedData['net_total'] / $cashFeeOnEveryAmount);
            }
            if (!$normalTransactionsFound) {
                $mergedArchivedData['misc_charge'] = $miscChargesArchive;
                $mergedArchivedData['weekly_fee'] = $weeklyFee;
                $mergedArchivedData['monthly_fee'] = $monthlyFee;

                $mergedArchivedData['management_fee'] =  $managementFee;
                $mergedArchivedData['handling_fee'] =  $handlingFee;

                $mergedArchivedData['misc_invoices'] = $miscInvoices;
                $mergedArchivedData['misc_invoice_amount'] = $miscInvoicesAmount;
                $mergedArchivedData['misc_charge'] = $mergedArchivedData['misc_charge'] + $mergedArchivedData['misc_invoice_amount'];

                $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] + $mergedArchivedData['monthly_fee'] + $mergedArchivedData['weekly_fee'] + $mergedArchivedData['total_archive_surcharges']; // +   $mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge'] 
                $mergedArchivedData['net_total'] = $mergedArchivedData['net_total']  - ($mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge'] + $mergedArchivedData['monthly_fee'] + $mergedArchivedData['weekly_fee'] + $mergedArchivedData['total_archive_surcharges']);

                // Add Payout charges
                $payoutCharge = PayoutCharge::whereRaw('(amount>= ? AND (SELECT pc.amount FROM payout_charges pc WHERE pc.business_id = ? ORDER BY pc.amount DESC LIMIT 1))', [$mergedArchivedData['net_total'], $business_id])->where('business_id', $business_id)->orderByRaw('amount ASC')->first();
                $payoutChargeAmount = 0;
                if (!empty($payoutCharge)) {
                    $payoutChargeAmount = $payoutCharge->charge;
                }
                $mergedArchivedData['payout_charge'] =  $payoutChargeAmount;
                $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] + $payoutChargeAmount;
                $mergedArchivedData['net_total'] = $mergedArchivedData['net_total'] - $payoutChargeAmount;
            } else {
                $mergedArchivedData['payout_charge'] =  0;
                $mergedArchivedData['management_fee'] =  $managementFee;
                $mergedArchivedData['handling_fee'] =  $handlingFee;

                $mergedArchivedData['misc_charge'] = $miscChargesArchive;

                $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee']; //  $mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge']
                $mergedArchivedData['net_total'] = $mergedArchivedData['net_total']  - ($mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge']);
            }
        } else {
            $mergedArchivedData = null;
        }

        if ($mergedData == null && $mergedArchivedData != null) {
            $mergedData['total_normal_surcharges'] = 0;
            $all_normal_surcharges = [];
            $weeklySurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Normal')->where('business_id', $business->id)->get();
            if (!$weeklySurcharges->isEmpty()) {
                foreach ($weeklySurcharges as $value) {
                    $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
                }
                $all_normal_surcharges = array_merge($all_normal_surcharges, $weeklySurcharges->toArray());
            }
            $monthlyFee = 0;
            $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $from_date")->format('Y-m-d');
            $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
            if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
                $monthlyFee = $business->business_commissions->monthly_fee;

                $monthlySurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Normal')->where('business_id', $business->id)->get();
                if (!$monthlySurcharges->isEmpty()) {
                    foreach ($monthlySurcharges as $value) {
                        $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
                    }
                    $all_normal_surcharges = array_merge($all_normal_surcharges, $monthlySurcharges->toArray());
                }
            }

            $mergedData['surcharges'] = $all_normal_surcharges;
            $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] + $mergedData['total_normal_surcharges'];
            $mergedArchivedData['net_total'] = $mergedArchivedData['net_total']  - $mergedData['total_normal_surcharges'];

            $mergedArchivedData['surcharges'] = array_merge($all_normal_surcharges, $all_archive_surcharges);
            $mergedData = null;
        }

        return ['normal' => $mergedData, 'archived' => $mergedArchivedData];
    }

    public function transactionQueries($from_date, $to_date, $archived, $business_id, $chargeDescription, $isConnectAccount)
    {
        $from_date = date('Y-m-d 00:00:00', strtotime($from_date));
        $to_date = date('Y-m-d 23:59:59', strtotime($to_date));
        $data = [];
        $codOrders = OnlineOrder::where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $codOrders = $codOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $codOrders = $codOrders->where('order_date', '>=', "$from_date");
        $codOrders = $codOrders->where('order_date', '<=', "$to_date");
        $codOrders = $codOrders->where('payment_method', '=', 'cod');
        $codOrders = $codOrders->where('online_orders.archived',  $archived);
        $codOrders = $codOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $codOrders = $codOrders->get();
        if (!$codOrders->isEmpty()) {
            $data['cod_report'] = $this->createReport($business_id, $codOrders);
        }

        $paypalOrders = OnlineOrder::select('*', 'online_orders.txn_id as online_txn_id', 'paypal_payments.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $paypalOrders = $paypalOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $paypalOrders = $paypalOrders->leftJoin('paypal_payments', 'sale_id', '=', 'online_orders.txn_id');
        $paypalOrders = $paypalOrders->where('paypal_payments.archived',  $archived);
        $paypalOrders = $paypalOrders->where('order_date', '>=', "$from_date");
        $paypalOrders = $paypalOrders->where('order_date', '<=', "$to_date");
        $paypalOrders = $paypalOrders->where('payment_method', '=', 'paypal');
        $paypalOrders = $paypalOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $paypalOrders = $paypalOrders->get();
        if (!$paypalOrders->isEmpty()) {
            $data['paypal_report'] = $this->createReport($business_id, $paypalOrders);;
        }

        $stripeOrders = OnlineOrder::select('*', 'stripe_transactions.txn_id as stripe_txn_id', 'online_orders.txn_id as online_txn_id', 'stripe_transactions.card_number as card_number', 'stripe_transactions.card_brand as card_brand', 'stripe_transactions.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $stripeOrders = $stripeOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $stripeOrders = $stripeOrders->leftJoin('stripe_transactions', function ($join) {
            $join->on('stripe_transactions.payment_intent_id', '=', 'online_orders.txn_id')->where('reporting_category', 'charge');
        });
        $stripeOrders = $stripeOrders->where('order_date', '>=', "$from_date");
        $stripeOrders = $stripeOrders->where('order_date', '<=', "$to_date");
        $stripeOrders = $stripeOrders->where('stripe_transactions.archived',  $archived);
        $stripeOrders = $stripeOrders->whereIn('payment_method', ['Credit Card', 'Online']);
        $stripeOrders = $stripeOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $stripeOrders = $stripeOrders->get();
        if (!$stripeOrders->isEmpty()) {
            $data['card_report'] = $this->createReport($business_id, $stripeOrders);;
        }

        $walletOrders = OnlineOrder::select('*',  'online_orders.txn_id as online_txn_id')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $walletOrders = $walletOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $walletOrders = $walletOrders->where('order_date', '>=', "$from_date");
        $walletOrders = $walletOrders->where('order_date', '<=', "$to_date");
        $walletOrders = $walletOrders->where('online_orders.archived',  $archived);
        $walletOrders = $walletOrders->whereIn('payment_method', ['Wallet']);
        $walletOrders = $walletOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $walletOrders = $walletOrders->get();
        if (!$walletOrders->isEmpty()) {
            $data['wallet_report'] = $this->createReport($business_id, $walletOrders);;
        }

        // $merchantTransactions = StripeTransaction::whereRaw('(descriptor LIKE ? OR business_id LIKE ?)', ["$chargeDescription", $business_id]);
        // $merchantTransactions = $merchantTransactions->where('stripe_transactions.archived',  $archived);
        // $merchantTransactions = $merchantTransactions->where('created', '>=', strtotime($from_date));
        // $merchantTransactions = $merchantTransactions->where('created', '<=', strtotime($to_date));
        // $merchantTransactions = $merchantTransactions->whereIn('reporting_category', ['charge', 'refund', 'dispute']);
        // $merchantTransactions = $merchantTransactions->whereIn('service', ['Merchant', 'Connect']);
        // $merchantTransactions = $merchantTransactions->get();
        // if (!$merchantTransactions->isEmpty()) {
        //     $data['merchant_report'] = $this->createMerchantReport($merchantTransactions, true, $isConnectAccount);
        // }
        return $data;
    }

    public function createReport($business_id, $orders)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_charge_back_transactions = 0;
        $total_charge_back_fee = 0.0;
        $total_fee = 0.0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_service_charge = 0;
        foreach ($orders as $order) {
            $orderTypeWiseCommission = 0.00;
            $orderTypeWiseServiceCharge = 0.00;
            $orderBasedOrTransactionWiseCharge = 0.00;
            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;
            $payment_method_fees = 0;
            $chargeBackFee = 0;
            $handlingFee = 0.00;
            $cardCountry = "";
            $brand = "";

            if (!empty($order->card_brand)) {
                $brand = $order->card_brand;
            }
            $business = Business::where('id', $business_id)->first();
            if ($business->service_charge_type == 'show') {
                $orderTypeWiseServiceCharge = $order->user_service_charge; // It would be mostly in flat.
            }

            $orderGrandTotalBeforeServiceCharge = $order->grand_total;
            $orderGrandTotalForDisplay = $order->grand_total;
            $refundedAmount = 0;

            if ($order->payment_method == 'paypal') {
                $refundedAmount = abs($order->refunded_amount);
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_paypal_fee, $order->online_paypal_fee_type);
            }
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online' || strtolower($order->payment_method) == 'wallet') {
                $refundedAmount = abs($order->refunded_amount) / 100;
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_card_fee, $order->online_card_fee_type);
            }

            if ($order->payment_method == 'cod')
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_cod_fee, $order->online_cod_fee_type);

            if ($order->order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->dinein_commission_fee, $order->dinein_commission_fee_type);
            } else {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_commission_fee, $order->online_commission_fee_type);
            }
            if ($order->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->per_order_charge, $order->per_order_charge_type);
            } else {
                if ($order->payment_method == 'paypal')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->paypal_charge, $order->paypal_charge_type);
                if (strtolower($order->payment_method) == 'credit card' || strtolower($order->payment_method) == 'online' || strtolower($order->payment_method) == 'wallet')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->card_charge, $order->card_charge_type);
                if ($order->payment_method == 'cod')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->cod_charge, $order->cod_charge_type);
            }


            $serviceCharge = $orderTypeWiseServiceCharge;
            $serviceCharge = 0;
            $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $serviceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);
            if ($order->order_status == 'Failed') {
                $transaction['order_grand_total'] = -$this->convertToDecimal($orderGrandTotalForDisplay);
                if ($order->charges_refundable == 'yes') {
                    $transaction['total_fee'] = 0;
                    $orderTypeWiseServiceCharge = 0;
                }
                $total_refund_transactions = $total_refund_transactions + 1;
                $transaction['net_total'] = $this->convertToDecimal(0 - $transaction['total_fee']);
            } else {
                $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotalForDisplay);
                if ($order->payment_method == 'cod') $transaction['order_grand_total'] = 0;

                $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $transaction['total_fee']);
                $gross_total = $gross_total + $transaction['order_grand_total'];
            }

            $transaction['order_id'] = "#ODR000" . $order->order_id;
            $transaction['restaurant_id'] = $order->restaurant_id;
            if (empty($order->stripe_txn_id)) {
                $transaction['txn_id'] = $order->online_txn_id;
            } else {
                $transaction['txn_id'] = $order->stripe_txn_id;
            }
            $transaction['order_date'] = $order->order_date;
            $transaction['order_type'] = $order->order_type;
            $transaction['order_status'] = $order->order_status;
            $transaction['payment_method'] = strtolower($order->payment_method) == 'credit card' ? 'Online' : $order->payment_method;
            $transaction['card_number'] = !empty($order->card_number) ? $order->card_number : null;
            $transaction['card_brand'] = $brand;
            $transaction['payment_method_fees'] = $payment_method_fees;
            $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
            $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
            $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;
            $transaction['country'] = $cardCountry;
            $transaction['country_charges'] = $countryWiseCommission;
            $transaction['charge_back_fee'] = $chargeBackFee;
            $transaction['handling_fee'] = $handlingFee;
            $transaction['card_brand'] = $brand;
            $transaction['card_brand_charge'] = $cardBrandWiseCommission;
            $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotalBeforeServiceCharge);

            $net_total = $net_total + $transaction['net_total'];
            $total_fee = $total_fee + abs($transaction['total_fee']);
            $total_service_charge = $total_service_charge + $orderTypeWiseServiceCharge;
            $total_charge_back_fee = $total_charge_back_fee + ($transaction['charge_back_fee']);
            $transactions[] = $transaction;
        }
        $report['transactions'] = $transactions;
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($net_total);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_non_domestic_fee'] = $total_non_domestic_fee;
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_charge_back_transactions'] = $this->convertToDecimal($total_charge_back_transactions);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['total_refund_transactions'] = $total_refund_transactions;
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        return $report;
    }

    public function createMerchantReport($merchantTransactions, $archivedReport, $isConnectAccount)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_fee = 0.0;

        $total_charge_back_transactions = 0;
        $total_refund_transactions = 0;
        $total_charge_back_fee = 0.0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $managementFee = 0;
        $archivedTransactions = 0;
        $cardCountry = "";
        $brand = "";
        $total_service_charge = 0;

        foreach ($merchantTransactions as $transaction) {
            $cardProcessingFee = 0.00;
            $cardAuthFee = 0.00;
            $managementFee = 0.00;
            $handlingFee = 0.00;
            $chargeBackFee = 0;
            $cardCountry = $transaction->card_country;
            $brand = $transaction->card_brand;

            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_auth_fee, $transaction->merchant_card_auth_fee_type));
                $cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_processing_fee, $transaction->merchant_card_processing_fee_type));
                if ($transaction->archived) {
                    if ($transaction->cash_handling_fee_type == 'percentage') {
                        $handlingFee  = $this->calculateFee($transaction->amount / 100, $transaction->cash_handling_fee, $transaction->cash_handling_fee_type);
                    } else {
                        if ($transaction->cash_handling_per_amount > 0)
                            $handlingFee = $transaction->cash_handling_fee * ceil($transaction->amount / $transaction->cash_handling_per_amount);
                    }
                    $handlingFee = 0;
                    $archivedTransactions = $archivedTransactions + 1;
                }

                $countryCommission = CountryCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();
                if (!empty($countryCommission)) {
                    $countryWiseCommission = abs($this->calculateFee($transaction->amount / 100, $countryCommission->stripe_percentage, 'percentage'));
                    $countryWiseCommission = $countryWiseCommission + $countryCommission->stripe_fixed;
                }
                $cardBrandCommission = CardBrandCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();

                if (!empty($cardBrandCommission)) {
                    $cardBrandWiseCommission = abs($this->calculateFee($transaction->amount / 100, $cardBrandCommission->stripe_percentage, 'percentage'));
                    $cardBrandWiseCommission = $cardBrandWiseCommission + $cardBrandCommission->stripe_fixed;
                }

                if ($transaction->reporting_category == 'refund') {
                    $cardAuthFee = -$cardAuthFee;
                    $cardProcessingFee = -$cardProcessingFee;
                    $cardBrandWiseCommission = -$cardBrandWiseCommission;
                    $countryWiseCommission = -$countryWiseCommission;
                }
            }
            if ($transaction->reporting_category == 'dispute') {
                $chargeBackFee = $this->calculateFee($transaction->amount / 100, $transaction->charge_back_fee, $transaction->charge_back_fee_type);
                $total_charge_back_transactions = $total_charge_back_transactions + 1;
            }
            if ($transaction->reporting_category == 'refund') {
                $total_refund_transactions = $total_refund_transactions + 1;
            }
            $applicationFee = ($this->convertToDecimal($managementFee + $handlingFee + $cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee));
            if ($transaction->service == 'Connect') {
                if ($transaction->reporting_category == 'refund' || $transaction->reporting_category == 'transfer_reversal') {
                    $applicationFee = 0;
                } else {
                    $applicationFeeTransaction = StripeTransaction::where('reporting_category', 'platform_earning')->whereNotNull('payment_intent_id')->where('business_id', $transaction->business_id)->where('payment_intent_id', $transaction->payment_intent_id)->first();
                    if (!empty($applicationFeeTransaction)) {
                        $applicationFee =  $this->convertToDecimal($applicationFeeTransaction->amount / 100);
                    }
                }
            }

            $merchantTransction['order_id'] = $transaction->txn_id;
            $merchantTransction['restaurant_id'] = null;
            $merchantTransction['txn_id'] = $transaction->txn_id;
            $merchantTransction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
            $merchantTransction['order_type'] = $transaction->service;
            $merchantTransction['order_status'] = $transaction->reporting_category;
            $merchantTransction['payment_method'] = $transaction->service;
            $merchantTransction['order_grand_total'] = $transaction->amount / 100;

            $merchantTransction['processing_fee'] = $cardProcessingFee;
            $merchantTransction['country'] = $cardCountry;
            $merchantTransction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
            $merchantTransction['card_brand'] = $brand;
            $merchantTransction['card_number'] = $transaction->card_number;
            $merchantTransction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
            $merchantTransction['auth_fee'] = $cardAuthFee;
            $merchantTransction['handling_fee'] = 0;
            $merchantTransction['management_fee'] = $managementFee;
            $merchantTransction['charge_back_fee'] = $chargeBackFee;
            $merchantTransction['total_fee'] =  $applicationFee;
            if ($transaction->non_domestic) {
                $total_non_domestic_transactions = $total_non_domestic_transactions + 1;
                $total_non_domestic_fee = $total_non_domestic_fee + $countryWiseCommission;
            }

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
                if ($transaction->service == 'Connect' && abs($applicationFee) > ($transaction->amount / 100) && $transaction->reporting_category != 'refund') $merchantTransction['net_total'] = 0; // Condition double check when charge refund yes.
            } else {
                if ($transaction->reporting_category == 'dispute') {
                    $merchantTransction['total_fee'] = $chargeBackFee;
                    $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
                } else {
                    $merchantTransction['total_fee'] = 0;
                    $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $merchantTransction['total_fee']);
                }
            }
            $gross_total = $gross_total + $merchantTransction['order_grand_total'];
            $net_total = $net_total + $merchantTransction['net_total'];
            $total_fee = $total_fee + abs($merchantTransction['total_fee']);
            $total_charge_back_fee = $total_charge_back_fee + $merchantTransction['charge_back_fee'];
            $transactions[] = $merchantTransction;
        }
        $report['transactions'] = $transactions;
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($net_total);
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        $report['total_charge_back_transactions'] = $this->convertToDecimal($total_charge_back_transactions);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['total_non_domestic_fee'] = $total_non_domestic_fee;
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        return $report;
    }

    public function mergeAllTransactionsAndSort($data)
    {
        $gross_total = 0;
        $total_fee = 0;
        $net_total = 0;
        $total_charge_back_fee = 0;
        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_refund_transactions = 0;
        $total_service_charge = 0;
        $misc_charge = ($data['misc_charge']);
        $weekly_fee = ($data['weekly_fee']);
        $monthly_fee = ($data['monthly_fee']);
        $all_transactions = [];
        if (isset($data['cod_report'])) {
            $gross_total = $gross_total + $data['cod_report']['gross_total'];
            $total_fee = $total_fee + $data['cod_report']['total_fee'];
            $net_total = $net_total + $data['cod_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['cod_report']['total_service_charge'];
            $total_refund_transactions = $total_refund_transactions + $data['cod_report']['total_refund_transactions'];
            $temp_transactions = $data['cod_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['card_report'])) {
            $gross_total = $gross_total + $data['card_report']['gross_total'];
            $total_fee = $total_fee + $data['card_report']['total_fee'];
            $net_total = $net_total + $data['card_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['card_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['card_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['card_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['card_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['card_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['card_report']['total_refund_transactions'];
            $temp_transactions = $data['card_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['wallet_report'])) {
            $gross_total = $gross_total + $data['wallet_report']['gross_total'];
            $total_fee = $total_fee + $data['wallet_report']['total_fee'];
            $net_total = $net_total + $data['wallet_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['wallet_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['wallet_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['wallet_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['wallet_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['wallet_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['wallet_report']['total_refund_transactions'];
            $temp_transactions = $data['wallet_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['paypal_report'])) {
            $gross_total = $gross_total + $data['paypal_report']['gross_total'];
            $total_fee = $total_fee + $data['paypal_report']['total_fee'];
            $net_total = $net_total + $data['paypal_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['paypal_report']['total_service_charge'];
            $temp_transactions = $data['paypal_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['paypal_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['paypal_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['paypal_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        // if (isset($data['merchant_report'])) {
        //     $gross_total = $gross_total + $data['merchant_report']['gross_total'];
        //     $total_fee = $total_fee + $data['merchant_report']['total_fee'];
        //     $net_total = $net_total + $data['merchant_report']['net_total'];
        //     $total_charge_back_fee = $total_charge_back_fee + $data['merchant_report']['total_charge_back_fee'];
        //     $total_charge_back_transactions = $total_charge_back_transactions + $data['merchant_report']['total_charge_back_transactions'];
        //     $temp_transactions = $data['merchant_report']['transactions'];
        //     $total_non_domestic_fee = $total_non_domestic_fee + $data['merchant_report']['total_non_domestic_fee'];
        //     $total_non_domestic_transactions = $total_non_domestic_transactions + $data['merchant_report']['total_non_domestic_transactions'];
        //     $total_refund_transactions = $total_refund_transactions + $data['merchant_report']['total_refund_transactions'];
        //     if (!empty($temp_transactions)) {
        //         $all_transactions = array_merge($all_transactions, $temp_transactions);
        //     }
        // }
        usort($all_transactions, $this->date_compare('order_date'));

        return [
            'from_date' => $data['from_date'],
            'to_date' => $data['to_date'],
            'payout_mode' => $data['payout_mode'],
            'payout_method' => $data['payout_method'],
            'payout_account' => $data['payout_account'],
            'transaction_type' => $data['transaction_type'],
            'account_type' => $data['account_type'],
            'payout_comment' => $data['payout_comment'],
            'gross_total' => $this->convertToDecimal($gross_total),
            'gross_fee' => $this->convertToDecimal($total_fee),
            'misc_charge' => $this->convertToDecimal($misc_charge),
            'weekly_fee' => $this->convertToDecimal($weekly_fee),
            'monthly_fee' => $this->convertToDecimal($monthly_fee),
            'total_charge_back_fee' => $this->convertToDecimal($total_charge_back_fee),
            'total_charge_back_transactions' => $this->convertToDecimal($total_charge_back_transactions),
            'total_non_domestic_transactions' => $this->convertToDecimal($total_non_domestic_transactions),
            'total_non_domestic_fee' => $this->convertToDecimal($total_non_domestic_fee),
            'total_service_charge' =>  $this->convertToDecimal($total_service_charge),
            'total_refund_transactions' => $total_refund_transactions,
            'net_fee' => $this->convertToDecimal($total_fee + $misc_charge + $monthly_fee + $weekly_fee + $total_service_charge),
            'net_total' => $this->convertToDecimal($net_total - $misc_charge - $monthly_fee - $weekly_fee - $total_service_charge),
            'transactions' => $all_transactions,
            'business' => $data['business']
        ];
    }

    public function date_compare($key)
    {
        return function ($a, $b) use ($key) {
            $t1 = strtotime($a[$key]);
            $t2 = strtotime($b[$key]);
            return $t2 - $t1;
        };
    }

    function nextDate($start_date, $interval_days, $output_format)
    {
        $start = strtotime($start_date);
        $end = strtotime(date('Y-m-d'));
        $days_ago = ($end - $start) / 24 / 60 / 60;
        if ($days_ago < 0) return date($output_format, $start);
        $remainder_days = $days_ago % $interval_days;
        if ($remainder_days > 0) {
            $new_date_string = "+" . ($interval_days - $remainder_days) . " days";
        } else {
            $new_date_string = "today";
        }

        return date($output_format, strtotime($new_date_string));
    }

    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
