<?php

namespace App\Http\Controllers\Connect;

use App\ModelsConnect\ConnectFile;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class ConnectFileController extends Controller
{
    /**
     * List files.
     * GET /connect/files
     */
    public function index(Request $req)
    {
        $files = ConnectFile::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $files->whereRaw('(s_file_id LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('connect_account_id', false)) {
            $files->where('connect_account_id', $query);
        }
        if ($query = $req->query('person_id', false)) {
            $files->where('person_id', $query);
        }
        if ($query = $req->query('company_id', false)) {
            $files->where('company_id', $query);
        }
        if ($query = $req->query('business_type', false)) {
            $files->where('business_type', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $files = $files->get();
        } else {
            $files = $files->paginate($req->query('per_page', 20));
        }

        if ($files->isEmpty()) Utils::error('No files found!', 404);


        return ($files);
    }

    /**
     * Create file.
     * POST /connect/files
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'connect_account_id' => 'required|exists:connect_accounts,id',
            'person_id' => 'exists:persons,id',
            'comapny_id' => 'exists:companies,id',
            's_account_id' => 'string',
            's_person_id' => 'string',

            'purpose' => 'required|string',
            'file' => 'required|file',
        ]);

        // create file
        $file = new ConnectFile;
        $file->fill($req->all());
        if ($req->hasFile('file')) {
            $fileName = Storage::disk('public')->put('files/connect/', $req->file('file'));
            $file->file = $req->file('file')->hashName();
        }
        $file->save();

        return $file;
    }

    /**
     * Show file.
     * GET /connect/files/{id}
     */
    public function show(Request $req, $id)
    {
        $file = ConnectFile::find($id);
        if (empty($file)) Utils::error('ConnectFile not found!', 404);
        return $file;
    }

    /**
     * Update file.
     * POST /connect/files/{id}
     */
    public function update(Request $req, $id)
    {
        $file = ConnectFile::find($id);
        if (empty($file)) Utils::error('ConnectFile not found!', 404);

        $this->validate($req, [
            'purpose' => 'string',
            'file' => 'required|file',

        ]);

        // update file
        if ($req->has('purpose')) $file->purpose = $req->input('purpose');
        if ($req->hasFile('file')) {
            $fileName = Storage::disk('public')->put('files/connect/', $req->file('file'));
            $file->file = $req->file('file')->hashName();
        }
        $file->save();

        return $file;
    }


    /**
     * Delete file by id.
     * DELETE /connect/files/{id}
     */
    public function delete(Request $req, $id)
    {
        $file = ConnectFile::find($id);
        if (empty($file)) Utils::error('ConnectFile not found!', 404);
        $file->delete();
        return $file;
    }
}
