<?php

namespace App\Http\Controllers\Connect;

use App\ModelsConnect\Person;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class PersonController extends Controller
{
    /**
     * List persons.
     * GET /connect/persons
     */
    public function index(Request $req)
    {
        $persons = Person::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $persons->whereRaw('(s_account_id LIKE ? OR name LIKE ? OR phone_number LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('connect_account_id', false)) {
            $persons->where('connect_account_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $persons = $persons->get();
        } else {
            $persons = $persons->paginate($req->query('per_page', 20));
        }

        if ($persons->isEmpty()) Utils::error('No persons found!', 404);


        return ($persons);
    }

    /**
     * Create person.
     * POST /connect/persons
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            's_account_id' => 'string',
            's_person_id' => 'string',
            'connect_account_id' => 'exists:connect_accounts.id',
            'company_id' => 'exists:companies.id',

            'first_name' => 'string',
            'last_name' => 'required|string',
            'email' => 'required|email',
            'phone_number' => 'required|string',
            'title' => 'required|string',
            'id_number' => 'required|string',
            'dob' => 'required|date',

            'line1' => 'string|required',
            'line2' => 'string',
            'city' => 'string|required',
            'state' => 'string',
            'postal_code' => 'string|required',

            'owner' => 'boolean',
            'director' => 'boolean',
            'executive' => 'boolean',
            'account_opener' => 'boolean',
            'representative' => 'boolean',

        ]);

        // create person
        $person = new Person;
        $person->fill($req->all());

        $person->save();

        return $person;
    }

    /**
     * Show person.
     * GET /connect/persons/{id}
     */
    public function show(Request $req, $id)
    {
        $person = Person::find($id);
        if (empty($person)) Utils::error('Person not found!', 404);
        return $person;
    }

    /**
     * Update person.
     * POST /connect/persons/{id}
     */
    public function update(Request $req, $id)
    {
        $person = Person::find($id);
        if (empty($person)) Utils::error('Person not found!', 404);

        $this->validate($req, [
            's_account_id' => 'string',
            's_person_id' => 'string',
            'connect_account_id' => 'exists:connect_accounts.id',
            'company_id' => 'exists:companies.id',

            'first_name' => 'string',
            'last_name' => 'required|string',
            'email' => 'required|email',
            'phone_number' => 'required|string',
            'title' => 'required|string',
            'id_number' => 'required|string',
            'dob' => 'required|date',

            'line1' => 'string|required',
            'line2' => 'string',
            'city' => 'string|required',
            'state' => 'string',
            'postal_code' => 'string|required',

            'owner' => 'boolean',
            'director' => 'boolean',
            'executive' => 'boolean',
            'account_opener' => 'boolean',
            'representative' => 'boolean',


        ]);

        // update person
        if ($req->has('s_account_id')) $person->s_account_id = $req->input('s_account_id');
        if ($req->has('s_person_id')) $person->s_person_id = $req->input('s_person_id');
        if ($req->has('first_name')) $person->first_name = $req->input('first_name');
        if ($req->has('last_name')) $person->last_name = $req->input('last_name');
        if ($req->has('email')) $person->email = $req->input('email');
        if ($req->has('phone_number')) $person->phone_number = $req->input('phone_number');
        if ($req->has('title')) $person->title = $req->input('title');
        if ($req->has('id_number')) $person->id_number = $req->input('id_number');
        if ($req->has('dob')) $person->dob = $req->input('dob');
        if ($req->has('line1')) $person->line1 = $req->input('line1');
        if ($req->has('line2')) $person->line2 = $req->input('line2');
        if ($req->has('city')) $person->city = $req->input('city');
        if ($req->has('state')) $person->state = $req->input('state');
        if ($req->has('postal_code')) $person->postal_code = $req->input('postal_code');

        $person->owner = $req->input('owner');
        $person->director = $req->input('director');
        $person->executive = $req->input('executive');
        $person->account_opener = $req->input('account_opener');
        $person->representive = $req->input('representive');

        $person->save();

        return $person;
    }


    /**
     * Delete person by id.
     * DELETE /connect/persons/{id}
     */
    public function delete(Request $req, $id)
    {
        $person = Person::find($id);
        if (empty($person)) Utils::error('Person not found!', 404);
        $person->delete();
        return $person;
    }
}
