<?php

namespace App\Http\Controllers\Connect;

use App\Business;
use App\BusinessCardReader;
use App\CardBrand;
use App\CardBrandCommission;
use App\Country;
use App\CountryCommission;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsConnect\ConnectAccount;
use App\StripeTransaction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use PDO;
use Stripe\PaymentIntent;
use Stripe\Stripe;
use Stripe\StripeClient;

class StripeController extends Controller
{
    public function payment_intent(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $stripeKeyColoumns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $stripeKeyColoumns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        // return $stripe->paymentMethods->retrieve("pm_1IiZM9KgAij6PRAuopr1r0I4");
        $this->validate($req, [
            'amount' => 'required|numeric|gt:0',  // multiplied by 100
            's_terminal_id' => 'string',
            'transaction_type' => 'required|in:online,merchant,connect',
            'business_id' => 'required_without_all:statement_descriptor,restaurant_id,s_terminal_id',

            'order_type' => 'required_if:transaction_type,online|in:dinein,pickup,delivery',

            'restaurant_id' => 'required_without_all:statement_descriptor,business_id,s_terminal_id',
            'statement_descriptor' => 'required_without_all:restaurant_id,business_id,s_terminal_id',

            'customer_id' => 'string',
            'customer_name' => 'string',
            'customer_email' => 'string|email|required_with:customer_name',

            'auto_confirm' => 'boolean'
        ]);
        $transaction_type = $req->input('transaction_type');
        $order_type = $req->input('order_type');
        $statement_descriptor = $req->input('statement_descriptor');
        $statement_description = "";
        $restaurant_id = $req->input('restaurant_id');
        $business_id = $req->input('business_id');
        $terminalId = $req->input('s_terminal_id');
        $descriptor_column = '';
        $business = null;
        $autoConfirm = $req->input('auto_confirm', false);
        if ($transaction_type == 'online') {
            $descriptor_column = 'online_statement_descriptor';
        } else if ($transaction_type == 'merchant') {
            $descriptor_column = 'statement_descriptor';
        }
        if ($statement_descriptor != null) {
            $business = Business::where($descriptor_column, $statement_descriptor)->orderByRaw('created_at DESC')->first();
        }
        if ($terminalId != null) {
            $businessCardReader = BusinessCardReader::where('s_terminal_id', $terminalId)->latest()->first();
            if (!empty($businessCardReader)) {
                $business_id = $businessCardReader->business_id;
            }
        }
        if ($business_id != null) {
            $business = Business::find($business_id);
        }

        if ($restaurant_id != null) {
            $business = Business::where('restaurant_id', $restaurant_id)->orderByRaw('created_at DESC')->first();
        }

        if (empty($business) && strtolower($terminalId) != 'simulator') {
            return Utils::error("Invalid restaurant or business", 400);
        }
        if (empty($business) && strtolower($terminalId) == 'simulator') $transaction_type = '';

        if ($transaction_type == 'online') {
            $statement_descriptor = $business->online_statement_descriptor;
            $statement_description = $business->online_statement_description;
        }
        if ($transaction_type == 'merchant') {
            $statement_descriptor = $business->statement_descriptor;
            $statement_description = $business->statement_description;
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $transaction_type = 'connect';
            }
        }
        $stripe_account_id = null;
        $transfer_connect = null;
        if ($transaction_type == 'connect') {
            $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
            $stripe_account_id = $business->s_account_id;
            $transfer_connect = $business->transfer_connect;
            $statement_descriptor = $business->statement_descriptor;
            // $statement_descriptor = $business->s_account_id;
            $statement_description = $business->statement_description;
            if ($mode == 'test') {
                $stripeKeyColoumns = 'connect_stripe_private_key_test';
            }
            if ($mode == 'live') {
                $stripeKeyColoumns = 'connect_stripe_private_key_live';
            }
            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        }

        $fee = 0;
        $country_stripe_fixed = 0;
        $country_stripe_percentage = 0;
        $card_stripe_fixed = 0;
        $card_stripe_percentage = 0;

        $humanAmount = $req->input('amount') / 100;
        if ($transaction_type != null && $transaction_type != 'online' && $req->has('payment_method_id')) {
            $stripe_payment_method = $stripe->paymentMethods->retrieve($req->input('payment_method_id'));
            $card_brand = $stripe_payment_method->card->brand;
            $card_country = $stripe_payment_method->card->country;
            $country = Country::where('iso', $card_country)->first();
            if (!empty($country)) {
                $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
            }
            if (empty($countryCommission)) {
                $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($countryCommission)) {
                $country_stripe_percentage = $this->calculateFee($humanAmount, $countryCommission->stripe_percentage, 'percentage');
                $country_stripe_fixed = $countryCommission->stripe_fixed;
            }
            $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$card_brand, $card_brand])->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
            }
            if (empty($cardBrandCommission)) {
                $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($cardBrandCommission)) {
                $card_stripe_percentage = $this->calculateFee($humanAmount, $cardBrandCommission->stripe_percentage, 'percentage');
                $card_stripe_fixed = $cardBrandCommission->stripe_fixed;
            }
        }

        $fee = $this->calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type);
        $fee = $fee + $card_stripe_fixed + $card_stripe_percentage + $country_stripe_fixed + $country_stripe_percentage;

        if ($req->has('currency')) {
            $currency = $req->input('currency');
        } else {
            $currency = 'GBP';
        }

        $intentParams = [
            'amount' => (float) ($humanAmount * 100),
            'currency' => $currency,
            // 'confirm' => true,
        ];
        if ($terminalId != null) {
            $intentParams['payment_method_types'] = ['card_present'];
            $intentParams['capture_method'] = 'manual';
        }
        if ($autoConfirm) {
            $intentParams['confirm'] = true;
            // $intentParams['error_on_requires_action'] = true;
        } else {
            $intentParams['confirmation_method'] = 'automatic';
        }
        if ($req->has('payment_method_id')) {
            $intentParams['payment_method'] = $req->input('payment_method_id');
        }
        if ($statement_descriptor != null) {
            $intentParams['statement_descriptor'] = $statement_descriptor;
            $intentParams['description'] = $statement_description;
        }
        if ($req->input('customer_id') != null) {
            $intentParams['customer'] = $req->input('customer_id');
            $intentParams['save_payment_method'] = true;
        }
        if ($req->input('customer_email') != null) {
            $customerParams['email'] = $req->input('customer_email');
            if ($req->input('customer_name') != null) {
                $customerParams['name'] = $req->input('customer_name');
            }
            $customer = $stripe->customers->create($customerParams);
            $intentParams['customer'] = $customer->id;
            $intentParams['save_payment_method'] = true;
        }
        if ($transaction_type == 'connect' && $stripe_account_id != null && $transfer_connect == 'yes') {
            $intentParams['transfer_data'] = [
                'destination' => $stripe_account_id
            ];
            if ($business->business_commissions->auto_fee == 'yes') {
                $intentParams['application_fee_amount'] = $fee * 100;
            }
        }

        if ($req->has('is_credit_card_tt')) {
            $intentParams['metadata']['is_credit_card_tt'] = $req->input('is_credit_card_tt');
            $intentParams['metadata']['order_id'] = $req->input('order_id');
        }

        if ($req->has('is_moto_order')) {
            $intentParams['metadata']['is_moto_order'] = $req->input('is_moto_order');
            $intentParams['metadata']['order_id'] = $req->input('order_id');
        }

        $intent = $stripe->paymentIntents->create($intentParams);
        return ['payment_intent_id' => $intent->client_secret, 'payment_intent' => $intent, "status" => $intent->status];
    }
    public function card_reader_connection_token(Request $req)
    {
        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($stripeMode == 'test') {
            $stripeKeyColoumns = 'stripe_private_key_test';
        } else {
            $stripeKeyColoumns = 'stripe_private_key_live';
        }
        if ($req->has('business_id')) {
            $business_id = $req->input('business_id');
            $business = Business::find($business_id);
            if (empty($business)) {
                Utils::error("Business not found", 400);
            }
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                $mode = strtolower($mode);
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
            }
        }

        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        $connectionParams = [];
        if ($req->has('s_location_id') && !empty($req->input('s_location_id'))) {
            $connectionParams['location'] = $req->input('s_location_id');
        }
        $connectionToken = $stripe->terminal->connectionTokens->create($connectionParams);
        return $data = ['token' => $connectionToken->secret];
    }

    public function capture_payment_intent(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $stripeKeyColoumns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $stripeKeyColoumns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;

        $this->validate($req, [
            's_terminal_id' => 'string',
            'payment_intent_id' => 'string|required',
            'business_id' => 'string',
        ]);
        $terminalId = $req->input('s_terminal_id');
        $business_id = $req->input('business_id');

        if ($terminalId != null) {
            $businessCardReader = BusinessCardReader::where('s_terminal_id', $terminalId)->latest()->first();
            if (!empty($businessCardReader)) {
                $business_id = $businessCardReader->business_id;
            }
        }

        if ($business_id != null) {
            $business = Business::find($business_id);
        }
        if (!empty($business) && $business->s_account_id != null && $business->connect_service == 1) {
            $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
            if ($mode == 'test') {
                $stripeKeyColoumns = 'connect_stripe_private_key_test';
            }
            if ($mode == 'live') {
                $stripeKeyColoumns = 'connect_stripe_private_key_live';
            }
            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        }

        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $intent =  $stripe->paymentIntents->retrieve($req->input('payment_intent_id'));
        $intent->capture();
        $intent = $stripe->paymentIntents->retrieve($req->input('payment_intent_id'));

        return $intent;
    }
    public function calculate_application_fee(Request $req)
    {
        $this->validate($req, [
            'transaction_type' => 'required|in:online,merchant,connect',
            'business_id' => 'required_without_all:statement_descriptor,restaurant_id',
            'restaurant_id' => 'required_without_all:statement_descriptor,business_id',
            'statement_descriptor' => 'required_without_all:restaurant_id,business_id',

            'amount' => 'required|numeric|gt:0',
            'order_type' => 'required_if:transaction_type,online|in:dinein,pickup,delivery'
        ]);
        $humanAmount = $req->input('amount') / 100;
        $transaction_type = $req->input('transaction_type');
        $order_type = $req->input('order_type');
        $statement_descriptor = $req->input('statement_descriptor');
        $restaurant_id = $req->input('restaurant_id');
        $business_id = $req->input('business_id');
        $business = Business::where('id', $business_id)->orderByRaw('created_at DESC')->first();
        $card_stripe_fixed = 0;
        $card_stripe_percentage = 0;
        $country_stripe_fixed = 0;
        $country_stripe_percentage = 0;

        if ($business->connect_service == 1 && $business->s_account_id != null) {
            $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
            if ($mode == 'test') {
                $stripeKeyColoumns = 'connect_stripe_private_key_test';
            }
            if ($mode == 'live') {
                $stripeKeyColoumns = 'connect_stripe_private_key_live';
            }
        } else {
            $mode = SiteSettingController::findByKey('stripe_mode')->value;
            if ($mode == 'test') {
                $stripeKeyColoumns = 'stripe_private_key_test';
            }
            if ($mode == 'live') {
                $stripeKeyColoumns = 'stripe_private_key_live';
            }
        }

        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        if ($transaction_type != null && $transaction_type != 'online' && $req->has('payment_method_id')) {
            $stripe_payment_method = $stripe->paymentMethods->retrieve($req->input('payment_method_id'));
            $card_brand = $stripe_payment_method->card->brand;
            $card_country = $stripe_payment_method->card->country;

            $country = Country::where('iso', $card_country)->first();

            if (!empty($country)) {
                $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
            }
            if (empty($countryCommission)) {
                $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($countryCommission)) {
                $country_stripe_percentage = $this->calculateFee($humanAmount, $countryCommission->stripe_percentage, 'percentage');
                $country_stripe_fixed = $countryCommission->stripe_fixed;
            }
            $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$card_brand, $card_brand])->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
            }
            if (empty($cardBrandCommission)) {
                $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($cardBrandCommission)) {
                $card_stripe_percentage = $this->calculateFee($humanAmount, $cardBrandCommission->stripe_percentage, 'percentage');
                $card_stripe_fixed = $cardBrandCommission->stripe_fixed;
            }
        }
        if (!empty($business)) {
            $fee =  $this->calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type);
            $fee = $fee + $card_stripe_fixed + $card_stripe_percentage + $country_stripe_fixed + $country_stripe_percentage;
            return ['fee' => $fee];
        } else {
            Utils::error("No business found", 404);
        }
    }
    public function calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type)
    {
        $payment_method_fees = 0;
        $orderTypeWiseCommission = 0;
        $orderTypeWiseServiceCharge = 0;
        $orderBasedOrTransactionWiseCharge = 0;

        $cardAuthFee = 0;
        $cardProcessingFee = 0;
        $fee = 0;
        if ($transaction_type == 'online') {

            $payment_method_fees  = $this->calculateFee($humanAmount, $business->business_commissions->online_card_fee, $business->business_commissions->online_card_fee_type);

            if ($order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->dinein_commission_fee, $business->business_commissions->dinein_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->dinein_service_fee, $business->business_commissions->dinein_service_fee_type);
            } else {
                $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->online_commission_fee, $business->business_commissions->online_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->online_service_fee, $business->business_commissions->online_service_fee_type);
            }
            if ($business->business_commissions->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($humanAmount, $business->business_commissions->per_order_charge, $business->business_commissions->per_order_charge_type);
            } else {
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($humanAmount, $business->business_commissions->card_charge, $business->business_commissions->card_charge_type);
            }

            $fee = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $cardAuthFee + $cardProcessingFee);
        }
        if ($transaction_type == 'merchant') {
            $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_auth_fee, $business->business_commissions->merchant_card_auth_fee_type);
            $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_processing_fee, $business->business_commissions->merchant_card_processing_fee_type);

            $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
        }
        if ($transaction_type == 'connect') {
            $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_auth_fee, $business->business_commissions->connect_card_auth_fee_type);
            $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_processing_fee, $business->business_commissions->connect_card_processing_fee_type);

            $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
        }
        return $fee;
    }

    public function calculateFee($humanAmount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($humanAmount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }
    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
