<?php

namespace App\Http\Controllers\Connect;

use App\Helpers\ImageHelper;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\SiteSetting;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Stripe\Account;
use Stripe\File;
use Stripe\Person;
use Stripe\Stripe;
use Stripe\StripeClient;

class TestConnectAccountController extends Controller
{
    public function create(Request $req)
    {
        $this->validate($req, [
            'country' => 'required|string',
            'email' => 'required|email',

            'business.name' => 'required|string',
            'business.url' => 'required|string',
            'business.tax_id' => 'required|string',
            'business.mcc' => 'required|string',

            'legal_entity.type' => 'required|string',
            'legal_entity.phone_number' => 'required|string',

            'legal_entity.address.line' => 'required|string',
            'legal_entity.address.city' => 'required|string',
            'legal_entity.address.state' => 'required|string',
            'legal_entity.address.post_code' => 'required|string',

            'legal_entity.person.dob' => 'required|date',
            'legal_entity.person.first_name' => 'required|string',
            'legal_entity.person.last_name' => 'required|string',
            'legal_entity.person.email' => 'required|string',
            'legal_entity.person.title' => 'required|string',
            'legal_entity.person.owner' => 'required|boolean',
            'legal_entity.person.phone_number' => 'required|string',

            'legal_entity.person.address.line' => 'required|string',
            'legal_entity.person.address.city' => 'required|string',
            'legal_entity.person.address.state' => 'required|string',
            'legal_entity.person.address.post_code' => 'required|string',

            'external_account.country' => 'required|string',
            'external_account.currency' => 'required|string',
            'external_account.account_holder_type' => 'required|string',
            'external_account.account_holder_name' => 'required|string',
            'external_account.account_number' => 'required|string',
            'external_account.routing_number' => 'required|string',
        ]);

        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $dob = Carbon::parse($req->input('legal_entity')['person']['dob']);
        $params = [
            'type' => 'custom',
            'country' => $req->input('country'),
            'email' => $req->input('email'),
            'capabilities' => [
                'card_payments' => [
                    'requested' => true,
                ],
                'transfers' => [
                    'requested' => true,
                ],
            ],
            'tos_acceptance' => [
                'ip' => $req->getHost(),
                'date' => Carbon::today()->unix()
            ],
            'business_profile' => [
                'name' => $req->input('business')['name'],
                'url' => $req->input('business')['url'],
                'mcc' => $req->input('business')['mcc'],
            ],
            'legal_entity' => [
                'type' => $req->input('legal_entity')['type'],
                'dob' => [
                    'day' => $dob->day,
                    'month' => $dob->month,
                    'year' => $dob->year,
                ],
                'address' => [
                    'line1' => $req->input('legal_entity')['address']['line'],
                    'city' => $req->input('legal_entity')['address']['city'],
                    'state' => $req->input('legal_entity')['address']['state'],
                    'postal_code' => $req->input('legal_entity')['address']['post_code'],
                ],
                'business_name' => $req->input('business')['name'],
                'business_tax_id' => $req->input('business')['tax_id'],
                'personal_address' => [
                    'line1' => $req->input('legal_entity')['person']['address']['line'],
                    'city' => $req->input('legal_entity')['person']['address']['city'],
                    'state' => $req->input('legal_entity')['person']['address']['state'],
                    'postal_code' => $req->input('legal_entity')['person']['address']['post_code'],
                ],
                'first_name' => $req->input('legal_entity')['person']['first_name'],
                'last_name' => $req->input('legal_entity')['person']['last_name'],
                'personal_title' => $req->input('legal_entity')['person']['title'],
                'personal_email' => $req->input('legal_entity')['person']['email'],
                'personal_phone_number' => $req->input('legal_entity')['person']['phone_number'],
                'phone_number' => $req->input('legal_entity')['phone_number'],
                'personal_id_number' => $req->input('legal_entity')['person']['id_number'],
            ],
            'external_account' => [
                "object" => "bank_account",
                'country' => $req->input('external_account')['country'],
                'currency' => $req->input('external_account')['currency'],
                'account_holder_type' => $req->input('external_account')['account_holder_type'],
                'account_holder_name' => $req->input('external_account')['account_holder_name'],
                'account_number' => $req->input('external_account')['account_number'],
                'routing_number' => $req->input('external_account')['routing_number'],
            ]
        ];

        if ($req->input('legal_entity')['type'] == 'company') {
            $owner = [
                'first_name' => $req->input('legal_entity')['person']['first_name'],
                'last_name' => $req->input('legal_entity')['person']['last_name'],
                'email' => $req->input('legal_entity')['person']['email'],
                'address' => [
                    'city' => $req->input('legal_entity')['person']['address']['city'],
                    'line1' => $req->input('legal_entity')['person']['address']['line'],
                    'postal_code' => $req->input('legal_entity')['person']['address']['post_code'],
                    'state' => $req->input('legal_entity')['person']['address']['state'],
                ],
                'dob' => [
                    'day' => $dob->day,
                    'month' => $dob->month,
                    'year' => $dob->year,
                ], 'relationship' => [
                    'owner' => true,
                    'title' => 'CEO'
                ],
            ];

            $params['legal_entity']['executives_provided'] = true;
            $params['legal_entity']['owners_provided'] = true;
            $params['legal_entity']['directors_provided'] = true;
        }
        $account = $stripe->accounts->create($params);
        if (!empty($owner)) {
            $person = $stripe->accounts->createPerson($account->id, $owner);
        }
        $account = $stripe->accounts->retrieve($account->id);
        return $account;
    }
    public function update(Request $req, $account_id)
    {

        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        $updateParams = [
            'legal_entity' => [
                'executives_provided' => true
            ]
        ];

        $account = $stripe->accounts->update($account_id, $updateParams);

        return $account;
    }
    public function show(Request $req, $id)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $account = $stripe->accounts->retrieve($id, []);
        return $account;
    }
    public function delete(Request $req, $id)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $account = $stripe->accounts->delete($id, []);
        return $account;
    }
    public function enable_payment(Request $req, $id)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;

        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $account = $stripe->accounts->allCapabilities($id);
        return $account;
    }
    public function external_account(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $account = $stripe->accounts->createExternalAccount('acct_1IfLiVQQpuDBrETU', [
            'external_account' => [
                "object" => "bank_account",
                'country' => 'US',
                'currency' => 'USD',
                'account_holder_type' => 'company',
                'account_holder_name' => 'company name',
                'account_number' => '000123456789',
                "routing_number" => "110000000",
            ]
        ]);
        return $account;
    }
    public function external_accounts(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $account = $stripe->accounts->allExternalAccounts('acct_1IfLiVQQpuDBrETU', []);
        return $account;
    }
    public function create_person(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        $account = $stripe->accounts->createPerson('acct_1IfLiVQQpuDBrETU', [
            'first_name' => 'Snehal',
            'email' => 'snehal@y1opmail.com',
            'last_name' => 'Patel',
            'dob' => [
                'day' => 1,
                'month' => 1,
                'year' => 1990,
            ],
            'relationship' => [
                'director' => true,
                'title' => 'Ye Walla Director'
            ],
        ]);
        return $account;
    }
    public function persons(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $account = $stripe->accounts->allPersons($req->input('account_id'), []);
        return $account;
    }
    public function update_person(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        // $account = Person::update(
        $account = $stripe->accounts->updatePerson(
            'acct_1IflPiQN2rVNxUcD',
            'person_4IflPp00wHMiXq1m',
            [
                'email' => 'jsmes@yopmail.com',
                'relationship' => [
                    'title' => 'CEO 1'
                ]
            ]
        );
        return $account;
    }
    public function upload_file(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $file = $stripe->files->create([
            'purpose' => 'identity_document',
            'file' => fopen('C:/upload.png', 'r'),
        ], ['acct_1IfLiVQQpuDBrETU']);

        //file_1IfOPfKgAij6PRAu2lzNHszM
        //file_1IfOVeKgAij6PRAutXDmF3KN
        return $file;
    }
    public function validate_file(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $account = $stripe->accounts->updatePerson(
            'acct_1IfLiVQQpuDBrETU',
            'person_4IfOjB009iNqIFsJ',
            [
                'verification' => [
                    'document' => [
                        'front' => 'file_1IfOPfKgAij6PRAu2lzNHszM',
                        'back' => 'file_1IfOVeKgAij6PRAutXDmF3KN',
                    ],
                ]
            ]
        );

        //file_1IfOPfKgAij6PRAu2lzNHszM
        return $account;
    }
}
