<?php

namespace App\Http\Controllers;

use App\ContactGroup;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class ContactGroupController extends Controller
{
    /**
     * List contactGroups.
     * GET /api/contact-groups
     */
    public function index(Request $req)
    {
        $contactGroups = ContactGroup::orderByRaw('name DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $contactGroups->whereRaw('(name LIKE ? OR description LIKE ? )', ["%$query%", "%$query%"]);
        }

        if (Auth::isUser()) {
            $contactGroups->where('disabled', 0);
            $contactGroups->where('user_id', Auth::id());
        } else {
            $query = $req->query('disabled');
            if ($query != null) {
                $contactGroups->where('disabled', $query == 0 ? '0' : '1');
            }
            if ($query = $req->query('user_id', false)) {
                $contactGroups->where('user_id', $query);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $contactGroups = $contactGroups->get();
        } else {
            $contactGroups = $contactGroups->paginate($req->query('per_page', 20));
        }

        if ($contactGroups->isEmpty()) Utils::error('No contactGroups found!', 404);


        return ($contactGroups);
    }

    /**
     * Create contactGroup.
     * POST /api/contact-groups
     */
    public function create(Request $req)
    {
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'description' => 'string',
            'user_id' => 'required|exists:users,id'
        ]);

        // create contactGroup
        $contactGroup = new ContactGroup;
        $contactGroup->fill($req->all());

        $contactGroup->save();

        return $contactGroup;
    }

    /**
     * Show contactGroup.
     * GET /api/contact-groups/{id}
     */
    public function show(Request $req, $id)
    {
        $contactGroup = ContactGroup::find($id);
        if (empty($contactGroup)) Utils::error('ContactGroup not found!', 404);
        return $contactGroup;
    }

    /**
     * Update contactGroup.
     * POST /api/contact-groups/{id}
     */
    public function update(Request $req, $id)
    {
        $contactGroup = ContactGroup::find($id);
        if (empty($contactGroup)) Utils::error('ContactGroup not found!', 404);
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'name' => 'string|max:255',
            'description' => 'string',
            'user_id' => 'exists:users,id'
        ]);

        // update contactGroup
        if ($req->has('name')) $contactGroup->name = $req->input('name');
        if ($req->has('description')) $contactGroup->description = $req->input('description');
        if ($req->has('user_id')) $contactGroup->user_id = $req->input('user_id');

        $contactGroup->save();

        return $contactGroup;
    }


    /**
     * Delete contactGroup by id.
     * DELETE /api/contact-groups/{id}
     */
    public function delete(Request $req, $id)
    {
        $contactGroup = ContactGroup::find($id);
        if (empty($contactGroup)) Utils::error('ContactGroup not found!', 404);
        $contactGroup->delete();
        return $contactGroup;
    }
}
