<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\CountryCommission;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class CountryCommissionController extends Controller
{
    /**
     * List countryCommissions.
     * GET /api/country-commissions
     */
    public function index(Request $req)
    {
        $countryCommissions = CountryCommission::orderByRaw("other ASC");

        if ($query = $req->query('business_id', false)) {
            $countryCommissions->where('business_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $countryCommissions = $countryCommissions->get();
        } else {
            $countryCommissions = $countryCommissions->paginate($req->query('per_page', 20));
        }

        if ($countryCommissions->isEmpty()) Utils::error('No countryCommissions found!', 404);


        return ($countryCommissions);
    }

    /**
     * Create countryCommission.
     * POST /api/country-commissions
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'country_id' => 'required_if:other,0|exists:countries,id',
            'business_id' => 'required|exists:businesses,id',
            'other' => 'required_if:country_id,null',
            'paypal_percentage' => 'numeric',
            'paypal_fixed' => 'numeric',
            'stripe_percentage' => 'numeric',
            'stripe_fixed' => 'numeric',

        ]);

        // create countryCommission
        $countryCommission = new CountryCommission;
        $countryCommission->fill($req->all());

        if ($countryCommission->other == 1) {
            $dbCountries = CountryCommission::where('other', '1')->where('business_id', $countryCommission->business_id)->first();
            if (!empty($dbCountries)) {
                Utils::error("Other Country already added in commissions", 400);
            }
        } else {
            $dbCountries = CountryCommission::where('country_id', $countryCommission->country_id)->where('business_id', $countryCommission->business_id)->get();
            if (!$dbCountries->isEmpty()) {
                Utils::error("Country already added in commissions", 400);
            }
        }
        $countryCommission->save();

        return $countryCommission;
    }

    /**
     * Show countryCommission.
     * GET /api/country-commissions/{id}
     */
    public function show(Request $req, $id)
    {
        $countryCommission = CountryCommission::find($id);
        if (empty($countryCommission)) Utils::error('CountryCommission not found!', 404);
        return $countryCommission;
    }

    /**
     * Update countryCommission.
     * POST /api/country-commissions/{id}
     */
    public function update(Request $req, $id)
    {
        $countryCommission = CountryCommission::find($id);
        if (empty($countryCommission)) Utils::error('CountryCommission not found!', 404);

        $this->validate($req, [
            'country_id' => 'required_if:other,0|exists:countries,id',
            'business_id' => 'exists:businesses,id',
            'other' => 'required_if:country_id,null',
            'paypal_percentage' => 'numeric',
            'paypal_fixed' => 'numeric',
            'stripe_percentage' => 'numeric',
            'stripe_fixed' => 'numeric',

        ]);
        if ($req->input('other') == 1) {
            $dbCountries = CountryCommission::where('other', '1')->whereNotIn('id', [$id])->where('business_id', $countryCommission->business_id)->first();
            if (!empty($dbCountries)) {
                Utils::error("Other Country already added in commissions", 400);
            }
            $countryCommission->country_id = null;
        } else {
            $dbCountries = CountryCommission::where('country_id', $countryCommission->country_id)->whereNotIn('id', [$id])->where('business_id', $countryCommission->business_id)->get();
            if (!$dbCountries->isEmpty()) {
                Utils::error("Country already added in commissions", 400);
            }
            if ($req->has('country_id')) $countryCommission->country_id = $req->input('country_id');
        }
        // update countryCommission

        if ($req->has('business_id')) $countryCommission->business_id = $req->input('business_id');

        $countryCommission->other = $req->input('other');
        $countryCommission->paypal_percentage = $req->input('paypal_percentage');
        $countryCommission->paypal_fixed = $req->input('paypal_fixed');

        $countryCommission->stripe_percentage = $req->input('stripe_percentage');
        $countryCommission->stripe_fixed = $req->input('stripe_fixed');


        $countryCommission->save();

        return $countryCommission;
    }


    /**
     * Delete countryCommission by id.
     * DELETE /api/country-commissions/{id}
     */
    public function delete(Request $req, $id)
    {
        $countryCommission = CountryCommission::find($id);
        if (empty($countryCommission)) Utils::error('CountryCommission not found!', 404);
        $countryCommission->delete();
        return $countryCommission;
    }
}
