<?php

namespace App\Http\Controllers;

use App\BankingRequest;
use App\Business;
use App\CaxtonTransfer;
use App\CaxtonWalletTransaction;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\OnlineOrder;
use App\PayoutInvoice;
use App\PaypalPayment;
use App\PouchHistory;
use App\ResellerPayoutInvoice;
use App\StripeTransaction;
use App\User;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class Dashboard1Controller extends Controller
{
    public function get(Request $req)
    {
        $start_time = Carbon::now()->subHours(3)->timestamp;
        $end_time = Carbon::now()->timestamp;
        return [
            'start_time' => $start_time,
            'end_time' => $end_time
        ];
    }
    public function index(Request $req)
    {
        $start_time = Carbon::now()->subHours(3)->timestamp;
        $end_time = Carbon::now()->timestamp;

        $activeBusinesses = Business::where('disabled', '0')->count();
        $disabledBusinesses = Business::where('disabled', '1')->count();

        $onlineServiceProviders = Business::where('online_service', '1')->count();
        $merchantServiceProviders = Business::where('merchant_service', '1')->count();
        $eposServiceProviders = Business::where('epos_service', '1')->count();

        $activeResellers = User::where('reseller', '1')->where('disabled', '0')->count();
        $disabledResellers = User::where('reseller', '1')->where('disabled', '1')->count();

        $activeUsers = User::where('reseller', '0')->where('disabled', '0')->count();
        $disabledUsers = User::where('reseller', '0')->where('disabled', '1')->count();

        $today = Carbon::today();
        $lastDay = Carbon::parse($today)->subDays(7)->startOfDay();
        $disputeTransactions = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->where('reporting_category', 'dispute')->orderByRaw('stripe_transactions.created_at DESC');
        $disputeTransactions->where('created', '>', $lastDay->getTimestamp());
        $disputeTransactions = $disputeTransactions->get();

        $bankRequests  = BankingRequest::select("banking_requests.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->where('status', 'Pending')->orderByRaw('id DESC')->get();

        $pouchWeight = (new SiteSettingController())->findByKey('pouch_weight');

        $businessesPouch = Business::where('disabled', '0')->sum('pouch_weight');

        $externalBalance = (new CaxtonfxController())->admin_balance($req);

        // Day Wise Report Internal Load
        $todayInternalLoad = PouchHistory::selectRaw('COUNT(*) as total_count, COALESCE(SUM(credit_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfDay())->where('description', 'Wallet Credited')->where('is_admin', '0')->get();
        $weekInternalLoad = PouchHistory::selectRaw('COUNT(*) as total_count, COALESCE(SUM(credit_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfWeek()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfWeek()->endOfDay())->where('description', 'Wallet Credited')->where('is_admin', '0')->get();
        $monthInternalLoad = PouchHistory::selectRaw('COUNT(*) as total_count, COALESCE(SUM(credit_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfMonth()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfMonth()->endOfDay())->where('description', 'Wallet Credited')->where('is_admin', '0')->get();

        // Day Wise Report External Load
        $todayExternalLoad = CaxtonWalletTransaction::selectRaw('COUNT(*) as total_count, COALESCE(SUM(credit_amount),0) as amount')->where('transaction_date', '>=', Carbon::today()->startOfDay())->where('transaction_date', '<=', Carbon::today()->endOfDay())->where('description', 'Card Load')->get();
        $weekExternalLoad = CaxtonWalletTransaction::selectRaw('COUNT(*) as total_count, COALESCE(SUM(credit_amount),0) as amount')->where('transaction_date', '>=', Carbon::today()->startOfWeek()->startOfDay())->where('transaction_date', '<=', Carbon::today()->endOfWeek()->endOfDay())->where('description', 'Card Load')->get();
        $monthExternalLoad = CaxtonWalletTransaction::selectRaw('COUNT(*) as total_count, COALESCE(SUM(credit_amount),0) as amount')->where('transaction_date', '>=', Carbon::today()->startOfMonth()->startOfDay())->where('transaction_date', '<=', Carbon::today()->endOfMonth()->endOfDay())->where('description', 'Card Load')->get();

        // Day Wise Report Internal Bank
        $caxtonUserId = (new SiteSettingController())->findByKey('caxton_user_id')->value;
        $todayInternalBank = CaxtonTransfer::selectRaw('COUNT(*) as total_count, COALESCE(SUM(buy_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfDay())->where('user_id', $caxtonUserId)->get();
        $weekInternalBank = CaxtonTransfer::selectRaw('COUNT(*) as total_count, COALESCE(SUM(buy_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfWeek()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfWeek()->endOfDay())->where('user_id', $caxtonUserId)->get();
        $monthInternalBank = CaxtonTransfer::selectRaw('COUNT(*) as total_count, COALESCE(SUM(buy_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfMonth()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfMonth()->endOfDay())->where('user_id', $caxtonUserId)->get();

        // Day Wise Report External Bank
        $todayExternalBank = CaxtonTransfer::selectRaw('COUNT(*) as total_count, COALESCE(SUM(buy_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfDay())->where('user_id', '!=', $caxtonUserId)->get();
        $weekExternalBank = CaxtonTransfer::selectRaw('COUNT(*) as total_count, COALESCE(SUM(buy_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfWeek()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfWeek()->endOfDay())->where('user_id', '!=', $caxtonUserId)->get();
        $monthExternalBank = CaxtonTransfer::selectRaw('COUNT(*) as total_count, COALESCE(SUM(buy_amount),0) as amount')->where('created_at', '>=', Carbon::today()->startOfMonth()->startOfDay())->where('created_at', '<=', Carbon::today()->endOfMonth()->endOfDay())->where('user_id', '!=', $caxtonUserId)->get();

        // $expireDate = Carbon::today()->addDays(15)->format('Y-m-d');
        // $upcomingDomainExpireBusinesses = Business::where('domain_valid_till', '<=', $expireDate)->limit('10')->get();

        // $stripeTransactions  = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->whereNotNull('business_id')->where('reporting_category', '!=', 'payout')->orderByRaw('id DESC')->get();
        // $stripeOtherTransactions  = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->whereNull('business_id')->where('reporting_category', '!=', 'payout')->orderByRaw('id DESC')->get();
        // $paypalTransactions  = PaypalPayment::select("paypal_payments.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->orderByRaw('id DESC')->get();
        // $onlineOrders  = OnlineOrder::select("online_orders.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->orderByRaw('id DESC')->get();
        // $businessPayoutInvoices  = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->orderByRaw('to_date DESC, payout_invoices.created_at DESC')->limit('10')->get();
        // $resellerPayoutInvoices  =  ResellerPayoutInvoice::select("reseller_payout_invoices.*", "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->orderByRaw('to_date DESC, reseller_payout_invoices.created_at DESC')->limit(10)->get();

        // $lastFewTransactions = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->where('created', '>', $start_time)->where('created', '<', $end_time)->orderByRaw('stripe_transactions.created_at DESC')->get();

        // $duplicateTransactions = [];
        // foreach ($lastFewTransactions as $trans) {
        //     if ($trans->reporting_category != 'charge') {
        //         continue;
        //     }
        //     $transaction = StripeTransaction::whereNotIn('id', [$trans->id])->where('reporting_category', 'charge')->where('amount', $trans->amount)->where('card_number', $trans->card_number)->where('descriptor', $trans->descriptor)->where('created', '>', $start_time)->where('created', '<', $end_time)->orderByRaw('created_at DESC')->first();
        //     if (!empty($transaction)) {
        //         $duplicateTransactions[] = $transaction;
        //     }
        // }

        $data = [
            'dispute_transactions' => $disputeTransactions,
            'bank_requests' => $bankRequests,
            'today' => [
                'internal_load' => $todayInternalLoad[0],
                'external_load' => $todayExternalLoad[0],
                'internal_bank' => $todayInternalBank[0],
                'external_bank' => $todayExternalBank[0],
            ],
            'this_week' => [
                'internal_load' => $weekInternalLoad[0],
                'external_load' => $weekExternalLoad[0],
                'internal_bank' => $weekInternalBank[0],
                'external_bank' => $weekExternalBank[0],
            ],
            'this_month' => [
                'internal_load' => $monthInternalLoad[0],
                'external_load' => $monthExternalLoad[0],
                'internal_bank' => $monthInternalBank[0],
                'external_bank' => $monthExternalBank[0],
            ],
            // 'stripe_transactions' => $stripeTransactions,
            // 'stripe_transactions_without_business' => $stripeOtherTransactions,
            // 'paypal_transactions' => $paypalTransactions,
            // 'online_orders' => $onlineOrders,
            // 'payout_invoices' => $businessPayoutInvoices,
            // 'reseller_payout_invoices' => $resellerPayoutInvoices,
            // 'duplicate_transactions' => $duplicateTransactions,
            // 'upcoming_domain_expire_businesses' => $upcomingDomainExpireBusinesses,
            'stats' => [
                'active_businesses' => $activeBusinesses,
                'disabled_businesses' => $disabledBusinesses,
                'online_service_businesses' => $onlineServiceProviders,
                'merchant_service_businesses' => $merchantServiceProviders,
                'epos_service_businesses' => $eposServiceProviders,
                'active_resellers' => $activeResellers,
                'disabled_resellers' => $disabledResellers,
                'active_users' => $activeUsers,
                'disabled_users' => $disabledUsers,
                'internal_balance' => $pouchWeight->value,
                'business_internal_balance' => $businessesPouch,
                'external_balance' => $externalBalance,
            ]
        ];
        return $data;
    }

    public function domain_expire_business(Request $req)
    {
        $businesses = Business::orderByRaw('name ASC, created_at DESC');

        $expireDate = Carbon::today()->addDays(15)->format('Y-m-d');
        $businesses->where('domain_valid_till', '<=', $expireDate);

        if ($req->query('nopaginate', false) == '1') {
            $businesses = $businesses->get();
        } else {
            $businesses = $businesses->paginate($req->query('per_page', 20));
        }

        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);


        return ($businesses);
    }
}