<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\OnlineOrder;
use App\PayoutInvoice;
use App\PaypalPayment;
use App\ResellerPayoutInvoice;
use App\StripeTransaction;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class DashboardController extends Controller
{
    public function get(Request $req)
    {
        $start_time = Carbon::now()->subHours(3)->timestamp;
        $end_time = Carbon::now()->timestamp;
        return [
            'start_time' => $start_time,
            'end_time' => $end_time
        ];
    }
    public function index(Request $req)
    {
        $start_time = Carbon::now()->subHours(3)->timestamp;
        $end_time = Carbon::now()->timestamp;

        $activeBusinesses = Business::where('disabled', '0')->count();
        $disabledBusinesses = Business::where('disabled', '1')->count();

        $onlineServiceProviders = Business::where('online_service', '1')->count();
        $merchantServiceProviders = Business::where('merchant_service', '1')->count();

        $activeResellers = User::where('reseller', '1')->where('disabled', '0')->count();
        $disabledResellers = User::where('reseller', '1')->where('disabled', '1')->count();

        $activeUsers = User::where('reseller', '0')->where('disabled', '0')->count();
        $disabledUsers = User::where('reseller', '0')->where('disabled', '1')->count();

        $stripeTransactions  = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->orderByRaw('id DESC')->get();
        $paypalTransactions  = PaypalPayment::select("paypal_payments.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->orderByRaw('id DESC')->get();
        $onlineOrders  = OnlineOrder::select("online_orders.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->limit('10')->orderByRaw('id DESC')->get();
        $businessPayoutInvoices  = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->orderByRaw('to_date DESC, payout_invoices.created_at DESC')->limit('10')->get();
        $resellerPayoutInvoices  =  ResellerPayoutInvoice::select("reseller_payout_invoices.*", "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->orderByRaw('to_date DESC, reseller_payout_invoices.created_at DESC')->limit(10)->get();

        $lastFewTransactions = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->where('created', '>', $start_time)->where('created', '<', $end_time)->orderByRaw('stripe_transactions.created_at DESC')->get();

        $duplicateTransactions = [];
        foreach ($lastFewTransactions as $trans) {
            $transaction = StripeTransaction::whereNotIn('id', [$trans->id])->where('reporting_category', 'charge')->where('amount', $trans->amount)->where('card_number', $trans->card_number)->where('descriptor', $trans->descriptor)->where('created', '>', $start_time)->where('created', '<', $end_time)->orderByRaw('created_at DESC')->first();
            if (!empty($transaction)) {
                $duplicateTransactions[] = $transaction;
            }
        }

        $data = [
            'stripe_transactions' => $stripeTransactions,
            'paypal_transactions' => $paypalTransactions,
            'online_orders' => $onlineOrders,
            'payout_invoices' => $businessPayoutInvoices,
            'reseller_payout_invoices' => $resellerPayoutInvoices,
            'duplicate_transactions' => $duplicateTransactions,
            'stats' => [
                'active_businesses' => $activeBusinesses,
                'disabled_businesses' => $disabledBusinesses,
                'online_service_businesses' => $onlineServiceProviders,
                'merchant_service_businesses' => $merchantServiceProviders,
                'active_resellers' => $activeResellers,
                'disabled_resellers' => $disabledResellers,
                'active_users' => $activeUsers,
                'disabled_users' => $disabledUsers,
            ]
        ];
        return $data;
    }
}
