<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\DatabaseServer;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class DatabaseServerController extends Controller
{
    /**
     * List databaseServers.
     * GET /api/database-servers
     */
    public function index(Request $req)
    {
        $databaseServers = DatabaseServer::orderByRaw('database_servers.name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $databaseServers->whereRaw('(database_servers.name LIKE ? OR public_ip LIKE ? OR local_ip LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $databaseServers = $databaseServers->get();
        } else {
            $databaseServers = $databaseServers->paginate($req->query('per_page', 20));
        }

        if ($databaseServers->isEmpty()) Utils::error('No databaseServers found!', 404);

        $databaseServers->each(function ($items) {
            $items->append('business');
        });

        return ($databaseServers);
    }

    /**
     * Create databaseServer.
     * POST /api/database-servers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'local_ip' => 'required',
            'public_ip' => 'required',
            'username' => 'required',
            'port' => 'string|max:255',
        ]);

        // create databaseServer
        $databaseServer = new DatabaseServer;
        $databaseServer->fill($req->all());

        $databaseServer->save();

        return $databaseServer;
    }

    /**
     * Show databaseServer.
     * GET /api/database-servers/{id}
     */
    public function show(Request $req, $id)
    {
        $databaseServer = DatabaseServer::find($id);
        if (empty($databaseServer)) Utils::error('DatabaseServer not found!', 404);
        return $databaseServer;
    }

    /**
     * Update databaseServer.
     * POST /api/database-servers/{id}
     */
    public function update(Request $req, $id)
    {
        $databaseServer = DatabaseServer::find($id);
        if (empty($databaseServer)) Utils::error('DatabaseServer not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'local_ip' => 'string',
            'public_ip' => 'string',
            'username' => 'string',
            'password' => 'string',
            'port' => 'string',
        ]);

        // update databaseServer
        if ($req->has('name')) $databaseServer->name = $req->input('name');
        if ($req->has('local_ip')) $databaseServer->local_ip = $req->input('local_ip');
        if ($req->has('public_ip')) $databaseServer->public_ip = $req->input('public_ip');
        if ($req->has('username')) $databaseServer->username = $req->input('username');
        if ($req->has('port')) $databaseServer->port = $req->input('port');
        $databaseServer->password = $req->input('password');

        $databaseServer->save();

        return $databaseServer;
    }


    /**
     * Delete databaseServer by id.
     * DELETE /api/database-servers/{id}
     */
    public function delete(Request $req, $id)
    {
        $databaseServer = DatabaseServer::find($id);
        if (empty($databaseServer)) Utils::error('DatabaseServer not found!', 404);
        $databaseServer->delete();
        if ($databaseServer->image != null) ImageHelper::delete(ImageHelper::$TYPE_BRAND, $databaseServer->image);
        return $databaseServer;
    }
}
