<?php

namespace App\Http\Controllers;

use App\Business;
use App\DeviceRegistration;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomRestaurant;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class DeviceRegistrationController extends Controller
{
    /**
     * List deviceRegistrations.
     * GET /api/device-registrations
     */
    public function index(Request $req)
    {
        $deviceRegistrations = DeviceRegistration::selectRaw('device_registrations.*,businesses.name as business_name')->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $deviceRegistrations->whereRaw('(device_id LIKE ? OR device_registrations.name LIKE ? )', ["%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $deviceRegistrations->where('business_id', $query);
        }
        if ($query = $req->query('status', false)) {
            $deviceRegistrations->where('status', $query);
        }
        if ($query = $req->query('device_type', false)) {
            $deviceRegistrations->where('device_type', $query);
        }
        if ($query = $req->query('service_type', false)) {
            $deviceRegistrations->where('service_type', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $deviceRegistrations = $deviceRegistrations->get();
        } else {
            $deviceRegistrations = $deviceRegistrations->paginate($req->query('per_page', 20));
        }

        if ($deviceRegistrations->isEmpty()) Utils::error('No deviceRegistrations found!', 404);


        return ($deviceRegistrations);
    }

    /**
     * Create deviceRegistration.
     * POST /api/device-registrations
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'device_id' => 'required|string',
            'serial_number' => 'string',
            'device_type' => 'required|in:Android,iOS',
            'status' => 'required|in:Pending,Approved,Rejected',
            'business_id' => 'required|exists:businesses,id'
        ]);

        // create deviceRegistration
        $deviceRegistration = new DeviceRegistration;
        $deviceRegistration->fill($req->all());

        $deviceRegistration->save();

        return $deviceRegistration;
    }

    /**
     * Show deviceRegistration.
     * GET /api/device-registrations/{id}
     */
    public function show(Request $req, $id)
    {
        $deviceRegistration = DeviceRegistration::find($id);
        if (empty($deviceRegistration)) Utils::error('DeviceRegistration not found!', 404);
        return $deviceRegistration;
    }

    /**
     * Update deviceRegistration.
     * POST /api/device-registrations/{id}
     */
    public function update(Request $req, $id)
    {
        $deviceRegistration = DeviceRegistration::find($id);
        if (empty($deviceRegistration)) Utils::error('DeviceRegistration not found!', 404);

        $this->validate($req, [
            'device_id' => 'string',
            'serial_number' => 'string',
            'device_type' => 'in:Android,iOS',
            'status' => 'in:Pending,Approved,Rejected',
            'business_id' => 'exists:businesses,id'

        ]);

        // update deviceRegistration
        if ($req->has('device_id')) $deviceRegistration->device_id = $req->input('device_id');
        if ($req->has('serial_number')) $deviceRegistration->serial_number = $req->input('serial_number');
        if ($req->has('status')) $deviceRegistration->status = $req->input('status');
        if ($req->has('device_type')) $deviceRegistration->device_type = $req->input('device_type');
        if ($req->has('business_id')) $deviceRegistration->business_id = $req->input('business_id');

        $deviceRegistration->save();

        return $deviceRegistration;
    }


    /**
     * Delete deviceRegistration by id.
     * DELETE /api/device-registrations/{id}
     */
    public function delete(Request $req, $id)
    {
        $deviceRegistration = DeviceRegistration::find($id);
        if (empty($deviceRegistration)) Utils::error('DeviceRegistration not found!', 404);
        $deviceRegistration->delete();
        return $deviceRegistration;
    }

    public function enable_weborder_admin(Request $req, $id)
    {
        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('DeviceRegistration not found!', 404);
        if ($device->status == 'Pending') {
            Utils::error('sorry device is disabled!', 404);
        }
        if ($device->status == 'Rejected') {
            Utils::error('sorry device is rejected!', 404);
        }

        if ($req->input('web_order') == 0) {

            $device->web_order = 1;
            $device->save();

            if ($device->device_token != '') {
                $business = Business::find($device->business_id);
                $restaurant_id = $business->restaurant_id;
                if (empty($res_id) || $restaurant_id == 0) Utils::error('restaurant id not set in settings!', 404);

                $updateDevice = TiffintomRestaurant::where('id', $restaurant_id)->update(['device_id' => '']);
                $weborder_devices_list = DeviceRegistration::where('web_order', '1')->where('business_id', $business->id)->get();
                foreach ($weborder_devices_list as $weborder_device) {
                    $restaurant = TiffintomRestaurant::find($restaurant_id);
                    if (empty($restaurant)) Utils::error("No restaurant found", 404);
                    if ($weborder_device->device_token) {
                        if ($restaurant->device_id) {
                            $restaurant->device_id = $restaurant->device_id . ',' . $weborder_device->device_token;
                        } else {
                            $restaurant->device_id = $weborder_device->device_token;
                        }
                    }
                    $restaurant->save();
                }
            }
        } else {
            $device = DeviceRegistration::find($id);
            if (empty($device)) Utils::error('DeviceRegistration not found!', 404);
            $device->web_order = 0;
            $device->save();
        }
        return $device;
    }

    public function enable_callerid_admin(Request $req, $id)
    {
        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('DeviceRegistration not found!', 404);

        if ($req->input('caller_id') == 0) {
            $device->caller_id = 1;
            $device->save();
        } else {
            $device = DeviceRegistration::find($id);
            if (empty($device)) Utils::error('DeviceRegistration not found!', 404);
            $device->caller_id = 0;
            $device->save();
        }
        return $device;
    }
}
