<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\Exports\PayoutReportExport;
use App\FactoryResetLog;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposOrder;
use App\ModelsEpos\EposTable;
use App\PayoutInvoice;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;
use Stripe\Stripe;
use Stripe\StripeClient;

class EposArtisanController extends Controller
{
    /**
     * Test
     */
    public function test(Request $req)
    {
        $ipAddress = $req->ip();
        $ipAddress = $ipAddress . " & " . $req->getHttpHost();
        return $ipAddress;
    }
    /**
     * Create new schema/table for epos run following command. 
     * php artisan make:migration --path="database/epos" create_{table_name}_table 
     */

    public function schema(Request $req)
    {
        set_time_limit(160);
        $dbName  = "test_epost_database";
        $host  = "localhost";
        $username  = "root";
        $password = "";
        if ($req->has('database')) {
            $dbName = $req->query('database');
        }
        if ($req->has('host')) {
            $host = $req->query('host');
        }
        if ($req->has('username')) {
            $username = $req->query('username');
        }
        if ($req->has('password')) {
            $password = $req->query('password');
        }

        $path = "database/epos";
        $config = app()->make('config');
        $config->set('database.connections.epos', [
            'driver'   => 'mysql',
            'host'   => $host,
            'database'   => $dbName,
            'username'   => $username,
            'password'   => $password,
        ]);

        DB::statement("CREATE DATABASE IF NOT EXISTS $dbName");
        Artisan::call('migrate', [
            '--path' => $path,
            '--force' => true,
            '--database' => 'epos'
        ]);
        $tables = DB::connection('epos')->select("SHOW TABLES");
        return $tables;
    }
    public function refresh_schema(Request $req)
    {
        $dbName  = "test_epost_database";
        $host  = "localhost";
        $username  = "root";
        $password = "";
        if ($req->has('database')) {
            $dbName = $req->query('database');
        }
        if ($req->has('host')) {
            $host = $req->query('host');
        }
        if ($req->has('username')) {
            $username = $req->query('username');
        }
        if ($req->has('password')) {
            $password = $req->query('password');
        }

        $path = "database/epos";
        $config = app()->make('config');
        $config->set('database.connections.epos', [
            'driver'   => 'mysql',
            'host'   => $host,
            'database'   => $dbName,
            'username'   => $username,
            'password'   => $password,
        ]);

        Artisan::call('migrate:fresh', [
            '--path' => $path,
            '--force' => true,
            '--database' => 'epos'
        ]);
        return ["message" => "Refreshed"];
    }

    public function load_data(Request $req)
    {

        $dbName  = "testing_epos";
        $host  = "localhost";
        $username  = "root";
        $password = "";
        if ($req->has('database')) {
            $dbName = $req->query('database');
        }
        if ($req->has('host')) {
            $host = $req->query('host');
        }
        if ($req->has('username')) {
            $username = $req->query('username');
        }
        if ($req->has('password')) {
            $password = $req->query('password');
        }

        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $host,
            'database'   => $dbName,
            'username'   => $username,
            'password'   => $password,
        ]);
        try {
            Artisan::call('db:seed', [
                '--class' => 'EposSeeder',
                '--force' => true,
                '--database' => 'epos_connection'
            ]);
        } catch (Exception $e) {
            return ['message' => $e->getMessage()];
        }
        return ['message' => 'Seeding done!'];
    }

    public function factory_reset(Request $req)
    {
        ini_set('max_execution_time', 240);
        (new EposOrderController())->delete_all($req);
        (new EposReservationController())->delete_all($req);
        (new EposCallLogController())->delete_all($req);
        (new EposCashdrawLogController())->delete_all($req);
        EposCustomer::where('name', '')->where('mobile', '')->delete();
        EposCustomer::where('name', '')->where('mobile', '0')->delete();
        EposCustomer::where('mobile', '')->where('email', '')->where('house_no', '')->where('street', '')->where('postcode', '')->delete();
        EposTable::whereNotNull('merge_table_id')->update(['merge_table_id' => NULL]);
        EposTable::whereNotNull('id')->update(['locked' => '0']);
        (new EposExternalPaymentController())->delete_all($req);

        $factoryResetLog = new FactoryResetLog;
        $factoryResetLog->business_id = @$req->business_id;
        $factoryResetLog->user_id = @$req->user_id;
        $factoryResetLog->device_id = @$req->device_id;
        $factoryResetLog->device_name = @$req->device_name;
        $factoryResetLog->sync_mode = @$req->sync_mode;
        $factoryResetLog->save();

        return ['message' => 'factory reset done! '. json_encode($req->all())];
    }

    public function epos_statement(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        $payoutInvoices = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->orderByRaw('to_date DESC, payout_invoices.created_at DESC');

        $business = Business::find(Auth::$business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $payoutInvoices->where('business_id', $business->id);

        if ($query = $req->query('from_date', false)) {
            $payoutInvoices->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $payoutInvoices->where('to_date', '<=', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $payoutInvoices = $payoutInvoices->get();
        } else {
            $payoutInvoices = $payoutInvoices->paginate($req->query('per_page', 20));
        }

        if ($payoutInvoices->isEmpty()) Utils::error('No payoutInvoices found!', 404);

        return ($payoutInvoices);
    }

    /**
     * Download payoutInvoice.
     * GET /epos/statement/{id}/download
     */
    public function payout_invoice_download(Request $req, $id)
    {
        $payoutInvoice = PayoutInvoice::select("payout_invoices.*", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->find($id);
        if (empty($payoutInvoice)) Utils::error('PayoutInvoice not found!', 404);
        $payoutInvoice->append('transactions');
        $payoutInvoice->append('business');
        $type = 'excel';
        if ($format = $req->query('format', false)) {
            $type = $format;
        }
        $fromDate = Carbon::parse($payoutInvoice['from_date']);
        $toDate = Carbon::parse($payoutInvoice['to_date']);
        $fileName = $payoutInvoice['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');

        // if ($type == 'excel')
        //     return Excel::download(new PayoutReportExport($payoutInvoice), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        // else if ($type == 'pdf') {
        return Excel::download(new PayoutReportExport($payoutInvoice), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        // } else {
        //     Utils::error("Invalid export format", 400);
        // }
    }

    /**
     * Create Reader.
     * POST /epos/create-reader
     */
    public function readerCreate(Request $req)
    {
        Stripe::setApiKey("sk_test_5J87rf5MsZJYIK36Jt9bveuI");
        \Stripe\Terminal\Reader::create([
            'registration_code' => "puppies-plug-could",
            'label' => "Demo",
        ]);

        return 'Reader Created!';
    }

    /**
     * Create Payment.
     * POST /epos/create-payment
     */
    public function paymentIntentCreate(Request $req)
    {
        Stripe::setApiKey("sk_test_5J87rf5MsZJYIK36Jt9bveuI");
        $intent = \Stripe\PaymentIntent::create([
            'amount' => 1000,
            'currency' => 'GBP',
            'payment_method_types' => ['card_present'],
            'capture_method' => 'manual',
        ]);
        return $intent->client_secret;
    }
}
