<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposCallLog;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class EposCallLogController extends Controller
{
    /**
     * List call_logs.
     * GET /api/call_logs
     */
    public function index(Request $req)
    {
        $call_logs = EposCallLog::selectRaw('call_logs.*,customers.name as customer_name')->leftJoin('customers', 'customers.id', '=', 'customer_id')->orderByRaw('call_logs.created_at DESC');

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after:from_date',
        ]);

        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";

            $call_logs = $call_logs->where('call_logs.created_at', '>=', "$from_date");
            $call_logs = $call_logs->where('call_logs.created_at', '<=', "$to_date");
        }

        if ($query = $req->query('query', false)) {
            $call_logs->whereRaw('(call_logs.number LIKE ?  )', ["%$query%"]);
        }
        if ($status = $req->query('status', false)) {
            $call_logs->where('call_logs.status', "$status");
        }
        if ($req->query('nopaginate', false) == '1') {
            $call_logs = $call_logs->get();
        } else {
            $call_logs = $call_logs->paginate($req->query('per_page', 20));
        }

        if ($call_logs->isEmpty()) Utils::error('No call logs found!', 404);


        return ($call_logs);
    }

    /**
     * Create call_log.
     * POST /api/call_logs
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'number' => 'required|string|max:255',
            'customer_id' => 'nullable|exists:epos_connection.customers,id',

            'status' => 'nullable|in:pending,rejected,accepted,parked',
        ]);

        // create call_log
        $call_log = new EposCallLog;
        $call_log->fill($req->all());
        if (!empty($req->input('customer_id'))) {
            if ($req->has('customer_id')) $call_log->customer_id = $req->input('customer_id');
        } else {
            $call_log->customer_id = null;
        }
        $call_log->save();

        return $call_log;
    }

    /**
     * Show call_log.
     * GET /api/call_logs/{id}
     */
    public function show(Request $req, $id)
    {
        $call_log = EposCallLog::find($id);
        if (empty($call_log)) Utils::error('EposCallLog not found!', 404);
        return $call_log;
    }

    /**
     * Update call_log.
     * POST /api/call_logs/{id}
     */
    public function update(Request $req, $id)
    {
        $call_log = EposCallLog::find($id);
        if (empty($call_log)) Utils::error('EposCallLog not found!', 404);

        $this->validate($req, [
            'number' => 'string|max:255',
            'customer_id' => 'nullable|exists:epos_connection.customers,id',

            'status' => 'nullable|in:pending,rejected,accepted,parked',
        ]);

        // update call_log
        if ($req->has('number')) $call_log->number = $req->input('number');
        if ($req->has('customer_id') && !empty($req->input('customer_id'))) $call_log->customer_id = $req->input('customer_id');
        if ($req->has('device_id')) $call_log->device_id = $req->input('device_id');
        if ($req->has('status')) $call_log->status = $req->input('status');

        $call_log->save();

        return $call_log;
    }


    /**
     * Delete call_log by id.
     * DELETE /api/call_logs/{id}
     */
    public function delete(Request $req, $id)
    {
        $call_log = EposCallLog::find($id);
        if (empty($call_log)) Utils::error('EposCallLog not found!', 404);
        $call_log->delete();
        return $call_log;
    }

    /**
     * Delete All reservation.
     * GET All/epos/call-logs/delete-all
     */
    public function delete_all(Request $req)
    {
        (new EposCallLog)->newQueryWithoutScopes()->forceDelete();
        DB::connection('epos_connection')->statement("ALTER TABLE call_logs AUTO_INCREMENT =  0");
        return ['message' => 'call logs reset done!'];
    }
}
