<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposCashdrawLog;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class EposCashdrawLogController extends Controller
{
    /**
     * List cashdraw_logs.
     * GET /api/cashdraw_logs
     */
    public function index(Request $req)
    {
        $cashdraw_logs = EposCashdrawLog::selectRaw('cashdraw_logs.*,users.username as user_name')->leftJoin('users', 'users.id', '=', 'user_id')->orderByRaw('cashdraw_logs.created_at DESC');

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after:from_date',
        ]);

        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";

            $cashdraw_logs = $cashdraw_logs->where('cashdraw_logs.created_at', '>=', "$from_date");
            $cashdraw_logs = $cashdraw_logs->where('cashdraw_logs.created_at', '<=', "$to_date");
        }

        if ($query = $req->query('query', false)) {
            $cashdraw_logs->whereRaw('(cashdraw_logs.value LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $cashdraw_logs = $cashdraw_logs->get();
        } else {
            $cashdraw_logs = $cashdraw_logs->paginate($req->query('per_page', 20));
        }

        if ($cashdraw_logs->isEmpty()) Utils::error('No cashdraw logs found!', 404);


        return ($cashdraw_logs);
    }

    /**
     * Create cashdraw_log.
     * POST /api/cashdraw_logs
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'value' => 'required|string|max:255',
            'user_id' => 'nullable|exists:epos_connection.users,id',

        ]);

        // create cashdraw_log
        $cashdraw_log = new EposCashdrawLog;
        $cashdraw_log->fill($req->all());
        if (!empty($req->input('user_id'))) {
            if ($req->has('user_id')) $cashdraw_log->user_id = $req->input('user_id');
        } else {
            $cashdraw_log->user_id = null;
        }
        $cashdraw_log->save();

        return $cashdraw_log;
    }

    /**
     * Show cashdraw_log.
     * GET /api/cashdraw_logs/{id}
     */
    public function show(Request $req, $id)
    {
        $cashdraw_log = EposCashdrawLog::find($id);
        if (empty($cashdraw_log)) Utils::error('EposCashdrawLog not found!', 404);
        return $cashdraw_log;
    }

    /**
     * Update cashdraw_log.
     * POST /api/cashdraw_logs/{id}
     */
    public function update(Request $req, $id)
    {
        $cashdraw_log = EposCashdrawLog::find($id);
        if (empty($cashdraw_log)) Utils::error('EposCashdrawLog not found!', 404);

        $this->validate($req, [
            'value' => 'required|string|max:255',
            'user_id' => 'nullable|exists:epos_connection.users,id',

        ]);

        // update cashdraw_log
        if ($req->has('value')) $cashdraw_log->value = $req->input('value');
        if ($req->has('user_id') && !empty($req->input('user_id'))) $cashdraw_log->user_id = $req->input('customer_id');
        if ($req->has('device_id')) $cashdraw_log->device_id = $req->input('device_id');

        $cashdraw_log->save();

        return $cashdraw_log;
    }


    /**
     * Delete cashdraw_log by id.
     * DELETE /api/cashdraw_logs/{id}
     */
    public function delete(Request $req, $id)
    {
        $cashdraw_log = EposCashdrawLog::find($id);
        if (empty($cashdraw_log)) Utils::error('EposCashdrawLog not found!', 404);
        $cashdraw_log->delete();
        return $cashdraw_log;
    }

    /**
     * Delete All cashdraw.
     * GET All/epos/cashdraw-logs/delete-all
     */
    public function delete_all(Request $req)
    {
        (new EposCashdrawLog)->newQueryWithoutScopes()->forceDelete();
        DB::connection('epos_connection')->statement("ALTER TABLE cashdraw_logs AUTO_INCREMENT =  0");
        return ['message' => 'cashdraw logs reset done!'];
    }
}
