<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposCustomer;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposCustomerController extends Controller
{
    /**
     * List customers.
     * GET /epos/customers
     */
    public function index(Request $req)
    {
        $customers = EposCustomer::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $customers->whereRaw('(name LIKE ? OR mobile LIKE ? OR available_credit LIKE ? OR membership_no LIKE ?  )', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $customers = $customers->get();
        } else {
            $customers = $customers->paginate($req->query('per_page', 20));
        }

        if ($customers->isEmpty()) Utils::error('No customers found!', 404);


        return ($customers);
    }

    /**
     * Create customer.
     * POST /epos/customers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'string',
            'mobile' => 'string',
            'email' => 'string',
            'landline' => 'string',
            'birth_date' => 'date',
            'anniversary_date' => 'date',
            'house_no' => 'string',
            'street' => 'string',
            'postcode' => 'string',
            'distance' => 'string',
            'city' => 'string',
            'country' => 'string',
            'state' => 'string',
            'membership_no' => 'string',
            'membership_points' => 'numeric',
            'available_credit' => 'numeric',
            'comment' => 'string',
            'unique_id' => 'string',
        ]);

        // create customer
        $customer = new EposCustomer;
        $customer->fill($req->all());
        $customer->membership_no = rand(100000000, 999999999);
        $customer->save();
        if (Auth::isUser())
            $customer->creator_id = Auth::id();
        $customer->refresh();

        return $customer;
    }

    /**
     * Show customer.
     * GET /epos/customers/{id}
     */
    public function show(Request $req, $id)
    {
        $customer = EposCustomer::find($id);
        if (empty($customer)) Utils::error('EposCustomer not found!', 404);
        return $customer;
    }

    /**
     * Update customer.
     * POST /epos/customers/{id}
     */
    public function update(Request $req, $id)
    {
        $customer = EposCustomer::find($id);
        if (empty($customer)) Utils::error('EposCustomer not found!', 404);

        $this->validate($req, [
            'name' => 'string',
            'mobile' => 'string',
            'email' => 'string',
            'landline' => 'string',
            'birth_date' => 'date',
            'anniversary_date' => 'date',
            'house_no' => 'string',
            'street' => 'string',
            'postcode' => 'string',
            'distance' => 'string',
            'city' => 'string',
            'country' => 'string',
            'state' => 'string',
            'membership_no' => 'string',
            'membership_points' => 'numeric',
            'available_credit' => 'numeric',
            'comment' => 'string',
            'news_letter' => 'boolean',
            'unique_id' => 'string',
        ]);

        // update customer
        if ($req->has('name')) $customer->name = $req->input('name');
        if ($req->has('mobile')) $customer->mobile = $req->input('mobile');
        if ($req->has('email')) $customer->email = $req->input('email');
        if ($req->has('landline')) $customer->landline = $req->input('landline');
        if ($req->has('birth_date')) $customer->birth_date = $req->input('birth_date');
        if ($req->has('anniversary_date')) $customer->anniversary_date = $req->input('anniversary_date');
        if ($req->has('house_no')) $customer->house_no = $req->input('house_no');
        if ($req->has('street')) $customer->street = $req->input('street');
        if ($req->has('postcode')) $customer->postcode = $req->input('postcode');
        if ($req->has('distance')) $customer->distance = $req->input('distance');
        if ($req->has('city')) $customer->city = $req->input('city');
        if ($req->has('country')) $customer->country = $req->input('country');
        if ($req->has('state')) $customer->state = $req->input('state');
        if ($req->has('membership_no')) $customer->membership_no = $req->input('membership_no');
        if ($req->has('membership_points')) $customer->membership_points = $req->input('membership_points');
        if ($req->has('available_credit')) $customer->available_credit = $req->input('available_credit');
        if ($req->has('comment')) $customer->comment = $req->input('comment');
        if ($req->has('news_letter')) $customer->news_letter = $req->input('news_letter');
        if ($req->has('unique_id')) $customer->unique_id = $req->input('unique_id');
        if (Auth::isUser())
            $customer->updater_id = Auth::id();
        $customer->save();

        return $customer;
    }


    /**
     * Delete customer by id.
     * DELETE /epos/customers/{id}
     */
    public function delete(Request $req, $id)
    {
        $customer = EposCustomer::find($id);
        if (empty($customer)) Utils::error('EposCustomer not found!', 404);
        $customer->delete();
        return $customer;
    }

    /**
     * Delete customer by id.
     * POST /epos/customers/delete-bulk
     */
    public function deleteBulk(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            foreach ($req->input('ids') as $id) {
                $customer = EposCustomer::find($id);
                if (!empty($customer)) $customer->delete();
            }
        }
        return $customer;
    }
}
