<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use Carbon\Carbon;
use App\Http\Controllers\Controller;
use App\Http\Controllers\PayoutReportController;
use App\MiscCharge;
use App\MiscInvoice;
use App\ModelsEpos\EposOrder;
use App\OnlineOrder;
use App\Passbook;
use App\PayoutCharge;
use App\PayoutInvoice;
use App\StripeTransaction;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposDashboardController extends Controller
{
    public function index(Request $req)
    {

        $business_id = Auth::$business_id;
        $business = Business::where('id', $business_id)->first();

        $currentWeekDay = Carbon::now()->dayOfWeek;
        if ($currentWeekDay <= 1) {
            $upcoming_startdate = Carbon::today()->startOfWeek(7)->subDays(12)->format('Y-m-d');
            $upcoming_enddate = Carbon::parse($upcoming_startdate)->addDays(6)->format('Y-m-d');
        } else {
            $upcoming_startdate = Carbon::today()->startOfWeek(7)->subDays(5)->format('Y-m-d');
            $upcoming_enddate = Carbon::parse($upcoming_startdate)->addDays(6)->format('Y-m-d');
        }

        $most_previous_startdate = Carbon::parse($upcoming_startdate)->subDays(7)->format('d-m-Y');
        $most_previous_enddate = Carbon::parse($most_previous_startdate)->addDays(6)->format('d-m-Y');
        $previous_startdate = Carbon::parse($most_previous_startdate)->subDays(7)->format('d-m-Y');
        $previous_enddate = Carbon::parse($previous_startdate)->addDays(6)->format('d-m-Y');


        //Most Previous Invoice Net Total
        $most_previous_payoutInvoices = PayoutInvoice::orderByRaw('to_date DESC,created_at DESC');
        $most_previous_payoutInvoices->where('business_id', $business_id);
        $most_previous_payoutInvoices->where('from_date', '>=', $most_previous_startdate);
        $most_previous_payoutInvoices->where('to_date', '<=', $most_previous_enddate);
        $most_previous_payoutInvoices = $most_previous_payoutInvoices->first();
        if (empty($most_previous_payoutInvoices)) {
            $most_previous_net_total = 0.00;
        } else {
            $most_previous_net_total = $most_previous_payoutInvoices['net_total'];
        }

        //Previous Invoice Net Total
        $previous_payoutInvoices = PayoutInvoice::orderByRaw('to_date DESC,created_at DESC');
        $previous_payoutInvoices->where('business_id', $business_id);
        $previous_payoutInvoices->where('from_date', '>=', $previous_startdate);
        $previous_payoutInvoices->where('to_date', '<=', $previous_enddate);
        $previous_payoutInvoices = $previous_payoutInvoices->first();
        if (empty($previous_payoutInvoices)) {
            $previous_net_total = 0.00;
        } else {
            $previous_net_total = $previous_payoutInvoices['net_total'];
        }

        //Upcoming Invoice net total
        $carbon = Carbon::today()->subDays(7);
        $currentDayOfWeek = $carbon->dayOfWeek;
        if ($currentDayOfWeek <= 1) {
            $invoice_from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
            $invoice_to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
        } else {
            $invoice_from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
            $invoice_to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
        }

        $req->merge(['from_date' => $invoice_from_date, 'to_date' => $invoice_to_date, 'is_job' => true]);
        $upcoming_report = (new PayoutReportController())->quick_report($req, $business_id);
        if (!empty($upcoming_report) && $upcoming_report != 'Saved') {
            $upcoming_net_total = $upcoming_report['normal']['net_total'];
        } else {
            $upcoming_net_total = 0.00;
        }

        //Top Product List With Percentage
        $ordersProductCount = EposOrder::selectRaw("CAST(sum(order_items.total) AS DECIMAL(7,2)) as total")->leftJoin('order_items', 'order_items.order_id', '=', 'orders.id')->orderByRaw('total DESC')->get();
        $ordersByProduct = EposOrder::selectRaw("products.name As product_name,products.description As description,COUNT(order_items.id) as count,CAST(sum(order_items.total) AS DECIMAL(7,2)) as total,products.image as report_product_image")->leftJoin('order_items', 'order_items.order_id', '=', 'orders.id')->leftJoin('products', 'products.id', '=', 'order_items.product_id')->groupBy('order_items.product_id')->orderByRaw('total DESC');
        $ordersByProduct->where('is_archived', "0");
        $ordersByProduct = $ordersByProduct->whereNotNull('order_items.product_id');
        $ordersByProduct = $ordersByProduct->limit('5')->get();

        //Append Image And Percentage By Calculate amount
        foreach ($ordersByProduct as $ordersByProducts) {
            $ordersByProducts->append('report_product_image_url');
            $ordersByProducts['percentage'] = $this->convertToDecimal($ordersByProducts->total * 100 / $ordersProductCount[0]['total']);
        }

        $onlineOrdersRevenue  = OnlineOrder::selectRaw("DATE_FORMAT(CAST(order_date as DATE),'%M %Y') as month_string, DATE_FORMAT(CAST(order_date as DATE),'%m') as month, ROUND(SUM(grand_total),2) as revenue, COUNT(id) as count")->groupByRaw("DATE_FORMAT(CAST(order_date as DATE),'%Y%m')")->get();
        $eposDeliveryOrdersRevenue  = EposOrder::where("order_type_id", "3")->selectRaw("DATE_FORMAT(CAST(created_at as DATE),'%M %Y') as month_string, DATE_FORMAT(CAST(created_at as DATE),'%Y%m') as month, ROUND(SUM(total),2) as revenue, COUNT(id) as count")->groupByRaw("DATE_FORMAT(CAST(created_at as DATE),'%Y%m')")->get();
        $eposCollectionOrdersRevenue  = EposOrder::where("order_type_id", "2")->selectRaw("DATE_FORMAT(CAST(created_at as DATE),'%M %Y') as month_string, DATE_FORMAT(CAST(created_at as DATE),'%Y%m') as month, ROUND(SUM(total),2) as revenue, COUNT(id) as count")->groupByRaw("DATE_FORMAT(CAST(created_at as DATE),'%Y%m')")->get();
        $eposWaitingOrdersRevenue  = EposOrder::where("order_type_id", "5")->selectRaw("DATE_FORMAT(CAST(created_at as DATE),'%M %Y') as month_string, DATE_FORMAT(CAST(created_at as DATE),'%Y%m') as month, ROUND(SUM(total),2) as revenue, COUNT(id) as count")->groupByRaw("DATE_FORMAT(CAST(created_at as DATE),'%Y%m')")->get();

        $data = [
            'business' => $business,
            'top_product_total' => $ordersProductCount[0]['total'],
            'top_product' => $ordersByProduct,
            'previous_net_total' => $previous_net_total,
            'most_previous_net_total' => $most_previous_net_total,
            'previous_net_total' => $previous_net_total,
            'upcoming_net_total' => $upcoming_net_total,
            'online_orders_revenue' => $onlineOrdersRevenue,
            'epos_delivery_orders_revenue' => $eposDeliveryOrdersRevenue,
            'epos_collection_orders_revenue' => $eposCollectionOrdersRevenue,
            'epos_waiting_orders_revenue' => $eposWaitingOrdersRevenue,
        ];
        return $data;
    }

    public function top_product(Request $req)
    {

        //Top Product List With Percentage
        $ordersProductCount = EposOrder::selectRaw("CAST(sum(order_items.total) AS DECIMAL(7,2)) as total")->leftJoin('order_items', 'order_items.order_id', '=', 'orders.id')->orderByRaw('total DESC')->get();
        $ordersByProduct = EposOrder::selectRaw("products.name As product_name,products.description As description,COUNT(order_items.id) as count,CAST(sum(order_items.total) AS DECIMAL(7,2)) as total,products.image as report_product_image")->leftJoin('order_items', 'order_items.order_id', '=', 'orders.id')->leftJoin('products', 'products.id', '=', 'order_items.product_id')->groupBy('order_items.product_id')->orderByRaw('total DESC');
        $ordersByProduct->where('is_archived', "0");
        $ordersByProduct = $ordersByProduct->whereNotNull('order_items.product_id');
        $ordersByProduct = $ordersByProduct->get();

        //Append Image And Percentage By Calculate amount
        foreach ($ordersByProduct as $ordersByProducts) {
            $ordersByProducts->append('report_product_image_url');
            $ordersByProducts['percentage'] = $this->convertToDecimal($ordersByProducts->total * 100 / $ordersProductCount[0]['total']);
        }

        $data = [
            'full_top_product_total' => $ordersProductCount[0]['total'],
            'full_top_product' => $ordersByProduct,
        ];
        return $data;
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
