<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposAddon;
use App\ModelsEpos\EposCategory;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposDepositType;
use App\ModelsEpos\EposDiscount;
use App\ModelsEpos\EposFloor;
use App\ModelsEpos\EposIngredient;
use App\ModelsEpos\EposOrderStatus;
use App\ModelsEpos\EposPaymentMethod;
use App\ModelsEpos\EposPrepLocation;
use App\ModelsEpos\EposPrintBlock;
use App\ModelsEpos\EposProduct;
use App\ModelsEpos\EposProductAddon;
use App\ModelsEpos\EposProductIngredient;
use App\ModelsEpos\EposReservation;
use App\ModelsEpos\EposReservationType;
use App\ModelsEpos\EposSetting;
use App\ModelsEpos\EposTable;
use App\ModelsEpos\EposTableStatus;
use App\ModelsEpos\EposUser;
use App\ModelsEpos\EposVoucher;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use stdClass;

class EposDatabaseController extends Controller
{
    public function pull(Request $req)
    {
        ini_set('max_execution_time', 240);

        if ($req->has('date')) {
            $created_date = Carbon::createFromTimestamp($req->input('date'))->toDateTimeString();
        }

        // Addons
        $addons  = EposAddon::orderByRaw('sequence ASC');
        if ($req->has('date')) {
            $addons = $addons->where('created_at', '>=', "$created_date");
        }
        $addons->where('disabled', 0);
        $addons = $addons->get();
        $addons->each(function ($addon) {
            $addon->makeHidden('children');
        });

        // ingredients
        // $ingredients  = EposIngredient::orderByRaw('sequence ASC, created_at DESC');
        // if ($req->has('date')) {
        //     $ingredients = $ingredients->where('created_at', '>=', "$created_date");
        // }
        // $ingredients = $ingredients->get();

        // prep_location
        $prep_location  = EposPrepLocation::selectRaw("preparation_locations.*,printers.ip as printer_ip")->leftJoin('printers', 'printers.id', '=', 'printer_id')->orderByRaw('preparation_locations.id DESC');
        if ($req->has('date')) {
            $prep_location = $prep_location->where('preparation_locations.created_at', '>=', "$created_date");
        }
        $prep_location = $prep_location->get();

        // customers
        $customers  = EposCustomer::orderByRaw('id DESC');
        if ($req->has('date')) {
            $customers = $customers->where('created_at', '>=', "$created_date");
        }
        $customers = $customers->get();

        // categories
        $categories  = EposCategory::orderByRaw('sequence ASC');
        if ($req->has('date')) {
            $categories = $categories->where('created_at', '>=', "$created_date");
        }
        $categories = $categories->get();
        $categories->each(function ($category) {
            $category->makeHidden('children');
            $category->append('assign_print_block');
            $category->append('print_block_id');
        });


        // products
        $products  = EposProduct::selectRaw("*,(SELECT COUNT(*) FROM product_addons WHERE product_addons.product_id = products.id AND product_addons.disabled=0) as total_addons")->orderByRaw('products.favourite DESC,products.sequence ASC');
        if ($req->has('date')) {
            $products = $products->where('created_at', '>=', "$created_date");
        }
        $products->where('misc', 0);
        $products = $products->get();
        foreach ($products as $product) {
            // $productAddons = EposProductAddon::select('product_addons.*', 'addons.name as addon_name')->leftJoin('addons', 'addons.id', '=', 'product_addons.addon_id')->where('product_id', $product->id)->latest()->get();
            $productAddons = EposProductAddon::where('product_id', $product->id)->where('disabled', '0')->orderByRaw('sequence ASC')->get();
            if (!$productAddons->isEmpty()) {
                $product->product_addons = $productAddons;
            }
            // if (!$productAddons->isEmpty()) {
            //     $productAddons->each(function ($productAddon) {
            //         $productAddon->append('addon');
            //     });
            //     $product->product_addons = $productAddons;
            // }
            // $productIngredients = EposProductIngredient::select('ingredient_id')->where('product_id', $product->id)->get();
            // if (!$productIngredients->isEmpty()) {
            //     $product->product_ingredients = $productIngredients;
            // }
            $product->append('product_ingredients');
        }

        // Floors
        $floors  = EposFloor::orderByRaw('id DESC');
        if ($req->has('date')) {
            $floors = $floors->where('created_at', '>=', "$created_date");
        }
        $floors = $floors->get();
        $floors->append('tables');
        $floors->each(function ($floor) {
            $floor->tables->each(function ($table) {
                $table->makeHidden('table_status');
            });
        });

        // Tables
        $table_statuses  = EposTableStatus::orderByRaw('id DESC');
        if ($req->has('date')) {
            $table_statuses = $table_statuses->where('created_at', '>=', "$created_date");
        }
        $table_statuses = $table_statuses->get();

        // Order Status
        $order_statuses  = EposOrderStatus::orderByRaw('id DESC');
        if ($req->has('date')) {
            $order_statuses = $order_statuses->where('created_at', '>=', "$created_date");
        }
        $order_statuses = $order_statuses->get();

        // Users
        $users  = EposUser::orderByRaw('id DESC')->get();
        $users = $users->each(function ($user) {
            $user->makeVisible(['password']);
        });

        // Sitesettings
        $settings  = EposSetting::orderByRaw('id ASC')->get();
        $settings[] = (new SiteSettingController())->findByKey('epos_android_app_version');
        $settings[] = (new SiteSettingController())->findByKey('epos_ios_app_version');

        //Business Detail
        if ($req->has('business_id')) {
            $business = Business::where('epos_service', 1)->where('id', $req->input('business_id'))->first();
            if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {
                $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
                $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
            }
            if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {
                $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
                $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
            }
            if ($business->s_account_id != null) {
                $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;
                if ($connectMode == 'test') {
                    $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                    $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
                }
                if ($connectMode == 'live') {
                    $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                    $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
                }
            }
            $business->makeHidden(['business_commissions', 'balance', 'user_counts']);
            $business->append('card_readers');
            $business->append('printers');
            $business->append('admin_permissions');
            $business->append('supervisor_permissions');
        } else {
            $business = new stdClass();
        }

        // Payment Methods
        $payment_method  = EposPaymentMethod::orderByRaw('id ASC')->get();

        //Print Blocks
        $print_blocks  = EposPrintBlock::orderByRaw('id ASC');
        if ($req->has('date')) {
            $print_blocks = $print_blocks->where('created_at', '>=', "$created_date");
        }
        $print_blocks = $print_blocks->get();

        //Print Blocks
        $deposit_types  = EposDepositType::orderByRaw('id ASC');
        if ($req->has('date')) {
            $deposit_types = $deposit_types->where('created_at', '>=', "$created_date");
        }
        $deposit_types = $deposit_types->get();

        // discounts
        $discounts  = EposDiscount::orderByRaw('id DESC');
        if ($req->has('date')) {
            $discounts = $discounts->where('created_at', '>=', "$created_date");
        }
        $discounts = $discounts->get();

        // vouchers
        $vouchers  = EposVoucher::selectRaw('vouchers.*,customers.name as customer_name,customers.email as customer_email,customers.mobile as customer_mobile')->leftJoin('customers', 'customers.id', '=', 'customer_id')->orderByRaw('vouchers.id DESC');
        if ($req->has('date')) {
            $vouchers = $vouchers->where('vouchers.created_at', '>=', "$created_date");
        }
        $vouchers = $vouchers->get();

        $reservations = EposReservation::selectRaw('reservations.*,customers.name as customer_name,tables.number as table_number,reservation_statuses.status as reservation_status,deposit_types.type as deposit_type')->leftJoin('deposit_types', 'deposit_types.id', '=', 'deposit_type_id')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('reservation_statuses', 'reservation_statuses.id', '=', 'reservation_status_id')->orderByRaw('reservations.created_at DESC');
        $reservations = $reservations->get();
        if (!$reservations->isEmpty()) {
            $reservations->append('customer');
            $reservations->append('table');
        }
        $data = [
            'addons' => $addons,
            // 'ingredients' => $ingredients,
            'prep_locations' => $prep_location,
            'customers' => $customers,
            'categories' => $categories,
            'products' => $products,
            'floors' => $floors,
            'table_statuses' => $table_statuses,
            'order_statuses' => $order_statuses,
            'users' => $users,
            'vouchers' => $vouchers,
            'site_settings' => $settings,
            'business' => $business,
            'payment_methods' => $payment_method,
            'print_blocks' => $print_blocks,
            'deposit_types' => $deposit_types,
            'discounts' => $discounts,
            'reservations' => $reservations
        ];

        return $data;
    }

    public function push(Request $req)
    {

        // return $data;
    }
}
