<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposDepositType;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposDepositTypeController extends Controller
{
    /**
     * List DepositType.
     * GET /epos/order-types
     */
    public function index(Request $req)
    {
        $DepositType = EposDepositType::orderByRaw('type ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $DepositType->whereRaw('(type LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $DepositType = $DepositType->get();
        } else {
            $DepositType = $DepositType->paginate($req->query('per_page', 20));
        }

        if ($DepositType->isEmpty()) Utils::error('No DepositType found!', 404);


        return ($DepositType);
    }

    /**
     * Create DepositType.
     * POST /epos/order-types
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'type' => 'required|string|max:255',

        ]);

        // create DepositType
        $DepositType = new EposDepositType;
        $DepositType->fill($req->all());

        $DepositType->save();

        return $DepositType;
    }

    /**
     * Show DepositType.
     * GET /epos/order-types/{id}
     */
    public function show(Request $req, $id)
    {
        $DepositType = EposDepositType::find($id);
        if (empty($DepositType)) Utils::error('EposDepositType not found!', 404);
        return $DepositType;
    }

    /**
     * Update DepositType.
     * POST /epos/order-types/{id}
     */
    public function update(Request $req, $id)
    {
        $DepositType = EposDepositType::find($id);
        if (empty($DepositType)) Utils::error('EposDepositType not found!', 404);

        $this->validate($req, [
            'type' => 'string|max:255',

        ]);

        // update DepositType
        if ($req->has('type')) $DepositType->type = $req->input('type');

        $DepositType->save();

        return $DepositType;
    }


    /**
     * Delete DepositType by id.
     * DELETE /epos/order-types/{id}
     */
    public function delete(Request $req, $id)
    {
        $DepositType = EposDepositType::find($id);
        if (empty($DepositType)) Utils::error('EposDepositType not found!', 404);
        $DepositType->delete();
        return $DepositType;
    }
}
