<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\BusinessCardReader;
use App\BusinessTip;
use App\DeviceRegistration;
use App\Helpers\Auth;
use App\ModelsEpos\EposDevice;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsTiffintom\TiffintomRestaurant;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class EposDeviceController extends Controller
{
    /**
     * List devices.
     * GET /api/devices
     */
    public function index(Request $req)
    {
        // $devices = EposDevice::orderByRaw('created_at DESC');
        $devices = DeviceRegistration::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $devices->whereRaw('(device_id LIKE ? )', ["%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $devices->where('business_id', $query);
        }
        if ($query = $req->query('device_type', false)) {
            $devices->where('device_type', $query);
        }
        if ($query = $req->query('status', false)) {
            $devices->where('status', $query);
        }
        $devices->where('service_type', 'Epos');

        if ($req->query('nopaginate', false) == '1') {
            $devices = $devices->get();
        } else {
            $devices = $devices->paginate($req->query('per_page', 20));
        }

        if ($devices->isEmpty()) Utils::error('No devices found!', 404);


        return ($devices);
    }

    /**
     * Create device.
     * POST /api/devices
     */
    public function create(Request $req)
    {
        if (Auth::role() == 'epos_user')
            $req->merge(['business_id' => Auth::$business_id]);
        $this->validate($req, [
            'device_id' => 'required|string|max:255',
            'device_token' => 'string|max:255',
            'device_type' => 'string|max:255',
            'name' => 'string|max:255',
            'business_id' => 'required'
        ]);

        $dbDevice = DeviceRegistration::where('device_id', $req->input('device_id'))->where('business_id', Auth::$business_id)->where('service_type', 'Epos')->orderBy('created_at', 'DESC')->first();
        if (!empty($dbDevice)) {
            $dbDevice->name = $req->input('name');
            if ($req->has('device_type')) $dbDevice->device_type = $req->input('device_type');
            $dbDevice->service_type = 'Epos';
            if ($dbDevice->business_id == '1025') $dbDevice->status = 'Approved';
            $dbDevice->save();
            $dbDevice->refresh();
            return $dbDevice;
        }
        // create device
        $device = new DeviceRegistration;
        $device->fill($req->all());
        $device->device_type =  $req->has('device_type') ? $req->input('device_type') : "Android";
        if ($req->input('business_id') == '1025') $device->status = 'Approved';
        $device->service_type = 'Epos';
        $device->save();
        $device->refresh();

        return $device;
    }

    /**
     * Show device.
     * GET /api/devices/{id}
     */
    public function show(Request $req, $id)
    {
        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        return $device;
    }
    /**
     * validate device.
     * GET /api/devices/validate/{device_id}
     */
    public function validate_device(Request $req, $device_id)
    {
        $device = DeviceRegistration::where('device_id', $device_id)->where('business_id', Auth::$business_id)->where('service_type', 'Epos')->orderBy('created_at', 'DESC')->first();
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        return $device;
    }
    /**
     * validate device.
     * POST /api/devices/validate/{device_id}
     */
    public function validate_device_token(Request $req, $device_id)
    {
        $device = DeviceRegistration::where('device_id', $device_id)->where('business_id', Auth::$business_id)->where('service_type', 'Epos')->orderBy('created_at', 'DESC')->first();
        if (empty($device)) Utils::error('EposDevice not found!', 404);

        $this->validate($req, [
            'device_id' => 'string|max:255',
            'device_token' => 'string|max:255',
            'device_type' => 'string|max:255',
            'name' => 'string|max:255',
        ]);

        // update device
        if ($req->has('device_id')) $device->device_id = $req->input('device_id');
        if ($req->has('device_token')) $device->device_token = $req->input('device_token');
        if ($req->has('device_type')) $device->device_type = $req->input('device_type');
        if ($req->has('name')) $device->name = $req->input('name');

        $device->save();
        $device->refresh();

        return $device;
    }
    /**
     * Update device.
     * POST /api/devices/{id}
     */
    public function update(Request $req, $id)
    {
        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);

        if ($device->web_order && $device->device_token != $req->input('device_token')) {
            $res_id = (new EposSettingController())->findByKey('res_id');
            $restaurant_id = $res_id->value;
            if (!empty($res_id) || $restaurant_id != 0) {

                // $response = (new Client())->request('POST', env('FETCH_REST_DETAIL'), [
                //     'form_params' => ['action' => 'EditStoreAllDetail', 'user_id' => $restaurant_id, 'device_id' => $device->device_token]
                // ]);

                $restaurant = TiffintomRestaurant::find($restaurant_id);
                if (empty($restaurant)) Utils::error("No restaurant found", 404);
                $restaurant->device_id = $restaurant->device_id . ',' . $device->device_token;
                $restaurant->save();
            }
        }


        if (Auth::role() == 'epos_user')
            $req->merge(['business_id' => Auth::$business_id]);
        $this->validate($req, [
            'device_id' => 'required|string|max:255',
            'device_token' => 'string|max:255',
            'device_type' => 'string|max:255',
            'name' => 'string|max:255',
            'business_id' => 'required'
        ]);

        if ($req->input('status') == 'Approved') {
            // $allow_permission = (new EposSettingController())->findByKey('device_permission_allow');
            // if (empty($allow_permission)) Utils::error('allow permission not set in settings!', 404);

            $allow_device = DeviceRegistration::where('status', 'Pending')->count();

            $allow_permission = Business::where('id', Auth::$business_id)->first();
            // $allow_permission = (int)$allow_permission->epos_device_allow;
            if ($allow_device < $allow_permission->epos_device_allow) {
                // update device
                if ($req->has('device_id')) $device->device_id = $req->input('device_id');
                if ($req->has('device_token')) $device->device_token = $req->input('device_token');
                if ($req->has('device_type')) $device->device_type = $req->input('device_type');
                if ($req->has('status')) $device->status = $req->input('status');
                $device->service_type = 'Epos';
                if ($req->has('name')) $device->name = $req->input('name');
                if ($req->has('web_order')) $device->web_order = $req->input('web_order');
                $device->save();
                $device->refresh();
            } else {
                Utils::error('Allow permission limit is over!', 404);
            }
        } else {
            // update device
            if ($req->has('device_id')) $device->device_id = $req->input('device_id');
            if ($req->has('device_token')) $device->device_token = $req->input('device_token');
            if ($req->has('device_type')) $device->device_type = $req->input('device_type');
            if ($req->has('status')) $device->status = $req->input('status');
            $device->service_type = 'Epos';
            if ($req->has('name')) $device->name = $req->input('name');
            if ($req->has('web_order')) $device->web_order = $req->input('web_order');
            $device->save();
            $device->refresh();
        }

        return $device;
    }


    /**
     * Delete device by id.
     * DELETE /api/devices/{id}
     */
    public function delete(Request $req, $id)
    {
        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        $device->delete();
        return $device;
    }
    /**
     * Enable device.
     * POST /epos/devices/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $allow_permission = (new EposSettingController())->findByKey('device_permission_allow');
        if (empty($allow_permission)) Utils::error('allow permission not set in settings!', 404);

        $allow_device = DeviceRegistration::where('disabled', 0)->count();

        $allow_permission = (int)$allow_permission->value;
        if ($allow_device < $allow_permission) {
            $device = DeviceRegistration::find($id);
            if (empty($device)) Utils::error('EposDevice not found!', 404);
            $device->disabled = 0;
            $device->save();
            return $device;
        } else {
            Utils::error('Allow permission limit is over!', 404);
        }
    }

    /**
     * Disable device.
     * POST /epos/devices/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        $device->disabled = 1;
        $device->save();
        return $device;
    }

    /**
     * Enable device.
     * POST /epos/devices/{id}/enable-weborder
     */
    public function enable_weborder(Request $req, $id)
    {

        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('EposDevice not found!', 404);
        if ($device->status == 'Pending') {
            Utils::error('sorry device is pending!', 404);
        }
        if ($device->status == 'Rejected') {
            Utils::error('sorry device is rejected!', 404);
        }

        if ($req->input('web_order') == 0) {
            $weborder_devices = DeviceRegistration::where('web_order', 1)->get();
            if (!empty($weborder_devices)) {
                foreach ($weborder_devices as $weborder_device) {
                    $device_web = DeviceRegistration::find($weborder_device['id']);
                    $device_web->web_order = 0;
                    $device_web->save();
                }
            }

            $device->web_order = 1;
            $device->save();

            if (!empty($device->device_token) || $device->device_token != '') {
                $res_id = (new EposSettingController())->findByKey('res_id');
                $restaurant_id = $res_id->value;
                if (empty($res_id) || $restaurant_id == 0) Utils::error('restaurant id not set in settings!', 404);

                $updateDevice = TiffintomRestaurant::where('id', $restaurant_id)->update(['device_id' => '']);
                $weborder_devices_list = DeviceRegistration::where('web_order', 1)->get();
                foreach ($weborder_devices_list as $weborder_device) {
                    $restaurant = TiffintomRestaurant::find($restaurant_id);
                    if (empty($restaurant)) Utils::error("No restaurant found", 404);
                    if ($weborder_device->device_token) {
                        if ($restaurant->device_id) {
                            $restaurant->device_id = $restaurant->device_id . ',' . $weborder_device->device_token;
                        } else {
                            $restaurant->device_id = $weborder_device->device_token;
                        }
                    }
                    $restaurant->save();
                }
            }
        } else {
            $device = DeviceRegistration::find($id);
            if (empty($device)) Utils::error('EposDevice not found!', 404);
            $device->web_order = 0;
            $device->save();
        }
        return $device;
    }

    public function enable_weborder_admin(Request $req, $id)
    {

        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('DeviceRegistration not found!', 404);
        if ($device->status == 'Pending') {
            Utils::error('sorry device is pending!', 404);
        }
        if ($device->status == 'Rejected') {
            Utils::error('sorry device is rejected!', 404);
        }


        if ($req->input('web_order') == 0) {

            $device->web_order = 1;
            $device->save();

            if (!empty($device->device_token) || $device->device_token != '') {
                // $res_id = (new EposSettingController())->findByKey('res_id');
                // $restaurant_id = $res_id->value;
                $business = Business::select('restaurant_id')->where('id', $device->business_id)->first();
                $restaurant_id = $business->restaurant_id;
                if (empty($business) || $restaurant_id == 0) Utils::error('restaurant id not set in settings!', 404);

                $updateDevice = TiffintomRestaurant::where('id', $restaurant_id)->update(['device_id' => '']);
                $weborder_devices_list = DeviceRegistration::where('business_id', $device->business_id)->where('web_order', 1)->get();
                foreach ($weborder_devices_list as $weborder_device) {
                    $restaurant = TiffintomRestaurant::find($restaurant_id);
                    if (empty($restaurant)) Utils::error("No restaurant found", 404);
                    if ($weborder_device->device_token) {
                        if ($restaurant->device_id) {
                            $restaurant->device_id = $restaurant->device_id . ',' . $weborder_device->device_token;
                        } else {
                            $restaurant->device_id = $weborder_device->device_token;
                        }
                    }
                    $restaurant->save();
                }
            }
        } else {
            $device = DeviceRegistration::find($id);
            if (empty($device)) Utils::error('EposDevice not found!', 404);
            $device->web_order = 0;
            $device->save();
        }
        return $device;
    }

    public function enable_callerid_admin(Request $req, $id)
    {

        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('DeviceRegistration not found!', 404);

        if ($req->input('caller_id') == 0) {
            $device->caller_id = 1;
            $device->save();
        } else {
            $device = DeviceRegistration::find($id);
            if (empty($device)) Utils::error('EposDevice not found!', 404);
            $device->caller_id = 0;
            $device->save();
        }
        return $device;
    }

    public function enable_sunmi_printer_admin(Request $req, $id)
    {

        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('DeviceRegistration not found!', 404);

        if ($req->input('sunmi_printer_enable') == 0) {
            $device->sunmi_printer_enable = 1;
            $device->save();
        } else {
            $device = DeviceRegistration::find($id);
            if (empty($device)) Utils::error('EposDevice not found!', 404);
            $device->sunmi_printer_enable = 0;
            $device->save();
        }
        return $device;
    }

    public function enable_tap_pay_admin(Request $req, $id)
    {
        $device = DeviceRegistration::find($id);
        if (empty($device)) Utils::error('DeviceRegistration not found!', 404);

        if ($req->input('is_tap_to_pay') == 0) {
            $device->is_tap_to_pay = 1;
            $device->save();

            $businessCardReaders = BusinessCardReader::where('business_id', $device->business_id)->where('connectivity', 'TAPTOPAY')->first();
            if (empty($businessCardReaders)) {
                $businessCardReader = new BusinessCardReader;
                $businessCardReader->business_id =  $device->business_id;
                $businessCardReader->name = 'Tap To Pay';
                $businessCardReader->connectivity =  'TAPTOPAY';
                $businessCardReader->save();

                $business = Business::find($device->business_id);
                if ($business->s_account_id != null && $business->connect_service == 1) {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $stripeKeyColoumns = 'connect_stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $stripeKeyColoumns = 'connect_stripe_private_key_live';
                    }
                } else {
                    $mode = SiteSettingController::findByKey('stripe_mode')->value;
                    if ($mode == 'test') {
                        $stripeKeyColoumns = 'stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $stripeKeyColoumns = 'stripe_private_key_live';
                    }
                }

                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
                $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                $locationParams = [
                    'display_name' => $business->name,
                    'address' => [
                        'line1' => $business->door_no . " " . $business->address,
                        'city' => $business->city,
                        'state' => $business->state,
                        'country' => $business->country->iso,
                        'postal_code' => $business->postcode
                    ]
                ];
                if ($business->s_account_id != null && $business->connect_service == 1) {
                    $locationParams['metadata'] = [
                        'connected_account' => $business->s_account_id
                    ];
                }
                // create location
                $stripeLocation = $stripe->terminal->locations->create($locationParams);

                $tips = BusinessTip::where('business_id', $business->id)->select('tip')->toBase()->get()->pluck('tip')->toArray();
                if (count($tips) > 3) {
                    $configuration = $stripe->terminal->configurations->create([
                        'tipping' => ['gbp' => ['fixed_amounts' => array_map('intval', array_values($tips))]],
                    ]);

                    $stripe->terminal->locations->update(
                        $stripeLocation->id,
                        [
                            'configuration_overrides' => $configuration->id
                        ]
                    );

                    $businessCardReader->s_location_id = $stripeLocation->id;
                    $businessCardReader->save();
                    $business->s_location_id = $stripeLocation->id;
                    $business->save();
                }
            }
        } else {
            $device->is_tap_to_pay = 0;
            $device->save();
        }

        return $device;
    }
}
