<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposExternalPayment;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class EposExternalPaymentController extends Controller
{
    /**
     * List external-payments.
     * GET /api/external-payments
     */
    public function index(Request $req)
    {
        $external_payments = EposExternalPayment::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $external_payments->whereRaw('(payment_method_name LIKE ?  )', ["%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $external_payments = $external_payments->get();
        } else {
            $external_payments = $external_payments->paginate($req->query('per_page', 20));
        }

        if ($external_payments->isEmpty()) Utils::error('No external payments found!', 404);


        return ($external_payments);
    }

    /**
     * Create external-payment.
     * POST /api/external-payments
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'payment_method_id' => 'required|string|max:255',
            'payment_method_name' => 'required|string|max:255',
            'amount' => 'required|numeric|gt:0',
            'status' => 'in:Success,Pending,Failed',
        ]);

        // create external_payment
        $external_payment = new EposExternalPayment;
        $external_payment->fill($req->all());
        $external_payment->save();

        return $external_payment;
    }

    /**
     * Show external_payment.
     * GET /api/external-payments/{id}
     */
    public function show(Request $req, $id)
    {
        $external_payment = EposExternalPayment::find($id);
        if (empty($external_payment)) Utils::error('External payment not found!', 404);
        $external_payment->append('tables');
        return $external_payment;
    }

    /**
     * Update external_payment.
     * POST /api/external-payments/{id}
     */
    public function update(Request $req, $id)
    {
        $external_payment = EposExternalPayment::find($id);
        if (empty($external_payment)) Utils::error('External payment not found!', 404);

        $this->validate($req, [
            'payment_method_id' => 'required|string|max:255',
            'payment_method_name' => 'required|string|max:255',
            'amount' => 'required|numeric|gt:0',
            'status' => 'in:Success,Pending,Failed',
        ]);

        // update external_payment
        if ($req->has('payment_method_id')) $external_payment->payment_method_id = $req->input('payment_method_id');
        if ($req->has('payment_method_name')) $external_payment->payment_method_name = $req->input('payment_method_name');
        if ($req->has('amount')) $external_payment->amount = $req->input('amount');
        if ($req->has('status')) $external_payment->status = $req->input('status');
        $external_payment->save();

        return $external_payment;
    }


    /**
     * Delete external_payment by id.
     * DELETE /api/external-payments/{id}
     */
    public function delete(Request $req, $id)
    {
        $external_payment = EposExternalPayment::find($id);
        if (empty($external_payment)) Utils::error('External payment not found!', 404);
        $external_payment->delete();
        return $external_payment;
    }

    /**
     * Delete All external-payments.
     * GET All/epos/external-payments/delete-all
     */
    public function delete_all(Request $req)
    {
        (new EposExternalPayment)->newQueryWithoutScopes()->forceDelete();
        DB::connection('epos_connection')->statement("ALTER TABLE external_payments AUTO_INCREMENT =  0");

        return ['message' => 'external payments reset done!'];
    }
}
