<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposFloor;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposFloorController extends Controller
{
    /**
     * List floors.
     * GET /api/floors
     */
    public function index(Request $req)
    {
        $floors = EposFloor::orderByRaw('sequence ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $floors->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $floors = $floors->get();
        } else {
            $floors = $floors->paginate($req->query('per_page', 20));
        }

        if ($floors->isEmpty()) Utils::error('No floors found!', 404);


        return ($floors);
    }

    /**
     * Create floor.
     * POST /api/floors
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'sequence' => 'numeric|gt:0',
        ]);

        // create floor
        $floor = new EposFloor;
        $floor->fill($req->all());
        $floor->save();

        // Sequence add
        $floor->sequence = $floor->id;
        $floor->save();

        return $floor;
    }

    /**
     * Show floor.
     * GET /api/floors/{id}
     */
    public function show(Request $req, $id)
    {
        $floor = EposFloor::find($id);
        if (empty($floor)) Utils::error('EposFloor not found!', 404);
        $floor->append('tables');
        return $floor;
    }

    /**
     * Update floor.
     * POST /api/floors/{id}
     */
    public function update(Request $req, $id)
    {
        $floor = EposFloor::find($id);
        if (empty($floor)) Utils::error('EposFloor not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'sequence' => 'numeric|gt:0',

        ]);

        // update floor
        if ($req->has('name')) $floor->name = $req->input('name');
        if ($req->has('sequence')) $floor->sequence = $req->input('sequence');

        $floor->save();

        return $floor;
    }


    /**
     * Delete floor by id.
     * DELETE /api/floors/{id}
     */
    public function delete(Request $req, $id)
    {
        $floor = EposFloor::find($id);
        if (empty($floor)) Utils::error('EposFloor not found!', 404);
        $floor->delete();
        return $floor;
    }

    public function floor_bulk(Request $req)
    {
        if ($req->has('floors') && !is_array($req->input('floors'))) {
            $req->merge(['floors' => json_decode($req->input('floors'))]);
        }
        $this->validate($req, [
            'floors' => 'required|array'
        ],);
        $floors = ($req->input('floors'));
        $count = 1;
        foreach ($floors as $table) {
            $tableObj = EposFloor::find($table->id);
            $tableObj->sequence = $count;
            $tableObj->save();
            $count++;
        }
        return $floors;
    }
}
