<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\BusinessCardReader;
use App\BusinessPrinter;
use App\Helpers\Auth;
use App\Helpers\OtpHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\Http\Controllers\UserController;
use App\ModelsEpos\EposDevice;
use App\User;
use App\UserSelectedBusiness;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;

class EposLoginController extends Controller
{
    /**
     * Login to Epos.
     * POST epos/login
     */
    public function login(Request $req)
    {
        $this->validate($req, [
            'username' => 'required',
            'password' => 'required',
        ]);
        $user = User::where('login_email', $req->input('username'))->first();
        if (empty($user)) Utils::error("Invalid login details", 400);

        if ($user->disabled == 1) {
            $otp = rand(100000, 999999);
            $user->otp = $otp;
            $user->save();
            $message = 'Your one-time password is ' . $otp . ' please use this to verify your phone number.';
            (new OtpHelper())->sendMessage($message, $user->contact_numbers, 'otp_service');
            (new UserController())->otpMail($user->id);
        }

        // if ($user->reseller) {
        //     Utils::error("You are not allowed to login", 400);
        // }
        $user->append('selected_business');
        // if (empty($user->selected_business)) {
        //     Utils::error("You are not allowed to login", 400);
        // }
        // if ($user->selected_business->epos_service == 0) {
        //     Utils::error("Your account does not have epos service", 400);
        // }
        if (!Hash::check($req->input('password'), $user->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $user->makeHidden(['business_counts', 'balance']);

            if ($req->has('application_type')) {
                $tokens = (new UserController())->getNewTokens($user->id);
            } else {
                $tokens = (new UserController())->getNewTokens($user->id, 'epos_admin');
            }

            if (Auth::$package_name != null) {
                $servicesCondition = "epos_service=1";
            } else {
                $servicesCondition = "epos_service=1 OR merchant_service=1 OR online_service=1 OR issuing_service=1 OR pouch_service=1";
            }

            $userBusinesses = Business::selectRaw("restaurant_id,online_service,issuing_service,merchant_service,pouch_service,load_wallet,load_bank,id,name,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,country_id,s_account_id,connect_service")->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$user->id])->orderByRaw('name ASC');
            if (Auth::$package_name == null) {
                $userBusinesses->whereRaw('(epos_service=1 OR merchant_service=1 OR online_service=1 OR issuing_service=1 OR pouch_service=1)', []);
            } else {
                $userBusinesses->where('epos_service', '1');
            }
            $userBusinesses = $userBusinesses->get();
            $userBusinesses->append('default_bank');
            $user->businesses = $userBusinesses;
            $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;

            if (!empty($user->businesses)) {
                foreach ($user->businesses as $business) {
                    if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {
                        $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
                        $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
                    }
                    if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {

                        $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
                        $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
                    }
                    $business->makeHidden(['balance', 'user_counts']);
                    $business->append('card_readers');
                    $business->append('printers');
                    $business->append('admin_permissions');
                    $business->append('supervisor_permissions');
                    $business->append('plans');
                    if ($business->s_account_id != null) {
                        if ($connectMode == 'test') {
                            $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                            $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
                        }
                        if ($connectMode == 'live') {
                            $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                            $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
                        }
                    }
                }
            }
            $user->append('selected_business');
            // dd($user);
            return response()->json($user, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Epos profile
     * GET epos/me
     */
    public function show(Request $req, $id = 'me')
    {
        if ($id == 'me') {
            $user = Auth::user();
        } else {
            $user = User::find($id);
        }
        // if ($user->reseller)  Utils::error("You are not allowed to access", 400);
        $user->append('selected_business');
        return $user;

        if (Auth::$package_name != null) {
            $servicesCondition = "epos_service=1";
        } else {
            $servicesCondition = "epos_service=1 OR merchant_service=1 OR online_service=1";
        }

        $userBusinesses = Business::selectRaw("restaurant_id,online_service,issuing_service,merchant_service,pouch_service,load_wallet,load_bank,id,name,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,country_id,supervisor_password,admin_password,s_account_id,connect_service")->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$user->id])->orderByRaw('name ASC');
        if (Auth::$package_name == null) {
            $userBusinesses->whereRaw('(epos_service=1 OR merchant_service=1 OR online_service=1 OR issuing_service=1 OR pouch_service=1)', []);
        } else {
            $userBusinesses->where('epos_service', '1');
        }
        $userBusinesses = $userBusinesses->get();
        $user->businesses = $userBusinesses;
        $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;

        if (!empty($user->businesses)) {
            foreach ($user->businesses as $business) {
                if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {

                    $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
                    $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
                }
                if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {

                    $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
                    $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
                }
                $business->makeHidden(['balance', 'user_counts']);
                $business->append('card_readers');
                $business->append('admin_permissions');
                $business->append('supervisor_permissions');
                if ($business->s_account_id != null) {
                    if ($connectMode == 'test') {
                        $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                        $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
                    }
                    if ($connectMode == 'live') {
                        $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                        $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
                    }
                }
            }
        }
        return $user;
    }
    /**
     * List epos businesses
     * GET epos/businesses
     */
    public function businesses_index(Request $req)
    {
        $user = Auth::user();
        // if ($user->reseller)  Utils::error("You are not allowed to access", 400);
        $user->makeHidden('business_counts');

        if (Auth::$package_name != null) {
            $servicesCondition = "epos_service=1";
        } else {
            $servicesCondition = "epos_service=1 OR merchant_service=1 OR online_service=1 OR connect_service=1 OR issuing_service=1 OR pouch_service=1";
        }

        $userBusinesses = Business::selectRaw("restaurant_id,online_service,issuing_service,merchant_service,pouch_service,load_wallet,load_bank,id,name,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,country_id,supervisor_password,admin_password,s_account_id,connect_service,pouch_weight,email")->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$user->id])->orderByRaw('name ASC');

        if (Auth::$package_name == null) {
            $userBusinesses->whereRaw('(epos_service=1 OR merchant_service=1 OR online_service=1 OR issuing_service=1 OR pouch_service=1 OR connect_service=1 OR connect_service=0)', []);
        } else {
            $userBusinesses->where('epos_service', '1');
        }

        $userBusinesses = $userBusinesses->get();

        if (empty($userBusinesses)) {
            Utils::error("No businesses found", 400);
        }
        $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;
        foreach ($userBusinesses as $business) {
            if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {

                $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
                $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
            }
            if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {

                $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
                $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
            }
            $business->makeHidden(['user_counts']);
            $business->append('card_readers');
            $business->append('admin_permissions');
            $business->append('supervisor_permissions');
            $business->append('printers');
            $business->append('plans');

            if ($business->s_account_id != null) {
                if ($connectMode == 'test') {
                    $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                    $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
                }
                if ($connectMode == 'live') {
                    $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                    $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
                }
            }
        }
        return $userBusinesses;
    }

    /**
     * Create business
     * Post epos/business
     */
    public function business_create(Request $req)
    {

        $req->merge(['online_service' => 0]);
        $req->merge(['merchant_service' => 0]);
        $req->merge(['epos_service' => 0]);
        $req->merge(['connect_service' => 0]);
        $req->merge(['issuing_service' => 0]);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'door_no' => 'string',
            'address' => 'required|string',
            'city' => 'required|string',
            'state' => 'required|string',
            'postcode' => 'required|string',
            'email' => 'required|email|unique:businesses,email',
            'contact_numbers' => 'required|string',
            'joining_date' => 'required|date',
            'type' => 'required|in:Sole trader,Partnership,Limited company,Not registered comapny,Other',
            'online_service' => 'required|boolean',
            'merchant_service' => 'required|boolean',
            'epos_service' => 'required|boolean',
            'connect_service' => 'required|boolean',
            'country_id' => 'required|exists:countries,id',
            'first_name' => 'required|string|max:255',
            'login_email' => 'required|email|unique:users,login_email',
            'password' => 'required|string',
        ]);

        $user = new User;
        $user->fill($req->all());
        $user->password = Hash::make($user->password);
        $user->save();

        $business = new Business;
        $business->fill($req->all());
        $business->disabled = 1;
        $business->save();

        return $business;
    }

    /**
     * Show epos business by id
     * GET epos/businesses/{business_id}
     */
    public function businesses_show(Request $req, $business_id)
    {
        $user = Auth::user();
        // if ($user->reseller) Utils::error("You are not allowed to access", 400);
        $business = Business::whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$user->id])->orderByRaw('name ASC')->where('id', $business_id)->first();
        // where('epos_service', 1)->
        if (empty($business)) {
            Utils::error("No business found", 400);
        }
        if (strtolower($business->stripe_mode) == 'test' && empty($business->stripe_public_key_test)) {

            $business->stripe_public_key_test = (new SiteSettingController())->findByKey('stripe_public_key_test')->value;
            $business->stripe_private_key_test = (new SiteSettingController())->findByKey('stripe_private_key_test')->value;
        }
        if (strtolower($business->stripe_mode) == 'live' && empty($business->stripe_public_key_live)) {

            $business->stripe_public_key_live = (new SiteSettingController())->findByKey('stripe_public_key_live')->value;
            $business->stripe_private_key_live = (new SiteSettingController())->findByKey('stripe_private_key_live')->value;
        }
        $business->append('admin_permissions');
        $business->append('supervisor_permissions');
        $business->append('default_bank_detail');
        $business->append('all_bank_detail');
        $business->printers = BusinessPrinter::where('business_id', $business->id)->get();
        $business->card_readers = BusinessCardReader::where('business_id', $business->id)->get();
        $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;
        if ($business->s_account_id != null) {
            if ($connectMode == 'test') {
                $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
            }
            if ($connectMode == 'live') {
                $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
            }
        }

        return $business;
    }


    /**
     * Select epos business by id
     * POST epos/businesses/{business_id}/select
     */
    public function businesses_select(Request $req, $business_id)
    {
        $user = Auth::user();
        // if ($user->reseller) Utils::error("You are not allowed to access", 400);

        if (Auth::$package_name != null) {
            $servicesCondition = "epos_service=1";
        } else {
            $servicesCondition = "epos_service=1 OR merchant_service=1 OR online_service=1 OR issuing_service=1 OR pouch_service=1";
        }

        $business = Business::selectRaw("restaurant_id,online_service,issuing_service,merchant_service,pouch_service,load_wallet,load_bank,id,name,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,country_id,supervisor_password,admin_password,s_account_id,connect_service")->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$user->id])->orderByRaw('name ASC')->where('id', $business_id);

        if (Auth::$package_name == null) {
            $business->whereRaw('(epos_service=1 OR merchant_service=1 OR online_service=1 OR issuing_service=1 OR pouch_service=1)', []);
        } else {
            $business->where('epos_service', '1');
        }
        $business = $business->first();

        if (empty($business)) {
            Utils::error("No business found", 400);
        }

        $userSelectedBusiness = new UserSelectedBusiness;
        $userSelectedBusiness->business_id = $business_id;
        $userSelectedBusiness->user_id = $user->id;
        $userSelectedBusiness->save();
        $user->append('selected_business');
        return $user;
    }

    /**
     * Update user password.
     * POST /epos/me/password
     */
    public function password(Request $req)
    {
        $user = Auth::user();
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'password' => 'required|string',
            'new_password' => 'required|string',
        ]);

        if (!Hash::check($req->input('password'), $user->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $user->password = Hash::make($req->input('new_password'));
            $user->save();

            $tokens = (new UserController())->getNewTokens($user->id, 'epos_admin');
            return response()->json($user, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }
}
