<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Auth;
use App\ModelsEpos\EposLoyaltyPoint;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposLoyaltyPointController extends Controller
{
    /**
     * List loyalty_points.
     * GET /api/loyalty_points
     */
    public function index(Request $req)
    {
        // $loyalty_points = EposLoyaltyPoint::orderByRaw('created_at DESC');
        $loyalty_points = EposLoyaltyPoint::selectRaw('loyalty_points.*,order_types.type as order_type')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->orderByRaw('loyalty_points.created_at DESC');

        if ($query = $req->query('query', false)) {
            $loyalty_points->whereRaw('(order_type_id LIKE ?  OR reward_type LIKE ? )', ["%$query%", "%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $loyalty_points = $loyalty_points->get();
        } else {
            $loyalty_points = $loyalty_points->paginate($req->query('per_page', 20));
        }

        if ($loyalty_points->isEmpty()) Utils::error('No loyalty points found!', 404);


        return ($loyalty_points);
    }

    /**
     * Create loyalty-point.
     * POST /api/loyalty-points
     */
    public function create(Request $req)
    {
        if (Auth::role() == 'epos_user')
            $req->merge(['business_id' => $req->header('business_id')]);
        $this->validate($req, [
            'order_type_id' => 'required|exists:epos_connection.order_types,id',
            'reward_type' => 'required|in:flat,percentage',
            'earn_amount' => 'required|numeric',
            'earn_point' => 'required|numeric',
            'spent_point' => 'required|numeric',
            'spent_percentage' => 'required|numeric',
            'minimum_order_value' => 'required|numeric',
        ]);

        // create loyalty_point
        $loyalty_point = new EposLoyaltyPoint;
        $loyalty_point->fill($req->all());

        $loyalty_point->save();
        $loyalty_point->refresh();

        return $loyalty_point;
    }

    /**
     * Show loyalty_point.
     * GET /api/loyalty_points/{id}
     */
    public function show(Request $req, $id)
    {
        $loyalty_point = EposLoyaltyPoint::find($id);
        if (empty($loyalty_point)) Utils::error('EposLoyaltyPoint not found!', 404);
        return $loyalty_point;
    }
    /**
     * Update discount.
     * POST /api/loyalty_points/{id}
     */
    public function update(Request $req, $id)
    {
        $loyalty_point = EposLoyaltyPoint::find($id);
        if (empty($loyalty_point)) Utils::error('EposLoyaltyPoint not found!', 404);
        if (Auth::role() == 'epos_user')
            $req->merge(['business_id' => $req->header('business_id')]);
        $this->validate($req, [
            'order_type_id' => 'required|exists:epos_connection.order_types,id',
            'reward_type' => 'required|in:flat,percentage',
            'earn_amount' => 'required|numeric',
            'earn_point' => 'required|numeric',
            'spent_point' => 'required|numeric',
            'spent_percentage' => 'required|numeric',
            'minimum_order_value' => 'required|numeric',
        ]);

        // update loyalty_point
        if ($req->has('order_type_id')) $loyalty_point->order_type_id = $req->input('order_type_id');
        if ($req->has('reward_type')) $loyalty_point->reward_type = $req->input('reward_type');
        if ($req->has('earn_amount')) $loyalty_point->earn_amount = $req->input('earn_amount');
        if ($req->has('earn_point')) $loyalty_point->earn_point = $req->input('earn_point');
        if ($req->has('spent_point')) $loyalty_point->spent_point = $req->input('spent_point');
        if ($req->has('spent_percentage')) $loyalty_point->spent_percentage = $req->input('spent_percentage');
        if ($req->has('minimum_order_value')) $loyalty_point->minimum_order_value = $req->input('minimum_order_value');

        $loyalty_point->save();
        $loyalty_point->refresh();

        return $loyalty_point;
    }


    /**
     * Delete loyalty_point by id.
     * DELETE /api/loyalty_points/{id}
     */
    public function delete(Request $req, $id)
    {
        $loyalty_point = EposLoyaltyPoint::find($id);
        if (empty($loyalty_point)) Utils::error('EposLoyaltyPoint not found!', 404);
        $loyalty_point->delete();
        return $loyalty_point;
    }
    
}
