<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\OnlineOrder;
use App\OnlineOrderCommission;
use App\PaypalPayment;
use App\ResellerCommission;
use App\ResellerOrderCommission;
use App\UserBusiness;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposOnlineOrderController extends Controller
{
    /**
     * List onlineOrders.
     * GET /epos/online-orders
     */
    public function index(Request $req)
    {
        $onlineOrders = OnlineOrder::orderByRaw('order_date DESC');
        $onlineOrders->select('online_orders.*', 'businesses.name as business_name');
        $onlineOrders->leftJoin('businesses', 'business_id', '=', 'businesses.id');
        if ($query = $req->query('query', false)) {
            $onlineOrders->whereRaw('(order_id LIKE ? OR order_type LIKE ? OR payment_method LIKE ? OR grand_total LIKE ? OR order_status LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('order_type', false)) {
            $onlineOrders->where('order_type', $query);
        }
        if ($query = $req->query('payment_method', false)) {
            $onlineOrders->where('payment_method', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $onlineOrders->where('business_id', $query);
        }
        if ($query = $req->query('restaurant_id', false)) {
            $onlineOrders->where('online_orders.restaurant_id', $query);
        }
        if ($query = $req->query('date_filter', false)) {
            if (strtolower($query) == 'today') {
                $from_date = Carbon::today()->startOfDay();
                $to_date = Carbon::today()->endOfDay();
            }
            if (strtolower($query) == 'yesterday') {
                $from_date = Carbon::yesterday()->startOfDay();
                $to_date = Carbon::yesterday()->endOfDay();
            }
            if (strtolower($query) == 'thisweek') {
                $from_date = Carbon::today()->getWeekStartsAt();
                $to_date = Carbon::today()->getWeekEndsAt();
            }
            if (strtolower($query) == 'thismonth') {
                $from_date = Carbon::today()->startOfMonth();
                $to_date = Carbon::today()->endOfMonth();
            }
            if (strtolower($query) == 'thisyear') {
                $from_date = Carbon::today()->startOfYear();
                $to_date = Carbon::today()->endOfYear();
            }
            if (!empty($from_date) && !empty($to_date)) {
                $onlineOrders->where('order_date', '>=', $from_date);
                $onlineOrders->where('order_date', '<=', $to_date);
            }
        }
        if ($query = $req->query('from_date', false)) {
            $onlineOrders->where('order_date', '>=', Carbon::parse($query)->startOfDay());
        }
        if ($query = $req->query('to_date', false)) {
            $onlineOrders->where('order_date', '<=', Carbon::parse($query)->endOfDay());
        }

        if ($req->query('nopaginate', false) == '1') {
            $onlineOrders = $onlineOrders->get();
        } else {
            $onlineOrders = $onlineOrders->paginate($req->query('per_page', 20));
        }

        if ($onlineOrders->isEmpty()) Utils::error('No orders found!', 404);


        return ($onlineOrders);
    }
    /**
     * List onlineOrders.
     * GET /epos/reservation
     */
    public function reservation(Request $req)
    {

        $business_id = $req->query('business_id', false);

        $business = Business::find($business_id);
        if (empty($business)) Utils::error('Business not found!', 404);

        $startdate = Carbon::parse($req->query('from_date', false))->format('Y-m-d');
        $enddate = Carbon::parse($req->query('to_date', false))->format('Y-m-d');
        $response = (new Client())->request('POST', env('FETCH_ORDERS_LIST'), [
            'form_params' => ['from' => $startdate, 'to' => $enddate, 'user_id' => $business->restaurant_id, 'action' => 'completedReservationFillter']
        ]);
        if ($response->getStatusCode() == 200) {
            $jsonBody = json_decode($response->getBody());
            if ($jsonBody->success == '1' && isset($jsonBody->reservationDetail)) {
                $onlineBookings = $jsonBody->reservationDetail;
            } else Utils::error("No data found", 400);
        } else {
            return $response->getBody();
        }

        return ($onlineBookings);
    }
    public function show(Request $req, $id)
    {
        $onlineOrder = OnlineOrder::find($id);
        if (empty($onlineOrder)) Utils::error('Order not found!', 404);
        return $onlineOrder;
    }
    public function delete(Request $req, $id)
    {
        $onlineOrder = OnlineOrder::find($id);
        if (empty($onlineOrder)) Utils::error('Order not found!', 404);
        $onlineOrder->delete();
        return $onlineOrder;
    }

    public function fetch(Request $req)
    {
        $startdate = Carbon::today()->subDays(15)->format('d-m-Y');
        $enddate = Carbon::today()->format('d-m-Y');

        if ($req->has('startdate')) {
            $startdate = $req->input('startdate');
        }
        if ($req->has('enddate')) {
            $enddate = $req->input('enddate');
        }
        return $this->orders($startdate, $enddate);
    }
    public function history(Request $req)
    {
        $startdate = Carbon::today()->subDays(15)->format('d-m-Y');
        $enddate = Carbon::today()->format('d-m-Y');
        $onlineOrder = OnlineOrder::orderByRaw('order_date ASC')->first();

        if (!empty($onlineOrder)) {
            $carbonDate = Carbon::parse($onlineOrder->order_date);
            $enddate = $carbonDate->format('d-m-Y');
            $startdate = $carbonDate->subDays(15)->format('d-m-Y');
        }
        if ($req->has('startdate')) {
            $startdate = $req->input('startdate');
        }
        if ($req->has('enddate')) {
            $enddate = $req->input('enddate');
        }
        return $this->orders($startdate, $enddate);
    }
    public function orders($startdate, $enddate)
    {
        $response = (new Client())->request('POST', env('FETCH_ORDERS_LIST', 'https://www.tiffintom.net/v2/StoreMobileApi/requests'), [
            'form_params' => ['startdate' => $startdate, 'enddate' => $enddate, 'action' => 'getOrderList']
            // 'form_params' => ['startdate' => '01-01-2021', 'enddate' => '20-02-2021', 'action' => 'StoreDetail', 'user_id' => '4']
        ]);
        if ($response->getStatusCode() == 200) {
            $jsonBody = json_decode($response->getBody());
            if ($jsonBody->success == '1' && isset($jsonBody->Orderlist)) {
                $this->saveOrder($jsonBody->Orderlist);
            }
            if ($jsonBody->success == '1' && isset($jsonBody->DineinOrderlist)) {
                $this->saveOrder($jsonBody->DineinOrderlist);
            }
            if (isset($jsonBody->Orderlist) && isset($jsonBody->DineinOrderlist)) {
                if (isset($jsonBody->Orderlist)) $jsonBody->Orderlist;
                if (isset($jsonBody->DineinOrderlist)) $jsonBody->DineinOrderlist;
            } else Utils::error("No data found", 400);
        } else {
            return $response->getBody();
        }
    }
    public function saveOrder($orderList)
    {
        foreach ($orderList as $order) {
            if (isset($order->order_type)) {
                $order_type = $order->order_type;
            } else {
                $order_type = 'dinein';
            }
            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            $onlineOrder = OnlineOrder::where('order_id', $order->id)->where('order_type', $order_type)->first();
            if (empty($onlineOrder)) {
                $onlineOrder = new OnlineOrder();
            }
            $shouldSetCommissions = ($onlineOrder->business_id == null) ? true : false;

            $onlineOrder->order_id = $order->id;
            $onlineOrder->restaurant_id = $order->restaurant_id;
            if (!empty($business)) {
                $onlineOrder->business_id = $business->id;
            }

            $onlineOrder->order_type = $order_type;
            if (isset($order->transaction_id)) {
                $onlineOrder->txn_id = $order->transaction_id;
            }
            $onlineOrder->payment_method = $order->payment_method;
            if (isset($order->order_sub_total)) {
                $onlineOrder->sub_total = $order->order_sub_total;
            }
            if (isset($order->delivery_charge)) {
                $onlineOrder->delivery_charge = $order->delivery_charge;
            }
            if (isset($order->driver_tip)) {
                $onlineOrder->driver_tip = $order->driver_tip;
            }
            if (isset($order->gratuity)) {
                $onlineOrder->gratuity_tip = $order->gratuity;
            }
            if (isset($order->order_grand_total)) {
                $onlineOrder->grand_total = $order->order_grand_total;
            }
            if (isset($order->charity_amount)) {
                $onlineOrder->charity_amount = $order->charity_amount;
            }
            if (isset($order->voucher_amount)) {
                $onlineOrder->voucher_amount = $order->voucher_amount;
            }
            if (isset($order->service_charge)) {
                $onlineOrder->user_service_charge = $order->service_charge;
            }

            $onlineOrder->order_date = $order->created;
            $onlineOrder->order_status = $order->status;

            $onlineOrder->save();
            $orderCommission = OnlineOrderCommission::where('online_order_id', $onlineOrder->id)->first();
            if (empty($orderCommission)) {
                $shouldSetCommissions = true;
                $orderCommission = new OnlineOrderCommission();
            }
            if ($shouldSetCommissions) {
                $orderCommission->online_order_id = $onlineOrder->id;
                if (!empty($business)) {
                    $orderCommission->fill(json_decode($business->business_commissions, true));
                }
            }

            $orderCommission->save();

            if (!empty($business)) {
                $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                if (!empty($reseller)) {
                    $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
                    if (empty($resellerCommission)) {
                        $resellerCommission = new ResellerCommission;
                        $resellerCommission->user_id = $reseller->user_id;
                        $resellerCommission->business_id = $business->id;
                        $resellerCommission->save();
                    }
                    $resellerOrderCommission = ResellerOrderCommission::where('online_order_id', $onlineOrder->id)->first();
                    if (empty($resellerOrderCommission)) {
                        $resellerOrderCommission = new ResellerOrderCommission();
                        $resellerOrderCommission->user_id = $reseller->user_id;
                        $resellerOrderCommission->online_order_id = $onlineOrder->id;
                        $resellerOrderCommission->fill(json_decode($resellerCommission, true));

                        $resellerOrderCommission->save();
                    }
                }
            }
        }
    }

    /**
     * Archive Bulk Transacitons.
     * POST online-orders/archive
     */
    public function archive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = PaypalPayment::whereRaw('(sale_id IN (SELECT txn_id from online_orders WHERE id IN (?))', $req->input('ids'))->update(['archived' => '1']);
            $transactions = PaypalPayment::whereRaw('(sale_id IN (SELECT txn_id from online_orders WHERE id IN (?))', $req->input('ids'))->get();
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }


    /**
     * UnArchive Bulk Transacitons.
     * POST online-orders/unarchive
     */
    public function unarchive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = PaypalPayment::whereRaw('(sale_id IN (SELECT txn_id from online_orders WHERE id IN (?))', $req->input('ids'))->update(['archived' => '0']);
            $transactions = PaypalPayment::whereRaw('(sale_id IN (SELECT txn_id from online_orders WHERE id IN (?))', $req->input('ids'))->get();
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }
}
