<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposOrderAction;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposOrderActionController extends Controller
{
    /**
     * List orderActions.
     * GET /epos/order-actions
     */
    public function index(Request $req)
    {
        $orderActions = EposOrderAction::orderByRaw('action ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $orderActions->whereRaw('(action LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $orderActions = $orderActions->get();
        } else {
            $orderActions = $orderActions->paginate($req->query('per_page', 20));
        }

        if ($orderActions->isEmpty()) Utils::error('No orderActions found!', 404);


        return ($orderActions);
    }

    /**
     * Create orderAction.
     * POST /epos/order-actions
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'action' => 'required|string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // create orderAction
        $orderAction = new EposOrderAction;
        $orderAction->fill($req->all());

        $orderAction->save();

        return $orderAction;
    }

    /**
     * Show orderAction.
     * GET /epos/order-actions/{id}
     */
    public function show(Request $req, $id)
    {
        $orderAction = EposOrderAction::find($id);
        if (empty($orderAction)) Utils::error('EposOrderAction not found!', 404);
        return $orderAction;
    }

    /**
     * Update orderAction.
     * POST /epos/order-actions/{id}
     */
    public function update(Request $req, $id)
    {
        $orderAction = EposOrderAction::find($id);
        if (empty($orderAction)) Utils::error('EposOrderAction not found!', 404);

        $this->validate($req, [
            'action' => 'string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // update orderAction
        if ($req->has('action')) $orderAction->action = $req->input('action');
        if ($req->has('top_color')) $orderAction->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $orderAction->bottom_color = $req->input('bottom_color');

        $orderAction->save();

        return $orderAction;
    }


    /**
     * Delete orderAction by id.
     * DELETE /epos/order-actions/{id}
     */
    public function delete(Request $req, $id)
    {
        $orderAction = EposOrderAction::find($id);
        if (empty($orderAction)) Utils::error('EposOrderAction not found!', 404);
        $orderAction->delete();
        return $orderAction;
    }
}
