<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Auth;
use App\Helpers\OtpHelper;
use App\Helpers\PushIosNotificationHelper;
use App\Helpers\PushNotificationHelper;
use App\ModelsEpos\EposOrder;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposDevice;
use App\ModelsEpos\EposLoyaltyPoint;
use App\ModelsEpos\EposOrderItem;
use App\ModelsEpos\EposOrderItemAddon;
use App\ModelsEpos\EposOrderItemIngredient;
use App\ModelsEpos\EposOrderPayment;
use App\ModelsEpos\EposOrderSplit;
use App\ModelsEpos\EposProduct;
use App\ModelsEpos\EposSetting;
use App\ModelsEpos\EposTable;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use stdClass;

class EposOrderController extends Controller
{
    /**
     * List orders.
     * GET /epos/orders
     */
    public function index(Request $req)
    {

        $orders = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')->orderByRaw('orders.id DESC');

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        $screen = $req->query("screen", null);
        $to_date = null;
        $from_date = null;
        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";
        } else {
            if ($screen != 'dashboard') {
                $to_date = Carbon::now()->endOfDay()->format('Y-m-d H:i:s');
                $from_date = Carbon::now()->subDays(11)->startOfDay()->format('Y-m-d H:i:s');
            } else {
                $to_date = Carbon::now()->endOfDay()->format('Y-m-d H:i:s');
                $from_date = Carbon::now()->subDays(1)->startOfDay()->format('Y-m-d H:i:s');
            }
        }

        if ($screen == 'dashboard') {
            $orders = $orders->whereRaw('((orders.created_at >= ? AND orders.created_at <= ?) OR orders.order_status_id != ? AND orders.order_status_id != ?)', [$from_date, $to_date, "5", "10"]);
        } else {
            $orders = $orders->where('orders.created_at', '>=', $from_date);
            $orders = $orders->where('orders.created_at', '<=', $to_date);
        }
        if ($query = $req->query('query', false)) {
            $orders->whereRaw('(orders.total ?  )', ["%$query%"]);
        }
        if ($table_id = $req->query('table_id', false)) {
            $orders->where('table_id', $table_id);
        }
        if ($order_type_id = $req->query('order_type_id', false)) {
            $orders->where('order_type_id', $order_type_id);
        }
        if ($order_status_id = $req->query('order_status_id', false)) {
            $orders->where('order_status_id', $order_status_id);
        }
        if ($query = $req->query('preparation_location_id', false)) {
            $orders->where('preparation_location_id', $query);
        }
        if ($query = $req->query('order_action_id', false)) {
            $orders->where('order_action_id', $query);
        }
        if ($query = $req->query('archived', false)) {
            $orders->where('is_archived', $query);
        }
        // $orders->with(['order_items', 'order_payments', 'order_splits', 'customer', 'table']);
        if ($req->query('nopaginate', false) == '1') {
            $orders = $orders->get();
        } else {
            $orders = $orders->paginate($req->query('per_page', 20));
        }

        if ($orders->isEmpty()) Utils::error('No orders found!', 404);

        $orders->append('order_items');
        $orders->append('order_payments');
        $orders->append('order_splits');
        $orders->append('customer');
        $orders->append('table');
        return ($orders);
    }
    /**
     * List orders.
     * GET /epos/orders/latest
     */
    public function index_latest(Request $req)
    {

        $orders = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status')
            ->leftJoin('customers', 'customers.id', '=', 'customer_id')
            ->leftJoin('tables', 'tables.id', '=', 'table_id')
            ->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')
            ->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')
            ->orderByRaw('orders.id DESC');

        $this->validate($req, [
            'from_updated_at' => 'date_format:Y-m-d H:i:s',
        ]);

        $from_date = null;
        if ($req->has('from_updated_at')) {
            $from_date = Carbon::parse($req->input('from_updated_at'))->subHours(2)->format('Y-m-d H:i:s');
        } else {
            $from_date = Carbon::now()->subDays(3)->startOfDay()->format('Y-m-d H:i:s');
        }
        // $lastUpdatedOrder = EposOrder::orderByRaw('updated_at DESC')->first();
        $orders = $orders->where('orders.updated_at', '>=', $from_date);

        if ($query = $req->query('query', false)) {
            $orders->whereRaw('(orders.total LIKE ? OR customers.name LIKE ? )', ["%$query%", "%$query%"]);
        }
        if ($table_id = $req->query('table_id', false)) {
            $orders->where('table_id', $table_id);
        }
        if ($order_type_id = $req->query('order_type_id', false)) {
            $orders->where('order_type_id', $order_type_id);
        }
        if ($order_status_id = $req->query('order_status_id', false)) {
            $orders->where('order_status_id', $order_status_id);
        }
        if ($query = $req->query('preparation_location_id', false)) {
            $orders->where('preparation_location_id', $query);
        }
        if ($query = $req->query('order_action_id', false)) {
            $orders->where('order_action_id', $query);
        }
        if ($query = $req->query('archived', false)) {
            $orders->where('is_archived', $query);
        }
        // $orders->with(['order_items', 'order_payments', 'order_splits', 'customer', 'table']);
        if ($req->query('nopaginate', false) == '1') {
            $orders = $orders->get();
        } else {
            $orders = $orders->paginate($req->query('per_page', 20));
        }

        if ($orders->isEmpty()) Utils::error('No orders found!', 404);

        $orders->append('order_items');
        $orders->append('order_payments');
        $orders->append('order_splits');
        $orders->append('customer');
        $orders->append('table');
        return ($orders);
    }
    /**
     * Create table.
     * POST /epos/orders
     */
    public function create(Request $req)
    {
        // Unique id check and update call and return
        $order = EposOrder::where('unique_id', $req->input('unique_id'))->first();
        if (!empty($order)) {
            $this->update($req, $order->id);
            return;
        }

        if (!is_array($req->order_items))
            $req->merge(['order_items' => (json_decode($req->order_items, true))]);
        if (!is_array($req->customer))
            $req->merge(['customer' => (json_decode($req->customer, true))]);

        $oldOrder = EposOrder::where('table_id', $req->input('table_id'))->where('order_type_id', $req->input('order_type_id'))->where('order_status_id', $req->input('order_status_id'))->latest()->first();
        if (!empty($oldOrder) && $req->input('table_id') != '') {
            $req->merge(['sub_total' => ($req->input('sub_total') + $oldOrder->sub_total)]);
            $req->merge(['total' => ($req->input('total') + $oldOrder->total)]);

            if ($req->input('discount') <= 0) {
                $req->merge(['discount' => ($oldOrder->discount)]);
            }
            if ($req->input('gratuity') <= 0) {
                $req->merge(['gratuity' => ($oldOrder->gratuity)]);
            }
            if ($req->input('delivery_charge') <= 0) {
                $req->merge(['delivery_charge' => ($oldOrder->delivery_charge)]);
            }
            if ($req->input('service_charge') <= 0) {
                $req->merge(['service_charge' => ($oldOrder->service_charge)]);
            }

            // $all_items = array_merge($req->input('order_items'), $oldOrder->order_items->toArray());
            // $req->merge(['order_items' => $all_items]);

            $items = $req->input('order_items');
            $olditems = $oldOrder->order_items->toArray();

            foreach ($olditems as $key => $value) {
                foreach ($items as $itemkey => $itemvalue) {
                    if ($value['product_id'] == $itemvalue['product_id']) {
                        $items[$itemkey]['quantity'] = $value['quantity'] + $itemvalue['quantity'];
                        $items[$itemkey]['addons_price'] = $value['addons_price'] + $itemvalue['addons_price'];
                        $items[$itemkey]['ingredients_price'] = $value['ingredients_price'] + $itemvalue['ingredients_price'];
                        $items[$itemkey]['instruction_price'] = $value['instruction_price'] + $itemvalue['instruction_price'];
                        $items[$itemkey]['sub_total'] = $value['sub_total'] + $itemvalue['sub_total'];
                        $items[$itemkey]['total'] = $value['total'] + $itemvalue['total'];
                        unset($olditems[$key]);
                    }
                }
            }
            $all_items = array_merge($items, $olditems);
            $req->merge(['order_items' => $all_items]);
            $order = EposOrder::find($oldOrder->id);
            $order->delete();
        }

        $this->validate($req, [
            'customer_id' => 'exists:epos_connection.customers,id',
            'table_id' => 'nullable|exists:epos_connection.tables,id',
            'order_type_id' => 'nullable|exists:epos_connection.order_types,id',
            'order_status_id' => 'nullable|exists:epos_connection.order_statuses,id',
            'preparation_location_id' => 'nullable|exists:epos_connection.preparation_locations,id',
            'order_action_id' => 'nullable|exists:epos_connection.order_actions,id',
            'sub_total' => 'numeric',
            'tax' => 'numeric',
            'service_charge' => 'numeric',
            'delivery_charge' => 'numeric',
            'gratuity' => 'numeric',
            'discount' => 'numeric',
            'total' => 'numeric',
            'total_paid' => 'numeric',
            'due_amount' => 'numeric',
            'comment' => 'string',
            'split_type' => 'in:enum,order_item,person',
            'split_count' => 'numeric',
            'no_guest' => 'string',
            'distance' => 'string',
            'delivery_date' => 'date',
            'order_items' => 'array',
            'is_archived' => 'boolean',
            'device_id' => 'nullable',
            'unique_id' => 'string',
            'delivery_collection_time_slot' => 'string',
            'is_delivery_collection_ask' => 'boolean',
            'display_order_id' => 'string',
            'is_auto_service_charge_remove' => 'boolean',
        ]);

        // Unique id add check on create
        $order = EposOrder::whereNotNull('unique_id')->where('unique_id', $req->input('unique_id'))->first();
        if (!empty($order)) {
            // update order
            $req->merge(['unique_id' => ($req->input('unique_id'))]);
            $order = $this->update($req, $order->id);
        } else {
            // create order
            $order = new EposOrder;
            $order->fill($req->all());
            if ($order->order_type_id != 1) {
                $order->table_id = null;
            }
            if (!empty($req->input('table_id'))) {
                if ($req->has('table_id')) $order->table_id = $req->input('table_id');
            } else {
                $order->table_id = null;
            }
            if (empty($req->input('customer_id'))) {
                $customerDetail = $req->input('customer');
                $customer = new EposCustomer((array)$customerDetail);
                $customer->membership_no = rand(100000000, 999999999);
                $customer->save();

                $order->customer_id = $customer->id;
            } else {
                $customerDetail = ($req->input('customer'));
                $customer = EposCustomer::find($customerDetail['id']);
                if (isset($customerDetail['name'])) $customer->name = $customerDetail['name'];
                if (isset($customerDetail['mobile'])) $customer->mobile = $customerDetail['mobile'];
                if (isset($customerDetail['email'])) $customer->email = $customerDetail['email'];
                if (isset($customerDetail['landline'])) $customer->landline = $customerDetail['landline'];
                if (isset($customerDetail['birth_date'])) $customer->birth_date = $customerDetail['birth_date'];
                if (isset($customerDetail['anniversary_date'])) $customer->anniversary_date = $customerDetail['anniversary_date'];
                if (isset($customerDetail['house_no'])) $customer->house_no = $customerDetail['house_no'];
                if (isset($customerDetail['street'])) $customer->street = $customerDetail['street'];
                if (isset($customerDetail['postcode'])) $customer->postcode = $customerDetail['postcode'];
                if (isset($customerDetail['distance'])) $customer->distance = $customerDetail['distance'];
                if (isset($customerDetail['city'])) $customer->city = $customerDetail['city'];
                if (isset($customerDetail['country'])) $customer->country = $customerDetail['country'];
                if (isset($customerDetail['state'])) $customer->state = $customerDetail['state'];
                if (isset($customerDetail['news_letter'])) $customer->news_letter = $customerDetail['news_letter'];
                $customer->save();

                $order->customer_id = $customer->id;
            }
            $order->save();

            // if (!empty($order->table_id)) {
            //     $table = EposTable::find($order->table_id);
            //     if (empty($table)) Utils::error('EposTable not found!', 404);
            //     $table->table_status_id = 3; // Order Occupied Status change
            //     $table->save();
            // }

            $items = ($req->input('order_items'));
            if (!empty($items)) {
                foreach ($items as $item) {
                    $reqItem = $item;

                    if (!isset($item['product_id']) || $item['product_id'] == '') {

                        $product = new EposProduct;
                        if ($item['product_name']) $product->name = $item['product_name'];
                        if ($item['preparation_location_id']) $product->preparation_location_id = $item['preparation_location_id'];
                        $product->top_color = '#8d8d8d';
                        $product->font_color = '#000000';
                        $product->price = $item['price'];
                        $product->takeaway_price = $item['takeaway_price'];
                        $product->web_price = $item['web_price'];
                        $product->delivery_price = $item['delivery_price'];
                        $product->waiting_price = $item['waiting_price'];
                        $product->favourite = 0;
                        $product->bar_favourite = 0;
                        $product->tax_included = 0;
                        $product->taxable = 0;
                        $product->auto_modify = 0;
                        $product->stockable = 0;
                        $product->misc = 1;
                        $product->save();
                        $item['product_id'] = $product->id;
                    }

                    $orderItem = new EposOrderItem((array)$item);
                    if (!isset($item['table_type'])) {
                        $orderItem->order_id = $order->id;
                    }
                    $orderItem->save();
                    if (isset($reqItem['order_item_addons'])) {
                        if (!is_array($reqItem['order_item_addons']))
                            $reqItem['order_item_addons'] = (json_decode($reqItem['order_item_addons'], true));

                        foreach ($reqItem['order_item_addons'] as $addon) {
                            $orderItemAddon = new EposOrderItemAddon($addon);
                            $orderItemAddon->order_item_id = $orderItem->id;
                            $orderItemAddon->save();
                        }
                    }
                    if (isset($reqItem['order_item_ingredients'])) {
                        if (!is_array($reqItem['order_item_ingredients']))
                            $reqItem['order_item_ingredients'] = (json_decode($reqItem['order_item_ingredients'], true));

                        foreach ($reqItem['order_item_ingredients'] as $ingredient) {
                            $orderItemAddon = new EposOrderItemIngredient($ingredient);
                            $orderItemAddon->order_item_id = $orderItem->id;
                            $orderItemAddon->save();
                        }
                    }
                }
            }

            $order = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status,preparation_locations.name as preparation_location_name,order_actions.action as order_action')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')->leftJoin('preparation_locations', 'preparation_locations.id', '=', 'preparation_location_id')->leftJoin('order_actions', 'order_actions.id', '=', 'order_action_id')->find($order->id);
            $order->append('order_items');
            $order->append('order_payments');
            $order->append('order_splits');
            $order->append('customer');
            $order->append('table');

            $message = 'You have a new ' . $order->order_type . ' order - ' . $order->id;

            $notification['title'] = "New " . $order->order_type . " order";
            $notification['body']  = $message;

            $notificationdata['data']['title'] = "New " . $order->order_type . " order";
            $notificationdata['data']['print']  = ($req->input('print') != '') ? true : false;
            $notificationdata['data']['message'] = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['order_id']  = $order->id;
            $notificationdata['data']['updater_id']  = $order->updater_id;
            $notificationdata['data']['order_type']  = ucfirst($order->order_type);
            $notificationdata['data']['notification_for']  = 'epos';
            $notificationdata['data']['notification_section']  = 'orders';
            $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
            $notificationdata['data']['business_id'] = Auth::$business_id;

            $topic = '/topics/business_' . Auth::$business_id;
            $fcm_api_key = (new SiteSettingController())->findByKey('fcm_api_key');
            if (!empty($fcm_api_key)) {
                $api_key = $fcm_api_key->value;
                if ($req->input('send_push', "1") == "1") {
                    (new PushNotificationHelper())->sendNotification($notificationdata, $topic, $api_key, $notification);
                }
            }

            $devices = EposDevice::orderByRaw('created_at DESC')->whereNotNull('device_token')->where('type', 'iOS')->where('disabled', 0)->get();
            if (!$devices->isEmpty()) {
                foreach ($devices as $device) {
                    $message = 'You have a new ' . $order->order_type . ' order - ' . $order->id;
                    (new PushIosNotificationHelper())->sendNotification($message, $device->device_token, $order->id, $order->order_type, 'orders');
                }
            }
            if ($order->order_status_id == "5") {
                $royaltyPointSetting = (new EposSettingController())->findByKey("how_much_purchase_in_pound_will_construct_one_loyalty_point");
                if (!empty($royaltyPointSetting) && $royaltyPointSetting->value != null && $royaltyPointSetting->value != 0) {
                    $points = (int)($order->sub_total / $royaltyPointSetting->value);
                    $customer = EposCustomer::find($order->customer_id);
                    if (!empty($customer)) {
                        $customer->membership_points = $points + $customer->membership_points;
                        $cust = $customer->save();
                        $order->customer = $cust;
                    }
                }
            }
            $order = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')->where('orders.id', $order->id)->first();
            $order->append('order_items');
            $order->append('order_payments');
            $order->append('order_splits');
            $order->append('customer');
            $order->append('table');

            if (!empty($req->input('table'))) {
                $table = Request::create("");
                $table->merge($req->table, true);
                (new EposTableController())->update($table, $table['id']);
            }

            if (!empty($req->input('order_splits'))) {
                $Allsplits = array();
                foreach ($req->order_splits as $split) {
                    $split['order_id'] = $order->id;
                    $Allsplits[] = $split;
                }
                $order_splits = Request::create("");
                $order_splits->merge(['splits' => json_encode($Allsplits, true), true]);
                (new EposOrderSplitController())->split_bulk($order_splits);
            }

            if (!empty($req->input('order_payments'))) {
                $Allpayment = array();
                foreach ($req->order_payments as $payment) {
                    $payment['order_id'] = $order->id;
                    $Allpayment[] = $payment;
                }
                $order_payments = Request::create("");
                $order_payments->merge(['payments' => json_encode($Allpayment, true), true]);
                (new EposOrderPaymentController())->payment_bulk($order_payments);
            }

            $splits = ($req->input('order_splits'));
            if (!empty($splits)) {
                foreach ($splits as $split) {
                    if (isset($split['unique_id'])) {
                        $orderSplit = EposOrderSplit::where('unique_id', $split['unique_id'])->first();
                        if (!empty($orderSplit)) {
                            EposOrderItem::where('unique_id', $split['unique_id'])->update(['order_split_id' => $orderSplit->id]);
                            EposOrderPayment::where('unique_id', $split['unique_id'])->update(['order_split_id' => $orderSplit->id]);
                        }
                    }
                }
            }
        }

        return $order;
    }

    /**
     * Show table.
     * GET /epos/orders/{id}
     */
    public function show(Request $req, $id)
    {
        $order = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status,preparation_locations.name as preparation_location_name,order_actions.action as order_action')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')->leftJoin('preparation_locations', 'preparation_locations.id', '=', 'preparation_location_id')->leftJoin('order_actions', 'order_actions.id', '=', 'order_action_id')->find($id);
        if (empty($order)) Utils::error('EposOrder not found!', 404);

        // Append Related Orders
        $order->append('order_items');
        $order->append('order_payments');
        $order->append('order_splits');
        $order->append('customer');
        $order->append('table');
        $order->each(function ($item) {
            $item->append('order_item_addons');
            $item->append('order_item_ingredients');
        });

        //Loyalty Calculate point
        $loyalty_points = EposLoyaltyPoint::where('order_type_id', $order->order_type_id)->where('minimum_order_value', '<=', $order->sub_total)->first();

        $order_payment = EposOrderPayment::where('payment_method_id', 6)->where('order_id', $order->id)->first();

        if (!empty($loyalty_points) && empty($order_payment)) {
            $customer = EposCustomer::where('id', $order->customer_id)->where('membership_points', '>=', "$loyalty_points->spent_point")->first();
            if (!empty($customer)) {
                $calculate_loyaltypoint = '';
                $calculate_loyaltyamount = '';
                if ($loyalty_points->reward_type == 'flat') {
                    $calculate_loyaltypoint = $loyalty_points->spent_point;
                    $calculate_loyaltyamount = $loyalty_points->spent_percentage;
                } else {
                    $calculate_loyaltypoint = $loyalty_points->spent_point;
                    $calculate_loyaltyamount = (($order->sub_total * $loyalty_points->spent_percentage) / $loyalty_points->spent_point);
                }

                $maxLoyaltyPoint = (new EposSettingController())->findByKey('maximum_loyalty_point')->value;
                if ($calculate_loyaltyamount > $maxLoyaltyPoint) {
                    $calculate_loyaltyamount = $maxLoyaltyPoint;
                    $calculate_loyaltyamount = (($loyalty_points->spent_point * $loyalty_points->spent_percentage) / $calculate_loyaltyamount);
                }
                $order->calculate_loyalty_point = $calculate_loyaltypoint;
                $order->calculate_loyalty_amount = $calculate_loyaltyamount;
            }
        }

        return $order;
    }

    /**
     * Update order.
     * POST /epos/orders/{id}
     */
    public function update(Request $req, $id)
    {
        $order = EposOrder::selectRaw('orders.*,order_types.type as order_type')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->find($id);
        if (empty($order)) Utils::error('EposOrder not found!', 404);
        $dbOrder = clone $order;

        if (!is_array($req->order_items))
            $req->merge(['order_items' => (json_decode($req->order_items, true))]);
        if (!is_array($req->customer))
            $req->merge(['customer' => (json_decode($req->customer, true))]);
        if (!is_array($req->table))
            $req->merge(['table' => (json_decode($req->table, true))]);
        if ($req->has('order_payments') && !is_array($req->input('order_payments'))) {
            $req->merge(['order_payments' => json_decode($req->input('order_payments'))]);
        }
        if ($req->has('order_splits') && !is_array($req->input('order_splits'))) {
            $req->merge(['order_splits' => json_decode($req->input('order_splits'))]);
        }

        $this->validate($req, [
            'customer_id' => 'exists:epos_connection.customers,id',
            'table_id' => 'nullable|exists:epos_connection.tables,id',
            'order_type_id' => 'exists:epos_connection.order_types,id',
            'order_status_id' => 'exists:epos_connection.order_statuses,id',
            'preparation_location_id' => 'exists:epos_connection.preparation_locations,id',
            'order_action_id' => 'exists:epos_connection.order_actions,id',
            'sub_total' => 'numeric',
            'tax' => 'numeric',
            'service_charge' => 'numeric',
            'delivery_charge' => 'numeric',
            'gratuity' => 'numeric',
            'discount' => 'numeric',
            'total' => 'numeric',
            'total_paid' => 'numeric',
            'due_amount' => 'numeric',
            'comment' => 'string',
            'split_type' => 'in:enum,order_item,person',
            'split_count' => 'numeric',
            'no_guest' => 'string',
            'distance' => 'string',
            'delivery_date' => 'date',
            'order_items' => 'array',
            'customer' => 'array',
            'device_id' => 'nullable',
            'is_archived' => 'boolean',
            'unique_id' => 'string',
            'delivery_collection_time_slot' => 'string',
            'is_delivery_collection_ask' => 'boolean',
            'display_order_id' => 'string',
            'is_auto_service_charge_remove' => 'boolean',
        ]);

        // update order
        if ($req->has('customer_id')) $order->customer_id = $req->input('customer_id');
        if ($req->has('updater_id')) $order->updater_id = ($req->input('updater_id') != '') ? $req->input('updater_id') : Auth::id();

        if ($req->has('updated_at')) $order->updated_at = $req->input('updated_at');
        if ($req->has('order_type_id')) $order->order_type_id = $req->input('order_type_id');
        if ($req->has('order_status_id') && $order->order_status_id != '5' && $order->order_status_id != '10') $order->order_status_id = $req->input('order_status_id');
        if ($req->has('order_status_id') && $req->input('order_status_id') == '10') $order->order_status_id = $req->input('order_status_id');
        if ($req->has('preparation_location_id')) $order->preparation_location_id = $req->input('preparation_location_id');
        if ($req->has('order_action_id')) $order->order_action_id = $req->input('order_action_id');
        if ($req->has('sub_total')) $order->sub_total = $req->input('sub_total');
        if ($req->has('tax')) $order->tax = $req->input('tax');
        if ($req->has('service_charge')) $order->service_charge = $req->input('service_charge');
        if ($req->has('delivery_charge')) $order->delivery_charge = $req->input('delivery_charge');
        if ($req->has('gratuity')) $order->gratuity = $req->input('gratuity');
        if ($req->has('discount')) $order->discount = $req->input('discount');
        if ($req->has('total')) $order->total = $req->input('total');
        if ($req->has('total_paid')) $order->total_paid = $req->input('total_paid');
        if ($req->has('due_amount')) $order->due_amount = $req->input('due_amount');
        if ($req->has('comment')) $order->comment = $req->input('comment');
        if ($req->has('split_type')) $order->split_type = $req->input('split_type');
        if ($req->has('split_count')) $order->split_count = $req->input('split_count');
        if ($req->has('no_guest')) $order->no_guest = $req->input('no_guest');
        if ($req->has('distance')) $order->distance = $req->input('distance');
        if ($req->has('delivery_date')) $order->delivery_date = $req->input('delivery_date');
        if ($req->has('device_id')) $order->device_id = $req->input('device_id');
        if ($req->has('unique_id')) $order->unique_id = $req->input('unique_id');
        $order->is_archived = $req->input('is_archived');
        if ($req->has('delivery_collection_time_slot')) $order->delivery_collection_time_slot = $req->input('delivery_collection_time_slot');
        $order->is_delivery_collection_ask = $req->input('is_delivery_collection_ask');
        $order->is_auto_service_charge_remove = $req->input('is_auto_service_charge_remove');
        if ($req->has('display_order_id')) $order->display_order_id = $req->input('display_order_id');

        if ($dbOrder->order_type_id == 1 && $dbOrder->order_type_id != $req->input('order_type_id')) {
            // $table = EposTable::find($dbOrder->table_id);
            // if (empty($table)) Utils::error('EposTable not found!', 404);
            // $table->table_status_id = 1; // Table Free Status
            // $table->save();
            // $order->table_id = null;
        } else {
            if (!empty($req->input('table_id'))) {
                if ($req->has('table_id')) $order->table_id = $req->input('table_id');
            } else {
                $order->table_id = null;
            }
        }
        if (empty($req->input('customer_id'))) {
            $customerDetail = $req->input('customer');
            $customer = new EposCustomer((array)$customerDetail);
            $customer->membership_no = rand(100000000, 999999999);
            $customer->save();

            $order->customer_id = $customer->id;
        } else {
            $customerDetail = $req->input('customer');
            $customer = EposCustomer::find($customerDetail['id']);
            if (isset($customerDetail['name'])) $customer->name = $customerDetail['name'];
            if (isset($customerDetail['mobile'])) $customer->mobile = $customerDetail['mobile'];
            if (isset($customerDetail['email'])) $customer->email = $customerDetail['email'];
            if (isset($customerDetail['landline'])) $customer->landline = $customerDetail['landline'];
            if (isset($customerDetail['birth_date'])) $customer->birth_date = $customerDetail['birth_date'];
            if (isset($customerDetail['anniversary_date'])) $customer->anniversary_date = $customerDetail['anniversary_date'];
            if (isset($customerDetail['house_no'])) $customer->house_no = $customerDetail['house_no'];
            if (isset($customerDetail['street'])) $customer->street = $customerDetail['street'];
            if (isset($customerDetail['postcode'])) $customer->postcode = $customerDetail['postcode'];
            if (isset($customerDetail['distance'])) $customer->distance = $customerDetail['distance'];
            if (isset($customerDetail['city'])) $customer->city = $customerDetail['city'];
            if (isset($customerDetail['country'])) $customer->country = $customerDetail['country'];
            if (isset($customerDetail['state'])) $customer->state = $customerDetail['state'];
            $customer->save();

            $order->customer_id = $customer->id;
        }
        $order->save();

        $items = ($req->input('order_items'));
        if (!empty($items)) {
            $orderItemIds = [];
            foreach ($items as $item) {
                $reqItem = $item;

                if (!isset($item['product_id']) || $item['product_id'] == '') {
                    $product = new EposProduct;
                    if ($item['product_name']) $product->name = $item['product_name'];
                    if ($item['preparation_location_id']) $product->preparation_location_id = $item['preparation_location_id'];
                    $product->top_color = '#8d8d8d';
                    $product->font_color = '#000000';
                    $product->price = $item['price'];
                    $product->takeaway_price = $item['takeaway_price'];
                    $product->web_price = $item['web_price'];
                    $product->delivery_price = $item['delivery_price'];
                    $product->waiting_price = $item['waiting_price'];
                    $product->favourite = 0;
                    $product->bar_favourite = 0;
                    $product->tax_included = 0;
                    $product->taxable = 0;
                    $product->auto_modify = 0;
                    $product->stockable = 0;
                    $product->misc = 1;
                    $product->save();
                    $item['product_id'] = $product->id;
                }

                if (!isset($item['id'])) {
                    $orderItem = new EposOrderItem((array)$item);
                    if (!isset($item['table_type'])) {
                        $orderItem->order_id = $order->id;
                    }
                    $orderItem->save();
                } else {
                    try {
                        $orderItem = EposOrderItem::find($item['id']);

                        if (empty($orderItem)) {
                            $orderItem = new EposOrderItem((array)$item);
                            if (!isset($item['table_type'])) {
                                $orderItem->order_id = $order->id;
                            }
                            $orderItem->save();
                        } else {
                            if (isset($item['order_split_id'])) {
                                $orderItem->order_split_id = $item['order_split_id'];
                            } else {
                                $orderItem->order_split_id = null;
                            }
                            $orderItem->price = $item['price'];
                            $orderItem->quantity = $item['quantity'];
                            $orderItem->sub_total = $item['sub_total'];
                            if (isset($item['is_delete'])) $orderItem->is_delete = $item['is_delete'];
                            if (isset($item['unique_id']))
                                $orderItem->unique_id = $item['unique_id'];
                            if (isset($item['sent_to_kitchen_quantity']))
                                $orderItem->sent_to_kitchen_quantity = $item['sent_to_kitchen_quantity'];
                            if (isset($item['item_unique_id']))
                                $orderItem->item_unique_id = $item['item_unique_id'];
                            if (isset($item['addons_price']))
                                $orderItem->addons_price = $item['addons_price'];
                            if (isset($item['ingredients_price']))
                                $orderItem->ingredients_price = $item['ingredients_price'];
                            if (isset($item['total']))
                                $orderItem->total = $item['total'];
                            if (isset($item['special_instruction']))
                                $orderItem->special_instruction = $item['special_instruction'];
                            if (isset($item['sent_to_kitchen']))
                                $orderItem->sent_to_kitchen = $item['sent_to_kitchen'];
                            if (isset($item['instruction_price']))
                                $orderItem->instruction_price = $item['instruction_price'];
                            $orderItem->save();
                        }
                    } catch (Exception $e) {
                        Log::debug($e->getMessage());
                    }
                }

                $orderItemIds[] = $orderItem->id;


                if (isset($reqItem['order_item_addons'])) {
                    if (!is_array($reqItem['order_item_addons']))
                        $reqItem['order_item_addons'] = (json_decode($reqItem['order_item_addons'], true));

                    $orderItemAddonIds = [];
                    foreach ($reqItem['order_item_addons'] as $addon) {
                        if (!isset($addon['id'])) {
                            $orderItemAddon = new EposOrderItemAddon($addon);
                            $orderItemAddon->order_item_id = $orderItem->id;
                            $orderItemAddon->save();
                        } else {
                            $orderItemAddon = EposOrderItemAddon::find($addon['id']);
                            if (empty($orderItemAddon)) {
                                $orderItemAddon = new EposOrderItemAddon($addon);
                                $orderItemAddon->order_item_id = $orderItem->id;
                                $orderItemAddon->save();
                            } else {
                                $orderItemAddon->order_item_id = $orderItem->id;
                                if (isset($addon['updater_id']))
                                    $orderItemAddon->updater_id = $addon['updater_id'];

                                $orderItemAddon->addon_id = $addon['addon_id'];
                                $orderItemAddon->price = $addon['price'];
                                $orderItemAddon->quantity = $addon['quantity'];
                                $orderItemAddon->total = $addon['total'];
                                $orderItemAddon->save();
                            }
                        }
                        $orderItemAddonIds[] = $orderItemAddon->id;
                    }
                    $orderItemAddonDelete = EposOrderItemAddon::whereNotIn('id', $orderItemAddonIds);
                    $orderItemAddonDelete->where('order_item_id', $orderItem->id);
                    $orderItemAddonDelete->delete();
                }

                if (isset($reqItem['order_item_ingredients'])) {
                    if (!is_array($reqItem['order_item_ingredients']))
                        $reqItem['order_item_ingredients'] = (json_decode($reqItem['order_item_ingredients'], true));
                    $orderItemIngredientIds = [];

                    foreach ($reqItem['order_item_ingredients'] as $ingredient) {
                        if (!isset($ingredient['id'])) {
                            $orderItemIngredient = new EposOrderItemIngredient($ingredient);
                            $orderItemIngredient->order_item_id = $orderItem->id;
                            $orderItemIngredient->save();
                        } else {
                            $orderItemIngredient = EposOrderItemIngredient::find($ingredient['id']);
                            if (empty($orderItemIngredient)) {
                                $orderItemIngredient = new EposOrderItemIngredient($ingredient);
                                $orderItemIngredient->order_item_id = $orderItem->id;
                                $orderItemIngredient->save();
                            } else {
                                $orderItemIngredient->order_item_id = $orderItem->id;
                                if (isset($ingredient['updater_id']))
                                    $orderItemIngredient->updater_id = $ingredient['updater_id'];
                                $orderItemIngredient->product_ingredient_id = $ingredient['product_ingredient_id'];
                                $orderItemIngredient->with = $ingredient['with'];
                                $orderItemIngredient->without = $ingredient['without'];
                                $orderItemIngredient->price = $ingredient['price'];
                                $orderItemIngredient->quantity = $ingredient['quantity'];
                                $orderItemIngredient->total = $ingredient['total'];
                                $orderItemIngredient->save();
                            }
                        }
                        $orderItemIngredientIds[] = $orderItemIngredient->id;
                    }

                    $orderItemIngredientDelete = EposOrderItemIngredient::whereNotIn('id', $orderItemIngredientIds);
                    $orderItemIngredientDelete->where('order_item_id', $orderItem->id);
                    $orderItemIngredientDelete->delete();
                }
            }
            $orderItemDelete = EposOrderItem::whereNotIn('id', $orderItemIds);
            $orderItemDelete->where('order_id', $order->id);
            $orderItemDelete->delete();
        } else {
            $delete = EposOrderItem::where('order_id', $order->id)->delete();
        }
        $order->append('order_items');
        // $order->append('order_splits');
        // $order->append('order_payments');

        //Loyalty Calculate point

        $loyalty_points = EposLoyaltyPoint::where('order_type_id', $order->order_type_id)->where('minimum_order_value', '<=', "$order->sub_total")->first();

        $order_payment = EposOrderPayment::where('payment_method_id', 6)->where('order_id', $order->id)->first();

        if (!empty($loyalty_points) && empty($order_payment)) {
            $customer = EposCustomer::where('id', $order->customer_id)->where('membership_points', '>=', "$loyalty_points->spent_point")->first();
            if (!empty($customer)) {
                $calculate_loyaltypoint = '';
                $calculate_loyaltyamount = '';
                if ($loyalty_points->reward_type == 'flat') {
                    $calculate_loyaltypoint = $loyalty_points->spent_point;
                    $calculate_loyaltyamount = $loyalty_points->spent_percentage;
                } else {
                    $calculate_loyaltypoint = $loyalty_points->spent_point;
                    $calculate_loyaltyamount = (($order->sub_total * $loyalty_points->spent_percentage) / $loyalty_points->spent_point);
                }

                $maxLoyaltyPoint = (new EposSettingController())->findByKey('maximum_loyalty_point')->value;
                if ($calculate_loyaltyamount > $maxLoyaltyPoint) {
                    $calculate_loyaltyamount = $maxLoyaltyPoint;
                    $calculate_loyaltyamount = (($loyalty_points->spent_point * $loyalty_points->spent_percentage) / $calculate_loyaltyamount);
                }

                $order->calculate_loyalty_point = $calculate_loyaltypoint;
                $order->calculate_loyalty_amount = $calculate_loyaltyamount;
            }
        }

        // if (!empty($order->table_id) && $req->input('order_status_id') == 1) {
        //     $table = EposTable::find($order->table_id);
        //     if (empty($table)) Utils::error('EposTable not found!', 404);
        //     $table->table_status_id = 3; // Table Taking Status
        //     $table->save();
        // }

        // if (!empty($order->table_id) && $req->input('order_status_id') == 2) {
        //     $table = EposTable::find($order->table_id);
        //     if (empty($table)) Utils::error('EposTable not found!', 404);
        //     $table->table_status_id = 2; // Table Taken Status
        //     $table->save();
        // }

        // if (!empty($order->table_id) && ($req->input('order_status_id') == 10 || $req->input('order_status_id') == 5)) {
        //     $table = EposTable::find($order->table_id);
        //     if (empty($table)) Utils::error('EposTable not found!', 404);
        //     $table->table_status_id = 1; // Table Free Status
        //     $table->save();
        // }
        $order->append('customer');
        $order->append('table');

        $message      = 'You have a new ' . $order->order_type . ' order - ' . $order->id;

        $notificationdata['data']['title']          = "New " . $order->order_type . " order";
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['print']  = ($req->input('print') != '') ? true : false;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['order_id']  = $order->id;
        $notificationdata['data']['updater_id']  = $order->updater_id;
        $notificationdata['data']['order_type']  = ucfirst($order->order_type);
        $notificationdata['data']['notification_for']  = 'epos';
        $notificationdata['data']['notification_section']  = 'orders';
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
        $notificationdata['data']['business_id']      = Auth::$business_id;

        $topic = '/topics/business_' . Auth::$business_id;
        $fcm_api_key = (new SiteSettingController())->findByKey('fcm_api_key');
        if (!empty($fcm_api_key)) {
            $api_key = $fcm_api_key->value;
            if ($req->input('send_push', "1") == "1") {
                (new PushNotificationHelper())->sendNotification($notificationdata, $topic, $api_key);
            }
        }

        $devices = EposDevice::orderByRaw('created_at DESC')->whereNotNull('device_token')->where('type', 'iOS')->where('disabled', 0)->get();
        if (!$devices->isEmpty()) {
            foreach ($devices as $device) {
                $message = 'You have ' . $order->order_type . ' order - ' . $order->id;
                (new PushIosNotificationHelper())->sendNotification($message, $device->device_token, $order->id, $order->order_type, 'orders');
            }
        }

        if ($order->order_status_id == "5") {
            $royaltyPointSetting = (new EposSettingController())->findByKey("how_much_purchase_in_pound_will_construct_one_loyalty_point");
            if (!empty($royaltyPointSetting) && $royaltyPointSetting->value != null && $royaltyPointSetting->value != 0) {
                $points = (int)($order->sub_total / $royaltyPointSetting->value);
                $customer = EposCustomer::find($order->customer_id);
                if (!empty($customer)) {
                    $customer->membership_points = $points + $customer->membership_points;
                    $cust = $customer->save();
                    $order->customer = $cust;
                }
            }
        }

        if (!empty($req->input('table'))) {
            $table = Request::create("");
            $table->merge($req->table, true);
            (new EposTableController())->update($table, $table['id']);
        }

        if (!empty($req->input('order_payments'))) {
            $Allpayment = array();
            foreach ($req->order_payments as $payment) {
                $payment['order_id'] = $order->id;
                $Allpayment[] = $payment;
            }
            $order_payments = Request::create("");
            $order_payments->merge(['payments' => json_encode($Allpayment, true), true]);
            // $order_payments = Request::create("");
            // $order_payments->merge(['payments' => json_encode($req->order_payments, true), true]);
            (new EposOrderPaymentController())->payment_bulk($order_payments);
        }

        if (!empty($req->input('order_splits'))) {
            $order_splits = Request::create("");
            $order_splits->merge(['splits' => json_encode($req->order_splits, true), true]);
            (new EposOrderSplitController())->split_bulk($order_splits);
        }

        $splits = ($req->input('order_splits'));
        if (!empty($splits)) {
            foreach ($splits as $split) {
                if (isset($split['unique_id'])) {
                    $orderSplit = EposOrderSplit::where('unique_id', $split['unique_id'])->first();
                    if (!empty($orderSplit)) {
                        EposOrderItem::where('unique_id', $split['unique_id'])->update(['order_split_id' => $orderSplit->id]);
                        EposOrderPayment::where('unique_id', $split['unique_id'])->update(['order_split_id' => $orderSplit->id]);
                    }
                }
            }
        }

        $order = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')->where('orders.id', $order->id)->first();
        $order->append('order_items');
        $order->append('order_payments');
        $order->append('order_splits');
        $order->append('customer');
        $order->append('table');

        $order->business_id = $req->input('business_id');

        return $order;
    }

    /**
     * Archive UnArchive Bulk Orders.
     * POST /epos/orders/bulk-archive
     */
    public function bulk_archive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $archivedOrders = EposOrder::whereIn('id', $req->input('ids'))->update(['is_archived' => '1']);
        }
        return  $req->input('ids');
    }
    /**
     * Archive UnArchive Bulk Orders.
     * POST /epos/orders/bulk-unarchive
     */
    public function bulk_unarchive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $unArchivedOrders = EposOrder::whereIn('id', $req->input('ids'))->update(['is_archived' => '0']);
        }
        return  $req->input('ids');
    }


    /**
     * Delete order by id.
     * DELETE /epos/orders/{id}
     */
    public function delete(Request $req, $id)
    {
        $order = EposOrder::find($id);
        if (empty($order)) Utils::error('EposOrder not found!', 404);
        $order->delete();

        if (!empty($order->table_id)) {
            $table = EposTable::find($order->table_id);
            if (empty($table)) Utils::error('EposTable not found!', 404);
            if (empty($table->merge_table_id)) {
                $table->table_status_id = 1; // Table Free Status
                $table->save();
            }
        }

        $message = 'order delete successfully!';
        $notification['title'] = "order delete successfully!";
        $notification['body']  = $message;
        $notificationdata['data']['title'] = "order delete successfully!";
        $notificationdata['data']['message'] = $message;
        $notificationdata['data']['order_id']  = $order->id;
        $notificationdata['data']['updater_id']  = $order->updater_id;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['notification_for']  = 'epos';
        $notificationdata['data']['notification_section']  = 'order_delete';
        $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
        $notificationdata['data']['business_id']      = Auth::$business_id;

        $topic = '/topics/business_' . Auth::$business_id;
        $fcm_api_key = (new SiteSettingController())->findByKey('fcm_api_key');
        if (!empty($fcm_api_key)) {
            $api_key = $fcm_api_key->value;
            (new PushNotificationHelper())->sendNotification($notificationdata, $topic, $api_key, $notification);
        }

        return $order;
    }

    /**
     * Swap Table change.
     * GET All/epos/orders/{id}/swap-table
     */
    public function swap_table(Request $req, $id)
    {
        $this->validate($req, [
            'old_table_id' => 'required',
            'new_table_id' => 'required',
        ]);

        // Fetch Order
        $order = EposOrder::find($id);
        if (empty($order)) Utils::error('EposOrder not found!', 404);
        if ($req->has('new_table_id')) $order->table_id = $req->input('new_table_id');
        $order->save();

        if (!empty($order->table_id)) {
            $old_table = EposTable::find($req->input('old_table_id'));
            if (empty($old_table)) Utils::error('EposTable not found!', 404);
            $old_tableStatusId = $old_table->table_status_id;
            $old_table->table_status_id = 1; // Table Free Status
            $old_table->save();
        }
        if (!empty($order->table_id)) {
            $new_table = EposTable::find($req->input('new_table_id'));
            if (empty($new_table)) Utils::error('EposTable not found!', 404);
            $new_table->table_status_id = $old_tableStatusId;
            $new_table->save();
        }

        return $order;
    }

    /**
     * ready to collect status change.
     * GET All/epos/orders/{id}/ready-to-collect
     */
    public function ready_to_collect(Request $req, $id)
    {
        $order = EposOrder::find($id);
        if (empty($order)) Utils::error('order not found!', 404);

        $customer = EposCustomer::find($order->customer_id);
        $message = "Your order #" . $id . " has been Ready.please collect your order";

        (new OtpHelper())->sendMessage($message, $customer->mobile, 'epos_sms');

        //Order Status Change Ready To Collect
        $order->order_status_id = 19;
        $order->save();

        return $order;
    }

    /**
     * on the way status change.
     * GET All/epos/orders/{id}/on-the-way
     */
    public function on_the_way(Request $req, $id)
    {
        $order = EposOrder::find($id);
        $customer = EposCustomer::find($order->customer_id);
        $message = "Your order #$id is on the way";
        (new OtpHelper())->sendMessage($message, $customer->mobile, 'epos_sms');

        //Order Status Change On the way
        $order->order_status_id = 20;
        $order->save();

        return $order;
    }

    /**
     * Delete All reservation.
     * GET All/epos/orders/delete-all
     */
    public function delete_all(Request $req)
    {
        (new EposOrder)->newQueryWithoutScopes()->forceDelete();
        DB::connection('epos_connection')->table('tables')->update(array('table_status_id' => 1));
        DB::connection('epos_connection')->statement("ALTER TABLE orders AUTO_INCREMENT =  0");

        EposTable::whereNotNull('merge_table_id')->update(['merge_table_id' => NULL]);
        EposTable::whereNotNull('id')->update(['locked' => '0']);

        // (new EposOrderItem())->newQueryWithoutScopes()->forceDelete();
        // DB::connection('epos_connection')->statement("ALTER TABLE order_items AUTO_INCREMENT =  0");

        // (new EposOrderItemAddon())->newQueryWithoutScopes()->forceDelete();
        // DB::connection('epos_connection')->statement("ALTER TABLE order_item_addons AUTO_INCREMENT =  0");

        // (new EposOrderItemIngredient())->newQueryWithoutScopes()->forceDelete();
        // DB::connection('epos_connection')->statement("ALTER TABLE order_item_ingredients AUTO_INCREMENT =  0");

        // (new EposOrderPayment())->newQueryWithoutScopes()->forceDelete();
        // DB::connection('epos_connection')->statement("ALTER TABLE order_payments AUTO_INCREMENT =  0");

        // Factory reset notification
        $message = 'order reset done!';
        $notification['title'] = "order reset done!";
        $notification['body']  = $message;
        $notificationdata['data']['title'] = "order reset done!";
        $notificationdata['data']['message'] = $message;
        $notificationdata['data']['business_id'] = Auth::$business_id;
        $notificationdata['data']['device_id'] = $req->input('device_id');
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['notification_for']  = 'epos';
        $notificationdata['data']['notification_section']  = 'order_factory_reset';
        $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
        $notificationdata['data']['business_id'] = Auth::$business_id;

        $topic = '/topics/business_' . Auth::$business_id;
        $fcm_api_key = (new SiteSettingController())->findByKey('fcm_api_key');
        if (!empty($fcm_api_key)) {
            $api_key = $fcm_api_key->value;
            (new PushNotificationHelper())->sendNotification($notificationdata, $topic, $api_key, $notification);
        }

        return ['message' => 'orders reset done!'];
    }
}
