<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposOrderItemAddon;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Illuminate\Http\Request;

class EposOrderItemAddonController extends Controller
{
    /**
     * List order_item_addons.
     * GET /epos/order_item_addons
     */
    public function index(Request $req)
    {
        $order_item_addons = EposOrderItemAddon::selectRaw('order_item_addons.*')->orderByRaw('order_item_addons.created_at DESC');


        if ($req->query('nopaginate', false) == '1') {
            $order_item_addons = $order_item_addons->get();
        } else {
            $order_item_addons = $order_item_addons->paginate($req->query('per_page', 20));
        }

        if ($order_item_addons->isEmpty()) Utils::error('No order_item_addons found!', 404);


        return ($order_item_addons);
    }

    /**
     * Create table.
     * POST /epos/order_item_addons
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_item_id' => 'required|exists:epos_connection.order_items,id',
            'addon_id' => 'required|exists:epos_connection.addons,id',
            'price' => 'required|numeric',
            'quantity' => 'required|numeric',
            'total' => 'required|numeric',
        ]);

        // create order
        $order = new EposOrderItemAddon;
        $order->fill($req->all());
        $order->save();

        return $order;
    }

    /**
     * Show table.
     * GET /epos/orders/{id}
     */
    public function show(Request $req, $id)
    {
        $order_item_addons = EposOrderItemAddon::selectRaw('order_item_addons.*')->find($id);
        if (empty($order_item_addons)) Utils::error('EposOrderItemAddon not found!', 404);
        return $order_item_addons;
    }

    /**
     * Update order.
     * POST /epos/orders/{id}
     */
    public function update(Request $req, $id)
    {
        $order_item_addons = EposOrderItemAddon::find($id);
        if (empty($order_item_addons)) Utils::error('EposOrderItemAddon not found!', 404);

        $this->validate($req, [
            'order_item_id' => 'required|exists:epos_connection.order_items,id',
            'addon_id' => 'required|exists:epos_connection.addons,id',
            'price' => 'required|numeric',
            'quantity' => 'required|numeric',
            'total' => 'required|numeric',
        ]);

        // update order
        if ($req->has('order_item_id')) $order_item_addons->order_item_id = $req->input('order_item_id');
        if ($req->has('addon_id')) $order_item_addons->addon_id = $req->input('addon_id');
        if ($req->has('price')) $order_item_addons->price = $req->input('price');
        if ($req->has('quantity')) $order_item_addons->quantity = $req->input('quantity');
        if ($req->has('total')) $order_item_addons->total = $req->input('total');

        $order_item_addons->save();

        return $order_item_addons;
    }


    /**
     * Delete order by id.
     * DELETE /epos/orders/{id}
     */
    public function delete(Request $req, $id)
    {
        $order = EposOrderItemAddon::find($id);
        if (empty($order)) Utils::error('EposOrderItemAddon not found!', 404);
        $order->delete();
        return $order;
    }
}
