<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposOrderItem;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposOrderItemAddon;
use App\ModelsEpos\EposOrderItemIngredient;
use Illuminate\Http\Request;

class EposOrderItemController extends Controller
{
    /**
     * List order_items.
     * GET /epos/order_items
     */
    public function index(Request $req)
    {
        $order_items = EposOrderItem::selectRaw('order_items.*')->orderByRaw('order_items.created_at DESC');

        if ($req->query('nopaginate', false) == '1') {
            $order_items = $order_items->get();
        } else {
            $order_items = $order_items->paginate($req->query('per_page', 20));
        }

        if ($order_items->isEmpty()) Utils::error('No order_items found!', 404);


        return ($order_items);
    }

    /**
     * Create table.
     * POST /epos/order_items
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'required|exists:epos_connection.orders,id',
            'product_id' => 'required|exists:epos_connection.products,id',
            'order_split_id' => 'exists:epos_connection.order_splits,id',
            'price' => 'required|numeric|gt:0',
            'quantity' => 'required|numeric',
            'sub_total' => 'required|numeric',
            'order_item_addons' => 'array',
            'order_item_ingredients' => 'array',
            'sent_to_kitchen_quantity' => 'numeric',
            'item_unique_id' => 'string',
            'is_delete' => 'boolean',
        ]);

        // create order
        $orderItem = new EposOrderItem;
        $orderItem->fill($req->all());
        $orderItem->save();

        $itemAddons = ($req->input('order_item_addons'));
        if (!empty($itemAddons)) {
            if (!is_array($itemAddons))
                $itemAddons = (json_decode($itemAddons, true));

            foreach ($itemAddons as $addon) {
                $orderItemAddon = new EposOrderItemAddon($addon);
                $orderItemAddon->order_item_id = $orderItem->id;
                $orderItemAddon->save();
            }
        }

        $itemIngredients = ($req->input('order_item_ingredients'));
        if (!empty($itemIngredients)) {
            if (!is_array($itemIngredients))
                $itemIngredients = (json_decode($itemIngredients, true));

            foreach ($itemIngredients as $ingredient) {
                $orderItemIngredient = new EposOrderItemIngredient($ingredient);
                $orderItemIngredient->order_item_id = $orderItem->id;
                $orderItemIngredient->save();
            }
        }
        return $orderItem;
    }

    /**
     * Show order_items.
     * GET /epos/order_items/{id}
     */
    public function show(Request $req, $id)
    {
        $orderItems = EposOrderItem::selectRaw('order_items.*')->find($id);
        if (empty($orderItems)) Utils::error('EposOrderItem not found!', 404);
        return $orderItems;
    }

    /**
     * Update order.
     * POST /epos/order_items/{id}
     */
    public function update(Request $req, $id)
    {
        $orderItem = EposOrderItem::find($id);
        if (empty($orderItem)) Utils::error('EposOrderItem not found!', 404);

        $this->validate($req, [
            'order_id' => 'required|exists:epos_connection.orders,id',
            'product_id' => 'required|exists:epos_connection.products,id',
            'order_split_id' => 'exists:epos_connection.order_splits,id',
            'price' => 'required|numeric|gt:0',
            'quantity' => 'required|numeric',
            'sub_total' => 'required|numeric',
            'order_item_addons' => 'array',
            'order_item_ingredients' => 'array',
            'sent_to_kitchen_quantity' => 'numeric',
            'item_unique_id' => 'string',
            'is_delete' => 'boolean',
        ]);

        // update orderItem
        if ($req->has('order_id')) $orderItem->order_id = $req->input('order_id');
        if ($req->has('product_id')) $orderItem->product_id = $req->input('product_id');
        if ($req->has('order_split_id')) $orderItem->order_split_id = $req->input('order_split_id');
        if ($req->has('price')) $orderItem->price = $req->input('price');
        if ($req->has('quantity')) $orderItem->quantity = $req->input('quantity');
        if ($req->has('sub_total')) $orderItem->sub_total = $req->input('sub_total');
        if ($req->has('sent_to_kitchen')) $orderItem->sent_to_kitchen = $req->input('sent_to_kitchen');
        if ($req->has('sent_to_kitchen_quantity')) $orderItem->sent_to_kitchen_quantity = $req->input('sent_to_kitchen_quantity');
        if ($req->has('item_unique_id')) $orderItem->item_unique_id = $req->input('item_unique_id');
        $orderItem->is_delete = $req->input('is_delete');

        $orderItem->save();

        $itemAddons = ($req->input('order_item_addons'));
        if (!empty($itemAddons)) {
            if (!is_array($itemAddons))
                $itemAddons = (json_decode($itemAddons, true));
            foreach ($itemAddons as $addon) {
                $orderItemAddon = new EposOrderItemAddon($addon);
                $orderItemAddon->order_item_id = $orderItem->id;
                $orderItemAddon->save();
            }
        }

        $itemIngredients = ($req->input('order_item_ingredients'));
        if (!empty($itemIngredients)) {
            if (!is_array($itemIngredients))
                $itemIngredients = (json_decode($itemIngredients, true));
            foreach ($itemIngredients as $ingredient) {
                $orderItemAddon = new EposOrderItemIngredient($ingredient);
                $orderItemAddon->order_item_id = $orderItem->id;
                $orderItemAddon->save();
            }
        }

        return $orderItem;
    }


    /**
     * Delete order by id.
     * DELETE /epos/order_items/{id}
     */
    public function delete(Request $req, $id)
    {
        $orderItem = EposOrderItem::find($id);
        if (empty($orderItem)) Utils::error('EposOrderItem not found!', 404);
        $orderItem->delete();
        return $orderItem;
    }
}