<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposOrderItemIngredient;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Illuminate\Http\Request;

class EposOrderItemIngredientController extends Controller
{
    /**
     * List order_item_ingredients.
     * GET /epos/order_item_ingredients
     */
    public function index(Request $req)
    {
        $order_item_ingredients = EposOrderItemIngredient::selectRaw('order_item_ingredients.*')->orderByRaw('order_item_ingredients.created_at DESC');

       
        if ($req->query('nopaginate', false) == '1') {
            $order_item_ingredients = $order_item_ingredients->get();
        } else {
            $order_item_ingredients = $order_item_ingredients->paginate($req->query('per_page', 20));
        }

        if ($order_item_ingredients->isEmpty()) Utils::error('No order_item_ingredients found!', 404);


        return ($order_item_ingredients);
    }

    /**
     * Create table.
     * POST /epos/order_item_ingredients
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_item_id' => 'required|exists:epos_connection.order_items,id',
            'product_ingredient_id' => 'required|exists:epos_connection.product_ingredients,id',
            'with' => 'boolean',
            'without' => 'boolean',
            'price' => 'required|numeric',
            'quantity' => 'required|numeric',
            'total' => 'required|numeric',
        ]);

        // create order
        $order = new EposOrderItemIngredient;
        $order->fill($req->all());
        $order->save();

        return $order;
    }

    /**
     * Show table.
     * GET /epos/orders/{id}
     */
    public function show(Request $req, $id)
    {
        $order_item_ingredients = EposOrderItemIngredient::selectRaw('order_item_ingredients.*')->find($id);
        if (empty($order_item_ingredients)) Utils::error('EposOrderItemIngredient not found!', 404);
        return $order_item_ingredients;
    }

    /**
     * Update order.
     * POST /epos/orders/{id}
     */
    public function update(Request $req, $id)
    {
        $order_item_ingredients = EposOrderItemIngredient::find($id);
        if (empty($order_item_ingredients)) Utils::error('EposOrderItemIngredient not found!', 404);

        $this->validate($req, [
            'order_item_id' => 'required|exists:epos_connection.order_items,id',
            'product_ingredient_id' => 'required|exists:epos_connection.product_ingredients,id',
            'with' => 'boolean',
            'without' => 'boolean',
            'price' => 'required|numeric',
            'quantity' => 'required|numeric',
            'total' => 'required|numeric',
        ]);

        // update order
        if ($req->has('order_item_id')) $order_item_ingredients->order_item_id = $req->input('order_item_id');
        if ($req->has('product_ingredient_id')) $order_item_ingredients->product_ingredient_id = $req->input('product_ingredient_id');
        $order_item_ingredients->with = $req->input('with');
        if ($req->has('price')) $order_item_ingredients->price = $req->input('price');
        if ($req->has('quantity')) $order_item_ingredients->quantity = $req->input('quantity');
        if ($req->has('total')) $order_item_ingredients->total = $req->input('total');

        $order_item_ingredients->save();

        return $order_item_ingredients;
    }


    /**
     * Delete order by id.
     * DELETE /epos/orders/{id}
     */
    public function delete(Request $req, $id)
    {
        $order = EposOrderItemIngredient::find($id);
        if (empty($order)) Utils::error('EposOrderItemIngredient not found!', 404);
        $order->delete();
        return $order;
    }

   
}
