<?php

namespace App\Http\Controllers\Epos;

use App\DeviceRegistration;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\PushIosNotificationHelper;
use App\Helpers\PushNotificationHelper;
use App\ModelsEpos\EposOrderLog;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposDevice;
use Illuminate\Http\Request;

class EposOrderLogController extends Controller
{
    /**
     * List order-log.
     * GET /epos/order-logs
     */
    public function index(Request $req)
    {
        $order_logs = EposOrderLog::orderByRaw('created_at DESC');

        if ($req->query('nopaginate', false) == '1') {
            $order_logs = $order_logs->get();
        } else {
            $order_logs = $order_logs->paginate($req->query('per_page', 20));
        }

        if ($order_logs->isEmpty()) Utils::error('No order logs found!', 404);

        return ($order_logs);
    }

    /**
     * Create order-log.
     * POST /epos/order-logs
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'log_file' => 'required',
        ]);

        // create order_log
        $order_log = new EposOrderLog;
        $order_log->fill($req->all());
        if ($req->has('log_file')) {
            $fileName = ImageHelper::saveImageToServer($req->file('log_file'), ImageHelper::$TYPE_EPOS_ORDER_LOG);
            $order_log->log_file = $fileName;
        }
        $order_log->save();

        return $order_log;
    }

    /**
     * Show order-log.
     * GET /epos/order-logs/{id}
     */
    public function show(Request $req, $id)
    {
        $order_log = EposOrderLog::find($id);
        if (empty($order_log)) Utils::error('Order log not found!', 404);
        return $order_log;
    }

    /**
     * Update order-log.
     * POST /epos/order-logs/{id}
     */
    public function update(Request $req, $id)
    {
        $order_log = EposOrderLog::find($id);
        if (empty($order_log)) Utils::error('Order log not found!', 404);

        $this->validate($req, [
            'log_file' => 'required',
        ]);

        // update order_log
        if ($req->has('log_file')) {
            $fileName = ImageHelper::saveImageToServer($req->file('log_file'), ImageHelper::$TYPE_EPOS_ORDER_LOG);
            $order_log->log_file = $fileName;
        }
        $order_log->save();

        return $order_log;
    }


    /**
     * Delete order-log by id.
     * DELETE /epos/order-logs/{id}
     */
    public function delete(Request $req, $id)
    {
        $order_log = EposOrderLog::find($id);
        if (empty($order_log)) Utils::error('Order log not found!', 404);
        $order_log->delete();
        return $order_log;
    }
}
