<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Auth;
use App\ModelsEpos\EposOrderSplit;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposOrderSplitController extends Controller
{
    /**
     * List order payments.
     * GET /epos/order payments
     */
    public function index(Request $req)
    {
        $order_splits = EposOrderSplit::selectRaw("order_splits.*, (SELECT SUM(amount) from order_payments WHERE order_split_id= order_splits.id)as paid_amount")->orderByRaw('created_at DESC');

        if ($query = $req->query('order_id', false)) {
            $order_splits->where('order_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $order_splits = $order_splits->get();
        } else {
            $order_splits = $order_splits->paginate($req->query('per_page', 20));
        }

        if ($order_splits->isEmpty()) Utils::error('No order splits found!', 404);


        return ($order_splits);
    }

    /**
     * Create table.
     * POST /epos/order_splits
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'required|exists:epos_connection.orders,id',
            'group_name' => 'string',
            'total' => 'numeric',
        ]);

        // create order_split
        $order_split = new EposOrderSplit;
        $order_split->fill($req->all());
        $order_split->save();

        return $order_split;
    }

    /**
     * Show table.
     * GET /epos/order_splits/{id}
     */
    public function show(Request $req, $id)
    {
        $order_split = EposOrderSplit::find($id);
        if (empty($order_split)) Utils::error('EposOrderSplit not found!', 404);
        return $order_split;
    }

    /**
     * Update order.
     * POST /epos/order_splits/{id}
     */
    public function update(Request $req, $id)
    {
        $order_split = EposOrderSplit::find($id);
        if (empty($order_split)) Utils::error('EposOrderSplit not found!', 404);

        $this->validate($req, [
            'order_id' => 'required|exists:epos_connection.orders,id',
            'group_name' => 'string',
            'total' => 'numeric',
        ]);

        // update order_split
        if ($req->has('order_id')) $order_split->order_id = $req->input('order_id');
        if ($req->has('group_name')) $order_split->group_name = $req->input('group_name');
        if ($req->has('total')) $order_split->total = $req->input('total');

        $order_split->save();

        return $order_split;
    }


    /**
     * Delete order by id.
     * DELETE /epos/order_splits/{id}
     */
    public function delete(Request $req, $id)
    {
        $order_split = EposOrderSplit::find($id);
        if (empty($order_split)) Utils::error('EposOrderSplit not found!', 404);
        $order_split->delete();
        return $order_split;
    }


    public function split_bulk(Request $req)
    {
        if ($req->has('splits') && !is_array($req->input('splits'))) {
            $req->merge(['splits' => json_decode($req->input('splits'))]);
        }
        $this->validate($req, [
            'splits' => 'required|array'
        ],);
        $splits = ($req->input('splits'));
        $orderSplits = [];
        foreach ($splits as $split) {
            if (empty($split->id)) {
                $orderSplit = new EposOrderSplit((array)$split);
                $orderSplit->save();
            } else {
                $orderSplit = EposOrderSplit::find($split->id);
                $orderSplit->order_id = $split->order_id;
                $orderSplit->group_name = $split->group_name;
                $orderSplit->total = $split->total;
                if (isset($split->unique_id)) $orderSplit->unique_id = $split->unique_id;
                $orderSplit->save();
            }
            $orderSplits[] = $orderSplit;
        }

        return $orderSplits;
    }
}
