<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposOrderStatus;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposOrderStatusController extends Controller
{
    /**
     * List orderStatuses.
     * GET /api/order-statuses
     */
    public function index(Request $req)
    {
        $orderStatuses = EposOrderStatus::orderByRaw('status ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $orderStatuses->whereRaw('(status LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $orderStatuses = $orderStatuses->get();
        } else {
            $orderStatuses = $orderStatuses->paginate($req->query('per_page', 20));
        }

        if ($orderStatuses->isEmpty()) Utils::error('No orderStatuses found!', 404);


        return ($orderStatuses);
    }

    /**
     * Create orderStatus.
     * POST /api/order-statuses
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'status' => 'required|string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // create orderStatus
        $orderStatus = new EposOrderStatus;
        $orderStatus->fill($req->all());

        $orderStatus->save();

        return $orderStatus;
    }

    /**
     * Show orderStatus.
     * GET /api/order-statuses/{id}
     */
    public function show(Request $req, $id)
    {
        $orderStatus = EposOrderStatus::find($id);
        if (empty($orderStatus)) Utils::error('EposOrderStatus not found!', 404);

        return $orderStatus;
    }

    /**
     * Update orderStatus.
     * POST /api/order-statuses/{id}
     */
    public function update(Request $req, $id)
    {
        $orderStatus = EposOrderStatus::find($id);
        if (empty($orderStatus)) Utils::error('EposOrderStatus not found!', 404);

        $this->validate($req, [
            'status' => 'string|max:255',
            'top_color' => 'string|max:9|min:7',
            'bottom_color' => 'string|max:9|min:7',
        ]);

        // update orderStatus
        if ($req->has('status')) $orderStatus->status = $req->input('status');
        if ($req->has('top_color')) $orderStatus->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $orderStatus->bottom_color = $req->input('bottom_color');

        $orderStatus->save();

        return $orderStatus;
    }


    /**
     * Delete orderStatus by id.
     * DELETE /api/order-statuses/{id}
     */
    public function delete(Request $req, $id)
    {
        $orderStatus = EposOrderStatus::find($id);
        if (empty($orderStatus)) Utils::error('EposOrderStatus not found!', 404);
        $orderStatus->delete();
        return $orderStatus;
    }
}
