<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposOrderType;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposOrderTypeController extends Controller
{
    /**
     * List orderTypes.
     * GET /epos/order-types
     */
    public function index(Request $req)
    {
        $orderTypes = EposOrderType::orderByRaw('type ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $orderTypes->whereRaw('(type LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $orderTypes = $orderTypes->get();
        } else {
            $orderTypes = $orderTypes->paginate($req->query('per_page', 20));
        }

        if ($orderTypes->isEmpty()) Utils::error('No orderTypes found!', 404);


        return ($orderTypes);
    }

    /**
     * Create orderType.
     * POST /epos/order-types
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'type' => 'required|string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // create orderType
        $orderType = new EposOrderType;
        $orderType->fill($req->all());

        $orderType->save();

        return $orderType;
    }

    /**
     * Show orderType.
     * GET /epos/order-types/{id}
     */
    public function show(Request $req, $id)
    {
        $orderType = EposOrderType::find($id);
        if (empty($orderType)) Utils::error('EposOrderType not found!', 404);
        return $orderType;
    }

    /**
     * Update orderType.
     * POST /epos/order-types/{id}
     */
    public function update(Request $req, $id)
    {
        $orderType = EposOrderType::find($id);
        if (empty($orderType)) Utils::error('EposOrderType not found!', 404);

        $this->validate($req, [
            'type' => 'string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // update orderType
        if ($req->has('type')) $orderType->type = $req->input('type');
        if ($req->has('top_color')) $orderType->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $orderType->bottom_color = $req->input('bottom_color');

        $orderType->save();

        return $orderType;
    }


    /**
     * Delete orderType by id.
     * DELETE /epos/order-types/{id}
     */
    public function delete(Request $req, $id)
    {
        $orderType = EposOrderType::find($id);
        if (empty($orderType)) Utils::error('EposOrderType not found!', 404);
        $orderType->delete();
        return $orderType;
    }
}
