<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposPaymentMethod;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposPaymentMethodController extends Controller
{
    /**
     * List payment-methods.
     * GET /api/payment-methods
     */
    public function index(Request $req)
    {
        $payment_methods = EposPaymentMethod::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $payment_methods->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($disabled = $req->query('disabled', false)) {
            $payment_methods->where('disabled', $disabled);
        }
        if ($req->query('nopaginate', false) == '1') {
            $payment_methods = $payment_methods->get();
        } else {
            $payment_methods = $payment_methods->paginate($req->query('per_page', 20));
        }

        if ($payment_methods->isEmpty()) Utils::error('No payment methods found!', 404);


        return ($payment_methods);
    }

    /**
     * Create payment-method.
     * POST /api/payment-methods
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
        ]);

        // create payment_method
        $payment_method = new EposPaymentMethod;
        $payment_method->fill($req->all());

        $payment_method->save();

        return $payment_method;
    }

    /**
     * Show payment_method.
     * GET /api/payment_methods/{id}
     */
    public function show(Request $req, $id)
    {
        $payment_method = EposPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('EposPaymentMethod not found!', 404);
        $payment_method->append('tables');
        return $payment_method;
    }

    /**
     * Update payment_method.
     * POST /api/payment_methods/{id}
     */
    public function update(Request $req, $id)
    {
        $payment_method = EposPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('EposPaymentMethod not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',

        ]);

        // update payment_method
        if ($req->has('name')) $payment_method->name = $req->input('name');

        $payment_method->save();

        return $payment_method;
    }


    /**
     * Delete payment_method by id.
     * DELETE /api/payment_methods/{id}
     */
    public function delete(Request $req, $id)
    {
        $payment_method = EposPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('EposPaymentMethod not found!', 404);
        $payment_method->delete();
        return $payment_method;
    }

    /**
     * Enable payment_method.
     * POST /epos/payment_methods/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $payment_method = EposPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('EposPaymentMethod not found!', 404);
        $payment_method->disabled = 0;
        $payment_method->save();
        return $payment_method;
    }

    /**
     * Disable payment_method.
     * POST /epos/payment_methods/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $payment_method = EposPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('EposPaymentMethod not found!', 404);
        $payment_method->disabled = 1;
        $payment_method->save();
        return $payment_method;
    }
}
