<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposPrepLocation;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposPrepLocationController extends Controller
{
    /**
     * List prepLocations.
     * GET /epos/preparation-locations
     */
    public function index(Request $req)
    {
        $prepLocations = EposPrepLocation::selectRaw("preparation_locations.*, printers.name as printer_name")->leftJoin('printers', 'printer_id', '=', 'printers.id')->orderByRaw('preparation_locations.name ASC, preparation_locations.created_at DESC');

        if ($query = $req->query('query', false)) {
            $prepLocations->whereRaw('(preparation_locations.name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $prepLocations = $prepLocations->get();
        } else {
            $prepLocations = $prepLocations->paginate($req->query('per_page', 20));
        }

        if ($prepLocations->isEmpty()) Utils::error('No prepLocations found!', 404);


        return ($prepLocations);
    }

    /**
     * Create prepLocation.
     * POST /epos/preparation-locations
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'cash_draw' => 'boolean',
            'printer_id' => 'exists:epos_connection.printers,id',
        ]);
        // create prepLocation
        $prepLocation = new EposPrepLocation;
        $prepLocation->fill($req->all());

        $prepLocation->save();

        return $prepLocation;
    }

    /**
     * Show prepLocation.
     * GET /epos/preparation-locations/{id}
     */
    public function show(Request $req, $id)
    {
        $prepLocation = EposPrepLocation::selectRaw("preparation_locations.*, printers.name as printer_name")->leftJoin('printers', 'printer_id', '=', 'printers.id')->find($id);
        if (empty($prepLocation)) Utils::error('EposPrepLocation not found!', 404);
        return $prepLocation;
    }

    /**
     * Update prepLocation.
     * POST /epos/preparation-locations/{id}
     */
    public function update(Request $req, $id)
    {
        $prepLocation = EposPrepLocation::find($id);
        if (empty($prepLocation)) Utils::error('EposPrepLocation not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'printer_id' => 'exists:epos_connection.printers,id',
            'cash_draw' => 'boolean',
        ]);
        // update prepLocation
        if ($req->has('name')) $prepLocation->name = $req->input('name');
        if ($req->has('printer_id')) $prepLocation->printer_id = $req->input('printer_id');
        $prepLocation->cash_draw = $req->input('cash_draw');

        $prepLocation->save();

        return $prepLocation;
    }


    /**
     * Delete prepLocation by id.
     * DELETE /epos/preparation-locations/{id}
     */
    public function delete(Request $req, $id)
    {
        $prepLocation = EposPrepLocation::find($id);
        if (empty($prepLocation)) Utils::error('EposPrepLocation not found!', 404);
        $prepLocation->delete();
        return $prepLocation;
    }

    /**
     * Enable prepLocation.
     * POST /epos/preparation-locaitons/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $prepLocation = EposPrepLocation::find($id);
        if (empty($prepLocation)) Utils::error('EposPrepLocation not found!', 404);
        $prepLocation->disabled = 0;
        $prepLocation->save();
        return $prepLocation;
    }

    /**
     * Disable prepLocation.
     * POST /epos/preparation-locaitons/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $prepLocation = EposPrepLocation::find($id);
        if (empty($prepLocation)) Utils::error('EposPrepLocation not found!', 404);
        $prepLocation->disabled = 1;
        $prepLocation->save();
        return $prepLocation;
    }
}
