<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\ImageHelper;
use App\ModelsEpos\EposPrintBlockAssign;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class EposPrintBlockAssignController extends Controller
{
    /**
     * List printBlockAssign.
     * GET /epos/product-print-blocks/
     */
    public function index(Request $req)
    {
        $printBlockAssign = EposPrintBlockAssign::select('print_block_assigns.*', 'print_blocks.name as print_block_name', 'categories.name as category_name')->leftJoin('categories', 'categories.id', '=', 'print_block_assigns.category_id')->leftJoin('print_blocks', 'print_blocks.id', '=', 'print_block_assigns.print_block_id')->latest();

        if ($query = $req->query('print_block_id', false)) {
            $printBlockAssign->where('print_block_id', $query);
        }
        if ($query = $req->query('category_id', false)) {
            $printBlockAssign->where('category_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $printBlockAssign = $printBlockAssign->get();
        } else {
            $printBlockAssign = $printBlockAssign->paginate($req->query('per_page', 20));
        }

        if ($printBlockAssign->isEmpty()) Utils::error('No printBlockAssign found!', 404);
        return $printBlockAssign;
    }

    public function create(Request $req)
    {
        $this->validate($req, [
            'category_id' =>  ['required', 'exists:epos_connection.categories,id', Rule::unique('epos_connection.print_block_assigns')->where(function ($query) use ($req) {
                return $query->where('category_id', $req->input('category_id'))->where('print_block_id', $req->input('print_block_id'));
            })],
            'print_block_id' =>  ['required', 'exists:epos_connection.categories,id', Rule::unique('epos_connection.print_block_assigns')->where(function ($query) use ($req) {
                return $query->where('category_id', $req->input('category_id'))->where('print_block_id', $req->input('print_block_id'));
            })],
        ], [
            'category_id.unique' => "Print block already assigned with the category",
            'print_block_id.unique' => "Category already assigned with the print block",
        ]);

        // create printBlockAssign
        $printBlockAssign = new EposPrintBlockAssign;
        $printBlockAssign->fill($req->all());
        $printBlockAssign->save();

        return $printBlockAssign;
    }


    /**
     * Add printBlockAssign multiple.
     * POST /epos/product-print_blocks/multiple
     */
    public function create_multiple(Request $req)
    {
        if ($req->has('category_ids') && !is_array($req->input('category_ids')))
            $req->merge(['category_ids' => json_decode($req->input('category_ids'))]);

        $this->validate($req, [
            'category_ids' => 'array|required_without:category_ids',
            'print_block_id' => 'required|exists:epos_connection.print_blocks,id'
        ]);
        $print_block_id = $req->input('print_block_id');

        if (!empty($req->input('category_ids')) && sizeof($req->input('category_ids')) > 0) {
            foreach ($req->input('category_ids') as $category_id) {
                $printBlockAssign = EposPrintBlockAssign::where('category_id', $category_id)->where('category_id', $category_id)->first();
                if (empty($printBlockAssign)) {
                    $printBlockAssign = new EposPrintBlockAssign();
                }
                $printBlockAssign->category_id = $category_id;
                $printBlockAssign->print_block_id = $print_block_id;
                $printBlockAssign->save();
            }
        }

        return ["message" => 'Print block assigned'];
    }

    /**
     * Show printBlockAssign profile.
     * GET /epos/print-block-assigns/{id}
     */
    public function show(Request $req, $id)
    {
        $printBlockAssign = EposPrintBlockAssign::select('print_block_assigns.*', 'print_blocks.name as print_block_name', 'categories.name as category_name')->leftJoin('categories', 'categories.id', '=', 'print_block_assigns.category_id')->leftJoin('print_blocks', 'print_blocks.id', '=', 'print_block_assigns.print_block_id')->find($id);
        if (empty($printBlockAssign)) Utils::error('EposPrintBlockAssign not found!', 404);
        return $printBlockAssign;
    }

    /**
     * Update printBlockAssign.
     * POST /epos/print-block-assigns/{id}
     */
    public function update(Request $req, $id)
    {
        $printBlockAssign = EposPrintBlockAssign::find($id);

        if (empty($printBlockAssign)) Utils::error('EposPrintBlockAssign not found!', 404);

        $this->validate($req, [
            'category_id' =>  ['required', 'exists:epos_connection.categories,id', Rule::unique('epos_connection.print_block_assigns')->where(function ($query) use ($req) {
                return $query->where('category_id', $req->input('category_id'))->where('print_block_id', $req->input('print_block_id'));
            })],
            'print_block_id' =>  ['required', 'exists:epos_connection.categories,id', Rule::unique('epos_connection.print_block_assigns')->where(function ($query) use ($req) {
                return $query->where('category_id', $req->input('category_id'))->where('print_block_id', $req->input('print_block_id'));
            })],
        ], [
            'category_id.unique' => "Print block already assigned with the category",
            'print_block_id.unique' => "Category already assigned with the print block",
        ]);


        // update printBlockAssign
        if ($req->has('category_id')) $printBlockAssign->category_id = $req->input('category_id');
        if ($req->has('print_block_id')) $printBlockAssign->print_block_id = $req->input('print_block_id');

        $printBlockAssign->save();

        return $printBlockAssign;
    }

    /**
     * Delete printBlockAssign by id.
     * DELETE /epos/print-block-assigns/{id}
     */
    public function delete(Request $req, $id)
    {
        $printBlockAssign = EposPrintBlockAssign::find($id);
        if (empty($printBlockAssign)) Utils::error('EposPrintBlockAssign not found!', 404);
        $printBlockAssign->delete();
        return $printBlockAssign;
    }
}
