<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\ImageHelper;
use App\ModelsEpos\EposPrintBlock;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposPrintBlockController extends Controller
{
    /**
     * List print-blocks.
     * GET /epos/print-blocks
     */
    public function index(Request $req)
    {
        $print_blocks = EposPrintBlock::orderByRaw('sequence ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $print_blocks->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $print_blocks = $print_blocks->get();
        } else {
            $print_blocks = $print_blocks->paginate($req->query('per_page', 20));
        }

        if ($print_blocks->isEmpty()) Utils::error('More print blocks coming soon!', 404);

        return $print_blocks;
    }

    /**
     * Create print_block.
     * POST /epos/print-blocks
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
        ]);

        // create print_block
        $print_block = new EposPrintBlock;
        $print_block->fill($req->all());
        $print_block->save();

        // Sequence add
        $print_block->sequence = $print_block->id;
        $print_block->save();

        return $print_block;
    }

    /**
     * Show print-block.
     * GET /epos/print-blocks/{id}
     */
    public function show(Request $req, $id)
    {
        $print_block = EposPrintBlock::find($id);
        if (empty($print_block)) Utils::error('EposPrintBlock not found!', 404);

        return $print_block;
    }

    /**
     * Update print-block.
     * POST /epos/print-blocks/{id}
     */
    public function update(Request $req, $id)
    {
        $print_block = EposPrintBlock::find($id);
        if (empty($print_block)) Utils::error('EposPrintBlock not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'sequence' => 'numeric',
        ]);

        // update print_block
        if ($req->has('name')) $print_block->name = $req->input('name');
        if ($req->has('sequence')) $print_block->sequence = $req->input('sequence');

        $print_block->save();

        return $print_block;
    }


    /**
     * Delete print-block by id.
     * DELETE /epos/print-blocks/{id}
     */
    public function delete(Request $req, $id)
    {
        $print_block = EposPrintBlock::find($id);
        if (empty($print_block)) Utils::error('EposPrintBlock not found!', 404);
        $print_block->delete();
        return $print_block;
    }

    public function print_block_bulk(Request $req)
    {
        if ($req->has('print_blocks') && !is_array($req->input('print_blocks'))) {
            $req->merge(['print_blocks' => json_decode($req->input('print_blocks'))]);
        }
        $this->validate($req, [
            'print_blocks' => 'required|array'
        ],);
        $print_blocks = ($req->input('print_blocks'));
        $count = 1;
        foreach ($print_blocks as $print_block) {
            $print_blockObj = EposPrintBlock::find($print_block->id);
            $print_blockObj->sequence = $count;
            $print_blockObj->save();
            $count++;
        }
        return $print_blocks;
    }
}
