<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposPrintSetting;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposPrintSettingController extends Controller
{
    /**
     * List print-settings.
     * GET /epos/print-settings
     */
    public function index(Request $req)
    {
        $print_settings = EposPrintSetting::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $print_settings->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($service = $req->query('service', false)) {
            $print_settings->where('service', $service);
        }
        if ($req->query('nopaginate', false) == '1') {
            $print_settings = $print_settings->get();
        } else {
            $print_settings = $print_settings->paginate($req->query('per_page', 20));
        }

        if ($print_settings->isEmpty()) Utils::error('No print settings found!', 404);


        return ($print_settings);
    }

    /**
     * Create print-setting.
     * POST /epos/print-settings
     */
    public function create(Request $req)
    {
        if (!is_array($req->print_structure)) {
            $req->merge(['print_structure' => (json_decode(stripslashes($req->print_structure), true))]);
        }

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'service' => 'required|in:epos,online',
            'print_structure' => 'required',
        ]);

        // create print-setting
        $print_setting = new EposPrintSetting;
        $print_setting->fill($req->all());
        if ($req->has('print_structure')) $print_setting->print_structure = $req->input('print_structure');
        $print_setting->save();

        return $print_setting;
    }

    /**
     * Show print-setting.
     * GET /epos/print-settings/{id}
     */
    public function show(Request $req, $id)
    {
        $print_setting = EposPrintSetting::find($id);
        if (empty($print_setting)) Utils::error('Print Setting not found!', 404);
        return $print_setting;
    }

    /**
     * Update printer.
     * POST /epos/print-settings/{id}
     */
    public function update(Request $req, $id)
    {
        $print_setting = EposPrintSetting::find($id);
        if (empty($print_setting)) Utils::error('EposPrintSetting not found!', 404);

        if (!is_array($req->print_structure)) {
            $req->merge(['print_structure' => (json_decode(stripslashes($req->print_structure), true))]);
        }

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'service' => 'required|in:epos,online',
            'print_structure' => 'required',
        ]);

        // update print-setting
        if ($req->has('name')) $print_setting->name = $req->input('name');
        if ($req->has('service')) $print_setting->service = $req->input('service');
        if ($req->has('print_structure')) $print_setting->print_structure = $req->input('print_structure');

        $print_setting->save();

        return $print_setting;
    }

    /**
     * Delete print-setting by id.
     * DELETE /epos/print-settings/{id}
     */
    public function delete(Request $req, $id)
    {
        $print_setting = EposPrintSetting::find($id);
        if (empty($print_setting)) Utils::error('Print Setting not found!', 404);
        $print_setting->delete();
        return $print_setting;
    }
}
