<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposPrinter;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposPrinterController extends Controller
{
    /**
     * List printers.
     * GET /epos/printers
     */
    public function index(Request $req)
    {
        // $printers = EposPrinter::orderByRaw('name ASC, created_at DESC');
        $printers = EposPrinter::selectRaw('printers.*,printer_models.name as printer_model_name')->leftJoin('printer_models', 'printer_models.id', '=', 'printer_model_id')->orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $printers->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('printer_model_id', false)) {
            $printers->where('printer_model_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $printers = $printers->get();
        } else {
            $printers = $printers->paginate($req->query('per_page', 20));
        }

        if ($printers->isEmpty()) Utils::error('No printers found!', 404);


        return ($printers);
    }

    /**
     * Create printer.
     * POST /epos/printers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'printer_model_id' => 'required|exists:epos_connection.printer_models,id',
            'name' => 'required|string|max:255',
            'ip' => 'required|string|max:255',
        ]);

        // create printer
        $printer = new EposPrinter;
        $printer->fill($req->all());

        $printer->save();

        return $printer;
    }

    /**
     * Show printer.
     * GET /epos/printers/{id}
     */
    public function show(Request $req, $id)
    {
        $printer = EposPrinter::find($id);
        if (empty($printer)) Utils::error('EposPrinter not found!', 404);
        return $printer;
    }

    /**
     * Update printer.
     * POST /epos/printers/{id}
     */
    public function update(Request $req, $id)
    {
        $printer = EposPrinter::find($id);
        if (empty($printer)) Utils::error('EposPrinter not found!', 404);

        $this->validate($req, [
            'printer_model_id' => 'exists:epos_connection.printer_models,id',
            'name' => 'string|max:255',
            'ip' => 'string|max:255',

        ]);

        // update printer
        if ($req->has('printer_model_id')) $printer->printer_model_id = $req->input('printer_model_id');
        if ($req->has('name')) $printer->name = $req->input('name');
        if ($req->has('ip')) $printer->ip = $req->input('ip');

        $printer->save();

        return $printer;
    }


    /**
     * Delete printer by id.
     * DELETE /epos/printers/{id}
     */
    public function delete(Request $req, $id)
    {
        $printer = EposPrinter::find($id);
        if (empty($printer)) Utils::error('EposPrinter not found!', 404);
        $printer->delete();
        return $printer;
    }
}
