<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposPrinterModel;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposPrinterModelController extends Controller
{
    /**
     * List printers.
     * GET /epos/printers
     */
    public function index(Request $req)
    {
        $printers = EposPrinterModel::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $printers->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $printers = $printers->get();
        } else {
            $printers = $printers->paginate($req->query('per_page', 20));
        }

        if ($printers->isEmpty()) Utils::error('No printers found!', 404);


        return ($printers);
    }

    /**
     * Create printer.
     * POST /epos/printers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
        ]);

        // create printer
        $printer = new EposPrinterModel;
        $printer->fill($req->all());

        $printer->save();

        return $printer;
    }

    /**
     * Show printer.
     * GET /epos/printers/{id}
     */
    public function show(Request $req, $id)
    {
        $printer = EposPrinterModel::find($id);
        if (empty($printer)) Utils::error('EposPrinterModel not found!', 404);
        return $printer;
    }

    /**
     * Update printer.
     * POST /epos/printers/{id}
     */
    public function update(Request $req, $id)
    {
        $printer = EposPrinterModel::find($id);
        if (empty($printer)) Utils::error('EposPrinterModel not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',

        ]);

        // update printer
        if ($req->has('name')) $printer->name = $req->input('name');

        $printer->save();

        return $printer;
    }


    /**
     * Delete printer by id.
     * DELETE /epos/printers/{id}
     */
    public function delete(Request $req, $id)
    {
        $printer = EposPrinterModel::find($id);
        if (empty($printer)) Utils::error('EposPrinterModel not found!', 404);
        $printer->delete();
        return $printer;
    }
}
