<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\ImageHelper;
use App\ModelsEpos\EposProductAddon;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposAddon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class EposProductAddonController extends Controller
{
    /**
     * List productAddons.
     * GET /epos/product-addons/
     */
    public function index(Request $req)
    {
        $productAddons = EposProductAddon::select('product_addons.*', 'addons.name as addon_name', 'addons.price', 'addons.web_price', 'addons.takeaway_price', 'addons.delivery_price', 'addons.waiting_price', 'addons.name as addon_name')->leftJoin('addons', 'addons.id', '=', 'product_addons.addon_id')->latest();

        if ($query = $req->query('product_id', false)) {
            $productAddons->where('product_id', $query);
        }

        if ($req->query('disabled') == '0' || $req->query('disabled') == '1') {
            $productAddons->where('disabled', $req->query('disabled'));
        }

        if (!$prefilled = $req->query('prefilled', false)) {
            $productAddons->whereNull('product_addons.parent_id');
        }

        if ($req->query('nopaginate', false) == '1') {
            $productAddons = $productAddons->get();
        } else {
            $productAddons = $productAddons->paginate($req->query('per_page', 20));
        }

        if ($productAddons->isEmpty()) Utils::error('No productAddons found!', 404);

        if ($query = $req->query('product_id', false)) {
            $productAddons->each(function ($productAddon) {
                $productAddon->append('addon');
            });
            $productAddons->append('subaddon_ids');
        }

        return $productAddons;
    }

    public function create(Request $req)
    {
        $this->validate($req, [
            'product_id' =>  ['exists:epos_connection.products,id', Rule::unique('epos_connection.product_addons')->where(function ($query) use ($req) {
                return $query->where('product_id', $req->input('product_id'))->where('addon_id', $req->input('addon_id'));
            })],
            'addon_id' =>  ['exists:epos_connection.addons,id', Rule::unique('epos_connection.product_addons')->where(function ($query) use ($req) {
                return $query->where('product_id', $req->input('product_id'))->where('addon_id', $req->input('addon_id'));
            })],
            'subaddon_ids' => 'string',
            // 'subaddon_ids.*' => ['exists:epos_connection.addons,id', Rule::unique('epos_connection.product_addons')->where(function ($query) use ($req) {
            //     return $query->where('product_id', $req->input('product_id'))->where('addon_id', $req->input('addon_id'));
            // })],
            // 'subaddon_ids.*' => Rule::forEach(function (string|null $value, string $attribute) use($req) {
            //     return [
            //         Rule::unique('epos_connection.product_addons')->where(function ($query) use ($value, $req) {
            //             return $query->where('product_id', $req->input('product_id'))->where('addon_id', $req->input('addon_id'))->where('parent_id', $value);
            //         })
            //     ];
            // }),
            'sequence' => 'numeric',
            'min_quantity' => 'numeric',
            'max_quantity' => 'numeric',
        ], [
            'product_id.unique' => "Addon already assigned with the product",
            'addon_id.unique' => "Product already assigned with the addon",
        ]);

        // create productAddon
        $productAddon = new EposProductAddon;
        $productAddon->fill($req->all());
        $productAddon->save();

        // Sequence add
        $productAddon->sequence = $productAddon->id;
        $productAddon->save();

        $subaddon_ids = explode(',', $req->subaddon_ids);

        foreach ($subaddon_ids as $subaddon_id) {
            $subaddon = EposAddon::find($subaddon_id);
            if (!empty($subaddon)) {
                $productSubaddon = new EposProductAddon;
                $productSubaddon->fill($req->all());
                $productSubaddon->product_id = $req->product_id;
                $productSubaddon->addon_id = $subaddon_id;
                $productSubaddon->parent_id = $productAddon->id;
                $productSubaddon->save();
            }
        }

        return $productAddon;
    }

    /**
     * Show productAddon profile.
     * GET /epos/product-addons/{id}
     */
    public function show(Request $req, $id)
    {
        $productAddon = EposProductAddon::select('product_addons.*', 'addons.price', 'addons.web_price', 'addons.takeaway_price', 'addons.delivery_price', 'addons.waiting_price', 'addons.name as addon_name')->leftJoin('addons', 'addons.id', '=', 'product_addons.addon_id')->find($id);
        if (empty($productAddon)) Utils::error('EposProductAddon not found!', 404);
        $productAddon->append('subaddon_ids');
        return $productAddon;
    }

    /**
     * Update productAddon.
     * POST /epos/product-addons/{id}
     */
    public function update(Request $req, $id)
    {
        $productAddon = EposProductAddon::find($id);
        if (empty($productAddon)) Utils::error('EposProductAddon not found!', 404);

        $this->validate($req, [
            'product_id' =>  ['exists:epos_connection.products,id', Rule::unique('epos_connection.product_addons')->where(function ($query) use ($req, $id) {
                return $query->where('product_id', $req->input('product_id'))->where('addon_id', $req->input('addon_id'))->whereNotIn('id', [$id]);
            })],
            'addon_id' =>  ['exists:epos_connection.addons,id', Rule::unique('epos_connection.product_addons')->where(function ($query) use ($req, $id) {
                return $query->where('product_id', $req->input('product_id'))->where('addon_id', $req->input('addon_id'))->whereNotIn('id', [$id]);
            })],
            'subaddon_ids' => 'string',
            'sequence' => 'numeric',
            'min_quantity' => 'numeric',
            'max_quantity' => 'numeric',
        ], [
            'product_id.unique' => "Addon already assigned with the product",
            'addon_id.unique' => "Product already assigned with the addon",
        ]);
        // update productAddon
        if ($req->has('product_id')) $productAddon->product_id = $req->input('product_id');
        if ($req->has('addon_id')) $productAddon->addon_id = $req->input('addon_id');
        $productAddon->sequence = $req->input('sequence');
        $productAddon->min_quantity = $req->input('min_quantity');
        $productAddon->max_quantity = $req->input('max_quantity');
        $productAddon->save();

        if ($req->has('subaddon_ids')) {
            $subaddon_ids = explode(',', $req->input('subaddon_ids'));
            foreach ($subaddon_ids as $subaddon_id) {
                if ($subaddon_id > 0) {
                    $subAddon = EposProductAddon::where('product_id', $productAddon->product_id)->where('addon_id', $subaddon_id)->where('parent_id', $productAddon->id)->first();
                    if (empty($subAddon)) $subAddon = new EposProductAddon();
                    $subAddon->product_id = $productAddon->product_id;
                    $subAddon->addon_id = $subaddon_id;
                    $subAddon->parent_id = $productAddon->id;
                    $subAddon->sequence = $productAddon->sequence;
                    $subAddon->min_quantity = $productAddon->min_quantity ?? 0;
                    $subAddon->max_quantity = $productAddon->max_quantity ?? 0;
                    $subAddon->save();
                }
            }
            EposProductAddon::where('product_id', $productAddon->product_id)->whereNotIn('addon_id', $subaddon_ids)->where('parent_id', $productAddon->id)->delete();
        }

        return $productAddon;
    }

    /**
     * Enable productAddon.
     * POST /epos/product-addons/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $productAddon = EposProductAddon::find($id);
        if (empty($productAddon)) Utils::error('EposProductAddon not found!', 404);
        $productAddon->disabled = 0;
        $productAddon->save();
        return $productAddon;
    }

    /**
     * Disable productAddon.
     * POST /epos/product-addons/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $productAddon = EposProductAddon::find($id);
        if (empty($productAddon)) Utils::error('EposProductAddon not found!', 404);
        $productAddon->disabled = 1;
        $productAddon->save();
        return $productAddon;
    }

    /**
     * Enable contain status productAddon.
     * POST /epos/product-addons/{id}/enable
     */
    public function containsEnable(Request $req, $id)
    {
        $productAddon = EposProductAddon::find($id);
        if (empty($productAddon)) Utils::error('EposProductAddon not found!', 404);
        $productAddon->contain = 0;
        $productAddon->save();
        return $productAddon;
    }

    /**
     * Disable contain status productAddon.
     * POST /epos/product-addons/{id}/disable
     */
    public function containDisable(Request $req, $id)
    {
        $productAddon = EposProductAddon::find($id);
        if (empty($productAddon)) Utils::error('EposProductAddon not found!', 404);
        $productAddon->contain = 1;
        $productAddon->save();
        return $productAddon;
    }

    /**
     * Delete productAddon by id.
     * DELETE /epos/product-addons/{id}
     */
    public function delete(Request $req, $id)
    {
        $productAddon = EposProductAddon::find($id);
        if (empty($productAddon)) Utils::error('EposProductAddon not found!', 404);

        EposProductAddon::where('parent_id', $id)->delete();

        $productAddon->delete();
        if ($productAddon->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_USER, $productAddon->image);
        return $productAddon;
    }

    public function subaddons($id)
    {
        $productAddon = EposAddon::find($id);
        if (empty($productAddon)) Utils::error('Addon not found!', 404);

        $subaddons = EposAddon::where('parent_id', $id)->where('parent_id', '!=', null)->get();
        if ($subaddons->isEmpty()) Utils::error('No subaddons found!', 404);

        return $subaddons;
    }
}
