<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Tiffintom\TiffintomBookingController;
use App\ModelsEpos\EposExternalPayment;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\ModelsEpos\EposOrder;
use App\ModelsEpos\EposOrderPayment;
use App\ModelsEpos\EposVoucher;
use App\ModelsTiffintom\TiffintomOrder;
use App\OnlineOrder;
use App\StripeTransaction;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class EposReportController extends Controller
{
    public function index(Request $req)
    {

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ]);

        $business = Business::find(Auth::$business_id);
        if (empty($business)) Utils::error("Business not found", 400);

        //Orders
        $orders = EposOrder::selectRaw("SUM(total) value, COUNT(*) count")->orderByRaw('delivery_date DESC');

        $duePaymentOrders = EposOrder::selectRaw('orders.*')->latest();

        // Online Order Count
        $online_order_count = OnlineOrder::selectRaw("COUNT(id) as count,CAST(sum(grand_total) AS DECIMAL(7,2)) as total")->where('business_id', $business->id)->orderByRaw('order_date DESC');

        // Web Online Order Count
        $web_online_order_count = TiffintomOrder::selectRaw("COUNT(id) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('type', 'Web')->where('restaurant_id', $business->restaurant_id)->orderByRaw('id DESC');

        // Andoid Online Order Count
        $android_online_order_count = TiffintomOrder::selectRaw("COUNT(id) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('type', 'Android')->where('restaurant_id', $business->restaurant_id)->orderByRaw('id DESC');

        // IOS Online Order Count
        $ios_online_order_count = TiffintomOrder::selectRaw("COUNT(id) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('restaurant_id', $business->restaurant_id)->orderByRaw('id DESC');
        $ios_online_order_count->whereRaw('(type LIKE ? OR type LIKE ? )', ["%iOS%", "%ios%"]);

        // Type Wise
        $ordersByType = EposOrder::selectRaw("COUNT(*) as count,CAST(sum(total) AS DECIMAL(7,2)) as total, order_types.type as type")->leftJoin('order_types', 'order_types.id', '=', 'orders.order_type_id')->groupBy('orders.order_type_id');

        // Status Wise
        $ordersByStatus = EposOrder::selectRaw("COUNT(*) as count,CAST(sum(total) AS DECIMAL(7,2)) as total, order_statuses.status as status")->leftJoin('order_statuses', 'order_statuses.id', '=', 'orders.order_status_id')->groupBy('orders.order_status_id');

        // Payment Wise
        $ordersByPayment = EposOrder::selectRaw("COUNT(order_payments.id) as count,CAST(sum(order_payments.amount) AS DECIMAL(7,2)) as total,payment_methods.name as name")->leftJoin('order_payments', 'order_payments.order_id', '=', 'orders.id')->join('payment_methods', 'payment_methods.id', '=', 'order_payments.payment_method_id')->groupBy('order_payments.payment_method_id');
        $ordersByRejctedPayment = EposOrder::leftJoin('order_payments', 'order_payments.order_id', '=', 'orders.id')->join('payment_methods', 'payment_methods.id', '=', 'order_payments.payment_method_id');

        //External Payment
        $externalPayments = EposExternalPayment::orderByRaw('created_at DESC');

        // Product Wise
        $ordersByProduct = EposOrder::selectRaw("COUNT(order_items.id) as count,CAST(sum(order_items.total) AS DECIMAL(7,2)) as total")->leftJoin('order_items', 'order_items.order_id', '=', 'orders.id');

        //Table No
        $total_no_table = EposOrder::selectRaw("COUNT(*) as count");

        //Vouchers
        $vouchers = EposVoucher::selectRaw("COUNT(*) as count,CAST(sum(voucher_amount) AS DECIMAL(7,2)) as total")->orderByRaw('id DESC');

        //Transactions
        $transactions = StripeTransaction::where('business_id', $business->id)->where('reporting_category', 'charge')->where('refunded', 0)->orderByRaw('created DESC');
        $transactions->whereRaw('(service LIKE ? OR service LIKE ?)', ["%Merchant%", "%Connect%"]);

        //Card Reader Transactions
        $cardReaderTransactions = StripeTransaction::selectRaw("COUNT(id) as count,CAST(sum(amount) / 100  AS DECIMAL(7,2)) as total")->where('business_id', $business->id)->where('reporting_category', 'charge')->where('refunded', 0)->orderByRaw('created DESC');
        $cardReaderTransactions->whereRaw('(service LIKE ? OR service LIKE ?)', ["%Merchant%", "%Connect%"]);

        //Moto Transactions
        $motoTransactions = StripeTransaction::selectRaw("COUNT(id) as count,CAST(sum(amount) / 100 AS DECIMAL(7,2)) as total")->where('business_id', $business->id)->where('reporting_category', 'charge')->where('refunded', 0)->orderByRaw('created DESC');
        $motoTransactions->whereRaw('(service LIKE ? OR service LIKE ?)', ["%Merchant%", "%Connect%"]);

        //Pay by link Transactions
        $payByLinkTransactions = StripeTransaction::selectRaw("COUNT(id) as count,CAST(sum(amount) / 100 AS DECIMAL(7,2)) as total")->where('business_id', $business->id)->where('reporting_category', 'charge')->where('refunded', 0)->orderByRaw('created DESC');
        $payByLinkTransactions->whereRaw('(service LIKE ? OR service LIKE ?)', ["%Merchant%", "%Connect%"]);

        if ($req->has('user_id')) {
            $orders = $orders->where('updater_id', $req->input('user_id'));
            $duePaymentOrders = $duePaymentOrders->where('orders.updater_id', $req->input('user_id'));
            $ordersByStatus = $ordersByStatus->where('orders.updater_id', $req->input('user_id'));
            $ordersByPayment = $ordersByPayment->where('orders.updater_id', $req->input('user_id'));
            $ordersByRejctedPayment = $ordersByRejctedPayment->where('orders.updater_id', $req->input('user_id'));
            $ordersByProduct = $ordersByProduct->where('orders.updater_id', $req->input('user_id'));
            $total_no_table = $total_no_table->where('orders.updater_id', $req->input('user_id'));
        }
        $is_archived = "0";
        if ($query = $req->query('archived', false)) {
            $is_archived = $query;
        }

        if ($req->has('from_date')) {

            $from_date_strtotime = strtotime($req->input('from_date'));
            $to_date_strtotime = strtotime($req->input('to_date'));

            while ($from_date_strtotime <= $to_date_strtotime) {
                $from_date = Carbon::createFromTimestamp($from_date_strtotime)->toDateString();

                if ($req->input('shift_status') == 'morning') {
                    $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                    $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                    if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time = $from_date . ' ' . $business_morning_start_time->value;
                    $end_time = $from_date . ' ' . $business_morning_end_time->value;
                } else if ($req->input('shift_status') == 'evening') {
                    $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                    $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                    if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                    $end_time = $from_date . ' ' . $business_evening_end_time->value;
                } else {
                    $start_time = $from_date . ' ' . "00:00:00";
                    $end_time = $from_date . ' ' . "23:59:59";
                }

                $start_time = Carbon::parse($start_time);
                $end_time = Carbon::parse($end_time);

                $orders->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $duePaymentOrders->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $ordersByType->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $ordersByStatus->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $ordersByPayment->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $ordersByRejctedPayment->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $ordersByProduct->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $total_no_table->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $total_no_table = $total_no_table->where('order_type_id', 1);

                $online_order_count->WhereRaw('(order_date > ? AND order_date < ?)', [$start_time, $end_time]);

                $orders = $orders->where('is_archived', $is_archived);
                $duePaymentOrders = $duePaymentOrders->where('is_archived', $is_archived);
                $ordersByType = $ordersByType->where('is_archived', $is_archived);
                $ordersByStatus = $ordersByStatus->where('is_archived', $is_archived);
                $ordersByPayment = $ordersByPayment->where('is_archived', $is_archived);
                $ordersByRejctedPayment = $ordersByRejctedPayment->where('is_archived', $is_archived);
                $ordersByProduct = $ordersByProduct->where('is_archived', $is_archived);
                $total_no_table = $total_no_table->where('is_archived', $is_archived);

                $from_date_strtotime = strtotime("+1 day", $from_date_strtotime);
            }
            $start_time = $req->input('from_date') . ' ' . "00:00:00";
            $end_time = $req->input('to_date') . ' ' . "23:59:59";
            $start_time = Carbon::parse($start_time);
            $end_time = Carbon::parse($end_time);
            $vouchers = $vouchers->where('created_at', '>', $start_time);
            $vouchers = $vouchers->where('created_at', '<', $end_time);

            $externalPayments = $externalPayments->where('created_at', '>', $start_time);
            $externalPayments = $externalPayments->where('created_at', '<', $end_time);

            $web_online_order_count->WhereRaw('(delivery_date >= ? AND delivery_date <= ?)', [$req->input('from_date'), $req->input('to_date')]);
            $android_online_order_count->WhereRaw('(delivery_date >= ? AND delivery_date <= ?)', [$req->input('from_date'), $req->input('to_date')]);
            $ios_online_order_count->WhereRaw('(delivery_date >= ? AND delivery_date <= ?)', [$req->input('from_date'), $req->input('to_date')]);

            $start_time_tr = Carbon::parse($req->input('from_date') . ' ' . "01:00:00")->getTimestamp();
            $end_time_tr = Carbon::parse($req->input('to_date') . ' ' . "23:59:59")->getTimestamp();
            $transactions->whereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
            $cardReaderTransactions->whereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
            $motoTransactions->whereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
            $payByLinkTransactions->whereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
        } else {

            $start_time = Carbon::today()->startOfDay();
            $end_time = Carbon::today()->endOfDay();

            if ($req->input('shift_status') == 'morning') {
                $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time = $from_date . ' ' . $business_morning_start_time->value;
                $end_time = $from_date . ' ' . $business_morning_end_time->value;
            } elseif ($req->input('shift_status') == 'evening') {
                $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                $end_time = $from_date . ' ' . $business_evening_end_time->value;
            }

            $orders->where('delivery_date', '>', $start_time);
            $orders->where('delivery_date', '<', $end_time);

            $duePaymentOrders->where('delivery_date', '>', $start_time);
            $duePaymentOrders->where('delivery_date', '<', $end_time);

            $online_order_count->where('order_date', '>', $start_time);
            $online_order_count->where('order_date', '<', $end_time);

            $web_online_order_count->where('delivery_date', '>', $req->input('from_date'));
            $web_online_order_count->where('delivery_date', '<', $req->input('to_date'));

            $android_online_order_count->where('delivery_date', '>', $req->input('from_date'));
            $android_online_order_count->where('delivery_date', '<', $req->input('to_date'));

            $ios_online_order_count->where('delivery_date', '>', $req->input('from_date'));
            $ios_online_order_count->where('delivery_date', '<', $req->input('to_date'));

            $ordersByType->where('delivery_date', '>', $start_time);
            $ordersByType->where('delivery_date', '<', $end_time);

            $ordersByStatus->where('delivery_date', '>', $start_time);
            $ordersByStatus->where('delivery_date', '<', $end_time);

            $ordersByPayment->where('delivery_date', '>', $start_time);
            $ordersByPayment->where('delivery_date', '<', $end_time);

            $ordersByRejctedPayment->where('delivery_date', '>', $start_time);
            $ordersByRejctedPayment->where('delivery_date', '<', $end_time);

            $ordersByProduct->where('delivery_date', '>', $start_time);
            $ordersByProduct->where('delivery_date', '<', $end_time);

            $total_no_table->where('delivery_date', '>', $start_time);
            $total_no_table->where('delivery_date', '<', $end_time);

            $vouchers->where('created_at', '>', $start_time);
            $vouchers->where('created_at', '<', $end_time);

            $externalPayments = $externalPayments->where('created_at', '>', $start_time);
            $externalPayments = $externalPayments->where('created_at', '<', $end_time);

            $orders = $orders->where('is_archived', $is_archived);
            $duePaymentOrders = $duePaymentOrders->where('is_archived', $is_archived);
            $ordersByType = $ordersByType->where('is_archived', $is_archived);
            $ordersByStatus = $ordersByStatus->where('is_archived', $is_archived);
            $ordersByPayment = $ordersByPayment->where('is_archived', $is_archived);
            $ordersByRejctedPayment = $ordersByRejctedPayment->where('is_archived', $is_archived);
            $ordersByProduct = $ordersByProduct->where('is_archived', $is_archived);
            $total_no_table = $total_no_table->where('is_archived', $is_archived);
            $total_no_table = $total_no_table->where('order_type_id', 1);

            $start_time_tr = Carbon::parse($req->input('from_date') . ' ' . "01:00:00")->getTimestamp();
            $end_time_tr = Carbon::parse($req->input('to_date') . ' ' . "23:59:59")->getTimestamp();
            $transactions->orWhereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
            $cardReaderTransactions->orWhereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
            $motoTransactions->orWhereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
            $payByLinkTransactions->orWhereRaw('(created > ? AND created < ?)', [$start_time_tr, $end_time_tr]);
        }

        $order_count = $orders->get();

        $duePaymentOrders = $duePaymentOrders->whereIn('id', EposOrderPayment::whereIn('payment_method_id', [9, 10, 11])->select('order_id')->get()->pluck('order_id')->toArray())->sum('due_amount');

        if (empty($order_count) || $order_count[0]['count'] == 0) Utils::error('No orders found!', 404);

        $online_order_count = $online_order_count->get();

        $web_online_order_count = $web_online_order_count->get();

        $android_online_order_count = $android_online_order_count->get();

        $ios_online_order_count = $ios_online_order_count->get();

        // Sum Of Values
        $total_discount = $orders->sum('discount');
        $total_gratuity = $orders->sum('gratuity');
        $total_service_charge = $orders->sum('service_charge');
        $total_no_guest = $orders->sum('no_guest');

        $ordersByType = $ordersByType->get();

        $ordersByStatus = $ordersByStatus->get();

        $ordersByPayment->where('order_status_id', '!=', "10");
        $ordersByPayment = $ordersByPayment->get();

        $ordersByRejctedPayment->where('order_status_id', "10")->whereRaw('(payment_method_id = ? OR  payment_method_id = ? OR  payment_method_id = ?)', ["9", "10", "11"]);
        $ordersByRejctedPayment = $ordersByRejctedPayment->sum('amount');

        $ordersByProduct->where('order_items.is_delete', '0');
        $ordersByProduct = $ordersByProduct->get();

        $total_no_table = $total_no_table->get();

        $vouchers = $vouchers->get();

        $transactions = $transactions->sum('amount');
        if ($transactions > 0) {
            $transactions =  ($transactions / 100);
            $transactions = $transactions - $ordersByRejctedPayment;
        }

        $orderIds = EposOrder::pluck("unique_id");
        // Transaction Added
        $cardReaderTransactions = $cardReaderTransactions->whereJsonContains('metadata->is_credit_card_tt', '1')->where(function ($query) use ($orderIds) {
            foreach ($orderIds as $value) {
                $query->orWhereJsonContains('metadata->order_id', $value);
            }
        })->get();
        $motoTransactions = $motoTransactions->whereJsonContains('metadata->is_moto_order', '1')->where(function ($query) use ($orderIds) {
            foreach ($orderIds as $value) {
                $query->orWhereJsonContains('metadata->order_id', $value);
            }
        })->get();
        $payByLinkTransactions = $payByLinkTransactions->whereJsonContains('metadata->is_payby_link', '1')->where(function ($query) use ($orderIds) {
            foreach ($orderIds as $value) {
                $query->orWhereJsonContains('metadata->order_id', $value);
            }
        })->get();
        $externalPayments = $externalPayments->sum('amount');

        $data = [
            'order_count' => $order_count[0]['count'],
            'order_total_business' => $order_count[0]['value'],
            'online_order_count' =>  $this->convertToDecimal($online_order_count[0]['count']),
            'online_order_total_business' => $online_order_count[0]['total'] != '' ?  $this->convertToDecimal($online_order_count[0]['total']) : '0.00',
            'web_online_order_count' =>  $web_online_order_count,
            'android_online_order_count' =>  $android_online_order_count,
            'ios_online_order_count' =>  $ios_online_order_count,
            'order_type_wise' => $ordersByType,
            'order_status_wise' => $ordersByStatus,
            'order_payment_wise' => $ordersByPayment,
            'orders_total_product' => $ordersByProduct[0]['count'],
            'orders_amount_product' => $ordersByProduct[0]['total'],
            'total_discount' => $total_discount,
            'total_gratuity' => $total_gratuity,
            'total_gratuity_change' => $duePaymentOrders,
            'total_service_charge' => $total_service_charge,
            'total_no_guest' => $total_no_guest,
            'total_no_table' => $total_no_table[0]['count'],
            'allocated_voucher' => $vouchers[0]['count'],
            'allocated_voucher_value' => $this->convertToDecimal($vouchers[0]['total']),
            'transaction_value' => $this->convertToDecimal($transactions),
            'external_payments' => $this->convertToDecimal($externalPayments),
            'card_reader_transactions' => $cardReaderTransactions,
            'moto_transactions' => $motoTransactions,
            'payby_link_transactions' => $payByLinkTransactions,
        ];
        return $data;
    }

    public function cancelList(Request $req)
    {

        // $orders = EposOrder::orderByRaw('delivery_date DESC');
        $orders = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')->orderByRaw('orders.delivery_date DESC');

        if ($req->has('user_id')) {
            $orders = $orders->where('updater_id', $req->input('user_id'));
        }

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ]);

        if ($req->has('from_date')) {

            $from_date_strtotime = strtotime($req->input('from_date'));
            $to_date_strtotime = strtotime($req->input('to_date'));

            while ($from_date_strtotime <= $to_date_strtotime) {
                $from_date = Carbon::createFromTimestamp($from_date_strtotime)->toDateString();

                if ($req->input('shift_status') == 'morning') {
                    $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                    $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                    if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time = $from_date . ' ' . $business_morning_start_time->value;
                    $end_time = $from_date . ' ' . $business_morning_end_time->value;
                } else if ($req->input('shift_status') == 'evening') {
                    $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                    $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                    if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                    $end_time = $from_date . ' ' . $business_evening_end_time->value;
                } else {
                    $start_time = $from_date . ' ' . "00:00:00";
                    $end_time = $from_date . ' ' . "23:59:59";
                }

                $start_time = Carbon::parse($start_time);
                $end_time = Carbon::parse($end_time);

                $orders->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);

                $orders->where('order_status_id', 10);
                $orders->where('is_archived', "0");
                if ($query = $req->query('query', false)) {
                    $orders->whereRaw('(orders.total ?  )', ["%$query%"]);
                }

                $from_date_strtotime = strtotime("+1 day", $from_date_strtotime);
            }
        } else {

            $start_time = Carbon::today()->startOfDay();
            $end_time = Carbon::today()->endOfDay();

            if ($req->input('shift_status') == 'morning') {
                $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time = $from_date . ' ' . $business_morning_start_time->value;
                $end_time = $from_date . ' ' . $business_morning_end_time->value;
            } elseif ($req->input('shift_status') == 'evening') {
                $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                $end_time = $from_date . ' ' . $business_evening_end_time->value;
            }

            $orders->where('delivery_date', '>', $start_time);
            $orders->where('delivery_date', '<', $end_time);

            $orders->where('order_status_id', 10);
            $orders->where('is_archived', "0");
            if ($query = $req->query('query', false)) {
                $orders->whereRaw('(orders.total ?  )', ["%$query%"]);
            }
        }

        if ($req->query('nopaginate', false) == '1') {
            $orders = $orders->get();
        } else {
            $orders = $orders->paginate($req->query('per_page', 20));
        }

        if ($orders->isEmpty()) Utils::error('No cancel orders found!', 404);


        return ($orders);
    }
    public function productWise(Request $req)
    {

        $ordersByProduct = EposOrder::selectRaw("products.name As product_name,COUNT(order_items.id) as count,CAST(sum(order_items.total) AS DECIMAL(7,2)) as total")->leftJoin('order_items', 'order_items.order_id', '=', 'orders.id')->leftJoin('products', 'products.id', '=', 'order_items.product_id')->where('order_items.is_delete', '0')->groupBy('order_items.product_id')->orderByRaw('orders.delivery_date DESC');

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ]);

        if ($req->has('from_date')) {

            $from_date_strtotime = strtotime($req->input('from_date'));
            $to_date_strtotime = strtotime($req->input('to_date'));

            while ($from_date_strtotime <= $to_date_strtotime) {
                $from_date = Carbon::createFromTimestamp($from_date_strtotime)->toDateString();

                if ($req->input('shift_status') == 'morning') {
                    $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                    $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                    if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time = $from_date . ' ' . $business_morning_start_time->value;
                    $end_time = $from_date . ' ' . $business_morning_end_time->value;
                } else if ($req->input('shift_status') == 'evening') {
                    $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                    $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                    if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                    $end_time = $from_date . ' ' . $business_evening_end_time->value;
                } else {
                    $start_time = $from_date . ' ' . "00:00:00";
                    $end_time = $from_date . ' ' . "23:59:59";
                }

                $start_time = Carbon::parse($start_time);
                $end_time = Carbon::parse($end_time);

                $ordersByProduct->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);
                $ordersByProduct->where('is_archived', "0");
                $ordersByProduct = $ordersByProduct->whereNotNull('order_items.product_id');

                $from_date_strtotime = strtotime("+1 day", $from_date_strtotime);
            }
        } else {

            $start_time = Carbon::today()->startOfDay();
            $end_time = Carbon::today()->endOfDay();

            if ($req->input('shift_status') == 'morning') {
                $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time = $from_date . ' ' . $business_morning_start_time->value;
                $end_time = $from_date . ' ' . $business_morning_end_time->value;
            } elseif ($req->input('shift_status') == 'evening') {
                $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                $end_time = $from_date . ' ' . $business_evening_end_time->value;
            }

            $ordersByProduct->where('delivery_date', '>', $start_time);
            $ordersByProduct->where('delivery_date', '<', $end_time);
            $ordersByProduct->where('is_archived', "0");
            $ordersByProduct = $ordersByProduct->whereNotNull('order_items.product_id');
        }

        if ($req->has('user_id')) {
            $ordersByProduct = $ordersByProduct->where('orders.updater_id', $req->input('user_id'));
        }

        if ($req->query('nopaginate', false) == '1') {
            $ordersByProduct = $ordersByProduct->get();
        } else {
            $ordersByProduct = $ordersByProduct->paginate($req->query('per_page', 20));
        }

        if ($ordersByProduct->isEmpty()) Utils::error('No orders found!', 404);

        return ($ordersByProduct);
    }

    public function onlineOrder(Request $req)
    {

        $business = Business::find(Auth::$business_id);
        if (empty($business)) Utils::error("Business not found", 400);

        if (empty($business->restaurant_id)) Utils::error("Business not found", 400);

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ]);

        // Online Order Count
        //$online_order_commission = OnlineOrder::where('business_id', $business->id)->orderByRaw('order_date DESC');
        $online_order_commission = TiffintomOrder::where('restaurant_id', $business->restaurant_id)->orderByRaw('delivery_date DESC');

        $online_order_count = TiffintomOrder::selectRaw("COUNT(id) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('restaurant_id', $business->restaurant_id)->orderByRaw('delivery_date DESC');

        // Web Online Order Count
        $web_online_order_count = TiffintomOrder::selectRaw("COUNT(id) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('type', 'Web')->where('restaurant_id', $business->restaurant_id)->orderByRaw('id DESC');

        // Andoid Online Order Count
        $android_online_order_count = TiffintomOrder::selectRaw("COUNT(id) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('type', 'Android')->where('restaurant_id', $business->restaurant_id)->orderByRaw('id DESC');

        // IOS Online Order Count
        $ios_online_order_count = TiffintomOrder::selectRaw("COUNT(id) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('restaurant_id', $business->restaurant_id)->orderByRaw('id DESC');
        $ios_online_order_count->whereRaw('(type LIKE ? OR type LIKE ? )', ["%iOS%", "%ios%"]);

        // Online Order By Type
        $OnlineOrderByType = TiffintomOrder::selectRaw("order_type,COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('restaurant_id', $business->restaurant_id)->groupBy('order_type');

        // Online Order By Payment
        $OnlineOrderByPayment = TiffintomOrder::selectRaw("payment_method,COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total")->where('restaurant_id', $business->restaurant_id)->groupBy('payment_method');

        if ($req->has('from_date')) {
            $start_time = Carbon::parse($req->input('from_date'))->startOfDay();
            $end_time = Carbon::parse($req->input('to_date'))->endOfDay();
            // $start_time = Carbon::parse($start_time)->format('Y-m-d');
            // $end_time = Carbon::parse($end_time)->format('Y-m-d');
            $online_order_commission->WhereRaw('(created > ? AND created < ?)', [$start_time, $end_time]);
            $online_order_count->WhereRaw('(created > ? AND created < ?)', [$start_time, $end_time]);
            $web_online_order_count->WhereRaw('(created > ? AND created < ?)', [$start_time, $end_time]);
            $android_online_order_count->WhereRaw('(created > ? AND created < ?)', [$start_time, $end_time]);
            $ios_online_order_count->WhereRaw('(created > ? AND created < ?)', [$start_time, $end_time]);
            $OnlineOrderByType->WhereRaw('(created > ? AND created < ?)', [$start_time, $end_time]);
            $OnlineOrderByPayment->WhereRaw('(created > ? AND created < ?)', [$start_time, $end_time]);
        } else {
            $start_time = Carbon::now()->startOfDay();
            $end_time = Carbon::now()->endOfDay();

            $online_order_commission->where('created', '>', $start_time);
            $online_order_commission->where('created', '<', $end_time);

            $online_order_count->where('created', '>', $start_time);
            $online_order_count->where('created', '<', $end_time);

            $web_online_order_count->where('created', '>', $start_time);
            $web_online_order_count->where('created', '<', $end_time);

            $android_online_order_count->where('created', '>', $start_time);
            $android_online_order_count->where('created', '<', $end_time);

            $ios_online_order_count->where('created', '>', $start_time);
            $ios_online_order_count->where('created', '<', $end_time);

            $OnlineOrderByType->where('created', '>', $start_time);
            $OnlineOrderByType->where('created', '<', $end_time);

            $OnlineOrderByPayment->where('created', '>', $start_time);
            $OnlineOrderByPayment->where('created', '<', $end_time);
        }

        $online_order_commission = $online_order_commission->get();
        $online_order_commission->each(function ($transaction) {
            $transaction->append('calculated_fees');
        });

        $Online_order_calculated_commission = 0;
        foreach ($online_order_commission as $key => $value) {
            $Online_order_calculated_commission += $value['calculated_fees']['total_fee'];
        }

        $online_order_count = $online_order_count->get();

        $web_online_order_count = $web_online_order_count->get();

        $android_online_order_count = $android_online_order_count->get();

        $ios_online_order_count = $ios_online_order_count->get();

        $OnlineOrderByType = $OnlineOrderByType->get();

        $OnlineOrderByPayment = $OnlineOrderByPayment->get();

        // Tiffintom Api Remove After this function Work
        if ($business->restaurant_id) {
            $req->merge(['restaurant_id' => $business->restaurant_id, 'status' => 'Pending', 'nopaginate' => 1, 'append' => 1]);
            $reservationDetail = (new TiffintomBookingController())->index($req);
        } else {
            $reservationDetail = [];
        }

        if (!empty($OnlineOrderByPayment)) {
            foreach ($OnlineOrderByPayment as $paymentKey => $paymentValue) {
                if (strtolower($paymentValue->payment_method) == 'stripe') {
                    $paymentValue->payment_method = 'Paid';
                }
            }
        }

        $data = [
            'online_order_count' =>  $this->convertToDecimal($online_order_count[0]['count']),
            'online_reservation_count' => sizeof($reservationDetail),
            'online_order_total_business' => $online_order_count[0]['total'] != '' ?  $this->convertToDecimal($online_order_count[0]['total']) : '0.00',
            'online_order_type' => $OnlineOrderByType,
            'online_order_payment' => $OnlineOrderByPayment,
            'web_online_order_count' =>  $web_online_order_count,
            'android_online_order_count' =>  $android_online_order_count,
            'ios_online_order_count' =>  $ios_online_order_count,
            'online_order_calculated_commission' =>  $this->convertToDecimal($Online_order_calculated_commission),
        ];

        return $data;
    }

    public function productCategoryWise(Request $req)
    {

        $ordersByProductCategory = EposOrder::selectRaw("IFNULL(categories.name,'MISC') As category_name,COUNT(order_items.id) as count,CAST(sum(order_items.total) AS DECIMAL(7,2)) as total")->leftJoin('order_items', 'order_items.order_id', '=', 'orders.id')->leftJoin('products', 'products.id', '=', 'order_items.product_id')->leftJoin('categories', 'categories.id', '=', 'products.category_id')->groupBy('products.category_id')->orderByRaw('orders.delivery_date DESC');

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ]);

        if ($req->has('from_date')) {

            $from_date_strtotime = strtotime($req->input('from_date'));
            $to_date_strtotime = strtotime($req->input('to_date'));

            while ($from_date_strtotime <= $to_date_strtotime) {
                $from_date = Carbon::createFromTimestamp($from_date_strtotime)->toDateString();

                if ($req->input('shift_status') == 'morning') {
                    $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                    $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                    if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time = $from_date . ' ' . $business_morning_start_time->value;
                    $end_time = $from_date . ' ' . $business_morning_end_time->value;
                } else if ($req->input('shift_status') == 'evening') {
                    $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                    $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                    if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                    $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                    $end_time = $from_date . ' ' . $business_evening_end_time->value;
                } else {
                    $start_time = $from_date . ' ' . "00:00:00";
                    $end_time = $from_date . ' ' . "23:59:59";
                }

                $start_time = Carbon::parse($start_time);
                $end_time = Carbon::parse($end_time);

                $ordersByProductCategory->orWhereRaw('(delivery_date > ? AND delivery_date < ?)', [$start_time, $end_time]);

                $ordersByProductCategory = $ordersByProductCategory->whereNotNull('order_items.product_id');
                $ordersByProductCategory->where('is_archived', "0");
                $from_date_strtotime = strtotime("+1 day", $from_date_strtotime);
            }
        } else {

            $start_time = Carbon::today()->startOfDay();
            $end_time = Carbon::today()->endOfDay();

            if ($req->input('shift_status') == 'morning') {
                $business_morning_start_time = (new EposSettingController())->findByKey('business_morning_start_time');
                $business_morning_end_time = (new EposSettingController())->findByKey('business_morning_end_time');
                if (empty($business_morning_start_time) || empty($business_morning_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time = $from_date . ' ' . $business_morning_start_time->value;
                $end_time = $from_date . ' ' . $business_morning_end_time->value;
            } elseif ($req->input('shift_status') == 'evening') {
                $business_evening_start_time = (new EposSettingController())->findByKey('business_evening_start_time');
                $business_evening_end_time = (new EposSettingController())->findByKey('business_evening_end_time');
                if (empty($business_evening_start_time) || empty($business_evening_start_time)) Utils::error('start or end morning time not set in settings!', 404);

                $from_date = Carbon::today()->startOfDay()->toDateString();
                $start_time =  $from_date . ' ' . $business_evening_start_time->value;
                $end_time = $from_date . ' ' . $business_evening_end_time->value;
            }

            $ordersByProductCategory->where('delivery_date', '>', $start_time);
            $ordersByProductCategory->where('delivery_date', '<', $end_time);
            $ordersByProductCategory->where('is_archived', "0");
            $ordersByProductCategory = $ordersByProductCategory->whereNotNull('order_items.product_id');
        }

        $ordersByProductCategory->where('order_items.is_delete', '0');
        if ($req->has('user_id')) {
            $ordersByProductCategory = $ordersByProductCategory->where('orders.updater_id', $req->input('user_id'));
        }

        if ($req->query('nopaginate', false) == '1') {
            $ordersByProductCategory = $ordersByProductCategory->get();
        } else {
            $ordersByProductCategory = $ordersByProductCategory->paginate($req->query('per_page', 20));
        }

        if ($ordersByProductCategory->isEmpty()) Utils::error('No orders found!', 404);

        return ($ordersByProductCategory);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
