<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposReservationConfirmation;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposReservationConfirmationController extends Controller
{
    /**
     * List reservationConfirmations.
     * GET /epos/reservation-confirmations
     */
    public function index(Request $req)
    {
        $reservationConfirmations = EposReservationConfirmation::orderByRaw('confirmation ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $reservationConfirmations->whereRaw('(confirmation LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $reservationConfirmations = $reservationConfirmations->get();
        } else {
            $reservationConfirmations = $reservationConfirmations->paginate($req->query('per_page', 20));
        }

        if ($reservationConfirmations->isEmpty()) Utils::error('No reservationConfirmations found!', 404);


        return ($reservationConfirmations);
    }

    /**
     * Create reservationConfirmation.
     * POST /epos/reservation-confirmations
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'confirmation' => 'required|string|max:255',
        ]);

        // create reservationConfirmation
        $reservationConfirmation = new EposReservationConfirmation;
        $reservationConfirmation->fill($req->all());

        $reservationConfirmation->save();

        return $reservationConfirmation;
    }

    /**
     * Show reservationConfirmation.
     * GET /epos/reservation-confirmations/{id}
     */
    public function show(Request $req, $id)
    {
        $reservationConfirmation = EposReservationConfirmation::find($id);
        if (empty($reservationConfirmation)) Utils::error('EposReservationConfirmation not found!', 404);
        return $reservationConfirmation;
    }

    /**
     * Update reservationConfirmation.
     * POST /epos/reservation-confirmations/{id}
     */
    public function update(Request $req, $id)
    {
        $reservationConfirmation = EposReservationConfirmation::find($id);
        if (empty($reservationConfirmation)) Utils::error('EposReservationConfirmation not found!', 404);

        $this->validate($req, [
            'confirmation' => 'string|max:255',
        ]);

        // update reservationConfirmation
        if ($req->has('confirmation')) $reservationConfirmation->confirmation = $req->input('confirmation');

        $reservationConfirmation->save();

        return $reservationConfirmation;
    }


    /**
     * Delete reservationConfirmation by id.
     * DELETE /epos/reservation-confirmations/{id}
     */
    public function delete(Request $req, $id)
    {
        $reservationConfirmation = EposReservationConfirmation::find($id);
        if (empty($reservationConfirmation)) Utils::error('EposReservationConfirmation not found!', 404);
        $reservationConfirmation->delete();
        return $reservationConfirmation;
    }
}
