<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposReservationStatus;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposReservationStatusController extends Controller
{
    /**
     * List tableStatuses.
     * GET /api/table-statuses
     */
    public function index(Request $req)
    {
        $tableStatuses = EposReservationStatus::orderByRaw('status ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $tableStatuses->whereRaw('(status LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $tableStatuses = $tableStatuses->get();
        } else {
            $tableStatuses = $tableStatuses->paginate($req->query('per_page', 20));
        }

        if ($tableStatuses->isEmpty()) Utils::error('No tableStatuses found!', 404);


        return ($tableStatuses);
    }

    /**
     * Create tableStatus.
     * POST /api/table-statuses
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'status' => 'required|string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // create tableStatus
        $tableStatus = new EposReservationStatus;
        $tableStatus->fill($req->all());

        $tableStatus->save();

        return $tableStatus;
    }

    /**
     * Show tableStatus.
     * GET /api/table-statuses/{id}
     */
    public function show(Request $req, $id)
    {
        $tableStatus = EposReservationStatus::find($id);
        if (empty($tableStatus)) Utils::error('EposReservationStatus not found!', 404);

        return $tableStatus;
    }

    /**
     * Update tableStatus.
     * POST /api/table-statuses/{id}
     */
    public function update(Request $req, $id)
    {
        $tableStatus = EposReservationStatus::find($id);
        if (empty($tableStatus)) Utils::error('EposReservationStatus not found!', 404);

        $this->validate($req, [
            'status' => 'string|max:255',
            'top_color' => 'string|max:9|min:7',
            'bottom_color' => 'string|max:9|min:7',
        ]);

        // update tableStatus
        if ($req->has('status')) $tableStatus->status = $req->input('status');
        if ($req->has('top_color')) $tableStatus->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $tableStatus->bottom_color = $req->input('bottom_color');

        $tableStatus->save();

        return $tableStatus;
    }


    /**
     * Delete tableStatus by id.
     * DELETE /api/table-statuses/{id}
     */
    public function delete(Request $req, $id)
    {
        $tableStatus = EposReservationStatus::find($id);
        if (empty($tableStatus)) Utils::error('EposReservationStatus not found!', 404);
        $tableStatus->delete();
        return $tableStatus;
    }
}
