<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposReservationType;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposReservationTypeController extends Controller
{
    /**
     * List reservationTypes.
     * GET /epos/reservation-types
     */
    public function index(Request $req)
    {
        $reservationTypes = EposReservationType::orderByRaw('type ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $reservationTypes->whereRaw('(type LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $reservationTypes = $reservationTypes->get();
        } else {
            $reservationTypes = $reservationTypes->paginate($req->query('per_page', 20));
        }

        if ($reservationTypes->isEmpty()) Utils::error('No reservationTypes found!', 404);


        return ($reservationTypes);
    }

    /**
     * Create reservationType.
     * POST /epos/reservation-types
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'type' => 'required|string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // create reservationType
        $reservationType = new EposReservationType;
        $reservationType->fill($req->all());

        $reservationType->save();

        return $reservationType;
    }

    /**
     * Show reservationType.
     * GET /epos/reservation-types/{id}
     */
    public function show(Request $req, $id)
    {
        $reservationType = EposReservationType::find($id);
        if (empty($reservationType)) Utils::error('EposReservationType not found!', 404);
        return $reservationType;
    }

    /**
     * Update reservationType.
     * POST /epos/reservation-types/{id}
     */
    public function update(Request $req, $id)
    {
        $reservationType = EposReservationType::find($id);
        if (empty($reservationType)) Utils::error('EposReservationType not found!', 404);

        $this->validate($req, [
            'type' => 'string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // update reservationType
        if ($req->has('type')) $reservationType->type = $req->input('type');
        if ($req->has('top_color')) $reservationType->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $reservationType->bottom_color = $req->input('bottom_color');

        $reservationType->save();

        return $reservationType;
    }


    /**
     * Delete reservationType by id.
     * DELETE /epos/reservation-types/{id}
     */
    public function delete(Request $req, $id)
    {
        $reservationType = EposReservationType::find($id);
        if (empty($reservationType)) Utils::error('EposReservationType not found!', 404);
        $reservationType->delete();
        return $reservationType;
    }
}
