<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Utils;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposSetting;
use Illuminate\Support\Facades\Log;

class EposSettingController extends Controller
{
    /**
     * List siteSettings.
     * GET /api/settings
     */
    public function index(Request $req)
    {
        $siteSettings = EposSetting::orderByRaw('sequence ASC, setting_group ASC');

        if ($query = $req->query('query', false)) {
            $siteSettings->whereRaw('(name LIKE ?)', ["%$query%"]);
        }

        if ($setting_group = $req->query('setting_group', false)) {
            $siteSettings->whereRaw('(setting_group LIKE ?)', ["%$setting_group%"]);
        }

        if ($req->query('disabled') == '0' || $req->query('disabled') == '1') {
            $siteSettings->where('disabled', $req->query('disabled'));
        }

        if ($req->query('nopaginate', false) == '1') {
            $siteSettings = $siteSettings->get();
        } else {
            $siteSettings = $siteSettings->paginate($req->query('per_page', 20));
        }

        if ($siteSettings->isEmpty()) Utils::error('No Settings found!', 404);
        if ($req->query('setting_group') == '') {
            $siteSettings[] = (new SiteSettingController())->findByKey('epos_android_app_version');
            $siteSettings[] = (new SiteSettingController())->findByKey('epos_ios_app_version');
        }

        return ($siteSettings);
    }

    /**
     * Create siteSettings.
     * POST /api/settings
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'key' => 'required|alpha_dash|string|unique',
            'value' => 'required',
            'comment' => 'string',
            'hint' => 'string',
            'setting_group' => 'string',
            'data_type' => 'in:enum,string,int,double,bool,json,json_array,text',
            'is_editable' => 'string',
            'disabled' => 'boolean',
        ]);

        // create siteSettings
        $siteSettings = new EposSetting;
        $siteSettings->fill($req->all());

        $siteSettings->save();

        return $siteSettings;
    }

    /**
     * Show siteSettings.
     * GET /api/settings/{id}
     */
    public function show(Request $req, $id)
    {
        $siteSettings = EposSetting::find($id);
        if (empty($siteSettings)) Utils::error('EposSetting not found!', 404);
        return $siteSettings;
    }

    public static function findByKey($key)
    {
        $siteSettings = EposSetting::where('key', $key)->first();
        return $siteSettings;
    }
    /**
     * Update siteSettings.
     * POST /api/settings/{id}
     */
    public function update(Request $req, $id)
    {
        $siteSettings = EposSetting::find($id);
        if (empty($siteSettings)) Utils::error('EposSetting not found!', 404);

        $this->validate($req, [
            'data_type' => 'in:enum,string,int,double,bool,json,json_array,text',
            'comment' => 'string',
            'hint' => 'string',
            'setting_group' => 'string',
            'is_editable' => 'string',
            'disabled' => 'boolean',
        ]);

        // update siteSettings
        if ($req->input('key') != 'epos_android_app_version' && $req->input('key') != 'epos_ios_app_version') {
            $siteSettings->value = $req->input('value');
            if ($req->has('data_type')) $siteSettings->data_type = $req->input('data_type');
            if ($req->has('comment')) $siteSettings->comment = $req->input('comment');
            if ($req->has('hint')) $siteSettings->hint = $req->input('hint');
            if ($req->has('setting_group')) $siteSettings->setting_group = $req->input('setting_group');
            if ($req->has('is_editable')) $siteSettings->is_editable = $req->input('is_editable');
            if ($req->has('disabled')) $siteSettings->disabled = $req->input('disabled');
            $siteSettings->save();
        } else {
        }
        return $siteSettings;
    }


    /**
     * Delete siteSettings by id.
     * DELETE /api/settings/{id}
     */
    public function delete(Request $req, $id)
    {
        $siteSettings = EposSetting::find($id);
        if (empty($siteSettings)) Utils::error('EposSetting not found!', 404);
        $siteSettings->delete();
        return $siteSettings;
    }

    /**
     * Load siteSettings.
     * POST /api/settings/load-settings
     */
    public function load_settings(Request $req)
    {
        $tables[] = [
            "name" => "Credit Card Mode",
            "key" => "stripe_mode",
            "comment" => "Credit card mode",
            "hint" => "test or live",
            "setting_group" => "Payment Method",
            "data_type" => "enum",
            "expected_values" => "live,test",
            "value" => 'test',
            "sequence" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Public Key for Test Mode",
            "key" => "stripe_public_key_test",
            "comment" => "Credit Card Public Key for Test mode",
            "hint" => "Enter public key for Test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => 'pk_test_AdQkfvDPjhh5OBqmlJ5DGCub',
            "sequence" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Private Key for Test Mode",
            "key" => "stripe_private_key_test",
            "comment" => "Credit Card Private Key for Test mode",
            "hint" => "Enter private key for Test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "sk_test_5J87rf5MsZJYIK36Jt9bveuI",
            "sequence" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Public Key for Live Mode",
            "key" => "stripe_public_key_live",
            "comment" => "Card Public Key for Live mode",
            "hint" => "Enter public key for Live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "pk_live_1vbUyNpmwlvwkoLmpZtn5Gk6",
            "sequence" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Private Key for Live Mode",
            "key" => "stripe_private_key_live",
            "comment" => "Card Private Key for Live mode",
            "hint" => "Enter private key for Live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "sk_live_M3enKM9GcFqbfhMGNkUU32oO",
            "sequence" => "0",
        ];
        $tables[] = [
            "name" => "Paypal Mode",
            "key" => "paypal_mode",
            "comment" => "Paypal mode",
            "hint" => "test or live",
            "setting_group" => "Payment Method",
            "data_type" => "enum",
            "expected_values" => "live,test",
            "value" => 'test',
            "sequence" => "5",
        ];
        $tables[] = [
            "name" => "Paypal Public Key for Test Mode",
            "key" => "paypal_public_key_test",
            "comment" => "Paypal Public Key for Test mode",
            "hint" => "Enter public key for test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "AY_5fJxxSwWtvcQfVcCqTgngHv9wmc1WEdgi0njaLoTEsS9oouuC2akeT78cEPFavcglH7LC2ynDWTcd",
            "sequence" => "5",
        ];
        $tables[] = [
            "name" => "Paypal Private Key for Test Mode",
            "key" => "paypal_private_key_test",
            "comment" => "Paypal Private Key for Test mode",
            "hint" => "Enter private key for test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "EDFMZIw9RhyKoYvh10vSq2KKhr6fD0SMJRFDcOWAQAZ-wEBBqvqUaa6v983oERQ6mq6-26PnC26gEpcQ",
            "sequence" => "5",
        ];

        $tables[] = [
            "name" => "Paypal Public Key for Live Mode",
            "key" => "paypal_public_key_live",
            "comment" => "Paypal Public Key for Live mode",
            "hint" => "Enter public key for live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "ARFbYu4nOwxgdMytHfC0btbilHBtuJTumJJUOY7OEcCTQgxdRPFuzd1Fn48EKpB2WbLckaSqodLJivXL",
            "sequence" => "5",
        ];
        $tables[] = [
            "name" => "Paypal Private Key for Live Mode",
            "key" => "paypal_private_key_live",
            "comment" => "Paypal Private Key for Live mode",
            "hint" => "Enter private key for live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "EFZGDsFgHF4ceSxhLJmQijmEL48sgVon_ciPTxeciyAnLCRWYut7-eZx4bD-llYL_g69-rQOxuZmd1e2",
            "sequence" => "5",
        ];
        $tables[] = [
            "name" => "Google Api key",
            "key" => "google_map_api_key",
            "comment" => "Google map api key",
            "hint" => "Enter google api key",
            "setting_group" => "Api Key",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "",
            "sequence" => "6",
        ];
        foreach ($tables as $table) {

            $siteSetting = $this->findByKey($table['key']);
            if (empty($siteSetting)) {
                $siteSetting = new EposSetting($table);
                $siteSetting->save();
            } else {
                error_log("Settings found for " . $table['key']);
            }
        }
        return EposSetting::get();
    }

    /**
     * Bulk Update siteSettings.
     * POST /api/settings/setting-bulk
     */
    public function setting_bulk(Request $req)
    {
        if ($req->has('siteSettings') && !is_array($req->input('siteSettings'))) {
            $req->merge(['siteSettings' => json_decode($req->input('siteSettings'))]);
        }
        $this->validate($req, [
            'siteSettings' => 'required|array'
        ],);
        $siteSettings = ($req->input('siteSettings'));
        foreach ($siteSettings as $siteSetting) {

            if ($siteSetting->key != "epos_android_app_version" && $siteSetting->key != "epos_ios_app_version") {
                $DBSiteSetting = EposSetting::find($siteSetting->id);
                if (!empty($DBSiteSetting)) {
                    $DBSiteSetting->value = $siteSetting->value;
                    if (isset($siteSetting->setting_group)) {
                        $DBSiteSetting->setting_group = $siteSetting->setting_group;
                    }
                    if ($DBSiteSetting->is_editable) $DBSiteSetting->is_editable = $siteSetting->is_editable;
                    $DBSiteSetting->save();
                }
            }
        }
        return $siteSettings;
    }
}
