<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\OtpHelper;
use App\ModelsEpos\EposSmsPackage;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposSmsPackageController extends Controller
{
    /**
     * List sms-packages.
     * GET /epos/sms-packages
     */
    public function index(Request $req)
    {
        $sms_packages = EposSmsPackage::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $sms_packages->whereRaw('(phone_number LIKE ?  )', ["%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $sms_packages = $sms_packages->get();
        } else {
            $sms_packages = $sms_packages->paginate($req->query('per_page', 20));
        }

        if ($sms_packages->isEmpty()) Utils::error('No sms_packages found!', 404);


        return ($sms_packages);
    }

    /**
     * Create sms_package.
     * POST /epos/sms_packages
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'ip' => 'required|string|max:255',
        ]);

        // create sms_package
        $sms_package = new EposSmsPackage;
        $sms_package->fill($req->all());

        $sms_package->save();

        return $sms_package;
    }

    /**
     * send message.
     * GET epos/sms_packages/send-message
     */
    public function send_message(Request $req)
    {
        $this->validate($req, [
            'message' => 'required|string',
            'bulk' => 'boolean',
            'phone_number' => 'required_if:bulk,0|string',
            'bulk_file' => 'required_if:bulk,1|mimes:xlsx,xls',
        ]);

        Log::debug($req->file('bulk_file')->getClientOriginalExtension());

        // send sms_package
        // (new OtpHelper())->sendMessage($req->input('message'), $req->input('phone_number'));

        return $req;
    }

    /**
     * Show sms_package.
     * GET /epos/sms_packages/{id}
     */
    public function show(Request $req, $id)
    {
        $sms_package = EposSmsPackage::find($id);
        if (empty($sms_package)) Utils::error('EposSmsPackage not found!', 404);
        return $sms_package;
    }

    /**
     * Update sms_package.
     * POST /epos/sms_packages/{id}
     */
    public function update(Request $req, $id)
    {
        $sms_package = EposSmsPackage::find($id);
        if (empty($sms_package)) Utils::error('Sms Package not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'ip' => 'string|max:255',

        ]);

        // update sms_package
        if ($req->has('name')) $sms_package->name = $req->input('name');
        if ($req->has('ip')) $sms_package->ip = $req->input('ip');

        $sms_package->save();

        return $sms_package;
    }


    /**
     * Delete sms_package by id.
     * DELETE /epos/sms_packages/{id}
     */
    public function delete(Request $req, $id)
    {
        $sms_package = EposSmsPackage::find($id);
        if (empty($sms_package)) Utils::error('EposSmsPackage not found!', 404);
        $sms_package->delete();
        return $sms_package;
    }
}
