<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposTableVisitor;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposTableVisitorController extends Controller
{
    /**
     * List tableVisitors.
     * GET /epos/table-visitors
     */
    public function index(Request $req)
    {
        $tableVisitors = EposTableVisitor::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $tableVisitors->whereRaw('(tableVisitors.number LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('table_id', false)) {
            $tableVisitors->where('table_id', $query);
        }
        if ($query = $req->query('customer_id', false)) {
            $tableVisitors->where('customer_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $tableVisitors = $tableVisitors->get();
        } else {
            $tableVisitors = $tableVisitors->paginate($req->query('per_page', 20));
        }

        if ($tableVisitors->isEmpty()) Utils::error('No tableVisitors found!', 404);


        return ($tableVisitors);
    }

    /**
     * Create tableVisitor.
     * POST /epos/table-visitors
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'table_id' => 'required|exists:epos_connection.tableVisitors,id',
            'customer_id' => 'required|exists:epos_connection.customers,id',
            'guests' => 'numeric',
            'start_time' => 'date_formate:Y-m-d H:i:s',
            'end_time' => 'date_formate:Y-m-d H:i:s',
        ]);

        // create tableVisitor
        $tableVisitor = new EposTableVisitor;
        $tableVisitor->fill($req->all());
        $tableVisitor->save();

        return $tableVisitor;
    }

    /**
     * Show tableVisitor.
     * GET /epos/table-visitors/{id}
     */
    public function show(Request $req, $id)
    {
        $tableVisitor = EposTableVisitor::selectRaw('tableVisitors.*,floors.name as floor_name,tableVisitor_statuses.status as status')->leftJoin('tableVisitor_statuses', 'tableVisitor_statuses.id', '=', 'tableVisitor_status_id')->leftJoin('floors', 'floors.id', '=', 'floor_id')->find($id);
        if (empty($tableVisitor)) Utils::error('EposTableVisitor not found!', 404);
        return $tableVisitor;
    }

    /**
     * Update tableVisitor.
     * POST /epos/table-visitors/{id}
     */
    public function update(Request $req, $id)
    {
        $tableVisitor = EposTableVisitor::find($id);
        if (empty($tableVisitor)) Utils::error('EposTableVisitor not found!', 404);

        $this->validate($req, [
            'table_id' => 'exists:epos_connection.tableVisitors,id',
            'customer_id' => 'exists:epos_connection.customers,id',
            'guests' => 'numeric',
            'start_time' => 'date_formate:Y-m-d H:i:s',
            'end_time' => 'date_formate:Y-m-d H:i:s',
        ]);

        // update tableVisitor
        if ($req->has('table_id')) $tableVisitor->table_id = $req->input('table_id');
        if ($req->has('customer_id')) $tableVisitor->customer_id = $req->input('customer_id');
        if ($req->has('guests')) $tableVisitor->guests = $req->input('guests');
        if ($req->has('start_time')) $tableVisitor->start_time = $req->input('start_time');
        if ($req->has('end_time')) $tableVisitor->end_time = $req->input('end_time');

        $tableVisitor->save();

        return $tableVisitor;
    }


    /**
     * Delete tableVisitor by id.
     * DELETE /epos/table-visitors/{id}
     */
    public function delete(Request $req, $id)
    {
        $tableVisitor = EposTableVisitor::find($id);
        if (empty($tableVisitor)) Utils::error('EposTableVisitor not found!', 404);
        $tableVisitor->delete();
        return $tableVisitor;
    }
}
