<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposUserGroup;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposUserGroupController extends Controller
{
    /**
     * List userGroups.
     * GET /epos/user-groups
     */
    public function index(Request $req)
    {
        $userGroups = EposUserGroup::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $userGroups->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $userGroups = $userGroups->get();
        } else {
            $userGroups = $userGroups->paginate($req->query('per_page', 20));
        }

        if ($userGroups->isEmpty()) Utils::error('No userGroups found!', 404);


        return ($userGroups);
    }

    /**
     * Create userGroup.
     * POST /epos/user-groups
     */
    public function create(Request $req)
    {
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'description' => 'string',
            'permissions' => 'required',
        ]);

        // create userGroup
        $userGroup = new EposUserGroup;
        $userGroup->fill($req->all());
        if ($req->has('permissions')) $userGroup->permissions = $req->input('permissions');
        $userGroup->save();

        return $userGroup;
    }

    /**
     * Show userGroup.
     * GET /epos/user-groups/{id}
     */
    public function show(Request $req, $id)
    {
        $userGroup = EposUserGroup::find($id);
        if (empty($userGroup)) Utils::error('EposUserGroup not found!', 404);
        return $userGroup;
    }

    /**
     * Update userGroup.
     * POST /epos/user-groups/{id}
     */
    public function update(Request $req, $id)
    {
        $userGroup = EposUserGroup::find($id);
        if (empty($userGroup)) Utils::error('EposUserGroup not found!', 404);

        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'name' => 'string|max:255',
            'description' => 'string',
            'permissions' => 'required',
        ]);

        // update userGroup
        if ($req->has('name')) $userGroup->name = $req->input('name');
        if ($req->has('description')) $userGroup->description = $req->input('description');
        if ($req->has('permissions')) $userGroup->permissions = $req->input('permissions');

        $userGroup->save();

        return $userGroup;
    }


    /**
     * Delete userGroup by id.
     * DELETE /epos/user-groups/{id}
     */
    public function delete(Request $req, $id)
    {
        $userGroup = EposUserGroup::find($id);
        if (empty($userGroup)) Utils::error('EposUserGroup not found!', 404);
        $userGroup->delete();
        return $userGroup;
    }
}
