<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposUserType;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class EposUserTypeController extends Controller
{
    /**
     * List userTypes.
     * GET /epos/user-types
     */
    public function index(Request $req)
    {
        $userTypes = EposUserType::orderByRaw('type ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $userTypes->whereRaw('(type LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $userTypes = $userTypes->get();
        } else {
            $userTypes = $userTypes->paginate($req->query('per_page', 20));
        }

        if ($userTypes->isEmpty()) Utils::error('No userTypes found!', 404);


        return ($userTypes);
    }

    /**
     * Create userType.
     * POST /epos/user-types
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'type' => 'required|string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // create userType
        $userType = new EposUserType;
        $userType->fill($req->all());

        $userType->save();

        return $userType;
    }

    /**
     * Show userType.
     * GET /epos/user-types/{id}
     */
    public function show(Request $req, $id)
    {
        $userType = EposUserType::find($id);
        if (empty($userType)) Utils::error('EposUserType not found!', 404);
        return $userType;
    }

    /**
     * Update userType.
     * POST /epos/user-types/{id}
     */
    public function update(Request $req, $id)
    {
        $userType = EposUserType::find($id);
        if (empty($userType)) Utils::error('EposUserType not found!', 404);

        $this->validate($req, [
            'type' => 'string|max:255',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
        ]);

        // update userType
        if ($req->has('type')) $userType->type = $req->input('type');
        if ($req->has('top_color')) $userType->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $userType->bottom_color = $req->input('bottom_color');

        $userType->save();

        return $userType;
    }


    /**
     * Delete userType by id.
     * DELETE /epos/user-types/{id}
     */
    public function delete(Request $req, $id)
    {
        $userType = EposUserType::find($id);
        if (empty($userType)) Utils::error('EposUserType not found!', 404);
        $userType->delete();
        return $userType;
    }
}
