<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposVoucher;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposOrder;
use App\ModelsEpos\EposOrderPayment;
use Carbon\Carbon;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposVoucherController extends Controller
{
    /**
     * List vouchers.
     * GET /epos/vouchers
     */
    public function index(Request $req)
    {
        $vouchers = EposVoucher::selectRaw('vouchers.*,customers.name as customer_name,customers.email as customer_email,customers.mobile as customer_mobile')->leftJoin('customers', 'customers.id', '=', 'customer_id')->orderByRaw('vouchers.created_at DESC');

        if ($query = $req->query('query', false)) {
            $vouchers->whereRaw('(vouchers.voucher_code LIKE ?  )', ["%$query%"]);
        }

        if ($customer_id = $req->query('customer_id', false)) {
            $vouchers->whereRaw('(vouchers.customer_id LIKE ? OR vouchers.customer_id is null)', ["%$customer_id%"]);
        }

        if ($order_type_id = $req->query('order_type_id', false)) {
            $vouchers->whereIn('vouchers.order_type_id', [$order_type_id]);
        }

        if ($from_date = $req->query('from_date', false)) {
            // $vouchers->where('vouchers.start_date', '>=', Carbon::parse($from_date)->startOfDay());
            // $vouchers->whereRaw('(vouchers.end_date <= ? OR vouchers.end_date is null)', [Carbon::parse($req->input('to_date'))->endOfDay()]);
            $vouchers->whereRaw('((vouchers.start_date <= ? AND vouchers.end_date >= ?) OR ((vouchers.start_date <= ? AND vouchers.end_date >= ?) OR vouchers.end_date is null) OR (vouchers.start_date >= ? AND vouchers.end_date <= ?))', [Carbon::parse($from_date)->startOfDay(), Carbon::parse($from_date)->startOfDay(), Carbon::parse($req->input('to_date'))->endOfDay(), Carbon::parse($req->input('to_date'))->endOfDay(), Carbon::parse($from_date)->startOfDay(), Carbon::parse($req->input('to_date'))->endOfDay()]);
        }

        if ($req->query('disabled') == '0' || $req->query('disabled') == '1') {
            $vouchers->where('disabled', $req->query('disabled'));
        }
        if ($req->query('nopaginate', false) == '1') {
            $vouchers = $vouchers->get();
        } else {
            $vouchers = $vouchers->paginate($req->query('per_page', 20));
        }
        if ($vouchers->isEmpty()) Utils::error('No vouchers found!', 404);

        return ($vouchers);
    }

    /**
     * Create voucher.
     * POST /epos/vouchers
     */
    public function create(Request $req)
    {
        if (!is_array($req->customer))
            $req->merge(['customer' => (json_decode($req->customer, true))]);

        $this->validate($req, [
            'customer_id' => 'exists:epos_connection.customers,id',
            'order_type_id' => 'string',
            'week_day' => 'string',
            'voucher_code' => 'required|string',
            'voucher_amount' => 'required|numeric|gt:0',
            'minimum_order_value' => 'numeric',
            'start_date' => 'required|date',
            'end_date' => 'date',
            'customer' => 'array',
            'unique_id' => 'string',
        ]);

        // create voucher
        $voucher = new EposVoucher;
        $voucher->fill($req->all());
        // $customerDetail = ($req->input('customer'));
        if ($req->has('customer') && $req->input('customer')['name'] == '' && empty($req->input('customer_id'))) {
            $customerDetail = $req->input('customer');
            $customer = new EposCustomer((array)$customerDetail);
            $customer->membership_no = rand(100000000, 999999999);
            $customer->save();

            $voucher->customer_id = $customer->id;
        } else {
            $customerDetail = ($req->input('customer'));
            $customer = EposCustomer::find($customerDetail['id']);
            if (!empty($customer)) {
                if (isset($customerDetail['name'])) $customer->name = $customerDetail['name'];
                if (isset($customerDetail['mobile'])) $customer->mobile = $customerDetail['mobile'];
                if (isset($customerDetail['email'])) $customer->email = $customerDetail['email'];
                if (isset($customerDetail['landline'])) $customer->landline = $customerDetail['landline'];
                if (isset($customerDetail['birth_date'])) $customer->birth_date = $customerDetail['birth_date'];
                if (isset($customerDetail['anniversary_date'])) $customer->anniversary_date = $customerDetail['anniversary_date'];
                if (isset($customerDetail['house_no'])) $customer->house_no = $customerDetail['house_no'];
                if (isset($customerDetail['street'])) $customer->street = $customerDetail['street'];
                if (isset($customerDetail['postcode'])) $customer->postcode = $customerDetail['postcode'];
                if (isset($customerDetail['distance'])) $customer->distance = $customerDetail['distance'];
                if (isset($customerDetail['city'])) $customer->city = $customerDetail['city'];
                if (isset($customerDetail['country'])) $customer->country = $customerDetail['country'];
                if (isset($customerDetail['state'])) $customer->state = $customerDetail['state'];
                $customer->save();

                $voucher->customer_id = $customer->id;
            }
        }
        $voucher->save();

        return $voucher;
    }

    /**
     * Show voucher.
     * GET /epos/vouchers/{id}
     */
    public function show(Request $req, $id)
    {
        $voucher = EposVoucher::find($id);
        if (empty($voucher)) Utils::error('EposVoucher not found!', 404);
        return $voucher;
    }


    /**
     * 
     * Search Voucher Code.
     * GET voucher/find/{code}
     * 
     */
    public function find(Request $req, $code)
    {

        $this->validate($req, [
            // 'order_id' => 'required',
        ]);

        $day_of_week = date('N', strtotime(date('l')));

        $from_date = Carbon::today();
        $to_date = Carbon::today();

        $voucher = EposVoucher::whereRaw('(voucher_code LIKE ? OR voucher_code LIKE ? ) ', [$code, str_replace(" ", "", $code)])->where('disabled', '0')->first();
        if (empty($voucher)) Utils::error('Invalid voucher!', 404);

        if ($voucher->end_date != null) {
            if (Carbon::parse($voucher->start_date)->timestamp > Carbon::today()->timestamp && Carbon::parse($voucher->end_date)->timestamp < Carbon::today()->timestamp) {
                Utils::error("Voucher expired", 400);
            }
        } else {
            if (Carbon::parse($voucher->start_date)->timestamp > Carbon::today()->endOfDay()->timestamp) {
                Utils::error("Voucher is not valid before date", 400);
            }
        }

        if (!empty($voucher->week_day) && str_contains($voucher->week_day, $day_of_week))
            Utils::error("Voucher not applicable for today", 400);

        $appliedVoucherPayment = EposOrderPayment::where('voucher_id', $voucher->id)->where('payment_method_id', '5')->get();
        $voucher->applied = !$appliedVoucherPayment->isEmpty();

        return $voucher;
    }

    /**
     * Update voucher.
     * POST /epos/vouchers/{id}
     */
    public function update(Request $req, $id)
    {
        if (!is_array($req->customer))
            $req->merge(['customer' => (json_decode($req->customer, true))]);

        $voucher = EposVoucher::find($id);
        if (empty($voucher)) Utils::error('EposVoucher not found!', 404);

        $this->validate($req, [
            'customer_id' => 'exists:epos_connection.customers,id',
            'order_type_id' => 'string',
            'week_day' => 'string',
            'voucher_code' => 'required|string',
            'voucher_amount' => 'required|numeric|gt:0',
            'minimum_order_value' => 'numeric',
            'start_date' => 'required|date',
            'end_date' => 'date',
            'customer' => 'array',
            'unique_id' => 'string',
        ]);

        // update voucher
        if ($req->has('customer_id')) $voucher->customer_id = $req->input('customer_id');
        if ($req->has('order_type_id')) $voucher->order_type_id = $req->input('order_type_id');
        if ($req->has('week_day')) $voucher->week_day = $req->input('week_day');
        if ($req->has('voucher_code')) $voucher->voucher_code = $req->input('voucher_code');
        if ($req->has('voucher_amount')) $voucher->voucher_amount = $req->input('voucher_amount');
        if ($req->has('minimum_order_value')) $voucher->minimum_order_value = $req->input('minimum_order_value');
        if ($req->has('start_date')) $voucher->start_date = $req->input('start_date');
        if ($req->has('unique_id')) $voucher->unique_id = $req->input('unique_id');
        $voucher->end_date = $req->input('end_date');

        if ($req->has('customer') && $req->input('customer')['name'] == '' && empty($req->input('customer_id'))) {
            $customerDetail = $req->input('customer');
            $customer = new EposCustomer((array)$customerDetail);
            $customer->membership_no = rand(100000000, 999999999);
            $customer->save();

            $voucher->customer_id = $customer->id;
        } else {
            $customerDetail = $req->input('customer');
            $customer = EposCustomer::find($customerDetail['id']);
            if (!empty($customer)) {
                if (isset($customerDetail['name'])) $customer->name = $customerDetail['name'];
                if (isset($customerDetail['mobile'])) $customer->mobile = $customerDetail['mobile'];
                if (isset($customerDetail['email'])) $customer->email = $customerDetail['email'];
                if (isset($customerDetail['landline'])) $customer->landline = $customerDetail['landline'];
                if (isset($customerDetail['birth_date'])) $customer->birth_date = $customerDetail['birth_date'];
                if (isset($customerDetail['anniversary_date'])) $customer->anniversary_date = $customerDetail['anniversary_date'];
                if (isset($customerDetail['house_no'])) $customer->house_no = $customerDetail['house_no'];
                if (isset($customerDetail['street'])) $customer->street = $customerDetail['street'];
                if (isset($customerDetail['postcode'])) $customer->postcode = $customerDetail['postcode'];
                if (isset($customerDetail['distance'])) $customer->distance = $customerDetail['distance'];
                if (isset($customerDetail['city'])) $customer->city = $customerDetail['city'];
                if (isset($customerDetail['country'])) $customer->country = $customerDetail['country'];
                if (isset($customerDetail['state'])) $customer->state = $customerDetail['state'];
                $customer->save();

                $voucher->customer_id = $customer->id;
            }
        }
        $voucher->save();

        return $voucher;
    }


    /**
     * Delete voucher by id.
     * DELETE /epos/vouchers/{id}
     */
    public function delete(Request $req, $id)
    {
        $voucher = EposVoucher::find($id);
        if (empty($voucher)) Utils::error('EposVoucher not found!', 404);
        $voucher->delete();
        return $voucher;
    }

    /**
     * Enable voucher.
     * POST /epos/vouchers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $voucher = EposVoucher::find($id);
        if (empty($voucher)) Utils::error('EposVoucher not found!', 404);
        $voucher->disabled = 0;
        $voucher->save();
        return $voucher;
    }

    /**
     * Disable voucher.
     * POST /epos/vouchers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $voucher = EposVoucher::find($id);
        if (empty($voucher)) Utils::error('EposVoucher not found!', 404);
        $voucher->disabled = 1;
        $voucher->save();
        return $voucher;
    }
}
