<?php

namespace App\Http\Controllers;

use App\Business;
use App\BusinessCommission;
use App\CardBrand;
use App\CardBrandCommission;
use App\CardBrandCommissionHistory;
use App\Country;
use App\CountryCommission;
use App\CountryCommissionHistory;
use App\Exports\BankReportExport;
use App\Exports\FinancialReportExport;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\MiscCharge;
use App\MiscInvoice;
use App\ModelsConnect\ExternalAccount;
use App\OnlineOrder;
use App\Passbook;
use App\PayoutCharge;
use App\PayoutInvoice;
use App\PaypalPayment;
use App\ResellerCommission;
use App\ResellerOrderCommission;
use App\ResellerStripeCommission;
use App\StripeTransaction;
use Carbon\Carbon;
use DateTime;
use Exception;
use Illuminate\Http\Request;
use GuzzleHttp\Client;
use Illuminate\Http\Client\Response;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Maatwebsite\Excel\Facades\Excel;
use Stripe\Stripe;
use Stripe\StripeClient;

class ExtraController extends Controller
{

    public function transactions(Request $req)
    {
        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;

        if ($stripeMode == 'test') {
            $secret_key = SiteSettingController::findByKey('stripe_private_key_test')->value;
        } else {
            $secret_key = SiteSettingController::findByKey('stripe_private_key_live')->value;
        }
        $lastTransaction = StripeTransaction::orderByRaw('created DESC')->first();
        $params = [];
        $options = [];
        if (!empty($lastTransaction) && !$req->has('account_id')) {
            // $params['ending_before'] = $lastTransaction->txn_id;
        }
        $params['limit'] = 50;

        if ($req->has('account_id'))
            $options['stripe_account'] = $req->input('account_id');
        // $params['destination'] = 'acct_1IhvXq3iYYuo9Vks';
        $stripe = new StripeClient($secret_key);
        $transactions = $stripe->balanceTransactions->all($params, $options);
        return $transactions;

        return $transactions;
    }

    public function card_reader_connection_token(Request $req)
    {
        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($stripeMode == 'test') {
            $stripeKeyColoumns = 'stripe_private_key_test';
        } else {
            $stripeKeyColoumns = 'stripe_private_key_live';
        }
        if ($req->has('business_id')) {
            $business_id = $req->input('business_id');
            $business = Business::find($business_id);
            if (empty($business)) {
                Utils::error("Business not found", 400);
            }
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                $mode = strtolower($mode);
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
            }
        }

        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $connectionParams = [];
        if ($req->has('s_location_id') && !empty($req->input('s_location_id'))) {
            $connectionParams['location'] = $req->input('s_location_id');
        }
        $connectionToken = $stripe->terminal->connectionTokens->create($connectionParams);
        return $data = ['token' => $connectionToken->secret];
    }

    public function registered_card_readers(Request $req)
    {
        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($stripeMode == 'test') {
            $key = SiteSettingController::findByKey('stripe_private_key_test')->value;
        } else {
            $key = SiteSettingController::findByKey('stripe_private_key_live')->value;
        }
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        return  $stripe->terminal->readers->all();
    }

    public function card_reader_intent(Request $req)
    {
        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($stripeMode == 'test') {
            $key = SiteSettingController::findByKey('stripe_private_key_test')->value;
        } else {
            $key = SiteSettingController::findByKey('stripe_private_key_live')->value;
        }
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $this->validate($req, [
            'amount' => 'required|gt:0'
        ]);
        return  $stripe->paymentIntents->create([
            'amount' => $req->input('amount'),
            'currency' => 'gbp',
            'payment_method_types' => ['card_present'],
            'capture_method' => 'manual',
        ]);
    }

    public function card_reader_intent_capture(Request $req)
    {
        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($stripeMode == 'test') {
            $key = SiteSettingController::findByKey('stripe_private_key_test')->value;
        } else {
            $key = SiteSettingController::findByKey('stripe_private_key_live')->value;
        }
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $this->validate($req, [
            'payment_intent' => 'required'
        ]);
        $intent =  $stripe->paymentIntents->retrieve($req->input('payment_intent'));
        $intent->capture();
        return  $stripe->paymentIntents->retrieve($req->input('payment_intent'));
    }

    public function calculateFee($humanAmount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($humanAmount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }

    public function admin_report(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after:from_date',
        ]);

        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";
        } else {
            $to_date = Carbon::now()->endOfDay()->format('Y-m-d H:i:s');
            $from_date = Carbon::now()->subDays(11)->startOfDay()->format('Y-m-d H:i:s');
        }
        $business_id = null;
        if ($req->has('business_id')) {
            $business_id = $req->input('business_id');
        }

        $data = [
            'from_date' => $from_date,
            'to_date' => $to_date
        ];
        $data['normal'] = $this->transactionQueries($from_date, $to_date, 0, $business_id);
        $data['archived'] = $this->transactionQueries($from_date, $to_date, 1, $business_id);

        return $data;
    }

    public function transactionQueries($from_date, $to_date, $archived, $business_id = null)
    {
        $response = [];
        $paypalOrders = OnlineOrder::select('online_orders.*', 'paypal_payments.transaction_fee as provider_fee', 'online_orders.txn_id as online_txn_id', 'businesses.name as business_name')->orderByRaw('order_date DESC');
        $paypalOrders = $paypalOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $paypalOrders = $paypalOrders->leftJoin('paypal_payments', 'sale_id', '=', 'online_orders.txn_id');
        $paypalOrders = $paypalOrders->leftJoin('businesses', 'online_orders.business_id', '=', 'businesses.id');
        $paypalOrders = $paypalOrders->where('online_orders.archived', $archived);
        $paypalOrders = $paypalOrders->where('order_date', '>=', "$from_date");
        $paypalOrders = $paypalOrders->where('order_date', '<=', "$to_date");
        $paypalOrders = $paypalOrders->where('payment_method', '=', 'paypal');
        $paypalOrders = $paypalOrders->whereNotNull('online_orders.business_id');
        $paypalOrders = $paypalOrders->whereIn('order_status', ['Delivered', 'Failed']);
        if ($business_id != null) {
            $paypalOrders = $paypalOrders->where('online_orders.business_id', '=', $business_id);
        }
        $paypalOrders = $paypalOrders->get();
        if (!$paypalOrders->isEmpty()) {
            $data['title'] = 'Paypal Report';
            $data['report'] = $this->createReport($paypalOrders);
            $response[] = $data;
        }

        $stripeOrders = OnlineOrder::select('online_orders.*', 'stripe_transactions.fee as provider_fee', 'stripe_transactions.txn_id as stripe_txn_id', 'online_orders.txn_id as online_txn_id', 'businesses.name as business_name')->orderByRaw('order_date DESC');
        $stripeOrders = $stripeOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $stripeOrders = $stripeOrders->leftJoin('stripe_transactions', function ($join) {
            $join->on('stripe_transactions.payment_intent_id', '=', 'online_orders.txn_id')->where('reporting_category', 'charge');
        });
        $stripeOrders = $stripeOrders->leftJoin('businesses', 'online_orders.business_id', '=', 'businesses.id');
        $stripeOrders = $stripeOrders->where('online_orders.archived', $archived);
        $stripeOrders = $stripeOrders->where('order_date', '>=', "$from_date");
        $stripeOrders = $stripeOrders->where('order_date', '<=', "$to_date");
        $stripeOrders = $stripeOrders->whereNotNull('online_orders.business_id');
        $stripeOrders = $stripeOrders->whereRaw('(payment_method LIKE ? OR payment_method LIKE ?)', ['Credit Card', 'Online']);
        $stripeOrders = $stripeOrders->whereIn('order_status', ['Delivered', 'Failed']);
        if ($business_id != null) {
            $stripeOrders = $stripeOrders->where('online_orders.business_id', '=', $business_id);
        }
        $stripeOrders = $stripeOrders->get();
        if (!$stripeOrders->isEmpty()) {
            $data['title'] = 'Card Report';
            $data['report'] = $this->createReport($stripeOrders);
            $response[] = $data;
        }

        $codOrders = OnlineOrder::select('online_orders.*', 'businesses.name as business_name')->orderByRaw('order_date DESC');
        $codOrders = $codOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $codOrders = $codOrders->leftJoin('businesses', 'online_orders.business_id', '=', 'businesses.id');
        $codOrders = $codOrders->where('order_date', '>=', "$from_date");
        $codOrders = $codOrders->where('order_date', '<=', "$to_date");
        $codOrders = $codOrders->where('payment_method', '=', 'cod');
        $codOrders = $codOrders->where('online_orders.archived', $archived);
        $codOrders = $codOrders->whereNotNull('online_orders.business_id');
        $codOrders = $codOrders->whereIn('order_status', ['Delivered', 'Failed']);
        if ($business_id != null) {
            $codOrders = $codOrders->where('online_orders.business_id', '=', $business_id);
        }
        $codOrders = $codOrders->get();
        if (!$codOrders->isEmpty()) {
            $data['title'] = 'COD Report';
            $data['report'] = $this->createReport($codOrders);
            $response[] = $data;
        }

        $merchantTransactions = StripeTransaction::select('stripe_transactions.*', 'businesses.name as business_name');
        $merchantTransactions = $merchantTransactions->leftJoin('businesses', 'stripe_transactions.business_id', '=', 'businesses.id');
        $merchantTransactions = $merchantTransactions->where('created', '>=', strtotime($from_date));
        $merchantTransactions = $merchantTransactions->where('created', '<=', strtotime($to_date));
        $merchantTransactions = $merchantTransactions->where('stripe_transactions.archived', $archived);
        $merchantTransactions = $merchantTransactions->where('service', '!=', 'Online');
        $merchantTransactions = $merchantTransactions->whereNotNull('stripe_transactions.business_id');
        $merchantTransactions = $merchantTransactions->whereIn('reporting_category', ['charge', 'refund']);
        if ($business_id != null) {
            $merchantTransactions = $merchantTransactions->where('stripe_transactions.business_id', '=', $business_id);
        }
        $merchantTransactions = $merchantTransactions->get();
        if (!$merchantTransactions->isEmpty()) {
            $data['title'] = 'Merchant Report';
            $data['report'] = $this->createMerchantReport($merchantTransactions);
            $response[] = $data;
        }
        return $response;
    }

    public function createReport($orders)
    {
        $transactions = array();

        $gross_total = 0.0;
        $net_total = 0.0;
        $total_fee = 0.0;
        $total_service_charge = 0;

        $total_provider_fee = 0.0;
        $total_business_fee = 0.0;
        $total_reseller_fee = 0.0;
        $total_non_domestic_fee = 0;
        $total_charge_back_fee = 0.0;

        $total_cash_payout = 0.0;
        $total_bank_payout = 0.0;

        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;

        foreach ($orders as $order) {

            $orderTypeWiseCommission = 0.00;
            $orderTypeWiseServiceCharge = 0.00;
            $orderBasedOrTransactionWiseCharge = 0.00;
            $payment_method_fees = 0;

            $reseller_orderTypeWiseCommission = 0.00;
            $reseller_orderTypeWiseServiceCharge = 0.00;
            $reseller_orderBasedOrTransactionWiseCharge = 0.00;
            $reseller_payment_method_fees = 0;

            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;

            $chargeBackFee = 0;
            $handlingFee = 0.00;

            $orderTypeWiseServiceCharge = $order->user_service_charge;
            $orderGrandTotal = $order->grand_total;

            $resellerCommission = ResellerOrderCommission::where('online_order_id', $order->id)->orderBy('id', 'DESC')->first();
            $providerFee = 0;
            if ($order->payment_method == 'paypal') {
                $providerFee = $order->provider_fee;
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_paypal_fee, $order->online_paypal_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_payment_method_fees  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_paypal_fee, $resellerCommission->online_paypal_fee_type);
                }
            }
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online') {
                $providerFee = $order->provider_fee / 100;
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_card_fee, $order->online_card_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_payment_method_fees  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_card_fee, $resellerCommission->online_card_fee_type);
                }
            }

            if ($order->payment_method == 'cod') {
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_cod_fee, $order->online_cod_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_payment_method_fees  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_cod_fee, $resellerCommission->online_cod_fee_type);
                }
            }

            if ($order->order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->dinein_commission_fee, $order->dinein_commission_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_orderTypeWiseCommission  = $this->calculateFee($orderGrandTotal, $resellerCommission->dinein_commission_fee, $resellerCommission->dinein_commission_fee_type);
                    $reseller_orderTypeWiseServiceCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->dinein_service_fee, $resellerCommission->dinein_service_fee_type);
                }
            } else {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->online_commission_fee, $order->online_commission_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_orderTypeWiseCommission  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_commission_fee, $resellerCommission->online_commission_fee_type);
                    $reseller_orderTypeWiseServiceCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_service_fee, $resellerCommission->online_service_fee_type);
                }
            }
            if ($order->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotal, $order->per_order_charge, $order->per_order_charge_type);
                if (!empty($resellerCommission)) {
                    $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->per_order_charge, $resellerCommission->per_order_charge_type);
                }
            } else {
                if ($order->payment_method == 'paypal') {
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->paypal_charge, $order->paypal_charge_type);
                    if (!empty($resellerCommission)) {
                        $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->paypal_charge, $resellerCommission->paypal_charge_type);
                    }
                }
                if (strtolower($order->payment_method) == 'credit card' || strtolower($order->payment_method) == 'online') {
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->card_charge, $order->card_charge_type);
                    if (!empty($resellerCommission)) {
                        $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->card_charge, $resellerCommission->card_charge_type);
                    }
                }
                if ($order->payment_method == 'cod') {
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->cod_charge, $order->cod_charge_type);
                    if (!empty($resellerCommission)) {
                        $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->cod_charge, $resellerCommission->cod_charge_type);
                    }
                }
            }
            $businessFee = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission);
            $resellerFee = $this->convertToDecimal($reseller_payment_method_fees + $reseller_orderTypeWiseCommission + $reseller_orderTypeWiseServiceCharge + $reseller_orderBasedOrTransactionWiseCharge);
            $serviceCharge = 0;
            $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $serviceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);

            if ($order->order_status == 'Failed') {
                $transaction['order_grand_total'] = -$this->convertToDecimal($order->grand_total);
                if ($order->charges_refundable == 'yes') {
                    if ($order->payment_method == 'cod') {
                        $businessFee = 0;
                        $resellerFee = 0;
                    } else {
                        $businessFee = -$businessFee;
                        $resellerFee = -$resellerFee;
                    }
                } else {
                    if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online') {
                        $businessFee = 0.0;
                        $resellerFee = 0.0;
                    } else {
                        $businessFee = $businessFee;
                        $resellerFee = $resellerFee;
                    }
                }
                $transaction['net_total'] = $this->convertToDecimal(0 - $businessFee);
                // $total_refund_transactions = $total_refund_transactions + 1;
                $total_refund_transactions = $total_refund_transactions + $this->convertToDecimal($orderGrandTotal);
            } else {
                $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotal);
                if ($order->payment_method == 'cod') $transaction['order_grand_total'] = 0;
                $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $businessFee);
            }

            if ($order->payment_method != 'cod')
                $gross_total = $gross_total + $transaction['order_grand_total'];
            $transaction['id'] =  $order->id;
            $transaction['order_id'] = "#ODR000" . $order->order_id;
            $transaction['restaurant_id'] = $order->restaurant_id;
            $transaction['business_id'] = $order->business_id;
            $transaction['business_name'] = $order->business_name;
            if (empty($order->stripe_txn_id)) {
                $transaction['txn_id'] = $order->online_txn_id;
            } else {
                $transaction['txn_id'] = $order->stripe_txn_id;
            }

            $transaction['order_date'] = $order->order_date;
            $transaction['order_type'] = strtolower($order->order_type) == 'delivery' || strtolower($order->order_type) == 'pickup' ? 'Online - ' . ($order->payment_method) : $order->order_type;
            $transaction['order_archived'] = $order->archived;
            if (!empty($order->online_reporting_category)) {
                $transaction['order_status'] = $order->online_reporting_category;
            } else {
                $transaction['order_status'] = $order->order_status;
            }
            $transaction['payment_method'] = strtolower($order->payment_method) == 'credit card' ? 'Online' : $order->payment_method;

            $transaction['payment_method_fees'] = $payment_method_fees;
            $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
            $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
            $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;

            $transaction['country_charges'] = $countryWiseCommission;
            $transaction['card_charges'] = $cardBrandWiseCommission;
            $transaction['charge_back_fee'] = 0;
            $transaction['provider_fee'] = $providerFee;

            $transaction['business_fee'] = $businessFee;
            $transaction['reseller_fee'] = $resellerFee;
            $transaction['total_fee'] = $transaction['business_fee'] - $transaction['reseller_fee'] - $transaction['provider_fee'];
            $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotal);

            if ($order->payment_method != 'Failed') {
                $total_provider_fee = $total_provider_fee + $transaction['provider_fee'];
                $total_business_fee = $total_business_fee + $transaction['business_fee'];
                $total_reseller_fee = $total_reseller_fee + $transaction['reseller_fee'];
                $total_fee = $total_fee + $transaction['total_fee'];

                if ($transaction['order_archived']) {
                    $total_cash_payout = $total_cash_payout + $transaction['net_total'];
                } else {
                    $total_bank_payout = $total_bank_payout + $transaction['net_total'];
                }
            }

            $total_charge_back_fee = $total_charge_back_fee + ($transaction['charge_back_fee']);
            $transactions[] = $transaction;
        }
        $report['transactions'] = $transactions;

        $report['total_business_fee'] = $this->convertToDecimal($total_business_fee);
        $report['total_reseller_fee'] = $this->convertToDecimal($total_reseller_fee);
        $report['total_provider_fee'] = $this->convertToDecimal($total_provider_fee);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['cash_payout'] = $this->convertToDecimal($total_cash_payout);
        $report['bank_payout'] = $this->convertToDecimal($total_bank_payout);
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($gross_total - $total_business_fee);
        $report['total_non_domestic_fee'] =  $this->convertToDecimal($total_non_domestic_fee);
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        $report['total_charge_back_transactions'] = $total_charge_back_transactions;
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        return $report;
    }

    public function createMerchantReport($merchantTransactions)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_fee = 0.0;

        $total_provider_fee = 0;
        $total_business_fee = 0.0;
        $total_reseller_fee = 0.0;
        $total_non_domestic_fee = 0;
        $total_charge_back_fee = 0.0;

        $total_cash_payout = 0.0;
        $total_bank_payout = 0.0;

        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;

        foreach ($merchantTransactions as $transaction) {
            $cardProcessingFee = 0.00;
            $cardAuthFee = 0.00;
            $chargeBackFee = 0;

            $reseller_cardProcessingFee = 0.00;
            $reseller_cardAuthFee = 0.00;
            $reseller_chargeBackFee = 0;

            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;

            $cardCountry = $transaction->card_country;
            $brand = $transaction->card_brand;

            $business = Business::select('id', 'name', 'connect_service')->find($transaction->business_id);
            $business->makeHidden('business_commissions');
            $business->makeHidden('user_counts');
            $isConnectAccount = $business->connect_service;

            $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $transaction->id)->first();

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_auth_fee, $transaction->merchant_card_auth_fee_type));
                $cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_processing_fee, $transaction->merchant_card_processing_fee_type));
                if (!empty($resellerStripeCommission)) {
                    $reseller_cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $resellerStripeCommission->merchant_card_auth_fee, $resellerStripeCommission->merchant_card_auth_fee_type));
                    $reseller_cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $resellerStripeCommission->merchant_card_processing_fee, $resellerStripeCommission->merchant_card_processing_fee_type));
                }
                $countryCommission = CountryCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();
                if (!empty($countryCommission)) {
                    $countryWiseCommission = abs($this->calculateFee($transaction->amount / 100, $countryCommission->stripe_percentage, 'percentage'));
                    $countryWiseCommission = $countryWiseCommission + $countryCommission->stripe_fixed;
                }
                $cardBrandCommission = CardBrandCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();

                if (!empty($cardBrandCommission)) {
                    $cardBrandWiseCommission = abs($this->calculateFee($transaction->amount / 100, $cardBrandCommission->stripe_percentage, 'percentage'));
                    $cardBrandWiseCommission = $cardBrandWiseCommission + $cardBrandCommission->stripe_fixed;
                }
            }
            if ($transaction->reporting_category == 'refund') {
                $cardAuthFee = -$cardAuthFee;
                $cardProcessingFee = -$cardProcessingFee;

                $reseller_cardAuthFee = -$reseller_cardAuthFee;
                $reseller_cardProcessingFee = -$reseller_cardProcessingFee;

                $cardBrandWiseCommission = -$cardBrandWiseCommission;
                $countryWiseCommission = -$countryWiseCommission;

                $total_refund_transactions = $total_refund_transactions + $this->convertToDecimal($transaction->amount / 100);
                // $total_refund_transactions = $total_refund_transactions + 1;
            }
            if ($transaction->reporting_category == 'dispute') {
                $chargeBackFee = $this->calculateFee($transaction->amount / 100, $transaction->charge_back_fee, $transaction->charge_back_fee_type);
                if (!empty($resellerStripeCommission)) {
                    $reseller_chargeBackFee = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->charge_back_fee, $resellerStripeCommission->charge_back_fee_type);
                }
                $total_charge_back_transactions = $total_charge_back_transactions + 1;
            }

            $businessFee = ($this->convertToDecimal($cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee));
            $resellerFee = ($this->convertToDecimal($reseller_cardAuthFee + $reseller_cardProcessingFee + $reseller_chargeBackFee));

            if ($isConnectAccount && $transaction->service == 'Connect') {
                if ($transaction->reporting_category == 'refund' || $transaction->reporting_category == 'transfer_reversal') {
                    $businessFee = 0;
                    $resellerFee = 0;
                } else {
                    $applicationFeeTransaction = StripeTransaction::where('reporting_category', 'platform_earning')->where('payment_intent_id', $transaction->payment_intent_id)->first();
                    if (!empty($applicationFeeTransaction)) {
                        // $businessFee =  $this->convertToDecimal($applicationFeeTransaction->amount / 100);
                    }
                    // else {
                    //     $cardAuthFee  = $this->calculateFee($transaction->amount / 100, $transaction->connect_card_auth_fee, $transaction->connect_card_auth_fee_type);
                    //     $cardProcessingFee  = $this->calculateFee($transaction->amount / 100, $transaction->connect_card_processing_fee, $transaction->connect_card_processing_fee_type);
                    //     $businessFee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
                    // }
                    if (!empty($resellerStripeCommission)) {
                        $reseller_cardAuthFee  = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->connect_card_auth_fee, $resellerStripeCommission->connect_card_auth_fee_type);
                        $reseller_cardProcessingFee  = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->connect_card_processing_fee, $resellerStripeCommission->connect_card_processing_fee_type);

                        $resellerFee = $this->convertToDecimal($reseller_cardAuthFee + $reseller_cardProcessingFee);
                    }
                }
            }
            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                // if ($isConnectAccount && abs($businessFee) > ($transaction->amount / 100)) $businessFee = $transaction->amount / 100;
            }

            $mTransaction['id'] = $transaction->id;
            $mTransaction['order_id'] = $transaction->txn_id;
            $mTransaction['restaurant_id'] = null;
            $mTransaction['business_id'] = $transaction->business_id;
            $mTransaction['business_name'] = $business->name;
            $mTransaction['txn_id'] = $transaction->txn_id;

            $mTransaction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
            $mTransaction['order_type'] = $transaction->service;
            $mTransaction['order_archived'] = $transaction->archived;
            $mTransaction['order_status'] = $transaction->reporting_category;
            $mTransaction['payment_method'] = $transaction->service;
            $mTransaction['order_grand_total'] = $transaction->amount / 100;

            $mTransaction['processing_fee'] = $cardProcessingFee;
            $mTransaction['country'] = $cardCountry;
            $mTransaction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
            $mTransaction['card_brand'] = $brand;
            $mTransaction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
            $mTransaction['auth_fee'] = $cardAuthFee;
            $mTransaction['handling_fee'] = 0;

            $mTransaction['country_charges'] = $countryWiseCommission;
            $mTransaction['card_brand_charge'] = $cardBrandWiseCommission;
            $mTransaction['charge_back_fee'] = $chargeBackFee;

            $mTransaction['provider_fee'] = $transaction->fee / 100;
            $mTransaction['business_fee'] = $businessFee;
            $mTransaction['reseller_fee'] = $resellerFee;
            $mTransaction['total_fee'] = $mTransaction['business_fee'] - $mTransaction['reseller_fee'] - $mTransaction['provider_fee'];

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $mTransaction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $mTransaction['business_fee']);
                //if ($transaction->service == 'Connect' && abs($mTransaction['business_fee']) > ($transaction->amount / 100)) $mTransaction['net_total'] = 0;
            } else {
                if ($transaction->reporting_category == 'dispute') {
                    $mTransaction['total_fee'] = $chargeBackFee;
                    $mTransaction['business_fee'] = $chargeBackFee;
                    $mTransaction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $mTransaction['business_fee']);
                } else {
                    $mTransaction['total_fee'] = 0;
                    $mTransaction['business_fee'] = 0;
                    $mTransaction['reseller_fee'] = 0;
                    if ($transaction->reporting_category != 'charge') {
                        $mTransaction['net_total'] = 0;
                    } else {
                        $mTransaction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $mTransaction['total_fee']);
                    }
                }
            }

            if ($transaction->non_domestic) {
                $total_non_domestic_transactions = $total_non_domestic_transactions + 1;
                $total_non_domestic_fee = $total_non_domestic_fee + $countryWiseCommission;
            }
            if ($transaction->reporting_category == 'charge' || $transaction->reporting_category == 'transfer_reversal' || $transaction->reporting_category == 'dispute' || $transaction->reporting_category == 'refund') {
                $gross_total = $gross_total + $mTransaction['order_grand_total'];
            }
            // if ($transaction->reporting_category != 'dispute' && $transaction->reporting_category != 'refund' && $transaction->reporting_category != 'failed') {
            if ($transaction->reporting_category == 'charge' || $transaction->reporting_category == 'transfer_reversal' || $transaction->reporting_category == 'dispute' || $transaction->reporting_category == 'refund') {
                if ($transaction->archived) {
                    $total_cash_payout = $total_cash_payout + $mTransaction['net_total'];
                } else {
                    $total_bank_payout = $total_bank_payout + $mTransaction['net_total'];
                }

                $total_provider_fee = $total_provider_fee + $mTransaction['provider_fee'];
                $total_business_fee = $total_business_fee + $mTransaction['business_fee'];
                $total_reseller_fee = $total_reseller_fee + $mTransaction['reseller_fee'];
                $total_fee = $total_fee + $mTransaction['total_fee'];
            }

            $total_charge_back_fee = $total_charge_back_fee + ($transaction['charge_back_fee']);
            $transactions[] = $mTransaction;
        }

        $report['transactions'] = $transactions;

        $report['total_business_fee'] = $this->convertToDecimal($total_business_fee);
        $report['total_reseller_fee'] = $this->convertToDecimal($total_reseller_fee);
        $report['total_provider_fee'] = $this->convertToDecimal($total_provider_fee);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['cash_payout'] = $this->convertToDecimal($total_cash_payout);
        $report['bank_payout'] = $this->convertToDecimal($total_bank_payout);
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($gross_total - $total_business_fee);
        $report['total_non_domestic_fee'] =  $this->convertToDecimal($total_non_domestic_fee);
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        $report['total_charge_back_transactions'] = $total_charge_back_transactions;
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        return $report;
    }

    public function createFinancialMerchantReport($merchantTransactions)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_fee = 0.0;

        $total_provider_fee = 0;
        $total_business_fee = 0.0;
        $total_reseller_fee = 0.0;
        $total_non_domestic_fee = 0;
        $total_charge_back_fee = 0.0;

        $total_cash_payout = 0.0;
        $total_bank_payout = 0.0;

        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;

        foreach ($merchantTransactions as $transaction) {
            $cardProcessingFee = 0.00;
            $cardAuthFee = 0.00;
            $chargeBackFee = 0;

            $reseller_cardProcessingFee = 0.00;
            $reseller_cardAuthFee = 0.00;
            $reseller_chargeBackFee = 0;

            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;

            $cardCountry = $transaction->card_country;
            $brand = $transaction->card_brand;

            $transaction->append('business');
            $business = $transaction->business;
            $isConnectAccount = $business->connect_service;

            $transaction->append('reseller_stripe_transactions');
            $resellerStripeCommission = $transaction->reseller_stripe_transactions;

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_auth_fee, $transaction->merchant_card_auth_fee_type));
                $cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_processing_fee, $transaction->merchant_card_processing_fee_type));
                if (!empty($resellerStripeCommission)) {
                    $reseller_cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $resellerStripeCommission->merchant_card_auth_fee, $resellerStripeCommission->merchant_card_auth_fee_type));
                    $reseller_cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $resellerStripeCommission->merchant_card_processing_fee, $resellerStripeCommission->merchant_card_processing_fee_type));
                }
                $transaction->append('country_commission_history');
                $countryCommission = $transaction->country_commission_history;
                if (!empty($countryCommission)) {
                    $countryWiseCommission = abs($this->calculateFee($transaction->amount / 100, $countryCommission->stripe_percentage, 'percentage'));
                    $countryWiseCommission = $countryWiseCommission + $countryCommission->stripe_fixed;
                }
                $transaction->append('card_brand_commission');
                $cardBrandCommission = $transaction->card_brand_commission;

                if (!empty($cardBrandCommission)) {
                    $cardBrandWiseCommission = abs($this->calculateFee($transaction->amount / 100, $cardBrandCommission->stripe_percentage, 'percentage'));
                    $cardBrandWiseCommission = $cardBrandWiseCommission + $cardBrandCommission->stripe_fixed;
                }
            }
            if ($transaction->reporting_category == 'refund') {
                $cardAuthFee = -$cardAuthFee;
                $cardProcessingFee = -$cardProcessingFee;

                $reseller_cardAuthFee = -$reseller_cardAuthFee;
                $reseller_cardProcessingFee = -$reseller_cardProcessingFee;

                $cardBrandWiseCommission = -$cardBrandWiseCommission;
                $countryWiseCommission = -$countryWiseCommission;

                $total_refund_transactions = $total_refund_transactions + $this->convertToDecimal($transaction->amount / 100);
            }
            if ($transaction->reporting_category == 'dispute') {
                $chargeBackFee = $this->calculateFee($transaction->amount / 100, $transaction->charge_back_fee, $transaction->charge_back_fee_type);
                if (!empty($resellerStripeCommission)) {
                    $reseller_chargeBackFee = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->charge_back_fee, $resellerStripeCommission->charge_back_fee_type);
                }
                $total_charge_back_transactions = $total_charge_back_transactions + 1;
            }

            $businessFee = ($this->convertToDecimal($cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee));
            $resellerFee = ($this->convertToDecimal($reseller_cardAuthFee + $reseller_cardProcessingFee + $reseller_chargeBackFee));

            if ($isConnectAccount && $transaction->service == 'Connect') {
                if ($transaction->reporting_category == 'refund' || $transaction->reporting_category == 'transfer_reversal') {
                    $businessFee = 0;
                    $resellerFee = 0;
                } else {
                    if (!empty($resellerStripeCommission)) {
                        $reseller_cardAuthFee  = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->connect_card_auth_fee, $resellerStripeCommission->connect_card_auth_fee_type);
                        $reseller_cardProcessingFee  = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->connect_card_processing_fee, $resellerStripeCommission->connect_card_processing_fee_type);

                        $resellerFee = $this->convertToDecimal($reseller_cardAuthFee + $reseller_cardProcessingFee);
                    }
                }
            }

            $mTransaction['id'] = $transaction->id;
            $mTransaction['order_id'] = $transaction->txn_id;
            $mTransaction['restaurant_id'] = null;
            $mTransaction['business_id'] = $transaction->business_id;
            $mTransaction['business_name'] = $business->name;
            $mTransaction['txn_id'] = $transaction->txn_id;

            $mTransaction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
            $mTransaction['order_type'] = $transaction->service;
            $mTransaction['order_archived'] = $transaction->archived;
            $mTransaction['order_status'] = $transaction->reporting_category;
            $mTransaction['payment_method'] = $transaction->service;
            $mTransaction['order_grand_total'] = $transaction->amount / 100;

            $mTransaction['processing_fee'] = $cardProcessingFee;
            $mTransaction['country'] = $cardCountry;
            $mTransaction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
            $mTransaction['card_brand'] = $brand;
            $mTransaction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
            $mTransaction['auth_fee'] = $cardAuthFee;
            $mTransaction['handling_fee'] = 0;

            $mTransaction['country_charges'] = $countryWiseCommission;
            $mTransaction['card_brand_charge'] = $cardBrandWiseCommission;
            $mTransaction['charge_back_fee'] = $chargeBackFee;

            $mTransaction['provider_fee'] = $transaction->fee / 100;
            $mTransaction['business_fee'] = $businessFee;
            $mTransaction['reseller_fee'] = $resellerFee;
            $mTransaction['total_fee'] = $mTransaction['business_fee'] - $mTransaction['reseller_fee'] - $mTransaction['provider_fee'];

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $mTransaction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $mTransaction['business_fee']);
            } else {
                if ($transaction->reporting_category == 'dispute') {
                    $mTransaction['total_fee'] = $chargeBackFee;
                    $mTransaction['business_fee'] = $chargeBackFee;
                    $mTransaction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $mTransaction['business_fee']);
                } else {
                    $mTransaction['total_fee'] = 0;
                    $mTransaction['business_fee'] = 0;
                    $mTransaction['reseller_fee'] = 0;
                    if ($transaction->reporting_category != 'charge') {
                        $mTransaction['net_total'] = 0;
                    } else {
                        $mTransaction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $mTransaction['total_fee']);
                    }
                }
            }

            if ($transaction->non_domestic) {
                $total_non_domestic_transactions = $total_non_domestic_transactions + 1;
                $total_non_domestic_fee = $total_non_domestic_fee + $countryWiseCommission;
            }
            if ($transaction->reporting_category == 'charge' || $transaction->reporting_category == 'transfer_reversal' || $transaction->reporting_category == 'dispute' || $transaction->reporting_category == 'refund') {
                $gross_total = $gross_total + $mTransaction['order_grand_total'];
            }
            if ($transaction->reporting_category == 'charge' || $transaction->reporting_category == 'transfer_reversal' || $transaction->reporting_category == 'dispute' || $transaction->reporting_category == 'refund') {
                if ($transaction->archived) {
                    $total_cash_payout = $total_cash_payout + $mTransaction['net_total'];
                } else {
                    $total_bank_payout = $total_bank_payout + $mTransaction['net_total'];
                }

                $total_provider_fee = $total_provider_fee + $mTransaction['provider_fee'];
                $total_business_fee = $total_business_fee + $mTransaction['business_fee'];
                $total_reseller_fee = $total_reseller_fee + $mTransaction['reseller_fee'];
                $total_fee = $total_fee + $mTransaction['total_fee'];
            }

            $total_charge_back_fee = $total_charge_back_fee + ($transaction['charge_back_fee']);
            $transactions[] = $mTransaction;
        }

        $report['transactions'] = $transactions;

        $report['total_business_fee'] = $this->convertToDecimal($total_business_fee);
        $report['total_reseller_fee'] = $this->convertToDecimal($total_reseller_fee);
        $report['total_provider_fee'] = $this->convertToDecimal($total_provider_fee);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['cash_payout'] = $this->convertToDecimal($total_cash_payout);
        $report['bank_payout'] = $this->convertToDecimal($total_bank_payout);
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($gross_total - $total_business_fee);
        $report['total_non_domestic_fee'] =  $this->convertToDecimal($total_non_domestic_fee);
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        $report['total_charge_back_transactions'] = $total_charge_back_transactions;
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        return $report;
    }

    public function bank_report(Request $req)
    {
        ini_set('max_execution_time', 240);
        $businesses = Business::orderByRaw('name ASC');
        $businesses = $businesses->get();
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);
        $businesses->append('default_bank');
        if ($req->input('service') != 'null' && $req->input('service') != '') {
            $new_businesses = [];
            foreach ($businesses as $business) {
                if ($req->input('service') == 'auto') {
                    if ($business->default_bank != null && $business->default_bank->payable_amount <= 0) {
                        $new_businesses[] = $business;
                    }
                }
                if ($req->input('service') == 'manual') {
                    if ($business->default_bank == null || $business->default_bank->payable_amount > 0) {
                        $new_businesses[] = $business;
                    }
                }
            }
        } else {
            return $businesses;
        }
        return $new_businesses;
    }

    public function bank_export(Request $req)
    {
        // $reports = ($this->financial_report($req));
        $reports = 'Financial Report';
        if ($reports != null) {
            $fileName = "Bank Report";
        } else {
            Utils::error("No data found", 400);
        }
        $fileName = "Bank Report";
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $reports = ($this->bank_report($req));
            return Excel::download(new BankReportExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            $reports = ($this->bank_report($req));
            return Excel::download(new BankReportExport($reports), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function export(Request $req)
    {
        // $reports = ($this->financial_report($req));
        $reports = 'Financial Report';
        if ($reports != null) {
            $fileName = "Financial Report";
        } else {
            Utils::error("No data found", 400);
        }
        $fileName = "Financial Report";
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $file = storage_path() . '/app/public/' . "FinancialReport.xlsx";
            return response()->download($file);
            // return Excel::download(new FinancialReportExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            $reports = ($this->financial_report($req));
            return Excel::download(new FinancialReportExport($reports), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function financial_report(Request $req)
    {
        ini_set('max_execution_time', 240);
        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after:from_date',
        ]);

        if ($date_filter = $req->query('date_filter_type', false)) {
            if ($req->filled('date_filter_type') == 1 && $req->input('date_filter_type') != 'null') {
                $now = Carbon::now();
                if ($date_filter == 'today') {
                    $from_date = $now->startOfDay()->format('Y-m-d');
                    $to_date = $now->endOfDay()->format('Y-m-d');
                } else if ($date_filter == 'this_week') {
                    $from_date = Carbon::today()->subDays(6)->startOfDay()->format('Y-m-d');
                    $to_date = $now->endOfDay()->format('Y-m-d');
                } else if ($date_filter == 'this_month') {
                    $from_date = Carbon::today()->subDays(29)->startOfDay()->format('Y-m-d');
                    $to_date = $now->endOfDay()->format('Y-m-d');
                } else if ($date_filter == 'this_quater') {
                    $from_date = Carbon::today()->subDays(89)->startOfDay()->format('Y-m-d');
                    $to_date = $now->endOfDay()->format('Y-m-d');
                } else if ($date_filter == 'last_six_month') {
                    $to_date = Carbon::parse($now)->format('Y-m-d');
                    $from_date = Carbon::today()->subDays(179)->startOfDay()->format('Y-m-d');
                } else if ($date_filter == 'this_year') {
                    $from_date = Carbon::today()->subDays(364)->startOfDay()->format('Y-m-d');
                    $to_date = $now->endOfDay()->format('Y-m-d');
                } else if ($date_filter == 'between') {
                    $date = $req->query('date');
                    $dates = explode(",", $date);
                    $from_date = Carbon::parse($dates[0])->startOfDay()->format('Y-m-d');
                    $to_date = Carbon::parse($dates[1])->endOfDay()->format('Y-m-d');
                } else {
                    Utils::error("Date field required with type", 400);
                }
                $req->merge(['from_date' => $from_date, 'to_date' => $to_date]);
            }
        }

        // return $req;
        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";
        } else {
            $from_date = null;
            $to_date = null;
        }
        $business_id = null;
        $business = null;
        if ($req->has('business_id') && $req->input('business_id') != 'null') {
            $business_id = $req->input('business_id');
            $business = Business::find($business_id);
            if (empty($business)) Utils::error("Business not found", 400);

            $business->append('default_bank_detail');
            $business->append('default_paypal_detail');
            $business->makeHidden(['business_commissions']);
        }

        $service = null;
        if ($req->has('service') && $req->input('service') != 'null') {
            $service = $req->input('service');
        }

        $fileName = "FinancialReport";
        $data = $this->setupTransactions($req, $business, $business_id, $from_date, $to_date, $service);
        Excel::store(new FinancialReportExport($data), $fileName . '.xlsx', 'public');
        Storage::disk('public')->delete($fileName);
        return $data;
    }

    public function setupTransactions($req, $business, $business_id, $from_date, $to_date, $service)
    {
        // $isConnectAccount = $business->connect_service;
        // $chargeDescription = $business->statement_desciptor;

        $data = $this->financial_transactionQueries($from_date, $to_date, $business_id, $service);
        $data['from_date'] = $from_date;
        $data['to_date'] = $to_date;
        $data['business'] = $business;

        $data['misc_charge'] = 0;
        $data['weekly_fee'] = 0;
        $data['monthly_fee'] = 0;

        $data['transaction_type'] = 'Normal';
        // $data['payout_mode'] = $business->payout_mode;
        // $data['payout_method'] = $business->payout_method;
        // $data['payout_comment'] = $business->payout_comment;
        // $data['account_type'] = $isConnectAccount ? 'Connect' : 'Normal';

        // if (strtolower($business->payout_method) == 'paypal') {
        //     if ($business->default_paypal_detail != null) {
        //         $data['payout_account'] = $business->default_paypal_detail;
        //     } else {
        //         $data['payout_account'] = "";
        //     }
        // } else if (strtolower($business->payout_method) == 'bank') {
        //     if ($business->default_bank_detail != null) {
        //         $data['payout_account'] = $business->default_bank_detail;
        //     } else {
        //         $data['payout_account'] = "";
        //     }
        // } else {
        //     $data['payout_account'] = "";
        // }

        $mergedData =  $this->mergeAllTransactionsAndSort($data);
        if (empty($mergedData['transactions'])) Utils::error("No transactions found!!", 400);
        $mergedData['transactions'] = collect($mergedData['transactions']);
        return $mergedData;
    }

    public function financial_transactionQueries($from_date, $to_date, $business_id = null, $service = null)
    {
        $data = [];
        $paypalOrders = OnlineOrder::select('*', 'paypal_payments.transaction_fee as provider_fee', 'paypal_payments.sale_state as online_reporting_category', 'online_orders.txn_id as online_txn_id', 'businesses.name as business_name')->orderByRaw('order_date DESC');
        $paypalOrders = $paypalOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $paypalOrders = $paypalOrders->leftJoin('paypal_payments', 'sale_id', '=', 'online_orders.txn_id');
        $paypalOrders = $paypalOrders->leftJoin('businesses', 'online_orders.business_id', '=', 'businesses.id');
        if ($from_date != null && $to_date != null) {
            $paypalOrders = $paypalOrders->where('order_date', '>=', "$from_date");
            $paypalOrders = $paypalOrders->where('order_date', '<=', "$to_date");
        }
        $paypalOrders = $paypalOrders->where('payment_method', '=', 'paypal');
        $paypalOrders = $paypalOrders->whereNotNull('online_orders.business_id');
        $paypalOrders = $paypalOrders->whereIn('order_status', ['Delivered', 'Failed']);
        if ($service == 'connect') {
            $paypalOrders = $paypalOrders->where('businesses.connect_service', '=', 1);
        }
        if ($service == 'normal') {
            $paypalOrders = $paypalOrders->where('businesses.connect_service', '=', 0);
        }
        if ($business_id != null) {
            $paypalOrders = $paypalOrders->where('online_orders.business_id', '=', $business_id);
        }
        $paypalOrders = $paypalOrders->get();
        if (!$paypalOrders->isEmpty()) {
            $data['paypal_report'] = $this->createReport($paypalOrders);
        }

        $stripeOrders = OnlineOrder::select('*', 'stripe_transactions.fee as provider_fee', 'stripe_transactions.txn_id as stripe_txn_id', 'stripe_transactions.reporting_category as online_reporting_category', 'online_orders.txn_id as online_txn_id', 'businesses.name as business_name')->orderByRaw('order_date DESC');
        $stripeOrders = $stripeOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $stripeOrders = $stripeOrders->leftJoin('stripe_transactions', function ($join) {
            $join->on('stripe_transactions.payment_intent_id', '=', 'online_orders.txn_id')->where('reporting_category', 'charge');
        });
        $stripeOrders = $stripeOrders->leftJoin('businesses', 'online_orders.business_id', '=', 'businesses.id');
        if ($from_date != null && $to_date != null) {
            $stripeOrders = $stripeOrders->where('order_date', '>=', "$from_date");
            $stripeOrders = $stripeOrders->where('order_date', '<=', "$to_date");
        }
        $stripeOrders = $stripeOrders->whereNotNull('online_orders.business_id');
        $stripeOrders = $stripeOrders->whereRaw('(payment_method LIKE ? OR payment_method LIKE ?)', ['Credit Card', 'Online']);
        $stripeOrders = $stripeOrders->whereIn('order_status', ['Delivered', 'Failed']);
        if ($service == 'connect') {
            $stripeOrders = $stripeOrders->where('businesses.connect_service', '=', 1);
        }
        if ($service == 'normal') {
            $stripeOrders = $stripeOrders->where('businesses.connect_service', '=', 0);
        }
        if ($business_id != null) {
            $stripeOrders = $stripeOrders->where('online_orders.business_id', '=', $business_id);
        }
        $stripeOrders = $stripeOrders->get();
        if (!$stripeOrders->isEmpty()) {
            $data['card_report'] = $this->createReport($stripeOrders);
        }

        $codOrders = OnlineOrder::select('*', 'businesses.name as business_name')->orderByRaw('order_date DESC');
        $codOrders = $codOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $codOrders = $codOrders->leftJoin('businesses', 'online_orders.business_id', '=', 'businesses.id');
        if ($from_date != null && $to_date != null) {
            $codOrders = $codOrders->where('order_date', '>=', "$from_date");
            $codOrders = $codOrders->where('order_date', '<=', "$to_date");
        }
        $codOrders = $codOrders->where('payment_method', '=', 'cod');
        $codOrders = $codOrders->whereNotNull('online_orders.business_id');
        $codOrders = $codOrders->whereIn('order_status', ['Delivered', 'Failed']);
        if ($service == 'connect') {
            $codOrders = $codOrders->where('businesses.connect_service', '=', 1);
        }
        if ($service == 'normal') {
            $codOrders = $codOrders->where('businesses.connect_service', '=', 0);
        }
        if ($business_id != null) {
            $codOrders = $codOrders->where('online_orders.business_id', '=', $business_id);
        }
        $codOrders = $codOrders->get();
        if (!$codOrders->isEmpty()) {
            $data['cod_report'] = $this->createReport($codOrders);
        }

        $merchantTransactions = StripeTransaction::select('stripe_transactions.*', 'stripe_transactions.reporting_category as online_reporting_category', 'businesses.name as business_name');
        $merchantTransactions = $merchantTransactions->leftJoin('businesses', 'stripe_transactions.business_id', '=', 'businesses.id');
        if ($from_date != null && $to_date != null) {
            $merchantTransactions = $merchantTransactions->where('created', '>=', strtotime($from_date));
            $merchantTransactions = $merchantTransactions->where('created', '<=', strtotime($to_date));
        }
        $merchantTransactions = $merchantTransactions->where('service', '!=', 'Online');
        $merchantTransactions = $merchantTransactions->whereNotNull('stripe_transactions.business_id');
        // $merchantTransactions = $merchantTransactions->whereIn('reporting_category', ['charge', 'refund']);
        if ($service == 'connect') {
            $merchantTransactions = $merchantTransactions->where('businesses.connect_service', '=', 1);
        }
        if ($service == 'normal') {
            $merchantTransactions = $merchantTransactions->where('businesses.connect_service', '=', 0);
        }
        if ($business_id != null) {
            $merchantTransactions = $merchantTransactions->where('stripe_transactions.business_id', '=', $business_id);
        }
        $merchantTransactions = $merchantTransactions->get();
        if (!$merchantTransactions->isEmpty()) {
            $data['merchant_report'] = $this->createFinancialMerchantReport($merchantTransactions);
        }

        return $data;
    }

    public function mergeAllTransactionsAndSort($data)
    {
        $gross_total = 0;
        $total_fee = 0;
        $total_cash_payout = 0;
        $total_bank_payout = 0;
        $net_total = 0;
        $total_charge_back_fee = 0;
        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_refund_transactions = 0;
        $total_service_charge = 0;
        $misc_charge = ($data['misc_charge']);
        $weekly_fee = ($data['weekly_fee']);
        $monthly_fee = ($data['monthly_fee']);
        $total_provider_fee = 0;
        $total_business_fee = 0.0;
        $total_reseller_fee = 0.0;
        $all_transactions = [];

        $report['total_business_fee'] = $this->convertToDecimal($total_business_fee);
        $report['total_reseller_fee'] = $this->convertToDecimal($total_reseller_fee);
        $report['total_provider_fee'] = $this->convertToDecimal($total_provider_fee);

        if (isset($data['cod_report'])) {
            $gross_total = $gross_total + $data['cod_report']['gross_total'];
            $total_fee = $total_fee + $data['cod_report']['total_fee'];
            $total_provider_fee = $total_provider_fee + $data['cod_report']['total_provider_fee'];
            $total_business_fee = $total_business_fee + $data['cod_report']['total_business_fee'];
            $total_reseller_fee = $total_reseller_fee + $data['cod_report']['total_reseller_fee'];
            $total_cash_payout = $total_cash_payout + $data['cod_report']['cash_payout'];
            $total_bank_payout = $total_bank_payout + $data['cod_report']['bank_payout'];
            $net_total = $net_total + $data['cod_report']['net_total'];
            // $total_service_charge = $total_service_charge + $data['cod_report']['total_service_charge'];
            $total_refund_transactions = $total_refund_transactions + $data['cod_report']['total_refund_transactions'];
            $temp_transactions = $data['cod_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['card_report'])) {
            $gross_total = $gross_total + $data['card_report']['gross_total'];
            $total_fee = $total_fee + $data['card_report']['total_fee'];
            $total_provider_fee = $total_provider_fee + $data['card_report']['total_provider_fee'];
            $total_business_fee = $total_business_fee + $data['card_report']['total_business_fee'];
            $total_reseller_fee = $total_reseller_fee + $data['card_report']['total_reseller_fee'];
            $total_cash_payout = $total_cash_payout + $data['card_report']['cash_payout'];
            $total_bank_payout = $total_bank_payout + $data['card_report']['bank_payout'];
            $net_total = $net_total + $data['card_report']['net_total'];
            // $total_service_charge = $total_service_charge + $data['card_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['card_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['card_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['card_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['card_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['card_report']['total_refund_transactions'];
            $temp_transactions = $data['card_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['wallet_report'])) {
            $gross_total = $gross_total + $data['wallet_report']['gross_total'];
            $total_fee = $total_fee + $data['wallet_report']['total_fee'];
            $total_provider_fee = $total_provider_fee + $data['wallet_report']['total_provider_fee'];
            $total_business_fee = $total_business_fee + $data['wallet_report']['total_business_fee'];
            $total_reseller_fee = $total_reseller_fee + $data['wallet_report']['total_reseller_fee'];
            $total_cash_payout = $total_cash_payout + $data['wallet_report']['cash_payout'];
            $total_bank_payout = $total_bank_payout + $data['wallet_report']['bank_payout'];
            $net_total = $net_total + $data['wallet_report']['net_total'];
            // $total_service_charge = $total_service_charge + $data['wallet_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['wallet_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['wallet_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['wallet_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['wallet_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['wallet_report']['total_refund_transactions'];
            $temp_transactions = $data['wallet_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['paypal_report'])) {
            $gross_total = $gross_total + $data['paypal_report']['gross_total'];
            $total_fee = $total_fee + $data['paypal_report']['total_fee'];
            $total_provider_fee = $total_provider_fee + $data['paypal_report']['total_provider_fee'];
            $total_business_fee = $total_business_fee + $data['paypal_report']['total_business_fee'];
            $total_reseller_fee = $total_reseller_fee + $data['paypal_report']['total_reseller_fee'];
            $total_cash_payout = $total_cash_payout + $data['paypal_report']['cash_payout'];
            $total_bank_payout = $total_bank_payout + $data['paypal_report']['bank_payout'];
            $net_total = $net_total + $data['paypal_report']['net_total'];
            // $total_service_charge = $total_service_charge + $data['paypal_report']['total_service_charge'];
            $temp_transactions = $data['paypal_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['paypal_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['paypal_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['paypal_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['merchant_report'])) {
            $gross_total = $gross_total + $data['merchant_report']['gross_total'];
            $total_fee = $total_fee + $data['merchant_report']['total_fee'];
            $total_provider_fee = $total_provider_fee + $data['merchant_report']['total_provider_fee'];
            $total_business_fee = $total_business_fee + $data['merchant_report']['total_business_fee'];
            $total_reseller_fee = $total_reseller_fee + $data['merchant_report']['total_reseller_fee'];
            $total_cash_payout = $total_cash_payout + $data['merchant_report']['cash_payout'];
            $total_bank_payout = $total_bank_payout + $data['merchant_report']['bank_payout'];
            $net_total = $net_total + $data['merchant_report']['net_total'];
            $total_charge_back_fee = $total_charge_back_fee + $data['merchant_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['merchant_report']['total_charge_back_transactions'];
            $temp_transactions = $data['merchant_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['merchant_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['merchant_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['merchant_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        usort($all_transactions, $this->date_compare('order_date'));
        return [
            'from_date' => $data['from_date'],
            'to_date' => $data['to_date'],
            'gross_total' => $this->convertToDecimal($gross_total),
            'gross_fee' => $this->convertToDecimal($total_fee),
            'gross_refund' => $total_refund_transactions,
            'gross_provider_fee' => $this->convertToDecimal($total_provider_fee),
            'gross_business_fee' => $this->convertToDecimal($total_business_fee),
            'gross_reseller_fee' => $this->convertToDecimal($total_reseller_fee),
            'cash_payout' => $this->convertToDecimal($total_cash_payout),
            'bank_payout' => $this->convertToDecimal($total_bank_payout),
            'net_fee' => $this->convertToDecimal($total_fee + $misc_charge + $monthly_fee + $weekly_fee + $total_service_charge),
            'net_total' => $this->convertToDecimal($net_total),
            'total_earning' => $this->convertToDecimal($total_business_fee - ($total_provider_fee + $total_reseller_fee)),
            'transactions' => $all_transactions,
            // 'net_total' => $this->convertToDecimal($net_total - $misc_charge - $monthly_fee - $weekly_fee - $total_service_charge),
            // 'payout_mode' => $data['payout_mode'],
            // 'payout_method' => $data['payout_method'],
            // 'payout_account' => $data['payout_account'],
            // 'transaction_type' => $data['transaction_type'],
            // 'account_type' => $data['account_type'],
            // 'payout_comment' => $data['payout_comment'],
            // 'misc_charge' => $this->convertToDecimal($misc_charge),
            // 'weekly_fee' => $this->convertToDecimal($weekly_fee),
            // 'monthly_fee' => $this->convertToDecimal($monthly_fee),
            // 'total_charge_back_fee' => $this->convertToDecimal($total_charge_back_fee),
            // 'total_charge_back_transactions' => $this->convertToDecimal($total_charge_back_transactions),
            // 'total_non_domestic_transactions' => $this->convertToDecimal($total_non_domestic_transactions),
            // 'total_non_domestic_fee' => $this->convertToDecimal($total_non_domestic_fee),
            // 'total_service_charge' =>  $this->convertToDecimal($total_service_charge),
            // 'total_refund_transactions' => $total_refund_transactions,
            // 'business' => $data['business']
        ];
    }

    public function userCheck($business_id, $mehtod_business_id)
    {
        if ($business_id != $mehtod_business_id) {
            Utils::error('User not authenticated.', 404);
        }

        return [
            'message' => 'data fetch successfully',
        ];
    }

    public function date_compare($key)
    {
        return function ($a, $b) use ($key) {
            $t1 = strtotime($a[$key]);
            $t2 = strtotime($b[$key]);
            return $t2 - $t1;
        };
    }

    public function convertToTitleCase($input)
    {
        $words = explode(' ', $input);
        $titleCasedWords = [];
        foreach ($words as $word) {
            $titleCasedWords[] = ucfirst(strtolower($word));
        }
        $titleCasedString = implode(' ', $titleCasedWords);
        return $titleCasedString;
    }
}
