<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\FactoryResetLog;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class FactoryResetLogController extends Controller
{
    /**
     * List factory-logs.
     * GET /api/factory-logs
     */
    public function index(Request $req)
    {
        $factory_reset_logs = FactoryResetLog::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $factory_reset_logs->whereRaw('(device_id LIKE ? OR sync_mode LIKE ? OR device_name LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }
        if ($business_id = $req->query('business_id', false)) {
            $factory_reset_logs->where('business_id', $business_id);
        }
        if ($user_id = $req->query('user_id', false)) {
            $factory_reset_logs->where('user_id', $user_id);
        }
        if ($req->query('nopaginate', false) == '1') {
            $factory_reset_logs = $factory_reset_logs->get();
        } else {
            $factory_reset_logs = $factory_reset_logs->paginate($req->query('per_page', 20));
        }

        if ($factory_reset_logs->isEmpty()) Utils::error('No logs found!', 404);


        return ($factory_reset_logs);
    }

    /**
     * Create factory-log.
     * POST /api/factory-logs
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'exists:businesses',
            'user_id' => 'exists:users',
            'device_id' => 'required|string',
            'device_name' => 'required|string',
            'sync_mode' => 'required|string',
        ]);

        // create factory_reset_log
        $factory_reset_log = new FactoryResetLog;
        $factory_reset_log->fill($req->all());
        $factory_reset_log->save();

        return $factory_reset_log;
    }

    /**
     * Show factory-log.
     * GET /api/factory-logs/{id}
     */
    public function show(Request $req, $id)
    {
        $factory_reset_log = FactoryResetLog::find($id);
        if (empty($factory_reset_log)) Utils::error('Log not found!', 404);
        return $factory_reset_log;
    }

    /**
     * Update factory-log.
     * POST /api/factory-logs/{id}
     */
    public function update(Request $req, $id)
    {
        $factory_reset_log = FactoryResetLog::find($id);
        if (empty($factory_reset_log)) Utils::error('Log not found!', 404);

        $this->validate($req, [
            'business_id' => 'exists:businesses',
            'user_id' => 'exists:users',
            'device_id' => 'required|string',
            'device_name' => 'required|string',
            'sync_mode' => 'required|string',
        ]);

        // update factory_reset_log
        if ($req->has('business_id')) $factory_reset_log->business_id = $req->input('business_id');
        if ($req->has('user_id')) $factory_reset_log->user_id = $req->input('user_id');
        if ($req->has('device_id')) $factory_reset_log->device_id = $req->input('device_id');
        if ($req->has('device_name')) $factory_reset_log->device_name = $req->input('device_name');
        if ($req->has('sync_mode')) $factory_reset_log->sync_mode = $req->input('sync_mode');
        $factory_reset_log->save();

        return $factory_reset_log;
    }


    /**
     * Delete factory-log by id.
     * DELETE /api/factory-logs/{id}
     */
    public function delete(Request $req, $id)
    {
        $factory_reset_log = FactoryResetLog::find($id);
        if (empty($factory_reset_log)) Utils::error('Log not found!', 404);
        $factory_reset_log->delete();
        return $factory_reset_log;
    }

    /**
     * Delete factory-log by id.
     * DELETE /api/factory-logs/bulk-delete
     */
    public function bulk_delete(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);

        $factory_reset_logs = FactoryResetLog::whereIn('id', $req->input('ids'))->delete();
        return $req;
    }
}
