<?php

namespace App\Http\Controllers;

use App\Faq;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class FaqController extends Controller
{
    /**
     * List faqs.
     * GET /api/faqs
     */
    public function index(Request $req)
    {
        $faqs = Faq::orderByRaw('id ASC');

        if ($query = $req->query('query', false)) {
            $faqs->whereRaw('(question LIKE ? OR  answer LIKE ? OR  page LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($prefilled = $req->query('prefilled', false)) {
            $faqs->whereNull('parent_id');
        }

        if ($page_type = $req->query('page_type', false)) {
            $faqs->where('page_type', $page_type);
        }

        if ($disabled = $req->query('disabled', false)) {
            $faqs->where('disabled', $disabled);
        }

        if ($req->query('nopaginate', false) == '1') {
            $faqs = $faqs->get();
        } else {
            $faqs = $faqs->paginate($req->query('per_page', 20));
        }

        if ($faqs->isEmpty()) Utils::error('No faqs found!', 404);

        if (!$prefilled = $req->query('prefilled', false)) {
            $faqs->each(function ($items) {
                $items->makeHidden('children');
            });
        }

        return $faqs;
    }

    /**
     * List faqs.
     * GET /api/faqs
     */
    public function home(Request $req)
    {
        $faqs = Faq::orderByRaw('id ASC');

        if ($query = $req->query('query', false)) {
            $faqs->whereRaw('(question LIKE ? OR  answer LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($page = $req->query('page', false)) {
            // $faqs->whereNotNull('parent_id');
            $faqs->where('page', $page);
        }

        if ($disabled = $req->query('disabled', false)) {
            $faqs->where('disabled', $disabled);
        }

        if ($req->query('nopaginate', false) == '1') {
            $faqs = $faqs->get();
        } else {
            $faqs = $faqs->paginate($req->query('per_page', 20));
        }

        if ($faqs->isEmpty()) Utils::error('No faqs found!', 404);

        return $faqs;
    }

    /**
     * Create faq.
     * POST /api/faqs
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'parent_id' => 'exists:faqs,id',
            'question' => 'string',
            'answer' => 'string',
            'page' => 'string',
        ]);

        // create faq
        $faq = new Faq;
        $faq->fill($req->all());
        $faq->save();

        return $faq;
    }

    /**
     * Show faq.
     * GET /api/faqs/{id}
     */
    public function show(Request $req, $id)
    {
        $faq = Faq::find($id);
        if (empty($faq)) Utils::error('Faq not found!', 404);
        return $faq;
    }

    /**
     * Update faq.
     * POST /api/faqs/{id}
     */
    public function update(Request $req, $id)
    {
        $faq = Faq::find($id);
        if (empty($faq)) Utils::error('Faq not found!', 404);

        $this->validate($req, [
            'parent_id' => 'exists:faqs,id',
            'question' => 'string',
            'answer' => 'string',
            'page' => 'string',
        ]);

        // update faq
        if ($req->has('parent_id')) $faq->parent_id = $req->input('parent_id');
        if ($req->has('question')) $faq->question = $req->input('question');
        if ($req->has('answer')) $faq->answer = $req->input('answer');
        if ($req->has('page')) $faq->page = $req->input('page');

        $faq->save();

        return $faq;
    }

    /**
     * Delete faq by id.
     * DELETE /api/faqs/{id}
     */
    public function delete(Request $req, $id)
    {
        $faq = Faq::find($id);
        if (empty($faq)) Utils::error('Faq not found!', 404);
        $faq->delete();

        Faq::where('parent_id', $id)->delete();

        return $faq;
    }

    /**
     * Enable faq.
     * POST /epos/faqs/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $faq = Faq::find($id);
        if (empty($faq)) Utils::error('Faq not found!', 404);
        $faq->disabled = 0;
        $faq->save();

        Faq::where('parent_id', $faq->id)->update(['disabled' => 0]);

        return $faq;
    }

    /**
     * Disable faq.
     * POST /epos/faqs/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $faq = Faq::find($id);
        if (empty($faq)) Utils::error('Faq not found!', 404);
        $faq->disabled = 1;
        $faq->save();

        Faq::where('parent_id', $faq->id)->update(['disabled' => 1]);

        return $faq;
    }
}
