<?php

namespace App\Http\Controllers;

use App\Business;
use App\BusinessCardReader;
use App\CardBrand;
use App\CardBrandCommission;
use App\Country;
use App\CountryCommission;
use App\Helpers\ImageHelper;
use App\HardwareOrder;
use App\HardwareOrderItem;
use App\HardwareProduct;
use App\Helpers\Auth;
use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\User;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Stripe\StripeClient;

class HardwareOrderController extends Controller
{
   /**
    * List hardware-orders.
    * GET /api/hardware-orders
    */
   public function index(Request $req)
   {
      $orders = HardwareOrder::selectRaw('hardware_orders.*,users.first_name as first_name')->leftJoin('users', 'users.id', '=', 'customer_id')->orderByRaw('hardware_orders.id DESC');

      if ($query = $req->query('query', false)) {
         $orders->whereRaw('(hardware_orders.total LIKE ?  )', ["%$query%"]);
      }

      if ($customer_id =  $req->query('customer_id', false)) {
         $orders->where('customer_id', $customer_id);
      }

      if ($req->has('from_date')) {
         $from_date = $req->input('from_date') . " 00:00:00";
         $to_date = $req->input('to_date') . " 23:59:59";

         $orders = $orders->where('hardware_orders.created_at', '>=', $from_date);
         $orders = $orders->where('hardware_orders.created_at', '<=', $to_date);
      }

      if ($req->query('nopaginate', false) == '1') {
         $orders = $orders->get();
      } else {
         $orders = $orders->paginate($req->query('per_page', 20));
      }

      if ($orders->isEmpty()) Utils::error('No orders found!', 404);

      $orders->append('order_items');
      $orders->append('customer');
      return $orders;
   }

   /**
    * Create hardware-order.
    * POST /api/hardware-orders
    */
   public function create(Request $req)
   {
      if (!is_array($req->order_items))
         $req->merge(['order_items' => (json_decode($req->order_items, true))]);

      $this->validate($req, [
         'customer_id' => 'required|exists:users,id',
         'flat_no' => 'string',
         'address' => 'required|string',
         'second_line' => 'string',
         'city' => 'string',
         'zipcode' => 'required|string',
         'sub_total' => 'required|numeric',
         'tax' => 'numeric',
         'voucher_code' => 'string',
         'offer_amount' => 'numeric',
         'total' => 'required|numeric',
         'txn_id' => 'string',
         'delivery_date' => 'required|date',
         'notes' => 'string',
         'status' => 'required|in:enum,Pending,Accepted,Rejected,Delivered',
      ]);

      // create hardware-order
      $order = new HardwareOrder;
      $order->fill($req->all());
      $order->save();

      // Items Add On Order
      $oldItems = HardwareOrderItem::where('customer_id', $order->customer_id)->where('order_id', 0)->first();
      if (!empty($oldItems)) {
         $dbItems = HardwareOrderItem::where('customer_id', $order->customer_id)->where('order_id', 0)->delete();
      }
      $items = ($req->input('order_items'));
      if (!empty($items)) {
         foreach ($items as $item) {
            if ($item['product_id']) {
               $productItems = HardwareProduct::where('id', $item['product_id'])->first();
               if ($productItems->stock_quantity >= $item['quantity']) {
                  $productItems->stock_quantity = $productItems->stock_quantity - $item['quantity'];
               } else {
                  $productItems->stock_quantity = 0;
               }
               if ($productItems->is_exclude == '1') {
                  $productItems->tax = $item['total'] * $productItems->tax_amount / 100;
               }
               $productItems->save();
            }
            $orderItem = new HardwareOrderItem((array)$item);
            $orderItem->order_id = $order->id;
            $orderItem->save();
         }
      }

      $this->orderMail($order->id);

      $order->append('order_items');
      $order->append('customer');
      return $order;
   }

   /**
    * Show hardware-order.
    * GET /api/hardware-orders/{id}
    */
   public function show(Request $req, $id)
   {
      $order = HardwareOrder::find($id);
      $order->append('order_items');
      $order->append('customer');
      if (empty($order)) Utils::error('Order not found!', 404);
      return $order;
   }

   /**
    * Update hardware-order.
    * POST /api/hardware-orders/{id}
    */
   public function update(Request $req, $id)
   {
      $order = HardwareOrder::find($id);
      if (empty($order)) Utils::error('Order not found!', 404);

      $this->validate($req, [
         'customer_id' => 'required|exists:users,id',
         'flat_no' => 'string',
         'address' => 'required|string',
         'second_line' => 'string',
         'city' => 'string',
         'zipcode' => 'string',
         'sub_total' => 'required|numeric',
         'tax' => 'numeric',
         'voucher_code' => 'string',
         'offer_amount' => 'numeric',
         'total' => 'required|numeric',
         'txn_id' => 'string',
         'delivery_date' => 'required|date',
         'notes' => 'string',
         'status' => 'required|in:enum,Pending,Accepted,Rejected,Delivered',
      ]);

      // update hardware-order
      if ($req->has('customer_id')) $order->customer_id = $req->input('customer_id');
      if ($req->has('flat_no')) $order->flat_no = $req->input('flat_no');
      if ($req->has('address')) $order->address = $req->input('address');
      if ($req->has('second_line')) $order->second_line = $req->input('second_line');
      if ($req->has('city')) $order->city = $req->input('city');
      if ($req->has('zipcode')) $order->zipcode = $req->input('zipcode');
      if ($req->has('sub_total')) $order->sub_total = $req->input('sub_total');
      if ($req->has('tax')) $order->tax = $req->input('tax');
      if ($req->has('voucher_code')) $order->voucher_code = $req->input('voucher_code');
      if ($req->has('offer_amount')) $order->offer_amount = $req->input('offer_amount');
      if ($req->has('total')) $order->total = $req->input('total');
      if ($req->has('delivery_date')) $order->delivery_date = $req->input('delivery_date');
      if ($req->has('notes')) $order->notes = $req->input('notes');
      if ($req->has('txn_id')) $order->txn_id = $req->input('txn_id');
      if ($req->has('status')) $order->status = $req->input('status');

      if ($req->has('status') != $order->status) {
         $this->orderMail($order->id);
      }

      $order->save();

      $order->append('order_items');
      $order->append('customer');
      return $order;
   }

   /**
    * Delete hardware-order by id.
    * DELETE /api/hardware-orders/{id}
    */
   public function delete(Request $req, $id)
   {
      $order = HardwareOrder::find($id);
      if (empty($order)) Utils::error('Order not found!', 404);
      $order->delete();
      return $order;
   }


   public function payment_intent(Request $req)
   {
      $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
      if ($mode == 'test') {
         $stripeKeyColoumns = 'connect_stripe_private_key_test';
      }
      if ($mode == 'live') {
         $stripeKeyColoumns = 'connect_stripe_private_key_live';
      }

      $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
      $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

      $this->validate($req, [
         'amount' => 'required|numeric|gt:0',  // multiplied by 100
         's_terminal_id' => 'string',
         'business_id' => 'string',
         'customer_id' => 'string',
         'first_name' => 'string',
         'customer_email' => 'string|email|required_with:first_name',
         'auto_confirm' => 'boolean'
      ]);

      $statement_descriptor = 'Tiffintom Ltd';
      $statement_description = 'Tiffintom Ltd';
      $business_id = $req->input('business_id');
      $terminalId = $req->input('s_terminal_id');
      $autoConfirm = $req->input('auto_confirm', false);

      $stripe_account_id = null;

      $humanAmount = $req->input('amount') / 100;

      if ($req->has('currency')) {
         $currency = $req->input('currency');
      } else {
         $currency = 'GBP';
      }

      $intentParams = [
         'amount' => (int) ($humanAmount * 100),
         'currency' => $currency,
      ];
      if ($terminalId != null) {
         $intentParams['payment_method_types'] = ['card_present'];
         $intentParams['capture_method'] = 'manual';
      }
      if ($autoConfirm) {
         $intentParams['confirm'] = true;
      } else {
         $intentParams['confirmation_method'] = 'automatic';
      }
      if ($req->has('payment_method_id')) {
         $intentParams['payment_method'] = $req->input('payment_method_id');
      }
      if ($statement_descriptor != null) {
         $intentParams['statement_descriptor'] = $statement_descriptor;
         $intentParams['description'] = $statement_description;
      }
      if ($req->input('customer_email') != null) {
         $customerParams['email'] = $req->input('customer_email');
         if ($req->input('first_name') != null) {
            $customerParams['name'] = $req->input('first_name');
         }
         $customer = $stripe->customers->create($customerParams);
         $intentParams['customer'] = $customer->id;
         $intentParams['save_payment_method'] = true;
      }

      // Default Connect Account Key
      $stripe_account_id = (new SiteSettingController())->findByKey('default_connect_account')->value;
      if ($stripe_account_id != null) {
         $intentParams['transfer_data'] = [
            'destination' => $stripe_account_id
         ];
      }

      $intent = $stripe->paymentIntents->create($intentParams);
      return ['payment_intent_id' => $intent->client_secret, 'payment_intent' => $intent, "status" => $intent->status];
   }

   public function calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type)
   {
      $payment_method_fees = 0;
      $orderTypeWiseCommission = 0;
      $orderTypeWiseServiceCharge = 0;
      $orderBasedOrTransactionWiseCharge = 0;

      $cardAuthFee = 0;
      $cardProcessingFee = 0;
      $fee = 0;
      if ($transaction_type == 'online') {

         $payment_method_fees  = $this->calculateFee($humanAmount, $business->business_commissions->online_card_fee, $business->business_commissions->online_card_fee_type);

         if ($order_type == 'dinein') {
            $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->dinein_commission_fee, $business->business_commissions->dinein_commission_fee_type);
            $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->dinein_service_fee, $business->business_commissions->dinein_service_fee_type);
         } else {
            $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->online_commission_fee, $business->business_commissions->online_commission_fee_type);
            $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->online_service_fee, $business->business_commissions->online_service_fee_type);
         }
         if ($business->business_commissions->order_charge_type == 'order_based') {
            $orderBasedOrTransactionWiseCharge = $this->calculateFee($humanAmount, $business->business_commissions->per_order_charge, $business->business_commissions->per_order_charge_type);
         } else {
            $orderBasedOrTransactionWiseCharge  = $this->calculateFee($humanAmount, $business->business_commissions->card_charge, $business->business_commissions->card_charge_type);
         }

         $fee = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $cardAuthFee + $cardProcessingFee);
      }
      if ($transaction_type == 'merchant') {
         $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_auth_fee, $business->business_commissions->merchant_card_auth_fee_type);
         $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_processing_fee, $business->business_commissions->merchant_card_processing_fee_type);

         $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
      }
      if ($transaction_type == 'connect') {
         $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_auth_fee, $business->business_commissions->connect_card_auth_fee_type);
         $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_processing_fee, $business->business_commissions->connect_card_processing_fee_type);

         $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
      }
      return $fee;
   }

   public function calculateFee($humanAmount, $fee, $type)
   {
      $calculatedFee = 0.00;
      if ($type == 'percentage') {
         if ($fee != 0)
            $calculatedFee = ($humanAmount * $fee / 100);
      } else {
         $calculatedFee = $fee;
      }
      return  $this->convertToDecimal($calculatedFee);
   }

   public function convertToDecimal($value)
   {
      return floatval(number_format($value, 2, '.', ''));
   }

   public function orderMail($id)
   {
      $order = HardwareOrder::find($id);
      if (empty($order)) Utils::error("order not found", 404);
      $order->append('order_items');

      $user = User::find($order->customer_id);
      if (empty($user)) Utils::error("No user found", 404);

      $siteSettings = TiffintomSiteSetting::find('1');
      if (empty($siteSettings)) Utils::error("No site-settings found", 404);

      $voucherDetails = '';
      $orderDescription = '';
      $tax_amount = '';

      if ($order['tax_amount'] > 0) {
         $tax_amount = ' <tr>
                          <td align="left" style="">
                             <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                <tr>
                                   <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                      Tax(' . $order['tax_percentage'] . ') %
                                   </td>
                                   <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                      ' . $siteSettings["site_currency"] . ' ' . number_format($order['tax_amount'], 2) . '
                                   </td>
                                </tr>
                             </table>
                          </td>
                       </tr>';
      }

      if ($order['voucher_amount'] > 0) {
         $voucherDetails = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Offer(' . $order['voucher_percentage'] . ') %
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['voucher_amount'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
      }

      $cartDetails = '';

      if (!empty($order['order_items'])) {
         foreach ($order['order_items'] as $cKey => $cValue) {
            $menuName = $cValue['product_name'];
            $cartDetails .= '
                <tr>
                    <td width="10%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . ($cKey + 1) . '</td>
                    <td width="45%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . $menuName . '</td>
                    <td width="10%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . $cValue['quantity'] . '</td>
                    <td width="15%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">
                        ' . $siteSettings["site_currency"] . ' ' . number_format($cValue['product_price'], 2) . '
                    </td>
                    <td width="20%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">
                        ' . $siteSettings["site_currency"] . ' ' . number_format($cValue['total'], 2) . '
                    </td>
                </tr>';
         }
      }

      if ($order['order_description'] != "") {
         $orderDescription = '<tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                           <tr>
                              <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Order Notes : ' . $order['notes'] . '</span>
                                 
                              </td>                              
                           </tr>
                        </table>
                     </td>
                  </tr>';
      }
      $statuss =  '';
      $comme = '';
      $statusss = '';
      if ($order['status'] == "Accepted") {
         $statuss .=  '  <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                           <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $user['first_name'] . '</strong></span>
                                    Thank you. your <span style="color:#000;font-weight:bold;">order ID ' . $order['id'] . '</span> has been Accepted.
                                    <br>
                                    Those lovely people should start cooking your food right about ... now.
                                 </p>
                              </td>
                           </tr>';
         $customerSubject = 'Order Accepted - ' . $order['id'];
      } else if ($order['status'] == "Delivered") {
         $statuss .= '   
                            <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                            <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $user['first_name'] . '</strong></span>
                                         Thank you for Order.Order number ' . $order['id'] . ' has been Delivered.
                                 </p>
                              </td>
                           </tr>';
         $customerSubject = 'Order Delivered - ' . $order['id'];
      } else if ($order['status'] == "Failed") {
         $statuss .= '   <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $user['first_name'] . '</strong></span>
                                         We are sorry for the inconvience caused,your Order ' . $order['id'] . ' has been Failed for the reason ' . $order['failed_reason'] . '.
                                 </p>
                              </td>
                           </tr>';
         $customerSubject = 'Order Reject - ' . $order['id'];

         $statusss .= ' <tr>
                             <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                                   <tr>
                                      <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                         <span style="font-weight: 700; display:block;">Failed Reason : ' . $order['failed_reason'] . '</span>
                                         
                                      </td>                              
                                   </tr>
                                </table>
                             </td>
                          </tr> ';
      } else {
         if ($order['status'] == "Pending") {
            $customerSubject = 'Order Placed Successful - ' . $order['id'];

            $statuss = '<tr>
                <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                   <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                      <span style="display:block;">Dear <strong style=" color:#000;">' . $user['first_name'] . '</strong></span>
                      Thank you for your order with us.Your order details below for the order ID <span style="color:#000;font-weight:bold;">order ID ' . $order['id'] . '</span>.
                   </p>
                </td>
             </tr>';
         }
      }

      $html = '
         <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
               <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                  <tr>
                     <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                        <div style="display:inline-block; vertical-align:middle; text-align:center">
                           <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                              <tr>
                                 <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                    <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                 </td>
                              </tr>
                           </table>
                        </div>
                     </td>
                  </tr>
                  <tr>
                     <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                           ' . $statuss . '
                           <tr>
                              <td align="left" style="padding-top: 20px;">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                   <td  width="10%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">S.No</td>
                                   <td width="45%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">Menu Name</td>
                                   <td width="10%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 700; line-height: 24px; padding: 10px;">Qty</td>
                                   <td width="15%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">
                                      Price
                                   </td>
                                   <td width="20%" align="right" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">
                                      Total Price
                                   </td>
                                </tr>
                                    ' . $cartDetails . '                                 
                                 </table>
                              </td>
                           </tr>   
                                 
                           
                           <tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Sub Total
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['sub_total'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>
                           ' . $tax_amount . '
                           ' . $voucherDetails . '
                           <tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 600; line-height: 24px; padding: 10px; border-top: 3px solid #eeeeee; border-bottom: 3px solid #eeeeee;">
                                          TOTAL
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 600; line-height: 24px; padding: 10px; border-top: 3px solid #eeeeee; border-bottom: 3px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['total'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>
                  <tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                           <tr>
                              <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Order Number/ID : ' . $order['id'] . '</span>
                                 <span style="font-weight: 700; display:block;">Payment method : Online</span>
                              </td>
                              <td align="right" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Payment status : Paid </span>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>
                  ' . $comme . '
                  
                  ' . $orderDescription . '
                  
                  ' . $statusss . '
                    <tr>   
                        	<td bgcolor="#fff" >
                        		<table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                        			<tr>
                        				<td style="padding: 30px 30px; text-align: right;">
                        					    <table align="left" style="text-align: left;">
                            						<tr>
                            							<td>
                            								   <p style="color:#000;"> Get The App : </p> 
                            							</td>
                            								<td width="10">&nbsp;</td>
                            							<td>
                            								<a>
                            								<img src="' . env('APP_URL') . 'images/playstore.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                            							</td>
                            								<td width="10">&nbsp;</td>
                            							<td>
                            								<a>
                            								<img src="' . env('APP_URL') . 'images/appstore.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                            							</td>
                            								<td width="10">&nbsp;</td>
                        					        </tr>
                            						
                            					</table>
                            					<table align="right" style="text-align: right;">
                        						<tr>
                        						    <td>
                        								    <p style="color:#000;">Follow Us : </p>
                        							</td>
                        								<td width="10">&nbsp;</td>
                        							<td>
                        								<a>
                        								<img src="' . env('APP_URL') . 'images/facebook.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td>
                        								<td width="10">&nbsp;</td>
                        							<td>
                        								<a>
                        								<img src="' . env('APP_URL') . 'images/twitter.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td>
                        								<td width="10">&nbsp;</td>
                        							<td>
                        								<a>
                        								<img src="' . env('APP_URL') . 'images/linkedin.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td><td width="10">&nbsp;</td>
                        							<td>
                        								<a>
                        								<img src="' . env('APP_URL') . 'images/Instragram.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td><td width="10">&nbsp;</td>
                        						</tr>
                        					</table>
                        				</td>
                        			</tr>
                        		</table>
                        	</td>
                        </tr>
                  <tr>
                     <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
                  </tr>
               </table>
            </td>
         </tr>';

      // User side mail
      $email['body'] = $html;
      $email['subject'] = $customerSubject;
      $email['mail_type'] = $siteSettings->mail_option;
      $email['to'] = $user->contact_emails;
      $email['from'] = $siteSettings->admin_email;
      (new MailHelper())->sendMail($email);

      return 'Mail Send Successfully';
   }
}
