<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\HardwareOrderItem;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class HardwareOrderItemController extends Controller
{
    /**
     * List hardware-order-items.
     * GET /api/hardware-order-items
     */
    public function index(Request $req)
    {
        $order_items = HardwareOrderItem::orderBy('id', 'DESC');

        if ($order_id = $req->query("order_id", false)) {
            $order_items->where('order_id', $order_id);
        }

        if ($customer_id = $req->query("customer_id", false)) {
            $order_items->where('customer_id', $customer_id);
            $order_items->where('order_id', '0');
        }

        if ($req->query('nopaginate', false) == '1') {
            $order_items = $order_items->get();
        } else {
            $order_items = $order_items->paginate($req->query('per_page', 20));
        }

        if ($order_items->isEmpty()) Utils::error('No order items found!', 404);

        return ($order_items);
    }

    /**
     * Create hardware-order-item.
     * POST /api/hardware-order-items
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'string',
            'customer_id' => 'required|exists:users,id',
            'product_name' => 'required|string',
            'product_price' => 'required|numeric',
            'total' => 'required|numeric',
            'quantity' => 'numeric',
        ]);

        $itemObj = HardwareOrderItem::where('product_name', $req->input('product_name'))->where('customer_id', $req->input('customer_id'))->where('order_id', '0')->first();
        if (empty($itemObj)) {
            // create hardware-order-item
            $order_item = new HardwareOrderItem;
            $order_item->fill($req->all());
            $order_item->save();
        } else {
            $itemObj->quantity = $itemObj->quantity + 1;
            $itemObj->product_price = $req->input('product_price');
            $itemObj->total = $itemObj->product_price * $itemObj->quantity;
            $itemObj->save();
        }

        $order_item = HardwareOrderItem::where('customer_id', $req->input('customer_id'))->where('order_id', 0)->orderBy('id', 'DESC')->get();
        if ($order_item->isEmpty()) Utils::error('No item found!', 404);
        return $order_item;
    }

    /**
     * Show hardware-order-item.
     * GET /api/hardware-order-items/{id}
     */
    public function show(Request $req, $id)
    {
        $order_item = HardwareOrderItem::find($id);
        if (empty($order_item)) Utils::error('Order items not found!', 404);
        return $order_item;
    }

    /**
     * Update hardware-order-item.
     * POST /api/hardware-order-items/{id}
     */
    public function update(Request $req, $id)
    {
        $order_item = HardwareOrderItem::find($id);
        if (empty($order_item)) Utils::error('Order items not found!', 404);

        $this->validate($req, [
            'order_id' => 'string',
            'customer_id' => 'required|exists:users,id',
            'product_name' => 'required|string',
            'product_price' => 'required|numeric',
            'total' => 'required|numeric',
            'quantity' => 'numeric',
        ]);

        // update hardware-order-item
        if ($req->has('order_id')) $order_item->order_id = $req->input('order_id');
        if ($req->has('customer_id')) $order_item->customer_id = $req->input('customer_id');
        if ($req->has('product_name')) $order_item->product_name = $req->input('product_name');
        if ($req->has('product_price')) $order_item->product_price = $req->input('product_price');
        if ($req->has('total')) $order_item->total = $req->input('total');
        if ($req->has('quantity')) $order_item->quantity = $req->input('quantity');
        $order_item->save();

        $order_item = HardwareOrderItem::where('customer_id', $req->input('customer_id'))->where('order_id', 0)->orderBy('id', 'DESC')->get();
        if ($order_item->isEmpty()) Utils::error('No item found!', 404);
        return $order_item;
    }

    /**
     * Delete hardware-order-item by id.
     * DELETE /api/hardware-order-items/{id}
     */
    public function delete(Request $req, $id)
    {
        $order_item = HardwareOrderItem::find($id);
        if (empty($order_item)) Utils::error('Order item not found!', 404);
        $order_item->delete();

        $carts = HardwareOrderItem::where('customer_id', $order_item->customer_id)->where('order_id', 0)->orderBy('id', 'DESC')->get();
        if ($carts->isEmpty()) Utils::error('Order item not found!', 404);

        return $order_item;
    }

    /**
     * Delete All cart.
     * DELETE /api/hardware-order-items//delete-all/{id}
     */
    public function delete_all(Request $req, $id)
    {
        $order_items = HardwareOrderItem::where('customer_id', $id)->where('order_id', 0)->delete();
        return ['message' => 'order items reset!'];
    }
}
