<?php

namespace App\Http\Controllers;

use App\HardwareProduct;
use App\Helpers\ImageHelper;
use App\HardwareProductImage;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class HardwareProductImageController extends Controller
{
    /**
     * List productImages.
     * GET /api/productImages
     */
    public function index(Request $req)
    {
        $productImages = HardwareProductImage::orderByRaw('priority DESC, created_at DESC');

        if ($query = $req->query('product_id', false)) {
            $productImages->where('product_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $productImages = $productImages->get();
        } else {
            $productImages = $productImages->paginate($req->query('per_page', 20));
        }

        if ($productImages->isEmpty()) Utils::error('No product images found!', 404);


        return ($productImages);
    }

    /**
     * Create productImage.
     * POST /api/hardware-products
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'product_id' => 'required|exists:product,id',
            'priority' => 'numeric',
            'image' => 'required|image'
        ]);

        // create productImage
        $productImage = new HardwareProductImage;
        $productImage->fill($req->all());
        if ($req->hasFile('image')) {
            $productImage->image = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_HARDWARE_PRODUCT);
        }
        $productImage->save();

        return $productImage;
    }

    /**
     * Show productImage.
     * GET /api/hardware-products/{id}
     */
    public function show(Request $req, $id)
    {
        $productImage = HardwareProductImage::find($id);
        if (empty($productImage)) Utils::error('Product image not found!', 404);
        return $productImage;
    }

    /**
     * Update productImage.
     * POST /api/hardware-products/{id}
     */
    public function update(Request $req, $id)
    {
        $productImage = HardwareProductImage::find($id);
        if (empty($productImage)) Utils::error('Product image not found!', 404);

        $this->validate($req, [
            'product_id' => 'exists:product,id',
            'priority' => 'numeric',
            'image' => 'image'
        ]);

        // update productImage
        if ($req->has('product_id')) $productImage->product_id = $req->input('product_id');
        if ($req->has('priority')) $productImage->priority = $req->input('priority');
        if ($req->hasFile('image')) {
            if ($productImage->image != null) ImageHelper::delete(ImageHelper::$TYPE_HARDWARE_PRODUCT, $productImage->image);
            $productImage->image = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_HARDWARE_PRODUCT);
        }
        $productImage->save();

        return $productImage;
    }


    /**
     * Delete productImage by id.
     * DELETE /api/hardware-products/{id}
     */
    public function delete(Request $req, $id)
    {
        $productImage = HardwareProductImage::find($id);
        if (empty($productImage)) Utils::error('Product image not found!', 404);
        $productImage->delete();
        if ($productImage->image != null) ImageHelper::delete(ImageHelper::$TYPE_HARDWARE_PRODUCT, $productImage->image);
        return $productImage;
    }
}
