<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\InvoiceItem;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class InvoiceItemController extends Controller
{
    /**
     * List invoiceItems.
     * GET /api/invoice-items
     */
    public function index(Request $req)
    {
        $invoiceItems = InvoiceItem::orderByRaw('created_at ASC');

        if ($query = $req->query('query', false)) {
            $invoiceItems->whereRaw('(description LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('invoice_id', false)) {
            $invoiceItems->where('invoice_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $invoiceItems = $invoiceItems->get();
        } else {
            $invoiceItems = $invoiceItems->paginate($req->query('per_page', 20));
        }

        if ($invoiceItems->isEmpty()) Utils::error('No invoice items found!', 404);

        return ($invoiceItems);
    }

    /**
     * Create invoiceItem.
     * POST /api/invoice-items
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'description' => 'required|string',
            'rate' => 'required|numeric',
            'quantity' => 'required|numeric',
            'total' => 'required|numeric',
            'invoice_id' => 'required|exists:invoices,id',
        ]);

        // create invoiceItem
        $invoiceItem = new InvoiceItem;
        $invoiceItem->fill($req->all());
        $invoiceItem->save();

        return $invoiceItem;
    }

    /**
     * Show invoiceItem.
     * GET /api/invoice-items/{id}
     */
    public function show(Request $req, $id)
    {
        $invoiceItem = InvoiceItem::find($id);
        if (empty($invoiceItem)) Utils::error('Invoice items not found!', 404);
        return $invoiceItem;
    }

    /**
     * Update invoiceItem.
     * POST /api/invoice-items/{id}
     */
    public function update(Request $req, $id)
    {
        $invoiceItem = InvoiceItem::find($id);
        if (empty($invoiceItem)) Utils::error('Invoice items not found!', 404);

        $this->validate($req, [
            'description' => 'string',
            'rate' => 'numeric',
            'quantity' => 'numeric',
            'total' => 'numeric',
            'invoice_id' => 'exists:invoices,id',
        ]);

        // update invoiceItem
        if ($req->has('description')) $invoiceItem->description = $req->input('description');
        if ($req->has('rate')) $invoiceItem->rate = $req->input('rate');
        if ($req->has('quantity')) $invoiceItem->quantity = $req->input('quantity');
        if ($req->has('total')) $invoiceItem->total = $req->input('total');
        if ($req->has('invoice_id')) $invoiceItem->invoice_id = $req->input('invoice_id');

        $invoiceItem->save();

        return $invoiceItem;
    }


    /**
     * Delete invoiceItem by id.
     * DELETE /api/invoice-items/{id}
     */
    public function delete(Request $req, $id)
    {
        $invoiceItem = InvoiceItem::find($id);
        if (empty($invoiceItem)) Utils::error('Invoice item not found!', 404);
        $invoiceItem->delete();
        return $invoiceItem;
    }
}
