<?php

namespace App\Http\Controllers;

use App\Business;
use App\CardBrand;
use App\CardBrandCommission;
use App\Country;
use App\CountryCommission;
use App\Helpers\Auth;
use App\Helpers\OtpHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\Invoice;
use App\InvoicePayLink;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class InvoicePayLinkController extends Controller
{
    /**
     * List invoice_paylink
     * GET /api/invoice-pay-links
     */
    public function index(Request $req)
    {
        $invoice_paylink = InvoicePayLink::orderBy('id', 'DESC');

        if ($business_id = $req->query("business_id", false)) {
            $business = Business::where('id', $business_id)->first();
            $invoice_paylink->whereRaw('(name LIKE ? )', ["%$business->name%"]);
        }

        if ($query = $req->query("query", false)) {
            $invoice_paylink->whereRaw('(name LIKE ? OR email LIKE ? OR mobile_number LIKE ? OR description LIKE ? OR descriptor LIKE ? OR amount LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($query = $req->query('status')) {
            $invoice_paylink->where('status', $query);
        }

        if ($query = $req->query('from_date')) {
            $fromDate = Carbon::parse($query)->startOfDay();
            $invoice_paylink->where('created', '>=', $fromDate);
        }
        if ($query = $req->query('to_date')) {
            $toDate = Carbon::parse($query)->endOfDay();
            $invoice_paylink->where('created', '<=', $toDate);
        }

        if ($req->query('nopaginate', false) == '1') {
            $invoice_paylink = $invoice_paylink->get();
        } else {
            $invoice_paylink = $invoice_paylink->paginate($req->query('per_page', 20));
        }

        if ($invoice_paylink->isEmpty()) Utils::error('payment links not found!', 404);

        return $invoice_paylink;
    }

    /**
     * List invoice_paylink
     * GET /api/invoice-pay-links
     */
    public function find(Request $req, $code)
    {
        $invoice_paylink = InvoicePayLink::whereRaw('(shortn LIKE ? )', ["%$code%"])->first();
        if (empty($invoice_paylink)) Utils::error('payment link not found!', 404);

        $business = Business::whereRaw('(name LIKE ? )', ["%$invoice_paylink->name%"])->first();
        if (empty($business)) Utils::error('Business not found!', 404);
        $invoice_paylink->business_id = $business->id;
        $invoice_paylink->image_url = $business->image_url;

        return $invoice_paylink;
    }

    /**
     * Create invoice_paylink.
     * POST /api/invoice-pay-links
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'string',
            'invoice_id' => 'string',
            'email' => 'string',
            'mobile_number' => 'required|string',
            'publishkey' => 'required|string',
            'secretkey' => 'required|string',
            'description' => 'required|string',
            'descriptor' => 'required|string',
            'amount' => 'required|numeric|gt:0',
        ]);

        // create invoice_paylink
        $invoice_paylink = new InvoicePayLink;
        $invoice_paylink->fill($req->all());
        if ($req->has('amount')) {
            $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
            if ($mode == 'test') {
                $columns = 'stripe_public_key_test';
                $columns1 = 'stripe_private_key_test';
            }
            if ($mode == 'live') {
                $columns = 'stripe_public_key_live';
                $columns1 = 'stripe_private_key_live';
            }

            $publishkey = SiteSettingController::findByKey($columns)->value;
            $secretkey = SiteSettingController::findByKey($columns1)->value;

            $invoice_paylink->publishkey = $publishkey;
            $invoice_paylink->secretkey = $secretkey;
        }


        // Rendom Shuffle Code Generate
        $arr = str_split('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789');
        shuffle($arr);
        $arr = array_slice($arr, 0, 9);
        $random = implode('', $arr);
        $invoice_paylink->shortn = $random;

        $invoice_paylink->save();

        $send_url = env('APP_URL') . 'invoice-payment?code=' . $random . '&'; // http://192.168.1.23:8300/invoice-payment?code=dZCMEwjIO&

        $message = 'This is payment request sent, please click link below to complete your payment.' . $send_url;
        (new OtpHelper())->sendMessage($message, $req->input('mobile_numer'), 'payment_sms_link');

        return [
            'message' => 'message send successfully!!',
            'payment_link' => $send_url,
            'payment_message' => $invoice_paylink,
        ];
    }

    /**
     * List invoice_paylink
     * GET /api/invoice-pay-links
     */
    public function change_status(Request $req, $id)
    {
        $invoice_paylink = InvoicePayLink::find($id);
        if (empty($invoice_paylink)) Utils::error('payment link not found!', 404);

        $this->validate($req, [
            'status' => 'required|in:Pending,Success,Failed',
            'payment_id' => 'required|string',
        ]);

        // update shortln url
        if ($req->has('status')) $invoice_paylink->status = $req->input('status');
        if ($req->has('payment_id')) $invoice_paylink->payment_id = $req->input('payment_id');
        $invoice_paylink->save();

        if ($req->input('status') == 'Success') {
            $invoice = Invoice::find($invoice_paylink->invoice_id);
            if (empty($invoice)) Utils::error('Invoice not found!', 404);
            $invoice->paid_amount = $invoice->paid_amount + $invoice_paylink->amount;
            if ($invoice->paid_amount >=  $invoice->grand_total) {
                $invoice->status = 'Paid';
            } else {
                $invoice->status = 'Partial';
            }
            $invoice->save();
        }

        return $invoice_paylink;
    }

    /**
     * List invoice_paylink
     * GET /api/invoice-pay-links/id/cancel-payment
     */
    public function change_cancel(Request $req, $id)
    {
        $invoice_paylink = InvoicePayLink::find($id);
        if (empty($invoice_paylink)) Utils::error('payment link not found!', 404);

        // cancel shortln url
        $invoice_paylink->status = 'Failed';
        $invoice_paylink->save();

        return $invoice_paylink;
    }

    /**
     * Update invoice_paylink.
     * POST  /api/invoice-pay-links/id
     */
    public function resend(Request $req, $id)
    {
        $invoice_paylink = InvoicePayLink::find($id);
        if (empty($invoice_paylink)) Utils::error('payment link not found!', 404);

        //Amount Update
        if ($req->has('amount')) $invoice_paylink->amount = $req->input('amount');
        $invoice_paylink->save();

        $send_url = env('APP_URL') . 'invoice-payment?code=' . $invoice_paylink->shortn . '&';

        // $message = 'Enter below link ' . $send_url;
        $message = 'This is payment request sent, please click link below to complete your payment.' . $send_url;
        (new OtpHelper())->sendMessage($message, $invoice_paylink->mobile_number, 'payment_sms_link');

        return [
            'message' => 'message send successfully!!',
            'payment_link' => $send_url,
            'payment_message' => $invoice_paylink,
        ];
    }

    /**
     * Create Payment Intent.
     * POST  /api/invoice-pay-links/id
     */
    public function payment_intent(Request $req)
    {

        $invoice_paylink = InvoicePayLink::find($req->input('id'));
        if ($invoice_paylink->status == 'Failed') Utils::error('payment link cancel,please try another link!', 404);
        if ($req->has('customer_email')) $invoice_paylink->email = $req->input('customer_email');
        $invoice_paylink->save();

        $key =  $req->input('secretkey');
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        $this->validate($req, [
            'amount' => 'required|numeric|gt:0',  // multiplied by 100
            's_terminal_id' => 'string',
            'transaction_type' => 'required|in:online,merchant,connect',
            'business_id' => 'required_without_all:statement_descriptor,restaurant_id,s_terminal_id',
            'order_type' => 'required_if:transaction_type,online|in:dinein,pickup,delivery',
            'restaurant_id' => 'required_without_all:statement_descriptor,business_id,s_terminal_id',
            'statement_descriptor' => 'required_without_all:restaurant_id,business_id,s_terminal_id',
            'customer_id' => 'string',
            'customer_name' => 'string',
            'customer_email' => 'string|email|required_with:customer_name',
            'auto_confirm' => 'boolean'
        ]);

        $transaction_type = $req->input('transaction_type');
        $order_type = $req->input('order_type');
        $statement_descriptor = $req->input('descriptor');
        $statement_description = $req->input('description');
        $business_id = $req->input('business_id');
        $business = null;
        $autoConfirm = $req->input('auto_confirm', false);

        if ($req->has('business_id') && $business_id != '') {
            $business = Business::find($business_id);
        } else {
            $business = Business::where('name', $invoice_paylink->name)->first();
        }

        $stripe_account_id = null;
        $transfer_connect = null;
        if (!empty($business)) {
            $stripe_account_id = $business->s_account_id;
            $transfer_connect = $business->transfer_connect;
        }

        $fee = 0;
        $country_stripe_fixed = 0;
        $country_stripe_percentage = 0;
        $card_stripe_fixed = 0;
        $card_stripe_percentage = 0;

        $humanAmount = $req->input('amount') / 100;
        if ($transaction_type != null && $transaction_type != 'online' && $req->has('payment_method_id')) {
            $stripe_payment_method = $stripe->paymentMethods->retrieve($req->input('payment_method_id'));
            $card_brand = $stripe_payment_method->card->brand;
            $card_country = $stripe_payment_method->card->country;
            $country = Country::where('iso', $card_country)->first();
            if (!empty($country)) {
                $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
            }
            if (empty($countryCommission)) {
                $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($countryCommission)) {
                $country_stripe_percentage = $this->calculateFee($humanAmount, $countryCommission->stripe_percentage, 'percentage');
                $country_stripe_fixed = $countryCommission->stripe_fixed;
            }
            $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$card_brand, $card_brand])->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
            }
            if (empty($cardBrandCommission)) {
                $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($cardBrandCommission)) {
                $card_stripe_percentage = $this->calculateFee($humanAmount, $cardBrandCommission->stripe_percentage, 'percentage');
                $card_stripe_fixed = $cardBrandCommission->stripe_fixed;
            }
        }

        $fee = $this->calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type);
        $fee = $fee + $card_stripe_fixed + $card_stripe_percentage + $country_stripe_fixed + $country_stripe_percentage;

        if ($req->has('currency')) {
            $currency = $req->input('currency');
        } else {
            $currency = 'GBP';
        }

        $intentParams = [
            'amount' => (int) ($humanAmount * 100),
            'currency' => $currency,
        ];
        if ($autoConfirm) {
            $intentParams['confirm'] = true;
        } else {
            $intentParams['confirmation_method'] = 'automatic';
        }
        if ($req->has('payment_method_id')) {
            $intentParams['payment_method'] = $req->input('payment_method_id');
        }
        if ($statement_descriptor != null) {
            $intentParams['statement_descriptor'] = $statement_descriptor;
            $intentParams['description'] = $statement_description;
        }
        if ($req->input('customer_id') != null) {
            $intentParams['customer'] = $req->input('customer_id');
            $intentParams['save_payment_method'] = true;
        }
        if ($req->input('customer_email') != null) {
            $customerParams['email'] = $req->input('customer_email');
            if ($req->input('customer_name') != null) {
                $customerParams['name'] = $req->input('customer_name');
            }
            $customer = $stripe->customers->create($customerParams);
            $intentParams['customer'] = $customer->id;
            $intentParams['save_payment_method'] = true;
        }
        if ($stripe_account_id != null && $transfer_connect == 'yes') {
            if ($business->business_commissions->auto_fee == 'yes') {
                $intentParams['application_fee_amount'] = $fee * 100;
            }
            $intentParams['transfer_data'] = [
                'destination' => $stripe_account_id
            ];
        }

        $intent = $stripe->paymentIntents->create($intentParams);
        return ['payment_intent_id' => $intent->client_secret, 'payment_intent' => $intent, "status" => $intent->status];
    }

    public function calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type)
    {
        $payment_method_fees = 0;
        $orderTypeWiseCommission = 0;
        $orderTypeWiseServiceCharge = 0;
        $orderBasedOrTransactionWiseCharge = 0;
        $cardAuthFee = 0;
        $cardProcessingFee = 0;
        $fee = 0;
        if ($transaction_type == 'online') {
            $payment_method_fees  = $this->calculateFee($humanAmount, $business->business_commissions->online_card_fee, $business->business_commissions->online_card_fee_type);
            if ($order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->dinein_commission_fee, $business->business_commissions->dinein_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->dinein_service_fee, $business->business_commissions->dinein_service_fee_type);
            } else {
                $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->online_commission_fee, $business->business_commissions->online_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->online_service_fee, $business->business_commissions->online_service_fee_type);
            }
            if ($business->business_commissions->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($humanAmount, $business->business_commissions->per_order_charge, $business->business_commissions->per_order_charge_type);
            } else {
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($humanAmount, $business->business_commissions->card_charge, $business->business_commissions->card_charge_type);
            }
            $fee = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $cardAuthFee + $cardProcessingFee);
        }
        if ($transaction_type == 'merchant') {
            $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_auth_fee, $business->business_commissions->merchant_card_auth_fee_type);
            $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_processing_fee, $business->business_commissions->merchant_card_processing_fee_type);

            $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
        }
        if ($transaction_type == 'connect') {
            $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_auth_fee, $business->business_commissions->connect_card_auth_fee_type);
            $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_processing_fee, $business->business_commissions->connect_card_processing_fee_type);

            $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
        }
        return $fee;
    }

    public function calculateFee($humanAmount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($humanAmount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }
}
