<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\InvoicePayment;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class InvoicePaymentController extends Controller
{
    /**
     * List invoice-payments.
     * GET /api/invoice-payments
     */
    public function index(Request $req)
    {
        $invoice_payments = InvoicePayment::orderByRaw('payment_mode ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $invoice_payments->whereRaw('(payment_mode LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $invoice_payments = $invoice_payments->get();
        } else {
            $invoice_payments = $invoice_payments->paginate($req->query('per_page', 20));
        }

        if ($invoice_payments->isEmpty()) Utils::error('No payments found!', 404);

        return ($invoice_payments);
    }

    /**
     * Create invoice-payment.
     * POST /api/invoice-payments
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'invoice_id' => 'required|exists:invoices,id',
            'payment_mode' => 'in:Bank,Card,Link,Merchant,Cash',
            'amount' => 'required|numeric',
            'reference' => 'string',
            'txn_id' => 'string',
        ]);

        // create invoice_payment
        $invoice_payment = new InvoicePayment;
        $invoice_payment->fill($req->all());
        $invoice_payment->updater_id = Auth::id();
        $invoice_payment->save();

        return $invoice_payment;
    }

    /**
     * Show invoice-payment.
     * GET /api/invoice-payments/{id}
     */
    public function show(Request $req, $id)
    {
        $invoice_payment = InvoicePayment::find($id);
        if (empty($invoice_payment)) Utils::error('Payment not found!', 404);
        return $invoice_payment;
    }

    /**
     * Update invoice-payment.
     * POST /api/invoice-payments/{id}
     */
    public function update(Request $req, $id)
    {
        $invoice_payment = InvoicePayment::find($id);
        if (empty($invoice_payment)) Utils::error('Payment not found!', 404);

        $this->validate($req, [
            'invoice_id' => 'required|exists:invoices,id',
            'payment_mode' => 'in:Bank,Card,Link,Merchant,Cash',
            'amount' => 'required|numeric',
            'reference' => 'string',
            'txn_id' => 'string',
        ]);

        // update invoice-payment
        if ($req->has('invoice_id')) $invoice_payment->invoice_id = $req->input('invoice_id');
        if ($req->has('payment_mode')) $invoice_payment->payment_mode = $req->input('payment_mode');
        if ($req->has('amount')) $invoice_payment->amount = $req->input('amount');
        if ($req->has('reference')) $invoice_payment->reference = $req->input('reference');
        if ($req->has('txn_id')) $invoice_payment->txn_id = $req->input('txn_id');
        if ($req->has('updater_id')) $invoice_payment->updater_id = $req->input('updater_id');
        $invoice_payment->archived = $req->input('archived');
        $invoice_payment->save();

        return $invoice_payment;
    }


    /**
     * Delete invoice-payment by id.
     * DELETE /api/invoice-payments/{id}
     */
    public function delete(Request $req, $id)
    {
        $invoice_payment = InvoicePayment::find($id);
        if (empty($invoice_payment)) Utils::error('Payment not found!', 404);
        $invoice_payment->delete();
        return $invoice_payment;
    }
}