<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\MiscCharge;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class MiscChargeController extends Controller
{
    /**
     * List miscCharges.
     * GET /api/misc-charges
     */
    public function index(Request $req)
    {
        $miscCharges = MiscCharge::orderByRaw('date DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $miscCharges->whereRaw('(description LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $miscCharges->where('business_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $miscCharges = $miscCharges->get();
        } else {
            $miscCharges = $miscCharges->paginate($req->query('per_page', 20));
        }

        if ($miscCharges->isEmpty()) Utils::error('No miscCharges found!', 404);


        return ($miscCharges);
    }

    /**
     * Create miscCharge.
     * POST /api/misc-charges
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'date' => 'required|date_format:Y-m-d',
            'amount' => 'required|numeric',
            'business_id' => 'required|exists:businesses,id',
            'archived' => 'boolean',
        ]);

        // create miscCharge
        $miscCharge = new MiscCharge;
        $miscCharge->fill($req->all());

        $miscCharge->save();

        return $miscCharge;
    }

    /**
     * Show miscCharge.
     * GET /api/misc-charges/{id}
     */
    public function show(Request $req, $id)
    {
        $miscCharge = MiscCharge::find($id);
        if (empty($miscCharge)) Utils::error('MiscCharge not found!', 404);
        return $miscCharge;
    }

    /**
     * Update miscCharge.
     * POST /api/misc-charges/{id}
     */
    public function update(Request $req, $id)
    {
        $miscCharge = MiscCharge::find($id);
        if (empty($miscCharge)) Utils::error('MiscCharge not found!', 404);

        $this->validate($req, [
            'date' => 'required|date_format:Y-m-d',
            'amount' => 'required|numeric',
            'business_id' => 'required|exists:businesses,id',
            'archived' => 'boolean',
        ]);

        // update miscCharge
        if ($req->has('date')) $miscCharge->date = $req->input('date');
        if ($req->has('description')) $miscCharge->description = $req->input('description');
        if ($req->has('business_id')) $miscCharge->business_id = $req->input('business_id');
        $miscCharge->archived = $req->input('archived');
        $miscCharge->amount = $req->input('amount');

        $miscCharge->save();

        return $miscCharge;
    }


    /**
     * Delete miscCharge by id.
     * DELETE /api/misc-charges/{id}
     */
    public function delete(Request $req, $id)
    {
        $miscCharge = MiscCharge::find($id);
        if (empty($miscCharge)) Utils::error('MiscCharge not found!', 404);
        $miscCharge->delete();
        return $miscCharge;
    }
}
