<?php

namespace App\Http\Controllers;

use App\Business;
use App\Exports\MiscInvoiceExport;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\MiscInvoice;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\MiscInvoiceItem;
use App\ModelsTiffintom\TiffintomShortlnUrl;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\Passbook;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Maatwebsite\Excel\Facades\Excel;
use Mpdf\Mpdf;

class MiscInvoiceController extends Controller
{
    /**
     * List miscInvoices.
     * GET /api/misc-invoices
     */
    public function index(Request $req)
    {
        $miscInvoices = MiscInvoice::selectRaw("misc_invoices.*,businesses.name as business_name,businesses.online_service as online_service,businesses.merchant_service as merchant_service,businesses.epos_service as epos_service,businesses.connect_service as connect_service")->leftJoin('businesses', 'business_id', '=', 'businesses.id')->orderByRaw('misc_invoices.date DESC, misc_invoices.created_at DESC');

        if ($query = $req->query('query', false)) {
            $miscInvoices->whereRaw('(misc_invoices.comment LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $miscInvoices->where('misc_invoices.business_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $miscInvoices = $miscInvoices->get();
        } else {
            $miscInvoices = $miscInvoices->paginate($req->query('per_page', 20));
        }

        if ($miscInvoices->isEmpty()) Utils::error('No miscInvoices found!', 404);


        return ($miscInvoices);
    }

    /**
     * Create miscInvoice.
     * POST /api/misc-invoices
     */
    public function create(Request $req)
    {
        if ($req->has('items') && !is_array($req->input('items'))) {
            $req->merge(['items' => json_decode($req->input('items'))]);
        }
        $this->validate($req, [
            'percentage' => 'required_if:payment_mode,Percentage|numeric',
            'grand_total' => 'required|numeric',
            'payment_mode' => 'required|in:Auto,Manual,Percentage',
            'status' => 'in:Paid,Partially Paid, Paid',
            'date' => 'required|date',
            'comment' => 'string',
            'business_id' => 'required|exists:businesses,id',
            'items' => 'required|array'
        ]);

        // create miscInvoice
        $miscInvoice = new MiscInvoice;
        $miscInvoice->fill($req->all());

        $miscInvoice->save();

        $passbook = new Passbook;
        $passbook->business_id = $miscInvoice->business_id;
        $passbook->entity_id = $miscInvoice->id;
        $passbook->entity = "misc_invoice";
        $passbook->amount =  -1 * $miscInvoice->grand_total;
        $passbook->date = Carbon::today();
        $passbook->from_date = Carbon::today()->toDateString();
        $passbook->to_date = Carbon::today()->toDateString();
        $passbook->description = "Debited for Miscellaneous invoice #$miscInvoice->id dated on " . Carbon::today()->toDateString();
        $passbook->save();

        $items = ($req->input('items'));
        foreach ($items as $item) {
            $miscInvoiceItem = new MiscInvoiceItem((array)$item);
            $miscInvoiceItem->misc_invoice_id = $miscInvoice->id;
            $miscInvoiceItem->save();
        }
        $miscInvoice->append('misc_invoice_items');
        return $miscInvoice;
    }

    /**
     * Show miscInvoice.
     * GET /api/misc-invoices/{id}
     */
    public function show(Request $req, $id)
    {
        $miscInvoice = MiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('MiscInvoice not found!', 404);
        $miscInvoice->append('misc_invoice_items');
        return $miscInvoice;
    }

    /**
     * Update miscInvoice.
     * POST /api/misc-invoices/{id}
     */
    public function update(Request $req, $id)
    {
        $miscInvoice = MiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('MiscInvoice not found!', 404);
        if ($req->has('items') && !is_array($req->input('items'))) {
            $req->merge(['items' => json_decode($req->input('items'))]);
        }

        $alreadyPaidAmount = $miscInvoice->total_paid_amount;
        $this->validate($req, [
            'grand_total' => 'numeric',
            'percentage' => 'required_if:payment_mode,Percentage|numeric',
            'payment_mode' => 'in:Auto,Manual,Percentage',
            'status' => 'in:Paid,Partially Paid, Paid',
            'date' => 'date',
            'comment' => 'string',
            'business_id' => 'exists:businesses,id',
            'items' => 'required|array'
        ]);

        // update miscInvoice
        if ($req->has('grand_total')) $miscInvoice->grand_total = $req->input('grand_total');
        if ($req->has('payment_mode')) $miscInvoice->payment_mode = $req->input('payment_mode');
        if ($req->has('percentage')) $miscInvoice->percentage = $req->input('percentage');
        if ($req->has('status')) $miscInvoice->status = $req->input('status');
        if ($req->has('date')) $miscInvoice->date = $req->input('date');
        if ($req->has('comment')) $miscInvoice->comment = $req->input('comment');
        if ($req->has('business_id')) $miscInvoice->business_id = $req->input('business_id');
        if ($miscInvoice->grand_total < $alreadyPaidAmount) {
            Utils::error('Invoice total should not be smaller than already paid amount', 400);
        }
        $miscInvoice->save();
        $passbook = Passbook::where('entity_id', $miscInvoice->id)->where('entity', 'misc_invoice')->first();
        if (empty($passbook)) {
            $passbook = new Passbook;
            $passbook->business_id = $miscInvoice->business_id;
            $passbook->entity_id = $miscInvoice->id;
            $passbook->entity = "misc_invoice";
            $passbook->date = Carbon::today();
            $passbook->from_date = Carbon::today()->toDateString();
            $passbook->to_date = Carbon::today()->toDateString();
            $passbook->description = "Debited for Miscellaneous invoice #$miscInvoice->id dated on " .  Carbon::today()->toDateString();
        }
        $passbook->amount = -1 * $miscInvoice->grand_total;
        $passbook->save();

        // $miscInvoice->append('misc_invoice_items');
        $oldItems = MiscInvoiceItem::where('misc_invoice_id', $miscInvoice->id)->delete();
        $items = ($req->input('items'));
        foreach ($items as $item) {
            $miscInvoiceItem = new MiscInvoiceItem((array)$item);
            $miscInvoiceItem->misc_invoice_id = $miscInvoice->id;
            $miscInvoiceItem->save();
        }
        $miscInvoice->append('misc_invoice_items');
        return $miscInvoice;
    }

    public function pay(Request $req, $id)
    {
        $miscInvoice = MiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('MiscInvoice not found!', 404);
        $this->validate($req, [
            'paid_amount' => 'required|numeric|gt:0',
        ]);
        if ($miscInvoice->total_paid_amount + $req->input('paid_amount') > $miscInvoice->grand_total) {
            Utils::error("Total paid amount is exceeding to invoice total", 400);
        }
        $passbook = new Passbook;
        $passbook->business_id = $miscInvoice->business_id;
        $passbook->entity_id = $id;
        $passbook->entity = "misc_invoice";
        $passbook->amount =  $req->input('paid_amount');
        $passbook->date = Carbon::today();
        $passbook->from_date = Carbon::today()->toDateString();
        $passbook->to_date = Carbon::today()->toDateString();
        $passbook->description = "Amount paid for misc invoice dated on $miscInvoice->date";
        $passbook->save();
    }

    /**
     * mail send miscInvoice by id.
     * POST /api/misc-invoices/{id}/send-message
     */
    public function send_message(Request $req, $id)
    {
        $this->validate($req, [
            'contact_numbers' => 'required|string',
        ]);

        // create shortln_url
        $shortln_url = new TiffintomShortlnUrl();
        if ($req->input('business_id') != '') {
            $bussiness = Business::where('id', $req->input('business_id'))->first();
            if (!empty($bussiness)) {
                if ($bussiness->connect_service == 1) {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $columns = 'connect_stripe_public_key_test';
                        $columns1 = 'connect_stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $columns = 'connect_stripe_public_key_live';
                        $columns1 = 'connect_stripe_private_key_live';
                    }
                    $publishkey = SiteSettingController::findByKey($columns)->value;
                    $secretkey = SiteSettingController::findByKey($columns1)->value;
                    $shortln_url->publishkey = $publishkey;
                    $shortln_url->secretkey = $secretkey;
                } else {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $columns = 'stripe_public_key_test';
                        $columns1 = 'stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $columns = 'stripe_public_key_live';
                        $columns1 = 'stripe_private_key_live';
                    }
                    $publishkey = SiteSettingController::findByKey($columns)->value;
                    $secretkey = SiteSettingController::findByKey($columns1)->value;
                    $shortln_url->publishkey = $publishkey;
                    $shortln_url->secretkey = $secretkey;
                }
            }
        }

        // Rendom Shuffle Code Generate
        $arr = str_split('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789');
        shuffle($arr);
        $arr = array_slice($arr, 0, 9);
        $random = implode('', $arr);
        $shortln_url->shortn = $random;
        $shortln_url->name = $bussiness->name;
        $shortln_url->email = $bussiness->email;
        $shortln_url->phone  = $req->input('contact_numbers');
        $shortln_url->description = $bussiness->statement_description;
        $shortln_url->descriptor = $bussiness->statement_descriptor;
        $shortln_url->amount =  $req->input('grand_total');
        $shortln_url->save();

        $send_url = env('APP_URL') . 'epos-payment?code=' . $random . '&';
        $message = 'This is payment request sent from ' . $bussiness->name . ', please click link below to complete your payment.' . $send_url;
        (new OtpHelper())->sendMessage($message, $req->input('contact_numbers'), 'payment_sms_link');

        return ['message' => 'message send successfully.'];
    }

    /**
     * mail send miscInvoice by id.
     * POST /api/misc-invoices/{id}/send-mail
     */
    public function send_mail(Request $req, $id)
    {
        $this->validate($req, [
            'email' => 'required|string',
        ]);

        // $this->makePdf($req->input('id'));

        $this->sendMail($req->input('email'), $req->input('id'), $req->input('message'));

        return ['message' => 'mail send successfully.'];
    }

    public function print(Request $req, $id)
    {
        $miscInvoice = MiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('MiscInvoice not found!', 404);
        $miscInvoice->append('misc_invoice_items');
        $miscInvoice->append('business');
        $fileName = $miscInvoice['business']['name'];

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            /*border-left: 1px solid #000000;
                border-top: 1px solid #000000;*/
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #73C6B6;
            font-weight:600;
            background-color:#138D75;
            color:#fff;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #73C6B6;
        }
        .items-table tr:nth-child(even){
            background-color:#E8F6F3   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h2 style = "text-align:right">Invoice</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td style="font-weight: 900;">Simple Merchant Payment Services</td>';
        $html = $html . '<td>' . Carbon::parse($miscInvoice->date)->format('F d, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td></td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td><h4>' . $miscInvoice->business->name . '</h4></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall</td>';
        $html = $html . '<td>' .   $miscInvoice->business->address  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>WS1 3RA</td>';
        $html = $html . '<td>' . $miscInvoice->business->country->name  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td>Email: ' . $miscInvoice->business->email . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Sr No.</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Rate</th>';
        $html = $html . '<th>Quantity</th>';
        $html = $html . '<th>Total</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        foreach ($miscInvoice->misc_invoice_items as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . $srNo . '</td>';
            $html = $html . '<td>' . $item->description . '</td>';
            $html = $html . '<td>' . $item->rate . '</td>';
            $html = $html . '<td>' . $item->quantity . '</td>';
            $html = $html . '<td>' . $item->total . '</td>';
            $html = $html . '</tr>';
            $srNo++;
        }
        $html = $html . '<tr>';
        $html = $html . '<th colspan="4" style="text-align:right;">Grand Total:</th>';
        $html = $html . '<th>' . $miscInvoice->grand_total . '</th>';
        $html = $html . '</tr>';
        if ($miscInvoice->status != "Paid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="4" style="text-align:right;">Paid Amount:</td>';
            $html = $html . '<td>' . $miscInvoice->total_paid_amount . '</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<th colspan="5">Invoice Paid</th>';
            $html = $html . '</tr>';
        }


        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';

        $mpdf = new Mpdf();
        $mpdf->WriteHTML($html);
        return $mpdf->Output("$fileName.pdf", "D");
        // return Excel::download(new MiscInvoiceExport($miscInvoice), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
    }

    /**
     * Delete miscInvoice by id.
     * DELETE /api/misc-invoices/{id}
     */
    public function delete(Request $req, $id)
    {
        $miscInvoice = MiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('MiscInvoice not found!', 404);
        $miscInvoice->delete();
        return $miscInvoice;
    }

    public function makePdf($id)
    {
        $miscInvoice = MiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('MiscInvoice not found!', 404);
        $miscInvoice->append('misc_invoice_items');
        $miscInvoice->append('business');
        $fileName = $miscInvoice['business']['name'];

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #73C6B6;
            font-weight:600;
            background-color:#138D75;
            color:#fff;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #73C6B6;
        }
        .items-table tr:nth-child(even){
            background-color:#E8F6F3   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h2 style = "text-align:right">Invoice</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td style="font-weight: 900;">Simple Merchant Payment Services</td>';
        $html = $html . '<td>' . Carbon::parse($miscInvoice->date)->format('F d, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td></td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td><h4>' . $miscInvoice->business->name . '</h4></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall</td>';
        $html = $html . '<td>' .   $miscInvoice->business->address  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>WS1 3RA</td>';
        $html = $html . '<td>' . $miscInvoice->business->country->name  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td>Email: ' . $miscInvoice->business->email . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Sr No.</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Rate</th>';
        $html = $html . '<th>Quantity</th>';
        $html = $html . '<th>Total</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        foreach ($miscInvoice->misc_invoice_items as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . $srNo . '</td>';
            $html = $html . '<td>' . $item->description . '</td>';
            $html = $html . '<td>' . $item->rate . '</td>';
            $html = $html . '<td>' . $item->quantity . '</td>';
            $html = $html . '<td>' . $item->total . '</td>';
            $html = $html . '</tr>';
            $srNo++;
        }
        $html = $html . '<tr>';
        $html = $html . '<th colspan="4" style="text-align:right;">Grand Total:</th>';
        $html = $html . '<th>' . $miscInvoice->grand_total . '</th>';
        $html = $html . '</tr>';
        if ($miscInvoice->status != "Paid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="4" style="text-align:right;">Paid Amount:</td>';
            $html = $html . '<td>' . $miscInvoice->total_paid_amount . '</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<th colspan="5">Invoice Paid</th>';
            $html = $html . '</tr>';
        }


        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';


        $mpdf = new Mpdf();
        $mpdf->SetHeader('Misc Invoice');
        $mpdf->SetFooter('{PAGENO}');
        $mpdf->defaultheaderfontsize = 10;
        $mpdf->defaultheaderfontstyle = 'B';
        $mpdf->defaultheaderline = 0;
        $mpdf->defaultfooterfontsize = 10;
        $mpdf->defaultfooterfontstyle = 'BI';
        $mpdf->defaultfooterline = 0;
        $mpdf->showWatermarkImage = true;
        $mpdf->WriteHTML($html);
        $fileName = str_replace(' ', '', $fileName);
        $path =  $fileName . '.pdf';
        $mpdf->Output('images/' . $path, "S");
        $file = storage_path() . '/app/public/' . $fileName . '.pdf';
        $file_local = Storage::disk('public')->url("images/$fileName.pdf");
        Storage::disk('ubsidi')->put(ImageHelper::$TYPE_MISC_INVOICE . '/' . $path, $file_local);
    }

    public function sendMail($sender_email, $id, $message)
    {
        $miscInvoice = MiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('MiscInvoice not found!', 404);
        $miscInvoice->append('misc_invoice_items');
        $miscInvoice->append('business');
        $fileName = $miscInvoice['business']['name'];

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        // Business Misc Invoice mail
        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #73C6B6;
            font-weight:600;
            background-color:#138D75;
            color:#fff;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #73C6B6;
        }
        .items-table tr:nth-child(even){
            background-color:#E8F6F3   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h2 style = "text-align:right">Invoice</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td style="font-weight: 900;">Simple Merchant Payment Services</td>';
        $html = $html . '<td>' . Carbon::parse($miscInvoice->date)->format('F d, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td></td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td><h4>' . $miscInvoice->business->name . '</h4></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall</td>';
        $html = $html . '<td>' .   $miscInvoice->business->address  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>WS1 3RA</td>';
        $html = $html . '<td>' . $miscInvoice->business->country->name  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td>Email: ' . $miscInvoice->business->email . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Sr No.</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Rate</th>';
        $html = $html . '<th>Quantity</th>';
        $html = $html . '<th>Total</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        foreach ($miscInvoice->misc_invoice_items as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . $srNo . '</td>';
            $html = $html . '<td>' . $item->description . '</td>';
            $html = $html . '<td>' . $item->rate . '</td>';
            $html = $html . '<td>' . $item->quantity . '</td>';
            $html = $html . '<td>' . $item->total . '</td>';
            $html = $html . '</tr>';
            $srNo++;
        }
        $html = $html . '<tr>';
        $html = $html . '<th colspan="4" style="text-align:right;">Grand Total:</th>';
        $html = $html . '<th>' . $miscInvoice->grand_total . '</th>';
        $html = $html . '</tr>';
        if ($miscInvoice->status != "Paid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="4" style="text-align:right;">Paid Amount:</td>';
            $html = $html . '<td>' . $miscInvoice->total_paid_amount . '</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<th colspan="5">Invoice Paid</th>';
            $html = $html . '</tr>';
        }


        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';


        $mpdf = new Mpdf();
        $mpdf->SetHeader('Misc Invoice');
        $mpdf->SetFooter('{PAGENO}');
        $mpdf->defaultheaderfontsize = 10;
        $mpdf->defaultheaderfontstyle = 'B';
        $mpdf->defaultheaderline = 0;
        $mpdf->defaultfooterfontsize = 10;
        $mpdf->defaultfooterfontstyle = 'BI';
        $mpdf->defaultfooterline = 0;
        $mpdf->showWatermarkImage = true;
        $mpdf->WriteHTML($html);
        $fileName = str_replace(' ', '', $fileName);
        $pdf = $mpdf->Output('', 'S');
        $attachment = chunk_split(base64_encode($pdf));

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#09c4a9">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-color.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                            ' . $message . '
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#09c4a9"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';



        $separator = md5(time());
        $eol = PHP_EOL;
        $filename = $fileName . ".pdf";

        $headers = '';
        $headers .= "MIME-Version: 1.0" . $eol;
        $headers .= "Content-Type: multipart/mixed; boundary=\"" . $separator . "\"" . $eol . $eol;

        $body = "Content-Transfer-Encoding: 7bit" . $eol;
        $body .= "This is a MIME encoded message." . $eol;
        $body .= "--" . $separator . $eol;
        $body .= "Content-Type: text/html; charset=\"UTF-8\"" . $eol;
        $body .= "Content-Transfer-Encoding: 8bit" . $eol . $eol;
        $body .= $emailinfo . $eol;
        $body .= "--" . $separator . $eol;
        $body .= "Content-Type: application/octet-stream; name=\"" . $filename . "\"" . $eol;
        $body .= "Content-Transfer-Encoding: base64" . $eol;
        $body .= "Content-Disposition: attachment" . $eol . $eol;
        $body .= $attachment . $eol;
        $emailinfo .= "--" . $separator . "--";

        $email = array(
            'body' => $body,
            'subject' => 'Misc Invoice',
            'mail_type' => $siteSettings->mail_option,
            'to' => $sender_email,
            'from' => $siteSettings->admin_email,
            'header_pdf' => $headers,
        );

        (new MailHelper())->sendMail($email);

        return 'Mail Send Successfully';
    }
}
