<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\MiscInvoiceItem;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class MiscInvoiceItemController extends Controller
{
    /**
     * List miscInvoiceItems.
     * GET /api/misc-charges
     */
    public function index(Request $req)
    {
        $miscInvoiceItems = MiscInvoiceItem::orderByRaw('created_at ASC');

        if ($query = $req->query('query', false)) {
            $miscInvoiceItems->whereRaw('(description LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('misc_invoice_id', false)) {
            $miscInvoiceItems->where('misc_invoice_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $miscInvoiceItems = $miscInvoiceItems->get();
        } else {
            $miscInvoiceItems = $miscInvoiceItems->paginate($req->query('per_page', 20));
        }

        if ($miscInvoiceItems->isEmpty()) Utils::error('No miscInvoiceItems found!', 404);


        return ($miscInvoiceItems);
    }

    /**
     * Create miscInvoiceItem.
     * POST /api/misc-charges
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'description' => 'required|string',
            'rate' => 'required|numeric',
            'quantity' => 'required|numeric',
            'total' => 'required|numeric',
            'misc_invoice_id' => 'required|exists:misc_invoices,id',
        ]);

        // create miscInvoiceItem
        $miscInvoiceItem = new MiscInvoiceItem;
        $miscInvoiceItem->fill($req->all());

        $miscInvoiceItem->save();

        return $miscInvoiceItem;
    }

    /**
     * Show miscInvoiceItem.
     * GET /api/misc-charges/{id}
     */
    public function show(Request $req, $id)
    {
        $miscInvoiceItem = MiscInvoiceItem::find($id);
        if (empty($miscInvoiceItem)) Utils::error('MiscInvoiceItem not found!', 404);
        return $miscInvoiceItem;
    }

    /**
     * Update miscInvoiceItem.
     * POST /api/misc-charges/{id}
     */
    public function update(Request $req, $id)
    {
        $miscInvoiceItem = MiscInvoiceItem::find($id);
        if (empty($miscInvoiceItem)) Utils::error('MiscInvoiceItem not found!', 404);

        $this->validate($req, [
            'description' => 'string',
            'rate' => 'numeric',
            'quantity' => 'numeric',
            'total' => 'numeric',
            'misc_invoice_id' => 'exists:misc_invoices,id',
        ]);

        // update miscInvoiceItem
        if ($req->has('description')) $miscInvoiceItem->description = $req->input('description');
        if ($req->has('rate')) $miscInvoiceItem->rate = $req->input('rate');
        if ($req->has('quantity')) $miscInvoiceItem->quantity = $req->input('quantity');
        if ($req->has('total')) $miscInvoiceItem->total = $req->input('total');
        if ($req->has('misc_invoice_id')) $miscInvoiceItem->misc_invoice_id = $req->input('misc_invoice_id');

        $miscInvoiceItem->save();

        return $miscInvoiceItem;
    }


    /**
     * Delete miscInvoiceItem by id.
     * DELETE /api/misc-charges/{id}
     */
    public function delete(Request $req, $id)
    {
        $miscInvoiceItem = MiscInvoiceItem::find($id);
        if (empty($miscInvoiceItem)) Utils::error('MiscInvoiceItem not found!', 404);
        $miscInvoiceItem->delete();
        return $miscInvoiceItem;
    }
}
