<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\Note;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class NoteController extends Controller
{
    /**
     * List notes.
     * GET /api/notes
     */
    public function index(Request $req)
    {
        $notes = Note::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $notes->whereRaw('(note LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $notes->where('business_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $notes = $notes->get();
        } else {
            $notes = $notes->paginate($req->query('per_page', 20));
        }

        if ($notes->isEmpty()) Utils::error('No notes found!', 404);


        return ($notes);
    }

    /**
     * Create note.
     * POST /api/notes
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'note' => 'required|string',
            'business_id' => 'required|exists:businesses,id',
        ]);

        // create note
        $note = new Note;
        $note->fill($req->all());
        if (Auth::isAdmin()) {
            $note->admin_id = Auth::id();
        }
        $note->save();

        return $note;
    }

    /**
     * Show note.
     * GET /api/notes/{id}
     */
    public function show(Request $req, $id)
    {
        $note = Note::find($id);
        if (empty($note)) Utils::error('Note not found!', 404);
        return $note;
    }

    /**
     * Update note.
     * POST /api/notes/{id}
     */
    public function update(Request $req, $id)
    {
        $note = Note::find($id);
        if (empty($note)) Utils::error('Note not found!', 404);

        $this->validate($req, [
            'note' => 'string',
            'business_id' => 'required|exists:businesses,id',
        ]);

        // update note
        if ($req->has('note')) $note->note = $req->input('note');
        if ($req->has('business_id')) $note->business_id = $req->input('business_id');

        $note->save();

        return $note;
    }


    /**
     * Delete note by id.
     * DELETE /api/notes/{id}
     */
    public function delete(Request $req, $id)
    {
        $note = Note::find($id);
        if (empty($note)) Utils::error('Note not found!', 404);
        $note->delete();
        return $note;
    }
}
