<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Tiffintom\TiffintomDineinOrderController;
use App\Http\Controllers\Tiffintom\TiffintomOrderController;
use App\ModelsTiffintom\TiffintomDineinOrder;
use App\ModelsTiffintom\TiffintomOrder;
use App\OnlineOrder;
use App\OnlineOrderCommission;
use App\PaypalPayment;
use App\ResellerCommission;
use App\ResellerOrderCommission;
use App\StripeTransaction;
use App\UserBusiness;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Hamcrest\Util;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class OnlineOrderController extends Controller
{
    private static $stripe;
    private static $key;
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'connect_stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'connect_stripe_private_key_live';
        }
        self::$key =  SiteSettingController::findByKey($columns)->value;
        self::$stripe = new StripeClient(["api_key" => self::$key, "stripe_version" => "2020-08-27"]);
    }

    /**
     * List onlineOrders.
     * GET /api/online-orders
     */
    public function index(Request $req)
    {
        $onlineOrders = OnlineOrder::orderByRaw('order_date DESC');
        $onlineOrders->select('online_orders.*', 'businesses.name as business_name');
        $onlineOrders->leftJoin('businesses', 'business_id', '=', 'businesses.id');
        if ($query = $req->query('query', false)) {
            $onlineOrders->whereRaw('(order_id LIKE ? OR order_type LIKE ? OR payment_method LIKE ? OR grand_total LIKE ? OR order_status LIKE ? OR businesses.name LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('order_type', false)) {
            $onlineOrders->where('order_type', $query);
        }
        if ($query = $req->query('order_status', false)) {
            $onlineOrders->where('order_status', $query);
        }
        if ($query = $req->query('payment_method', false)) {
            $onlineOrders->where('payment_method', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $onlineOrders->where('business_id', $query);
        }
        if ($query = $req->query('archive_status', false)) {
            if ($query == 'archived') {
                $onlineOrders->where('archived', '1');
            }
            if ($query == 'unarchived') {
                $onlineOrders->where('archived', '0');
            }
        }
        if ($query = $req->query('restaurant_id', false)) {
            $onlineOrders->where('online_orders.restaurant_id', $query);
        }
        if ($query = $req->query('date_filter', false)) {
            if (strtolower($query) == 'today') {
                $from_date = Carbon::today()->startOfDay();
                $to_date = Carbon::today()->endOfDay();
            }
            if (strtolower($query) == 'yesterday') {
                $from_date = Carbon::yesterday()->startOfDay();
                $to_date = Carbon::yesterday()->endOfDay();
            }
            if (strtolower($query) == 'thisweek') {
                $from_date = Carbon::today()->getWeekStartsAt();
                $to_date = Carbon::today()->getWeekEndsAt();
            }
            if (strtolower($query) == 'thismonth') {
                $from_date = Carbon::today()->startOfMonth();
                $to_date = Carbon::today()->endOfMonth();
            }
            if (strtolower($query) == 'thisyear') {
                $from_date = Carbon::today()->startOfYear();
                $to_date = Carbon::today()->endOfYear();
            }
            if (strtolower($query) == 'this_week') {
                $from_date = Carbon::today()->startOfWeek();
                $to_date = Carbon::today()->endOfWeek();
            }
            if (strtolower($query) == 'this_month') {
                $from_date = Carbon::today()->startOfMonth();
                $to_date = Carbon::today()->endOfMonth();
            }
            if (strtolower($query) == 'this_year') {
                $from_date = Carbon::today()->startOfYear();
                $to_date = Carbon::today()->endOfYear();
            }
            if (isset($from_date) && isset($to_date)) {
                $onlineOrders->where('order_date', '>=', $from_date);
                $onlineOrders->where('order_date', '<=', $to_date);
            }
        }
        if ($query = $req->query('from_date', false)) {
            $onlineOrders->where('order_date', '>=', Carbon::parse($query)->startOfDay());
        }
        if ($query = $req->query('to_date', false)) {
            $onlineOrders->where('order_date', '<=', Carbon::parse($query)->endOfDay());
        }

        if ($req->query('nopaginate', false) == '1') {
            $onlineOrders = $onlineOrders->get();
        } else {
            $onlineOrders = $onlineOrders->paginate($req->query('per_page', 20));
        }

        if ($onlineOrders->isEmpty()) Utils::error('No orders found!', 404);


        return ($onlineOrders);
    }
    public function show(Request $req, $id)
    {
        $onlineOrder = OnlineOrder::find($id);
        if (empty($onlineOrder)) Utils::error('Order not found!', 404);
        return $onlineOrder;
    }
    public function delete(Request $req, $id)
    {
        $onlineOrder = OnlineOrder::find($id);
        if (empty($onlineOrder)) Utils::error('Order not found!', 404);
        $onlineOrder->delete();
        return $onlineOrder;
    }

    public function fetch(Request $req)
    {
        $startdate = Carbon::today()->subDays(20)->format('d-m-Y');
        $enddate = Carbon::today()->format('d-m-Y');

        if ($req->has('startdate')) {
            $startdate = $req->input('startdate');
        }
        if ($req->has('enddate')) {
            $enddate = $req->input('enddate');
        }
        return $this->orders($startdate, $enddate);
    }
    public function history(Request $req)
    {
        $startdate = Carbon::today()->subDays(15)->format('d-m-Y');
        $enddate = Carbon::today()->format('d-m-Y');
        $onlineOrder = OnlineOrder::orderByRaw('order_date ASC')->first();

        if (!empty($onlineOrder)) {
            $carbonDate = Carbon::parse($onlineOrder->order_date);
            $enddate = $carbonDate->format('d-m-Y');
            $startdate = $carbonDate->subDays(7)->format('d-m-Y');
        }
        if ($req->has('startdate')) {
            $startdate = $req->input('startdate');
        }
        if ($req->has('enddate')) {
            $enddate = $req->input('enddate');
        }
        return $this->orders($startdate, $enddate);
    }
    public function orders($startdate, $enddate)
    {
        $req = Request::create("");
        $req->merge(['from_date' => $startdate, 'to_date' => $enddate, 'nopaginate' => 1, 'append' => 1]);

        // Order fetch
        $orders = TiffintomOrder::orderBy('id', 'DESC');
        if (isset($startdate)) {
            $fromDate = Carbon::parse($startdate)->startOfDay();
            $orders->where('created', '>', $fromDate);
        }
        if (isset($enddate)) {
            $toDate = Carbon::parse($enddate)->endOfDay();
            $orders->where('created', '<', $toDate);
        }
        $orders = $orders->get();
        if (!empty($orders)) {
            $this->saveOrder($orders);
        }

        // Dine in order fetch
        $dineinOrders = TiffintomDineinOrder::orderBy('id', 'DESC');
        if (isset($startdate)) {
            $fromDate = Carbon::parse($startdate)->startOfDay();
            $dineinOrders->where('created', '>', $fromDate);
        }
        if (isset($enddate)) {
            $toDate = Carbon::parse($enddate)->endOfDay();
            $dineinOrders->where('created', '<', $toDate);
        }
        $dineinOrders = $dineinOrders->get();
        if (!empty($dineinOrders)) {
            $this->saveDineinOrder($dineinOrders);
        }

        if (!empty($orders) && !empty($dineinOrders)) {
            if (!empty($orders)) return $orders;
            if (!empty($dineinOrders)) return $dineinOrders;
        } else Utils::error("No data found", 400);

        // $response = (new Client())->request('POST', env('FETCH_ORDERS_LIST', 'https://www.tiffintom.net/v2/StoreMobileApi/requests'), [
        //     'form_params' => ['startdate' => $startdate, 'enddate' => $enddate, 'action' => 'getOrderList']
        // ]);

        // if ($response->getStatusCode() == 200) {
        //     $jsonBody = json_decode($response->getBody());
        //     if ($jsonBody->success == '1' && isset($jsonBody->Orderlist)) {
        //         $this->saveOrder($jsonBody->Orderlist);
        //     }
        //     if ($jsonBody->success == '1' && isset($jsonBody->DineinOrderlist)) {
        //         $this->saveOrder($jsonBody->DineinOrderlist);
        //     }
        //     if (isset($jsonBody->Orderlist) && isset($jsonBody->DineinOrderlist)) {
        //         if (isset($jsonBody->Orderlist)) return $jsonBody->Orderlist;
        //         if (isset($jsonBody->DineinOrderlist)) return $jsonBody->DineinOrderlist;
        //     } else Utils::error("No data found", 400);
        // } else {
        //     return $response->getBody();
        // }
    }
    public function saveOrder($orderList)
    {
        foreach ($orderList as $order) {
            // if (isset($order->order_type)) {
            $order_type = $order->order_type;
            // } else {
            //     $order_type = 'dinein';
            // }

            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            $onlineOrder = OnlineOrder::where('order_id', $order->id)->where('order_type', '!=', 'dinein')->first(); // where('order_type', $order_type)->
            if (!empty($onlineOrder)) {
                if ($onlineOrder->payment_method == 'cod' && $order->status === 'Delivered' && $onlineOrder->order_status != 'Delivered' && isset($business->business_commissions)) {
                    // $this->debit_cod_fees($onlineOrder, $business);
                }
                $onlineOrder->order_status = $order->status;
                $onlineOrder->save();
                continue;
            }
            if (empty($onlineOrder)) {
                $onlineOrder = new OnlineOrder();
            }

            $shouldSetCommissions = ($onlineOrder->business_id == null) ? true : false;

            $onlineOrder->order_id = $order->id;
            $onlineOrder->restaurant_id = $order->restaurant_id;
            if (!empty($business)) {
                $onlineOrder->business_id = $business->id;
                if ($business->auto_archive == 1) {
                    $onlineOrder->archived = 1;
                }
            }

            $onlineOrder->order_type = $order_type;
            if (isset($order->transaction_id)) {
                $onlineOrder->txn_id = $order->transaction_id;
            }
            // $onlineOrder->payment_method = $order->payment_method;
            $onlineOrder->payment_method = $order->payment_method == 'Stripe' ? 'Credit Card' : $order->payment_method;;
            if (isset($order->order_sub_total)) {
                $onlineOrder->sub_total = $order->order_sub_total;
            }
            if (isset($order->delivery_charge)) {
                $onlineOrder->delivery_charge = $order->delivery_charge;
            }
            if (isset($order->driver_tip)) {
                $onlineOrder->driver_tip = $order->driver_tip;
            }
            if (isset($order->gratuity)) {
                $onlineOrder->gratuity_tip = $order->gratuity;
            }
            if (isset($order->order_grand_total)) {
                $onlineOrder->grand_total = $order->order_grand_total;
            }
            if (isset($order->charity_amount)) {
                $onlineOrder->charity_amount = $order->charity_amount;
            }
            if (isset($order->voucher_amount)) {
                $onlineOrder->voucher_amount = $order->voucher_amount;
            }
            if (isset($order->service_charge) && $order->status != 'Failed') {
                $onlineOrder->user_service_charge = $order->service_charge;
            }

            $onlineOrder->order_date = $order->created;
            $onlineOrder->order_status = $order->status;

            $onlineOrder->save();

            if ($onlineOrder->payment_method == 'cod' && $onlineOrder->order_status == 'Delivered' && isset($business->business_commissions)) {
                // $this->debit_cod_fees($onlineOrder, $business);
            }

            $orderCommission = OnlineOrderCommission::where('online_order_id', $onlineOrder->id)->first();
            if (empty($orderCommission)) {
                $shouldSetCommissions = true;
                $orderCommission = new OnlineOrderCommission();
            }
            if ($shouldSetCommissions) {
                $orderCommission->online_order_id = $onlineOrder->id;
                if (!empty($business)) {
                    $orderCommission->fill(json_decode($business->business_commissions, true));
                }
            }

            $orderCommission->save();

            if (!empty($business)) {
                $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                if (!empty($reseller)) {
                    $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
                    if (empty($resellerCommission)) {
                        $resellerCommission = new ResellerCommission;
                        $resellerCommission->user_id = $reseller->user_id;
                        $resellerCommission->business_id = $business->id;
                        $resellerCommission->save();
                    }
                    $resellerOrderCommission = ResellerOrderCommission::where('online_order_id', $onlineOrder->id)->first();
                    if (empty($resellerOrderCommission)) {
                        $resellerOrderCommission = new ResellerOrderCommission();
                        $resellerOrderCommission->user_id = $reseller->user_id;
                        $resellerOrderCommission->online_order_id = $onlineOrder->id;
                        $resellerOrderCommission->fill(json_decode($resellerCommission, true));

                        $resellerOrderCommission->save();
                    }
                }
            }
        }
        return $orderList;
    }

    public function debit_cod_fees($order, $business)
    {
        $stripe = self::$stripe;
        $orderTypeWiseCommission = $this->calculateFee(($order->grand_total - $order->charity_amount), $business->business_commissions->online_commission_fee, $business->business_commissions->online_commission_fee_type);

        if ($order->payment_method == 'cod')
            $codFeeAmount  = $this->calculateFee(($order->grand_total - $order->charity_amount), $business->business_commissions->online_cod_fee, $business->business_commissions->online_cod_fee_type);

        if ($business->business_commissions->order_charge_type == 'order_based') {
            $orderBasedOrTransactionWiseCharge = $this->calculateFee(($order->grand_total - $order->charity_amount), $business->business_commissions->per_order_charge, $business->business_commissions->per_order_charge_type);
        } else {
            if ($order->payment_method == 'cod')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee(($order->grand_total - $order->charity_amount), $business->business_commissions->cod_charge, $business->business_commissions->cod_charge_type);
        }

        $codAmount = $orderTypeWiseCommission + $codFeeAmount + $orderBasedOrTransactionWiseCharge + $order->user_service_charge;

        $description = "Cod charge - $order->order_id";

        if ($codAmount > 0 && $business->s_account_id) {
            $charge = $stripe->charges->create([
                'amount'   => (int)($codAmount * 100),
                'currency' => 'gbp',
                'source' => $business->s_account_id,
                'description' => $description,
                'statement_descriptor' => $business->s_account_id
            ]);
        }

        return $order;
    }

    public function saveDineinOrder($orderList)
    {
        foreach ($orderList as $order) {

            $order_type = 'dinein';

            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            $onlineOrder = OnlineOrder::where('order_id', $order->id)->where('order_type', 'dinein')->first(); // where('order_type', $order_type)->
            if (!empty($onlineOrder)) {
                $onlineOrder->order_status = $order->status;
                $onlineOrder->save();
                continue;
            }
            if (empty($onlineOrder)) {
                $onlineOrder = new OnlineOrder();
            }
            $shouldSetCommissions = ($onlineOrder->business_id == null) ? true : false;

            $onlineOrder->order_id = $order->id;
            $onlineOrder->restaurant_id = $order->restaurant_id;
            if (!empty($business)) {
                $onlineOrder->business_id = $business->id;
                if ($business->auto_archive == 1) {
                    $onlineOrder->archived = 1;
                }
            }

            $onlineOrder->order_type = $order_type;
            if (isset($order->transaction_id)) {
                $onlineOrder->txn_id = $order->transaction_id;
            }
            // $onlineOrder->payment_method = $order->payment_method;
            $onlineOrder->payment_method = $order->payment_method == 'Stripe' ? 'Credit Card' : $order->payment_method;;
            if (isset($order->order_sub_total)) {
                $onlineOrder->sub_total = $order->order_sub_total;
            }
            if (isset($order->delivery_charge)) {
                $onlineOrder->delivery_charge = $order->delivery_charge;
            }
            if (isset($order->driver_tip)) {
                $onlineOrder->driver_tip = $order->driver_tip;
            }
            if (isset($order->gratuity)) {
                $onlineOrder->gratuity_tip = $order->gratuity;
            }
            if (isset($order->order_grand_total)) {
                $onlineOrder->grand_total = $order->order_grand_total;
            }
            if (isset($order->charity_amount)) {
                $onlineOrder->charity_amount = $order->charity_amount;
            }
            if (isset($order->voucher_amount)) {
                $onlineOrder->voucher_amount = $order->voucher_amount;
            }
            if (isset($order->service_charge) && $order->status != 'Failed') {
                $onlineOrder->user_service_charge = $order->service_charge;
            }

            $onlineOrder->order_date = $order->created;
            $onlineOrder->order_status = $order->status;

            $onlineOrder->save();
            $orderCommission = OnlineOrderCommission::where('online_order_id', $onlineOrder->id)->first();
            if (empty($orderCommission)) {
                $shouldSetCommissions = true;
                $orderCommission = new OnlineOrderCommission();
            }
            if ($shouldSetCommissions) {
                $orderCommission->online_order_id = $onlineOrder->id;
                if (!empty($business)) {
                    $orderCommission->fill(json_decode($business->business_commissions, true));
                }
            }

            $orderCommission->save();

            if (!empty($business)) {
                $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                if (!empty($reseller)) {
                    $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
                    if (empty($resellerCommission)) {
                        $resellerCommission = new ResellerCommission;
                        $resellerCommission->user_id = $reseller->user_id;
                        $resellerCommission->business_id = $business->id;
                        $resellerCommission->save();
                    }
                    $resellerOrderCommission = ResellerOrderCommission::where('online_order_id', $onlineOrder->id)->first();
                    if (empty($resellerOrderCommission)) {
                        $resellerOrderCommission = new ResellerOrderCommission();
                        $resellerOrderCommission->user_id = $reseller->user_id;
                        $resellerOrderCommission->online_order_id = $onlineOrder->id;
                        $resellerOrderCommission->fill(json_decode($resellerCommission, true));

                        $resellerOrderCommission->save();
                    }
                }
            }
        }
        return $orderList;
    }

    /**
     * Archive Bulk Transacitons.
     * POST online-orders/archive
     */
    public function archive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = OnlineOrder::whereIn('id', $req->input('ids'))->update(['archived' => '1']);
            $transactions = OnlineOrder::whereIn('id', $req->input('ids'))->get();
            foreach ($transactions as $transaction) {
                if ($transaction->payment_method == 'Credit Card') {
                    $transactions = StripeTransaction::where('payment_intent_id', $transaction->txn_id)->update(['archived' => '1']);
                }
                if ($transaction->payment_method == 'paypal') {
                    $transactions = PaypalPayment::where('sale_id', $transaction->txn_id)->update(['archived' => '1']);
                }
            }
            // $transactions = PaypalPayment::whereRaw('(sale_id IN (SELECT txn_id from online_orders WHERE id IN (?)))', $req->input('ids'))->update(['archived' => '1']);
            // $transactions = StripeTransaction::whereRaw('(payment_intent_id IN (SELECT txn_id from online_orders WHERE ids IN (?)))', $req->input('ids'))->update(['archived' => '1']);
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }


    /**
     * UnArchive Bulk Transacitons.
     * POST online-orders/unarchive
     */
    public function unarchive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = OnlineOrder::whereIn('id', $req->input('ids'))->update(['archived' => '0']);
            $transactions = OnlineOrder::whereIn('id', $req->input('ids'))->get();
            foreach ($transactions as $transaction) {
                if ($transaction->payment_method == 'Credit Card') {
                    $transactions = StripeTransaction::where('payment_intent_id', $transaction->txn_id)->update(['archived' => '0']);
                }
                if ($transaction->payment_method == 'paypal') {
                    $transactions = PaypalPayment::where('sale_id', $transaction->txn_id)->update(['archived' => '0']);
                }
            }
            // $transactions = PaypalPayment::whereRaw('(sale_id IN (SELECT txn_id from online_orders WHERE id IN (?)))', $req->input('ids'))->update(['archived' => '0']);
            // $transactions = StripeTransaction::whereRaw('(payment_intent_id IN (SELECT txn_id from online_orders WHERE id IN (?)))', $req->input('ids'))->update(['archived' => '0']);
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }

    /**
     * Refund Online order.
     * POST online-orders/{id}/refund/
     * @param int amount
     * @param string password
     */
    public function refund(Request $req, $id)
    {
        $this->validate($req, [
            'amount' => 'gt:0|numeric|required',
            'password' => 'required', // Human amount
        ]);
        if (Auth::isAdmin()) {
            if (!Hash::check($req->input('password'), Auth::admin()->password)) {
                Utils::error("Password is incorrect", 400);
            }
        }
        if (Auth::isUser()) {
            if (!Hash::check($req->input('password'), Auth::user()->password)) {
                Utils::error("Password is incorrect", 400);
            }
        }

        $onlineOrder = OnlineOrder::find($id);
        if (empty($onlineOrder)) Utils::error("Order not found", 404);
        if ($onlineOrder->archived == 1) Utils::error("Archived Order cannot be refunded", 400);
        if (strtolower($onlineOrder->payment_method) == 'cod') Utils::error("COD Order cannot be refunded", 400);
        if (strtolower($onlineOrder->order_status) != 'delivered') Utils::error("Only delivered order can be refunded", 400);
        if ($req->input('amount') > $onlineOrder->grand_total) {
            Utils::error("Refund amount should not be greater than order total", 400);
        }
        if (strtolower($onlineOrder->payment_method) == 'paypal') {
            //Refund Paypal
            error_log($onlineOrder->txn_id);
            $paypalPayment = PaypalPayment::where('sale_id',  $onlineOrder->txn_id)->latest()->first();
            if (empty($paypalPayment)) {
                Utils::error("No transaction found with this order", 404);
            }
            $refundResponse = (new PaypalPaymentController())->refund($req, $paypalPayment->id);
            return $onlineOrder;
        }
        if (strtolower($onlineOrder->payment_method) == 'credit card' || strtolower($onlineOrder->payment_method) == 'online' || strtolower($onlineOrder->payment_method) == 'card' || strtolower($onlineOrder->payment_method) == 'stripe') {
            //Refund Stripe
            $stripeTransaction = StripeTransaction::where('payment_intent_id', $onlineOrder->txn_id)->latest()->first();
            if (empty($stripeTransaction)) {
                Utils::error("No transaction found with this order", 404);
            }
            $refundResponse = (new StripeTransactionController())->refund($req, $stripeTransaction->id);
            //TODO Change online order status if require.
            return $onlineOrder;
        }
    }

    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}