<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\StripeTransaction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Stripe\StripeClient;

class PayViaDirectController extends Controller
{
    /**
     * call inner page view.
     * GET /api/pay-via-direct
     */
    public function index(Request $req)
    {

        $this->validate($req, [
            'amount' => 'gt:0|numeric|required',
            'business_id' => 'required|string',
            'success_url' => 'required|url',
            'currency' => 'required|string',
            'customer_name' => 'string',
            'customer_email' => 'string',
            'failed_url' => 'url',
        ]);
        $name = '';
        $email = '';
        if ($req->has("customer_name")) {
            $name = '&payment_customer_name=' . $req->input("customer_name");
        }
        if ($req->has("customer_email")) {
            $email = '&payment_customer_email=' . $req->input("customer_email");
        }

        $link = '/direct-payment?key=' . base64_encode('payment_direct_amount=' . $req->input("amount") . '&payment_direct_business_id=' . $req->input("business_id") . '&payment_success_url=' . $req->input("success_url") . '&payment_failed_url=' . $req->input("failed_url") . '&payment_currency=' . $req->input("currency") . $name . $email);
        return redirect($link);
    }
    /**
     * Refund Stripe Transaction.
     * POST /stripe-transactions/{id}/refund
     * @param int amount
     * @param string password
     */
    public function refund(Request $req, $id)
    {
        $this->validate($req, [
            'amount' => 'required|numeric|gt:0'  // Human Amount
        ]);

        $Mode = SiteSettingController::findByKey('connect_stripe_mode');
        if (!empty($Mode)) {
            $stripeMode = trim($Mode->value);
        }
        if ($stripeMode == 'test') {
            $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        $stripe = new StripeClient(["api_key" => $stripeKey, "stripe_version" => "2020-08-27"]);

        // retrive payment intent
        $paymentIntent =  $stripe->paymentIntents->retrieve($id);
        $txn_id = $paymentIntent->charges->data[0]->balance_transaction;

        $stripeTransaction = StripeTransaction::where('txn_id', $txn_id)->first();
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);

        if ($stripeTransaction->source == null) {
            Utils::error("This transaction has no source", 400);
        }
        if ($stripeTransaction->reporting_category != 'charge') {
            Utils::error("This transaction cannot be refund", 400);
        }
        if ($stripeTransaction->refunded) {
            Utils::error("This transaction already refunded", 400);
        }
        if ($stripeTransaction->service == 'Connect' && $stripeTransaction->transfer_status != 'pending') {
            // Utils::error("Cannot refund this transaction as it may not available", 400);
        }
        $account_type = $stripeTransaction->account_type;
        $stripeMode = "test";
        $stripeMode = $stripeTransaction->mode;

        if ($account_type == 'normal') {
            $business = Business::find($stripeTransaction->business_id);
            if (!empty($business)) {
                $mode = $stripeTransaction->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);
        $chargeDetails = $stripe->charges->retrieve($stripeTransaction->source);
        $refundParams = [
            'charge' => $stripeTransaction->source,
            'amount' => $req->input('amount') * 100
        ];
        if ($chargeDetails->transfer_data != null) {
            $refundParams['reverse_transfer'] = true;
        }
        try {
            $stripe->refunds->create($refundParams);
            Log::error("Refund Complete");
        } catch (\Stripe\Exception\InvalidRequestException $e) {
            // Invalid parameters were supplied to Stripe's API
            Log::error("Invalid Request Exception");
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\CardException $e) {
            // Since it's a decline, \Stripe\Exception\CardException will be caught
            Log::error("Card Exception");
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\ApiErrorException $e) {
            // Display a very generic error to the user, and maybe send
            // yourself an email
            Log::error("Api Error Exception");
            Utils::error($e->getError()->message, 400);
        }
        return $stripeTransaction;
    }
}
