<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\PayoutCharge;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class PayoutChargeController extends Controller
{
    /**
     * List payoutCharges.
     * GET /api/payout-charges
     */
    public function index(Request $req)
    {
        $payoutCharges = PayoutCharge::orderByRaw("amount DESC");

        if ($query = $req->query('business_id', false)) {
            $payoutCharges->where('business_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $payoutCharges = $payoutCharges->get();
        } else {
            $payoutCharges = $payoutCharges->paginate($req->query('per_page', 20));
        }

        if ($payoutCharges->isEmpty()) Utils::error('No payoutCharges found!', 404);


        return ($payoutCharges);
    }

    /**
     * Create payoutCharge.
     * POST /api/payout-charges
     */
    public function create(Request $req)
    {
        $this->validate($req, [

            'business_id' => 'required|exists:businesses,id',
            'amount' => 'required|numeric',
            'charge' => 'required|numeric',

        ]);

        // create payoutCharge
        $payoutCharge = new PayoutCharge;
        $payoutCharge->fill($req->all());

        $payoutCharge->save();

        return $payoutCharge;
    }

    /**
     * Show payoutCharge.
     * GET /api/payout-charges/{id}
     */
    public function show(Request $req, $id)
    {
        $payoutCharge = PayoutCharge::find($id);
        if (empty($payoutCharge)) Utils::error('PayoutCharge not found!', 404);
        return $payoutCharge;
    }

    /**
     * Update payoutCharge.
     * POST /api/payout-charges/{id}
     */
    public function update(Request $req, $id)
    {
        $payoutCharge = PayoutCharge::find($id);
        if (empty($payoutCharge)) Utils::error('PayoutCharge not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'amount' => 'required|numeric',
            'charge' => 'required|numeric',
        ]);

        if ($req->has('business_id')) $payoutCharge->business_id = $req->input('business_id');

        $payoutCharge->amount = $req->input('amount');
        $payoutCharge->charge = $req->input('charge');

        $payoutCharge->save();

        return $payoutCharge;
    }


    /**
     * Delete payoutCharge by id.
     * DELETE /api/payout-charges/{id}
     */
    public function delete(Request $req, $id)
    {
        $payoutCharge = PayoutCharge::find($id);
        if (empty($payoutCharge)) Utils::error('PayoutCharge not found!', 404);
        $payoutCharge->delete();
        return $payoutCharge;
    }
}
