<?php

namespace App\Http\Controllers;

use App\Business;
use App\CaxtonCard;
use App\CaxtonWallet;
use App\Exports\AdminMiscReportExport;
use App\Exports\AdminPayoutReportExport;
use App\Exports\AdminWalletLoadReportExport;
use App\Exports\MultiplePayoutReportExport;
use App\Exports\PayoutReportExport;
use App\PayoutInvoice;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsConnect\ExternalAccount;
use App\Passbook;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;

class PayoutInvoiceController extends Controller
{
    /**
     * List payoutInvoices.
     * GET /api/payout-invoices
     */
    public function index(Request $req)
    {
        $payoutInvoices = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('businesses.disabled', '0')->orderByRaw('to_date DESC, businesses.name ASC');

        if ($query = $req->query('query', false)) {
            $payoutInvoices->whereRaw('(comment LIKE ? OR name LIKE ? OR gross_total LIKE ? OR gross_fee LIKE ? OR transaction_type LIKE ? OR payout_invoices.payout_mode LIKE ? OR payout_invoices.payout_method LIKE ? OR net_total LIKE ? OR net_fee LIKE ?  )', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $payoutInvoices->where('business_id', $query);
        }

        if ($query = $req->query('transaction_type', false)) {
            $payoutInvoices->where('transaction_type', $query);
        }
        if ($query = $req->query('status', false)) {
            $payoutInvoices->where('status', $query);
        }
        if ($query = $req->query('payout_mode', false)) {
            $payoutInvoices->where('payout_mode', $query);
        }
        if ($query = $req->query('payout_method', false)) {
            $payoutInvoices->where('payout_method', $query);
        }
        if ($query = $req->query('from_date', false)) {
            $payoutInvoices->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $payoutInvoices->where('to_date', '<=', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $payoutInvoices = $payoutInvoices->get();
        } else {
            $payoutInvoices = $payoutInvoices->paginate($req->query('per_page', 20));
        }

        if ($payoutInvoices->isEmpty()) Utils::error('No payoutInvoices found!', 404);
        $payoutInvoices->append('payable_tot_amount');
        return ($payoutInvoices);
    }

    /*
    |---------------------------------------
    | Modern Report
    | GET /api/payout-invoices/modern
    |---------------------------------------
    */
    public function modern(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
        ]);
        $fromDate = $req->query('from_date');
        $toDate = $req->query('to_date');
        $normalAutoBankPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('payout_invoices.payout_mode', 'Auto')->where('payout_invoices.payout_method', 'Bank')->orderByRaw('businesses.name ASC')->get();
        $normalAutoPaypalPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('payout_invoices.payout_mode', 'Auto')->where('payout_invoices.payout_method', 'Paypal')->orderByRaw('businesses.name ASC')->get();
        // $normalAutoCashPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date','>=', $req->query('from_date'))->where('to_date','<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('payout_invoices.payout_mode', 'Auto')->where('payout_invoices.payout_method', 'Cash')->orderByRaw('businesses.name ASC')->get();

        $normalManualBankPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('payout_invoices.payout_mode', 'Manual')->where('payout_invoices.payout_method', 'Bank')->orderByRaw('businesses.name ASC')->get();
        $normalManualPaypalPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('payout_invoices.payout_mode', 'Manual')->where('payout_invoices.payout_method', 'Paypal')->orderByRaw('businesses.name ASC')->get();
        $normalManualCashPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('payout_invoices.payout_mode', 'Manual')->where('payout_invoices.payout_method', 'Cash')->orderByRaw('businesses.name ASC')->get();

        $manualPayouts =  PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->whereRaw('(payout_invoices.payout_mode = ? OR (payout_invoices.payout_mode = ? AND payout_invoices.payout_method = ? ))', ["Manual", "Auto", "Cash"])->orderByRaw('businesses.name ASC')->get();

        $archivedAutoBankPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Archived')->where('payout_invoices.payout_mode', 'Auto')->where('payout_invoices.payout_method', 'Bank')->orderByRaw('businesses.name ASC')->get();
        $archivedAutoPaypalPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Archived')->where('payout_invoices.payout_mode', 'Auto')->where('payout_invoices.payout_method', 'Paypal')->orderByRaw('businesses.name ASC')->get();
        $archivedAutoCashPayouts = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Archived')->where('payout_invoices.payout_mode', 'Auto')->where('payout_invoices.payout_method', 'Cash')->orderByRaw('businesses.name ASC')->get();

        $normalPayoutSummary = PayoutInvoice::selectRaw("
        (
            SELECT
                SUM(net_total)
            FROM
                payout_invoices
            WHERE
                payout_mode = 'Auto' AND payout_method = 'Bank'
                AND from_date >= '$fromDate' AND to_date <='$toDate'
                limit 1
        ) AS 'bank_payout',
        (
            SELECT
                SUM(net_total)
            FROM
                payout_invoices
            WHERE
                payout_mode = 'Auto' AND payout_method = 'Paypal'
                AND from_date >= '$fromDate' AND to_date <='$toDate'
                limit 1
        ) AS 'paypal_payout',
        (
            SELECT
                SUM(net_total)
            FROM
                payout_invoices
            WHERE
                payout_mode = 'Auto' AND payout_method = 'Cash'
                AND from_date >= '$fromDate' AND to_date <='$toDate'
                limit 1
        ) AS 'cash_payout',
        (
            SELECT
                SUM(net_total)
            FROM
                payout_invoices
            WHERE
                payout_mode = 'Manual'
                AND from_date >= '$fromDate' AND to_date <='$toDate'
                limit 1
        ) AS 'manual_payout',
        (
            SELECT SUM(net_total)
            FROM
                payout_invoices 
            WHERE from_date >= '$fromDate' AND to_date <='$toDate'
            limit 1
        )as total
        
        ")->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->first();

        if (empty($normalPayoutSummary)) Utils::error("No data found for selected range", 400);
        $data['normal_auto_bank_payouts'] = $normalAutoBankPayouts;
        $data['normal_auto_paypal_payouts'] = $normalAutoPaypalPayouts;
        $data['normal_auto_cash_payouts'] = null;

        $data['archived_auto_bank_payouts'] = $archivedAutoBankPayouts;
        $data['archived_auto_paypal_payouts'] = $archivedAutoPaypalPayouts;
        $data['archived_auto_cash_payouts'] = $archivedAutoCashPayouts;

        $data['normal_manual_bank_payouts'] = $normalManualBankPayouts;
        $data['normal_manual_paypal_payouts'] = $normalManualPaypalPayouts;
        $data['normal_manual_cash_payouts'] = $normalManualCashPayouts;
        $data['manual_payouts'] = $manualPayouts;

        $data['normal_payout_summary'] = $normalPayoutSummary;
        $data['from_date'] = $fromDate;
        $data['to_date'] = $toDate;
        return $data;
    }

    public function export(Request $req)
    {
        $exportableData = ($this->modern($req));
        $fromDate = Carbon::parse($exportableData['from_date']);
        $toDate = Carbon::parse($exportableData['to_date']);
        $fileName = $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
        $type = $req->query('format', 'excel');
        if ($type == 'excel')
            return Excel::download(new AdminPayoutReportExport($exportableData), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new AdminPayoutReportExport($exportableData), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function walletLoadExport(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
        ]);
        $fromDate = $req->query('from_date');
        $toDate = $req->query('to_date');
        $data = [];

        $business_ids = PayoutInvoice::select('id', 'business_id', 'net_total')
            ->where('from_date', '>=', $fromDate)
            ->where('to_date', '<=', $toDate)
            ->where('transaction_type', 'Archived')
            ->where('payout_mode', 'Auto')
            ->whereIn('payout_method', ['Bank', 'Paypal', 'Cash'])
            ->get()
            ->pluck('business_id', 'net_total')
            ->toArray();
        $business_ids = array_unique($business_ids);

        $caxton_wallets = CaxtonWallet::whereIn('business_id', $business_ids)->select('id', 'name', 'business_id', 'wallet_id')->get();
        $caxton_cards = CaxtonCard::whereIn('wallet_id', $caxton_wallets->pluck('wallet_id')->toArray())->get();

        foreach ($business_ids as $net_total => $business_id) {
            $business_wallets = $caxton_wallets->where('business_id', $business_id);
            foreach ($business_wallets as $wallet) {
                print_r($net_total);
                // echo "<pre>";
                // echo "</pre>";
                $business_cards = $caxton_cards->where('wallet_id', $wallet->wallet_id);
                foreach ($business_cards as $card) {
                    $data[$card->card_id]['CardHolderName'] = $card->card_holder_name;
                    $data[$card->card_id]['WalletGroupName'] = $wallet->name;
                    $data[$card->card_id]['WalletId'] = $wallet->wallet_id;
                    $data[$card->card_id]['LoadCurrency'] = 'GBP';
                    $data[$card->card_id]['LoadAmount'] = $net_total;
                }
            }
        }
        die;

        $fileName = $fromDate . " to " . $toDate;

        return Excel::download(new AdminWalletLoadReportExport($data, $fromDate, $toDate), "$fileName.csv", \Maatwebsite\Excel\Excel::CSV);
    }

    public function paymentExport(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
        ]);
        $fromDate = $req->query('from_date');
        $toDate = $req->query('to_date');
        $data = [];

        $business_ids = PayoutInvoice::select('id', 'business_id')
            ->where('from_date', '>=', $fromDate)
            ->where('to_date', '<=', $toDate)
            ->where('transaction_type', 'Normal')
            ->where('payout_mode', 'Manual')
            ->whereIn('payout_method', ['Bank', 'Paypal', 'Cash'])
            ->get()
            ->pluck('business_id')
            ->toArray();



        return $business_ids;
    }

    /*
    |---------------------------------------
    | Misc Modern Report
    | GET /api/payout-invoices/misc-report
    |---------------------------------------
    */
    public function modern_misc(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
        ]);
        $fromDate = $req->query('from_date');
        $toDate = $req->query('to_date');
        $miscReport = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('misc_charge', '>', 0)->orderByRaw('businesses.name ASC')->get();

        $totalSummary = PayoutInvoice::selectRaw("
        (
            SELECT SUM(misc_charge)
            FROM
                payout_invoices 
            WHERE from_date >= '$fromDate' AND to_date <='$toDate'
            limit 1
        )as total
        
        ")->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('misc_charge', '>', 0)->first();

        if (empty($totalSummary)) Utils::error("No data found for selected range", 400);
        $data['misc_report'] = $miscReport;
        $data['total__summary'] = $totalSummary;
        $data['from_date'] = $fromDate;
        $data['to_date'] = $toDate;
        return $data;
    }


    public function export_misc(Request $req)
    {
        $exportableData = ($this->modern_misc($req));
        $fromDate = Carbon::parse($exportableData['from_date']);
        $toDate = Carbon::parse($exportableData['to_date']);
        $fileName = $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
        $type = $req->query('format', 'excel');
        if ($type == 'excel')
            return Excel::download(new AdminMiscReportExport($exportableData), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new AdminMiscReportExport($exportableData), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function export_all(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
        ]);
        $fromDate = $req->query('from_date');
        $toDate = $req->query('to_date');

        $payoutInvoices = PayoutInvoice::select("payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->where('businesses.disabled', '0')->orderByRaw('to_date DESC, businesses.name ASC');
        if ($query = $req->query('from_date', false)) {
            $payoutInvoices->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $payoutInvoices->where('to_date', '<=', $query);
        }
        $payoutInvoices = $payoutInvoices->get();
        if ($payoutInvoices->isEmpty()) Utils::error("No invoices found", 400);
        $businesses = [];

        $payoutInvoices->each(function ($invoice) {
            $invoice->append('transactions');
            $invoice->append('business');
            $businesses[] = $invoice;
        });
        $fromDate = Carbon::parse($fromDate);
        $toDate = Carbon::parse($toDate);

        $fileName = $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');

        $type = $req->query('format', 'excel');

        if ($type == 'excel')
            return Excel::download(new MultiplePayoutReportExport($payoutInvoices), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new MultiplePayoutReportExport($payoutInvoices), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }
    /**
     * Create payoutInvoice.
     * POST /api/payout-invoices
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:business,id',
            'gross_total' => 'numeric',
            'misc_charge' => 'numeric',
            'weekly_fee' => 'numeric',
            'monthly_fee' => 'numeric',
            'total_fee' => 'numeric',
            'net_total' => 'numeric',
            'paid_amount' => 'numeric',
            'transaction_type' => 'required|in:Normal,Archived',
            'payout_mode' => 'required|in:Manual,Auto',
            'payout_method' => 'required|in:Cash,Bank,Paypal',
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
            'status' => 'required|in:Pending,Settled,Partially Settled',
            'comment' => 'string',
        ]);

        // create payoutInvoice
        $payoutInvoice = new PayoutInvoice;
        $payoutInvoice->fill($req->all());

        $payoutInvoice->save();

        return $payoutInvoice;
    }

    /**
     * Show payoutInvoice.
     * GET /api/payout-invoices/{id}
     */
    public function show(Request $req, $id)
    {
        $payoutInvoice = PayoutInvoice::select("payout_invoices.*", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->find($id);
        if (empty($payoutInvoice)) Utils::error('PayoutInvoice not found!', 404);
        $payoutInvoice->append('transactions');
        $payoutInvoice->append('surcharges');
        $payoutInvoice->append('invoices');
        $payoutInvoice->append('business');
        $payoutInvoice->append('payable_tot_amount');
        Log::channel('debug')->info([$payoutInvoice->toArray()]);
        return $payoutInvoice;
    }

    /**
     * Download payoutInvoice.
     * GET /api/payout-invoices/{id}/download
     */
    public function download(Request $req, $id)
    {
        $payoutInvoice = PayoutInvoice::select("payout_invoices.*", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'payout_invoices.business_id')->find($id);
        if (empty($payoutInvoice)) Utils::error('PayoutInvoice not found!', 404);
        $payoutInvoice->append('transactions');
        $payoutInvoice->append('business');
        $payoutInvoice->append('payable_tot_amount');
        $type = 'excel';
        if ($format = $req->query('format', false)) {
            $type = $format;
        }
        $fromDate = Carbon::parse($payoutInvoice['from_date']);
        $toDate = Carbon::parse($payoutInvoice['to_date']);
        $fileName = $payoutInvoice['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');

        if ($type == 'excel')
            return Excel::download(new PayoutReportExport($payoutInvoice), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new PayoutReportExport($payoutInvoice), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }
    /**
     * Update payoutInvoice.
     * POST /api/payout-invoices/{id}
     */
    public function update(Request $req, $id)
    {
        $payoutInvoice = PayoutInvoice::find($id);
        if (empty($payoutInvoice)) Utils::error('PayoutInvoice not found!', 404);

        $this->validate($req, [
            'business_id' => 'exists:business,id',
            'gross_total' => 'numeric',
            'misc_charge' => 'numeric',
            'weekly_fee' => 'numeric',
            'monthly_fee' => 'numeric',
            'total_fee' => 'numeric',
            'net_total' => 'numeric',
            'paid_amount' => 'numeric',
            'transaction_type' => 'in:Normal,Archived',
            'payout_mode' => 'in:Manual,Auto',
            'payout_method' => 'in:Cash,Bank,Paypal',
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'date_format:Y-m-d|after:from_date',
            'status' => 'in:Pending,Settled,Partially Settled',
            'comment' => 'string',
        ]);

        // update payoutInvoice
        if ($req->has('gross_total')) $payoutInvoice->gross_total = $req->input('gross_total');
        if ($req->has('misc_charge')) $payoutInvoice->misc_charge = $req->input('misc_charge');
        if ($req->has('weekly_fee')) $payoutInvoice->weekly_fee = $req->input('weekly_fee');
        if ($req->has('monthly_fee')) $payoutInvoice->monthly_fee = $req->input('monthly_fee');
        if ($req->has('total_fee')) $payoutInvoice->total_fee = $req->input('total_fee');
        if ($req->has('net_total')) $payoutInvoice->net_total = $req->input('net_total');
        if ($req->has('transaction_type')) $payoutInvoice->transaction_type = $req->input('transaction_type');
        if ($req->has('payment_mode')) $payoutInvoice->payment_mode = $req->input('payment_mode');
        if ($req->has('payment_method')) $payoutInvoice->payment_method = $req->input('payment_method');
        if ($req->has('from_date')) $payoutInvoice->from_date = $req->input('from_date');
        if ($req->has('to_date')) $payoutInvoice->to_date = $req->input('to_date');
        if ($req->has('status')) $payoutInvoice->status = $req->input('status');
        if ($req->has('comment')) $payoutInvoice->comment = $req->input('comment');

        $payoutInvoice->save();

        return $payoutInvoice;
    }


    /**
     * Delete payoutInvoice by id.
     * DELETE /api/payout-invoices/{id}
     */
    public function delete(Request $req, $id)
    {
        $payoutInvoice = PayoutInvoice::find($id);
        if (empty($payoutInvoice)) Utils::error('PayoutInvoice not found!', 404);
        $payoutInvoice->delete();

        #Check if passbook entry is there or not and then delete
        $passbook = Passbook::where('entity_id', $payoutInvoice->id)->where('entity', 'payout_invoice')->where('business_id', $payoutInvoice->business_id)->first();
        if (!empty($passbook)) $passbook->delete();

        return $payoutInvoice;
    }
    public function pay_bulk_invoices(Request $req)
    {
        if ($req->has('invoices') && !is_array($req->input('invoices'))) {
            $req->merge(['invoices' => json_decode($req->input('invoices'))]);
        }
        $this->validate($req, [
            'invoices' => 'required|array'
        ], ['invoices.required' => 'No invoices updated']);
        $invoices = ($req->input('invoices'));
        $invoiceIds = [];
        $finalInvoices = [];
        foreach ($invoices as $invoice) {
            $dbInvoice = PayoutInvoice::find($invoice->id);
            if (empty($dbInvoice)) continue;
            $dbBusiness = Business::find($dbInvoice->business_id);
            $payableAmount = ($invoice->net_total > 0 ? -1 * ($invoice->net_total - (-1 * ($invoice->total_paid_amount))) : (-1 * ($invoice->net_total)) - $invoice->total_paid_amount);
            if ($payableAmount > 0 && $invoice->paid_amount > $payableAmount) {
                Utils::error("Paid amount should not be greater than payable amount for " . $dbBusiness->name, 400);
            }
            if ($payableAmount < 0 &&  $invoice->paid_amount < $payableAmount) {
                Utils::error("Paid amount should not be lesser than payable amount for " . $dbBusiness->name, 400);
            }
            $invoice->business_id = $dbInvoice->business_id;
            $invoice->from_date = $dbInvoice->from_date;
            $invoice->to_date = $dbInvoice->to_date;
            $finalInvoices[] = $invoice;
        }
        foreach ($finalInvoices as $invoice) {
            $invoiceIds[] = $invoice->id;
            $passbook = new Passbook();
            $passbook->business_id = $invoice->business_id;
            $passbook->amount = $invoice->paid_amount;
            $passbook->entity = "payout_invoice";
            $passbook->entity_id = $invoice->id;
            $passbook->date = Carbon::today();
            $passbook->from_date = $invoice->from_date;
            $passbook->to_date = $invoice->to_date;
            if ($invoice->paid_amount > 0) {
                $passbook->description = "Invoice settled for $invoice->from_date to $invoice->to_date";
            } else {
                $passbook->description = "Invoice paid for $invoice->from_date to $invoice->to_date";
            }

            $passbook->save();
        }
        if (!empty($invoices)) {
            $invoices = PayoutInvoice::whereRaw('(id IN (?))', $invoiceIds)->get();
        }
        return $invoices;
    }
}
