<?php

namespace App\Http\Controllers;

use App\Business;
use App\Invoice;
use App\BusinessSurcharge;
use App\CardBrandCommissionHistory;
use App\CountryCommissionHistory;
use App\Exports\MultiplePayoutReportExport;
use App\Exports\PayoutReportExport;
use App\Helpers\Utils;
use App\InvoicePayment;
use App\InvoiceSurcharge;
use App\InvoiceTransaction;
use App\MiscCharge;
use App\MiscInvoice;
use App\ModelsConnect\ExternalAccount;
use App\OnlineOrder;
use App\Passbook;
use App\PayoutCharge;
use App\PayoutInvoice;
use App\PaypalPayment;
use App\StripeTransaction;
use App\User;
use App\UserBusiness;
use Carbon\Carbon;
use DateTime;
use Illuminate\Support\Facades\Queue;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;
use Stripe\StripeClient;

class PayoutReportController extends Controller
{

    /**
     * @param  Request  $req
     * @param  string  $business_id
     *-----------------------------------------
     * Get Report from to to date all businesses
     * GET /businesses/{business_id}/report
     */
    public function all_business_quick_report(Request $req)
    {
        $req->merge(['is_job' => "true"]);
        $businesses = Business::where('disabled', '0')->orderBy('id', 'DESC')->get();

        $carbon = Carbon::today()->subDays(7);

        $currentDayOfWeek = $carbon->dayOfWeek;
        if ($currentDayOfWeek <= 1) {
            $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
            $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
        } else {
            $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
            $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
        }

        $req->merge(['from_date' => $from_date]);
        $req->merge(['to_date' => $to_date]);

        foreach ($businesses as $business) {
            $this->quick_report($req, $business->id);
        }

        return ["message" => "Invoice Generated Successfully!"];
    }

    /**
     * Get Report by user from to to date
     * GET /generate-report/{userId}
     */
    public function generateReportByUser(Request $req, $userId)
    {
        set_time_limit(86400);
        $userBusinesses = UserBusiness::where('user_id', $userId)->select('business_id')->get()->pluck('business_id')->toArray();
        $businesses = Business::whereIn('id', $userBusinesses)->where('disabled', '0')->get();
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);
        $req->merge(['save' => "true"]);
        $reports = [];
        foreach ($businesses as $business) {
            $reports[] = (new ResellerPayoutReportController())->quick_report($req, $userId, $business->id);
        }
        return $reports;
    }

    /**
     * Get Report from to to date
     * GET /generate-report
     */
    public function generate_report(Request $req)
    {
        $businesses = Business::where('disabled', '0')->get();
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);

        $req->merge(['save' => "true"]);
        $time = Carbon::now();
        foreach ($businesses as $business) {
            try {
                $time->addMinute();
                Queue::later($time, (new PayoutReportController())->quick_report($req, $business->id));
                // $fromDate = Carbon::parse($req->input('from_date'))->format('Y-m-d');
                // $toDate = Carbon::parse($req->input('to_date'))->format('Y-m-d');
                // $isConnectAccount = $business->connect_service;
                // $accountType = $isConnectAccount ? 'Connect' : 'Normal';

                // $payoutInvoice = PayoutInvoice::where('from_date', $fromDate)->where('to_date', $toDate)->where('account_type', $accountType)->where('business_id', $business->id)->first(); // where('transaction_type', $report['transaction_type'])->
                // if (empty($payoutInvoice)) {
                //     $payoutInvoice = new PayoutInvoice();
                // }
                // $payoutInvoice->business_id = $business->id;
                // $payoutInvoice->from_date = $fromDate;
                // $payoutInvoice->to_date = $toDate;
                // $payoutInvoice->save();
            } catch (\Exception $e) {
                continue;
            }
        }

        return $businesses;
    }

    /**
     * @param  Request  $req
     * @param  string  $business_id
     *-----------------------------------------
     * Get Report from to to date with business id
     * GET /businesses/{business_id}/report
     */
    public function quick_report(Request $req, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $business->append('default_bank_detail');
        $business->append('default_paypal_detail');
        $business->makeHidden(['business_commissions']);

        $fromDate = Carbon::parse($req->input('from_date'))->format('Y-m-d');
        $toDate = Carbon::parse($req->input('to_date'))->format('Y-m-d');
        $req->merge(['from_date' => $fromDate]);
        $req->merge(['to_date' => $toDate]);

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ]);

        if ($req->has('save')) {
            $carbon = Carbon::today()->subDays(7);

            $currentDayOfWeek = $carbon->dayOfWeek;
            if ($currentDayOfWeek <= 1) {
                $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
            } else {
                $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
            }
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";

            $req->merge(['from_date' => $from_date]);
            $req->merge(['to_date' => $to_date]);
        } else {
            if ($req->has('from_date')) {
                $from_date = $req->input('from_date') . " 00:00:00";
                $to_date = $req->input('to_date') . " 23:59:59";
            } else {
                // $to_date = Carbon::now()->endOfDay()->format('Y-m-d H:i:s');
                // $from_date = Carbon::now()->subDays(11)->startOfDay()->format('Y-m-d H:i:s');
                $carbon = Carbon::today()->subDays(7);

                $currentDayOfWeek = $carbon->dayOfWeek;
                if ($currentDayOfWeek <= 1) {
                    $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                    $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
                } else {
                    $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
                    $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
                }
            }
        }

        return $this->setupTransactions($req, $business, $business_id, $from_date, $to_date);
    }

    public function setupTransactions($req, $business, $business_id, $from_date, $to_date)
    {
        $isConnectAccount = $business->connect_service;
        $accountType = $isConnectAccount ? 'Connect' : 'Normal';
        $chargeDescription = $business->statement_desciptor;

        $data = $this->transactionQueries($from_date, $to_date, 0, $business_id, $chargeDescription, $isConnectAccount);
        $data['from_date'] = $from_date;
        $data['to_date'] = $to_date;
        $data['business'] = $business;

        $data['misc_charge'] = 0;
        $data['weekly_fee'] = 0;
        $data['monthly_fee'] = 0;

        $data['transaction_type'] = 'Normal';
        $data['payout_mode'] = $business->payout_mode;
        $data['payout_method'] = $business->payout_method;
        $data['payout_comment'] = $business->payout_comment;
        $data['surcharges'] = $business->normal_surcharges;
        $data['account_type'] = $isConnectAccount ? 'Connect' : 'Normal';

        if (strtolower($business->payout_method) == 'paypal') {
            if ($business->default_paypal_detail != null) {
                $data['payout_account'] = $business->default_paypal_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else if (strtolower($business->payout_method) == 'bank') {
            if ($business->default_bank_detail != null) {
                $data['payout_account'] = $business->default_bank_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else {
            $data['payout_account'] = "";
        }

        $mergedData =  $this->mergeAllTransactionsAndSort($data);

        $archivedData = $this->transactionQueries($from_date, $to_date, 1, $business_id, $chargeDescription, $isConnectAccount);
        $archivedData['from_date'] = $from_date;
        $archivedData['to_date'] = $to_date;
        $archivedData['business'] = $business;
        $archivedData['misc_charge'] = 0;
        $archivedData['weekly_fee'] = 0;
        $archivedData['monthly_fee'] = 0;
        $archivedData['transaction_type'] = 'Archived';
        $archivedData['payout_mode'] = $business->archived_payout_mode;
        $archivedData['payout_method'] = $business->archived_payout_method;
        $archivedData['payout_comment'] = $business->payout_comment;
        $archivedData['surcharges'] = $business->archive_surcharges;
        $archivedData['account_type'] = $isConnectAccount ? 'Connect' : 'Normal';

        if (strtolower($business->archived_payout_method) == 'paypal') {
            if ($business->default_paypal_detail != null) {
                $archivedData['payout_account'] = $business->default_paypal_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else if (strtolower($business->archived_payout_method) == 'bank') {
            if ($business->default_bank_detail != null) {
                $archivedData['payout_account'] = $business->default_bank_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else {
            $archivedData['payout_account'] = "";
        }

        $mergedArchivedData =  $this->mergeAllTransactionsAndSort($archivedData);

        // Create emi for misc invoice amount
        $payoutInvoice = PayoutInvoice::where('from_date', explode(" ", $mergedData['from_date'])[0])->where('to_date', explode(" ", $mergedData['to_date'])[0])->where('transaction_type', $mergedData['transaction_type'])->where('business_id', $mergedData['business']['id'])->first();

        $miscInvoices = MiscInvoice::where('payment_mode', 'percentage')->where('business_id', $business->id)->where('status', '!=', 'Paid')->get();
        $miscInvoicesAmount = 0;
        if (!$miscInvoices->isEmpty()) {
            foreach ($miscInvoices as $invoice) {
                $total_paid_amount = $invoice->total_paid_amount;
                if (!empty($payoutInvoice))
                    $total_paid_amount = -1 * Passbook::where('business_id', $business->id)->where('entity_id', $invoice->id)->where('entity', 'misc_invoice')->where('ref_entity_id', '!=', $payoutInvoice->id)->where('ref_entity', '!=', 'payout_invoice')->sum('amount');
                $payableAmount = $invoice->grand_total - $total_paid_amount;
                if ($payableAmount > 0) {
                    $actualEmi = $this->calculateFee($invoice->grand_total, $invoice->percentage, 'percentage');
                    if ($actualEmi < $payableAmount) {
                        $actualEmi = $this->calculateFee($payableAmount, $invoice->percentage, 'percentage');
                    } else {
                        $actualEmi = $payableAmount;
                    }
                    $miscInvoicesAmount = $miscInvoicesAmount + $actualEmi;
                }
            };
        }

        $miscCharges = MiscCharge::where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('archived', '0')->where('business_id', $business->id)->sum('amount');
        $miscChargesArchive = MiscCharge::where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('archived', '1')->where('business_id', $business->id)->sum('amount');
        $weeklyFee = 0;
        if ($business->business_commissions->weekly_fee !== null) {
            $weeklyFee = $business->business_commissions->weekly_fee;
        }

        // Invoice ADD Non-Archive
        $invoices = Invoice::selectRaw('invoices.*,invoice_payments.amount as amount')->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('business_id', $business->id)->where('invoice_payments.archived', '0')->where('invoice_payments.payment_mode', 'Merchant')->get();
        $invoiceIds = Invoice::select("invoices.id")->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('business_id', $business->id)->where('invoice_payments.archived', '0')->where('invoice_payments.payment_mode', 'Merchant')->get();
        $invoiceIds->makeHidden('image_url');
        $invoiceIds->makeHidden('thumb_url');
        $mergedData['total_invoices'] =  InvoicePayment::whereIn('invoice_id', $invoiceIds)->where('payment_mode', 'Merchant')->where('archived', '0')->sum('amount');

        $mergedData['total_normal_surcharges'] = 0;
        $all_normal_surcharges = [];
        $weeklySurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Normal')->where('business_id', $business->id)->get();
        if (!$weeklySurcharges->isEmpty()) {
            foreach ($weeklySurcharges as $value) {
                $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
            }
            $all_normal_surcharges = array_merge($all_normal_surcharges, $weeklySurcharges->toArray());
        }
        $monthlyFee = 0;
        $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $from_date")->format('Y-m-d');
        $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
        if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
            $monthlyFee = $business->business_commissions->monthly_fee;

            $monthlySurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Normal')->where('business_id', $business->id)->get();
            if (!$monthlySurcharges->isEmpty()) {
                foreach ($monthlySurcharges as $value) {
                    $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
                }
                $all_normal_surcharges = array_merge($all_normal_surcharges, $monthlySurcharges->toArray());
            }
        }

        $mergedData['surcharges'] = $all_normal_surcharges;

        $thisMonthInvoices = PayoutInvoice::whereRaw('(DATE_FORMAT(CAST(to_date as DATE), "%m/%Y") LIKE ? )', Carbon::today()->format('m/Y'))->where('business_id', $business_id)->count();
        if ($thisMonthInvoices > 0) {
            // $monthlyFee = 0;
        }
        $normalTransactionsFound = false;
        if (sizeof($mergedData['transactions']) > 0 || sizeof($invoices) > 0 || (($weeklyFee > 0 || $monthlyFee > 0 || $miscCharges > 0) && sizeof($mergedArchivedData['transactions']) == 0)) {
            $normalTransactionsFound = true;

            $mergedArchivedData['total_archive_surcharges'] = 0;
            $all_archive_surcharges = [];
            $weeklyArchiveSurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Archive')->where('business_id', $business->id)->get();
            if (!$weeklyArchiveSurcharges->isEmpty()) {
                foreach ($weeklyArchiveSurcharges as $value) {
                    $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                }
                $all_archive_surcharges = array_merge($all_archive_surcharges, $weeklyArchiveSurcharges->toArray());
            }
            $fromDate1 = Carbon::parse($from_date)->format('Y-m-d');
            $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $fromDate1")->format('Y-m-d');
            $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
            if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
                $monthlyFee = $business->business_commissions->monthly_fee;
                $monthlyArchiveSurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Archive')->where('business_id', $business->id)->get();
                if (!$monthlyArchiveSurcharges->isEmpty()) {
                    foreach ($monthlyArchiveSurcharges as $value) {
                        $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                    }
                    $all_archive_surcharges = array_merge($all_archive_surcharges, $monthlyArchiveSurcharges->toArray());
                }
            }

            $mergedData['surcharges'] = array_merge($all_normal_surcharges, $all_archive_surcharges);
            $mergedData['invoices'] = $invoices;

            $mergedData['misc_charge'] = $miscCharges;
            $mergedData['weekly_fee'] = $weeklyFee;
            $mergedData['monthly_fee'] = $monthlyFee;

            $mergedData['management_fee'] =  0;
            $mergedData['handling_fee'] =  0;

            $mergedData['misc_invoices'] = $miscInvoices;
            $mergedData['misc_invoice_amount'] = $miscInvoicesAmount;
            $mergedData['misc_charge'] = $mergedData['misc_charge'] +  $mergedData['misc_invoice_amount'];

            $mergedData['net_fee'] = $mergedData['net_fee'] +   $mergedData['handling_fee'] +  $mergedData['management_fee'] +  $mergedData['misc_charge'] + $mergedData['monthly_fee'] + $mergedData['weekly_fee'] + $mergedArchivedData['total_archive_surcharges'] + $mergedData['total_invoices'];
            $mergedData['net_total'] = $mergedData['net_total']  - ($mergedData['handling_fee'] +  $mergedData['management_fee'] +  $mergedData['misc_charge'] + $mergedData['monthly_fee'] + $mergedData['weekly_fee'] + $mergedArchivedData['total_archive_surcharges'] + $mergedData['total_invoices']);

            // Add payout charges

            $payoutCharge = PayoutCharge::whereRaw('(amount>= ? AND (SELECT pc.amount FROM payout_charges pc WHERE pc.business_id = ? ORDER BY pc.amount DESC LIMIT 1))', [$mergedData['net_total'], $business_id])->where('business_id', $business_id)->orderByRaw('amount ASC')->first();
            $payoutChargeAmount = 0;
            if (!empty($payoutCharge)) {
                $payoutChargeAmount = $payoutCharge->charge;
            }
            $mergedData['payout_charge'] =  $payoutChargeAmount;
            $mergedData['net_fee'] = $mergedData['net_fee'] + $mergedData['payout_charge'];
            $mergedData['net_total'] = $mergedData['net_total'] - $mergedData['payout_charge'];
        } else {
            $mergedData = null;
        }

        // Invoice ADD Archive
        $archiveInvoices = Invoice::selectRaw('invoices.*,invoice_payments.amount as amount')->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('business_id', $business->id)->where('invoice_payments.archived', '1')->where('invoice_payments.payment_mode', 'Merchant')->get();
        $archiveInvoiceIds = Invoice::select("invoices.id")->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('business_id', $business->id)->where('invoice_payments.archived', '1')->where('invoice_payments.payment_mode', 'Merchant')->get();
        $archiveInvoiceIds->makeHidden('image_url');
        $archiveInvoiceIds->makeHidden('thumb_url');
        $mergedArchivedData['total_invoices'] =  InvoicePayment::whereIn('invoice_id', $archiveInvoiceIds)->where('payment_mode', 'Merchant')->where('archived', '1')->sum('amount');

        if (sizeof($mergedArchivedData['transactions']) > 0 || $miscChargesArchive > 0 || sizeof($archiveInvoices) > 0) {
            $mergedArchivedData['total_archive_surcharges'] = 0;

            $all_archive_surcharges = [];
            $weeklyArchiveSurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Archive')->where('business_id', $business->id)->get();
            if (!$weeklyArchiveSurcharges->isEmpty()) {
                foreach ($weeklyArchiveSurcharges as $value) {
                    $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                }
                $all_archive_surcharges = array_merge($all_archive_surcharges, $weeklyArchiveSurcharges->toArray());
            }
            $fromDate1 = Carbon::parse($from_date)->format('Y-m-d');
            $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $fromDate1")->format('Y-m-d');
            $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
            if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
                $monthlyFee = $business->business_commissions->monthly_fee;
                $monthlyArchiveSurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Archive')->where('business_id', $business->id)->get();
                if (!$monthlyArchiveSurcharges->isEmpty()) {
                    foreach ($monthlyArchiveSurcharges as $value) {
                        $mergedArchivedData['total_archive_surcharges'] = $mergedArchivedData['total_archive_surcharges'] + $value['surcharge'];
                    }
                    $all_archive_surcharges = array_merge($all_archive_surcharges, $monthlyArchiveSurcharges->toArray());
                }
            }
            $mergedArchivedData['surcharges'] = $all_archive_surcharges;
            $mergedArchivedData['invoices'] = $archiveInvoices;

            $cashFee = $business->business_commissions->cash_handling_fee;
            $managementFee = $business->business_commissions->management_fee;
            $cashFeeOnEveryAmount = $business->business_commissions->cash_handling_per_amount;
            $handlingFee = 0;
            if ($cashFeeOnEveryAmount > 0) {
                $handlingFee = $cashFee * ceil($mergedArchivedData['net_total'] / $cashFeeOnEveryAmount);
            }
            if (!$normalTransactionsFound) {
                $mergedArchivedData['misc_charge'] = $miscChargesArchive;
                $mergedArchivedData['weekly_fee'] = $weeklyFee;
                $mergedArchivedData['monthly_fee'] = $monthlyFee;

                $mergedArchivedData['management_fee'] =  $managementFee;
                $mergedArchivedData['handling_fee'] =  $handlingFee;

                $mergedArchivedData['misc_invoices'] = $miscInvoices;
                $mergedArchivedData['misc_invoice_amount'] = $miscInvoicesAmount;
                $mergedArchivedData['misc_charge'] = $mergedArchivedData['misc_charge'] + $mergedArchivedData['misc_invoice_amount'];

                $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] +   $mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge'] + $mergedArchivedData['monthly_fee'] + $mergedArchivedData['weekly_fee'] + $mergedArchivedData['total_archive_surcharges'] + $mergedArchivedData['total_invoices'];
                $mergedArchivedData['net_total'] = $mergedArchivedData['net_total']  - ($mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge'] + $mergedArchivedData['monthly_fee'] + $mergedArchivedData['weekly_fee'] + $mergedArchivedData['total_archive_surcharges'] + $mergedArchivedData['total_invoices']);

                // Add Payout charges

                $payoutCharge = PayoutCharge::whereRaw('(amount>= ? AND (SELECT pc.amount FROM payout_charges pc WHERE pc.business_id = ? ORDER BY pc.amount DESC LIMIT 1))', [$mergedArchivedData['net_total'], $business_id])->where('business_id', $business_id)->orderByRaw('amount ASC')->first();
                $payoutChargeAmount = 0;
                if (!empty($payoutCharge)) {
                    $payoutChargeAmount = $payoutCharge->charge;
                }
                $mergedArchivedData['payout_charge'] =  $payoutChargeAmount;
                $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] + $payoutChargeAmount;
                $mergedArchivedData['net_total'] = $mergedArchivedData['net_total'] - $payoutChargeAmount;
            } else {
                $mergedArchivedData['payout_charge'] =  0;
                $mergedArchivedData['management_fee'] =  $managementFee;
                $mergedArchivedData['handling_fee'] =  $handlingFee;

                $mergedArchivedData['misc_charge'] = $miscChargesArchive;

                $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] +   $mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge'] + $mergedArchivedData['total_archive_surcharges'] + $mergedArchivedData['total_invoices'];
                $mergedArchivedData['net_total'] = $mergedArchivedData['net_total']  - ($mergedArchivedData['handling_fee'] +  $mergedArchivedData['management_fee'] +  $mergedArchivedData['misc_charge'] + $mergedArchivedData['total_archive_surcharges'] + $mergedArchivedData['total_invoices']);
            }
        } else {
            $mergedArchivedData = null;
        }
        if ($mergedData == null && $mergedArchivedData != null) {

            // Invoice ADD Non-Archive
            $invoices = Invoice::selectRaw('invoices.*,invoice_payments.amount as amount')->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('business_id', $business->id)->where('invoice_payments.archived', '0')->where('invoice_payments.payment_mode', 'Merchant')->get();
            $invoiceIds = Invoice::select("invoices.id")->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $from_date)->where('date', '<=', $to_date)->where('business_id', $business->id)->where('invoice_payments.archived', '0')->where('invoice_payments.payment_mode', 'Merchant')->get();
            $invoiceIds->makeHidden('image_url');
            $invoiceIds->makeHidden('thumb_url');
            $mergedData['total_invoices'] =  InvoicePayment::whereIn('invoice_id', $invoiceIds)->where('payment_mode', 'Merchant')->where('archived', '0')->sum('amount');

            $mergedData['total_normal_surcharges'] = 0;
            $all_normal_surcharges = [];
            $weeklySurcharges = BusinessSurcharge::where('interval', 'Weekly')->where('type', 'Normal')->where('business_id', $business->id)->get();
            if (!$weeklySurcharges->isEmpty()) {
                foreach ($weeklySurcharges as $value) {
                    $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
                }
                $all_normal_surcharges = array_merge($all_normal_surcharges, $weeklySurcharges->toArray());
            }
            $monthlyFee = 0;
            $fromDate1 = Carbon::parse($from_date)->format('Y-m-d');
            $firstTuesdayDayOfMonth = Carbon::parse("first tuesday of $fromDate1")->format('Y-m-d');
            $firstMondayDayOfMonth = Carbon::parse($firstTuesdayDayOfMonth)->addDays(6)->format('Y-m-d');
            if (Carbon::parse($firstMondayDayOfMonth)->between($from_date, $to_date)) {
                $monthlyFee = $business->business_commissions->monthly_fee;

                $monthlySurcharges = BusinessSurcharge::where('interval', 'Monthly')->where('type', 'Normal')->where('business_id', $business->id)->get();
                if (!$monthlySurcharges->isEmpty()) {
                    foreach ($monthlySurcharges as $value) {
                        $mergedData['total_normal_surcharges'] = $mergedData['total_normal_surcharges'] + $value['surcharge'];
                    }
                    $all_normal_surcharges = array_merge($all_normal_surcharges, $monthlySurcharges->toArray());
                }
            }

            $mergedData['surcharges'] = $all_normal_surcharges;
            $mergedData['invoices'] = $invoices;

            $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] + $mergedData['total_normal_surcharges'] + $mergedData['total_invoices'];
            $mergedArchivedData['net_total'] = $mergedArchivedData['net_total']  - $mergedData['total_normal_surcharges'] - $mergedData['total_invoices'];

            $mergedArchivedData['surcharges'] = array_merge($all_normal_surcharges, $all_archive_surcharges);
            $mergedArchivedData['invoices'] = $archiveInvoices;
            $mergedData = null;
        }

        if ($mergedData == null && $mergedArchivedData == null && !$req->has('payout_all')) {
            if ($req->has('is_job')) {
                return "Saved";
            } else {
                if ($req->has('is_reseller_job')) {
                } else {
                    Utils::error("No data found for $business->name", 400);
                }
            }
        }

        if ($format = $req->query('format', false)) {
            $payable_tot_amount = 0;
            $externalAccount = ExternalAccount::where('business_id', $business->id)->where('default', 1)->orderByRaw('id DESC')->first();
            if (!empty($externalAccount)) {
                // $stringValue = 'UBSIDI LIMITED,Tiffin Tom,TIFFINTOMUK LTD,Tiffin Tom Limited,Ubsidi Ltd,TIFFINTOM LIMITED,TiffinTomuk Ltd';
                $stringValue = (new SiteSettingController())->findByKey('bank_string')->value;
                if ($externalAccount->account_holder_name == '' || stripos($stringValue, $externalAccount->account_holder_name) !== false) {
                    $payable_tot_amount = 1;
                }
            }
            if ($mergedData != null) {
                $mergedData['payable_tot_amount'] = $payable_tot_amount;
            }
            if ($mergedArchivedData != null) {
                $mergedArchivedData['payable_tot_amount'] = $payable_tot_amount;
            }
            return $this->export($mergedData, $mergedArchivedData, $format);
        } else if ($req->has('save')) {

            if ($mergedData != null)
                $this->saveReport($mergedData, $accountType);
            else {
                $payoutInvoice = PayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'normal')->where('business_id', $business->id)->first();
                if (!empty($payoutInvoice)) {
                    if ($payoutInvoice->status != 'Settled') {
                        $payoutInvoice->delete();
                        Passbook::where('business_id', $business->id)->where('entity_id', $payoutInvoice->id)->where('entity', 'payout_invoice')->delete();
                    }
                }
            }
            if ($mergedArchivedData != null)
                $this->saveReport($mergedArchivedData, $accountType);
            else {
                $payoutInvoice = PayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'archived')->where('business_id', $business->id)->first();
                if (!empty($payoutInvoice)) {
                    if ($payoutInvoice->status != 'Settled') {
                        $payoutInvoice->delete();
                        Passbook::where('business_id', $business->id)->where('entity_id', $payoutInvoice->id)->where('entity', 'payout_invoice')->delete();
                    }
                }
            }
            if ($mergedData != null || $mergedArchivedData != null) {
                $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                if (!empty($reseller)) {
                    $reseller = User::where('id', $reseller->user_id)->first();
                    if (!empty($reseller)) {
                        $resellerPayoutController = new ResellerPayoutReportController();
                        $resellerBusinessReports = $resellerPayoutController->reseller_business_report($from_date, $to_date, $reseller, $business);
                        foreach ($resellerBusinessReports as $resellerBusinessReport)
                            $saved = $resellerPayoutController->saveReport($resellerBusinessReport);
                    }
                }
            }
        }
        return ['normal' => $mergedData, 'archived' => $mergedArchivedData];
    }

    public function transactionQueries($from_date, $to_date, $archived, $business_id, $chargeDescription, $isConnectAccount)
    {
        $data = [];
        $codOrders = OnlineOrder::where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $codOrders = $codOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $codOrders = $codOrders->where('order_date', '>=', "$from_date");
        $codOrders = $codOrders->where('order_date', '<=', "$to_date");
        $codOrders = $codOrders->where('payment_method', '=', 'cod');
        $codOrders = $codOrders->where('online_orders.archived',  $archived);
        $codOrders = $codOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $codOrders = $codOrders->get();
        if (!$codOrders->isEmpty()) {
            $data['cod_report'] = $this->createReport($business_id, $codOrders);
        }

        $paypalOrders = OnlineOrder::select('*', 'online_orders.txn_id as online_txn_id', 'paypal_payments.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $paypalOrders = $paypalOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $paypalOrders = $paypalOrders->leftJoin('paypal_payments', 'sale_id', '=', 'online_orders.txn_id');
        $paypalOrders = $paypalOrders->where('paypal_payments.archived',  $archived);
        $paypalOrders = $paypalOrders->where('order_date', '>=', "$from_date");
        $paypalOrders = $paypalOrders->where('order_date', '<=', "$to_date");
        $paypalOrders = $paypalOrders->where('payment_method', '=', 'paypal');
        $paypalOrders = $paypalOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $paypalOrders = $paypalOrders->get();
        if (!$paypalOrders->isEmpty()) {
            $data['paypal_report'] = $this->createReport($business_id, $paypalOrders);;
        }

        $stripeOrders = OnlineOrder::select('*', 'stripe_transactions.txn_id as stripe_txn_id', 'online_orders.txn_id as online_txn_id', 'stripe_transactions.card_number as card_number', 'stripe_transactions.card_brand as card_brand', 'stripe_transactions.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $stripeOrders = $stripeOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $stripeOrders = $stripeOrders->leftJoin('stripe_transactions', function ($join) {
            $join->on('stripe_transactions.payment_intent_id', '=', 'online_orders.txn_id')->where('reporting_category', 'charge');
        });
        $stripeOrders = $stripeOrders->where('order_date', '>=', "$from_date");
        $stripeOrders = $stripeOrders->where('order_date', '<=', "$to_date");
        $stripeOrders = $stripeOrders->where('stripe_transactions.archived',  $archived);
        $stripeOrders = $stripeOrders->whereIn('payment_method', ['Credit Card', 'Online']);
        $stripeOrders = $stripeOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $stripeOrders = $stripeOrders->get();
        if (!$stripeOrders->isEmpty()) {
            $data['card_report'] = $this->createReport($business_id, $stripeOrders);;
        }


        $walletOrders = OnlineOrder::select('*',  'online_orders.txn_id as online_txn_id')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $walletOrders = $walletOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $walletOrders = $walletOrders->where('order_date', '>=', "$from_date");
        $walletOrders = $walletOrders->where('order_date', '<=', "$to_date");
        $walletOrders = $walletOrders->where('online_orders.archived',  $archived);
        $walletOrders = $walletOrders->whereIn('payment_method', ['Wallet']);
        $walletOrders = $walletOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $walletOrders = $walletOrders->get();
        if (!$walletOrders->isEmpty()) {
            $data['wallet_report'] = $this->createReport($business_id, $walletOrders);;
        }

        $merchantTransactions = StripeTransaction::whereRaw('(descriptor LIKE ? OR business_id LIKE ?)', ["$chargeDescription", $business_id]);
        $merchantTransactions = $merchantTransactions->where('stripe_transactions.archived',  $archived);
        $merchantTransactions = $merchantTransactions->where('created', '>=', strtotime($from_date));
        $merchantTransactions = $merchantTransactions->where('created', '<=', strtotime($to_date));
        $merchantTransactions = $merchantTransactions->whereIn('reporting_category', ['charge', 'refund', 'dispute']);
        $merchantTransactions = $merchantTransactions->whereIn('service', ['Merchant', 'Connect']);
        $merchantTransactions = $merchantTransactions->get();
        if (!$merchantTransactions->isEmpty()) {
            $data['merchant_report'] = $this->createMerchantReport($merchantTransactions, true, $isConnectAccount);
        }

        $onlineRefundTransactions = StripeTransaction::whereRaw('(descriptor LIKE ? OR business_id LIKE ?)', ["$chargeDescription", $business_id]);
        $onlineRefundTransactions = $onlineRefundTransactions->where('stripe_transactions.archived',  $archived);
        $onlineRefundTransactions = $onlineRefundTransactions->where('created', '>=', strtotime($from_date));
        $onlineRefundTransactions = $onlineRefundTransactions->where('created', '<=', strtotime($to_date));
        $onlineRefundTransactions = $onlineRefundTransactions->where('reporting_category', 'refund');
        $onlineRefundTransactions = $onlineRefundTransactions->whereIn('service', ['online']);
        $onlineRefundTransactions = $onlineRefundTransactions->get();
        if (!$onlineRefundTransactions->isEmpty()) {
            $data['online_refund_report'] = $this->createMerchantReport($onlineRefundTransactions, true, $isConnectAccount);
        }
        return $data;
    }

    public function createReport($business_id, $orders)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_charge_back_transactions = 0;
        $total_charge_back_fee = 0.0;
        $total_fee = 0.0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_service_charge = 0;
        foreach ($orders as $order) {
            $orderTypeWiseCommission = 0.00;
            $orderTypeWiseServiceCharge = 0.00;
            $orderBasedOrTransactionWiseCharge = 0.00;
            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;
            $payment_method_fees = 0;
            $chargeBackFee = 0;
            $handlingFee = 0.00;
            $cardCountry = "";
            $brand = "";

            if (!empty($order->card_brand)) {
                $brand = $order->card_brand;
            }
            $business = Business::where('id', $business_id)->first();
            if ($business->service_charge_type == 'show') {
                $orderTypeWiseServiceCharge = $order->user_service_charge; // It would be mostly in flat.
            }

            $orderGrandTotalBeforeServiceCharge = $order->grand_total;
            $orderGrandTotalForDisplay = $order->grand_total;
            $refundedAmount = 0;

            if ($order->payment_method == 'paypal') {
                $refundedAmount = abs($order->refunded_amount);
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_paypal_fee, $order->online_paypal_fee_type);
            }
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online' || strtolower($order->payment_method) == 'wallet') {
                $refundedAmount = abs($order->refunded_amount) / 100;
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_card_fee, $order->online_card_fee_type);
            }

            if ($order->payment_method == 'cod')
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_cod_fee, $order->online_cod_fee_type);

            if ($order->order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->dinein_commission_fee, $order->dinein_commission_fee_type);
            } else {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_commission_fee, $order->online_commission_fee_type);
            }
            if ($order->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->per_order_charge, $order->per_order_charge_type);
            } else {
                if ($order->payment_method == 'paypal')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->paypal_charge, $order->paypal_charge_type);
                if (strtolower($order->payment_method) == 'credit card' || strtolower($order->payment_method) == 'online' || strtolower($order->payment_method) == 'wallet')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->card_charge, $order->card_charge_type);
                if ($order->payment_method == 'cod')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->cod_charge, $order->cod_charge_type);
            }


            $serviceCharge = $orderTypeWiseServiceCharge;
            $serviceCharge = 0;
            $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $serviceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);
            if ($order->order_status == 'Failed') {
                $transaction['order_grand_total'] = -$this->convertToDecimal($orderGrandTotalForDisplay);
                if ($order->charges_refundable == 'yes') {
                    $transaction['total_fee'] = 0;
                    $orderTypeWiseServiceCharge = 0;
                }
                $total_refund_transactions = $total_refund_transactions + 1;
                $transaction['net_total'] = $this->convertToDecimal(0 - $transaction['total_fee']);
            } else {
                $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotalForDisplay);
                if ($order->payment_method == 'cod') $transaction['order_grand_total'] = 0;

                $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $transaction['total_fee']);
                $gross_total = $gross_total + $transaction['order_grand_total'];
            }

            $transaction['order_id'] = "#ODR000" . $order->order_id;
            $transaction['restaurant_id'] = $order->restaurant_id;
            if (empty($order->stripe_txn_id)) {
                $transaction['txn_id'] = $order->online_txn_id;
            } else {
                $transaction['txn_id'] = $order->stripe_txn_id;
            }
            $transaction['order_date'] = $order->order_date;
            $transaction['order_type'] = $order->order_type;
            $transaction['order_status'] = $order->order_status;
            $transaction['payment_method'] = strtolower($order->payment_method) == 'credit card' ? 'Online' : $order->payment_method;
            $transaction['card_number'] = !empty($order->card_number) ? $order->card_number : null;
            $transaction['card_brand'] = $brand;
            $transaction['payment_method_fees'] = $payment_method_fees;
            $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
            $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
            $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;
            $transaction['country'] = $cardCountry;
            $transaction['country_charges'] = $countryWiseCommission;
            $transaction['charge_back_fee'] = $chargeBackFee;
            $transaction['handling_fee'] = $handlingFee;
            $transaction['card_brand'] = $brand;
            $transaction['card_brand_charge'] = $cardBrandWiseCommission;
            $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotalBeforeServiceCharge);

            $net_total = $net_total + $transaction['net_total'];
            $total_fee = $total_fee + abs($transaction['total_fee']);
            $total_service_charge = $total_service_charge + $orderTypeWiseServiceCharge;
            $total_charge_back_fee = $total_charge_back_fee + ($transaction['charge_back_fee']);
            $transactions[] = $transaction;
        }
        $report['transactions'] = $transactions;
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($net_total);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_non_domestic_fee'] = $total_non_domestic_fee;
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_charge_back_transactions'] = $this->convertToDecimal($total_charge_back_transactions);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['total_refund_transactions'] = $total_refund_transactions;
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        return $report;
    }

    public function createMerchantReport($merchantTransactions, $archivedReport, $isConnectAccount)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_fee = 0.0;

        $total_charge_back_transactions = 0;
        $total_refund_transactions = 0;
        $total_charge_back_fee = 0.0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $managementFee = 0;
        $archivedTransactions = 0;
        $cardCountry = "";
        $brand = "";
        $total_service_charge = 0;

        foreach ($merchantTransactions as $transaction) {
            $cardProcessingFee = 0.00;
            $cardAuthFee = 0.00;
            $managementFee = 0.00;
            $handlingFee = 0.00;
            $chargeBackFee = 0;
            $cardCountry = $transaction->card_country;
            $brand = $transaction->card_brand;

            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;

            // if ($transaction->reporting_category == 'refund') {
            //     $checkRefundTransactions = StripeTransaction::where('payment_intent_id', $transaction->payment_intent_id)->where('txn_id', '!=', $transaction->txn_id)->first();
            //     if (empty($checkRefundTransactions)) {
            //         continue;
            //     }
            // }

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_auth_fee, $transaction->merchant_card_auth_fee_type));
                $cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_processing_fee, $transaction->merchant_card_processing_fee_type));
                if ($transaction->archived) {
                    if ($transaction->cash_handling_fee_type == 'percentage') {
                        $handlingFee  = $this->calculateFee($transaction->amount / 100, $transaction->cash_handling_fee, $transaction->cash_handling_fee_type);
                    } else {
                        if ($transaction->cash_handling_per_amount > 0)
                            $handlingFee = $transaction->cash_handling_fee * ceil($transaction->amount / $transaction->cash_handling_per_amount);
                    }
                    $handlingFee = 0;
                    $archivedTransactions = $archivedTransactions + 1;
                }

                $countryCommission = CountryCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();
                if (!empty($countryCommission)) {
                    $countryWiseCommission = abs($this->calculateFee($transaction->amount / 100, $countryCommission->stripe_percentage, 'percentage'));
                    $countryWiseCommission = $countryWiseCommission + $countryCommission->stripe_fixed;
                }
                $cardBrandCommission = CardBrandCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();

                if (!empty($cardBrandCommission)) {
                    $cardBrandWiseCommission = abs($this->calculateFee($transaction->amount / 100, $cardBrandCommission->stripe_percentage, 'percentage'));
                    $cardBrandWiseCommission = $cardBrandWiseCommission + $cardBrandCommission->stripe_fixed;
                }

                if ($transaction->reporting_category == 'refund') {
                    $cardAuthFee = -$cardAuthFee;
                    $cardProcessingFee = -$cardProcessingFee;
                    $cardBrandWiseCommission = -$cardBrandWiseCommission;
                    $countryWiseCommission = -$countryWiseCommission;
                }
            }
            if ($transaction->reporting_category == 'dispute') {
                $chargeBackFee = $this->calculateFee($transaction->amount / 100, $transaction->charge_back_fee, $transaction->charge_back_fee_type);
                $total_charge_back_transactions = $total_charge_back_transactions + 1;
            }
            if ($transaction->reporting_category == 'refund') {
                $total_refund_transactions = $total_refund_transactions + 1;
            }
            $applicationFee = ($this->convertToDecimal($managementFee + $handlingFee + $cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee));
            if ($transaction->service == 'Connect') {
                if ($transaction->reporting_category == 'refund' || $transaction->reporting_category == 'transfer_reversal') {
                    $applicationFee = 0;
                } else {
                    $applicationFeeTransaction = StripeTransaction::where('reporting_category', 'platform_earning')->whereNotNull('payment_intent_id')->where('business_id', $transaction->business_id)->where('payment_intent_id', $transaction->payment_intent_id)->first();
                    if (!empty($applicationFeeTransaction)) {
                        $applicationFee =  $this->convertToDecimal($applicationFeeTransaction->amount / 100);
                    }
                }
            }
            if ($transaction->service == 'Online' && $transaction->reporting_category == 'refund') {
                $applicationFee = 0;
            }

            $merchantTransction['order_id'] = $transaction->txn_id;
            $merchantTransction['restaurant_id'] = null;
            $merchantTransction['txn_id'] = $transaction->txn_id;
            $merchantTransction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
            $merchantTransction['order_type'] = $transaction->service;
            $merchantTransction['order_status'] = $transaction->reporting_category;
            $merchantTransction['payment_method'] = $transaction->service;
            $merchantTransction['order_grand_total'] = $transaction->amount / 100;

            $merchantTransction['processing_fee'] = $cardProcessingFee;
            $merchantTransction['country'] = $cardCountry;
            $merchantTransction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
            $merchantTransction['card_brand'] = $brand;
            $merchantTransction['card_number'] = $transaction->card_number;
            $merchantTransction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
            $merchantTransction['auth_fee'] = $cardAuthFee;
            $merchantTransction['handling_fee'] = 0;
            $merchantTransction['management_fee'] = $managementFee;
            $merchantTransction['charge_back_fee'] = $chargeBackFee;
            $merchantTransction['total_fee'] =  $applicationFee;
            if ($transaction->non_domestic) {
                $total_non_domestic_transactions = $total_non_domestic_transactions + 1;
                $total_non_domestic_fee = $total_non_domestic_fee + $countryWiseCommission;
            }

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
                if ($transaction->service == 'Connect' && abs($applicationFee) > ($transaction->amount / 100) && $transaction->reporting_category != 'refund') $merchantTransction['net_total'] = 0; // Condition double check when charge refund yes.
            } else {
                if ($transaction->reporting_category == 'dispute') {
                    $merchantTransction['total_fee'] = $chargeBackFee;
                    $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
                } else {
                    $merchantTransction['total_fee'] = 0;
                    $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $merchantTransction['total_fee']);
                }
            }
            $gross_total = $gross_total + $merchantTransction['order_grand_total'];
            $net_total = $net_total + $merchantTransction['net_total'];
            $total_fee = $total_fee + abs($merchantTransction['total_fee']);
            $total_charge_back_fee = $total_charge_back_fee + $merchantTransction['charge_back_fee'];
            $transactions[] = $merchantTransction;
        }
        $report['transactions'] = $transactions;
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($net_total);
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        $report['total_charge_back_transactions'] = $this->convertToDecimal($total_charge_back_transactions);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['total_non_domestic_fee'] = $total_non_domestic_fee;
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        return $report;
    }

    public function export($report = null, $archivedReport = null, $type = "excel")
    {
        if ($report != null) {
            $fromDate = Carbon::parse($report['from_date']);
            $toDate = Carbon::parse($report['to_date']);
            $fileName = $report['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
        } else if ($archivedReport != null) {
            $fromDate = Carbon::parse($archivedReport['from_date']);
            $toDate = Carbon::parse($archivedReport['to_date']);
            $fileName = $archivedReport['business']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
        } else {
            Utils::error("No data found", 400);
        }
        if ($report != null)
            $reports[] = $report;
        if ($archivedReport != null) {
            $reports[] = $archivedReport;
        }
        if ($type == 'excel')
            return Excel::download(new MultiplePayoutReportExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new MultiplePayoutReportExport($reports), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function saveReport($report, $accountType)
    {
        $business = $report['business'];
        $shouldDeleteOldTransactions = false;
        $payoutInvoice = PayoutInvoice::where('from_date', explode(" ", $report['from_date'])[0])->where('to_date', explode(" ", $report['to_date'])[0])->where('account_type', $accountType)->where('transaction_type', $report['transaction_type'])->where('business_id', $report['business']['id'])->first();
        if (empty($payoutInvoice)) {
            $payoutInvoice = new PayoutInvoice();
        } else {
            if ($payoutInvoice->status == 'Settled') return;
            $shouldDeleteOldTransactions = true;
        }
        $payoutInvoice->business_id = $business->id;
        $payoutInvoice->gross_total = $report['gross_total'];
        $payoutInvoice->gross_fee = $report['gross_fee'];
        $payoutInvoice->payout_charge = $report['payout_charge'];
        $payoutInvoice->misc_charge = $report['misc_charge'];
        $payoutInvoice->monthly_fee = $report['monthly_fee'];
        $payoutInvoice->handling_fee = $report['handling_fee'];
        $payoutInvoice->management_fee = $report['management_fee'];
        $payoutInvoice->weekly_fee = $report['weekly_fee'];
        $payoutInvoice->net_fee = $report['net_fee'];
        $payoutInvoice->net_total = $report['net_total'];
        $payoutInvoice->payout_comment = $report['payout_comment'];
        $payoutInvoice->total_charge_back_transactions = $report['total_charge_back_transactions'];
        $payoutInvoice->total_charge_back_fee = $report['total_charge_back_fee'];
        $payoutInvoice->total_non_domestic_transactions = $report['total_non_domestic_transactions'];
        $payoutInvoice->total_non_domestic_fee = $report['total_non_domestic_fee'];
        $payoutInvoice->total_refund_transactions = $report['total_refund_transactions'];
        $payoutInvoice->total_service_charge = $report['total_service_charge'];
        $payoutInvoice->transaction_type = $report['transaction_type'];
        if ($report['transaction_type'] == 'Normal') {
            $payoutInvoice->surcharge = $report['total_normal_surcharges'];
        }
        if ($report['transaction_type'] == 'Archived') {
            $payoutInvoice->surcharge = $report['total_archive_surcharges'];
        }
        $payoutInvoice->invoices = $report['total_invoices'];
        $payoutInvoice->account_type = $accountType;
        if ($payoutInvoice->transaction_type == 'Archived') {
            $payoutInvoice->payout_mode = $business->archived_payout_mode;
            $payoutInvoice->payout_method = $business->archived_payout_method;
        } else {
            $payoutInvoice->payout_mode = $business->payout_mode;
            $payoutInvoice->payout_method = $business->payout_method;
        }
        $payoutInvoice->from_date = $report['from_date'];
        $payoutInvoice->to_date = $report['to_date'];
        $payoutInvoice->payout_account = $report['payout_account'];
        $payoutInvoice->payout_invoice_raw = $report;
        $payoutInvoice->save();

        $savedTransactionIds = [];
        foreach ($report['transactions'] as $transaction) {
            $invoiceTransaction = InvoiceTransaction::where('payout_invoice_id', $payoutInvoice->id)->where('order_id', $transaction['order_id'])->first();
            if (empty($invoiceTransaction))
                $invoiceTransaction = new InvoiceTransaction();
            $invoiceTransaction->fill($transaction);
            $invoiceTransaction->payout_invoice_id = $payoutInvoice->id;
            $invoiceTransaction->save();
            $savedTransactionIds[] = $invoiceTransaction->id;
        }
        # Delete old transactions if new transactions are not matching with it.
        if ($shouldDeleteOldTransactions) {
            InvoiceTransaction::where('payout_invoice_id', $payoutInvoice->id)->whereNotIn('id', $savedTransactionIds)->delete();
        }

        # Surcharges Invoice Wise
        $savedSurchargeIds = [];
        InvoiceSurcharge::where('payout_invoice_id', $payoutInvoice->id)->delete();
        foreach ($report['surcharges'] as $surcharge) {
            $invoiceSurcharge = new InvoiceSurcharge();
            $invoiceSurcharge->fill($surcharge);
            $invoiceSurcharge->payout_invoice_id = $payoutInvoice->id;
            $invoiceSurcharge->save();
            $savedSurchargeIds[] = $invoiceSurcharge->id;
        }

        # Save passbook entry for generated invoice
        $passbook = Passbook::where('entity_id', $payoutInvoice->id)->where('entity', 'payout_invoice')->where('business_id', $payoutInvoice->business_id)->first();
        if (empty($passbook)) {
            $passbook = new Passbook();
        }
        $passbook->business_id = $payoutInvoice->business_id;
        $passbook->amount = $payoutInvoice->net_total;
        $passbook->entity_id = $payoutInvoice->id;
        $passbook->entity = 'payout_invoice';
        $passbook->date = Carbon::today();
        $passbook->from_date = Carbon::parse($payoutInvoice->from_date)->toDateString();
        $passbook->to_date = Carbon::parse($payoutInvoice->to_date)->toDateString();
        $passbook->description = 'Invoice for ' . Carbon::parse($payoutInvoice->from_date)->toDateString() . ' to ' . Carbon::parse($payoutInvoice->to_date)->toDateString();
        $passbook->save();

        if (isset($report['misc_invoices'])) {
            $miscInvoices = $report['misc_invoices'];
            foreach ($miscInvoices as $invoice) {
                $total_paid_amount = $invoice->total_paid_amount;
                if (!empty($payoutInvoice))
                    $total_paid_amount = -1 * Passbook::where('business_id', $business->id)->where('entity_id', $invoice->id)->where('entity', 'misc_invoice')->where('ref_entity_id', '!=', $payoutInvoice->id)->where('ref_entity', '!=', 'payout_invoice')->sum('amount');
                $payableAmount = $invoice->grand_total - $total_paid_amount;
                $actualEmi = 0;
                if ($payableAmount > 0) {
                    $actualEmi = $this->calculateFee($invoice->grand_total, $invoice->percentage, 'percentage');
                    if ($actualEmi < $payableAmount) {
                        $actualEmi = $this->calculateFee($payableAmount, $invoice->percentage, 'percentage');
                    } else {
                        $actualEmi = $payableAmount;
                    }
                }
                if ($actualEmi > 0) {
                    # Save passbook entry for generated invoice
                    $passbook = Passbook::where('entity_id', $invoice->id)->where('entity', 'misc_invoice')->where('ref_entity_id', $payoutInvoice->id)->where('ref_entity', 'payout_invoice')->where('business_id', $payoutInvoice->business_id)->first();
                    if (empty($passbook)) {
                        $passbook = new Passbook();
                    }
                    $passbook->business_id = $payoutInvoice->business_id;
                    $passbook->amount =  $actualEmi;
                    $passbook->entity_id = $invoice->id;
                    $passbook->entity = 'misc_invoice';
                    $passbook->ref_entity_id = $payoutInvoice->id;
                    $passbook->ref_entity = 'payout_invoice';
                    $passbook->date = Carbon::today();
                    $passbook->from_date = Carbon::parse($invoice->date);
                    $passbook->to_date = Carbon::parse($invoice->date);
                    $passbook->description = 'Auto paid from Payout invoice #' . $payoutInvoice->id . ' for miscellaneous invoice #' . $invoice->id . ' dated on ' . Carbon::parse($invoice->date)->toDateString();
                    $passbook->save();
                }
            };
        }
        if (false && $passbook->amount > 0 && $payoutInvoice->transaction_type == 'Normal' && $business->s_account_id != null && $business->connect_service) {
            $this->payout_to_business($passbook, $payoutInvoice);
        }
    }

    private function payout_to_business($passbook, $payoutInvoice)
    {
        $business = Business::find($passbook->business_id);
        if (empty($business)) return;

        if ($business->connect_service == 0) return;

        $key = "";
        $stripeConnectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;
        if (strtolower($stripeConnectMode) == 'live') {
            $key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
        } else {
            $key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
        }
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $amount = $passbook->amount;
        $transfer = $stripe->transfers->create([
            'amount' => $passbook->amount * 100,
            "currency" => "gbp",
            'description' =>  "$business->name payout from " . Carbon::parse($payoutInvoice->from_date)->toDateString() . ' to ' . Carbon::parse($payoutInvoice->to_date)->toDateString(),
            'destination' => $business->s_account_id,
        ]);
        // create passbook entry for transfer
        $passbook = new Passbook();
        $passbook->business_id = $payoutInvoice->business_id;
        $passbook->amount = $amount * -1;
        $passbook->entity = "payout_invoice";
        $passbook->entity_id = $payoutInvoice->id;
        $passbook->from_date = $payoutInvoice->from_date;
        $passbook->to_date = $payoutInvoice->to_date;
        $passbook->date = Carbon::today();
        if ($payoutInvoice->paid_amount > 0) {
            $passbook->description = "Invoice settled for " . Carbon::parse($payoutInvoice->from_date)->toDateString() . " to " . Carbon::parse($payoutInvoice->to_date)->toDateString() . " Transfer ID: " . $transfer->id;
        } else {
            $passbook->description = "Invoice paid for " . Carbon::parse($payoutInvoice->from_date)->toDateString() . " to " . Carbon::parse($payoutInvoice->to_date)->toDateString() . " Transfer ID: " . $transfer->id;
        }

        $passbook->save();
    }
    /*
    |=================================
    |   Util functions 
    |=================================
    */
    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }

    public function mergeAllTransactionsAndSort($data)
    {
        $gross_total = 0;
        $total_fee = 0;
        $net_total = 0;
        $total_charge_back_fee = 0;
        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_refund_transactions = 0;
        $total_service_charge = 0;
        $misc_charge = ($data['misc_charge']);
        $weekly_fee = ($data['weekly_fee']);
        $monthly_fee = ($data['monthly_fee']);
        $all_transactions = [];
        if (isset($data['cod_report'])) {
            $gross_total = $gross_total + $data['cod_report']['gross_total'];
            $total_fee = $total_fee + $data['cod_report']['total_fee'];
            $net_total = $net_total + $data['cod_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['cod_report']['total_service_charge'];
            $total_refund_transactions = $total_refund_transactions + $data['cod_report']['total_refund_transactions'];
            $temp_transactions = $data['cod_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['card_report'])) {
            $gross_total = $gross_total + $data['card_report']['gross_total'];
            $total_fee = $total_fee + $data['card_report']['total_fee'];
            $net_total = $net_total + $data['card_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['card_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['card_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['card_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['card_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['card_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['card_report']['total_refund_transactions'];
            $temp_transactions = $data['card_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['wallet_report'])) {
            $gross_total = $gross_total + $data['wallet_report']['gross_total'];
            $total_fee = $total_fee + $data['wallet_report']['total_fee'];
            $net_total = $net_total + $data['wallet_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['wallet_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['wallet_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['wallet_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['wallet_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['wallet_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['wallet_report']['total_refund_transactions'];
            $temp_transactions = $data['wallet_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['paypal_report'])) {
            $gross_total = $gross_total + $data['paypal_report']['gross_total'];
            $total_fee = $total_fee + $data['paypal_report']['total_fee'];
            $net_total = $net_total + $data['paypal_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['paypal_report']['total_service_charge'];
            $temp_transactions = $data['paypal_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['paypal_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['paypal_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['paypal_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['merchant_report'])) {
            $gross_total = $gross_total + $data['merchant_report']['gross_total'];
            $total_fee = $total_fee + $data['merchant_report']['total_fee'];
            $net_total = $net_total + $data['merchant_report']['net_total'];
            $total_charge_back_fee = $total_charge_back_fee + $data['merchant_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['merchant_report']['total_charge_back_transactions'];
            $temp_transactions = $data['merchant_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['merchant_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['merchant_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['merchant_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['online_refund_report'])) {
            // $gross_total = $gross_total + $data['online_refund_report']['gross_total'];
            // $total_fee = $total_fee + $data['online_refund_report']['total_fee'];
            // $net_total = $net_total + $data['online_refund_report']['net_total'];
            // $total_charge_back_fee = $total_charge_back_fee + $data['online_refund_report']['total_charge_back_fee'];
            // $total_charge_back_transactions = $total_charge_back_transactions + $data['online_refund_report']['total_charge_back_transactions'];
            // $total_non_domestic_fee = $total_non_domestic_fee + $data['online_refund_report']['total_non_domestic_fee'];
            // $total_non_domestic_transactions = $total_non_domestic_transactions + $data['online_refund_report']['total_non_domestic_transactions'];
            // $total_refund_transactions = $total_refund_transactions + $data['online_refund_report']['total_refund_transactions'];
            $temp_transactions = $data['online_refund_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }

        usort($all_transactions, $this->date_compare('order_date'));

        $payable_amount = 0;
        $externalAccount = ExternalAccount::where('business_id', $data['business']['id'])->where('default', 1)->orderByRaw('id DESC')->first();
        if (!empty($externalAccount)) {
            // $stringValue = 'UBSIDI LIMITED,Tiffin Tom,TIFFINTOMUK LTD,Tiffin Tom Limited,Ubsidi Ltd,TIFFINTOM LIMITED,TiffinTomuk Ltd';
            $stringValue = (new SiteSettingController())->findByKey('bank_string')->value;
            if ($externalAccount->account_holder_name == '' || stripos($stringValue, $externalAccount->account_holder_name) !== false) {
                $payable_amount = 1;
            } else {
                $payable_amount = 0;
            }
        }

        return [
            'from_date' => $data['from_date'],
            'to_date' => $data['to_date'],
            'payout_mode' => $data['payout_mode'],
            'payout_method' => $data['payout_method'],
            'payout_account' => $data['payout_account'],
            'transaction_type' => $data['transaction_type'],
            'account_type' => $data['account_type'],
            'payout_comment' => $data['payout_comment'],
            'gross_total' => $this->convertToDecimal($gross_total),
            'gross_fee' => $this->convertToDecimal($total_fee),
            'misc_charge' => $this->convertToDecimal($misc_charge),
            'weekly_fee' => $this->convertToDecimal($weekly_fee),
            'monthly_fee' => $this->convertToDecimal($monthly_fee),
            'total_charge_back_fee' => $this->convertToDecimal($total_charge_back_fee),
            'total_charge_back_transactions' => $this->convertToDecimal($total_charge_back_transactions),
            'total_non_domestic_transactions' => $this->convertToDecimal($total_non_domestic_transactions),
            'total_non_domestic_fee' => $this->convertToDecimal($total_non_domestic_fee),
            'total_service_charge' =>  $this->convertToDecimal($total_service_charge),
            'total_refund_transactions' => $total_refund_transactions,
            'net_fee' => $this->convertToDecimal($total_fee + $misc_charge + $monthly_fee + $weekly_fee + $total_service_charge),
            'net_total' => $this->convertToDecimal($net_total - $misc_charge - $monthly_fee - $weekly_fee - $total_service_charge),
            'payable_amount' => $this->convertToDecimal($payable_amount),
            'transactions' => $all_transactions,
            'business' => $data['business']
        ];
    }

    public function date_compare($key)
    {
        return function ($a, $b) use ($key) {
            $t1 = strtotime($a[$key]);
            $t2 = strtotime($b[$key]);
            return $t2 - $t1;
        };
    }
}
