<?php

namespace App\Http\Controllers;

use App\Business;
use App\Exports\PayoutRequestReportExport;
use App\Helpers\ImageHelper;
use App\PayoutRequest;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsConnect\ConnectAccount;
use App\ModelsConnect\ExternalAccount;
use App\Passbook;
use App\PayoutCharge;
use App\PayoutInvoice;
use App\User;
use Carbon\Carbon;
use Exception;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;
use Stripe\StripeClient;

class PayoutRequestController extends Controller
{
    /**
     * List payoutRequest.
     * GET /api/payout-requests
     */
    public function index(Request $req)
    {
        $payoutRequests = PayoutRequest::select("payout_requests.*", "businesses.name as business_name", "users.first_name", "users.last_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->leftJoin('users', 'users.id', '=', 'user_id')->orderByRaw('created DESC');

        if ($query = $req->query('query', false)) {
            $payoutRequests->whereRaw('(businesses.name LIKE ? OR first_name LIKE ? OR last_name LIKE ? OR amount LIKE ? OR note LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('status', false)) {
            $payoutRequests->where('status', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $payoutRequests->where('business_id', $query);
        }
        if ($query = $req->query('from_date', false)) {
            $this->validate($req, [
                'from_date' => 'date_format:Y-m-d',
                'to_date' => 'required_with:from_date|date_format:Y-m-d',
            ]);

            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();

            $payoutRequests = $payoutRequests->where('payout_requests.created', '>=', $from_date->getTimestamp());
            $payoutRequests = $payoutRequests->where('payout_requests.created', '<=', $to_date->getTimestamp());
        }

        if ($query = $req->query('user_id', false)) {
            $payoutRequests->where('user_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $payoutRequests = $payoutRequests->get();
        } else {
            $payoutRequests = $payoutRequests->paginate($req->query('per_page', 20));
        }

        if ($payoutRequests->isEmpty()) Utils::error('No payoutRequests found!', 404);

        return ($payoutRequests);
    }


    /**
     * Export payoutRequest.
     * POST /api/payout-requests/export
     */
    public function export(Request $req)
    {
        if ($req->input('from_date') == '' || $req->input('from_date') == 'null') {
            $req->request->remove('from_date');
            $req->request->remove('to_date');
        }
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $fileName = "PayoutRequest";
            $reports = ($this->index($req));
            return Excel::download(new PayoutRequestReportExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    /**
     * Create payoutRequest.
     * POST /api/payout-requests
     */
    public function create(Request $req)
    {

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // create payoutRequest
        $payoutRequest = new PayoutRequest;
        $payoutRequest->fill($req->all());
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
            if (!empty($business) && $business->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        if ($req->has('user_id')) {
            $user = User::find($req->input('user_id'));
            if (!empty($user) && $user->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }

        try {
            if ($req->has('external_account_id') && $req->has('description')) {
                $external_account = ExternalAccount::find($req->input('external_account_id'));
                if (empty($external_account)) Utils::error('external account not found!', 404);

                $created_start_date = Carbon::today()->format('Y-m-d 00:00:00');
                $created_end_date = Carbon::today()->format('Y-m-d 23:59:59');
                // $payout_requests = PayoutRequest::where('bank_account_id', $external_account->s_bank_account_id)->where('created_at', '>=', $created_start_date)->where('created_at', '<=', $created_end_date)->first();
                // if (!empty($payout_requests)) Utils::error('payout request already done for today!', 404);

                if ($business->s_account_id != null && $business->connect_service == 1) {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $stripeKeyColoumns = 'connect_stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $stripeKeyColoumns = 'connect_stripe_private_key_live';
                    }
                } else {
                    $mode = SiteSettingController::findByKey('stripe_mode')->value;
                    if ($mode == 'test') {
                        $stripeKeyColoumns = 'stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $stripeKeyColoumns = 'stripe_private_key_live';
                    }
                }

                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
                $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                $amount = $req->input('amount');
                // Add payout charges
                $payoutCharge = PayoutCharge::whereRaw('(amount>= ? AND (SELECT pc.amount FROM payout_charges pc WHERE pc.business_id = ? ORDER BY pc.amount DESC LIMIT 1))', [$amount, $business->id])->where('business_id', $business->id)->orderByRaw('amount ASC')->first();
                $calculatedFee = 0;
                if (!empty($payoutCharge)) {
                    $calculatedFee = $payoutCharge->charge;
                }
                // $transferrableAmount = $this->convertToDecimal($amount - $calculatedFee);
                $transferrableAmount = $this->convertToDecimal($amount);

                $payout = $stripe->payouts->create([
                    'amount' => $transferrableAmount * 100,
                    'currency' => 'gbp',
                    'description' => $req->input('description'),
                    'statement_descriptor' => $req->input('statement_descriptor'),
                    "destination" => $external_account->s_bank_account_id,
                ], [
                    "stripe_account" => $business->s_account_id,
                ]);
                $payoutRequest->payout_id = $payout->id;
                $payoutRequest->payout_status = $payout->status;
                $payoutRequest->created = $payout->created;
                $payoutRequest->arrival_date = $payout->arrival_date;
                if ($payout->status == 'paid') {
                    $payoutRequest->status = 'Accepted';
                }
                if ($payout->status == 'canceled') {
                    $payoutRequest->status = 'Rejected';
                }
                $payoutRequest->bank_account_id = $external_account->s_bank_account_id;
                $payoutRequest->save();
            } else {
                $payoutRequest->save();
            }
        } catch (Exception $e) {
            Utils::error($e->getMessage(), 400);
        }

        return $payoutRequest;
    }

    /**
     * Create summary payoutRequest.
     * POST /api/payout-requests/payout-summary
     */
    public function create_summary(Request $req)
    {
        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // create payoutRequest
        $business = Business::find($req->input('business_id'));
        if (!empty($business)) {
            $amount = $req->input('amount');
            // Add payout charges
            $payoutCharge = PayoutCharge::whereRaw('(amount>= ? AND (SELECT pc.amount FROM payout_charges pc WHERE pc.business_id = ? ORDER BY pc.amount DESC LIMIT 1))', [$amount, $business->id])->where('business_id', $business->id)->orderByRaw('amount ASC')->first();
            $calculatedFee = 0;
            if (!empty($payoutCharge)) {
                $calculatedFee = $payoutCharge->charge;
            }
            $transferrableAmount = $this->convertToDecimal($amount - $calculatedFee);
        } else {
            $amount = $req->input('amount');
            $calculatedFee = 0;
            $transferrableAmount = $this->convertToDecimal($amount - $calculatedFee);
        }

        return [
            'withdrawal_amount' => $this->convertToDecimal($amount),
            'withdrawal_fee' => $calculatedFee,
            'total_amount' => $transferrableAmount,
        ];
    }

    /**
     * Create bulk payoutRequest.
     * POST /api/payout-requests/bulk
     */
    public function bulk_create(Request $req)
    {
        if ($req->has('payouts') && !is_array($req->input('payouts')))
            $req->merge(['payouts' => json_decode($req->input('payouts'))]);

        $this->validate($req, [
            'payouts' => 'array',
        ]);

        if ($req->has('payouts') && !empty($req->input('payouts')) || sizeof($req->input('payouts')) > 0) {
            // create payoutRequest
            $payouts = ($req->input('payouts'));
            foreach ($payouts as $payout) {
                if ($payout->amount > 0) {
                    $payoutRequest = new PayoutRequest;
                    $payoutRequest->business_id = $payout->business_id;
                    $payoutRequest->note = $payout->description;
                    $payoutRequest->amount = $payout->amount;
                    $payoutRequest->status = $payout->status;

                    if ($payout->business_id) {
                        $business = Business::find($payout->business_id);
                    }
                    if ($payout->external_account_id && $payout->description) {
                        $external_account = ExternalAccount::where('id', $payout->external_account_id)->first();
                        if (empty($external_account)) Utils::error('external account not found!', 404);

                        $created_start_date = Carbon::today()->format('Y-m-d 00:00:00');
                        $created_end_date = Carbon::today()->format('Y-m-d 23:59:59');
                        // $payout_requests = PayoutRequest::where('bank_account_id', $external_account->s_bank_account_id)->where('created_at', '>=', $created_start_date)->where('created_at', '<=', $created_end_date)->first();
                        // if (!empty($payout_requests)) continue;

                        if ($business->s_account_id != null && $business->connect_service == 1) {
                            $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                            if ($mode == 'test') {
                                $stripeKeyColoumns = 'connect_stripe_private_key_test';
                            }
                            if ($mode == 'live') {
                                $stripeKeyColoumns = 'connect_stripe_private_key_live';
                            }
                        } else {
                            $mode = SiteSettingController::findByKey('stripe_mode')->value;
                            if ($mode == 'test') {
                                $stripeKeyColoumns = 'stripe_private_key_test';
                            }
                            if ($mode == 'live') {
                                $stripeKeyColoumns = 'stripe_private_key_live';
                            }
                        }

                        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
                        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                        $amount = $payout->amount;
                        // Add payout charges
                        $payoutCharge = PayoutCharge::whereRaw('(amount>= ? AND (SELECT pc.amount FROM payout_charges pc WHERE pc.business_id = ? ORDER BY pc.amount DESC LIMIT 1))', [$amount, $business->id])->where('business_id', $business->id)->orderByRaw('amount ASC')->first();
                        $calculatedFee = 0;
                        if (!empty($payoutCharge)) {
                            $calculatedFee = $payoutCharge->charge;
                        }
                        // $transferrableAmount = $this->convertToDecimal($amount - $calculatedFee);
                        try {

                            $transferrableAmount = $this->convertToDecimal($amount);
                            $payout = $stripe->payouts->create([
                                'amount' => $transferrableAmount * 100,
                                'currency' => 'gbp',
                                'description' => $payout->description,
                                'statement_descriptor' => $payout->statement_descriptor,
                                "destination" => $external_account->s_bank_account_id,
                            ], [
                                "stripe_account" => $business->s_account_id,
                            ]);
                            $payoutRequest->payout_id = $payout->id;
                            $payoutRequest->payout_status = $payout->status;
                            $payoutRequest->created = $payout->created;
                            $payoutRequest->arrival_date = $payout->arrival_date;
                            if ($payout->status == 'paid') {
                                $payoutRequest->status = 'Accepted';
                            }
                            if ($payout->status == 'canceled') {
                                $payoutRequest->status = 'Rejected';
                            }
                            $payoutRequest->bank_account_id = $external_account->s_bank_account_id;
                            $payoutRequest->save();
                        } catch (Exception $e) {
                            Log::debug($business->name . ' ' . $e);
                            continue;
                        }
                    }
                }
            }
        }

        return $payouts;
    }

    /**
     * Show payoutRequest.
     * GET /api/payout-requests/{id}
     */
    public function show(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);
        return $payoutRequest;
    }

    /**
     * Create payoutRequest.
     * POST /api/payout-requests
     */
    public function fetch(Request $req)
    {
        $businesses = Business::where('connect_service', '1')->whereNotNull('s_account_id')->where('disabled', '0')->orderByRaw('name ASC');
        if ($req->has('merchant')) {
            if ($req->query('merchant', false) == '1') {
                $businesses = $businesses->where('merchant_service', '1');
            } else {
                $businesses = $businesses->where('merchant_service', '0');
            }
        }
        if ($payout_mode = $req->query('payout_mode', false) != 'all') {
            $businesses = $businesses->where('payout_mode', $req->query('payout_mode'));
        }
        if ($req->query('nopaginate', false) == '1') {
            $businesses = $businesses->get();
        } else {
            $businesses = $businesses->paginate($req->query('per_page', 20));
        }
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);

        $businesses->makeHidden('business_commissions');
        $businesses->makeHidden('country');
        foreach ($businesses as $business) {
            $bankDescription = ExternalAccount::where('business_id', $business->id)->where('default', '1')->whereNotNull('description')->first();
            if (empty($bankDescription)) {
                $bankDescription = ExternalAccount::where('business_id', $business->id)->whereNotNull('description')->first();
            }

            if (!empty($bankDescription)) {
                if ($bankDescription->description) {
                    $business->payout_description = $bankDescription->description;
                } else {
                    $business->payout_description = 'Payout for ' . $business->name;
                }
            } else {
                $business->payout_description = 'Payout for ' . $business->name;
            }

            $bankDescriptor = ExternalAccount::where('business_id', $business->id)->where('default', '1')->whereNotNull('statement_descriptor')->first();
            if (empty($bankDescriptor)) {
                $bankDescriptor = ExternalAccount::where('business_id', $business->id)->whereNotNull('statement_descriptor')->first();
            }

            if (!empty($bankDescriptor)) {
                if ($bankDescriptor->statement_descriptor) {
                    $business->payout_statement_descriptor = $bankDescriptor->statement_descriptor;
                } else {
                    $business->payout_statement_descriptor = 'Payout for ' . $business->name;
                }
            } else {
                $business->payout_statement_descriptor = 'Payout for ' . $business->name;
            }

            $business->payable_amount = 0;
            $business->select_checkbox = true;
            $business->external_account_id = '';

            $carbon = Carbon::today()->subDays(7);
            if ($carbon->dayOfWeek <= 3) {
                $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
            } else {
                $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
            }
            // $req->merge(['from_date' => $from_date]);
            // $req->merge(['to_date' => $to_date]);
            // $req->merge(['payout_all' => true]);
            // $data = (new PayoutReportController())->quick_report($req, $business->id);
            // if ($data['normal'] != NULL) {
            //     if ($data['normal']['net_total'] > 0) {
            //         $business->payable_amount = $data['normal']['net_total'];
            //     }
            // }
            $carbon = Carbon::today()->subDays(7);
            $currentDayOfWeek = $carbon->dayOfWeek;
            if ($currentDayOfWeek <= 1) {
                $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
            } else {
                $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
            }
            $payoutInvoice = PayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'Normal')->where('business_id', $business->id)->first();
            if (!empty($payoutInvoice)) {
                if ($payoutInvoice->net_total > 0) {
                    $externalAccount = ExternalAccount::where('business_id', $business->id)->where('default', 1)->orderByRaw('id DESC')->first();
                    if (!empty($externalAccount)) {
                        // $stringValue = 'UBSIDI LIMITED,Tiffin Tom,TIFFINTOMUK LTD,Tiffin Tom Limited,Ubsidi Ltd,TIFFINTOM LIMITED,TiffinTomuk Ltd';
                        $stringValue = (new SiteSettingController())->findByKey('bank_string')->value;
                        if ($externalAccount->account_holder_name == '' || stripos($stringValue, $externalAccount->account_holder_name) !== false) {
                            $payable_amount = 1;
                            $business->payable_amount = $payoutInvoice->net_total;
                        } else {
                            $business->payable_amount = 0;
                        }
                    }
                }
            } else {
                $business->payable_amount = 0;
            }



            if ($business->s_account_id != null && $business->connect_service == 1) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
            }

            $business->bank_details = ExternalAccount::where('business_id', $business->id)->get();

            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
            try {
                $account = ConnectAccount::where('business_id', $business->id)->first();
                if (empty($account)) Utils::error('ConnectAccount not found!', 404);

                if ($account->s_account_id != null) {
                    $account_balace = $stripe->balance->retrieve([], ['stripe_account' => $account->s_account_id]);
                    if ($account_balace != null) {
                        $business->available_balance = $this->convertToDecimal($account_balace->available[0]->amount);
                    }
                    // $stripe_account = $stripe->accounts->retrieve($account->s_account_id);
                    // $bankIds = [];
                    // foreach ($stripe_account->external_accounts->data as $bankAccount) {
                    //     $dbExternalAccount = ExternalAccount::where('s_bank_account_id', $bankAccount->id)->latest()->first();
                    //     // if (empty($dbExternalAccount)) {
                    //     //     $dbExternalAccount = ExternalAccount::where('s_account_id', $bankAccount->account)->whereRaw('account_number LIKE ?', ["%$bankAccount->last4%"])->where('account_holder_name', $bankAccount->account_holder_name)->latest()->first();
                    //     // }
                    //     if (empty($dbExternalAccount)) {
                    //         $external_account = new ExternalAccount();
                    //         $external_account->connect_account_id = $account->id;
                    //         $external_account->s_account_id = $bankAccount->account;
                    //         $external_account->s_bank_account_id = $bankAccount->id;
                    //         $external_account->business_id = $business->id;
                    //         $external_account->country = $bankAccount->country;
                    //         $external_account->currency = $bankAccount->currency;
                    //         $external_account->account_holder_name = $bankAccount->account_holder_name;
                    //         $external_account->account_holder_type = $bankAccount->account_holder_type;
                    //         $external_account->account_number = $bankAccount->last4;
                    //         $external_account->routing_number = $bankAccount->routing_number;
                    //         $external_account->save();
                    //     } else {
                    //         $external_account = clone ($dbExternalAccount);
                    //     }
                    //     $external_account->connect_account_id = $account->id;
                    //     $external_account->business_id = $business->id;

                    //     $external_account->s_account_id = $bankAccount->account;
                    //     $external_account->default = $bankAccount->default_for_currency;

                    //     $external_account->save();
                    //     $bankIds[] = $external_account->id;
                    // }
                    // $business->bank_details = $stripe_account->external_accounts->data;
                    // foreach ($business->bank_details as $bankDetails) {
                    //     $checkExternalAccount = ExternalAccount::where('s_bank_account_id', $bankDetails->id)->first();
                    //     $bankDetails->description = $checkExternalAccount->description;
                    // }
                    // if (!empty($bankIds)) {
                    //     $deleteExternalAccounts = ExternalAccount::where('business_id', $business->id)->whereNotIn('id', $bankIds)->delete();
                    // }
                }
            } catch (Exception $e) {
                // Log::debug($e);
            }
        }

        return $businesses;
    }

    /**
     * Update payoutRequest.
     * POST /api/payout-requests/{id}
     */
    public function update(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        $dbPayoutRequest = $payoutRequest->clone();
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'rejection_note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // update payoutRequest
        if ($req->has('amount')) $payoutRequest->amount = $req->input('amount');
        if ($req->has('note')) $payoutRequest->note = $req->input('note');
        if ($req->has('status')) $payoutRequest->status = $req->input('status');
        if ($req->has('business_id')) $payoutRequest->business_id = $req->input('business_id');
        if ($req->has('user_id')) $payoutRequest->user_id = $req->input('user_id');
        if ($req->has('rejection_note')) $payoutRequest->rejection_note = $req->input('rejection_note');
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
            if (!empty($business) && $business->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        if ($req->has('user_id')) {
            $user = User::find($req->input('user_id'));
            if (!empty($user) && $user->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        $passbook = Passbook::where('entity', 'payout_request')->where('entity_id', $payoutRequest->id);
        if ($req->has('user_id')) {
            $passbook->where('user_id', $payoutRequest->user_id);
        }
        if ($req->has('business_id')) {
            $passbook->where('business_id', $payoutRequest->business_id);
        }
        $passbook = $passbook->first();
        if ($payoutRequest->status == 'Accepted') {
            if (empty($passbook)) {
                $passbook = new Passbook;
            }
            $passbook->entity = "payout_request";
            $passbook->entity_id = $payoutRequest->id;
            $passbook->business_id = $payoutRequest->business_id;
            $passbook->user_id = $payoutRequest->user_id;
            $passbook->date = Carbon::today()->toDateString();
            $passbook->from_date = $payoutRequest->created_at;
            $passbook->to_date = $payoutRequest->created_at;
            $passbook->amount = -1 * $payoutRequest->amount;
            $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($payoutRequest->created_at)->toDateString();
            $passbook->save();
        } else {
            if (!empty($passbook))
                $passbook->delete();
        }

        if ($payoutRequest->payout_id && $payoutRequest->status == 'Rejected') {

            if ($business->s_account_id != null && $business->connect_service == 1) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
            }

            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

            $stripe->payouts->cancel(
                $payoutRequest->payout_id,
                [],
                [
                    "stripe_account" => $business->s_account_id,
                ]
            );
        }
        $payoutRequest->save();

        return $payoutRequest;
    }

    /**
     * Enable payoutRequest.
     * POST /api/payout-requests/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);
        $payoutRequest->disabled = 0;
        $payoutRequest->save();
        return $payoutRequest;
    }

    /**
     * Disable payoutRequest.
     * POST /api/payout-requests/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);
        $payoutRequest->disabled = 1;
        $payoutRequest->save();
        return $payoutRequest;
    }

    /**
     * Delete payoutRequest by id.
     * DELETE /api/payout-requests/{id}
     */
    public function delete(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);
        $payoutRequest->delete();
        return $payoutRequest;
    }

    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
