<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\PaypalDetail;
use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\User;
use Faker\Factory;
use Illuminate\Http\Request;

class PaypalDetailController extends Controller
{
    /**
     * List paypalDetails.
     * GET /api/paypal-details
     */
    public function index(Request $req)
    {
        $paypalDetails = PaypalDetail::orderByRaw('name DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $paypalDetails->whereRaw('(name LIKE ? OR email LIKE ? )', ["%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $paypalDetails->where('business_id', $query);
        }
        if ($query = $req->query('user_id', false)) {
            $paypalDetails->where('user_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $paypalDetails = $paypalDetails->get();
        } else {
            $paypalDetails = $paypalDetails->paginate($req->query('per_page', 20));
        }

        if ($paypalDetails->isEmpty()) Utils::error('No paypalDetails found!', 404);


        return ($paypalDetails);
    }

    /**
     * Create paypalDetail.
     * POST /api/paypal-details
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => 'required|email',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // create paypalDetail
        $paypalDetail = new PaypalDetail;
        $paypalDetail->fill($req->all());

        $paypalDetail->save();

        return $paypalDetail;
    }

    /**
     * Show paypalDetail.
     * GET /api/paypal-details/{id}
     */
    public function show(Request $req, $id)
    {
        $paypalDetail = PaypalDetail::find($id);
        if (empty($paypalDetail)) Utils::error('PaypalDetail not found!', 404);
        return $paypalDetail;
    }

    /**
     * Update paypalDetail.
     * POST /api/paypal-details/{id}
     */
    public function update(Request $req, $id)
    {
        $paypalDetail = PaypalDetail::find($id);
        if (empty($paypalDetail)) Utils::error('PaypalDetail not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'email' => 'email',
            'business_id' => 'exists:businesses,id',
            'user_id' => 'exists:users,id',
        ]);

        // update paypalDetail
        if ($req->has('email')) $paypalDetail->email = $req->input('email');
        if ($req->has('name')) $paypalDetail->name = $req->input('name');
        if ($req->has('business_id')) $paypalDetail->business_id = $req->input('business_id');
        if ($req->has('user_id')) $paypalDetail->user_id = $req->input('user_id');

        $paypalDetail->save();

        return $paypalDetail;
    }


    /**
     * Delete paypalDetail by id.
     * DELETE /api/paypal-details/{id}/default
     */
    public function default(Request $req, $id)
    {
        $paypalDetail = PaypalDetail::find($id);
        if (empty($paypalDetail)) Utils::error('PaypalDetail not found!', 404);
        if ($paypalDetail->business_id != null) {
            $business = Business::find($paypalDetail->business_id);
            if (empty($paypalDetail)) Utils::error('Business not found!', 404);
            $business->paypal_detail_id = $paypalDetail->id;
            $business->save();
        } else 
        if ($paypalDetail->user_id != null) {
            $user = User::find($paypalDetail->user_id);
            if (empty($paypalDetail)) Utils::error('User not found!', 404);
            $user->paypal_detail_id = $paypalDetail->id;
            $user->save();
        }

        return $paypalDetail;
    }

    /**
     * Delete paypalDetail by id.
     * DELETE /api/paypal-details/{id}
     */
    public function delete(Request $req, $id)
    {
        $paypalDetail = PaypalDetail::find($id);
        if (empty($paypalDetail)) Utils::error('PaypalDetail not found!', 404);
        $paypalDetail->delete();
        if ($paypalDetail->image != null) ImageHelper::delete(ImageHelper::$TYPE_BRAND, $paypalDetail->image);
        return $paypalDetail;
    }
}
