<?php

namespace App\Http\Controllers;

use App\Business;
use App\BusinessCommission;
use App\Country;
use App\CountryCommission;
use App\CountryCommissionHistory;
use App\Exports\PaypalTransactionExports;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\PaypalPayment;
use Carbon\Carbon;
use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;
use PayPal\Api\Amount;
use PayPal\Api\Refund;
use PayPal\Api\Sale;
use PayPal\Rest\ApiContext;
use PayPal\Auth\OAuthTokenCredential;
use PayPal\Api\Payer;
use PayPal\Api\Item;
use PayPal\Api\ItemList;
use PayPal\Api\Transaction;
use PayPal\Api\RedirectUrls;
use PayPal\Api\Payment;
use PayPal\Exception\PayPalConnectionException;

class PaypalPaymentController extends Controller
{
    /**
     * List paypalPayments.
     * GET /api/paypal-payments
     */
    public function index(Request $req)
    {
        $paypalPayments = PaypalPayment::select("paypal_payments.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('sale_created_at DESC');

        if ($query = $req->query('query', false)) {
            $paypalPayments->whereRaw('(businesses.name LIKE ? OR description LIKE ? OR transaction_amount LIKE ? OR payment_id LIKE ? OR payer_country LIKE ? OR sale_state LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            // $paypalPayments->where('business_id', $query);
            $business = Business::where('id', $query)->first();
            $paypalPayments->whereRaw('(custom_description LIKE ?)', ["$business->online_paypal_description"]);
        }
        if ($query = $req->query('sale_state', false)) {
            $paypalPayments->where('sale_state', $query);
        }
        //We will remove this query soon. TOBE Deprecated
        if ($query = $req->query('transaction_status', false)) {
            if ($query == 'archived') {
                $paypalPayments->where('archived', '1');
            }
            if ($query == 'unarchived') {
                $paypalPayments->where('archived', '0');
            }
        }
        // After removing above query this will be usable
        if ($query = $req->query('archive_status', false)) {
            if ($query == 'archived') {
                $paypalPayments->where('archived', '1');
            }
            if ($query == 'unarchived') {
                $paypalPayments->where('archived', '0');
            }
        }
        if ($date_filter = $req->query('date_filter_type', false)) {
            if ($date = $req->query('date', false)) {
                if ($date_filter == 'in_last') {
                    $today = Carbon::today()->endOfDay();
                    $lastDay = Carbon::today()->subDays($date)->startOfDay();
                    $paypalPayments->where('sale_created_at', '>', $lastDay);
                    $paypalPayments->where('sale_created_at', '<', $today);
                } else if ($date_filter == 'before') {
                    $date = Carbon::parse($date)->startOfDay();
                    $paypalPayments->where('sale_created_at', '<', $date);
                } else if ($date_filter == 'after') {
                    $date = Carbon::parse($date)->endOfDay();
                    $paypalPayments->where('sale_created_at', '>', $date);
                } else if ($date_filter == 'between') {
                    $date = $req->query('date');
                    $dates = explode(",", $date);
                    $dates[0] = Carbon::parse($dates[0])->startOfDay();
                    $dates[1] = Carbon::parse($dates[1])->endOfDay();
                    $paypalPayments->where('sale_created_at', '>',  $dates[0]);
                    $paypalPayments->where('sale_created_at', '<',  $dates[1]);
                } else if ($date_filter == 'equal') {
                    $startTime = Carbon::parse($date)->startOfDay();
                    $endTime = Carbon::parse($date)->startOfDay();
                    $paypalPayments->where('sale_created_at', '>',  $startTime);
                    $paypalPayments->where('sale_created_at', '<',  $endTime);
                }
            } else {
                Utils::error("Date field required with type", 400);
            }
        }
        if ($query = $req->query('format', false)) {
            $paypalPayments = $paypalPayments->get();
            if ($query == 'excel') {
                return Excel::download(new PaypalTransactionExports($paypalPayments), "Paypal Transactions.xls", \Maatwebsite\Excel\Excel::XLS);
            }
            if ($query == 'pdf') {
                return Excel::download(new PaypalTransactionExports($paypalPayments), "Paypal Transactions.pdf", \Maatwebsite\Excel\Excel::MPDF);
            }
            Utils::error('Invalid format!', 404);
        }
        if ($req->query('nopaginate', false) == '1') {
            $paypalPayments = $paypalPayments->get();
        } else {
            $paypalPayments = $paypalPayments->paginate($req->query('per_page', 20));
        }

        if ($paypalPayments->isEmpty()) Utils::error('No paypal payments found!', 404);


        return ($paypalPayments);
    }
    public function show(Request $req, $id)
    {
        $paypalPayment = PaypalPayment::find($id);
        if (empty($paypalPayment)) Utils::error('Payment not found!', 404);
        $paypalPayment->append('online_order');
        return $paypalPayment;
    }
    public function transaction(Request $req)
    {
        $siteSettingMode = SiteSettingController::findByKey('paypal_mode');

        if ($siteSettingMode->value == 'live') {
            $siteSettingpublicKey = SiteSettingController::findByKey('paypal_public_key_live');
            $siteSettingprivateKey = SiteSettingController::findByKey('paypal_private_key_live');
            $url = 'https://api.paypal.com/v1/payments/payment';
            $publicKey = $siteSettingpublicKey->value;
            $privateKey = $siteSettingprivateKey->value;
        } else {
            $siteSettingpublicKey = SiteSettingController::findByKey('paypal_public_key_test');
            $siteSettingprivateKey = SiteSettingController::findByKey('paypal_private_key_test');
            $url = 'https://api.sandbox.paypal.com/v1/payments/payment';
            $publicKey =  $siteSettingpublicKey->value;
            $privateKey =  $siteSettingprivateKey->value;
        }
        return $this->fetch(null, $publicKey, $privateKey, $url);
    }
    public function fetch(Request $req = null, $publicKey, $privateKey, $url)
    {
        $payments = $this->makeApiCall(null,  $publicKey, $privateKey, $url);
        $reversedArray = array_reverse($payments);
        $this->calculateCommissionsAndSave($reversedArray);
        return $reversedArray;
    }
    public function makeApiCall($nextId = null,  $publicKey, $privateKey, $url)
    {
        // NOTE:: Change SubDays to get last how many days of transactions you want
        $query = [];
        $query['count'] = 20;
        $query['start_time'] = Carbon::today()->subDays(env('PAYPAL_DAYS_INTERVAL', 7))->startOfDay()->format('Y-m-d\TH:i:s\Z');
        $query['end_time'] = Carbon::today()->endOfDay()->format('Y-m-d\TH:i:s\Z');
        $query['sort_by'] = 'create_time';
        if ($nextId != null) {
            $query['start_id'] = $nextId;
        }
        $response = (new Client())->get($url, [
            'auth' => [$publicKey, $privateKey], // username, password
            'query' => $query
        ]);
        if ($response->getStatusCode() == 200) {
            $jsonBody = json_decode($response->getBody());
            $all_payments = $jsonBody->payments;

            if (isset($jsonBody->next_id) && $jsonBody->next_id != $nextId) {
                $all_payments1 = $this->makeApiCall($jsonBody->next_id,  $publicKey, $privateKey, $url);
                foreach ($all_payments1 as $payment) {
                    $all_payments[] = $payment;
                }
                return $all_payments;
            } else {
                return $all_payments;
            }
        } else if ($response->getStatusCode() == 400) {
            return $response->getBody();
        }
    }
    public function calculateCommissionsAndSave($payments)
    {
        foreach ($payments as $item) {
            $dbPaypalPayment = PaypalPayment::where('payment_id', $item->id)->first();
            $paypalPayment = new PaypalPayment();
            if (!empty($dbPaypalPayment)) {
                $paypalPayment = $dbPaypalPayment;
            }
            $paypalPayment->payment_id = $item->id;
            $paypalPayment->state = $item->state;
            $paypalPayment->description = $item->transactions[0]->description;
            if (isset($item->transactions[0]->custom))
                $paypalPayment->custom_description = $item->transactions[0]->custom;
            else $paypalPayment->custom_description = $item->transactions[0]->description;

            if (isset($item->payer) && $item->payer != null && isset($item->payer->payer_info) && $item->payer->payer_info != null) {
                $paypalPayment->payer_country = $item->payer->payer_info->country_code;
                if (isset($item->payer->payer_info->email) && $item->payer->payer_info->email != null)
                    $paypalPayment->email = $item->payer->payer_info->email;
            }
            $paypalPayment->refunded_amount = 0;
            foreach ($item->transactions[0]->related_resources as $resource) {
                if (isset($resource->sale)) {
                    $paypalPayment->sale_created_at = $resource->sale->create_time;
                    $paypalPayment->sale_id = $item->transactions[0]->related_resources[0]->sale->id;
                    $paypalPayment->sale_state = $item->transactions[0]->related_resources[0]->sale->state;
                    $paypalPayment->transaction_amount = $item->transactions[0]->related_resources[0]->sale->amount->total;
                    $paypalPayment->transaction_fee = $item->transactions[0]->related_resources[0]->sale->transaction_fee->value;
                    $paypalPayment->currency = $item->transactions[0]->related_resources[0]->sale->amount->currency;
                }
                if (isset($resource->refund)) {
                    $paypalPayment->refund_created_at = $resource->refund->create_time;
                    $paypalPayment->refunded_amount = $paypalPayment->refunded_amount + $resource->refund->amount->total;
                    if (isset($resource->refund->capture_id)) {
                        $paypalPayment->capture_id = $resource->refund->capture_id;
                    }
                }
            }

            $business = Business::where('restaurant_name', $paypalPayment->custom_description)->first();
            if (!empty($business)) {
                $paypalPayment->business_id = $business->id;
                if ($business->auto_archive == 1) {
                    $paypalPayment->archived = 1;
                }
                if ($paypalPayment->id != null) {
                    $countryCommissionHistory = CountryCommissionHistory::where('paypal_payment_id', $paypalPayment->id)->first();
                }
                if (empty($countryCommissionHistory) || !empty($countryCommissionHistory)) {
                    $country = Country::where('iso', $paypalPayment->payer_country)->first();
                    if (!empty($country)) {
                        $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();

                        if ($country->id == $business->country_id) {
                            $paypalPayment->non_domestic = 0;
                        } else {
                            $paypalPayment->non_domestic = 1;
                        }
                    }
                    if (empty($countryCommission)) {
                        $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                        if (empty($country))
                            $paypalPayment->non_domestic = 1;
                    }
                    $paypalPayment->save();
                    $dbCountryCommissionHistory = CountryCommissionHistory::where('paypal_payment_id', $paypalPayment->id)->first();
                    if (empty($dbCountryCommissionHistory)) {
                        $countryCommissionHistory = new CountryCommissionHistory();
                        if (!empty($countryCommission)) {
                            $countryCommissionHistory->paypal_payment_id = $paypalPayment->id;
                            $countryCommissionHistory->paypal_percentage = $countryCommission->paypal_percentage;
                            $countryCommissionHistory->paypal_fixed = $countryCommission->paypal_fixed;
                            $countryCommissionHistory->save();
                        } else {
                            $countryCommissionHistory->paypal_payment_id = $paypalPayment->id;
                            $countryCommissionHistory->paypal_percentage = 0;
                            $countryCommissionHistory->paypal_fixed = 0;
                            $countryCommissionHistory->save();
                        }
                    }
                }
            } else {
                $paypalPayment->save();
                $countryCommissionHistory = CountryCommissionHistory::where('paypal_payment_id', $paypalPayment->id)->first();
                if (empty($countryCommissionHistory)) {
                    $countryCommissionHistory = new CountryCommissionHistory();
                }
                $countryCommissionHistory->paypal_payment_id = $paypalPayment->id;
                $countryCommissionHistory->paypal_percentage = 0;
                $countryCommissionHistory->paypal_fixed = 0;
                $countryCommissionHistory->save();
            }
        }
    }
    /**
     * Archive Bulk Transacitons.
     * POST paypal-payments/archive
     */
    public function archive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = PaypalPayment::whereIn('id', $req->input('ids'))->update(['archived' => '1']);
            $transactions = PaypalPayment::whereIn('id', $req->input('ids'))->get();
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }


    /**
     * UnArchive Bulk Transacitons.
     * POST paypal-payments/unarchive
     */
    public function unarchive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = PaypalPayment::whereIn('id', $req->input('ids'))->update(['archived' => '0']);
            $transactions = PaypalPayment::whereIn('id', $req->input('ids'))->get();
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }

    /**
     * Refund Paypal Transaction.
     * POST paypal-payments/{id}/refund
     * @param int amount
     * @param string password
     */
    public function refund(Request $req, $id)
    {
        $this->validate($req, [
            'amount' => 'gt:0|numeric|required',
            'password' => 'required', // Human amount
        ]);
        if (Auth::isAdmin()) {
            if (!Hash::check($req->input('password'), Auth::admin()->password)) {
                Utils::error("Password is incorrect", 400);
            }
        }
        if (Auth::isUser()) {
            if (!Hash::check($req->input('password'), Auth::user()->password)) {
                Utils::error("Password is incorrect", 400);
            }
        }
        $paypalPayment = PaypalPayment::find($id);
        if (empty($paypalPayment)) Utils::error("Payment not found", 404);
        if (strtolower($paypalPayment->sale_state) == 'refunded') Utils::error("This payment is already refunded", 400);
        if ($req->input('amount') > $this->convertToDecimal($paypalPayment->transaction_amount - abs($paypalPayment->refunded_amount))) Utils::error("Cannot refund more than transaction amount", 400);

        $siteSettingMode = SiteSettingController::findByKey('paypal_mode');

        if ($siteSettingMode->value == 'live') {
            $siteSettingpublicKey = SiteSettingController::findByKey('paypal_public_key_live');
            $siteSettingprivateKey = SiteSettingController::findByKey('paypal_private_key_live');
        } else {
            $siteSettingpublicKey = SiteSettingController::findByKey('paypal_public_key_test');
            $siteSettingprivateKey = SiteSettingController::findByKey('paypal_private_key_test');
        }
        $publicKey = $siteSettingpublicKey->value;
        $privateKey = $siteSettingprivateKey->value;
        try {
            $apiContext = new ApiContext(new OAuthTokenCredential($publicKey, $privateKey));
            $singlePayment = Payment::get($paypalPayment->payment_id, $apiContext);
            $paypalPayment->sale_state = $singlePayment->transactions[0]->related_resources[0]->sale->state;
            $refunded_amount = 0;
            foreach ($singlePayment->transactions[0]->related_resources as $resource) {
                if (isset($resource->refund)) {
                    $paypalPayment->refund_created_at = $resource->refund->create_time;
                    $refunded_amount = $refunded_amount + $resource->refund->amount->total;
                    if (isset($resource->refund->capture_id)) {
                        $paypalPayment->capture_id = $resource->refund->capture_id;
                    }
                }
            }
            $paypalPayment->refunded_amount = $refunded_amount;
            $paypalPayment->save();


            if (strtolower($paypalPayment->sale_state) == 'refunded') Utils::error("Payment is already refunded", 400);
            $totalRefundableAmount = $paypalPayment->transaction_amount - $paypalPayment->refunded_amount;
            if ($req->input('amount') > $totalRefundableAmount) {
                Utils::error("Transaction cannot refund more than transaction amount", 400);
            }

            //Create refund
            $amt = new Amount();
            $amt->setTotal($req->input('amount'))->setCurrency($paypalPayment->currency);

            $refund = new Refund();
            $refund->setAmount($amt);
            $sale = new Sale();
            $sale->setId($paypalPayment->sale_id);

            //Execute Refund
            $sale->refund($refund, $apiContext);

            //Retrive single payment
            $singlePayment = Payment::get($paypalPayment->payment_id, $apiContext);
            $paypalPayment->sale_state = $singlePayment->transactions[0]->related_resources[0]->sale->state;
            $refunded_amount = 0;
            foreach ($singlePayment->transactions[0]->related_resources as $resource) {
                if (isset($resource->refund)) {
                    $paypalPayment->refund_created_at = $resource->refund->create_time;
                    $refunded_amount = $refunded_amount + $resource->refund->amount->total;
                    if (isset($resource->refund->capture_id)) {
                        $paypalPayment->capture_id = $resource->refund->capture_id;
                    }
                }
            }
            $paypalPayment->refunded_amount = $refunded_amount;

            $paypalPayment->save();
            return $paypalPayment;
        } catch (PayPalConnectionException $ex) {
            return $ex->getData();
        } catch (Exception $ex) {
            return $ex->getMessage();
        }
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
