<?php

namespace App\Http\Controllers;

use App\Position;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class PositionController extends Controller
{
    /**
     * List positions.
     * GET /api/positions
     */
    public function index(Request $req)
    {
        $positions = Position::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $positions->whereRaw('(name LIKE ?)', ["%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $positions = $positions->get();
        } else {
            $positions = $positions->paginate($req->query('per_page', 20));
        }

        if ($positions->isEmpty()) Utils::error('No positions found!', 404);

        return $positions;
    }

    /**
     * List positions.
     * GET /api/positions/mini
     */
    public function mini(Request $req)
    {
        $positions = Position::orderByRaw('name ASC');
        $positions->where('disabled', 0);
        if ($query = $req->query('query', false)) {
            $positions->whereRaw('(name LIKE ? )', ["%$query%"]);
        }
        $positions = $positions->get();
        if ($positions->isEmpty()) Utils::error('No positions found!', 404);

        return ($positions);
    }


    /**
     * Create position.
     * POST /api/positions
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
        ]);

        // create position
        $position = new Position;
        $position->fill($req->all());
        $position->save();

        return $position;
    }

    /**
     * Show position.
     * GET /api/positions/{id}
     */
    public function show(Request $req, $id)
    {
        $position = Position::find($id);
        if (empty($position)) Utils::error('Position not found!', 404);
        return $position;
    }

    /**
     * Update position.
     * POST /api/positions/{id}
     */
    public function update(Request $req, $id)
    {
        $position = Position::find($id);
        if (empty($position)) Utils::error('Position not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
        ]);

        // update position
        if ($req->has('name')) $position->name = $req->input('name');
        $position->save();

        return $position;
    }

    /**
     * Delete position by id.
     * DELETE /api/positions/{id}
     */
    public function delete(Request $req, $id)
    {
        $position = Position::find($id);
        if (empty($position)) Utils::error('Position not found!', 404);
        $position->delete();
        return $position;
    }

    /**
     * Enable position.
     * POST /api/positions/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $position = Position::find($id);
        if (empty($position)) Utils::error('Position not found!', 404);
        $position->disabled = 0;
        $position->save();
        return $position;
    }

    /**
     * Disable position.
     * POST /api/positions/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $position = Position::find($id);
        if (empty($position)) Utils::error('Position not found!', 404);
        $position->disabled = 1;
        $position->save();
        return $position;
    }
}
