<?php

namespace App\Http\Controllers;

use App\Business;
use App\CaxtonBeneficiary;
use App\CaxtonPayoutRequest;
use App\CaxtonTransfer;
use App\CaxtonUser;
use App\CaxtonWallet;
use App\Exports\UbsidiWalletTransferTransactionsExport;
use App\Helpers\ImageHelper;
use App\PouchHistory;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Jobs\WalletTransferCreatorJob;
use App\Passbook;
use App\PayoutInvoice;
use Carbon\Carbon;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Date;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Queue;

class PouchHistoryController extends Controller
{
    /**
     * List pouch-histories.
     * GET /api/pouch-histories
     */
    public function index(Request $req)
    {
        $pouch_histories = PouchHistory::selectRaw('pouch_history.*,businesses.name as business_name')->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('pouch_history.created_at DESC');

        if ($query = $req->query('query', false)) {
            $pouch_histories->whereRaw('(pouch_history.debit_amount LIKE ? OR pouch_history.credit_amount LIKE ? OR pouch_history.description LIKE ? OR pouch_history.transaction_category LIKE ? OR pouch_history.device_type LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($business_id = $req->query('business_id', false)) {
            if ($business_id != 'null') {
                $pouch_histories->where('pouch_history.business_id', $business_id);
            }
        }

        if ($req->input('is_admin') != '') {
            $is_admin = $req->input('is_admin');
            $pouch_histories->where('pouch_history.is_admin', $is_admin);
        }

        if ($author = $req->query('author', false)) {
            if ($author != 'null') {
                if ($author == 'user') {
                    $pouch_histories->where('pouch_history.is_admin', 0);
                }
                if ($author == 'admin') {
                    $pouch_histories->where('pouch_history.is_admin', 1);
                }
            }
        }

        if ($transaction_category = $req->query('transaction_category', false)) {
            if ($transaction_category != 'null') {
                $pouch_histories->where('transaction_category', $transaction_category);
            }
        }
        if ($status = $req->query('status', false)) {
            if ($status != 'null') {
                $pouch_histories->where('status', $status);
            }
        }
        if ($query = $req->query('from_date', false)) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $this->validate($req, [
                    'from_date' => 'date_format:Y-m-d',
                    'to_date' => 'required_with:from_date|date_format:Y-m-d',
                ]);

                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();

                $pouch_histories = $pouch_histories->where('pouch_history.created_at', '>=', $from_date);
                $pouch_histories = $pouch_histories->where('pouch_history.created_at', '<=', $to_date);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $pouch_histories = $pouch_histories->get();
        } else {
            $pouch_histories = $pouch_histories->paginate($req->query('per_page', 20));
        }

        if ($pouch_histories->isEmpty()) Utils::error('No wallet transaction found!', 404);

        return ($pouch_histories);
    }

    /**
     * Count pouch-histories.
     * GET /api/pouch-histories/count
     */
    public function index_count(Request $req)
    {
        $pouch_histories = PouchHistory::selectRaw('pouch_history.*,businesses.name as business_name')->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $pouch_histories->whereRaw('(debit_amount LIKE ? OR credit_amount LIKE ? OR description LIKE ? OR transaction_category LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($business_id = $req->query('business_id', false)) {
            if ($business_id != 'null') {
                $pouch_histories->where('business_id', $business_id);
            }
        }

        if ($req->input('is_admin') != '') {
            $is_admin = $req->input('is_admin');
            $pouch_histories->where('is_admin', $is_admin);
        }

        if ($author = $req->query('author', false)) {
            if ($author != 'null') {
                if ($author == 'user') {
                    $pouch_histories->where('pouch_history.is_admin', 0);
                }
                if ($author == 'admin') {
                    $pouch_histories->where('pouch_history.is_admin', 1);
                }
            }
        }

        if ($transaction_category = $req->query('transaction_category', false)) {
            $pouch_histories->where('transaction_category', $transaction_category);
        }

        if ($query = $req->query('from_date', false)) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $this->validate($req, [
                    'from_date' => 'date_format:Y-m-d',
                    'to_date' => 'required_with:from_date|date_format:Y-m-d',
                ]);

                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();

                $pouch_histories = $pouch_histories->where('pouch_history.created_at', '>=', $from_date);
                $pouch_histories = $pouch_histories->where('pouch_history.created_at', '<=', $to_date);
            }
        }

        $pouch_histories = $pouch_histories->count();

        $data = [
            'no_of_transactions' => (int)$pouch_histories,
        ];

        return $data;
    }

    /**
     * Export pouch-histories.
     * GET /api/pouch-histories/export
     */
    public function export_index(Request $req)
    {
        $reports = '';
        $req->merge(['nopaginate' => 1]);

        if ($req->has('from_date')) {
            if ($req->input('from_date') != 'null' || $req->input('to_date') != 'null') {
                $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
                $to_date = Carbon::parse($req->input('to_date'))->endOfDay();
                $fileName =  $from_date->format('d-m-Y') . " to " . $to_date->format('d-m-Y');
            } else {
                $fileName = "Wallet Transactions";
            }
        } else {
            $fileName = "Wallet Transactions";
        }

        $type = $req->query('format', 'excel');
        $reports = ($this->index($req));
        if ($type == 'excel') {
            return Excel::download(new UbsidiWalletTransferTransactionsExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else if ($type == 'pdf') {
            return Excel::download(new UbsidiWalletTransferTransactionsExport($reports), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    /**
     * Create pouch-history.
     * POST /api/pouch-histories
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'debit_amount' => 'numeric',
            'credit_amount' => 'numeric',
            'description' => 'string',
            'transaction_category' => 'in:credit,debit',
            'remark' => 'string',
            'wallet_id' => 'string',
            'status_description' => 'string',
            'device_type' => 'in:Web,Android,iOS',
            'status' => 'in:Pending,Accepted,Rejected',
            'is_admin' => 'boolean',
        ]);

        $pouchWeight = (new SiteSettingController())->findByKey('pouch_weight');
        if ($req->input('debit_amount') > $pouchWeight->value) {
            Utils::error('you have insufficient balance', 404);
        }

        // create pouch-history
        $pouch_history = new PouchHistory;
        $pouch_history->fill($req->all());
        $pouch_history->transaction_category = 'debit';
        $pouch_history->description = 'Wallet Debited';
        $pouch_history->is_admin = 1;
        $pouch_history->status_description = 'create';
        $pouch_history->updater_id = Auth::id();
        $pouch_history->save();

        return $pouch_history;
    }

    /**
     * Load Main Wallet pouch-history.
     * POST /api/pouch-histories/load-main-pouch
     */
    public function load_main_pouch(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'debit_amount' => 'numeric',
            'credit_amount' => 'numeric',
            'description' => 'string',
            'transaction_category' => 'in:credit,debit',
            'remark' => 'string',
            'wallet_id' => 'string',
            'status_description' => 'string',
            'device_type' => 'in:Web,Android,iOS',
            'status' => 'in:Pending,Accepted,Rejected',
            'is_admin' => 'boolean',
        ]);

        // create pouch-history
        $pouch_history = new PouchHistory;
        $pouch_history->fill($req->all());
        $pouch_history->transaction_category = 'credit';
        $pouch_history->description = 'Wallet Credited';
        $pouch_history->updater_id = Auth::id();
        $pouch_history->is_admin = 1;
        $pouch_history->status_description = 'load-main-pouch';
        $pouch_history->save();

        return $pouch_history;
    }

    /**
     * UnLoad Main Wallet pouch-history.
     * POST /api/pouch-histories/unload-main-pouch
     */
    public function unload_main_pouch(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'debit_amount' => 'numeric',
            'credit_amount' => 'numeric',
            'description' => 'string',
            'transaction_category' => 'in:credit,debit',
            'remark' => 'string',
            'wallet_id' => 'string',
            'status_description' => 'string',
            'device_type' => 'in:Web,Android,iOS',
            'status' => 'in:Pending,Accepted,Rejected',
            'is_admin' => 'boolean',
        ]);

        $business = Business::find($req->input('business_id'));
        if (empty($business)) Utils::error('business not found!', 404);

        if ($req->input('credit_amount') > $business->pouch_weight) {
            Utils::error('you have insufficient balance', 404);
        }

        // create pouch-history
        $pouch_history = new PouchHistory;
        $pouch_history->fill($req->all());
        $pouch_history->transaction_category = 'credit';
        $pouch_history->description = 'Wallet Credited';
        $pouch_history->updater_id = Auth::id();
        $pouch_history->is_admin = 1;
        $pouch_history->status_description = 'unload-main-pouch';
        $pouch_history->save();

        return $pouch_history;
    }

    /**
     * Create Wallet pouch-history.
     * POST /api/pouch-histories/wallet-transfer
     */
    public function wallet_create(Request $req)
    {
        $this->validate($req, [
            'wallet_id' => 'required|string',
            'business_id' => 'required|exists:businesses,id',
            'debit_amount' => 'required|numeric|gte:10',
            'description' => 'string',
            'remark' => 'string',
            'wallet_id' => 'string',
            'status_description' => 'string',
            'device_type' => 'in:Web,Android,iOS',
            'status' => 'in:Pending,Accepted,Rejected',
        ]);

        $business = Business::find($req->input('business_id'));
        if (empty($business)) Utils::error('business not found!', 404);

        if ($req->input('debit_amount') > $business->pouch_weight) {
            Utils::error('you have insufficient balance', 404);
        }

        $jobs = DB::table('jobs')->where('queue', 'wallet_transfer')->get();
        if ($jobs->isEmpty()) {
            $time = Carbon::now();
            $time->addMinute();
        } else {
            foreach ($jobs as $result) {
                $time = $result->available_at;
                $time = Date::parse($time)->addMinute();
            }
        }

        $caxtonWallet = CaxtonWallet::where('wallet_id', $req->input('wallet_id'))->first();

        $bankPayoutRequest = new CaxtonPayoutRequest();
        $bankPayoutRequest->business_id = $req->input('business_id');
        $bankPayoutRequest->wallet_id = $caxtonWallet->id;
        $bankPayoutRequest->note = 'Wallet Transfer ' . $caxtonWallet->name;
        $bankPayoutRequest->amount = $req->input('debit_amount');
        $bankPayoutRequest->status = 'Pending';
        $bankPayoutRequest->available_at = $time;
        $bankPayoutRequest->save();

        // Business Wallet Update
        $business->pouch_weight = $business->pouch_weight - $req->input('debit_amount');
        $business->save();

        Queue::later($time, new WalletTransferCreatorJob($bankPayoutRequest, true), '', 'wallet_transfer');

        return $bankPayoutRequest;
    }

    /**
     * Create Bank pouch-history.
     * POST /api/pouch-histories/bank-transfer
     */
    public function bank_create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'debit_amount' => 'required|numeric|gte:0',
            'otp' => 'required|string|max:6',
            'fee' => 'numeric',
            'description' => 'string',
            'remark' => 'string',
            'wallet_id' => 'string',
            'status_description' => 'string',
            'device_type' => 'in:Web,Android,iOS',
            'status' => 'in:Pending,Accepted,Rejected',
        ]);

        $business = Business::find($req->input('business_id'));
        if (empty($business)) Utils::error('business not found!', 404);

        $amount = $req->input('debit_amount') + $req->input('fee');

        if ($amount > $business->pouch_weight) {
            Utils::error('you have insufficient balance', 404);
        }

        $user = CaxtonUser::where('business_id', $req->input('business_id'))->first();
        if (empty($user)) Utils::error('Bank not found!', 404);

        if ($req->input('otp') !=  $user->otp) {
            return Utils::error('otp not matched.please enter correct otp.', 400);
        }

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $endPoint = 'account/balances/third-party/transfer';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "UserId":"' . $user->account_id . '",
                    "SendCurrency": "GBP",
                    "SendAmount": ' . (float)$req->input('debit_amount') . ',
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                // create pouch-history
                $pouch_history = new PouchHistory;
                $pouch_history->fill($req->all());
                $pouch_history->transaction_category = 'debit';
                if (!$req->input('description')) {
                    $pouch_history->description = 'Bank Debited';
                }
                $pouch_history->status_description = 'bank-transfer';
                $pouch_history->status = 'Accepted';
                $pouch_history->updater_id = Auth::id();
                $pouch_history->save();

                $business->pouch_weight = $business->pouch_weight - $amount;
                $business->save();

                // Total Balance Update
                $pouch_history->total_balance = $business->pouch_weight;
                $pouch_history->save();
            }
        } catch (\Exception $e) {
            Log::debug($e->getMessage());
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            }
        }
        return $pouch_history;
    }

    /**
     * Load Wallet pouch-history.
     * POST /api/pouch-histories/wallet-load
     */
    public function wallet_load(Request $req)
    {
        $business = Business::find($req->input('business_id'));
        if (empty($business)) Utils::error('business not found!', 404);

        $this->validate($req, [
            'wallet_id' => 'required|string',
            'business_id' => 'required|exists:businesses,id',
            'credit_amount' => 'required|numeric|gte:10',
            'description' => 'string',
            'remark' => 'string',
            'wallet_id' => 'string',
            'status_description' => 'string',
            'device_type' => 'in:Web,Android,iOS',
            'status' => 'in:Pending,Accepted,Rejected',
        ]);

        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $wallet_id = $req->input('wallet_id');
        $credit_amount = $req->input('credit_amount');

        $endPointWallet = 'wallets/wallet/' . $wallet_id . '/balances/balance/specific/ccy/GBP';

        $client = new \GuzzleHttp\Client();
        $dataWallet = $client->request('GET', env('CAXTON_BASEURL') . $endPointWallet, [
            'headers' => [
                'Authorization' => 'bearer ' . $apiKey,
                'UserApiToken' => $user_apiKey,
                'content-type' => 'application/json',
            ],
        ]);
        if ($dataWallet->getStatusCode() >= 200 && $dataWallet->getStatusCode() <= 210) {
            $dataWallet = json_decode($dataWallet->getBody()->getContents());
            $balances = $dataWallet->Content->Model->Balances;
            $balance_amount = 0;
            foreach ($balances as $balance) {
                $balance_amount = $balance->Balance;
            }
            if ($req->input('credit_amount') > $balance_amount) {
                Utils::error('you have insufficient balance', 404);
            }

            $endPoint = 'wallets/balances/redeem/' . $wallet_id;

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                        "Currency":"GBP",
                        "Amount":"' . $credit_amount . '",
                    }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                // Create pouch-history
                $pouch_history = new PouchHistory;
                $pouch_history->fill($req->all());
                $pouch_history->transaction_category = 'credit';
                $pouch_history->description = 'Wallet Load';
                $pouch_history->status_description = 'wallet-load';
                $pouch_history->status = 'Accepted';
                $pouch_history->updater_id = Auth::id();
                $pouch_history->save();

                // Business Wallet Update
                $business->pouch_weight = $business->pouch_weight + $pouch_history->credit_amount;
                $business->save();

                // Total Balance Update
                $pouch_history->total_balance = $business->pouch_weight;
                $pouch_history->save();

                return $pouch_history;
            }
        }
    }

    /**
     * Bank Wallet pouch-history.
     * POST /api/pouch-histories/bank-load
     */
    public function bank_load(Request $req)
    {
        $business = Business::find($req->input('business_id'));
        if (empty($business)) Utils::error('business not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'credit_amount' => 'required|numeric',
            'wallet_id' => 'required|string',
            'fee' => 'numeric',
        ]);

        try {
            $caxtonUser = CaxtonUser::where('business_id', $business->id)->first();
            if (empty($caxtonUser)) Utils::error('user not found!', 404);
            $user_id = SiteSettingController::findByKey('caxton_user_id')->value;

            // Create pouch-history
            $pouch_history = new PouchHistory;
            $pouch_history->fill($req->all());
            $pouch_history->transaction_category = 'credit';
            $pouch_history->description = 'Bank Load';
            $pouch_history->status = 'Pending';
            $pouch_history->status_description = 'bank-load';
            $pouch_history->updater_id = Auth::id();
            $pouch_history->save();

            $client = new \GuzzleHttp\Client();
            $endPoint = 'account/balances/third-party/transfer';

            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "UserId":"' . $user_id . '",
                    "SendCurrency": "GBP",
                    "SendAmount": ' . (float)$req->input('credit_amount') . ',
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $caxtonUser->token,
                    'UserApiToken' => $caxtonUser->human_password,
                    'content-type' => 'application/json',
                ],
            ]);

            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $endPointFee = 'account/balances/third-party/transfer';
                $dataFee = $client->request('POST', env('CAXTON_BASEURL') . $endPointFee, [
                    'body' => '{
                    "UserId":"' . $user_id . '",
                    "SendCurrency": "GBP",
                    "SendAmount": ' . (float)$req->input('fee') . ',
                }',
                    'headers' => [
                        'Authorization' => 'bearer ' . $caxtonUser->token,
                        'UserApiToken' => $caxtonUser->human_password,
                        'content-type' => 'application/json',
                    ],
                ]);
                if ($dataFee->getStatusCode() >= 200 && $dataFee->getStatusCode() <= 210) {
                    $this->external_wallet_load($req);
                    // Create pouch-history
                    // $pouch_history = new PouchHistory;
                    // $pouch_history->fill($req->all());
                    // $pouch_history->transaction_category = 'credit';
                    // $pouch_history->description = 'Bank Load';
                    // $pouch_history->status_description = 'bank-load';
                    // $pouch_history->updater_id = Auth::id();
                    $pouch_history->status = 'Accepted';
                    $pouch_history->save();

                    // Business Wallet Update
                    $business->pouch_weight = $business->pouch_weight + $pouch_history->credit_amount;
                    $business->save();

                    // Total Balance Update
                    $pouch_history->total_balance = $business->pouch_weight;
                    $pouch_history->save();

                    return $pouch_history;
                }
            }
        } catch (\Exception $e) {
            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                // $message = str_replace(
                //     rtrim($e->getMessage()),
                //     (string) $e->getResponse()->getBody(),
                //     (string) $e
                // );
                // Log::debug($message);
                $pouch_history->status = 'Rejected';
                $pouch_history->save();
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            }
        }
    }

    public function external_wallet_load($req)
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        try {
            $wallet_id = $req->input('wallet_id');
            $amount = $req->input('credit_amount');

            $endPoint = 'payments/currency-bank/card-load';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                    "WalletId":"' . $wallet_id . '",
                    "Currency":"GBP",
                    "Amount":"' . $amount . '",
                }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());
                $shop_order_id  = $data->Content->Model->ShopOrderId;
                $processEndPoint = 'payments/currency-bank/process';
                $walletDetails = $client->request('POST', env('CAXTON_BASEURL') . $processEndPoint, [
                    'body' => '{
                        "ShopOrderId":"' . $shop_order_id . '",
                    }',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);

                if ($walletDetails->getStatusCode() >= 200 && $walletDetails->getStatusCode() <= 210) {
                    $walletDetails = json_decode($walletDetails->getBody()->getContents());
                }
            }

            return ['message' => 'wallet load successfully!'];
        } catch (\Exception $e) {
        }
    }

    /**
     * Show pouch-history.
     * GET /api/pouch-histories/{id}
     */
    public function show(Request $req, $id)
    {
        $pouch_history = PouchHistory::find($id);
        if (empty($pouch_history)) Utils::error('Wallet transaction not found!', 404);
        return $pouch_history;
    }

    /**
     * Business beneficiary list.
     * GET /api/pouch-histories/beneficiary/{id}
     */
    public function beneficiary_index(Request $req, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error('Business not found!', 404);

        $beneficiaries = CaxtonBeneficiary::where('business_id', $business->id)->get();
        if (empty($beneficiaries)) Utils::error('Beneficiary not found!', 404);

        return $beneficiaries;
    }

    /**
     * Business wallet list.
     * GET /api/pouch-histories/wallet/{id}
     */
    public function wallet_index(Request $req, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error('Business not found!', 404);

        $wallets = CaxtonWallet::where('business_id', $business->id);
        if (empty($wallets)) Utils::error('Wallet not found!', 404);

        if ($bank = $req->query('bank', false)) {
            if ($bank == 'bank') {
                $wallets =  $wallets->where('bank', '1');
            }
            if ($bank == 'not_bank') {
                $wallets =  $wallets->where('bank', '0');
            }
        }
        $wallets =  $wallets->get();

        return $wallets;
    }

    /**
     * Update pouch-history.
     * POST /api/pouch-histories/{id}
     */
    public function update(Request $req, $id)
    {
        $pouch_history = PouchHistory::find($id);
        if (empty($pouch_history)) Utils::error('Wallet transaction not found!', 404);

        $this->validate($req, [
            'business_id' => 'exists:businesses,id',
            'debit_amount' => 'numeric',
            'credit_amount' => 'numeric',
            'description' => 'string',
            'transaction_category' => 'in:credit,debit',
            'remark' => 'string',
            'wallet_id' => 'string',
            'status_description' => 'string',
            'device_type' => 'in:Web,Android,iOS',
            'status' => 'in:Pending,Accepted,Rejected',
            'is_admin' => 'boolean',
        ]);

        // update pouch-history
        if ($req->has('business_id')) $pouch_history->business_id = $req->input('business_id');
        if ($req->has('debit_amount')) $pouch_history->debit_amount = $req->input('debit_amount');
        if ($req->has('credit_amount')) $pouch_history->credit_amount = $req->input('credit_amount');
        if ($req->has('description')) $pouch_history->description = $req->input('description');
        if ($req->has('transaction_category')) $pouch_history->transaction_category = $req->input('transaction_category');
        if ($req->has('remark')) $pouch_history->remark = $req->input('remark');
        if ($req->has('wallet_id')) $pouch_history->wallet_id = $req->input('wallet_id');
        if ($req->has('device_type')) $pouch_history->device_type = $req->input('device_type');
        if ($req->has('status')) $pouch_history->status = $req->input('status');
        if ($req->has('status_description')) $pouch_history->status_description = $req->input('status_description');
        $pouch_history->is_admin = $req->input('is_admin');
        $pouch_history->save();


        if ($req->input('status') == 'Accepted') {
            if ($req->has('business_id')) {
                $business = Business::find($req->input('business_id'));
                if (empty($business)) Utils::error('business not found!', 404);
            }
            if ($pouch_history->status_description == 'create') {
                if ($req->has('business_id')) {
                    $business = Business::find($req->input('business_id'));
                    if (empty($business)) Utils::error('business not found!', 404);
                    $business->pouch_weight = $business->pouch_weight + $pouch_history->debit_amount;
                    $business->save();

                    // User Wallet Credited
                    $business_pouch_history = new PouchHistory;
                    $business_pouch_history->business_id = $req->input('business_id');
                    $business_pouch_history->transaction_category = 'credit';
                    $business_pouch_history->description = 'Wallet Credited';
                    $business_pouch_history->credit_amount = $pouch_history->debit_amount;
                    $business_pouch_history->device_type = $pouch_history->device_type;
                    $business_pouch_history->remark = $pouch_history->remark;
                    $business_pouch_history->status = 'Accepted';
                    $business_pouch_history->updater_id = Auth::id();
                    $business_pouch_history->save();

                    // Total Balance Update
                    $business_pouch_history->total_balance = $business->pouch_weight;
                    $business_pouch_history->save();

                    // Admin Wallet Debited
                    $pouchWeight = (new SiteSettingController())->findByKey('pouch_weight');
                    $pouchWeight->value =  $pouchWeight->value - $pouch_history->debit_amount;
                    $pouchWeight->save();

                    // Total Balance Update
                    $pouch_history->total_balance = $pouchWeight->value;
                    $pouch_history->save();
                }
            } else if ($pouch_history->status_description == 'load-main-pouch') {
                // Admin Wallet Debited
                $pouchWeight = (new SiteSettingController())->findByKey('pouch_weight');
                $pouchWeight->value =  $this->convertToDecimal($pouchWeight->value + $pouch_history->credit_amount);
                $pouchWeight->save();

                // Total Balance Update
                $pouch_history->total_balance = $pouchWeight->value;
                $pouch_history->save();
            } else if ($pouch_history->status_description == 'unload-main-pouch') {
                $business_pouch_history = new PouchHistory;
                $business_pouch_history->fill($req->all());
                $business_pouch_history->transaction_category = 'debit';
                $business_pouch_history->description = 'Redemption';
                $business_pouch_history->debit_amount = $pouch_history->credit_amount;
                $business_pouch_history->device_type = $pouch_history->device_type;
                $business_pouch_history->remark = $pouch_history->remark;
                $business_pouch_history->credit_amount = 0;
                $business_pouch_history->status = 'Accepted';
                $business_pouch_history->save();

                // Business Wallet Update
                $business->pouch_weight = $business->pouch_weight - $business_pouch_history->debit_amount;
                $business->save();

                // Total Balance Update
                $business_pouch_history->total_balance = $business->pouch_weight;
                $business_pouch_history->save();

                // Admin Wallet Debited
                $pouchWeight = (new SiteSettingController())->findByKey('pouch_weight');
                $pouchWeight->value =  $this->convertToDecimal($pouchWeight->value + $pouch_history->credit_amount);
                $pouchWeight->save();

                // Total Balance Update
                $pouch_history->total_balance = $pouchWeight->value;
                $pouch_history->save();
            } else if ($pouch_history->status_description == 'bulk-payout') {
                $passbook = new Passbook;
                $passbook->entity = "payout_request";
                $passbook->entity_id = $pouch_history->id;
                $passbook->business_id = $pouch_history->business_id;
                $passbook->date = Carbon::today()->toDateString();
                $passbook->from_date = $pouch_history->created_at;
                $passbook->to_date = $pouch_history->created_at;
                $passbook->amount = -1 * $pouch_history->debit_amount;
                $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($pouch_history->created_at)->toDateString();
                $passbook->save();

                $business = Business::find($pouch_history->business_id);
                if (empty($business)) Utils::error('business not found!', 404);

                $business_pouch_history = new PouchHistory;
                $business_pouch_history->business_id = $pouch_history->business_id;
                $business_pouch_history->transaction_category = 'credit';
                $business_pouch_history->description = 'Wallet Credited';
                $business_pouch_history->updater_id = Auth::id();
                $business_pouch_history->credit_amount = $pouch_history->debit_amount;
                $business_pouch_history->debit_amount = 0;
                $business_pouch_history->status = 'Accepted';
                $business_pouch_history->save();

                // Business Wallet Update
                $business->pouch_weight = $business->pouch_weight + $business_pouch_history->credit_amount;
                $business->save();

                // Total Balance Update
                $business_pouch_history->total_balance = $business->pouch_weight;
                $business_pouch_history->save();

                // Admin Wallet Debited
                $pouchWeight = (new SiteSettingController())->findByKey('pouch_weight');
                $pouchWeight->value =  $this->convertToDecimal($pouchWeight->value - $business_pouch_history->credit_amount);
                $pouchWeight->save();

                // Total Balance Update
                $pouch_history->total_balance = $pouchWeight->value;
                $pouch_history->save();
            }
            // else if ($pouch_history->status_description == 'wallet-transfer') {
            //     $apiKey = SiteSettingController::findByKey('caxton_token')->value;
            //     $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

            //     try {
            //         $endPoint = 'payments/currency-bank/card-load';

            //         $client = new \GuzzleHttp\Client();
            //         $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
            //             'body' => '{
            //                 "WalletId":"' . $pouch_history->wallet_id . '",
            //                 "Currency":"GBP",
            //                 "Amount":"' . $pouch_history->debit_amount . '",
            //             }',
            //             'headers' => [
            //                 'Authorization' => 'bearer ' . $apiKey,
            //                 'UserApiToken' => $user_apiKey,
            //                 'content-type' => 'application/json',
            //             ],
            //         ]);
            //         if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            //             $data = json_decode($data->getBody()->getContents());
            //             $shop_order_id  = $data->Content->Model->ShopOrderId;
            //             $processEndPoint = 'payments/currency-bank/process';
            //             $walletDetails = $client->request('POST', env('CAXTON_BASEURL') . $processEndPoint, [
            //                 'body' => '{
            //                     "ShopOrderId":"' . $shop_order_id . '",
            //                 }',
            //                 'headers' => [
            //                     'Authorization' => 'bearer ' . $apiKey,
            //                     'UserApiToken' => $user_apiKey,
            //                     'content-type' => 'application/json',
            //                 ],
            //             ]);

            //             if ($walletDetails->getStatusCode() >= 200 && $walletDetails->getStatusCode() <= 210) {
            //                 // Business Wallet Update
            //                 $business->pouch_weight = $business->pouch_weight - $pouch_history->debit_amount;
            //                 $business->save();
            //             }
            //         }
            //     } catch (\Exception $e) {
            //         Log::debug($e->getMessage());
            //         if ($e instanceof \GuzzleHttp\Exception\RequestException) {
            //             $data = json_decode($e->getResponse()->getBody());
            //             Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            //         }
            //     }
            // } else if ($pouch_history->status_description == 'bank-transfer') {
            //     $user = CaxtonUser::where('business_id', $pouch_history->business_id)->first();
            //     if (empty($user)) Utils::error('Bank not found!', 404);

            //     $apiKey = SiteSettingController::findByKey('caxton_token')->value;
            //     $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

            //     try {
            //         $endPoint = 'account/balances/third-party/transfer';

            //         $client = new \GuzzleHttp\Client();
            //         $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
            //             'body' => '{
            //                 "UserId":"' . $user->account_id . '",
            //                 "SendCurrency": "GBP",
            //                 "SendAmount": ' . (float)$pouch_history->debit_amount . ',
            //             }',
            //             'headers' => [
            //                 'Authorization' => 'bearer ' . $apiKey,
            //                 'UserApiToken' => $user_apiKey,
            //                 'content-type' => 'application/json',
            //             ],
            //         ]);
            //         if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            //             $business->pouch_weight = $business->pouch_weight - ($pouch_history->debit_amount + $pouch_history->fee);
            //             $business->save();
            //         }
            //     } catch (\Exception $e) {
            //         Log::debug($e->getMessage());
            //         if ($e instanceof \GuzzleHttp\Exception\RequestException) {
            //             $data = json_decode($e->getResponse()->getBody());
            //             Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            //         }
            //     }
            // } else if ($pouch_history->status_description == 'wallet-load') {
            //     $apiKey = SiteSettingController::findByKey('caxton_token')->value;
            //     $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

            //     $endPointWallet = 'wallets/wallet/' . $pouch_history->wallet_id . '/balances/balance/specific/ccy/GBP';

            //     $client = new \GuzzleHttp\Client();
            //     $dataWallet = $client->request('GET', env('CAXTON_BASEURL') . $endPointWallet, [
            //         'headers' => [
            //             'Authorization' => 'bearer ' . $apiKey,
            //             'UserApiToken' => $user_apiKey,
            //             'content-type' => 'application/json',
            //         ],
            //     ]);
            //     if ($dataWallet->getStatusCode() >= 200 && $dataWallet->getStatusCode() <= 210) {
            //         $dataWallet = json_decode($dataWallet->getBody()->getContents());
            //         $balances = $dataWallet->Content->Model->Balances;
            //         $balance_amount = 0;
            //         foreach ($balances as $balance) {
            //             $balance_amount = $balance->Balance;
            //         }
            //         if ($req->input('credit_amount') > $balance_amount) {
            //             Utils::error('you have insufficient balance', 404);
            //         }

            //         $endPoint = 'wallets/balances/redeem/' . $pouch_history->wallet_id;

            //         $client = new \GuzzleHttp\Client();
            //         $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
            //             'body' => '{
            //                     "Currency":"GBP",
            //                     "Amount":"' . $pouch_history->credit_amount . '",
            //                 }',
            //             'headers' => [
            //                 'Authorization' => 'bearer ' . $apiKey,
            //                 'UserApiToken' => $user_apiKey,
            //                 'content-type' => 'application/json',
            //             ],
            //         ]);
            //         if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            //             // Business Wallet Update
            //             $business->pouch_weight = $business->pouch_weight + $pouch_history->credit_amount;
            //             $business->save();
            //             return $pouch_history;
            //         }
            //     }
            // } else if ($pouch_history->status_description == 'bank-load') {
            //     try {
            //         $caxtonUser = CaxtonUser::where('business_id', $business->id)->first();
            //         if (empty($caxtonUser)) Utils::error('user not found!', 404);
            //         $user_id = SiteSettingController::findByKey('caxton_user_id')->value;

            //         $client = new \GuzzleHttp\Client();
            //         $endPoint = 'account/balances/third-party/transfer';

            //         $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
            //             'body' => '{
            //                 "UserId":"' . $user_id . '",
            //                 "SendCurrency": "GBP",
            //                 "SendAmount": ' . (float)$pouch_history->credit_amount . ',
            //             }',
            //             'headers' => [
            //                 'Authorization' => 'bearer ' . $caxtonUser->token,
            //                 'UserApiToken' => $caxtonUser->human_password,
            //                 'content-type' => 'application/json',
            //             ],
            //         ]);

            //         if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210 && $pouch_history->fee > 0) {
            //             $endPointFee = 'account/balances/third-party/transfer';
            //             $dataFee = $client->request('POST', env('CAXTON_BASEURL') . $endPointFee, [
            //                 'body' => '{
            //                     "UserId":"' . $user_id . '",
            //                     "SendCurrency": "GBP",
            //                     "SendAmount": ' . (float)$pouch_history->fee . ',
            //                 }',
            //                 'headers' => [
            //                     'Authorization' => 'bearer ' . $caxtonUser->token,
            //                     'UserApiToken' => $caxtonUser->human_password,
            //                     'content-type' => 'application/json',
            //                 ],
            //             ]);
            //             if ($dataFee->getStatusCode() >= 200 && $dataFee->getStatusCode() <= 210) {
            //                 // Business Wallet Update
            //                 $business->pouch_weight = $business->pouch_weight + $pouch_history->credit_amount;
            //                 $business->save();
            //                 return $pouch_history;
            //             }
            //         }
            //     } catch (\Exception $e) {
            //         if ($e instanceof \GuzzleHttp\Exception\RequestException) {
            //             $data = json_decode($e->getResponse()->getBody());
            //             Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            //         }
            //     }
            // }
        }

        return $pouch_history;
    }


    /**
     * Delete pouch-story by id.
     * DELETE /api/pouch-histories/{id}
     */
    public function delete(Request $req, $id)
    {
        $pouch_history = PouchHistory::find($id);
        if (empty($pouch_history)) Utils::error('Wallet transaction not found!', 404);
        $pouch_history->delete();
        return $pouch_history;
    }

    /**
     * Fetch pouch-history payout.
     * POST /api/pouch-histories/payout
     */
    public function payout_fetch(Request $req)
    {
        $businesses = Business::where('disabled', '0')->where('pouch_service', '1')->orderByRaw('name ASC');

        if ($query = $req->query('query', false)) {
            $businesses->whereRaw('(name LIKE ?)', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $businesses = $businesses->get();
        } else {
            $businesses = $businesses->paginate($req->query('per_page', 20));
        }
        if ($businesses->isEmpty()) Utils::error('No businesses found!', 404);

        //Business Fields Hidden
        $businesses->makeHidden('created_at');
        $businesses->makeHidden('user_counts');
        $businesses->makeHidden('business_commissions');
        $businesses->makeHidden('country');
        $businesses->makeHidden('balance');
        $businesses->makeHidden('finance_user_id');
        $businesses->makeHidden('finance_user_phone_number');
        $businesses->makeHidden('image_url');
        $businesses->makeHidden('thumb_url');

        foreach ($businesses as $business) {
            $business->business_name = $business->name;
            $business->payout_description = 'Payout for ' . $business->name;
            $business->payout_statement_descriptor = 'Payout for ' . $business->name;
            $business->select_checkbox = true;
            $carbon = Carbon::today()->subDays(7);
            $currentDayOfWeek = $carbon->dayOfWeek;
            if ($currentDayOfWeek <= 1) {
                $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
            } else {
                $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
                $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
            }
            $payoutInvoice = PayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'Archived')->where('business_id', $business->business_id)->first();
            if (!empty($payoutInvoice)) {
                if ($payoutInvoice->net_total > 0) {
                    $business->amount = $payoutInvoice->net_total;
                } else {
                    $business->amount = 0;
                }
            } else {
                $business->amount = 0;
            }
        }
        return $businesses;
    }

    /**
     * Create bulk internal wallet payout.
     * POST /api/pouch-histories/bulk-payout
     */
    public function bulk_payout(Request $req)
    {
        if ($req->has('payouts') && !is_array($req->input('payouts')))
            $req->merge(['payouts' => json_decode($req->input('payouts'))]);

        $this->validate($req, [
            'payouts' => 'array',
        ]);

        if ($req->has('payouts') && !empty($req->input('payouts')) || sizeof($req->input('payouts')) > 0) {
            // create payoutRequest
            $payouts = ($req->input('payouts'));
            foreach ($payouts as $payout) {
                if ($payout->amount > 0) {
                    try {
                        $pouchWeightCheck = (new SiteSettingController())->findByKey('pouch_weight');
                        if ($req->input('amount') > $pouchWeightCheck->value) {
                            continue; // Utils::error('you have insufficient balance', 404);
                        }
                        $bankPayoutRequest = new CaxtonPayoutRequest;
                        $bankPayoutRequest->business_id = $payout->business_id;
                        $bankPayoutRequest->note = $payout->description;
                        $bankPayoutRequest->amount = $payout->amount;
                        $bankPayoutRequest->status = 'Accepted';
                        $bankPayoutRequest->save();

                        // create pouch-history
                        $pouch_history = new PouchHistory;
                        $pouch_history->business_id = $bankPayoutRequest->business_id;
                        $pouch_history->transaction_category = 'debit';
                        $pouch_history->description = 'Wallet Debited';
                        $pouch_history->updater_id = Auth::id();
                        $pouch_history->debit_amount = $bankPayoutRequest->amount;
                        $pouch_history->status_description = 'bulk-payout';
                        $pouch_history->credit_amount = 0;
                        $pouch_history->is_admin = 1;
                        $pouch_history->save();
                    } catch (\Exception $e) {
                        Log::debug($e);
                        continue;
                    }
                }
            }
        }

        return $payouts;
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}