<?php

namespace App\Http\Controllers;

use App\ResellerCommission;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class ResellerCommissionController extends Controller
{
    /**
     * List resellerCommissions.
     * GET /api/reseller-commissions
     */
    public function index(Request $req)
    {
        $resellerCommissions = ResellerCommission::orderByRaw('type DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $resellerCommissions->whereRaw('(type LIKE ? OR id_number LIKE ? )', ["%$query%", "%$query%"]);
        }

        if (Auth::isUser()) {
            $resellerCommissions->where('user_id', Auth::id());
        } else {

            if ($query = $req->query('user_id', false)) {
                $resellerCommissions->where('user_id', $query);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $resellerCommissions = $resellerCommissions->get();
        } else {
            $resellerCommissions = $resellerCommissions->paginate($req->query('per_page', 20));
        }

        if ($resellerCommissions->isEmpty()) Utils::error('No resellerCommissions found!', 404);


        return ($resellerCommissions);
    }

    /**
     * Create resellerCommission.
     * POST /api/reseller-commissions
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'user_id' => 'required|exists:users,id',
            'business_id' => 'required|exists:businesses,id',

            'cash_handling_fee_type' => 'required|in:flat,percentage',
            'cash_handling_fee' => 'required|numeric|min:0',
            'cash_handling_per_amount' => 'required|numeric|min:0',
            'management_fee_type' => 'required|in:flat,percentage',
            'management_fee' => 'required|numeric|min:0',

            'merchant_card_processing_fee_type' => 'required|in:flat,percentage',
            'merchant_card_processing_fee' => 'required|numeric|min:0',
            'merchant_card_auth_fee_type' => 'required|in:flat,percentage',
            'merchant_card_auth_fee' => 'required|numeric|min:0',

            'connect_card_processing_fee_type' => 'required|in:flat,percentage',
            'connect_card_processing_fee' => 'required|numeric|min:0',
            'connect_card_auth_fee_type' => 'required|in:flat,percentage',
            'connect_card_auth_fee' => 'required|numeric|min:0',

            'online_commission_fee_type' => 'required|in:flat,percentage',
            'online_commission_fee' => 'required|numeric|min:0',
            'dinein_commission_fee_type' => 'required|in:flat,percentage',
            'dinein_commission_fee' => 'required|numeric|min:0',

            'online_cod_fee_type' => 'required|in:flat,percentage',
            'online_cod_fee' => 'required|numeric|min:0',
            'online_paypal_fee_type' => 'required|in:flat,percentage',
            'online_paypal_fee' => 'required|numeric|min:0',
            'online_card_fee_type' => 'required|in:flat,percentage',
            'online_card_fee' => 'required|numeric|min:0',

            'online_service_fee_status' => 'required|in:show,hide',
            'online_service_fee_type' => 'required|in:flat,percentage',
            'online_service_fee' => 'required|numeric|min:0',
            'dinein_service_fee_status' => 'required|in:show,hide',
            'dinein_service_fee_type' => 'required|in:flat,percentage',
            'dinein_service_fee' => 'required',

            'order_charge_type' => 'required|in:order_based,transaction_based',

            'per_order_charge_type' => 'required|in:flat,percentage',
            'per_order_charge' => 'required',

            'cod_charge_type' => 'required|in:flat,percentage',
            'cod_charge' => 'required',
            'paypal_charge_type' => 'required|in:flat,percentage',
            'paypal_charge' => 'required',
            'card_charge_type' => 'required|in:flat,percentage',
            'card_charge' => 'required',
            'charge_back_fee_type' => 'required|in:flat,percentage',
            'charge_back_fee' => 'required',

            'charges_refundable' => 'in:no,yes',
        ]);

        // create resellerCommission
        $resellerCommission = new ResellerCommission;
        $resellerCommission->fill($req->all());

        $resellerCommission->save();

        return $resellerCommission;
    }

    /**
     * Show resellerCommission.
     * GET /api/reseller-commissions/{id}
     */
    public function show(Request $req, $id)
    {
        $resellerCommission = ResellerCommission::find($id);
        if (empty($resellerCommission)) Utils::error('ResellerCommission not found!', 404);
        return $resellerCommission;
    }

    /**
     * Update resellerCommission.
     * POST /api/reseller-commissions/{id}
     */
    public function update(Request $req, $id)
    {
        $resellerCommission = ResellerCommission::find($id);
        if (empty($resellerCommission)) Utils::error('ResellerCommission not found!', 404);

        $this->validate($req, [
            'user_id' => 'required|exists:users,id',
            'business_id' => 'required|exists:businesses,id',

            'cash_handling_fee_type' => 'required|in:flat,percentage',
            'cash_handling_fee' => 'required|numeric|min:0',
            'cash_handling_per_amount' => 'required|numeric|min:0',

            'management_fee_type' => 'required|in:flat,percentage',
            'management_fee' => 'required|numeric|min:0',

            'connect_card_processing_fee_type' => 'required|in:flat,percentage',
            'connect_card_processing_fee' => 'required|numeric|min:0',
            'connect_card_auth_fee_type' => 'required|in:flat,percentage',
            'connect_card_auth_fee' => 'required|numeric|min:0',

            'merchant_card_processing_fee_type' => 'required|in:flat,percentage',
            'merchant_card_processing_fee' => 'required|numeric|min:0',
            'merchant_card_auth_fee_type' => 'required|in:flat,percentage',
            'merchant_card_auth_fee' => 'required|numeric|min:0',

            'online_commission_fee_type' => 'required|in:flat,percentage',
            'online_commission_fee' => 'required|numeric|min:0',
            'dinein_commission_fee_type' => 'required|in:flat,percentage',
            'dinein_commission_fee' => 'required|numeric|min:0',

            'online_cod_fee_type' => 'required|in:flat,percentage',
            'online_cod_fee' => 'required|numeric|min:0',
            'online_paypal_fee_type' => 'required|in:flat,percentage',
            'online_paypal_fee' => 'required|numeric|min:0',
            'online_card_fee_type' => 'required|in:flat,percentage',
            'online_card_fee' => 'required|numeric|min:0',

            'online_service_fee_status' => 'required|in:show,hide',
            'online_service_fee_type' => 'required|in:flat,percentage',
            'online_service_fee' => 'required|numeric|min:0',
            'dinein_service_fee_status' => 'required|in:show,hide',
            'dinein_service_fee_type' => 'required|in:flat,percentage',
            'dinein_service_fee' => 'required',

            'order_charge_type' => 'required|in:order_based,transaction_based',

            'per_order_charge_type' => 'required|in:flat,percentage',
            'per_order_charge' => 'required',

            'cod_charge_type' => 'required|in:flat,percentage',
            'cod_charge' => 'required',
            'paypal_charge_type' => 'required|in:flat,percentage',
            'paypal_charge' => 'required',
            'card_charge_type' => 'required|in:flat,percentage',
            'card_charge' => 'required',
            'charge_back_fee_type' => 'required|in:flat,percentage',
            'charge_back_fee' => 'required',

            'charges_refundable' => 'in:no,yes',
        ]);

        // update resellerCommission
        if ($req->has('cash_handling_fee_type')) $resellerCommission->cash_handling_fee_type = $req->input('cash_handling_fee_type');
        $resellerCommission->cash_handling_fee = $req->input('cash_handling_fee');
        $resellerCommission->cash_handling_per_amount = $req->input('cash_handling_per_amount');
        if ($req->has('management_fee_type')) $resellerCommission->management_fee_type = $req->input('management_fee_type');
        $resellerCommission->management_fee = $req->input('management_fee');

        if ($req->has('merchant_card_processing_fee_type')) $resellerCommission->merchant_card_processing_fee_type = $req->input('merchant_card_processing_fee_type');
        $resellerCommission->merchant_card_processing_fee = $req->input('merchant_card_processing_fee');
        if ($req->has('merchant_card_auth_fee_type')) $resellerCommission->merchant_card_auth_fee_type = $req->input('merchant_card_auth_fee_type');
        $resellerCommission->merchant_card_auth_fee = $req->input('merchant_card_auth_fee');

        if ($req->has('connect_card_processing_fee_type')) $resellerCommission->connect_card_processing_fee_type = $req->input('connect_card_processing_fee_type');
        $resellerCommission->connect_card_processing_fee = $req->input('connect_card_processing_fee');
        if ($req->has('connect_card_auth_fee_type')) $resellerCommission->connect_card_auth_fee_type = $req->input('connect_card_auth_fee_type');
        $resellerCommission->connect_card_auth_fee = $req->input('connect_card_auth_fee');

        if ($req->has('online_commission_fee_type')) $resellerCommission->online_commission_fee_type = $req->input('online_commission_fee_type');
        $resellerCommission->online_commission_fee = $req->input('online_commission_fee');
        if ($req->has('dinein_commission_fee_type')) $resellerCommission->dinein_commission_fee_type = $req->input('dinein_commission_fee_type');
        $resellerCommission->dinein_commission_fee = $req->input('dinein_commission_fee');

        if ($req->has('online_cod_fee_type')) $resellerCommission->online_cod_fee_type = $req->input('online_cod_fee_type');
        $resellerCommission->online_cod_fee = $req->input('online_cod_fee');
        if ($req->has('online_paypal_fee_type')) $resellerCommission->online_paypal_fee_type = $req->input('online_paypal_fee_type');
        $resellerCommission->online_paypal_fee = $req->input('online_paypal_fee');
        if ($req->has('online_card_fee_type')) $resellerCommission->online_card_fee_type = $req->input('online_card_fee_type');
        $resellerCommission->online_card_fee = $req->input('online_card_fee');

        if ($req->has('online_service_fee_status')) $resellerCommission->online_service_fee_status = $req->input('online_service_fee_status');
        if ($req->has('online_service_fee_type')) $resellerCommission->online_service_fee_type = $req->input('online_service_fee_type');
        $resellerCommission->online_service_fee = $req->input('online_service_fee');
        if ($req->has('dinein_service_fee_status')) $resellerCommission->dinein_service_fee_status = $req->input('dinein_service_fee_status');
        if ($req->has('dinein_service_fee_type')) $resellerCommission->dinein_service_fee_type = $req->input('dinein_service_fee_type');
        $resellerCommission->dinein_service_fee = $req->input('dinein_service_fee');

        if ($req->has('order_charge_type')) $resellerCommission->order_charge_type = $req->input('order_charge_type');

        if ($req->has('per_order_charge_type')) $resellerCommission->per_order_charge_type = $req->input('per_order_charge_type');
        $resellerCommission->per_order_charge = $req->input('per_order_charge');

        if ($req->has('cod_charge_type')) $resellerCommission->cod_charge_type = $req->input('cod_charge_type');
        $resellerCommission->cod_charge = $req->input('cod_charge');
        if ($req->has('paypal_charge_type')) $resellerCommission->paypal_charge_type = $req->input('paypal_charge_type');
        $resellerCommission->paypal_charge = $req->input('paypal_charge');
        if ($req->has('card_charge_type')) $resellerCommission->card_charge_type = $req->input('card_charge_type');
        $resellerCommission->card_charge = $req->input('card_charge');

        if ($req->has('charge_back_fee_type')) $resellerCommission->charge_back_fee_type = $req->input('charge_back_fee_type');
        $resellerCommission->charge_back_fee = $req->input('charge_back_fee');

        if ($req->has('charges_refundable')) $resellerCommission->charges_refundable = $req->input('charges_refundable');

        $resellerCommission->save();

        return $resellerCommission;
    }


    /**
     * Delete resellerCommission by id.
     * DELETE /api/reseller-commissions/{id}
     */
    public function delete(Request $req, $id)
    {
        $resellerCommission = ResellerCommission::find($id);
        if (empty($resellerCommission)) Utils::error('ResellerCommission not found!', 404);
        $resellerCommission->delete();
        return $resellerCommission;
    }
}
