<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\ResellerMiscCharge;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class ResellerMiscChargeController extends Controller
{
    /**
     * List miscCharges.
     * GET /api/misc-charges
     */
    public function index(Request $req)
    {
        $miscCharges = ResellerMiscCharge::orderByRaw('date DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $miscCharges->whereRaw('(description LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('user_id', false)) {
            $miscCharges->where('user_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $miscCharges = $miscCharges->get();
        } else {
            $miscCharges = $miscCharges->paginate($req->query('per_page', 20));
        }

        if ($miscCharges->isEmpty()) Utils::error('No miscCharges found!', 404);


        return ($miscCharges);
    }

    /**
     * Create miscCharge.
     * POST /api/misc-charges
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'date' => 'required|date_format:Y-m-d',
            'amount' => 'required|numeric',
            'user_id' => 'required|exists:users,id',
        ]);

        // create miscCharge
        $miscCharge = new ResellerMiscCharge;
        $miscCharge->fill($req->all());

        $miscCharge->save();

        return $miscCharge;
    }

    /**
     * Show miscCharge.
     * GET /api/misc-charges/{id}
     */
    public function show(Request $req, $id)
    {
        $miscCharge = ResellerMiscCharge::find($id);
        if (empty($miscCharge)) Utils::error('ResellerMiscCharge not found!', 404);
        return $miscCharge;
    }

    /**
     * Update miscCharge.
     * POST /api/misc-charges/{id}
     */
    public function update(Request $req, $id)
    {
        $miscCharge = ResellerMiscCharge::find($id);
        if (empty($miscCharge)) Utils::error('ResellerMiscCharge not found!', 404);

        $this->validate($req, [
            'date' => 'required|date_format:Y-m-d',
            'amount' => 'required|numeric',
            'user_id' => 'required|exists:users,id',
        ]);

        // update miscCharge
        if ($req->has('date')) $miscCharge->date = $req->input('date');
        if ($req->has('description')) $miscCharge->description = $req->input('description');
        if ($req->has('user_id')) $miscCharge->user_id = $req->input('user_id');
        $miscCharge->amount = $req->input('amount');

        $miscCharge->save();

        return $miscCharge;
    }


    /**
     * Delete miscCharge by id.
     * DELETE /api/misc-charges/{id}
     */
    public function delete(Request $req, $id)
    {
        $miscCharge = ResellerMiscCharge::find($id);
        if (empty($miscCharge)) Utils::error('Reseller MiscCharge not found!', 404);
        $miscCharge->delete();
        return $miscCharge;
    }
}
