<?php

namespace App\Http\Controllers;

use App\Exports\MiscInvoiceExport;
use App\ResellerMiscInvoice;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ResellerMiscInvoiceItem;
use App\Passbook;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Mpdf\Mpdf;

class ResellerMiscInvoiceController extends Controller
{
    /**
     * List miscInvoices.
     * GET /api/misc-invoices
     */
    public function index(Request $req)
    {
        $miscInvoices = ResellerMiscInvoice::selectRaw("reseller_misc_invoices.*,users.first_name as user_name")->leftJoin('users', 'user_id', '=', 'users.id')->orderByRaw('reseller_misc_invoices.date DESC, reseller_misc_invoices.created_at DESC');

        if ($query = $req->query('query', false)) {
            $miscInvoices->whereRaw('(reseller_misc_invoices.comment LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('user_id', false)) {
            $miscInvoices->where('reseller_misc_invoices.user_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $miscInvoices = $miscInvoices->get();
        } else {
            $miscInvoices = $miscInvoices->paginate($req->query('per_page', 20));
        }

        if ($miscInvoices->isEmpty()) Utils::error('No miscInvoices found!', 404);


        return ($miscInvoices);
    }

    /**
     * Create miscInvoice.
     * POST /api/misc-invoices
     */
    public function create(Request $req)
    {
        if ($req->has('items') && !is_array($req->input('items'))) {
            $req->merge(['items' => json_decode($req->input('items'))]);
        }
        $this->validate($req, [
            'percentage' => 'required_if:payment_mode,Percentage|numeric',
            'grand_total' => 'required|numeric',
            'payment_mode' => 'required|in:Auto,Manual,Percentage',
            'status' => 'in:Paid,Partially Paid, Paid',
            'date' => 'required|date',
            'comment' => 'string',
            'user_id' => 'required|exists:users,id',
            'items' => 'required|array'
        ]);

        // create miscInvoice
        $miscInvoice = new ResellerMiscInvoice;
        $miscInvoice->fill($req->all());

        $miscInvoice->save();

        $passbook = new Passbook;
        $passbook->user_id = $miscInvoice->user_id;
        $passbook->entity_id = $miscInvoice->id;
        $passbook->entity = "misc_invoice";
        $passbook->amount =  -1 * $miscInvoice->grand_total;
        $passbook->date = Carbon::today();
        $passbook->from_date = Carbon::today();
        $passbook->to_date = Carbon::today();
        $passbook->description = "Debited for Miscellaneous invoice #$miscInvoice->id dated on " . Carbon::today()->toDateString();
        $passbook->save();

        $items = ($req->input('items'));
        foreach ($items as $item) {
            $miscInvoiceItem = new ResellerMiscInvoiceItem((array)$item);
            $miscInvoiceItem->misc_invoice_id = $miscInvoice->id;
            $miscInvoiceItem->save();
        }
        $miscInvoice->append('misc_invoice_items');
        return $miscInvoice;
    }

    /**
     * Show miscInvoice.
     * GET /api/misc-invoices/{id}
     */
    public function show(Request $req, $id)
    {
        $miscInvoice = ResellerMiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('ResellerMiscInvoice not found!', 404);
        $miscInvoice->append('misc_invoice_items');
        return $miscInvoice;
    }

    /**
     * Update miscInvoice.
     * POST /api/misc-invoices/{id}
     */
    public function update(Request $req, $id)
    {
        $miscInvoice = ResellerMiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('ResellerMiscInvoice not found!', 404);
        if ($req->has('items') && !is_array($req->input('items'))) {
            $req->merge(['items' => json_decode($req->input('items'))]);
        }

        $alreadyPaidAmount = $miscInvoice->total_paid_amount;
        $this->validate($req, [
            'grand_total' => 'numeric',
            'percentage' => 'required_if:payment_mode,Percentage|numeric',
            'payment_mode' => 'in:Auto,Manual,Percentage',
            'status' => 'in:Paid,Partially Paid, Paid',
            'date' => 'date',
            'comment' => 'string',
            'user_id' => 'exists:users,id',
            'items' => 'required|array'
        ]);

        // update miscInvoice
        if ($req->has('grand_total')) $miscInvoice->grand_total = $req->input('grand_total');
        if ($req->has('payment_mode')) $miscInvoice->payment_mode = $req->input('payment_mode');
        if ($req->has('percentage')) $miscInvoice->percentage = $req->input('percentage');
        if ($req->has('status')) $miscInvoice->status = $req->input('status');
        if ($req->has('date')) $miscInvoice->date = $req->input('date');
        if ($req->has('comment')) $miscInvoice->comment = $req->input('comment');
        if ($req->has('user_id')) $miscInvoice->user_id = $req->input('user_id');
        if ($miscInvoice->grand_total < $alreadyPaidAmount) {
            Utils::error('Invoice total should not be smaller than already paid amount', 400);
        }
        $miscInvoice->save();
        $passbook = Passbook::where('entity_id', $miscInvoice->id)->where('entity', 'misc_invoice')->first();
        if (empty($passbook)) {
            $passbook = new Passbook;
            $passbook->user_id = $miscInvoice->user_id;
            $passbook->entity_id = $miscInvoice->id;
            $passbook->entity = "misc_invoice";
            $passbook->date = Carbon::today();
            $passbook->from_date = Carbon::today();
            $passbook->to_date = Carbon::today();
            $passbook->description = "Debited for Miscellaneous invoice #$miscInvoice->id dated on " .  Carbon::today()->toDateString();
        }
        $passbook->amount = -1 * $miscInvoice->grand_total;
        $passbook->save();

        // $miscInvoice->append('misc_invoice_items');
        $oldItems = ResellerMiscInvoiceItem::where('misc_invoice_id', $miscInvoice->id)->delete();
        $items = ($req->input('items'));
        foreach ($items as $item) {
            $miscInvoiceItem = new ResellerMiscInvoiceItem((array)$item);
            $miscInvoiceItem->misc_invoice_id = $miscInvoice->id;
            $miscInvoiceItem->save();
        }
        $miscInvoice->append('misc_invoice_items');
        return $miscInvoice;
    }

    public function pay(Request $req, $id)
    {
        $miscInvoice = ResellerMiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('ResellerMiscInvoice not found!', 404);
        $this->validate($req, [
            'paid_amount' => 'required|numeric|gt:0',
        ]);
        if ($miscInvoice->total_paid_amount + $req->input('paid_amount') > $miscInvoice->grand_total) {
            Utils::error("Total paid amount is exceeding to invoice total", 400);
        }
        $passbook = new Passbook;
        $passbook->user_id = $miscInvoice->user_id;
        $passbook->entity_id = $id;
        $passbook->entity = "misc_invoice";
        $passbook->amount =  $req->input('paid_amount');
        $passbook->date = Carbon::today();
        $passbook->from_date = Carbon::today();
        $passbook->to_date = Carbon::today();
        $passbook->description = "Amount paid for misc invoice dated on $miscInvoice->date";
        $passbook->save();
    }
    public function print(Request $req, $id)
    {
        $miscInvoice = ResellerMiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('ResellerMiscInvoice not found!', 404);
        $miscInvoice->append('misc_invoice_items');
        $miscInvoice->append('user');
        $fileName = $miscInvoice['user']['first_name'];

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            /*border-left: 1px solid #000000;
                border-top: 1px solid #000000;*/
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #73C6B6;
            font-weight:600;
            background-color:#138D75;
            color:#fff;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #73C6B6;
        }
        .items-table tr:nth-child(even){
            background-color:#E8F6F3   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h2 style = "text-align:right">Invoice</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td style="font-weight: 900;">Simple Merchant Payment Services</td>';
        $html = $html . '<td>' . Carbon::parse($miscInvoice->date)->format('F d, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td></td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td><h4>' . $miscInvoice->user->first_name . '</h4></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall</td>';
        $html = $html . '<td>' .   $miscInvoice->user->residential_address  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>WS1 3RA</td>';
        $html = $html . '<td>' . $miscInvoice->user->country->name  . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td>Email: ' . $miscInvoice->user->login_email . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Sr No.</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Rate</th>';
        $html = $html . '<th>Quantity</th>';
        $html = $html . '<th>Total</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        foreach ($miscInvoice->misc_invoice_items as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . $srNo . '</td>';
            $html = $html . '<td>' . $item->description . '</td>';
            $html = $html . '<td>' . $item->rate . '</td>';
            $html = $html . '<td>' . $item->quantity . '</td>';
            $html = $html . '<td>' . $item->total . '</td>';
            $html = $html . '</tr>';
            $srNo++;
        }
        $html = $html . '<tr>';
        $html = $html . '<th colspan="4" style="text-align:right;">Grand Total:</th>';
        $html = $html . '<th>' . $miscInvoice->grand_total . '</th>';
        $html = $html . '</tr>';
        if ($miscInvoice->status != "Paid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="4" style="text-align:right;">Paid Amount:</td>';
            $html = $html . '<td>' . $miscInvoice->total_paid_amount . '</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<th colspan="5">Invoice Paid</th>';
            $html = $html . '</tr>';
        }


        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';

        $mpdf = new Mpdf();
        $mpdf->WriteHTML($html);
        return $mpdf->Output("$fileName.pdf", "D");
        // return Excel::download(new ResellerMiscInvoiceExport($miscInvoice), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
    }

    /**
     * Delete miscInvoice by id.
     * DELETE /api/misc-invoices/{id}
     */
    public function delete(Request $req, $id)
    {
        $miscInvoice = ResellerMiscInvoice::find($id);
        if (empty($miscInvoice)) Utils::error('ResellerMiscInvoice not found!', 404);
        $miscInvoice->delete();
        return $miscInvoice;
    }
}
