<?php

namespace App\Http\Controllers;

use App\Business;
use App\Exports\AdminPayoutReportExport;
use App\Exports\AdminResellerPayoutReportExport;
use App\Exports\MultiplePayoutReportExport;
use App\Exports\PayoutReportExport;
use App\Exports\ResellerMultiplePayoutReportExport;
use App\Exports\ResellerPayoutReportExport;
use App\ResellerPayoutInvoice;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Passbook;
use App\ResellerPayoutInvoiceGroup;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;

class ResellerPayoutInvoiceController extends Controller
{
    /**
     * List payoutInvoices.
     * GET /api/payout-invoices
     */
    public function index(Request $req)
    {
        $payoutInvoices = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->orderByRaw('to_date DESC, reseller_payout_invoices.created_at DESC');

        if ($query = $req->query('query', false)) {
            $payoutInvoices->whereRaw('(comment LIKE ? OR name LIKE ? OR gross_total LIKE ? OR gross_fee LIKE ? OR transaction_type LIKE ? OR reseller_payout_invoices.payout_mode LIKE ? OR reseller_payout_invoices.payout_method LIKE ? OR net_total LIKE ? OR net_fee LIKE ?  )', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $payoutInvoices->where('business_id', $query);
        }
        if ($query = $req->query('user_id', false)) {
            $payoutInvoices->where('user_id', $query);
        }
        if ($query = $req->query('transaction_type', false)) {
            $payoutInvoices->where('transaction_type', $query);
        }
        if ($query = $req->query('status', false)) {
            $payoutInvoices->where('status', $query);
        }
        if ($query = $req->query('payout_mode', false)) {
            $payoutInvoices->where('payout_mode', $query);
        }
        if ($query = $req->query('payout_method', false)) {
            $payoutInvoices->where('payout_method', $query);
        }
        if ($query = $req->query('from_date', false)) {
            $payoutInvoices->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $payoutInvoices->where('to_date', '<=', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $payoutInvoices = $payoutInvoices->get();
        } else {
            $payoutInvoices = $payoutInvoices->paginate($req->query('per_page', 20));
        }

        if ($payoutInvoices->isEmpty()) Utils::error('No payoutInvoices found!', 404);


        return ($payoutInvoices);
    }

    /*
    |---------------------------------------
    | Modern Report
    | GET /api/payout-invoices/modern
    |---------------------------------------
    */
    public function modern(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
        ]);
        $fromDate = $req->query('from_date');
        $toDate = $req->query('to_date');
        $normalAutoBankPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('reseller_payout_invoices.payout_mode', 'Auto')->where('reseller_payout_invoices.payout_method', 'Bank')->get();
        $normalAutoPaypalPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('reseller_payout_invoices.payout_mode', 'Auto')->where('reseller_payout_invoices.payout_method', 'Paypal')->get();

        $normalManualBankPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('reseller_payout_invoices.payout_mode', 'Manual')->where('reseller_payout_invoices.payout_method', 'Bank')->get();
        $normalManualPaypalPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('reseller_payout_invoices.payout_mode', 'Manual')->where('reseller_payout_invoices.payout_method', 'Paypal')->get();
        $normalManualCashPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->where('reseller_payout_invoices.payout_mode', 'Manual')->where('reseller_payout_invoices.payout_method', 'Cash')->get();

        $manualPayouts =  ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Normal')->whereRaw('(reseller_payout_invoices.payout_mode = ? OR (reseller_payout_invoices.payout_mode = ? AND reseller_payout_invoices.payout_method = ? ))', ["Manual", "Auto", "Cash"])->get();

        $archivedAutoBankPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Archived')->where('reseller_payout_invoices.payout_mode', 'Auto')->where('reseller_payout_invoices.payout_method', 'Bank')->get();
        $archivedAutoPaypalPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Archived')->where('reseller_payout_invoices.payout_mode', 'Auto')->where('reseller_payout_invoices.payout_method', 'Paypal')->get();
        $archivedAutoCashPayouts = ResellerPayoutInvoice::selectRaw("reseller_payout_invoices.*, users.first_name, users.last_name, SUM(net_fee) as net_fee")->select("reseller_payout_invoices.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->groupBy(['from_date', 'to_date'])->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->where('transaction_type', 'Archived')->where('reseller_payout_invoices.payout_mode', 'Auto')->where('reseller_payout_invoices.payout_method', 'Cash')->get();

        $normalPayoutSummary = ResellerPayoutInvoice::selectRaw("
        (
            SELECT
                SUM(net_fee)
            FROM
                reseller_payout_invoices
            WHERE
                payout_mode = 'Auto' AND payout_method = 'Bank'
                AND from_date >= '$fromDate' AND to_date<='$toDate'
                limit 1
        ) AS 'bank_payout',
        (
            SELECT
                SUM(net_fee)
            FROM
                reseller_payout_invoices
            WHERE
                payout_mode = 'Auto' AND payout_method = 'Paypal'
                AND from_date >= '$fromDate' AND to_date<='$toDate'
                limit 1
        ) AS 'paypal_payout',
        (
            SELECT
                SUM(net_fee)
            FROM
                reseller_payout_invoices
            WHERE
                payout_mode = 'Auto' AND payout_method = 'Cash'
                AND from_date >= '$fromDate' AND to_date<='$toDate'
                limit 1
        ) AS 'cash_payout',
        (
            SELECT
                SUM(net_fee)
            FROM
                reseller_payout_invoices
            WHERE
                payout_mode = 'Manual'
                AND from_date >= '$fromDate' AND to_date<='$toDate'
                limit 1
        ) AS 'manual_payout',
        (
            SELECT SUM(net_fee)
            FROM
                reseller_payout_invoices 
            WHERE from_date >= '$fromDate' AND to_date<='$toDate'
            limit 1
        )as total
        
        ")->where('from_date', '>=', $req->query('from_date'))->where('to_date', '<=', $req->query('to_date'))->first();

        if (empty($normalPayoutSummary)) Utils::error("No data found for selected range", 400);
        $data['normal_auto_bank_payouts'] = $normalAutoBankPayouts;
        $data['normal_auto_paypal_payouts'] = $normalAutoPaypalPayouts;
        $data['normal_auto_cash_payouts'] = null;

        $data['archived_auto_bank_payouts'] = $archivedAutoBankPayouts;
        $data['archived_auto_paypal_payouts'] = $archivedAutoPaypalPayouts;
        $data['archived_auto_cash_payouts'] = $archivedAutoCashPayouts;

        $data['normal_manual_bank_payouts'] = $normalManualBankPayouts;
        $data['normal_manual_paypal_payouts'] = $normalManualPaypalPayouts;
        $data['normal_manual_cash_payouts'] = $normalManualCashPayouts;
        $data['manual_payouts'] = $manualPayouts;

        $data['normal_payout_summary'] = $normalPayoutSummary;
        $data['from_date'] = $fromDate;
        $data['to_date'] = $toDate;
        return $data;
    }

    public function export(Request $req)
    {
        $exportableData = ($this->modern($req));
        $fromDate = Carbon::parse($exportableData['from_date']);
        $toDate = Carbon::parse($exportableData['to_date']);
        $fileName = $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
        $type = $req->query('format', 'excel');
        if ($type == 'excel')
            return Excel::download(new AdminResellerPayoutReportExport($exportableData), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new AdminResellerPayoutReportExport($exportableData), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function export_all(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
        ]);
        $fromDate = $req->query('from_date');
        $toDate = $req->query('to_date');

        $payoutInvoices = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "businesses.name", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->orderByRaw('to_date DESC, reseller_payout_invoices.created_at DESC');
        if ($query = $req->query('from_date', false)) {
            $payoutInvoices->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $payoutInvoices->where('to_date', '<=', $query);
        }
        $payoutInvoices = $payoutInvoices->get();
        if ($payoutInvoices->isEmpty()) Utils::error("No invoices found", 400);
        $businesses = [];

        $payoutInvoices->each(function ($invoice) {
            $invoice->append('transactions');
            $invoice->append('business');
            $businesses[] = $invoice;
        });
        $fromDate = Carbon::parse($fromDate);
        $toDate = Carbon::parse($toDate);

        $fileName = $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');

        $type = $req->query('format', 'excel');

        if ($type == 'excel')
            return Excel::download(new MultiplePayoutReportExport($payoutInvoices), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new MultiplePayoutReportExport($payoutInvoices), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }
    /**
     * Create payoutInvoice.
     * POST /api/payout-invoices
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:business,id',
            'gross_total' => 'required|numeric',
            'misc_charge' => 'required|numeric',
            'total_fee' => 'required|numeric',
            'net_total' => 'required|numeric',
            'paid_amount' => 'numeric',
            'transaction_type' => 'required|in:Normal,Archived',
            'payout_mode' => 'required|in:Manual,Auto',
            'payout_method' => 'required|in:Cash,Bank,Paypal',
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required|date_format:Y-m-d|after:from_date',
            'status' => 'required|in:Pending,Settled,Partially Settled',
            'comment' => 'string',
        ]);

        // create payoutInvoice
        $payoutInvoice = new ResellerPayoutInvoice;
        $payoutInvoice->fill($req->all());

        $payoutInvoice->save();

        return $payoutInvoice;
    }

    /**
     * Show payoutInvoice.
     * GET /api/payout-invoices/{id}
     */
    public function show(Request $req, $id)
    {
        $payoutInvoice = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->find($id);
        if (empty($payoutInvoice)) Utils::error('ResellerPayoutInvoice not found!', 404);
        $payoutInvoice->append('transactions');
        $payoutInvoice->append('business');
        $payoutInvoice->append('reseller');
        return $payoutInvoice;
    }

    /**
     * Download payoutInvoice.
     * GET /api/payout-invoices/{id}/download
     */
    public function download(Request $req, $id)
    {
        $payoutInvoice = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->find($id);
        if (empty($payoutInvoice)) Utils::error('ResellerPayoutInvoice not found!', 404);

        $fromDate = Carbon::parse($payoutInvoice['from_date']);
        $toDate = Carbon::parse($payoutInvoice['to_date']);
        $type = 'excel';
        if ($format = $req->query('format', false)) {
            $type = $format;
        }

        if ($style = $req->query('style', false)) {
            $resellerPayoutInvoices = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->where('user_id', $payoutInvoice->user_id)->orderByRaw('to_date DESC, reseller_payout_invoices.created_at DESC');
            $resellerPayoutInvoices->where('from_date', '=', $fromDate);
            $resellerPayoutInvoices->where('to_date', '=', $toDate);
            $resellerPayoutInvoices = $resellerPayoutInvoices->get();

            foreach ($resellerPayoutInvoices as $payoutInvoice) {
                $payoutInvoice->append('transactions');
                $payoutInvoice->append('business');
                $payoutInvoice->append('reseller');
            }
            $fileName = $payoutInvoice['reseller']['first_name'] . " " . $payoutInvoice['reseller']['last_name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
            if ($type == 'excel')
                return Excel::download(new ResellerMultiplePayoutReportExport($resellerPayoutInvoices), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
            else if ($type == 'pdf') {
                return Excel::download(new ResellerMultiplePayoutReportExport($resellerPayoutInvoices), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
            } else {
                Utils::error("Invalid export format", 400);
            }
        } else {

            $payoutInvoice->append('transactions');
            $payoutInvoice->append('business');
            $payoutInvoice->append('reseller');

            $fromDate = Carbon::parse($payoutInvoice['from_date']);
            $toDate = Carbon::parse($payoutInvoice['to_date']);
            $fileName = $payoutInvoice['reseller']['first_name'] . " " . $payoutInvoice['reseller']['last_name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
            if ($type == 'excel')
                return Excel::download(new ResellerPayoutReportExport($payoutInvoice), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
            else if ($type == 'pdf') {
                return Excel::download(new ResellerPayoutReportExport($payoutInvoice), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
            } else {
                Utils::error("Invalid export format", 400);
            }
        }
    }
    /**
     * Update payoutInvoice.
     * POST /api/payout-invoices/{id}
     */
    public function update(Request $req, $id)
    {
        $payoutInvoice = ResellerPayoutInvoice::find($id);
        if (empty($payoutInvoice)) Utils::error('ResellerPayoutInvoice not found!', 404);

        $this->validate($req, [
            'business_id' => 'exists:business,id',
            'gross_total' => 'numeric',
            'misc_charge' => 'numeric',
            'total_fee' => 'numeric',
            'net_total' => 'numeric',
            'paid_amount' => 'numeric',
            'transaction_type' => 'in:Normal,Archived',
            'payout_mode' => 'in:Manual,Auto',
            'payout_method' => 'in:Cash,Bank,Paypal',
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'date_format:Y-m-d|after:from_date',
            'status' => 'in:Pending,Settled,Partially Settled',
            'comment' => 'string',
        ]);

        // update payoutInvoice
        if ($req->has('gross_total')) $payoutInvoice->gross_total = $req->input('gross_total');
        if ($req->has('misc_charge')) $payoutInvoice->misc_charge = $req->input('misc_charge');
        if ($req->has('total_fee')) $payoutInvoice->total_fee = $req->input('total_fee');
        if ($req->has('net_total')) $payoutInvoice->net_total = $req->input('net_total');
        if ($req->has('transaction_type')) $payoutInvoice->transaction_type = $req->input('transaction_type');
        if ($req->has('payment_mode')) $payoutInvoice->payment_mode = $req->input('payment_mode');
        if ($req->has('payment_method')) $payoutInvoice->payment_method = $req->input('payment_method');
        if ($req->has('from_date')) $payoutInvoice->from_date = $req->input('from_date');
        if ($req->has('to_date')) $payoutInvoice->to_date = $req->input('to_date');
        if ($req->has('status')) $payoutInvoice->status = $req->input('status');
        if ($req->has('comment')) $payoutInvoice->comment = $req->input('comment');

        $payoutInvoice->save();

        return $payoutInvoice;
    }


    /**
     * Delete payoutInvoice by id.
     * DELETE /api/payout-invoices/{id}
     */
    public function delete(Request $req, $id)
    {
        $payoutInvoice = ResellerPayoutInvoice::find($id);
        if (empty($payoutInvoice)) Utils::error('ResellerPayoutInvoice not found!', 404);
        $payoutInvoice->delete();

        #Check if passbook entry is there or not and then delete
        $passbook = Passbook::where('entity_id', $payoutInvoice->id)->where('entity', 'reseller_payout_invoice')->where('user_id', $payoutInvoice->user_id)->first();
        if (!empty($passbook)) $passbook->delete();

        return $payoutInvoice;
    }
    public function pay_bulk_invoices(Request $req)
    {
        if ($req->has('invoices') && !is_array($req->input('invoices'))) {
            $req->merge(['invoices' => json_decode($req->input('invoices'))]);
        }
        $this->validate($req, [
            'invoices' => 'required|array'
        ], ['invoices.required' => 'No invoices updated']);
        $invoices = ($req->input('invoices'));
        $invoiceIds = [];
        $finalInvoices = [];
        foreach ($invoices as $invoice) {
            $dbInvoice = ResellerPayoutInvoice::find($invoice->id);
            if (empty($dbInvoice)) continue;
            $dbUser = User::find($dbInvoice->user_id);
            $payableAmount = ($invoice->net_fee > 0 ? -1 * ($invoice->net_fee - (-1 * ($invoice->total_paid_amount))) : (-1 * ($invoice->net_fee)) - $invoice->total_paid_amount);
            if ($payableAmount > 0 && $invoice->paid_amount > $payableAmount) {
                Utils::error("Paid amount should not be greater than payable amount for " . $dbUser->first_name, 400);
            }
            if ($payableAmount < 0 &&  $invoice->paid_amount < $payableAmount) {
                Utils::error("Paid amount should not be lesser than payable amount for " . $dbUser->first_name, 400);
            }
            $invoice->user_id = $dbInvoice->user_id;
            $invoice->from_date = $dbInvoice->from_date;
            $invoice->to_date = $dbInvoice->to_date;
            $finalInvoices[] = $invoice;
        }
        foreach ($finalInvoices as $invoice) {
            $invoiceIds[] = $invoice->id;
            $passbook = new Passbook();
            $passbook->user_id = $invoice->user_id;
            $passbook->amount = $invoice->paid_amount;
            $passbook->entity = "reseller_payout_invoice";
            $passbook->entity_id = $invoice->id;
            $passbook->date = Carbon::today();
            $passbook->from_date = $invoice->from_date;
            $passbook->to_date = $invoice->to_date;
            if ($invoice->paid_amount > 0) {
                $passbook->description = "Invoice settled for $invoice->from_date to $invoice->to_date";
            } else {
                $passbook->description = "Invoice paid for $invoice->from_date to $invoice->to_date";
            }

            $passbook->save();
        }
        if (!empty($invoices)) {
            $invoices = ResellerPayoutInvoice::whereRaw('(id IN (?))', $invoiceIds)->get();
        }
        return $invoices;
    }
    /**
     * Pay grouped reseller payout invoices
     * POST api/payout-invoices/groups
     */
    public function group_pay_bulk_invoices(Request $req)
    {
        if ($req->has('invoices') && !is_array($req->input('invoices'))) {
            $req->merge(['invoices' => json_decode($req->input('invoices'))]);
        }
        $this->validate($req, [
            'invoices' => 'required|array'
        ], ['invoices.required' => 'No invoices updated']);
        $invoices = ($req->input('invoices'));
        $invoiceIds = [];
        $finalInvoices = [];
        foreach ($invoices as $invoice) {
            $dbInvoice = ResellerPayoutInvoiceGroup::find($invoice->id);
            if (empty($dbInvoice)) continue;
            $dbUser = User::find($dbInvoice->user_id);
            $payableAmount = ($invoice->total_amount > 0 ? -1 * ($invoice->total_amount - (-1 * ($invoice->total_paid_amount))) : (-1 * ($invoice->total_amount)) - $invoice->total_paid_amount);
            if ($payableAmount > 0 && $invoice->paid_amount > $payableAmount) {
                Utils::error("Paid amount should not be greater than payable amount for " . $dbUser->first_name, 400);
            }
            if ($payableAmount < 0 &&  $invoice->paid_amount < $payableAmount) {
                Utils::error("Paid amount should not be lesser than payable amount for " . $dbUser->first_name, 400);
            }
            $invoice->user_id = $dbInvoice->user_id;
            $invoice->from_date = $dbInvoice->from_date;
            $invoice->to_date = $dbInvoice->to_date;
            $finalInvoices[] = $invoice;
        }
        foreach ($finalInvoices as $invoice) {
            $invoiceIds[] = $invoice->id;
            $passbook = new Passbook();
            if ($invoice->paid_amount == 0) continue;
            $passbook->user_id = $invoice->user_id;
            $passbook->amount = $invoice->paid_amount;
            $passbook->entity = "reseller_payout_invoice_group";
            $passbook->entity_id = $invoice->id;
            $passbook->date = Carbon::today();
            $passbook->from_date = $invoice->from_date;
            $passbook->to_date = $invoice->to_date;
            if ($invoice->paid_amount > 0) {
                $passbook->description = "Invoice settled for $invoice->from_date to $invoice->to_date";
            } else {
                $passbook->description = "Invoice paid for $invoice->from_date to $invoice->to_date";
            }

            $passbook->save();
        }
        if (!empty($invoices)) {
            $invoices = ResellerPayoutInvoiceGroup::whereRaw('(id IN (?))', $invoiceIds)->get();
        }
        return $invoices;
    }
    /**
     * Get grouped reseller payout invoices
     * GET api/payout-invoices/groups
     */
    public function groups(Request $req)
    {
        $resellerPayoutInvoiceGroups = ResellerPayoutInvoiceGroup::select("reseller_payout_invoice_groups.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoice_groups.user_id')->orderByRaw('to_date DESC');
        if ($query = $req->query('user_id', false)) {
            $resellerPayoutInvoiceGroups->where('user_id', $query);
        }
        if ($query = $req->query('from_date', false)) {
            $resellerPayoutInvoiceGroups->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $resellerPayoutInvoiceGroups->where('to_date', '<=', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $resellerPayoutInvoiceGroups = $resellerPayoutInvoiceGroups->get();
        } else {
            $resellerPayoutInvoiceGroups = $resellerPayoutInvoiceGroups->paginate($req->query('per_page', 20));
        }
        if ($resellerPayoutInvoiceGroups->isEmpty()) Utils::error("No data found", 400);
        return $resellerPayoutInvoiceGroups;
    }

    /**
     * Get grouped reseller payout invoices
     * GET api/payout-invoices/groups/{id}
     */
    public function group_show(Request $req, $id)
    {
        $resellerPayoutInvoiceGroup = ResellerPayoutInvoiceGroup::select("reseller_payout_invoice_groups.*", "users.first_name", "users.last_name")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoice_groups.user_id')->find($id);
        if (empty($resellerPayoutInvoiceGroup)) Utils::error("No data found", 400);
        $resellerPayoutInvoiceGroup->invoices = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->where('from_date', $resellerPayoutInvoiceGroup->from_date)->where('to_date', $resellerPayoutInvoiceGroup->to_date)->where('user_id', $resellerPayoutInvoiceGroup->user_id)->get();

        if ($req->has('format')) {
            foreach ($resellerPayoutInvoiceGroup->invoices as $payoutInvoice) {
                $payoutInvoice->append('transactions');
                $payoutInvoice->append('business');
                $payoutInvoice->append('reseller');
            }
            $type = 'excel';
            if ($format = $req->query('format', false)) {
                $type = $format;
            }
            $fromDate = Carbon::parse($req->input('from_date'));
            $toDate = Carbon::parse($req->input('to_date'));
            $fileName = $payoutInvoice['reseller']['first_name'] . " " . $payoutInvoice['reseller']['last_name'] . " " . Carbon::parse($resellerPayoutInvoiceGroup->from_date)->format('d-m-Y') . " to " . Carbon::parse($resellerPayoutInvoiceGroup->to_date)->format('d-m-Y');
            if ($type == 'excel')
                return Excel::download(new ResellerMultiplePayoutReportExport($resellerPayoutInvoiceGroup->invoices), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
            else if ($type == 'pdf') {
                return Excel::download(new ResellerMultiplePayoutReportExport($resellerPayoutInvoiceGroup->invoices), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
            } else {
                Utils::error("Invalid export format", 400);
            }
        }

        return $resellerPayoutInvoiceGroup;
    }
}
