<?php

namespace App\Http\Controllers;

use App\Business;
use App\CardBrand;
use App\CardBrandCommission;
use App\CardBrandCommissionHistory;
use App\Country;
use App\CountryCommission;
use App\CountryCommissionHistory;
use App\Exports\ResellerMultiplePayoutReportExport;
use App\Exports\ResellerPayoutReportExport;
use App\Helpers\Utils;
use App\InvoiceTransaction;
use App\MiscCharge;
use App\ModelsConnect\ExternalAccount;
use App\OnlineOrder;
use App\Passbook;
use App\ResellerPayoutInvoice;
use App\PaypalPayment;
use App\ResellerCommission;
use App\ResellerInvoiceTransaction;
use App\ResellerOrderCommission;
use App\ResellerPayoutInvoiceGroup;
use App\ResellerStripeCommission;
use App\StripeTransaction;
use App\User;
use App\UserBusiness;
use Carbon\Carbon;
use DateTime;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;
use PDO;

class ResellerPayoutReportController extends Controller
{
    /*
    |-----------------------------------------
    | Get Report from to to date with business id
    | GET /users/{user_id}/report/{business_id}
    */
    public function quick_report(Request $req, $user_id, $business_id)
    {
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);

        $isConnectAccount = $business->connect_service;
        $accountType = $isConnectAccount ? 'Connect' : 'Normal';

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ]);

        $to_date = Carbon::now()->endOfDay()->format('Y-m-d H:i:s');
        $from_date = Carbon::now()->subDays(11)->startOfDay()->format('Y-m-d H:i:s');
        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";
        }
        $userbusiness = UserBusiness::where('business_id', $business->id)->where('user_id', $user_id)->first();
        if (empty($userbusiness)) Utils::error("No reseller found for this business", 400);
        if ($userbusiness->user_id != $user_id) Utils::error("Invalid reseller", 400);

        $reseller = User::where('id', $userbusiness->user_id)->first();
        if (empty($reseller)) Utils::error("No reseller found", 400);
        $reseller->append('default_bank_detail');
        $reseller->append('default_paypal_detail');
        $reseller->reseller_commissions = $userbusiness->reseller_commissions;
        if (empty($reseller->reseller_commissions)) {
            Utils::error("No commissions set for this reseller and $business->name", 400);
        }
        $chargeDescription = $business->statement_desciptor;

        $data = $this->transactionQueries($from_date, $to_date, 0, $business->id, $chargeDescription, $isConnectAccount);
        $data['from_date'] = $from_date;
        $data['to_date'] = $to_date;
        $data['business'] = $business;
        $data['reseller'] = $reseller;

        $data['misc_charge'] = 0;
        $data['weekly_fee'] = 0;
        $data['monthly_fee'] = 0;

        $data['transaction_type'] = 'Normal';
        $data['payout_mode'] = $reseller->payout_mode;
        $data['payout_method'] = $reseller->payout_method;
        $data['payout_comment'] = $reseller->payout_comment;
        if (strtolower($reseller->payout_method) == 'paypal') {
            if ($reseller->default_paypal_detail != null) {
                $data['payout_account'] = $reseller->default_paypal_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else if (strtolower($reseller->payout_method) == 'bank') {
            if ($reseller->default_bank_detail != null) {
                $data['payout_account'] = $reseller->default_bank_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else {
            $data['payout_account'] = "";
        }

        $mergedData =  $this->mergeAllTransactionsAndSort($data);
        $mergedData['management_fee'] = 0;
        $mergedData['handling_fee'] =  0;
        $mergedData['payout_charge'] =  0;
        $mergedData['net_fee'] = $mergedData['net_fee'] +   $mergedData['management_fee']  + $mergedData['handling_fee'] - $mergedData['payout_charge'];
        $mergedData['net_total'] = $mergedData['net_total'] -   $mergedData['management_fee']  - $mergedData['handling_fee'] + $mergedData['payout_charge'];

        $archivedData = $this->transactionQueries($from_date, $to_date, 1, $business->id, $chargeDescription, $isConnectAccount);
        $archivedData['from_date'] = $from_date;
        $archivedData['to_date'] = $to_date;
        $archivedData['business'] = $business;
        $archivedData['reseller'] = $reseller;

        if (strtolower($reseller->payout_method) == 'paypal') {
            if ($reseller->default_paypal_detail != null) {
                $archivedData['payout_account'] = $reseller->default_paypal_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else if (strtolower($reseller->payout_method) == 'bank') {
            if ($reseller->default_bank_detail != null) {
                $archivedData['payout_account'] = $reseller->default_bank_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else {
            $archivedData['payout_account'] = "";
        }
        $archivedData['misc_charge'] = 0;
        $archivedData['weekly_fee'] = 0;
        $archivedData['monthly_fee'] = 0;

        $archivedData['payout_mode'] = $reseller->archived_payout_mode;
        $archivedData['payout_method'] = $reseller->archived_payout_method;
        $archivedData['transaction_type'] = 'Archived';
        $archivedData['payout_comment'] = $reseller->payout_comment;

        $mergedArchivedData =  $this->mergeAllTransactionsAndSort($archivedData);
        $businessmergedArchivedData =  $this->businessmergeAllTransactionsAndSort($archivedData);

        $managementFee =  $reseller->reseller_commissions->management_fee;
        $cashFee =  $reseller->reseller_commissions->cash_handling_fee;
        $cashFeeOnEveryAmount =  $reseller->reseller_commissions->cash_handling_per_amount;
        $handlingFee = 0;
        if ($cashFeeOnEveryAmount > 0) {
            $handlingFee = $cashFee * ceil($businessmergedArchivedData['net_total'] / $cashFeeOnEveryAmount);
        }

        $mergedArchivedData['management_fee'] =  $managementFee;
        $mergedArchivedData['handling_fee'] =  $handlingFee;
        $mergedArchivedData['payout_charge'] =  0;
        $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] + $mergedArchivedData['management_fee']  + $mergedArchivedData['handling_fee'] - $mergedArchivedData['payout_charge'];
        $mergedArchivedData['net_total'] = $mergedArchivedData['net_total'] - $mergedArchivedData['management_fee']  - $mergedArchivedData['handling_fee'] + $mergedArchivedData['payout_charge'];

        if (sizeof($mergedData['transactions']) == 0) {
            $mergedData = null;
        }

        if (sizeof($mergedArchivedData['transactions']) == 0) {
            $mergedArchivedData = null;
        }

        if ($mergedData == null && $mergedArchivedData == null) {
            if ($req->has('is_job')) {
                return "No data found";
            } else {
                if (!$req->has('is_reseller_job')) {
                    Utils::error("No data found for $business->name", 400);
                }
            }
        }

        if ($format = $req->query('format', false)) {
            return $this->export($mergedData, $mergedArchivedData, $format);
        } else if ($req->has('save')) {
            if ($mergedData != null)
                $saved = $this->saveReport($mergedData);
            else {
                ResellerPayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'Normal')->where('business_id', $business->id)->where('user_id', $reseller->id)->delete();
            }
            if ($mergedArchivedData != null)
                $saved = $this->saveReport($mergedArchivedData);
            else {
                ResellerPayoutInvoice::where('from_date', Carbon::parse($from_date)->toDateString())->where('to_date', Carbon::parse($to_date)->toDateString())->where('transaction_type', 'Archived')->where('business_id', $business->id)->where('user_id', $reseller->id)->delete();
            }
        }
        return ['normal' => $mergedData, 'archived' => $mergedArchivedData];
    }

    public function reseller_business_report($from_date = null, $to_date = null, $reseller, $business)
    {
        $business->append('default_bank_detail');
        $business->append('default_paypal_detail');
        $business->makeHidden(['business_commissions']);
        $isConnectAccount = $business->connect_service;
        $accountType = $isConnectAccount ? 'Connect' : 'Normal';

        if ($from_date == null)
            $to_date = Carbon::now()->endOfDay()->format('Y-m-d H:i:s');
        if ($to_date == null)
            $from_date = Carbon::now()->subDays(11)->startOfDay()->format('Y-m-d H:i:s');

        $reseller->reseller_commissions = ResellerCommission::where('business_id', $business->id)->where('user_id', $reseller->id)->latest()->first();
        if (empty($reseller->reseller_commissions)) {
            return [];
        }

        $reseller->append('default_bank_detail');
        $reseller->append('default_paypal_detail');
        $chargeDescription = $business->statement_desciptor;

        $data = $this->transactionQueries($from_date, $to_date, 0, $business->id, $chargeDescription, $isConnectAccount);
        $data['from_date'] = $from_date;
        $data['to_date'] = $to_date;
        $data['business'] = $business;
        $data['reseller'] = $reseller;

        $data['misc_charge'] = 0;
        $data['weekly_fee'] = 0;
        $data['monthly_fee'] = 0;

        $data['transaction_type'] = 'Normal';
        $data['payout_mode'] = $reseller->payout_mode;
        $data['payout_method'] = $reseller->payout_method;
        $data['payout_comment'] = $reseller->payout_comment;
        if (strtolower($reseller->payout_method) == 'paypal') {
            if ($reseller->default_paypal_detail != null) {
                $data['payout_account'] = $reseller->default_paypal_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else if (strtolower($reseller->payout_method) == 'bank') {
            if ($reseller->default_bank_detail != null) {
                $data['payout_account'] = $reseller->default_bank_detail;
            } else {
                $data['payout_account'] = "";
            }
        } else {
            $data['payout_account'] = "";
        }

        $mergedData =  $this->mergeAllTransactionsAndSort($data);

        $mergedData['management_fee'] = 0;
        $mergedData['handling_fee'] =  0;
        $mergedData['payout_charge'] =  0;
        $mergedData['net_fee'] = $mergedData['net_fee'] +   $mergedData['management_fee']  + $mergedData['handling_fee'] + $mergedData['payout_charge'];
        $mergedData['net_total'] = $mergedData['net_total'] -   $mergedData['management_fee']  - $mergedData['handling_fee'] - $mergedData['payout_charge'];

        // $applicationFee = ($this->convertToDecimal($cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee));
        // $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $serviceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);

        $reports[] = $mergedData;

        $archivedData = $this->transactionQueries($from_date, $to_date, 1, $business->id, $chargeDescription, $isConnectAccount);
        $archivedData['from_date'] = $from_date;
        $archivedData['to_date'] = $to_date;
        $archivedData['business'] = $business;
        $archivedData['reseller'] = $reseller;

        $archivedData['misc_charge'] = 0;
        $archivedData['weekly_fee'] = 0;
        $archivedData['monthly_fee'] = 0;

        $archivedData['transaction_type'] = 'Archived';
        $archivedData['payout_mode'] = $reseller->archived_payout_mode;
        $archivedData['payout_method'] = $reseller->archived_payout_method;
        $archivedData['payout_comment'] = $reseller->payout_comment;
        if (strtolower($reseller->payout_method) == 'paypal') {
            if ($reseller->default_paypal_detail != null) {
                $archivedData['payout_account'] = $reseller->default_paypal_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else if (strtolower($reseller->payout_method) == 'bank') {
            if ($reseller->default_bank_detail != null) {
                $archivedData['payout_account'] = $reseller->default_bank_detail;
            } else {
                $archivedData['payout_account'] = "";
            }
        } else {
            $archivedData['payout_account'] = "";
        }

        $mergedArchivedData =  $this->mergeAllTransactionsAndSort($archivedData);

        $managementFee =  $reseller->reseller_commissions->management_fee;
        $cashFee =  $reseller->reseller_commissions->cash_handling_fee;
        $cashFeeOnEveryAmount =  $reseller->reseller_commissions->cash_handling_per_amount;
        $handlingFee = 0;
        if ($cashFeeOnEveryAmount > 0) {
            $handlingFee = $cashFee * ceil($mergedArchivedData['net_total'] / $cashFeeOnEveryAmount);
        }

        $mergedArchivedData['management_fee'] =  $managementFee;
        $mergedArchivedData['handling_fee'] =  $handlingFee;
        $mergedArchivedData['payout_charge'] =  $handlingFee;
        $mergedArchivedData['net_fee'] = $mergedArchivedData['net_fee'] + $mergedArchivedData['management_fee']  + $mergedArchivedData['handling_fee'] + $mergedArchivedData['payout_charge'];
        $mergedArchivedData['net_total'] = $mergedArchivedData['net_total'] - $mergedArchivedData['management_fee']  - $mergedArchivedData['handling_fee'] - $mergedArchivedData['payout_charge'];

        $reports[] = $mergedArchivedData;

        return $reports;
    }
    public function transactionQueries($from_date, $to_date, $archived, $business_id, $chargeDescription, $isConnectAccount)
    {
        $data = [];
        $codOrders = OnlineOrder::where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $codOrders = $codOrders->leftJoin('reseller_order_commissions', 'reseller_order_commissions.online_order_id', '=', 'online_orders.id');
        $codOrders = $codOrders->where('order_date', '>=', "$from_date");
        $codOrders = $codOrders->where('order_date', '<=', "$to_date");
        $codOrders = $codOrders->where('payment_method', '=', 'cod');
        $codOrders = $codOrders->where('online_orders.archived',  $archived);
        $codOrders = $codOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $codOrders = $codOrders->get();
        if (!$codOrders->isEmpty()) {
            $data['cod_report'] = $this->createReport($codOrders);
        }

        $paypalOrders = OnlineOrder::select('*', 'online_orders.txn_id as online_txn_id', 'paypal_payments.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $paypalOrders = $paypalOrders->leftJoin('reseller_order_commissions', 'reseller_order_commissions.online_order_id', '=', 'online_orders.id');
        $paypalOrders = $paypalOrders->leftJoin('paypal_payments', 'sale_id', '=', 'online_orders.txn_id');
        $paypalOrders = $paypalOrders->where('paypal_payments.archived',  $archived);
        $paypalOrders = $paypalOrders->where('order_date', '>=', "$from_date");
        $paypalOrders = $paypalOrders->where('order_date', '<=', "$to_date");
        $paypalOrders = $paypalOrders->where('payment_method', '=', 'paypal');
        $paypalOrders = $paypalOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $paypalOrders = $paypalOrders->get();
        if (!$paypalOrders->isEmpty()) {
            $data['paypal_report'] = $this->createReport($paypalOrders);;
        }

        $stripeOrders = OnlineOrder::select('*', 'stripe_transactions.txn_id as stripe_txn_id', 'online_orders.txn_id as online_txn_id', 'stripe_transactions.card_number as card_number', 'stripe_transactions.card_brand as card_brand', 'stripe_transactions.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $stripeOrders = $stripeOrders->leftJoin('reseller_order_commissions', 'reseller_order_commissions.online_order_id', '=', 'online_orders.id');
        $stripeOrders = $stripeOrders->leftJoin('stripe_transactions', function ($join) {
            $join->on('stripe_transactions.payment_intent_id', '=', 'online_orders.txn_id')->where('reporting_category', 'charge');
        });
        $stripeOrders = $stripeOrders->where('order_date', '>=', "$from_date");
        $stripeOrders = $stripeOrders->where('order_date', '<=', "$to_date");
        $stripeOrders = $stripeOrders->where('stripe_transactions.archived',  $archived);
        $stripeOrders = $stripeOrders->whereIn('payment_method', ['Credit Card', 'Online']);
        $stripeOrders = $stripeOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $stripeOrders = $stripeOrders->get();
        if (!$stripeOrders->isEmpty()) {
            $data['card_report'] = $this->createReport($stripeOrders);;
        }

        $walletOrders = OnlineOrder::select('*',  'online_orders.txn_id as online_txn_id')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $walletOrders = $walletOrders->leftJoin('reseller_order_commissions', 'reseller_order_commissions.online_order_id', '=', 'online_orders.id');
        $walletOrders = $walletOrders->where('order_date', '>=', "$from_date");
        $walletOrders = $walletOrders->where('order_date', '<=', "$to_date");
        $walletOrders = $walletOrders->where('online_orders.archived',  $archived);
        $walletOrders = $walletOrders->whereIn('payment_method', ['Wallet']);
        $walletOrders = $walletOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $walletOrders = $walletOrders->get();
        if (!$walletOrders->isEmpty()) {
            $data['wallet_report'] = $this->createReport($walletOrders);;
        }

        $merchantTransactions = StripeTransaction::whereRaw('(descriptor LIKE ? OR business_id LIKE ?)', ["$chargeDescription", $business_id]);
        $merchantTransactions = $merchantTransactions->where('stripe_transactions.archived',  $archived);
        $merchantTransactions = $merchantTransactions->where('created', '>=', strtotime($from_date));
        $merchantTransactions = $merchantTransactions->where('created', '<=', strtotime($to_date));
        $merchantTransactions = $merchantTransactions->whereIn('reporting_category', ['charge', 'refund']);
        $merchantTransactions = $merchantTransactions->whereIn('service', ['Merchant', 'Connect']);
        $merchantTransactions = $merchantTransactions->get();
        if (!$merchantTransactions->isEmpty()) {
            $data['merchant_report'] = $this->createMerchantReport($merchantTransactions, $isConnectAccount);
        }

        $businesscodOrders = OnlineOrder::where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $businesscodOrders = $businesscodOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $businesscodOrders = $businesscodOrders->where('order_date', '>=', "$from_date");
        $businesscodOrders = $businesscodOrders->where('order_date', '<=', "$to_date");
        $businesscodOrders = $businesscodOrders->where('payment_method', '=', 'cod');
        $businesscodOrders = $businesscodOrders->where('online_orders.archived',  $archived);
        $businesscodOrders = $businesscodOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $businesscodOrders = $businesscodOrders->get();
        if (!$businesscodOrders->isEmpty()) {
            $data['business_cod_report'] = $this->createbusinessReport($business_id, $businesscodOrders);
        }

        $businesspaypalOrders = OnlineOrder::select('*', 'online_orders.txn_id as online_txn_id', 'paypal_payments.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $businesspaypalOrders = $businesspaypalOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $businesspaypalOrders = $businesspaypalOrders->leftJoin('paypal_payments', 'sale_id', '=', 'online_orders.txn_id');
        $businesspaypalOrders = $businesspaypalOrders->where('paypal_payments.archived',  $archived);
        $businesspaypalOrders = $businesspaypalOrders->where('order_date', '>=', "$from_date");
        $businesspaypalOrders = $businesspaypalOrders->where('order_date', '<=', "$to_date");
        $businesspaypalOrders = $businesspaypalOrders->where('payment_method', '=', 'paypal');
        $businesspaypalOrders = $businesspaypalOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $businesspaypalOrders = $businesspaypalOrders->get();
        if (!$businesspaypalOrders->isEmpty()) {
            $data['business_paypal_report'] = $this->createbusinessReport($business_id, $businesspaypalOrders);;
        }

        $businessstripeOrders = OnlineOrder::select('*', 'stripe_transactions.txn_id as stripe_txn_id', 'online_orders.txn_id as online_txn_id', 'stripe_transactions.card_number as card_number', 'stripe_transactions.card_brand as card_brand', 'stripe_transactions.refunded_amount as refunded_amount')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $businessstripeOrders = $businessstripeOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $businessstripeOrders = $businessstripeOrders->leftJoin('stripe_transactions', function ($join) {
            $join->on('stripe_transactions.payment_intent_id', '=', 'online_orders.txn_id')->where('reporting_category', 'charge');
        });
        $businessstripeOrders = $businessstripeOrders->where('order_date', '>=', "$from_date");
        $businessstripeOrders = $businessstripeOrders->where('order_date', '<=', "$to_date");
        $businessstripeOrders = $businessstripeOrders->where('stripe_transactions.archived',  $archived);
        $businessstripeOrders = $businessstripeOrders->whereIn('payment_method', ['Credit Card', 'Online']);
        $businessstripeOrders = $businessstripeOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $businessstripeOrders = $businessstripeOrders->get();
        if (!$businessstripeOrders->isEmpty()) {
            $data['business_card_report'] = $this->createbusinessReport($business_id, $businessstripeOrders);;
        }


        $businesswalletOrders = OnlineOrder::select('*',  'online_orders.txn_id as online_txn_id')->where('online_orders.business_id', $business_id)->orderByRaw('order_date DESC');
        $businesswalletOrders = $businesswalletOrders->leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id');
        $businesswalletOrders = $businesswalletOrders->where('order_date', '>=', "$from_date");
        $businesswalletOrders = $businesswalletOrders->where('order_date', '<=', "$to_date");
        $businesswalletOrders = $businesswalletOrders->where('online_orders.archived',  $archived);
        $businesswalletOrders = $businesswalletOrders->whereIn('payment_method', ['Wallet']);
        $businesswalletOrders = $businesswalletOrders->whereIn('order_status', ['Delivered', 'Failed']);
        $businesswalletOrders = $businesswalletOrders->get();
        if (!$businesswalletOrders->isEmpty()) {
            $data['business_wallet_report'] = $this->createbusinessReport($business_id, $businesswalletOrders);;
        }

        $businessmerchantTransactions = StripeTransaction::whereRaw('(descriptor LIKE ? OR business_id LIKE ?)', ["$chargeDescription", $business_id]);
        $businessmerchantTransactions = $businessmerchantTransactions->where('stripe_transactions.archived',  $archived);
        $businessmerchantTransactions = $businessmerchantTransactions->where('created', '>=', strtotime($from_date));
        $businessmerchantTransactions = $businessmerchantTransactions->where('created', '<=', strtotime($to_date));
        $businessmerchantTransactions = $businessmerchantTransactions->whereIn('reporting_category', ['charge', 'refund', 'dispute']);
        $businessmerchantTransactions = $businessmerchantTransactions->whereIn('service', ['Merchant', 'Connect']);
        $businessmerchantTransactions = $businessmerchantTransactions->get();
        if (!$businessmerchantTransactions->isEmpty()) {
            $data['business_merchant_report'] = $this->createMerchantBusinessReport($businessmerchantTransactions, true, $isConnectAccount);
        }

        $businessonlineRefundTransactions = StripeTransaction::whereRaw('(descriptor LIKE ? OR business_id LIKE ?)', ["$chargeDescription", $business_id]);
        $businessonlineRefundTransactions = $businessonlineRefundTransactions->where('stripe_transactions.archived',  $archived);
        $businessonlineRefundTransactions = $businessonlineRefundTransactions->where('created', '>=', strtotime($from_date));
        $businessonlineRefundTransactions = $businessonlineRefundTransactions->where('created', '<=', strtotime($to_date));
        $businessonlineRefundTransactions = $businessonlineRefundTransactions->where('reporting_category', 'refund');
        $businessonlineRefundTransactions = $businessonlineRefundTransactions->whereIn('service', ['online']);
        $businessonlineRefundTransactions = $businessonlineRefundTransactions->get();
        if (!$businessonlineRefundTransactions->isEmpty()) {
            $data['business_online_refund_report'] = $this->createMerchantBusinessReport($businessonlineRefundTransactions, true, $isConnectAccount);
        }

        return $data;
    }
    public function createReport($orders)
    {
        $transactions = array();

        $gross_total = 0.0;
        $net_total = 0.0;
        $total_fee = 0.0;
        $total_service_charge = 0;

        $total_provider_fee = 0.0;
        $total_business_fee = 0.0;
        $total_reseller_fee = 0.0;
        $total_non_domestic_fee = 0;
        $total_charge_back_fee = 0.0;

        $total_cash_payout = 0.0;
        $total_bank_payout = 0.0;

        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;

        foreach ($orders as $order) {
            $orderTypeWiseCommission = 0.00;
            $orderTypeWiseServiceCharge = 0.00;
            $orderBasedOrTransactionWiseCharge = 0.00;
            $payment_method_fees = 0;

            $reseller_orderTypeWiseCommission = 0.00;
            $reseller_orderTypeWiseServiceCharge = 0.00;
            $reseller_orderBasedOrTransactionWiseCharge = 0.00;
            $reseller_payment_method_fees = 0;

            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;

            $orderTypeWiseServiceCharge = $order->user_service_charge;
            $orderGrandTotal = $order->grand_total;

            $chargeBackFee = 0;
            $handlingFee = 0;
            $cardCountry = "";
            $brand = "";

            $resellerCommission = ResellerOrderCommission::where('online_order_id', $order->id)->orderBy('id', 'DESC')->first();
            $providerFee = 0;
            if ($order->payment_method == 'paypal') {
                $providerFee = $order->provider_fee;
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_paypal_fee, $order->online_paypal_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_payment_method_fees  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_paypal_fee, $resellerCommission->online_paypal_fee_type);
                }
            }
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online' || strtolower($order->payment_method) == 'wallet') {
                $providerFee = $order->provider_fee / 100;
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_card_fee, $order->online_card_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_payment_method_fees  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_card_fee, $resellerCommission->online_card_fee_type);
                }
            }
            if ($order->payment_method == 'cod') {
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_cod_fee, $order->online_cod_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_payment_method_fees  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_cod_fee, $resellerCommission->online_cod_fee_type);
                }
            }

            if ($order->order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->dinein_commission_fee, $order->dinein_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($orderGrandTotal, $order->dinein_service_fee, $order->dinein_service_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_orderTypeWiseCommission  = $this->calculateFee($orderGrandTotal, $resellerCommission->dinein_commission_fee, $resellerCommission->dinein_commission_fee_type);
                    $reseller_orderTypeWiseServiceCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->dinein_service_fee, $resellerCommission->dinein_service_fee_type);
                }
            } else {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->online_commission_fee, $order->online_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($orderGrandTotal, $order->online_service_fee, $order->online_service_fee_type);
                if (!empty($resellerCommission)) {
                    $reseller_orderTypeWiseCommission  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_commission_fee, $resellerCommission->online_commission_fee_type);
                    $reseller_orderTypeWiseServiceCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->online_service_fee, $resellerCommission->online_service_fee_type);
                }
            }
            if ($order->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotal, $order->per_order_charge, $order->per_order_charge_type);
                if (!empty($resellerCommission)) {
                    $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->per_order_charge, $resellerCommission->per_order_charge_type);
                }
            } else {
                if ($order->payment_method == 'paypal') {
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->paypal_charge, $order->paypal_charge_type);
                    if (!empty($resellerCommission)) {
                        $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->paypal_charge, $resellerCommission->paypal_charge_type);
                    }
                }
                if (strtolower($order->payment_method) == 'credit card' || strtolower($order->payment_method) == 'online' || strtolower($order->payment_method) == 'wallet') {
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->card_charge, $order->card_charge_type);
                    if (!empty($resellerCommission)) {
                        $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->card_charge, $resellerCommission->card_charge_type);
                    }
                }
                if ($order->payment_method == 'cod') {
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->cod_charge, $order->cod_charge_type);
                    if (!empty($resellerCommission)) {
                        $reseller_orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $resellerCommission->cod_charge, $resellerCommission->cod_charge_type);
                    }
                }
            }

            $businessFee = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission);
            $resellerFee = $this->convertToDecimal($reseller_payment_method_fees + $reseller_orderTypeWiseCommission + $reseller_orderTypeWiseServiceCharge + $reseller_orderBasedOrTransactionWiseCharge);
            $serviceCharge = 0;
            $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $serviceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);

            if ($order->order_status == 'Failed') {
                $transaction['order_grand_total'] = -$this->convertToDecimal($order->grand_total);
                if ($order->charges_refundable == 'yes') {
                    if ($order->payment_method == 'cod') {
                        $businessFee = 0;
                        $resellerFee = 0;
                    } else {
                        $businessFee = -$businessFee;
                        $resellerFee = -$resellerFee;
                    }
                } else {
                    if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online') {
                        $businessFee = 0.0;
                        $resellerFee = 0.0;
                    } else {
                        $businessFee = $businessFee;
                        $resellerFee = $resellerFee;
                    }
                }
                $transaction['net_total'] = $this->convertToDecimal(0 - $businessFee);
                // $total_refund_transactions = $total_refund_transactions + 1;
                $total_refund_transactions = $total_refund_transactions + $this->convertToDecimal($orderGrandTotal);
            } else {
                $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotal);
                if ($order->payment_method == 'cod') $transaction['order_grand_total'] = 0;
                $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $businessFee);
            }

            if ($order->payment_method != 'cod')
                $gross_total = $gross_total + $transaction['order_grand_total'];
            $transaction['id'] =  $order->id;
            $transaction['order_id'] = "#ODR000" . $order->order_id;
            $transaction['restaurant_id'] = $order->restaurant_id;
            $transaction['business_id'] = $order->business_id;
            $transaction['business_name'] = $order->business_name;
            if (empty($order->stripe_txn_id)) {
                $transaction['txn_id'] = $order->online_txn_id;
            } else {
                $transaction['txn_id'] = $order->stripe_txn_id;
            }

            $transaction['order_date'] = $order->order_date;
            $transaction['order_type'] = strtolower($order->order_type) == 'delivery' || strtolower($order->order_type) == 'pickup' ? 'Online - ' . ($order->payment_method) : $order->order_type;
            $transaction['order_archived'] = $order->archived;
            if (!empty($order->online_reporting_category)) {
                $transaction['order_status'] = $order->online_reporting_category;
            } else {
                $transaction['order_status'] = $order->order_status;
            }
            $transaction['payment_method'] = strtolower($order->payment_method) == 'credit card' ? 'Online' : $order->payment_method;

            $transaction['payment_method_fees'] = $payment_method_fees;
            $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
            $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
            $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;

            $transaction['country_charges'] = $countryWiseCommission;
            $transaction['card_charges'] = $cardBrandWiseCommission;
            $transaction['charge_back_fee'] = 0;
            $transaction['provider_fee'] = $providerFee;

            $transaction['business_fee'] = $businessFee;
            $transaction['reseller_fee'] = $resellerFee;
            $transaction['total_fee'] = $transaction['business_fee'] - $transaction['reseller_fee'] - $transaction['provider_fee'];
            $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotal);

            if ($order->payment_method != 'Failed') {
                $total_provider_fee = $total_provider_fee + $transaction['provider_fee'];
                $total_business_fee = $total_business_fee + $transaction['business_fee'];
                $total_reseller_fee = $total_reseller_fee + $transaction['reseller_fee'];
                $total_fee = $total_fee + $transaction['total_fee'];

                if ($transaction['order_archived']) {
                    $total_cash_payout = $total_cash_payout + $transaction['net_total'];
                } else {
                    $total_bank_payout = $total_bank_payout + $transaction['net_total'];
                }
            }

            $total_charge_back_fee = $total_charge_back_fee + ($transaction['charge_back_fee']);
            $transactions[] = $transaction;
        }
        $report['transactions'] = $transactions;

        $report['total_business_fee'] = $this->convertToDecimal($total_business_fee);
        $report['total_reseller_fee'] = $this->convertToDecimal($total_reseller_fee);
        $report['total_provider_fee'] = $this->convertToDecimal($total_provider_fee);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['cash_payout'] = $this->convertToDecimal($total_cash_payout);
        $report['bank_payout'] = $this->convertToDecimal($total_bank_payout);
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($gross_total - $total_business_fee);
        $report['total_non_domestic_fee'] =  $this->convertToDecimal($total_non_domestic_fee);
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        $report['total_charge_back_transactions'] = $total_charge_back_transactions;
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        return $report;
    }
    public function createMerchantReport($merchantTransactions, $isConnectAccount)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_charge_back_transactions = 0;
        $total_charge_back_fee = 0.0;
        $total_fee = 0.0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_service_charge = 0;

        foreach ($merchantTransactions as $transaction) {
            $cardProcessingFee = 0.00;
            $cardAuthFee = 0.00;
            $managementFee = 0.00;
            $handlingFee = 0.00;
            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;
            $chargeBackFee = 0;
            $cardCountry = $transaction->card_country;
            $brand = $transaction->card_brand;
            $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $transaction->id)->first();
            $applicationFee = 0; // means total fee for particular transaction
            if (!empty($resellerStripeCommission)) {
                $transaction->fill(json_decode($resellerStripeCommission, true));

                if ($transaction->reporting_category == 'charge') {
                    $cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $resellerStripeCommission->merchant_card_auth_fee, $resellerStripeCommission->merchant_card_auth_fee_type));
                    $cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $resellerStripeCommission->merchant_card_processing_fee, $resellerStripeCommission->merchant_card_processing_fee_type));
                }
                if ($transaction->reporting_category == 'refund') {
                    $cardAuthFee = -$cardAuthFee;
                    $cardProcessingFee = -$cardProcessingFee;
                    $cardBrandWiseCommission = -$cardBrandWiseCommission;
                    $countryWiseCommission = -$countryWiseCommission;
                    $total_refund_transactions = $total_refund_transactions + 1;
                }
                if ($transaction->reporting_category == 'dispute') {
                    $chargeBackFee = -$this->calculateFee($transaction->amount / 100, $resellerStripeCommission->charge_back_fee, $resellerStripeCommission->charge_back_fee_type);
                    $total_charge_back_transactions = $total_charge_back_transactions + 1;
                }
                $applicationFee = ($this->convertToDecimal($managementFee + $handlingFee + $cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee));
                if ($transaction->reporting_category == 'refund' || $transaction->reporting_category == 'transfer_reversal' || $transaction->refunded) {
                    $applicationFee = 0;
                } else {
                    $cardAuthFee  = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->connect_card_auth_fee, $resellerStripeCommission->connect_card_auth_fee_type);
                    $cardProcessingFee  = $this->calculateFee($transaction->amount / 100, $resellerStripeCommission->connect_card_processing_fee, $resellerStripeCommission->connect_card_processing_fee_type);
                    //country wise & card brand wise charges are not yet calculated
                    $applicationFee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
                }
            }


            $merchantTransction['order_id'] = $transaction->txn_id;
            $merchantTransction['restaurant_id'] = null;
            $merchantTransction['txn_id'] = $transaction->txn_id;
            $merchantTransction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
            $merchantTransction['order_type'] = $transaction->service;
            $merchantTransction['order_status'] = $transaction->reporting_category;
            $merchantTransction['payment_method'] = $transaction->service;
            $merchantTransction['order_grand_total'] = $transaction->amount / 100;

            $merchantTransction['processing_fee'] = $cardProcessingFee;
            $merchantTransction['country'] = $cardCountry;
            $merchantTransction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
            $merchantTransction['card_brand'] = $brand;
            $merchantTransction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
            $merchantTransction['auth_fee'] = $cardAuthFee;
            $merchantTransction['handling_fee'] = $handlingFee;
            $merchantTransction['management_fee'] = $managementFee;
            $merchantTransction['charge_back_fee'] = $chargeBackFee;
            $merchantTransction['total_fee'] =  $applicationFee;
            if ($transaction->non_domestic) {
                $total_non_domestic_transactions = $total_non_domestic_transactions + 1;
                $total_non_domestic_fee = $total_non_domestic_fee + $countryWiseCommission;
            }

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
                if ($isConnectAccount && abs($applicationFee) > ($transaction->amount / 100)) $merchantTransction['net_total'] = 0;
            } else {
                $merchantTransction['total_fee'] = 0;
                $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $merchantTransction['total_fee']);
            }

            $gross_total = $gross_total + $merchantTransction['order_grand_total'];
            $net_total = $net_total + $merchantTransction['net_total'];
            $total_fee = $total_fee + abs($merchantTransction['total_fee']);
            $total_charge_back_fee = $total_charge_back_fee + $merchantTransction['charge_back_fee'];
            $transactions[] = $merchantTransction;
        }
        $report['transactions'] = $transactions;
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($net_total);
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        $report['total_charge_back_transactions'] = $this->convertToDecimal($total_charge_back_transactions);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['total_non_domestic_fee'] = $total_non_domestic_fee;
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        return $report;
    }
    public function createbusinessReport($business_id, $orders)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_charge_back_transactions = 0;
        $total_charge_back_fee = 0.0;
        $total_fee = 0.0;
        $total_non_domestic_transactions = 0;
        $total_refund_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_service_charge = 0;
        foreach ($orders as $order) {
            $orderTypeWiseCommission = 0.00;
            $orderTypeWiseServiceCharge = 0.00;
            $orderBasedOrTransactionWiseCharge = 0.00;
            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;
            $payment_method_fees = 0;
            $chargeBackFee = 0;
            $handlingFee = 0.00;
            $cardCountry = "";
            $brand = "";

            if (!empty($order->card_brand)) {
                $brand = $order->card_brand;
            }
            $business = Business::where('id', $business_id)->first();
            if ($business->service_charge_type == 'show') {
                $orderTypeWiseServiceCharge = $order->user_service_charge; // It would be mostly in flat.
            }

            $orderGrandTotalBeforeServiceCharge = $order->grand_total;
            $orderGrandTotalForDisplay = $order->grand_total;
            $refundedAmount = 0;

            if ($order->payment_method == 'paypal') {
                $refundedAmount = abs($order->refunded_amount);
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_paypal_fee, $order->online_paypal_fee_type);
            }
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online' || strtolower($order->payment_method) == 'wallet') {
                $refundedAmount = abs($order->refunded_amount) / 100;
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_card_fee, $order->online_card_fee_type);
            }

            if ($order->payment_method == 'cod')
                $payment_method_fees  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_cod_fee, $order->online_cod_fee_type);

            if ($order->order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->dinein_commission_fee, $order->dinein_commission_fee_type);
            } else {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->online_commission_fee, $order->online_commission_fee_type);
            }
            if ($order->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->per_order_charge, $order->per_order_charge_type);
            } else {
                if ($order->payment_method == 'paypal')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->paypal_charge, $order->paypal_charge_type);
                if (strtolower($order->payment_method) == 'credit card' || strtolower($order->payment_method) == 'online' || strtolower($order->payment_method) == 'wallet')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->card_charge, $order->card_charge_type);
                if ($order->payment_method == 'cod')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotalBeforeServiceCharge, $order->cod_charge, $order->cod_charge_type);
            }


            $serviceCharge = $orderTypeWiseServiceCharge;
            $serviceCharge = 0;
            $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $serviceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);
            if ($order->order_status == 'Failed') {
                $transaction['order_grand_total'] = -$this->convertToDecimal($orderGrandTotalForDisplay);
                if ($order->charges_refundable == 'yes') {
                    $transaction['total_fee'] = 0;
                    $orderTypeWiseServiceCharge = 0;
                }
                $total_refund_transactions = $total_refund_transactions + 1;
                $transaction['net_total'] = $this->convertToDecimal(0 - $transaction['total_fee']);
            } else {
                $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotalForDisplay);
                if ($order->payment_method == 'cod') $transaction['order_grand_total'] = 0;

                $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $transaction['total_fee']);
                $gross_total = $gross_total + $transaction['order_grand_total'];
            }

            $transaction['order_id'] = "#ODR000" . $order->order_id;
            $transaction['restaurant_id'] = $order->restaurant_id;
            if (empty($order->stripe_txn_id)) {
                $transaction['txn_id'] = $order->online_txn_id;
            } else {
                $transaction['txn_id'] = $order->stripe_txn_id;
            }
            $transaction['order_date'] = $order->order_date;
            $transaction['order_type'] = $order->order_type;
            $transaction['order_status'] = $order->order_status;
            $transaction['payment_method'] = strtolower($order->payment_method) == 'credit card' ? 'Online' : $order->payment_method;
            $transaction['card_number'] = !empty($order->card_number) ? $order->card_number : null;
            $transaction['card_brand'] = $brand;
            $transaction['payment_method_fees'] = $payment_method_fees;
            $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
            $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
            $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;
            $transaction['country'] = $cardCountry;
            $transaction['country_charges'] = $countryWiseCommission;
            $transaction['charge_back_fee'] = $chargeBackFee;
            $transaction['handling_fee'] = $handlingFee;
            $transaction['card_brand'] = $brand;
            $transaction['card_brand_charge'] = $cardBrandWiseCommission;
            $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotalBeforeServiceCharge);

            $net_total = $net_total + $transaction['net_total'];
            $total_fee = $total_fee + abs($transaction['total_fee']);
            $total_service_charge = $total_service_charge + $orderTypeWiseServiceCharge;
            $total_charge_back_fee = $total_charge_back_fee + ($transaction['charge_back_fee']);
            $transactions[] = $transaction;
        }
        $report['transactions'] = $transactions;
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($net_total);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_non_domestic_fee'] = $total_non_domestic_fee;
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_charge_back_transactions'] = $this->convertToDecimal($total_charge_back_transactions);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['total_refund_transactions'] = $total_refund_transactions;
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        return $report;
    }
    public function createMerchantBusinessReport($merchantTransactions, $archivedReport, $isConnectAccount)
    {
        $transactions = array();
        $gross_total = 0.0;
        $net_total = 0.0;
        $total_fee = 0.0;

        $total_charge_back_transactions = 0;
        $total_refund_transactions = 0;
        $total_charge_back_fee = 0.0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $managementFee = 0;
        $archivedTransactions = 0;
        $cardCountry = "";
        $brand = "";
        $total_service_charge = 0;

        foreach ($merchantTransactions as $transaction) {
            $cardProcessingFee = 0.00;
            $cardAuthFee = 0.00;
            $managementFee = 0.00;
            $handlingFee = 0.00;
            $chargeBackFee = 0;
            $cardCountry = $transaction->card_country;
            $brand = $transaction->card_brand;

            $countryWiseCommission = 0.0;
            $cardBrandWiseCommission = 0.0;

            // if ($transaction->reporting_category == 'refund') {
            //     $checkRefundTransactions = StripeTransaction::where('payment_intent_id', $transaction->payment_intent_id)->where('txn_id', '!=', $transaction->txn_id)->first();
            //     if (empty($checkRefundTransactions)) {
            //         continue;
            //     }
            // }

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_auth_fee, $transaction->merchant_card_auth_fee_type));
                $cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_processing_fee, $transaction->merchant_card_processing_fee_type));
                if ($transaction->archived) {
                    if ($transaction->cash_handling_fee_type == 'percentage') {
                        $handlingFee  = $this->calculateFee($transaction->amount / 100, $transaction->cash_handling_fee, $transaction->cash_handling_fee_type);
                    } else {
                        if ($transaction->cash_handling_per_amount > 0)
                            $handlingFee = $transaction->cash_handling_fee * ceil($transaction->amount / $transaction->cash_handling_per_amount);
                    }
                    $handlingFee = 0;
                    $archivedTransactions = $archivedTransactions + 1;
                }

                $countryCommission = CountryCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();
                if (!empty($countryCommission)) {
                    $countryWiseCommission = abs($this->calculateFee($transaction->amount / 100, $countryCommission->stripe_percentage, 'percentage'));
                    $countryWiseCommission = $countryWiseCommission + $countryCommission->stripe_fixed;
                }
                $cardBrandCommission = CardBrandCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();

                if (!empty($cardBrandCommission)) {
                    $cardBrandWiseCommission = abs($this->calculateFee($transaction->amount / 100, $cardBrandCommission->stripe_percentage, 'percentage'));
                    $cardBrandWiseCommission = $cardBrandWiseCommission + $cardBrandCommission->stripe_fixed;
                }

                if ($transaction->reporting_category == 'refund') {
                    $cardAuthFee = -$cardAuthFee;
                    $cardProcessingFee = -$cardProcessingFee;
                    $cardBrandWiseCommission = -$cardBrandWiseCommission;
                    $countryWiseCommission = -$countryWiseCommission;
                }
            }
            if ($transaction->reporting_category == 'dispute') {
                $chargeBackFee = $this->calculateFee($transaction->amount / 100, $transaction->charge_back_fee, $transaction->charge_back_fee_type);
                $total_charge_back_transactions = $total_charge_back_transactions + 1;
            }
            if ($transaction->reporting_category == 'refund') {
                $total_refund_transactions = $total_refund_transactions + 1;
            }
            $applicationFee = ($this->convertToDecimal($managementFee + $handlingFee + $cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee));
            if ($transaction->service == 'Connect') {
                if ($transaction->reporting_category == 'refund' || $transaction->reporting_category == 'transfer_reversal') {
                    $applicationFee = 0;
                } else {
                    $applicationFeeTransaction = StripeTransaction::where('reporting_category', 'platform_earning')->whereNotNull('payment_intent_id')->where('business_id', $transaction->business_id)->where('payment_intent_id', $transaction->payment_intent_id)->first();
                    if (!empty($applicationFeeTransaction)) {
                        $applicationFee =  $this->convertToDecimal($applicationFeeTransaction->amount / 100);
                    }
                }
            }
            if ($transaction->service == 'Online' && $transaction->reporting_category == 'refund') {
                $applicationFee = 0;
            }

            $merchantTransction['order_id'] = $transaction->txn_id;
            $merchantTransction['restaurant_id'] = null;
            $merchantTransction['txn_id'] = $transaction->txn_id;
            $merchantTransction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
            $merchantTransction['order_type'] = $transaction->service;
            $merchantTransction['order_status'] = $transaction->reporting_category;
            $merchantTransction['payment_method'] = $transaction->service;
            $merchantTransction['order_grand_total'] = $transaction->amount / 100;

            $merchantTransction['processing_fee'] = $cardProcessingFee;
            $merchantTransction['country'] = $cardCountry;
            $merchantTransction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
            $merchantTransction['card_brand'] = $brand;
            $merchantTransction['card_number'] = $transaction->card_number;
            $merchantTransction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
            $merchantTransction['auth_fee'] = $cardAuthFee;
            $merchantTransction['handling_fee'] = 0;
            $merchantTransction['management_fee'] = $managementFee;
            $merchantTransction['charge_back_fee'] = $chargeBackFee;
            $merchantTransction['total_fee'] =  $applicationFee;
            if ($transaction->non_domestic) {
                $total_non_domestic_transactions = $total_non_domestic_transactions + 1;
                $total_non_domestic_fee = $total_non_domestic_fee + $countryWiseCommission;
            }

            if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
                $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
                if ($transaction->service == 'Connect' && abs($applicationFee) > ($transaction->amount / 100) && $transaction->reporting_category != 'refund') $merchantTransction['net_total'] = 0; // Condition double check when charge refund yes.
            } else {
                if ($transaction->reporting_category == 'dispute') {
                    $merchantTransction['total_fee'] = $chargeBackFee;
                    $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
                } else {
                    $merchantTransction['total_fee'] = 0;
                    $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $merchantTransction['total_fee']);
                }
            }
            $gross_total = $gross_total + $merchantTransction['order_grand_total'];
            $net_total = $net_total + $merchantTransction['net_total'];
            $total_fee = $total_fee + abs($merchantTransction['total_fee']);
            $total_charge_back_fee = $total_charge_back_fee + $merchantTransction['charge_back_fee'];
            $transactions[] = $merchantTransction;
        }
        $report['transactions'] = $transactions;
        $report['gross_total'] = $this->convertToDecimal($gross_total);
        $report['net_total'] = $this->convertToDecimal($net_total);
        $report['total_charge_back_fee'] = $this->convertToDecimal($total_charge_back_fee);
        $report['total_charge_back_transactions'] = $this->convertToDecimal($total_charge_back_transactions);
        $report['total_fee'] = $this->convertToDecimal($total_fee);
        $report['total_service_charge'] = $this->convertToDecimal($total_service_charge);
        $report['total_non_domestic_fee'] = $total_non_domestic_fee;
        $report['total_non_domestic_transactions'] = $total_non_domestic_transactions;
        $report['total_refund_transactions'] = $total_refund_transactions;
        return $report;
    }

    public function export($report = null, $archivedReport = null, $type = "excel")
    {
        if ($report != null) {
            $fromDate = Carbon::parse($report['from_date']);
            $toDate = Carbon::parse($report['to_date']);
            $fileName = $report['reseller']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
        } else if ($archivedReport != null) {
            $fromDate = Carbon::parse($archivedReport['from_date']);
            $toDate = Carbon::parse($archivedReport['to_date']);
            $fileName = $archivedReport['reseller']['name'] . " " . $fromDate->format('d-m-Y') . " to " . $toDate->format('d-m-Y');
        } else {
            Utils::error("No data found", 400);
        }
        if ($report != null)
            $reports[] = $report;
        if ($archivedReport != null) {
            $reports[] = $archivedReport;
        }

        if ($type == 'excel')
            return Excel::download(new ResellerMultiplePayoutReportExport($reports), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new ResellerMultiplePayoutReportExport($reports), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }
    public function exportMultiple($report = null, $fileName, $type = "excel")
    {
        if ($type == 'excel')
            return Excel::download(new ResellerMultiplePayoutReportExport($report), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        else if ($type == 'pdf') {
            return Excel::download(new ResellerMultiplePayoutReportExport($report), "$fileName.pdf", \Maatwebsite\Excel\Excel::MPDF);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    public function saveReport($report)
    {
        $business = $report['business'];
        $reseller = $report['reseller'];
        $shouldDeleteOldTransactions = false;
        $payoutInvoice = ResellerPayoutInvoice::where('from_date', explode(" ", $report['from_date'])[0])->where('to_date', explode(" ", $report['to_date'])[0])->where('transaction_type', $report['transaction_type'])->where('business_id', $report['business']['id'])->where('user_id', $report['reseller']['id'])->first();
        if (sizeof($report['transactions']) == 0 || $report['net_fee'] == 0) {
            if (!empty($payoutInvoice)) $payoutInvoice->delete();
            return $report;
        }
        if (empty($payoutInvoice)) {
            $payoutInvoice = new ResellerPayoutInvoice();
        } else {
            $shouldDeleteOldTransactions = true;
        }
        $payoutInvoice->business_id = $business->id;
        $payoutInvoice->user_id = $reseller->id;
        $payoutInvoice->gross_total = $report['gross_total'];
        $payoutInvoice->gross_fee = $report['gross_fee'];
        $payoutInvoice->misc_charge = $report['misc_charge'];
        $payoutInvoice->monthly_fee = $report['monthly_fee'];
        $payoutInvoice->handling_fee = $report['handling_fee'];
        $payoutInvoice->management_fee = $report['management_fee'];
        $payoutInvoice->payout_charge = $report['payout_charge'];
        $payoutInvoice->weekly_fee = $report['weekly_fee'];
        $payoutInvoice->net_fee = $report['net_fee'];
        $payoutInvoice->net_total = $report['net_total'];
        $payoutInvoice->payout_comment = $report['payout_comment'];
        $payoutInvoice->total_charge_back_transactions = $report['total_charge_back_transactions'];
        $payoutInvoice->total_charge_back_fee = $report['total_charge_back_fee'];
        $payoutInvoice->total_non_domestic_transactions = $report['total_non_domestic_transactions'];
        $payoutInvoice->total_non_domestic_fee = $report['total_non_domestic_fee'];
        $payoutInvoice->total_refund_transactions = $report['total_refund_transactions'];
        $payoutInvoice->total_service_charge = $report['total_service_charge'];
        $payoutInvoice->transaction_type = $report['transaction_type'];
        if ($payoutInvoice->transaction_type == 'Archived') {
            $payoutInvoice->payout_mode = $reseller->archived_payout_mode;
            $payoutInvoice->payout_method = $reseller->archived_payout_method;
        } else {
            $payoutInvoice->payout_mode = $reseller->payout_mode;
            $payoutInvoice->payout_method = $reseller->payout_method;
        }
        $payoutInvoice->from_date = $report['from_date'];
        $payoutInvoice->to_date = $report['to_date'];
        $payoutInvoice->payout_account = $report['payout_account'];
        $payoutInvoice->payout_invoice_raw = $report;
        $payoutInvoice->save();

        $savedTransactionIds = [];
        foreach ($report['transactions'] as $transaction) {
            $invoiceTransaction = ResellerInvoiceTransaction::where('reseller_payout_invoice_id', $payoutInvoice->id)->where('order_id', $transaction['order_id'])->first();
            if (empty($invoiceTransaction))
                $invoiceTransaction = new ResellerInvoiceTransaction();
            $invoiceTransaction->fill($transaction);
            $invoiceTransaction->reseller_payout_invoice_id = $payoutInvoice->id;
            $invoiceTransaction->save();
            $savedTransactionIds[] = $invoiceTransaction->id;
        }
        # Delete old transactions if new transactions are not matching with it.
        if ($shouldDeleteOldTransactions) {
            InvoiceTransaction::where('payout_invoice_id', $payoutInvoice->id)->whereNotIn('id', $savedTransactionIds)->delete();
        }
        #Check if group for given from and to date is already there or not
        $resellerPayoutInvoiceGroup = ResellerPayoutInvoiceGroup::where('from_date', Carbon::parse($payoutInvoice->from_date)->toDateString())->where('to_date', Carbon::parse($payoutInvoice->to_date)->toDateString())->where('user_id', $payoutInvoice->user_id)->first();
        if (empty($resellerPayoutInvoiceGroup)) {
            $resellerPayoutInvoiceGroup = new ResellerPayoutInvoiceGroup;
            $resellerPayoutInvoiceGroup->from_date = $payoutInvoice->from_date;
            $resellerPayoutInvoiceGroup->to_date = $payoutInvoice->to_date;
            $resellerPayoutInvoiceGroup->user_id = $payoutInvoice->user_id;
            $resellerPayoutInvoiceGroup->save();
            $resellerPayoutInvoiceGroup->refresh();
        }

        # Save passbook entry for generated invoice
        $passbook = Passbook::where('entity_id', $resellerPayoutInvoiceGroup->id)->where('entity', 'reseller_payout_invoice_group')->where('user_id', $payoutInvoice->user_id)->first();
        if (empty($passbook)) {
            $passbook = new Passbook();
        }
        $passbook->user_id = $payoutInvoice->user_id;
        $passbook->amount = $resellerPayoutInvoiceGroup->total_amount;
        $passbook->entity_id = $resellerPayoutInvoiceGroup->id;
        $passbook->entity = 'reseller_payout_invoice_group';
        $passbook->date = Carbon::today();
        $passbook->from_date = $payoutInvoice->from_date;
        $passbook->to_date = $payoutInvoice->to_date;
        $passbook->description = 'Invoice for ' . Carbon::parse($payoutInvoice->from_date)->toDateString() . ' to ' . Carbon::parse($payoutInvoice->to_date)->toDateString();
        $passbook->save();
    }
    public function saveReportMultiple($all_report)
    {
        foreach ($all_report as $report) {
            $this->saveReport($report);
        }
    }

    /*
    |
    |   Util functions 
    |
    */
    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }
    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
    public function mergeAllTransactionsAndSort($data)
    {
        $gross_total = 0;
        $total_fee = 0;
        $net_total = 0;
        $total_charge_back_fee = 0;
        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_refund_transactions = 0;
        $total_service_charge = 0;
        $misc_charge = ($data['misc_charge']);
        $weekly_fee = ($data['weekly_fee']);
        $monthly_fee = ($data['monthly_fee']);
        $all_transactions = [];
        if (isset($data['cod_report'])) {
            $gross_total = $gross_total + $data['cod_report']['gross_total'];
            $total_fee = $total_fee + $data['cod_report']['total_fee'];
            $net_total = $net_total + $data['cod_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['cod_report']['total_service_charge'];
            $total_refund_transactions = $total_refund_transactions + $data['cod_report']['total_refund_transactions'];
            $temp_transactions = $data['cod_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['card_report'])) {
            $gross_total = $gross_total + $data['card_report']['gross_total'];
            $total_fee = $total_fee + $data['card_report']['total_fee'];
            $net_total = $net_total + $data['card_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['card_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['card_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['card_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['card_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['card_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['card_report']['total_refund_transactions'];
            $temp_transactions = $data['card_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['wallet_report'])) {
            $gross_total = $gross_total + $data['wallet_report']['gross_total'];
            $total_fee = $total_fee + $data['wallet_report']['total_fee'];
            $net_total = $net_total + $data['wallet_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['wallet_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['wallet_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['wallet_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['wallet_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['wallet_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['wallet_report']['total_refund_transactions'];
            $temp_transactions = $data['wallet_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['paypal_report'])) {
            $gross_total = $gross_total + $data['paypal_report']['gross_total'];
            $total_fee = $total_fee + $data['paypal_report']['total_fee'];
            $net_total = $net_total + $data['paypal_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['paypal_report']['total_service_charge'];
            $temp_transactions = $data['paypal_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['paypal_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['paypal_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['paypal_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['merchant_report'])) {
            $gross_total = $gross_total + $data['merchant_report']['gross_total'];
            $total_fee = $total_fee + $data['merchant_report']['total_fee'];
            $net_total = $net_total + $data['merchant_report']['net_total'];
            $total_charge_back_fee = $total_charge_back_fee + $data['merchant_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['merchant_report']['total_charge_back_transactions'];
            $temp_transactions = $data['merchant_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['merchant_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['merchant_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['merchant_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        usort($all_transactions, $this->date_compare('order_date'));
        return [
            'from_date' => $data['from_date'],
            'to_date' => $data['to_date'],
            'payout_mode' => $data['payout_mode'],
            'payout_method' => $data['payout_method'],
            'payout_account' => $data['payout_account'],
            'transaction_type' => $data['transaction_type'],
            'payout_comment' => $data['payout_comment'],
            'gross_total' => $this->convertToDecimal($gross_total),
            'gross_fee' => $this->convertToDecimal($total_fee),
            'weekly_fee' => $this->convertToDecimal($weekly_fee),
            'monthly_fee' => $this->convertToDecimal($monthly_fee),
            'misc_charge' => $this->convertToDecimal($misc_charge),
            'total_charge_back_fee' => $this->convertToDecimal($total_charge_back_fee),
            'total_charge_back_transactions' => $this->convertToDecimal($total_charge_back_transactions),
            'total_non_domestic_transactions' => $this->convertToDecimal($total_non_domestic_transactions),
            'total_non_domestic_fee' => $this->convertToDecimal($total_non_domestic_fee),
            'total_refund_transactions' => $total_refund_transactions,
            'total_service_charge' =>  $this->convertToDecimal($total_service_charge),
            'net_fee' => $this->convertToDecimal($total_fee - $misc_charge - $monthly_fee - $weekly_fee),
            'net_total' => $this->convertToDecimal($net_total + $misc_charge + $monthly_fee + $weekly_fee),
            'transactions' => $all_transactions,
            'business' => $data['business'],
            'reseller' => $data['reseller']
        ];
    }
    public function businessmergeAllTransactionsAndSort($data)
    {
        $gross_total = 0;
        $total_fee = 0;
        $net_total = 0;
        $total_charge_back_fee = 0;
        $total_charge_back_transactions = 0;
        $total_non_domestic_transactions = 0;
        $total_non_domestic_fee = 0;
        $total_refund_transactions = 0;
        $total_service_charge = 0;
        $misc_charge = ($data['misc_charge']);
        $weekly_fee = ($data['weekly_fee']);
        $monthly_fee = ($data['monthly_fee']);
        $all_transactions = [];
        if (isset($data['business_cod_report'])) {
            $gross_total = $gross_total + $data['business_cod_report']['gross_total'];
            $total_fee = $total_fee + $data['business_cod_report']['total_fee'];
            $net_total = $net_total + $data['business_cod_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['business_cod_report']['total_service_charge'];
            $total_refund_transactions = $total_refund_transactions + $data['business_cod_report']['total_refund_transactions'];
            $temp_transactions = $data['business_cod_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['business_card_report'])) {
            $gross_total = $gross_total + $data['business_card_report']['gross_total'];
            $total_fee = $total_fee + $data['business_card_report']['total_fee'];
            $net_total = $net_total + $data['business_card_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['business_card_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['business_card_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['business_card_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['business_card_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['business_card_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['business_card_report']['total_refund_transactions'];
            $temp_transactions = $data['business_card_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['business_wallet_report'])) {
            $gross_total = $gross_total + $data['business_wallet_report']['gross_total'];
            $total_fee = $total_fee + $data['business_wallet_report']['total_fee'];
            $net_total = $net_total + $data['business_wallet_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['business_wallet_report']['total_service_charge'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['business_wallet_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['business_wallet_report']['total_non_domestic_transactions'];
            $total_charge_back_fee = $total_charge_back_fee + $data['business_wallet_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['business_wallet_report']['total_charge_back_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['business_wallet_report']['total_refund_transactions'];
            $temp_transactions = $data['business_wallet_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['business_paypal_report'])) {
            $gross_total = $gross_total + $data['business_paypal_report']['gross_total'];
            $total_fee = $total_fee + $data['business_paypal_report']['total_fee'];
            $net_total = $net_total + $data['business_paypal_report']['net_total'];
            $total_service_charge = $total_service_charge + $data['business_paypal_report']['total_service_charge'];
            $temp_transactions = $data['business_paypal_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['business_paypal_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['business_paypal_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['business_paypal_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['business_merchant_report'])) {
            $gross_total = $gross_total + $data['business_merchant_report']['gross_total'];
            $total_fee = $total_fee + $data['business_merchant_report']['total_fee'];
            $net_total = $net_total + $data['business_merchant_report']['net_total'];
            $total_charge_back_fee = $total_charge_back_fee + $data['business_merchant_report']['total_charge_back_fee'];
            $total_charge_back_transactions = $total_charge_back_transactions + $data['business_merchant_report']['total_charge_back_transactions'];
            $temp_transactions = $data['business_merchant_report']['transactions'];
            $total_non_domestic_fee = $total_non_domestic_fee + $data['business_merchant_report']['total_non_domestic_fee'];
            $total_non_domestic_transactions = $total_non_domestic_transactions + $data['business_merchant_report']['total_non_domestic_transactions'];
            $total_refund_transactions = $total_refund_transactions + $data['business_merchant_report']['total_refund_transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }
        if (isset($data['business_online_refund_report'])) {
            // $gross_total = $gross_total + $data['business_online_refund_report']['gross_total'];
            // $total_fee = $total_fee + $data['business_online_refund_report']['total_fee'];
            // $net_total = $net_total + $data['business_online_refund_report']['net_total'];
            // $total_charge_back_fee = $total_charge_back_fee + $data['business_online_refund_report']['total_charge_back_fee'];
            // $total_charge_back_transactions = $total_charge_back_transactions + $data['business_online_refund_report']['total_charge_back_transactions'];
            // $total_non_domestic_fee = $total_non_domestic_fee + $data['business_online_refund_report']['total_non_domestic_fee'];
            // $total_non_domestic_transactions = $total_non_domestic_transactions + $data['business_online_refund_report']['total_non_domestic_transactions'];
            // $total_refund_transactions = $total_refund_transactions + $data['business_online_refund_report']['total_refund_transactions'];
            $temp_transactions = $data['business_online_refund_report']['transactions'];
            if (!empty($temp_transactions)) {
                $all_transactions = array_merge($all_transactions, $temp_transactions);
            }
        }

        usort($all_transactions, $this->date_compare('order_date'));

        $payable_amount = 0;
        $externalAccount = ExternalAccount::where('business_id', $data['business']['id'])->where('default', 1)->orderByRaw('id DESC')->first();
        if (!empty($externalAccount)) {
            // $stringValue = 'UBSIDI LIMITED,Tiffin Tom,TIFFINTOMUK LTD,Tiffin Tom Limited,Ubsidi Ltd,TIFFINTOM LIMITED,TiffinTomuk Ltd';
            $stringValue = (new SiteSettingController())->findByKey('bank_string')->value;
            if ($externalAccount->account_holder_name == '' || stripos($stringValue, $externalAccount->account_holder_name) !== false) {
                $payable_amount = 1;
            } else {
                $payable_amount = 0;
            }
        }

        return [
            'from_date' => $data['from_date'],
            'to_date' => $data['to_date'],
            'payout_mode' => $data['payout_mode'],
            'payout_method' => $data['payout_method'],
            'payout_account' => $data['payout_account'],
            'transaction_type' => $data['transaction_type'],
            'payout_comment' => $data['payout_comment'],
            'gross_total' => $this->convertToDecimal($gross_total),
            'gross_fee' => $this->convertToDecimal($total_fee),
            'misc_charge' => $this->convertToDecimal($misc_charge),
            'weekly_fee' => $this->convertToDecimal($weekly_fee),
            'monthly_fee' => $this->convertToDecimal($monthly_fee),
            'total_charge_back_fee' => $this->convertToDecimal($total_charge_back_fee),
            'total_charge_back_transactions' => $this->convertToDecimal($total_charge_back_transactions),
            'total_non_domestic_transactions' => $this->convertToDecimal($total_non_domestic_transactions),
            'total_non_domestic_fee' => $this->convertToDecimal($total_non_domestic_fee),
            'total_service_charge' =>  $this->convertToDecimal($total_service_charge),
            'total_refund_transactions' => $total_refund_transactions,
            'net_fee' => $this->convertToDecimal($total_fee + $misc_charge + $monthly_fee + $weekly_fee + $total_service_charge),
            'net_total' => $this->convertToDecimal($net_total - $misc_charge - $monthly_fee - $weekly_fee - $total_service_charge),
            'payable_amount' => $this->convertToDecimal($payable_amount),
            'transactions' => $all_transactions,
            'business' => $data['business']
        ];
    }
    public function date_compare($key)
    {
        return function ($a, $b) use ($key) {
            $t1 = strtotime($a[$key]);
            $t2 = strtotime($b[$key]);
            return $t2 - $t1;
        };
    }
}
