<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\Role;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class RoleController extends Controller
{
    /**
     * List roles.
     * GET /api/roles
     */
    public function index(Request $req)
    {
        $roles = Role::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $roles->whereRaw('(title LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('type', false)) {
            $roles->where('type', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $roles = $roles->get();
        } else {
            $roles = $roles->paginate($req->query('per_page', 20));
        }

        if ($roles->isEmpty()) Utils::error('No roles found!', 404);


        return ($roles);
    }

    /**
     * Create role.
     * POST /api/roles
     */
    public function create(Request $req)
    {
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'title' => 'required|string',
            'type' => 'in:admin,user',
            'permissions' => 'required',
        ]);

        // create role
        $role = new Role;
        $role->fill($req->all());
        if ($req->has('permissions')) $role->permissions = $req->input('permissions');
        $role->save();

        return $role;
    }

    /**
     * Show role.
     * GET /api/roles/{id}
     */
    public function show(Request $req, $id)
    {
        $role = Role::find($id);
        if (empty($role)) Utils::error('Role not found!', 404);
        return $role;
    }

    /**
     * Update role.
     * POST /api/roles/{id}
     */
    public function update(Request $req, $id)
    {
        $role = Role::find($id);
        if (empty($role)) Utils::error('Role not found!', 404);

        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'title' => 'string',
            'type' => 'in:admin,user',
            'permissions' => 'required',
        ]);

        // update role
        if ($req->has('title')) $role->title = $req->input('title');
        if ($req->has('type')) $role->type = $req->input('type');
        if ($req->has('permissions')) $role->permissions = $req->input('permissions');
        $role->save();

        return $role;
    }


    /**
     * Delete role by id.
     * DELETE /api/roles/{id}
     */
    public function delete(Request $req, $id)
    {
        $role = Role::find($id);
        if (empty($role)) Utils::error('Role not found!', 404);
        $role->delete();
        return $role;
    }
}
