<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\SMSLog;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class SMSLogController extends Controller
{
    /**
     * List sms-logs.
     * GET /api/sms-logs
     */
    public function index(Request $req)
    {
        $sms_logs = SMSLog::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $sms_logs->whereRaw('(event LIKE ? OR mobile_number LIKE ? OR message LIKE ? OR sms_package LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $sms_logs = $sms_logs->get();
        } else {
            $sms_logs = $sms_logs->paginate($req->query('per_page', 20));
        }

        if ($sms_logs->isEmpty()) Utils::error('No sms logs found!', 404);

        return ($sms_logs);
    }

    /**
     * Create sms-log.
     * POST /api/sms-logs
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'event' => 'required|string|max:255',
            'mobile_number' => 'required|string',
            'message' => 'required|string',
            'sms_package' => 'required|string',
        ]);

        // create sms_log
        $sms_log = new SMSLog;
        $sms_log->fill($req->all());
        $sms_log->save();

        return $sms_log;
    }

    /**
     * Show sms_log.
     * GET /api/sms_logs/{id}
     */
    public function show(Request $req, $id)
    {
        $sms_log = SMSLog::find($id);
        if (empty($sms_log)) Utils::error('SMS log not found!', 404);
        return $sms_log;
    }

    /**
     * Update sms_log.
     * POST /api/sms_logs/{id}
     */
    public function update(Request $req, $id)
    {
        $sms_log = SMSLog::find($id);
        if (empty($sms_log)) Utils::error('SMS log not found!', 404);

        $this->validate($req, [
            'event' => 'required|string|max:255',
            'mobile_number' => 'required|string',
            'message' => 'required|string',
            'sms_package' => 'required|string',
        ]);

        // update sms_log
        if ($req->has('event')) $sms_log->event = $req->input('event');
        if ($req->has('mobile_number')) $sms_log->mobile_number = $req->input('mobile_number');
        if ($req->has('message')) $sms_log->message = $req->input('message');
        if ($req->has('sms_package')) $sms_log->sms_package = $req->input('sms_package');

        $sms_log->save();

        return $sms_log;
    }


    /**
     * Delete sms_log by id.
     * DELETE /api/sms_logs/{id}
     */
    public function delete(Request $req, $id)
    {
        $sms_log = SMSLog::find($id);
        if (empty($sms_log)) Utils::error('SMS log not found!', 404);
        $sms_log->delete();
        return $sms_log;
    }

    /**
     * Delete All sms_log
     * GET All /api/sms-log/delete-all/
     */
    public function delete_previous(Request $req)
    {
        $to_date = Carbon::today()->subDays(90)->startOfDay();
        SMSLog::where('created', '<=', $to_date)->delete();
        return ['message' => 'sms logs rejected done!'];
    }
}
