<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\SecureIp;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class SecureIpController extends Controller
{
    /**
     * List secure-ips.
     * GET /api/secure-ips
     */
    public function index(Request $req)
    {
        $secure_ips = SecureIp::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $secure_ips->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $secure_ips = $secure_ips->get();
        } else {
            $secure_ips = $secure_ips->paginate($req->query('per_page', 20));
        }

        if ($secure_ips->isEmpty()) Utils::error('No ips found!', 404);


        return ($secure_ips);
    }

    /**
     * Create secure_ip.
     * POST /api/secure_ips
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'ip' => 'required|string|max:255|unique:secure_ips,ip',
        ]);

        // create secure-ip
        $secure_ip = new SecureIp;
        $secure_ip->fill($req->all());
        $secure_ip->save();

        return $secure_ip;
    }

    /**
     * Show secure-ip.
     * GET /api/secure-ips/{id}
     */
    public function show(Request $req, $id)
    {
        $secure_ip = SecureIp::find($id);
        if (empty($secure_ip)) Utils::error('Ip not found!', 404);
        return $secure_ip;
    }

    /**
     * Update secure-ip.
     * POST /api/secure-ips/{id}
     */
    public function update(Request $req, $id)
    {
        $secure_ip = SecureIp::find($id);
        if (empty($secure_ip)) Utils::error('Ip not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'ip' => 'required|string|max:255|unique:secure_ips,ip',
        ]);

        // update secure_ip
        if ($req->has('name')) $secure_ip->name = $req->input('name');
        if ($req->has('ip')) $secure_ip->ip = $req->input('ip');
        $secure_ip->save();

        return $secure_ip;
    }

    /**
     * Enable secure-ip.
     * POST /api/secure-ip/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $secure_ip = SecureIp::find($id);
        if (empty($secure_ip)) Utils::error('Ip not found!', 404);
        $secure_ip->disabled = 0;
        $secure_ip->save();
        return $secure_ip;
    }

    /**
     * Disable secure-ip.
     * POST /api/secure-ip/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $secure_ip = SecureIp::find($id);
        if (empty($secure_ip)) Utils::error('Ip not found!', 404);
        $secure_ip->disabled = 1;
        $secure_ip->save();
        return $secure_ip;
    }

    /**
     * Delete secure-ip by id.
     * DELETE /api/secure-ips/{id}
     */
    public function delete(Request $req, $id)
    {
        $secure_ip = SecureIp::find($id);
        if (empty($secure_ip)) Utils::error('Ip not found!', 404);
        $secure_ip->delete();
        return $secure_ip;
    }
}
