<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Epos\EposCustomerController;
use App\Http\Controllers\Epos\EposOrderController;
use App\Http\Controllers\Epos\EposOrderSplitController;
use App\Http\Controllers\Epos\EposReservationController;
use App\Http\Controllers\Epos\EposTableController;
use App\Http\Controllers\Epos\EposVoucherController;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

use function Psy\debug;

class SocketController extends Controller
{
    /**
     * List socket.
     * GET /api/socket
     */
    public function index(Request $req)
    {
        return;
    }

    /**
     * Create socket object.
     * POST /api/socket
     */
    public function create(Request $req)
    {
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
            if (empty($business)) Utils::error("Business not found", 400);
            $this->establishEposConnection($business);
        }

        // create socket object
        if ($req->input('socket_type') == 'order') {
            try {
                if ($req->has('id')) {
                    (new EposOrderController())->update($req, $req->input('id'));
                } else {
                    (new EposOrderController())->create($req);
                }
            } catch (Exception $e) {
                Log::debug($e->getMessage());
            }
        } else if ($req->input('socket_type') == 'table') {
            try {
                if ($req->has('id')) {
                    (new EposTableController())->update($req, $req->input('id'));
                }
            } catch (Exception $e) {
                Log::debug($e->getMessage());
            }
        } else if ($req->input('socket_type') == 'reservation') {
            try {
                if ($req->has('id')) {
                    (new EposReservationController())->update($req, $req->input('id'));
                } else {
                    (new EposReservationController())->create($req);
                }
            } catch (Exception $e) {
                Log::debug($e->getMessage());
            }
        } else if ($req->input('socket_type') == 'customer') {
            try {
                if ($req->has('id')) {
                    (new EposCustomerController())->update($req, $req->input('id'));
                } else {
                    (new EposCustomerController())->create($req);
                }
            } catch (Exception $e) {
                Log::debug($e->getMessage());
            }
        } else if ($req->input('socket_type') == 'voucher') {
            try {
                if ($req->has('id')) {
                    (new EposVoucherController())->update($req, $req->input('id'));
                } else {
                    (new EposVoucherController())->create($req);
                }
            } catch (Exception $e) {
                Log::debug($e->getMessage());
            }
        } else {
        }

        return $req;
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }
}
