<?php

namespace App\Http\Controllers;

use App\Business;
use App\BusinessCommission;
use App\CardBrand;
use App\CardBrandCommission;
use App\CardBrandCommissionHistory;
use App\Country;
use App\CountryCommission;
use App\CountryCommissionHistory;
use App\Exports\StripeTransactionExports;
use App\FetchLog;
use App\StripeTransaction;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Tiffintom\TiffintomOrderController;
use App\PayoutRequest;
use App\ModelsConnect\ExternalAccount;
use App\ResellerCommission;
use App\ResellerStripeCommission;
use App\SiteSetting;
use App\UserBusiness;
use Carbon\Carbon;
use Exception;
use Hamcrest\Util;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Maatwebsite\Excel\Facades\Excel;
use PDO;
use Stripe\Stripe;
use Stripe\StripeClient;
use Stripe\Webhook;

class StripeTransactionController extends Controller
{
    /**
     * List sripeTransactions.
     * GET /api/stripe-transactions
     */
    public function index(Request $req)
    {
        $sripeTransactions = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('created DESC, id DESC');

        if ($business_null = $req->query('business_null', false)) {
            $sripeTransactions->whereNull('business_id');
            $sripeTransactions->whereRaw('(reporting_category NOT LIKE ? OR reporting_category NOT LIKE ? OR reporting_category NOT LIKE ?)', ["%fee%", "%network_cost%", "%connect_reserved_funds%"]);
        } else {
            if ($reporting_type = $req->query('reporting_type', false)) {
            } else {
                // $sripeTransactions->whereNotNull('business_id');
            }
        }

        // if ($reporting_type = $req->query('reporting_type', false)) {
        //     $sripeTransactions->where('reporting_category', 'payout');
        // } else {
        //     $sripeTransactions->where('reporting_category', '!=', 'payout');
        // }

        if ($query = $req->query('query', false)) {
            if (is_numeric($query)) {
                $query = $query * 100;
            }
            $sripeTransactions->whereRaw('(businesses.name LIKE ? OR description LIKE ? OR amount LIKE ? OR txn_id LIKE ? OR stripe_transactions.type LIKE ? OR reporting_category LIKE ? OR payment_intent_id LIKE ? OR card_brand LIKE ? OR card_country LIKE ? OR card_type LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($service = $req->query('service', false)) {
            $sripeTransactions->where('service', $service);
        }
        if ($query = $req->query('business_id', false)) {
            $sripeTransactions->where('business_id', $query);
        }
        if ($query = $req->query('type', false)) {
            $sripeTransactions->where('type', $query);
        }
        //We will remove this query soon. TOBE Deprecated
        if ($query = $req->query('transaction_status', false)) {
            if ($query == 'archived') {
                $sripeTransactions->where('archived', '1');
            }
            if ($query == 'unarchived') {
                $sripeTransactions->where('archived', '0');
            }
        }
        // After removing above query this will be usable
        if ($query = $req->query('archive_status', false)) {
            if ($query == 'archived') {
                $sripeTransactions->where('archived', '1');
            }
            if ($query == 'unarchived') {
                $sripeTransactions->where('archived', '0');
            }
        }

        if ($tab = $req->query('tab', false)) {

            if ($tab == 'stripe_transaction') {
                $sripeTransactions->whereNotNull('business_id');
            }
            if ($tab == 'other_transaction') {
                $sripeTransactions->whereRaw('(reporting_category LIKE ?)', ["%charge%"]);
                $sripeTransactions->whereNull('business_id');
            }
            if ($tab == 'dispute_transaction') {
                $sripeTransactions->whereRaw('(reporting_category LIKE ? OR reporting_category LIKE ?)', ["%dispute%", "%dispute_reversal%"]);
            }
            if ($tab == 'payout_transaction') {
                $sripeTransactions->whereRaw('(reporting_category LIKE ?)', ["%payout%"]);
            }
            if ($tab == 'platform_earning') {
                $sripeTransactions->whereRaw('(reporting_category LIKE ?)', ["%platform_earning%"]);
            }
            if ($tab == 'connect_charge') {
                $sripeTransactions->whereRaw('(reporting_category LIKE ? OR reporting_category LIKE ?)', ["%fee%", "%network_cost%"]);
            }
            if ($tab == 'connect_payout') {
                $sripeTransactions->whereRaw('(reporting_category LIKE ? OR reporting_category LIKE ? OR reporting_category LIKE ?)', ["%transfer%", "%transfer_reversal%", "%connect_reserved_funds%"]);
            }
        }

        if ($query = $req->query('charge', false)) {
            $sripeTransactions->whereRaw('(reporting_category LIKE ? OR reporting_category LIKE ? OR reporting_category LIKE ?)', ["%charge%", "%refund%", "%failed%"]);
        }
        if ($query = $req->query('connect_payout', false)) {
            $sripeTransactions->whereRaw('(reporting_category LIKE ? OR reporting_category LIKE ?)', ["%transfer%", "%transfer_reversal%"]);
        }
        if ($query = $req->query('connect_charge', false)) {
            $sripeTransactions->whereRaw('(reporting_category LIKE ? OR reporting_category LIKE ? OR reporting_category LIKE ?)', ["%fee%", "%network_cost%", "%connect_reserved_funds%"]);
        }
        if ($query = $req->query('status', false)) {
            $sripeTransactions->where('reporting_category', $query);
        }
        if ($amount_filter = $req->query('amount_filter_type', false)) {
            if ($req->has('amount')) {
                $amount = $req->query('amount');
                if ($amount_filter == 'greater_than') {
                    $sripeTransactions->where('amount', '>', $amount * 100);
                } else if ($amount_filter == 'less_than') {
                    $sripeTransactions->where('amount', '<', $amount * 100);
                } else if ($amount_filter == 'between') {
                    $amounts = explode(",", $amount);
                    $sripeTransactions->where('amount', '>',  $amounts[0] * 100);
                    $sripeTransactions->where('amount', '<',  $amounts[1] * 100);
                } else if ($amount_filter == 'equal') {
                    $sripeTransactions->where('amount', '=', $amount * 100);
                }
            } else {
                Utils::error("Amount field required with type", 400);
            }
        }
        if ($date_filter = $req->query('date_filter_type', false)) {
            if ($date = $req->query('date', false)) {
                if ($date_filter == 'in_last') {
                    $today = Carbon::today()->endOfDay();
                    $lastDay = Carbon::today()->subDays($date)->startOfDay();
                    $sripeTransactions->where('created', '>', $lastDay->getTimestamp());
                    $sripeTransactions->where('created', '<', $today->getTimestamp());
                } else if ($date_filter == 'before') {
                    $date = Carbon::parse($date)->startOfDay();
                    $sripeTransactions->where('created', '<', $date->getTimestamp());
                } else if ($date_filter == 'after') {
                    $date = Carbon::parse($date)->endOfDay();
                    $sripeTransactions->where('created', '>', $date->getTimestamp());
                } else if ($date_filter == 'between') {
                    $date = $req->query('date');
                    $dates = explode(",", $date);
                    $dates[0] = Carbon::parse($dates[0])->startOfDay();
                    $dates[1] = Carbon::parse($dates[1])->endOfDay();
                    $sripeTransactions->where('created', '>',  $dates[0]->getTimestamp());
                    $sripeTransactions->where('created', '<',  $dates[1]->getTimestamp());
                } else if ($date_filter == 'equal') {
                    $startTime = Carbon::parse($date)->startOfDay();
                    $endTime = Carbon::parse($date)->endOfDay();
                    $sripeTransactions->where('created', '>',  $startTime->getTimestamp());
                    $sripeTransactions->where('created', '<',  $endTime->getTimestamp());
                }
            } else {
                Utils::error("Date field required with type", 400);
            }
        }
        if ($req->has('gateway_mode_test')) {
            $sripeTransactions->where('is_gateway_mode_test', (bool)$req->gateway_mode_test);
        }
        if ($query = $req->query('format', false)) {
            $sripeTransactions = $sripeTransactions->get();
            if ($sripeTransactions->isEmpty()) Utils::error('No sripeTransactions found to export!', 404);
            $sripeTransactions->each(function ($transaction) {
                $transaction->append('calculated_fees');
            });

            if ($query == 'excel') {
                return Excel::download(new StripeTransactionExports($sripeTransactions), "Credit Card Transactions.xls", \Maatwebsite\Excel\Excel::XLS);
            }
            if ($query == 'pdf') {
                return Excel::download(new StripeTransactionExports($sripeTransactions), "Credit Card Transactions.pdf", \Maatwebsite\Excel\Excel::MPDF);
            }
            Utils::error('Invalid format!', 404);
        } else {

            if ($req->query('nopaginate', false) == '1') {
                $sripeTransactions = $sripeTransactions->get();
            } else {
                $sripeTransactions = $sripeTransactions->paginate($req->query('per_page', 20));
            }

            if ($sripeTransactions->isEmpty()) Utils::error('No sripeTransactions found!', 404);
        }

        $sripeTransactions->each(function ($transaction) {
            $transaction->append('calculated_fees');
        });
        return ($sripeTransactions);
    }

    /**
     * Create Stripe Transaction.
     * POST /api/stripe-transactions
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'txn_id' => 'required',
            'business_id' => 'required|exists:businesses,id',
            'cash_handling_fee_type' => 'required|in:flat,percentage',
            'cash_handling_fee' => 'required|numeric|min:0',
            'cash_handling_per_amount' => 'required',
            'management_fee_type' => 'required|in:flat,percentage',
            'management_fee' => 'required|numeric|min:0',
            'merchant_card_processing_fee_type' => 'required|in:flat,percentage',
            'merchant_card_processing_fee' => 'required|numeric|min:0',
            'merchant_card_auth_fee_type' => 'required|in:flat,percentage',
            'merchant_card_auth_fee' => 'required|numeric|min:0',
            'charge_back_fee_type' => 'required|in:flat,percentage',
            'charge_back_fee' => 'required|numeric|min:0',
            'charges_refundable' => 'required|in:no,yes',
        ]);
        $business = Business::find($req->input('business_id'));
        $businessCommission = BusinessCommission::where('business_id', $business->id)->first();

        if ($business->connect_service == 1) {
            $connectSiteSettingMode = SiteSettingController::findByKey('connect_stripe_mode');
            $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_live');
            if (!empty($connectSiteSettingMode)) {
                $mode = $connectSiteSettingMode->value;
            }
            if (!empty($connectSiteSettingsecret_key) && $mode == 'live') {
                $key = $connectSiteSettingsecret_key->value;
            } else {
                $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_test');
                $key = $connectSiteSettingsecret_key->value;
            }
        } else {
            $siteSettingMode = SiteSettingController::findByKey('stripe_mode');
            $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
            if (!empty($siteSettingMode)) {
                $mode = $siteSettingMode->value;
            }
            if (!empty($siteSettingsecret_key) && $mode == 'live') {
                $key = $siteSettingsecret_key->value;
            } else {
                $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                $key = $siteSettingsecret_key->value;
            }
        }

        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $txn_id = $req->input('txn_id');
        if (str_starts_with($txn_id, "pi_")) {
            try {
                $paymentIntent = $stripe->paymentIntents->retrieve($txn_id);
            } catch (Exception $e) {
                // when Payment-Intent Blank Then Get Normal Account
                $siteSettingMode = SiteSettingController::findByKey('stripe_mode');
                $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
                if (!empty($siteSettingMode)) {
                    $mode = $siteSettingMode->value;
                }
                if (!empty($siteSettingsecret_key) && $mode == 'live') {
                    $key = $siteSettingsecret_key->value;
                } else {
                    $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                    $key = $siteSettingsecret_key->value;
                }
                $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
                $paymentIntent = $stripe->paymentIntents->retrieve($txn_id);
            }
            $txn_id = $paymentIntent->charges->data[0]->balance_transaction;
        }
        if (str_starts_with($txn_id, "ch_")) {
            try {
                $charge = $stripe->charges->retrieve($txn_id);
            } catch (Exception $e) {
                // when Charge Blank Then Get Normal Account
                $siteSettingMode = SiteSettingController::findByKey('stripe_mode');
                $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
                if (!empty($siteSettingMode)) {
                    $mode = $siteSettingMode->value;
                }
                if (!empty($siteSettingsecret_key) && $mode == 'live') {
                    $key = $siteSettingsecret_key->value;
                } else {
                    $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                    $key = $siteSettingsecret_key->value;
                }
                $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
                $charge = $stripe->charges->retrieve($txn_id);
            }
            $txn_id = $charge->balance_transaction;
        }
        if (str_starts_with($txn_id, 'txn_')) {
            try {
                $trans = $stripe->balanceTransactions->retrieve($txn_id);
            } catch (Exception $e) {
                // when Balance Transaction Blank Then Get Normal Account
                $siteSettingMode = SiteSettingController::findByKey('stripe_mode');
                $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
                if (!empty($siteSettingMode)) {
                    $mode = $siteSettingMode->value;
                }
                if (!empty($siteSettingsecret_key) && $mode == 'live') {
                    $key = $siteSettingsecret_key->value;
                } else {
                    $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                    $key = $siteSettingsecret_key->value;
                }
                $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
                $trans = $stripe->balanceTransactions->retrieve($txn_id);
            }
        } else {
            Utils::error("Invalid Trnasaction ID", 400);
        }

        $chargeDetails = null;
        if ($trans->type == 'charge') {
            $chargeDetails = $stripe->charges->retrieve(
                $trans->source,
                []
            );
        }
        $stripeTransaction = StripeTransaction::where('txn_id', $trans->id)->first();
        if (!empty($stripeTransaction)) {
            Utils::error("Transaction already exists", 400);
            return $stripeTransaction;
        }

        $stripeTransaction = new StripeTransaction();
        $stripeTransaction->txn_id = $trans->id;
        $stripeTransaction->business_id = $business->id;

        if ($trans->reporting_category == 'charge') {
            $chargeDetails = $stripe->charges->retrieve($trans->source, []);
            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
        }
        if ($trans->reporting_category == 'refund') {
            $refundDetails = $stripe->refunds->retrieve($trans->source, []);
            $stripeTransaction->payment_intent_id = $refundDetails->payment_intent;
            $chargeDetails = $stripe->charges->retrieve($refundDetails->charge, []);
            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
        }
        if ($trans->reporting_category == 'transfer') {
            $transferDetails = $stripe->transfers->retrieve($trans->source, []);
            if ($transferDetails->source_transaction != null) {
                $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
            } else {
                $stripeTransaction->descriptor = $transferDetails->destination;
            }
        }
        if ($trans->reporting_category == 'transfer_reversal') {
            $transferDetails = $stripe->transfers->retrieve($trans->source, []);
            if ($transferDetails->source_transaction != null) {
                $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                $dbStripeTransactions = StripeTransaction::where('source', $transferDetails->source_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded]);
            } else {
                $stripeTransaction->descriptor = $transferDetails->destination;
            }
        }
        if ($trans->reporting_category == 'platform_earning') {
            $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
            $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
        }
        if ($trans->reporting_category == 'platform_earning_refund') {
            $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
            $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
            $dbStripeTransactions = StripeTransaction::where('source', $applicationFee->originating_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded]);
        }
        if ($trans->reporting_category == 'dispute') {
            $disputeDetails = $stripe->disputes->retrieve($trans->source, []);
            $chargeDetails = $stripe->charges->retrieve($disputeDetails->charge, []);
            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
        }
        if (!empty($chargeDetails)) {
            if ($chargeDetails->customer != null) {
                $customerDetails = $stripe->customers->retrieve($chargeDetails->customer, []);
                $stripeTransaction->email = $customerDetails->email;
            }
            try {
                if ($chargeDetails->payment_method_details->type != null && $chargeDetails->payment_method_details->type == 'card_present') {
                    $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                    $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                    $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                    $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                } else {
                    $stripeTransaction->card_number = $chargeDetails->payment_method_details->card->last4;
                    $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card->brand;
                    $stripeTransaction->card_type = $chargeDetails->payment_method_details->card->funding;
                    $stripeTransaction->card_country = $chargeDetails->payment_method_details->card->country;
                }
            } catch (Exception $e) {
            }
            $dbStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded]);
        }
        $stripeTransaction->object = $trans->object;
        $stripeTransaction->description = $trans->description;
        $stripeTransaction->amount = $trans->amount;
        $stripeTransaction->fee = $trans->fee;
        $stripeTransaction->net = $trans->net;
        $stripeTransaction->currency = $trans->currency;
        $stripeTransaction->reporting_category = $trans->reporting_category;
        $stripeTransaction->source = $trans->source;
        $stripeTransaction->status = $trans->status;
        $stripeTransaction->type = $trans->type;
        $stripeTransaction->mode = $mode;
        $stripeTransaction->available_on = $trans->available_on;
        $stripeTransaction->created = $trans->created;
        if (isset($chargeDetails->metadata)) {
            $stripeTransaction->metadata = $chargeDetails->metadata;
        }
        $stripeTransaction->service = 'Other';


        $stripeTransaction->cash_handling_fee_type = $req->input('cash_handling_fee_type');
        $stripeTransaction->cash_handling_fee = $req->input('cash_handling_fee');
        $stripeTransaction->cash_handling_per_amount = $req->input('cash_handling_per_amount');
        $stripeTransaction->management_fee_type = $req->input('management_fee_type');
        $stripeTransaction->management_fee = 0; //$req->input('management_fee');
        $stripeTransaction->merchant_card_processing_fee_type = $req->input('merchant_card_processing_fee_type');
        $stripeTransaction->merchant_card_processing_fee = $req->input('merchant_card_processing_fee');
        $stripeTransaction->merchant_card_auth_fee_type = $req->input('merchant_card_auth_fee_type');
        $stripeTransaction->merchant_card_auth_fee = $req->input('merchant_card_auth_fee');

        $stripeTransaction->charge_back_fee_type = $req->input('charge_back_fee_type');
        $stripeTransaction->charge_back_fee = $req->input('charge_back_fee');

        $stripeTransaction->connect_card_processing_fee_type = $businessCommission->connect_card_processing_fee_type;
        $stripeTransaction->connect_card_processing_fee = $businessCommission->connect_card_processing_fee;
        $stripeTransaction->connect_card_auth_fee_type = $businessCommission->connect_card_auth_fee_type;
        $stripeTransaction->connect_card_auth_fee = $businessCommission->connect_card_auth_fee;

        $stripeTransaction->charges_refundable = $req->input('charges_refundable');

        //Set transaction service type Like from which service this transaction comes
        if ($business->online_statement_descriptor != null && ($business->online_statement_descriptor == $stripeTransaction->descriptor || $business->online_statement_description == $stripeTransaction->description)) {
            $stripeTransaction->service = 'Online';
            $stripeTransaction->transfer_status = 'not_applicable';
        } else if ($business->s_account_id != null && $business->connect_service == 1) {
            $stripeTransaction->service = 'Connect';
            if ($stripeTransaction->refunded == 1) {
                $stripeTransaction->transfer_status = 'not_applicable';
            }
        } else if ($business->statement_descriptor != null && ($business->statement_descriptor == $stripeTransaction->descriptor || $business->statement_description == $stripeTransaction->description)) {
            $stripeTransaction->service = 'Merchant';
            $stripeTransaction->transfer_status = 'not_applicable';
        } else {
            $stripeTransaction->service = 'Other';
            $stripeTransaction->transfer_status = 'not_applicable';
        }

        $country = null;
        $countryCommission = null;
        $cardBrand = null;
        $cardBrandCommission = null;

        if (!empty($stripeTransaction->card_country)) {
            $country = Country::where('iso', $stripeTransaction->card_country)->first();
            if (!empty($country)) {
                $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                if ($country->id == $business->country_id) {
                    $stripeTransaction->non_domestic = 0;
                } else {
                    $stripeTransaction->non_domestic = 1;
                }
            }
        }
        if (empty($countryCommission)) {
            $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
            if (empty($country))
                $stripeTransaction->non_domestic = 1;
        }
        $stripeTransaction->save();

        if (!empty($countryCommission)) {
            $countryCommissionHistory = new CountryCommissionHistory();
            $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
            $countryCommissionHistory->stripe_percentage = $countryCommission->stripe_percentage;
            $countryCommissionHistory->stripe_fixed = $countryCommission->stripe_fixed;
            $countryCommissionHistory->save();
        } else {
            $countryCommissionHistory = new CountryCommissionHistory();
            $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
            $countryCommissionHistory->stripe_percentage = 0;
            $countryCommissionHistory->stripe_fixed = 0;
            $countryCommissionHistory->save();
        }

        // Save Reseller Commissions
        $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
        if (!empty($reseller)) {
            $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
            if (empty($resellerCommission)) {
                $resellerCommission = new ResellerCommission;
                $resellerCommission->user_id = $reseller->user_id;
                $resellerCommission->business_id = $business->id;
                $resellerCommission->save();
            }
            $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $stripeTransaction->id)->first();
            if (empty($resellerStripeCommission)) {
                $resellerStripeCommission = new ResellerStripeCommission();
                $resellerStripeCommission->user_id = $reseller->user_id;
                $resellerStripeCommission->stripe_transaction_id = $stripeTransaction->id;
                $resellerStripeCommission->fill(json_decode($resellerCommission, true));

                $resellerStripeCommission->save();
            }
        }


        $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$stripeTransaction->card_brand, $stripeTransaction->card_brand])->first();
        if (!empty($cardBrand)) {
            $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
        }
        if (empty($cardBrandCommission)) {
            $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
        }
        if (!empty($cardBrandCommission)) {
            $cardBrandCommissionHistory = new CardBrandCommissionHistory();
            $cardBrandCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
            $cardBrandCommissionHistory->stripe_percentage = $cardBrandCommission->stripe_percentage;
            $cardBrandCommissionHistory->stripe_fixed = $cardBrandCommission->stripe_fixed;
            $cardBrandCommissionHistory->save();
        }

        return $stripeTransaction;
    }

    public function handleWebhook(Request $req)
    {
        $siteSettingWebhookKey = SiteSettingController::findByKey('webhook_key');
        if (!empty($siteSettingWebhookKey)) {
            $key = $siteSettingWebhookKey->value;
        } else {
            $key = $siteSettingWebhookKey->value;
        }
        $payload = @file_get_contents('php://input');
        $sig_header = $req->header('Stripe-Signature');
        $event = null;
        try {
            $event = Webhook::constructEvent(
                $payload,
                $sig_header,
                $key,
                ''
            );
        } catch (\Stripe\Exception\SignatureVerificationException $e) {
            Log::debug('Webhook Error: ' . $e->getMessage());
        }

        // Handle specific webhook events
        switch ($event->type) {
            case 'payment_intent.succeeded':
                $intent = $event->data->object;
                if ($intent->statement_descriptor_suffix != '') {
                    $data = ((new TiffintomOrderController())->orderReplica($intent));
                }
                $this->WebHookData($req);
                break;
            case 'payment_intent.payment_failed':
                $intent = $event->data->object;
                if ($intent->statement_descriptor_suffix != '') {
                    $data = ((new TiffintomOrderController())->orderReplica($intent));
                }
                $this->WebHookData($req);
                break;
            default:
                // Log::debug('Received unknown event type');
                break;
        }
        return 'Webhook Handled';
    }

    public function WebHookData(Request $req)
    {
        $account_type = 'connect';
        $siteSettingMode = SiteSettingController::findByKey('connect_stripe_mode');
        $siteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_live');
        if (!empty($siteSettingMode)) {
            $mode = $siteSettingMode->value;
        }
        if (!empty($siteSettingsecret_key) && $mode == 'live') {
            $key = $siteSettingsecret_key->value;
        } else {
            $siteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_test');
            $key = $siteSettingsecret_key->value;
        }

        $lastTransaction = StripeTransaction::orderByRaw('created DESC, id DESC');
        // if ($req != null) {
        //     $lastTransaction->whereRaw('(descriptor = ? OR descriptor = ? )', [$req->statement_descriptor, $req->statement_descriptor]);
        // } else {
        //     // $mode_column = "stripe_mode";
        //     $mode_column = "connect_stripe_mode";
        //     $siteSettingMode = SiteSettingController::findByKey($mode_column);
        //     if (!empty($siteSettingMode)) {
        //         $mode = $siteSettingMode->value;
        //     }
        // }
        $lastTransaction->where('mode', $mode);
        $lastTransaction->where('account_type', $account_type);
        $lastTransaction->where('failed', '0');
        // $lastTransaction->whereNotNull('txn_id');
        $lastTransaction = $lastTransaction->first();

        $params = [];
        $params['limit'] = 5;
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        if (!empty($lastTransaction)) {
            $params['ending_before'] = $lastTransaction->txn_id;
            try {
                $transactionListes = $stripe->balanceTransactions->all($params);
            } catch (Exception $e) {
                $params = [];
                $params['limit'] = 5;
            }
        }
        try {
            $transactions = $stripe->balanceTransactions->all($params);
            foreach (array_reverse($transactions->data) as $trans) {
                $chargeDetails = null;
                $country = null;
                $countryCommission = null;
                $cardBrand = null;
                $cardBrandCommission = null;

                if ($trans->type == 'charge') {
                    $chargeDetails = $stripe->charges->retrieve(
                        $trans->source,
                        []
                    );
                }
                $stripeTransaction = StripeTransaction::where('txn_id', $trans->id)->first();
                if (!empty($stripeTransaction)) {
                    $stripeTransaction->mode = $mode;
                    $stripeTransaction->account_type = $account_type;
                    $stripeTransaction->save();
                    continue;
                }

                $stripeTransaction = new StripeTransaction();
                $stripeTransaction->txn_id = $trans->id;
                $stripeTransaction->mode = $mode;
                $stripeTransaction->service = 'Other';
                $stripeTransaction->account_type = $account_type;
                $stripeTransaction->transfer_status = 'pending';

                if ($trans->reporting_category == 'charge') {
                    $chargeDetails = $stripe->charges->retrieve($trans->source, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    if ($chargeDetails->statement_descriptor == null && $chargeDetails->source != null) {
                        // $stripeTransaction->descriptor = $chargeDetails->source->id;
                    }
                    $stripeTransaction->transfer_status = 'pending';
                }

                if ($trans->reporting_category == 'refund') {
                    $refundDetails = $stripe->refunds->retrieve($trans->source, []);
                    $stripeTransaction->payment_intent_id = $refundDetails->payment_intent;
                    $chargeDetails = $stripe->charges->retrieve($refundDetails->charge, []);

                    // if parent charge archive then auto archive refund transaction
                    $chargeStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->first();
                    if (!empty($chargeStripeTransactions)) {
                        $stripeTransaction->archived = $chargeStripeTransactions->archived;
                    }

                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    $stripeTransaction->transfer_status = 'not_applicable';
                    if ($chargeDetails->refunded && $chargeDetails->payment_intent != null) {
                        $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'not_applicable']);
                    }
                }
                if ($trans->reporting_category == 'transfer') {
                    $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                    if ($transferDetails->source_transaction != null) {
                        $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                        if ($chargeDetails->payment_intent != null) {
                            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                            $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'completed']);
                        }
                        $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    } else {
                        $stripeTransaction->descriptor = $transferDetails->destination;
                    }
                    $stripeTransaction->transfer_status = 'not_applicable';
                }
                if ($trans->reporting_category == 'transfer_reversal') {
                    $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                    if ($transferDetails->source_transaction != null) {
                        $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                        $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                        $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                        $dbStripeTransactions = StripeTransaction::where('source', $transferDetails->source_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'transfer_status' => 'reversed']);
                    } else {
                        $stripeTransaction->descriptor = $transferDetails->destination;
                    }
                    $stripeTransaction->transfer_status = 'not_applicable';
                }
                if ($trans->reporting_category == 'platform_earning') {
                    $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                    $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                }
                if ($trans->reporting_category == 'platform_earning_refund') {
                    $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                    $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    $dbStripeTransactions = StripeTransaction::where('source', $applicationFee->originating_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);
                }
                if ($trans->reporting_category == 'dispute') {
                    $disputeDetails = $stripe->disputes->retrieve($trans->source, []);
                    $stripeTransaction->dispute_status = $disputeDetails->status;
                    $chargeDetails = $stripe->charges->retrieve($disputeDetails->charge, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                }
                if ($trans->reporting_category == 'payout') {
                    $payoutDetails = $stripe->payouts->retrieve($trans->source, []);
                    $stripeTransaction->payout_status = $payoutDetails->status;
                }

                if (!empty($chargeDetails)) {
                    if ($chargeDetails->customer != null) {
                        $customerDetails = $stripe->customers->retrieve($chargeDetails->customer, []);
                        $stripeTransaction->email = $customerDetails->email;
                    }
                    // try {
                    if ($chargeDetails->payment_method_details->type == 'card_present') { // $chargeDetails->payment_method_details->type != null && 
                        $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                        $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                        $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                        $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                    } else {
                        if (isset($chargeDetails->payment_method_details->card)) {
                            $stripeTransaction->card_number = $chargeDetails->payment_method_details->card->last4;
                            $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card->brand;
                            $stripeTransaction->card_type = $chargeDetails->payment_method_details->card->funding;
                            $stripeTransaction->card_country = $chargeDetails->payment_method_details->card->country;
                        }
                    }
                    // } catch (Exception $e) {
                    // }
                    $dbStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);
                }

                $stripeTransaction->object = $trans->object;
                $stripeTransaction->description = $trans->description;
                $stripeTransaction->amount = $trans->amount;
                $stripeTransaction->fee = $trans->fee;
                $stripeTransaction->net = $trans->net;
                $stripeTransaction->currency = $trans->currency;
                $stripeTransaction->reporting_category = $trans->reporting_category;
                $stripeTransaction->source = $trans->source;
                $stripeTransaction->status = $trans->status;
                $stripeTransaction->type = $trans->type;
                $stripeTransaction->available_on = $trans->available_on;
                $stripeTransaction->created = $trans->created;
                if (isset($chargeDetails->metadata)) {
                    $stripeTransaction->metadata = $chargeDetails->metadata;
                }

                $searchableDescriptor = $stripeTransaction->descriptor;
                if (empty($stripeTransaction->descriptor)) {
                    $searchableDescriptor = $stripeTransaction->description;
                }

                $business = Business::whereRaw('(statement_descriptor = ? OR online_statement_descriptor = ? OR s_account_id = ?)', ["$searchableDescriptor", "$searchableDescriptor", "$searchableDescriptor"])->first();
                if (!empty($business)) {
                    $stripeTransaction->business_id = $business->id;

                    if ($business->auto_archive == 1) {
                        $stripeTransaction->archived = 1;
                    }
                    $businessCommission = BusinessCommission::where('business_id', $business->id)->first();
                    $stripeTransaction->cash_handling_fee_type = $businessCommission->cash_handling_fee_type;
                    $stripeTransaction->cash_handling_fee = $businessCommission->cash_handling_fee;
                    $stripeTransaction->cash_handling_per_amount = $businessCommission->cash_handling_per_amount;
                    $stripeTransaction->management_fee_type = $businessCommission->management_fee_type;
                    $stripeTransaction->management_fee = 0; //$businessCommission->management_fee;
                    $stripeTransaction->merchant_card_processing_fee_type = $businessCommission->merchant_card_processing_fee_type;
                    $stripeTransaction->merchant_card_processing_fee = $businessCommission->merchant_card_processing_fee;
                    $stripeTransaction->merchant_card_auth_fee_type = $businessCommission->merchant_card_auth_fee_type;
                    $stripeTransaction->merchant_card_auth_fee = $businessCommission->merchant_card_auth_fee;

                    $stripeTransaction->charge_back_fee_type = $businessCommission->charge_back_fee_type;
                    $stripeTransaction->charge_back_fee = $businessCommission->charge_back_fee;

                    $stripeTransaction->connect_card_processing_fee_type = $businessCommission->connect_card_processing_fee_type;
                    $stripeTransaction->connect_card_processing_fee = $businessCommission->connect_card_processing_fee;
                    $stripeTransaction->connect_card_auth_fee_type = $businessCommission->connect_card_auth_fee_type;
                    $stripeTransaction->connect_card_auth_fee = $businessCommission->connect_card_auth_fee;

                    $stripeTransaction->charges_refundable = $businessCommission->charges_refundable;

                    //Set transaction service type Like from which service this transaction comes
                    if ($business->online_statement_descriptor != null && ($business->online_statement_descriptor == $stripeTransaction->descriptor || $business->online_statement_description == $stripeTransaction->description)) {
                        $stripeTransaction->service = 'Online';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    } else if ($business->s_account_id != null && $business->connect_service == 1) {
                        $stripeTransaction->service = 'Connect';
                        if ($stripeTransaction->refunded == 1) {
                            $stripeTransaction->transfer_status = 'not_applicable';
                        }
                    } else if ($business->statement_descriptor != null && ($business->statement_descriptor == $stripeTransaction->descriptor || $business->statement_description == $stripeTransaction->description)) {
                        $stripeTransaction->service = 'Merchant';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    } else {
                        $stripeTransaction->service = 'Other';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    }

                    $country = null;
                    $countryCommission = null;
                    $cardBrand = null;
                    $cardBrandCommission = null;
                    if (!empty($stripeTransaction->card_country)) {
                        $country = Country::where('iso', $stripeTransaction->card_country)->first();
                        if (!empty($country)) {
                            $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                            if ($country->id == $business->country_id) {
                                $stripeTransaction->non_domestic = 0;
                            } else {
                                $stripeTransaction->non_domestic = 1;
                            }
                        }
                    }

                    if (empty($countryCommission)) {
                        $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                        if (empty($country))
                            $stripeTransaction->non_domestic = 1;
                    }

                    $stripeTransaction->save();

                    if (!empty($countryCommission)) {
                        $countryCommissionHistory = new CountryCommissionHistory();
                        $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $countryCommissionHistory->stripe_percentage = $countryCommission->stripe_percentage;
                        $countryCommissionHistory->stripe_fixed = $countryCommission->stripe_fixed;
                        $countryCommissionHistory->save();
                    } else {
                        $countryCommissionHistory = new CountryCommissionHistory();
                        $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $countryCommissionHistory->stripe_percentage = 0;
                        $countryCommissionHistory->stripe_fixed = 0;
                        $countryCommissionHistory->save();
                    }

                    // Save Reseller Commissions
                    $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                    if (!empty($reseller)) {
                        $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
                        if (empty($resellerCommission)) {
                            $resellerCommission = new ResellerCommission;
                            $resellerCommission->user_id = $reseller->user_id;
                            $resellerCommission->business_id = $business->id;
                            $resellerCommission->save();
                        }
                        $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $stripeTransaction->id)->first();
                        if (empty($resellerStripeCommission)) {
                            $resellerStripeCommission = new ResellerStripeCommission();
                            $resellerStripeCommission->user_id = $reseller->user_id;
                            $resellerStripeCommission->stripe_transaction_id = $stripeTransaction->id;
                            $resellerStripeCommission->fill(json_decode($resellerCommission, true));

                            $resellerStripeCommission->save();
                        }
                    }

                    $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$stripeTransaction->card_brand, $stripeTransaction->card_brand])->first();
                    if (!empty($cardBrand)) {
                        $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
                    }
                    if (empty($cardBrandCommission)) {
                        $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
                    }
                    if (!empty($cardBrandCommission)) {
                        $cardBrandCommissionHistory = new CardBrandCommissionHistory();
                        $cardBrandCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $cardBrandCommissionHistory->stripe_percentage = $cardBrandCommission->stripe_percentage;
                        $cardBrandCommissionHistory->stripe_fixed = $cardBrandCommission->stripe_fixed;
                        $cardBrandCommissionHistory->save();
                    }
                } else {
                    $stripeTransaction->save();
                }
            }
            return $transactions;
        } catch (Exception $e) {
            Log::debug($e);
            return null;
        }
    }


    /**
     * Show Stripe Transaction
     * GET api/stripe-transactions/retrive/{id}
     */
    public function showTransaction(Request $req, $id)
    {
        $stripeMode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($stripeMode == 'test') {
            $secret_key = SiteSettingController::findByKey('stripe_private_key_test')->value;
        } else {
            $secret_key = SiteSettingController::findByKey('stripe_private_key_live')->value;
        }
        $stripe = new \Stripe\StripeClient($secret_key);
        $transactions = $stripe->balanceTransactions->retrieve($id, []);
        return $transactions;
    }


    /**
     * Show Stripe Transaction
     * GET api/stripe-transactions/{txn_id}
     */
    public function transactionFetch(Request $req, $txn_id)
    {
        $stripeTransactions = StripeTransaction::where('txn_id', $txn_id)->first();
        if (empty($stripeTransactions)) Utils::error('Transaction not found!', 404);

        $business = Business::where('id', $stripeTransactions->business_id)->first();

        $data = [
            'stripeTransactions' => $stripeTransactions,
            'business' => $business,
        ];

        return view('invoice-send-recepit', $data);
    }
    /**
     * List sripeTransactions connect.
     * GET /api/stripe-transactions/connect
     */
    public function connect(Request $req)
    {
        $sripeTransactions = StripeTransaction::select("stripe_transactions.*", "businesses.name as business_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('created DESC, id DESC');

        if ($query = $req->query('query', false)) {
            if (is_numeric($query)) {
                $query = $query * 100;
            }
            $sripeTransactions->whereRaw('(businesses.name LIKE ? OR description LIKE ? OR amount LIKE ? OR txn_id LIKE ? OR stripe_transactions.type LIKE ? OR reporting_category LIKE ? OR payment_intent_id LIKE ? OR card_brand LIKE ? OR card_country LIKE ? OR card_type LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        $sripeTransactions->where('service', 'Connect');

        if ($query = $req->query('business_id', false)) {
            $sripeTransactions->where('business_id', $query);
        }
        if ($query = $req->query('type', false)) {
            $sripeTransactions->where('type', $query);
        }
        if ($query = $req->query('transaction_status', false)) {
            if ($query == 'archived') {
                $sripeTransactions->where('archived', '1');
            }
            if ($query == 'unarchived') {
                $sripeTransactions->where('archived', '0');
            }
        }
        if ($query = $req->query('status', false)) {
            if ($query == 'charge') {
                $sripeTransactions->whereIn('reporting_category', [
                    'transfer',  'platform_earning'
                ]);
            }
            if ($query == 'refund') {
                $sripeTransactions->whereIn('reporting_category', [
                    'transfer_reversal', 'platform_earning_refund'
                ]);
            }
        } else {
            // $sripeTransactions->whereIn('reporting_category', [
            //     'transfer', 'transfer_reversal', 'platform_earning', 'platform_earning_refund'
            // ]);
        }
        if ($amount_filter = $req->query('amount_filter_type', false)) {
            if ($req->has('amount')) {
                $amount = $req->query('amount');
                if ($amount_filter == 'greater_than') {
                    $sripeTransactions->where('amount', '>', $amount * 100);
                } else if ($amount_filter == 'less_than') {
                    $sripeTransactions->where('amount', '<', $amount * 100);
                } else if ($amount_filter == 'between') {
                    $amounts = explode(",", $amount);
                    $sripeTransactions->where('amount', '>',  $amounts[0] * 100);
                    $sripeTransactions->where('amount', '<',  $amounts[1] * 100);
                } else if ($amount_filter == 'equal') {
                    $sripeTransactions->where('amount', '=', $amount * 100);
                }
            } else {
                Utils::error("Amount field required with type", 400);
            }
        }
        if ($date_filter = $req->query('date_filter_type', false)) {
            if ($date = $req->query('date', false)) {
                if ($date_filter == 'in_last') {
                    $today = Carbon::today()->endOfDay();
                    $lastDay = Carbon::today()->subDays($date)->startOfDay();
                    $sripeTransactions->where('created', '>', $lastDay->getTimestamp());
                    $sripeTransactions->where('created', '<', $today->getTimestamp());
                } else if ($date_filter == 'before') {
                    $date = Carbon::parse($date)->startOfDay();
                    $sripeTransactions->where('created', '<', $date->getTimestamp());
                } else if ($date_filter == 'after') {
                    $date = Carbon::parse($date)->endOfDay();
                    $sripeTransactions->where('created', '>', $date->getTimestamp());
                } else if ($date_filter == 'between') {
                    $date = $req->query('date');
                    $dates = explode(",", $date);
                    $dates[0] = Carbon::parse($dates[0])->startOfDay();
                    $dates[1] = Carbon::parse($dates[1])->endOfDay();
                    $sripeTransactions->where('created', '>',  $dates[0]->getTimestamp());
                    $sripeTransactions->where('created', '<',  $dates[1]->getTimestamp());
                } else if ($date_filter == 'equal') {
                    $startTime = Carbon::parse($date)->startOfDay();
                    $endTime = Carbon::parse($date)->startOfDay();
                    $sripeTransactions->where('created', '>',  $startTime->getTimestamp());
                    $sripeTransactions->where('created', '<',  $endTime->getTimestamp());
                }
            } else {
                Utils::error("Date field required with type", 400);
            }
        }
        if ($query = $req->query('format', false)) {
            $sripeTransactions = $sripeTransactions->get();
            if ($sripeTransactions->isEmpty()) Utils::error('No sripeTransactions found to export!', 404);
            $sripeTransactions->each(function ($transaction) {
                $transaction->append('calculated_fees');
            });

            if ($query == 'excel') {
                return Excel::download(new StripeTransactionExports($sripeTransactions), "Credit Card Transactions.xls", \Maatwebsite\Excel\Excel::XLS);
            }
            if ($query == 'pdf') {
                return Excel::download(new StripeTransactionExports($sripeTransactions), "Credit Card Transactions.pdf", \Maatwebsite\Excel\Excel::MPDF);
            }
            Utils::error('Invalid format!', 404);
        } else {

            if ($req->query('nopaginate', false) == '1') {
                $sripeTransactions = $sripeTransactions->get();
            } else {
                $sripeTransactions = $sripeTransactions->paginate($req->query('per_page', 20));
            }

            if ($sripeTransactions->isEmpty()) Utils::error('No sripeTransactions found!', 404);
        }

        $sripeTransactions->each(function ($transaction) {
            $transaction->append('calculated_fees');
        });
        return ($sripeTransactions);
    }
    public function show(Request $req, $id)
    {
        $stripeTransaction = StripeTransaction::find($id);
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);
        $stripeTransaction->append('online_order');
        $stripeTransaction->append('calculated_fees');
        return $stripeTransaction;
    }
    /**
     * Refund Stripe Transaction.
     * POST /stripe-transactions/{id}/refund
     * @param int amount
     * @param string password
     */
    public function refund(Request $req, $id)
    {

        $stripeTransaction = StripeTransaction::find($id);
        if (empty($stripeTransaction)) Utils::error('Transaction not found!', 404);

        $this->validate($req, [
            'password' => 'required|string',
            'amount' => 'required|numeric|gt:0'  // Human Amount
        ]);
        if ($req->input('amount') * 100 != $stripeTransaction->amount) {
            // Utils::error('This transaction does not support partial refund!', 404);
        }
        if (Auth::isAdmin()) {
            if (!Hash::check($req->input('password'), Auth::admin()->password)) {
                Utils::error("Password is incorrect", 400);
            }
        }
        if (Auth::isUser()) {
            if (!Hash::check($req->input('password'), Auth::user()->password)) {
                Utils::error("Password is incorrect", 400);
            }
        }
        // if ($stripeTransaction->service != 'Merchant') {
        //     Utils::error("Only payment service transactions can be refunded", 400);
        // }
        if ($stripeTransaction->source == null) {
            Utils::error("This transaction has no source", 400);
        }
        if ($stripeTransaction->reporting_category != 'charge') {
            Utils::error("This transaction cannot be refund", 400);
        }
        // if ($stripeTransaction->archived == '1') {
        //     Utils::error("Archived transaction cannot be refund", 400);
        // }
        if ($stripeTransaction->refunded) {
            Utils::error("This transaction already refunded", 400);
        }
        if ($stripeTransaction->service == 'Connect' && $stripeTransaction->transfer_status != 'pending') {
            // Utils::error("Cannot refund this transaction as it may not available", 400);
        }
        $account_type = $stripeTransaction->account_type;
        $stripeMode = "test";
        $stripeMode = $stripeTransaction->mode;
        $stripeKey = "";

        if ($stripeMode == 'test') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        if ($account_type == 'normal') {
            $business = Business::find($stripeTransaction->business_id);
            if (!empty($business)) {
                $mode = $stripeTransaction->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);
        $chargeDetails = $stripe->charges->retrieve($stripeTransaction->source);
        $refundParams = [
            'charge' => $stripeTransaction->source,
            'amount' => $req->input('amount') * 100
        ];
        if ($chargeDetails->transfer_data != null) {
            $refundParams['reverse_transfer'] = true;
        }
        try {
            $stripe->refunds->create($refundParams);
            Log::error("Refund Complete");
        } catch (\Stripe\Exception\InvalidRequestException $e) {
            // Invalid parameters were supplied to Stripe's API
            Log::error("Invalid Request Exception");
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\CardException $e) {
            // Since it's a decline, \Stripe\Exception\CardException will be caught
            Log::error("Card Exception");
            Utils::error($e->getError()->message, 400);
        } catch (\Stripe\Exception\ApiErrorException $e) {
            // Display a very generic error to the user, and maybe send
            // yourself an email
            Log::error("Api Error Exception");
            Utils::error($e->getError()->message, 400);
        }
        $this->transaction($req);
        return $stripeTransaction;
    }

    public function transaction(Request $req)
    {
        $account_type = "normal";
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
            if (empty($business)) Utils::error("Business not found", 400);
            if ($business->s_account_id != null && ($business->connect_service == 1)) {
                $account_type = 'connect';
            }
            $mode = $business->stripe_mode;
            $secret_key = $business->stripe_private_key_test;
            if ($mode == 'live') {
                $secret_key = $business->stripe_private_key_live;
            }
            return $this->fetch(null, $secret_key, $mode, $account_type);
        } else {
            $siteSettingMode = SiteSettingController::findByKey('stripe_mode');
            $connectSiteSettingMode = SiteSettingController::findByKey('connect_stripe_mode');
            $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
            $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_live');
            if (!empty($siteSettingMode)) {
                $mode = $siteSettingMode->value;
            }
            if (!empty($connectSiteSettingMode)) {
                $connectMode = $connectSiteSettingMode->value;
            }
            if (!empty($siteSettingsecret_key) && $mode == 'live') {
                $secret_key = $siteSettingsecret_key->value;
            } else {
                $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                $secret_key = $siteSettingsecret_key->value;
            }

            $data[] = $this->fetch(null, $secret_key, $mode, 'normal');

            if (!empty($connectSiteSettingsecret_key) && $connectMode == 'live') {
                $secret_key = $connectSiteSettingsecret_key->value;
            } else {
                $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_test');
                $secret_key = $connectSiteSettingsecret_key->value;
            }
            $data[] = $this->fetch(null, $secret_key, $connectMode, 'connect');

            return $data;
        }
    }

    public function charge($param_business = null, $key, $mode = 'test', $account_type = 'normal')
    {
        if ($account_type == 'connect') {
            $normalCharge = FetchLog::where('event', Utils::$CONNECT_CHARGE)->latest()->first();
        } else {
            $normalCharge = FetchLog::where('event', Utils::$NORMAL_CHARGE)->latest()->first();
        }
        if (!empty($normalCharge)) {
            $createGt = $normalCharge->created_at;
        } else {
            $createGt = Carbon::today();
        }

        $paramCharges = [];
        $paramCharges['limit'] = 50;
        $createGt = Carbon::parse($createGt);
        $paramCharges['created[gt]'] =  $createGt->getTimestamp();
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        $charges =  $stripe->charges->all($paramCharges);
        if (!empty($charges)) {
            foreach ($charges as $charge) {
                if ($charge->status == 'failed') {

                    $stripeFailedTransaction = StripeTransaction::where('source', $charge->id)->first();
                    if (!empty($stripeFailedTransaction)) {
                        $stripeFailedTransaction->mode = $mode;
                        $stripeFailedTransaction->account_type = $account_type;
                        $stripeFailedTransaction->save();
                        continue;
                    }

                    $stripeFailedTransaction = new StripeTransaction();
                    $stripeFailedTransaction->txn_id = $charge->id ? $charge->id : '';
                    $stripeFailedTransaction->payment_intent_id = $charge->payment_intent;
                    $stripeFailedTransaction->object = $charge->object;
                    $stripeFailedTransaction->descriptor = $charge->statement_descriptor;
                    $stripeFailedTransaction->description = $charge->statement_descriptor_suffix;
                    $stripeFailedTransaction->amount = $charge->amount;
                    $stripeFailedTransaction->net = $charge->amount;
                    $stripeFailedTransaction->currency = $charge->currency;
                    $stripeFailedTransaction->reporting_category = 'failed';
                    $stripeFailedTransaction->source = $charge->id;
                    $stripeFailedTransaction->status = 'failed'; // Pending / Available
                    $stripeFailedTransaction->type = 'failed'; // charge / refund
                    if (isset($charge) && isset($charge->payment_method_details) && isset($charge->payment_method_details->card)) {
                        $stripeFailedTransaction->card_type = $charge->payment_method_details->card->funding;
                        $stripeFailedTransaction->card_number = $charge->payment_method_details->card->last4;
                        $stripeFailedTransaction->card_brand = $charge->payment_method_details->card->brand;
                        $stripeFailedTransaction->card_country = $charge->payment_method_details->card->country;
                    }
                    if (isset($charge)) {
                        $stripeFailedTransaction->available_on = $charge->created;
                    } else {
                        $stripeFailedTransaction->available_on = Carbon::now();
                    }
                    $stripeFailedTransaction->created = $charge->created;
                    $stripeFailedTransaction->mode = $mode;
                    $stripeFailedTransaction->account_type = $account_type;
                    $stripeFailedTransaction->failed = 1;
                    $stripeFailedTransaction->failure_message = $charge->failure_message;
                    $business = Business::whereRaw('(statement_descriptor = ? OR online_statement_descriptor = ? OR s_account_id = ?)', ["$charge->statement_descriptor", "$charge->statement_descriptor", "$charge->statement_descriptor"])->first();
                    if (!empty($business)) {
                        $stripeFailedTransaction->business_id = $business->id;
                        if ($business->auto_archive == 1) {
                            $stripeFailedTransaction->archived = 1;
                        }
                        if ($business->online_statement_descriptor != null && ($business->online_statement_descriptor == $charge->statement_descriptor || $business->online_statement_description == $charge->statement_descriptor)) {
                            $stripeFailedTransaction->service = 'Online';
                            $stripeFailedTransaction->transfer_status = 'not_applicable';
                        } else if ($business->s_account_id != null && $business->connect_service == 1) {
                            $stripeFailedTransaction->service = 'Connect';
                            if ($stripeFailedTransaction->refunded == 1) {
                                $stripeFailedTransaction->transfer_status = 'not_applicable';
                            }
                        } else if ($business->statement_descriptor != null && ($business->statement_descriptor == $stripeFailedTransaction->descriptor || $business->statement_description == $stripeFailedTransaction->description)) {
                            $stripeFailedTransaction->service = 'Merchant';
                            $stripeFailedTransaction->transfer_status = 'not_applicable';
                        } else {
                            $stripeFailedTransaction->service = 'Other';
                            $stripeFailedTransaction->transfer_status = 'not_applicable';
                        }
                    }
                    $stripeFailedTransaction->save();

                    $fetch_log = new FetchLog();
                    if ($account_type == 'connect') {
                        $fetch_log->event = Utils::$CONNECT_CHARGE;
                    } else {
                        $fetch_log->event = Utils::$NORMAL_CHARGE;
                    }
                    $fetch_log->save();
                }
            }
        }
    }

    public function fetch($param_business = null, $key, $mode = 'test', $account_type = 'normal')
    {
        $failedTransaction = $this->charge($param_business, $key, $mode, $account_type);

        $lastTransaction = StripeTransaction::orderByRaw('created DESC, id DESC');
        if ($param_business != null) {
            $lastTransaction->whereRaw('(descriptor = ? OR descriptor = ? )', [$param_business->online_statement_descriptor, $param_business->statement_descriptor]);
        } else {
            $mode_column = "stripe_mode";
            if ($account_type == 'connect') $mode_column = "connect_stripe_mode";
            $siteSettingMode = SiteSettingController::findByKey($mode_column);
            if (!empty($siteSettingMode)) {
                $mode = $siteSettingMode->value;
            }
        }
        $lastTransaction->where('mode', $mode);
        $lastTransaction->where('account_type', $account_type);
        $lastTransaction->where('failed', '0');
        // $lastTransaction->whereNotNull('txn_id');
        $lastTransaction = $lastTransaction->first();

        $params = [];
        $params['limit'] = 95;
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        if (!empty($lastTransaction)) {
            $params['ending_before'] = $lastTransaction->txn_id;
            try {
                $transactionListes = $stripe->balanceTransactions->all($params);
            } catch (Exception $e) {
                $params = [];
                $params['limit'] = 95;
            }
        }
        try {
            $transactions = $stripe->balanceTransactions->all($params);
            foreach (array_reverse($transactions->data) as $trans) {
                $chargeDetails = null;
                $country = null;
                $countryCommission = null;
                $cardBrand = null;
                $cardBrandCommission = null;

                if ($trans->type == 'charge') {
                    $chargeDetails = $stripe->charges->retrieve(
                        $trans->source,
                        []
                    );
                }
                $stripeTransaction = StripeTransaction::where('txn_id', $trans->id)->first();
                if (!empty($stripeTransaction)) {
                    $stripeTransaction->mode = $mode;
                    $stripeTransaction->account_type = $account_type;
                    $stripeTransaction->save();
                    continue;
                }

                $stripeTransaction = new StripeTransaction();
                $stripeTransaction->txn_id = $trans->id;
                $stripeTransaction->mode = $mode;
                $stripeTransaction->service = 'Other';
                $stripeTransaction->account_type = $account_type;
                $stripeTransaction->transfer_status = 'pending';

                if ($trans->reporting_category == 'charge') {
                    $chargeDetails = $stripe->charges->retrieve($trans->source, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    if ($chargeDetails->statement_descriptor == null && $chargeDetails->source != null) {
                        // $stripeTransaction->descriptor = $chargeDetails->source->id;
                    }
                    $stripeTransaction->transfer_status = 'pending';
                }

                if ($trans->reporting_category == 'refund') {
                    $refundDetails = $stripe->refunds->retrieve($trans->source, []);
                    $stripeTransaction->payment_intent_id = $refundDetails->payment_intent;
                    $chargeDetails = $stripe->charges->retrieve($refundDetails->charge, []);

                    // if parent charge archive then auto archive refund transaction
                    $chargeStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->first();
                    if (!empty($chargeStripeTransactions)) {
                        $stripeTransaction->archived = $chargeStripeTransactions->archived;
                    }

                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    $stripeTransaction->transfer_status = 'not_applicable';
                    if ($chargeDetails->refunded && $chargeDetails->payment_intent != null) {
                        $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'not_applicable']);
                    }
                }
                if ($trans->reporting_category == 'transfer') {
                    $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                    if ($transferDetails->source_transaction != null) {
                        $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                        if ($chargeDetails->payment_intent != null) {
                            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                            $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'completed']);
                        }
                        $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    } else {
                        $stripeTransaction->descriptor = $transferDetails->destination;
                    }
                    $stripeTransaction->transfer_status = 'not_applicable';
                }
                if ($trans->reporting_category == 'transfer_reversal') {
                    $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                    if ($transferDetails->source_transaction != null) {
                        $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                        $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                        $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                        $dbStripeTransactions = StripeTransaction::where('source', $transferDetails->source_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'transfer_status' => 'reversed']);
                    } else {
                        $stripeTransaction->descriptor = $transferDetails->destination;
                    }
                    $stripeTransaction->transfer_status = 'not_applicable';
                }
                if ($trans->reporting_category == 'platform_earning') {
                    $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                    $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                }
                if ($trans->reporting_category == 'platform_earning_refund') {
                    $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                    $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    $dbStripeTransactions = StripeTransaction::where('source', $applicationFee->originating_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);
                }
                if ($trans->reporting_category == 'dispute') {
                    $disputeDetails = $stripe->disputes->retrieve($trans->source, []);
                    $stripeTransaction->dispute_status = $disputeDetails->status;
                    $chargeDetails = $stripe->charges->retrieve($disputeDetails->charge, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                }
                if ($trans->reporting_category == 'payout') {
                    $payoutDetails = $stripe->payouts->retrieve($trans->source, []);
                    $stripeTransaction->payout_status = $payoutDetails->status;
                }

                if (!empty($chargeDetails)) {
                    if ($chargeDetails->customer != null) {
                        $customerDetails = $stripe->customers->retrieve($chargeDetails->customer, []);
                        $stripeTransaction->email = $customerDetails->email;
                    }
                    // try {
                    if ($chargeDetails->payment_method_details->type == 'card_present') { // $chargeDetails->payment_method_details->type != null && 
                        $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                        $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                        $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                        $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                    } else {
                        if (isset($chargeDetails->payment_method_details->card)) {
                            $stripeTransaction->card_number = $chargeDetails->payment_method_details->card->last4;
                            $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card->brand;
                            $stripeTransaction->card_type = $chargeDetails->payment_method_details->card->funding;
                            $stripeTransaction->card_country = $chargeDetails->payment_method_details->card->country;
                        }
                    }
                    // } catch (Exception $e) {
                    // }
                    $dbStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);

                    if (isset($chargeDetails->metadata->is_gateway_mode_test)) {
                        if ($chargeDetails->metadata->is_gateway_mode_test == 'true') {
                            $stripeTransaction->is_gateway_mode_test = 1;
                        } elseif ($chargeDetails->metadata->is_gateway_mode_test == 'false') {
                            $stripeTransaction->is_gateway_mode_test = 0;
                        }
                    }
                }

                $stripeTransaction->object = $trans->object;
                $stripeTransaction->description = $trans->description;
                $stripeTransaction->amount = $trans->amount;
                $stripeTransaction->fee = $trans->fee;
                $stripeTransaction->net = $trans->net;
                $stripeTransaction->currency = $trans->currency;
                $stripeTransaction->reporting_category = $trans->reporting_category;
                $stripeTransaction->source = $trans->source;
                $stripeTransaction->status = $trans->status;
                $stripeTransaction->type = $trans->type;
                $stripeTransaction->available_on = $trans->available_on;
                $stripeTransaction->created = $trans->created;
                if (isset($chargeDetails->metadata)) {
                    $stripeTransaction->metadata = $chargeDetails->metadata;
                }

                $searchableDescriptor = $stripeTransaction->descriptor;
                if (empty($stripeTransaction->descriptor)) {
                    $searchableDescriptor = $stripeTransaction->description;
                }

                $business = Business::whereRaw('(statement_descriptor = ? OR online_statement_descriptor = ? OR s_account_id = ?)', ["$searchableDescriptor", "$searchableDescriptor", "$searchableDescriptor"])->first();
                if (!empty($business)) {
                    $stripeTransaction->business_id = $business->id;

                    if ($business->auto_archive == 1) {
                        $stripeTransaction->archived = 1;
                    }
                    $businessCommission = BusinessCommission::where('business_id', $business->id)->first();
                    $stripeTransaction->cash_handling_fee_type = $businessCommission->cash_handling_fee_type;
                    $stripeTransaction->cash_handling_fee = $businessCommission->cash_handling_fee;
                    $stripeTransaction->cash_handling_per_amount = $businessCommission->cash_handling_per_amount;
                    $stripeTransaction->management_fee_type = $businessCommission->management_fee_type;
                    $stripeTransaction->management_fee = 0; //$businessCommission->management_fee;
                    $stripeTransaction->merchant_card_processing_fee_type = $businessCommission->merchant_card_processing_fee_type;
                    $stripeTransaction->merchant_card_processing_fee = $businessCommission->merchant_card_processing_fee;
                    $stripeTransaction->merchant_card_auth_fee_type = $businessCommission->merchant_card_auth_fee_type;
                    $stripeTransaction->merchant_card_auth_fee = $businessCommission->merchant_card_auth_fee;

                    $stripeTransaction->charge_back_fee_type = $businessCommission->charge_back_fee_type;
                    $stripeTransaction->charge_back_fee = $businessCommission->charge_back_fee;

                    $stripeTransaction->connect_card_processing_fee_type = $businessCommission->connect_card_processing_fee_type;
                    $stripeTransaction->connect_card_processing_fee = $businessCommission->connect_card_processing_fee;
                    $stripeTransaction->connect_card_auth_fee_type = $businessCommission->connect_card_auth_fee_type;
                    $stripeTransaction->connect_card_auth_fee = $businessCommission->connect_card_auth_fee;

                    $stripeTransaction->charges_refundable = $businessCommission->charges_refundable;

                    //Set transaction service type Like from which service this transaction comes
                    if ($business->online_statement_descriptor != null && ($business->online_statement_descriptor == $stripeTransaction->descriptor || $business->online_statement_description == $stripeTransaction->description)) {
                        $stripeTransaction->service = 'Online';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    } else if ($business->s_account_id != null && $business->connect_service == 1) {
                        $stripeTransaction->service = 'Connect';
                        if ($stripeTransaction->refunded == 1) {
                            $stripeTransaction->transfer_status = 'not_applicable';
                        }
                    } else if ($business->statement_descriptor != null && ($business->statement_descriptor == $stripeTransaction->descriptor || $business->statement_description == $stripeTransaction->description)) {
                        $stripeTransaction->service = 'Merchant';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    } else {
                        $stripeTransaction->service = 'Other';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    }

                    $country = null;
                    $countryCommission = null;
                    $cardBrand = null;
                    $cardBrandCommission = null;
                    if (!empty($stripeTransaction->card_country)) {
                        $country = Country::where('iso', $stripeTransaction->card_country)->first();
                        if (!empty($country)) {
                            $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                            if ($country->id == $business->country_id) {
                                $stripeTransaction->non_domestic = 0;
                            } else {
                                $stripeTransaction->non_domestic = 1;
                            }
                        }
                    }

                    if (empty($countryCommission)) {
                        $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                        if (empty($country))
                            $stripeTransaction->non_domestic = 1;
                    }

                    $stripeTransaction->save();

                    if (!empty($countryCommission)) {
                        $countryCommissionHistory = new CountryCommissionHistory();
                        $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $countryCommissionHistory->stripe_percentage = $countryCommission->stripe_percentage;
                        $countryCommissionHistory->stripe_fixed = $countryCommission->stripe_fixed;
                        $countryCommissionHistory->save();
                    } else {
                        $countryCommissionHistory = new CountryCommissionHistory();
                        $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $countryCommissionHistory->stripe_percentage = 0;
                        $countryCommissionHistory->stripe_fixed = 0;
                        $countryCommissionHistory->save();
                    }

                    // Save Reseller Commissions
                    $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                    if (!empty($reseller)) {
                        $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
                        if (empty($resellerCommission)) {
                            $resellerCommission = new ResellerCommission;
                            $resellerCommission->user_id = $reseller->user_id;
                            $resellerCommission->business_id = $business->id;
                            $resellerCommission->save();
                        }
                        $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $stripeTransaction->id)->first();
                        if (empty($resellerStripeCommission)) {
                            $resellerStripeCommission = new ResellerStripeCommission();
                            $resellerStripeCommission->user_id = $reseller->user_id;
                            $resellerStripeCommission->stripe_transaction_id = $stripeTransaction->id;
                            $resellerStripeCommission->fill(json_decode($resellerCommission, true));

                            $resellerStripeCommission->save();
                        }
                    }

                    $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$stripeTransaction->card_brand, $stripeTransaction->card_brand])->first();
                    if (!empty($cardBrand)) {
                        $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
                    }
                    if (empty($cardBrandCommission)) {
                        $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
                    }
                    if (!empty($cardBrandCommission)) {
                        $cardBrandCommissionHistory = new CardBrandCommissionHistory();
                        $cardBrandCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $cardBrandCommissionHistory->stripe_percentage = $cardBrandCommission->stripe_percentage;
                        $cardBrandCommissionHistory->stripe_fixed = $cardBrandCommission->stripe_fixed;
                        $cardBrandCommissionHistory->save();
                    }
                } else {
                    $stripeTransaction->save();
                }
            }

            $this->fetchReverse($param_business, $key, $mode, $account_type);
            return $transactions;
        } catch (Exception $e) {
            // $this->fetchReverse($param_business, $key, $mode, $account_type);
            return null;
        }
    }

    public function fetchReverse($param_business = null, $key, $mode = 'test', $account_type = 'normal')
    {
        $lastTransaction = StripeTransaction::orderByRaw('created DESC, id DESC');
        if ($param_business != null) {
            $lastTransaction->whereRaw('(descriptor = ? OR descriptor = ? )', [$param_business->online_statement_descriptor, $param_business->statement_descriptor]);
        } else {
            $mode_column = "connect_stripe_mode";
            if ($account_type == 'connect') $mode_column = "stripe_mode";
            $siteSettingMode = SiteSettingController::findByKey($mode_column);
            if (!empty($siteSettingMode)) {
                $mode = $siteSettingMode->value;
            }
        }
        $lastTransaction->where('mode', $mode);
        $lastTransaction->where('account_type', $account_type);
        $lastTransaction->where('failed', '0');
        // $lastTransaction->whereNotNull('txn_id');
        $lastTransaction = $lastTransaction->first();

        $params = [];
        if (!empty($lastTransaction)) {
            $params['ending_before'] = $lastTransaction->txn_id;
        }
        $params['limit'] = 50;
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        try {
            $transactions = $stripe->balanceTransactions->all($params);

            foreach (array_reverse($transactions->data) as $trans) {
                $chargeDetails = null;
                if ($trans->type == 'charge') {
                    $chargeDetails = $stripe->charges->retrieve(
                        $trans->source,
                        []
                    );
                }
                $stripeTransaction = StripeTransaction::where('txn_id', $trans->id)->first();
                if (!empty($stripeTransaction)) {
                    $stripeTransaction->mode = $mode;
                    $stripeTransaction->account_type = $account_type;
                    $stripeTransaction->save();
                    continue;
                }
                $stripeTransaction = new StripeTransaction();
                $stripeTransaction->txn_id = $trans->id;
                $stripeTransaction->mode = $mode;
                $stripeTransaction->service = 'Other';
                $stripeTransaction->account_type = $account_type;
                $stripeTransaction->transfer_status = 'pending';
                if ($trans->reporting_category == 'charge') {
                    $chargeDetails = $stripe->charges->retrieve($trans->source, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    if ($chargeDetails->statement_descriptor == null && $chargeDetails->source != null) {
                        // $stripeTransaction->descriptor = $chargeDetails->source->id;
                    }
                    $stripeTransaction->transfer_status = 'pending';
                }
                if ($trans->reporting_category == 'refund') {
                    $refundDetails = $stripe->refunds->retrieve($trans->source, []);
                    $stripeTransaction->payment_intent_id = $refundDetails->payment_intent;
                    $chargeDetails = $stripe->charges->retrieve($refundDetails->charge, []);

                    // if parent charge archive then auto archive refund transaction
                    $chargeStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->first();
                    if (!empty($chargeStripeTransactions)) {
                        $stripeTransaction->archived = $chargeStripeTransactions->archived;
                    }

                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    $stripeTransaction->transfer_status = 'not_applicable';
                    if ($chargeDetails->refunded)
                        $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'not_applicable']);
                }
                if ($trans->reporting_category == 'transfer') {
                    $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                    if ($transferDetails->source_transaction != null) {
                        $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                        $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                        $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                        $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'completed']);
                    } else {
                        $stripeTransaction->descriptor = $transferDetails->destination;
                    }
                    $stripeTransaction->transfer_status = 'not_applicable';
                }
                if ($trans->reporting_category == 'transfer_reversal') {
                    $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                    if ($transferDetails->source_transaction != null) {
                        $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                        $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                        $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                        $dbStripeTransactions = StripeTransaction::where('source', $transferDetails->source_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'transfer_status' => 'reversed']);
                    } else {
                        $stripeTransaction->descriptor = $transferDetails->destination;
                    }
                    $stripeTransaction->transfer_status = 'not_applicable';
                }
                if ($trans->reporting_category == 'platform_earning') {
                    $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                    $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                }
                if ($trans->reporting_category == 'platform_earning_refund') {
                    $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                    $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                    $dbStripeTransactions = StripeTransaction::where('source', $applicationFee->originating_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);
                }
                if ($trans->reporting_category == 'dispute') {
                    $disputeDetails = $stripe->disputes->retrieve($trans->source, []);
                    $stripeTransaction->dispute_status = $disputeDetails->status;
                    $chargeDetails = $stripe->charges->retrieve($disputeDetails->charge, []);
                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                    $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                }
                if ($trans->reporting_category == 'payout') {
                    $payoutDetails = $stripe->payouts->retrieve($trans->source, []);
                    $stripeTransaction->payout_status = $payoutDetails->status;
                }

                if (!empty($chargeDetails)) {
                    if ($chargeDetails->customer != null) {
                        $customerDetails = $stripe->customers->retrieve($chargeDetails->customer, []);
                        $stripeTransaction->email = $customerDetails->email;
                    }
                    try {
                        if ($chargeDetails->payment_method_details->type != null && $chargeDetails->payment_method_details->type == 'card_present') {
                            if (isset($chargeDetails->payment_method_details->card)) {
                                $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                                $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                                $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                                $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                            }
                            if (isset($chargeDetails->payment_method_details->card_present)) {
                                $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                                $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                                $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                                $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                            }
                        } else {
                            if (isset($chargeDetails->payment_method_details->card)) {
                                $stripeTransaction->card_number = $chargeDetails->payment_method_details->card->last4;
                                $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card->brand;
                                $stripeTransaction->card_type = $chargeDetails->payment_method_details->card->funding;
                                $stripeTransaction->card_country = $chargeDetails->payment_method_details->card->country;
                            }
                        }
                    } catch (Exception $e) {
                    }
                    $dbStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);
                }

                $stripeTransaction->object = $trans->object;
                $stripeTransaction->description = $trans->description;
                $stripeTransaction->amount = $trans->amount;
                $stripeTransaction->fee = $trans->fee;
                $stripeTransaction->net = $trans->net;
                $stripeTransaction->currency = $trans->currency;
                $stripeTransaction->reporting_category = $trans->reporting_category;
                $stripeTransaction->source = $trans->source;
                $stripeTransaction->status = $trans->status;
                $stripeTransaction->type = $trans->type;
                $stripeTransaction->available_on = $trans->available_on;
                $stripeTransaction->created = $trans->created;
                if (isset($chargeDetails->metadata)) {
                    $stripeTransaction->metadata = $chargeDetails->metadata;
                }

                $searchableDescriptor = $stripeTransaction->descriptor;
                if (empty($stripeTransaction->descriptor)) {
                    $searchableDescriptor = $stripeTransaction->description;
                }
                $business = Business::whereRaw('(statement_descriptor = ? OR online_statement_descriptor = ? OR s_account_id = ?)', ["$searchableDescriptor", "$searchableDescriptor", "$searchableDescriptor"])->first();
                if (!empty($business)) {
                    $stripeTransaction->business_id = $business->id;
                    if ($business->auto_archive == 1) {
                        $stripeTransaction->archived = 1;
                    }
                    $businessCommission = BusinessCommission::where('business_id', $business->id)->first();
                    $stripeTransaction->cash_handling_fee_type = $businessCommission->cash_handling_fee_type;
                    $stripeTransaction->cash_handling_fee = $businessCommission->cash_handling_fee;
                    $stripeTransaction->cash_handling_per_amount = $businessCommission->cash_handling_per_amount;
                    $stripeTransaction->management_fee_type = $businessCommission->management_fee_type;
                    $stripeTransaction->management_fee = 0; //$businessCommission->management_fee;
                    $stripeTransaction->merchant_card_processing_fee_type = $businessCommission->merchant_card_processing_fee_type;
                    $stripeTransaction->merchant_card_processing_fee = $businessCommission->merchant_card_processing_fee;
                    $stripeTransaction->merchant_card_auth_fee_type = $businessCommission->merchant_card_auth_fee_type;
                    $stripeTransaction->merchant_card_auth_fee = $businessCommission->merchant_card_auth_fee;

                    $stripeTransaction->charge_back_fee_type = $businessCommission->charge_back_fee_type;
                    $stripeTransaction->charge_back_fee = $businessCommission->charge_back_fee;

                    $stripeTransaction->connect_card_processing_fee_type = $businessCommission->connect_card_processing_fee_type;
                    $stripeTransaction->connect_card_processing_fee = $businessCommission->connect_card_processing_fee;
                    $stripeTransaction->connect_card_auth_fee_type = $businessCommission->connect_card_auth_fee_type;
                    $stripeTransaction->connect_card_auth_fee = $businessCommission->connect_card_auth_fee;

                    $stripeTransaction->charges_refundable = $businessCommission->charges_refundable;

                    //Set transaction service type Like from which service this transaction comes
                    if ($business->online_statement_descriptor != null && ($business->online_statement_descriptor == $stripeTransaction->descriptor || $business->online_statement_description == $stripeTransaction->description)) {
                        $stripeTransaction->service = 'Online';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    } else if ($business->s_account_id != null && $business->connect_service == 1) {
                        $stripeTransaction->service = 'Connect';
                        if ($stripeTransaction->refunded == 1) {
                            $stripeTransaction->transfer_status = 'not_applicable';
                        }
                    } else if ($business->statement_descriptor != null && ($business->statement_descriptor == $stripeTransaction->descriptor || $business->statement_description == $stripeTransaction->description)) {
                        $stripeTransaction->service = 'Merchant';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    } else {
                        $stripeTransaction->service = 'Other';
                        $stripeTransaction->transfer_status = 'not_applicable';
                    }

                    $country = null;
                    $countryCommission = null;
                    $cardBrand = null;
                    $cardBrandCommission = null;
                    if (!empty($stripeTransaction->card_country)) {
                        $country = Country::where('iso', $stripeTransaction->card_country)->first();
                        if (!empty($country)) {
                            $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                            if ($country->id == $business->country_id) {
                                $stripeTransaction->non_domestic = 0;
                            } else {
                                $stripeTransaction->non_domestic = 1;
                            }
                        }
                    }
                    if (empty($countryCommission)) {
                        $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                        if (empty($country))
                            $stripeTransaction->non_domestic = 1;
                    }
                    $stripeTransaction->save();

                    if (!empty($countryCommission)) {
                        $countryCommissionHistory = new CountryCommissionHistory();
                        $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $countryCommissionHistory->stripe_percentage = $countryCommission->stripe_percentage;
                        $countryCommissionHistory->stripe_fixed = $countryCommission->stripe_fixed;
                        $countryCommissionHistory->save();
                    } else {
                        $countryCommissionHistory = new CountryCommissionHistory();
                        $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $countryCommissionHistory->stripe_percentage = 0;
                        $countryCommissionHistory->stripe_fixed = 0;
                        $countryCommissionHistory->save();
                    }

                    // Save Reseller Commissions
                    $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                    if (!empty($reseller)) {
                        $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
                        if (empty($resellerCommission)) {
                            $resellerCommission = new ResellerCommission;
                            $resellerCommission->user_id = $reseller->user_id;
                            $resellerCommission->business_id = $business->id;
                            $resellerCommission->save();
                        }
                        $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $stripeTransaction->id)->first();
                        if (empty($resellerStripeCommission)) {
                            $resellerStripeCommission = new ResellerStripeCommission();
                            $resellerStripeCommission->user_id = $reseller->user_id;
                            $resellerStripeCommission->stripe_transaction_id = $stripeTransaction->id;
                            $resellerStripeCommission->fill(json_decode($resellerCommission, true));

                            $resellerStripeCommission->save();
                        }
                    }


                    $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$stripeTransaction->card_brand, $stripeTransaction->card_brand])->first();
                    if (!empty($cardBrand)) {
                        $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
                    }
                    if (empty($cardBrandCommission)) {
                        $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
                    }
                    if (!empty($cardBrandCommission)) {
                        $cardBrandCommissionHistory = new CardBrandCommissionHistory();
                        $cardBrandCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                        $cardBrandCommissionHistory->stripe_percentage = $cardBrandCommission->stripe_percentage;
                        $cardBrandCommissionHistory->stripe_fixed = $cardBrandCommission->stripe_fixed;
                        $cardBrandCommissionHistory->save();
                    }
                } else {
                    $stripeTransaction->save();
                }
            }
            return $transactions;
        } catch (Exception $e) {
            return null;
        }
    }

    public function transfer(Request $req)
    {
        Utils::error("This feature is discontinued", 400);
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'array',
            'from_date' => 'date',
            'to_date' => 'date',
        ]);
        $business = Business::find($req->input('business_id'));
        $stripeTransactions = StripeTransaction::where('service', 'Connect')->where('transfer_status', 'pending')->where('refunded', '0')->where('amount', '>', 'refunded_amount')->where('archived', '0');
        if ($req->has('ids') && !empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $stripeTransactions = $stripeTransactions->whereIn('id', $req->input('ids'))->get();
            if ($stripeTransactions->isEmpty()) Utils::error('No transactions can be transfer', 400);
            return $this->transferTransactions($stripeTransactions, $business);
        } else {
            $fromDate = Carbon::now()->subDays(10)->startOfDay();
            $toDate = Carbon::now()->subDays(4)->endOfDay();

            if ($req->has('from_date')) {
                $fromDate = Carbon::parse($req->input('from_date'))->subDays(10)->startOfDay();
            }
            if ($req->has('to_date')) {
                $toDate = Carbon::parse($req->input('to_date'))->subDays(10)->startOfDay();
            }

            $stripeTransactions->where('created', '>', $fromDate->timestamp);
            $stripeTransactions->where('created', '<', $toDate->timestamp);
            $stripeTransactions = $stripeTransactions->get();
            $stripeTransactions->limit(10);

            if ($stripeTransactions->isEmpty()) {
                Utils::error('No transactions can be transfer', 400);
            }
            return $this->transferTransactions($stripeTransactions, $business);
        }
    }

    public function transferTransactions($transactions, $business = null)
    {
        $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
        if ($mode == 'test') {
            $stripeKeyColoumns = 'connect_stripe_private_key_test';
        }
        if ($mode == 'live') {
            $stripeKeyColoumns = 'connect_stripe_private_key_live';
        }
        $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        foreach ($transactions as $transaction) {
            if ($transaction->archived == 1 || $transaction->service != 'Connect' || $transaction->transfer_status != 'pending' || $transaction->refunded == 1 || $transaction->amount <= $transaction->refunded_amount) {
                continue;
            }
            if ($business == null) {
                $business = Business::find($transaction->business_id);
            }
            if (empty($business) || $business->s_account_id == null) continue;
            $transaction->append('calculated_fees');
            $transferrableAmount = $transaction->calculated_fees['net_total'];
            if ($transferrableAmount > 0) {
                $charge = $stripe->charges->retrieve($transaction->source);
                $transferrableAmount = $transferrableAmount - $charge->amount_refunded;
                if ($transferrableAmount > 0) {
                    $transfer = $stripe->transfers->create([
                        'amount' => $transferrableAmount * 100,
                        "currency" => "gbp",
                        'description' => $business->statement_description,
                        'source_transaction' => $transaction->source,
                        'destination' => $business->s_account_id,
                        'metadata' => [
                            'total_fee' => $transaction->total_fee
                        ]
                    ]);
                    $transaction->transfer_status = 'completed';
                } else {
                    $transaction->transfer_status = 'not_applicable';
                    $transaction->refunded_amount = $charge->amount_refunded;
                    $transaction->refunded = $charge->refunded;
                }
            } else {
                $transaction->transfer_status = 'not_applicable';
            }
            $transaction->save();
        }
        return $transactions;
    }

    /**
     * Archive Bulk Transacitons.
     * POST stripe-transactions/archive
     */
    public function archive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->update(['archived' => '1']); // where('refunded_amount', '=', '0')->

            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Online')->update(['archived' => '1']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Merchant')->update(['archived' => '1']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Connect')->where('transfer_status', 'pending')->update(['archived' => '1']);
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->get();

            // When refund transaction archive then same charge archive
            foreach ($transactions as $tran) {
                $this->refundAction($tran, 1);
            }
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }

    /**
     * UnArchive Bulk Transacitons.
     * POST stripe-transactions/unarchive
     */
    public function unarchive(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->update(['archived' => '0']); //where('reporting_category', 'charge')->
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Online')->update(['archived' => '0']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Merchant')->update(['archived' => '0']);
            // $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->where('reporting_category', 'charge')->where('service', 'Connect')->where('transfer_status', 'pending')->update(['archived' => '0']);
            $transactions = StripeTransaction::whereIn('id', $req->input('ids'))->get();

            // When refund transaction Unarchive then same charge Unarchive
            foreach ($transactions as $tran) {
                $this->refundAction($tran, 0);
            }
        } else {
            Utils::error("No transactions found", 400);
        }
        return $transactions;
    }

    public function refundAction($tran, $archive)
    {
        // When refund transaction Archive / Unarchive then same charge Archive / Unarchive
        $account_type = $tran->account_type;
        $stripeMode = "test";
        $stripeMode = $tran->mode;
        $stripeKey = "";

        if ($stripeMode == 'test') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            if ($account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        if ($account_type == 'normal') {
            $business = Business::find($tran->business_id);
            if (!empty($business)) {
                $mode = $tran->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);

        // Archive Charge Transaction 
        if ($tran->refunded == 1) { // && $tran->refunded_amount == 0
            $chargeDetail = $stripe->charges->retrieve($tran->source, []);
            $chargetransaction = StripeTransaction::where('source', $chargeDetail->refunds->data[0]->id)->first();
            if (!empty($chargetransaction)) {
                $chargetransaction->archived = $archive;
                $chargetransaction->save();
            }
        }

        // Unarchive Refund Transaction 
        if ($tran->reporting_category == 'refund') { // refund
            $refundDetail = $stripe->refunds->retrieve($tran->source, []);

            $refundtransaction = StripeTransaction::where('source', $refundDetail->charge)->first();
            if (!empty($refundtransaction)) {
                $refundtransaction->archived = $archive;
                $refundtransaction->save();
            }
        }
    }

    public function link(Request $req)
    {
        if ($req->has('stripe_transaction_ids') && !is_array($req->input('stripe_transaction_ids')))
            $req->merge(['stripe_transaction_ids' => json_decode($req->input('stripe_transaction_ids'))]);

        $this->validate($req, [
            'business_id' => 'required',
            'stripe_transaction_id' => 'required_without:stripe_transaction_ids',
            'stripe_transaction_ids' => 'array|required_without:stripe_transaction_id'
        ]);
        if ($req->has('stripe_transaction_id')) {
            $this->link_single_business($req->input('business_id'), $req->input('stripe_transaction_id'));
        }
        if ($req->has('stripe_transaction_ids')) {
            foreach ($req->input('stripe_transaction_ids') as $stripe_transaction_id) {
                $this->link_single_business($req->input('business_id'), $stripe_transaction_id);
            }
        }
    }

    public function link_single_business($business_id, $stripe_transaction_id)
    {
        $stripeTransaction = StripeTransaction::find($stripe_transaction_id);
        if (empty($stripeTransaction)) Utils::error('No stripeTransaction found!', 404);

        $business = Business::find($business_id);
        if (empty($business)) Utils::error('No business found!', 404);

        $this->saveBusinessCommissionsWithTransaction($stripeTransaction, $business);
    }

    public function link_business(Request $req)
    {
        $stripeTransactions = StripeTransaction::whereNull('business_id')->limit(500)->get();
        $count = 0;
        $unAssigned = sizeof($stripeTransactions);
        foreach ($stripeTransactions as $stripeTransaction) {
            $searchableDescriptor = $stripeTransaction->descriptor;
            if (empty($stripeTransaction->descriptor)) {
                $searchableDescriptor = $stripeTransaction->description;
            }
            $business = Business::whereRaw('(statement_descriptor = ? OR online_statement_descriptor = ? OR s_account_id = ?)', ["$searchableDescriptor", "$searchableDescriptor", "$searchableDescriptor"])->first();

            if (!empty($business)) {
                $count = $count + 1;
                $this->saveBusinessCommissionsWithTransaction($stripeTransaction, $business);
            }
        }
        return ["message" => "Businesses assigned", "assigned" => $count, "total_unassigned" => $unAssigned];
    }

    public function saveBusinessCommissionsWithTransaction($stripeTransaction, $business)
    {

        $stripeTransaction->business_id = $business->id;
        $businessCommission = BusinessCommission::where('business_id', $business->id)->first();
        $stripeTransaction->cash_handling_fee_type = $businessCommission->cash_handling_fee_type;
        $stripeTransaction->cash_handling_fee = $businessCommission->cash_handling_fee;
        $stripeTransaction->cash_handling_per_amount = $businessCommission->cash_handling_per_amount;
        $stripeTransaction->management_fee_type = $businessCommission->management_fee_type;
        $stripeTransaction->management_fee = 0; //$businessCommission->management_fee;
        $stripeTransaction->merchant_card_processing_fee_type = $businessCommission->merchant_card_processing_fee_type;
        $stripeTransaction->merchant_card_processing_fee = $businessCommission->merchant_card_processing_fee;
        $stripeTransaction->merchant_card_auth_fee_type = $businessCommission->merchant_card_auth_fee_type;
        $stripeTransaction->merchant_card_auth_fee = $businessCommission->merchant_card_auth_fee;

        $stripeTransaction->charge_back_fee_type = $businessCommission->charge_back_fee_type;
        $stripeTransaction->charge_back_fee = $businessCommission->charge_back_fee;

        $stripeTransaction->connect_card_processing_fee_type = $businessCommission->connect_card_processing_fee_type;
        $stripeTransaction->connect_card_processing_fee = $businessCommission->connect_card_processing_fee;
        $stripeTransaction->connect_card_auth_fee_type = $businessCommission->connect_card_auth_fee_type;
        $stripeTransaction->connect_card_auth_fee = $businessCommission->connect_card_auth_fee;

        $stripeTransaction->charges_refundable = $businessCommission->charges_refundable;
        $stripeTransaction->account_type = 'Normal';
        //Set transaction service type Like from which service this transaction comes
        if ($business->online_statement_descriptor != null && ($business->online_statement_descriptor == $stripeTransaction->descriptor || $business->online_statement_description == $stripeTransaction->description)) {
            $stripeTransaction->service = 'Online';
        } else if ($business->s_account_id != null && $business->connect_service == 1) {
            $stripeTransaction->service = 'Connect';
            $stripeTransaction->account_type = 'Connect';
        } else if ($business->statement_descriptor != null && ($business->statement_descriptor == $stripeTransaction->descriptor || $business->statement_description == $stripeTransaction->description)) {
            $stripeTransaction->service = 'Merchant';
        } else {
            $stripeTransaction->service = 'Other';
        }

        $country = null;
        $countryCommission = null;
        $cardBrand = null;
        $cardBrandCommission = null;

        if (!empty($stripeTransaction->card_country)) {
            $country = Country::where('iso', $stripeTransaction->card_country)->first();
            if (!empty($country)) {
                $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                if ($country->id == $business->country_id) {
                    $stripeTransaction->non_domestic = 0;
                } else {
                    $stripeTransaction->non_domestic = 1;
                }
            }
        }
        if (empty($countryCommission)) {
            $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
            if (empty($country))
                $stripeTransaction->non_domestic = 1;
        }

        $dbCountryCommissionHistory = CountryCommissionHistory::where('stripe_transaction_id', $stripeTransaction->id)->first();
        if (empty($dbCountryCommissionHistory)) {
            $countryCommissionHistory = new CountryCommissionHistory();
            if (!empty($countryCommission)) {
                $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                $countryCommissionHistory->stripe_percentage = $countryCommission->stripe_percentage;
                $countryCommissionHistory->stripe_fixed = $countryCommission->stripe_fixed;
                $countryCommissionHistory->save();
            } else {
                $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                $countryCommissionHistory->stripe_percentage = 0;
                $countryCommissionHistory->stripe_fixed = 0;
                $countryCommissionHistory->save();
            }
        }

        $stripeTransaction->save();

        // Save Reseller Commissions
        $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
        if (!empty($reseller)) {
            $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
            if (empty($resellerCommission)) {
                $resellerCommission = new ResellerCommission;
                $resellerCommission->user_id = $reseller->user_id;
                $resellerCommission->business_id = $business->id;
                $resellerCommission->save();
            }
            $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $stripeTransaction->id)->first();
            if (empty($resellerStripeCommission)) {
                $resellerStripeCommission = new ResellerStripeCommission();
                $resellerStripeCommission->user_id = $reseller->user_id;
                $resellerStripeCommission->stripe_transaction_id = $stripeTransaction->id;
                $resellerStripeCommission->fill(json_decode($resellerCommission, true));

                $resellerStripeCommission->save();
            }
        }


        $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$stripeTransaction->card_brand, $stripeTransaction->card_brand])->first();
        if (!empty($cardBrand)) {
            $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
        }
        if (empty($cardBrandCommission)) {
            $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
        }
        if (!empty($cardBrandCommission)) {
            $cardBrandCommissionHistory = new CardBrandCommissionHistory();
            $cardBrandCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
            $cardBrandCommissionHistory->stripe_percentage = $cardBrandCommission->stripe_percentage;
            $cardBrandCommissionHistory->stripe_fixed = $cardBrandCommission->stripe_fixed;
            $cardBrandCommissionHistory->save();
        }
    }

    public function saveBlankChargeDetail()
    {
        $lastBlankTransaction = StripeTransaction::orderBy('id', 'ASC');
        $lastBlankTransaction->whereNull('card_brand');
        $lastBlankTransaction->where('reporting_category', 'charge');
        $lastBlankTransaction->where('type', 'charge');
        $lastBlankTransaction = $lastBlankTransaction->get();

        if (!empty($lastBlankTransaction)) {
            foreach ($lastBlankTransaction as $trans) {

                try {
                    // if ($trans->account_type == 'normal') {
                    //     $siteSettingMode = SiteSettingController::findByKey('stripe_mode');
                    //     $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
                    //     if (!empty($siteSettingMode)) {
                    //         $mode = $siteSettingMode->value;
                    //     }
                    //     if (!empty($siteSettingsecret_key) && $mode == 'live') {
                    //         $key = $siteSettingsecret_key->value;
                    //     } else {
                    //         $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                    //         $key = $siteSettingsecret_key->value;
                    //     }
                    // } else {
                    //     $connectSiteSettingMode = SiteSettingController::findByKey('connect_stripe_mode');
                    //     $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_live');
                    //     if (!empty($connectSiteSettingMode)) {
                    //         $connectMode = $connectSiteSettingMode->value;
                    //     }
                    //     if (!empty($connectSiteSettingsecret_key) && $connectMode == 'live') {
                    //         $key = $connectSiteSettingsecret_key->value;
                    //     } else {
                    //         $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_test');
                    //         $key = $connectSiteSettingsecret_key->value;
                    //     }
                    // }

                    $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
                    if (!empty($siteSettingsecret_key) && $trans->mode == 'live') {
                        $key = $siteSettingsecret_key->value;
                    } else {
                        $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                        $key = $siteSettingsecret_key->value;
                    }

                    $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                    $balance_transaction = $stripe->balanceTransactions->retrieve(
                        $trans->txn_id,
                        []
                    );

                    $chargeDetails = $stripe->charges->retrieve(
                        $balance_transaction->source,
                        []
                    );

                    $stripeTransaction = StripeTransaction::where('id', $trans->id)->first();
                    if (!empty($stripeTransaction)) {
                        if ($chargeDetails->payment_method_details->type == 'card_present') {
                            $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                            $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                            $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                            $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                        } else {
                            if (isset($chargeDetails->payment_method_details->card)) {
                                $stripeTransaction->card_number = $chargeDetails->payment_method_details->card->last4;
                                $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card->brand;
                                $stripeTransaction->card_type = $chargeDetails->payment_method_details->card->funding;
                                $stripeTransaction->card_country = $chargeDetails->payment_method_details->card->country;
                            }
                        }
                        $stripeTransaction->save();
                    } else {
                        continue;
                    }
                } catch (Exception $e) {
                    try {
                        $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_live');

                        if (!empty($connectSiteSettingsecret_key) && $trans->mode == 'live') {
                            $key = $connectSiteSettingsecret_key->value;
                        } else {
                            $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_test');
                            $key = $connectSiteSettingsecret_key->value;
                        }

                        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                        $balance_transaction = $stripe->balanceTransactions->retrieve(
                            $trans->txn_id,
                            []
                        );

                        $chargeDetails = $stripe->charges->retrieve(
                            $balance_transaction->source,
                            []
                        );

                        $stripeTransaction = StripeTransaction::where('id', $trans->id)->first();
                        if (!empty($stripeTransaction)) {
                            if ($chargeDetails->payment_method_details->type == 'card_present') {
                                $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                                $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                                $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                                $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                            } else {
                                if (isset($chargeDetails->payment_method_details->card)) {
                                    $stripeTransaction->card_number = $chargeDetails->payment_method_details->card->last4;
                                    $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card->brand;
                                    $stripeTransaction->card_type = $chargeDetails->payment_method_details->card->funding;
                                    $stripeTransaction->card_country = $chargeDetails->payment_method_details->card->country;
                                }
                            }
                            $stripeTransaction->save();
                        } else {
                            continue;
                        }
                    } catch (Exception $e) {
                        continue;
                    }
                }
            }
        }

        return $lastBlankTransaction;
    }

    public function missTransactions()
    {
        $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
        if ($mode == 'test') {
            $stripeKeyColoumns = 'connect_stripe_private_key_test';
        }
        if ($mode == 'live') {
            $stripeKeyColoumns = 'connect_stripe_private_key_live';
        }
        $key = SiteSettingController::findByKey($stripeKeyColoumns)->value;
        $params = [];
        $params['created']['gte'] = Carbon::today()->subDays(2)->timestamp;
        $params['type'] = 'charge';
        $params['limit'] = 100;
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        try {
            $missChargeTransaction = [];
            $transactions = $stripe->balanceTransactions->all($params);
            foreach (array_reverse($transactions->data) as $trans) {
                $fetchTransaction = StripeTransaction::orderByRaw('created DESC');
                $fetchTransaction->where('reporting_category', 'charge');
                $fetchTransaction->where('source', $trans->source);
                $fetchTransactions = $fetchTransaction->first();
                if (empty($fetchTransactions)) {
                    $missChargeTransaction[] = $trans;
                }
            }
        } catch (Exception $e) {
        }

        return $missChargeTransaction;
    }

    public function updateTransactionDetails()
    {
        $stripeTransactions = StripeTransaction::orderByRaw('created DESC');
        // $carbon = Carbon::today()->subDays(1);
        // $startDate = $carbon->startOfDay();
        // $endDate = $carbon->endOfDay();
        $carbon = Carbon::today()->subDays(1);
        $endCarbon = Carbon::today()->subDays(1);
        $startDate = $carbon->startOfDay();
        $endDate = $endCarbon->endOfDay();
        // $endDate = Carbon::today()->endOfDay();
        $stripeTransactions->where('created', '>', $startDate->getTimestamp());
        $stripeTransactions->where('created', '<', $endDate->getTimestamp());
        $stripeTransactions = $stripeTransactions->get();

        foreach ($stripeTransactions as $trans) {
            $country = null;
            $countryCommission = null;
            $cardBrand = null;
            $cardBrandCommission = null;

            $business = Business::find($trans->business_id);
            try {
                $country = Country::where('iso', $trans->card_country)->first();
                if (!empty($country)) {
                    $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                    if ($country->id == $business->country_id) {
                        $trans->non_domestic = 0;
                    } else {
                        $trans->non_domestic = 1;
                    }
                }
                if (empty($countryCommission)) {
                    $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                    if (empty($country))
                        $trans->non_domestic = 1;
                }

                $countryCommissionHistory = CountryCommissionHistory::where('stripe_transaction_id', $trans->id)->first();
                if (!empty($countryCommission) && !empty($countryCommissionHistory)) {
                    $countryCommissionHistory->stripe_percentage = $countryCommission->stripe_percentage;
                    $countryCommissionHistory->stripe_fixed = $countryCommission->stripe_fixed;
                    $countryCommissionHistory->save();
                }


                $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$trans->card_brand, $trans->card_brand])->first();
                if (!empty($cardBrand)) {
                    $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
                }
                if (empty($cardBrandCommission)) {
                    $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
                }

                $cardBrandCommissionHistory = CardBrandCommissionHistory::where('stripe_transaction_id', $trans->id)->first();
                if (!empty($cardBrandCommission) && !empty($cardBrandCommissionHistory)) {
                    $cardBrandCommissionHistory->stripe_percentage = $cardBrandCommission->stripe_percentage;
                    $cardBrandCommissionHistory->stripe_fixed = $cardBrandCommission->stripe_fixed;
                    $cardBrandCommissionHistory->save();
                }
            } catch (Exception $e) {
                Log::debug($e->getMessage());
            }
            Log::debug('Transaction Commission updated successfully!');
        }
    }

    public function missTransactionDetail()
    {
        $missCharge = FetchLog::where('event', Utils::$MISS_CHARGE)->latest()->first();
        if (!empty($missCharge)) {
            $txnId = $missCharge->extras;
            $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
            if ($mode == 'test') {
                $stripeKeyColoumns = 'connect_stripe_private_key_test';
            }
            if ($mode == 'live') {
                $stripeKeyColoumns = 'connect_stripe_private_key_live';
            }
            $key = SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

            $params = [];
            $params['limit'] = 100;
            $params['type'] = 'charge';
            $params['ending_before'] = $txnId;

            $txid = '';
            try {
                $transactions = $stripe->balanceTransactions->all($params);

                foreach (array_reverse($transactions->data) as $trans) {
                    $country = null;
                    $countryCommission = null;
                    $cardBrand = null;
                    $cardBrandCommission = null;

                    $fetchTransaction = StripeTransaction::orderByRaw('created DESC');
                    $fetchTransaction->where('reporting_category', 'charge');
                    $fetchTransaction->where('source', $trans->source);
                    $fetchTransactions = $fetchTransaction->first();
                    if (empty($fetchTransactions)) {
                        $chargeDetails = null;
                        if ($trans->type == 'charge') {
                            $chargeDetails = $stripe->charges->retrieve(
                                $trans->source,
                                []
                            );
                        }
                        $account_type = 'connect';
                        $stripeTransaction = new StripeTransaction();
                        $stripeTransaction->txn_id = $trans->id;
                        $stripeTransaction->mode = $mode;
                        $stripeTransaction->service = 'Other';
                        $stripeTransaction->account_type = $account_type;
                        $stripeTransaction->transfer_status = 'pending';

                        if ($trans->reporting_category == 'charge') {
                            $chargeDetails = $stripe->charges->retrieve($trans->source, []);
                            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                            if ($chargeDetails->statement_descriptor == null && $chargeDetails->source != null) {
                            }
                            $stripeTransaction->transfer_status = 'pending';
                        }

                        if ($trans->reporting_category == 'refund') {
                            $refundDetails = $stripe->refunds->retrieve($trans->source, []);
                            $stripeTransaction->payment_intent_id = $refundDetails->payment_intent;
                            $chargeDetails = $stripe->charges->retrieve($refundDetails->charge, []);
                            // if parent charge archive then auto archive refund transaction
                            $chargeStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->first();
                            if (!empty($chargeStripeTransactions)) {
                                $stripeTransaction->archived = $chargeStripeTransactions->archived;
                            }

                            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                            $stripeTransaction->transfer_status = 'not_applicable';
                            if ($chargeDetails->refunded && $chargeDetails->payment_intent != null) {
                                $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'not_applicable']);
                            }
                        }
                        if ($trans->reporting_category == 'transfer') {
                            $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                            if ($transferDetails->source_transaction != null) {
                                $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                                if ($chargeDetails->payment_intent != null) {
                                    $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                                    $dbStripeTransactions = StripeTransaction::where('payment_intent_id', $chargeDetails->payment_intent)->where('reporting_category', 'charge')->update(['transfer_status' => 'completed']);
                                }
                                $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                            } else {
                                $stripeTransaction->descriptor = $transferDetails->destination;
                            }
                            $stripeTransaction->transfer_status = 'not_applicable';
                        }
                        if ($trans->reporting_category == 'transfer_reversal') {
                            $transferDetails = $stripe->transfers->retrieve($trans->source, []);
                            if ($transferDetails->source_transaction != null) {
                                $chargeDetails = $stripe->charges->retrieve($transferDetails->source_transaction, []);
                                $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                                $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                                $dbStripeTransactions = StripeTransaction::where('source', $transferDetails->source_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'transfer_status' => 'reversed']);
                            } else {
                                $stripeTransaction->descriptor = $transferDetails->destination;
                            }
                            $stripeTransaction->transfer_status = 'not_applicable';
                        }
                        if ($trans->reporting_category == 'platform_earning') {
                            $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                            $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                        }
                        if ($trans->reporting_category == 'platform_earning_refund') {
                            $applicationFee = $stripe->applicationFees->retrieve($trans->source, []);
                            $chargeDetails = $stripe->charges->retrieve($applicationFee->originating_transaction, []);
                            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                            $dbStripeTransactions = StripeTransaction::where('source', $applicationFee->originating_transaction)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);
                        }
                        if ($trans->reporting_category == 'dispute') {
                            $disputeDetails = $stripe->disputes->retrieve($trans->source, []);
                            $stripeTransaction->dispute_status = $disputeDetails->status;
                            $chargeDetails = $stripe->charges->retrieve($disputeDetails->charge, []);
                            $stripeTransaction->payment_intent_id = $chargeDetails->payment_intent;
                            $stripeTransaction->descriptor = $chargeDetails->statement_descriptor;
                        }
                        if ($trans->reporting_category == 'payout') {
                            $payoutDetails = $stripe->payouts->retrieve($trans->source, []);
                            $stripeTransaction->payout_status = $payoutDetails->status;
                        }

                        if (!empty($chargeDetails)) {
                            if ($chargeDetails->customer != null) {
                                $customerDetails = $stripe->customers->retrieve($chargeDetails->customer, []);
                                $stripeTransaction->email = $customerDetails->email;
                            }
                            // try {
                            if ($chargeDetails->payment_method_details->type == 'card_present') { // $chargeDetails->payment_method_details->type != null && 
                                $stripeTransaction->card_number = $chargeDetails->payment_method_details->card_present->last4;
                                $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card_present->brand;
                                $stripeTransaction->card_type = $chargeDetails->payment_method_details->card_present->funding;
                                $stripeTransaction->card_country = $chargeDetails->payment_method_details->card_present->country;
                            } else {
                                if (isset($chargeDetails->payment_method_details->card)) {
                                    $stripeTransaction->card_number = $chargeDetails->payment_method_details->card->last4;
                                    $stripeTransaction->card_brand = $chargeDetails->payment_method_details->card->brand;
                                    $stripeTransaction->card_type = $chargeDetails->payment_method_details->card->funding;
                                    $stripeTransaction->card_country = $chargeDetails->payment_method_details->card->country;
                                }
                            }
                            // } catch (Exception $e) {
                            // }
                            $dbStripeTransactions = StripeTransaction::where('source', $chargeDetails->id)->where('reporting_category', 'charge')->update(['refunded' => $chargeDetails->refunded, 'refunded_amount' => $chargeDetails->amount_refunded]);
                        }

                        $stripeTransaction->object = $trans->object;
                        $stripeTransaction->description = $trans->description;
                        $stripeTransaction->amount = $trans->amount;
                        $stripeTransaction->fee = $trans->fee;
                        $stripeTransaction->net = $trans->net;
                        $stripeTransaction->currency = $trans->currency;
                        $stripeTransaction->reporting_category = $trans->reporting_category;
                        $stripeTransaction->source = $trans->source;
                        $stripeTransaction->status = $trans->status;
                        $stripeTransaction->type = $trans->type;
                        $stripeTransaction->available_on = $trans->available_on;
                        $stripeTransaction->created = $trans->created;
                        if (isset($chargeDetails->metadata)) {
                            $stripeTransaction->metadata = $chargeDetails->metadata;
                        }

                        $searchableDescriptor = $stripeTransaction->descriptor;
                        if (empty($stripeTransaction->descriptor)) {
                            $searchableDescriptor = $stripeTransaction->description;
                        }

                        $business = Business::whereRaw('(statement_descriptor = ? OR online_statement_descriptor = ? OR s_account_id = ?)', ["$searchableDescriptor", "$searchableDescriptor", "$searchableDescriptor"])->first();
                        if (!empty($business)) {
                            $stripeTransaction->business_id = $business->id;

                            if ($business->auto_archive == 1) {
                                $stripeTransaction->archived = 1;
                            }
                            $businessCommission = BusinessCommission::where('business_id', $business->id)->first();
                            $stripeTransaction->cash_handling_fee_type = $businessCommission->cash_handling_fee_type;
                            $stripeTransaction->cash_handling_fee = $businessCommission->cash_handling_fee;
                            $stripeTransaction->cash_handling_per_amount = $businessCommission->cash_handling_per_amount;
                            $stripeTransaction->management_fee_type = $businessCommission->management_fee_type;
                            $stripeTransaction->management_fee = 0; //$businessCommission->management_fee;
                            $stripeTransaction->merchant_card_processing_fee_type = $businessCommission->merchant_card_processing_fee_type;
                            $stripeTransaction->merchant_card_processing_fee = $businessCommission->merchant_card_processing_fee;
                            $stripeTransaction->merchant_card_auth_fee_type = $businessCommission->merchant_card_auth_fee_type;
                            $stripeTransaction->merchant_card_auth_fee = $businessCommission->merchant_card_auth_fee;

                            $stripeTransaction->charge_back_fee_type = $businessCommission->charge_back_fee_type;
                            $stripeTransaction->charge_back_fee = $businessCommission->charge_back_fee;

                            $stripeTransaction->connect_card_processing_fee_type = $businessCommission->connect_card_processing_fee_type;
                            $stripeTransaction->connect_card_processing_fee = $businessCommission->connect_card_processing_fee;
                            $stripeTransaction->connect_card_auth_fee_type = $businessCommission->connect_card_auth_fee_type;
                            $stripeTransaction->connect_card_auth_fee = $businessCommission->connect_card_auth_fee;

                            $stripeTransaction->charges_refundable = $businessCommission->charges_refundable;

                            //Set transaction service type Like from which service this transaction comes
                            if ($business->online_statement_descriptor != null && ($business->online_statement_descriptor == $stripeTransaction->descriptor || $business->online_statement_description == $stripeTransaction->description)) {
                                $stripeTransaction->service = 'Online';
                                $stripeTransaction->transfer_status = 'not_applicable';
                            } else if ($business->s_account_id != null && $business->connect_service == 1) {
                                $stripeTransaction->service = 'Connect';
                                if ($stripeTransaction->refunded == 1) {
                                    $stripeTransaction->transfer_status = 'not_applicable';
                                }
                            } else if ($business->statement_descriptor != null && ($business->statement_descriptor == $stripeTransaction->descriptor || $business->statement_description == $stripeTransaction->description)) {
                                $stripeTransaction->service = 'Merchant';
                                $stripeTransaction->transfer_status = 'not_applicable';
                            } else {
                                $stripeTransaction->service = 'Other';
                                $stripeTransaction->transfer_status = 'not_applicable';
                            }
                            if (!empty($stripeTransaction->card_country)) {
                                $country = Country::where('iso', $stripeTransaction->card_country)->first();
                                if (!empty($country)) {
                                    $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                                    if ($country->id == $business->country_id) {
                                        $stripeTransaction->non_domestic = 0;
                                    } else {
                                        $stripeTransaction->non_domestic = 1;
                                    }
                                }
                            }
                            if (empty($countryCommission)) {
                                $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                                if (empty($country))
                                    $stripeTransaction->non_domestic = 1;
                            }

                            $stripeTransaction->save();

                            if (!empty($countryCommission)) {
                                $countryCommissionHistory = new CountryCommissionHistory();
                                $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                                $countryCommissionHistory->stripe_percentage = $countryCommission->stripe_percentage;
                                $countryCommissionHistory->stripe_fixed = $countryCommission->stripe_fixed;
                                $countryCommissionHistory->save();
                            } else {
                                $countryCommissionHistory = new CountryCommissionHistory();
                                $countryCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                                $countryCommissionHistory->stripe_percentage = 0;
                                $countryCommissionHistory->stripe_fixed = 0;
                                $countryCommissionHistory->save();
                            }

                            // Save Reseller Commissions
                            $reseller = UserBusiness::where('business_id', $business->id)->whereRaw('(user_id IN (SELECT id from users WHERE reseller = 1))')->first();
                            if (!empty($reseller)) {
                                $resellerCommission = ResellerCommission::where('user_id', $reseller->user_id)->where('business_id', $business->id)->latest()->first();
                                if (empty($resellerCommission)) {
                                    $resellerCommission = new ResellerCommission;
                                    $resellerCommission->user_id = $reseller->user_id;
                                    $resellerCommission->business_id = $business->id;
                                    $resellerCommission->save();
                                }
                                $resellerStripeCommission = ResellerStripeCommission::where('stripe_transaction_id', $stripeTransaction->id)->first();
                                if (empty($resellerStripeCommission)) {
                                    $resellerStripeCommission = new ResellerStripeCommission();
                                    $resellerStripeCommission->user_id = $reseller->user_id;
                                    $resellerStripeCommission->stripe_transaction_id = $stripeTransaction->id;
                                    $resellerStripeCommission->fill(json_decode($resellerCommission, true));

                                    $resellerStripeCommission->save();
                                }
                            }


                            $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$stripeTransaction->card_brand, $stripeTransaction->card_brand])->first();
                            if (!empty($cardBrand)) {
                                $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
                            }
                            if (empty($cardBrandCommission)) {
                                $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
                            }
                            if (!empty($cardBrandCommission)) {
                                $cardBrandCommissionHistory = new CardBrandCommissionHistory();
                                $cardBrandCommissionHistory->stripe_transaction_id = $stripeTransaction->id;
                                $cardBrandCommissionHistory->stripe_percentage = $cardBrandCommission->stripe_percentage;
                                $cardBrandCommissionHistory->stripe_fixed = $cardBrandCommission->stripe_fixed;
                                $cardBrandCommissionHistory->save();
                            }
                        } else {
                            $stripeTransaction->save();
                        }
                    }
                    $txid = $trans->id;
                }

                if ($txid != '') {
                    $fetch_log = new FetchLog();
                    $fetch_log->event = Utils::$MISS_CHARGE;
                    $fetch_log->extras = $txid;
                    $fetch_log->save();
                }
            } catch (Exception $e) {
            }
        }
    }

    public function payoutTransactionDetail()
    {

        // $payoutRequests = PayoutRequest::whereNotNull('business_id')->whereNotNull('payout_id')->orderByRaw('created_at DESC')->get();
        $businesses = Business::where('connect_service', '1')->where('disabled', '0')->get();

        foreach ($businesses as $business) {
            try {

                if ($business->s_account_id != null && $business->connect_service == 1) {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $stripeKeyColoumns = 'connect_stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $stripeKeyColoumns = 'connect_stripe_private_key_live';
                    }
                } else {
                    $mode = SiteSettingController::findByKey('stripe_mode')->value;
                    if ($mode == 'test') {
                        $stripeKeyColoumns = 'stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $stripeKeyColoumns = 'stripe_private_key_live';
                    }
                }

                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
                $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
                $payouts = $stripe->payouts->all(
                    ['limit' => 50],
                    [
                        "stripe_account" => $business->s_account_id,
                    ]
                );
                if (!empty($payouts->data)) {
                    foreach ($payouts->data as $payout) {
                        $payoutRequest = PayoutRequest::where('payout_id', $payout->id)->first();
                        $externalAccount = ExternalAccount::where('s_bank_account_id', $payout->destination)->first();
                        if (!empty($externalAccount)) {
                            $payoutRequest->business_id  = $business->id;
                        }
                        if (!empty($payoutRequest)) {
                            $payoutRequest->payout_status = $payout->status;
                            $payoutRequest->created = $payout->created;
                            $payoutRequest->arrival_date = $payout->arrival_date;
                            if ($payout->status == 'paid') {
                                $payoutRequest->status = 'Accepted';
                            }
                            if ($payout->status == 'canceled') {
                                $payoutRequest->status = 'Rejected';
                            }
                            $payoutRequest->save();
                            continue;
                        }
                        $payoutRequest = new PayoutRequest();
                        $payoutRequest->amount = $payout->amount / 100;
                        $payoutRequest->status = 'Pending';
                        $payoutRequest->note = $payout->statement_descriptor;
                        $payoutRequest->bank_account_id = $payout->destination;
                        $payoutRequest->payout_id = $payout->id;
                        $payoutRequest->payout_status = $payout->status;
                        $payoutRequest->created = $payout->created;
                        $payoutRequest->arrival_date = $payout->arrival_date;
                        $payoutRequest->rejection_note = $payout->failure_message;
                        $payoutRequest->save();
                    }
                }
            } catch (Exception $e) {
            }
        }
    }

    public function updateDisputeTransaction()
    {
        $disputeTransaction = StripeTransaction::orderBy('id', 'ASC');
        $disputeTransaction->whereRaw('(dispute_status != ? OR dispute_status != ?)', ['win', 'lost']);
        $disputeTransaction->where('reporting_category', 'dispute');
        $disputeTransaction = $disputeTransaction->get();

        if (!empty($disputeTransaction)) {
            foreach ($disputeTransaction as $trans) {

                try {
                    $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_live');
                    if (!empty($siteSettingsecret_key) && $trans->mode == 'live') {
                        $key = $siteSettingsecret_key->value;
                    } else {
                        $siteSettingsecret_key = SiteSettingController::findByKey('stripe_private_key_test');
                        $key = $siteSettingsecret_key->value;
                    }

                    $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                    $disputeDetails = $stripe->disputes->retrieve(
                        $trans->source,
                        []
                    );

                    $stripeTransaction = StripeTransaction::where('id', $trans->id)->first();
                    if (!empty($stripeTransaction)) {
                        $stripeTransaction->dispute_status = $disputeDetails->status;
                        $stripeTransaction->save();
                    } else {
                        continue;
                    }
                } catch (Exception $e) {
                    try {
                        $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_live');

                        if (!empty($connectSiteSettingsecret_key) && $trans->mode == 'live') {
                            $key = $connectSiteSettingsecret_key->value;
                        } else {
                            $connectSiteSettingsecret_key = SiteSettingController::findByKey('connect_stripe_private_key_test');
                            $key = $connectSiteSettingsecret_key->value;
                        }

                        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                        $disputeDetails = $stripe->disputes->retrieve(
                            $trans->source,
                            []
                        );

                        $stripeTransaction = StripeTransaction::where('id', $trans->id)->first();
                        if (!empty($stripeTransaction)) {
                            $stripeTransaction->dispute_status = $disputeDetails->status;
                            $stripeTransaction->save();
                        } else {
                            continue;
                        }
                    } catch (Exception $e) {
                        continue;
                    }
                }
            }
        }

        return $disputeTransaction;
    }

    public function stripePaymentConfirmation(Request $request)
    {
        return view('stripe-payment');
    }

    public function stripeCustomPayment(Request $request)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_public_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_public_key_live';
        }
        $key = SiteSettingController::findByKey($columns)->value;
        return view('stripe-custom-payment', compact('key'));
    }

    public function createPaymentIntent(Request $request)
    {
        if (!$request->has('amount') || $request->amount == null || $request->amount == 0) {
            $json['status'] = false;
            $json['message'] = 'Payment amount must be greater then 0.';
            return response()->json($json);
        }
        $json = [];
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key = SiteSettingController::findByKey($columns)->value;
        $stripe = new StripeClient($key);

        $paymentIntent = $stripe->paymentIntents->create([
            'amount' => 1500,
            'currency' => 'gbp',
            'payment_method' => $request->paymentIntent,
            'confirm' => true,
            'capture_method' => 'automatic'
        ]);

        $json['status'] = true;
        $json['data']['paymentIntentSecret'] = $paymentIntent->client_secret;
        return response()->json($json);
    }
}
