<?php

namespace App\Http\Controllers;

use App\Exports\SubscriptionEmailExport;
use App\SubscriptionEmail;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Carbon\Carbon;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Maatwebsite\Excel\Facades\Excel;

class SubscriptionEmailController extends Controller
{
    /**
     * List subsciption-emails.
     * GET /api/subsciption-emails
     */
    public function index(Request $req)
    {
        $subsciption_emails = SubscriptionEmail::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $subsciption_emails->whereRaw('(email LIKE ?)', ["%$query%"]);
        }

        if ($query = $req->query('from_date', false)) {
            $this->validate($req, [
                'from_date' => 'required|date_format:Y-m-d',
                'to_date' => 'required|date_format:Y-m-d|after_or_equal:from_date',
            ]);

            $from_date = Carbon::parse($req->query('from_date'))->format('Y-m-d 00:00:00');
            $to_date = Carbon::parse($req->query('to_date'))->format('Y-m-d 23:59:59');
            $subsciption_emails->where('created_at', '>=', $from_date)->where('created_at', '<=', $to_date);
        }

        if ($req->query('nopaginate', false) == '1') {
            $subsciption_emails = $subsciption_emails->get();
        } else {
            $subsciption_emails = $subsciption_emails->paginate($req->query('per_page', 20));
        }

        if ($subsciption_emails->isEmpty()) Utils::error('No subscription emails found!', 404);

        return $subsciption_emails;
    }

    /**
     * Create subsciption-email.
     * POST /api/subsciption-emails
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'email' => 'required|email',
        ]);

        // create subsciption-email
        $subsciption_email = new SubscriptionEmail;
        $subsciption_email->fill($req->all());
        $subsciption_email->save();

        $this->sendMail($subsciption_email->id);

        return $subsciption_email;
    }

    /**
     * Export subsciption-email.
     * POST /api/subsciption-emails/export
     */
    public function export(Request $req)
    {
        if ($req->input('from_date') == '' || $req->input('from_date') == 'null') {
            $req->request->remove('from_date');
            $req->request->remove('to_date');
        }
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $fileName = "subsciption_emails";
            $data = ($this->index($req));
            return Excel::download(new SubscriptionEmailExport($data), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }

    /**
     * Show subsciption-email.
     * GET /api/subsciption-emails/{id}
     */
    public function show(Request $req, $id)
    {
        $subsciption_email = SubscriptionEmail::find($id);
        if (empty($subsciption_email)) Utils::error('Subscription email not found!', 404);
        return $subsciption_email;
    }

    /**
     * Update subsciption-email.
     * POST /api/subsciption-emails/{id}
     */
    public function update(Request $req, $id)
    {
        $subsciption_email = SubscriptionEmail::find($id);
        if (empty($subsciption_email)) Utils::error('Subscription email not found!', 404);

        $this->validate($req, [
            'email' => 'required|email',
        ]);

        $respond = $subsciption_email->respond;

        // update subsciption_email
        if ($req->has('email')) $subsciption_email->email = $req->input('email');

        $subsciption_email->save();

        return $subsciption_email;
    }

    /**
     * Delete subsciption-email by id.
     * DELETE /api/subsciption-emails/{id}
     */
    public function delete(Request $req, $id)
    {
        $subsciption_email = SubscriptionEmail::find($id);
        if (empty($subsciption_email)) Utils::error('Subscription email not found!', 404);
        $subsciption_email->delete();
        return $subsciption_email;
    }

    public function sendMail($id)
    {
        $subsciption_email = SubscriptionEmail::find($id);
        if (empty($subsciption_email)) Utils::error("No contact found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Thank you for subscribing to UBSIDI newsletter! We are excited to have you as a new member of our community, and we look forward to keeping you up-to-date on our latest news, updates, and promotions.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Our newsletter will be delivered to your inbox on a regular basis, and you can expect to receive a variety of content, including product information, special offers, and industry insights. We aim to provide you with valuable information that you can use to make informed decisions and stay informed about the latest trends in your field.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                As a valued subscriber, you will also receive exclusive access to special promotions and discounts, which are only available to our newsletter subscribers. Be sure to check your inbox regularly to take advantage of these exciting offers.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                If you ever have any questions or concerns, please do not hesitate to contact us by replying to this email. Our team is always available to assist you, and we are committed to providing you with exceptional customer service.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Thank you again for joining our community, and we look forward to staying connected with you!
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Welcome to UBSIDI Newsletter!',
            'mail_type' => $siteSettings->mail_option,
            'to' => $subsciption_email->email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }
}
