<?php

namespace App\Http\Controllers;

use App\SupportContact;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class SupportContactController extends Controller
{
    /**
     * List support-contacts.
     * GET /api/support-contacts
     */
    public function index(Request $req)
    {
        $support_contacts = SupportContact::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $support_contacts->whereRaw('(first_name LIKE ? OR  last_name LIKE ? OR  email LIKE ? OR  subject LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $support_contacts = $support_contacts->get();
        } else {
            $support_contacts = $support_contacts->paginate($req->query('per_page', 20));
        }

        if ($support_contacts->isEmpty()) Utils::error('No contacts found!', 404);

        return $support_contacts;
    }

    /**
     * Create support-contact.
     * POST /api/support-contacts
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'first_name' => 'required|string',
            'last_name' => 'required|string',
            'email' => 'required|email',
            'subject' => 'required|string',
            'message' => 'required|string',
            'respond' => 'string',
        ]);

        // create support-contact
        $support_contact = new SupportContact;
        $support_contact->fill($req->all());
        $support_contact->save();

        $this->supportMail($support_contact->id);

        return $support_contact;
    }

    /**
     * Show support-contact.
     * GET /api/support-contacts/{id}
     */
    public function show(Request $req, $id)
    {
        $support_contact = SupportContact::find($id);
        if (empty($support_contact)) Utils::error('Contact not found!', 404);
        return $support_contact;
    }

    /**
     * Update support-contact.
     * POST /api/support-contacts/{id}
     */
    public function update(Request $req, $id)
    {
        $support_contact = SupportContact::find($id);
        if (empty($support_contact)) Utils::error('Contact not found!', 404);

        $this->validate($req, [
            'first_name' => 'required|string',
            'last_name' => 'required|string',
            'email' => 'required|email',
            'subject' => 'required|string',
            'message' => 'required|string',
            'respond' => 'string',
        ]);

        $respond = $support_contact->respond;

        // update support_contact
        if ($req->has('first_name')) $support_contact->first_name = $req->input('first_name');
        if ($req->has('last_name')) $support_contact->last_name = $req->input('last_name');
        if ($req->has('email')) $support_contact->email = $req->input('email');
        if ($req->has('subject')) $support_contact->subject = $req->input('subject');
        if ($req->has('message')) $support_contact->message = $req->input('message');
        if ($req->has('respond')) $support_contact->respond = $req->input('respond');

        $support_contact->save();

        if ($respond == '' && $req->input('respond') != '') {
            $this->supportRespondMail($support_contact->id);
        }

        return $support_contact;
    }

    /**
     * Delete support-contact by id.
     * DELETE /api/support-contacts/{id}
     */
    public function delete(Request $req, $id)
    {
        $support_contact = SupportContact::find($id);
        if (empty($support_contact)) Utils::error('Contact not found!', 404);
        $support_contact->delete();
        return $support_contact;
    }

    public function supportMail($id)
    {
        $support_contact = SupportContact::find($id);
        if (empty($support_contact)) Utils::error("No contact found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Dear ' . $support_contact->first_name . ' ' . $support_contact->last_name . ',<br>
                             </p>

                             <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Thank you for contacting us through our website UBSIDI. We are delighted to hear from you and appreciate the opportunity to assist you.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Our team has received your inquiry, and we are currently reviewing it. Please allow us 24-48 hours to respond to your message. We will do our best to address your concerns or queries as soon as possible.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Meanwhile, you may find the answers to some of your questions on our websites <a href="https://www.ubsidi.com/web/faqs" targe="_blank" style="text-decoration: underline;">FAQ section</a>. If you need immediate assistance, please call our customer service hotline at 08712884040 , ext: 107 / 108, and our representatives will be happy to assist you.<br>
                            </p>
                            <p style="font-size: 15px; line-height: 24px; color: #000;">
                                Once again, thank you for choosing UBSIDI, and we look forward to serving you soon.
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Thank you for contacting us - UBSIDI for ' . $support_contact->subject,
            'mail_type' => $siteSettings->mail_option,
            'to' => $support_contact->email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function supportRespondMail($id)
    {
        $support_contact = SupportContact::find($id);
        if (empty($support_contact)) Utils::error("No contact found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table width="100%" style="width:767px;">
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $support_contact->first_name . ' ' . $support_contact->last_name . ',<br>
                             ' . $support_contact->respond . '<br>
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Thank you for contacting us - UBSIDI for ' . $support_contact->subject,
            'mail_type' => $siteSettings->mail_option,
            'to' => $support_contact->email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }
}
