<?php

namespace App\Http\Controllers;

use App\TapRequest;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class TapRequestController extends Controller
{
    /**
     * List tap-to-pay-requests.
     * GET /api/tap-to-pay-requests
     */
    public function index(Request $req)
    {
        $tap_to_pay_requests = TapRequest::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $tap_to_pay_requests->whereRaw('(name LIKE ? OR email LIKE ? OR  phone LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }
        if ($business_id = $req->query('business_id', false)) {
            $tap_to_pay_requests->where('business_id', $business_id);
        }
        if ($status = $req->query('status', false)) {
            $tap_to_pay_requests->where('status', $status);
        }
        if ($req->query('nopaginate', false) == '1') {
            $tap_to_pay_requests = $tap_to_pay_requests->get();
        } else {
            $tap_to_pay_requests = $tap_to_pay_requests->paginate($req->query('per_page', 20));
        }

        if ($tap_to_pay_requests->isEmpty()) Utils::error('No tap-to pay requests found!', 404);

        return $tap_to_pay_requests;
    }

    /**
     * Create tap-to-pay-request.
     * POST /api/tap-to-pay-requests
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'name' => 'required|string',
            'email' => 'required|email',
            'phone' => 'required|string',
            'message' => 'string',
            'status' => 'in:pending,accepted,rejected',
        ]);

        // create tap_to_pay_request
        $tap_to_pay_request = new TapRequest;
        $tap_to_pay_request->fill($req->all());
        $tap_to_pay_request->status = 'pending';
        $tap_to_pay_request->save();
        return $tap_to_pay_request;
    }


    /**
     * Count tap-to-pay-requests.
     * GET /api/tap-to-pay-requests/count
     */
    public function count(Request $req)
    {
        $tapCount = TapRequest::orderByRaw('id DESC');

        if ($business_id = $req->query('business_id', false)) {
            $tapCount->where('business_id', $business_id);
        }

        $tapCount = $tapCount->count();

        $data = [
            "tap_count" => $tapCount,
        ];

        return $data;
    }


    /**
     * Show tap-to-pay-request.
     * GET /api/tap-to-pay-requests/{id}
     */
    public function show(Request $req, $id)
    {
        $tap_to_pay_request = TapRequest::find($id);
        if (empty($tap_to_pay_request)) Utils::error('Tap-to pay request not found!', 404);
        return $tap_to_pay_request;
    }

    /**
     * Update tap-to-pay-request.
     * POST /api/tap-to-pay-requests/{id}
     */
    public function update(Request $req, $id)
    {
        $tap_to_pay_request = TapRequest::find($id);
        if (empty($tap_to_pay_request)) Utils::error('Tap-to pay request not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'name' => 'required|string',
            'email' => 'required|email',
            'phone' => 'required|string',
            'message' => 'string',
            'status' => 'in:pending,accepted,rejected',
        ]);

        // update tap_to_pay_request
        if ($req->has('business_id')) $tap_to_pay_request->business_id = $req->input('business_id');
        if ($req->has('name')) $tap_to_pay_request->name = $req->input('name');
        if ($req->has('email')) $tap_to_pay_request->email = $req->input('email');
        if ($req->has('phone')) $tap_to_pay_request->phone = $req->input('phone');
        if ($req->has('message')) $tap_to_pay_request->message = $req->input('message');
        if ($req->has('status')) $tap_to_pay_request->status = $req->input('status');
        $tap_to_pay_request->save();

        return $tap_to_pay_request;
    }

    /**
     * Delete tap-to-pay-request by id.
     * DELETE /api/tap-to-pay-requests/{id}
     */
    public function delete(Request $req, $id)
    {
        $tap_to_pay_request = TapRequest::find($id);
        if (empty($tap_to_pay_request)) Utils::error('Tap-to pay request not found!', 404);
        $tap_to_pay_request->delete();
        return $tap_to_pay_request;
    }
}
