<?php

namespace App\Http\Controllers;

use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\Ticket;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\TicketMessage;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class TicketController extends Controller
{
    /**
     * List tickets.
     * GET /api/tickets
     */
    public function index(Request $req)
    {
        $tickets = Ticket::orderByRaw('id DESC');

        if ($query = $req->query('query', false)) {
            $tickets->whereRaw('(name LIKE ? OR email LIKE ? OR phone_number LIKE ? OR ticket_number LIKE ? )', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($email = $req->query('email', false)) {
            $tickets->whereRaw('(email LIKE ?  )', ["%$email%"]);
        }
        if ($phone_number = $req->query('phone_number', false)) {
            $tickets->where('phone_number', $phone_number);
        }
        if ($status = $req->query('status', false)) {
            $tickets->where('status', $status);
        }
        if ($req->has('verified')) {
            $tickets->where('verified', $req->query('verified', false));
        }
        if ($ticket_number = $req->query('ticket', false)) {
            $tickets->whereRaw('(email LIKE ? OR phone_number LIKE ? OR ticket_number LIKE ? )', ["%$ticket_number%", "%$ticket_number%", "%$ticket_number%"]);
            $tickets->where('verified', '1');
        }
        if ($req->query('nopaginate', false) == '1') {
            $tickets = $tickets->get();
        } else {
            $tickets = $tickets->paginate($req->query('per_page', 20));
        }

        if ($tickets->isEmpty()) Utils::error('No tickets found!', 404);

        if ($ticket_number = $req->query('ticket', false)) {

            // $ticket = Ticket::find($tickets[0]['id']);
            // $otp = rand(100000, 999999);
            // $ticket->otp = $otp;
            // $ticket->save();

            // $message = 'Your one-time password is ' . $otp . ' please use this to verify your phone number.';
            // (new OtpHelper())->sendMessage($message, $ticket->phone_number);

            // $this->otpMail($ticket->id);
        }

        return ($tickets);
    }

    /**
     * Create ticket.
     * POST /api/tickets
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => 'required|string|max:255',
            'phone_number' => 'required|string|max:255',
            'subject' => 'required',
            'message' => 'required',
            'status' => 'in:Pending,Progress,Issue,Solved',
            'verified' => 'boolean',
            'disabled' => 'boolean',
        ]);

        // create plan
        $otp = rand(100000, 999999);
        $ticket = new Ticket;
        $ticket->fill($req->all());
        $ticket->ticket_number = rand(100000000, 1000000000);
        $ticket->otp = $otp;
        $ticket->save();

        $message = 'Your one-time password is ' . $otp . ' please use this to verify your phone number.';
        (new OtpHelper())->sendMessage($message, $req->input('phone_number'), 'otp_service');

        $this->otpMail($ticket->id);

        if ($ticket->id) {
            $ticket_message = new TicketMessage;
            $ticket_message->ticket_id = $ticket->id;
            $ticket_message->message = $ticket->message;
            $ticket_message->admin = 0;
            $ticket_message->close = 0;
            $ticket_message->save();
        }

        return $ticket;
    }

    /**
     * Show ticket.
     * GET /api/tickets/{id}
     */
    public function show(Request $req, $id)
    {
        $ticket = Ticket::find($id);
        if (empty($ticket)) Utils::error('Ticket not found!', 404);
        return $ticket;
    }

    /**
     * Update ticket.
     * POST /api/tickets/{id}
     */
    public function update(Request $req, $id)
    {
        $ticket = Ticket::find($id);
        if (empty($ticket)) Utils::error('Ticket not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => 'required|string|max:255',
            'phone_number' => 'required|string|max:255',
            'subject' => 'required',
            'message' => 'required',
            'otp' => 'string',
            'status' => 'in:Pending,Progress,Issue,Solved',
            'verified' => 'boolean',
            'disabled' => 'boolean',
        ]);

        if ($ticket->otp != $req->input('otp')) {
            Utils::error('One time password not matched,please try again!', 404);
        }

        // update ticket
        if ($req->has('name')) $ticket->name = $req->input('name');
        if ($req->has('email')) $ticket->email = $req->input('email');
        if ($req->has('phone_number')) $ticket->phone_number = $req->input('phone_number');
        if ($req->has('subject')) $ticket->subject = $req->input('subject');
        if ($req->has('message')) $ticket->message = $req->input('message');
        if ($req->has('status')) $ticket->status = $req->input('status');
        if ($req->has('verified')) $ticket->verified = $req->input('verified');
        if ($req->has('disabled')) $ticket->disabled = $req->input('disabled');

        $ticket->save();

        return $ticket;
    }

    /**
     * Update ticket.
     * POST /api/tickets/{id}
     */
    public function all_update(Request $req, $id)
    {
        $ticket = Ticket::find($id);
        if (empty($ticket)) Utils::error('Ticket not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => 'required|string|max:255',
            'phone_number' => 'required|string|max:255',
            'subject' => 'required',
            'message' => 'required',
            'otp' => 'string',
            'status' => 'in:Pending,Progress,Issue,Solved',
            'verified' => 'boolean',
            'disabled' => 'boolean',
        ]);

        // update ticket
        if ($req->has('name')) $ticket->name = $req->input('name');
        if ($req->has('email')) $ticket->email = $req->input('email');
        if ($req->has('phone_number')) $ticket->phone_number = $req->input('phone_number');
        if ($req->has('subject')) $ticket->subject = $req->input('subject');
        if ($req->has('message')) $ticket->message = $req->input('message');
        if ($req->has('status')) $ticket->status = $req->input('status');
        $ticket->save();

        return $ticket;
    }

    public function resend_otp(Request $req, $id)
    {
        $ticket = Ticket::find($id);
        if (empty($ticket)) Utils::error('Ticket not found!', 404);

        $message = 'Your one-time password is ' . $ticket->otp . ' please use this to verify your phone number.';
        (new OtpHelper())->sendMessage($message, $ticket->phone_number, 'otp_service');

        $this->otpMail($ticket->id);

        if ($req->has('phone_number')) $ticket->phone_number = $req->input('phone_number');
        if ($req->has('email')) $ticket->email = $req->input('email');

        $ticket->save();

        return $ticket;
    }


    /**
     * Delete ticket by id.
     * DELETE /api/tickets/{id}
     */
    public function delete(Request $req, $id)
    {
        $ticket = Ticket::find($id);
        if (empty($ticket)) Utils::error('Ticket not found!', 404);
        $ticket->delete();
        return $ticket;
    }

    public function otpMail($id)
    {
        $ticket = Ticket::find($id);
        if (empty($ticket)) Utils::error("No ticket found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $ticket->name . ',<br>
                             Your one-time password is ' . $ticket->otp . ' please use this to verify your Email.<br>
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $ticket->email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }
}
