<?php

namespace App\Http\Controllers;

use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\TicketMessage;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class TicketMessageController extends Controller
{
    /**
     * List ticket messages.
     * GET /api/ticket-messages
     */
    public function index(Request $req)
    {
        $ticket_messages = TicketMessage::orderByRaw('created_at ASC');

        if ($ticket_id = $req->query('ticket_id', false)) {
            $ticket_messages->where('ticket_id', $ticket_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $ticket_messages = $ticket_messages->get();
        } else {
            $ticket_messages = $ticket_messages->paginate($req->query('per_page', 20));
        }

        if ($ticket_messages->isEmpty()) Utils::error('No messages found!', 404);

        return ($ticket_messages);
    }

    /**
     * Create ticket_message.
     * POST /api/ticket-messages
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'ticket_id' => 'required|exists:tickets,id',
            'message' => 'required',
            'admin' => 'boolean',
            'close' => 'boolean',
        ]);

        // create ticket message
        $ticket_message = new TicketMessage;
        $ticket_message->fill($req->all());
        $ticket_message->save();

        return $ticket_message;
    }

    /**
     * Show ticket_message.
     * GET /api/ticket-messages/{id}
     */
    public function show(Request $req, $id)
    {
        $ticket_message = TicketMessage::find($id);
        if (empty($ticket_message)) Utils::error('message not found!', 404);
        return $ticket_message;
    }

    /**
     * Update ticket_message.
     * POST /api/ticket-messages/{id}
     */
    public function update(Request $req, $id)
    {
        $ticket_message = TicketMessage::find($id);
        if (empty($ticket_message)) Utils::error('message not found!', 404);

        $this->validate($req, [
            'ticket_id' => 'required|exists:tickets,id',
            'message' => 'required',
            'admin' => 'boolean',
            'close' => 'boolean',
        ]);

        // update ticket message
        if ($req->has('ticket_id')) $ticket_message->ticket_id = $req->input('ticket_id');
        if ($req->has('message')) $ticket_message->message = $req->input('message');
        if ($req->has('admin')) $ticket_message->admin = $req->input('admin');
        if ($req->has('close')) $ticket_message->close = $req->input('close');

        $ticket_message->save();

        return $ticket_message;
    }

    /**
     * Update ticket_message.
     * POST /api/ticket-messages/{id}/close
     */
    public function close(Request $req, $id)
    {
        $ticket_message = TicketMessage::where('ticket_id', $id)->first();
        if (empty($ticket_message)) Utils::error('message not found!', 404);

        TicketMessage::where('ticket_id', $id)->update(['close' => 1]);

        return $ticket_message;
    }

    /**
     * Delete ticket message by id.
     * DELETE /api/ticket-messages/{id}
     */
    public function delete(Request $req, $id)
    {
        $ticket = TicketMessage::find($id);
        if (empty($ticket)) Utils::error('message not found!', 404);
        $ticket->delete();
        return $ticket;
    }
}
