<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomAccountAnswer;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomAccountAnswerController extends Controller
{
    /**
     * List account-answers
     * GET /tiffintom/account-answers/history
     */
    public function index(Request $req)
    {
        $account_answers = TiffintomAccountAnswer::selectRaw("account_help_answer.*,account_help.name as account_name")->leftJoin('account_help', 'account_help.id', '=', 'account_id')->orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $account_answers->whereRaw('(name)', ["%$query%"]);
        }

        if ($account_id = $req->query("account_id", false)) {
            $account_answers->where('account_id', $account_id);
        }

        if ($status = $req->query("status", false)) {
            $account_answers->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $account_answers = $account_answers->get();
        } else {
            $account_answers = $account_answers->paginate($req->query('per_page', 20));
        }

        if ($account_answers->isEmpty()) Utils::error('Account answers not found!', 404);

        return $account_answers;
    }

    /**
     * Create account-answer.
     * POST /tiffintom/account-answers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'account_id' => 'required|exists:tiffintom_connection.account_help,id',
            'question' => 'required|string',
            'answer' => 'required|string',
            'sortorder' => 'string',
        ]);

        // create account-answer
        $account_answer = new TiffintomAccountAnswer;
        $account_answer->fill($req->all());
        $account_answer->save();

        $account_answer->sortorder = $account_answer->id;
        $account_answer->save();

        return $account_answer;
    }

    /**
     * List account-answers
     * GET /tiffintom/account-answers/id
     */
    public function show(Request $req, $id)
    {
        $account_answer = TiffintomAccountAnswer::find($id);
        if (empty($account_answer)) Utils::error("Account answer not found", 404);
        return $account_answer;
    }

    /**
     * Update account-answer.
     * POST  /tiffintom/account-answers/id
     */
    public function update(Request $req, $id)
    {
        $account_answer = TiffintomAccountAnswer::find($id);
        if (empty($account_answer)) Utils::error('Account answer not found!', 404);

        $this->validate($req, [
            'account_id' => 'required|exists:tiffintom_connection.account_help,id',
            'question' => 'required|string',
            'answer' => 'required|string',
            'sortorder' => 'string',
        ]);

        // update account-answer
        if ($req->has('account_id')) $account_answer->account_id = $req->input('account_id');
        if ($req->has('question')) $account_answer->question = $req->input('question');
        if ($req->has('answer')) $account_answer->answer = $req->input('answer');
        if ($req->has('sortorder')) $account_answer->sortorder = $req->input('sortorder');
        $account_answer->save();

        return $account_answer;
    }

    /**
     * Delete account-answer by id.
     * DELETE /tiffintom/account-answers/{id}
     */
    public function delete(Request $req, $id)
    {
        $account_answer = TiffintomAccountAnswer::find($id);
        if (empty($account_answer)) Utils::error('Account answer not found!', 404);
        $account_answer->delete();

        return $account_answer;
    }

    /**
     * Enable account-answer.
     * POST /tiffintom/account-answers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $account_answer = TiffintomAccountAnswer::find($id);
        if (empty($account_answer)) Utils::error('Account answer not found!', 404);
        $account_answer->status = '1';
        $account_answer->save();
        return $account_answer;
    }

    /**
     * Disable account-answer.
     * POST /tiffintom/account-answers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $account_answer = TiffintomAccountAnswer::find($id);
        if (empty($account_answer)) Utils::error('Account answer not found!', 404);
        $account_answer->status = '0';
        $account_answer->save();
        return $account_answer;
    }
}