<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomAccountHelp;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomAccountHelpController extends Controller
{


    /**
     * List account-help
     * GET /tiffintom/account-help
     */
    public function index(Request $req)
    {
        $account_help = TiffintomAccountHelp::orderBy('id', 'DESC')->get();

        if ($account_help->isEmpty()) Utils::error('No account help found!', 404);

        return $account_help;
    }

    /**
     * List account-helps
     * GET /tiffintom/account-helps/history
     */
    public function history(Request $req)
    {
        $account_helps = TiffintomAccountHelp::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $account_helps->whereRaw('(name)', ["%$query%"]);
        }

        if ($status = $req->query("status", false)) {
            $account_helps->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $account_helps = $account_helps->get();
        } else {
            $account_helps = $account_helps->paginate($req->query('per_page', 20));
        }

        if ($account_helps->isEmpty()) Utils::error('Account helps not found!', 404);

        return $account_helps;
    }

    /**
     * Create account-help.
     * POST /tiffintom/account-helps
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'sortorder' => 'string',
        ]);

        // create account-help
        $account_help = new TiffintomAccountHelp;
        $account_help->fill($req->all());
        $account_help->save();

        $account_help->sortorder = $account_help->id;
        $account_help->save();

        return $account_help;
    }

    /**
     * List account-helps
     * GET /tiffintom/account-helps/id
     */
    public function show(Request $req, $id)
    {
        $account_help = TiffintomAccountHelp::find($id);
        if (empty($account_help)) Utils::error("Account help not found", 404);
        return $account_help;
    }

    /**
     * Update account-help.
     * POST  /tiffintom/account-helps/id
     */
    public function update(Request $req, $id)
    {
        $account_help = TiffintomAccountHelp::find($id);
        if (empty($account_help)) Utils::error('Account help not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'sortorder' => 'string',
        ]);

        // update account-help
        if ($req->has('name')) $account_help->name = $req->input('name');
        if ($req->has('sortorder')) $account_help->sortorder = $req->input('sortorder');
        $account_help->save();

        return $account_help;
    }

    /**
     * Delete account-help by id.
     * DELETE /tiffintom/account-helps/{id}
     */
    public function delete(Request $req, $id)
    {
        $account_help = TiffintomAccountHelp::find($id);
        if (empty($account_help)) Utils::error('Account help not found!', 404);
        $account_help->delete();

        return $account_help;
    }

    /**
     * Enable account-help.
     * POST /tiffintom/account-helps/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $account_help = TiffintomAccountHelp::find($id);
        if (empty($account_help)) Utils::error('Account help not found!', 404);
        $account_help->status = '1';
        $account_help->save();
        return $account_help;
    }

    /**
     * Disable account-help.
     * POST /tiffintom/account-helps/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $account_help = TiffintomAccountHelp::find($id);
        if (empty($account_help)) Utils::error('Account help not found!', 404);
        $account_help->status = '0';
        $account_help->save();
        return $account_help;
    }
}
