<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomAddressBook;
use App\ModelsTiffintom\TiffintomCart;
use App\ModelsTiffintom\TiffintomRestaurant;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomAddressBookController extends Controller
{


    /**
     * List address-book
     * GET /tiffintom/address-book
     */
    public function index(Request $req)
    {
        $address_book = TiffintomAddressBook::orderBy('id', 'DESC');

        // $customer = Auth::user();
        // if ($customer) {
        //     $address_book->where('user_id', $customer->id);
        // }
        if ($query = $req->query("customer_id", false)) {
            $address_book->where('user_id', $query);
            $address_book->where('delete_status', 'N');
        }
        if ($status = $req->query("status", false)) {
            $address_book->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $address_book = $address_book->get();
        } else {
            $address_book = $address_book->paginate($req->query('per_page', 20));
        }

        if ($address_book->isEmpty()) Utils::error('No address book found!', 404);
        if ($query = $req->query("restaurant_id", false)) {
            $restaurants = TiffintomRestaurant::where('id', $req->query("restaurant_id"))->first();
            if (empty($restaurants)) Utils::error("No restaurant found", 404);
            $restaurants->append('delivery_setting');
            $restaurants->append('areamaps');
            $addressBookLists = array();

            $cart = TiffintomCart::where('customer_id', $req->query("customer_id"))->where('order_id', 0)->orderBy('id', 'ASC')->get();
            $subTotal = 0;
            foreach ($cart as $cart) {
                $subTotal += $cart->total_price;
            }

            $deliveryadd = '';
            $deliverytitle = '';

            if (!empty($address_book)) {
                foreach ($address_book as $key => $value) {
                    $sourcelatitude = $value['latitude'];
                    $sourcelongitude = $value['longitude'];
                    if ($sourcelatitude != '' && $sourcelongitude != '') {
                        $latitudeTo  = $restaurants['sourcelatitude'];
                        $longitudeTo = $restaurants['sourcelongitude'];
                        $unit = 'K';
                        $distance = $this->getDistanceCalculate(
                            $sourcelatitude,
                            $sourcelongitude,
                            $latitudeTo,
                            $longitudeTo,
                            $unit
                        );
                        $distance = str_replace(',', '', $distance);

                        $count = 0;
                        if ($restaurants['map_mode'] == 'Circle') {
                            foreach ($restaurants['delivery_setting'] as $value1) {
                                if ($distance <= $value1['delivery_miles']) {
                                    $deliveryCharge = $value1['delivery_charge'];
                                    $deliveryadd = $value['address'];
                                    $deliverytitle = $value['title'];
                                    $count = 1;
                                    break;
                                }
                            }
                            if ($count == 1) {
                                $message = 'success';
                            } else {
                                $message = 'failed';
                            }
                        } elseif ($restaurants['map_mode'] == 'Polygon') {
                            foreach ($restaurants['areamaps'] as $key => $values) {
                                $xyArray = $this->getXYAxis($values['mappath']);
                                if (!empty($xyArray) && is_array($xyArray)) {
                                    $objPolygon = new TiffintomPolygonsController(array('x' => $sourcelatitude, 'y' => $sourcelongitude), $xyArray);
                                    if ($objPolygon->check()) {
                                        $deliveryCharge = $values['service_delivery_charge'];
                                        $message = 'success';
                                        $deliveryadd = $value['address'];
                                        $deliverytitle = $value['title'];
                                        break;
                                    } else {
                                        $message = 'failed1';
                                    }
                                }
                            }
                        } else {
                            $message = 'failed2';
                        }

                        if ($message == 'success') {
                            $deliveryadd = $value['address'];
                            $deliverytitle = $value['title'];
                            $value['to_distance'] = $distance;
                            if ($restaurants['free_delivery'] > 0 && $restaurants['free_delivery'] <= $subTotal) {
                                $deliveryCharge = 0.00;
                            }
                            $value['title'] = $deliverytitle;
                            $value['address'] = $deliveryadd;
                            $value['deliveryCharge'] = $deliveryCharge;
                            $value['deliveryMin'] = $restaurants['minimum_order'];
                            $value['delivery_status'] = 'delivery_available';
                            $addressBookLists[] = $value;
                        } else {
                            $value['to_distance'] = $distance;
                            $value['delivery_status'] = 'out_of_delivery';
                            $addressBookLists[] = $value;
                        }
                    }
                }
            }
            return $addressBookLists;
        }
        return $address_book;
    }

    /**
     * Create address-book.
     * POST /tiffintom/address-book
     */
    public function create(Request $req)
    {
        $address_book = TiffintomAddressBook::where('flat_no', $req->input('flat_no'))->where('zipcode', $req->input('zipcode'))->where('user_id', $req->input('user_id'))->where('status', '1')->where('delete_status', 'N')->first();
        if (!empty($address_book)) Utils::error("Already Address Exist", 404);

        $this->validate($req, [
            'user_id' => 'required|exists:tiffintom_connection.users,id',
            'title' => 'required|string',
            'flat_no' => 'string',
            'address' => 'required|string',
            'latitude' => 'required|string',
            'longitude' => 'required|string',
            'zipcode' => 'required|string',
        ], [
            'latitude.required' => "Please enter valid postcode and press lookup button",
            'longitude.required' => "Please enter valid postcode and press lookup button",
        ]);

        // create cart
        $address_book = new TiffintomAddressBook;
        $address_book->fill($req->all());
        $address_book->status = '1';
        $address_book->save();

        if ($req->has('restaurant_id')) {
            $sourcelatitude = $req->input('latitude');
            $sourcelongitude = $req->input('longitude');

            $restaurants = TiffintomRestaurant::where('id', $req->input("restaurant_id"))->first();
            if (empty($restaurants)) Utils::error("No restaurant found", 404);
            $restaurants->append('delivery_setting');
            $restaurants->append('areamaps');

            if ($sourcelatitude != '' && $sourcelongitude != '') {
                $latitudeTo  = $restaurants['sourcelatitude'];
                $longitudeTo = $restaurants['sourcelongitude'];
                $unit = 'K';
                $distance = $this->getDistanceCalculate(
                    $sourcelatitude,
                    $sourcelongitude,
                    $latitudeTo,
                    $longitudeTo,
                    $unit
                );
                $distance = str_replace(',', '', $distance);

                $count = 0;
                if ($restaurants['map_mode'] == 'Circle') {
                    foreach ($restaurants['delivery_setting'] as $value1) {
                        if ($distance <= $value1['delivery_miles']) {
                            $deliveryCharge = $value1['delivery_charge'];
                            $count = 1;
                            break;
                        }
                    }

                    if ($count == 1) {
                        $message = 'success';
                    } else {
                        $message = 'failed';
                    }
                } elseif ($restaurants['map_mode'] == 'Polygon') {
                    foreach ($restaurants['areamaps'] as $key => $values) {
                        $xyArray = $this->getXYAxis($values['mappath']);
                        if (!empty($xyArray) && is_array($xyArray)) {
                            $objPolygon = new TiffintomPolygonsController(array('x' => $sourcelatitude, 'y' => $sourcelongitude), $xyArray);
                            if ($objPolygon->check()) {
                                $deliveryCharge = $values['service_delivery_charge'];
                                $message = 'success';
                                break;
                            } else {
                                $message = 'failed1';
                            }
                        }
                    }
                } else {
                    $message = 'failed2';
                }
                if ($message == 'success') {
                    $cart = TiffintomCart::where('customer_id', $req->input("customer_id"))->where('order_id', 0)->orderBy('id', 'ASC')->get();
                    $subTotal = 0;
                    foreach ($cart as $cart) {
                        $subTotal += $cart->total_price;
                    }
                    if ($restaurants['free_delivery'] > 0 && $restaurants['free_delivery'] <= $subTotal) {
                        $deliveryCharge = 0.00;
                    }
                    $address_book['to_distance'] = $distance;
                    $address_book['deliveryCharge'] = $deliveryCharge;
                    $address_book['deliveryMin'] = $restaurants['minimum_order'];
                    $address_book['delivery_status'] = 'delivery_available';
                } else {
                    $address_book['to_distance'] = $distance;
                    $address_book['delivery_status'] = 'out_of_delivery';
                }
            }
        }

        return $address_book;
    }

    /**
     * List address-book
     * GET /tiffintom/address-book/id
     */
    public function show(Request $req, $id)
    {
        $address_book = TiffintomAddressBook::where('id', $id);
        if (empty($address_book)) Utils::error("No address book found", 404);
        $address_book = $address_book->first();

        if ($req->has('restaurant_id')) {
            $restaurants = TiffintomRestaurant::where('id', $req->input('restaurant_id'))->first();
            if (empty($restaurants)) Utils::error("No restaurant found", 404);
            $restaurants->append('delivery_setting');
            $restaurants->append('areamaps');

            $subTotal = 0;

            if (!empty($address_book)) {
                $cart = TiffintomCart::where('customer_id', $address_book['user_id'])->where('order_id', 0)->orderBy('id', 'ASC')->get();
                foreach ($cart as $cart) {
                    $subTotal += $cart->total_price;
                }

                $sourcelatitude = $address_book['latitude'];
                $sourcelongitude = $address_book['longitude'];
                if ($sourcelatitude != '' && $sourcelongitude != '') {

                    $deliveryadd = '';
                    $deliverytitle = '';

                    $latitudeTo  = $restaurants['sourcelatitude'];
                    $longitudeTo = $restaurants['sourcelongitude'];
                    $unit = 'K';
                    $distance = $this->getDistanceCalculate(
                        $sourcelatitude,
                        $sourcelongitude,
                        $latitudeTo,
                        $longitudeTo,
                        $unit
                    );
                    $distance = str_replace(',', '', $distance);

                    $count = 0;
                    if ($restaurants['map_mode'] == 'Circle') {
                        foreach ($restaurants['delivery_setting'] as $value1) {
                            if ($distance <= $value1['delivery_miles']) {
                                $deliveryCharge = $value1['delivery_charge'];
                                $deliveryadd = $address_book['address'];
                                $deliverytitle = $address_book['title'];
                                $count = 1;
                                break;
                            }
                        }

                        if ($count == 1) {
                            $message = 'success';
                        } else {
                            $message = 'failed';
                        }
                    } elseif ($restaurants['map_mode'] == 'Polygon') {
                        foreach ($restaurants['areamaps'] as $key => $values) {
                            $xyArray = $this->getXYAxis($values['mappath']);
                            if (!empty($xyArray) && is_array($xyArray)) {
                                $objPolygon = new TiffintomPolygonsController(array('x' => $sourcelatitude, 'y' => $sourcelongitude), $xyArray);
                                if ($objPolygon->check()) {
                                    $deliveryCharge = $values['service_delivery_charge'];
                                    $message = 'success';
                                    $deliveryadd = $address_book['address'];
                                    $deliverytitle = $address_book['title'];
                                    break;
                                } else {
                                    $message = 'failed1';
                                }
                            }
                        }
                    } else {
                        $message = 'failed2';
                    }

                    if ($message == 'success') {
                        $deliveryadd = $address_book['address'];
                        $deliverytitle = $address_book['title'];
                        $address_book['to_distance'] = $distance;
                        if ($restaurants['free_delivery'] > 0 && $restaurants['free_delivery'] <= $subTotal) {
                            $deliveryCharge = 0.00;
                        }
                        $address_book['title'] = $deliverytitle;
                        $address_book['address'] = $deliveryadd;
                        $address_book['deliveryCharge'] = $deliveryCharge;
                        $address_book['deliveryMin'] = $restaurants['minimum_order'];
                        $address_book['delivery_status'] = 'delivery_available';
                    } else {
                        $address_book['to_distance'] = $distance;
                        $address_book['delivery_status'] = 'out_of_delivery';
                    }
                }
            }
            return $address_book;
        }

        return $address_book;
    }

    /**
     * Update address-book.
     * POST  /tiffintom/address-book/id
     */
    public function update(Request $req, $id)
    {
        $address_book = TiffintomAddressBook::find($id);
        if (empty($address_book)) Utils::error('address book not found!', 404);

        $old_address_book = TiffintomAddressBook::where('flat_no', $req->input('flat_no'))->where('zipcode', $req->input('zipcode'))->where('user_id', $req->input('user_id'))->where('id', '!=', $id)->where('status', '1')->where('delete_status', 'N')->first();
        if (!empty($old_address_book)) Utils::error("Already Address Exist", 404);

        $this->validate($req, [
            'user_id' => 'required|exists:tiffintom_connection.users,id',
            'title' => 'required|string',
            'flat_no' => 'string',
            'address' => 'required|string',
            'latitude' => 'required|string',
            'longitude' => 'required|string',
            'zipcode' => 'required|string',
        ]);

        // update menu
        if ($req->has('user_id')) $address_book->user_id = $req->input('user_id');
        if ($req->has('title')) $address_book->title = $req->input('title');
        if ($req->has('flat_no')) $address_book->flat_no = $req->input('flat_no');
        if ($req->has('address')) $address_book->address = $req->input('address');
        if ($req->has('latitude')) $address_book->latitude = $req->input('latitude');
        if ($req->has('longitude')) $address_book->longitude = $req->input('longitude');
        if ($req->has('zipcode')) $address_book->zipcode = $req->input('zipcode');

        $address_book->save();

        if ($req->has('restaurant_id')) {
            $sourcelatitude = $req->input('latitude');
            $sourcelongitude = $req->input('longitude');

            $restaurants = TiffintomRestaurant::where('id', $req->input("restaurant_id"))->first();
            if (empty($restaurants)) Utils::error("No restaurant found", 404);
            $restaurants->append('delivery_setting');
            $restaurants->append('areamaps');

            if ($sourcelatitude != '' && $sourcelongitude != '') {
                $latitudeTo  = $restaurants['sourcelatitude'];
                $longitudeTo = $restaurants['sourcelongitude'];
                $unit = 'K';
                $distance = $this->getDistanceCalculate(
                    $sourcelatitude,
                    $sourcelongitude,
                    $latitudeTo,
                    $longitudeTo,
                    $unit
                );
                $distance = str_replace(',', '', $distance);

                $count = 0;
                if ($restaurants['map_mode'] == 'Circle') {
                    foreach ($restaurants['delivery_setting'] as $value1) {
                        if ($distance <= $value1['delivery_miles']) {
                            $deliveryCharge = $value1['delivery_charge'];
                            $count = 1;
                            break;
                        }
                    }
                    if ($count == 1) {
                        $message = 'success';
                    } else {
                        $message = 'failed';
                    }
                } elseif ($restaurants['map_mode'] == 'Polygon') {
                    foreach ($restaurants['areamaps'] as $key => $values) {
                        $xyArray = $this->getXYAxis($values['mappath']);
                        if (!empty($xyArray) && is_array($xyArray)) {
                            $objPolygon = new TiffintomPolygonsController(array('x' => $sourcelatitude, 'y' => $sourcelongitude), $xyArray);
                            if ($objPolygon->check()) {
                                $deliveryCharge = $values['service_delivery_charge'];
                                $message = 'success';
                                break;
                            } else {
                                $message = 'failed1';
                            }
                        }
                    }
                } else {
                    $message = 'failed2';
                }

                if ($message == 'success') {
                    $cart = TiffintomCart::where('customer_id', $req->input("customer_id"))->where('order_id', 0)->orderBy('id', 'ASC')->get();
                    $subTotal = 0;
                    foreach ($cart as $cart) {
                        $subTotal += $cart->total_price;
                    }
                    if ($restaurants['free_delivery'] > 0 && $restaurants['free_delivery'] <= $subTotal) {
                        $deliveryCharge = 0.00;
                    }
                    $address_book['to_distance'] = $distance;
                    $address_book['deliveryCharge'] = $deliveryCharge;
                    $address_book['deliveryMin'] = $restaurants['minimum_order'];
                    $address_book['delivery_status'] = 'delivery_available';
                } else {
                    $address_book['to_distance'] = $distance;
                    $address_book['delivery_status'] = 'out_of_delivery';
                }
            }
        }

        return $address_book;
    }

    /**
     * Delete address-book by id.
     * DELETE /tiffintom/address-book/{id}
     */
    public function delete(Request $req, $id)
    {
        $address_book = TiffintomAddressBook::find($id);
        if (empty($address_book)) Utils::error('address book not found!', 404);
        $address_book->delete();

        return $address_book;
    }

    /**
     * Enable address-book.
     * POST /tiffintom/address-book/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $address_book = TiffintomAddressBook::find($id);
        if (empty($address_book)) Utils::error('address book not found!', 404);
        $address_book->status = '1';
        $address_book->save();
        return $address_book;
    }

    /**
     * Disable address-book.
     * POST /tiffintom/address-book/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $address_book = TiffintomAddressBook::find($id);
        if (empty($address_book)) Utils::error('address book not found!', 404);
        $address_book->status = '0';
        $address_book->save();
        return $address_book;
    }

    //Distance calculate
    public function getDistanceCalculate($latitudeFrom, $longitudeFrom, $latitudeTo, $longitudeTo, $unit)
    {
        $theta = $longitudeFrom - $longitudeTo;
        $dist = sin(deg2rad((float)$latitudeFrom)) * sin(deg2rad((float)$latitudeTo)) +  cos(deg2rad((float)$latitudeFrom)) * cos(deg2rad((float)$latitudeTo)) * cos(deg2rad((float)$theta));
        $dist = acos($dist);
        $dist = rad2deg($dist);
        $miles = $dist * 60 * 1.1515;
        $unit = strtoupper($unit);
        if ($unit == "K") {
            return number_format(($miles), 2);
        } else if ($unit == "N") {
            return ($miles * 0.8684) . ' nm';
        } else {
            return $miles . ' mi';
        }
    }

    //-------------------------------------------------------------------------------------------------------------
    public function getDeliveryCharge($resId, $Durations, $lat, $lng)
    {
        $message = '';
        $restDetails = TiffintomRestaurant::where('id', $resId)->first();
        $restDetails->append('delivery_setting');
        $restDetails->append('areamaps');

        // $SettingsList = TiffintomSiteSetting::where('id', '1')->first();

        $deliveryCharge = $count = 0;

        if ($restDetails['map_mode'] == 'Circle') {
            foreach ($restDetails['delivery_settings'] as $key => $value) {
                if ($Durations <= $value['delivery_miles']) { // + $SettingsList['site_mile'] 
                    $deliveryCharge = $value['delivery_charge'];
                    $count = 1;
                    break;
                }
            }
            if ($count == 1) {
                $message = 'success';
            } else {
                $message = 'failed';
            }
        } else if ($restDetails['map_mode'] == 'Polygon') {
            foreach ($restDetails['areamaps'] as $key => $value) {
                $xyArray = $this->getXYAxis($value['mappath']);
                if (!empty($xyArray) && is_array($xyArray)) {
                    $objPolygon = new TiffintomPolygonsController(array('x' => $lat, 'y' => $lng), $xyArray);
                    if ($objPolygon->check()) {
                        $deliveryCharge = $value['service_delivery_charge'];
                        $count = 1;
                        break;
                    }
                }
            }
            if ($count == 1) {
                $message = 'success';
            } else {
                $message = 'failed';
            }
        } else {
            $message = 'failed';
        }

        return array($deliveryCharge, $message);
    }

    public function getXYAxis($mapPath)
    {
        $xy = explode(',', $mapPath);
        $xyArray = array();
        if (!empty($xy) && is_array($xy)) {
            foreach ($xy as $x => $y) {
                $xyAxis = explode(' ', trim($y));
                $xyArray[] = array('y' => $xyAxis[0], 'x' => $xyAxis[1]);
            }
        }
        return $xyArray;
    }
}