<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomAppFeedback;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomAppFeedbackController extends Controller
{

    /**
     * List app-feedback
     * GET /tiffintom/app-feedback
     */
    public function index(Request $req)
    {
        $app_feedback = TiffintomAppFeedback::orderBy('id', 'DESC');

        $customer = Auth::user();
        if ($customer) {
            $app_feedback->where('customer_id', $customer->id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $app_feedback = $app_feedback->get();
        } else {
            $app_feedback = $app_feedback->paginate($req->query('per_page', 20));
        }

        if ($app_feedback->isEmpty()) Utils::error('No app feedbacks found!', 404);

        return $app_feedback;
    }

    /**
     * List app-feedback
     * GET /tiffintom/app-feedback/history
     */
    public function history(Request $req)
    {
        $app_feedback = TiffintomAppFeedback::selectRaw("app_feedback.*,users.first_name as customer_name")->leftJoin('users', 'users.id', '=', 'customer_id')->orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $app_feedback->whereRaw('(subject  LIKE ? OR message  LIKE ? OR users.first_name LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($customer_id = $req->query("customer_id", false)) {
            $app_feedback->where('customer_id', $customer_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $app_feedback = $app_feedback->get();
        } else {
            $app_feedback = $app_feedback->paginate($req->query('per_page', 20));
        }

        if ($app_feedback->isEmpty()) Utils::error('No app feedbacks found!', 404);

        return $app_feedback;
    }

    /**
     * Create app-feedback.
     * POST /tiffintom/app-feedback
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'customer_id' => 'required|string',
            'subject' => 'required|string',
            'message' => 'required|string',
            'response' => 'string',
        ]);

        // create app feedback
        $app_feedback = new TiffintomAppFeedback;
        $app_feedback->fill($req->all());
        $app_feedback->save();
        return $app_feedback;
    }

    /**
     * List app-feedback
     * GET /tiffintom/app-feedback/id
     */
    public function show(Request $req, $id)
    {
        $app_feedback = TiffintomAppFeedback::find($id);
        if (empty($app_feedback)) Utils::error("No app feedback found", 404);
        return $app_feedback;
    }

    /**
     * Update app-feedback.
     * POST  /tiffintom/app-feedback/id
     */
    public function update(Request $req, $id)
    {
        $app_feedback = TiffintomAppFeedback::find($id);
        if (empty($app_feedback)) Utils::error('app feedback not found!', 404);

        $this->validate($req, [
            'customer_id' => 'required|string',
            'subject' => 'required|string',
            'message' => 'required|string',
            'response' => 'string',
        ]);

        // update menu
        if ($req->has('customer_id')) $app_feedback->customer_id = $req->input('customer_id');
        if ($req->has('subject')) $app_feedback->subject = $req->input('subject');
        if ($req->has('message')) $app_feedback->message = $req->input('message');
        if ($req->has('response')) $app_feedback->response = $req->input('response');

        $app_feedback->save();
        return $app_feedback;
    }

    /**
     * Delete app-feedback by id.
     * DELETE /tiffintom/app-feedback/{id}
     */
    public function delete(Request $req, $id)
    {
        $app_feedback = TiffintomAppFeedback::find($id);
        if (empty($app_feedback)) Utils::error('app feedback not found!', 404);
        $app_feedback->delete();

        return $app_feedback;
    }
}