<?php

namespace App\Http\Controllers\Tiffintom;

use App\ActionLog;
use App\Business;
use App\Helpers\Auth;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\Helpers\PushIosNotificationTiffintomHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomBooking;
use App\ModelsTiffintom\TiffintomCart;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomReview;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomSystemNotification;
use App\ModelsTiffintom\TiffintomUser;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomBookingController extends Controller
{


    /**
     * List booking
     * GET /tiffintom/booking
     */
    public function index(Request $req)
    {
        $booking = TiffintomBooking::orderBy('id', 'DESC');

        if ($query = $req->query("restaurant_id", false)) {
            $booking->where('restaurant_id', $query);
            if ($req->hasHeader('business-id')) {
                $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
                if (empty($business)) Utils::error('User not authenticated.', 404);
                (new ExtraController())->userCheck($req->header('business-id'), $business->id);
            }
        }

        if ($query = $req->query("customer_id", false)) {
            $booking->where('customer_id', $query);
        }

        if ($query = $req->query('status')) {
            $booking->where('status', $query);
        }
        if ($previous_status = $req->query('previous_status')) {
            // $booking->whereRaw('(status LIKE ? OR status LIKE ? )', ["%Approved%", "%Cancel%"]);
            $fromDate = Carbon::today()->startOfDay()->format('Y-m-d');
            $booking->whereRaw('(status LIKE ? OR status LIKE ? AND booking_date <= ? )', ["%Cancel%", "%Approved%", $fromDate]);
        }
        if ($upcoming_status = $req->query('upcoming_status')) {
            $fromDate = Carbon::today()->startOfDay()->format('Y-m-d');
            $booking->whereRaw('(status LIKE ? OR status LIKE ? AND booking_date >= ? )', ["%Pending%", "%Approved%", $fromDate]);
        }
        if ($query = $req->query('from_date')) {
            $fromDate = Carbon::parse($query)->startOfDay();
            $booking->where('booking_date', '>=', $query);
        }
        if ($query = $req->query('to_date')) {
            $toDate = Carbon::parse($query)->endOfDay();
            $booking->where('booking_date', '<=', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $booking = $booking->get();
            $bookings = array();
            // $booking->each(function ($bookingd) {
            //     $restaurants = TiffintomRestaurant::where('id', $bookingd->restaurant_id)->first();
            //     $bookingd->restaurant = $restaurants;

            //     $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurants->id)->where('status', '1')->first();
            //     if (!empty($reviews)) {
            //         if ($reviews->count > 0) {
            //             $bookingd->restaurant->average_rating = $this->convertToDecimal($reviews->total_rating / $reviews->count);
            //             $bookingd->restaurant->total_reviews = $reviews->count;
            //             $bookingd['restaurant']['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
            //             $bookingd['restaurant']['total_reviews'] = $reviews->count;
            //         }
            //     }
            // });
            if (empty($booking) && !$req->query('append')) Utils::error('No booking found!', 404);
            return $booking;
        } else {
            $booking = $booking->paginate($req->query('per_page', 20));
        }

        if ($booking->isEmpty() && !$req->query('append')) Utils::error('No booking found!', 404);

        return $booking;
    }

    /**
     * Only for partner
     * List booking-home
     * GET /tiffintom/bookings/home
     */
    public function home(Request $req)
    {
        $startDate = Carbon::now()->startOfDay();
        $endDate = Carbon::now()->endOfDay();

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $pendingBookings = TiffintomBooking::where('status', 'Pending')->where('restaurant_id', $req->input('restaurant_id'))->orderBy('booking_date', "DESC")->get();
        $ApprovedBookings = TiffintomBooking::where('status', '!=', 'Pending')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->orderBy('booking_date', "DESC")->get();

        $response = [
            'pending_bookings' => $pendingBookings,
            'without_pending_bookings' => $ApprovedBookings,
        ];

        return $response;
    }

    /**
     * Only for partner
     * Update bookings.
     * POST  /tiffintom/bookings/id
     */
    public function change_status(Request $req, $id)
    {
        $booking = TiffintomBooking::find($id);
        if (empty($booking)) Utils::error('TiffintomBooking not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $booking->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'status' => 'required|string',
            'cancel_reason' => 'required_if:status,Cancel|string',
        ]);

        // update booking
        if ($req->has('status')) $booking->status = $req->input('status');
        if ($req->has('cancel_reason')) $booking->cancel_reason = $req->input('cancel_reason');

        $booking->save();

        $user = TiffintomUser::find($booking->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($booking->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($booking->status == 'Cancel') {
            $message = 'Hi ' . $user->first_name . ', we are sorry ' . $booking->cancel_reason . ' so we have had to cancel it.Dont worry you can still phone the restaurant on ' . $restaurant->restaurant_phone . ' if you are looking for alternative solution.';
            $title = "Your Reservation Is Declined";
            $sms_event = 'reservation_reject';
        } else if ($booking->status == 'Approved') {
            $message = 'Your reservation ' . $booking->booking_id . ' has been confirmed by ' . $restaurant->restaurant_name . '! See you soon! ';
            $title = "Your Reservation Is Accepted";
            $sms_event = 'reservation_accept';
        } else {
            Utils::error('Please pass valid booking status', 404);
        }

        // if ($user->phone_verify == 1) {
        if ($siteSettings->user_booking_sms == 'Yes') {
            if ($user->book_status == 2 || $user->book_status == 3 || $user->book_status == 6 || $user->book_status == 7) {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
        }
        // }

        if ($booking->status == 'Cancel') {
            $message = 'Your Reservation ' . $booking->booking_id . ' is declined due to ' . $booking->cancel_reason . ' Your Reservation decline.';
            $title = "Your Reservation Is Declined";
        } elseif ($booking->status == 'Approved') {
            $message = 'Your Reservation ' . $booking->booking_id . ' is accepted. See you soon! Your Reservation Accepted.';
            $title = "Your Reservation Is Accepted";
        } else {
            Utils::error('Please pass valid booking status', 404);
        }
        $notification['title']          = $title;
        $notification['body']        = $message;

        $notificationdata['data']['title']          = $title;
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type'    => "booking");
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');

        if ($user->book_status == 7 || $user->book_status == 1 || $user->book_status == 3 || $user->book_status == 5) {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        $this->bookingMail($booking->id);

        //Call Socket
        (new TiffintomRestaurantController())->restaurant_pending_count($req, $booking->restaurant_id);

        return $booking;
    }


    /**
     * Create booking.
     * POST /tiffintom/booking
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'booking_id' => 'string',
            'customer_name' => 'required|string',
            'guest_count' => 'required|string',
            'booking_email' => 'required|string',
            'booking_phone' => 'required|string',
            'booking_instruction' => 'string',
            'booking_date' => 'required|string',
            'booking_time' => 'required|string',
            'type' => 'required|string',
            'status' => 'string',
        ]);

        // create cart
        $booking = new TiffintomBooking;
        $booking->fill($req->all());
        $booking->save();

        $booking->booking_id = '#Book000' . $booking->id;
        $booking->booking_date = date('Y-m-d', strtotime($req->input('booking_date')));
        $booking->booking_time = date('D', strtotime($req->input('booking_date'))) . ' ' . $req->input('booking_time');
        $booking->save();

        $user = TiffintomUser::find($booking->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($booking->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $business = Business::where('restaurant_id', $booking->restaurant_id)->first();
        $auto_accept = false;
        if (!empty($business)) {
            if ($business->auto_accept_fields && $business->auto_accept_reservation == 'yes') {
                $today = strtolower(Carbon::parse($booking->booking_date)->format('l'));
                if ($business->auto_accept_fields[$today]['list'] == 1) {
                    $auto_accept = true;
                }
            }
        }

        // Restaurant Fcm Notification
        $message      = 'You have a new reservation request - ' . $booking->booking_id;
        $notification['title']          = "Newtable";
        $notification['body']        = $message;

        $notificationdata['data']['title']          = "Newtable";
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['booking_id']  = $booking->id;
        $notificationdata['data']['auto_accept']  = $auto_accept;
        $notificationdata['data']['order_status']  = 'Pending';
        $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type'    => "booking");
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
        $devices = explode(',', $restaurant->device_id);
        foreach ($devices as $device) {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $device, $notification, 'restaurant');
        }

        // User Fcm Notification
        if ($user->book_table == 7 || $user->book_table == 1 || $user->book_table == 5 || $user->book_table == 3) {
            $message      = 'Your Reservation ' . $booking->booking_id . ' is placed successfully.Please wait for restaurant confirmation.';
            $notification['title']          = "Your Reservation Has Been Placed";
            $notification['body']        = $message;

            $notificationdata['data']['title']          = "Your Reservation Has Been Placed";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['order_status']  = 'Pending';
            $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type'    => "booking");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        // All side Text message send on user mobile
        if ($user->book_table == 6 || $user->book_table == 2 || $user->book_table == 3 || $user->book_table == 7) {
            $message  = 'Your Reservation ' . $booking->booking_id . ' is placed successfully.Please wait for restaurant confirmation.';
            // if ($user->phone_verify == 1) {
            if ($siteSettings->user_booking_sms == 'Yes') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, 'reservation_place', $restaurant->sms_sender_id);
            }
            // }

            if ($siteSettings->restaurant_order_all == 'Yes') {
                $message = 'New online reservation from Tiffin Tom - ' . $booking->booking_id . ' Please check now! Thank you';
                // (new OtpHelper())->sendMessage($message, $restaurant->sms_phonenumber, 'Restaurant');
                (new OtpHelper())->sendMessage($message, $restaurant->sms_phonenumber, 'reservation_place', $restaurant->sms_sender_id);
            }

            if ($siteSettings->admin_order_all == 'Yes') {
                $message = 'New online reservation from Tiffin Tom - ' . $booking->booking_id;
                // (new OtpHelper())->sendMessage($message, $siteSettings->admin_phone, 'Sitesetting');
                (new OtpHelper())->sendMessage($message, $siteSettings->admin_phone, 'reservation_place', $restaurant->sms_sender_id);
            }

            $UsersList = TiffintomUser::where('status', '1')->where('role_id', '4')->get();
            foreach ($UsersList as $key => $value) {
                if ($value['sms_permissions'] != '') {
                    $sms_permissions = explode(',', $value['sms_permissions']);
                    if (in_array("Order", $sms_permissions)) {
                        $message = 'New online reservation from Tiffin Tom - ' . $booking->booking_id;
                        // (new OtpHelper())->sendMessage($message, $value['phone_number'], 'Sitesetting');
                        (new OtpHelper())->sendMessage($message, $value['phone_number'], 'reservation_place', $restaurant->sms_sender_id);
                    }
                }
            }
        }

        $this->bookingMail($booking->id);

        //Call Socket
        (new TiffintomRestaurantController())->restaurant_pending_count($req, $booking->restaurant_id);

        return $booking;
    }

    /**
     * List booking
     * GET /tiffintom/booking/id
     */
    public function show(Request $req, $id)
    {
        $booking = TiffintomBooking::find($id);
        if (empty($booking)) Utils::error("No booking found", 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $booking->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        return $booking;
    }

    /**
     * Update booking.
     * POST  /tiffintom/booking/id
     */
    public function update(Request $req, $id)
    {
        $booking = TiffintomBooking::find($id);
        if (empty($booking)) Utils::error('booking not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $booking->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'booking_id' => 'string',
            'customer_name' => 'required|string',
            'guest_count' => 'required|string',
            'booking_email' => 'required|string',
            'booking_phone' => 'required|string',
            'booking_instruction' => 'string',
            'booking_date' => 'required|string',
            'booking_time' => 'required|string',
            'type' => 'required|string',
            'status' => 'string',
            'cancel_reason' => 'string',
        ]);

        // update menu
        if ($req->has('customer_id')) $booking->customer_id = $req->input('customer_id');
        if ($req->has('restaurant_id')) $booking->restaurant_id = $req->input('restaurant_id');
        if ($req->has('booking_id')) $booking->booking_id = $req->input('booking_id');
        if ($req->has('customer_name')) $booking->customer_name = $req->input('customer_name');
        if ($req->has('guest_count')) $booking->guest_count = $req->input('guest_count');
        if ($req->has('booking_email')) $booking->booking_email = $req->input('booking_email');
        if ($req->has('booking_phone')) $booking->booking_phone = $req->input('booking_phone');
        if ($req->has('booking_instruction')) $booking->booking_instruction = $req->input('booking_instruction');
        if ($req->has('booking_date')) $booking->booking_date = $req->input('booking_date');
        if ($req->has('booking_time')) $booking->booking_time = $req->input('booking_time');
        if ($req->has('type')) $booking->type = $req->input('type');
        if ($req->has('status')) $booking->status = $req->input('status');
        if ($req->has('cancel_reason')) $booking->cancel_reason = $req->input('cancel_reason');

        $booking->save();

        //Call Socket
        (new TiffintomRestaurantController())->restaurant_pending_count($req, $booking->restaurant_id);

        return $booking;
    }

    /**
     * Delete booking by id.
     * DELETE /tiffintom/booking/{id}
     */
    public function delete(Request $req, $id)
    {
        $booking = TiffintomBooking::find($id);
        if (empty($booking)) Utils::error('booking not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $booking->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $booking->delete();

        return $booking;
    }

    /**
     * Delete booking by id.
     * DELETE /tiffintom/booking/bulk-delete
     */
    public function bulk_delete(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);

        $bookings = TiffintomBooking::whereIn('id', $req->input('ids'))->delete();
        $ids = $req->input('ids');
        foreach ($ids as $id) {

            $user = Auth::user();

            if (isset($_SERVER['HTTP_CLIENT_IP']))
                $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
            else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
                $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
            else if (isset($_SERVER['HTTP_X_FORWARDED']))
                $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
            else if (isset($_SERVER['HTTP_FORWARDED_FOR']))
                $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
            else if (isset($_SERVER['HTTP_FORWARDED']))
                $ipaddress = $_SERVER['HTTP_FORWARDED'];
            else if (isset($_SERVER['REMOTE_ADDR']))
                $ipaddress = $_SERVER['REMOTE_ADDR'];
            else
                $ipaddress = '';

            $action_log = new ActionLog();
            $action_log->user_type =  'tiffintom';
            $action_log->user_id =  $user->id;
            $action_log->business_id =  Auth::$business_id;
            $action_log->method =  "DELETE";
            $action_log->prefix =  'bookings';
            $action_log->end_point =  $id;
            $action_log->full_path =  '/epos/tiffintom/bookings/';
            $action_log->request_raw =  [];
            $action_log->ip =  $ipaddress;
            $action_log->save();
        }

        return $req;
    }

    public function bookingMail($id)
    {
        $booking = TiffintomBooking::find($id);
        if (empty($booking)) Utils::error("No booking found", 404);

        $restaurant = TiffintomRestaurant::find($booking->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $user = TiffintomUser::find($booking->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        if ($user->role_id == 5) {
            $colorCode = '#fc353a';
            $fontCode = '#FFFFFF';
            $buttonCode = '#fc353a';
            $logoUrl = env('APP_URL') . 'images/logo-gogrubz.png';
        } else {
            $colorCode = '#f1e0c6';
            $logoUrl = env('APP_URL') . 'images/logo-home.png';
            $buttonCode = '#f3cc2c';
            $fontCode = '#000000';
        }

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="' . $colorCode . '">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . $logoUrl . '">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';
        if ($booking->status == 'Pending') {
            $emailinfo .= '<tr>
            <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                    your reservation is not confirmed until the restaurant accepts or declines your request.
                </p>
            </td>
        </tr>';
        }
        $emailinfo .= '<tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
            <td style="display:inline-block; width:185px;vertical-align: top;">Restaurant Name</td>
            <td style="display:inline-block; width:50px; text-align:center;">:</td>
            <td style="display:inline-block; width:350px; color:#ee541e;">' . $restaurant->restaurant_name . '</td>
        </tr>
        <tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
            <td style="display:inline-block; width:185px;vertical-align: top;">Book Table id</td>
            <td style="display:inline-block; width:50px; text-align:center;">:</td>
            <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->booking_id . '</td>
        </tr>
        <tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
            <td style="display:inline-block; width:185px;vertical-align: top;">Guest Count</td>
            <td style="display:inline-block; width:50px; text-align:center;">:</td>
            <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->guest_count . '</td>
        </tr>
        <tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
            <td style="display:inline-block; width:185px;vertical-align: top;">Customer Name</td>
            <td style="display:inline-block; width:50px; text-align:center;">:</td>
            <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->customer_name . '</td>
        </tr> 
        <tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
            <td style="display:inline-block; width:185px;vertical-align: top;">Customer Email</td>
            <td style="display:inline-block; width:50px; text-align:center;">:</td>
            <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->booking_email . '</td>
        </tr> 
        <tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
            <td style="display:inline-block; width:185px;vertical-align: top;">Customer Phone</td>
            <td style="display:inline-block; width:50px; text-align:center;">:</td>
            <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->booking_phone . '</td>
        </tr>';

        if (!empty($booking->booking_instruction)) {
            $emailinfo .= '<tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
                <td style="display:inline-block; width:185px;vertical-align: top;">Instruction</td>
                <td style="display:inline-block; width:50px; text-align:center;">:</td>
                <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->booking_instruction . '</td>
            </tr>';
        }

        $emailinfo .= '<tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
                <td style="display:inline-block; width:185px;vertical-align: top;">Status</td>
                <td style="display:inline-block; width:50px; text-align:center;">:</td>
                <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->status . '</td>
            </tr>';

        $emailinfo .= '<tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
                <td style="display:inline-block; width:185px;vertical-align: top;">Booking Date</td>
                <td style="display:inline-block; width:50px; text-align:center;">:</td>
                <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->booking_date . '</td>
            </tr>';

        $emailinfo .= '<tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
                <td style="display:inline-block; width:185px;vertical-align: top;">Booking Time</td>
                <td style="display:inline-block; width:50px; text-align:center;">:</td>
                <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->booking_time . '</td>
            </tr>';


        if ($booking->status == 'Cancel') {
            $emailinfo .= '
            <tr style="display:block; width:610px; margin-bottom:15px;font:13px Arial, Helvetica, sans-serif;">
                <td style="display:inline-block; width:185px;vertical-align: top;">Cancel Reason</td>
                <td style="display:inline-block; width:50px; text-align:center;">:</td>
                <td style="display:inline-block; width:350px; color:#ee541e;">' . $booking->cancel_reason . '</td>
            </tr>';
        }
        if ($user->role_id != 5) {
            $emailinfo .= '  <tr>   
                   <td bgcolor="#fff" >
                       <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                           <tr>
                               <td style="padding: 30px 30px; text-align: right;">
                                       <table align="left" style="text-align: left;">
                                           <tr>
                                               <td>
                                                      <p style="color:#000;"> Get The App : </p> 
                                               </td>
                                                   <td width="10">&nbsp;</td>
                                               <td>
                                                   <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                   <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                               </td>
                                                   <td width="10">&nbsp;</td>
                                               <td>
                                                   <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                   <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                               </td>
                                                   <td width="10">&nbsp;</td>
                                           </tr>
                                           
                                       </table>
                                       <table align="right" style="text-align: right;">
                                       <tr>
                                           <td>
                                                   <p style="color:#000;">Follow Us : </p>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td><td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td><td width="10">&nbsp;</td>
                                       </tr>
                                   </table>
                               </td>
                           </tr>
                       </table>
                   </td>
                </tr>';
        }
        $emailinfo .= '<tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="' . $colorCode . '"><div style="color:' . $fontCode . ';font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        if ($user->book_table == 7 || $user->book_table == 4 || $user->book_table == 5 || $user->book_table == 6) {
            if ($user->email_verify == 1) {
                $email['body'] = $emailinfo;
                $email['subject'] = 'Your reservation status for ' . $booking->booking_id . ' has been ' . $booking->status;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $booking->booking_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }
        }

        // Restaurant side Mail
        $email['body'] = $emailinfo;
        $email['subject'] = 'Your reservation status for ' . $booking->booking_id . ' has been ' . $booking->status;
        $email['mail_type'] = $siteSettings->mail_option;
        $email['to'] = $restaurant->contact_email;
        $email['from'] = $booking->booking_email;
        (new MailHelper())->sendMail($email);

        // Admin side Mail with permission
        if ($siteSettings->admin_booking_place_email == 'Yes' && $booking->status == 'Pending') {
            $email['body'] = $emailinfo;
            $email['subject'] = 'Your reservation status for ' . $booking->booking_id . ' has been ' . $booking->status;
            $email['mail_type'] = $siteSettings->mail_option;
            if ($user->role_id != 5) {
                $email['to'] = $siteSettings->gogrubz_order_email;
            } else {
                $email['to'] = $siteSettings->order_email;
            }
            $email['from'] = $booking->booking_email;
            (new MailHelper())->sendMail($email);
        }
        if ($siteSettings->admin_booking_accept_email == 'Yes' && $booking->status == 'Approved') {
            $email['body'] = $emailinfo;
            $email['subject'] = 'Your reservation status for ' . $booking->booking_id . ' has been ' . $booking->status;
            $email['mail_type'] = $siteSettings->mail_option;
            if ($user->role_id != 5) {
                $email['to'] = $siteSettings->gogrubz_order_email;
            } else {
                $email['to'] = $siteSettings->order_email;
            }
            $email['from'] = $booking->booking_email;
            (new MailHelper())->sendMail($email);
        }
        if ($siteSettings->admin_booking_reject_email == 'Yes' && $booking->status == 'Cancel') {
            $email['body'] = $emailinfo;
            $email['subject'] = 'Your reservation status for ' . $booking->booking_id . ' has been ' . $booking->status;
            $email['mail_type'] = $siteSettings->mail_option;
            if ($user->role_id != 5) {
                $email['to'] = $siteSettings->gogrubz_order_email;
            } else {
                $email['to'] = $siteSettings->order_email;
            }
            $email['from'] = $booking->booking_email;
            (new MailHelper())->sendMail($email);
        }

        return 'Mail Send Successfully';
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
