<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCard;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCardController extends Controller
{
    /**
     * List cards
     * GET /tiffintom/cards
     */
    public function index(Request $req)
    {
        $cards = TiffintomCard::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $cards->whereRaw('(name LIKE ? OR email LIKE ? OR phone LIKE ? OR status LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $cards = $cards->get();
        } else {
            $cards = $cards->paginate($req->query('per_page', 20));
        }

        if ($cards->isEmpty()) Utils::error('Cards not found!', 404);

        return $cards;
    }

    /**
     * Create card.
     * POST /tiffintom/cards
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'email' => 'string',
            'phone' => 'string',
            'status' => 'string',
        ]);

        // create card
        $card = new TiffintomCard;
        $card->fill($req->all());
        $card->save();
        return $card;
    }

    /**
     * List cards
     * GET /tiffintom/cards/id
     */
    public function show(Request $req, $id)
    {
        $cards = TiffintomCard::find($id);
        if (empty($cards)) Utils::error("Card not found", 404);
        return $cards;
    }

    /**
     * Update card.
     * POST  /tiffintom/cards/id
     */
    public function update(Request $req, $id)
    {
        $card = TiffintomCard::find($id);
        if (empty($card)) Utils::error('Card not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
            'email' => 'string',
            'phone' => 'string',
            'status' => 'string',
        ]);

        // update card
        if ($req->has('name')) $card->name = $req->input('name');
        if ($req->has('email')) $card->email = $req->input('email');
        if ($req->has('phone')) $card->phone = $req->input('email');
        if ($req->has('status')) $card->status = $req->input('email');
        $card->save();

        return $card;
    }

    /**
     * Delete card by id.
     * DELETE /tiffintom/cards/{id}
     */
    public function delete(Request $req, $id)
    {
        $card = TiffintomCard::find($id);
        if (empty($card)) Utils::error('Card not found!', 404);
        $card->delete();
        return $card;
    }
}
