<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCart;
use App\ModelsTiffintom\TiffintomMenu;
use App\ModelsTiffintom\TiffintomMenuVariant;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCartController extends Controller
{


    /**
     * List cart
     * GET /tiffintom/cart
     */
    public function index(Request $req)
    {
        $cart = TiffintomCart::orderBy('id', 'DESC');

        if ($query = $req->query("restaurant_id", false)) {
            $cart->where('restaurant_id', $query);
        }
        $customer = Auth::user();
        if ($customer) {
            $cart->where('customer_id', $customer->id);
            $cart->where('order_id', 0);
        }

        // if ($customer_id = $req->query("customer_id", false)) {
        //     $cart->where('customer_id', $customer_id);
        //     $cart->where('order_id', 0);
        // }

        if ($req->query('nopaginate', false) == '1') {
            $cart = $cart->get();
        } else {
            $cart = $cart->paginate($req->query('per_page', 20));
        }

        if ($cart->isEmpty()) Utils::error('No cart found!', 404);

        return $cart;
    }

    /**
     * List cart
     * GET /tiffintom/cart/history
     */
    public function history(Request $req)
    {
        $cart = TiffintomCart::orderBy('id', 'DESC');

        if ($query = $req->query("restaurant_id", false)) {
            $cart->where('restaurant_id', $query);
        }

        if ($req->input("order_id") != '') {
            $cart->where('order_id', $req->input("order_id"));
        }

        if ($customer_id = $req->query("customer_id", false)) {
            $cart->where('customer_id', $customer_id);
        }

        if ($restaurant_id = $req->query("restaurant_id", false)) {
            $cart->where('restaurant_id', $restaurant_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $cart = $cart->get();
        } else {
            $cart = $cart->paginate($req->query('per_page', 20));
        }

        if ($cart->isEmpty()) Utils::error('No cart found!', 404);

        return $cart;
    }

    /**
     * Create cart.
     * POST /tiffintom/cart
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'order_id' => 'exists:tiffintom_connection.orders,id',
            'menu_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'menu_name' => 'string',
            'subaddons_name' => 'string',
            'menu_price' => 'numeric',
            'total_price' => 'numeric',
            'quantity' => 'numeric',
            'menu_description' => 'numeric',
        ]);

        // create cart

        //Other Restaurant cart
        $Excarts = TiffintomCart::where('customer_id', $req->input('customer_id'))->where('restaurant_id', '!=', $req->input('restaurant_id'))->where('order_id', 0)->get();
        if (!$Excarts->isEmpty()) $carts = TiffintomCart::where('customer_id', $req->input('customer_id'))->where('order_id', 0)->delete();

        $cartObj = TiffintomCart::where('menu_id', $req->input('menu_id'))->where('subaddons_name', '')->where('customer_id', $req->input('customer_id'))->where('order_id', 0)->first();
        if (empty($cartObj)) {
            $cartObj = TiffintomCart::where('menu_id', $req->input('menu_id'))->where('subaddons_name', $req->input('subaddons_name'))->where('customer_id', $req->input('customer_id'))->whereNotNull('subaddons_name')->where('order_id', 0)->first();
        }
        if (empty($cartObj)) {
            $cartObj = new TiffintomCart;
            $cartObj->fill($req->all());
            $cartObj->save();
        } else {
            $cartObj->quantity = $cartObj->quantity + 1;
            // $cartMenuObj = TiffintomMenuVariant::where('menu_id', $req->input('menu_id'))->where('orginal_price', '!=', $cartObj->menu_price)->first();
            if (!$req->has('subaddons_name')) {
                $cartMenuObj = TiffintomMenu::selectRaw('menu_details.orginal_price as orginal_price')->leftJoin('menu_details', 'menu_details.menu_id', '=', 'restaurant_menus.id')->where('menu_details.menu_id', $req->input('menu_id'))->where('menu_details.orginal_price', '!=', $cartObj->menu_price)->where('restaurant_menus.product_percentage', '<=', 0)->first();
                if (empty($cartMenuObj)) {
                    $cartObj->total_price = $cartObj->menu_price * $cartObj->quantity;
                } else {
                    $cartObj->menu_price = $cartMenuObj->orginal_price;
                    $cartObj->total_price = $cartMenuObj->orginal_price * $cartObj->quantity;
                }
            } else {
                $cartObj->menu_price = $req->input('menu_price');
                $cartObj->total_price = $cartObj->menu_price * $cartObj->quantity;
            }
            $cartObj->save();
        }

        $carts = TiffintomCart::where('customer_id', $req->input('customer_id'))->where('order_id', 0)->orderBy('id', 'DESC')->get();
        if ($carts->isEmpty()) Utils::error('No cart found!', 404);
        return $carts;
    }

    /**
     * List cart
     * GET /tiffintom/cart/id
     */
    public function show(Request $req, $id)
    {
        $cart = TiffintomCart::where('id', $id);
        if (empty($cart)) Utils::error("No product found", 404);
        $cart = $cart->first();

        return $cart;
    }

    /**
     * Update cart.
     * POST  /tiffintom/cart/id
     */
    public function update(Request $req, $id)
    {
        $cart = TiffintomCart::find($id);
        if (empty($cart)) Utils::error('cart not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'order_id' => 'exists:tiffintom_connection.orders,id',
            'menu_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'menu_name' => 'string',
            'subaddons_name' => 'string',
            'menu_price' => 'numeric',
            'total_price' => 'numeric',
            'quantity' => 'numeric',
            'menu_description' => 'numeric',
        ]);

        // update menu
        if ($req->has('restaurant_id')) $cart->restaurant_id = $req->input('restaurant_id');
        if ($req->has('customer_id')) $cart->customer_id = $req->input('customer_id');
        if ($req->has('order_id')) $cart->order_id = $req->input('order_id');
        if ($req->has('menu_id')) $cart->menu_id = $req->input('menu_id');
        if ($req->has('menu_name')) $cart->menu_name = $req->input('menu_name');
        if ($req->has('subaddons_name')) $cart->subaddons_name = $req->input('subaddons_name');

        if ($req->has('subaddons_name')) {
            $str_arr = explode(",", $req->input('subaddons_name'));
            $subProductName = $str_arr;
            $cartMenuObj = array();
            // $cartMenuObj = TiffintomMenu::selectRaw('menu_details.orginal_price as orginal_price')->leftJoin('menu_details', 'menu_details.menu_id', '=', 'restaurant_menus.id')->where('menu_details.menu_id', $req->input('menu_id'))->where('menu_details.sub_name', $subProductName)->where('menu_details.orginal_price', '!=', $req->input('menu_price'))->where('restaurant_menus.product_percentage', '<=', 0)->first();
        } else {
            $cartMenuObj = TiffintomMenu::selectRaw('menu_details.orginal_price as orginal_price')->leftJoin('menu_details', 'menu_details.menu_id', '=', 'restaurant_menus.id')->where('menu_details.menu_id', $req->input('menu_id'))->where('menu_details.orginal_price', '!=', $req->input('menu_price'))->where('restaurant_menus.product_percentage', '<=', 0)->first();
        }

        // $cartMenuObj = TiffintomMenuVariant::where('menu_id', $req->input('menu_id'))->where('orginal_price', '!=', $req->input('menu_price'))->first();
        if (empty($cartMenuObj)) {
            if ($req->has('menu_price')) $cart->menu_price = $req->input('menu_price');
            if ($req->has('total_price')) $cart->total_price = $req->input('menu_price') * $req->input('quantity');
        } else {
            $cart->menu_price = $cartMenuObj->orginal_price;
            $cart->total_price = $cartMenuObj->orginal_price * $req->input('quantity');
        }
        if ($req->has('quantity')) $cart->quantity = $req->input('quantity');
        if ($req->has('menu_description')) $cart->menu_description = $req->input('menu_description');
        $cart->save();

        $carts = TiffintomCart::where('customer_id', $req->input('customer_id'))->where('order_id', 0)->orderBy('id', 'DESC')->get();
        if ($carts->isEmpty()) Utils::error('No cart found!', 404);
        return $carts;
    }

    /**
     * Delete cart by id.
     * DELETE /tiffintom/cart/{id}
     */
    public function delete(Request $req, $id)
    {
        $cart = TiffintomCart::find($id);
        if (empty($cart)) Utils::error('cart not found!', 404);
        $cart->delete();

        $carts = TiffintomCart::where('customer_id', $cart->customer_id)->where('order_id', 0)->orderBy('id', 'DESC')->get();
        if ($carts->isEmpty()) Utils::error('No cart found!', 404);
        return $carts;
    }

    /**
     * bulk cart by id.
     * POST /tiffintom/cart/bulk-cart
     */
    public function cart_bulk(Request $req)
    {
        if ($req->has('carts') && !is_array($req->input('carts'))) {
            $req->merge(['carts' => json_decode($req->input('carts'))]);
        }
        $this->validate($req, [
            'carts' => 'required|array'
        ]);

        if ($req->has('customer_id')) {
            $carts = TiffintomCart::where('customer_id', $req->input('customer_id'))->where('order_id', 0)->delete();
        }

        $carts = ($req->input('carts'));
        foreach ($carts as $cart) {
            $cartObj = new TiffintomCart((array)$cart);
            $cartObj->save();
        }
        return $carts;
    }

    /**
     * Delete All cart.
     * GET All/epos/cart/delete-all/
     */
    public function delete_all(Request $req)
    {
        $customer = Auth::user();
        $carts = TiffintomCart::where('customer_id', $customer->id)->where('order_id', 0)->delete();

        return ['message' => 'orders reset done!'];
    }
}
