<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomChat;
use App\ModelsTiffintom\TiffintomOrderHelp;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class TiffintomChatController extends Controller
{


    /**
     * List chat
     * GET /tiffintom/chat
     */
    public function index(Request $req)
    {
        $chat = TiffintomChat::selectRaw("chat.*,users.first_name as first_name")->leftJoin('users', 'users.id', '=', 'user_id')->orderBy('id', 'ASC');

        if ($query = $req->query("query", false)) {
            $chat->whereRaw('(users.first_name LIKE ?)', ["%$query%"]);
        }

        if ($user_group = $req->query('user_group', false)) {
            $chat->join(DB::raw('(SELECT user_id, MAX(created) as latest FROM chat GROUP BY user_id) as r'), 'chat.created', '=', 'r.latest');
            // $chat->groupBy('user_id');
        }

        if ($restaurant_group = $req->query('restaurant_group', false)) {
            $user_id = $req->input('user_id');
            $chat->join(DB::raw('(SELECT restaurant_id, MAX(created) as latest FROM chat where user_id=' . $user_id . ' GROUP BY restaurant_id) as r'), 'chat.created', '=', 'r.latest');
            // $chat->groupBy('restaurant_id');
        }

        if ($admin_group = $req->query('admin_group', false)) {
            $chat->join(DB::raw('(SELECT user_id, MAX(created) as latest FROM chat GROUP BY user_id) as r'), 'chat.created', '=', 'r.latest');
        }

        if ($order_id = $req->query('order_id', false)) {
            $chat->where('order_id', $order_id);
        }

        // if ($restaurant_id = $req->query('restaurant_id', false)) {
        //     $chat->where('restaurant_id', $restaurant_id);
        // }

        if ($user_id = $req->query('user_id', false)) {
            $chat->where('user_id', $user_id);
        }

        if ($admin = $req->query('admin', false)) {
            $chat->where('admin', 1);
        }

        if ($last_id = $req->query('last_id', false)) {
            $chat->where('id', '>', $last_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $chat = $chat->get();
        } else {
            $chat = $chat->paginate($req->query('per_page', 20));
        }

        if ($chat->isEmpty()) Utils::error('No chat found!', 404);

        //Mark Messages as seen
        if (Auth::isTiffintomUser()) {
            TiffintomChat::where('user_id', $req->query('user_id'))->where('admin', '1')->update(['status' => '0']);
        }
        return $chat;
    }

    /**
     * Create chat.
     * POST /tiffintom/chat
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'user_id' => 'exists:tiffintom_connection.users,id',
            'admin' => 'string',
            'message' => 'string',
            'status' => 'string',
            'is_attach' => 'string',
            'attach' => 'mimes:jpeg,png,jpg,mp3,m4a,mp4',
        ]);

        // create chat
        $chat = new TiffintomChat;
        $chat->fill($req->all());
        if ($req->has('attach')) {
            $fileName = ImageHelper::saveImageToServer($req->file('attach'), ImageHelper::$TYPE_TIFFINTOM_ORDERHELP, 'tiffintom');
            $chat->message = $fileName;
        }
        $chat->status = 1;
        $chat->save();

        $user = TiffintomUser::find($chat->user_id);

        $restaurant = TiffintomRestaurant::find($chat->restaurant_id);

        if ($req->input('admin') == 0 && !empty($restaurant)) {
            $message = 'You have a new message from ' . $user->first_name;
            $notification['title']          = "Chat Message";
            $notification['body']        = $message;

            $notificationdata['data']['title'] = "Chat message";
            $notificationdata['data']['message'] = $message;
            $notificationdata['data']['is_background'] = false;
            $notificationdata['data']['order_id'] = $chat->order_id;
            $notificationdata['data']['restaurant_id'] = $chat->restaurant_id;
            $notificationdata['data']['payload'] = array('ChatDetails' => "", 'type' => "tiffintomchat");
            $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
            $Devices = explode(',', $restaurant->device_id);
            foreach ($Devices as $Device) {
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device, $notification, 'restaurant');
            }
        }

        if ($req->input('admin') == 1) {
            $message = 'You have a new message from TiffinTom support.';
            $notification['title']          = "Chat Message";
            $notification['body']        = $message;

            $notificationdata['data']['title'] = "Chat message";
            $notificationdata['data']['message'] = $message;
            $notificationdata['data']['is_background'] = false;
            $notificationdata['data']['order_id'] = $chat->order_id;
            $notificationdata['data']['payload'] = array('ChatDetails' => "", 'type' => "tiffintomchat");
            $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        return $chat;
    }

    /**
     * List chat
     * GET /tiffintom/chat/id
     */
    public function show(Request $req, $id)
    {
        $chat = TiffintomChat::find($id);
        if (empty($chat)) Utils::error("No chat found", 404);
        return $chat;
    }

    /**
     * Update chat.
     * POST  /tiffintom/chat/id
     */
    public function update(Request $req, $id)
    {
        $chat = TiffintomChat::find($id);
        if (empty($chat)) Utils::error('chat not found!', 404);

        $this->validate($req, [
            'order_id' => 'exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'user_id' => 'exists:tiffintom_connection.users,id',
            'admin' => 'string',
            'message' => 'string',
            'status' => 'string',
            'is_attach' => 'string',
        ]);

        // update menu
        if ($req->has('user_id')) $chat->user_id = $req->input('user_id');
        if ($req->has('order_id')) $chat->order_id = $req->input('order_id');
        if ($req->has('restaurant_id')) $chat->restaurant_id = $req->input('restaurant_id');
        if ($req->has('message')) $chat->message = $req->input('message');
        $chat->admin = $req->input('admin');
        $chat->status = $req->input('status');
        $chat->is_attach = $req->input('is_attach');

        $chat->save();
        return $chat;
    }

    /**
     * Delete chat by id.
     * DELETE /tiffintom/chat/{id}
     */
    public function delete(Request $req, $id)
    {
        $chat = TiffintomChat::find($id);
        if (empty($chat)) Utils::error('chat not found!', 404);
        $chat->delete();

        return $chat;
    }

    /**
     * Delete chat by id.
     * DELETE /tiffintom/chat/{id}/delete-user
     */
    public function delete_user(Request $req, $id)
    {
        $order_helps = TiffintomChat::where('user_id', $id)->delete();
        return ['message' => 'chat deleted successfully!'];
    }

    /**
     * get chat count.
     * GET /tiffintom/chat/chat-all-count
     */
    public function chat_all_count(Request $req)
    {
        $chat = TiffintomChat::orderBy('id', 'ASC');
        if ($customer_id = $req->query('customer_id', false)) {
            $chat->where('user_id', $customer_id);
        }
        // if ($restaurant_id = $req->query('restaurant_id', false)) {
        //     $chat->where('restaurant_id', $restaurant_id);
        // }
        $chat->where('admin', 1);
        $chat->where('status', 1);
        $chat = $chat->count();

        $order_help = TiffintomOrderHelp::orderBy('id', 'ASC');
        if ($customer_id = $req->query('customer_id', false)) {
            $order_help->where('customer_id', $customer_id);
        }
        if ($restaurant_id = $req->query('restaurant_id', false)) {
            $order_help->where('restaurant_id', $restaurant_id);
        }
        $order_help->where('admin', 1);
        $order_help->where('status', 1);
        $order_help = $order_help->count();

        $Total = $chat + $order_help;

        $data = [
            'chat_count' => $chat,
            'order_help_count' => $order_help,
            'total_count' => $Total,
        ];
        return $data;
    }

    /**
     * get static chat after Order.
     * GET /tiffintom/chat/chat-order-question
     */
    public function order_question_chat(Request $req)
    {
        $someJSON = '[
        {"question":"Please select the issue with this order.",
            "questions":[
                {"question":"I have the wrong amount of items in my order","type":"sub","sub_question":[{"question":"We are very sorry that you havent received your order. You can view the status of your order by clicking the link below. Alternatively you can call the restaurant and request an update on your order.Check your order status."}]},
                {"question":"I have not received my order","type":"sub","sub_question":[{"question":"We are very sorry that you havent received your order. You can view the status of your order by clicking the link below. Alternatively you can call the restaurant and request an update on your order.Check your order status."}]},
                {"question":"Items are missing or incorrect in order","type":"sub","sub_question":[{"question":"We are very sorry that you havent received your order. You can view the status of your order by clicking the link below. Alternatively you can call the restaurant and request an update on your order.Check your order status."}]},
                {"question":"I have a food taste or quality issue with my order","type":"sub","sub_question":[{ "question":"We are very sorry that you havent received your order. You can view the status of your order by clicking the link below. Alternatively you can call the restaurant and request an update on your order.Check your order status."}]},
                {"question":"I have a referral code related query","type":"sub","sub_question":[{ "question":"We are very sorry that you havent received your order. You can view the status of your order by clicking the link below. Alternatively you can call the restaurant and request an update on your order.Check your order status."}]},
                {"question":"Other Help","type":"sub","sub_question":[{ "question":"We are very sorry that you havent received your order. You can view the status of your order by clicking the link below. Alternatively you can call the restaurant and request an update on your order.Check your order status."}]}
            ]
        }]';
        $someArray = json_decode($someJSON, true);

        return $someArray;
    }

    /**
     * get static chat general question.
     * POST /tiffintom/chat/chat-general-question
     */
    public function order_general_chat(Request $req)
    {
        $user = TiffintomUser::where('id', $req->input('id'))->first();
        if (empty($user)) Utils::error("No user found", 404);

        $usersDetails = $user->first_name . " (" . $user->username . ", " . $user->phone_number . ")";

        $someJSON = '[
        {"question":"Welcome to TiffinTom chat support, how can we help today?",
            "questions":[
                {"question":"Hello, ' . $usersDetails . ', please select the department you need assistance with"},
                {"question":"Admin","type":"sub","sub_question":[{"question":"We are connecting you to one of our support executives."}]},
                {"question":"Sales & Marketing","type":"sub","sub_question":[{"question":"We are connecting you to one of our support executives."}]},
                {"question":"Order Related Help","type":"order_help"},
                {"question":"Other Support","type":"sub","sub_question":[{"question":"We are connecting you to one of our support executives."}]},
                {"question":"Join as a new Restaurant","type":"sub","sub_question":[{"question":"We are connecting you to one of our support executives."}]}
            ]
        }]';
        $someArray = json_decode($someJSON, true);

        return $someArray;
    }
}
