<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCheckoutList;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCheckoutListController extends Controller
{


    /**
     * List checkout-list
     * GET /tiffintom/checkout-list
     */
    public function index(Request $req)
    {
        $checkout_list = TiffintomCheckoutList::selectRaw("checkout_list.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('checkout_list.id', 'DESC');

        if ($restaurant_id = $req->query("restaurant_id", false)) {
            $checkout_list->where('restaurant_id', $restaurant_id);
        }

        if ($order_type = $req->query("order_type", false)) {
            $checkout_list->where('checkout_list.order_type', $order_type);
        }

        if ($status = $req->query("status", false)) {
            $checkout_list->where('checkout_list.status', $status);
        }

        if ($all_type = $req->query("all_type", false)) {
            $checkout_list->whereRaw('(checkout_list.order_type = ? OR checkout_list.order_type = ?)', [$all_type, 'both']);
        }

        $checkout_list = $checkout_list->get();

        if ($checkout_list->isEmpty()) Utils::error('No checkout list found!', 404);

        return $checkout_list;
    }

    /**
     * List checkout-list
     * GET /tiffintom/checkout-list/history
     */
    public function history(Request $req)
    {
        $checkout_list = TiffintomCheckoutList::selectRaw("checkout_list.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('checkout_list.id', 'DESC');

        if ($restaurant_id = $req->query("restaurant_id", false)) {
            $checkout_list->where('restaurant_id', $restaurant_id);
        }

        if ($order_type = $req->query("order_type", false)) {
            $checkout_list->where('checkout_list.order_type', $order_type);
        }

        if ($status = $req->query("status", false)) {
            $checkout_list->where('checkout_list.status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $checkout_list = $checkout_list->get();
        } else {
            $checkout_list = $checkout_list->paginate($req->query('per_page', 20));
        }

        if ($checkout_list->isEmpty()) Utils::error('No checkout list found!', 404);

        return $checkout_list;
    }

    /**
     * Create checkout-list.
     * POST /tiffintom/checkout-lists
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'order_type' => 'required|string',
            'message' => 'required|string',
            'status' => 'boolean',
        ]);

        // create checkout_list
        $checkout_list = new TiffintomCheckoutList;
        $checkout_list->fill($req->all());
        $checkout_list->status = '1';
        $checkout_list->save();
        return $checkout_list;
    }

    /**
     * List checkout-lists
     * GET /tiffintom/checkout-lists/id
     */
    public function show(Request $req, $id)
    {
        $checkout_lists = TiffintomCheckoutList::find($id);
        if (empty($checkout_lists)) Utils::error("Checkout list not found", 404);
        return $checkout_lists;
    }

    /**
     * Update checkout-list.
     * POST  /tiffintom/checkout-lists/id
     */
    public function update(Request $req, $id)
    {
        $checkout_list = TiffintomCheckoutList::find($id);
        if (empty($checkout_list)) Utils::error('Checkout List not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'order_type' => 'required|string',
            'message' => 'required|string',
            'status' => 'boolean',
        ]);

        // update checkout_list
        if ($req->has('restaurant_id')) $checkout_list->restaurant_id = $req->input('restaurant_id');
        if ($req->has('order_type')) $checkout_list->order_type = $req->input('order_type');
        if ($req->has('message')) $checkout_list->message = $req->input('message');
        $checkout_list->status = $req->input('status');
        $checkout_list->save();

        return $checkout_list;
    }

    /**
     * Delete checkout-list by id.
     * DELETE /tiffintom/checkout-lists/{id}
     */
    public function delete(Request $req, $id)
    {
        $checkout_list = TiffintomCheckoutList::find($id);
        if (empty($checkout_list)) Utils::error('Checkout List not found!', 404);
        $checkout_list->delete();
        return $checkout_list;
    }

    /**
     * Enable checkout-list.
     * POST /tiffintom/checkout-lists/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $checkout_list = TiffintomCheckoutList::find($id);
        if (empty($checkout_list)) Utils::error('Checkout List not found!', 404);
        $checkout_list->status = '1';
        $checkout_list->save();
        return $checkout_list;
    }

    /**
     * Disable checkout-list.
     * POST /tiffintom/checkout-lists/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $checkout_list = TiffintomCheckoutList::find($id);
        if (empty($checkout_list)) Utils::error('Checkout List not found!', 404);
        $checkout_list->status = '0';
        $checkout_list->save();
        return $checkout_list;
    }
}
