<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCity;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCityController extends Controller
{
    /**
     * List cities
     * GET /tiffintom/cities
     */
    public function index(Request $req)
    {
        $cities = TiffintomCity::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $cities->whereRaw('(city_name LIKE ?)', ["%$query%"]);
        }

        if ($country_id = $req->query("country_id", false)) {
            $cities->where('country_id', $country_id);
        }

        if ($state_id = $req->query("state_id", false)) {
            $cities->where('state_id', $state_id);
        }

        if ($status = $req->query("status", false)) {
            $cities->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $cities = $cities->get();
        } else {
            $cities = $cities->paginate($req->query('per_page', 20));
        }

        if ($cities->isEmpty()) Utils::error('Cities not found!', 404);

        return $cities;
    }
    /**
     * List cities
     * GET /tiffintom/cities/history
     */
    public function history(Request $req)
    {
        $cities = TiffintomCity::orderBy('id', 'DESC')->groupBy('city_name');
        if ($status = $req->query("status", false)) {
            $cities->where('status', $status);
        }
        $cities = $cities->get();

        if ($cities->isEmpty()) Utils::error('Cities not found!', 404);

        $groupedArray = [];
        foreach ($cities as $element) {
            $initialLetter = strtoupper(substr($element->city_name, 0, 1));
            if (!isset($groupedArray[$initialLetter])) {
                $groupedArray[$initialLetter] = [];
            }
            $groupedArray[$initialLetter][] = $element;
        }

        return $groupedArray;
    }

    /**
     * Create city.
     * POST /tiffintom/cities
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'country_id' => 'required|exists:tiffintom_connection.countries,id',
            'state_id' => 'required|exists:tiffintom_connection.states,id',
            'city_name' => 'required|string',
        ]);

        // create city
        $city = new TiffintomCity;
        $city->fill($req->all());
        $city->status = '1';
        $city->save();
        return $city;
    }

    /**
     * List cities
     * GET /tiffintom/cities/id
     */
    public function show(Request $req, $id)
    {
        $cities = TiffintomCity::find($id);
        if (empty($cities)) Utils::error("City not found", 404);
        return $cities;
    }

    /**
     * Update city.
     * POST  /tiffintom/cities/id
     */
    public function update(Request $req, $id)
    {
        $city = TiffintomCity::find($id);
        if (empty($city)) Utils::error('City not found!', 404);

        $this->validate($req, [
            'country_id' => 'required|exists:tiffintom_connection.countries,id',
            'state_id' => 'required|exists:tiffintom_connection.states,id',
            'city_name' => 'required|string',
        ]);

        // update country
        if ($req->has('country_id')) $city->country_id = $req->input('country_id');
        if ($req->has('state_id')) $city->state_id = $req->input('state_id');
        if ($req->has('city_name')) $city->city_name = $req->input('city_name');
        $city->save();

        return $city;
    }

    /**
     * Delete city by id.
     * DELETE /tiffintom/cities/{id}
     */
    public function delete(Request $req, $id)
    {
        $city = TiffintomCity::find($id);
        if (empty($city)) Utils::error('City not found!', 404);
        $city->delete();
        return $city;
    }

    /**
     * Enable city.
     * POST /tiffintom/cities/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $city = TiffintomCity::find($id);
        if (empty($city)) Utils::error('City not found!', 404);
        $city->status = '1';
        $city->save();
        return $city;
    }

    /**
     * Disable city.
     * POST /tiffintom/cities/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $city = TiffintomCity::find($id);
        if (empty($city)) Utils::error('City not found!', 404);
        $city->status = '0';
        $city->save();
        return $city;
    }
}
