<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\OtpHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomComplaint;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class TiffintomComplaintController extends Controller
{


    /**
     * List complaints
     * GET /tiffintom/complaints
     */
    public function index(Request $req)
    {
        $complaints = TiffintomComplaint::orderBy('id', 'ASC');

        if ($admin_group = $req->query('admin_group', false)) {
            $complaints->join(DB::raw('(SELECT order_id, MAX(created) as latest FROM complaint GROUP BY order_id) as r'), 'complaint.created', '=', 'r.latest');
        }

        if ($order_id = $req->query('order_id', false)) {
            $complaints->where('order_id', $order_id);
        }

        if ($user_id = $req->query('user_id', false)) {
            $complaints->where('user_id', $user_id);
        }

        if ($admin = $req->query('admin', false)) {
            $complaints->where('admin', 1);
        }

        if ($status = $req->query('status', false)) {
            if ($status == 'open') {
                $complaints->where('close', 0);
            }
            if ($status == 'close') {
                $complaints->where('close', 1);
            }
        }

        if ($last_id = $req->query('last_id', false)) {
            $complaints->where('id', '>', $last_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $complaints = $complaints->get();
        } else {
            $complaints = $complaints->paginate($req->query('per_page', 20));
        }

        if ($complaints->isEmpty()) Utils::error('No complaints found!', 404);

        //Mark Messages as seen
        if (Auth::isTiffintomUser()) {
            TiffintomComplaint::where('user_id', $req->query('user_id'))->where('admin', '1')->update(['status' => '0']);
        }

        return $complaints;
    }

    /**
     * Create complaints.
     * POST /tiffintom/complaints
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'user_id' => 'exists:tiffintom_connection.users,id',
            'admin' => 'string',
            'message' => 'string',
            'status' => 'boolean',
            'close' => 'boolean',
        ]);

        // create complaints
        $complaint = new TiffintomComplaint;
        $complaint->fill($req->all());
        $complaint->status = 1;
        $complaint->save();

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $user = TiffintomUser::find($complaint->user_id);

        if ($siteSettings->complaint_all == 'Yes') {
            $message = 'Complaint - ORD00' . $complaint->order_id . ',' . $complaint->message;
            (new OtpHelper())->sendMessage($message, $user->phone_number, 'otp_service');
        }

        return $complaint;
    }

    /**
     * List complaints
     * GET /tiffintom/complaints/id
     */
    public function show(Request $req, $id)
    {
        $complaint = TiffintomComplaint::find($id);
        if (empty($complaint)) Utils::error("Complaint not found", 404);
        return $complaint;
    }

    /**
     * Update complaints.
     * POST  /tiffintom/complaints/id
     */
    public function update(Request $req, $id)
    {
        $complaint = TiffintomComplaint::find($id);
        if (empty($complaint)) Utils::error('complaint not found!', 404);

        $this->validate($req, [
            'order_id' => 'exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'user_id' => 'exists:tiffintom_connection.users,id',
            'admin' => 'string',
            'message' => 'string',
            'status' => 'boolean',
            'close' => 'boolean',
        ]);

        // update menu
        if ($req->has('user_id')) $complaint->user_id = $req->input('user_id');
        if ($req->has('order_id')) $complaint->order_id = $req->input('order_id');
        if ($req->has('restaurant_id')) $complaint->restaurant_id = $req->input('restaurant_id');
        if ($req->has('message')) $complaint->message = $req->input('message');
        $complaint->admin = $req->input('admin');
        $complaint->status = $req->input('status');
        $complaint->close = $req->input('close');

        $complaint->save();
        return $complaint;
    }

    /**
     * Delete complaints by id.
     * DELETE /tiffintom/complaints/{id}
     */
    public function delete(Request $req, $id)
    {
        $complaint = TiffintomComplaint::find($id);
        if (empty($complaint)) Utils::error('complaint not found!', 404);
        $complaint->delete();

        return $complaint;
    }

    /**
     * Delete complaints by id.
     * DELETE /tiffintom/complaints/{id}/delete-user
     */
    public function delete_user(Request $req, $id)
    {
        $complaint = TiffintomComplaint::where('user_id', $id)->delete();
        return ['message' => 'complaints deleted successfully!'];
    }

    /**
     * Enable account-help.
     * POST /tiffintom/account-helps/{id}/open
     */
    public function open(Request $req, $id)
    {
        $complaints = TiffintomComplaint::where('order_id', $id)->update(['close' => '0']);
        return $complaints;
    }

    /**
     * Disable account-help.
     * POST /tiffintom/account-helps/{id}/close
     */
    public function close(Request $req, $id)
    {
        $complaints = TiffintomComplaint::where('order_id', $id)->update(['close' => '1']);
        return $complaints;
    }
}
