<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomContact;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomContactController extends Controller
{
    /**
     * List contacts
     * GET /tiffintom/contacts
     */
    public function index(Request $req)
    {
        $contacts = TiffintomContact::orderBy('id', 'DESC');

        if ($query = $req->query('query', false)) {
            $contacts->whereRaw('(name LIKE ? OR email LIKE ? OR phone LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($verify = $req->query("verify", false)) {
            $contacts->where('verify', $verify);
        }

        if ($req->query('nopaginate', false) == '1') {
            $contacts = $contacts->get();
        } else {
            $contacts = $contacts->paginate($req->query('per_page', 20));
        }

        if ($contacts->isEmpty()) Utils::error('contacts not found!', 404);

        return $contacts;
    }


    /**
     * Create cart.
     * POST /tiffintom/contacts
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'email' => 'required|string',
            'phone' => 'required|string',
            'subject' => 'required|string',
            'message' => 'required|string',
            'department' => 'required|string',
            'otp' => 'string',
            'status' => 'boolean',
            'verify' => 'boolean',
        ]);

        $contact = new TiffintomContact;
        $contact->fill($req->all());
        $contact->save();

        return $contact;
    }

    /**
     * List contacts
     * GET /tiffintom/contacts/id
     */
    public function show(Request $req, $id)
    {
        $contact = TiffintomContact::find($id);
        if (empty($contact)) Utils::error("contact not found", 404);
        return $contact;
    }

    /**
     * Update contacts.
     * POST  /tiffintom/contacts/id
     */
    public function update(Request $req, $id)
    {
        $contact = TiffintomContact::find($id);
        if (empty($contact)) Utils::error('contact not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
            'email' => 'required|string',
            'phone' => 'required|string',
            'subject' => 'required|string',
            'message' => 'required|string',
            'department' => 'required|string',
            'otp' => 'string',
            'response' => 'string',
            'status' => 'boolean',
            'verify' => 'boolean',
        ]);

        // update contact
        if ($req->has('response')) $contact->response = $req->input('response');
        $contact->save();

        if ($contact->response) {
            $this->sendMail($contact->id);
        }
        return $contact;
    }

    /**
     * Delete contacts by id.
     * DELETE /tiffintom/contacts/{id}
     */
    public function delete(Request $req, $id)
    {
        $contact = TiffintomContact::find($id);
        if (empty($contact)) Utils::error('contact not found!', 404);
        $contact->delete();
        return $contact;
    }

    /**
     * Enable contacts.
     * POST /tiffintom/contacts/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $contact = TiffintomContact::find($id);
        if (empty($contact)) Utils::error('contact not found!', 404);
        $contact->status = '1';
        $contact->save();
        return $contact;
    }

    /**
     * Disable contacts.
     * POST /tiffintom/contacts/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $contact = TiffintomContact::find($id);
        if (empty($contact)) Utils::error('contact not found!', 404);
        $contact->status = '0';
        $contact->save();
        return $contact;
    }

    public function sendMail($id)
    {
        $contact = TiffintomContact::find($id);
        if (empty($contact)) Utils::error("No contact found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#f1e0c6">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/logo-home.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $contact->first_name . ',<br>
                             ' . $contact->response . '<br>
                             </p>
                          </td>
                       </tr>
                         <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                            <center><a href="#"><button style="background-color: #f3cc2c;border: none;color: black;padding: 15px 32px;text-align: center;text-decoration: none;display: inline-block;font-size: 20px;margin: 4px 2px;cursor: pointer;">Log In</button></a></center>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#f1e0c6"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Tiffintom Response',
            'mail_type' => $siteSettings->mail_option,
            'to' => $contact->email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }
}
