<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCountry;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCountryController extends Controller
{
    /**
     * List countries
     * GET /tiffintom/countries
     */
    public function index(Request $req)
    {
        $countries = TiffintomCountry::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $countries->whereRaw('(country_name LIKE ? OR currency_name LIKE ? OR currency_code LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($status = $req->query("status", false)) {
            $countries->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $countries = $countries->get();
        } else {
            $countries = $countries->paginate($req->query('per_page', 20));
        }

        if ($countries->isEmpty()) Utils::error('Countries not found!', 404);

        return $countries;
    }

    /**
     * Create country.
     * POST /tiffintom/countries
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'country_name' => 'required|string',
            'currency_name' => 'required|string',
            'currency_code' => 'required|string',
            'currency_symbol' => 'required|string',
            'phone_code' => 'required|string',
            'iso_code' => 'required|string',
            'status' => 'boolean',
        ]);

        // create country
        $country = new TiffintomCountry;
        $country->fill($req->all());
        $country->status = '1';
        $country->save();
        return $country;
    }

    /**
     * List countries
     * GET /tiffintom/countries/id
     */
    public function show(Request $req, $id)
    {
        $countries = TiffintomCountry::find($id);
        if (empty($countries)) Utils::error("Country not found", 404);
        return $countries;
    }

    /**
     * Update country.
     * POST  /tiffintom/countries/id
     */
    public function update(Request $req, $id)
    {
        $country = TiffintomCountry::find($id);
        if (empty($country)) Utils::error('Country not found!', 404);

        $this->validate($req, [
            'country_name' => 'required|string',
            'currency_name' => 'required|string',
            'currency_code' => 'required|string',
            'currency_symbol' => 'required|string',
            'phone_code' => 'required|string',
            'iso_code' => 'required|string',
            'status' => 'boolean',
        ]);

        // update country
        if ($req->has('country_name')) $country->country_name = $req->input('country_name');
        if ($req->has('currency_name')) $country->currency_name = $req->input('currency_name');
        if ($req->has('currency_code')) $country->currency_code = $req->input('currency_code');
        if ($req->has('currency_symbol')) $country->currency_symbol = $req->input('currency_symbol');
        if ($req->has('phone_code')) $country->phone_code = $req->input('phone_code');
        if ($req->has('iso_code')) $country->iso_code= $req->input('iso_code');
        $country->save();

        return $country;
    }

    /**
     * Delete country by id.
     * DELETE /tiffintom/countries/{id}
     */
    public function delete(Request $req, $id)
    {
        $country = TiffintomCountry::find($id);
        if (empty($country)) Utils::error('Country not found!', 404);
        $country->delete();
        return $country;
    }

    /**
     * Enable country.
     * POST /tiffintom/countries/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $country = TiffintomCountry::find($id);
        if (empty($country)) Utils::error('Country not found!', 404);
        $country->status = '1';
        $country->save();
        return $country;
    }

    /**
     * Disable country.
     * POST /tiffintom/countries/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $country = TiffintomCountry::find($id);
        if (empty($country)) Utils::error('Country not found!', 404);
        $country->status = '0';
        $country->save();
        return $country;
    }
}