<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomCuisine;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCuisineController extends Controller
{

    /**
     * List cuisine
     * GET /tiffintom/cuisine
     */
    public function index(Request $req)
    {
        $cuisine = TiffintomCuisine::orderBy('id', 'DESC');
        if (empty($cuisine)) Utils::error("No cuisine found", 404);

        if ($req->query('nopaginate', false) == '1') {
            $cuisine = $cuisine->get();
        } else {
            $cuisine = $cuisine->paginate($req->query('per_page', 20));
        }

        if ($cuisine->isEmpty()) Utils::error('No cuisine found!', 404);

        return $cuisine;
    }

    /**
     * Create cuisine.
     * POST /tiffintom/cuisine
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'cuisine_name' => 'string',
            'cuisine_image' => 'image',
            'cuisine_seo' => 'string',
            'first_bg' => 'string',
            'second_bg' => 'string',
            'status' => 'boolean',
            'delete_status' => 'string',
        ]);

        // create cuisine
        $cuisine = new TiffintomCuisine;
        $cuisine->fill($req->all());
        if ($req->has('cuisine_name')) $cuisine->cuisine_name = (new ExtraController())->convertToTitleCase($req->input('cuisine_name'));;
        if ($req->has('cuisine_name')) $cuisine->cuisine_seo = $this->seoUrl($cuisine->cuisine_name);
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_CUISINE, 'tiffintom');
            $cuisine->cuisine_image = $fileName;
        }
        $cuisine->save();

        return $cuisine;
    }

    /**
     * List cuisine
     * GET /tiffintom/cuisine/id
     */
    public function show(Request $req, $id)
    {
        $cuisine = TiffintomCuisine::find($id);
        if (empty($cuisine)) Utils::error("No cuisine found", 404);

        return $cuisine;
    }

    /**
     * Update cuisine.
     * POST  /tiffintom/cuisine/id
     */
    public function update(Request $req, $id)
    {
        $cuisine = TiffintomCuisine::find($id);
        if (empty($cuisine)) Utils::error('cuisine not found!', 404);

        $this->validate($req, [
            'cuisine_name' => 'string',
            'cuisine_image' => 'image',
            'cuisine_seo' => 'string',
            'first_bg' => 'string',
            'second_bg' => 'string',
            'status' => 'boolean',
            'delete_status' => 'string',
        ]);

        // update menu
        if ($req->has('cuisine_name')) $cuisine->cuisine_name = $req->input('cuisine_name');
        if ($req->has('cuisine_seo')) $cuisine->cuisine_seo = $req->input('cuisine_seo');
        if ($req->has('first_bg')) $cuisine->first_bg = $req->input('first_bg');
        if ($req->has('second_bg')) $cuisine->second_bg = $req->input('second_bg');
        if ($req->has('delete_status')) $cuisine->delete_status = $req->input('delete_status');
        $cuisine->status = $req->input('status');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_CUISINE, 'tiffintom');
            $cuisine->cuisine_image = $fileName;
        }

        $cuisine->save();
        return $cuisine;
    }

    /**
     * Delete cuisine by id.
     * DELETE /tiffintom/cuisine/{id}
     */
    public function delete(Request $req, $id)
    {
        $cuisine = TiffintomCuisine::find($id);
        if (empty($cuisine)) Utils::error('cuisine not found!', 404);
        $cuisine->delete();

        return $cuisine;
    }

    /**
     * Enable cuisine.
     * POST /tiffintom/cuisine/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $cuisine = TiffintomCuisine::find($id);
        if (empty($cuisine)) Utils::error('cuisine not found!', 404);
        $cuisine->status = '1';
        $cuisine->save();
        return $cuisine;
    }

    /**
     * Disable cuisine.
     * POST /tiffintom/cuisine/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $cuisine = TiffintomCuisine::find($id);
        if (empty($cuisine)) Utils::error('cuisine not found!', 404);
        $cuisine->status = '0';
        $cuisine->save();
        return $cuisine;
    }

    //Convert to seo url name
    public function seoUrl($string)
    {
        $text = preg_replace('~[^\\pL\d]+~u', '-', $string);

        // trim
        $text = trim($text, '-');

        // lowercase
        $text = strtolower($text);

        // remove unwanted characters
        $text = preg_replace('~[^-\w]+~', '', $text);

        if (strlen($text) > 70) {
            $text = substr($text, 0, 70);
        }

        if (empty($text)) {
            //return 'n-a';
            return time();
        }

        return $text;
    }
}