<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCustomerReward;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomReward;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCustomerRewardController extends Controller
{

    /**
     * List reward
     * GET /tiffintom/reward
     */
    public function index(Request $req)
    {
        $rewards = TiffintomCustomerReward::orderBy('id', 'DESC');

        $customer = Auth::user();
        if ($customer) {
            $rewards->where('customer_id', $customer->id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $rewards = $rewards->get();
        } else {
            $rewards = $rewards->paginate($req->query('per_page', 20));
        }

        if ($rewards->isEmpty()) Utils::error('No reward found!', 404);

        if (!$rewards->isEmpty()) {
            $rewards->each(function ($reward) {
                $main_reward = TiffintomReward::where('id', 1)->first();
                if ($reward->type == 'Spent') {
                    $reward->reward_totalpoint = $main_reward->reward_totalpoint;
                } else {
                    $reward->reward_totalpoint = '';
                }
                $reward->reward_validity = $main_reward->reward_validity;
                $order = TiffintomOrder::where('id', $reward->order_id)->where('reward_used', 'Y')->first();
                if (!empty($order)) {
                    $reward->reward_offer = $order->reward_offer;
                } else {
                    $reward->reward_offer = '';
                }
            });
        }
        return $rewards;
    }

    /**
     * Create reward.
     * POST /tiffintom/reward
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'required|exists:tiffintom_connection.orders,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'restaurant_name' => 'string',
            'total' => 'string',
            'points' => 'string',
            'type' => 'string',
            'status' => 'string',
        ]);

        // create reward
        $reward = new TiffintomCustomerReward;
        $reward->fill($req->all());
        $reward->save();
        return $reward;
    }

    /**
     * List reward
     * GET /tiffintom/reward/id
     */
    public function show(Request $req, $id)
    {
        $reward = TiffintomCustomerReward::find($id);
        if (empty($reward)) Utils::error("No reward found", 404);
        return $reward;
    }

    /**
     * Update reward.
     * POST  /tiffintom/reward/id
     */
    public function update(Request $req, $id)
    {
        $reward = TiffintomCustomerReward::find($id);
        if (empty($reward)) Utils::error('reward not found!', 404);

        $this->validate($req, [
            'order_id' => 'required|exists:tiffintom_connection.orders,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'restaurant_name' => 'string',
            'total' => 'string',
            'points' => 'string',
            'type' => 'string',
            'status' => 'string',
        ]);

        // update menu
        if ($req->has('order_id')) $reward->order_id = $req->input('order_id');
        if ($req->has('customer_id')) $reward->customer_id = $req->input('customer_id');
        if ($req->has('restaurant_name')) $reward->restaurant_name = $req->input('restaurant_name');
        if ($req->has('total')) $reward->total = $req->input('total');
        if ($req->has('points')) $reward->points = $req->input('points');
        if ($req->has('type')) $reward->type = $req->input('type');
        if ($req->has('status')) $reward->status = $req->input('status');

        $reward->save();
        return $reward;
    }

    /**
     * Delete reward by id.
     * DELETE /tiffintom/reward/{id}
     */
    public function delete(Request $req, $id)
    {
        $reward = TiffintomCustomerReward::find($id);
        if (empty($reward)) Utils::error('reward not found!', 404);
        $reward->delete();

        return $reward;
    }
}
