<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomInvoices;
use App\ModelsTiffintom\TiffintomBooking;
use App\ModelsTiffintom\TiffintomCart;
use App\ModelsTiffintom\TiffintomChat;
use App\ModelsTiffintom\TiffintomComplaint;
use App\ModelsTiffintom\TiffintomContact;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomPartnerMedia;
use App\ModelsTiffintom\TiffintomPaymentTransaction;
use App\ModelsTiffintom\TiffintomReferralPending;
use App\ModelsTiffintom\TiffintomReview;
use App\ModelsTiffintom\TiffintomUser;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomRestaurantRegistration;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomDashboardController extends Controller
{

    /**
     * List dashboard/admin-dashboard
     * GET /tiffintom/dashboard/admin-dashboard
     */
    public function admin_dashboard()
    {
        $orders = collect(TiffintomOrder::orderByRaw('id DESC')->get());
        $bookings = collect(TiffintomBooking::orderByRaw('id DESC')->get());
        $users = collect(TiffintomUser::where('role_id', '3')->where('deleted_status', 'N')->orderByRaw('id DESC')->get());
        $restaurants = collect(TiffintomRestaurant::orderByRaw('id DESC')->get());
        $complaints = collect(TiffintomComplaint::orderByRaw('id DESC')->get());
        $chats = collect(TiffintomChat::orderByRaw('id DESC')->get());

        // Users
        $total_users = $users->count();
        $total_active_users = $users->where('status', '0')->count();
        $total_deactive_users = $users->where('status', '1')->count();

        // Restaurants
        $currentDay = strtolower(date('l'));
        $total_restaurants = $restaurants->count();
        $total_active_restaurants = $restaurants->where('status', '1')->count();
        $total_close_restaurants = $restaurants->where('close', '1')->count();
        $total_deactive_restaurants = $restaurants->where('status', '0')->count();
        $total_close_delivery_restaurants = $restaurants->where('status', '1')->where('restaurant_delivery', 'Yes')->where($currentDay . '_status', 'Close')->count();
        $total_deactive_order_restaurants = $restaurants->where('online_order', 'No')->count();
        $total_close_pickup_restaurants = TiffintomRestaurant::leftJoin('restaurant_timings', 'restaurant_timings.restaurant_id', '=', 'restaurants.id')->where('status', '1')->where('restaurant_pickup', 'Yes')->where('delete_status', 'N')->where('restaurant_timings.pick_' . $currentDay . '_status', 'Close')->count();

        //Orders
        $total_orders = $orders->count();
        $total_incomplete_orders = $orders->where('status', 'Accepted')->count();
        $total_delivered_orders = $orders->where('status', 'Delivered')->count();
        $value_delivered_orders = $orders->where('status', 'Delivered')->sum('order_grand_total');
        $total_reject_orders = $orders->where('status', 'Failed')->count();
        $value_reject_orders = $orders->where('status', 'Failed')->sum('order_grand_total');
        $total_delivery_pending_orders = $orders->where('status', 'Pending')->where('order_type', 'delivery')->count();
        $value_delivery_pending_orders = $orders->where('status', 'Pending')->where('order_type', 'delivery')->sum('order_grand_total');
        $total_pickup_pending_orders = $orders->where('status', 'Pending')->where('order_type', 'pickup')->count();
        $value_pickup_pending_orders = $orders->where('status', 'Pending')->where('order_type', 'pickup')->sum('order_grand_total');

        //Bookings
        $total_bookings = $bookings->count();
        $total_pending_bookings = $bookings->where('status', 'Pending')->count();
        $total_accepted_bookings = $bookings->where('status', 'Approve')->count();
        $total_rejected_bookings = $bookings->where('status', 'Cancel')->count();

        // Contacts
        $total_unverify_contacts = TiffintomContact::where('status', '0')->count();
        $total_verify_contacts = TiffintomContact::where('status', '1')->count();

        // Carts
        $total_carts = TiffintomCart::where('order_id', '0')->where('customer_id', '!=', '0')->groupBy('customer_id')->count();

        // Complaints
        $total_complaints = $complaints->where('close', '!=', '1')->groupBy('order_id')->count();
        $total_unread_complaints = $complaints->where('status', '1')->where('admin', '0')->where('close', '0')->groupBy('order_id')->count();

        // Chats
        $total_chats = $chats->where('admin', '0')->where('status', '1')->groupBy('user_id')->count();

        // Referrals
        $total_referrals = TiffintomReferralPending::where('status', 'Eligible')->count();

        // Payment Disable
        $cod_payments = TiffintomRestaurant::leftJoin('restaurant_payments', 'restaurant_payments.restaurant_id', '=', 'restaurants.id')->where('restaurant_payments.payment_id', '7')->whereRaw('(restaurant_payments.payment_status = ? OR restaurant_payments.android_status = ? OR restaurant_payments.ios_status = ?)', ['N', 'N', 'N'])->orderByRaw('restaurants.id DESC')->count();
        $stipe_payments = TiffintomRestaurant::leftJoin('restaurant_payments', 'restaurant_payments.restaurant_id', '=', 'restaurants.id')->where('restaurant_payments.payment_id', '8')->whereRaw('(restaurant_payments.payment_status = ? OR restaurant_payments.android_status = ? OR restaurant_payments.ios_status = ?)', ['N', 'N', 'N'])->orderByRaw('restaurants.id DESC')->count();
        $paypal_payments = TiffintomRestaurant::leftJoin('restaurant_payments', 'restaurant_payments.restaurant_id', '=', 'restaurants.id')->where('restaurant_payments.payment_id', '9')->whereRaw('(restaurant_payments.payment_status = ? OR restaurant_payments.android_status = ? OR restaurant_payments.ios_status = ?)', ['N', 'N', 'N'])->orderByRaw('restaurants.id DESC')->count();

        // Restaurant Registrations
        $total_registrations = TiffintomRestaurantRegistration::where('status', '1')->count();

        $data = [
            // 'complete_order_list' => $complete_orders,
            'stats' => [
                'total_users' => $total_users,
                'total_active_users' => $total_active_users,
                'total_deactive_users' => $total_deactive_users,
                'total_restaurants' => $total_restaurants,
                'total_active_restaurants' => $total_active_restaurants,
                'total_close_restaurants' => $total_close_restaurants,
                'total_deactive_restaurants' => $total_deactive_restaurants,
                'total_close_delivery_restaurants' => $total_close_delivery_restaurants,
                'total_deactive_order_restaurants' => $total_deactive_order_restaurants,
                'total_close_pickup_restaurants' => $total_close_pickup_restaurants,
                'total_orders' => $total_orders,
                'total_incomplete_orders' => $total_incomplete_orders,
                'total_delivered_orders' => $total_delivered_orders,
                'value_delivered_orders' => $value_delivered_orders,
                'total_reject_orders' => $total_reject_orders,
                'value_reject_orders' => $value_reject_orders,
                'total_delivery_pending_orders' => $total_delivery_pending_orders,
                'value_delivery_pending_orders' => $value_delivery_pending_orders,
                'total_pickup_pending_orders' => $total_pickup_pending_orders,
                'value_pickup_pending_orders' => $value_pickup_pending_orders,
                'total_bookings' => $total_bookings,
                'total_pending_bookings' => $total_pending_bookings,
                'total_accepted_bookings' => $total_accepted_bookings,
                'total_rejected_bookings' => $total_rejected_bookings,
                'total_carts' => $total_carts,
                'total_complaints' => $total_complaints,
                'total_unread_complaints' => $total_unread_complaints,
                'total_chats' => $total_chats,
                'total_referrals' => $total_referrals,
                'cod_payments' => $cod_payments,
                'stipe_payments' => $stipe_payments,
                'paypal_payments' => $paypal_payments,
                'total_registrations' => $total_registrations,
                'total_verify_contacts' => $total_verify_contacts,
                'total_unverify_contacts' => $total_unverify_contacts,
            ]
        ];
        return $data;
    }


    /**
     * List dashboard/overview
     * GET /tiffintom/dashboard/overview
     */
    public function dashboard_overview(Request $req)
    {
        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $start_date = date('Y-m-d');
        $end_date = date('Y-m-d');
        $week_start_date = date('Y-m-d', strtotime("monday this week"));
        $week_end_date = date('Y-m-d', strtotime("sunday this week"));
        $month_start_date = date('Y-m-d', strtotime("first day of this month"));
        $month_end_date = date('Y-m-d', strtotime("last day of this month"));
        $previous_start_date = date('Y-m-d', strtotime($start_date . ' - 1 days'));
        $previous_end_date = date('Y-m-d', strtotime($end_date . ' - 1 days'));
        $previous_week_start_date = date('Y-m-d', strtotime("last week monday"));
        $previous_week_end_date = date('Y-m-d', strtotime("last week sunday"));
        $previous_month_start_date = date('Y-m-d', strtotime('first day of last month'));
        $previous_month_end_date = date('Y-m-d', strtotime('last day of last month'));

        // Payment Wise
        $todayOrderPayments = TiffintomOrder::selectRaw("COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total,payment_method as payment_method_name")->where('restaurant_id', $req->input('restaurant_id'))->where('status', 'Delivered')->whereRaw('(delivery_date >= ? AND delivery_date <= ?)', [$start_date, $end_date])->groupBy('payment_method')->get();

        $weekOrderPayments = TiffintomOrder::selectRaw("COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total,payment_method as payment_method_name")->where('restaurant_id', $req->input('restaurant_id'))->where('status', 'Delivered')->whereRaw('(delivery_date > ? AND delivery_date < ?)', [$week_start_date, $week_end_date])->groupBy('payment_method')->get();

        $monthOrderPayments = TiffintomOrder::selectRaw("COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total,payment_method as payment_method_name")->where('restaurant_id', $req->input('restaurant_id'))->where('status', 'Delivered')->whereRaw('(delivery_date > ? AND delivery_date < ?)', [$month_start_date, $month_end_date])->groupBy('payment_method')->get();

        $previousDayOrderPayments = TiffintomOrder::selectRaw("COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total,payment_method as payment_method_name")->where('restaurant_id', $req->input('restaurant_id'))->where('status', 'Delivered')->whereRaw('(delivery_date > ? AND delivery_date < ?)', [$previous_start_date, $previous_end_date])->groupBy('payment_method')->get();

        $previousWeekOrderPayments = TiffintomOrder::selectRaw("COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total,payment_method as payment_method_name")->where('restaurant_id', $req->input('restaurant_id'))->where('status', 'Delivered')->whereRaw('(delivery_date > ? AND delivery_date < ?)', [$previous_week_start_date, $previous_week_end_date])->groupBy('payment_method')->get();

        $previousMonthOrderPayments = TiffintomOrder::selectRaw("COUNT(*) as count,CAST(sum(order_grand_total) AS DECIMAL(7,2)) as total,payment_method as payment_method_name")->where('restaurant_id', $req->input('restaurant_id'))->where('status', 'Delivered')->whereRaw('(delivery_date > ? AND delivery_date < ?)', [$previous_month_start_date, $previous_month_end_date])->groupBy('payment_method')->get();

        //COD ARRAY
        $codarray = array();
        $codarray['daily']['count'] = $this->search($todayOrderPayments, 'cod', 'count');
        $codarray['daily']['price'] = $this->search($todayOrderPayments, 'cod', 'total');
        $codarray['daily']['previous_count'] = $this->convertToDecimal(($this->search($todayOrderPayments, 'cod', 'count') <= 0 || $this->search($previousDayOrderPayments, 'cod', 'total') <= 0) ? (($this->search($todayOrderPayments, 'cod', 'count') > 0 && $this->search($previousDayOrderPayments, 'cod', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'cod', 'count') - $this->search($previousDayOrderPayments, 'cod', 'total')) * 100) / $this->search($previousDayOrderPayments, 'cod', 'total'));
        $codarray['daily']['previous_percentage'] = $this->convertToDecimal(($this->search($todayOrderPayments, 'cod', 'total') <= 0 || $this->search($previousDayOrderPayments, 'cod', 'total') <= 0) ? (($this->search($todayOrderPayments, 'cod', 'total') > 0 && $this->search($previousDayOrderPayments, 'cod', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'cod', 'total') - $this->search($previousDayOrderPayments, 'cod', 'total')) * 100) / $this->search($previousDayOrderPayments, 'cod', 'total'));
        $codarray['weekly']['count'] = $this->search($weekOrderPayments, 'cod', 'count');
        $codarray['weekly']['price'] = $this->search($weekOrderPayments, 'cod', 'total');
        $codarray['weekly']['previous_count'] = $this->convertToDecimal(($this->search($weekOrderPayments, 'cod', 'count') <= 0 || $this->search($previousWeekOrderPayments, 'cod', 'count') <= 0) ? (($this->search($weekOrderPayments, 'cod', 'count') > 0 && $this->search($previousWeekOrderPayments, 'cod', 'count') <= 0) ? 100 : 0) : (($this->search($weekOrderPayments, 'cod', 'count') - $this->search($previousWeekOrderPayments, 'cod', 'count')) * 100) / $this->search($previousWeekOrderPayments, 'cod', 'count'));
        $codarray['weekly']['previous_percentage'] = $this->convertToDecimal(($this->search($weekOrderPayments, 'cod', 'total') <= 0 || $this->search($previousWeekOrderPayments, 'cod', 'total') <= 0) ? (($this->search($weekOrderPayments, 'cod', 'total') > 0 && $this->search($previousWeekOrderPayments, 'cod', 'total') <= 0) ? 100 : 0) : (($this->search($weekOrderPayments, 'cod', 'total') - $this->search($previousWeekOrderPayments, 'cod', 'total')) * 100) / $this->search($previousWeekOrderPayments, 'cod', 'total'));
        $codarray['monthly']['count'] = $this->search($monthOrderPayments, 'cod', 'count');
        $codarray['monthly']['price'] = $this->search($monthOrderPayments, 'cod', 'total');
        $codarray['monthly']['previous_count'] = $this->convertToDecimal(($this->search($monthOrderPayments, 'cod', 'count') <= 0 || $this->search($previousMonthOrderPayments, 'cod', 'count') <= 0) ? (($this->search($monthOrderPayments, 'cod', 'count') > 0 && $this->search($previousMonthOrderPayments, 'cod', 'count') <= 0) ? 100 : 0) : (($this->search($monthOrderPayments, 'cod', 'count') - $this->search($previousMonthOrderPayments, 'cod', 'count')) * 100) / $this->search($previousMonthOrderPayments, 'cod', 'count'));
        $codarray['monthly']['previous_percentage'] = $this->convertToDecimal(($this->search($monthOrderPayments, 'cod', 'total') <= 0 || $this->search($previousMonthOrderPayments, 'cod', 'total') <= 0) ? (($this->search($monthOrderPayments, 'cod', 'total') > 0 && $this->search($previousMonthOrderPayments, 'cod', 'total') <= 0) ? 100 : 0) : (($this->search($monthOrderPayments, 'cod', 'total') - $this->search($previousMonthOrderPayments, 'cod', 'total')) * 100) / $this->search($previousMonthOrderPayments, 'cod', 'total'));

        //PAYPAL ARRAY
        $paypalarray = array();
        $paypalarray['daily']['count'] = $this->search($todayOrderPayments, 'paypal', 'count');
        $paypalarray['daily']['price'] = $this->search($todayOrderPayments, 'paypal', 'total');
        $paypalarray['daily']['previous_count'] = $this->convertToDecimal(($this->search($todayOrderPayments, 'paypal', 'count') <= 0 || $this->search($previousDayOrderPayments, 'paypal', 'total') <= 0) ? (($this->search($todayOrderPayments, 'paypal', 'count') > 0 && $this->search($previousDayOrderPayments, 'paypal', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'paypal', 'count') - $this->search($previousDayOrderPayments, 'paypal', 'total')) * 100) / $this->search($previousDayOrderPayments, 'paypal', 'total'));
        $paypalarray['daily']['previous_percentage'] = $this->convertToDecimal(($this->search($todayOrderPayments, 'paypal', 'total') <= 0 || $this->search($previousDayOrderPayments, 'paypal', 'total') <= 0) ? (($this->search($todayOrderPayments, 'paypal', 'total') > 0 && $this->search($previousDayOrderPayments, 'paypal', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'paypal', 'total') - $this->search($previousDayOrderPayments, 'paypal', 'total')) * 100) / $this->search($previousDayOrderPayments, 'paypal', 'total'));
        $paypalarray['weekly']['count'] = $this->search($weekOrderPayments, 'paypal', 'count');
        $paypalarray['weekly']['price'] = $this->search($weekOrderPayments, 'paypal', 'total');
        $paypalarray['weekly']['previous_count'] = $this->convertToDecimal(($this->search($weekOrderPayments, 'paypal', 'count') <= 0 || $this->search($previousWeekOrderPayments, 'paypal', 'count') <= 0) ? (($this->search($weekOrderPayments, 'paypal', 'count') > 0 && $this->search($previousWeekOrderPayments, 'paypal', 'count') <= 0) ? 100 : 0) : (($this->search($weekOrderPayments, 'paypal', 'count') - $this->search($previousWeekOrderPayments, 'paypal', 'count')) * 100) / $this->search($previousWeekOrderPayments, 'paypal', 'count'));
        $paypalarray['weekly']['previous_percentage'] = $this->convertToDecimal(($this->search($weekOrderPayments, 'paypal', 'total') <= 0 || $this->search($previousWeekOrderPayments, 'paypal', 'total') <= 0) ? (($this->search($weekOrderPayments, 'paypal', 'total') > 0 && $this->search($previousWeekOrderPayments, 'paypal', 'total') <= 0) ? 100 : 0) : (($this->search($weekOrderPayments, 'paypal', 'total') - $this->search($previousWeekOrderPayments, 'paypal', 'total')) * 100) / $this->search($previousWeekOrderPayments, 'paypal', 'total'));
        $paypalarray['monthly']['count'] = $this->search($monthOrderPayments, 'paypal', 'count');
        $paypalarray['monthly']['price'] = $this->search($monthOrderPayments, 'paypal', 'total');
        $paypalarray['monthly']['previous_count'] = $this->convertToDecimal(($this->search($monthOrderPayments, 'paypal', 'count') <= 0 || $this->search($previousMonthOrderPayments, 'paypal', 'count') <= 0) ? (($this->search($monthOrderPayments, 'paypal', 'count') > 0 && $this->search($previousMonthOrderPayments, 'paypal', 'count') <= 0) ? 100 : 0) : (($this->search($monthOrderPayments, 'paypal', 'count') - $this->search($previousMonthOrderPayments, 'paypal', 'count')) * 100) / $this->search($previousMonthOrderPayments, 'paypal', 'count'));
        $paypalarray['monthly']['previous_percentage'] = $this->convertToDecimal(($this->search($monthOrderPayments, 'paypal', 'total') <= 0 || $this->search($previousMonthOrderPayments, 'paypal', 'total') <= 0) ? (($this->search($monthOrderPayments, 'paypal', 'total') > 0 && $this->search($previousMonthOrderPayments, 'paypal', 'total') <= 0) ? 100 : 0) : (($this->search($monthOrderPayments, 'paypal', 'total') - $this->search($previousMonthOrderPayments, 'paypal', 'total')) * 100) / $this->search($previousMonthOrderPayments, 'paypal', 'total'));

        //CARD ARRAY
        $cardarray = array();
        $cardarray['daily']['count'] = $this->search($todayOrderPayments, 'Stripe', 'count');
        $cardarray['daily']['price'] = $this->search($todayOrderPayments, 'Stripe', 'total');
        $cardarray['daily']['previous_count'] = $this->convertToDecimal(($this->search($todayOrderPayments, 'Stripe', 'count') <= 0 || $this->search($previousDayOrderPayments, 'Stripe', 'total') <= 0) ? (($this->search($todayOrderPayments, 'Stripe', 'count') > 0 && $this->search($previousDayOrderPayments, 'Stripe', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'Stripe', 'count') - $this->search($previousDayOrderPayments, 'Stripe', 'total')) * 100) / $this->search($previousDayOrderPayments, 'Stripe', 'total'));
        $cardarray['daily']['previous_percentage'] = $this->convertToDecimal(($this->search($todayOrderPayments, 'Stripe', 'total') <= 0 || $this->search($previousDayOrderPayments, 'Stripe', 'total') <= 0) ? (($this->search($todayOrderPayments, 'Stripe', 'total') > 0 && $this->search($previousDayOrderPayments, 'Stripe', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'Stripe', 'total') - $this->search($previousDayOrderPayments, 'Stripe', 'total')) * 100) / $this->search($previousDayOrderPayments, 'Stripe', 'total'));
        $cardarray['weekly']['count'] = $this->search($weekOrderPayments, 'Stripe', 'count');
        $cardarray['weekly']['price'] = $this->search($weekOrderPayments, 'Stripe', 'total');
        $cardarray['weekly']['previous_count'] = $this->convertToDecimal(($this->search($weekOrderPayments, 'Stripe', 'count') <= 0 || $this->search($previousWeekOrderPayments, 'Stripe', 'count') <= 0) ? (($this->search($weekOrderPayments, 'Stripe', 'count') > 0 && $this->search($previousWeekOrderPayments, 'Stripe', 'count') <= 0) ? 100 : 0) : (($this->search($weekOrderPayments, 'Stripe', 'count') - $this->search($previousWeekOrderPayments, 'Stripe', 'count')) * 100) / $this->search($previousWeekOrderPayments, 'Stripe', 'count'));
        $cardarray['weekly']['previous_percentage'] = $this->convertToDecimal(($this->search($weekOrderPayments, 'Stripe', 'total') <= 0 || $this->search($previousWeekOrderPayments, 'Stripe', 'total') <= 0) ? (($this->search($weekOrderPayments, 'Stripe', 'total') > 0 && $this->search($previousWeekOrderPayments, 'Stripe', 'total') <= 0) ? 100 : 0) : (($this->search($weekOrderPayments, 'Stripe', 'total') - $this->search($previousWeekOrderPayments, 'Stripe', 'total')) * 100) / $this->search($previousWeekOrderPayments, 'Stripe', 'total'));
        $cardarray['monthly']['count'] = $this->search($monthOrderPayments, 'Stripe', 'count');
        $cardarray['monthly']['price'] = $this->search($monthOrderPayments, 'Stripe', 'total');
        $cardarray['monthly']['previous_count'] = $this->convertToDecimal(($this->search($monthOrderPayments, 'Stripe', 'count') <= 0 || $this->search($previousMonthOrderPayments, 'Stripe', 'count') <= 0) ? (($this->search($monthOrderPayments, 'Stripe', 'count') > 0 && $this->search($previousMonthOrderPayments, 'Stripe', 'count') <= 0) ? 100 : 0) : (($this->search($monthOrderPayments, 'Stripe', 'count') - $this->search($previousMonthOrderPayments, 'Stripe', 'count')) * 100) / $this->search($previousMonthOrderPayments, 'Stripe', 'count'));
        $cardarray['monthly']['previous_percentage'] = $this->convertToDecimal(($this->search($monthOrderPayments, 'Stripe', 'total') <= 0 || $this->search($previousMonthOrderPayments, 'Stripe', 'total') <= 0) ? (($this->search($monthOrderPayments, 'Stripe', 'total') > 0 && $this->search($previousMonthOrderPayments, 'Stripe', 'total') <= 0) ? 100 : 0) : (($this->search($monthOrderPayments, 'Stripe', 'total') - $this->search($previousMonthOrderPayments, 'Stripe', 'total')) * 100) / $this->search($previousMonthOrderPayments, 'Stripe', 'total'));

        $startDate = Carbon::now()->startOfDay();
        $endDate = Carbon::now()->endOfDay();

        $newReviewCount = TiffintomReview::where('status', '1')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->where('delete_status', 'N')->orderBy('id', "DESC")->count();

        $reviewsHistory = TiffintomReview::where('status', '1')->where('restaurant_id', $req->input('restaurant_id'))->orderBy('id', "DESC")->get();

        if (!empty($reviewsHistory)) {
            $reviewCount = 0;
            $totalRating = count($reviewsHistory);
            if ($totalRating > 0) {
                foreach ($reviewsHistory as $rValue) {
                    $reviewCount  = $rValue['rating'] + $reviewCount;
                }
                $finalReview = ($reviewCount / $totalRating);
            } else {
                $totalRating = 0;
                $finalReview = 0;
            }
        } else {
            $totalRating = 0;
            $finalReview = 0;
        }


        $first_day_this_month = date('Y-m-01');
        $last_day_this_month  = date('Y-m-t');
        $this_month_invoices = TiffintomInvoices::where('restaurant_id', $req->input('restaurant_id'))->where('start_date', '>=', $first_day_this_month)->where('end_date', '<=', $last_day_this_month)->orderBy('id', "DESC")->get();

        $prevoius_first_day_this_month = date('Y-m-d', strtotime('first day of last month'));
        $prevoius_last_day_this_month  = date('Y-m-d', strtotime('last day of last month'));
        $previous_month_invoices = TiffintomInvoices::where('restaurant_id', $req->input('restaurant_id'))->where('start_date', '>=', $prevoius_first_day_this_month)->where('end_date', '<=', $prevoius_last_day_this_month)->orderBy('id', "DESC")->get();

        $first_day_this_year = date('Y-01-01');
        $last_day_this_year  = date('Y-12-31');
        $this_year_invoices = TiffintomInvoices::where('restaurant_id', $req->input('restaurant_id'))->where('start_date', '>=', $first_day_this_year)->where('end_date', '<=', $last_day_this_year)->orderBy('id', "DESC")->get();

        $this_month_grand_total = 0;
        foreach ($this_month_invoices as $value) {
            $this_month_grand_total += $value['grand_total'];
        }
        $last_month_grand_total = 0;
        foreach ($previous_month_invoices as $value) {
            $last_month_grand_total += $value['grand_total'];
        }
        $this_year_grand_total = 0;
        foreach ($this_year_invoices as $value) {
            $this_year_grand_total += $value['grand_total'];
        }

        $partnerMedia = TiffintomPartnerMedia::where('status', '1')->whereRaw('(restaurant_id LIKE ? OR restaurant_id LIKE ? ) ', [$req->input('restaurant_id'), 'all'])->get();

        $response = [
            'partner_medias' => $partnerMedia,
            'new_review_count' => $newReviewCount,
            'total_rating' => $totalRating,
            'final_review' => $this->convertToDecimal($finalReview),
            'cod' => $codarray,
            'paypal' => $paypalarray,
            'card' => $cardarray,
            'cod_count' =>  $this->search($todayOrderPayments, 'cod', 'count'),
            'paypal_count' =>  $this->search($todayOrderPayments, 'paypal', 'count'),
            'card_count' =>  $this->search($todayOrderPayments, 'Stripe', 'count'),
            'cod_price' =>  $this->search($todayOrderPayments, 'cod', 'total'),
            'paypal_price' =>  $this->search($todayOrderPayments, 'paypal', 'total'),
            'card_price' =>  $this->search($todayOrderPayments, 'Stripe', 'total'),
            'previous_cod_count' => ($this->search($todayOrderPayments, 'cod', 'count') <= 0 || $this->search($previousDayOrderPayments, 'cod', 'count') <= 0) ? (($this->search($todayOrderPayments, 'cod', 'count') > 0 && $this->search($previousDayOrderPayments, 'cod', 'count') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'cod', 'count') - $this->search($previousDayOrderPayments, 'cod', 'count')) * 100) / $this->search($previousDayOrderPayments, 'cod', 'count'),
            'previous_paypal_count' => ($this->search($todayOrderPayments, 'paypal', 'count') <= 0 || $this->search($previousDayOrderPayments, 'paypal', 'count') <= 0) ? (($this->search($todayOrderPayments, 'paypal', 'count') > 0 && $this->search($previousDayOrderPayments, 'paypal', 'count') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'paypal', 'count') - $this->search($previousDayOrderPayments, 'paypal', 'count')) * 100) / $this->search($previousDayOrderPayments, 'paypal', 'count'),
            'previous_card_count' => ($this->search($todayOrderPayments, 'Stripe', 'count') <= 0 || $this->search($previousDayOrderPayments, 'Stripe', 'count') <= 0) ? (($this->search($todayOrderPayments, 'Stripe', 'count') > 0 && $this->search($previousDayOrderPayments, 'Stripe', 'count') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'Stripe', 'count') - $this->search($previousDayOrderPayments, 'Stripe', 'count')) * 100) / $this->search($previousDayOrderPayments, 'Stripe', 'count'),
            'previous_cod_percentage' => ($this->search($todayOrderPayments, 'cod', 'total') <= 0 || $this->search($previousDayOrderPayments, 'cod', 'total') <= 0) ? (($this->search($todayOrderPayments, 'cod', 'total') > 0 && $this->search($previousDayOrderPayments, 'cod', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'cod', 'total') - $this->search($previousDayOrderPayments, 'cod', 'total')) * 100) / $this->search($previousDayOrderPayments, 'cod', 'total'),
            'previous_paypal_percentage' => ($this->search($todayOrderPayments, 'paypal', 'total') <= 0 || $this->search($previousDayOrderPayments, 'paypal', 'total') <= 0) ? (($this->search($todayOrderPayments, 'paypal', 'total') > 0 && $this->search($previousDayOrderPayments, 'paypal', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'paypal', 'total') - $this->search($previousDayOrderPayments, 'paypal', 'total')) * 100) / $this->search($previousDayOrderPayments, 'paypal', 'total'),
            'previous_card_percentage' => ($this->search($todayOrderPayments, 'paypal', 'total') <= 0 || $this->search($previousDayOrderPayments, 'Stripe', 'total') <= 0) ? (($this->search($todayOrderPayments, 'paypal', 'total') > 0 && $this->search($previousDayOrderPayments, 'Stripe', 'total') <= 0) ? 100 : 0) : (($this->search($todayOrderPayments, 'paypal', 'total') - $this->search($previousDayOrderPayments, 'Stripe', 'total')) * 100) / $this->search($previousDayOrderPayments, 'Stripe', 'total'),
            'this_month_invoices' => $this_month_invoices,
            'previous_month_invoices' => $previous_month_invoices,
            'this_year_invoices' => $this_year_invoices,
            'this_month_grand_total' => $this_month_grand_total,
            'previous_month_grand_total' => $last_month_grand_total,
            'this_year_grand_total' => $this_year_grand_total,
            'this_month_percentage' => ($this_month_grand_total <= 0 || $last_month_grand_total <= 0) ? (($this_month_grand_total > 0 && $last_month_grand_total <= 0) ? 100 : 0) : ((($this_month_grand_total - $last_month_grand_total) * 100) / $last_month_grand_total),
        ];

        return $response;
    }

    /**
     * List dashboard/payment-overview
     * GET /tiffintom/dashboard/payment-overview
     */
    public function payment_overview(Request $req)
    {

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        if (date('l') != 'Monday' || date('l') != 'Tuesday') {
            $next_tuesday = date('Y-m-d', strtotime('-2 weeks tuesday'));
            $next_monday = date('Y-m-d', strtotime('-1 weeks monday'));

            $latest_tuesday = date('Y-m-d', strtotime('-3 weeks tuesday'));
            $latest_monday = date('Y-m-d', strtotime('-2 weeks monday'));
        } else {
            $next_tuesday = date('Y-m-d', strtotime('-3 weeks tuesday'));
            $next_monday = date('Y-m-d', strtotime('-2 weeks monday'));

            $latest_tuesday = date('Y-m-d', strtotime('-4 weeks tuesday'));
            $latest_monday = date('Y-m-d', strtotime('-3 weeks monday'));
        }

        $next_schedule_date = date('Y-m-d', strtotime('wednesday next week'));
        $latest_schedule_date = date('Y-m-d', strtotime('wednesday this week'));

        $payment_transactions = TiffintomPaymentTransaction::where('restaurant_id', $req->input('restaurant_id'))->orderBy('id', "DESC")->limit('3')->get();

        $first_day_this_month = date('Y-m-01');
        $last_day_this_month  = date('Y-m-t');
        $this_month_invoices = TiffintomInvoices::where('restaurant_id', $req->input('restaurant_id'))->where('start_date', '>=', $first_day_this_month)->where('end_date', '<=', $last_day_this_month)->orderBy('id', "DESC")->get();

        $prevoius_first_day_this_month = date('Y-m-d', strtotime('first day of last month'));
        $prevoius_last_day_this_month  = date('Y-m-d', strtotime('last day of last month'));
        $previous_month_invoices = TiffintomInvoices::where('restaurant_id', $req->input('restaurant_id'))->where('start_date', '>=', $prevoius_first_day_this_month)->where('end_date', '<=', $prevoius_last_day_this_month)->orderBy('id', "DESC")->get();

        $first_day_this_year = date('Y-01-01');
        $last_day_this_year  = date('Y-12-31');
        $this_year_invoices = TiffintomInvoices::where('restaurant_id', $req->input('restaurant_id'))->where('start_date', '>=', $first_day_this_year)->where('end_date', '<=', $last_day_this_year)->orderBy('id', "DESC")->get();

        $this_month_grand_total = 0;
        foreach ($this_month_invoices as $value) {
            $this_month_grand_total += $value['grand_total'];
        }
        $last_month_grand_total = 0;
        foreach ($previous_month_invoices as $value) {
            $last_month_grand_total += $value['grand_total'];
        }
        $this_year_grand_total = 0;
        foreach ($this_year_invoices as $value) {
            $this_year_grand_total += $value['grand_total'];
        }

        $response = [
            'payment_transactions' => $payment_transactions,
            'next_schedule_date' => $next_schedule_date,
            'latest_schedule_date' => $latest_schedule_date,
            'for_week_next_start' => $next_tuesday,
            'for_week_next_end' => $next_monday,
            'for_week_latest_start' => $latest_tuesday,
            'for_week_latest_end' => $latest_monday,
            'this_month_invoices' => $this_month_invoices,
            'previous_month_invoices' => $previous_month_invoices,
            'this_year_invoices' => $this_year_invoices,
            'this_month_grand_total' => $this_month_grand_total,
            'previous_month_grand_total' => $last_month_grand_total,
            'this_year_grand_total' => $this_year_grand_total,
            'this_month_percentage' => ($this_month_grand_total <= 0 || $last_month_grand_total <= 0) ? (($this_month_grand_total > 0 && $last_month_grand_total <= 0) ? 100 : 0) : ((($this_month_grand_total - $last_month_grand_total) * 100) / $last_month_grand_total),
        ];

        return $response;
    }

    public function search($items, $name, $key)
    {
        foreach ($items as $item) {
            if ($item->payment_method_name == $name) {
                if ($item->$key) {
                    return $item->$key;
                } else {
                    return 0;
                }
                break;
            }
        }
        return 0;
    }

    /**
     * List Pending Count All of Orders & Reservations.
     * GET  /tiffintom/dashboard/pendingcounts
     */
    public function pending_count(Request $req)
    {
        $PendingOrders = TiffintomOrder::where('status', 'Pending')->count();
        $PendingBookings = TiffintomBooking::where('status', 'Pending')->count();

        $data = [
            'pending_orders' => 0,
            'pending_bookings' => 0,
        ];

        return $data;
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
