<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomDeliverySetting;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomDeliverySettingController extends Controller
{
    /**
     * List delivery-setting
     * GET /tiffintom/delivery-settings
     */
    public function index(Request $req)
    {
        $delivery_settings = TiffintomDeliverySetting::orderBy('id', 'ASC');

        if ($query = $req->query("restaurant_id", false)) {
            $delivery_settings->where('resid', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $delivery_settings = $delivery_settings->get();
        } else {
            $delivery_settings = $delivery_settings->paginate($req->query('per_page', 20));
        }

        if ($delivery_settings->isEmpty()) Utils::error('TiffintomDeliverySetting not found!', 404);

        return $delivery_settings;
    }

    /**
     * Create delivery_setting.
     * POST /tiffintom/delivery-settings
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'delivery_miles' => 'required|numeric|gt:0',
            'delivery_charge' => 'required|numeric|gt:0',
            'driver_chargehint' => 'string',
            'radius_color' => 'string',
            'map_type' => 'in:Radius,Polygon',
        ]);

        // create delivery_setting
        $delivery_setting = new TiffintomDeliverySetting;
        $delivery_setting->fill($req->all());
        $delivery_setting->save();
        return $delivery_setting;
    }

    /**
     * List delivery-settings
     * GET /tiffintom/delivery-settings/id
     */
    public function show(Request $req, $id)
    {
        $delivery_settings = TiffintomDeliverySetting::find($id);
        if (empty($delivery_settings)) Utils::error("TiffintomDeliverySetting not found", 404);
        return $delivery_settings;
    }

    /**
     * Update delivery-setting.
     * POST  /tiffintom/delivery-settings/id
     */
    public function update(Request $req, $id)
    {
        $delivery_setting = TiffintomDeliverySetting::find($id);
        if (empty($delivery_setting)) Utils::error('TiffintomDeliverySetting not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'delivery_miles' => 'required|numeric|gt:0',
            'delivery_charge' => 'required|numeric|gt:0',
            'driver_chargehint' => 'string',
            'radius_color' => 'string',
            'map_type' => 'in:Radius,Polygon',
        ]);

        // update delivery_setting
        if ($req->has('restaurant_id')) $delivery_setting->restaurant_id = $req->input('restaurant_id');
        if ($req->has('delivery_miles')) $delivery_setting->delivery_miles = $req->input('delivery_miles');
        if ($req->has('delivery_charge')) $delivery_setting->delivery_charge = $req->input('delivery_charge');
        if ($req->has('driver_chargehint')) $delivery_setting->driver_chargehint = $req->input('driver_chargehint');
        if ($req->has('radius_color')) $delivery_setting->radius_color = $req->input('radius_color');
        if ($req->has('map_type')) $delivery_setting->map_type = $req->input('map_type');
        $delivery_setting->save();

        return $delivery_setting;
    }

    /**
     * Delete delivery-setting by id.
     * DELETE /tiffintom/delivery-settings/{id}
     */
    public function delete(Request $req, $id)
    {
        $delivery_setting = TiffintomDeliverySetting::find($id);
        if (empty($delivery_setting)) Utils::error('TiffintomDeliverySetting not found!', 404);
        $delivery_setting->delete();
        return $delivery_setting;
    }
}
