<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomDietary;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomDietaryController extends Controller
{

    /**
     * List dietaries
     * GET /tiffintom/dietary
     */
    public function index(Request $req)
    {
        $dietaries = TiffintomDietary::orderBy('id', 'DESC');

        if ($req->query('nopaginate', false) == '1') {
            $dietaries = $dietaries->get();
        } else {
            $dietaries = $dietaries->paginate($req->query('per_page', 20));
        }

        if ($dietaries->isEmpty()) Utils::error('No dietaries found!', 404);

        return $dietaries;
    }

    /**
     * Create dietary.
     * POST /tiffintom/dietary
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'string',
            'status' => 'boolean',
        ]);

        // create dietary
        $dietary = new TiffintomDietary;
        $dietary->fill($req->all());
        if ($req->has('name')) $dietary->name = (new ExtraController())->convertToTitleCase($req->input('name'));;
        $dietary->save();

        return $dietary;
    }

    /**
     * List dietary
     * GET /tiffintom/dietary/id
     */
    public function show(Request $req, $id)
    {
        $dietary = TiffintomDietary::find($id);
        if (empty($dietary)) Utils::error("No dietary found", 404);
        return $dietary;
    }

    /**
     * Update dietary.
     * POST  /tiffintom/dietary/id
     */
    public function update(Request $req, $id)
    {
        $dietary = TiffintomDietary::find($id);
        if (empty($dietary)) Utils::error('dietary not found!', 404);

        $this->validate($req, [
            'name' => 'string',
            'status' => 'boolean',
        ]);

        // update menu
        if ($req->has('name')) $dietary->name = $req->input('name');
        $dietary->status = $req->input('status');
        $dietary->save();
        return $dietary;
    }

    /**
     * Delete dietary by id.
     * DELETE /tiffintom/dietary/{id}
     */
    public function delete(Request $req, $id)
    {
        $dietary = TiffintomDietary::find($id);
        if (empty($dietary)) Utils::error('dietary not found!', 404);
        $dietary->delete();
        return $dietary;
    }

    /**
     * Enable dietary.
     * POST /tiffintom/dietary/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $dietary = TiffintomDietary::find($id);
        if (empty($dietary)) Utils::error('dietary not found!', 404);
        $dietary->status = '1';
        $dietary->save();
        return $dietary;
    }

    /**
     * Disable dietary.
     * POST /tiffintom/dietary/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $dietary = TiffintomDietary::find($id);
        if (empty($dietary)) Utils::error('dietary not found!', 404);
        $dietary->status = '0';
        $dietary->save();
        return $dietary;
    }

    //Convert to seo url name
    public function seoUrl($string)
    {
        $text = preg_replace('~[^\\pL\d]+~u', '-', $string);
        $text = trim($text, '-');
        $text = strtolower($text);
        $text = preg_replace('~[^-\w]+~', '', $text);
        if (strlen($text) > 70) {
            $text = substr($text, 0, 70);
        }
        if (empty($text)) {
            return time();
        }
        return $text;
    }
}
