<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomDineinCategory;
use App\ModelsTiffintom\TiffintomRestaurant;
use Illuminate\Http\Request;

class TiffintomDineinCategoryController extends Controller
{


    /**
     * List dinein-categories
     * GET /tiffintom/dinein-categories
     */
    public function index(Request $req)
    {
        if ($qr_code = $req->query("qr_code", false)) {
            $restaurant = TiffintomRestaurant::where('qr_code', $qr_code)->where('dine_in', 1)->first();
            if (empty($restaurant)) Utils::error("No restaurant found", 404);
            $req->merge(['restaurant_id' => $restaurant->id]);
        }

        $categories = TiffintomDineinCategory::orderBy('sortorder', 'ASC');

        if ($query = $req->query("query", false)) {
            $categories->whereRaw('(category_name)', ["%$query%"]);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $categories->where('restaurant_id', $query);
        }

        if (Auth::role() == 'guest' || Auth::role() == 'tiffintom_user') {
            $categories->where('status', '1');
        }
        $categories->where('delete_status', 'N');


        if ($req->query('nopaginate', false) == '1') {
            $categories = $categories->get();
        } else {
            $categories = $categories->paginate($req->query('per_page', 20));
        }

        if ($categories->isEmpty()) Utils::error('No categories found!', 404);

        if ($query = $req->query("prefilled", false) && $req->query("is_suggested", false) != 1) {
            $categories->each(function ($items) {
                $items->append('menu');
            });
        }
        if ($query = $req->query("prefilled", false) && $req->query("is_suggested", false) == 1) {
            $categories->each(function ($items) {
                $items->append('menu_suggested');
            });
        }

        return $categories;
    }

    /**
     * Create dinein-category.
     * POST /tiffintom/dinein-categories
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_name' => 'required|string',
            'category_seo' => 'string',
            'sortorder' => 'numeric',
            'description' => 'string',
            'delete_status' => 'in:Y,N',
            'status' => 'boolean',
        ]);

        // create category
        $category = new TiffintomDineinCategory;
        $category->fill($req->all());
        if ($req->has('category_name')) $category->category_seo = $this->seoUrl($category->category_name);
        $category->save();
        return $category;
    }

    /**
     * List dinein-category
     * GET /tiffintom/dinein-categories/id
     */
    public function show(Request $req, $id)
    {
        $categories = TiffintomDineinCategory::find($id);
        if (empty($categories)) Utils::error("categories not found", 404);
        return $categories;
    }

    /**
     * Update dinein-category.
     * POST  /tiffintom/dinein-categories/id
     */
    public function update(Request $req, $id)
    {
        $category = TiffintomDineinCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_name' => 'required|string',
            'category_seo' => 'string',
            'sortorder' => 'numeric',
            'description' => 'string',
            'delete_status' => 'in:Y,N',
            'status' => 'boolean',
        ]);

        // update category
        if ($req->has('restaurant_id')) $category->restaurant_id = $req->input('restaurant_id');
        if ($req->has('category_name')) $category->category_name = $req->input('category_name');
        if ($req->has('category_name')) $category->category_seo = $this->seoUrl($category->category_name);
        if ($req->has('description')) $category->description = $req->input('description');
        if ($req->has('delete_status')) $category->delete_status = $req->input('delete_status');
        $category->sortorder = $req->input('sortorder');
        $category->status = $req->input('status');
        $category->save();

        return $category;
    }

    /**
     * Delete dinein-category by id.
     * DELETE /tiffintom/dinein-categories/{id}
     */
    public function delete(Request $req, $id)
    {
        $category = TiffintomDineinCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);
        $category->delete();
        return $category;
    }

    /**
     * Enable dinein-category.
     * POST /tiffintom/dinein-categories/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $category = TiffintomDineinCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);
        $category->status = '1';
        $category->save();
        return $category;
    }

    /**
     * Disable dinein-category.
     * POST /tiffintom/dinein-categories/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $category = TiffintomDineinCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);
        $category->status = '0';
        $category->save();
        return $category;
    }

    //Convert to seo url name
    public function seoUrl($string)
    {
        $text = preg_replace('~[^\\pL\d]+~u', '-', $string);

        // trim
        $text = trim($text, '-');

        // lowercase
        $text = strtolower($text);

        // remove unwanted characters
        $text = preg_replace('~[^-\w]+~', '', $text);

        if (strlen($text) > 70) {
            $text = substr($text, 0, 70);
        }

        if (empty($text)) {
            //return 'n-a';
            return time();
        }

        return $text;
    }
}
