<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomDineinMenu;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomDineinMenuController extends Controller
{

    /**
     * List menu
     * GET /tiffintom/menu
     */
    public function index(Request $req)
    {
        $menu = TiffintomDineinMenu::selectRaw('dinein_restaurant_menus.*,dinein_categories.category_name as category_name')->leftJoin('dinein_categories', 'dinein_categories.id', '=', 'category_id')->orderBy('dinein_restaurant_menus.sortorder', 'ASC');

        if ($query = $req->query("query", false)) {
            $menu->whereRaw('(menu_name)', ["%$query%"]);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $menu->where('dinein_restaurant_menus.restaurant_id', $query);
        }
        if ($query = $req->query("category_id", false)) {
            $menu->where('category_id', $query);
        }
        $menu->with(['variants']);

        if ($req->query('nopaginate', false) == '1') {
            $menu = $menu->get();
        } else {
            $menu = $menu->paginate($req->query('per_page', 20));
        }

        if ($menu->isEmpty()) Utils::error('No menu found!', 404);

        return $menu;
    }

    /**
     * Create menu.
     * POST /tiffintom/categories
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_id' => 'required|exists:tiffintom_connection.categories,id',
            'menu_name' => 'required|string',
            'is_suggest' => 'string',
            'price_option' => 'in:single,multiple',
            'menu_description' => 'string',
            'menu_type' => 'in:veg,nonveg,other',
            'menu_addon' => 'in:Yes,No',
            'popular_dish' => 'in:Yes,No',
            'featured' => 'in:Yes,No',
            'spicy_dish' => 'in:mild,medium,extra_hot,slightly_hot,hot,no_option',
            'favourite' => 'in:Y,N',
            'status' => 'boolean',
            'delete_status' => 'in:Y,N',
            'sortorder' => 'numeric',
            'short_name' => 'string',
            'vegetarian' => 'numeric',
            'contains_nuts' => 'numeric',
            'milk' => 'numeric',
            'mustard' => 'numeric',
            'eggs' => 'numeric',
            'fish' => 'numeric',
            'gluten_free' => 'numeric',
            'halal' => 'numeric',
            'koshar' => 'numeric',
            'vegan' => 'numeric',
            'crustaceans' => 'numeric',
            'molluscs' => 'numeric',
            'sulphur_dioxide' => 'numeric',
            'sulphites' => 'numeric',
            'product_percentage' => 'numeric',
        ]);

        // create menu
        $menu = new TiffintomDineinMenu;
        $menu->fill($req->all());
        $menu->save();
        return $menu;
    }

    /**
     * List category
     * GET /tiffintom/categories/id
     */
    public function show(Request $req, $id)
    {
        $menu = TiffintomDineinMenu::find($id);
        if (empty($menu)) Utils::error("No product found", 404);
        $menu->with(['variants']);
        $menu->variants->each(function ($variant) {
            $variant->append('main_addons');
        });

        return $menu;
    }

    /**
     * Update menu.
     * POST  /tiffintom/menus/id
     */
    public function update(Request $req, $id)
    {
        $menu = TiffintomDineinMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_id' => 'required|exists:tiffintom_connection.categories,id',
            'menu_name' => 'required|string',
            'is_suggest' => 'string',
            'price_option' => 'in:single,multiple',
            'menu_description' => 'string',
            'menu_type' => 'in:veg,nonveg,other',
            'menu_addon' => 'in:Yes,No',
            'popular_dish' => 'in:Yes,No',
            'featured' => 'in:Yes,No',
            'spicy_dish' => 'in:mild,medium,extra_hot,slightly_hot,hot,no_option',
            'favourite' => 'in:Y,N',
            'status' => 'boolean',
            'delete_status' => 'in:Y,N',
            'sortorder' => 'numeric',
            'short_name' => 'string',
            'vegetarian' => 'numeric',
            'contains_nuts' => 'numeric',
            'milk' => 'numeric',
            'mustard' => 'numeric',
            'eggs' => 'numeric',
            'fish' => 'numeric',
            'gluten_free' => 'numeric',
            'halal' => 'numeric',
            'koshar' => 'numeric',
            'vegan' => 'numeric',
            'crustaceans' => 'numeric',
            'molluscs' => 'numeric',
            'sulphur_dioxide' => 'numeric',
            'sulphites' => 'numeric',
            'product_percentage' => 'numeric',
        ]);

        // update menu
        if ($req->has('restaurant_id')) $menu->restaurant_id = $req->input('restaurant_id');
        if ($req->has('category_id')) $menu->category_id = $req->input('category_id');
        if ($req->has('menu_name')) $menu->menu_name = $req->input('menu_name');
        if ($req->has('is_suggest')) $menu->is_suggest = $req->input('is_suggest');
        if ($req->has('price_option')) $menu->price_option = $req->input('price_option');
        if ($req->has('menu_description')) $menu->menu_description = $req->input('menu_description');
        if ($req->has('menu_type')) $menu->menu_type = $req->input('menu_type');
        if ($req->has('menu_addon')) $menu->menu_addon = $req->input('menu_addon');
        if ($req->has('popular_dish')) $menu->popular_dish = $req->input('popular_dish');
        if ($req->has('featured')) $menu->featured = $req->input('featured');
        if ($req->has('spicy_dish')) $menu->spicy_dish = $req->input('spicy_dish');
        if ($req->has('favourite')) $menu->favourite = $req->input('favourite');
        if ($req->has('delete_status')) $menu->delete_status = $req->input('delete_status');
        if ($req->has('sortorder')) $menu->sortorder = $req->input('sortorder');
        if ($req->has('short_name')) $menu->short_name = $req->input('short_name');
        if ($req->has('product_percentage')) $menu->product_percentage = $req->input('product_percentage');
        $menu->vegetarian = $req->input('vegetarian');
        $menu->contains_nuts = $req->input('contains_nuts');
        $menu->milk = $req->input('milk');
        $menu->mustard = $req->input('mustard');
        $menu->eggs = $req->input('eggs');
        $menu->fish = $req->input('fish');
        $menu->gluten_free = $req->input('gluten_free');
        $menu->halal = $req->input('halal');
        $menu->koshar = $req->input('koshar');
        $menu->vegan = $req->input('vegan');
        $menu->crustaceans = $req->input('crustaceans');
        $menu->molluscs = $req->input('molluscs');
        $menu->sulphur_dioxide = $req->input('sulphur_dioxide');
        $menu->sulphites = $req->input('sulphites');
        $menu->status = $req->input('status');
        $menu->save();

        return $menu;
    }

    /**
     * Delete menu by id.
     * DELETE /tiffintom/menus/{id}
     */
    public function delete(Request $req, $id)
    {
        $menu = TiffintomDineinMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);
        $menu->delete();
        return $menu;
    }

    /**
     * Enable menu.
     * POST /tiffintom/menus/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $menu = TiffintomDineinMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);
        $menu->status = '1';
        $menu->save();
        return $menu;
    }

    /**
     * Disable menu.
     * POST /tiffintom/menus/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $menu = TiffintomDineinMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);
        $menu->status = '0';
        $menu->save();
        return $menu;
    }
}
