<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposOrder;
use App\ModelsTiffintom\TiffintomDineinCart;
use App\ModelsTiffintom\TiffintomCustomerReward;
use App\ModelsTiffintom\TiffintomMenu;
use App\ModelsTiffintom\TiffintomOffer;
use App\ModelsTiffintom\TiffintomOfferDay;
use App\ModelsTiffintom\TiffintomOfferDayPrice;
use App\ModelsTiffintom\TiffintomOfferDayProduct;
use App\ModelsTiffintom\TiffintomOfferMultiplePrice;
use App\ModelsTiffintom\TiffintomOfferPrice;
use App\ModelsTiffintom\TiffintomOfferProduct;
use App\ModelsTiffintom\TiffintomDineinOrder;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomReview;
use App\ModelsTiffintom\TiffintomReward;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomUser;
use App\ModelsTiffintom\TiffintomWalletHistory;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomDineinOrderController extends Controller
{
    /**
     * List order
     * GET /tiffintom/order
     */
    public function index(Request $req)
    {
        $orders = TiffintomDineinOrder::orderBy('id', 'DESC');

        if ($query = $req->query("restaurant_id", false)) {
            $orders->where('restaurant_id', $query);
        }
        if ($query = $req->query("customer_id", false)) {
            $orders->where('customer_id', $query);
        }
        if ($query = $req->query('from_date')) {
            $fromDate = Carbon::parse($query)->startOfDay();
            $orders->where('created', '>', $fromDate);
        }
        if ($query = $req->query('to_date')) {
            $toDate = Carbon::parse($query)->endOfDay();
            $orders->where('created', '<', $toDate);
        }
        if ($req->query('nopaginate', false) == '1') {
            $orders = $orders->get();
            foreach ($orders as $key => $value) {
                $restaurants = TiffintomRestaurant::where('id', $value->restaurant_id)->first();
                if (!empty($restaurants)) {
                    $value['restaurant'] = $restaurants;

                    $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurants['id'])->where('status', '1')->first();
                    if (!empty($reviews)) {
                        if ($reviews->count > 0) {
                            $value['restaurant']['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                            $value['restaurant']['total_reviews'] = $reviews->count;
                        }
                    }
                    $orders[] = $value;
                }
            }
        } else {
            $orders = $orders->paginate($req->query('per_page', 20));
            if (!$orders->isEmpty()) {
                $orders->each(function ($order) {
                    $restaurants = TiffintomRestaurant::where('id', $order->restaurant_id)->first();
                    $order->restaurant = $restaurants;

                    $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $order->restaurant_id)->where('status', '1')->first();
                    if (!empty($reviews)) {
                        if ($reviews->count > 0) {
                            $order->restaurant->average_rating = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                            $order->restaurant->total_reviews = $reviews->count;
                        }
                    }
                });
            }
        }

        if ($orders->isEmpty()) Utils::error('order not found!', 404);

        if (!$orders->isEmpty() && !$req->query('append')) {
            $orders->each(function ($order) {
                $order->append('cart_view');
                $order->append('reviews');
            });
        }

        return $orders;
    }

    /**
     * List order
     * GET /tiffintom/order/order-history
     */
    public function order_history(Request $req)
    {
        if (!Auth::$business_id) {
            $business_id = $req->header('business_id');
        } else {
            $business_id = Auth::$business_id;
        }
        if (empty($business_id)) Utils::error("Business id not found", 400);
        $business = Business::find($business_id);
        if (empty($business)) Utils::error("Business not found", 400);
        $this->establishEposConnection($business);

        $orders = EposOrder::selectRaw('orders.*,customers.name as customer_name,tables.number as table_number,order_types.type as order_type,order_statuses.status as order_status')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'order_status_id')->orderByRaw('orders.id DESC');

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        $to_date = null;
        $from_date = null;

        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . " 00:00:00";
            $to_date = $req->input('to_date') . " 23:59:59";
            $orders = $orders->where('orders.created_at', '>=', $from_date);
            $orders = $orders->where('orders.created_at', '<=', $to_date);
        }


        if ($query = $req->query('query', false)) {
            $orders->whereRaw('(orders.total ?  )', ["%$query%"]);
        }
        if ($table_id = $req->query('table_id', false)) {
            $orders->where('table_id', $table_id);
        }
        if ($customer_id = $req->query('customer_id', false)) {
            $orders->where('customer_id', $customer_id);
            // Create connection
            $this->establishEposConnection($business);
            $Customer = EposCustomer::find($customer_id);
            $epos_customer = $Customer;
        }
        if ($user_id = $req->query('user_id', false)) {
            $user = TiffintomUser::find($user_id);
            $this->establishEposConnection($business);
            $name = $user->first_name . ' ' . $user->last_name;
            $Customer = EposCustomer::whereRaw('(name LIKE ?  )', ["%$name%"])->first();
            $epos_customer = $Customer;
        }
        if ($order_type_id = $req->query('order_type_id', false)) {
            $orders->where('order_type_id', $order_type_id);
        }
        if ($order_status_id = $req->query('order_status_id', false)) {
            $orders->where('order_status_id', $order_status_id);
        }
        if ($query = $req->query('preparation_location_id', false)) {
            $orders->where('preparation_location_id', $query);
        }
        if ($query = $req->query('order_action_id', false)) {
            $orders->where('order_action_id', $query);
        }
        if ($query = $req->query('archived', false)) {
            $orders->where('is_archived', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $orders = $orders->get();
        } else {
            $orders = $orders->paginate($req->query('per_page', 20));
        }

        if ($orders->isEmpty()) Utils::error('No orders found!', 404);

        $orders->append('order_items');
        $orders->append('order_payments');
        $orders->append('order_splits');
        $orders->append('customer');
        $orders->append('table');

        $data = [
            'order_history' => $orders,
            'epos_customer' => $epos_customer,
        ];

        return $data;
    }

    /**
     * Create order.
     * POST /tiffintom/order
     */
    public function create(Request $req)
    {
        if ($req->has('carts') && !is_array($req->input('carts'))) {
            $req->merge(['carts' => json_decode($req->input('carts'))]);
        }

        $this->validate($req, [
            'order_number'  => 'string',
            'restaurant_id'  => 'required|string',
            'customer_id'  => 'required|string',
            'customer_name'  => 'required|string',
            'customer_email'  => 'required|string',
            'customer_phone'  => 'required|string',
            'order_description'  => 'string',
            'offer_percentage'  => 'numeric',
            'offer_amount'  => 'numeric',
            'day_offer'  => 'numeric',
            'service_charge'  => 'numeric',
            'voucher_code'  => 'string',
            'voucher_percentage'  => 'numeric',
            'voucher_amount'  => 'numeric',
            'order_sub_total'  => 'required|numeric|gt:0',
            'order_grand_total'  => 'required|numeric|gt:0',
            'payment_status'  => 'string',
            'payment_method'  => 'required|string',
            'card_id'  => 'string',
            'payment_wallet'  => 'string',
            'paid_full'  => 'string',
            'split_payment'  => 'string',
            'wallet_amount'  => 'string',
            'transaction_id'  => 'string',
            'reward_used'  => 'string',
            'order_point'  => 'numeric',
            'spent_point'  => 'numeric',
            'reward_offer'  => 'numeric',
            'reward_offer_percentage'  => 'numeric',
            'payerID'  => 'string',
            'paymentToken'  => 'string',
            'paymentID'  => 'string',
            'status'  => 'string',
            'type'  => 'string',
            'charity_amount'  => 'numeric',
            'charity_message'  => 'string',
            'carts' => 'required|array'
        ]);

        // create order
        $order = new TiffintomDineinOrder;
        $order->fill($req->all());
        $order->save();

        $order->order_number = 'ORD00' . $order->id;
        $order->save();

        //Cart Add
        $carts = ($req->input('carts'));
        foreach ($carts as $cart) {
            $cartItem = new TiffintomDineinCart((array)$cart);
            $cartItem->order_id = $order->id;
            $cartItem->customer_id = $req->input('customer_id');
            $cartItem->restaurant_id = $req->input('restaurant_id');
            $cartItem->save();
        }

        // Wallet Payment 
        if ($req->input('payment_wallet') == 'Yes') {
            $user = TiffintomUser::find($req->input('customer_id'));
            // if ($user->wallet_amount < $req->input('order_grand_total')) {
            $user->wallet_amount = $user->wallet_amount - $req->input('wallet_amount'); //$req->input('order_grand_total');
            // } else {
            //     $user->wallet_amount = 0;
            // }
            $user->save();

            $wallet_history = new TiffintomWalletHistory;
            $wallet_history->customer_id = $req->input('customer_id');
            $wallet_history->purpose = "Amount Paid";
            $wallet_history->transaction_type = "Amount Paid";
            $wallet_history->transaction_details = $order->order_number;
            $wallet_history->amount = $req->input('wallet_amount');
            $wallet_history->save();
        }

        //Reward Points Update 
        if ($req->input('reward_used') == 'Y') {
            $rewardPoints = TiffintomReward::orderBy('id', 'ASC')->first();

            $order->spent_point =  $rewardPoints['reward_totalpoint'];
            $order->save();

            $today = date("d-m-Y");
            $expireday = '-' . $rewardPoints['reward_validity'] . ' days';
            $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
            $startdate = date("Y-m-d 23:59:59", strtotime($today));
            $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

            $customerPoints = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->where('created', '<=', $startdate)->where('created', '>=', $endDate)->sum('points');

            $rewardPointSpent = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('type', 'Spent')->where('status', '1')->get();

            if (!empty($rewardPointSpent)) {
                foreach ($rewardPointSpent as $value) {
                    $customerPointsave = TiffintomCustomerReward::where('id', $value['id'])->first();
                    $customerPointsave->status = '0';
                    $customerPointsave->save();
                }
            }
            $orderPoints = $customerPoints - $rewardPoints['reward_totalpoint'];
            $customerPoint = new TiffintomCustomerReward;
            $restaurantDetails = TiffintomRestaurant::where('id', $req->input('restaurant_id'))->first();
            $customerPoint->order_id = $order->id;
            $customerPoint->restaurant_name = $restaurantDetails['restaurant_name'];
            $customerPoint->customer_id = $req->input('customer_id');
            $customerPoint->total = $req->input('order_grand_total');
            $customerPoint->points = $orderPoints;
            $customerPoint->type = 'Spent';
            $customerPoint->save();

            $today = date("d-m-Y");
            $expireday = '-' . $rewardPoints['reward_validity'] . ' days';
            $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
            $startdate = date("Y-m-d 23:59:59", strtotime($today));
            $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

            $rewardPointEarn = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->where('type', 'Earned')->get();

            if (!empty($rewardPointEarn)) {
                foreach ($rewardPointEarn as $value) {
                    $customerPointsave = TiffintomCustomerReward::where('id', $value['id'])->first();
                    $customerPointsave->status = '0';
                    $customerPointsave->save();
                }
            }
        }

        $user = TiffintomUser::find($order->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        // Restaurant Fcm Notification
        $message      = 'You have a new dinin order - ' . $order->order_number;
        $notification['title']          = "NewDineinorder";
        $notification['body']        = $message;

        $notificationdata['data']['title']          = "NewDineinorder";
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['order_id']  = $order->id;
        $notificationdata['data']['order_type']  = ucfirst($order->order_type);
        $notificationdata['data']['order_status']  = $order->status;
        $notificationdata['data']['payment_status']  = $order->payment_status != '' ? 'Paid' : 'Not Paid';
        $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "dineinorder");
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
        $Devices = explode(',', $restaurant->device_id);
        foreach ($Devices as $Device) {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device, $notification, 'restaurant');
        }

        if ($user->order_place == 7) {
            // $this->orderEmail($order->id);
            $message  = 'Your order from ' . $restaurant->restaurant_name . ' has been successfully placed. Please wait for the restaurant to accept.';
            $notification['title']          = "Dine in Order placed";
            $notification['body']        = $message;

            $notificationdata['data']['title']          = "Dine in Order placed";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['order_id']  = $order->id;
            $notificationdata['data']['order_status']  = $order->status;
            $notificationdata['data']['payment_status']  = $order->payment_status != '' ? 'Paid' : 'Not Paid';
            $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "dineinorder");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_place == 4 || $user->order_place == 5 || $user->order_place == 6) {
            // $this->orderEmail($order->id);
        }
        if ($user->order_place == 1 || $user->order_place == 3 || $user->order_place == 5) {
            $message  = 'Your order from ' . $restaurant->restaurant_name . ' has been successfully placed. Please wait for the restaurant to accept.';
            $notification['title']          = "Dine in Order placed";
            $notification['body']        = $message;

            $notificationdata['data']['title']          = "Dine in Order placed";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['order_id']  = $order->id;
            $notificationdata['data']['order_status']  = $order->status;
            $notificationdata['data']['payment_status']  = $order->payment_status != '' ? 'Paid' : 'Not Paid';
            $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "dineinorder");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        if ($user->order_place == 2 || $user->order_place == 3 || $user->order_place == 6 || $user->order_place == 7) {
            // if ($user->phone_verify == 1) {
            $message = 'Hey ' . $user->first_name . '! Thank you for placing your dinein order with TiffinTom. Please wait for the restaurant to receive and confirm your order. Your order number is ' . $order->order_number;
            (new OtpHelper())->sendMessage($message, $user->phone_number, 'reservation_place', $restaurant->sms_sender_id);
            // }
            if ($siteSettings->admin_order_all == 'Yes') {
                $message = 'New Dinein order from Tiffintom - ' . $order->order_number;
                // (new OtpHelper())->sendMessage($message, $siteSettings->admin_phone, 'Sitesetting');
                (new OtpHelper())->sendMessage($message, $siteSettings->admin_phone, 'reservation_place', $restaurant->sms_sender_id);
            }

            $UsersList = TiffintomUser::where('status', '1')->where('role_id', '4')->get();
            foreach ($UsersList as $key => $value) {
                if ($value['sms_permissions'] != '') {
                    $sms_permissions = explode(',', $value['sms_permissions']);
                    if (in_array("Order", $sms_permissions)) {
                        $message = 'New Dinein order from Tiffintom - ' . $order->order_number;
                        // (new OtpHelper())->sendMessage($message, $value['phone_number'], 'Sitesetting');
                        (new OtpHelper())->sendMessage($message, $value['phone_number'], 'reservation_place', $restaurant->sms_sender_id);
                    }
                }
            }

            $messages = 'New Dinein order from Tiffintom -  ' . $order->order_number . ' Please check now! Thank you';
            if ($siteSettings['restaurant_order_all'] == 'Yes') {
                // (new OtpHelper())->sendMessage($messages, $restaurant->sms_phonenumber, 'Restaurant');
                (new OtpHelper())->sendMessage($messages, $restaurant->sms_phonenumber, 'reservation_place', $restaurant->sms_sender_id);
            }
        }

        $this->orderMail($order->id);

        $order = TiffintomDineinOrder::find($order->id);
        $order->append('cart_view');
        return $order;
    }

    /**
     * List order
     * GET /tiffintom/order/id
     */
    public function show(Request $req, $id)
    {
        $order = TiffintomDineinOrder::find($id);
        if (empty($order)) Utils::error("order not found", 404);
        $order->append('cart_view');
        $order->append('card_view');
        $order->append('reviews');
        return $order;
    }

    /**
     * Update orders.
     * POST  /tiffintom/orders/id/change-status
     */
    public function changeStatus(Request $req, $id)
    {
        $dinein_order = TiffintomDineinOrder::find($id);
        if (empty($dinein_order)) Utils::error('TiffintomDineinOrder not found!', 404);

        $this->validate($req, [
            'status' => 'required|string',
            'preparation' => 'string',
            'failed_reason' => 'required_if:status,Failed|string',
        ]);

        // update dinein_order
        if ($req->has('status')) $dinein_order->status = $req->input('status');
        if ($req->has('preparation')) $dinein_order->preparation = $req->input('preparation');
        if ($req->has('failed_reason')) $dinein_order->failed_reason = $req->input('failed_reason');

        $dinein_order->save();

        $user = TiffintomUser::find($dinein_order->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($dinein_order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($req->input('status') == 'Failed') {
            $message = 'Sorry, but your dine in order from ' . $restaurant->restaurant_name . ' cannot be accepted at the moment. Please try again later or order in person.';
            $title = "Dine in order rejected";
            $sms_event = 'order_reject';
        } else if ($req->input('status') == 'Delivered') {
            $title = "Dine in order delivered";
            $message = 'Your dine in order from ' . $restaurant->restaurant_name . ' has been successfully delivered. Enjoy your meal.';
            $sms_event = 'order_update';
        } else if ($req->input('status') == 'Accepted') {
            $title = "Dine in order accepted";
            $message = 'Your dine in order from ' . $restaurant->restaurant_name . ' has been accepted.You should be served soon.';
            $sms_event = 'order_accept';
        } else {
        }

        // All status wise user text message send
        // if ($user->phone_verify == 1) {
        if ($user->order_accept == 7 && $req->input('status') == 'Accepted') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_delivered == 7 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_reject == 7 && $req->input('status') == 'Failed') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_accept == 2 && $req->input('status') == 'Accepted') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_delivered == 2 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_reject == 2 && $req->input('status') == 'Failed') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_accept == 3 && $req->input('status') == 'Accepted') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_delivered == 3 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_reject == 3 && $req->input('status') == 'Failed') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_accept == 6 && $req->input('status') == 'Accepted') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_delivered == 6 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        if ($user->order_reject == 6 && $req->input('status') == 'Failed') {
            (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
        }
        // }

        // All Status Wise Notification Set
        $notification['title']          = $title;
        $notification['body']        = $message;

        $notificationdata['data']['title']          = $title;
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['order_id']  = $dinein_order->id;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['order_status']  = $dinein_order->status;
        $notificationdata['data']['payload']        = array('DineinorderDetails' => "", 'type'    => "dineinorder");
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
        if ($user->order_accept == 7 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 7 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 7 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_accept == 1 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 1 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 1 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 3 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 3 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_accept == 3 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 5 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 5 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_accept == 5 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        $this->orderMail($dinein_order->id);

        return $dinein_order;
    }


    /**
     * List order
     * POST /tiffintom/order/offer-check/{code}
     */
    public function offer_check(Request $req)
    {
        if ($req->has('carts') && !is_array($req->input('carts'))) {
            $req->merge(['carts' => json_decode($req->input('carts'))]);
        }

        $order = new TiffintomDineinOrder;
        $order->fill($req->all());

        $subtotal = 0;
        $carts = ($req->input('carts'));
        $order['carts'] = $carts;
        foreach ($carts as $cart) {
            if (isset($cart->total_price)) {
                $subtotal = $subtotal + $cart->total_price;
            }
        }
        $subtotal = $this->convertToDecimal($subtotal);


        $order['order_sub_total'] = $this->convertToDecimal($subtotal);
        $order['order_grand_total'] = $this->convertToDecimal($subtotal);

        $restaurantDetails = TiffintomRestaurant::where('id', $order['restaurant_id'])->first();
        $reward = TiffintomReward::orderBy('id', 'ASC')->first();

        if (!empty($reward) && $restaurantDetails['reward_option'] == 'Yes') {

            $customerPoints = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->first();

            $createdDate = explode(' ', $reward['created']);
            $date1 = date_create($createdDate[0]);
            $date2 = date_create(date('Y-m-d'));

            $diff = date_diff($date1, $date2);
            $diff = $diff->format("%R%a");

            $remainingDays = $reward['reward_validity'];
            // $remainingDays = ($reward['reward_validity'] - $diff);
            $resPercent = $reward['reward_percentage'];
            if ($remainingDays > 0) {

                $today = date("d-m-Y");
                $expireday = '-' . $reward['reward_validity'] . ' days';
                $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
                $startdate = date("Y-m-d 23:59:59", strtotime($today));
                $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

                $customerPoints = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->where('created', '<=', $startdate)->where('created', '>=', $endDate)->sum('points');

                $userPoints = $customerPoints;
                $resPoints = $reward['reward_totalpoint'];

                if ($subtotal > $reward['minimum_order']) {
                    if ($userPoints >= $resPoints) {
                        $totPercent = $reward['reward_percentage'];
                        $offPercent = $subtotal * abs($totPercent / 100);
                        if ($reward['miximum_value'] > $offPercent) {
                            $order['rewardPoint'] = $this->convertToDecimal($offPercent);
                            $order['rewardPercentage'] = $totPercent;
                        } else {
                            if ($reward['value_type'] == 0) {
                                $offPercent = $reward['miximum_value'];
                                $order['rewardPoint'] = $offPercent;
                                $order['rewardPercentage'] = '';
                            } else {
                                $offPercent = $reward['miximum_value'];
                                $order['rewardPoint'] = $offPercent;
                                $order['rewardPercentage'] = $reward['miximum_value'];
                            }
                        }
                    }
                }
            }
        }

        $rewardAmount = $reward['reward_amount'];
        $rewardPoints = $reward['reward_point'];

        $getRewardFromTotal = $subtotal / $rewardAmount;

        $orderPoint = $getRewardFromTotal * $rewardPoints;
        $order['order_point'] = $this->convertToDecimal((int) $orderPoint);

        $appliedOffers = [];
        $Offers = [];
        $customerOrdersCount = TiffintomDineinOrder::where('customer_id', $order['customer_id'])->count();

        foreach ($carts as $key => $cartItem) {
            //Multiple Price Offer Section
            $multiplePriceOffers = null;
            if ($order['order_type'] ==  'delivery') {
                $multiplePriceOffers = TiffintomOfferMultiplePrice::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('purchase_price', '<=', $subtotal)->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('status', '1')->orderBy('purchase_price', 'DESC')->first();
            } else if ($order['order_type'] ==  'pickup') {
                $multiplePriceOffers = TiffintomOfferMultiplePrice::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('purchase_price', '<=', $subtotal)->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('status', '1')->orderBy('purchase_price', 'DESC')->first();
            }
            if (!empty($multiplePriceOffers)) {
                $offerIdExpload = explode(',', $multiplePriceOffers['offer_id']);
                foreach ($offerIdExpload as $multiOfferSubKey => $multiOfferSubValue) {
                    $Offers[$multiOfferSubKey]['quantity'] = $multiplePriceOffers['offer_qty'];
                    $Offers[$multiOfferSubKey]['menu_name'] = $multiOfferSubValue;
                    $Offers[$multiOfferSubKey]['menu_price'] = 0;
                    $Offers[$multiOfferSubKey]['total_price'] = 0;
                }
                $appliedOffers[] = $Offers;
                break;
            }


            //Day-Wise Offer Section
            $dayOffers = null;
            if ($order['order_type'] ==  'delivery') {
                $dayOffers = TiffintomOfferDay::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('day',  date('l'))->where('purchase_price', '<=', $subtotal)->where('status', '1')->orderBy('id', 'DESC')->first();
            } else if ($order['order_type'] ==  'pickup') {
                $dayOffers = TiffintomOfferDay::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('day',  date('l'))->where('purchase_price', '<=', $subtotal)->where('status', '1')->orderBy('id', 'DESC')->first();
            }
            if (!empty($dayOffers)) {
                if ($dayOffers['day_offer_type'] == 0) {
                    $dayofferAmount = $dayOffers['offer_amount'];
                    $order['offer_amount'] = $dayOffers['offer_amount'];
                    $order['offer_percentage'] = 0;
                } else {
                    $dayofferAmount = $subtotal * $dayOffers['offer_amount'] / 100;
                    $order['offer_amount'] = $this->convertToDecimal($dayofferAmount);
                    $order['offer_percentage'] = $dayOffers['offer_amount'];
                }
                break;
            }

            // Day Wise Product Offer Section
            $dayProductOffer = null;
            if ($order['order_type'] ==  'delivery') {
                $dayProductOffer = TiffintomOfferDayProduct::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('day_product',  date('l'))->where('status', '1')->orderBy('id', 'DESC')->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $dayProductOffer = TiffintomOfferDayProduct::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('day_product',  date('l'))->where('status', '1')->orderBy('id', 'DESC')->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            }
            if (!empty($dayProductOffer)) {
                $product = TiffintomMenu::where('id', $dayProductOffer['offer_id'])->first();
                $Offers['menu_id'] = $dayProductOffer['offer_id'];
                $Offers['quantity'] = $dayProductOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }

            // Day Wise Price Offer Section
            $dayPriceOffer = null;
            if ($order['order_type'] ==  'delivery') {
                $dayPriceOffer = TiffintomOfferDayPrice::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('day',  date('l'))->where('status', '1')->orderBy('id', 'DESC')->where('purchase_price', '<=', $subtotal)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $dayPriceOffer = TiffintomOfferDayPrice::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('day',  date('l'))->where('status', '1')->orderBy('id', 'DESC')->where('purchase_price', '<=', $subtotal)->first();
            }
            if (!empty($dayPriceOffer)) {
                $product = TiffintomMenu::where('id', $dayPriceOffer['offer_id'])->first();
                $Offers['menu_id'] = $dayPriceOffer['offer_id'];
                $Offers['quantity'] = $dayPriceOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }

            //Offer Section
            $offer = null;
            if ($order['order_type'] ==  'delivery') {
                if ($customerOrdersCount == 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->where('first_user', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('free_price', '<=', $subtotal)->first();
                }
                if (($customerOrdersCount == 0 && empty($offer)) || $customerOrdersCount > 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->where('normal', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('normal_price', '<=', $subtotal)->first();
                }
            } else if ($order['order_type'] ==  'pickup') {
                if ($customerOrdersCount == 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->where('first_user', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('free_price', '<=', $subtotal)->first();
                }
                if (($customerOrdersCount == 0 && empty($offer)) || $customerOrdersCount > 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->where('normal', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('normal_price', '<=', $subtotal)->first();
                }
            }
            if (!empty($offer)) {
                if ($offer['first_user'] == 'Y') {
                    $offersAmount = $offer['free_percentage'];
                } else {
                    $offersAmount = $offer['normal_percentage'];
                }
                if ($offer['offer_mode'] == 'flat') {
                    $offerAmount = $offersAmount;
                    $offerPercentage = 0;
                } else {
                    $offerAmount = $this->convertToDecimal($subtotal * $offersAmount / 100);
                    $offerPercentage = $offersAmount;
                }
                $order['offer_amount'] = $offerAmount;
                $order['offer_percentage'] = $offerPercentage;
                break;
            }

            //Product Offer Section
            if ($order['order_type'] ==  'delivery') {
                $productOffer = TiffintomOfferProduct::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $productOffer = TiffintomOfferProduct::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            }
            if (!empty($productOffer)) {
                $product = TiffintomMenu::where('id', $productOffer['offer_id'])->first();
                $Offers['menu_id'] = $productOffer['offer_id'];
                $Offers['quantity'] = $productOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }

            //Price Offer Section
            if ($order['order_type'] ==  'delivery') {
                $priceOffer = TiffintomOfferPrice::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('purchase_price', '<=', $subtotal)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $priceOffer = TiffintomOfferPrice::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('purchase_price', '<=', $subtotal)->first();
            }
            if (!empty($priceOffer)) {
                $product = TiffintomMenu::where('id', $priceOffer['offer_id'])->first();
                $Offers['menu_id'] = $priceOffer['offer_id'];
                $Offers['quantity'] = $priceOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }
        }

        $order['applied_offers'] = $appliedOffers;
        return $order;
    }

    /**
     * Create order.
     * POST /tiffintom/order
     */
    public function reOrder(Request $req, $id)
    {

        $customer = Auth::user();

        //Other Restaurant cart
        $Excarts = TiffintomDineinCart::where('customer_id', $customer->id)->where('order_id', 0)->get();
        if (!$Excarts->isEmpty()) $cartss = TiffintomDineinCart::where('customer_id', $customer->id)->where('order_id', 0)->delete();

        $carts = TiffintomDineinCart::where('order_id', $id)->where('total_price', '>', 0)->orderBy('id', 'DESC')->get();

        foreach ($carts as $cart) {
            $cartObj = new TiffintomDineinCart;
            $cartObj->restaurant_id = $cart->restaurant_id;
            $cartObj->customer_id = $cart->customer_id;
            $cartObj->menu_id = $cart->menu_id;
            $cartObj->menu_name = $cart->menu_name;
            $cartObj->subaddons_name = $cart->subaddons_name;
            $cartObj->menu_price = $cart->menu_price;
            $cartObj->total_price = $cart->total_price;
            $cartObj->quantity = $cart->quantity;
            $cartObj->menu_description = $cart->menu_description;
            $cartObj->save();
        }

        return ['message' => 'cart added!'];
    }

    public function orderMail($id)
    {
        $order = TiffintomDineinOrder::find($id);
        if (empty($order)) Utils::error("order not found", 404);
        $order->append('cart_view');

        $user = TiffintomUser::find($order->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $servicecharge = '';
        $offerDetails = '';
        $voucherDetails = '';
        $rewardDetails = '';
        $orderDescription = '';

        if ($order['split_payment'] == 'Yes') {
            $paymentMethod = strtoupper(($order['payment_method'] == 'Stripe') ? 'Card' : $order['payment_method']) . ' & ' . 'Wallet';
        } else {
            $paymentMethod = strtoupper(($order['payment_method'] == 'Stripe') ? 'Card' : $order['payment_method']);
        }

        $paymentStatus = ($order['payment_status'] == 'P') ? 'Paid' : 'Not Paid';

        if ($order['offer_amount'] > 0) {
            $offerDetails = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Offer(' . $order['offer_percentage'] . ') %
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['offer_amount'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }
        if ($order['service_charge'] > 0) {
            $servicecharge = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Service Charge
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['service_charge'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }
        $tax_amount = '';
        if ($order['tax_amount'] > 0) {

            $tax_amount = ' <tr>
                          <td align="left" style="">
                             <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                <tr>
                                   <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                      Tax(' . $order['tax_percentage'] . ') %
                                   </td>
                                   <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                      ' . $siteSettings["site_currency"] . ' ' . number_format($order['tax_amount'], 2) . '
                                   </td>
                                </tr>
                             </table>
                          </td>
                       </tr>';
        }

        if ($order['voucher_amount'] > 0) {
            $voucherDetails = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Offer(' . $order['voucher_percentage'] . ') %
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['voucher_amount'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }

        if ($order['reward_used'] == 'N' && $order['reward_offer'] > 0) {
            $rewardDetails = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Offer(' . $order['reward_offer_percentage'] . ') %
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['reward_offer'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }
        $cartDetails = '';

        if (!empty($order['cart_view'])) {
            foreach ($order['cart_view'] as $cKey => $cValue) {
                $menuName = $cValue['menu_name'];
                $subAddons = ($cValue['subaddons_name'] != "") ? $cValue['subaddons_name'] : "";
                $cartDetails .= '
                <tr>
                    <td width="10%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . ($cKey + 1) . '</td>
                    <td width="45%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . $menuName . ' ' . $subAddons . '</td>
                    <td width="10%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . $cValue['quantity'] . '</td>
                    <td width="15%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">
                        ' . $siteSettings["site_currency"] . ' ' . number_format($cValue['menu_price'], 2) . '
                    </td>
                    <td width="20%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">
                        ' . $siteSettings["site_currency"] . ' ' . number_format($cValue['total_price'], 2) . '
                    </td>
                </tr>';
            }
        }

        if ($order['order_description'] != "") {
            $orderDescription = '<tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                           <tr>
                              <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Order Instruction : ' . $order['order_description'] . '</span>
                                 
                              </td>                              
                           </tr>
                        </table>
                     </td>
                  </tr>';
        }
        $statuss =  '';
        $comme = '';
        $statusss = '';
        if ($order['status'] == "Accepted") {
            $statuss .=  '  <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                           <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                    Thank you. your <span style="color:#000;font-weight:bold;">order ID ' . $order['order_number'] . '</span> has been Accepted.
                                    <br>
                                    Those lovely people should start cooking your food right about ... now.
                                 </p>
                              </td>
                           </tr>';
            $customerSubject = 'Order Accepted - ' . $order['order_number'];
            // $preparation .=  '<span style="font-weight: 700; display:block;">Preparation Time : ' . $order['preparation'] . '</span>';

            if ($order['order_type'] == "delivery") {
                $comme .= '<tr>
                            <td align="center" height="100%" valign="top" width="100%" style="padding:0px 0px 0px 0px;background-color:#fff">
                                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;">
                                   <tr>
                                      <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 14px; font-weight: 400; line-height: 20px; padding-top:5px">
                                            <b>Comments: </b> The order is to be delivered AS SOON AS POSSIBLE on ' . date('Y-m-d', strtotime($order['delivery_date'])) . ' Please note that changes in delivery time can occur. Any changes can be seen by <a href="#">clicking here</a>.
                   
                                      </td>
                                    </tr>
                                </table>
                             </td>
                        </tr>';
            } else {


                $comme .= '<tr>
                            <td align="center" height="100%" valign="top" width="100%" style="padding:0px 0px 0px 0px;background-color:#fff">
                                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;">
                                   <tr>
                                      <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 14px; font-weight: 400; line-height: 20px; padding-top:5px">
                                            <b>Comments: </b>  The order is to be Prepared AS SOON AS POSSIBLE on ' . date('Y-m-d', strtotime($order['delivery_date'])) . ' Please note that changes in delivery time can occur. Any changes can be seen by <a href="#">clicking here</a>.
                                         </td>
                                    </tr>
                                </table>
                             </td>
                        </tr>';
            }
        } else if ($order['status'] == "Delivered" && $order['order_type'] == "delivery") {
            $statuss .= '   
                            <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                            <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                         Thank you for Order.Order number ' . $order['order_number'] . ' has been Delivered.
                                 </p>
                              </td>
                           </tr>';
            $customerSubject = 'Order Delivered - ' . $order['order_number'];
        } else if ($order['status'] == "Failed") {
            $statuss .= '   <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                         We are sorry for the inconvience caused,your Order ' . $order['order_number'] . ' has been Failed for the reason ' . $order['failed_reason'] . '.
                                 </p>
                              </td>
                           </tr>';
            $customerSubject = 'Order Reject - ' . $order['order_number'];

            $statusss .= ' <tr>
                             <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                                   <tr>
                                      <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                         <span style="font-weight: 700; display:block;">Failed Reason : ' . $order['failed_reason'] . '</span>
                                         
                                      </td>                              
                                   </tr>
                                </table>
                             </td>
                          </tr> ';
        } else {
            if ($order['status'] == "Pending") {
                $customerSubject = 'Dinein Order Placed Successful - ' . $order['order_number'];
            }
        }

        if ($order['status'] == "Delivered" && $order['order_type'] == "pickup") {
            $statuss .= '   
                            <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                            <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                         Thank you for order. Order number ' . $order['order_number'] . ' has been Collected.
                                 </p>
                              </td>
                           </tr>';



            $customerSubject = 'Order Collected - ' . $order['order_number'];
        }
        $html = '
         <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
               <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                  <tr>
                     <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#f1e0c6">
                        <div style="display:inline-block; vertical-align:middle; text-align:center">
                           <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                              <tr>
                                 <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                    <img width="170" src="' . env('APP_URL') . 'images/logo-home.png">
                                 </td>
                              </tr>
                           </table>
                        </div>
                     </td>
                  </tr>
                  <tr>
                     <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                           ' . $statuss . '
                           
                           <tr>
                              <td align="left" style="padding-top: 20px;">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                   <td  width="10%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">S.No</td>
                                   <td width="45%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">Menu Name</td>
                                   <td width="10%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 700; line-height: 24px; padding: 10px;">Qty</td>
                                   <td width="15%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">
                                      Price
                                   </td>
                                   <td width="20%" align="right" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">
                                      Total Price
                                   </td>
                                </tr>
                                    ' . $cartDetails . '                                 
                                 </table>
                              </td>
                           </tr>   
                                 
                           
                           <tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Sub Total
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['order_sub_total'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>
                           ' . $tax_amount . '
                           ' . $servicecharge . '
                           ' . $offerDetails . '
                           ' . $voucherDetails . '
                           ' . $rewardDetails . '

                           <tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 600; line-height: 24px; padding: 10px; border-top: 3px solid #eeeeee; border-bottom: 3px solid #eeeeee;">
                                          TOTAL
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 600; line-height: 24px; padding: 10px; border-top: 3px solid #eeeeee; border-bottom: 3px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['order_grand_total'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>
                  <tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                           <tr>
                              <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Order Number/ID : ' . $order['order_number'] . '</span>
                                 <span style="font-weight: 700; display:block;">Payment method : ' . $paymentMethod . '</span>
                              </td>
                              <td align="right" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Order type : ' . ucfirst($order['order_type']) . '</span>
                                 <span style="font-weight: 700; display:block;">Payment status : ' . $paymentStatus . ' </span>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>
                  ' . $comme . '
                  ' . $orderDescription . '
                  ' . $statusss . '
                    <tr>   
                        	<td bgcolor="#fff" >
                        		<table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                        			<tr>
                        				<td style="padding: 30px 30px; text-align: right;">
                        					    <table align="left" style="text-align: left;">
                            						<tr>
                            							<td>
                            								   <p style="color:#000;"> Get The App : </p> 
                            							</td>
                            								<td width="10">&nbsp;</td>
                            							<td>
                            								<a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                            								<img src="' . env('APP_URL') . 'images/playstore.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                            							</td>
                            								<td width="10">&nbsp;</td>
                            							<td>
                            								<a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                            								<img src="' . env('APP_URL') . 'images/appstore.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                            							</td>
                            								<td width="10">&nbsp;</td>
                        					        </tr>
                            						
                            					</table>
                            					<table align="right" style="text-align: right;">
                        						<tr>
                        						    <td>
                        								    <p style="color:#000;">Follow Us : </p>
                        							</td>
                        								<td width="10">&nbsp;</td>
                        							<td>
                        								<a href="https://www.facebook.com/tiffintom/" target="_blank">
                        								<img src="' . env('APP_URL') . 'images/facebook.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td>
                        								<td width="10">&nbsp;</td>
                        							<td>
                        								<a href="https://twitter.com/TiffinTomUK/" target="_blank">
                        								<img src="' . env('APP_URL') . 'images/twitter.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td>
                        								<td width="10">&nbsp;</td>
                        							<td>
                        								<a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                        								<img src="' . env('APP_URL') . 'images/linkedin.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td><td width="10">&nbsp;</td>
                        							<td>
                        								<a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                        								<img src="' . env('APP_URL') . 'images/Instragram.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                        							</td><td width="10">&nbsp;</td>
                        						</tr>
                        					</table>
                        				</td>
                        			</tr>
                        		</table>
                        	</td>
                        </tr>
                  <tr>
                     <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#f1e0c6"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
                  </tr>
               </table>
            </td>
         </tr>';

        // User side mail
        if ($user->email_verify == 1) {
            if ($order->status == 'Pending' && ($user->order_place == 7 || $user->order_place == 4 || $user->order_place == 5 || $user->order_place == 6)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }

            if ($order->status == 'Accepted' && ($user->order_accept == 4 || $user->order_accept == 5 || $user->order_accept == 6 || $user->order_accept == 7)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }
            if ($order->status == 'Delivered' && ($user->order_delivered == 4 || $user->order_delivered == 5 || $user->order_delivered == 6 || $user->order_delivered == 7)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }
            if ($order->status == 'Failed' && ($user->order_reject == 4 || $user->order_reject == 5 || $user->order_reject == 6 || $user->order_reject == 7)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }
        }

        // Restaurant side Mail
        $email['body'] = $html;
        $email['subject'] = $customerSubject;
        $email['mail_type'] = $siteSettings->mail_option;
        $email['to'] = $restaurant->contact_email;
        $email['from'] = $order->customer_email;
        (new MailHelper())->sendMail($email);

        return 'Mail Send Successfully';
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
