<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomDomain;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomDomainController extends Controller
{
    /**
     * List domains
     * GET /tiffintom/domains
     */
    public function index(Request $req)
    {
        $domains = TiffintomDomain::orderBy('id', 'DESC');

        if ($query = $req->query('query', false)) {
            $domains->whereRaw('(domain_name LIKE ?)', ["%$query%"]);
        }

        if ($status = $req->query("status", false)) {
            $domains->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $domains = $domains->get();
        } else {
            $domains = $domains->paginate($req->query('per_page', 20));
        }

        if ($domains->isEmpty()) Utils::error('domains not found!', 404);

        return $domains;
    }


    /**
     * Create domain.
     * POST /tiffintom/domains
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'domain_name' => 'required|string',
            'status' => 'boolean',
        ]);

        $domains = new TiffintomDomain;
        $domains->fill($req->all());
        $domains->status = '1';
        $domains->save();

        return $domains;
    }

    /**
     * List domains
     * GET /tiffintom/domains/id
     */
    public function show(Request $req, $id)
    {
        $domains = TiffintomDomain::find($id);
        if (empty($domains)) Utils::error("domains not found", 404);
        return $domains;
    }

    /**
     * Update domains.
     * POST  /tiffintom/domains/id
     */
    public function update(Request $req, $id)
    {
        $domains = TiffintomDomain::find($id);
        if (empty($domains)) Utils::error('domains not found!', 404);

        $this->validate($req, [
            'domain_name' => 'required|string',
            'status' => 'boolean',
        ]);

        // update domains
        if ($req->has('domain_name')) $domains->domain_name = $req->input('domain_name');
        $domains->save();

        return $domains;
    }

    /**
     * Delete domains by id.
     * DELETE /tiffintom/domains/{id}
     */
    public function delete(Request $req, $id)
    {
        $domains = TiffintomDomain::find($id);
        if (empty($domains)) Utils::error('domains not found!', 404);
        $domains->delete();
        return $domains;
    }

    /**
     * Enable domains.
     * POST /tiffintom/domains/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $domains = TiffintomDomain::find($id);
        if (empty($domains)) Utils::error('domains not found!', 404);
        $domains->status = '1';
        $domains->save();
        return $domains;
    }

    /**
     * Disable domains.
     * POST /tiffintom/domains/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $domains = TiffintomDomain::find($id);
        if (empty($domains)) Utils::error('domains not found!', 404);
        $domains->status = '0';
        $domains->save();
        return $domains;
    }
}
