<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomDownloadDocument;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomDownloadDocumentController extends Controller
{

    /**
     * List download-documents
     * GET /tiffintom/download-documents
     */
    public function index(Request $req)
    {
        $download_documents = TiffintomDownloadDocument::orderBy('id', 'DESC');

        if ($status = $req->query('status', false)) {
            $download_documents->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $download_documents = $download_documents->get();
        } else {
            $download_documents = $download_documents->paginate($req->query('per_page', 20));
        }

        if ($download_documents->isEmpty()) Utils::error('No import documents found!', 404);

        return $download_documents;
    }


    /**
     * Create download-document.
     * POST /tiffintom/download-documents
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'type' => 'in:file,link',
            'path' => 'required_if:type,link',
        ]);

        // create download-document
        $download_document = new TiffintomDownloadDocument;
        $download_document->fill($req->all());

        if ($req->has('image') && $req->input('type') == 'file') {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PAYMENT_METHOD, 'tiffintom');
            $download_document->path = $fileName;
        }

        $download_document->save();
        return $download_document;
    }

    /**
     * List download-documents
     * GET /tiffintom/download-documents/id
     */
    public function show(Request $req, $id)
    {
        $download_documents = TiffintomDownloadDocument::find($id);
        if (empty($download_documents)) Utils::error("Import Document not found", 404);
        return $download_documents;
    }

    /**
     * Update payment method.
     * POST  /tiffintom/download-documents/id
     */
    public function update(Request $req, $id)
    {
        $download_document = TiffintomDownloadDocument::find($id);
        if (empty($download_document)) Utils::error('Import Document not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
            'type' => 'in:file,link',
            'path' => 'required_if:type,link',
        ]);

        // update download-document
        if ($req->has('name')) $download_document->name = $req->input('name');
        if ($req->has('type')) $download_document->type = $req->input('type');
        if ($req->has('image') && $req->input('type') == 'file') {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PAYMENT_METHOD, 'tiffintom');
            $download_document->payment_method_image = $fileName;
        }
        if ($req->has('path') && $req->input('type') == 'link') $download_document->path = $req->input('path');

        $download_document->save();

        return $download_document;
    }

    /**
     * Delete download-document by id.
     * DELETE /tiffintom/download-documents/{id}
     */
    public function delete(Request $req, $id)
    {
        $download_document = TiffintomDownloadDocument::find($id);
        if (empty($download_document)) Utils::error('Import Document not found!', 404);
        $download_document->delete();
        return $download_document;
    }
}