<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomDeliverySetting;
use App\ModelsTiffintom\TiffintomFavouriteRestaurant;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomReview;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomUser;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomFavouriteRestaurantController extends Controller
{


    /**
     * List favourite-restaurant
     * GET /tiffintom/favourite-restaurant
     */
    public function index(Request $req)
    {
        $favourite_restaurant = TiffintomFavouriteRestaurant::orderBy('id', 'ASC');

        if ($customer_id = $req->query('customer_id', false)) {
            $favourite_restaurant->where('customer_id', $customer_id);
        }

        if ($restaurant_id = $req->query('restaurant_id', false)) {
            $favourite_restaurant->where('restaurant_id', $restaurant_id);
        }

        if ($is_favourite = $req->query('is_favourite', false)) {
            $favourite_restaurant->where('is_favourite', $is_favourite);
        }

        if ($req->query('nopaginate', false) == '1') {
            $favourite_restaurant = $favourite_restaurant->get();
            $favourite_restaurants = array();
            foreach ($favourite_restaurant as $key => $value) {
                $restaurants = TiffintomRestaurant::where('id', $value->restaurant_id)->first();

                $value['restaurant'] = $restaurants;
                $sourcelatitude = $req->input('latitude');
                $sourcelongitude = $req->input('longitude');
                $latitudeTo  = $restaurants['sourcelatitude'];
                $longitudeTo = $restaurants['sourcelongitude'];
                $unit = 'K';

                $distance = $this->getDistanceCalculate(
                    $sourcelatitude,
                    $sourcelongitude,
                    $latitudeTo,
                    $longitudeTo,
                    $unit
                );
                $value['restaurant']['distance'] = (float)$distance;

                list($deliveryCharge, $message) = $this->getDeliveryCharge($restaurants['id'], $distance, $restaurants['sourcelatitude'], $restaurants['sourcelongitude']);
                if ($deliveryCharge <= 0) {
                    $DeliverySettings = TiffintomDeliverySetting::where('restaurant_id', $restaurants['id'])->first();
                    $deliveryCharge = $DeliverySettings['delivery_charge'];
                }
                $value['restaurant']['deliveryCharges'] = $deliveryCharge;

                $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurants['id'])->where('status', '1')->first();
                if (!empty($reviews)) {
                    if ($reviews->count > 0) {
                        $value['restaurant']['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                        $value['restaurant']['total_reviews'] = $reviews->count;
                    }
                }

                $restaurants->append('restaurant_timing');

                $currentTime = strtotime(date('h:i A'));
                $currentDay = strtolower(date('l'));

                $firstOpenTime = strtotime($restaurants[$currentDay . '_first_opentime']);
                $firstCloseTime = strtotime($restaurants[$currentDay . '_first_closetime']);

                $secondOpenTime = strtotime($restaurants[$currentDay . '_second_opentime']);
                $secondCloseTime = strtotime($restaurants[$currentDay . '_second_closetime']);

                $pickfirstOpenTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM');
                $pickfirstCloseTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM');

                $picksecondOpenTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM');
                $picksecondCloseTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM');

                $req = Request::create("");
                $req->merge(['restaurant_id' => $restaurants['id'], 'date' => date('Y-m-d')]);
                $TimeSlots = $this->restaurant_timeslot($req);
                $deliveryTimeSlots = $TimeSlots['delivery'];
                $pickupTimeSlots = $TimeSlots['pickup'];

                if ($deliveryTimeSlots['today'] != '' || $pickupTimeSlots['today'] != '') {
                    if (($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $pickfirstCloseTime)) {
                        $restaurant['currentStatus'] = 'Open';
                    } else if (($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) || ($currentTime > $picksecondOpenTime && $currentTime <= $picksecondCloseTime)) {
                        $restaurant['currentStatus'] = 'Open';
                    } else {
                        $restaurant['currentStatus'] = 'PreOrder';
                    }
                } else if ($deliveryTimeSlots["tomorrow"] != '' || $pickupTimeSlots["tomorrow"] != '' || $deliveryTimeSlots["dayaftertomorrow"] != '' || $pickupTimeSlots["dayaftertomorrow"] != '') {
                    $restaurant['currentStatus'] = 'PreOrder';
                } else {
                    $restaurant['currentStatus'] = 'Closed';
                }

                $order_count = TiffintomOrder::where('customer_id', $value->customer_id)->where('restaurant_id', $value->restaurant_id)->count();
                $value['order_count'] = $order_count;
                $favourite_restaurants[] = $value;
            }
            if (empty($favourite_restaurants)) Utils::error('No favourite restaurant found!', 404);

            return $favourite_restaurants;
        } else {
            $favourite_restaurant = $favourite_restaurant->paginate($req->query('per_page', 20));
        }

        if ($favourite_restaurant->isEmpty()) Utils::error('No favourite restaurant found!', 404);

        return $favourite_restaurant;
    }

    /**
     * Create chat.
     * POST /tiffintom/chat
     */
    public function create_update(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'exists:tiffintom_connection.users,id',
            'is_favourite' => 'boolean',
        ]);

        $favouriteRestaurant = TiffintomFavouriteRestaurant::where('restaurant_id', $req->input('restaurant_id'))->where('customer_id', $req->input('customer_id'))->first();
        if (empty($favouriteRestaurant)) {
            $favouriteRestaurant = new TiffintomFavouriteRestaurant();
        }
        $favouriteRestaurant->fill($req->all());
        $favouriteRestaurant->save();

        return $favouriteRestaurant;
    }

    //Distance calculate
    public function getDistanceCalculate($latitudeFrom, $longitudeFrom, $latitudeTo, $longitudeTo, $unit)
    {
        $theta = $longitudeFrom - $longitudeTo;
        $dist = sin(deg2rad((float)$latitudeFrom)) * sin(deg2rad((float)$latitudeTo)) +  cos(deg2rad((float)$latitudeFrom)) * cos(deg2rad((float)$latitudeTo)) * cos(deg2rad((float)$theta));
        $dist = acos($dist);
        $dist = rad2deg($dist);
        $miles = $dist * 60 * 1.1515;
        $unit = strtoupper($unit);
        if ($unit == "K") {
            return number_format(($miles), 2);
        } else if ($unit == "N") {
            return ($miles * 0.8684) . ' nm';
        } else {
            return $miles . ' mi';
        }
    }

    //-------------------------------------------------------------------------------------------------------------
    public function getDeliveryCharge($resId, $Durations, $lat, $lng)
    {

        $message = '';

        $restDetails = TiffintomRestaurant::leftJoin('delivery_settings', 'delivery_settings.restaurant_id', '=', 'restaurants.id')->where('restaurants.id', $resId)->orderBy('delivery_settings.delivery_miles', 'ASC')->first();

        $SettingsList = TiffintomSiteSetting::where('id', '1')->first();

        $deliveryCharge = $count = 0;

        if ($restDetails['map_mode'] == 'Circle') {
            foreach ($restDetails['delivery_settings'] as $key => $value) {
                if ($Durations <= $value['delivery_miles'] + $SettingsList['site_mile']) {
                    $deliveryCharge = $value['delivery_charge'];
                    $count = '1';
                    break;
                }
            }
            if ($count == 1) {
                $message = 'success';
            } else {
                $message = 'failed';
            }
        } else {
            $message = 'failed';
        }
        return array($deliveryCharge, $message);
    }

    /**
     * Update restaurant.
     * POST  /tiffintom/restaurants/time-slot
     */
    public function restaurant_timeslot(Request $req)
    {
        $restaurants = TiffintomRestaurant::find($req->query("restaurant_id"));
        if (empty($restaurants)) Utils::error("No restaurant found", 404);
        $restaurants->append('restaurant_timing');

        $deliveryTimeSlots = $this->getTimeSlot($req->query("date"), "delivery", $req->query("restaurant_id"));
        $pickupTimeSlots = $this->getTimeSlot($req->query("date"), "pickup", $req->query("restaurant_id"));

        $timeSlot = array();
        $timeSlot['delivery'] = $deliveryTimeSlots;
        $timeSlot['pickup'] = $pickupTimeSlots;

        return $timeSlot;
    }


    public function getTimeSlot($date, $type, $resid)
    {
        if ($date != '') {
            $restaurantDetails = TiffintomRestaurant::find($resid);
            if (empty($restaurantDetails)) Utils::error("No restaurant found", 404);
            $restaurantDetails->append('restaurant_timing');

            //Timing Section
            $nowTime = date('h:i A');
            $currentTime = strtotime($nowTime);
            $currentDates = date("d-m-Y", strtotime($date));
            $carbonDate = Carbon::parse($date);
            $tomorrow = $carbonDate->addDays(1)->format('d-m-Y');
            $dayaftertomorrow = $carbonDate->addDays(1)->format('d-m-Y');
            $timesArray = array($currentDates, $tomorrow, $dayaftertomorrow);
            $timeSlots = array();
            $deliveryTimeSlots = array();
            $pickupTimeSlots = array();
            foreach ($timesArray as $key => $date) {
                $today = date('d-m-Y');
                $currentDate = date("d-m-Y", strtotime($date));

                $currentDay = strtolower(date('l', strtotime($currentDate)));
                if ($type == 'delivery' || $type == '') {
                    $firstStartTime = $restaurantDetails[$currentDay . '_first_opentime'];
                    $firstEndTime = $restaurantDetails[$currentDay . '_first_closetime'];
                    $secondStartTime = $restaurantDetails[$currentDay . '_second_opentime'];
                    $secondEndTime = $restaurantDetails[$currentDay . '_second_closetime'];

                    $firstOpenTime = strtotime($restaurantDetails[$currentDay . '_first_opentime']);
                    $firstCloseTime = strtotime($restaurantDetails[$currentDay . '_first_closetime']);
                    $secondOpenTime = strtotime($restaurantDetails[$currentDay . '_second_opentime']);
                    $secondCloseTime = strtotime($restaurantDetails[$currentDay . '_second_closetime']);
                    $restentitytiming =    $restaurantDetails[$currentDay . '_status'];
                    $estimate_time = ($restaurantDetails[$currentDay . '_estimate_time'] > 0) ? $restaurantDetails[$currentDay . '_estimate_time'] : $restaurantDetails['estimate_time'];
                } else {

                    $firstStartTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM';
                    $firstEndTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM';
                    $secondStartTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM';
                    $secondEndTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM';

                    $firstOpenTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM');
                    $firstCloseTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM');
                    $secondOpenTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM');
                    $secondCloseTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM');
                    $restentitytiming = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_status']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_status'] : 'Close';
                    $estimate_time = 0;
                }
                //In first Timing section
                $array_of_time = array();

                if ($restentitytiming != 'Close' && $restaurantDetails['close'] != 0) {

                    $restaurantDetails['currentStatus'] = 'Open';
                    $final[] = $restaurantDetails;
                    if (strtotime($today) != strtotime($currentDate)) {
                        $nowTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $nowTime);
                        $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                            if ($firstStartTime != $firstEndTime) {
                                if ($counts == 0) {
                                    $fifteen_minsss = $estimate_time * 60;
                                    $start_time += $fifteen_minsss;
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $counts++;
                                } else {
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $counts++;
                                }
                            }
                            $start_time += $fifteen_mins;
                        }

                        $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                        $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                        $fifteen_mins = 15 * 60;
                        $countsd = 0;

                        while ($start_time <= $end_time) {
                            $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                            if ($secondStartTime != $secondEndTime) {
                                if ($countsd == 0) {
                                    $fifteen_minsss = $estimate_time * 60;
                                    $start_time += $fifteen_minsss;
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $countsd++;
                                } else {
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $countsd++;
                                }
                            }
                            $start_time += $fifteen_mins;
                        }
                    } else {
                        if ($currentTime < $firstOpenTime) {
                            $nowTimes = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $nowTimes);
                            $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                    if ($firstStartTime != $firstEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                        if ($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) {
                            $firstOpenTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $firstOpenTime);
                            $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                    if ($firstStartTime != $firstEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                            $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                            $start_time1 = strtotime($currentDate . ' ' . $secondStartTimes);
                            $end_time1 = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $countsd = 0;
                            while ($start_time1 <= $end_time1) {
                                if ($currentTime <= $start_time1) {
                                    $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStartTime != $secondEndTime) {
                                        if ($countsd == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time1 += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time1);
                                            $countsd++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time1);
                                            $countsd++;
                                        }
                                    }
                                }
                                $start_time1 += $fifteen_mins;
                            }
                        }
                        if (empty($array_of_time)) {
                            if ($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime) {
                                $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                                $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                                $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                                $fifteen_mins = 15 * 60;
                                $counts = 0;
                                while ($start_time <= $end_time) {
                                    if ($currentTime <= $start_time) {
                                        $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                        if ($secondStartTime != $secondEndTime) {
                                            if ($counts == 0) {
                                                $fifteen_minsss = $estimate_time * 60;
                                                $start_time += $fifteen_minsss;
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $counts++;
                                            } else {
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $counts++;
                                            }
                                        }
                                    }
                                    $start_time += $fifteen_mins;
                                }
                            } else {
                                if ($currentTime < $secondOpenTime) {
                                    $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                                    $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                                    $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                                    $fifteen_mins = 15 * 60;
                                    $countsd = 0;
                                    while ($start_time <= $end_time) {
                                        if ($currentTime <= $start_time) {
                                            $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                            if ($secondStartTime != $secondEndTime) {
                                                if ($countsd == 0) {
                                                    $fifteen_minsss = $estimate_time * 60;
                                                    $start_time += $fifteen_minsss;
                                                    $array_of_time[] = date("h:i A", $start_time);
                                                    $countsd++;
                                                } else {
                                                    $array_of_time[] = date("h:i A", $start_time);
                                                    $countsd++;
                                                }
                                            }
                                        }
                                        $start_time += $fifteen_mins;
                                    }
                                }
                            }
                        } else {
                            if ($currentTime < $secondOpenTime) {
                                $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                                $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                                $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                                $fifteen_mins = 15 * 60;
                                $countsd = 0;
                                while ($start_time <= $end_time) {
                                    if ($currentTime <= $start_time) {
                                        $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                        if ($secondStartTime != $secondEndTime) {
                                            if ($countsd == 0) {
                                                $fifteen_minsss = $estimate_time * 60;
                                                $start_time += $fifteen_minsss;
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $countsd++;
                                            } else {
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $countsd++;
                                            }
                                        }
                                    }
                                    $start_time += $fifteen_mins;
                                }
                            }
                        }
                        if ($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) {
                            $currentTime = date("h:i A", strtotime('+0 minutes', $currentTime));
                            $start_time = strtotime($currentDate . ' ' . $currentTime);
                            $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStartTime != $secondEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                    }
                }
                $timeList = '';
                if (!empty($array_of_time)) {
                    foreach ($array_of_time as $value) {
                        $timeList .= $value . ',';
                    }
                }
                if ($type == 'delivery' || $type == '') {
                    if ($key == 0) {
                        $deliveryTimeSlots["today"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 1) {
                        $deliveryTimeSlots["tomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 2) {
                        $deliveryTimeSlots["dayaftertomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                } else {
                    if ($key == 0) {
                        $pickupTimeSlots["today"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 1) {
                        $pickupTimeSlots["tomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 2) {
                        $pickupTimeSlots["dayaftertomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                }
            }
        }
        if ($type == 'delivery' || $type == '') {
            $data = $deliveryTimeSlots;
        }
        if ($type == 'pickup') {
            $data = $pickupTimeSlots;
        }
        return $data;
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
