<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomKeyword;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomKeywordController extends Controller
{
    /**
     * List keywords
     * GET /tiffintom/keywords
     */
    public function index(Request $req)
    {
        $keywords = TiffintomKeyword::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $keywords->whereRaw('(key_type LIKE ? OR keyword_type LIKE ?  OR keyword_value LIKE ?  OR page_type LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($status = $req->query("status", false)) {
            $keywords->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $keywords = $keywords->get();
        } else {
            $keywords = $keywords->paginate($req->query('per_page', 20));
        }

        if ($keywords->isEmpty()) Utils::error('Keywords not found!', 404);

        return $keywords;
    }

    /**
     * Create keyword.
     * POST /tiffintom/keywords
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'key_type' => 'required|string',
            'keyword_type' => 'required|string',
            'keyword_value' => 'required|string',
            'page_type' => 'required|string',
        ]);

        // create keyword
        $keyword = new TiffintomKeyword;
        $keyword->fill($req->all());
        $keyword->status = '1';
        $keyword->save();
        return $keyword;
    }

    /**
     * List keywords
     * GET /tiffintom/keywords/id
     */
    public function show(Request $req, $id)
    {
        $keyword = TiffintomKeyword::find($id);
        if (empty($keyword)) Utils::error("Keyword not found", 404);
        return $keyword;
    }

    /**
     * Update keyword.
     * POST  /tiffintom/keywords/id
     */
    public function update(Request $req, $id)
    {
        $keyword = TiffintomKeyword::find($id);
        if (empty($keyword)) Utils::error('Keyword not found!', 404);

        $this->validate($req, [
            'key_type' => 'required|string',
            'keyword_type' => 'required|string',
            'keyword_value' => 'required|string',
            'page_type' => 'required|string',
        ]);

        // update keyword
        if ($req->has('key_type')) $keyword->key_type = $req->input('key_type');
        if ($req->has('keyword_type')) $keyword->keyword_type = $req->input('keyword_type');
        if ($req->has('keyword_value')) $keyword->keyword_value = $req->input('keyword_value');
        if ($req->has('page_type')) $keyword->page_type = $req->input('page_type');
        $keyword->save();

        return $keyword;
    }

    /**
     * Delete keyword by id.
     * DELETE /tiffintom/keywords/{id}
     */
    public function delete(Request $req, $id)
    {
        $keyword = TiffintomKeyword::find($id);
        if (empty($keyword)) Utils::error('Keyword not found!', 404);
        $keyword->delete();

        return $keyword;
    }

    /**
     * Enable keyword.
     * POST /tiffintom/keywords/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $keyword = TiffintomKeyword::find($id);
        if (empty($keyword)) Utils::error('Keyword not found!', 404);
        $keyword->status = '1';
        $keyword->save();

        return $keyword;
    }

    /**
     * Disable keyword.
     * POST /tiffintom/keywords/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $keyword = TiffintomKeyword::find($id);
        if (empty($keyword)) Utils::error('Keyword not found!', 404);
        $keyword->status = '0';
        $keyword->save();

        return $keyword;
    }
}
