<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomLanguageContent;
use App\ModelsTiffintom\TiffintomLanguageSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomLanguageSettingController extends Controller
{
    /**
     * List language-settings
     * GET /tiffintom/language-settings
     */
    public function index(Request $req)
    {
        $language_settings = TiffintomLanguageSetting::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $language_settings->whereRaw('(language_name LIKE ? OR language_code LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($status = $req->query("status", false)) {
            $language_settings->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $language_settings = $language_settings->get();
        } else {
            $language_settings = $language_settings->paginate($req->query('per_page', 20));
        }

        if ($language_settings->isEmpty()) Utils::error('Language settings not found!', 404);

        return $language_settings;
    }

    /**
     * Create language-setting.
     * POST /tiffintom/language-settings
     */
    public function create(Request $req)
    {
        if ($req->has('language_contents') && !is_array($req->input('language_contents'))) {
            $req->merge(['language_contents' => json_decode($req->input('language_contents'))]);
        }

        $this->validate($req, [
            'language_name' => 'required|string',
            'language_code' => 'required|string',
            'status' => 'boolean',
            'language_default' => 'boolean',
            'language_contents' => 'array'
        ]);

        // create language-setting
        $language_setting = new TiffintomLanguageSetting;
        $language_setting->fill($req->all());
        $language_setting->status = '1';
        $language_setting->save();

        if ($req->has('language_contents')) {
            $language_contents = ($req->input('language_contents'));
            foreach ($language_contents as $item) {
                $languageContent = new TiffintomLanguageContent((array)$item);
                $languageContent->language_id = $language_setting->id;
                $languageContent->save();
            }
        }

        return $language_setting;
    }

    /**
     * List language-settings
     * GET /tiffintom/language-settings/id
     */
    public function show(Request $req, $id)
    {
        $language_setting = TiffintomLanguageSetting::find($id);
        if (empty($language_setting)) Utils::error("Language setting not found", 404);
        return $language_setting;
    }

    /**
     * Update language-setting.
     * POST  /tiffintom/language-settings/id
     */
    public function update(Request $req, $id)
    {
        $language_setting = TiffintomLanguageSetting::find($id);
        if (empty($language_setting)) Utils::error('Language setting not found!', 404);

        if ($req->has('language_contents') && !is_array($req->input('language_contents'))) {
            $req->merge(['language_contents' => json_decode($req->input('language_contents'))]);
        }

        $this->validate($req, [
            'language_name' => 'required|string',
            'language_code' => 'required|string',
            'status' => 'boolean',
            'language_default' => 'boolean',
            'language_contents' => 'array'
        ]);

        // update language-setting
        if ($req->has('language_name')) $language_setting->language_name = $req->input('language_name');
        if ($req->has('language_code')) $language_setting->language_code = $req->input('language_code');
        $language_setting->status = $req->input('status');
        $language_setting->language_default = $req->input('language_default');
        $language_setting->save();

        if ($req->has('language_contents')) {
            $oldItems = TiffintomLanguageContent::where('language_id', $language_setting->id)->delete();
            $language_contents = ($req->input('language_contents'));
            foreach ($language_contents as $item) {
                $languageContent = new TiffintomLanguageContent((array)$item);
                $languageContent->language_id = $language_setting->id;
                $languageContent->save();
            }
        }

        return $language_setting;
    }

    /**
     * Delete language-setting by id.
     * DELETE /tiffintom/language-settings/{id}
     */
    public function delete(Request $req, $id)
    {
        $language_setting = TiffintomLanguageSetting::find($id);
        if (empty($language_setting)) Utils::error('Language setting not found!', 404);
        $language_setting->delete();

        $language_contents = TiffintomLanguageContent::where('language_id', $id)->delete();

        return $language_setting;
    }

    /**
     * Enable language-setting.
     * POST /tiffintom/language-settings/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $language_setting = TiffintomLanguageSetting::find($id);
        if (empty($language_setting)) Utils::error('Language setting not found!', 404);
        $language_setting->status = '1';
        $language_setting->save();

        return $language_setting;
    }

    /**
     * Disable language-setting.
     * POST /tiffintom/language-settings/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $language_setting = TiffintomLanguageSetting::find($id);
        if (empty($language_setting)) Utils::error('Language setting not found!', 404);
        $language_setting->status = '0';
        $language_setting->save();

        return $language_setting;
    }

    /**
     * Enable language-setting.
     * POST /tiffintom/language-settings/{id}/enable-default
     */
    public function enable_default(Request $req, $id)
    {
        $language_setting = TiffintomLanguageSetting::where('language_default', '1')->update(['language_default' => 0]);

        $language_setting = TiffintomLanguageSetting::find($id);
        if (empty($language_setting)) Utils::error('Language setting not found!', 404);
        $language_setting->language_default = '1';
        $language_setting->save();

        return $language_setting;
    }

    /**
     * Disable language-setting.
     * POST /tiffintom/language-settings/{id}/disable-default
     */
    public function disable_default(Request $req, $id)
    {
        $language_setting = TiffintomLanguageSetting::find($id);
        if (empty($language_setting)) Utils::error('Language setting not found!', 404);
        $language_setting->language_default = '0';
        $language_setting->save();

        return $language_setting;
    }
}
