<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomLocation;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomLocationController extends Controller
{
    /**
     * List locations
     * GET /tiffintom/locations
     */
    public function index(Request $req)
    {
        $locations = TiffintomLocation::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $locations->whereRaw('(area_name LIKE ? OR zip_code LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($city_id = $req->query("city_id", false)) {
            $locations->where('city_id', $city_id);
        }

        if ($state_id = $req->query("state_id", false)) {
            $locations->where('state_id', $state_id);
        }

        if ($status = $req->query("status", false)) {
            $locations->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $locations = $locations->get();
        } else {
            $locations = $locations->paginate($req->query('per_page', 20));
        }

        if ($locations->isEmpty()) Utils::error('Locations not found!', 404);

        return $locations;
    }

    /**
     * Create location.
     * POST /tiffintom/locations
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'state_id' => 'required|exists:tiffintom_connection.states,id',
            'city_id' => 'required|exists:tiffintom_connection.cities,id',
            'area_name' => 'required|string',
            'zip_code' => 'required|string',
        ]);

        // create location
        $location = new TiffintomLocation;
        $location->fill($req->all());
        $location->status = '1';
        $location->save();
        return $location;
    }

    /**
     * List locations
     * GET /tiffintom/locations/id
     */
    public function show(Request $req, $id)
    {
        $locations = TiffintomLocation::find($id);
        if (empty($locations)) Utils::error("Location not found", 404);
        return $locations;
    }

    /**
     * Update location.
     * POST  /tiffintom/locations/id
     */
    public function update(Request $req, $id)
    {
        $location = TiffintomLocation::find($id);
        if (empty($location)) Utils::error('Location not found!', 404);

        $this->validate($req, [
            'state_id' => 'required|exists:tiffintom_connection.states,id',
            'city_id' => 'required|exists:tiffintom_connection.cities,id',
            'area_name' => 'required|string',
            'zip_code' => 'required|string',
        ]);

        // update country
        if ($req->has('country_id')) $location->country_id = $req->input('country_id');
        if ($req->has('state_id')) $location->state_id = $req->input('state_id');
        if ($req->has('location_name')) $location->location_name = $req->input('location_name');
        $location->save();

        return $location;
    }

    /**
     * Delete location by id.
     * DELETE /tiffintom/locations/{id}
     */
    public function delete(Request $req, $id)
    {
        $location = TiffintomLocation::find($id);
        if (empty($location)) Utils::error('Location not found!', 404);
        $location->delete();
        return $location;
    }

    /**
     * Enable location.
     * POST /tiffintom/locations/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $location = TiffintomLocation::find($id);
        if (empty($location)) Utils::error('Location not found!', 404);
        $location->status = '1';
        $location->save();
        return $location;
    }

    /**
     * Disable location.
     * POST /tiffintom/locations/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $location = TiffintomLocation::find($id);
        if (empty($location)) Utils::error('Location not found!', 404);
        $location->status = '0';
        $location->save();
        return $location;
    }
}