<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomMainAddon;
use App\ModelsTiffintom\TiffintomSubAddon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class TiffintomMainAddonController extends Controller
{

    /**
     * List main-addons
     * GET /tiffintom/main-addons
     */
    public function index(Request $req)
    {
        $main_addons = TiffintomMainAddon::selectRaw('mainaddons.*,categories.category_name as category_name,restaurants.restaurant_name as restaurant_name')->leftJoin('categories', 'categories.id', '=', 'category_id')->leftJoin('restaurants', 'restaurants.id', '=', 'mainaddons.restaurant_id')->orderBy('mainaddons.sortorder', 'ASC');

        if ($query = $req->query("query", false)) {
            $main_addons->whereRaw('(mainaddons_name LIKE ? OR admin_name LIKE ?)', ["%$query%", "%$query%"]);
        }
        if ($category_id = $req->query("category_id", false)) {
            $main_addons->where('category_id', $category_id);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $main_addons->where('mainaddons.restaurant_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $main_addons = $main_addons->get();
        } else {
            $main_addons = $main_addons->paginate($req->query('per_page', 20));
        }

        if ($main_addons->isEmpty()) Utils::error('No addons found!', 404);

        return $main_addons;
    }

    /**
     * Create main-addon.
     * POST /tiffintom/main-addons
     */
    public function create(Request $req)
    {
        if ($req->has('sub_addons') && !is_array($req->input('sub_addons'))) {
            $req->merge(['sub_addons' => json_decode($req->input('sub_addons'))]);
        }

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_id' => 'required|exists:tiffintom_connection.categories,id',
            'mainaddons_name' => 'required|string|unique:tiffintom_connection.mainaddons,mainaddons_name',
            'admin_name' => 'string',
            'mainaddons_mini_count' => 'numeric',
            'mainaddons_count' => 'numeric',
            'sortorder' => 'numeric',
            'delete_status' => 'in:Y,N',
            'status' => 'boolean',
            'sub_addons' => 'required|array'
        ]);

        // create main_addon
        $main_addon = new TiffintomMainAddon;
        $main_addon->fill($req->all());
        if ($req->has('mainaddons_name')) $main_addon->mainaddons_name = strtoupper($main_addon->mainaddons_name . ':');
        $main_addon->save();
        $sub_addons = ($req->input('sub_addons'));
        foreach ($sub_addons as $sub_addon) {
            $subAddons = new TiffintomSubAddon((array)$sub_addon);
            $subAddons->subaddons_name = strtoupper($subAddons->subaddons_name);
            $subAddons->mainaddons_id = $main_addon->id;
            $subAddons->save();
        }
        return $main_addon;
    }

    /**
     * List main-addon
     * GET /tiffintom/main-addons/id
     */
    public function show(Request $req, $id)
    {
        $main_addons = TiffintomMainAddon::find($id);
        if (empty($main_addons)) Utils::error("addons not found", 404);
        return $main_addons;
    }

    /**
     * Update main-addon.
     * POST  /tiffintom/main-addons/id
     */
    public function update(Request $req, $id)
    {
        if ($req->has('sub_addons') && !is_array($req->input('sub_addons'))) {
            $req->merge(['sub_addons' => json_decode($req->input('sub_addons'))]);
        }
        $main_addon = TiffintomMainAddon::find($id);
        if (empty($main_addon)) Utils::error('main_addons not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_id' => 'required|exists:tiffintom_connection.categories,id',
            'mainaddons_name' =>  ['required', Rule::unique('tiffintom_connection.mainaddons')->where(function ($query) use ($req) {
                return $query->where('restaurant_id', $req->input('restaurant_id'))->where('category_id', $req->input('category_id'))->where('id', '!=', $req->input('id'));
            })],
            'admin_name' => 'string',
            'mainaddons_mini_count' => 'numeric',
            'mainaddons_count' => 'numeric',
            'sortorder' => 'numeric',
            'delete_status' => 'in:Y,N',
            'status' => 'boolean',
            'sub_addons' => 'required|array'
        ]);

        // update main_addon
        if ($req->has('restaurant_id')) $main_addon->restaurant_id = $req->input('restaurant_id');
        if ($req->has('category_id')) $main_addon->category_id = $req->input('category_id');
        if ($req->has('mainaddons_name')) $main_addon->mainaddons_name = strtoupper($req->input('mainaddons_name'));
        if ($req->has('admin_name')) $main_addon->admin_name = $req->input('admin_name');
        if ($req->has('delete_status')) $main_addon->delete_status = $req->input('delete_status');
        $main_addon->mainaddons_mini_count = $req->input('mainaddons_mini_count');
        $main_addon->mainaddons_count = $req->input('mainaddons_count');
        $main_addon->sortorder = $req->input('sortorder');
        $main_addon->status = $req->input('status');
        $main_addon->save();

        $sub_addons = ($req->input('sub_addons'));
        foreach ($sub_addons as $sub_addon) {
            if (!isset($sub_addon->id)) {
                $subAddons = new TiffintomSubAddon((array)$sub_addon);
                $subAddons->subaddons_name = strtoupper($subAddons->subaddons_name);
                $subAddons->mainaddons_id = $main_addon->id;
                $subAddons->save();
            } else {
                $subAddons = TiffintomSubAddon::find($sub_addon->id);
                $subAddons->mainaddons_id = $main_addon->id;
                $subAddons->category_id = $main_addon->category_id;
                $subAddons->subaddons_name = strtoupper($sub_addon->subaddons_name);
                $subAddons->subaddons_price = $sub_addon->subaddons_price;
                $subAddons->save();
            }
        }

        return $main_addon;
    }

    /**
     * Delete main-addon by id.
     * DELETE /tiffintom/main-addons/{id}
     */
    public function delete(Request $req, $id)
    {
        $main_addon = TiffintomMainAddon::find($id);
        if (empty($main_addon)) Utils::error('addons not found!', 404);
        $main_addon->delete();
        return $main_addon;
    }

    /**
     * Delete main-addon by id.
     * DELETE /tiffintom/main-addons/{id}
     */
    public function delete_subaddon(Request $req, $id)
    {
        $sub_addon = TiffintomSubAddon::find($id);
        if (empty($sub_addon)) Utils::error('sub addons not found!', 404);
        $sub_addon->delete();
        return $sub_addon;
    }

    /**
     * Enable main-addon.
     * POST /tiffintom/main-addons/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $main_addon = TiffintomMainAddon::find($id);
        if (empty($main_addon)) Utils::error('addons not found!', 404);
        $main_addon->status = '1';
        $main_addon->save();
        return $main_addon;
    }

    /**
     * Disable main-addon.
     * POST /tiffintom/main-addons/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $main_addon = TiffintomMainAddon::find($id);
        if (empty($main_addon)) Utils::error('addons not found!', 404);
        $main_addon->status = '0';
        $main_addon->save();
        return $main_addon;
    }

    /**
     * Bulk Upload addon.
     * POST /tiffintom/addons/bulk
     */
    public function bulk_addon(Request $req)
    {
        if ($req->has('addons') && !is_array($req->input('addons')))
            $req->merge(['addons' => json_decode($req->input('addons'))]);
        $this->validate($req, [
            'addons' => 'required|array'
        ]);

        $restaurantAddons = ($req->input('addons'));
        $count = 1;
        foreach ($restaurantAddons as $addon) {
            $addonObj = TiffintomMainAddon::find($addon->id);
            $addonObj->sortorder = $count;
            $addonObj->save();
            $count++;
        }
        return $restaurantAddons;
    }
}
