<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomMainAddon;
use App\ModelsTiffintom\TiffintomMenu;
use App\ModelsTiffintom\TiffintomMenuAddons;
use App\ModelsTiffintom\TiffintomMenuVariant;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomMenuController extends Controller
{


    /**
     * List menu
     * GET /tiffintom/menu
     */
    public function index(Request $req)
    {
        $menu = TiffintomMenu::selectRaw('restaurant_menus.*,categories.category_name as category_name,restaurants.restaurant_name as restaurant_name')->leftJoin('categories', 'categories.id', '=', 'category_id')->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_menus.restaurant_id')->orderBy('restaurant_menus.sortorder', 'ASC');

        if ($query = $req->query("query", false)) {
            $menu->whereRaw('(menu_name LIKE ? )', ["%$query%"]);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $menu->where('restaurant_menus.restaurant_id', $query);
        }
        if ($query = $req->query("category_id", false)) {
            $menu->where('category_id', $query);
        }
        $menu->with(['variants']);

        if ($req->query('nopaginate', false) == '1') {
            $menu = $menu->get();
        } else {
            $menu = $menu->paginate($req->query('per_page', 20));
        }

        if ($menu->isEmpty()) Utils::error('No menu found!', 404);

        return $menu;
    }

    /**
     * Create menu.
     * POST /tiffintom/categories
     */
    public function create(Request $req)
    {
        if ($req->has('menu_details') && !is_array($req->input('menu_details'))) {
            $req->merge(['menu_details' => json_decode($req->input('menu_details'))]);
        }

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_id' => 'required|exists:tiffintom_connection.categories,id',
            'menu_name' => 'required|string',
            'is_suggest' => 'string',
            'price_option' => 'in:single,multiple',
            'menu_description' => 'string',
            'menu_type' => 'in:veg,nonveg,other',
            'menu_addon' => 'in:Yes,No',
            'popular_dish' => 'in:Yes,No',
            'featured' => 'in:Yes,No',
            'spicy_dish' => 'in:mild,medium,extra_hot,slightly_hot,hot,no_option',
            'favourite' => 'in:Y,N',
            'status' => 'boolean',
            'delete_status' => 'in:Y,N',
            'sortorder' => 'numeric',
            'short_name' => 'string',
            'vegetarian' => 'numeric',
            'contains_nuts' => 'numeric',
            'milk' => 'numeric',
            'mustard' => 'numeric',
            'eggs' => 'numeric',
            'fish' => 'numeric',
            'gluten_free' => 'numeric',
            'halal' => 'numeric',
            'koshar' => 'numeric',
            'vegan' => 'numeric',
            'crustaceans' => 'numeric',
            'molluscs' => 'numeric',
            'sulphur_dioxide' => 'numeric',
            'sulphites' => 'numeric',
            'product_percentage' => 'numeric',
            'image' => 'image',
            'menu_details' => 'array'
        ]);

        // create menu
        $menu = new TiffintomMenu;
        $menu->fill($req->all());
        $menu->save();
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_MENU . '/' . $menu->id, 'tiffintom');
            $menu->menu_image = $fileName;
        }
        if ($req->has('menu_image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('menu_image'), ImageHelper::$TYPE_TIFFINTOM_MENU . '/' . $menu->id, 'tiffintom');
            $menu->menu_image = $fileName;
        }
        if ($req->has('menu_name')) $menu->menu_name = (new ExtraController())->convertToTitleCase($req->input('menu_name'));
        $menu->save();

        if ($menu->price_option == "single") {
            $menu_details = ($req->input('menu_details'));
            $menuVariant = new TiffintomMenuVariant;
            $menuVariant->menu_id = $menu->id;
            $menuVariant->sub_name = (new ExtraController())->convertToTitleCase($req->input('menu_name'));
            $menuVariant->orginal_price = $req->input('orginal_price');
            $menuVariant->save();
            if ($menu->menu_addon == "Yes") {
                $menu_details = ($req->input('menu_details'));
                foreach ($menu_details as $key => $value) {
                    $main_addons = $value->main_addons;
                    // $main_addons = json_decode($main_addons);
                    foreach ($main_addons as $addonkey => $addonvalue) {
                        foreach ($addonvalue->sub_addonses as $sub_addon_key => $sub_addon_value) {
                            if (!empty($sub_addon_value->selected) && $sub_addon_value->selected == '1') {
                                $menuAddon = new TiffintomMenuAddons;
                                $menuAddon->menu_id = $menu->id;
                                $menuAddon->restaurant_id = $menu->restaurant_id;
                                $menuAddon->category_id = $menu->category_id;
                                $menuAddon->mainaddons_id = $addonvalue->id;
                                $menuAddon->subaddons_id = $sub_addon_value->id;
                                $menuAddon->price_option = $menu->price_option;
                                $menuAddon->menudetails_id = $menuVariant->id;
                                $menuAddon->subaddons_price = $sub_addon_value->subaddons_price;
                                $menuAddon->save();
                            }
                        }
                    }
                }
            }
        } else {
            $menu_details = $req->input('menu_details');
            foreach ($menu_details as $key => $value) {
                $menuVariant = new TiffintomMenuVariant;
                $menuVariant->menu_id = $menu->id;
                $menuVariant->sub_name = $value->sub_name;
                $menuVariant->orginal_price = $value->orginal_price;
                $menuVariant->save();
                if ($menu->menu_addon == "Yes") {
                    $main_addons = $value->main_addons;
                    // $main_addons = json_decode($main_addons);
                    foreach ($main_addons as $addonkey => $addonvalue) {
                        foreach ($addonvalue->sub_addonses as $sub_addon_key => $sub_addon_value) {
                            if (!empty($sub_addon_value->selected) && $sub_addon_value->selected == '1') {
                                $menuAddon = new TiffintomMenuAddons;
                                $menuAddon->menu_id = $menu->id;
                                $menuAddon->restaurant_id = $menu->restaurant_id;
                                $menuAddon->category_id = $menu->category_id;
                                $menuAddon->mainaddons_id = $addonvalue->id;
                                $menuAddon->subaddons_id = $sub_addon_value->id;
                                $menuAddon->price_option = $menu->price_option;
                                $menuAddon->menudetails_id = $menuVariant->id;
                                $menuAddon->subaddons_price = $sub_addon_value->subaddons_price;
                                $menuAddon->save();
                            }
                        }
                    }
                }
            }
        }

        // Sequence add
        $menu->sortorder = $menu->id;
        $menu->save();

        return $menu;
    }

    /**
     * List category
     * GET /tiffintom/categories/id
     */
    public function show(Request $req, $id)
    {
        $menu = TiffintomMenu::where('id', $id);
        if (empty($menu)) Utils::error("No product found", 404);
        $menu->with(['variants']);
        $menu = $menu->first();
        $menu->variants->each(function ($variant) {
            $variant->append('main_addons');
            $variant->append('menu_addons');
            $variant->append('all_addons');
        });
        // $menu = TiffintomMenu::with('variants.menuAddons.mainAddons.sub_addonses')->where('id', $id)->first();
        // if (empty($menu)) Utils::error("No product found", 404);
        return $menu;
    }

    /**
     * Update menu.
     * POST  /tiffintom/menus/id
     */
    public function update(Request $req, $id)
    {
        if ($req->has('menu_details') && !is_array($req->input('menu_details'))) {
            $req->merge(['menu_details' => json_decode($req->input('menu_details'))]);
        }

        $menu = TiffintomMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_id' => 'required|exists:tiffintom_connection.categories,id',
            'menu_name' => 'required|string',
            'is_suggest' => 'string',
            'price_option' => 'in:single,multiple',
            'menu_description' => 'string',
            'menu_type' => 'in:veg,nonveg,other',
            'menu_addon' => 'in:Yes,No',
            'popular_dish' => 'in:Yes,No',
            'featured' => 'in:Yes,No',
            'spicy_dish' => 'in:mild,medium,extra_hot,slightly_hot,hot,no_option',
            'favourite' => 'in:Y,N',
            'status' => 'boolean',
            'delete_status' => 'in:Y,N',
            'sortorder' => 'numeric',
            'short_name' => 'string',
            'vegetarian' => 'numeric',
            'contains_nuts' => 'numeric',
            'milk' => 'numeric',
            'mustard' => 'numeric',
            'eggs' => 'numeric',
            'fish' => 'numeric',
            'gluten_free' => 'numeric',
            'halal' => 'numeric',
            'koshar' => 'numeric',
            'vegan' => 'numeric',
            'crustaceans' => 'numeric',
            'molluscs' => 'numeric',
            'sulphur_dioxide' => 'numeric',
            'sulphites' => 'numeric',
            'product_percentage' => 'numeric',
            'image' => 'image',
            'menu_details' => 'array'
        ]);

        // update menu
        if ($req->has('restaurant_id')) $menu->restaurant_id = $req->input('restaurant_id');
        if ($req->has('category_id')) $menu->category_id = $req->input('category_id');
        if ($req->has('menu_name')) $menu->menu_name = (new ExtraController())->convertToTitleCase($req->input('menu_name'));
        if ($req->has('is_suggest')) $menu->is_suggest = $req->input('is_suggest');
        if ($req->has('price_option')) $menu->price_option = $req->input('price_option');
        if ($req->has('menu_addons')) $menu->menu_addons = $req->input('menu_addons');
        if ($req->has('menu_description')) $menu->menu_description = $req->input('menu_description');
        if ($req->has('menu_addon')) $menu->menu_addon = $req->input('menu_addon');
        if ($req->has('menu_type')) {
            if ($req->input('menu_type') == '') {
                $menu->menu_type = 'veg';
            } else {
                $menu->menu_type = $req->input('menu_type');
            }
        }
        if ($req->has('popular_dish')) {
            if ($req->input('popular_dish') == '') {
                $menu->popular_dish = 'No';
            } else {
                $menu->popular_dish = $req->input('popular_dish');
            }
        }
        if ($req->has('featured')) {
            if ($req->input('featured') == '') {
                $menu->featured = 'No';
            } else {
                $menu->featured = $req->input('featured');
            }
        }
        if ($req->has('spicy_dish')) {
            if ($req->input('spicy_dish') == '') {
                $menu->spicy_dish = 'mild';
            } else {
                $menu->spicy_dish = $req->input('spicy_dish');
            }
        }
        if ($req->has('favourite')) {
            if ($req->input('favourite') == '') {
                $menu->favourite = 'N';
            } else {
                $menu->favourite = $req->input('favourite');
            }
        }
        if ($req->has('delete_status')) {
            if ($req->input('delete_status') == '') {
                $menu->delete_status = 'N';
            } else {
                $menu->delete_status = $req->input('delete_status');
            }
        }
        if ($req->has('sortorder')) $menu->sortorder = $req->input('sortorder');
        if ($req->has('short_name')) $menu->short_name = $req->input('short_name');
        if ($req->has('product_percentage')) $menu->product_percentage = $req->input('product_percentage');
        $menu->vegetarian = $req->input('vegetarian');
        $menu->contains_nuts = $req->input('contains_nuts');
        $menu->milk = $req->input('milk');
        $menu->mustard = $req->input('mustard');
        $menu->eggs = $req->input('eggs');
        $menu->fish = $req->input('fish');
        $menu->gluten_free = $req->input('gluten_free');
        $menu->halal = $req->input('halal');
        $menu->koshar = $req->input('koshar');
        $menu->vegan = $req->input('vegan');
        $menu->crustaceans = $req->input('crustaceans');
        $menu->molluscs = $req->input('molluscs');
        $menu->sulphur_dioxide = $req->input('sulphur_dioxide');
        $menu->sulphites = $req->input('sulphites');
        $menu->status = $req->input('status');
        $menu->save();
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_MENU . '/' . $menu->id, 'tiffintom');
            $menu->menu_image = $fileName;
        }
        if ($req->has('menu_image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('menu_image'), ImageHelper::$TYPE_TIFFINTOM_MENU . '/' . $menu->id, 'tiffintom');
            $menu->menu_image = $fileName;
        }
        $menu->save();

        $variantDeleteAll = TiffintomMenuVariant::where('menu_id', $menu->id)->delete();
        if ($menu->price_option == "single") {
            $menu_details = ($req->input('menu_details'));
            $menuVariant = new TiffintomMenuVariant;
            $menuVariant->menu_id = $menu->id;
            $menuVariant->sub_name = (new ExtraController())->convertToTitleCase($req->input('menu_name'));
            $menuVariant->orginal_price = $req->input('orginal_price');
            $menuVariant->save();
            if ($menu->menu_addon == "Yes") {
                $menu_details = ($req->input('menu_details'));
                $menuAddonDeleteAll = TiffintomMenuAddons::where('menu_id', $menu->id)->delete();
                foreach ($menu_details as $key => $value) {
                    $main_addons = $value->main_addons;
                    // $main_addons = json_decode($main_addons);
                    foreach ($main_addons as $addonkey => $addonvalue) {
                        foreach ($addonvalue->sub_addonses as $sub_addon_key => $sub_addon_value) {
                            if (!empty($sub_addon_value->selected) && $sub_addon_value->selected == '1') {
                                $menuAddon = new TiffintomMenuAddons;
                                $menuAddon->menu_id = $menu->id;
                                $menuAddon->restaurant_id = $menu->restaurant_id;
                                $menuAddon->category_id = $menu->category_id;
                                $menuAddon->mainaddons_id = $addonvalue->id;
                                $menuAddon->subaddons_id = $sub_addon_value->id;
                                $menuAddon->price_option = $menu->price_option;
                                $menuAddon->menudetails_id = $menuVariant->id;
                                $menuAddon->subaddons_price = $sub_addon_value->subaddons_price;
                                $menuAddon->save();
                            }
                        }
                    }
                }
            }
        } else {
            $menu_details = $req->input('menu_details');
            if ($menu->menu_addon == "Yes") {
                $menuAddonDeleteAll = TiffintomMenuAddons::where('menu_id', $menu->id)->delete();
            }
            foreach ($menu_details as $key => $value) {
                $menuVariant = new TiffintomMenuVariant;
                $menuVariant->menu_id = $menu->id;
                $menuVariant->sub_name = $value->sub_name;
                $menuVariant->orginal_price = $value->orginal_price;
                $menuVariant->save();
                if ($menu->menu_addon == "Yes") {
                    $main_addons = $value->main_addons;
                    // $main_addons = json_decode($main_addons);
                    foreach ($main_addons as $addonkey => $addonvalue) {
                        foreach ($addonvalue->sub_addonses as $sub_addon_key => $sub_addon_value) {
                            if (!empty($sub_addon_value->selected) && $sub_addon_value->selected == 1) {
                                $menuAddon = new TiffintomMenuAddons;
                                $menuAddon->menu_id = $menu->id;
                                $menuAddon->restaurant_id = $menu->restaurant_id;
                                $menuAddon->category_id = $menu->category_id;
                                $menuAddon->mainaddons_id = $addonvalue->id;
                                $menuAddon->subaddons_id = $sub_addon_value->id;
                                $menuAddon->price_option = $menu->price_option;
                                $menuAddon->menudetails_id = $menuVariant->id;
                                $menuAddon->subaddons_price = $sub_addon_value->subaddons_price;
                                $menuAddon->save();
                            }
                        }
                    }
                }
            }
        }

        return $menu;
    }

    /**
     * Delete menu by id.
     * DELETE /tiffintom/menus/{id}
     */
    public function delete(Request $req, $id)
    {
        $menu = TiffintomMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);
        // $menu->delete();
        $menu->delete_status = 'Y';
        $menu->save();
        return $menu;
    }

    /**
     * Delete variant by id.
     * DELETE /tiffintom/menus/{id}/variant
     */
    public function delete_variant(Request $req, $id)
    {
        $variant = TiffintomMenuVariant::find($id);
        if (empty($variant)) Utils::error('variant not found!', 404);
        $variant->delete();
        $menuAddons = TiffintomMenuAddons::where('menudetails_id', $id)->delete();
        return $variant;
    }

    /**
     * Enable menu.
     * POST /tiffintom/menus/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $menu = TiffintomMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);
        $menu->status = '1';
        $menu->save();
        return $menu;
    }

    /**
     * Disable menu.
     * POST /tiffintom/menus/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $menu = TiffintomMenu::find($id);
        if (empty($menu)) Utils::error('menu not found!', 404);
        $menu->status = '0';
        $menu->save();
        return $menu;
    }

    /**
     * Bulk Upload menu.
     * POST /tiffintom/menus/bulk
     */
    public function bulk_menu(Request $req)
    {
        if ($req->has('menus') && !is_array($req->input('menus')))
            $req->merge(['menus' => json_decode($req->input('menus'))]);

        $this->validate($req, [
            'menus' => 'required|array'
        ]);

        $restaurantMenus = ($req->input('menus'));
        $count = 1;
        foreach ($restaurantMenus as $menu) {
            $menuObj = TiffintomMenu::find($menu->id);
            $menuObj->sortorder = $count;
            $menuObj->save();
            $count++;
        }
        return $restaurantMenus;
    }
}
