<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomMessage;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;

class TiffintomMessageController extends Controller
{
    /**
     * List sms-messages
     * GET /tiffintom/sms-messages
     */
    public function index(Request $req)
    {
        $sms_messages = TiffintomMessage::selectRaw('messages.*,urls.package_name as package_name')->leftJoin('urls', 'urls.id', '=', 'sms_plan')->orderBy('id', 'DESC');

        if ($query = $req->query("number", false)) {
            $sms_messages->where('number', $query);
        }

        if ($type = $req->query("type", false)) {
            $sms_messages->where('type', $type);
        }

        if ($req->query('nopaginate', false) == '1') {
            $sms_messages = $sms_messages->get();
        } else {
            $sms_messages = $sms_messages->paginate($req->query('per_page', 20));
        }

        if ($sms_messages->isEmpty()) Utils::error('sms messages not found!', 404);

        return $sms_messages;
    }


    /**
     * Create sms-message.
     * POST /tiffintom/sms-messages
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'number' =>  ['required', Rule::unique('tiffintom_connection.messages')->where(function ($query) use ($req) {
                return $query->where('number', $req->input('number'))->where('delete_status', 'N');
                // ->where('number_count', $req->input('number_count'))
            })],
            'sms_plan' => 'string',
            'number_count' => 'required|string',
            'type' => 'required|in:send,verify',
            'minimum_digit' => 'numeric|gt:0',
            'maximum_digit' => 'numeric|gt:0',
            'device_type' => 'in:mobile,landline,other',
            'mobile' => 'boolean',
        ]);

        $sms_message = new TiffintomMessage;
        $sms_message->fill($req->all());
        $sms_message->status = 1;
        $sms_message->save();

        return $sms_message;
    }

    /**
     * List sms-messages
     * GET /tiffintom/sms-messages/id
     */
    public function show(Request $req, $id)
    {
        $sms_message = TiffintomMessage::find($id);
        if (empty($sms_message)) Utils::error("sms message not found", 404);
        return $sms_message;
    }

    /**
     * Update sms-messages.
     * POST  /tiffintom/sms-messages/id
     */
    public function update(Request $req, $id)
    {
        $sms_message = TiffintomMessage::find($id);
        if (empty($sms_message)) Utils::error('sms message not found!', 404);

        $this->validate($req, [
            'number' =>  ['required', Rule::unique('tiffintom_connection.messages')->ignore($sms_message->id)->where(function ($query) use ($req) {
                return $query->where('number', $req->input('number'))->where('delete_status', 'N');
                // ->where('number_count', $req->input('number_count'))
            })],
            'sms_plan' => 'string',
            'number_count' => 'required|string',
            'type' => 'required|in:send,verify',
            'minimum_digit' => 'numeric|gt:0',
            'maximum_digit' => 'numeric|gt:0',
            'device_type' => 'in:mobile,landline,other',
            'mobile' => 'boolean',
        ]);

        // update sms message
        if ($req->has('number')) $sms_message->number = $req->input('number');
        if ($req->has('sms_plan')) $sms_message->sms_plan = $req->input('sms_plan');
        if ($req->has('number_count')) $sms_message->number_count = $req->input('number_count');
        if ($req->has('type')) $sms_message->type = $req->input('type');
        if ($req->has('minimum_digit')) $sms_message->minimum_digit = $req->input('minimum_digit');
        if ($req->has('maximum_digit')) $sms_message->maximum_digit = $req->input('maximum_digit');
        if ($req->has('device_type')) $sms_message->device_type = $req->input('device_type');
        $sms_message->mobile = $req->input('mobile');

        $sms_message->save();

        return $sms_message;
    }

    /**
     * Delete sms-messages by id.
     * DELETE /tiffintom/sms-messages/{id}
     */
    public function delete(Request $req, $id)
    {
        $sms_message = TiffintomMessage::find($id);
        if (empty($sms_message)) Utils::error('sms message not found!', 404);
        $sms_message->delete();
        return $sms_message;
    }

    /**
     * Enable sms-messages.
     * POST /tiffintom/sms-messages/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $sms_message = TiffintomMessage::find($id);
        if (empty($sms_message)) Utils::error('sms message not found!', 404);
        $sms_message->status = '0';
        $sms_message->save();
        return $sms_message;
    }

    /**
     * Disable sms-messages.
     * POST /tiffintom/sms-messages/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $sms_message = TiffintomMessage::find($id);
        if (empty($sms_message)) Utils::error('sms message not found!', 404);
        $sms_message->status = '1';
        $sms_message->save();
        return $sms_message;
    }

    public function validate_number($contact_number)
    {
        $messageSend = -1;
        $id = 0;
        $checkMessages =  TiffintomMessage::where('status', '1')->where('delete_status', 'N')->orderByRaw('id ASC')->get();
        if (!$checkMessages->isEmpty()) {
            foreach ($checkMessages as $CheckMessage) {
                $sub = substr($contact_number, 0, $CheckMessage['number_count']);
                if ($CheckMessage['number'] == $sub) {
                    $phone_length =  Str::length($contact_number);
                    if ($CheckMessage['minimum_digit'] <= $phone_length && $CheckMessage['maximum_digit'] >= $phone_length) {
                        if ($CheckMessage['mobile'] == '1') {
                            $messageSend = 1;
                            $id = $CheckMessage['id'];
                        } else {
                            $messageSend = 0;
                            $id = $CheckMessage['id'];
                        }
                    } else {
                        return Utils::error('Please enter ' . $CheckMessage['minimum_digit'] . ' to ' . $CheckMessage['maximum_digit'] . ' digit ' . $CheckMessage['device_type'] . ' number', 400);
                    }
                }
            }
        }
        if ($messageSend == -1) {
            return Utils::error('Please enter valid mobile number', 400);
        }
        $data = [
            'id' => $id,
            'send_verify' => $messageSend,
        ];
        return $data;
    }

    public function message_number($contact_number)
    {
        $messageSend = false;
        $checkMessages =  TiffintomMessage::where('status', '1')->where('delete_status', 'N')->orderByRaw('id ASC')->get();
        if (!$checkMessages->isEmpty()) {
            foreach ($checkMessages as $CheckMessage) {
                $sub = substr($contact_number, 0, $CheckMessage['number_count']);
                if ($CheckMessage['number'] == $sub) {
                    if ($CheckMessage['device_type'] == 'mobile') {
                        $messageSend = true;
                    }
                }
            }
        }

        return $messageSend;
    }
}
