<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomUser;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomNewsLetterController extends Controller
{

    /**
     * List news-letters
     * GET /tiffintom/news-letters
     */
    public function index(Request $req)
    {
        $users = TiffintomUser::orderBy('id', 'DESC');
        $users->where('newsletter', 'Y')->where('status', '1');

        if ($query = $req->query('query', false)) {
            $users->whereRaw('(first_name LIKE ? OR last_name LIKE ? OR phone_number LIKE ? OR username LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($role_id = $req->query("role_id", false)) {
            $users->where('role_id', $role_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $users = $users->get();
        } else {
            $users = $users->paginate($req->query('per_page', 20));
        }

        if ($users->isEmpty()) Utils::error('Users not found!', 404);

        return $users;
    }

    /**
     * Create send-mail-all.
     * POST /tiffintom/news-letters/send-mail-all
     */
    public function send_all(Request $req)
    {
        $this->validate($req, [
            'subject'  => 'required|string',
            'content'  => 'required|string',
        ]);

        $users = TiffintomUser::where('newsletter', 'Y')->where('status', '1')->orderBy('id', 'DESC')->get();

        foreach ($users as $user) {
            $user->subject = $req->input('subject');
            $user->content = $req->input('content');
            $this->sendMail($user);
        }

        return [
            'message' => 'Mail send successfully!'
        ];
    }

    /**
     * Create send-mail-customer.
     * POST /tiffintom/news-letters/send-customer
     */
    public function send_customer(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);

        $this->validate($req, [
            'subject'  => 'required|string',
            'content'  => 'required|string',
            'ids' => 'required|array'
        ]);

        $users = TiffintomUser::whereIn('id', $req->input('ids'))->orderBy('id', 'DESC')->get();

        foreach ($users as $user) {
            $user->subject = $req->input('subject');
            $user->content = $req->input('content');
            $this->sendMail($user);
        }

        return [
            'message' => 'Mail send successfully!'
        ];
    }

    public function sendMail($user)
    {
        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#f1e0c6">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/logo-home.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>
                    <tr>
                        <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                            <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                                <tr>
                                    <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                    <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                        ' . $user->content . '
                                    </h2>
                                    </td>
                                </tr>
                            </table>
                        </td>
                    </tr>
                    <tr>   
                        <td bgcolor="#fff" >
                            <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                                <tr>
                                    <td style="padding: 30px 30px; text-align: right;">
                                            <table align="left" style="text-align: left;">
                                                <tr>
                                                    <td>
                                                            <p style="color:#000;"> Get The App : </p> 
                                                    </td>
                                                        <td width="10">&nbsp;</td>
                                                    <td>
                                                        <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                        <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                    </td>
                                                        <td width="10">&nbsp;</td>
                                                    <td>
                                                        <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                        <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                    </td>
                                                        <td width="10">&nbsp;</td>
                                                </tr>
                                                
                                            </table>
                                            <table align="right" style="text-align: right;">
                                            <tr>
                                                <td>
                                                        <p style="color:#000;">Follow Us : </p>
                                                </td>
                                                    <td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td>
                                                    <td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td>
                                                    <td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td><td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td><td width="10">&nbsp;</td>
                                            </tr>
                                        </table>
                                    </td>
                                </tr>
                            </table>
                        </td>
                    </tr>
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#f1e0c6"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
                    </tr>
                </table>
            </td>
        </tr>
        </table>';

        // User side mail
        $email['body'] = $emailinfo;
        $email['subject'] = $user->subject;
        $email['mail_type'] = $siteSettings->mail_option;
        $email['to'] = $user->username;
        $email['from'] = $siteSettings->admin_email;
        (new MailHelper())->sendMail($email);

        return 'Mail Send Successfully';
    }
}
