<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomNote;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class TiffintomNoteController extends Controller
{
    /**
     * List notes
     * GET /tiffintom/notes
     */
    public function index(Request $req)
    {
        $notes = TiffintomNote::selectRaw("notes.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $notes->whereRaw('(restaurants.restaurant_name LIKE ?)', ["%$query%"]);
        }

        if ($admin_group = $req->query('admin_group', false)) {
            $notes->join(DB::raw('(SELECT restaurant_id, MAX(created) as latest FROM orderhelp GROUP BY customer_id,restaurant_id) as r'), 'orderhelp.created', '=', 'r.latest');
        }

        if ($restaurant_id = $req->query('restaurant_id', false)) {
            $restaurant_id = $req->input('restaurant_id');
            $notes->where('restaurant_id', $restaurant_id);
        }

        if ($order_id = $req->query('order_id', false)) {
            $notes->where('order_id', $order_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $notes = $notes->get();
        } else {
            $notes = $notes->paginate($req->query('per_page', 20));
        }

        if ($notes->isEmpty()) Utils::error('No notess found!', 404);

        return $notes;
    }

    /**
     * Create notes.
     * POST /tiffintom/notes
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'note' => 'string',
            'status' => 'boolean',
        ]);

        // create notes
        $note = new TiffintomNote;
        $note->fill($req->all());
        $note->creates = Auth::id();
        $note->status = 1;
        $note->save();

        return $note;
    }

    /**
     * List notes
     * GET /tiffintom/notes/id
     */
    public function show(Request $req, $id)
    {
        $note = TiffintomNote::find($id);
        if (empty($note)) Utils::error("No note found", 404);
        return $note;
    }

    /**
     * Update notes.
     * POST  /tiffintom/notes/id
     */
    public function update(Request $req, $id)
    {
        $note = TiffintomNote::find($id);
        if (empty($note)) Utils::error('note not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'note' => 'string',
            'status' => 'boolean',
        ]);

        // update menu
        if ($req->has('restaurant_id')) $note->restaurant_id = $req->input('restaurant_id');
        $note->creates = Auth::id();
        if ($req->has('note')) $note->note = $req->input('note');
        $note->status = $req->input('status');
        $note->save();

        return $note;
    }

    /**
     * Delete notes by id.
     * DELETE /tiffintom/notes/{id}
     */
    public function delete(Request $req, $id)
    {
        $note = TiffintomNote::find($id);
        if (empty($note)) Utils::error('note not found!', 404);
        $note->delete();

        return $note;
    }

    /**
     * Enable note.
     * POST /tiffintom/notes/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $note = TiffintomNote::find($id);
        if (empty($note)) Utils::error('Note not found!', 404);
        $note->status = '1';
        $note->save();
        return $note;
    }

    /**
     * Disable note.
     * POST /tiffintom/notes/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $note = TiffintomNote::find($id);
        if (empty($note)) Utils::error('Note not found!', 404);
        $note->status = '0';
        $note->save();
        return $note;
    }
}
