<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomNotifications;
use App\ModelsTiffintom\TiffintomUser;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomNotificationsController extends Controller
{

    /**
     * List notification
     * GET /tiffintom/notifications
     */
    public function index(Request $req)
    {
        $notifications = TiffintomNotifications::selectRaw("notifications.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $notifications->whereRaw('(type LIKE ? OR title LIKE ?  OR content LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($status = $req->query('status', false)) {
            $notifications->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $notifications = $notifications->get();
        } else {
            $notifications = $notifications->paginate($req->query('per_page', 20));
        }

        if ($notifications->isEmpty()) Utils::error('No notifications found!', 404);

        return $notifications;
    }


    /**
     * Create notification.
     * POST /tiffintom/notifications
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'type' => 'required|in:all,ios,android',
            'title' => 'required|string',
            'content' => 'required|string',
        ]);

        // create notification
        $notification = new TiffintomNotifications;
        $notification->fill($req->all());
        $notification->status = '1';
        $notification->save();

        try {
            $users = TiffintomUser::whereNotNull('device_id')->where('role_id', '3')->where('status', '1')->where('deleted_status', 'N')->get();

            foreach ($users as $user) {
                if ($user->important_update == 7 || $user->important_update == 5 || $user->important_update == 3 || $user->important_update == 1) {
                    $notification['title'] = $notification->title;
                    $notification['body'] = $notification->content;

                    $notificationdata['data']['title'] = $notification->title;
                    $notificationdata['data']['message'] = $notification->content;
                    $notificationdata['data']['is_background']  = false;
                    $notificationdata['data']['restaurant_id'] = $notification->restaurant_id;
                    $notificationdata['data']['order_id'] = 0;
                    $system_notification['user_id']  = $user->id;
                    $notificationdata['data']['payload'] = array('OrderDetails' => "", 'type'    => "order");
                    $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
                    (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
                }
            }
        } catch (Exception $e) {
            return $notification;
        }

        return $notification;
    }

    /**
     * List notifications
     * GET /tiffintom/notifications/id
     */
    public function show(Request $req, $id)
    {
        $notification = TiffintomNotifications::find($id);
        if (empty($notification)) Utils::error("Notification not found", 404);
        return $notification;
    }

    /**
     * Update notification.
     * POST  /tiffintom/notifications/id
     */
    public function update(Request $req, $id)
    {
        $notification = TiffintomNotifications::find($id);
        if (empty($notification)) Utils::error('Notification not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'type' => 'required|in:all,ios,android',
            'title' => 'required|string',
            'content' => 'required|string',
        ]);

        // update notification
        if ($req->has('restaurant_id')) $notification->restaurant_id = $req->input('restaurant_id');
        if ($req->has('type')) $notification->type = $req->input('type');
        if ($req->has('title')) $notification->title = $req->input('title');
        if ($req->has('content')) $notification->content = $req->input('content');
        $notification->save();

        try {
            $users = TiffintomUser::whereNotNull('device_id')->where('role_id', '3')->where('status', '1')->where('deleted_status', 'N')->get();

            foreach ($users as $user) {
                if ($user->important_update == 7 || $user->important_update == 5 || $user->important_update == 3 || $user->important_update == 1) {
                    $notification['title'] = $notification->title;
                    $notification['body'] = $notification->content;

                    $notificationdata['data']['title'] = $notification->title;
                    $notificationdata['data']['message'] = $notification->content;
                    $notificationdata['data']['is_background']  = false;
                    $notificationdata['data']['restaurant_id'] = $notification->restaurant_id;
                    $notificationdata['data']['order_id'] = 0;
                    $system_notification['user_id']  = $user->id;
                    $notificationdata['data']['payload'] = array('OrderDetails' => "", 'type'    => "order");
                    $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
                    (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
                }
            }
        } catch (Exception $e) {
            return $notification;
        }

        return $notification;
    }

    /**
     * Delete notification by id.
     * DELETE /tiffintom/notifications/{id}
     */
    public function delete(Request $req, $id)
    {
        $notification = TiffintomNotifications::find($id);
        if (empty($notification)) Utils::error('Notification not found!', 404);
        $notification->delete();

        return $notification;
    }

    /**
     * Enable notification.
     * POST /tiffintom/notifications/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $notification = TiffintomNotifications::find($id);
        if (empty($notification)) Utils::error('Notification not found!', 404);
        $notification->status = '1';
        $notification->save();

        return $notification;
    }

    /**
     * Disable notification.
     * POST /tiffintom/notifications/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $notification = TiffintomNotifications::find($id);
        if (empty($notification)) Utils::error('Notification not found!', 404);
        $notification->status = '0';
        $notification->save();

        return $notification;
    }
}
