<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomOfferDay;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomOfferDayController extends Controller
{
    /**
     * List offer-day
     * GET /tiffintom/offer-day
     */
    public function index(Request $req)
    {
        $offer_days = TiffintomOfferDay::selectRaw("day_offer.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'resid')->orderBy('id', 'ASC');

        if ($query = $req->query("restaurant_id", false)) {
            $offer_days->where('resid', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $offer_days = $offer_days->get();
        } else {
            $offer_days = $offer_days->paginate($req->query('per_page', 20));
        }

        if ($offer_days->isEmpty()) Utils::error('TiffintomOfferDay not found!', 404);

        return $offer_days;
    }

    /**
     * Create offer-product.
     * POST /tiffintom/offer-day
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'purchase_price' => 'required|numeric|gt:0',
            'day_offer_type' => 'required|string',
            'offer_amount' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // create offer_day
        $offer_day = new TiffintomOfferDay();
        $offer_day->fill($req->all());
        $offer_day->save();
        return $offer_day;
    }

    /**
     * List offer-day
     * GET /tiffintom/offer-day/id
     */
    public function show(Request $req, $id)
    {
        $offer_days = TiffintomOfferDay::find($id);
        if (empty($offer_days)) Utils::error("TiffintomOfferDay not found", 404);
        return $offer_days;
    }

    /**
     * Update offer-product.
     * POST  /tiffintom/offer-day/id
     */
    public function update(Request $req, $id)
    {
        $offer_day = TiffintomOfferDay::find($id);
        if (empty($offer_day)) Utils::error('TiffintomOfferDay not found!', 404);

        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'purchase_price' => 'required|numeric|gt:0',
            'day_offer_type' => 'required|string',
            'offer_amount' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // update offer_day
        if ($req->has('resid')) $offer_day->resid = $req->input('resid');
        if ($req->has('day')) $offer_day->day = $req->input('day');
        if ($req->has('purchase_price')) $offer_day->purchase_price = $req->input('purchase_price');
        $offer_day->day_offer_type = $req->input('day_offer_type');
        if ($req->has('offer_amount')) $offer_day->offer_amount = $req->input('offer_amount');
        $offer_day->status = $req->input('status');
        $offer_day->delivery_type = $req->input('delivery_type');
        $offer_day->pickup_type = $req->input('pickup_type');
        $offer_day->dinein_type = $req->input('dinein_type');

        $offer_day->save();

        return $offer_day;
    }

    /**
     * Delete offer-product by id.
     * DELETE /tiffintom/offer-day/{id}
     */
    public function delete(Request $req, $id)
    {
        $offer_day = TiffintomOfferDay::find($id);
        if (empty($offer_day)) Utils::error('TiffintomOfferDay not found!', 404);
        $offer_day->delete();
        return $offer_day;
    }

    /**
     * Enable offer-product.
     * POST /tiffintom/offer-day/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $offer_day = TiffintomOfferDay::find($id);
        if (empty($offer_day)) Utils::error('TiffintomOfferDay not found!', 404);
        $offer_day->status = '1';
        $offer_day->save();
        return $offer_day;
    }

    /**
     * Disable offer-product.
     * POST /tiffintom/offer-day/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $offer_day = TiffintomOfferDay::find($id);
        if (empty($offer_day)) Utils::error('TiffintomOfferDay not found!', 404);
        $offer_day->status = '0';
        $offer_day->save();
        return $offer_day;
    }
}
