<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomOfferDayPrice;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomOfferDayPriceController extends Controller
{
    /**
     * List offer-day-price
     * GET /tiffintom/offer-day-price
     */
    public function index(Request $req)
    {
        $offer_day_prices = TiffintomOfferDayPrice::selectRaw("day_price_offer.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'resid')->orderBy('id', 'ASC');

        if ($query = $req->query("restaurant_id", false)) {
            $offer_day_prices->where('resid', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $offer_day_prices = $offer_day_prices->get();
        } else {
            $offer_day_prices = $offer_day_prices->paginate($req->query('per_page', 20));
        }

        if ($offer_day_prices->isEmpty()) Utils::error('Day offer prices not found!', 404);

        return $offer_day_prices;
    }

    /**
     * Create offer-product.
     * POST /tiffintom/offer-day-price
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'purchase_price' => 'required|numeric|gt:0',
            'offer_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'offer_qty' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // create offer_day_price
        $offer_day_price = new TiffintomOfferDayPrice();
        $offer_day_price->fill($req->all());
        $offer_day_price->save();
        return $offer_day_price;
    }

    /**
     * List offer-day-price
     * GET /tiffintom/offer-day-price/id
     */
    public function show(Request $req, $id)
    {
        $offer_day_prices = TiffintomOfferDayPrice::find($id);
        if (empty($offer_day_prices)) Utils::error("Day offer price not found", 404);
        return $offer_day_prices;
    }

    /**
     * Update offer-product.
     * POST  /tiffintom/offer-day-price/id
     */
    public function update(Request $req, $id)
    {
        $offer_day_price = TiffintomOfferDayPrice::find($id);
        if (empty($offer_day_price)) Utils::error('Day offer price not found!', 404);

        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'purchase_price' => 'required|numeric|gt:0',
            'offer_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'offer_qty' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // update offer_day_price
        if ($req->has('resid')) $offer_day_price->resid = $req->input('resid');
        if ($req->has('day')) $offer_day_price->day = $req->input('day');
        if ($req->has('purchase_price')) $offer_day_price->purchase_price = $req->input('purchase_price');
        if ($req->has('offer_id')) $offer_day_price->offer_id = $req->input('offer_id');
        if ($req->has('offer_qty')) $offer_day_price->offer_qty = $req->input('offer_qty');
        $offer_day_price->status = $req->input('status');
        $offer_day_price->delivery_type = $req->input('delivery_type');
        $offer_day_price->pickup_type = $req->input('pickup_type');
        $offer_day_price->dinein_type = $req->input('dinein_type');

        $offer_day_price->save();

        return $offer_day_price;
    }

    /**
     * Delete offer-product by id.
     * DELETE /tiffintom/offer-day-price/{id}
     */
    public function delete(Request $req, $id)
    {
        $offer_day_price = TiffintomOfferDayPrice::find($id);
        if (empty($offer_day_price)) Utils::error('Day offer price not found!', 404);
        $offer_day_price->delete();
        return $offer_day_price;
    }

    /**
     * Enable offer-product.
     * POST /tiffintom/offer-day-price/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $offer_day_price = TiffintomOfferDayPrice::find($id);
        if (empty($offer_day_price)) Utils::error('Day offer price not found!', 404);
        $offer_day_price->status = '1';
        $offer_day_price->save();
        return $offer_day_price;
    }

    /**
     * Disable offer-product.
     * POST /tiffintom/offer-day-price/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $offer_day_price = TiffintomOfferDayPrice::find($id);
        if (empty($offer_day_price)) Utils::error('Day offer price not found!', 404);
        $offer_day_price->status = '0';
        $offer_day_price->save();
        return $offer_day_price;
    }
}
