<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomOfferDayProduct;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomOfferDayProductController extends Controller
{
    /**
     * List offer-day-product
     * GET /tiffintom/offer-day-product
     */
    public function index(Request $req)
    {
        $offer_day_products = TiffintomOfferDayProduct::selectRaw("day_product_offer.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'resid')->orderBy('id', 'ASC');

        if ($query = $req->query("restaurant_id", false)) {
            $offer_day_products->where('resid', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $offer_day_products = $offer_day_products->get();
        } else {
            $offer_day_products = $offer_day_products->paginate($req->query('per_page', 20));
        }

        if ($offer_day_products->isEmpty()) Utils::error('Day product offers not found!', 404);

        return $offer_day_products;
    }

    /**
     * Create offer-product.
     * POST /tiffintom/offer-day-product
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'day_product' => 'required|string',
            'menu_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'offer_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'product_qty' => 'required|numeric|gt:0',
            'offer_qty' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // create offer_day_product
        $offer_day_product = new TiffintomOfferDayProduct();
        $offer_day_product->fill($req->all());
        $offer_day_product->save();
        return $offer_day_product;
    }

    /**
     * List offer-day-product
     * GET /tiffintom/offer-day-product/id
     */
    public function show(Request $req, $id)
    {
        $offer_day_products = TiffintomOfferDayProduct::find($id);
        if (empty($offer_day_products)) Utils::error("Day product offer not found", 404);
        return $offer_day_products;
    }

    /**
     * Update offer-product.
     * POST  /tiffintom/offer-day-product/id
     */
    public function update(Request $req, $id)
    {
        $offer_day_product = TiffintomOfferDayProduct::find($id);
        if (empty($offer_day_product)) Utils::error('Day product offer not found!', 404);

        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'day_product' => 'required|string',
            'menu_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'offer_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'product_qty' => 'required|numeric|gt:0',
            'offer_qty' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // update offer_day_product
        if ($req->has('resid')) $offer_day_product->resid = $req->input('resid');
        if ($req->has('day_product')) $offer_day_product->day_product = $req->input('day_product');
        if ($req->has('menu_id')) $offer_day_product->menu_id = $req->input('menu_id');
        if ($req->has('offer_id')) $offer_day_product->offer_id = $req->input('offer_id');
        if ($req->has('product_qty')) $offer_day_product->product_qty = $req->input('product_qty');
        if ($req->has('offer_qty')) $offer_day_product->offer_qty = $req->input('offer_qty');
        $offer_day_product->status = $req->input('status');
        $offer_day_product->delivery_type = $req->input('delivery_type');
        $offer_day_product->pickup_type = $req->input('pickup_type');
        $offer_day_product->dinein_type = $req->input('dinein_type');

        $offer_day_product->save();

        return $offer_day_product;
    }

    /**
     * Delete offer-product by id.
     * DELETE /tiffintom/offer-day-product/{id}
     */
    public function delete(Request $req, $id)
    {
        $offer_day_product = TiffintomOfferDayProduct::find($id);
        if (empty($offer_day_product)) Utils::error('Day product offer not found!', 404);
        $offer_day_product->delete();
        return $offer_day_product;
    }

    /**
     * Enable offer-product.
     * POST /tiffintom/offer-day-product/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $offer_day_product = TiffintomOfferDayProduct::find($id);
        if (empty($offer_day_product)) Utils::error('Day product offer not found!', 404);
        $offer_day_product->status = '1';
        $offer_day_product->save();
        return $offer_day_product;
    }

    /**
     * Disable offer-product.
     * POST /tiffintom/offer-day-product/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $offer_day_product = TiffintomOfferDayProduct::find($id);
        if (empty($offer_day_product)) Utils::error('Day product offer not found!', 404);
        $offer_day_product->status = '0';
        $offer_day_product->save();
        return $offer_day_product;
    }
}
