<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomOfferMultiplePrice;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomOfferMultiplePriceController extends Controller
{
    /**
     * List offer-multiple-price
     * GET /tiffintom/offer-multiple-price
     */
    public function index(Request $req)
    {
        $offer_multiple_prices = TiffintomOfferMultiplePrice::selectRaw("multiple_price_offer.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'resid')->orderBy('id', 'ASC');

        if ($query = $req->query("restaurant_id", false)) {
            $offer_multiple_prices->where('resid', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $offer_multiple_prices = $offer_multiple_prices->get();
        } else {
            $offer_multiple_prices = $offer_multiple_prices->paginate($req->query('per_page', 20));
        }

        if ($offer_multiple_prices->isEmpty()) Utils::error('TiffintomOfferMultiplePrice not found!', 404);

        return $offer_multiple_prices;
    }

    /**
     * Create offer-multiple-price.
     * POST /tiffintom/offer-multiple-price
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'offer_id' => 'required|string',
            'offer_qty' => 'required|numeric|gt:0',
            'purchase_price' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
            'offer_from' => 'required|date_format:Y-m-d',
            'offer_to' => 'required|date_format:Y-m-d|after:offer_from',
        ]);

        // create offer_multiple_price
        $offer_multiple_price = new TiffintomOfferMultiplePrice;
        $offer_multiple_price->fill($req->all());
        $offer_multiple_price->save();
        return $offer_multiple_price;
    }

    /**
     * List offer-multiple-price
     * GET /tiffintom/offer-multiple-price/id
     */
    public function show(Request $req, $id)
    {
        $offer_multiple_prices = TiffintomOfferMultiplePrice::find($id);
        if (empty($offer_multiple_prices)) Utils::error("TiffintomOfferMultiplePrice not found", 404);
        return $offer_multiple_prices;
    }

    /**
     * Update offer-multiple-price.
     * POST  /tiffintom/offer-multiple-price/id
     */
    public function update(Request $req, $id)
    {
        $offer_multiple_price = TiffintomOfferMultiplePrice::find($id);
        if (empty($offer_multiple_price)) Utils::error('TiffintomOfferMultiplePrice not found!', 404);

        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'offer_id' => 'required|string',
            'offer_qty' => 'required|numeric|gt:0',
            'purchase_price' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
            'offer_from' => 'required|date_format:Y-m-d',
            'offer_to' => 'required|date_format:Y-m-d|after:offer_from',
        ]);

        // update offer_multiple_price
        if ($req->has('resid')) $offer_multiple_price->resid = $req->input('resid');
        if ($req->has('purchase_price')) $offer_multiple_price->purchase_price = $req->input('purchase_price');
        if ($req->has('offer_id')) $offer_multiple_price->offer_id = $req->input('offer_id');
        if ($req->has('offer_qty')) $offer_multiple_price->offer_qty = $req->input('offer_qty');
        if ($req->has('offer_from')) $offer_multiple_price->offer_from = $req->input('offer_from');
        if ($req->has('offer_to')) $offer_multiple_price->offer_to = $req->input('offer_to');
        $offer_multiple_price->status = $req->input('status');
        $offer_multiple_price->delivery_type = $req->input('delivery_type');
        $offer_multiple_price->pickup_type = $req->input('pickup_type');
        $offer_multiple_price->dinein_type = $req->input('dinein_type');

        $offer_multiple_price->save();

        return $offer_multiple_price;
    }

    /**
     * Delete offer-multiple-price by id.
     * DELETE /tiffintom/offer-multiple-price/{id}
     */
    public function delete(Request $req, $id)
    {
        $offer_multiple_price = TiffintomOfferMultiplePrice::find($id);
        if (empty($offer_multiple_price)) Utils::error('TiffintomOfferMultiplePrice not found!', 404);
        $offer_multiple_price->delete();
        return $offer_multiple_price;
    }

    /**
     * Enable offer-multiple-price.
     * POST /tiffintom/offer-multiple-price/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $offer_multiple_price = TiffintomOfferMultiplePrice::find($id);
        if (empty($offer_multiple_price)) Utils::error('TiffintomOfferMultiplePrice not found!', 404);
        $offer_multiple_price->status = '1';
        $offer_multiple_price->save();
        return $offer_multiple_price;
    }

    /**
     * Disable offer-multiple-price.
     * POST /tiffintom/offer-multiple-price/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $offer_multiple_price = TiffintomOfferMultiplePrice::find($id);
        if (empty($offer_multiple_price)) Utils::error('TiffintomOfferMultiplePrice not found!', 404);
        $offer_multiple_price->status = '0';
        $offer_multiple_price->save();
        return $offer_multiple_price;
    }
}
