<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomOfferProduct;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomOfferProductController extends Controller
{
    /**
     * List offer-products
     * GET /tiffintom/offer-products
     */
    public function index(Request $req)
    {
        $offer_products = TiffintomOfferProduct::selectRaw("product_offer.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'resid')->orderBy('id', 'ASC');

        if ($query = $req->query("restaurant_id", false)) {
            $offer_products->where('resid', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $offer_products = $offer_products->get();
        } else {
            $offer_products = $offer_products->paginate($req->query('per_page', 20));
        }

        if ($offer_products->isEmpty()) Utils::error('Product Offers not found!', 404);

        return $offer_products;
    }

    /**
     * Create offer-product.
     * POST /tiffintom/offer-products
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'menu_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'offer_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'product_qty' => 'required|numeric|gt:0',
            'offer_qty' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
            'offer_from' => 'required|date_format:Y-m-d',
            'offer_to' => 'required|date_format:Y-m-d|after:offer_from',
        ]);

        // create offer_product
        $offer_product = new TiffintomOfferProduct;
        $offer_product->fill($req->all());
        $offer_product->save();
        return $offer_product;
    }

    /**
     * List offer-products
     * GET /tiffintom/offer-products/id
     */
    public function show(Request $req, $id)
    {
        $offer_products = TiffintomOfferProduct::find($id);
        if (empty($offer_products)) Utils::error("Product Offer not found", 404);
        return $offer_products;
    }

    /**
     * Update offer-product.
     * POST  /tiffintom/offer-products/id
     */
    public function update(Request $req, $id)
    {
        $offer_product = TiffintomOfferProduct::find($id);
        if (empty($offer_product)) Utils::error('Product Offer not found!', 404);

        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'menu_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'offer_id' => 'required|exists:tiffintom_connection.restaurant_menus,id',
            'product_qty' => 'required|numeric|gt:0',
            'offer_qty' => 'required|numeric|gt:0',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
            'offer_from' => 'required|date_format:Y-m-d',
            'offer_to' => 'required|date_format:Y-m-d|after:offer_from',
        ]);

        // update offer_product
        if ($req->has('resid')) $offer_product->resid = $req->input('resid');
        if ($req->has('menu_id')) $offer_product->menu_id = $req->input('menu_id');
        if ($req->has('offer_id')) $offer_product->offer_id = $req->input('offer_id');
        if ($req->has('product_qty')) $offer_product->product_qty = $req->input('product_qty');
        if ($req->has('offer_qty')) $offer_product->offer_qty = $req->input('offer_qty');
        if ($req->has('offer_from')) $offer_product->offer_from = $req->input('offer_from');
        if ($req->has('offer_to')) $offer_product->offer_to = $req->input('offer_to');
        $offer_product->status = $req->input('status');
        $offer_product->delivery_type = $req->input('delivery_type');
        $offer_product->pickup_type = $req->input('pickup_type');
        $offer_product->dinein_type = $req->input('dinein_type');

        $offer_product->save();

        return $offer_product;
    }

    /**
     * Delete offer-product by id.
     * DELETE /tiffintom/offer-products/{id}
     */
    public function delete(Request $req, $id)
    {
        $offer_product = TiffintomOfferProduct::find($id);
        if (empty($offer_product)) Utils::error('Product Offer not found!', 404);
        $offer_product->delete();
        return $offer_product;
    }

    /**
     * Enable offer-product.
     * POST /tiffintom/offer-products/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $offer_product = TiffintomOfferProduct::find($id);
        if (empty($offer_product)) Utils::error('Product Offer not found!', 404);
        $offer_product->status = '1';
        $offer_product->save();
        return $offer_product;
    }

    /**
     * Disable offer-product.
     * POST /tiffintom/offer-products/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $offer_product = TiffintomOfferProduct::find($id);
        if (empty($offer_product)) Utils::error('Product Offer not found!', 404);
        $offer_product->status = '0';
        $offer_product->save();
        return $offer_product;
    }
}
