<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomOfferText;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomOfferTextController extends Controller
{
    /**
     * List offer-texts
     * GET /tiffintom/offer-texts
     */
    public function index(Request $req)
    {
        $offer_texts_texts = TiffintomOfferText::selectRaw("offertexts.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $offer_texts_texts->whereRaw('(offer_name)', ["%$query%"]);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $offer_texts_texts->where('restaurant_id', $query);
        }
        if ($status = $req->query("status", false)) {
            $offer_texts_texts->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $offer_texts_texts = $offer_texts_texts->get();
        } else {
            $offer_texts_texts = $offer_texts_texts->paginate($req->query('per_page', 20));
        }

        if ($offer_texts_texts->isEmpty()) Utils::error('Offer texts not found!', 404);

        return $offer_texts_texts;
    }

    /**
     * Create offer.
     * POST /tiffintom/offer-texts
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'offer_name' => 'required|string',
            'status' => 'boolean',
        ]);

        // create offer
        $offer_texts = new TiffintomOfferText;
        $offer_texts->fill($req->all());
        $offer_texts->save();
        return $offer_texts;
    }

    /**
     * List offer-texts
     * GET /tiffintom/offer-texts/id
     */
    public function show(Request $req, $id)
    {
        $offer_texts_texts = TiffintomOfferText::find($id);
        if (empty($offer_texts_texts)) Utils::error("Offer text not found", 404);
        return $offer_texts_texts;
    }

    /**
     * Update offer.
     * POST  /tiffintom/offer-texts/id
     */
    public function update(Request $req, $id)
    {
        $offer_texts = TiffintomOfferText::find($id);
        if (empty($offer_texts)) Utils::error('Offer text not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'offer_name' => 'required|string',
            'status' => 'boolean',
        ]);

        // update offer
        if ($req->has('restaurant_id')) $offer_texts->restaurant_id = $req->input('restaurant_id');
        if ($req->has('offer_name')) $offer_texts->offer_name = $req->input('offer_name');
        $offer_texts->status = $req->input('status');
        $offer_texts->save();

        return $offer_texts;
    }

    /**
     * Delete offer by id.
     * DELETE /tiffintom/offer-texts/{id}
     */
    public function delete(Request $req, $id)
    {
        $offer_texts = TiffintomOfferText::find($id);
        if (empty($offer_texts)) Utils::error('Offer text not found!', 404);
        $offer_texts->delete();
        return $offer_texts;
    }

    /**
     * Enable offer.
     * POST /tiffintom/offer-texts/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $offer_texts = TiffintomOfferText::find($id);
        if (empty($offer_texts)) Utils::error('Offer text not found!', 404);
        $offer_texts->status = '1';
        $offer_texts->save();
        return $offer_texts;
    }

    /**
     * Disable offer.
     * POST /tiffintom/offer-texts/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $offer_texts = TiffintomOfferText::find($id);
        if (empty($offer_texts)) Utils::error('Offer text not found!', 404);
        $offer_texts->status = '0';
        $offer_texts->save();
        return $offer_texts;
    }
}
