<?php

namespace App\Http\Controllers\Tiffintom;

use App\ActionLog;
use App\Business;
use App\Helpers\Auth;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\Http\Controllers\SiteSettingController;
use App\ModelsTiffintom\TiffintomCart;
use App\ModelsTiffintom\TiffintomCustomerReward;
use App\ModelsTiffintom\TiffintomDineinOrder;
use App\ModelsTiffintom\TiffintomMenu;
use App\ModelsTiffintom\TiffintomOffer;
use App\ModelsTiffintom\TiffintomOfferDay;
use App\ModelsTiffintom\TiffintomOfferDayPrice;
use App\ModelsTiffintom\TiffintomOfferDayProduct;
use App\ModelsTiffintom\TiffintomOfferMultiplePrice;
use App\ModelsTiffintom\TiffintomOfferPrice;
use App\ModelsTiffintom\TiffintomOfferProduct;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomReview;
use App\ModelsTiffintom\TiffintomReward;
use App\ModelsTiffintom\TiffintomRTC;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomUser;
use App\ModelsTiffintom\TiffintomWalletHistory;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use PayPal\Api\Amount;
use PayPal\Api\Refund;
use PayPal\Api\Sale;
use PayPal\Rest\ApiContext;
use PayPal\Auth\OAuthTokenCredential;
use PayPal\Api\Payment;
use PayPal\Exception\PayPalConnectionException;

class TiffintomOrderController extends Controller
{
    /**
     * List order
     * GET /tiffintom/order
     */
    public function index(Request $req)
    {
        $orders = TiffintomOrder::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $orders->whereRaw('(order_number LIKE ? OR customer_name LIKE ?  OR customer_email LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($query = $req->query("restaurant_id", false)) {
            $orders->where('restaurant_id', $query);
        }
        if ($query = $req->query("customer_id", false)) {
            $orders->where('customer_id', $query);
        }
        // Useful for partner starts
        if ($query = $req->query('order_type')) {
            if ($query == 'dinein') {
                $orders->whereNull('order_type');
            } else if ($query == 'pickup') {
                $orders->where('order_type', 'pickup');
            } else if ($query == 'delivery') {
                $orders->where('order_type', 'delivery');
            }
        }
        if ($query = $req->query('status')) {
            $orders->where('status', $query);
        }
        if ($processed = $req->query('processed')) {
            $orders->whereRaw('(status LIKE ? OR status LIKE ? )', ["%Processing%", "%PaymentCancel%"]);
        } else {
            $orders->where('status', '!=', 'Processing');
            $orders->where('status', '!=', 'PaymentCancel');
        }
        if ($query = $req->query('upcoming_status')) {
            $orders->whereRaw('(status LIKE ? OR status LIKE ? )', ["%Accepted%", "%Pending%"]);
        }
        if ($query = $req->query('previous_status')) {
            $orders->whereRaw('(status LIKE ? OR status LIKE ? )', ["%Delivered%", "%Failed%"]);
        }
        if ($req->has('from_date')) {
            if ($query = $req->query('from_date')) {
                $fromDate = Carbon::parse($query)->startOfDay();
                $orders->where('created', '>', $fromDate);
            }
            if ($query = $req->query('to_date')) {
                $toDate = Carbon::parse($query)->endOfDay();
                $orders->where('created', '<', $toDate);
            }
        } else {
            $business = Business::where('restaurant_id', Auth::$business_id)->first();
            if (!empty($business) && $business->online_order_limit > 0) {
                $fromDate = Carbon::today()->startOfDay();
                $toDate = Carbon::today()->subDays($business->online_order_limit)->startOfDay();

                $orders->where('created', '>', $fromDate);
                $orders->where('created', '<', $toDate);
            }
        }

        //Useful for partner ends

        if ($req->query('nopaginate', false) == '1') {
            $orders = $orders->get();
            if (!$orders->isEmpty() && !$req->query('append')) {
                $orders->each(function ($value) {
                    $restaurants = TiffintomRestaurant::where('id', $value->restaurant_id)->first();
                    $value['restaurant'] = $restaurants;
                    $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurants['id'])->where('status', '1')->first();
                    if (!empty($reviews)) {
                        if ($reviews->count > 0) {
                            $value['restaurant']['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                            $value['restaurant']['total_reviews'] = $reviews->count;
                        }
                    }
                });
            }
        } else {
            $orders = $orders->paginate($req->query('per_page', 20));
            if (!$orders->isEmpty()) {
                $orders->each(function ($order) {
                    $restaurants = TiffintomRestaurant::where('id', $order->restaurant_id)->first();
                    $order->restaurant = $restaurants;

                    $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $order->restaurant_id)->where('status', '1')->first();
                    if (!empty($reviews)) {
                        if ($reviews->count > 0) {
                            $order->restaurant->average_rating = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                            $order->restaurant->total_reviews = $reviews->count;
                        }
                    }
                });
            }
        }

        if ($orders->isEmpty() && !$req->query('append')) Utils::error('order not found!', 404);

        if (!$orders->isEmpty() && !$req->query('append')) {
            $orders->each(function ($order) {
                $order->append('cart_view');
                $order->append('reviews');
                $order->append('card_view');
            });
        }

        // User wise check
        if ($req->hasHeader('business-id') && $req->input('restaurant_id') != '') {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        return $orders;
    }

    /**
     * Only for partner
     * List order-home
     * GET /tiffintom/orders/home
     */
    public function home(Request $req)
    {
        $startDate = Carbon::now()->startOfDay();
        $endDate = Carbon::now()->endOfDay();

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $pendingOrders = TiffintomOrder::where('status', 'Pending')->where('restaurant_id', $req->input('restaurant_id'))->orderBy('id', "DESC")->get();
        $completedOrders = TiffintomOrder::where('status', 'Delivered')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->orderBy('id', "DESC")->get();
        $acceptedOrders = TiffintomOrder::where('status', 'Accepted')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->orderBy('preparation', "ASC")->orderBy('id', "DESC")->get();

        $pendingDineinOrders = TiffintomDineinOrder::where('status', 'Pending')->where('restaurant_id', $req->input('restaurant_id'))->orderBy('id', "DESC")->get();
        $completedDineinOrders = TiffintomDineinOrder::where('status', 'Delivered')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->orderBy('id', "DESC")->get();
        $acceptedDineinOrders = TiffintomDineinOrder::where('status', 'Accepted')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->orderBy('preparation', "ASC")->orderBy('id', "DESC")->get();

        $response = [
            'pending_orders' => $pendingOrders,
            'accepted_orders' => $acceptedOrders,
            'completed_orders' => $completedOrders,
            'pending_dinein_orders' => $pendingDineinOrders,
            'accepted_dinein_orders' => $acceptedDineinOrders,
            'completed_dinein_orders' => $completedDineinOrders,
        ];

        return $response;
    }

    /**
     * Only for partner
     * List order-all-order
     * GET /tiffintom/orders/all-order
     */
    public function all_orders(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        if ($query = $req->query('from_date')) {
            $startDate = Carbon::parse($query)->startOfDay();
        }
        if ($query = $req->query('to_date')) {
            $endDate = Carbon::parse($query)->endOfDay();
        }


        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $normalOrders = TiffintomOrder::whereIn('status', ['Delivered', 'Failed'])->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->orderBy('id', "DESC")->get();
        $dineinOrders = TiffintomDineinOrder::where('status', 'Delivered')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->orderBy('id', "DESC")->get();

        $response = [
            'normal_orders' => $normalOrders,
            'dinein_orders' => $dineinOrders,
        ];

        return $response;
    }

    /**
     * Only for partner
     * List order-history
     * GET /tiffintom/orders/history
     */
    public function history(Request $req)
    {

        $startDate = Carbon::now()->startOfDay();
        $endDate = Carbon::now()->endOfDay();

        if ($req->has('start_date')) {
            $startDate = Carbon::parse($req->input('start_date'))->startOfDay();
        }
        if ($req->has('end_date')) {
            $endDate = Carbon::parse($req->input('end_date'))->endOfDay();
        }
        // $startDate = $startDate->format('Y-m-d H:i:s');
        // $endDate = $endDate->format('Y-m-d H:i:s');

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $completedOrders = TiffintomOrder::where('status', 'Delivered')->where('restaurant_id', $req->input('restaurant_id'))->whereBetween('created', [$startDate, $endDate])->orderBy('id', "DESC");
        $completedDineinOrders = TiffintomDineinOrder::where('status', 'Delivered')->where('restaurant_id', $req->input('restaurant_id'))->whereBetween('created', [$startDate, $endDate])->orderBy('id', "DESC");


        $sql = $completedOrders->toSql();
        $bindings = $completedOrders->getBindings();

        $sql_with_bindings = preg_replace_callback('/\?/', function ($match) use ($sql, &$bindings) {
            return "'" . array_shift($bindings) . "'";
        }, $sql);

        $completedOrders = $completedOrders->get();
        $completedDineinOrders = $completedDineinOrders->get();

        $response = [
            'sql' => $sql,
            'binding' => $bindings,
            'query' => print_r($sql_with_bindings, true),
            'start_date' => $startDate,
            'end_date' => $endDate,
            'completed_orders' => $completedOrders,
            'completed_dinein_orders' => $completedDineinOrders,
        ];

        return $response;
    }

    /**
     * Only for partner
     * GET order time-slot.
     * POST  /tiffintom/orders/{id}/time-slot
     */
    public function order_timeslot(Request $req, $id)
    {
        $order = TiffintomOrder::find($id);
        if (empty($order)) Utils::error("No order found", 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);
        $restaurant->append('restaurant_timing');

        $timeSlots = $this->getTimeSlot($order);

        return $timeSlots;
    }

    /**
     * Only for partner
     */
    public function getTimeSlot($order)
    {
        // if ($order->status != 'Pending') Utils::error("time slot fetch only pending order", 404);

        if ($order->delivery_date != '') {
            $restaurantDetails = TiffintomRestaurant::find($order->restaurant_id);
            if (empty($restaurantDetails)) Utils::error("No restaurant found", 404);
            // $restaurantDetails = $restaurantDetails->first();
            $restaurantDetails->append('restaurant_timing');

            //Timing Section
            $nowTime = date('h:i A');
            $currentTime = strtotime($nowTime);
            $date = date("d-m-Y", strtotime($order->delivery_date));
            $TimeSlots = array();

            $today = date('d-m-Y');
            $currentDate = date("d-m-Y", strtotime($date));
            $type = $order->order_type;

            $order->delivery_time = date('h:i A', strtotime('+0 minutes', strtotime($order->delivery_time)));

            $currentDay = strtolower(date('l', strtotime($currentDate)));
            if ($type == 'delivery' || $type == '') {
                $firstStartTime = $order->delivery_time;
                $firstStringStartTime = $restaurantDetails[$currentDay . '_first_opentime'];
                $firstEndTime = $restaurantDetails[$currentDay . '_first_closetime'];
                $secondStartTime = $order->delivery_time;
                $secondStringStartTime = $restaurantDetails[$currentDay . '_second_opentime'];
                $secondEndTime = $restaurantDetails[$currentDay . '_second_closetime'];

                $firstOpenTime = strtotime($order->delivery_time);
                // $firstOpenTime = strtotime($restaurantDetails[$currentDay . '_first_opentime']);
                $firstCloseTime = strtotime($restaurantDetails[$currentDay . '_first_closetime']);
                $secondOpenTime = strtotime($order->delivery_time);
                // $secondOpenTime = strtotime($restaurantDetails[$currentDay . '_second_opentime']);
                $secondCloseTime = strtotime($restaurantDetails[$currentDay . '_second_closetime']);
                $restentitytiming =    $restaurantDetails[$currentDay . '_status'];
                // $estimate_time = ($restaurantDetails[$currentDay . '_estimate_time'] > 0) ? $restaurantDetails[$currentDay . '_estimate_time'] : $restaurantDetails['estimate_time'];
                $estimate_time = 0;
            } else {
                $firstStartTime = $order->delivery_time;
                $firstStringStartTime = $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime'];
                $firstEndTime = $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime'];
                $secondStartTime = $order->delivery_time;
                $secondStringStartTime = $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime'];
                $secondEndTime = $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime'];

                $firstOpenTime = strtotime($order->delivery_time);
                $firstCloseTime = strtotime($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime']);
                $secondOpenTime = strtotime($order->delivery_time);
                $secondCloseTime = strtotime($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime']);
                $restentitytiming =    $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_status'];
                $estimate_time = 0;
            }
            //In first Timing section
            $array_of_time = array();
            $array_of_times = array();
            if ($restentitytiming != 'Close' && $restaurantDetails['close'] != 0) {
                $restaurantDetails['currentStatus'] = 'Open';
                $final[] = $restaurantDetails;
                if (strtotime($today) != strtotime($currentDate)) {
                    $nowTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                    $start_time = strtotime($currentDate . ' ' . $nowTime);
                    $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                    $fifteen_mins = 15 * 60;
                    $counts = 0;
                    while ($start_time <= $end_time) {
                        $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                        if ($firstStringStartTime != $firstEndTime) {
                            if ($counts == 0) {
                                $fifteen_minsss = $estimate_time * 60;
                                $start_time += $fifteen_minsss;
                                $array_of_time[] = date("h:i A", $start_time);
                                $counts++;
                            } else {
                                $array_of_time[] = date("h:i A", $start_time);
                                $counts++;
                            }
                        }
                        $start_time += $fifteen_mins;
                    }

                    $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                    $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                    $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                    $fifteen_mins = 15 * 60;
                    $countsd = 0;

                    while ($start_time <= $end_time) {
                        $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                        if ($secondStringStartTime != $secondEndTime) {
                            if ($countsd == 0) {
                                $fifteen_minsss = $estimate_time * 60;
                                $start_time += $fifteen_minsss;
                                $array_of_time[] = date("h:i A", $start_time);
                                $countsd++;
                            } else {
                                $array_of_time[] = date("h:i A", $start_time);
                                $countsd++;
                            }
                        }
                        $start_time += $fifteen_mins;
                    }
                } else {
                    if ($currentTime <= $firstOpenTime) {
                        $nowTimes = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $nowTimes);
                        $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            if ($currentTime <= $start_time) {
                                $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                if ($firstStringStartTime != $firstEndTime) {
                                    if ($counts == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    }
                                }
                            }
                            $start_time += $fifteen_mins;
                        }

                        $secondStartTimes = date("h:i A", strtotime('+0 minutes', strtotime($secondStringStartTime)));
                        $start_time1 = strtotime($currentDate . ' ' . $secondStartTimes);
                        $end_time1 = strtotime($currentDate . ' ' . $secondEndTime);
                        $fifteen_mins = 15 * 60;
                        $countsd = 0;
                        while ($start_time1 <= $end_time1) {
                            if ($currentTime <= $start_time1) {
                                $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                if ($secondStringStartTime != $secondEndTime) {
                                    if ($countsd == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time1 += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time1);
                                        $countsd++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time1);
                                        $countsd++;
                                    }
                                }
                            }
                            $start_time1 += $fifteen_mins;
                        }
                    }
                    if ($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) {
                        $firstOpenTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $firstOpenTime);
                        $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            if ($currentTime <= $start_time) {
                                $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                if ($firstStringStartTime != $firstEndTime) {
                                    if ($counts == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    }
                                }
                            }
                            $start_time += $fifteen_mins;
                        }
                        $secondStartTimes = date("h:i A", strtotime('+0 minutes', strtotime($secondStringStartTime)));
                        $start_time1 = strtotime($currentDate . ' ' . $secondStartTimes);
                        $end_time1 = strtotime($currentDate . ' ' . $secondEndTime);
                        $fifteen_mins = 15 * 60;
                        $countsd = 0;
                        while ($start_time1 <= $end_time1) {
                            if ($currentTime <= $start_time1) {
                                $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                if ($secondStringStartTime != $secondEndTime) {
                                    if ($countsd == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time1 += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time1);
                                        $countsd++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time1);
                                        $countsd++;
                                    }
                                }
                            }
                            $start_time1 += $fifteen_mins;
                        }
                    }
                    if (empty($array_of_time)) {
                        if ($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime) {
                            $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                            $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStringStartTime != $secondEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                        // else {
                        if ($currentTime < $secondOpenTime && empty($array_of_time)) {
                            $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                            $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $countsd = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStringStartTime != $secondEndTime) {
                                        if ($countsd == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $countsd++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $countsd++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                        // }
                    } else {
                        if ($currentTime < $secondOpenTime && empty($array_of_time)) {
                            $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                            $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $countsd = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStringStartTime != $secondEndTime) {
                                        if ($countsd == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $countsd++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $countsd++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                    }
                    if ($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) {
                        $currentTime = date("h:i A", strtotime('+0 minutes', $currentTime));
                        $start_time = strtotime($currentDate . ' ' . $currentTime);
                        $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            if ($currentTime <= $start_time) {
                                $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                if ($secondStringStartTime != $secondEndTime) {
                                    if ($counts == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    }
                                }
                            }
                            $start_time += $fifteen_mins;
                        }
                    }
                }
            }
        }

        if (!empty($array_of_time)) {
            foreach ($array_of_time as $time) {
                if ($firstOpenTime <= strtotime($currentDate . ' ' . $time)) {
                    $array_of_times[] = $time;
                }
            }
        }

        return $array_of_times;
    }

    /**
     * Only for partner
     * Update orders.
     * POST  /tiffintom/orders/id
     */
    public function changeStatus(Request $req, $id)
    {
        $order = TiffintomOrder::find($id);
        if (empty($order)) Utils::error('TiffintomOrder not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'status' => 'required|string',
            'preparation' => 'required_if:status,Accepted|string',
            'failed_reason' => 'required_if:status,Failed|string',
        ]);

        // update order
        if ($req->has('status')) $order->status = $req->input('status');
        if ($req->has('preparation')) $order->preparation = $req->input('preparation');
        if ($req->has('failed_reason')) $order->failed_reason = $req->input('failed_reason');

        $order->save();

        if ($req->input('status') == 'Failed' && $order->payment_method == 'paypal') {
            $siteSetting = TiffintomSiteSetting::where('id', '1')->first();
            if ($siteSetting['paypal_mode'] == 'Test') {
                $publicKey = $siteSetting['test_clientid'];
                $privateKey = $siteSetting['test_secretkey'];
            }
            if ($siteSetting['paypal_mode'] == 'Live') {
                $publicKey = $siteSetting['live_clientid'];
                $privateKey = $siteSetting['live_secretkey'];
            }

            try {
                $apiContext = new ApiContext(new OAuthTokenCredential($publicKey, $privateKey));
                // $singlePayment = Payment::get($paypalPayment->payment_id, $apiContext);

                //Create refund
                $amt = new Amount();
                $amt->setTotal($order->order_grand_total)->setCurrency('GBP');

                $refund = new Refund();
                $refund->setAmount($amt);
                $sale = new Sale();
                $sale->setId($order->transaction_id);

                //Execute Refund
                $sale->refund($refund, $apiContext);
            } catch (PayPalConnectionException $ex) {
                return $ex->getData();
            } catch (Exception $ex) {
                return $ex->getMessage();
            }
        }

        // if ($req->input('status') == 'Failed' && $order->payment_method == 'Stripe') {
        //     $stripeKey = "";
        //     $siteSetting = TiffintomSiteSetting::where('id', '1')->first();
        //     if ($siteSetting['stripe_mode'] == 'Test') {
        //         $stripeKey = $siteSetting['stripe_secretkeyTest'];
        //     }
        //     if ($siteSetting['stripe_mode'] == 'Live') {
        //         $stripeKey = $siteSetting['stripe_secretkey'];
        //     }

        //     $business = Business::where('restaurant_id', $order->restaurant_id)->where('connect_service', '1')->whereNotNull('s_account_id')->first();
        //     if (!empty($business)) {
        //         $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
        //         if ($mode == 'test') {
        //             $stripeKeyColoumns = 'connect_stripe_private_key_test';
        //         }
        //         if ($mode == 'live') {
        //             $stripeKeyColoumns = 'connect_stripe_private_key_live';
        //         }
        //         $stripeKey =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
        //     }

        //     $stripe = new \Stripe\StripeClient($stripeKey);

        //     $paymentIntent =  $stripe->paymentIntents->retrieve($order->transaction_id);
        //     $chargeDetails = $stripe->charges->retrieve($paymentIntent->charges->data[0]->id);

        //     $refundParams = [
        //         'charge' => $paymentIntent->charges->data[0]->id,
        //     ];

        //     try {
        //         $stripe->refunds->create($refundParams);
        //         Log::error("Refund Complete");
        //     } catch (\Stripe\Exception\InvalidRequestException $e) {
        //         Log::error("Invalid Request Exception");
        //         Utils::error($e->getError()->message, 400);
        //     } catch (\Stripe\Exception\CardException $e) {
        //         Log::error("Card Exception");
        //         Utils::error($e->getError()->message, 400);
        //     } catch (\Stripe\Exception\ApiErrorException $e) {
        //         Log::error("Api Error Exception");
        //         Utils::error($e->getError()->message, 400);
        //     }
        // }

        $user = TiffintomUser::find($order->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($req->input('status') == 'Delivered') {

            $reward = TiffintomReward::orderBy('id', 'ASC')->first();

            if (!empty($reward) && $restaurant->reward_option == 'Yes') {
                $getRewardFromTotal = $order->order_sub_total / $reward->reward_amount;
                $orderPoint = $getRewardFromTotal * $reward->reward_point;

                $customerReward = new TiffintomCustomerReward;
                $customerReward->order_id = $order->id;
                $customerReward->restaurant_name = $restaurant->restaurant_name;
                $customerReward->customer_id = $order->customer_id;
                $customerReward->total = $order->order_sub_total;
                $customerReward->points = $orderPoint;
                $customerReward->type = 'Earned';
                $customerReward->save();
            }
        }

        if ($req->input('status') == 'Failed') {
            $message = 'Sorry, but your order from ' . $restaurant->restaurant_name . ' cannot be accepted at the moment. Sorry for any inconvenience caused.';
            $title = "Order rejected";
            $sms_event = 'order_reject';
        } else if ($req->input('status') == 'Delivered') {
            $title = "Order delivered";
            $message = 'Your order from ' . $restaurant->restaurant_name . ' has been successfully delivered. Enjoy your meal.';
            $sms_event = 'order_update';
        } else if ($req->input('status') == 'Accepted') {
            $title = "Order accepted";
            $sms_event = 'order_accept';
            if ($order->order_type == 'delivery') {
                $message = 'Your order from ' . $restaurant->restaurant_name . ' has been accepted. Your food will be with you at ' . $order->preparation;
            } else {
                $message = 'Your order from ' . $restaurant->restaurant_name . ' has been accepted. Your food will be with you at ' . $order->preparation;
            }
        } else {
            $title = "";
            $message = "";
            if ($req->input('status') == 'accepted') {
                $title = "Order accepted";
                $sms_event = 'order_accept';
                if ($order->order_type == 'delivery') {
                    $message = 'Your order from ' . $restaurant->restaurant_name . ' has been accepted. Your food will be with you at ' . $order->preparation;
                } else {
                    $message = 'Your order from ' . $restaurant->restaurant_name . ' has been accepted. Your food will be with you at ' . $order->preparation;
                }
            }
        }

        // All status wise user text message send
        // if ($user->phone_verify == 1) {
        if ($siteSettings->user_order_sms == 'Yes') {
            if ($user->order_accept == 7 && $req->input('status') == 'Accepted') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_delivered == 7 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_reject == 7 && $req->input('status') == 'Failed') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_accept == 2 && $req->input('status') == 'Accepted') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_delivered == 2 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_reject == 2 && $req->input('status') == 'Failed') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_accept == 3 && $req->input('status') == 'Accepted') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_delivered == 3 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_reject == 3 && $req->input('status') == 'Failed') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_accept == 6 && $req->input('status') == 'Accepted') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_delivered == 6 && $req->input('status') == 'Delivered' && $restaurant->delivered_message == 'Yes') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
            if ($user->order_reject == 6 && $req->input('status') == 'Failed') {
                (new OtpHelper())->sendMessage($message, $user->phone_number, $sms_event, $restaurant->sms_sender_id);
            }
        }
        // }

        // All Status Wise Notification Set
        $notification['title']          = $title;
        $notification['body']        = $message;

        $notificationdata['data']['title']          = $title;
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['order_id']  = $order->id;
        $notificationdata['data']['order_status']  = $order->status;
        $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type'    => "order");
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
        if ($user->order_accept == 7 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 7 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 7 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_accept == 1 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 1 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 1 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 3 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 3 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_accept == 3 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_delivered == 5 && $req->input('status') == 'Delivered') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_reject == 5 && $req->input('status') == 'Failed') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }
        if ($user->order_accept == 5 && $req->input('status') == 'Accepted') {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        $this->orderMail($order->id);

        $order->append('cart_view');

        //Call Socket
        (new TiffintomRestaurantController())->restaurant_pending_count($req, $order->restaurant_id);

        return $order;
    }

    /**
     * Only for partner
     * Update orders.
     * POST  /tiffintom/orders/id/time-extend
     */
    public function time_extend(Request $req, $id)
    {
        $order = TiffintomOrder::find($id);
        if (empty($order)) Utils::error('TiffintomOrder not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'status' => 'string',
            'preparation' => 'required|string',
        ]);

        // update order
        // if ($req->has('status')) $order->status = $req->input('status');
        if ($req->has('preparation')) $order->preparation = $req->input('preparation');
        $order->save();

        $user = TiffintomUser::find($order->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $message = 'Your order ' . $order->order_number . ' has been ' . $order->preparation . ' Time Extended.';

        // All status wise user text message send
        // if ($user->phone_verify == 1) {
        if ($siteSettings->user_order_sms == 'Yes') {
            if ($user->order_accept == 7 && $user->order_accept == 2) {
                (new OtpHelper())->sendMessage($message, $user->phone_number, 'order_update', $restaurant->sms_sender_id);
            }
        }
        // }

        // All Status Wise Notification Set
        $notification['title']          = "Order accepted";
        $notification['body']        = $message;

        $notificationdata['data']['title']          = 'Order accepted';
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['order_status']  = $order->status;
        $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type'    => "OrderStatus");
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
        if ($user->order_accept == 7 && $user->order_accept == 5 && $user->order_accept == 3 && $user->order_accept == 1) {
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        $this->orderMail($order->id);

        return $order;
    }


    /**
     * Only for partner
     * Update orders.
     * POST  /tiffintom/orders/id/rtc
     */
    public function order_rtc(Request $req, $id)
    {
        $order = TiffintomOrder::selectRaw('orders.*,restaurants.restaurant_name as restaurant_name,users.first_name as first_name')->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->leftJoin('users', 'users.id', '=', 'customer_id')->where('orders.id', $id)->first();
        if (empty($order)) Utils::error('order not found!', 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $fetchRTC = TiffintomRTC::where('res_id', $order->restaurant_id)->first();
        if (!empty($fetchRTC)) {
            $fetchRTC->count = $fetchRTC->count + 1;
            $fetchRTC->save();
        } else {
            $fetchRTC = new TiffintomRTC;
            $fetchRTC->res_id = $order->restaurant_id;
            $fetchRTC->count = 1;
            $fetchRTC->save();
        }
        if ($order->order_type == 'delivery') {
            $message = 'Hello ' . $order->first_name . ' your order ' . $order->order_number . ' at ' . $order->restaurant_name . ' is on the way.Thank you';
        } else {
            $message = 'Hello ' . $order->first_name . ' your order ' . $order->order_number . ' at ' . $order->restaurant_name . ' is ready for collection.Thank you';
        }
        (new OtpHelper())->sendMessage($message, $order->customer_phone, 'order_update', $restaurant->sms_sender_id);

        return ['message' => 'message send successfully!'];
    }

    /**
     * Create order.
     * POST /tiffintom/order
     */
    public function create(Request $req)
    {
        if ($req->has('carts') && !is_array($req->input('carts'))) {
            $req->merge(['carts' => json_decode($req->input('carts'))]);
        }
        if ($req->has('surcharges') && !is_array($req->input('surcharges'))) {
            $req->merge(['surcharges' => json_decode($req->input('surcharges'))]);
        }

        $this->validate($req, [
            'order_number'  => 'string',
            'restaurant_id'  => 'required|string',
            'customer_id'  => 'required|string',
            'customer_name'  => 'required|string',
            'customer_email'  => 'string', // required|
            'customer_phone'  => 'required|string',
            'source_latitude'  => 'string',
            'source_longitude'  => 'string',
            'destination_latitude'  => 'string',
            'destination_longitude'  => 'string',
            'flat_no'  => 'string',
            'address'  => 'string',
            'order_type'  => 'required|string',
            'assoonas'  => 'string',
            'order_description'  => 'string',
            'delivery_date'  => 'required|string',
            'delivery_time'  => 'required|string',
            'offer_percentage'  => 'numeric',
            'offer_amount'  => 'numeric',
            'day_offer'  => 'numeric',
            'delivery_charge'  => 'numeric',
            'service_charge'  => 'numeric',
            'voucher_code'  => 'string',
            'voucher_percentage'  => 'numeric',
            'voucher_amount'  => 'numeric',
            'tip_percentage'  => 'numeric',
            'tip_amount'  => 'numeric',
            'order_sub_total'  => 'required|numeric|gt:0',
            'order_grand_total'  => 'required|numeric|gt:0',
            'payment_status'  => 'string',
            'payment_method'  => 'required|string',
            'card_id'  => 'string',
            'payment_wallet'  => 'string',
            'paid_full'  => 'string',
            'split_payment'  => 'string',
            'wallet_amount'  => 'string',
            'transaction_id'  => 'string',
            'reward_used'  => 'string',
            'order_point'  => 'numeric',
            'spent_point'  => 'numeric',
            'reward_offer'  => 'numeric',
            'reward_offer_percentage'  => 'numeric',
            'payerID'  => 'string',
            'paymentToken'  => 'string',
            'paymentID'  => 'string',
            'status'  => 'string',
            'type'  => 'string',
            'dont_bell'  => 'string',
            'dont_cutlery'  => 'string',
            'is_favourite'  => 'string',
            'charity_amount'  => 'numeric',
            'driver_tip'  => 'string',
            'charity_message'  => 'string',
            'delivery_instruction'  => 'string',
            'carts' => 'required|array',
            'surcharges' => 'array',
            'surcharge_amount'  => 'numeric',
        ]);

        // create order
        $order = new TiffintomOrder;
        $order->fill($req->all());
        if ($req->input('payment_method') == 'Stripe' && !$req->has('transaction_id')) {
            $order->status = 'Processing';
        } else {
            $order->status = 'Pending';
        }
        $order->save();

        $order->order_number = 'ORD00' . $order->id;
        if ($req->input('flat_no') != '' && (strtolower($req->input('type')) == 'ios' || strtolower($req->input('type')) == 'android')) {
            $order->address = $req->input('flat_no') . ',' . $req->input('address');
        }
        if (strtolower($req->input('assoonas')) == 'now') {
            $order->delivery_date = date('Y-m-d');
            $order->delivery_time = date('h:i A');
        }

        if ($req->has('delivery_date')) {
            if (strtotime($req->input('delivery_date')) < strtotime(date('Y-m-d'))) {
                $order->delivery_date = date('Y-m-d');
            }
        }

        if ($req->has('surcharge_amount')) {
            $order->surcharges = $req->input('surcharges');
            $order->surcharge_amount = $order->surcharge_amount;
        }

        if ($req->hasHeader('package-name')) {
            $order->package_name = $req->header('package-name');
        }

        if ($req->hasHeader('app-version')) {
            $order->app_version = $req->header('app-version');
        }

        $order->save();

        //Cart Add
        $oldCart = TiffintomCart::where('customer_id', $order->customer_id)->where('order_id', 0)->first();
        if (!empty($oldCart)) {
            $dbCart = TiffintomCart::where('customer_id', $order->customer_id)->where('order_id', 0)->delete();
        }
        $carts = ($req->input('carts'));
        foreach ($carts as $cart) {
            $cartItem = new TiffintomCart((array)$cart);
            $cartItem->order_id = $order->id;
            $cartItem->customer_id = $req->input('customer_id');
            $cartItem->restaurant_id = $req->input('restaurant_id');
            $cartItem->save();
        }

        if ($order->status != 'Processing') {
            // Wallet Payment 
            if ($req->input('payment_wallet') == 'Yes') {
                $user = TiffintomUser::find($req->input('customer_id'));
                // if ($user->wallet_amount < $req->input('order_grand_total')) {
                $user->wallet_amount = $user->wallet_amount - $req->input('wallet_amount'); //$req->input('order_grand_total');
                // } else {
                //     $user->wallet_amount = 0;
                // }
                $user->save();

                $wallet_history = new TiffintomWalletHistory;
                $wallet_history->customer_id = $req->input('customer_id');
                $wallet_history->purpose = "Amount Paid";
                $wallet_history->transaction_type = "Debited";
                $wallet_history->transaction_details = $order->order_number;
                $wallet_history->amount = $req->input('wallet_amount');
                $wallet_history->save();
            }

            //Reward Points Update 
            if ($req->input('reward_used') == 'Y') {
                $rewardPoints = TiffintomReward::orderBy('id', 'ASC')->first();

                $order->spent_point =  $rewardPoints['reward_totalpoint'];
                $order->save();

                $today = date("d-m-Y");
                $expireday = '-' . $rewardPoints['reward_validity'] . ' days';
                $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
                $startdate = date("Y-m-d 23:59:59", strtotime($today));
                $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

                $customerPoints = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->where('created', '<=', $startdate)->where('created', '>=', $endDate)->sum('points');

                $rewardPointSpent = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('type', 'Spent')->where('status', '1')->get();

                if (!empty($rewardPointSpent)) {
                    foreach ($rewardPointSpent as $value) {
                        $customerPointsave = TiffintomCustomerReward::where('id', $value['id'])->first();
                        $customerPointsave->status = '0';
                        $customerPointsave->save();
                    }
                }
                $orderPoints = $customerPoints - $rewardPoints['reward_totalpoint'];
                $customerPoint = new TiffintomCustomerReward;
                $restaurantDetails = TiffintomRestaurant::where('id', $req->input('restaurant_id'))->first();
                $customerPoint->order_id = $order->id;
                $customerPoint->restaurant_name = $restaurantDetails['restaurant_name'];
                $customerPoint->customer_id = $req->input('customer_id');
                $customerPoint->total = $req->input('order_grand_total');
                $customerPoint->points = $orderPoints;
                $customerPoint->type = 'Spent';
                $customerPoint->save();

                $today = date("d-m-Y");
                $expireday = '-' . $rewardPoints['reward_validity'] . ' days';
                $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
                $startdate = date("Y-m-d 23:59:59", strtotime($today));
                $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

                $rewardPointEarn = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->where('type', 'Earned')->get();

                if (!empty($rewardPointEarn)) {
                    foreach ($rewardPointEarn as $value) {
                        $customerPointsave = TiffintomCustomerReward::where('id', $value['id'])->first();
                        $customerPointsave->status = '0';
                        $customerPointsave->save();
                    }
                }
            }

            $user = TiffintomUser::find($order->customer_id);
            if (empty($user)) Utils::error("No user found", 404);

            $restaurant = TiffintomRestaurant::find($order->restaurant_id);
            if (empty($restaurant)) Utils::error("No restaurant found", 404);

            $siteSettings = TiffintomSiteSetting::find('1');
            if (empty($siteSettings)) Utils::error("No site-settings found", 404);


            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            $preparation_time = '';
            if (!empty($business)) {
                if ($business->auto_accept_fields && $business->auto_accept_order == 'yes') {
                    $today = strtolower(Carbon::parse($order->delivery_date)->format('l'));
                    $allocate_time = $business->auto_accept_fields[$today]['allocate_time'];
                    if ($allocate_time != 0) {
                        $delivery_time = Carbon::parse($order->delivery_time)->format('H:i:s');
                        $preparation_times = Carbon::createFromFormat('H:i:s', $delivery_time)->addMinutes($allocate_time)->toTimeString();
                        $preparation_time = Carbon::parse($preparation_times)->format('h:i A');
                    }
                }
            }
            // Restaurant Fcm Notification
            $message = 'You have a new order - ' . $order->order_number;

            $notification['title'] = "Neworder";
            $notification['body'] = $message;

            $notificationdata['data']['title'] = "Neworder";
            $notificationdata['data']['message'] = $message;
            $notificationdata['data']['is_background'] = false;
            $notificationdata['data']['order_id'] = $order->id;
            $notificationdata['data']['preparation_time'] = $preparation_time;
            $notificationdata['data']['order_status'] = $order->status;
            $notificationdata['data']['order_type'] = ucfirst($order->order_type);
            $notificationdata['data']['payment_status'] = $order->payment_status != '' ? 'Paid' : 'Not Paid';
            $notificationdata['data']['payload'] = array('OrderDetails' => "", 'type' => "order");
            $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
            $Devices = explode(',', $restaurant->device_id);
            foreach ($Devices as $Device) {
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device, $notification, 'restaurant');
            }

            if ($user->order_place == 7) {
                // $this->orderEmail($order->id);
                $message  = 'Your order from ' . $restaurant->restaurant_name . ' has been successfully placed. Please wait for the restaurant to accept.';

                $notification['title']          = "Order placed";
                $notification['body']        = $message;

                $notificationdata['data']['title']          = "Order placed";
                $notificationdata['data']['message']        = $message;
                $notificationdata['data']['is_background']  = false;
                $notificationdata['data']['order_id']  = $order->id;
                $notificationdata['data']['order_status']  = $order->status;
                $notificationdata['data']['order_type']  = ucfirst($order->order_type);
                $notificationdata['data']['payment_status']  = $order->payment_status != '' ? 'Paid' : 'Not Paid';
                $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "order");
                $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
            }
            if ($user->order_place == 4 || $user->order_place == 5 || $user->order_place == 6) {
                // $this->orderEmail($order->id);
            }
            if ($user->order_place == 1 || $user->order_place == 3 || $user->order_place == 5) {
                $message  = 'Your order from ' . $restaurant->restaurant_name . ' has been successfully placed. Please wait for the restaurant to accept.';

                $notification['title']          = "Order placed";
                $notification['body']        = $message;

                $notificationdata['data']['title']          = "Order placed";
                $notificationdata['data']['message']        = $message;
                $notificationdata['data']['is_background']  = false;
                $notificationdata['data']['order_id']  = $order->id;
                $notificationdata['data']['order_status']  = $order->status;
                $notificationdata['data']['order_type']  = ucfirst($order->order_type);
                $notificationdata['data']['payment_status']  = $order->payment_status != '' ? 'Paid' : 'Not Paid';
                $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "order");
                $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
            }
            if ($user->order_place == 2 || $user->order_place == 3 || $user->order_place == 6 || $user->order_place == 7) {
                //if ($user->phone_verify == 1) {
                if ($siteSettings->user_order_sms == 'Yes') {
                    $message = 'Hey ' . $user->first_name . '! Thank you for placing your order with TiffinTom. Please wait for the restaurant to receive and confirm your order. Your order number is ' . $order->order_number;
                    (new OtpHelper())->sendMessage($message, $user->phone_number, 'order_place', $restaurant->sms_sender_id);
                }
                // }
                if ($siteSettings->admin_order_all == 'Yes') {
                    $message = 'New online order from Tiffintom - ' . $order->order_number;
                    (new OtpHelper())->sendMessage($message, $siteSettings->admin_phone, 'Sitesetting', 'order_place', $restaurant->sms_sender_id);
                }

                $UsersList = TiffintomUser::where('status', '1')->where('role_id', '4')->get();
                foreach ($UsersList as $key => $value) {
                    if ($value['sms_permissions'] != '') {
                        $sms_permissions = explode(',', $value['sms_permissions']);
                        if (in_array("Order", $sms_permissions)) {
                            $message = 'New online order from Tiffintom - ' . $order->order_number;
                            // (new OtpHelper())->sendMessage($message, $value['phone_number'], 'Sitesetting');
                            (new OtpHelper())->sendMessage($message, $value['phone_number'], 'order_place', $restaurant->sms_sender_id);
                        }
                    }
                }

                $messages = 'New online order from Tiffintom -  ' . $order->order_number . ' Please check now! Thank you';
                if ($siteSettings['restaurant_order_all'] == 'Yes') {
                    // (new OtpHelper())->sendMessage($messages, $restaurant->sms_phonenumber, 'Restaurant');
                    (new OtpHelper())->sendMessage($messages, $restaurant->sms_phonenumber, 'order_place', $restaurant->sms_sender_id);
                }
            }

            $this->orderMail($order->id);
        }

        $order = TiffintomOrder::find($order->id);
        $order->append('cart_view');

        //Call Socket
        (new TiffintomRestaurantController())->restaurant_pending_count($req, $order->restaurant_id);

        return $order;
    }


    /**
     * Order Replica To Place / Failed
     * Update orders.
     * POST  /tiffintom/orders/id
     */
    public function orderReplica($req)
    {
        $id = str_replace("ORD00", "", $req->statement_descriptor_suffix);
        $order = TiffintomOrder::find($id);
        if (empty($order)) Utils::error('Order not found!', 404);
        if ($order->status != 'Processing') Utils::error('Order is not processing status!', 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        // update order
        if ($req->status == 'succeeded') {
            $order->status = 'Pending';
            $order->transaction_id = $req->id;
            $order->payment_status = 'P';

            $cartsDetails = TiffintomCart::where('temp_orderid', $order->id)->get();
            if (!$cartsDetails->isEmpty()) {
                foreach ($cartsDetails as $value) {
                    $cartObj = TiffintomCart::where('id', $value->id)->first();
                    $cartObj->order_id = $order->id;
                    $cartObj->temp_orderid = 0;
                    $cartObj->save();
                }
            }
        }
        if ($req->status == 'canceled' || $req->status == 'requires_payment_method') {
            $order->status = 'PaymentCancel';
            $order->payment_status = 'NP';
            $order->failed_reason = 'abandoned';
        }
        $order->save();

        if ($req->status == 'succeeded') {
            // Wallet Payment 
            if ($order->payment_wallet == 'Yes') {
                $user = TiffintomUser::find($order->customer_id);
                $user->wallet_amount = $user->wallet_amount - $order->wallet_amount;
                $user->save();

                $wallet_history = new TiffintomWalletHistory;
                $wallet_history->customer_id = $order->customer_id;
                $wallet_history->purpose = "Amount Paid";
                $wallet_history->transaction_type = "Debited";
                $wallet_history->transaction_details = $order->order_number;
                $wallet_history->amount = $order->wallet_amount;
                $wallet_history->save();
            }

            //Reward Points Update 
            if ($req->input('reward_used') == 'Y') {
                $rewardPoints = TiffintomReward::orderBy('id', 'ASC')->first();

                $order->spent_point =  $rewardPoints['reward_totalpoint'];
                $order->save();

                $today = date("d-m-Y");
                $expireday = '-' . $rewardPoints['reward_validity'] . ' days';
                $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
                $startdate = date("Y-m-d 23:59:59", strtotime($today));
                $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

                $customerPoints = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->where('created', '<=', $startdate)->where('created', '>=', $endDate)->sum('points');

                $rewardPointSpent = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('type', 'Spent')->where('status', '1')->get();

                if (!empty($rewardPointSpent)) {
                    foreach ($rewardPointSpent as $value) {
                        $customerPointsave = TiffintomCustomerReward::where('id', $value['id'])->first();
                        $customerPointsave->status = '0';
                        $customerPointsave->save();
                    }
                }
                $orderPoints = $customerPoints - $rewardPoints['reward_totalpoint'];
                $customerPoint = new TiffintomCustomerReward;
                $restaurantDetails = TiffintomRestaurant::where('id', $req->input('restaurant_id'))->first();
                $customerPoint->order_id = $order->id;
                $customerPoint->restaurant_name = $restaurantDetails['restaurant_name'];
                $customerPoint->customer_id = $req->input('customer_id');
                $customerPoint->total = $req->input('order_grand_total');
                $customerPoint->points = $orderPoints;
                $customerPoint->type = 'Spent';
                $customerPoint->save();

                $today = date("d-m-Y");
                $expireday = '-' . $rewardPoints['reward_validity'] . ' days';
                $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
                $startdate = date("Y-m-d 23:59:59", strtotime($today));
                $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

                $rewardPointEarn = TiffintomCustomerReward::where('customer_id', $order->customer_id)->where('status', '1')->where('type', 'Earned')->get();

                if (!empty($rewardPointEarn)) {
                    foreach ($rewardPointEarn as $value) {
                        $customerPointsave = TiffintomCustomerReward::where('id', $value['id'])->first();
                        $customerPointsave->status = '0';
                        $customerPointsave->save();
                    }
                }
            }
            $user = TiffintomUser::find($order->customer_id);
            if (empty($user)) Utils::error("No user found", 404);

            $restaurant = TiffintomRestaurant::find($order->restaurant_id);
            if (empty($restaurant)) Utils::error("No restaurant found", 404);

            $siteSettings = TiffintomSiteSetting::find('1');
            if (empty($siteSettings)) Utils::error("No site-settings found", 404);

            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            $preparation_time = '';
            if (!empty($business)) {
                if ($business->auto_accept_fields && $business->auto_accept_order == 'yes') {
                    $today = strtolower(Carbon::parse($order->delivery_date)->format('l'));
                    $allocate_time = $business->auto_accept_fields[$today]['allocate_time'];
                    if ($allocate_time != 0) {
                        $delivery_time = Carbon::parse($order->delivery_time)->format('H:i:s');
                        $preparation_times = Carbon::createFromFormat('H:i:s', $delivery_time)->addMinutes($allocate_time)->toTimeString();
                        $preparation_time = Carbon::parse($preparation_times)->format('h:i A');
                    }
                }
            }

            // Restaurant Fcm Notification
            $message = 'You have a new order - ' . $order->order_number;

            $notification['title'] = "Neworder";
            $notification['body'] = $message;

            $notificationdata['data']['title'] = "Neworder";
            $notificationdata['data']['message'] = $message;
            $notificationdata['data']['is_background'] = false;
            $notificationdata['data']['order_id'] = $order->id;
            $notificationdata['data']['preparation_time'] = $preparation_time;
            $notificationdata['data']['order_status'] = $order->status;
            $notificationdata['data']['order_type'] = ucfirst($order->order_type);
            $notificationdata['data']['payment_status'] = $order->payment_status != '' ? 'Paid' : 'Not Paid';
            $notificationdata['data']['payload'] = array('OrderDetails' => "", 'type' => "order");
            $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
            $Devices = explode(',', $restaurant->device_id);
            foreach ($Devices as $Device) {
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device, $notification, 'restaurant');
            }

            if ($user->order_place == 7) {
                // $this->orderEmail($order->id);
                $message  = 'Your order from ' . $restaurant->restaurant_name . ' has been successfully placed. Please wait for the restaurant to accept.';

                $notification['title']          = "Order placed";
                $notification['body']        = $message;

                $notificationdata['data']['title']          = "Order placed";
                $notificationdata['data']['message']        = $message;
                $notificationdata['data']['is_background']  = false;
                $notificationdata['data']['order_id']  = $order->id;
                $notificationdata['data']['order_status']  = $order->status;
                $notificationdata['data']['order_type']  = ucfirst($order->order_type);
                $notificationdata['data']['payment_status']  = $order->payment_status != '' ? 'Paid' : 'Not Paid';
                $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "order");
                $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
            }
            if ($user->order_place == 1 || $user->order_place == 3 || $user->order_place == 5) {
                $message  = 'Your order from ' . $restaurant->restaurant_name . ' has been successfully placed. Please wait for the restaurant to accept.';

                $notification['title']          = "Order placed";
                $notification['body']        = $message;

                $notificationdata['data']['title']          = "Order placed";
                $notificationdata['data']['message']        = $message;
                $notificationdata['data']['is_background']  = false;
                $notificationdata['data']['order_id']  = $order->id;
                $notificationdata['data']['order_status']  = $order->status;
                $notificationdata['data']['order_type']  = ucfirst($order->order_type);
                $notificationdata['data']['payment_status']  = $order->payment_status != '' ? 'Paid' : 'Not Paid';
                $notificationdata['data']['payload']        = array('OrderDetails' => "", 'type' => "order");
                $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
            }
            if ($user->order_place == 2 || $user->order_place == 3 || $user->order_place == 6 || $user->order_place == 7) {
                if ($siteSettings->user_order_sms == 'Yes') {
                    $message = 'Hey ' . $user->first_name . '! Thank you for placing your order with TiffinTom. Please wait for the restaurant to receive and confirm your order. Your order number is ' . $order->order_number;
                    (new OtpHelper())->sendMessage($message, $user->phone_number, 'order_place');
                }
                if ($siteSettings->admin_order_all == 'Yes') {
                    $message = 'New online order from Tiffintom - ' . $order->order_number;
                    (new OtpHelper())->sendMessage($message, $siteSettings->admin_phone, 'Sitesetting', 'order_place', $restaurant->sms_sender_id);
                }

                $UsersList = TiffintomUser::where('status', '1')->where('role_id', '4')->get();
                foreach ($UsersList as $key => $value) {
                    if ($value['sms_permissions'] != '') {
                        $sms_permissions = explode(',', $value['sms_permissions']);
                        if (in_array("Order", $sms_permissions)) {
                            $message = 'New online order from Tiffintom - ' . $order->order_number;
                            (new OtpHelper())->sendMessage($message, $value['phone_number'], 'order_place', $restaurant->sms_sender_id);
                        }
                    }
                }

                $messages = 'New online order from Tiffintom -  ' . $order->order_number . ' Please check now! Thank you';
                if ($siteSettings['restaurant_order_all'] == 'Yes') {
                    (new OtpHelper())->sendMessage($messages, $restaurant->sms_phonenumber, 'order_place', $restaurant->sms_sender_id);
                }

                $this->orderMail($order->id);
            }
        }

        return $order;
    }


    /**
     * List order
     * GET /tiffintom/order/id
     */
    public function show(Request $req, $id)
    {
        $order = TiffintomOrder::find($id);
        if (empty($order)) Utils::error("order not found", 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $order->append('cart_view');
        $order->append('card_view');
        $order->append('reviews');
        return $order;
    }

    /**
     * Delete order by id.
     * DELETE /tiffintom/orders/{id}
     */
    public function delete(Request $req, $id)
    {
        $order = TiffintomOrder::find($id);
        if (empty($order)) Utils::error('order not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $order->delete();

        return $order;
    }

    /**
     * Delete order by id.
     * DELETE /tiffintom/orders/bulk-delete
     */
    public function bulk_delete(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);

        $orders = TiffintomOrder::whereIn('id', $req->input('ids'))->delete();
        $ids = $req->input('ids');
        foreach ($ids as $id) {

            $user = Auth::user();

            if (isset($_SERVER['HTTP_CLIENT_IP']))
                $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
            else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
                $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
            else if (isset($_SERVER['HTTP_X_FORWARDED']))
                $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
            else if (isset($_SERVER['HTTP_FORWARDED_FOR']))
                $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
            else if (isset($_SERVER['HTTP_FORWARDED']))
                $ipaddress = $_SERVER['HTTP_FORWARDED'];
            else if (isset($_SERVER['REMOTE_ADDR']))
                $ipaddress = $_SERVER['REMOTE_ADDR'];
            else
                $ipaddress = '';

            $action_log = new ActionLog();
            $action_log->user_type =  'tiffintom';
            $action_log->user_id =  $user->id;
            $action_log->business_id =  Auth::$business_id;
            $action_log->method =  "DELETE";
            $action_log->prefix =  'orders';
            $action_log->end_point =  $id;
            $action_log->full_path =  '/epos/tiffintom/orders/';
            $action_log->request_raw =  [];
            $action_log->ip =  $ipaddress;
            $action_log->save();
        }

        return $req;
    }


    /**
     * List order
     * POST /tiffintom/order/offer-check/{code}
     */
    public function offer_check(Request $req)
    {
        if ($req->has('carts') && !is_array($req->input('carts'))) {
            $req->merge(['carts' => json_decode($req->input('carts'))]);
        }

        $order = new TiffintomOrder;
        $order->fill($req->all());

        $subtotal = 0;
        $carts = ($req->input('carts'));
        $order['carts'] = $carts;
        if (!empty($carts)) {
            foreach ($carts as $cart) {
                if (isset($cart->total_price)) {
                    $subtotal = $subtotal + $cart->total_price;
                }
            }
        }
        $subtotal = $this->convertToDecimal($subtotal);


        $order['order_sub_total'] = $this->convertToDecimal($subtotal);
        $order['order_grand_total'] = $this->convertToDecimal($subtotal);

        $order['offer_amount'] = 0;
        $order['offer_percentage'] = '';

        $restaurantDetails = TiffintomRestaurant::where('id', $order['restaurant_id'])->first();
        $reward = TiffintomReward::orderBy('id', 'ASC')->first();

        if (!empty($reward) && $restaurantDetails['reward_option'] == 'Yes') {

            $customerPoints = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->first();

            $createdDate = explode(' ', $reward['created']);
            $date1 = date_create($createdDate[0]);
            $date2 = date_create(date('Y-m-d'));

            $diff = date_diff($date1, $date2);
            $diff = $diff->format("%R%a");

            $remainingDays = $reward['reward_validity'];
            // $remainingDays = ($reward['reward_validity'] - $diff);
            $resPercent = $reward['reward_percentage'];

            if ($remainingDays > 0) {

                $today = date("d-m-Y");
                $expireday = '-' . $reward['reward_validity'] . ' days';
                $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
                $startdate = date("Y-m-d 23:59:59", strtotime($today));
                $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));

                $customerPoints = TiffintomCustomerReward::where('customer_id', $order['customer_id'])->where('status', '1')->where('created', '<=', $startdate)->where('created', '>=', $endDate)->sum('points');

                $userPoints = $customerPoints;
                $resPoints = $reward['reward_totalpoint'];

                if ($subtotal > $reward['minimum_order']) {
                    if ($userPoints >= $resPoints) {
                        $totPercent = $reward['reward_percentage'];
                        $offPercent = $subtotal * abs($totPercent / 100);
                        if ($reward['miximum_value'] > $offPercent) {
                            $order['redeemPoint'] = $resPoints;
                            $order['rewardPoint'] = $this->convertToDecimal($offPercent);
                            $order['rewardPercentage'] = $totPercent;
                        } else {
                            $order['redeemPoint'] = $resPoints;
                            if ($reward['value_type'] == 0) {
                                $order['rewardPoint'] = $reward['miximum_value'];
                                $order['rewardPercentage'] = '';
                            } else {
                                $order['rewardPoint'] = $reward['miximum_value'];
                                $order['rewardPercentage'] = $reward['reward_percentage'];
                            }
                        }
                    }
                }
            }
        }

        $rewardAmount = $reward['reward_amount'];
        $rewardPoints = $reward['reward_point'];

        $getRewardFromTotal = $subtotal / $rewardAmount;

        $orderPoint = $getRewardFromTotal * $rewardPoints;
        $order['order_point'] = $this->convertToDecimal((int) $orderPoint);

        $appliedOffers = [];
        $eligibleOffers = [];
        $Offers = [];
        $customerOrdersCount = TiffintomOrder::where('customer_id', $order['customer_id'])->count();

        if (isset($order['delivery_date'])) {
            $offer_date = Carbon::parse($order['delivery_date'])->format('Y-m-d');
            $day = Carbon::parse($order['delivery_date'])->format('l');
        } else {
            $offer_date = date('Y-m-d');
            $day = date('l');
        }


        foreach ($carts as $key => $cartItem) {
            //Multiple Price Offer Section
            $multiplePriceOffers = null;
            if ($order['order_type'] ==  'delivery') {
                $multiplePriceOffers = TiffintomOfferMultiplePrice::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('purchase_price', '<=', $subtotal)->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('status', '1')->orderBy('purchase_price', 'DESC')->first();
            } else if ($order['order_type'] ==  'pickup') {
                $multiplePriceOffers = TiffintomOfferMultiplePrice::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('purchase_price', '<=', $subtotal)->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('status', '1')->orderBy('purchase_price', 'DESC')->first();
            } else if ($order['order_type'] ==  'dinein') {
                $multiplePriceOffers = TiffintomOfferMultiplePrice::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('purchase_price', '<=', $subtotal)->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('status', '1')->orderBy('purchase_price', 'DESC')->first();
            }
            if (!empty($multiplePriceOffers)) {
                $offerIdExpload = explode(',', $multiplePriceOffers['offer_id']);
                foreach ($offerIdExpload as $multiOfferSubKey => $multiOfferSubValue) {
                    $Offers[$multiOfferSubKey]['quantity'] = $multiplePriceOffers['offer_qty'];
                    $Offers[$multiOfferSubKey]['menu_name'] = $multiOfferSubValue;
                }
                $eligibleOffers = $Offers;
                break;
            }

            //Day-Wise Offer Section
            $dayOffers = null;
            if ($order['order_type'] ==  'delivery') {
                $dayOffers = TiffintomOfferDay::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('day',  $day)->where('purchase_price', '<=', $subtotal)->where('status', '1')->orderBy('id', 'DESC')->first();
            } else if ($order['order_type'] ==  'pickup') {
                $dayOffers = TiffintomOfferDay::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('day',  $day)->where('purchase_price', '<=', $subtotal)->where('status', '1')->orderBy('id', 'DESC')->first();
            } else if ($order['order_type'] ==  'dinein') {
                $dayOffers = TiffintomOfferDay::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('day',  $day)->where('purchase_price', '<=', $subtotal)->where('status', '1')->orderBy('id', 'DESC')->first();
            }
            if (!empty($dayOffers)) {
                if ($dayOffers['day_offer_type'] == 0) {
                    $dayofferAmount = $dayOffers['offer_amount'];
                    $order['offer_amount'] = $dayOffers['offer_amount'];
                    $order['offer_percentage'] = 0;
                } else {
                    $dayofferAmount = $subtotal * $dayOffers['offer_amount'] / 100;
                    $order['offer_amount'] = $this->convertToDecimal($dayofferAmount);
                    $order['offer_percentage'] = $dayOffers['offer_amount'];
                }
                break;
            }

            // Day Wise Product Offer Section
            $dayProductOffer = null;
            if ($order['order_type'] ==  'delivery') {
                $dayProductOffer = TiffintomOfferDayProduct::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('day_product',  $day)->where('status', '1')->orderBy('id', 'DESC')->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $dayProductOffer = TiffintomOfferDayProduct::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('day_product',  $day)->where('status', '1')->orderBy('id', 'DESC')->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            } else if ($order['order_type'] ==  'dinein') {
                $dayProductOffer = TiffintomOfferDayProduct::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('day_product',  $day)->where('status', '1')->orderBy('id', 'DESC')->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            }
            if (!empty($dayProductOffer)) {
                $product = TiffintomMenu::where('id', $dayProductOffer['offer_id'])->first();
                $Offers['menu_id'] = $dayProductOffer['offer_id'];
                $Offers['quantity'] = $dayProductOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }

            // Day Wise Price Offer Section
            $dayPriceOffer = null;
            if ($order['order_type'] ==  'delivery') {
                $dayPriceOffer = TiffintomOfferDayPrice::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('day',  $day)->where('status', '1')->orderBy('id', 'DESC')->where('purchase_price', '<=', $subtotal)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $dayPriceOffer = TiffintomOfferDayPrice::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('day',  $day)->where('status', '1')->orderBy('id', 'DESC')->where('purchase_price', '<=', $subtotal)->first();
            } else if ($order['order_type'] ==  'dinein') {
                $dayPriceOffer = TiffintomOfferDayPrice::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('day',  $day)->where('status', '1')->orderBy('id', 'DESC')->where('purchase_price', '<=', $subtotal)->first();
            }
            if (!empty($dayPriceOffer)) {
                $product = TiffintomMenu::where('id', $dayPriceOffer['offer_id'])->first();
                $Offers['menu_id'] = $dayPriceOffer['offer_id'];
                $Offers['quantity'] = $dayPriceOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }

            //Offer Section
            $offer = null;
            if ($order['order_type'] ==  'delivery') {
                if ($customerOrdersCount == 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->where('first_user', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('free_price', '<=', $subtotal)->first();
                }
                if (($customerOrdersCount == 0 && empty($offer)) || $customerOrdersCount > 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->where('normal', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('normal_price', '<=', $subtotal)->first();
                }
            } else if ($order['order_type'] ==  'pickup') {
                if ($customerOrdersCount == 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->where('first_user', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('free_price', '<=', $subtotal)->first();
                }
                if (($customerOrdersCount == 0 && empty($offer)) || $customerOrdersCount > 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->where('normal', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('normal_price', '<=', $subtotal)->first();
                }
            } else if ($order['order_type'] ==  'dinein') {
                if ($customerOrdersCount == 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('status', '1')->where('first_user', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('free_price', '<=', $subtotal)->first();
                }
                if (($customerOrdersCount == 0 && empty($offer)) || $customerOrdersCount > 0) {
                    $offer = TiffintomOffer::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('status', '1')->where('normal', 'Y')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('normal_price', '<=', $subtotal)->first();
                }
            }
            if (!empty($offer)) {
                if ($customerOrdersCount == 0) {
                    $offersAmount = $offer['free_percentage'];
                } else {
                    $offersAmount = $offer['normal_percentage'];
                }
                if ($offer['offer_mode'] == 'flat') {
                    $offerAmount = $offersAmount;
                    $offerPercentage = 0;
                } else {
                    $offerAmount = $this->convertToDecimal($subtotal * $offersAmount / 100);
                    $offerPercentage = $offersAmount;
                }
                $order['offer_amount'] = $offerAmount;
                $order['offer_percentage'] = $offerPercentage;
                break;
            }

            //Product Offer Section
            if ($order['order_type'] ==  'delivery') {
                $productOffer = TiffintomOfferProduct::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $productOffer = TiffintomOfferProduct::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            } else if ($order['order_type'] ==  'dinein') {
                $productOffer = TiffintomOfferProduct::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('menu_id', $cartItem->menu_id)->where('product_qty', '<=', $cartItem->quantity)->first();
            }
            if (!empty($productOffer)) {
                $product = TiffintomMenu::where('id', $productOffer['offer_id'])->first();
                $Offers['menu_id'] = $productOffer['offer_id'];
                $Offers['quantity'] = $productOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }

            //Price Offer Section
            if ($order['order_type'] ==  'delivery') {
                $priceOffer = TiffintomOfferPrice::where('resid', $order['restaurant_id'])->where('delivery_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('purchase_price', '<=', $subtotal)->first();
            } else if ($order['order_type'] ==  'pickup') {
                $priceOffer = TiffintomOfferPrice::where('resid', $order['restaurant_id'])->where('pickup_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('purchase_price', '<=', $subtotal)->first();
            } else if ($order['order_type'] ==  'dinein') {
                $priceOffer = TiffintomOfferPrice::where('resid', $order['restaurant_id'])->where('dinein_type', '1')->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', $offer_date)->where('offer_to', '>=', $offer_date)->where('purchase_price', '<=', $subtotal)->first();
            }
            if (!empty($priceOffer)) {
                $product = TiffintomMenu::where('id', $priceOffer['offer_id'])->first();
                $Offers['menu_id'] = $priceOffer['offer_id'];
                $Offers['quantity'] = $priceOffer['offer_qty'];
                $Offers['menu_name'] = $product['menu_name'];
                $Offers['menu_price'] = 0;
                $Offers['total_price'] = 0;
                $appliedOffers[] = $Offers;
                break;
            }
        }

        $order['applied_offers'] = $appliedOffers;
        $order['eligible_offers'] = $eligibleOffers;

        return $order;
    }

    /**
     * Create order.
     * POST /tiffintom/order
     */
    public function reOrder(Request $req, $id)
    {

        $customer = Auth::user();

        //Other Restaurant cart
        $Excarts = TiffintomCart::where('customer_id', $customer->id)->where('order_id', 0)->get();
        if (!$Excarts->isEmpty()) $cartss = TiffintomCart::where('customer_id', $customer->id)->where('order_id', 0)->delete();

        $carts = TiffintomCart::where('order_id', $id)->where('total_price', '>', 0)->orderBy('id', 'DESC')->get();

        foreach ($carts as $cart) {
            $cartObj = new TiffintomCart;
            $cartObj->restaurant_id = $cart->restaurant_id;
            $cartObj->customer_id = $cart->customer_id;
            $cartObj->menu_id = $cart->menu_id;
            $cartObj->menu_name = $cart->menu_name;
            if ($cart->subaddons_name != '') {
                $cartObj->subaddons_name = $cart->subaddons_name;
            }
            $cartObj->menu_price = $cart->menu_price;
            $cartObj->total_price = $cart->total_price;
            $cartObj->quantity = $cart->quantity;
            $cartObj->menu_description = $cart->menu_description;
            $cartObj->save();
        }

        return ['message' => 'cart added!'];
    }

    public function orderMail($id)
    {
        $order = TiffintomOrder::find($id);
        if (empty($order)) Utils::error("order not found", 404);
        $order->append('cart_view');

        $user = TiffintomUser::find($order->customer_id);
        if (empty($user)) Utils::error("No user found", 404);

        $restaurant = TiffintomRestaurant::find($order->restaurant_id);
        if (empty($restaurant)) Utils::error("No restaurant found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $servicecharge = '';
        $offerDetails = '';
        $voucherDetails = '';
        $rewardDetails = '';
        $orderDescription = '';
        $preparation = '';

        if ($order['split_payment'] == 'Yes') {
            $paymentMethod = strtoupper(($order['payment_method'] == 'Stripe') ? 'Card' : $order['payment_method']) . ' & ' . 'Wallet';
        } else {
            $paymentMethod = strtoupper(($order['payment_method'] == 'Stripe') ? 'Card' : $order['payment_method']);
        }

        $paymentStatus = ($order['payment_status'] == 'P') ? 'Paid' : 'Not Paid';

        if ($user->role_id == 5) {
            $colorCode = '#fc353a';
            $fontCode = '#FFFFFF';
            $buttonCode = '#fc353a';
            $logoUrl = env('APP_URL') . 'images/logo-gogrubz.png';
        } else {
            $colorCode = '#f1e0c6';
            $logoUrl = env('APP_URL') . 'images/logo-home.png';
            $buttonCode = '#f3cc2c';
            $fontCode = '#000000';
        }

        if ($order['order_type'] == 'delivery') {
            $deliveryAmount = ($order['delivery_charge'] > 0) ? $siteSettings['site_currency'] . ' ' . number_format($order['delivery_charge'], 2) : 'Free';

            $deliveryCharge = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600;padding: 10px;border-top: 1px solid #eeeeee;">
                                          Delivery Fee
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $deliveryAmount . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';

            $deliveryAddress = '<tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:0px 20px 20px 20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;padding:10px;">
                           <tr>
                              <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top:15px">
                                 <span style="font-weight: 700;">Delivery Address</span>
                                 <span style="padding-top:0px;display:block">' . $order['customer_name'] . '<br>' . $order['flat_no'] . ', ' . $order['address'] . '<br>' . $order['customer_phone'] . '</span>
                              </td>
                              <td align="right" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top:15px">
                                 <span style="font-weight: 700;">Requested Delivery Time / Date</span>
                                 <span style="padding-top:0px;display:block">' . date('Y-m-d', strtotime($order['delivery_date'])) . ' ' . $order['delivery_time'] . '</span>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>';
        } else {
            $deliveryCharge = '';
            $deliveryAddress = '<tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:0px 20px 20px 20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;padding:10px;">
                           <tr>
                              <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top:15px">
                                 <span style="font-weight: 700;">Pickup Address</span>
                                 <span style="padding-top:0px;display:block">' . $restaurant['restaurant_name'] . '<br>' . $restaurant['street_address'] . '<br>' . $restaurant['restaurant_phone'] . '</span>
                              </td>
                              <td align="right" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top:15px">
                                 <span style="font-weight: 700;">Pickup Time / Date</span>
                                 <span style="padding-top:0px;display:block">' . date('Y-m-d', strtotime($order['delivery_date'])) . ' ' . $order['delivery_time'] . '</span>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>';
        }

        if ($order['offer_amount'] > 0) {
            $offerDetails = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Offer(' . $order['offer_percentage'] . ') %
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['offer_amount'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }
        if ($order['service_charge'] > 0) {
            $servicecharge = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Service Charge
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['service_charge'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }
        $tax_amount = '';
        if ($order['tax_amount'] > 0) {

            $tax_amount = ' <tr>
                          <td align="left" style="">
                             <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                <tr>
                                   <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                      Tax(' . $order['tax_percentage'] . ') %
                                   </td>
                                   <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                      ' . $siteSettings["site_currency"] . ' ' . number_format($order['tax_amount'], 2) . '
                                   </td>
                                </tr>
                             </table>
                          </td>
                       </tr>';
        }

        if ($order['voucher_amount'] > 0) {
            $voucherDetails = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Offer(' . $order['voucher_percentage'] . ') %
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['voucher_amount'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }

        if ($order['reward_used'] == 'N' && $order['reward_offer'] > 0) {
            $rewardDetails = '<tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Offer(' . $order['reward_offer_percentage'] . ') %
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['reward_offer'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>';
        }
        $cartDetails = '';

        if (!empty($order['cart_view'])) {
            foreach ($order['cart_view'] as $cKey => $cValue) {
                $menuName = $cValue['menu_name'];
                $subAddons = ($cValue['subaddons_name'] != "") ? $cValue['subaddons_name'] : "";
                $cartDetails .= '
                <tr>
                    <td width="10%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . ($cKey + 1) . '</td>
                    <td width="45%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . $menuName . ' ' . $subAddons . '</td>
                    <td width="10%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">' . $cValue['quantity'] . '</td>
                    <td width="15%" align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">
                        ' . $siteSettings["site_currency"] . ' ' . number_format($cValue['menu_price'], 2) . '
                    </td>
                    <td width="20%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; padding: 15px 10px 5px 10px;border-bottom:1px solid #eee;">
                        ' . $siteSettings["site_currency"] . ' ' . number_format($cValue['total_price'], 2) . '
                    </td>
                </tr>';
            }
        }

        if ($order['order_description'] != "") {
            $orderDescription = '<tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                           <tr>
                              <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Order Instruction : ' . $order['order_description'] . '</span>
                                 
                              </td>                              
                           </tr>
                        </table>
                     </td>
                  </tr>';
        }
        $statuss =  '';
        $comme = '';
        $statusss = '';
        if ($order['status'] == "Accepted") {
            $statuss .=  '  <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                           <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                    Thank you. your <span style="color:#000;font-weight:bold;">order ID ' . $order['order_number'] . '</span> has been Accepted.
                                    <br>
                                    Those lovely people should start cooking your food right about ... now.
                                 </p>
                              </td>
                           </tr>';
            $customerSubject = 'Order Accepted - ' . $order['order_number'];
            $preparation .=  '<span style="font-weight: 700; display:block;">Expected Delivery Time : ' . $order['preparation'] . '</span>';

            if ($order['order_type'] == "delivery") {
                $comme .= '<tr>
                            <td align="center" height="100%" valign="top" width="100%" style="padding:0px 0px 0px 0px;background-color:#fff">
                                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;">
                                   <tr>
                                      <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 14px; font-weight: 400; line-height: 20px; padding-top:5px">
                                            <b>Comments: </b> The order is to be delivered AS SOON AS POSSIBLE on ' . date('Y-m-d', strtotime($order['delivery_date'])) . ' Please note that changes in delivery time can occur. Any changes can be seen by <a href="#">clicking here</a>.
                   
                                      </td>
                                    </tr>
                                </table>
                             </td>
                        </tr>';
            } else {


                $comme .= '<tr>
                            <td align="center" height="100%" valign="top" width="100%" style="padding:0px 0px 0px 0px;background-color:#fff">
                                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;">
                                   <tr>
                                      <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 14px; font-weight: 400; line-height: 20px; padding-top:5px">
                                            <b>Comments: </b>  The order is to be Prepared AS SOON AS POSSIBLE on ' . date('Y-m-d', strtotime($order['delivery_date'])) . ' Please note that changes in delivery time can occur. Any changes can be seen by <a href="#">clicking here</a>.
                                         </td>
                                    </tr>
                                </table>
                             </td>
                        </tr>';
            }
        } else if ($order['status'] == "Delivered" && $order['order_type'] == "delivery") {
            $statuss .= '   
                            <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                            <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                         Thank you for Order.Order number ' . $order['order_number'] . ' has been Delivered.
                                 </p>
                              </td>
                           </tr>';
            $customerSubject = 'Order Delivered - ' . $order['order_number'];
        } else if ($order['status'] == "Failed") {
            $statuss .= '   <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                         We are sorry for the inconvience caused,your Order ' . $order['order_number'] . ' has been Failed for the reason ' . $order['failed_reason'] . '.
                                 </p>
                              </td>
                           </tr>';
            $customerSubject = 'Order Reject - ' . $order['order_number'];

            $statusss .= ' <tr>
                             <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                                   <tr>
                                      <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                         <span style="font-weight: 700; display:block;">Failed Reason : ' . $order['failed_reason'] . '</span>
                                         
                                      </td>                              
                                   </tr>
                                </table>
                             </td>
                          </tr> ';
        } else {
            if ($order['status'] == "Pending") {
                $customerSubject = 'Order Placed Successful - ' . $order['order_number'];

                $statuss = '<tr>
                <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                   <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                      <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                      Thank you for your order with us.Your order details below for the order ID <span style="color:#000;font-weight:bold;">order ID ' . $order['order_number'] . '</span>, We gladly confirm your online order for Collection posted to restaurant. Please wait for the restaurant to confirm if they will accept or deny the order.
                   </p>
                </td>
             </tr>';
            }
        }

        if ($order['status'] == "Delivered" && $order['order_type'] == "pickup") {
            $statuss .= '   
                            <tr>
                              <td align="center" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <img src="images/tick.png" width="100" style="display: block; border: 0px;" />
                                 <h2 style="font-size: 28px; font-weight: 800; line-height: 36px; color: #333333; margin: 0;">
                                    Thank You For Your Order!
                                 </h2>
                              </td>
                           </tr>
                            <tr>
                              <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                 <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                    <span style="display:block;">Dear <strong style=" color:#000;">' . $order['customer_name'] . '</strong></span>
                                         Thank you for order. Order number ' . $order['order_number'] . ' has been Collected.
                                 </p>
                              </td>
                           </tr>';



            $customerSubject = 'Order Collected - ' . $order['order_number'];
        }
        $html = '
         <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
               <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                  <tr>
                     <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="' . $colorCode . '">
                        <div style="display:inline-block; vertical-align:middle; text-align:center">
                           <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                              <tr>
                                 <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                    <img width="170" src="' . $logoUrl . '">
                                 </td>
                              </tr>
                           </table>
                        </div>
                     </td>
                  </tr>
                  <tr>
                     <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                           ' . $statuss . '
                           
                           <tr>
                              <td align="left" style="padding-top: 20px;">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                   <td  width="10%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">S.No</td>
                                   <td width="45%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">Menu Name</td>
                                   <td width="10%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 700; line-height: 24px; padding: 10px;">Qty</td>
                                   <td width="15%" align="left" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">
                                      Price
                                   </td>
                                   <td width="20%" align="right" bgcolor="#eeeeee" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 700; line-height: 24px; padding: 10px;">
                                      Total Price
                                   </td>
                                </tr>
                                    ' . $cartDetails . '                                 
                                 </table>
                              </td>
                           </tr>   
                                 
                           
                           <tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          Sub Total
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 15px; font-weight: 600; line-height: 24px; padding: 10px;border-top: 1px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['order_sub_total'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>
                           ' . $tax_amount . '
                           ' . $deliveryCharge . '
                           ' . $servicecharge . '
                           ' . $offerDetails . '
                           ' . $voucherDetails . '
                           ' . $rewardDetails . '

                           <tr>
                              <td align="left" style="">
                                 <table cellspacing="0" cellpadding="0" border="0" width="100%">
                                    <tr>
                                       <td width="75%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 600; line-height: 24px; padding: 10px; border-top: 3px solid #eeeeee; border-bottom: 3px solid #eeeeee;">
                                          TOTAL
                                       </td>
                                       <td width="25%" align="right" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 600; line-height: 24px; padding: 10px; border-top: 3px solid #eeeeee; border-bottom: 3px solid #eeeeee;">
                                          ' . $siteSettings["site_currency"] . ' ' . number_format($order['order_grand_total'], 2) . '
                                       </td>
                                    </tr>
                                 </table>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>
                  <tr>
                     <td align="center" height="100%" valign="top" width="100%" style="padding:20px;background-color:#fff">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="width:670px;background:#eee;padding:10px;">
                           <tr>
                              <td align="left" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 ' . $preparation . '
                                 <span style="font-weight: 700; display:block;">Order Number/ID : ' . $order['order_number'] . '</span>
                                 <span style="font-weight: 700; display:block;">Payment method : ' . $paymentMethod . '</span>
                              </td>
                              <td align="right" width="50%" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                                 <span style="font-weight: 700; display:block;">Order type : ' . ucfirst($order['order_type']) . '</span>
                                 <span style="font-weight: 700; display:block;">Payment status : ' . $paymentStatus . ' </span>
                              </td>
                           </tr>
                        </table>
                     </td>
                  </tr>
                  ' . $comme . '
                  
                  ' . $deliveryAddress . '
                  
                  ' . $orderDescription . '
                  
                  ' . $statusss;
        if ($user->role_id != 5) {
            $html .= '<tr>   
                        <td bgcolor="#fff" >
                            <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                                <tr>
                                    <td style="padding: 30px 30px; text-align: right;">
                                            <table align="left" style="text-align: left;">
                                                <tr>
                                                    <td>
                                                            <p style="color:#000;"> Get The App : </p> 
                                                    </td>
                                                        <td width="10">&nbsp;</td>
                                                    <td>
                                                        <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                        <img src="' . env('APP_URL') . 'images/playstore.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                    </td>
                                                        <td width="10">&nbsp;</td>
                                                    <td>
                                                        <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                        <img src="' . env('APP_URL') . 'images/appstore.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                    </td>
                                                        <td width="10">&nbsp;</td>
                                                </tr>
                                                
                                            </table>
                                            <table align="right" style="text-align: right;">
                                            <tr>
                                                <td>
                                                        <p style="color:#000;">Follow Us : </p>
                                                </td>
                                                    <td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/facebook.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td>
                                                    <td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/twitter.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td>
                                                    <td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/linkedin.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td><td width="10">&nbsp;</td>
                                                <td>
                                                    <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/Instragram.png" width="25" height="25" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                                </td><td width="10">&nbsp;</td>
                                            </tr>
                                        </table>
                                    </td>
                                </tr>
                            </table>
                        </td>
                    </tr>';
        }
        $html .= '<tr>
                    <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="' . $colorCode . '"><div style="color:' . $fontCode . ';font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
                </tr>
               </table>
            </td>
         </tr>';

        // User side mail
        if ($user->email_verify == 1) {
            if ($order->status == 'Pending' && ($user->order_place == 7 || $user->order_place == 4 || $user->order_place == 5 || $user->order_place == 6)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }

            if ($order->status == 'Accepted' && ($user->order_accept == 4 || $user->order_accept == 5 || $user->order_accept == 6 || $user->order_accept == 7)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }
            if ($order->status == 'Delivered' && ($user->order_delivered == 4 || $user->order_delivered == 5 || $user->order_delivered == 6 || $user->order_delivered == 7)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }
            if ($order->status == 'Failed' && ($user->order_reject == 4 || $user->order_reject == 5 || $user->order_reject == 6 || $user->order_reject == 7)) {
                $email['body'] = $html;
                $email['subject'] = $customerSubject;
                $email['mail_type'] = $siteSettings->mail_option;
                $email['to'] = $order->customer_email;
                $email['from'] = $siteSettings->admin_email;
                (new MailHelper())->sendMail($email);
            }
        }

        // Restaurant side Mail
        $email['body'] = $html;
        $email['subject'] = $customerSubject;
        $email['mail_type'] = $siteSettings->mail_option;
        $email['to'] = $restaurant->contact_email;
        $email['from'] = $order->customer_email;
        (new MailHelper())->sendMail($email);

        // Admin side Mail with permission
        if ($siteSettings->admin_order_place_email == 'Yes' && $order['status'] == "Pending") {
            $email['body'] = $html;
            $email['subject'] = $customerSubject;
            $email['mail_type'] = $siteSettings->mail_option;
            if ($user->role_id != 5) {
                $email['to'] = $siteSettings->gogrubz_order_email;
            } else {
                $email['to'] = $siteSettings->order_email;
            }
            $email['from'] = $order->customer_email;
            (new MailHelper())->sendMail($email);
        }
        if ($siteSettings->admin_order_accept_email == 'Yes' && $order['status'] == "Accepted") {
            $email['body'] = $html;
            $email['subject'] = $customerSubject;
            $email['mail_type'] = $siteSettings->mail_option;
            if ($user->role_id != 5) {
                $email['to'] = $siteSettings->gogrubz_order_email;
            } else {
                $email['to'] = $siteSettings->order_email;
            }
            $email['from'] = $order->customer_email;
            (new MailHelper())->sendMail($email);
        }
        if ($siteSettings->admin_order_reject_email == 'Yes' && $order['status'] == "Failed") {
            $email['body'] = $html;
            $email['subject'] = $customerSubject;
            $email['mail_type'] = $siteSettings->mail_option;
            if ($user->role_id != 5) {
                $email['to'] = $siteSettings->gogrubz_order_email;
            } else {
                $email['to'] = $siteSettings->order_email;
            }
            $email['from'] = $order->customer_email;
            (new MailHelper())->sendMail($email);
        }

        return 'Mail Send Successfully';
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}