<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomOrderHelp;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class TiffintomOrderHelpController extends Controller
{


    /**
     * List order-help
     * GET /tiffintom/order-help
     */
    public function index(Request $req)
    {

        $shallAdminReadAll = true;
        $restaurant_id = $req->input('restaurant_id');
        if ($user_group = $req->query('user_group', false)) {
            $shallAdminReadAll = false;
            $order_help = TiffintomOrderHelp::orderBy('id', 'DESC');
            $order_help->join(DB::raw('(SELECT customer_id, MAX(created) as latest FROM orderhelp  WHERE restaurant_id=' . $restaurant_id . ' GROUP BY customer_id) as r'), 'orderhelp.created', '=', 'r.latest');
        } else {
            if ($restaurant_group = $req->query('restaurant_group', false)) {
                $order_help = TiffintomOrderHelp::orderBy('id', 'DESC');
            } else {
                $order_help = TiffintomOrderHelp::orderBy('id', 'ASC');
            }
        }

        if ($restaurant_group = $req->query('restaurant_group', false)) {
            $customer_id = $req->input('customer_id');
            $order_help->join(DB::raw('(SELECT restaurant_id, MAX(created) as latest FROM orderhelp where customer_id=' . $customer_id . ' GROUP BY restaurant_id) as r'), 'orderhelp.created', '=', 'r.latest');
        }

        if ($admin_group = $req->query('admin_group', false)) {
            $order_help->join(DB::raw('(SELECT restaurant_id, MAX(created) as latest FROM orderhelp GROUP BY customer_id,restaurant_id) as r'), 'orderhelp.created', '=', 'r.latest');
        }

        if ($req->query('restaurant_id', false) && $req->input('admin_groups') != '') {
            $restaurant_id = $req->input('restaurant_id');
            $order_help->where('restaurant_id', $restaurant_id);
        }

        if ($order_id = $req->query('order_id', false)) {
            $order_help->where('order_id', $order_id);
        }

        if ($req->query('restaurant_id', false) && $req->input('admin_groups') == '') {
            // User wise check
            // if ($req->hasHeader('business-id')) {
            //     $business = Business::where('restaurant_id', $restaurant_id)->first();
            //     if (empty($business)) Utils::error('User not authenticated.', 404);
            //     (new ExtraController())->userCheck($req->header('business-id'), $business->id);
            // }
            $order_help->where('restaurant_id', $restaurant_id);
            // $business = Business::where('restaurant_id', $restaurant_id)->first();
            // if (empty($business)) Utils::error('User not authenticated.', 404);
            // (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        if ($user_id = $req->query('user_id', false)) {
            $order_help->where('customer_id', $user_id);
        }

        if ($customer_id = $req->query('customer_id', false)) {
            $order_help->where('customer_id', $customer_id);
        }

        if ($admin = $req->query('admin', false)) {
            $order_help->where('admin', 1);
        }

        if ($last_id = $req->query('last_id', false)) {
            $order_help->where('id', '>', $last_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $order_help = $order_help->get();
        } else {
            $order_help = $order_help->paginate($req->query('per_page', 20));
        }

        if ($order_help->isEmpty()) Utils::error('No order helps found!', 404);

        //Mark Messages as seen
        if (Auth::isTiffintomUser() && $req->query('restaurant_id', false) && (!Auth::isEposUser() || !Auth::isEposAdmin() || !Auth::isTiffintomAdmin())) {
            //Read all admin messages
            TiffintomOrderHelp::where('restaurant_id', $req->query('restaurant_id'))->where('customer_id', $req->query('customer_id'))->where('admin', '1')->update(['status' => '0']);
        }
        if ($shallAdminReadAll && $req->query('customer_id', false)) { // && (Auth::isEposUser() || Auth::isEposAdmin() || Auth::isTiffintomAdmin()) 
            //Read All User messages
            TiffintomOrderHelp::where('restaurant_id', $req->query('restaurant_id'))->where('customer_id', $req->query('customer_id'))->where('admin', '0')->update(['status' => '0']);
        }

        //Mark Messages as seen
        if (Auth::isTiffintomUser() && $req->query('restaurant_id', false) && (!Auth::isEposUser() || !Auth::isEposAdmin() || !Auth::isTiffintomAdmin())) {
            //Read all admin messages
            TiffintomOrderHelp::where('restaurant_id', $req->query('restaurant_id'))->where('customer_id', $req->query('user_id'))->where('admin', '1')->update(['status' => '0']);
        }
        if ($shallAdminReadAll && $req->query('user_id', false)) { // && (Auth::isEposUser() || Auth::isEposAdmin() || Auth::isTiffintomAdmin()) 
            //Read All User messages
            TiffintomOrderHelp::where('restaurant_id', $req->query('restaurant_id'))->where('customer_id', $req->query('user_id'))->where('admin', '0')->update(['status' => '0']);
        }
        return $order_help;
    }

    /**
     * Create order-help.
     * POST /tiffintom/order-help
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'required|exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'admin' => 'string',
            'message' => 'string',
            'status' => 'string',
            'is_attach' => 'string',
            'attach' => 'mimes:jpeg,png,jpg,mp3,m4a,mp4',
        ]);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        // create order-help
        $order_help = new TiffintomOrderHelp;
        $order_help->fill($req->all());
        if ($req->has('attach')) {
            $fileName = ImageHelper::saveImageToServer($req->file('attach'), ImageHelper::$TYPE_TIFFINTOM_ORDERHELP, 'tiffintom');
            $order_help->message = $fileName;
        }
        $order_help->status = 1;
        $order_help->save();

        $user = TiffintomUser::find($order_help->customer_id);

        $restaurant = TiffintomRestaurant::find($order_help->restaurant_id);

        if ($req->input('admin') == 0 && !empty($restaurant)) {
            $message      = 'You have a new message from ' . $user->first_name;
            $notification['title']          = "Order Help message";
            $notification['body']        = $message;

            $notificationdata['data']['title']          = "Order Help message";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['order_id']  = $order_help->order_id;
            $notificationdata['data']['restaurant_id'] = $order_help->restaurant_id;
            $notificationdata['data']['payload']        = array('OrderhelpDetails' => "", 'type' => "restaurantchat");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            $Devices = explode(',', $restaurant->device_id);
            foreach ($Devices as $Device) {
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device, $notification, 'restaurant');
            }
        }

        if ($req->input('admin') == 1 && !empty($restaurant)) {
            $message      = 'You have a new message from ' . $restaurant->restaurant_name;
            $notification['title']          = "Order Help message";
            $notification['body']        = $message;

            $notificationdata['data']['title']          = "Order Help message";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['order_id']  = $order_help->order_id;
            $notificationdata['data']['restaurant_id'] = $order_help->restaurant_id;
            $notificationdata['data']['payload']        = array('OrderhelpDetails' => "", 'type' => "restaurantchat");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $user->device_id, $notification, 'customer');
        }

        return $order_help;
    }

    /**
     * List order-help
     * GET /tiffintom/order-help/id
     */
    public function show(Request $req, $id)
    {
        $order_help = TiffintomOrderHelp::where('id', $id);
        if (empty($order_help)) Utils::error("No order_help found", 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order_help->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        return $order_help;
    }

    /**
     * Update order-help.
     * POST  /tiffintom/order-help/id
     */
    public function update(Request $req, $id)
    {
        $order_help = TiffintomOrderHelp::find($id);
        if (empty($order_help)) Utils::error('order_help not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order_help->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'order_id' => 'required|exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'admin' => 'string',
            'message' => 'string',
            'status' => 'string',
            'is_attach' => 'string',
        ]);

        // update menu
        if ($req->has('customer_id')) $order_help->customer_id = $req->input('customer_id');
        if ($req->has('order_id')) $order_help->order_id = $req->input('order_id');
        if ($req->has('restaurant_id')) $order_help->restaurant_id = $req->input('restaurant_id');
        if ($req->has('message')) $order_help->message = $req->input('message');
        $order_help->admin = $req->input('admin');
        $order_help->status = $req->input('status');
        $order_help->is_attach = $req->input('is_attach');

        $order_help->save();
        return $order_help;
    }

    /**
     * Delete order-help by id.
     * DELETE /tiffintom/order-help/{id}
     */
    public function delete(Request $req, $id)
    {
        $order_help = TiffintomOrderHelp::find($id);
        if (empty($order_help)) Utils::error('order_help not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $order_help->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $order_help->delete();

        return $order_help;
    }

    /**
     * Delete order-help by id.
     * DELETE /tiffintom/order-help/{id}/delete-user
     */
    public function delete_user(Request $req, $id)
    {
        $order_helps = TiffintomOrderHelp::where('customer_id', $id)->delete();
        return ['message' => 'order help deleted successfully!'];
    }
}