<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomPartnerMedia;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomPartnerMediaController extends Controller
{

    /**
     * List partner-medias
     * GET /tiffintom/partner-medias
     */
    public function index(Request $req)
    {
        $partner_medias = TiffintomPartnerMedia::selectRaw("partner_media.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('id', 'DESC');

        if ($restaurant_id = $req->query('restaurant_id', false)) {
            $partner_medias->where('restaurant_id', $restaurant_id);
        }

        if ($status = $req->query('status', false)) {
            $partner_medias->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $partner_medias = $partner_medias->get();
        } else {
            $partner_medias = $partner_medias->paginate($req->query('per_page', 20));
        }

        if ($partner_medias->isEmpty()) Utils::error('No partner messages found!', 404);

        return $partner_medias;
    }


    /**
     * Create partner-medias.
     * POST /tiffintom/partner-medias
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'string',
            'title' => 'required|string',
            'type' => 'in:0,1',
            'url' => 'required_if:type,1',
            'all' => 'boolean',
        ]);

        // create partner-medias
        $partner_media = new TiffintomPartnerMedia;
        $partner_media->fill($req->all());
        if ($req->has('all')) $partner_media->restaurant_id = 'all';
        $partner_media->status = '1';
        if ($req->has('image') && $req->input('type') == '0') {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PARTNERMEDIA, 'tiffintom');
            $partner_media->url = $fileName;
        }
        $partner_media->save();

        return $partner_media;
    }

    /**
     * List partner-medias
     * GET /tiffintom/partner-medias/id
     */
    public function show(Request $req, $id)
    {
        $partner_medias = TiffintomPartnerMedia::find($id);
        if (empty($partner_medias)) Utils::error("Partner Media not found", 404);
        return $partner_medias;
    }

    /**
     * Update payment method.
     * POST  /tiffintom/partner-medias/id
     */
    public function update(Request $req, $id)
    {
        $partner_media = TiffintomPartnerMedia::find($id);
        if (empty($partner_media)) Utils::error('Partner Media not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'string',
            'title' => 'required|string',
            'type' => 'in:0,1',
            'url' => 'required_if:type,1',
            'all' => 'boolean',
        ]);

        // update partner-medias
        if ($req->has('restaurant_id')) $partner_media->restaurant_id = $req->input('restaurant_id');
        if ($req->has('all')) $partner_media->restaurant_id = 'all';
        if ($req->has('title')) $partner_media->title = $req->input('title');
        $partner_media->type = $req->input('type');

        if ($req->has('image') && $req->input('type') == '0') {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PARTNERMEDIA, 'tiffintom');
            $partner_media->url = $fileName;
        }
        if ($req->has('url') && $req->input('type') == '1') $partner_media->url = $req->input('url');

        $partner_media->save();

        return $partner_media;
    }

    /**
     * Delete partner-medias by id.
     * DELETE /tiffintom/partner-medias/{id}
     */
    public function delete(Request $req, $id)
    {
        $partner_media = TiffintomPartnerMedia::find($id);
        if (empty($partner_media)) Utils::error('Partner Media not found!', 404);
        $partner_media->delete();
        return $partner_media;
    }

    /**
     * Enable partner-medias.
     * POST /tiffintom/partner-medias/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $partner_media = TiffintomPartnerMedia::find($id);
        if (empty($partner_media)) Utils::error('Partner Media not found!', 404);
        $partner_media->status = '1';
        $partner_media->save();
        return $partner_media;
    }

    /**
     * Disable partner-medias.
     * POST /tiffintom/partner-medias/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $partner_media = TiffintomPartnerMedia::find($id);
        if (empty($partner_media)) Utils::error('Partner Media not found!', 404);
        $partner_media->status = '0';
        $partner_media->save();
        return $partner_media;
    }
}
