<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomPartnerMessage;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomPartnerMessageController extends Controller
{

    /**
     * List partner-message
     * GET /tiffintom/partner-message
     */
    public function index(Request $req)
    {
        $partner_messages = TiffintomPartnerMessage::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $partner_messages->whereRaw('(message LIKE ? OR font_color LIKE ?  OR bg_color LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($status = $req->query('status', false)) {
            $partner_messages->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $partner_messages = $partner_messages->get();
        } else {
            $partner_messages = $partner_messages->paginate($req->query('per_page', 20));
        }

        if ($partner_messages->isEmpty()) Utils::error('No partner messages found!', 404);

        return $partner_messages;
    }


    /**
     * Create partner-message.
     * POST /tiffintom/partner-messages
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'message' => 'required|string',
            'font_color' => 'required|string',
            'bg_color' => 'required|string',
        ]);

        // create partner-message
        $partner_message = new TiffintomPartnerMessage;
        $partner_message->fill($req->all());
        $partner_message->status = '1';
        $partner_message->save();
        return $partner_message;
    }

    /**
     * List partner-messages
     * GET /tiffintom/partner-messages/id
     */
    public function show(Request $req, $id)
    {
        $partner_message = TiffintomPartnerMessage::find($id);
        if (empty($partner_message)) Utils::error("Partner message not found", 404);
        return $partner_message;
    }

    /**
     * Update partner-message.
     * POST  /tiffintom/partner-messages/id
     */
    public function update(Request $req, $id)
    {
        $partner_message = TiffintomPartnerMessage::find($id);
        if (empty($partner_message)) Utils::error('Partner message not found!', 404);

        $this->validate($req, [
            'message' => 'required|string',
            'font_color' => 'required|string',
            'bg_color' => 'required|string',
        ]);

        // update partner-message
        if ($req->has('message')) $partner_message->message = $req->input('message');
        if ($req->has('font_color')) $partner_message->font_color = $req->input('font_color');
        if ($req->has('bg_color')) $partner_message->bg_color = $req->input('bg_color');
        $partner_message->save();

        return $partner_message;
    }

    /**
     * Delete partner-message by id.
     * DELETE /tiffintom/partner-messages/{id}
     */
    public function delete(Request $req, $id)
    {
        $partner_message = TiffintomPartnerMessage::find($id);
        if (empty($partner_message)) Utils::error('Partner message not found!', 404);
        $partner_message->delete();

        return $partner_message;
    }

    /**
     * Enable partner-message.
     * POST /tiffintom/partner-messages/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $partner_message = TiffintomPartnerMessage::find($id);
        if (empty($partner_message)) Utils::error('Partner message not found!', 404);
        $partner_message->status = '1';
        $partner_message->save();

        return $partner_message;
    }

    /**
     * Disable partner-message.
     * POST /tiffintom/partner-messages/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $partner_message = TiffintomPartnerMessage::find($id);
        if (empty($partner_message)) Utils::error('Partner message not found!', 404);
        $partner_message->status = '0';
        $partner_message->save();

        return $partner_message;
    }
}