<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomPaymentMethod;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomPaymentMethodController extends Controller
{
    /**
     * List payment-methods
     * GET /tiffintom/payment-methods
     */
    public function index(Request $req)
    {
        $payment_methods = TiffintomPaymentMethod::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $payment_methods->whereRaw('(payment_method_name LIKE ?)', ["%$query%"]);
        }

        if ($status = $req->query("status", false)) {
            $payment_methods->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $payment_methods = $payment_methods->get();
        } else {
            $payment_methods = $payment_methods->paginate($req->query('per_page', 20));
        }

        if ($payment_methods->isEmpty()) Utils::error('Payment Methods not found!', 404);

        return $payment_methods;
    }

    /**
     * Create payment-method.
     * POST /tiffintom/payment-methods
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'payment_method_name' => 'required|string',
            'payment_method_image' => 'image',
        ]);

        // create payment-method
        $payment_method = new TiffintomPaymentMethod;
        $payment_method->fill($req->all());

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PAYMENT_METHOD, 'tiffintom');
            $payment_method->payment_method_image = $fileName;
        }

        $payment_method->save();
        return $payment_method;
    }

    /**
     * List payment-methods
     * GET /tiffintom/payment-methods/id
     */
    public function show(Request $req, $id)
    {
        $payment_methods = TiffintomPaymentMethod::find($id);
        if (empty($payment_methods)) Utils::error("Payment Method not found", 404);
        return $payment_methods;
    }

    /**
     * Update payment method.
     * POST  /tiffintom/payment-methods/id
     */
    public function update(Request $req, $id)
    {
        $payment_method = TiffintomPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('Payment Method not found!', 404);

        $this->validate($req, [
            'payment_method_name' => 'required|string',
            'payment_method_image' => 'image',
        ]);

        // update payment-method
        if ($req->has('payment_method_name')) $payment_method->payment_method_name = $req->input('payment_method_name');

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PAYMENT_METHOD, 'tiffintom');
            $payment_method->payment_method_image = $fileName;
        }

        $payment_method->save();

        return $payment_method;
    }

    /**
     * Delete payment-method by id.
     * DELETE /tiffintom/payment-methods/{id}
     */
    public function delete(Request $req, $id)
    {
        $payment_method = TiffintomPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('Payment Method not found!', 404);
        $payment_method->delete();
        return $payment_method;
    }

    /**
     * Enable payment-method.
     * POST /tiffintom/payment-methods/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $payment_method = TiffintomPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('Payment Method not found!', 404);
        $payment_method->status = '1';
        $payment_method->save();
        return $payment_method;
    }

    /**
     * Disable payment-method.
     * POST /tiffintom/payment-methods/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $payment_method = TiffintomPaymentMethod::find($id);
        if (empty($payment_method)) Utils::error('Payment Method not found!', 404);
        $payment_method->status = '0';
        $payment_method->save();
        return $payment_method;
    }
}
