<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomPaymentPlan;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomPaymentPlanController extends Controller
{

    /**
     * List payment-plan
     * GET /tiffintom/payment-plans
     */
    public function index(Request $req)
    {
        $payment_plans = TiffintomPaymentPlan::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $payment_plans->whereRaw('(plan_name LIKE ? OR stripe_mode LIKE ?  OR paypal_mode LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($status = $req->query('status', false)) {
            $payment_plans->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $payment_plans = $payment_plans->get();
        } else {
            $payment_plans = $payment_plans->paginate($req->query('per_page', 20));
        }

        if ($payment_plans->isEmpty()) Utils::error('No payment plans found!', 404);

        return $payment_plans;
    }

    /**
     * Create payment-plan.
     * POST /tiffintom/payment-plans
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'plan_name' => 'required|string',
            'stripe_mode' => 'required|in:Live,Test',
            'stripe_secretkey' => 'required_if:stripe_mode,Live',
            'stripe_publishkey' => 'required_if:stripe_mode,Live',
            'stripe_secretkeyTest' => 'required_if:stripe_mode,Test',
            'stripe_publishkeyTest' => 'required_if:stripe_mode,Test',
            'paypal_mode' => 'required|in:Live,Test',
            'live_clientid' => 'required_if:paypal_mode,Live',
            'live_secretkey' => 'required_if:paypal_mode,Live',
            'test_clientid' => 'required_if:paypal_mode,Test',
            'test_secretkey' => 'required_if:paypal_mode,Test',
        ]);

        // create payment-plan
        $payment_plan = new TiffintomPaymentPlan;
        $payment_plan->fill($req->all());
        $payment_plan->status = '1';
        $payment_plan->save();

        return $payment_plan;
    }

    /**
     * List payment-plans
     * GET /tiffintom/payment-plans/id
     */
    public function show(Request $req, $id)
    {
        $payment_plan = TiffintomPaymentPlan::find($id);
        if (empty($payment_plan)) Utils::error("Payment Plan not found", 404);
        return $payment_plan;
    }

    /**
     * Update payment-plan.
     * POST  /tiffintom/payment-plans/id
     */
    public function update(Request $req, $id)
    {
        $payment_plan = TiffintomPaymentPlan::find($id);
        if (empty($payment_plan)) Utils::error('Payment Plan not found!', 404);

        $this->validate($req, [
            'plan_name' => 'required|string',
            'stripe_mode' => 'required|in:Live,Test',
            'stripe_secretkey' => 'required_if:stripe_mode,Live',
            'stripe_publishkey' => 'required_if:stripe_mode,Live',
            'stripe_secretkeyTest' => 'required_if:stripe_mode,Test',
            'stripe_publishkeyTest' => 'required_if:stripe_mode,Test',
            'paypal_mode' => 'required|in:Live,Test',
            'live_clientid' => 'required_if:paypal_mode,Live',
            'live_secretkey' => 'required_if:paypal_mode,Live',
            'test_clientid' => 'required_if:paypal_mode,Test',
            'test_secretkey' => 'required_if:paypal_mode,Test',
        ]);

        // update payment-plan
        if ($req->has('plan_name')) $payment_plan->plan_name = $req->input('plan_name');
        if ($req->has('stripe_mode')) $payment_plan->stripe_mode = $req->input('stripe_mode');
        if ($req->has('stripe_secretkey')) $payment_plan->stripe_secretkey = $req->input('stripe_secretkey');
        if ($req->has('stripe_publishkey')) $payment_plan->stripe_publishkey = $req->input('stripe_publishkey');
        if ($req->has('stripe_secretkeyTest')) $payment_plan->stripe_secretkeyTest = $req->input('stripe_secretkeyTest');
        if ($req->has('stripe_publishkeyTest')) $payment_plan->stripe_publishkeyTest = $req->input('stripe_publishkeyTest');
        if ($req->has('paypal_mode')) $payment_plan->paypal_mode = $req->input('paypal_mode');
        if ($req->has('test_clientid')) $payment_plan->test_clientid = $req->input('test_clientid');
        if ($req->has('test_secretkey')) $payment_plan->test_secretkey = $req->input('test_secretkey');
        if ($req->has('live_clientid')) $payment_plan->live_clientid = $req->input('live_clientid');
        if ($req->has('live_secretkey')) $payment_plan->live_secretkey = $req->input('live_secretkey');

        $payment_plan->save();

        return $payment_plan;
    }

    /**
     * Delete payment-plan by id.
     * DELETE /tiffintom/payment-plans/{id}
     */
    public function delete(Request $req, $id)
    {
        $payment_plan = TiffintomPaymentPlan::find($id);
        if (empty($payment_plan)) Utils::error('Payment Plan not found!', 404);
        $payment_plan->delete();

        return $payment_plan;
    }

    /**
     * Enable payment-plan.
     * POST /tiffintom/payment-plans/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $payment_plan = TiffintomPaymentPlan::find($id);
        if (empty($payment_plan)) Utils::error('Payment Plan not found!', 404);
        $payment_plan->status = '1';
        $payment_plan->save();

        return $payment_plan;
    }

    /**
     * Disable payment-plan.
     * POST /tiffintom/payment-plans/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $payment_plan = TiffintomPaymentPlan::find($id);
        if (empty($payment_plan)) Utils::error('Payment Plan not found!', 404);
        $payment_plan->status = '0';
        $payment_plan->save();

        return $payment_plan;
    }
}