<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomPreMessages;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomPreMessagesController extends Controller
{
    /**
     * List pre-messages
     * GET /tiffintom/pre-messages
     */
    public function index(Request $req)
    {
        $pre_messages = TiffintomPreMessages::orderBy('id', 'DESC');

        if ($query = $req->query('query', false)) {
            $pre_messages->whereRaw('(premessage LIKE ?)', ["%$query%"]);
        }

        if ($status = $req->query("status", false)) {
            $pre_messages->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $pre_messages = $pre_messages->get();
        } else {
            $pre_messages = $pre_messages->paginate($req->query('per_page', 20));
        }

        if ($pre_messages->isEmpty()) Utils::error('pre-messages not found!', 404);

        return $pre_messages;
    }


    /**
     * Create pre-message.
     * POST /tiffintom/pre-messages
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'premessage' => 'required|string',
            'status' => 'boolean',
        ]);

        $pre_messages = new TiffintomPreMessages;
        $pre_messages->fill($req->all());
        $pre_messages->status = '1';
        $pre_messages->save();

        return $pre_messages;
    }

    /**
     * List pre-messages
     * GET /tiffintom/pre-messages/id
     */
    public function show(Request $req, $id)
    {
        $pre_messages = TiffintomPreMessages::find($id);
        if (empty($pre_messages)) Utils::error("pre-message not found", 404);
        return $pre_messages;
    }

    /**
     * Update pre-messages.
     * POST  /tiffintom/pre-messages/id
     */
    public function update(Request $req, $id)
    {
        $pre_messages = TiffintomPreMessages::find($id);
        if (empty($pre_messages)) Utils::error('pre-message not found!', 404);

        $this->validate($req, [
            'premessage' => 'required|string',
            'status' => 'boolean',
        ]);

        // update pre_messages
        if ($req->has('premessage')) $pre_messages->premessage = $req->input('premessage');
        $pre_messages->save();

        return $pre_messages;
    }

    /**
     * Delete pre-messages by id.
     * DELETE /tiffintom/pre-messages/{id}
     */
    public function delete(Request $req, $id)
    {
        $pre_messages = TiffintomPreMessages::find($id);
        if (empty($pre_messages)) Utils::error('pre-message not found!', 404);
        $pre_messages->delete();
        return $pre_messages;
    }

    /**
     * Enable pre-messages.
     * POST /tiffintom/pre-messages/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $pre_messages = TiffintomPreMessages::find($id);
        if (empty($pre_messages)) Utils::error('pre-messages not found!', 404);
        $pre_messages->status = '1';
        $pre_messages->save();
        return $pre_messages;
    }

    /**
     * Disable pre-messages.
     * POST /tiffintom/pre-messages/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $pre_messages = TiffintomPreMessages::find($id);
        if (empty($pre_messages)) Utils::error('pre-messages not found!', 404);
        $pre_messages->status = '0';
        $pre_messages->save();
        return $pre_messages;
    }
}