<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomPromotionBanner;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomPromotionBannerController extends Controller
{

    /**
     * List promotion-banners
     * GET /tiffintom/promotion-banners
     */
    public function index(Request $req)
    {
        $promotion_banners = TiffintomPromotionBanner::selectRaw("promotion_banners.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('id', 'DESC');

        if ($req->query('nopaginate', false) == '1') {
            $promotion_banners = $promotion_banners->get();
        } else {
            $promotion_banners = $promotion_banners->paginate($req->query('per_page', 20));
        }

        if ($promotion_banners->isEmpty()) Utils::error('No promotion banners found!', 404);

        return $promotion_banners;
    }


    /**
     * Create promotion-banner.
     * POST /tiffintom/promotion-banners
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'banner_link' => 'string',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'image' => 'image',
        ]);

        // create promotion-banner
        $promotion_banners = new TiffintomPromotionBanner;
        $promotion_banners->fill($req->all());
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PROMOTION_BANNER, 'tiffintom');
            $promotion_banners->banner_image = $fileName;
        }
        $promotion_banners->save();

        return $promotion_banners;
    }

    /**
     * List promotion-banners
     * GET /tiffintom/promotion-banners/id
     */
    public function show(Request $req, $id)
    {
        $promotion_banners = TiffintomPromotionBanner::find($id);
        if (empty($promotion_banners)) Utils::error("Promotion banner not found", 404);
        return $promotion_banners;
    }

    /**
     * Update payment method.
     * POST  /tiffintom/promotion-banners/id
     */
    public function update(Request $req, $id)
    {
        $promotion_banners = TiffintomPromotionBanner::find($id);
        if (empty($promotion_banners)) Utils::error('Promotion banner not found!', 404);

        $this->validate($req, [
            'banner_link' => 'string',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'image' => 'image',
        ]);

        // update promotion-banner
        if ($req->has('banner_link')) $promotion_banners->banner_link = $req->input('banner_link');
        if ($req->has('restaurant_id')) $promotion_banners->restaurant_id = $req->input('restaurant_id');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_PROMOTION_BANNER, 'tiffintom');
            $promotion_banners->banner_image = $fileName;
        }
        $promotion_banners->save();

        return $promotion_banners;
    }

    /**
     * Delete promotion-banner by id.
     * DELETE /tiffintom/promotion-banners/{id}
     */
    public function delete(Request $req, $id)
    {
        $promotion_banners = TiffintomPromotionBanner::find($id);
        if (empty($promotion_banners)) Utils::error('Promotion banner not found!', 404);
        $promotion_banners->delete();
        return $promotion_banners;
    }
}