<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomPromotion;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomPromotionController extends Controller
{

    /**
     * List promotions
     * GET /tiffintom/promotions
     */
    public function index(Request $req)
    {
        $promotions = TiffintomPromotion::orderBy('id', 'DESC');

        if ($query = $req->query('restaurant_id', false)) {
            $promotions->where('restaurant_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $promotions = $promotions->get();
        } else {
            $promotions = $promotions->paginate($req->query('per_page', 20));
        }

        if ($promotions->isEmpty()) Utils::error('No promotions found!', 404);

        return $promotions;
    }


    /**
     * Create promotion-banner.
     * POST /tiffintom/promotions
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'promo_image_url' => 'string',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'image' => 'image',
        ]);

        // create promotion-banner
        $promotion = new TiffintomPromotion;
        $promotion->fill($req->all());
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_RESTAURANT . $req->input('restaurant_id') . '/storeBanners/', 'tiffintom');
            $promotion->promo_image = $fileName;
        }
        $promotion->save();

        return $promotion;
    }

    /**
     * List promotions
     * GET /tiffintom/promotions/id
     */
    public function show(Request $req, $id)
    {
        $promotion = TiffintomPromotion::find($id);
        if (empty($promotion)) Utils::error("Promotion not found", 404);
        return $promotion;
    }

    /**
     * Update payment method.
     * POST  /tiffintom/promotions/id
     */
    public function update(Request $req, $id)
    {
        $promotion = TiffintomPromotion::find($id);
        if (empty($promotion)) Utils::error('Promotion not found!', 404);

        $this->validate($req, [
            'promo_image_url' => 'string',
            'restaurant_id' => 'exists:tiffintom_connection.restaurants,id',
            'image' => 'image',
        ]);

        // update promotion-banner
        if ($req->has('promo_image_url')) $promotion->promo_image_url = $req->input('promo_image_url');
        if ($req->has('restaurant_id')) $promotion->restaurant_id = $req->input('restaurant_id');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_RESTAURANT . $req->input('restaurant_id') . '/storeBanners/', 'tiffintom');
            $promotion->promo_image = $fileName;
        }
        $promotion->save();

        return $promotion;
    }

    /**
     * Delete promotion-banner by id.
     * DELETE /tiffintom/promotions/{id}
     */
    public function delete(Request $req, $id)
    {
        $promotion = TiffintomPromotion::find($id);
        if (empty($promotion)) Utils::error('Promotion not found!', 404);
        $promotion->delete();
        return $promotion;
    }
}
