<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomReferral;
use App\ModelsTiffintom\TiffintomReferralPending;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomReferralController extends Controller
{

    /**
     * List referral
     * GET /tiffintom/referral
     */
    public function index(Request $req)
    {
        $referral = TiffintomReferral::orderBy('id', 'DESC')->first();
        if (empty($referral)) Utils::error("No referral found", 404);
        return $referral;
    }

    /**
     * Create referral.
     * POST /tiffintom/referral
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'required|exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'rating' => 'integer',
            'message' => 'string',
            'responce' => 'string',
            'order_type' => 'string',
        ]);

        // create referral
        $referral = new TiffintomReferral;
        $referral->fill($req->all());
        $referral->save();

        return $referral;
    }

    /**
     * List referral
     * GET /tiffintom/referral/id
     */
    public function show(Request $req, $id)
    {
        $referral = TiffintomReferral::orderBy('id', 'ASC')->first();
        if (empty($referral)) Utils::error("No referral found", 404);
        // $referral = $referral->first();

        return $referral;
    }

    /**
     * Update referral.
     * POST  /tiffintom/referral/id
     */
    public function update(Request $req, $id)
    {
        $referral = TiffintomReferral::find($id);
        if (empty($referral)) Utils::error('referral not found!', 404);

        $this->validate($req, [
            'order_id' => 'required|exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'rating' => 'integer',
            'message' => 'string',
            'responce' => 'string',
            'order_type' => 'string',
        ]);

        // update menu
        if ($req->has('customer_id')) $referral->customer_id = $req->input('customer_id');
        if ($req->has('order_id')) $referral->order_id = $req->input('order_id');
        if ($req->has('restaurant_id')) $referral->restaurant_id = $req->input('restaurant_id');
        if ($req->has('rating')) $referral->rating = $req->input('rating');
        if ($req->has('message')) $referral->message = $req->input('message');
        if ($req->has('responce')) $referral->responce = $req->input('responce');
        $referral->order_type = $req->input('order_type');

        $referral->save();
        return $referral;
    }

    /**
     * Update referral.
     * POST  /tiffintom/referral/id
     */
    public function update_referral(Request $req, $id)
    {
        $referral = TiffintomReferral::find($id);
        if (empty($referral)) Utils::error('referral not found!', 404);

        $this->validate($req, [
            'referral_option' => 'required',
            'referral_type' => 'string',
            'invite_amount' => 'string',
            'receive_amount' => 'string',
            'minimum_order' => 'string',
        ]);

        // update menu
        if ($req->has('referral_option')) $referral->referral_option = $req->input('referral_option');
        if ($req->has('referral_type')) $referral->referral_type = $req->input('referral_type');
        if ($req->has('invite_amount')) $referral->invite_amount = $req->input('invite_amount');
        if ($req->has('invite_amount')) $referral->invite_amount = $req->input('invite_amount');
        if ($req->has('receive_amount')) $referral->receive_amount = $req->input('receive_amount');
        if ($req->has('minimum_order')) $referral->minimum_order = $req->input('minimum_order');

        $referral->save();

        return $referral;
    }

    /**
     * Delete referral by id.
     * DELETE /tiffintom/referral/{id}
     */
    public function delete(Request $req, $id)
    {
        $referral = TiffintomReferral::find($id);
        if (empty($referral)) Utils::error('referral not found!', 404);
        $referral->delete();

        return $referral;
    }

    /**
     * Delete referral by id.
     * DELETE /tiffintom/referral/{id}
     */
    public function refferal_pending(Request $req)
    {
        $referral_pending = TiffintomReferralPending::selectRaw('referrals_pending.*,users.first_name as first_name,users.last_name as last_name,users.username as username')->leftJoin('users', 'users.id', '=', 'referrals_pending.recieve_user')->orderBy('id', 'DESC');

        $customer = Auth::user();
        if ($customer) {
            $referral_pending->where('invite_user', $customer->id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $referral_pending = $referral_pending->get();
        } else {
            $referral_pending = $referral_pending->paginate($req->query('per_page', 20));
        }

        if ($referral_pending->isEmpty()) Utils::error('No referral pending found!', 404);

        return $referral_pending;
    }

    /**
     * Delete referral by id.
     * DELETE /tiffintom/referral/history
     */
    public function refferal_pending_index(Request $req)
    {
        $referral_pending = TiffintomReferralPending::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $referral_pending->whereRaw('(invite_user LIKE ? OR invite_amount LIKE ? OR recieve_user LIKE ? OR receive_amount LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($status = $req->query('status')) {
            $referral_pending->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $referral_pending = $referral_pending->get();
        } else {
            $referral_pending = $referral_pending->paginate($req->query('per_page', 20));
        }

        if ($referral_pending->isEmpty()) Utils::error('No referral pending found!', 404);

        return $referral_pending;
    }
}
