<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\PostcodeController;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposTable;
use App\ModelsTiffintom\TiffintomBooking;
use App\ModelsTiffintom\TiffintomCuisine;
use App\ModelsTiffintom\TiffintomAreaMap;
use App\ModelsTiffintom\TiffintomDeliverySetting;
use App\ModelsTiffintom\TiffintomDietary;
use App\ModelsTiffintom\TiffintomDineinOrder;
use App\ModelsTiffintom\TiffintomFavouriteRestaurant;
use App\ModelsTiffintom\TiffintomMenu;
use App\ModelsTiffintom\TiffintomOffer;
use App\ModelsTiffintom\TiffintomOfferDay;
use App\ModelsTiffintom\TiffintomOfferDayPrice;
use App\ModelsTiffintom\TiffintomOfferDayProduct;
use App\ModelsTiffintom\TiffintomOfferMultiplePrice;
use App\ModelsTiffintom\TiffintomOfferPrice;
use App\ModelsTiffintom\TiffintomOfferProduct;
use App\ModelsTiffintom\TiffintomOfferText;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomOrderHelp;
use App\ModelsTiffintom\TiffintomPromotion;
use App\ModelsTiffintom\TiffintomPromotionBanner;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomRestaurantPaymentMethod;
use App\ModelsTiffintom\TiffintomRestaurantTiming;
use App\ModelsTiffintom\TiffintomRestaurantVoucher;
use App\ModelsTiffintom\TiffintomReview;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomUser;
use App\ModelsTiffintom\TiffintomUserVoucher;
use App\Postcode;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use stdClass;

class TiffintomRestaurantController extends Controller
{
    /**
     * List restaurants
     * GET /tiffintom/restaurants
     */
    public function index(Request $req)
    {
        $restaurants = TiffintomRestaurant::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $restaurants->whereRaw('(restaurant_name LIKE ? OR contact_name LIKE ? OR contact_phone LIKE ? OR street_address LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($query = $req->query("restaurant_id", false)) {
            $restaurants->where('restaurant_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $restaurants = $restaurants->get();
        } else {
            $restaurants = $restaurants->paginate($req->query('per_page', 20));
        }

        if ($restaurants->isEmpty()) Utils::error('No restaurant found!', 404);

        return $restaurants;
    }

    /**
     * List businesses.
     * GET /api/businesses/mini
     */
    public function mini(Request $req)
    {
        $restaurants = TiffintomRestaurant::orderByRaw('id ASC');
        $restaurants->where('status', '1');

        if ($query = $req->query("query", false)) {
            $restaurants->whereRaw('(restaurant_name)', ["%$query%"]);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $restaurants->where('restaurant_id', $query);
        }

        $restaurants = $restaurants->get();

        if ($restaurants->isEmpty()) Utils::error('No restaurants found!', 404);

        return ($restaurants);
    }

    /**
     * List restaurants
     * GET /tiffintom/restaurants/fetch
     */
    public function fetch(Request $req)
    {
        $restaurants = TiffintomRestaurant::orderBy('id', 'ASC');

        if ($qr_code = $req->query("qr_code", false)) {
            $restaurants->where('qr_code', $qr_code);
        }
        if ($query = $req->query("query", false)) {
            $restaurants->whereRaw('(restaurant_name)', ["%$query%"]);
        }
        if ($id = $req->query("id", false)) {
            $restaurants->where('id', $id);
        }

        $restaurants = $restaurants->first();
        if (empty($restaurants)) Utils::error('No restaurant found!', 404);

        $restaurants->append('restaurant_timing');
        $restaurants->append('business');
        $restaurants->append('payment_methods');
        $restaurants->append('delivery_setting');
        // $restaurants->append('site_setting');
        // $restaurants->append('meta_tag');
        $restaurants->makeHidden(['modified', 'otp', 'smart_mps_id', 'dinein_restaurant_commission', 'device_id', 'device_name', 'sms_option', 'sms_phonenumber', 'restaurant_commission', 'cash_commission', 'card_commission', 'paypal_commission', 'invoice_period', 'offer_texts', 'fb_page_id', 'dinein_stripe_description', 'dinein_stripe_descriptor',   'dinein_service_charge_status', 'dinein_restaurant_commission', 'map_mode', 'email_order', 'order_email', 'is_logged', 'delete_status', 'qr_code', 'state_id', 'country_id', 'location_id',  'restaurant_tax', 'restaurant_logo']);

        $currentTime = strtotime(date('h:i A'));
        $currentDay = strtolower(date('l'));

        $firstOpenTime = strtotime($restaurants[$currentDay . '_first_opentime']);
        $firstCloseTime = strtotime($restaurants[$currentDay . '_first_closetime']);
        $secondOpenTime = strtotime($restaurants[$currentDay . '_second_opentime']);
        $secondCloseTime = strtotime($restaurants[$currentDay . '_second_closetime']);

        $pickfirstOpenTime = strtotime($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_opentime']);
        $pickfirstCloseTime = strtotime($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_closetime']);
        $picksecondOpenTime = strtotime($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_opentime']);
        $picksecondCloseTime = strtotime($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_closetime']);

        if ($restaurants['response_status'] != 0 && ($restaurants[$currentDay . '_status'] != 'Close' || $restaurants['restaurant_timing']['pick_' . $currentDay . '_status'] != 'Close')) {
            if (($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $pickfirstCloseTime)) {
                $restaurants['currentStatus'] = 'Open';
            } else if (($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) || ($currentTime > $picksecondOpenTime && $currentTime <= $picksecondCloseTime)) {
                $restaurants['currentStatus'] = 'Open';
            } else if (($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $picksecondOpenTime)) {
                $restaurants['currentStatus'] = 'PreOrder';
            } else {
                $restaurants['currentStatus'] = 'Closed';
            }
        } else {
            $restaurants['currentStatus'] = 'Closed';
        }

        $req->merge(['restaurant_id' => $restaurants->id, 'date' => date('Y-m-d')]);
        $TimeSlots = $this->restaurant_timeslot($req);
        $deliveryTimeSlots = $TimeSlots['delivery'];
        $pickupTimeSlots = $TimeSlots['pickup'];

        if ($deliveryTimeSlots['today'] != '' || $deliveryTimeSlots['dayaftertomorrow'] != '' || $deliveryTimeSlots['tomorrow'] != '' || $pickupTimeSlots['today'] != '' || $pickupTimeSlots['dayaftertomorrow'] != '' || $pickupTimeSlots['tomorrow'] != '') {
            $restaurants['restaurant_status'] = 'Open';
        } else {
            $restaurants['restaurant_status'] = 'Closed';
        }

        $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurants->id)->where('status', '1')->first();
        if (!empty($reviews) && $reviews->count > 0) {
            $restaurants['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
            $restaurants['total_reviews'] = $reviews->count;
        }

        if ($customer_id = $req->query("customer_id", false)) {
            $favourite = TiffintomFavouriteRestaurant::where('customer_id', $customer_id)->where('restaurant_id', $restaurants->id)->where('is_favourite', '1')->first();
            if (!empty($favourite)) {
                $restaurants['is_favourite'] = 1;
            } else {
                $restaurants['is_favourite'] = 0;
            }

            $business = Business::find($restaurants->business->id);
            if (empty($business)) Utils::error("Business not found", 400);
            $this->establishEposConnection($business);
            $User = TiffintomUser::where('id', $customer_id)->first();

            $name = $User->first_name . ' ' . $User->last_name;
            $Customer = EposCustomer::whereRaw('(name LIKE ?  )', ["%$name%"])->first();
            if (empty($Customer)) {
                $Customer = new EposCustomer;
                $Customer->membership_no = rand(100000000, 999999999);
                $Customer->name = $name;
                $Customer->mobile = $User->phone_number;
                $Customer->email = $User->username;
                $Customer->save();
            }
            $restaurants['epos_customer'] = $Customer;
        }

        if ($table_id = $req->query("table_id", false)) {
            $business = Business::find($restaurants->business->id);
            if (empty($business)) Utils::error("Business not found", 400);
            $this->establishEposConnection($business);
            $restaurants['table'] = EposTable::where('id', $table_id)->first();
        }

        return $restaurants;
    }

    /**
     * List restaurants
     * POST /tiffintom/restaurants/search
     */
    public function search(Request $req)
    {
        $siteSettings = TiffintomSiteSetting::orderBy('id', 'ASC')->first();
        $addmiles = $siteSettings->site_mile;

        $postcode = Postcode::whereRaw('(post_code LIKE ? OR post_code LIKE ? )', [$req->input('postcode'), str_replace("%20", "", $req->input('postcode'))])->first();
        if (empty($postcode)) {
            $newPostcode = $this->findFromGoogleAndReturn($req->input('postcode'));
            $postcode = $newPostcode;
        }

        $sourcelatitude = $postcode->latitude;
        $sourcelongitude = $postcode->longitude;

        if ($sourcelatitude != '' && $sourcelongitude != '') {
            $allCuisinesList = array();
            $allCuisinesLists = array();
            $allDietaryLists = array();

            if ($req->has('cuisine_id')) {
                $cuisine_id = $req->input('cuisine_id');
                $restaurants = TiffintomRestaurant::whereNotNull('id')->whereRaw('(restaurant_pickup LIKE ? OR restaurant_delivery LIKE ? )', ["%Yes%", "%Yes%"])->where('status', '1')->where('delete_status', 'N')->whereRaw("FIND_IN_SET($cuisine_id,restaurant_cuisine)")->whereNotNull('sourcelatitude')->whereNotNull('sourcelongitude')->orderBy('id', 'ASC');
            } else {
                $restaurants = TiffintomRestaurant::whereNotNull('id')->whereRaw('(restaurant_pickup LIKE ? OR restaurant_delivery LIKE ? )', ["%Yes%", "%Yes%"])->where('status', '1')->where('delete_status', 'N')->whereNotNull('sourcelatitude')->whereNotNull('sourcelongitude')->orderBy('id', 'ASC');
            }

            if ($req->has('name')) {
                $name = $req->input('contact_name');
                $restaurants->whereRaw('(restaurant_name LIKE ? )', ["%$name%"]);
            }
            if ($id = $req->query("id", false)) {
                $restaurants->where('id', $id);
            }

            $restaurants = $restaurants->get();
            if ($restaurants->isEmpty()) Utils::error('No restaurants found!', 404);
            $restaurantses = array();

            foreach ($restaurants as $key => $restaurant) {
                if (!empty($restaurant)) {
                    $restaurant->append('restaurant_timing');
                    $restaurant->append('delivery_setting');
                    $restaurant->append('payment_methods');
                    $restaurant->append('currentStatus');
                    $restaurants->makeHidden(['modified', 'otp', 'smart_mps_id', 'dinein_restaurant_commission', 'device_id', 'device_name', 'sms_option', 'sms_phonenumber', 'restaurant_commission', 'cash_commission', 'card_commission', 'paypal_commission', 'invoice_period', 'offer_texts', 'fb_page_id', 'dinein_stripe_description', 'dinein_stripe_descriptor',   'dinein_service_charge_status', 'dinein_restaurant_commission', 'map_mode', 'email_order', 'order_email', 'is_logged', 'delete_status', 'qr_code', 'state_id', 'country_id', 'location_id',  'restaurant_tax', 'restaurant_logo']);
                    $restaurant->total_orders = TiffintomOrder::where('restaurant_id', $restaurant->id)->count();
                    $sourcelatitude = $sourcelatitude;
                    $sourcelongitude = $sourcelongitude;
                    $latitudeTo  = $restaurant->sourcelatitude;
                    $longitudeTo = $restaurant->sourcelongitude;
                    $unit = 'K';
                    if ($latitudeTo != '' && $longitudeTo != '') {
                        $distance = $this->getDistanceCalculate(
                            $sourcelatitude,
                            $sourcelongitude,
                            $latitudeTo,
                            $longitudeTo,
                            $unit
                        );
                    } else {
                        continue;
                    }
                    $restaurant->distance = (float)$distance;

                    if ($latitudeTo != '' && $longitudeTo != '') {
                        list($deliveryCharge, $message) = $this->getDeliveryCharge($restaurant->id, $distance, $sourcelatitude, $sourcelongitude);
                        $restaurant->deliveryCharges = $deliveryCharge;
                    } else {
                        continue;
                    }


                    $customerId = $req->input('customer_id');
                    $req = Request::create("");
                    $req->merge(['restaurant_id' => $restaurant->id, 'date' => date('Y-m-d'), 'customer_id' => $customerId]);
                    $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurant->id)->where('status', '1')->first();
                    if (!empty($reviews) && $reviews->count > 0) {
                        $restaurant['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                        $restaurant['total_reviews'] = $reviews->count;
                    }

                    if ($message == 'success') {
                        $cuisineList = array();
                        $restaurantCuisine = explode(',', $restaurant->restaurant_cuisine);
                        if (!empty($restaurantCuisine)) {
                            $restaurant_cuisine_list = [];
                            foreach ($restaurantCuisine as $ckey => $cvalue) {
                                $cuisines = TiffintomCuisine::where('id', $cvalue)->where('delete_status', 'N')->first();
                                if (!empty($cuisines)) {
                                    if (!in_array($cvalue, $allCuisinesLists)) {
                                        $allCuisinesLists[] = $cvalue;
                                    }
                                    $restaurant_cuisine_list[] = $cuisines->cuisine_name;
                                }
                            }
                            $restaurant['cuisine_list'] = $restaurant_cuisine_list;
                        }
                        $cuisineList = TiffintomCuisine::whereIn('id', $allCuisinesLists)->get();

                        $dietaryList = array();
                        $restaurantDietaries = explode(',', $restaurant->restaurant_dietary);
                        if (!empty($restaurantDietaries)) {
                            $restaurant_dietary_list = [];
                            foreach ($restaurantDietaries as $ckey => $cvalue) {
                                $dietarys = TiffintomDietary::where('id', $cvalue)->first();
                                if (!empty($dietarys)) {
                                    if (!in_array($cvalue, $allDietaryLists)) {
                                        $allDietaryLists[] = $cvalue;
                                    }
                                    $restaurant_dietary_list[] = $dietarys->dietary_name;
                                }
                            }
                            $restaurant['dietary_list'] = $restaurant_dietary_list;
                        }
                        $dietaryList = TiffintomDietary::whereIn('id', $allDietaryLists)->get();

                        if ($req->has('customer_id')) {
                            $order_count = TiffintomOrder::where('customer_id', $req->input('customer_id'))->where('restaurant_id', $restaurant->id)->count();
                            $restaurant->order_count = $order_count;

                            $favourite = TiffintomFavouriteRestaurant::where('customer_id', $req->input('customer_id'))->where('restaurant_id', $restaurant->id)->where('is_favourite', 1)->first();
                            if (!empty($favourite)) {
                                $restaurant['is_favourite'] = 1;
                            } else {
                                $restaurant['is_favourite'] = 0;
                            }
                        }

                        $restaurantses[] = $restaurant;
                    } else {
                        continue;
                    }
                }
            }
            $bannerLists = TiffintomPromotionBanner::whereNotNull('banner_image')->get();
        }

        usort($restaurantses, function ($a, $b) {
            return $a['distance'] <=> $b['distance'];
        });

        $data = [
            'restaurants' => $restaurantses,
            'cuisines' => $cuisineList,
            'dietaries' => $dietaryList,
            'banner_lists' => $bannerLists,
        ];

        return $data;
    }

    public function find_city(Request $req)
    {
        $postcode = Postcode::whereRaw('(post_town LIKE ? OR post_code LIKE ? OR post_town LIKE ? )', [$req->input('city_name'), str_replace("%20", "", $req->input('city_name')), str_replace("%20", "", $req->input('city_name'))])->first();
        if (empty($postcode)) Utils::error('No city found!', 404);

        return $postcode;
    }

    public function nearby(Request $req)
    {
         // $postcode = Postcode::whereRaw('(post_code LIKE ? OR post_code LIKE ? )', [$req->input('postcode'), str_replace("%20", "", $req->input('postcode'))])->first();
        //if (empty($postcode)) { remove //jaydip 12-02-20024
            //$newPostcode = $this->findFromGoogleAndReturn($req->input('postcode'));
            //$postcode = $newPostcode;
       // }
        $newPostcode = $this->findFromGoogleAndReturn($req->input('postcode'));
        $postcode = $newPostcode;

        $sourcelatitude = $postcode->latitude ?? '';
        $sourcelongitude = $postcode->longitude ?? '';

        $allCuisinesLists = array();
        $allDietariesLists = array();
        $restaurantses = array();

        if ($sourcelatitude != '' && $sourcelongitude != '') {

            if ($req->has('cuisine_id')) {
                $cuisine_id = $req->input('cuisine_id');
                $restaurants = TiffintomRestaurant::with(['offerTexts'])->whereNotNull('id')->whereRaw('(restaurant_pickup LIKE ? OR restaurant_delivery LIKE ? )', ["%Yes%", "%Yes%"])->where('status', '1')->where('delete_status', 'N')->whereRaw("FIND_IN_SET($cuisine_id,restaurant_cuisine)")->whereNotNull('sourcelatitude')->whereNotNull('sourcelongitude')->orderBy('id', 'ASC');
            } else {
                $restaurants = TiffintomRestaurant::with(['offerTexts'])->whereNotNull('id')->whereRaw('(restaurant_pickup LIKE ? OR restaurant_delivery LIKE ? )', ["%Yes%", "%Yes%"])->where('status', '1')->where('delete_status', 'N')->whereNotNull('sourcelatitude')->whereNotNull('sourcelongitude')->orderBy('id', 'ASC');
            }

            if ($req->has('name')) {
                $name = $req->input('contact_name');
                $restaurants->whereRaw('(restaurant_name LIKE ? )', ["%$name%"]);
            }
            if ($id = $req->query("id", false)) {
                $restaurants->where('id', $id);
            }

            $carbonDate = Carbon::parse(date('d-m-Y'));
            $currentDay = strtolower(date('l'));
            $tomorrowDay = strtolower($carbonDate->addDays(1)->format('l'));
            $dayaftertomorrowDay = strtolower($carbonDate->addDays(1)->format('l'));

            $restaurants = $restaurants
                ->select('id', 'restaurant_name', 'seo_url', 'city_id', 'sourcelatitude', 'sourcelongitude', 'restaurant_cuisine', 'restaurant_dietary', 'minimum_order', 'paypal_minimum_order', 'card_minimum_order', 'online_order', 'logo_name', 'delivery_charge', $currentDay . '_first_opentime', $currentDay . '_first_closetime', $currentDay . '_second_opentime', $currentDay . '_second_closetime', $currentDay . '_status', $tomorrowDay . '_first_opentime', $tomorrowDay . '_first_closetime', $tomorrowDay . '_second_opentime', $tomorrowDay . '_second_closetime', $tomorrowDay . '_status', $dayaftertomorrowDay . '_first_opentime', $dayaftertomorrowDay . '_first_closetime', $dayaftertomorrowDay . '_second_opentime', $dayaftertomorrowDay . '_second_closetime', $dayaftertomorrowDay . '_status', 'map_mode', 'pickup_estimate_time', 'estimate_time', 'free_delivery', 'new', 'restaurant_delivery', 'restaurant_pickup')
                ->withCount('orders')->orderBy('orders_count')->get();
            // ->toBase()

            if ($restaurants->isEmpty()) Utils::error('No restaurants found!', 404);

            foreach ($restaurants as $key => $restaurant) {
                if (!empty($restaurant)) {
                    $bannerImage = TiffintomPromotion::where('restaurant_id', $restaurant->id)->select('promo_image')->first();
                    if ($bannerImage) {
                        $restaurant->promotion = ImageHelper::getServerImageUrl(ImageHelper::$TYPE_TIFFINTOM_RESTAURANT, $restaurant->id . '/storeBanners/' . $bannerImage->promo_image, env('FTP_IMAGE_PATH_TIFFINTOM'));
                    } else {
                        $restaurant->promotion = null;
                    }
                    $restaurant->offer_texts = TiffintomOfferText::where('restaurant_id', $restaurant->id)->where('status', '1')->where('delete_status', 'N')->select('offer_name')->toBase()->get();
                    $restaurant->image_url = ImageHelper::getServerImageUrl(ImageHelper::$TYPE_TIFFINTOM_RESTAURANT, $restaurant->id . '/storeLogo/' . $restaurant->logo_name, env('FTP_IMAGE_PATH_TIFFINTOM'));
                    $restaurant->currentStatus = $this->currentStatus($restaurant);
                    $restaurant->city_name = $this->seoUrl($restaurant->city_id);
                    $sourcelatitude = $sourcelatitude;
                    $sourcelongitude = $sourcelongitude;
                    $latitudeTo  = $restaurant->sourcelatitude;
                    $longitudeTo = $restaurant->sourcelongitude;
                    $unit = 'K';
                    if ($latitudeTo != '' && $longitudeTo != '') {
                        $distance = $this->getDistanceCalculate(
                            $sourcelatitude,
                            $sourcelongitude,
                            $latitudeTo,
                            $longitudeTo,
                            $unit
                        );
                    } else {
                        continue;
                    }
                    $restaurant->distance = (float)$distance;

                    if ($latitudeTo != '' && $longitudeTo != '') {
                        list($deliveryCharge, $message) = $this->getDeliveryCharge($restaurant->id, $distance, $sourcelatitude, $sourcelongitude);
                        $restaurant->deliveryCharges = $deliveryCharge;
                        // $delivery_settings = TiffintomDeliverySetting::where('restaurant_id', $restaurant->id)->select('delivery_miles', 'delivery_charge')->toBase()->get();
                        // $deliveryCharge = $count = 0;
                        // if ($restaurant->map_mode == 'Circle') {
                        //     foreach ($delivery_settings as $key => $value) {
                        //         if ($distance <= $value->delivery_miles) {
                        //             $deliveryCharge = $value->delivery_charge;
                        //             $count = 1;
                        //             break;
                        //         }
                        //     }
                        //     if ($count == 1) {
                        //         $message = 'success';
                        //     } else {
                        //         $message = 'failed';
                        //     }
                        // } else if ($restaurant->map_mode == 'Polygon') {
                        //     $areamaps = TiffintomAreaMap::where('res_id', $restaurant->id)->toBase()->get();
                        //     foreach ($areamaps as $key => $value) {
                        //         $xyArray = $this->getXYAxis($value['mappath']);
                        //         if (!empty($xyArray) && is_array($xyArray)) {
                        //             $objPolygon = new TiffintomPolygonsController(array('x' => $latitudeTo, 'y' => $longitudeTo), $xyArray);
                        //             if ($objPolygon->check()) {
                        //                 $deliveryCharge = $value['service_delivery_charge'];
                        //                 $message = 'success';
                        //                 break;
                        //             } else {
                        //                 $message = 'failed';
                        //             }
                        //         }
                        //     }
                        // } else {
                        //     $message = 'failed';
                        // }
                    } else {
                        continue;
                    }

                    $customerId = $req->input('customer_id');
                    $req = Request::create("");
                    $req->merge(['restaurant_id' => $restaurant->id, 'date' => date('Y-m-d'), 'customer_id' => $customerId]);
                    $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurant->id)->where('status', '1')->first();
                    if (!empty($reviews) && $reviews->count > 0) {
                        $restaurant->average_rating = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                        $restaurant->total_reviews = $reviews->count;
                    } else {
                        $restaurant->average_rating = null;
                    }

                    if ($message == 'success') {
                        $restaurantCuisine = explode(',', $restaurant->restaurant_cuisine);
                        $allCuisinesLists = array_merge($allCuisinesLists, $restaurantCuisine);
                        $restaurantDietary = explode(',', $restaurant->restaurant_dietary);
                        $allDietariesLists = array_merge($allDietariesLists, $restaurantDietary);
                        $restaurantses[] = $restaurant;
                    } else {
                        continue;
                    }
                }
            }
            $bannerLists = TiffintomPromotionBanner::whereNotNull('banner_image')->get();
        }
        $allCuisinesLists = array_unique($allCuisinesLists);
        if (count($allCuisinesLists) > 8) {
            array_splice($allCuisinesLists, 8);
        }

        $allCuisinesLists = TiffintomCuisine::whereIn('id', $allCuisinesLists)->where('status', '1')->select('id', 'cuisine_name', 'cuisine_image', 'first_bg', 'second_bg')->get()->append('image_url');
        $allDietariesLists = array_unique($allDietariesLists);
        $allDietariesLists = TiffintomDietary::whereIn('id', $allDietariesLists)->where('status', '1')->select('id', 'name')->get();

        if ($req->has('customer_id') && $req->customer_id != null) {
            $favouriteRestaurants = collect(TiffintomFavouriteRestaurant::where('customer_id', $req->input('customer_id'))->where('is_favourite', 1)->select('restaurant_id')->toBase()->get()->pluck('restaurant_id')->toArray())->values(); // unique('id')->
        } else {
            $favouriteRestaurants = [];
        }
        $restaurantses = collect($restaurantses)->sortBy('distance')->unique('id')->values();

        if ($restaurantses->isEmpty()) Utils::error('No restaurant found!', 404);

        $data = [
            'restaurants' => $restaurantses,
            'cuisines' => $allCuisinesLists,
            'dietaries' => $allDietariesLists,
            'favoriteRestaurants' => $favouriteRestaurants,
            'banner_lists' => $bannerLists,
        ];
        return $data;
    }

    public function currentStatus($restaurant)
    {
        $restaurant = collect($restaurant)->toArray();
        $currentTime = strtotime(date('h:i A'));
        $carbonDate = Carbon::parse(date('d-m-Y'));
        $currentDay = strtolower(date('l'));
        $tomorrowDay = strtolower($carbonDate->addDays(1)->format('l'));
        $dayaftertomorrowDay = strtolower($carbonDate->addDays(1)->format('l'));

        $firstOpenTime = strtotime($restaurant[$currentDay . '_first_opentime']);
        $firstCloseTime = strtotime($restaurant[$currentDay . '_first_closetime']);
        $secondOpenTime = strtotime($restaurant[$currentDay . '_second_opentime']);
        $secondCloseTime = strtotime($restaurant[$currentDay . '_second_closetime']);
        $Daystatus = isset($restaurant[$currentDay . '_status']) ? $restaurant[$currentDay . '_status'] : 'Close';

        $delivery_tomorrow_firstOpenTime = $restaurant[$tomorrowDay . '_first_opentime'];
        $delivery_tomorrow_firstCloseTime = $restaurant[$tomorrowDay . '_first_closetime'];
        $delivery_tomorrow_secondOpenTime = $restaurant[$tomorrowDay . '_second_opentime'];
        $delivery_tomorrow_secondCloseTime = $restaurant[$tomorrowDay . '_second_closetime'];
        $delivery_tomorrow_Daystatus = isset($restaurant[$currentDay . '_status']) ? $restaurant[$currentDay . '_status'] : 'Close';

        $delivery_after_firstOpenTime = $restaurant[$dayaftertomorrowDay . '_first_opentime'];
        $delivery_after_firstCloseTime = $restaurant[$dayaftertomorrowDay . '_first_closetime'];
        $delivery_after_secondOpenTime = $restaurant[$dayaftertomorrowDay . '_second_opentime'];
        $delivery_after_secondCloseTime = $restaurant[$dayaftertomorrowDay . '_second_closetime'];
        $delivery_after_Daystatus = isset($restaurant[$currentDay . '_status']) ? $restaurant[$currentDay . '_status'] : 'Close';

        $restaurant['restaurant_timing'] = TiffintomRestaurantTiming::where('restaurant_id', $restaurant['id'])->first();
        if (isset($restaurant['restaurant_timing']) && $restaurant['restaurant_timing'] != null) {
            $pickfirstOpenTime = strtotime($restaurant['restaurant_timing']['pick_' . $currentDay . '_first_opentime']);
            $pickfirstCloseTime = strtotime($restaurant['restaurant_timing']['pick_' . $currentDay . '_first_closetime']);
            $picksecondOpenTime = strtotime($restaurant['restaurant_timing']['pick_' . $currentDay . '_second_opentime']);
            $picksecondCloseTime = strtotime($restaurant['restaurant_timing']['pick_' . $currentDay . '_second_closetime']);
            $pickstatus = $restaurant['restaurant_timing']['pick_' . $currentDay . '_status'];

            $pick_tomorrow_firstOpenTime = $restaurant['restaurant_timing']['pick_' . $tomorrowDay . '_first_opentime'];
            $pick_tomorrow_firstCloseTime = $restaurant['restaurant_timing']['pick_' . $tomorrowDay . '_first_closetime'];
            $pick_tomorrow_secondOpenTime = $restaurant['restaurant_timing']['pick_' . $tomorrowDay . '_second_opentime'];
            $pick_tomorrow_secondCloseTime = $restaurant['restaurant_timing']['pick_' . $tomorrowDay . '_second_closetime'];
            $pick_tomorrow_status = $restaurant['restaurant_timing']['pick_' . $tomorrowDay . '_status'];

            $pick_day_after_firstOpenTime = $restaurant['restaurant_timing']['pick_' . $dayaftertomorrowDay . '_first_opentime'];
            $pick_day_after_firstCloseTime = $restaurant['restaurant_timing']['pick_' . $dayaftertomorrowDay . '_first_closetime'];
            $pick_day_after_secondOpenTime = $restaurant['restaurant_timing']['pick_' . $dayaftertomorrowDay . '_second_opentime'];
            $pick_day_after_secondCloseTime = $restaurant['restaurant_timing']['pick_' . $dayaftertomorrowDay . '_second_closetime'];
            $pick_day_after_status = $restaurant['restaurant_timing']['pick_' . $dayaftertomorrowDay . '_status'];
        } else {
            $pickfirstOpenTime = strtotime('12:00 PM');
            $pickfirstCloseTime = strtotime('12:00 PM');
            $picksecondOpenTime = strtotime('12:00 PM');
            $picksecondCloseTime = strtotime('12:00 PM');
            $pickstatus = 'Close';

            $pick_tomorrow_firstOpenTime = '12:00 PM';
            $pick_tomorrow_firstCloseTime = '12:00 PM';
            $pick_tomorrow_secondOpenTime = '12:00 PM';
            $pick_tomorrow_secondCloseTime = '12:00 PM';
            $pick_tomorrow_status = 'Close';

            $pick_day_after_firstOpenTime = '12:00 PM';
            $pick_day_after_firstCloseTime = '12:00 PM';
            $pick_day_after_secondOpenTime = '12:00 PM';
            $pick_day_after_secondCloseTime = '12:00 PM';
            $pick_day_after_status = 'Close';
        }

        if ($Daystatus != 'Close' || $pickstatus != 'Close') {
            if (($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $pickfirstCloseTime)) {
                return 'Open';
            } else if (($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) || ($currentTime > $picksecondOpenTime && $currentTime <= $picksecondCloseTime)) {
                return 'Open';
            } else if (
                $delivery_tomorrow_Daystatus != 'Close' && $pick_tomorrow_status != 'Close' && (!($delivery_tomorrow_firstOpenTime == '12:00 PM' && $delivery_tomorrow_firstCloseTime == '12:00 PM') || !($pick_tomorrow_firstOpenTime == '12:00 PM' && $pick_tomorrow_firstCloseTime == '12:00 PM'))
            ) {
                return 'PreOrder';
            } else if (
                $delivery_tomorrow_Daystatus != 'Close' && $pick_tomorrow_status != 'Close' && (!($delivery_tomorrow_secondOpenTime == '12:00 PM' && $delivery_tomorrow_secondCloseTime == '12:00 PM')  || !($pick_tomorrow_secondOpenTime == '12:00 PM' && $pick_tomorrow_secondCloseTime == '12:00 PM'))
            ) {
                return 'PreOrder';
            } else if (
                $delivery_after_Daystatus != 'Close' && $pick_day_after_status != 'Close' && (!($delivery_after_firstOpenTime == '12:00 PM' && $delivery_after_firstCloseTime == '12:00 PM') || !($pick_day_after_firstOpenTime == '12:00 PM' && $pick_day_after_firstCloseTime == '12:00 PM'))
            ) {
                return 'PreOrder';
            } else if (
                $delivery_after_Daystatus != 'Close' && $pick_day_after_status != 'Close' && (!($delivery_after_secondOpenTime == '12:00 PM' && $delivery_after_secondCloseTime == '12:00 PM') || !($pick_day_after_secondOpenTime == '12:00 PM' && $pick_day_after_secondCloseTime == '12:00 PM'))
            ) {
                return 'PreOrder';
            } else {
                return 'Closed';
            }
        } else {
            return 'Closed';
        }
    }

    /**
     * Create restaurant.
     * POST  /tiffintom/restaurants
     */
    public function create(Request $req)
    {
        if ($req->has('restaurant_timing') && !is_array($req->restaurant_timing))
            $req->merge(['restaurant_timing' => (json_decode($req->restaurant_timing, true))]);

        if ($req->has('bank_detail') && !is_array($req->bank_detail))
            $req->merge(['bank_detail' => (json_decode($req->bank_detail, true))]);

        if ($req->has('delivery_setting') && !is_array($req->delivery_setting))
            $req->merge(['delivery_setting' => (json_decode($req->delivery_setting, true))]);

        if ($req->has('areamaps') && !is_array($req->areamaps))
            $req->merge(['areamaps' => (json_decode($req->areamaps, true))]);

        if ($req->has('meta_tag') && !is_array($req->meta_tag))
            $req->merge(['meta_tag' => (json_decode($req->meta_tag, true))]);

        if ($req->has('payment_methods') && !is_array($req->payment_methods))
            $req->merge(['payment_methods' => (json_decode($req->payment_methods, true))]);

        $this->validate($req, [
            'contact_name' => 'required|string|max:255',
            'contact_phone' => 'required|string',
            'contact_email' => 'required|string',
            'contact_address' => 'required|string',
            'city_id' => 'required|string',
            'currency' => 'required|string',
            'timezone' => 'required|string',
            'currency_code' => 'required|string',
            'zipcode' => 'string',
            'restaurant_name' => 'required|string',
            'restaurant_phone' => 'required|string',
            'street_address' => 'required|string',
            'working_time' => 'in:morning,evening,both',
            'monday_first_opentime' => 'required|string',
            'monday_first_closetime' => 'required|string',
            'monday_second_opentime' => 'required|string',
            'monday_second_closetime' => 'required|string',
            'tuesday_first_opentime' => 'required|string',
            'tuesday_first_closetime' => 'required|string',
            'tuesday_second_opentime' => 'required|string',
            'tuesday_second_closetime' => 'required|string',
            'wednesday_first_opentime' => 'required|string',
            'wednesday_first_closetime' => 'required|string',
            'wednesday_second_opentime' => 'required|string',
            'wednesday_second_closetime' => 'required|string',
            'thursday_first_opentime' => 'required|string',
            'thursday_first_closetime' => 'required|string',
            'thursday_second_opentime' => 'required|string',
            'thursday_second_closetime' => 'required|string',
            'friday_first_opentime' => 'required|string',
            'friday_first_closetime' => 'required|string',
            'friday_second_opentime' => 'required|string',
            'friday_second_closetime' => 'required|string',
            'saturday_first_opentime' => 'required|string',
            'saturday_first_closetime' => 'required|string',
            'saturday_second_opentime' => 'required|string',
            'saturday_second_closetime' => 'required|string',
            'sunday_first_opentime' => 'required|string',
            'sunday_first_closetime' => 'required|string',
            'sunday_second_opentime' => 'required|string',
            'sunday_second_closetime' => 'required|string',
            'monday_status' => 'string',
            'tuesday_status' => 'string',
            'wednesday_status' => 'string',
            'thursday_status' => 'string',
            'friday_status' => 'string',
            'saturday_status' => 'string',
            'sunday_status' => 'string',
            'facebook_login' => 'in:Y,N',
            'google_login' => 'in:Y,N',
            'twitter_login' => 'in:Y,N',
            'apple_login' => 'in:Y,N',
            'alcahol_available' => 'boolean',
            'alcahol_not_available' => 'boolean',
            'bring_your_alcahol' => 'boolean',
            'soft_drink' => 'boolean',
            'restaurant_tax' => 'string',
            'pickup_estimate_time' => 'string',
            'restaurant_cuisine' => 'string',
            'restaurant_dietary' => 'string',
            'restaurant_visibility' => 'in:FOS,external,both',
            'restaurant_dispatch' => 'in:Yes,No',
            'online_order' => 'in:Yes,No',
            'restaurant_pickup' => 'in:Yes,No',
            'restaurant_delivery' => 'in:Yes,No',
            'restaurant_booktable' => 'in:Yes,No',
            'booking_status' => 'in:Yes,No',
            'image_type' => 'in:Yes,No',
            'restaurant_about' => 'string',
            'username' => 'required|string',
            'password' => 'required|string',
            'delivery_charge' => 'in:Yes,No',
            'estimate_time' => 'string',
            'minimum_order' => 'string',
            'paypal_minimum_order' => 'string',
            'card_minimum_order' => 'string',
            'free_delivery' => 'string',
            'booking_start_days' => 'string',
            'booking_start' => 'string',
            'booking_end' => 'string',
            'no_of_guest' => 'string',
            'map_mode' => 'in:Circle,Polygon',
            'email_order' => 'in:Yes,No',
            'order_email' => 'string',
            'sms_option' => 'in:Yes,No',
            'sms_phonenumber' => 'string',
            'service_charge_type' => 'string',
            'restaurant_commission' => 'numeric',
            'cash_commission' => 'numeric',
            'card_commission' => 'numeric',
            'paypal_commission' => 'numeric',
            'service_charge_status' => 'in:Yes,No',
            'order_charge_type' => 'string',
            'cash_charge_type' => 'string',
            'cash_charge' => 'numeric',
            'card_charge_type' => 'string',
            'card_charge' => 'numeric',
            'paypal_charge_type' => 'string',
            'paypal_charge' => 'numeric',
            'per_order_charge' => 'numeric',
            'reward_option' => 'in:Yes,No',
            'service_charge' => 'numeric',
            'new' => 'string',
            'RTC' => 'in:Yes,No',
            'app_setting' => 'in:Yes,No',
            'delivered_message' => 'in:Yes,No',
            'is_allergy' => 'in:0,1',
            'allergy_message' => 'string',
            'wallet_payment' => 'in:Yes,No',
            'payment_plan' => 'in:0,1',
            'stripe_description' => 'required|string',
            'stripe_descriptor' => 'required|string',
            'invoice_period' => 'in:Yes,No',
            'servicecharge_delivery' => 'boolean',
            'servicecharge_picked' => 'boolean',
            'response_status' => 'string',
            'delete_status' => 'string',
            'status' => 'boolean',
            'close' => 'boolean',
            'is_featured' => 'boolean',
            'meta_title' => 'string',
            'meta_keyword' => 'string',
            'meta_description' => 'string',
            'food_rating' => 'string',
            'dine_in' => 'in:1,0',
            'gratuity' => 'in:1,0',
            'device_id' => 'string',
            'restaurant_timing' => 'required|array',
            'delivery_setting' => 'required_if:map_mode,Circle|array',
            'areamaps' => 'required_if:map_mode,Polygon|array',
            'bank_detail' => 'array',
            'meta_tag' => 'array',
            'image' => 'image',
        ]);

        $restaurant = new TiffintomRestaurant;
        //echo "<pre>";print_r($req->all());exit;
        $restaurant->fill($req->all());
        if ($req->has('restaurant_name')) $restaurant->seo_url = $this->seoUrl($restaurant->restaurant_name);
        $restaurant->status = '1';
        $restaurant->delete_status = "N";
        $restaurant->save();

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_RESTAURANT . $restaurant->id . '/storeLogo/', 'tiffintom');
            $restaurant->logo_name = $fileName;
        }
        $restaurant->save();

        // Pickup Timing
        if (!empty($req->input('restaurant_timing'))) {
            $req = $req->merge($req->restaurant_timing);
            $req->merge(['restaurant_id' => $restaurant->id]);
            $pickup_timings = (new TiffintomRestaurantTimingController())->create_update($req);
        }

        // Meta tag
        if (!empty($req->input('meta_tag'))) {
            $req = $req->merge($req->restaurant_timing);
            $req->merge(['restaurant_id' => $restaurant->id]);
            $metaTags = (new TiffintomRestaurantMetatagController())->create_update($req);
        }

        // Pickup TIming
        if (!empty($req->input('bank_detail'))) {
            $req = $req->merge($req->bank_detail);
            $req->merge(['restaurant_id' => $restaurant->id]);
            $bank_detail = (new TiffintomBankDetailController())->create_update($req);
        }

        // Delivery Setting Add
        if (!empty($req->input('delivery_setting'))) {
            $oldItems = TiffintomDeliverySetting::where('restaurant_id', $restaurant->id)->delete();
            $delivery_settings = ($req->input('delivery_setting'));
            $color = ["#800080", "#00FF00", "#FF00FF", "#FF0000", "#0000FF", "#808000", "#008000", "#00FFFF", "#C71585", "#B0E0E6", "#FFDAB9", "#edc50c", "#a0ed0c", "#0ceda6", "#0c66ed", "#0c17ed", "#6b0ced", "#d50ced", "#ed0cbb", "#ed0c51", "#f69d9d", "#f69de2", "#da9df6", "#b29df6", "#9dacf6", "#9de6f6", "#1b535f", "#052228", "#1e2f33", "#486166", "#0b6b7f", "#0b7f5d", "#0a5a42", "#13dca1", "#8dcebb"];
            foreach ($delivery_settings as $key => $item) {
                $deliverySettings = new TiffintomDeliverySetting((array)$item);
                $deliverySettings->restaurant_id = $restaurant->id;
                $deliverySettings->radius_color = $color[$key];
                $deliverySettings->save();
            }
        }

        // Areamaps Add
        if (!empty($req->input('areamaps'))) {
            $oldItems = TiffintomAreamap::where('res_id', $restaurant->id)->delete();
            $areamaps = ($req->input('areamaps'));
            $color = ["#800080", "#00FF00", "#FF00FF", "#FF0000", "#0000FF", "#808000", "#008000", "#00FFFF", "#C71585", "#B0E0E6", "#FFDAB9", "#edc50c", "#a0ed0c", "#0ceda6", "#0c66ed", "#0c17ed", "#6b0ced", "#d50ced", "#ed0cbb", "#ed0c51", "#f69d9d", "#f69de2", "#da9df6", "#b29df6", "#9dacf6", "#9de6f6", "#1b535f", "#052228", "#1e2f33", "#486166", "#0b6b7f", "#0b7f5d", "#0a5a42", "#13dca1", "#8dcebb"];
            foreach ($areamaps as $key => $item) {
                $deliverySettings = new TiffintomAreamap((array)$item);
                $deliverySettings->res_id = $restaurant->id;
                $deliverySettings->colorcode = $color[$key];
                $deliverySettings->save();
            }
        }

        // Payment Methods
        if (!empty($req->input('payment_methods'))) {
            $req = $req->merge($req->payment_methods);
            $oldItems = TiffintomRestaurantPaymentMethod::where('restaurant_id', $restaurant->id)->delete();
            $payment_methods = ($req->input('payment_methods'));
            foreach ($payment_methods as $item) {
                $paymentMethods = new TiffintomRestaurantPaymentMethod((array)$item);
                $paymentMethods->restaurant_id = $restaurant->id;
                $paymentMethods->save();
            }
        }

        $restaurant->append('restaurant_timing');
        $restaurant->append('bank_detail');
        $restaurant->append('delivery_setting');
        $restaurant->append('payment_methods');
        return $restaurant;
    }

    /**
     * List restaurants
     * GET /tiffintom/restaurants/id
     */
    public function show(Request $req, $id)
    {
        $restaurants = TiffintomRestaurant::find($id);
        // $restaurants = TiffintomRestaurant::where('seo_url', $id)->first();
        if (empty($restaurants)) Utils::error("No restaurant found", 404);
        $restaurants->append('restaurant_timing');
        // $restaurants->append('bank_detail');
        $restaurants->append('business');
        $restaurants->append('delivery_setting');
        $restaurants->append('payment_methods');
        $restaurants->append('site_setting');
        $restaurants->append('meta_tag');

        // Cuisine List Name Add
        $restaurants['cuisine_names'] = TiffintomCuisine::select('cuisine_name')->whereRaw('(id IN (?))', $restaurants->restaurant_cuisine)->get();;
        //$restaurants['dietary_names'] = TiffintomDietary::select('name')->whereRaw('(id IN (?))', $restaurants->restaurant_dietary)->get();;
        if(!empty($restaurants->restaurant_dietary)) {
            $restaurants['dietary_names'] = TiffintomDietary::select('name')->whereRaw('(id IN (?))', $restaurants->restaurant_dietary)->get();
        } else {
            $restaurants['dietary_names'] = "";
        }
        $currentTime = strtotime(date('h:i A'));
        $currentDay = strtolower(date('l'));

        $firstOpenTime = strtotime($restaurants[$currentDay . '_first_opentime']);
        $firstCloseTime = strtotime($restaurants[$currentDay . '_first_closetime']);

        $secondOpenTime = strtotime($restaurants[$currentDay . '_second_opentime']);
        $secondCloseTime = strtotime($restaurants[$currentDay . '_second_closetime']);

        $pickfirstOpenTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM');
        $pickfirstCloseTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM');
        $picksecondOpenTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM');
        $picksecondCloseTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM');
        $restentitytiming = isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_status']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_status'] : 'Close';

        if ($restaurants['response_status'] != 0 && ($restaurants[$currentDay . '_status'] != 'Close' || $restentitytiming != 'Close')) {
            if (($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $pickfirstCloseTime)) {
                $restaurants['currentStatus'] = 'Open';
            } else if (($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) || ($currentTime > $picksecondOpenTime && $currentTime <= $picksecondCloseTime)) {
                $restaurants['currentStatus'] = 'Open';
            } else if (($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $picksecondOpenTime)) {
                $restaurants['currentStatus'] = 'PreOrder';
            } else {
                $restaurants['currentStatus'] = 'Close';
            }


            if (($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime)) {
                $restaurants['currentDeliveryStatus'] = 'Open';
            } else if (($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime)) {
                $restaurants['currentDeliveryStatus'] = 'Open';
            } else if (($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime)) {
                $restaurants['currentDeliveryStatus'] = 'PreOrder';
            } else {
                $restaurants['currentDeliveryStatus'] = 'Close';
            }

            if (($currentTime > $pickfirstOpenTime && $currentTime <= $pickfirstCloseTime)) {
                $restaurants['currentPickupStatus'] = 'Open';
            } else if (($currentTime > $picksecondOpenTime && $currentTime <= $picksecondCloseTime)) {
                $restaurants['currentPickupStatus'] = 'Open';
            } else if (($currentTime > $pickfirstOpenTime && $currentTime <= $picksecondOpenTime)) {
                $restaurants['currentPickupStatus'] = 'PreOrder';
            } else {
                $restaurants['currentPickupStatus'] = 'Close';
            }
        } else {
            $restaurants['currentStatus'] = 'Close';
        }
        $restaurants->append('currentStatus');
        $req->merge(['restaurant_id' => $restaurants->id, 'date' => date('Y-m-d')]);
        $TimeSlots = $this->restaurant_timeslot($req);
        $deliveryTimeSlots = $TimeSlots['delivery'];
        $pickupTimeSlots = $TimeSlots['pickup'];

        if ($deliveryTimeSlots['today'] != '' || $deliveryTimeSlots['dayaftertomorrow'] != '' || $deliveryTimeSlots['tomorrow'] != '' || $pickupTimeSlots['today'] != '' || $pickupTimeSlots['dayaftertomorrow'] != '' || $pickupTimeSlots['tomorrow'] != '') {
            $restaurants['restaurant_status'] = 'Open';
        } else {
            $restaurants['restaurant_status'] = 'Closed';
        }

        $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurants->id)->where('status', '1')->first();
        if (!empty($reviews)) {
            if ($reviews->count > 0) {
                $restaurants['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                $restaurants['total_reviews'] = $reviews->count;
            }
        }
        $restaurants['suggest_product_count'] = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('is_suggest', '1')->where('price_option', 'single')->where('menu_addon', 'no')->count();

        if ($req->input("latitude") != '' && $req->input("longitude") != '') {
            $sourcelatitude  = $req->input("latitude");
            $sourcelongitude  = $req->input("longitude");
            $latitudeTo  = $restaurants->sourcelatitude;
            $longitudeTo = $restaurants->sourcelongitude;
            $unit = 'K';

            $distance = $this->getDistanceCalculate(
                $sourcelatitude,
                $sourcelongitude,
                $latitudeTo,
                $longitudeTo,
                $unit
            );

            $restaurants['distance'] = (float)$distance;

            list($deliveryCharge, $message) = $this->getDeliveryCharge($restaurants->id, $distance, $sourcelatitude, $sourcelongitude);
            $restaurants['deliveryCharges'] = $deliveryCharge;
        }

        if ($req->input("postcode") != '') {
            $postcode = Postcode::whereRaw('(post_code LIKE ? OR post_code LIKE ? )', [$req->input('postcode'), str_replace("%20", "", $req->input('postcode'))])->first();
            if (empty($postcode)) {
                $newPostcode = $this->findFromGoogleAndReturn($req->input('postcode'));
                $postcode = $newPostcode;
            }

            $sourcelatitude = $postcode->latitude ?? '';;
            $sourcelongitude = $postcode->longitude ?? '';;
            $latitudeTo  = $restaurants->sourcelatitude;
            $longitudeTo = $restaurants->sourcelongitude;
            $unit = 'K';

            if ($sourcelatitude != '' && $sourcelongitude != '' && $latitudeTo != '' && $latitudeTo != '') {
                $distance = $this->getDistanceCalculate(
                    $sourcelatitude,
                    $sourcelongitude,
                    $latitudeTo,
                    $longitudeTo,
                    $unit
                );

                $restaurants['distance'] = (float)$distance;

                list($deliveryCharge, $message) = $this->getDeliveryCharge($restaurants->id, $distance, $sourcelatitude, $sourcelongitude);
                $restaurants['deliveryCharges'] = $deliveryCharge;
            }
        }

        $halalcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('halal', '1')->count();
        $vegancount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('vegan', '1')->count();
        $vegetariancount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('vegetarian', '1')->count();
        $contains_nutscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('contains_nuts', '1')->count();
        $gluten_freecount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('gluten_free', '1')->count();
        $milkcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('milk', '1')->count();
        $crustaceanscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('crustaceans', '1')->count();
        $molluscscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('molluscs', '1')->count();
        $fishcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('fish', '1')->count();
        $sulphur_dioxidecount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('sulphur_dioxide', '1')->count();
        $eggscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('eggs', '1')->count();
        $sulphitescount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('sulphites', '1')->count();
        $mustardcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('mustard', '1')->count();
        $kosharcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('koshar', '1')->count();

        $allergies = [];
        if ($halalcount > 0) {
            $allergies[] = 'Halal';
        }
        if ($vegancount > 0) {
            $allergies[] = 'Vegan';
        }
        if ($vegetariancount > 0) {
            $allergies[] = 'Vegetarian';
        }
        if ($contains_nutscount > 0) {
            $allergies[] = 'Nuts';
        }
        if ($gluten_freecount > 0) {
            $allergies[] = 'Gluten';
        }
        if ($milkcount > 0) {
            $allergies[] = 'Dairy';
        }
        if ($crustaceanscount > 0) {
            $allergies[] = 'Crustaceans';
        }
        if ($molluscscount > 0) {
            $allergies[] = 'Molluscs';
        }
        if ($fishcount > 0) {
            $allergies[] = 'Fish';
        }
        if ($sulphur_dioxidecount > 0) {
            $allergies[] = 'Sulphur dioxide';
        }
        if ($eggscount > 0) {
            $allergies[] = 'Eggs';
        }
        if ($sulphitescount > 0) {
            $allergies[] = 'Sulphites';
        }
        if ($mustardcount > 0) {
            $allergies[] = 'Mustard';
        }
        if ($kosharcount > 0) {
            $allergies[] = 'Koshar';
        }

        if ($customer_id = $req->query("customer_id", false)) {
            $order_count = TiffintomOrder::where('customer_id', $customer_id)->where('restaurant_id', $restaurants->id)->count();
            $restaurants['order_count'] = $order_count;

            $favourite = TiffintomFavouriteRestaurant::where('customer_id', $customer_id)->where('restaurant_id', $restaurants->id)->where('is_favourite', '1')->first();
            if (!empty($favourite)) {
                $restaurants['is_favourite'] = 1;
            } else {
                $restaurants['is_favourite'] = 0;
            }
        }

        $restaurants['allergies'] = $allergies;
        $restaurants->makeHidden(['modified', 'otp', 'smart_mps_id', 'dinein_restaurant_commission', 'device_id', 'device_name', 'sms_option', 'sms_phonenumber', 'restaurant_commission', 'cash_commission', 'card_commission', 'paypal_commission', 'invoice_period', 'offer_texts', 'fb_page_id', 'dinein_stripe_description', 'dinein_stripe_descriptor',   'dinein_service_charge_status', 'dinein_restaurant_commission', 'map_mode', 'email_order', 'order_email', 'is_logged', 'delete_status', 'qr_code', 'state_id', 'country_id', 'location_id',  'restaurant_tax', 'restaurant_logo']);

        return $restaurants;
    }

    /**
     * List restaurants
     * GET /tiffintom/restaurants/name/fetch
     */
    public function show_name(Request $req, $name)
    {
        $restaurants = TiffintomRestaurant::where('seo_url', $name)->first();
        if (empty($restaurants)) Utils::error("No restaurant found", 404);
        $restaurants->append('restaurant_timing');
        // $restaurants->append('bank_detail');
        $restaurants->append('business');
        $restaurants->append('delivery_setting');
        $restaurants->append('payment_methods');
        $restaurants->append('site_setting');
        $restaurants->append('meta_tag');

        // Cuisine List Name Add
        $restaurants['cuisine_names'] = TiffintomCuisine::select('cuisine_name')->whereRaw('(id IN (?))', $restaurants->restaurant_cuisine)->get();
        if(!empty($restaurants->restaurant_dietary)) {
            $restaurants['dietary_names'] = TiffintomDietary::select('name')->whereRaw('(id IN (?))', $restaurants->restaurant_dietary)->get();
        } else {
            $restaurants['dietary_names'] = "";
        }

        $currentTime = strtotime(date('h:i A'));
        $currentDay = strtolower(date('l'));

        $firstOpenTime = strtotime($restaurants[$currentDay . '_first_opentime']);
        $firstCloseTime = strtotime($restaurants[$currentDay . '_first_closetime']);

        $secondOpenTime = strtotime($restaurants[$currentDay . '_second_opentime']);
        $secondCloseTime = strtotime($restaurants[$currentDay . '_second_closetime']);

        $pickfirstOpenTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM');
        $pickfirstCloseTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM');
        $picksecondOpenTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM');
        $picksecondCloseTime = strtotime(isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM');
        $restentitytiming = isset($restaurants['restaurant_timing']['pick_' . $currentDay . '_status']) ? $restaurants['restaurant_timing']['pick_' . $currentDay . '_status'] : 'Close';

        if ($restaurants['response_status'] != 0 && ($restaurants[$currentDay . '_status'] != 'Close' || $restentitytiming != 'Close')) {
            if (($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $pickfirstCloseTime)) {
                $restaurants['currentStatus'] = 'Open';
            } else if (($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) || ($currentTime > $picksecondOpenTime && $currentTime <= $picksecondCloseTime)) {
                $restaurants['currentStatus'] = 'Open';
            } else if (($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime) || ($currentTime > $pickfirstOpenTime && $currentTime <= $picksecondOpenTime)) {
                $restaurants['currentStatus'] = 'PreOrder';
            } else {
                $restaurants['currentStatus'] = 'Close';
            }


            if (($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime)) {
                $restaurants['currentDeliveryStatus'] = 'Open';
            } else if (($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime)) {
                $restaurants['currentDeliveryStatus'] = 'Open';
            } else if (($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime)) {
                $restaurants['currentDeliveryStatus'] = 'PreOrder';
            } else {
                $restaurants['currentDeliveryStatus'] = 'Close';
            }

            if (($currentTime > $pickfirstOpenTime && $currentTime <= $pickfirstCloseTime)) {
                $restaurants['currentPickupStatus'] = 'Open';
            } else if (($currentTime > $picksecondOpenTime && $currentTime <= $picksecondCloseTime)) {
                $restaurants['currentPickupStatus'] = 'Open';
            } else if (($currentTime > $pickfirstOpenTime && $currentTime <= $picksecondOpenTime)) {
                $restaurants['currentPickupStatus'] = 'PreOrder';
            } else {
                $restaurants['currentPickupStatus'] = 'Close';
            }
        } else {
            $restaurants['currentStatus'] = 'Close';
        }
        $restaurants->append('currentStatus');
        $req->merge(['restaurant_id' => $restaurants->id, 'date' => date('Y-m-d')]);
        $TimeSlots = $this->restaurant_timeslot($req);
        $deliveryTimeSlots = $TimeSlots['delivery'];
        $pickupTimeSlots = $TimeSlots['pickup'];

        if ($deliveryTimeSlots['today'] != '' || $deliveryTimeSlots['dayaftertomorrow'] != '' || $deliveryTimeSlots['tomorrow'] != '' || $pickupTimeSlots['today'] != '' || $pickupTimeSlots['dayaftertomorrow'] != '' || $pickupTimeSlots['tomorrow'] != '') {
            $restaurants['restaurant_status'] = 'Open';
        } else {
            $restaurants['restaurant_status'] = 'Closed';
        }

        $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurants->id)->where('status', '1')->first();
        if (!empty($reviews)) {
            if ($reviews->count > 0) {
                $restaurants['average_rating'] = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                $restaurants['total_reviews'] = $reviews->count;
            }
        }
        $restaurants['suggest_product_count'] = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('is_suggest', '1')->where('price_option', 'single')->where('menu_addon', 'no')->count();

        if ($req->input("latitude") != '' && $req->input("longitude") != '') {
            $sourcelatitude  = $req->input("latitude");
            $sourcelongitude  = $req->input("longitude");
            $latitudeTo  = $restaurants->sourcelatitude;
            $longitudeTo = $restaurants->sourcelongitude;
            $unit = 'K';

            $distance = $this->getDistanceCalculate(
                $sourcelatitude,
                $sourcelongitude,
                $latitudeTo,
                $longitudeTo,
                $unit
            );

            $restaurants['distance'] = (float)$distance;

            list($deliveryCharge, $message) = $this->getDeliveryCharge($restaurants->id, $distance, $sourcelatitude, $sourcelongitude);
            $restaurants['deliveryCharges'] = $deliveryCharge;
        }

        if ($req->input("postcode") != '') {
            $postcode = Postcode::whereRaw('(post_code LIKE ? OR post_code LIKE ? )', [$req->input('postcode'), str_replace("%20", "", $req->input('postcode'))])->first();
            if (empty($postcode)) {
                $newPostcode = $this->findFromGoogleAndReturn($req->input('postcode'));
                $postcode = $newPostcode;
            }

            $sourcelatitude = $postcode->latitude;
            $sourcelongitude = $postcode->longitude;
            $latitudeTo  = $restaurants->sourcelatitude;
            $longitudeTo = $restaurants->sourcelongitude;
            $unit = 'K';

            $distance = $this->getDistanceCalculate(
                $sourcelatitude,
                $sourcelongitude,
                $latitudeTo,
                $longitudeTo,
                $unit
            );

            $restaurants['distance'] = (float)$distance;

            list($deliveryCharge, $message) = $this->getDeliveryCharge($restaurants->id, $distance, $sourcelatitude, $sourcelongitude);
            $restaurants['deliveryCharges'] = $deliveryCharge;
        }

        $halalcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('halal', '1')->count();
        $vegancount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('vegan', '1')->count();
        $vegetariancount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('vegetarian', '1')->count();
        $contains_nutscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('contains_nuts', '1')->count();
        $gluten_freecount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('gluten_free', '1')->count();
        $milkcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('milk', '1')->count();
        $crustaceanscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('crustaceans', '1')->count();
        $molluscscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('molluscs', '1')->count();
        $fishcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('fish', '1')->count();
        $sulphur_dioxidecount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('sulphur_dioxide', '1')->count();
        $eggscount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('eggs', '1')->count();
        $sulphitescount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('sulphites', '1')->count();
        $mustardcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('mustard', '1')->count();
        $kosharcount = TiffintomMenu::where('restaurant_id', $restaurants->id)->where('koshar', '1')->count();

        $allergies = [];
        if ($halalcount > 0) {
            $allergies[] = 'Halal';
        }
        if ($vegancount > 0) {
            $allergies[] = 'Vegan';
        }
        if ($vegetariancount > 0) {
            $allergies[] = 'Vegetarian';
        }
        if ($contains_nutscount > 0) {
            $allergies[] = 'Nuts';
        }
        if ($gluten_freecount > 0) {
            $allergies[] = 'Gluten';
        }
        if ($milkcount > 0) {
            $allergies[] = 'Dairy';
        }
        if ($crustaceanscount > 0) {
            $allergies[] = 'Crustaceans';
        }
        if ($molluscscount > 0) {
            $allergies[] = 'Molluscs';
        }
        if ($fishcount > 0) {
            $allergies[] = 'Fish';
        }
        if ($sulphur_dioxidecount > 0) {
            $allergies[] = 'Sulphur dioxide';
        }
        if ($eggscount > 0) {
            $allergies[] = 'Eggs';
        }
        if ($sulphitescount > 0) {
            $allergies[] = 'Sulphites';
        }
        if ($mustardcount > 0) {
            $allergies[] = 'Mustard';
        }
        if ($kosharcount > 0) {
            $allergies[] = 'Koshar';
        }

        if ($customer_id = $req->query("customer_id", false)) {
            $order_count = TiffintomOrder::where('customer_id', $customer_id)->where('restaurant_id', $restaurants->id)->count();
            $restaurants['order_count'] = $order_count;

            $favourite = TiffintomFavouriteRestaurant::where('customer_id', $customer_id)->where('restaurant_id', $restaurants->id)->where('is_favourite', '1')->first();
            if (!empty($favourite)) {
                $restaurants['is_favourite'] = 1;
            } else {
                $restaurants['is_favourite'] = 0;
            }
        }

        $restaurants['allergies'] = $allergies;
        $restaurants->makeHidden(['modified', 'otp', 'smart_mps_id', 'dinein_restaurant_commission', 'device_id', 'device_name', 'sms_option', 'sms_phonenumber', 'restaurant_commission', 'cash_commission', 'card_commission', 'paypal_commission', 'invoice_period', 'offer_texts', 'fb_page_id', 'dinein_stripe_description', 'dinein_stripe_descriptor',   'dinein_service_charge_status', 'dinein_restaurant_commission', 'map_mode', 'email_order', 'order_email', 'is_logged', 'delete_status', 'qr_code', 'state_id', 'country_id', 'location_id',  'restaurant_tax', 'restaurant_logo']);

        return $restaurants;
    }


    /**
     * List restaurants
     * GET /tiffintom/restaurants/id/all
     */
    public function show_all(Request $req, $id)
    {
        $restaurants = TiffintomRestaurant::find($id);
        if (empty($restaurants)) Utils::error("No restaurant found", 404);
        $restaurants->append('restaurant_timing');
        $restaurants->append('bank_detail');
        $restaurants->append('business');
        $restaurants->append('delivery_setting');
        $restaurants->append('areamaps');
        $restaurants->append('payment_methods');
        // $restaurants->append('site_setting');
        $restaurants->append('meta_tag');
        return $restaurants;
    }

    /**
     * Update restaurant.
     * POST  /tiffintom/restaurants/id
     */
    public function update(Request $req, $id)
    {
        $restaurant = TiffintomRestaurant::find($id);
        if (empty($restaurant)) Utils::error('Restaurant not found!', 404);

        if ($req->has('restaurant_timing') && !is_array($req->restaurant_timing))
            $req->merge(['restaurant_timing' => (json_decode($req->restaurant_timing, true))]);

        if ($req->has('bank_detail') && !is_array($req->bank_detail))
            $req->merge(['bank_detail' => (json_decode($req->bank_detail, true))]);

        if ($req->has('delivery_setting') && !is_array($req->delivery_setting))
            $req->merge(['delivery_setting' => (json_decode($req->delivery_setting, true))]);

        if ($req->has('areamaps') && !is_array($req->areamaps))
            $req->merge(['areamaps' => (json_decode($req->areamaps, true))]);

        if ($req->has('meta_tag') && !is_array($req->meta_tag))
            $req->merge(['meta_tag' => (json_decode($req->meta_tag, true))]);

        if ($req->has('payment_methods') && !is_array($req->payment_methods))
            $req->merge(['payment_methods' => (json_decode($req->payment_methods, true))]);

        $this->validate($req, [
            'contact_name' => 'required|string|max:255',
            'contact_phone' => 'required|string',
            'contact_email' => 'required|string',
            'contact_address' => 'required|string',
            'city_id' => 'required|string',
            'currency' => 'required|string',
            'timezone' => 'required|string',
            'currency_code' => 'required|string',
            'zipcode' => 'string',
            'restaurant_name' => 'required|string',
            'restaurant_phone' => 'required|string',
            'street_address' => 'required|string',
            'working_time' => 'in:morning,evening,both',
            'monday_first_opentime' => 'required|string',
            'monday_first_closetime' => 'required|string',
            'monday_second_opentime' => 'required|string',
            'monday_second_closetime' => 'required|string',
            'tuesday_first_opentime' => 'required|string',
            'tuesday_first_closetime' => 'required|string',
            'tuesday_second_opentime' => 'required|string',
            'tuesday_second_closetime' => 'required|string',
            'wednesday_first_opentime' => 'required|string',
            'wednesday_first_closetime' => 'required|string',
            'wednesday_second_opentime' => 'required|string',
            'wednesday_second_closetime' => 'required|string',
            'thursday_first_opentime' => 'required|string',
            'thursday_first_closetime' => 'required|string',
            'thursday_second_opentime' => 'required|string',
            'thursday_second_closetime' => 'required|string',
            'friday_first_opentime' => 'required|string',
            'friday_first_closetime' => 'required|string',
            'friday_second_opentime' => 'required|string',
            'friday_second_closetime' => 'required|string',
            'saturday_first_opentime' => 'required|string',
            'saturday_first_closetime' => 'required|string',
            'saturday_second_opentime' => 'required|string',
            'saturday_second_closetime' => 'required|string',
            'sunday_first_opentime' => 'required|string',
            'sunday_first_closetime' => 'required|string',
            'sunday_second_opentime' => 'required|string',
            'sunday_second_closetime' => 'required|string',
            'monday_status' => 'string',
            'tuesday_status' => 'string',
            'wednesday_status' => 'string',
            'thursday_status' => 'string',
            'friday_status' => 'string',
            'saturday_status' => 'string',
            'sunday_status' => 'string',
            'facebook_login' => 'in:Y,N',
            'google_login' => 'in:Y,N',
            'twitter_login' => 'in:Y,N',
            'apple_login' => 'in:Y,N',
            'alcahol_available' => 'boolean',
            'alcahol_not_available' => 'boolean',
            'bring_your_alcahol' => 'boolean',
            'soft_drink' => 'boolean',
            'restaurant_tax' => 'string',
            'pickup_estimate_time' => 'string',
            'restaurant_cuisine' => 'string',
            'restaurant_dietary' => 'string',
            'restaurant_visibility' => 'in:FOS,external,both',
            'restaurant_dispatch' => 'in:Yes,No',
            'online_order' => 'in:Yes,No',
            'restaurant_pickup' => 'in:Yes,No',
            'restaurant_delivery' => 'in:Yes,No',
            'restaurant_booktable' => 'in:Yes,No',
            'booking_status' => 'in:Yes,No',
            'image_type' => 'in:Yes,No',
            'restaurant_about' => 'string',
            'username' => 'required|string',
            'password' => 'string',
            'delivery_charge' => 'in:Yes,No',
            'estimate_time' => 'string',
            'minimum_order' => 'string',
            'paypal_minimum_order' => 'string',
            'card_minimum_order' => 'string',
            'free_delivery' => 'string',
            'booking_start_days' => 'string',
            'booking_start' => 'string',
            'booking_end' => 'string',
            'no_of_guest' => 'string',
            'map_mode' => 'in:Circle,Polygon',
            'email_order' => 'in:Yes,No',
            'order_email' => 'string',
            'sms_option' => 'in:Yes,No',
            'sms_phonenumber' => 'string',
            'service_charge_type' => 'string',
            'restaurant_commission' => 'numeric',
            'cash_commission' => 'numeric',
            'card_commission' => 'numeric',
            'paypal_commission' => 'numeric',
            'service_charge_status' => 'in:Yes,No',
            'order_charge_type' => 'string',
            'cash_charge_type' => 'string',
            'cash_charge' => 'numeric',
            'card_charge_type' => 'string',
            'card_charge' => 'numeric',
            'paypal_charge_type' => 'string',
            'paypal_charge' => 'numeric',
            'per_order_charge' => 'numeric',
            'reward_option' => 'in:Yes,No',
            'service_charge' => 'numeric',
            'new' => 'string',
            'RTC' => 'in:Yes,No',
            'app_setting' => 'in:Yes,No',
            'delivered_message' => 'in:Yes,No',
            'is_allergy' => 'in:0,1',
            'allergy_message' => 'string',
            'wallet_payment' => 'in:Yes,No',
            'payment_plan' => 'in:0,1',
            'stripe_description' => 'required|string',
            'stripe_descriptor' => 'required|string',
            'invoice_period' => 'in:Yes,No',
            'servicecharge_delivery' => 'boolean',
            'servicecharge_picked' => 'boolean',
            'response_status' => 'string',
            'delete_status' => 'string',
            'status' => 'boolean',
            'close' => 'boolean',
            'is_featured' => 'boolean',
            'meta_title' => 'string',
            'meta_keyword' => 'string',
            'meta_description' => 'string',
            'food_rating' => 'string',
            'dine_in' => 'in:1,0',
            'gratuity' => 'in:1,0',
            'device_id' => 'string',
            'restaurant_timing' => 'required|array',
            'delivery_setting' => 'required_if:map_mode,Circle|array',
            'areamaps' => 'required_if:map_mode,Polygon|array',
            'bank_detail' => 'array',
            'meta_tag' => 'array',
            'image' => 'image',
        ]);

        // update business
        if ($req->has('contact_name')) $restaurant->contact_name = $req->input('contact_name');
        if ($req->has('contact_phone')) $restaurant->contact_phone = $req->input('contact_phone');
        if ($req->has('contact_email')) $restaurant->contact_email = $req->input('contact_email');
        if ($req->has('contact_address')) $restaurant->contact_address = $req->input('contact_address');
        if ($req->has('city_id')) $restaurant->city_id = $req->input('city_id');
        if ($req->has('currency')) $restaurant->currency = $req->input('currency');
        if ($req->has('timezone')) $restaurant->timezone = $req->input('timezone');
        if ($req->has('zipcode')) $restaurant->zipcode = $req->input('zipcode');

        if ($req->has('restaurant_name')) $restaurant->restaurant_name = $req->input('restaurant_name');
        if ($req->has('restaurant_name')) $restaurant->seo_url = $this->seoUrl($restaurant->restaurant_name);
        if ($req->has('restaurant_phone')) $restaurant->restaurant_phone = $req->input('restaurant_phone');
        if ($req->has('street_address')) $restaurant->street_address = $req->input('street_address');
        // Delivery Timing
        if ($req->has('working_time')) $restaurant->working_time = $req->input('working_time');
        if ($req->has('monday_first_opentime')) $restaurant->monday_first_opentime = $req->input('monday_first_opentime');
        if ($req->has('monday_first_closetime')) $restaurant->monday_first_closetime = $req->input('monday_first_closetime');
        if ($req->has('monday_second_opentime')) $restaurant->monday_second_opentime = $req->input('monday_second_opentime');
        if ($req->has('monday_second_closetime')) $restaurant->monday_second_closetime = $req->input('monday_second_closetime');
        if ($req->has('tuesday_first_opentime')) $restaurant->tuesday_first_opentime = $req->input('tuesday_first_opentime');
        if ($req->has('tuesday_first_closetime')) $restaurant->tuesday_first_closetime = $req->input('tuesday_first_closetime');
        if ($req->has('tuesday_second_opentime')) $restaurant->tuesday_second_opentime = $req->input('tuesday_second_opentime');
        if ($req->has('tuesday_second_closetime')) $restaurant->tuesday_second_closetime = $req->input('tuesday_second_closetime');
        if ($req->has('wednesday_first_opentime')) $restaurant->wednesday_first_opentime = $req->input('wednesday_first_opentime');
        if ($req->has('wednesday_first_closetime')) $restaurant->wednesday_first_closetime = $req->input('wednesday_first_closetime');
        if ($req->has('wednesday_second_opentime')) $restaurant->wednesday_second_opentime = $req->input('wednesday_second_opentime');
        if ($req->has('wednesday_second_closetime')) $restaurant->wednesday_second_closetime = $req->input('wednesday_second_closetime');
        if ($req->has('thursday_first_opentime')) $restaurant->thursday_first_opentime = $req->input('thursday_first_opentime');
        if ($req->has('thursday_first_closetime')) $restaurant->thursday_first_closetime = $req->input('thursday_first_closetime');
        if ($req->has('thursday_second_opentime')) $restaurant->thursday_second_opentime = $req->input('thursday_second_opentime');
        if ($req->has('thursday_second_closetime')) $restaurant->thursday_second_closetime = $req->input('thursday_second_closetime');
        if ($req->has('friday_first_opentime')) $restaurant->friday_first_opentime = $req->input('friday_first_opentime');
        if ($req->has('friday_first_closetime')) $restaurant->friday_first_closetime = $req->input('friday_first_closetime');
        if ($req->has('friday_second_opentime')) $restaurant->friday_second_opentime = $req->input('friday_second_opentime');
        if ($req->has('friday_second_closetime')) $restaurant->friday_second_closetime = $req->input('friday_second_closetime');
        if ($req->has('saturday_first_opentime')) $restaurant->saturday_first_opentime = $req->input('saturday_first_opentime');
        if ($req->has('saturday_first_closetime')) $restaurant->saturday_first_closetime = $req->input('saturday_first_closetime');
        if ($req->has('saturday_second_opentime')) $restaurant->saturday_second_opentime = $req->input('saturday_second_opentime');
        if ($req->has('saturday_second_closetime')) $restaurant->saturday_second_closetime = $req->input('saturday_second_closetime');
        if ($req->has('sunday_first_opentime')) $restaurant->sunday_first_opentime = $req->input('sunday_first_opentime');
        if ($req->has('sunday_first_closetime')) $restaurant->sunday_first_closetime = $req->input('sunday_first_closetime');
        if ($req->has('sunday_second_opentime')) $restaurant->sunday_second_opentime = $req->input('sunday_second_opentime');
        if ($req->has('sunday_second_closetime')) $restaurant->sunday_second_closetime = $req->input('sunday_second_closetime');
        if ($req->has('monday_status')) $restaurant->monday_status = $req->input('monday_status');
        if ($req->has('tuesday_status')) $restaurant->tuesday_status = $req->input('tuesday_status');
        if ($req->has('wednesday_status')) $restaurant->wednesday_status = $req->input('wednesday_status');
        if ($req->has('thursday_status')) $restaurant->thursday_status = $req->input('thursday_status');
        if ($req->has('friday_status')) $restaurant->friday_status = $req->input('friday_status');
        if ($req->has('saturday_status')) $restaurant->saturday_status = $req->input('saturday_status');
        if ($req->has('sunday_status')) $restaurant->sunday_status = $req->input('sunday_status');
        if ($req->has('facebook_login')) $restaurant->facebook_login = $req->input('facebook_login');
        if ($req->has('google_login')) $restaurant->google_login = $req->input('google_login');
        if ($req->has('twitter_login')) $restaurant->twitter_login = $req->input('twitter_login');
        if ($req->has('apple_login')) $restaurant->apple_login = $req->input('apple_login');
        $restaurant->alcahol_available = $req->input('alcahol_available');
        $restaurant->alcahol_not_available = $req->input('alcahol_not_available');
        $restaurant->bring_your_alcahol = $req->input('bring_your_alcahol');
        $restaurant->soft_drink = $req->input('soft_drink');
        $restaurant->restaurant_tax  = $req->input('restaurant_tax');
        $restaurant->pickup_estimate_time  = $req->input('pickup_estimate_time');
        if ($req->has('restaurant_cuisine')) $restaurant->restaurant_cuisine = $req->input('restaurant_cuisine');
        if ($req->has('restaurant_dietary')) $restaurant->restaurant_dietary = $req->input('restaurant_dietary');
        if ($req->has('restaurant_visibility')) $restaurant->restaurant_visibility = $req->input('restaurant_visibility');
        if ($req->has('restaurant_dispatch')) $restaurant->restaurant_dispatch = $req->input('restaurant_dispatch');
        if ($req->has('online_order')) $restaurant->online_order = $req->input('online_order');
        if ($req->has('restaurant_pickup')) $restaurant->restaurant_pickup = $req->input('restaurant_pickup');
        if ($req->has('restaurant_delivery')) $restaurant->restaurant_delivery = $req->input('restaurant_delivery');
        if ($req->has('restaurant_booktable')) $restaurant->restaurant_booktable = $req->input('restaurant_booktable');
        if ($req->has('image_type')) $restaurant->image_type = $req->input('image_type');
        if ($req->has('booking_status')) $restaurant->booking_status = $req->input('booking_status');
        if ($req->has('restaurant_about')) $restaurant->restaurant_about = $req->input('restaurant_about');
        if ($req->has('username')) $restaurant->username = $req->input('username');

        if ($req->has('delivery_charge')) $restaurant->delivery_charge = $req->input('delivery_charge');
        if ($req->has('estimate_time')) $restaurant->estimate_time = $req->input('estimate_time');
        if ($req->has('minimum_order')) $restaurant->minimum_order = $req->input('minimum_order');
        if ($req->has('paypal_minimum_order')) $restaurant->paypal_minimum_order = $req->input('paypal_minimum_order');
        if ($req->has('card_minimum_order')) $restaurant->card_minimum_order = $req->input('card_minimum_order');
        if ($req->has('free_delivery')) $restaurant->free_delivery = $req->input('free_delivery');
        if ($req->has('booking_start_days')) $restaurant->booking_start_days = $req->input('booking_start_days');
        if ($req->has('booking_start')) $restaurant->booking_start = $req->input('booking_start');
        if ($req->has('booking_end')) $restaurant->booking_end = $req->input('booking_end');
        if ($req->has('delivery_charge')) $restaurant->delivery_charge = $req->input('delivery_charge');
        if ($req->has('no_of_guest')) $restaurant->no_of_guest = $req->input('no_of_guest');
        if ($req->has('map_mode')) $restaurant->map_mode = $req->input('map_mode');

        if ($req->has('email_order')) $restaurant->email_order = $req->input('email_order');
        if ($req->has('order_email')) $restaurant->order_email = $req->input('order_email');
        if ($req->has('sms_option')) $restaurant->sms_option = $req->input('sms_option');
        if ($req->has('sms_phonenumber')) $restaurant->sms_phonenumber = $req->input('sms_phonenumber');

        if ($req->has('service_charge_type')) $restaurant->service_charge_type = $req->input('service_charge_type');
        if ($req->has('restaurant_commission')) $restaurant->restaurant_commission = $req->input('restaurant_commission');
        if ($req->has('cash_commission')) $restaurant->cash_commission = $req->input('cash_commission');
        if ($req->has('card_commission')) $restaurant->card_commission = $req->input('card_commission');
        if ($req->has('paypal_commission')) $restaurant->paypal_commission = $req->input('paypal_commission');
        if ($req->has('service_charge_status')) $restaurant->service_charge_status = $req->input('service_charge_status');
        if ($req->has('order_charge_type')) $restaurant->order_charge_type = $req->input('order_charge_type');
        if ($req->has('cash_charge_type')) $restaurant->cash_charge_type = $req->input('cash_charge_type');
        if ($req->has('cash_charge')) $restaurant->cash_charge = $req->input('cash_charge');
        if ($req->has('card_charge_type')) $restaurant->card_charge_type = $req->input('card_charge_type');
        if ($req->has('card_charge')) $restaurant->card_charge = $req->input('card_charge');
        if ($req->has('paypal_charge_type')) $restaurant->paypal_charge_type = $req->input('paypal_charge_type');
        if ($req->has('paypal_charge')) $restaurant->paypal_charge = $req->input('paypal_charge');
        if ($req->has('per_order_charge')) $restaurant->per_order_charge = $req->input('per_order_charge');
        if ($req->has('reward_option')) $restaurant->reward_option = $req->input('reward_option');
        if ($req->has('service_charge')) $restaurant->service_charge = $req->input('service_charge');
        if ($req->has('new')) $restaurant->new = $req->input('new');
        if ($req->has('RTC')) $restaurant->RTC = $req->input('RTC');
        if ($req->has('app_setting')) $restaurant->app_setting = $req->input('app_setting');
        if ($req->has('delivered_message')) $restaurant->delivered_message = $req->input('delivered_message');
        if ($req->has('is_allergy')) $restaurant->is_allergy = $req->input('is_allergy');
        if ($req->has('allergy_message')) $restaurant->allergy_message = $req->input('allergy_message');

        if ($req->has('wallet_payment')) $restaurant->wallet_payment = $req->input('wallet_payment');
        if ($req->has('payment_plan')) $restaurant->payment_plan = $req->input('payment_plan');
        if ($req->has('stripe_description')) $restaurant->stripe_description = $req->input('stripe_description');
        if ($req->has('stripe_descriptor')) $restaurant->stripe_descriptor = $req->input('stripe_descriptor');

        if ($req->has('invoice_period')) $restaurant->invoice_period = $req->input('invoice_period');

        if ($req->has('response_status')) $restaurant->response_status = $req->input('response_status');
        if ($req->has('delete_status')) $restaurant->delete_status = $req->input('delete_status');

        if ($req->has('meta_title')) $restaurant->meta_title = $req->input('meta_title');
        if ($req->has('meta_keyword')) $restaurant->meta_keyword = $req->input('meta_keyword');
        if ($req->has('meta_description')) $restaurant->meta_description = $req->input('meta_description');
        if ($req->has('food_rating')) $restaurant->food_rating = $req->input('food_rating');

        $restaurant->gratuity = $req->input('gratuity');
        $restaurant->dine_in = $req->input('dine_in');
        $restaurant->servicecharge_delivery = $req->input('servicecharge_delivery');
        $restaurant->servicecharge_picked = $req->input('servicecharge_picked');
        $restaurant->status = $req->input('status');
        $restaurant->close = $req->input('close');
        $restaurant->is_featured = $req->input('is_featured');
        // if ($req->has('device_id')) $restaurant->device_id = $req->input('device_id');

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_RESTAURANT . $restaurant->id . '/storeLogo/', 'tiffintom');
            $restaurant->logo_name = $fileName;
        }
        $restaurant->save();

        // Pickup Timing
        if (!empty($req->input('restaurant_timing'))) {
            $req = $req->merge($req->restaurant_timing);
            $pickup_timings = (new TiffintomRestaurantTimingController())->create_update($req);
        }

        // Meta tag
        if (!empty($req->input('meta_tag'))) {
            $req = $req->merge($req->meta_tag);
            $metaTags = (new TiffintomRestaurantMetatagController())->create_update($req);
        }

        // Pickup TIming
        if (!empty($req->input('bank_detail'))) {
            $req = $req->merge($req->bank_detail);
            $bank_detail = (new TiffintomBankDetailController())->create_update($req);
        }

        // Delivery Setting Add
        if (!empty($req->input('delivery_setting'))) {
            $oldItems = TiffintomDeliverySetting::where('restaurant_id', $restaurant->id)->delete();
            $delivery_settings = ($req->input('delivery_setting'));
            $color = ["#800080", "#00FF00", "#FF00FF", "#FF0000", "#0000FF", "#808000", "#008000", "#00FFFF", "#C71585", "#B0E0E6", "#FFDAB9", "#edc50c", "#a0ed0c", "#0ceda6", "#0c66ed", "#0c17ed", "#6b0ced", "#d50ced", "#ed0cbb", "#ed0c51", "#f69d9d", "#f69de2", "#da9df6", "#b29df6", "#9dacf6", "#9de6f6", "#1b535f", "#052228", "#1e2f33", "#486166", "#0b6b7f", "#0b7f5d", "#0a5a42", "#13dca1", "#8dcebb"];
            foreach ($delivery_settings as $key => $item) {
                $deliverySettings = new TiffintomDeliverySetting((array)$item);
                $deliverySettings->restaurant_id = $restaurant->id;
                $deliverySettings->radius_color = $color[$key];
                $deliverySettings->save();
            }
        }

        // Areamaps Add
        if (!empty($req->input('areamaps'))) {
            $oldItems = TiffintomAreamap::where('res_id', $restaurant->id)->delete();
            $areamaps = ($req->input('areamaps'));
            $color = ["#800080", "#00FF00", "#FF00FF", "#FF0000", "#0000FF", "#808000", "#008000", "#00FFFF", "#C71585", "#B0E0E6", "#FFDAB9", "#edc50c", "#a0ed0c", "#0ceda6", "#0c66ed", "#0c17ed", "#6b0ced", "#d50ced", "#ed0cbb", "#ed0c51", "#f69d9d", "#f69de2", "#da9df6", "#b29df6", "#9dacf6", "#9de6f6", "#1b535f", "#052228", "#1e2f33", "#486166", "#0b6b7f", "#0b7f5d", "#0a5a42", "#13dca1", "#8dcebb"];
            foreach ($areamaps as $key => $item) {
                $deliverySettings = new TiffintomAreamap((array)$item);
                $deliverySettings->res_id = $restaurant->id;
                $deliverySettings->colorcode = $color[$key];
                $deliverySettings->save();
            }
        }

        // Payment Methods
        if (!empty($req->input('payment_methods'))) {
            $req = $req->merge($req->payment_methods);
            $oldItems = TiffintomRestaurantPaymentMethod::where('restaurant_id', $restaurant->id)->delete();
            $payment_methods = ($req->input('payment_methods'));
            foreach ($payment_methods as $item) {
                $paymentMethods = new TiffintomRestaurantPaymentMethod((array)$item);
                $paymentMethods->restaurant_id = $restaurant->id;
                $paymentMethods->save();
            }
        }

        $restaurant->append('restaurant_timing');
        $restaurant->append('bank_detail');
        $restaurant->append('delivery_setting');
        $restaurant->append('payment_methods');

        return $restaurant;
    }

    /**
     * Delete restaurant by id.
     * DELETE /tiffintom/restaurants/{id}
     */
    public function delete(Request $req, $id)
    {
        $restaurant = TiffintomRestaurant::find($id);
        if (empty($restaurant)) Utils::error('restaurant not found!', 404);
        $restaurant->delete();
        return $restaurant;
    }

    /**
     * Enable restaurant.
     * POST /tiffintom/restaurants/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $restaurant = TiffintomRestaurant::find($id);
        if (empty($restaurant)) Utils::error('restaurant not found!', 404);
        $restaurant->status = '1';
        $restaurant->save();
        return $restaurant;
    }

    /**
     * Disable restaurant.
     * POST /tiffintom/restaurants/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $restaurant = TiffintomRestaurant::find($id);
        if (empty($restaurant)) Utils::error('restaurant not found!', 404);
        $restaurant->status = '0';
        $restaurant->save();
        return $restaurant;
    }

    /**
     * Enable restaurant.
     * POST /tiffintom/restaurants/{id}/enable
     */
    public function enable_featured(Request $req, $id)
    {
        $restaurant = TiffintomRestaurant::find($id);
        if (empty($restaurant)) Utils::error('restaurant not found!', 404);
        $restaurant->is_featured = '1';
        $restaurant->save();
        return $restaurant;
    }

    /**
     * Disable restaurant.
     * POST /tiffintom/restaurants/{id}/disable
     */
    public function disable_featured(Request $req, $id)
    {
        $restaurant = TiffintomRestaurant::find($id);
        if (empty($restaurant)) Utils::error('restaurant not found!', 404);
        $restaurant->is_featured = '0';
        $restaurant->save();
        return $restaurant;
    }

    /**
     * Update restaurant.
     * POST  /tiffintom/restaurants/time-slot
     */
    public function restaurant_timeslot(Request $req)
    {
        // $restaurants = TiffintomRestaurant::find($req->query("restaurant_id"));
        // if (empty($restaurants)) Utils::error("No restaurant found", 404);
        // $restaurants->append('restaurant_timing');

        $deliveryTimeSlots = $this->getTimeSlot($req->query("date"), "delivery", $req->query("restaurant_id"));
        $pickupTimeSlots = $this->getTimeSlot($req->query("date"), "pickup", $req->query("restaurant_id"));

        $timeSlot = array();
        $timeSlot['delivery'] = $deliveryTimeSlots;
        $timeSlot['pickup'] = $pickupTimeSlots;

        return $timeSlot;
    }

    public function getTimeSlot($date, $type, $resid)
    {
        if ($date != '') {
            $restaurantDetails = TiffintomRestaurant::find($resid);
            if (empty($restaurantDetails)) Utils::error("No restaurant found", 404);
            $restaurantDetails->append('restaurant_timing');

            //Timing Section
            $nowTime = date('h:i A');
            $currentTime = strtotime($nowTime);
            $currentDates = date("d-m-Y", strtotime($date));
            $carbonDate = Carbon::parse($date);
            $tomorrow = $carbonDate->addDays(1)->format('d-m-Y');
            $dayaftertomorrow = $carbonDate->addDays(2)->format('d-m-Y');
            $timesArray = array($currentDates, $tomorrow, $dayaftertomorrow);
            $timeSlots = array();
            $deliveryTimeSlots = array();
            $pickupTimeSlots = array();
            foreach ($timesArray as $key => $date) {
                $today = date('d-m-Y');
                $currentDate = date("d-m-Y", strtotime($date));

                $currentDay = strtolower(date('l', strtotime($currentDate)));
                if ($type == 'delivery' || $type == '') {
                    $firstStartTime = $restaurantDetails[$currentDay . '_first_opentime'];
                    $firstEndTime = $restaurantDetails[$currentDay . '_first_closetime'];
                    $secondStartTime = $restaurantDetails[$currentDay . '_second_opentime'];
                    $secondEndTime = $restaurantDetails[$currentDay . '_second_closetime'];

                    $firstOpenTime = strtotime($restaurantDetails[$currentDay . '_first_opentime']);
                    $firstCloseTime = strtotime($restaurantDetails[$currentDay . '_first_closetime']);
                    $secondOpenTime = strtotime($restaurantDetails[$currentDay . '_second_opentime']);
                    $secondCloseTime = strtotime($restaurantDetails[$currentDay . '_second_closetime']);
                    $restentitytiming =    $restaurantDetails[$currentDay . '_status'];
                    $estimate_time = ($restaurantDetails[$currentDay . '_estimate_time'] > 0) ? $restaurantDetails[$currentDay . '_estimate_time'] : $restaurantDetails['estimate_time'];
                } else {

                    $firstStartTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM';
                    $firstEndTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM';
                    $secondStartTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM';
                    $secondEndTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM';

                    $firstOpenTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM');
                    $firstCloseTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM');
                    $secondOpenTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM');
                    $secondCloseTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM');
                    $restentitytiming = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_status']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_status'] : 'Close';
                    $estimate_time = 0;
                }
                //In first Timing section
                $array_of_time = array();

                if ($restentitytiming != 'Close' && $restaurantDetails['close'] != 0) {

                    $restaurantDetails['currentStatus'] = 'Open';
                    $final[] = $restaurantDetails;
                    if (strtotime($today) != strtotime($currentDate)) {
                        $nowTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $nowTime);
                        $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            // $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                            if ($firstStartTime != $firstEndTime) {
                                if ($counts == 0) {
                                    $fifteen_minsss = $estimate_time * 60;
                                    $start_time += $fifteen_minsss;
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $counts++;
                                } else {
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $counts++;
                                }
                            }
                            $start_time += $fifteen_mins;
                        }

                        $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                        $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                        $fifteen_mins = 15 * 60;
                        $countsd = 0;

                        while ($start_time <= $end_time) {
                            // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                            if ($secondStartTime != $secondEndTime) {
                                if ($countsd == 0) {
                                    $fifteen_minsss = $estimate_time * 60;
                                    $start_time += $fifteen_minsss;
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $countsd++;
                                } else {
                                    $array_of_time[] = date("h:i A", $start_time);
                                    $countsd++;
                                }
                            }
                            $start_time += $fifteen_mins;
                        }
                    } else {
                        if ($currentTime < $firstOpenTime) {
                            $nowTimes = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $nowTimes);
                            $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    // $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                    if ($firstStartTime != $firstEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                        if ($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) {
                            $firstOpenTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $firstOpenTime);
                            $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    // $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                    if ($firstStartTime != $firstEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                            $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                            $start_time1 = strtotime($currentDate . ' ' . $secondStartTimes);
                            $end_time1 = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $countsd = 0;
                            while ($start_time1 <= $end_time1) {
                                if ($currentTime <= $start_time1) {
                                    // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStartTime != $secondEndTime) {
                                        if ($countsd == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time1 += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time1);
                                            $countsd++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time1);
                                            $countsd++;
                                        }
                                    }
                                }
                                $start_time1 += $fifteen_mins;
                            }
                        }
                        if (empty($array_of_time)) {
                            if ($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime) {
                                $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                                $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                                $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                                $fifteen_mins = 15 * 60;
                                $counts = 0;
                                while ($start_time <= $end_time) {
                                    if ($currentTime <= $start_time) {
                                        // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                        if ($secondStartTime != $secondEndTime) {
                                            if ($counts == 0) {
                                                $fifteen_minsss = $estimate_time * 60;
                                                $start_time += $fifteen_minsss;
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $counts++;
                                            } else {
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $counts++;
                                            }
                                        }
                                    }
                                    $start_time += $fifteen_mins;
                                }
                            } else {
                                if ($currentTime < $secondOpenTime) {
                                    $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                                    $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                                    $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                                    $fifteen_mins = 15 * 60;
                                    $countsd = 0;
                                    while ($start_time <= $end_time) {
                                        if ($currentTime <= $start_time) {
                                            // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                            if ($secondStartTime != $secondEndTime) {
                                                if ($countsd == 0) {
                                                    $fifteen_minsss = $estimate_time * 60;
                                                    $start_time += $fifteen_minsss;
                                                    $array_of_time[] = date("h:i A", $start_time);
                                                    $countsd++;
                                                } else {
                                                    $array_of_time[] = date("h:i A", $start_time);
                                                    $countsd++;
                                                }
                                            }
                                        }
                                        $start_time += $fifteen_mins;
                                    }
                                }
                            }
                        } else {
                            if ($currentTime < $secondOpenTime) {
                                $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                                $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                                $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                                $fifteen_mins = 15 * 60;
                                $countsd = 0;
                                while ($start_time <= $end_time) {
                                    if ($currentTime <= $start_time) {
                                        // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                        if ($secondStartTime != $secondEndTime) {
                                            if ($countsd == 0) {
                                                $fifteen_minsss = $estimate_time * 60;
                                                $start_time += $fifteen_minsss;
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $countsd++;
                                            } else {
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $countsd++;
                                            }
                                        }
                                    }
                                    $start_time += $fifteen_mins;
                                }
                            }
                        }
                        if ($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) {
                            $currentTime = date("h:i A", strtotime('+0 minutes', $currentTime));
                            $start_time = strtotime($currentDate . ' ' . $currentTime);
                            $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStartTime != $secondEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                    }
                }
                $timeList = '';
                if (!empty($array_of_time)) {
                    foreach ($array_of_time as $value) {
                        $timeList .= $value . ',';
                    }
                }
                if ($type == 'delivery' || $type == '') {
                    if ($key == 0) {
                        $deliveryTimeSlots["today"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 1) {
                        $deliveryTimeSlots["tomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 2) {
                        $deliveryTimeSlots["dayaftertomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                } else {
                    if ($key == 0) {
                        $pickupTimeSlots["today"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 1) {
                        $pickupTimeSlots["tomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                    if ($key == 2) {
                        $pickupTimeSlots["dayaftertomorrow"] = ($timeList != '') ? $timeList : '';
                    }
                }
            }
        }
        if ($type == 'delivery' || $type == '') {
            $data = $deliveryTimeSlots;
        }
        if ($type == 'pickup') {
            $data = $pickupTimeSlots;
        }
        return $data;
    }

    /**
     * Update restaurant.
     * POST  /tiffintom/restaurants/time-slot
     */
    public function restaurant_booking_timeslot(Request $req)
    {
        $deliveryTimeSlots = $this->getBookingTimeSlot($req->query("date"), "delivery", $req->query("restaurant_id"));
        $pickupTimeSlots = $this->getBookingTimeSlot($req->query("date"), "pickup", $req->query("restaurant_id"));

        $timeSlot = array();
        if ($deliveryTimeSlots['today'] != '') {
            $timeSlot['booking'] = $deliveryTimeSlots['today'];
        } else {
            $timeSlot['booking'] = $pickupTimeSlots['today'];
        }

        return $timeSlot;
    }

    public function getBookingTimeSlot($date, $type, $resid)
    {
        $deliveryTimeSlots = array();
        $pickupTimeSlots = array();
        $array_of_time = array();
        if ($date != '') {
            $restaurantDetails = TiffintomRestaurant::find($resid);
            if (empty($restaurantDetails)) Utils::error("No restaurant found", 404);
            $restaurantDetails->append('restaurant_timing');

            //Timing Section
            $nowTime = date('h:i A');
            $currentTime = strtotime($nowTime);
            $currentDates = date("d-m-Y", strtotime($date));
            $date = Carbon::parse($date);
            // $carbonDate = Carbon::parse($date);
            // $tomorrow = $carbonDate->addDays(1)->format('d-m-Y');
            // $dayaftertomorrow = $carbonDate->addDays(2)->format('d-m-Y');
            // $timesArray = array($currentDates, $tomorrow, $dayaftertomorrow);
            // $timeSlots = array();
            // foreach ($timesArray as $key => $date) {
            $today = date('d-m-Y');
            $currentDate = date("d-m-Y", strtotime($date));
            // if (strtotime($today) == strtotime($currentDate)) {
            //     if ($restaurantDetails['booking_start'] == '4320') {
            //         $currentDate = date("Y-m-d", strtotime('+3 days', strtotime(date("d-m-Y", strtotime($date)))));
            //     } else if ($restaurantDetails['booking_start'] == '2880') {
            //         $currentDate = date("Y-m-d", strtotime('+2 days', strtotime(date("d-m-Y", strtotime($date)))));
            //     } else if ($restaurantDetails['booking_start'] == '1440') {
            //         $currentDate = date("Y-m-d", strtotime('+1 days', strtotime(date("d-m-Y", strtotime($date)))));
            //     } else {
            //     }
            // }
            $currentDay = strtolower(date('l', strtotime($currentDate)));
            $toDay = strtolower(date('l'));
            $bookingTiming = '';
            $currentDay = strtolower(date('l', strtotime($currentDate)));
            if ($today == $currentDate && preg_match('/\b' . $currentDay . '\b/', $restaurantDetails['booking_start_days'])) {
                $bookingTiming = 'close';
            }
            if ($type == 'delivery' || $type == '') {
                $firstStartTime = $restaurantDetails[$currentDay . '_first_opentime'];
                $firstEndTime = $restaurantDetails[$currentDay . '_first_closetime'];
                $secondStartTime = $restaurantDetails[$currentDay . '_second_opentime'];
                $secondEndTime = $restaurantDetails[$currentDay . '_second_closetime'];

                $firstOpenTime = strtotime($restaurantDetails[$currentDay . '_first_opentime']);
                $firstCloseTime = strtotime($restaurantDetails[$currentDay . '_first_closetime']);
                $secondOpenTime = strtotime($restaurantDetails[$currentDay . '_second_opentime']);
                $secondCloseTime = strtotime($restaurantDetails[$currentDay . '_second_closetime']);
                $restentitytiming =    $restaurantDetails[$currentDay . '_status'];
                // $estimate_time = ($restaurantDetails[$currentDay . '_estimate_time'] > 0) ? $restaurantDetails[$currentDay . '_estimate_time'] : $restaurantDetails['estimate_time'];
                $estimate_time = 0;
            } else {
                $firstStartTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM';
                $firstEndTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM';
                $secondStartTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM';
                $secondEndTime = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM';

                $firstOpenTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_opentime'] : '12:00 PM');
                $firstCloseTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_first_closetime'] : '12:00 PM');
                $secondOpenTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_opentime'] : '12:00 PM');
                $secondCloseTime = strtotime(isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_second_closetime'] : '12:00 PM');
                $restentitytiming = isset($restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_status']) ? $restaurantDetails['restaurant_timing']['pick_' . $currentDay . '_status'] : 'Close';
                $estimate_time = 0;
            }
            //In first Timing section
            if ($restentitytiming != 'Close' && $restaurantDetails['close'] != 0 && $bookingTiming != 'close') {

                $restaurantDetails['currentStatus'] = 'Open';
                $final[] = $restaurantDetails;
                if (strtotime($today) != strtotime($currentDate)) {
                    $nowTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                    $start_time = strtotime($currentDate . ' ' . $nowTime);
                    $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                    $fifteen_mins = 15 * 60;
                    $counts = 0;
                    while ($start_time <= $end_time) {
                        // $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                        if ($firstStartTime != $firstEndTime) {
                            if ($counts == 0) {
                                $fifteen_minsss = $estimate_time * 60;
                                $start_time += $fifteen_minsss;
                                $array_of_time[] = date("h:i A", $start_time);
                                $counts++;
                            } else {
                                $array_of_time[] = date("h:i A", $start_time);
                                $counts++;
                            }
                        }
                        $start_time += $fifteen_mins;
                    }

                    $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                    $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                    $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                    $fifteen_mins = 15 * 60;
                    $countsd = 0;

                    while ($start_time <= $end_time) {
                        // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                        if ($secondStartTime != $secondEndTime) {
                            if ($countsd == 0) {
                                $fifteen_minsss = $estimate_time * 60;
                                $start_time += $fifteen_minsss;
                                $array_of_time[] = date("h:i A", $start_time);
                                $countsd++;
                            } else {
                                $array_of_time[] = date("h:i A", $start_time);
                                $countsd++;
                            }
                        }
                        $start_time += $fifteen_mins;
                    }
                } else {
                    if ($currentTime < $firstOpenTime) {
                        $nowTimes = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $nowTimes);
                        $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            if ($currentTime <= $start_time) {
                                // $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                if ($firstStartTime != $firstEndTime) {
                                    if ($counts == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    }
                                }
                            }
                            $start_time += $fifteen_mins;
                        }
                    }
                    if ($currentTime > $firstOpenTime && $currentTime <= $firstCloseTime) {
                        $firstOpenTime = date("h:i A", strtotime('+0 minutes', $firstOpenTime));
                        $start_time = strtotime($currentDate . ' ' . $firstOpenTime);
                        $end_time = strtotime($currentDate . ' ' . $firstEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            if ($currentTime <= $start_time) {
                                // $newarrival = $firstStartTime . ' - ' . $firstEndTime;
                                if ($firstStartTime != $firstEndTime) {
                                    if ($counts == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    }
                                }
                            }
                            $start_time += $fifteen_mins;
                        }
                        $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                        $start_time1 = strtotime($currentDate . ' ' . $secondStartTimes);
                        $end_time1 = strtotime($currentDate . ' ' . $secondEndTime);
                        $fifteen_mins = 15 * 60;
                        $countsd = 0;
                        while ($start_time1 <= $end_time1) {
                            if ($currentTime <= $start_time1) {
                                // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                if ($secondStartTime != $secondEndTime) {
                                    if ($countsd == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time1 += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time1);
                                        $countsd++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time1);
                                        $countsd++;
                                    }
                                }
                            }
                            $start_time1 += $fifteen_mins;
                        }
                    }
                    if (empty($array_of_time)) {
                        if ($currentTime > $firstCloseTime && $currentTime <= $secondOpenTime) {
                            $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                            $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $counts = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStartTime != $secondEndTime) {
                                        if ($counts == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $counts++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        } else {
                            if ($currentTime < $secondOpenTime) {
                                $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                                $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                                $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                                $fifteen_mins = 15 * 60;
                                $countsd = 0;
                                while ($start_time <= $end_time) {
                                    if ($currentTime <= $start_time) {
                                        // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                        if ($secondStartTime != $secondEndTime) {
                                            if ($countsd == 0) {
                                                $fifteen_minsss = $estimate_time * 60;
                                                $start_time += $fifteen_minsss;
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $countsd++;
                                            } else {
                                                $array_of_time[] = date("h:i A", $start_time);
                                                $countsd++;
                                            }
                                        }
                                    }
                                    $start_time += $fifteen_mins;
                                }
                            }
                        }
                    } else {
                        if ($currentTime < $secondOpenTime) {
                            $secondStartTimes = date("h:i A", strtotime('+0 minutes', $secondOpenTime));
                            $start_time = strtotime($currentDate . ' ' . $secondStartTimes);
                            $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                            $fifteen_mins = 15 * 60;
                            $countsd = 0;
                            while ($start_time <= $end_time) {
                                if ($currentTime <= $start_time) {
                                    // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                    if ($secondStartTime != $secondEndTime) {
                                        if ($countsd == 0) {
                                            $fifteen_minsss = $estimate_time * 60;
                                            $start_time += $fifteen_minsss;
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $countsd++;
                                        } else {
                                            $array_of_time[] = date("h:i A", $start_time);
                                            $countsd++;
                                        }
                                    }
                                }
                                $start_time += $fifteen_mins;
                            }
                        }
                    }
                    if ($currentTime > $secondOpenTime && $currentTime <= $secondCloseTime) {
                        $currentTime = date("h:i A", strtotime('+0 minutes', $currentTime));
                        $start_time = strtotime($currentDate . ' ' . $currentTime);
                        $end_time = strtotime($currentDate . ' ' . $secondEndTime);
                        $fifteen_mins = 15 * 60;
                        $counts = 0;
                        while ($start_time <= $end_time) {
                            if ($currentTime <= $start_time) {
                                // $newarrival = $secondStartTime . ' - ' . $secondEndTime;
                                if ($secondStartTime != $secondEndTime) {
                                    if ($counts == 0) {
                                        $fifteen_minsss = $estimate_time * 60;
                                        $start_time += $fifteen_minsss;
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    } else {
                                        $array_of_time[] = date("h:i A", $start_time);
                                        $counts++;
                                    }
                                }
                            }
                            $start_time += $fifteen_mins;
                        }
                    }
                }
            }
            $timeList = '';
            if (!empty($array_of_time)) {
                foreach ($array_of_time as $value) {
                    $timeList .= $value . ',';
                }
            }
            if ($type == 'delivery' || $type == '') {
                $deliveryTimeSlots["today"] = ($timeList != '') ? $timeList : '';
            } else {
                $pickupTimeSlots["today"] = ($timeList != '') ? $timeList : '';
            }
            // }
        }
        if ($type == 'delivery' || $type == '') {
            $data = $deliveryTimeSlots;
        }
        if ($type == 'pickup') {
            $data = $pickupTimeSlots;
        }
        return $data;
    }

    /**
     * List Offers of restaurant.
     * POST  /tiffintom/restaurants/offer-all
     */
    public function restaurant_offer(Request $req)
    {
        $multiplePriceOffers = TiffintomOfferMultiplePrice::where('resid', $req->query("restaurant_id"))->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->where('status', '1')->orderBy('purchase_price', 'DESC')->get();

        $dayOffers = TiffintomOfferDay::where('resid', $req->query("restaurant_id"))->where('day',  date('l'))->where('status', '1')->orderBy('id', 'DESC')->get();

        $dayProductOffer = TiffintomOfferDayProduct::where('resid', $req->query("restaurant_id"))->where('day_product',  date('l'))->where('status', '1')->orderBy('id', 'DESC')->get();

        $dayPriceOffer = TiffintomOfferDayPrice::where('resid', $req->query("restaurant_id"))->where('day',  date('l'))->where('status', '1')->orderBy('id', 'DESC')->get();

        $offers = TiffintomOffer::where('resid', $req->query("restaurant_id"))->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->get();

        $productOffer = TiffintomOfferProduct::where('resid', $req->query("restaurant_id"))->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->get();

        $priceOffer = TiffintomOfferPrice::where('resid', $req->query("restaurant_id"))->where('status', '1')->orderBy('id', 'DESC')->where('offer_from', '<=', date('Y-m-d'))->where('offer_to', '>=', date('Y-m-d'))->get();

        $customer = Auth::user();
        $restDetails = TiffintomRestaurant::find($req->query("restaurant_id"));
        if ($customer) {
            $userVoucher = TiffintomUserVoucher::where('status', '1')->whereRaw('(device_type = ? OR device_type=?)', ['app', 'all'])->whereRaw('(restaurant_id = ? OR restaurant_id=?)', [$req->query("restaurant_id"), 'all'])->where('voucher_from', '<=', date('Y-m-d'))->where('voucher_to', '>=', date('Y-m-d'))->where('user_id', $customer->id)->get();
            foreach ($userVoucher as $uservoucher) {

                if ($uservoucher['delivery_type'] == 1 && $uservoucher['pickup_type'] == 1) {
                    $type = 'Delivery and Collection';
                } else if ($uservoucher['delivery_type'] == 0 && $uservoucher['pickup_type'] == 1) {
                    $type = 'Collection Only';
                } else if ($uservoucher['delivery_type'] == 1 && $uservoucher['pickup_type'] == 0) {
                    $type = 'Delivery Only';
                } else {
                    $type = '';
                }

                $device = ($uservoucher['device_type'] == 'all') ? 'application or Website, Minimum ' : 'application, Minimum ';

                if ($uservoucher['offer_mode'] == 'price') {
                    $offer_mode = $restDetails['currency'] . $uservoucher['offer_value'] . ' off ';
                } else if ($uservoucher['offer_mode'] == 'percentage') {
                    $offer_mode = $uservoucher['offer_value'] . '% off ';
                } else if ($uservoucher['offer_mode'] == 'free_delivery') {
                    $offer_mode = 'free delivery';
                } else {
                    $offer_mode = '';
                }

                if ($uservoucher['restaurant_id'] == 'all') {
                    $restaurant_id = '';
                } else {
                    $restaurant_id = ' at ' . $restDetails['restaurant_name'];
                }
                $uservoucher['voucher_message'] = $uservoucher['voucher_code'] . ' Get ' . $offer_mode . $type . $restaurant_id . ' ,Apply in ' . $device . $restDetails['currency'] . $uservoucher['minimum_value'] . ' order, Expires ' . $uservoucher['voucher_to'];
            }
        } else {
            $userVoucher = array();
        }

        $restaurantVoucher = TiffintomRestaurantVoucher::whereRaw('(device_type = ? OR device_type=?)', ['app', 'all'])->where('restaurant_id', $req->query("restaurant_id"))->where('status', '1')->get();
        $restaurantvouchers = [];
        if (!$restaurantVoucher->isEmpty()) {
            $currentDate  = strtotime(date("Y-m-d"));
            foreach ($restaurantVoucher as $restaurantkey => $restaurantvoucher) {
                if ($currentDate >= strtotime($restaurantvoucher['voucher_from']) && $currentDate <= strtotime($restaurantvoucher['voucher_to'])) {
                    if ($restaurantvoucher['delivery_type'] == 1 && $restaurantvoucher['pickup_type'] == 1) {
                        $type = 'Delivery and Collection';
                    } else if ($restaurantvoucher['delivery_type'] == 0 && $restaurantvoucher['pickup_type'] == 1) {
                        $type = 'Collection Only';
                    } else if ($restaurantvoucher['delivery_type'] == 1 && $restaurantvoucher['pickup_type'] == 0) {
                        $type = 'Delivery Only';
                    } else {
                        $type = '';
                    }

                    $device = ($restaurantvoucher['device_type'] == 'all') ? 'application or Website, Minimum ' : 'application, Minimum ';

                    if ($restaurantvoucher['offer_mode'] == 'price') {
                        $offer_mode = $restDetails['currency'] . $restaurantvoucher['offer_value'] . ' off ';
                    } else if ($restaurantvoucher['offer_mode'] == 'percentage') {
                        $offer_mode = $restaurantvoucher['offer_value'] . '% off ';
                    } else if ($restaurantvoucher['offer_mode'] == 'free_delivery') {
                        $offer_mode = 'free delivery';
                    } else {
                        $offer_mode = '';
                    }

                    if ($restaurantvoucher['restaurant_id'] == 'all') {
                        $restaurant_id = '';
                    } else {
                        $restaurant_id = ' at ' . $restDetails['restaurant_name'];
                    }
                    $restaurantvoucher['voucher_message'] = $restaurantvoucher['voucher_code'] . ' Get ' . $offer_mode . $type . $restaurant_id . ' ,Apply in ' . $device . $restDetails['currency'] . $restaurantvoucher['minimum_value'] . ' order, Expires ' . $restaurantvoucher['voucher_to'];
                    $restaurantvouchers[] = $restaurantvoucher;
                }
            }
        }

        $offerText = TiffintomOfferText::where('restaurant_id', $req->query("restaurant_id"))->where('status', '1')->where('delete_status', 'N')->get();

        $data = [
            'multiple_price_offers' => $multiplePriceOffers,
            'day_offers' => $dayOffers,
            'day_product_offers' => $dayProductOffer,
            'day_price_offers' => $dayPriceOffer,
            'offers' => $offers,
            'product_offers' => $productOffer,
            'price_offers' => $priceOffer,
            'user_vouchers' => $userVoucher,
            'restaurant_vouchers' => $restaurantvouchers,
            'offer_text' => $offerText,
        ];

        return $data;
    }

    /**
     * List Pending Count All of restaurant.
     * GET  /tiffintom/restaurants/{id}/counts
     */
    public function restaurant_pending_count(Request $req, $id)
    {
        $PendingOrders = TiffintomOrder::where('restaurant_id', $id)->where('status', 'Pending')->where('delivery_date', '=', date('Y-m-d'))->count();

        $PendingDineinOrders = TiffintomDineinOrder::where('restaurant_id', $id)->where('status', 'Pending')->where('created', '>=', date('Y-m-d 00:00:00'))->where('created', '<=', date('Y-m-d 23:59:59'))->count();

        // $DelayOrders = TiffintomOrder::selectRaw("COUNT(id) as count,CONCAT_WS('',delivery_date,preparation) <= " . date('Y-m-d h:i A') . "")->where('restaurant_id', $id)->where('status', 'Accepted')->where('delivery_date', '=', date('Y-m-d'))->get();
        $DelayOrders = TiffintomOrder::where('restaurant_id', $id)->where('status', 'Accepted')->where('delivery_date', '=', date('Y-m-d'))->whereRaw('CONCAT_WS(" ",delivery_date,STR_TO_DATE(preparation, "%h:%i %p")) <= ?', [date('Y-m-d H:i:s')])->count();

        $PendingBookings = TiffintomBooking::where('restaurant_id', $id)->where('status', 'Pending')->count();

        $orderhelpDetailscount = TiffintomOrderHelp::where('restaurant_id', $id)->where('admin', '0')->where('status', '1')->count();

        $newReviewscount = TiffintomReview::where('restaurant_id', $id)->where('delete_status', 'N')->where('status', '1')->where('created', '>=', date("Y-m-d 00:00:00"))->where('created', '<=', date("Y-m-d 23:59:59"))->count();

        $data = [
            'pending_orders' => (int)$PendingOrders + (int)$PendingDineinOrders,
            'pending_bookings' => (int)$PendingBookings,
            'unread_chat' => (int)$orderhelpDetailscount,
            'new_reviews' => (int)$newReviewscount,
            'delay_orders' => (int)$DelayOrders,
        ];

        return $data;
    }

    public function findFromGoogleAndReturn($code)
    {
        $siteSetting = SiteSettingController::findByKey('google_map_api_key');
        $key = "AIzaSyBX3HSrqBRxfsvfuLBV0ZdK_BfHSymdS-U";
        if (!empty($siteSetting)) {
            $key = $siteSetting->value;
        }
        $response = (new Client())->request('GET', "https://maps.google.com/maps/api/geocode/json", [
            'query' => ['address' => $code . '.UK', 'key' => $key, 'region' => 'UK']
        ]);
        if ($response->getStatusCode() == 200) {
            $jsonBody = json_decode($response->getBody());
            if ($jsonBody->status != 'ZERO_RESULTS') {

                $results = $jsonBody->results;
                if (!empty($results) && sizeof($results) > 0) {
                    $singleResult = $results[0];
                    $addressComponnet = $singleResult->address_components;

                    $newPostcode = new Postcode;
                    $newPostcode->latitude = $singleResult->geometry->location->lat;
                    $newPostcode->longitude = $singleResult->geometry->location->lng;

                    if (!empty($addressComponnet) && sizeof($addressComponnet) > 0) {
                        foreach ($addressComponnet as $key => $value) {
                            if ($value->types[0] == 'street_number') {
                                $newPostcode->street = $value->long_name;
                            }
                            if ($value->types[0] == 'premise') {
                                $newPostcode->thorough_fare = $value->long_name;
                            }
                            if ($value->types[0] == 'postal_town') {
                                $newPostcode->post_town = $value->long_name;
                            }
                            if ($value->types[0] == 'postal_code') {
                                $newPostcode->post_code = str_replace(" ", "", $value->long_name);
                            }
                        }
                    }
                    return $newPostcode;
                }
            }
            return null;
        }
    }

    //Distance calculate
    public function getDistanceCalculate($latitudeFrom, $longitudeFrom, $latitudeTo, $longitudeTo, $unit)
    {
        $theta = $longitudeFrom - $longitudeTo;
        $dist = sin(deg2rad((float)$latitudeFrom)) * sin(deg2rad((float)$latitudeTo)) +  cos(deg2rad((float)$latitudeFrom)) * cos(deg2rad((float)$latitudeTo)) * cos(deg2rad((float)$theta));
        $dist = acos($dist);
        $dist = rad2deg($dist);
        $miles = $dist * 60 * 1.1515;
        $unit = strtoupper($unit);
        if ($unit == "K") {
            return number_format(($miles), 2);
        } else if ($unit == "N") {
            return ($miles * 0.8684) . ' nm';
        } else {
            return $miles . ' mi';
        }
    }

    //-------------------------------------------------------------------------------------------------------------
    public function getDeliveryCharge($resId, $Durations, $lat, $lng)
    {
        $message = '';
        $restDetails = TiffintomRestaurant::where('id', $resId)->first();
        $restDetails->append('delivery_setting');
        $restDetails->append('areamaps');

        // $SettingsList = TiffintomSiteSetting::where('id', '1')->first();

        $deliveryCharge = $count = 0;

        if ($restDetails['map_mode'] == 'Circle') {
            foreach ($restDetails['delivery_settings'] as $key => $value) {
                if ($Durations <= $value['delivery_miles']) { // + $SettingsList['site_mile'] 
                    $deliveryCharge = $value['delivery_charge'];
                    $count = 1;
                    break;
                }
            }
            if ($count == 1) {
                $message = 'success';
            } else {
                $message = 'failed';
            }
        } else if ($restDetails['map_mode'] == 'Polygon') {
            foreach ($restDetails['areamaps'] as $key => $value) {
                $xyArray = $this->getXYAxis($value['mappath']);
                if (!empty($xyArray) && is_array($xyArray)) {
                    $objPolygon = new TiffintomPolygonsController(array('x' => $lat, 'y' => $lng), $xyArray);
                    if ($objPolygon->check()) {
                        $deliveryCharge = $value['service_delivery_charge'];
                        $count = 1;
                        break;
                    }
                }
            }
            if ($count == 1) {
                $message = 'success';
            } else {
                $message = 'failed';
            }
        } else {
            $message = 'failed';
        }

        return array($deliveryCharge, $message);
    }

    public function getXYAxis($mapPath)
    {
        $xy = explode(',', $mapPath);
        $xyArray = array();
        if (!empty($xy) && is_array($xy)) {
            foreach ($xy as $x => $y) {
                $xyAxis = explode(' ', trim($y));
                $xyArray[] = array('y' => $xyAxis[0], 'x' => $xyAxis[1]);
            }
        }
        return $xyArray;
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }

    //Convert to seo url name
    public function seoUrl($string)
    {
        $text = preg_replace('~[^\\pL\d]+~u', '-', $string);

        // trim
        $text = trim($text, '-');

        // lowercase
        $text = strtolower($text);

        // remove unwanted characters
        $text = preg_replace('~[^-\w]+~', '', $text);

        if (strlen($text) > 70) {
            $text = substr($text, 0, 70);
        }

        if (empty($text)) {
            //return 'n-a';
            return time();
        }

        return $text;
    }


    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
