<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomRestaurantRegistration;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Mpdf\Mpdf;

class TiffintomRestaurantRegistrationController extends Controller
{

    /**
     * List restaurant_registration
     * GET /tiffintom/restaurant-registration
     */
    public function index(Request $req)
    {
        $restaurant_registration = TiffintomRestaurantRegistration::orderBy('id', 'DESC');
        if (empty($restaurant_registration)) Utils::error("No restaurant registration found", 404);

        if ($req->query('nopaginate', false) == '1') {
            $restaurant_registration = $restaurant_registration->get();
        } else {
            $restaurant_registration = $restaurant_registration->paginate($req->query('per_page', 20));
        }

        if ($restaurant_registration->isEmpty()) Utils::error('No restaurant registration found!', 404);

        return $restaurant_registration;
    }

    /**
     * Create restaurant_registration.
     * POST /tiffintom/restaurant-registration
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_name' => 'string',
            'restaurant_address' => 'string',
            'restaurant_city' => 'string',
            'restaurant_postcode' => 'string',
            'first_name' => 'string',
            'surname' => 'string',
            'email' => 'string',
            'phone' => 'string',
            'company_type' => 'string',
            'restaurant_cuisine' => 'string',
            'restaurant_dietary' => 'string',
            'no_of_location' => 'string',
            'deliver' => 'string',
            'payment' => 'string',
            'company_name' => 'string',
            'company_first_address' => 'string',
            'company_second_address' => 'string',
            'company_city' => 'string',
            'company_postcode' => 'string',
            'status' => 'boolean',
            'menu_image.*' => 'image',
            'logo' => 'image',
        ]);

        // create restaurant_registration
        $restaurant_registration = new TiffintomRestaurantRegistration;
        $restaurant_registration->fill($req->all());
        if ($req->has('menu_image')) {
            $refFiles = array();
            foreach ($req->file('menu_image') as $key => $value) {
                $fileName = ImageHelper::saveImageToServer($value, ImageHelper::$TYPE_TIFFINTOM_BECOME, 'tiffintom');
                $refFiles[] = $fileName;
            }
            $restaurantrefFile = implode(',', $refFiles);
            $restaurant_registration->menu_image = $restaurantrefFile;
        }
        if ($req->has('logo')) {
            $fileName = ImageHelper::saveImageToServer($req->file('logo'), ImageHelper::$TYPE_TIFFINTOM_BECOME, 'tiffintom');
            $restaurant_registration->logo = $fileName;
        }
        $restaurant_registration->save();

        // Pdf Save
        $this->becomePdf($restaurant_registration->id);

        // Mail send
        $this->becomeMail($restaurant_registration->id);

        return $restaurant_registration;
    }

    /**
     * List restaurant registration
     * GET /tiffintom/restaurant-registration/id
     */
    public function show(Request $req, $id)
    {
        $restaurant_registration = TiffintomRestaurantRegistration::find($id);
        if (empty($restaurant_registration)) Utils::error("No restaurant registration found", 404);
        return $restaurant_registration;
    }

    /**
     * Update restaurant registration.
     * POST  /tiffintom/restaurant-registration/id
     */
    public function update(Request $req, $id)
    {
        $restaurant_registration = TiffintomRestaurantRegistration::find($id);
        if (empty($restaurant_registration)) Utils::error('restaurant registration not found!', 404);

        $this->validate($req, [
            'restaurant_name' => 'string',
            'restaurant_address' => 'string',
            'restaurant_city' => 'string',
            'restaurant_postcode' => 'string',
            'first_name' => 'string',
            'surname' => 'string',
            'email' => 'string',
            'phone' => 'string',
            'company_type' => 'string',
            'restaurant_cuisine' => 'string',
            'restaurant_dietary' => 'string',
            'no_of_location' => 'string',
            'deliver' => 'string',
            'payment' => 'string',
            'company_name' => 'string',
            'company_first_address' => 'string',
            'company_second_address' => 'string',
            'company_city' => 'string',
            'company_postcode' => 'string',
            'status' => 'boolean',
            'menu_image' => 'image',
            'logo' => 'image',
        ]);

        // update menu
        if ($req->has('restaurant_name')) $restaurant_registration->restaurant_name = $req->input('restaurant_name');
        if ($req->has('restaurant_address')) $restaurant_registration->restaurant_address = $req->input('restaurant_address');
        if ($req->has('restaurant_city')) $restaurant_registration->restaurant_city = $req->input('restaurant_city');
        if ($req->has('restaurant_postcode')) $restaurant_registration->restaurant_postcode = $req->input('restaurant_postcode');
        if ($req->has('first_name')) $restaurant_registration->first_name = $req->input('first_name');
        if ($req->has('surname')) $restaurant_registration->surname = $req->input('surname');
        if ($req->has('email')) $restaurant_registration->email = $req->input('email');
        if ($req->has('phone')) $restaurant_registration->phone = $req->input('phone');
        if ($req->has('company_type')) $restaurant_registration->company_type = $req->input('company_type');
        if ($req->has('restaurant_cuisine')) $restaurant_registration->restaurant_cuisine = $req->input('restaurant_cuisine');
        if ($req->has('restaurant_dietary')) $restaurant_registration->restaurant_dietary = $req->input('restaurant_dietary');
        if ($req->has('no_of_location')) $restaurant_registration->no_of_location = $req->input('no_of_location');
        if ($req->has('deliver')) $restaurant_registration->deliver = $req->input('deliver');
        if ($req->has('payment')) $restaurant_registration->payment = $req->input('payment');
        if ($req->has('company_name')) $restaurant_registration->company_name = $req->input('company_name');
        if ($req->has('company_first_address')) $restaurant_registration->company_first_address = $req->input('company_first_address');
        if ($req->has('company_second_address')) $restaurant_registration->company_second_address = $req->input('company_second_address');
        if ($req->has('company_city')) $restaurant_registration->company_city = $req->input('company_city');
        if ($req->has('company_postcode')) $restaurant_registration->company_postcode = $req->input('company_postcode');
        $restaurant_registration->status = $req->input('status');
        if ($req->has('menu_image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('menu_image'), ImageHelper::$TYPE_TIFFINTOM_BECOME, 'tiffintom');
            $restaurant_registration->menu_image = $fileName;
        }
        if ($req->has('logo')) {
            $fileName = ImageHelper::saveImageToServer($req->file('logo'), ImageHelper::$TYPE_TIFFINTOM_BECOME, 'tiffintom');
            $restaurant_registration->logo = $fileName;
        }

        $restaurant_registration->save();
        return $restaurant_registration;
    }

    /**
     * Delete restaurant registration by id.
     * DELETE /tiffintom/restaurant-registration/{id}
     */
    public function delete(Request $req, $id)
    {
        $restaurant_registration = TiffintomRestaurantRegistration::find($id);
        if (empty($restaurant_registration)) Utils::error('restaurant registration not found!', 404);
        $restaurant_registration->delete();

        return $restaurant_registration;
    }

    public function becomePdf($id)
    {
        $restaurant_registration = TiffintomRestaurantRegistration::find($id);
        if (empty($restaurant_registration)) Utils::error("restaurant registration not found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $html = '<!DOCTYPE html>
				<html>
					<head>
						<style>
							table {
								font-family: arial, sans-serif;
								border-collapse: collapse;
								width: 100%;
							}

							html {
								font-family: arial, sans-serif;
							}

							td,
							th {
								border: 1px solid #000;
								text-align: left;
								padding: 12px;
							}

							h2 {
								text-align: center;
								font-family: arial, sans-serif;
							}

							.bold {
								font-weight: bold;
							}
						</style>
					</head>
					<body>
						<h2>RESTAURANT AGREEMENT</h2>
						<p class="bold">BETWEEN</p>
						<p>
							<b>Tiffin Tom LTD</b><br />
							Trading address: 66 Little Caldmore<br />
							Walsall, West Midlands, WS1 3RA<br />
							Tel: 08712 884044
						</p>

						<font style="font-size: 12px;">(hereafter, “<b>Tiffin Tom LTD</b>”) </font>

						<p class="bold">AND</p>
						<p>
							NAME OF CONTRACTING PARTY <b style="font-size:18px;">' . $restaurant_registration->company_name . '</b> <br />
							Which is [Tick one]:[Insert company name if incorporated OR Insert<br />
							[ ] a COMPANY owner name if sole trader/partnership]with COMPANY NO: ;or <br />
							[ ] an INDIVIDUAL; or <br />
							[ ] a PARTNERSHIP]
						</p>

						<p class="bold">OWNER DETAILS</p>
						<p>CONTACT NAME&nbsp;<b style="font-size:18px;">' . $restaurant_registration->company_name . '</b></p>
						<p>FULL ADDRESS&nbsp;&nbsp;<b style="font-size:18px;">' . $restaurant_registration->company_first_address . '</b></p>
						<p>TEL NO&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;________________________________________</p>
						<p>EMAIL&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;________________________________________</p>

						<p class="bold">RESTAURANT DETAILS</p>
						<p>
							<span style="font-size: 10px;">For and on behalf of</span> <br />
							NAME OF RESTAURANT <b style="font-size:18px;">' . $restaurant_registration->restaurant_name . '</b>
						</p>
						<p>VAT NUMBER&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;_________________________________________</p>
						<p>FULL ADDRESS&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b style="font-size:18px;">' . $restaurant_registration->restaurant_address . '</b></p>
						<p>TEL NO&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b style="font-size:18px;">' . $restaurant_registration->phone . '</b></p>
						<p>EMAIL&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b style="font-size:18px;">' . $restaurant_registration->email . '</b></p>

						<p style="font-size: 12px;">(the contracting party listed above, hereafter, the “<b>Restaurant</b>”)</p>
						<p class="bold">BANKING DETAILS</p>
						<p>NAME OF BANK OR BUILDING SOCIETY &nbsp;&nbsp; _________________________________________</p>
						<p>ACCOUNT HOLDERS NAME &nbsp;&nbsp; _________________________________________</p>
						<p>SORT CODE &nbsp;&nbsp; _________________________________________</p>
						<p>ACCOUNT NUMBER &nbsp;&nbsp; _________________________________________</p>

						<p style="font-size: 12px;">
							<b>THIS AGREEMENT</b> set out in parts 1 and 2 the terms and conditions applicable to the restaurant’s use of Tiffin Tom Ltd.’s proprietary WE- Connect Technology and Tiffin Tom Ltd.’s website for the receipt. Preparation and
							delivery of orders by customers for goods from the Restaurant and has been concluded as of the date at the end of this document (<b>the “Agreement”</b>).
						</p>

						<p class="bold" style="text-align: center;">PART 1. OVERVIEW OF KEY TERMS</p>

						<table>
							<tr>
								<td>
									Deposit for WCT Provision,<br />
									Installation & Connection Fee <br />
									Or Ful Epos System<br />
									Installation & Connection Fee
								</td>
								<td>
									£___________N/A___________ paid by:<br />
									<br />
									[N/A ] Electronic card payment<br />
									[N/A ] Cash<br />
									[N/A ] Cheque<br />
								</td>
							</tr>
							<tr>
								<td>
									Charges by Tiffin Tom Ltd to Restaurant<br />
									<b>(“Restaurant charges”)</b>
								</td>
								<td>
									<p>Tiffin Tom Ltd will charge the restaurant:<br /></p>
									<ul>
										<li>A5% commission fee per order (charged on Gross Order Value)</li>
										<li>£0.00admin charge per order paid for online</li>
										<li>£ (N/A) %Merchant Fee + £ (N/A) Processing Fee</li>
										<li>£ 0 Rental Charge + Terms</li>

										<br />
										<b>“Gross Order Value”</b>
										means:
										<br />
										<br />

										<li>
											The total amount charged by the Restaurant to the Customer for an Order, including the value of the goods plus the Restaurant’s delivery charges (if any) plus the Customer Admin Charge (if any) plus applicable
											taxes
										</li>
									</ul>
								</td>
							</tr>
							<tr>
								<td>
									Charges by Restaurant to Customer<br />
									<b>(“Customer Admin Charge”)</b>
								</td>
								<td>
									<p>Tiffin Tom Ltd will charge the Customer:<br /></p>
									<ul>
										<li>A £0 admin charge per Order paid for online</li>
									</ul>
								</td>
							</tr>
							<tr>
								<td>
									<p>Term</p>
								</td>
								<td>The Agreement is valid from signature until partnership is seized or cancelled.</td>
							</tr>
						</table>

						<p class="bold" style="text-align: center;">PART 2. GENERAL TERMS AND CONDITIONS</p>
						<p style="font-size: 11px;">
							<b>1. PURPOSE</b><br />
							<br />

							1.1. Tiffin Tom Ltd, primarily via its website or its affiliated websites, applications and platforms (together, the “Website”), and its proprietary Tiffin Tom Ltd-Connect Technology (“WCT”), WCT terminal , and other related
							equipment and software (together) the “Product”), will enable customers to place Orders (as defined in clause 3.1) for takeaway food and drinks from the Restaurant, for delivery by, or collection at, the Restaurant (the
							“Service”) on the terms and conditions set out in this Agreement.<br />

							1.2. In consideration of the mutual covenants contained in this Agreement, the parties agree to comply with their respective obligations set out herein.<br />
							<br />

							<b>2. THE PRODUCT</b><br />
							<br />

							2.1. Payment: The Restaurant agrees to pay Tiffin Tom Ltd the “WCT Provision, Installation & Connection Fee” indicated in Part 1 od this Agreement, of which: the Restaurant has paid or will pay the “refundable deposit” indicated
							in part 1<br />

							2.2. Delivery and Installation: Delivery and Installation of the Product will take place at a mutually agreed time between Tiffin Tom Ltd and the Restaurant and will only be carried out after payment in full of the refundable
							deposit has been made by the Restaurant.<br />

							2.3. Profile set-up: Within 14 days of the later of (i) Tiffin Tom Ltd having received payment in full of the refundable deposit in cleared funds, and (ii) Tiffin Tom Ltd having received all the information that it shall
							reasonably request from the Restaurant (including in relation to the Restaurant’s name, logo, address, contact information and menu (the “website profile)) in complete and correct form, Tiffin Tom Ltd undertakes to set up the
							Restaurant’s Website Profile on the Website.<br />

							2.4. Product Care: The Restaurant will use, store and maintain the Product using reasonable care, including without limitation establishing and maintaining the Product in such a way as to allow it to receive a GPRS signal, and
							otherwise so that Orders received will be processed instantly.<br />

							2.5. Product malfunctions: The Restaurant must notify Tiffin Tom Ltd without delay if the Product malfunctions.<br />

							2.6 Product repair- general: During the first (12) months from the date of this Agreement, Tiffin Tom Ltd will reasonable promptly repair or replace the Product in case of malfunction, save where in Tiffin Tom Ltd.’s sole
							opinion the malfunction is a result of misuse, unauthorised alteration or accidental damage by the Restaurant or of repairs not performed by Tiffin Tom Ltd or its authorised representatives (“Restaurant Product Damage”).<br />

							2.7. Product repair – charges: After twelve (12) months from the date of this Agreement, or at any time where the malfunction is in Tiffin Tom Ltd.’s sole opinion a result of Restaurant Product damage, Tiffin Tom Ltd reserves
							the right to charge the Restaurant a reasonable fee for repairing the malfunctioning Product or to charge the Restaurant for a replacement Product at Tiffin Tom Ltd.’s then prevailing price.<br />

							2.8. Product upgrades: Tiffin Tom Ltd may upgrade or alter the Product on notice to the Restaurant at any time. Tiffin Tom Ltd reserves the right to charge the Restaurant for any upgraded or replaced Product at Tiffin Tom Ltd.’s
							then prevailing price. Tiffin Tom Ltd will notify the Restaurant in advance of any such upgrade or alteration of the amount of charges applicable.<br />
							<br />

							<b>3. ORDERS</b><br />
							<br />

							3.1. Execution of Orders: The Restaurant undertakes to receive, process, prepare and deliver the customers’ orders placed via the Website and received from Tiffin Tom Ltd by means of any of the Product, fax, phone or email
							(“Orders”) using the best care, skill and diligence, and in accordance with best practise in the Restaurant’s industry, profession or trade. Food prepared, processed and delivered must correspond to what is stated in the Order,
							and must take into account any customer comments submitted with the Order’s including without limitation in relation to customer allergies.<br />

							3.2. Rejection of Orders: The Restaurant shall use its best efforts to accept all Orders. Where the Restaurant wishes to reject an Order, it shall immediately communicate such rejection by pressing the relevant button on the WCT
							terminal.<br />

							3.3. Payment for Orders-Acceptance of Card Orders: The Restaurant agrees that it will at all times allow customers the option to pay for Orders using either a credit or debit card (“Card Orders”) or, subject to clause 3.4, cash
							(“Cash Orders”)<br />

							3.4. Payment for Orders - Card only Orders: If, at any time after the first 120 days of the term of this Agreement, the aggregate Gross Order Value of Card Orders is not more than the aggregate value of any debts owing by the
							Restaurant to Tiffin Tom Ltd (including for the avoidance of doubt any debts relating to Restaurant Charges for Cash Orders), the Restaurant agrees that Tiffin Tom Ltd may at its discretion process Card Orders only on behalf of
							the Restaurant until all such debts are cleared.<br />

							3.5. Delivery of Orders – Order Number: When the Restaurant delivers the Order, the Restaurant must check that the Order number given by the customer corresponds with the Order number received by the Restaurant from Tiffin Tom
							Ltd.<br />

							3.6. Delivery of Orders- Customer ID: Where an order is a Card Order, on delivery of the Order the Restaurant must request proof of identification from the customer in the form of the signed card used to place the Order, and
							check that the credit or debit card conforms with the receipt data for the Order.<br />

							3.7. Delivery of Orders- Alcohol etc: Where an order contains alcohol, or any other goods which are subject to statutory age restrictions, the Restaurant must request proof of age from the customer (in accordance with applicable
							laws) on delivery. The Restaurant will take every other precaution necessary to ensure the customer is over the legal age for the purchase of alcohol and to ensure that the delivery occurs within the legal timeframe allowed for
							sale of alcohol.<br />

							3.8. Receipts: The Restaurant will provide each customer with an official receipt (and a VAT receipt, if applicable) in respect of an Order, if the customer so requests.<br />

							3.9. Redelivery and refunds: The Restaurant shall use its best efforts to satisfy any request for a redelivery or refund or proportionate price reduction of any Order made by a customer or Tiffin Tom Ltd on behalf of a customer.
							In the event that such a redelivery, refund or proportionate price reduction is granted by the Restaurant or Tiffin Tom Ltd, this shall not affect the Restaurant Charges that are due to Tiffin Tom Ltd in relation to the original
							order. The calculation of all Restaurant Charges shall therefore be based on the Gross Order Value of the faultless Order.<br />
							<br />

							<b>4. RESTAURANT & MENU INFORMATION</b><br />
							<br />

							4.1. Information supplied to Tiffin Tom Ltd: The Restaurant undertakes to check that any information contained in its menu (the “Menu”) or otherwise provided to Tiffin Tom Ltd for inclusion to the Website is at all times
							accurate and complies with all applicable regulations and laws including but not limited to consumer information about ingredients which may cause allergic reactions (“Allergen Information”) such as nuts), and is up to date.
							<br />

							4.2. Allergen Information: The restaurant must provide and update Tiffin Tom Ltd with details of any Allergen Information in relation to the dishes that the Restaurant prepares. It is the Restaurant’s sole responsibility to
							provide this information to Tiffin Tom Ltd without delay, and check that the Website contains the correct Allergen Information. The safety of the customer is paramount. If Tiffin Tom Ltd is not satisfied with the commitment of
							the Restaurant to providing accurate and timely information about ingredients (and in particular, Allergen Information), Tiffin Tom Ltd reserves the right to suspend the Restaurant from the Website and/ or terminate this
							agreement immediately.<br />

							4.3. Menu prices: The restaurant agrees that the menu prices (including any delivery charges) it provides for display on the Website shall be the same as the prices offered in its premises or otherwise offered) and that it shall
							not offer a customer any discounts or redeem any coupons which are not also offered via the Website.<br />

							4.4. Updates & errors – The Restaurant shall be solely responsible for ensuring that the information displayed on the website is accurate and up to date and corresponds exactly to the information contained in the Menu. The
							restaurant shall promptly report to Tiffin Tom Ltd any errors in the menu information or Website Profile displayed on the Website or any changes to the Menu or Website profile. The restaurant agrees and acknowledges that if
							there is any conflict between the information provided on the Website and Information provided by the Restaurant to Tiffin Tom Ltd, the information on the Website shall prevail.<br />

							4.5. Changes and Costs-Tiffin Tom Ltd will update and/or change the contents of the Information about the Restaurant displayed on the website as soon as practicable following receipt of a written request from the Restaurant to
							correct errors or make minor changes to such information. Tiffin Tom Ltd reserves the right to charge an administrative fee to the Restaurant for any changes. In such a case, Tiffin Tom Ltd will notify the Restaurant in advance
							of the change of the amount of the administrative fees applicable.<br />

							4.6. Opening hours: The restaurant must inform Tiffin Tom Ltd of its hours of operation (the “Opening Hours”), and of any changes to such Opening Hours. If the hours of operation are stated on the menu, Tiffin Tom Ltd shall be
							entitles to treat these as the Opening Hours unless the Restaurant informs Tiffin Tom Ltd otherwise.<br />

							4.7. Offline times during opening hours: The restaurant must remain online and able to accept Orders using the Product at all times during the Opening Hours, save in case of closures for emergencies or planned closures, the
							duration of which has been communicated to Tiffin Tom Ltd in writing in advance. Where closures are planned, the Restaurant must provide Tiffin Tom Ltd with a date from which Opening Hours will return to normal.<br />
							<br />

							<b>5. CHARGES</b><br />
							<br />

							5.1. Restaurant Charges: Tiffin Tom Ltd is entitled to charge the Restaurant the Restaurant charges set forth in Part 1 of this agreement, compromising;<br />
							5.1.1.1. A commission at the rate set forth in Part 1 on the total Gross Order Value of each Order calculated in accordance with clause 5.2 placed by a customer using the Service (plus applicable taxes) and<br />
							5.1.1.2. An administrative charge in the amount set forth in Part 1 for each order that is paid for online (plus applicable taxes).<br />

							5.2. Gross order value: The Gross Order Value is the total amount charged by the Restaurant to the Customer for an order, including the value of the goods plus the Restaurant’s delivery charges (if any) plus the Customer Admin
							charge (if any) plus applicable taxes).<br />

							5.3. Customer Admin charge: Tiffin Tom Ltd will, on behalf of the Restaurant, charge each customer using the Service and who pays for an Order online, a Customer Admin Charge in the amount set forth in Part 1. Where the Customer
							Admin Charge applies, the Restaurant shall include it in that customer’s receipts for goods.<br />
							<br />

							<b>6. INVOICING AND PAYMENT</b><br />
							<br />

							6.1. Statements: Twice per month, Tiffin Tom Ltd shall provide a statement of outstanding accounts between the Restaurant and Tiffin Tom Ltd( a “Statement”) relating to the previous bi-monthly period. The statement will include:
							<br />
							6.1.1 The aggregate Gross Order Value of all orders for the relevant period, split between Cash Orders and Card Orders;<br />
							6.1.2 any sums owed by the Restaurant to Tiffin Tom Ltd in relation to Restaurant Charges, the Product or the Balance (if any) and any other services provided by Tiffin Tom Ltd to the Restaurant, in each case for the relevant
							period;<br />
							6.1.3 any balance brought forward from, and any amounts paid or received by Tiffin Tom Ltd since the date of, the previous Statement<br />

							6.2 Invoicing: If any monies are owed to the Restaurant prior to the next Statement, JE will remit the sum to the Restaurant prior to the next statement.If any monies are owed by the Restaurant to Tiffin Tom Ltd according to the
							Statement, such sums shall be due on the date of the invoice, and payable by the Restaurant within 7 days. Thereafter, Tiffin Tom Ltd may charge interest and set off any unpaid amounts in accordance with clause 7.1.<br />

							6.3 Card Orders: If a customer pays for an Order by credit or debit card, and the payment is withheld due to faults in the Restaurant’s preparation or delivery of the Order, or due to misuse of the card, the Restaurant is not
							entitled to any payment from Tiffin Tom Ltd in relation to such order.<br />

							6.4. Disputed Statements: If the Restaurant disagrees with the Statement, the Restaurant must notify Tiffin Tom Ltd of its disagreement within 14 days of the delivery of the Statement to the Restaurant, setting out in detail the
							reasons for the disagreement. If the Restaurant fails to notify Tiffin Tom Ltd of any such disagreement within 14 days of the Statement date, the Statement shall be deemed to be accepted by the Restaurant.<br />
							<br />

							<b>7. CHANGES TO OWNERSHIP OR BANK ACCOUNT DETAILS</b><br />
							<br />

							The Restaurant must notify Tiffin Tom Ltd in writing prior to any change of ownership of the Restaurant or any change to its bank account details, and in any case as soon as practicable following such change. Failure to notify
							Tiffin Tom Ltd may result in Tiffin Tom Ltd paying monies to a bank account controlled by the outgoing owner(s) or an incorrect bank account. The Restaurant shall fully indemnify Tiffin Tom Ltd and hold Tiffin Tom Ltd harmless
							against any losses, damages or claims made against Tiffin Tom Ltd by the new owner(s) or otherwise incurred by Tiffin Tom Ltd due to any failure by the Restaurant to provide timely notification of a change in accordance with the
							clause.<br />
							<br />

							<b>8. RESTAURANT REVIEWS</b><br />
							<br />

							8.1. General: The Restaurant acknowledges and agrees that Tiffin Tom Ltd may display on the Website ratings and comments (“Reviews”) provided by customers regarding the Restaurant or an Order.<br />
							8.2. Removal:Tiffin Tom Ltd will only remove those Reviews that Tiffin Tom Ltd determines in its sole discretion contain explicit, offensive or derogatory language or otherwise breach Tiffin Tom Ltd.’s review guidelines from
							time to time. The Restaurant acknowledges that Tiffin Tom Ltd is under no obligation (but reserves the right and sole discretion at any time and for any reason) to remove or edit any other Reviews.<br />
							8.3. Reviews by Restaurant: The Restaurant agrees that it shall not itself provide or cause any other party to provide any Reviews that are fraudulent or otherwise breach Tiffin Tom Ltd.’s review guidelines.<br />
							8.4. Liability: To the fullest extent permitted by law, Tiffin Tom Ltd assumes no responsibility or liability to the Restaurant for any Reviews.<br />
							<br />

							<b>9.CONSENTS & COMPLIANCE WITH LAWS</b><br />
							<br />
							9.1. General: The Restaurant confirms and undertakes, that it has obtained and will obtain and maintain any consents, licenses, permits, approvals or authorisations (“Consents”) of any person that may be required in connection
							with, and it is not party to and will not enter into any agreement which would be breached by, or under which any default would occur as a result of, signing this Agreement or performing any of its terms or generally running its
							business.<br />

							9.2. Franchises: If the Restaurant’s business, name, brand or logo is or becomes subject to any license or franchise arrangements, the Restaurant confirms it has obtained or will obtain any required Consents from its licensors
							or franchisors for the use and inclusion on the Website of such business, name, brand or logo, and shall fully indemnify Tiffin Tom Ltd for any losses, damages or claims made against or incurred by Tiffin Tom Ltd due to any
							failure to obtain such Consents.<br />

							9.3. Loss of Consents: The Restaurant will notify Tiffin Tom Ltd in writing immediately if any of the Consents described in clause 10.1 or 10.2 are revoked or suspended, or the Restaurant is otherwise unable to rely on or
							benefit from any such Consents for any reason.<br />

							9.4. Compliance with laws: The Restaurant will comply with all applicable laws and regulations including, without limitation in relation to health and safety, VAT, data protection and food standards, hygiene and information, and
							will provide reasonable evidence to Tiffin Tom Ltd of such compliance upon request.<br />

							9.5. Website terms & conditions: The Restaurant shall at all times comply with the Website terms and conditions (available on the Website, and as amended from time to time), and in particular (but not limited to) compliance with
							applicable security and data protection provisions when handling customer information or receiving, processing and delivering Orders. The Website terms and conditions are hereby incorporated into this Agreement. In the event of
							any conflict between this Agreement and any Website terms and conditions, the terms of this Agreement shall prevail.<br />

							9.6. Taxes: The Restaurant shall be solely responsible for self-assessing, claiming and remitting all its applicable taxes.<br />

							9.7. Inspections: The Restaurant agrees that Tiffin Tom Ltd has the right to inspect the Restaurant’s premises of no less than 24 hours’ notice, in order for Tiffin Tom Ltd to satisfy itself of the Restaurant’s compliance with
							this clause 9, to collect the Product in accordance with Tiffin Tom Ltd.’s ownership rights as set forth in clause 2.9 or for any other reasonable purpose.<br />
							<br />

							<b>10. MARKETING AND EXCLUSIVITY</b><br />
							<br />

							10.1 Display of Tiffin Tom Ltd branding: Where the Restaurant agrees to market Tiffin Tom Ltd and/or the Website by means of menus, stickers or other relevant advertising material, the Restaurant agrees to do so in accordance
							with Tiffin Tom Ltd.’s guidelines and instructions.<br />

							10.2 Tiffin Tom Ltd marketing: The Restaurant consents to receiving from time to time direct marketing communications relating to Tiffin Tom Ltd.’s products or services or those of Tiffin Tom Ltd.’s selected partners via e-mail,
							post, fax or other means of communication.<br />

							10.3 Non- disparagement: The Restaurant undertakes to refer positively to Tiffin Tom Ltd in relation to any publicity regarding the Orders and the Service in accordance with guidelines provided by Tiffin Tom Ltd and must not at
							any time actively encourage customers who have places Orders to place future Orders directly with the Restaurant, nor attempt to direct Orders placed with the Restaurant through any website other than the Website.<br />

							10.4 Exclusivity: For any duration of this Agreement and for a period of 12 months after its expiry or termination (the “Period”), the Restaurant agrees that it shall not have any direct or indirect financial interest in any
							business that carries out Competing Activities to Tiffin Tom Ltd within the United Kingdom. During the Period the Restaurant shall not (unless it notifies Tiffin Tom Ltd and receives Tiffin Tom Ltd.’s consent, which consent may
							be revoked at Tiffin Tom Ltd.’s sole discretion on one months notice) be a member of any other association or cooperate or be otherwise involved in any way with any third party which carries out competing activities to Tiffin
							Tom Ltd. “Competing Activities” means the same or similar services as Tiffin Tom Ltd is providing to the Restaurant under this Agreement or other activities having a similar purpose.<br />
							<br />

							<b>11. INTELLECTUAL PROPERTY RIGHTS</b><br />
							<br />

							11.1 Ownership of IPR: All IPR (as defined below) in or arising out of or in connection with the Service, the Product or the Website shall be owned by Tiffin Tom Ltd and nothing in this Agreement shall constitute a transfer of
							those IPR to the Restaurant.<br />

							11.2 Licence of Restaurant IPR: The Restaurant grants Tiffin Tom Ltd a licence to use the Restaurant’s name, menu, logo and other IPR for the purposes of providing the Service for the duration of this Agreement.<br />

							11.3 Third- party IP rights: The Restaurant confirms irrevocably to Tiffin Tom Ltd that the Restaurant’s name, Menu, logo and other material that the Restaurant may provide to Tiffin Tom Ltd for inclusion on the website do not
							violate, infringe or conflict with the IPR of any third party (including without limitation any licensor or franchisor).<br />

							11.4 Third- party IP claims: If a third party makes a claim against Tiffin Tom Ltd for the violation of the third party’s IPR relating to the Restaurant’s name, Menu, logo and/or other material provided by the Restaurant, the
							Restaurant shall fully indemnify and keep Tiffin Tom Ltd indemnified against any losses, damages or claims of any nature and all costs resulting therefrom.<br />

							11.5 “IPR” means: all patents, rights to inventions, utility models, copyright and related rights, trademarks, service marks, trade, business and domain names, rights in trade dress or get up, rights in goodwill or to sue for
							passing off, unfair competition rights, rights in designs, rights in computer software, database right, topography rights, moral rights, rights in confidential information (including know-how and trade secrets) and any other
							intellectual property rights, in each case whether registered or unregistered and including all applications for and renewals or extensions of such rights, and all similar or equivalent rights or forms of protection in any part
							of the world.<br />
							<br />

							<b>12. LIMITATIONS OF LIABILITY & INDEMNITY</b><br />
							<br />

							12.1. General: Nothing in this Agreement shall limit or exclude Tiffin Tom Ltd.’s of the Restaurant’s liability for death or personal injury caused by its negligence, or the negligence of its employees, agents or subcontractors;
							fraud or fraudulent misrepresentation; breach of the terms implied by section 2 of the Supply of Goods and Services act 1982 (title and quiet possession); or any other liability that cannot be limited or excluded under
							applicable law.<br />

							12.2. Exclusion of Tiffin Tom Ltd liability: Subject always to clause 16.1. Tiffin Tom Ltd shall not be liable to the Restaurant, whether in contract, tort (including negligence) breach of statutory duty, or otherwise, for any
							damages, costs, direct or indirect losses including without limitation loss of profit, or any consequential loss suffered by the Restaurant and arising out of or in connection with this Agreement, including resulting from
							faults, breakdowns or other interruptions to the Service for any reason.<br />

							12.3. Limitation of Tiffin Tom Ltd.’s liability: Subjects always to clauses 16.1 and 16.2, WCY’s total liability to the Restaurant in respect of all other losses arising under or in connection with this agreement howsoever
							caused (and whether in contract, tort (including negligence), breach of statutory duty, or otherwise, and including losses caused by Tiffin Tom Ltd.’s repudiatory breach or a deliberate breach of the Agreement by Tiffin Tom Ltd,
							its employees, agents or subcontractors) shall never exceed the aggregate amount of Restaurant Charges paid to Tiffin Tom Ltd by the Restaurant pursuant to this Agreement in the 12 months prior to the date that the liability
							arose.<br />

							12.4. Indemnity: The Restaurant fully indemnifies Tiffin Tom Ltd and will keep Tiffin Tom Ltd indemnified against any losses, damages or claims (and all related costs) made against Tiffin Tom Ltd by a customer or any third party
							in connection with the Restaurant’s failure to deliver or imperfect delivery of an Order or the restaurants failure to comply with this Agreement and/or any applicable laws, rules, and regulations in force at the relevant time.
							<br />

							12.5. Joinder: The Restaurant shall accept and shall not object to being included by Tiffin Tom Ltd in any manner to any third-party notice or otherwise in any proceedings instituted against Tiffin Tom Ltd, relating to the
							Restaurant’s acts or omissions in connection with this Agreement.<br />
							<br />

							<b>13. FORCE MAJEURE</b><br />
							<br />

							Tiffin Tom Ltd shall not be liable to the Restaurant as a result of any delay or failure to perform its obligations under this agreement because of a Force Majeure Event. A “Force Majeure Event” means an event beyond the
							reasonable control of Tiffin Tom Ltd including but not limited to strikes, lock outs or other industrial disputes (whether involving the workforce of Tiffin Tom Ltd or any other party), failure of a utility service or transport
							network, act of God, war, riot, civil commotion, malicious damage, compliance with any law or governmental order, rule, regulation or direction, accident, breakdown of plant or machinery, fire, flood, storm or default of
							suppliers or subcontractors, import and export restrictions, faults, breakdowns or other operational interruptions.<br />
							<br />

							<b>14. ENTIRE AGREEMENT</b><br />
							<br />

							This Agreement constitutes the entire agreement between the parties and supersedes and extinguishes all previous drafts, agreements, arrangements and understandings between them, whether written or oral, relating to its subject
							matter, and no party shall have any claim for innocent or negligent misrepresentation based upon any statement in this Agreement.<br />
							<br />

							<b>15. AMENDMENTS</b><br />
							<br />

							Upon 1 months’ prior written notice, Tiffin Tom Ltd may amend the terms of this Agreement. The Restaurant shall be notified of such amendments in writing either by fax, email or post. Unless the Restaurant responds to such
							notification within 2 calendar weeks objecting to the amendments or terminates this Agreement in accordance with clause 15.3, such amendments shall thereafter be construed as forming part of this Agreement and accepted by the
							Restaurant.<br />
							<br />

							<b>16. ASSIGNMENT</b> <br />
							<br />

							16.1. Tiffin Tom Ltd shall be entitled to assign all or any of its rights and obligations under this Agreement to any third party.<br />

							16.2. The Restaurant may only assign, transfer, charge, subcontract or deal in another manner with all or any of its right or obligations under the Agreement to a third party with Tiffin Tom Ltd.’s express prior written consent.
							<br />
							<br />

							<b>17.CONFIDENTIALITY</b><br />
							<br />

							The content of this Agreement and any information concerning the other party is to be treated as confidential and shall not be disclosed during the term of this agreement or at any time thereafter saves as required by law,
							provided that Tiffin Tom Ltd is entitled to use the Restaurant’s name as a reference.<br />
							<br />

							<b>18. INVALIDITY CLAUSE</b><br />
							<br />

							If any of the terms or conditions of this Agreement are declared wholly or partly invalid, illegal or unenforceable, the remainder of this Agreement shall remain in full force and effect and any wholly or partly invalid terms or
							conditions shall be modified to the minimum extent possible to make it valid, legal and enforceable.<br />
							<br />

							<b>19. GOVERNING LAW AND JURISDICTION</b><br />
							<br />

							20.1. <b>Governing Law:</b> This Agreement and any dispute or claim arising out or in connection with this Agreement or its subject matter or formation (including any dispute or claim relating to non- contractual obligations)
							shall be governed by and construed in accordance with English Law.<br />
							<br />

							21.2. <b>Jurisdiction:</b> The parties agree to submit any dispute arising in connection with this agreement to the exclusive jurisdiction of the courts of England and Wales (including any dispute or claim relating to non-
							contractual obligations).<br />
							<br />

							<b>22. E- SIGNATURE</b><br />
							<br />

							The parties consent to use an electronic signature service for the purposes of electronically executing this Agreement.<br />
						</p>

						<p class="bold" style="font-size: 13px; ">Signed for and behalf of Tiffin Tom Ltd (Office use only)</p>
						<p class="bold" style="font-size: 13px; ">Signed __________________________________</p>
						<p class="bold" style="font-size: 13px; ">Name __________________________________</p>
						<p class="bold" style="font-size: 13px; ">Date __________________________________</p>
						<p class="bold" style="font-size: 13px; ">Signed for and behalf of named RESTAURANT</p>
						<p class="bold" style="font-size: 13px; ">Signed __________________________________</p>
						<p class="bold" style="font-size: 13px; ">Name __________________________________</p>
						<p class="bold" style="font-size: 13px; ">Date __________________________________</p>
					</body>
				</html>';

        $mpdf = new Mpdf();
        $mpdf->SetHeader('Restaurant agreement');
        $mpdf->SetFooter('{PAGENO}');
        $mpdf->defaultheaderfontsize = 10;
        $mpdf->defaultheaderfontstyle = 'B';
        $mpdf->defaultheaderline = 0;
        $mpdf->defaultfooterfontsize = 10;
        $mpdf->defaultfooterfontstyle = 'BI';
        $mpdf->defaultfooterline = 0;
        // $mpdf->SetWatermarkImage(SITESETTINGSS_FEVICON_URL . $siteSettingss['site_fav']);
        $mpdf->showWatermarkImage = true;
        $mpdf->WriteHTML($html);
        $path =  $restaurant_registration->id . '_as.pdf';
        // $mpdf->Output($path, "F");
        // $pdfContent = ;
        // $file_local = Storage::disk('public')->get('images/' . $restaurant_registration->id . '_as.pdf');
        // $file_local = Storage::disk('public')->url("images/$restaurant_registration->id . '_as.pdf'");
        Storage::disk('tiffintom')->put(ImageHelper::$TYPE_TIFFINTOM_BECOME . '/' . $path, $mpdf->Output('', 'S'));
        // $fileName = ImageHelper::saveImageToServer($file_local, ImageHelper::$TYPE_TIFFINTOM_BECOME, 'tiffintom');
    }

    public function becomeMail($id)
    {
        $restaurant_registration = TiffintomRestaurantRegistration::find($id);
        if (empty($restaurant_registration)) Utils::error("restaurant registration not found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $html = '<html xmlns="http://www.w3.org/1999/xhtml" xmlns:v="urn:schemas-microsoft-com:vml" xmlns:o="urn:schemas-microsoft-com:office:office">
        <head>
            <meta charset="UTF-8">
            <meta http-equiv="X-UA-Compatible" content="IE=edge">
            <meta name="viewport" content="width=device-width, initial-scale=1">
            <title>*|MC:SUBJECT|*</title>
            
        <style type="text/css">
            p{
                margin:10px 0;
                padding:0;
            }
            table{
                border-collapse:collapse;
            }
            h1,h2,h3,h4,h5,h6{
                display:block;
                margin:0;
                padding:0;
            }
            img,a img{
                border:0;
                height:auto;
                outline:none;
                text-decoration:none;
            }
            body,#bodyTable,#bodyCell{
                height:100%;
                margin:0;
                padding:0;
                width:100%;
            }
            .mcnPreviewText{
                display:none !important;
            }
            #outlook a{
                padding:0;
            }
            img{
                -ms-interpolation-mode:bicubic;
            }
            table{
                mso-table-lspace:0pt;
                mso-table-rspace:0pt;
            }
            .ReadMsgBody{
                width:100%;
            }
            .ExternalClass{
                width:100%;
            }
            p,a,li,td,blockquote{
                mso-line-height-rule:exactly;
            }
            a[href^=tel],a[href^=sms]{
                color:inherit;
                cursor:default;
                text-decoration:none;
            }
            p,a,li,td,body,table,blockquote{
                -ms-text-size-adjust:100%;
                -webkit-text-size-adjust:100%;
            }
            .ExternalClass,.ExternalClass p,.ExternalClass td,.ExternalClass div,.ExternalClass span,.ExternalClass font{
                line-height:100%;
            }
            a[x-apple-data-detectors]{
                color:inherit !important;
                text-decoration:none !important;
                font-size:inherit !important;
                font-family:inherit !important;
                font-weight:inherit !important;
                line-height:inherit !important;
            }
            #bodyCell{
                padding:10px;
            }
            .templateContainer{
                max-width:600px !important;
            }
            a.mcnButton{
                display:block;
            }
            .mcnImage,.mcnRetinaImage{
                vertical-align:bottom;
            }
            .mcnTextContent{
                word-break:break-word;
            }
            .mcnTextContent img{
                height:auto !important;
            }
            .mcnDividerBlock{
                table-layout:fixed !important;
            }
        /*
        @tab Page
        @section Background Style
        @tip Set the background color and top border for your email. You may want to choose colors that match your companys branding.
        */
            body,#bodyTable{
                /*@editable*/background-color:#FFFFFF;
            }
        /*
        @tab Page
        @section Background Style
        @tip Set the background color and top border for your email. You may want to choose colors that match your companys branding.
        */
            #bodyCell{
                /*@editable*/border-top:0;
            }
        /*
        @tab Page
        @section Email Border
        @tip Set the border for your email.
        */
            .templateContainer{
                /*@editable*/border:0;
            }
        /*
        @tab Page
        @section Heading 1
        @tip Set the styling for all first-level headings in your emails. These should be the largest of your headings.
        @style heading 1
        */
            h1{
                /*@editable*/color:#FFFFFF;
                /*@editable*/font-family:Arial, "Helvetica Neue", Helvetica, sans-serif;
                /*@editable*/font-size:32px;
                /*@editable*/font-style:normal;
                /*@editable*/font-weight:bold;
                /*@editable*/line-height:125%;
                /*@editable*/letter-spacing:normal;
                /*@editable*/text-align:center;
            }
        /*
        @tab Page
        @section Heading 2
        @tip Set the styling for all second-level headings in your emails.
        @style heading 2
        */
            h2{
                /*@editable*/color:#FFFFFF;
                /*@editable*/font-family:"Times New Roman", Times, Baskerville, Georgia, serif;
                /*@editable*/font-size:22px;
                /*@editable*/font-style:normal;
                /*@editable*/font-weight:bold;
                /*@editable*/line-height:125%;
                /*@editable*/letter-spacing:normal;
                /*@editable*/text-align:center;
            }
        /*
        @tab Page
        @section Heading 3
        @tip Set the styling for all third-level headings in your emails.
        @style heading 3
        */
            h3{
                /*@editable*/color:#FFFFFF;
                /*@editable*/font-family:Arial, "Helvetica Neue", Helvetica, sans-serif;
                /*@editable*/font-size:24px;
                /*@editable*/font-style:normal;
                /*@editable*/font-weight:bold;
                /*@editable*/line-height:125%;
                /*@editable*/letter-spacing:normal;
                /*@editable*/text-align:center;
            }
        /*
        @tab Page
        @section Heading 4
        @tip Set the styling for all fourth-level headings in your emails. These should be the smallest of your headings.
        @style heading 4
        */
            h4{
                /*@editable*/color:#FFFFFF;
                /*@editable*/font-family:Arial, "Helvetica Neue", Helvetica, sans-serif;
                /*@editable*/font-size:22px;
                /*@editable*/font-style:normal;
                /*@editable*/font-weight:normal;
                /*@editable*/line-height:125%;
                /*@editable*/letter-spacing:normal;
                /*@editable*/text-align:center;
            }
        /*
        @tab Preheader
        @section Preheader Style
        @tip Set the background color and borders for your emails preheader area.
        */
            #templatePreheader{
                /*@editable*/background-color:#ffffff;
                /*@editable*/border-top:0;
                /*@editable*/border-bottom:0;
                /*@editable*/padding-top:0px;
                /*@editable*/padding-bottom:0px;
            }
        /*
        @tab Preheader
        @section Preheader Text
        @tip Set the styling for your emails preheader text. Choose a size and color that is easy to read.
        */
            #templatePreheader .mcnTextContent,#templatePreheader .mcnTextContent p{
                /*@editable*/color:#656565;
                /*@editable*/font-family:Helvetica;
                /*@editable*/font-size:12px;
                /*@editable*/line-height:150%;
                /*@editable*/text-align:center;
            }
        /*
        @tab Preheader
        @section Preheader Link
        @tip Set the styling for your emails preheader links. Choose a color that helps them stand out from your text.
        */
            #templatePreheader .mcnTextContent a,#templatePreheader .mcnTextContent p a{
                /*@editable*/color:#656565;
                /*@editable*/font-weight:normal;
                /*@editable*/text-decoration:underline;
            }
        /*
        @tab Header
        @section Header Style
        @tip Set the background color and borders for your emails header area.
        */
            #templateHeader{
                /*@editable*/background-color:#FFFFFF;
                /*@editable*/border-top:0;
                /*@editable*/border-bottom:0;
                /*@editable*/padding-top:0;
                /*@editable*/padding-bottom:9px;
            }
        /*
        @tab Header
        @section Header Text
        @tip Set the styling for your emails header text. Choose a size and color that is easy to read.
        */
            #templateHeader .mcnTextContent,#templateHeader .mcnTextContent p{
                /*@editable*/color:#202020;
                /*@editable*/font-family:Helvetica;
                /*@editable*/font-size:16px;
                /*@editable*/line-height:150%;
                /*@editable*/text-align:left;
            }
        /*
        @tab Header
        @section Header Link
        @tip Set the styling for your emails header links. Choose a color that helps them stand out from your text.
        */
            #templateHeader .mcnTextContent a,#templateHeader .mcnTextContent p a{
                /*@editable*/color:#2BAADF;
                /*@editable*/font-weight:normal;
                /*@editable*/text-decoration:underline;
            }
        /*
        @tab Body
        @section Body Style
        @tip Set the background color and borders for your emails body area.
        */
            #templateBody{
                /*@editable*/background-color:#3a606e;
                /*@editable*/border-top:0;
                /*@editable*/border-bottom:0;
                /*@editable*/padding-top:0;
                /*@editable*/padding-bottom:36px;
            }
        /*
        @tab Body
        @section Body Text
        @tip Set the styling for your emails body text. Choose a size and color that is easy to read.
        */
            #templateBody .mcnTextContent,#templateBody .mcnTextContent p{
                /*@editable*/color:#FFFFFF;
                /*@editable*/font-family:Georgia, Times, "Times New Roman", serif;
                /*@editable*/font-size:16px;
                /*@editable*/line-height:150%;
                /*@editable*/text-align:left;
            }
        /*
        @tab Body
        @section Body Link
        @tip Set the styling for your emails body links. Choose a color that helps them stand out from your text.
        */
            #templateBody .mcnTextContent a,#templateBody .mcnTextContent p a{
                /*@editable*/color:#FFFFFF;
                /*@editable*/font-weight:normal;
                /*@editable*/text-decoration:underline;
            }
        /*
        @tab Footer
        @section Footer Style
        @tip Set the background color and borders for your emails footer area.
        */
            #templateFooter{
                /*@editable*/background-color:#FFFFFF;
                /*@editable*/border-top:0;
                /*@editable*/border-bottom:0;
                /*@editable*/padding-top:9px;
                /*@editable*/padding-bottom:9px;
            }
        /*
        @tab Footer
        @section Footer Text
        @tip Set the styling for your emails footer text. Choose a size and color that is easy to read.
        */
            #templateFooter .mcnTextContent,#templateFooter .mcnTextContent p{
                /*@editable*/color:#656565;
                /*@editable*/font-family:Helvetica;
                /*@editable*/font-size:12px;
                /*@editable*/line-height:150%;
                /*@editable*/text-align:center;
            }
        /*
        @tab Footer
        @section Footer Link
        @tip Set the styling for your emails footer links. Choose a color that helps them stand out from your text.
        */
            #templateFooter .mcnTextContent a,#templateFooter .mcnTextContent p a{
                /*@editable*/color:#656565;
                /*@editable*/font-weight:normal;
                /*@editable*/text-decoration:underline;
            }
        @media only screen and (min-width:768px){
            .templateContainer{
                width:600px !important;
            }

    }	@media only screen and (max-width: 480px){
            body,table,td,p,a,li,blockquote{
                -webkit-text-size-adjust:none !important;
            }

    }	@media only screen and (max-width: 480px){
            body{
                width:100% !important;
                min-width:100% !important;
            }

    }	@media only screen and (max-width: 480px){
            #bodyCell{
                padding-top:10px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnRetinaImage{
                max-width:100% !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnImage{
                width:100% !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnCartContainer,.mcnCaptionTopContent,.mcnRecContentContainer,.mcnCaptionBottomContent,.mcnTextContentContainer,.mcnBoxedTextContentContainer,.mcnImageGroupContentContainer,.mcnCaptionLeftTextContentContainer,.mcnCaptionRightTextContentContainer,.mcnCaptionLeftImageContentContainer,.mcnCaptionRightImageContentContainer,.mcnImageCardLeftTextContentContainer,.mcnImageCardRightTextContentContainer,.mcnImageCardLeftImageContentContainer,.mcnImageCardRightImageContentContainer{
                max-width:100% !important;
                width:100% !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnBoxedTextContentContainer{
                min-width:100% !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnImageGroupContent{
                padding:9px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnCaptionLeftContentOuter .mcnTextContent,.mcnCaptionRightContentOuter .mcnTextContent{
                padding-top:9px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnImageCardTopImageContent,.mcnCaptionBottomContent:last-child .mcnCaptionBottomImageContent,.mcnCaptionBlockInner .mcnCaptionTopContent:last-child .mcnTextContent{
                padding-top:18px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnImageCardBottomImageContent{
                padding-bottom:9px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnImageGroupBlockInner{
                padding-top:0 !important;
                padding-bottom:0 !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnImageGroupBlockOuter{
                padding-top:9px !important;
                padding-bottom:9px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnTextContent,.mcnBoxedTextContentColumn{
                padding-right:18px !important;
                padding-left:18px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcnImageCardLeftImageContent,.mcnImageCardRightImageContent{
                padding-right:18px !important;
                padding-bottom:0 !important;
                padding-left:18px !important;
            }

    }	@media only screen and (max-width: 480px){
            .mcpreview-image-uploader{
                display:none !important;
                width:100% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Heading 1
        @tip Make the first-level headings larger in size for better readability on small screens.
        */
            h1{
                /*@editable*/font-size:22px !important;
                /*@editable*/line-height:125% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Heading 2
        @tip Make the second-level headings larger in size for better readability on small screens.
        */
            h2{
                /*@editable*/font-size:20px !important;
                /*@editable*/line-height:125% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Heading 3
        @tip Make the third-level headings larger in size for better readability on small screens.
        */
            h3{
                /*@editable*/font-size:18px !important;
                /*@editable*/line-height:125% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Heading 4
        @tip Make the fourth-level headings larger in size for better readability on small screens.
        */
            h4{
                /*@editable*/font-size:16px !important;
                /*@editable*/line-height:150% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Boxed Text
        @tip Make the boxed text larger in size for better readability on small screens. We recommend a font size of at least 16px.
        */
            .mcnBoxedTextContentContainer .mcnTextContent,.mcnBoxedTextContentContainer .mcnTextContent p{
                /*@editable*/font-size:14px !important;
                /*@editable*/line-height:150% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Preheader Visibility
        @tip Set the visibility of the emails preheader on small screens. You can hide it to save space.
        */
            #templatePreheader{
                /*@editable*/display:block !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Preheader Text
        @tip Make the preheader text larger in size for better readability on small screens.
        */
            #templatePreheader .mcnTextContent,#templatePreheader .mcnTextContent p{
                /*@editable*/font-size:14px !important;
                /*@editable*/line-height:150% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Header Text
        @tip Make the header text larger in size for better readability on small screens.
        */
            #templateHeader .mcnTextContent,#templateHeader .mcnTextContent p{
                /*@editable*/font-size:16px !important;
                /*@editable*/line-height:150% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Body Text
        @tip Make the body text larger in size for better readability on small screens. We recommend a font size of at least 16px.
        */
            #templateBody .mcnTextContent,#templateBody .mcnTextContent p{
                /*@editable*/font-size:16px !important;
                /*@editable*/line-height:150% !important;
            }

    }	@media only screen and (max-width: 480px){
        /*
        @tab Mobile Styles
        @section Footer Text
        @tip Make the footer content text larger in size for better readability on small screens.
        */
            #templateFooter .mcnTextContent,#templateFooter .mcnTextContent p{
                /*@editable*/font-size:14px !important;
                /*@editable*/line-height:150% !important;
            }

    }</style></head>
        <body>
           <span class="mcnPreviewText" style="display:none; font-size:0px; line-height:0px; max-height:0px; max-width:0px; opacity:0; overflow:hidden; visibility:hidden; mso-hide:all;">*|MC_PREVIEW_TEXT|*</span>
            
            <center>
                <table align="center" border="0" cellpadding="0" cellspacing="0" height="100%" width="100%" id="bodyTable">
                    <tr>
                        <td align="center" valign="top" id="bodyCell">
                         
                            <table border="0" cellpadding="0" cellspacing="0" width="100%" class="templateContainer">
                                <tr>
                                    <td valign="top" id="templatePreheader"></td>
                                </tr>
                                <tr>
                                    <td valign="top" id="templateHeader"><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnImageBlock" style="min-width:100%;">
        <tbody class="mcnImageBlockOuter">
                <tr>
                    <td valign="top" style="padding:9px" class="mcnImageBlockInner">
                        <table align="left" width="100%" border="0" cellpadding="0" cellspacing="0" class="mcnImageContentContainer" style="min-width:100%;">
                            <tbody><tr>
                                <td class="mcnImageContent" valign="top" style="padding-right: 9px; padding-left: 9px; padding-top: 0; padding-bottom: 0; text-align:center;">
                                    
                                        
                                            <img align="center" alt="" src="https://mcusercontent.com/2a8fd892458fd770525566d3f/images/24149036-8c5b-4a06-a6c8-bb635544be3d.png" width="186.12" style="max-width:1200px; padding-bottom: 0; display: inline !important; vertical-align: bottom;" class="mcnImage">
                                        
                                    
                                </td>
                            </tr>
                        </tbody></table>
                    </td>
                </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnDividerBlock" style="min-width:100%;">
        <tbody class="mcnDividerBlockOuter">
            <tr>
                <td class="mcnDividerBlockInner" style="min-width:100%; padding:18px;">
                    <table class="mcnDividerContent" border="0" cellpadding="0" cellspacing="0" width="100%" style="min-width: 100%;border-top: 2px solid #EAEAEA;">
                        <tbody><tr>
                            <td>
                                <span></span>
                            </td>
                        </tr>
                    </tbody></table>
                </td>
            </tr>
        </tbody>
    </table></td>
                                </tr>
                                <tr>
                                    <td valign="top" id="templateBody"><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnTextBlock" style="min-width:100%;">
        <tbody class="mcnTextBlockOuter">
            <tr>
                <td valign="top" class="mcnTextBlockInner" style="padding-top:9px;">
                    <table align="left" border="0" cellpadding="0" cellspacing="0" style="max-width:100%; min-width:100%;" width="100%" class="mcnTextContentContainer">
                        <tbody><tr>
                            
                            <td valign="top" class="mcnTextContent" style="padding: 0px 18px 9px; line-height: 100%;">
                            
                                <h1 style="text-align: center;"><br>
    <span style="font-family:roboto,helvetica neue,helvetica,arial,sans-serif">Welcome on board!</span></h1>

    <div style="text-align: center;"><br>
    <span style="font-family:roboto,helvetica neue,helvetica,arial,sans-serif">You have almost got your free app, website and marketing...</span></div>

                            </td>
                        </tr>
                    </tbody></table>
                    
                </td>
            </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnImageBlock" style="min-width:100%;">
        <tbody class="mcnImageBlockOuter">
                <tr>
                    <td valign="top" style="padding:0px" class="mcnImageBlockInner">
                        <table align="left" width="100%" border="0" cellpadding="0" cellspacing="0" class="mcnImageContentContainer" style="min-width:100%;">
                            <tbody><tr>
                                <td class="mcnImageContent" valign="top" style="padding-right: 0px; padding-left: 0px; padding-top: 0; padding-bottom: 0; text-align:center;">
                                    
                                        
                                            <img align="center" alt="" src="https://mcusercontent.com/2a8fd892458fd770525566d3f/images/a3cdb0d7-e3ff-4b79-904b-2879b8458ff8.png" width="252" style="max-width:638px; padding-bottom: 0; display: inline !important; vertical-align: bottom;" class="mcnImage">
                                        
                                    
                                </td>
                            </tr>
                        </tbody></table>
                    </td>
                </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnTextBlock" style="min-width:100%;">
        <tbody class="mcnTextBlockOuter">
            <tr>
                <td valign="top" class="mcnTextBlockInner" style="padding-top:9px;">
                  
                    <table align="left" border="0" cellpadding="0" cellspacing="0" style="max-width:100%; min-width:100%;" width="100%" class="mcnTextContentContainer">
                        <tbody><tr>
                            
                            <td valign="top" class="mcnTextContent" style="padding: 0px 18px 9px; font-family: &quot;Courier New&quot;, Courier, &quot;Lucida Sans Typewriter&quot;, &quot;Lucida Typewriter&quot;, monospace; line-height: 150%;">
                            
                                <div style="text-align: center;"><span style="font-family:roboto,helvetica neue,helvetica,arial,sans-serif"><span style="font-size:21px">Thank you for registering to become a TiffinTom partner. We&nbsp;just need a few more details from you and you&nbsp;will be up and&nbsp;running in no time!</span></span></div>

                            </td>
                        </tr>
                    </tbody></table>
                    
                </td>
            </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnTextBlock" style="min-width:100%;">
        <tbody class="mcnTextBlockOuter">
            <tr>
                <td valign="top" class="mcnTextBlockInner" style="padding-top:9px;">
                  
                    <table align="left" border="0" cellpadding="0" cellspacing="0" style="max-width:100%; min-width:100%;" width="100%" class="mcnTextContentContainer">
                        <tbody><tr>
                            
                            <td valign="top" class="mcnTextContent" style="padding: 0px 18px 9px; font-style: normal; font-weight: normal; line-height: 150%;">
                            
                                <div style="text-align: center;"><br>
    <span style="font-family:roboto,helvetica neue,helvetica,arial,sans-serif"><strong><u><span style="font-size:24px">Please provide:</span></u></strong><br>
    <br>
    - Bank account details&nbsp;to receive your payments&nbsp;(account holders name, sort code &amp; account&nbsp;number)<br>
    <br>
    - Personal ID (passport, drivers license or national ID)</span><br>
    <br>
    <strong><span style="font-family:roboto,helvetica neue,helvetica,arial,sans-serif">Just reply back to us on this email with the above infomation and your ID attached.</span></strong><br>
    &nbsp;</div>

                            </td>
                        </tr>
                    </tbody></table>
                    
                </td>
            </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnButtonBlock" style="min-width:100%;">
        <tbody class="mcnButtonBlockOuter">
            <tr>
                <td style="padding-top:0; padding-right:18px; padding-bottom:18px; padding-left:18px;" valign="top" align="center" class="mcnButtonBlockInner">
                    <table border="0" cellpadding="0" cellspacing="0" class="mcnButtonContentContainer" style="border-collapse: separate !important;border-radius: 0px;background-color: #FFFFFF;">
                        <tbody>
                            <tr>
                                <td align="center" valign="middle" class="mcnButtonContent" style="font-family: Arial, &quot;Helvetica Neue&quot;, Helvetica, sans-serif; font-size: 16px; padding: 15px;">
                                    <a class="mcnButton " title="shop.tiffintom.com" href="http://shop.tiffintom.com/epos/epos%20hardware/Sunmi%20v2%20moile%20pos" target="_blank" style="font-weight: bold;letter-spacing: normal;line-height: 100%;text-align: center;text-decoration: none;color: #F24934;">PURCHASE DEVICE</a>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                </td>
            </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnButtonBlock" style="min-width:100%;">
        <tbody class="mcnButtonBlockOuter">
            <tr>
                <td style="padding-top:0; padding-right:18px; padding-bottom:18px; padding-left:18px;" valign="top" align="center" class="mcnButtonBlockInner">
                    <table border="0" cellpadding="0" cellspacing="0" class="mcnButtonContentContainer" style="border-collapse: separate !important;border: 1px none;border-radius: 0px;background-color: #FFFFFF;">
                        <tbody>
                            <tr>
                                <td align="center" valign="middle" class="mcnButtonContent" style="font-family: Arial, &quot;Helvetica Neue&quot;, Helvetica, sans-serif; font-size: 16px; padding: 15px;">
                                    <a class="mcnButton " title="Download partner app" href="http://www.tiffintom.com/users/download" target="_blank" style="font-weight: bold;letter-spacing: normal;line-height: 100%;text-align: center;text-decoration: none;color: #F24934;">DOWNLOAD PARTNER APP</a>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                </td>
            </tr>
        </tbody>
    </table></td>
                                </tr>
                                <tr>
                                    <td align="center" valign="top">
                                        <img src="https://cdn-images.mailchimp.com/template_images/gallery/8a7030a1-3d71-45ca-aa5d-e244b31efdf5.png" width="100%" height="auto" style="display:block; margin:0; padding:0;">
                                    </td>
                                </tr>
                                <tr>
                                    <td valign="top" id="templateFooter"><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnDividerBlock" style="min-width:100%;">
        <tbody class="mcnDividerBlockOuter">
            <tr>
                <td class="mcnDividerBlockInner" style="min-width:100%; padding:18px;">
                    <table class="mcnDividerContent" border="0" cellpadding="0" cellspacing="0" width="100%" style="min-width: 100%;border-top: 2px solid #EAEAEA;">
                        <tbody><tr>
                            <td>
                                <span></span>
                            </td>
                        </tr>
                    </tbody></table>
                </td>
            </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnTextBlock" style="min-width:100%;">
        <tbody class="mcnTextBlockOuter">
            <tr>
                <td valign="top" class="mcnTextBlockInner" style="padding-top:9px;">
                    <table align="left" border="0" cellpadding="0" cellspacing="0" style="max-width:100%; min-width:100%;" width="100%" class="mcnTextContentContainer">
                        <tbody><tr>
                            
                            <td valign="top" class="mcnTextContent" style="padding-top:0; padding-right:18px; padding-bottom:9px; padding-left:18px;">
                            
                                <img data-file-id="2013686" height="97" src="https://mcusercontent.com/2a8fd892458fd770525566d3f/images/7d76a5c2-b696-4991-a2e0-536935007c19.png" style="border: 0px  ; width: 200px; height: 97px; margin: 0px;" width="200">
                            </td>
                        </tr>
                    </tbody></table>
                </td>
            </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnFollowBlock" style="min-width:100%;">
        <tbody class="mcnFollowBlockOuter">
            <tr>
                <td align="center" valign="top" style="padding:9px" class="mcnFollowBlockInner">
                    <table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnFollowContentContainer" style="min-width:100%;">
        <tbody><tr>
            <td align="center" style="padding-left:9px;padding-right:9px;">
                <table border="0" cellpadding="0" cellspacing="0" width="100%" style="min-width:100%;" class="mcnFollowContent">
                    <tbody><tr>
                        <td align="center" valign="top" style="padding-top:9px; padding-right:9px; padding-left:9px;">
                            <table align="center" border="0" cellpadding="0" cellspacing="0">
                                <tbody><tr>
                                    <td align="center" valign="top">
                                                <table align="left" border="0" cellpadding="0" cellspacing="0" style="display:inline;">
                                                    <tbody><tr>
                                                        <td valign="top" style="padding-right:10px; padding-bottom:9px;" class="mcnFollowContentItemContainer">
                                                            <table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnFollowContentItem">
                                                                <tbody><tr>
                                                                    <td align="left" valign="middle" style="padding-top:5px; padding-right:10px; padding-bottom:5px; padding-left:9px;">
                                                                        <table align="left" border="0" cellpadding="0" cellspacing="0" width="">
                                                                            <tbody><tr>
                                                                                
                                                                                    <td align="center" valign="middle" width="24" class="mcnFollowIconContent">
                                                                                        <a href="http://www.twitter.com/tiffintomuk" target="_blank"><img src="https://cdn-images.mailchimp.com/icons/social-block-v2/light-twitter-48.png" alt="Twitter" style="display:block;" height="24" width="24" class=""></a>
                                                                                    </td>
                                                                                
                                                                                
                                                                            </tr>
                                                                        </tbody></table>
                                                                    </td>
                                                                </tr>
                                                            </tbody></table>
                                                        </td>
                                                    </tr>
                                                </tbody></table>
                                                <table align="left" border="0" cellpadding="0" cellspacing="0" style="display:inline;">
                                                    <tbody><tr>
                                                        <td valign="top" style="padding-right:10px; padding-bottom:9px;" class="mcnFollowContentItemContainer">
                                                            <table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnFollowContentItem">
                                                                <tbody><tr>
                                                                    <td align="left" valign="middle" style="padding-top:5px; padding-right:10px; padding-bottom:5px; padding-left:9px;">
                                                                        <table align="left" border="0" cellpadding="0" cellspacing="0" width="">
                                                                            <tbody><tr>
                                                                                
                                                                                    <td align="center" valign="middle" width="24" class="mcnFollowIconContent">
                                                                                        <a href="http://www.facebook.com/tiffintom" target="_blank"><img src="https://cdn-images.mailchimp.com/icons/social-block-v2/light-facebook-48.png" alt="Facebook" style="display:block;" height="24" width="24" class=""></a>
                                                                                    </td>
                                                                                
                                                                                
                                                                            </tr>
                                                                        </tbody></table>
                                                                    </td>
                                                                </tr>
                                                            </tbody></table>
                                                        </td>
                                                    </tr>
                                                </tbody></table>
                                                <table align="left" border="0" cellpadding="0" cellspacing="0" style="display:inline;">
                                                    <tbody><tr>
                                                        <td valign="top" style="padding-right:10px; padding-bottom:9px;" class="mcnFollowContentItemContainer">
                                                            <table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnFollowContentItem">
                                                                <tbody><tr>
                                                                    <td align="left" valign="middle" style="padding-top:5px; padding-right:10px; padding-bottom:5px; padding-left:9px;">
                                                                        <table align="left" border="0" cellpadding="0" cellspacing="0" width="">
                                                                            <tbody><tr>
                                                                                
                                                                                    <td align="center" valign="middle" width="24" class="mcnFollowIconContent">
                                                                                        <a href="https://www.tiffintom.com/" target="_blank"><img src="https://cdn-images.mailchimp.com/icons/social-block-v2/light-link-48.png" alt="Website" style="display:block;" height="24" width="24" class=""></a>
                                                                                    </td>
                                                                                
                                                                                
                                                                            </tr>
                                                                        </tbody></table>
                                                                    </td>
                                                                </tr>
                                                            </tbody></table>
                                                        </td>
                                                    </tr>
                                                </tbody></table>
                                            
                                                <table align="left" border="0" cellpadding="0" cellspacing="0" style="display:inline;">
                                                    <tbody><tr>
                                                        <td valign="top" style="padding-right:10px; padding-bottom:9px;" class="mcnFollowContentItemContainer">
                                                            <table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnFollowContentItem">
                                                                <tbody><tr>
                                                                    <td align="left" valign="middle" style="padding-top:5px; padding-right:10px; padding-bottom:5px; padding-left:9px;">
                                                                        <table align="left" border="0" cellpadding="0" cellspacing="0" width="">
                                                                            <tbody><tr>
                                                                                
                                                                                    <td align="center" valign="middle" width="24" class="mcnFollowIconContent">
                                                                                        <a href="http://instagram.com/tiffintomuk" target="_blank"><img src="https://cdn-images.mailchimp.com/icons/social-block-v2/light-instagram-48.png" alt="Instagram" style="display:block;" height="24" width="24" class=""></a>
                                                                                    </td>
                                                                                
                                                                                
                                                                            </tr>
                                                                        </tbody></table>
                                                                    </td>
                                                                </tr>
                                                            </tbody></table>
                                                        </td>
                                                    </tr>
                                                </tbody></table>
                                                <table align="left" border="0" cellpadding="0" cellspacing="0" style="display:inline;">
                                                    <tbody><tr>
                                                        <td valign="top" style="padding-right:0; padding-bottom:9px;" class="mcnFollowContentItemContainer">
                                                            <table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnFollowContentItem">
                                                                <tbody><tr>
                                                                    <td align="left" valign="middle" style="padding-top:5px; padding-right:10px; padding-bottom:5px; padding-left:9px;">
                                                                        <table align="left" border="0" cellpadding="0" cellspacing="0" width="">
                                                                            <tbody><tr>
                                                                                
                                                                                    <td align="center" valign="middle" width="24" class="mcnFollowIconContent">
                                                                                        <a href="https://www.youtube.com/channel/UC00x1XQnGTxJOE6AgHH1QYA" target="_blank"><img src="https://cdn-images.mailchimp.com/icons/social-block-v2/light-youtube-48.png" alt="YouTube" style="display:block;" height="24" width="24" class=""></a>
                                                                                    </td>
                                                                                
                                                                                
                                                                            </tr>
                                                                        </tbody></table>
                                                                    </td>
                                                                </tr>
                                                            </tbody></table>
                                                        </td>
                                                    </tr>
                                                </tbody></table>
                                    </td>
                                </tr>
                            </tbody></table>
                        </td>
                    </tr>
                </tbody></table>
            </td>
        </tr>
    </tbody></table>

                </td>
            </tr>
        </tbody>
    </table><table border="0" cellpadding="0" cellspacing="0" width="100%" class="mcnTextBlock" style="min-width:100%;">
        <tbody class="mcnTextBlockOuter">
            <tr>
                <td valign="top" class="mcnTextBlockInner" style="padding-top:9px;">
                  <table align="left" border="0" cellpadding="0" cellspacing="0" style="max-width:100%; min-width:100%;" width="100%" class="mcnTextContentContainer">
                        <tbody><tr>
                            
                            <td valign="top" class="mcnTextContent" style="padding-top:0; padding-right:18px; padding-bottom:9px; padding-left:18px;">
                            
                                <em>Copyright © 2020 Tiffin Tom Limited, All rights reserved.</em><br>
    <br>
    <strong>Our mailing address is:</strong><br>
    admin@tiffintom.com
                            </td>
                        </tr>
                    </tbody></table>
                </td>
            </tr>
        </tbody>
    </table></td>
                                </tr>
                            </table>
                        </td>
                    </tr>
                </table>
            </center>
        </body>
    </html>';

        $customerSubject = 'Become Restaurant';

        $separator = md5(time());
        $eol = PHP_EOL;
        $filename = "Restaurant_agreement.pdf";
        $fileatt     = env('FTP_IMAGE_PATH_TIFFINTOM') . ImageHelper::$TYPE_TIFFINTOM_BECOME . '/' . $restaurant_registration->id . "_as.pdf";
        //$fileatt     = "https://www.tiffintom.net/images/340_as.pdf";
        // $file = fopen($fileatt, 'rb');
        // $data = fread($file, filesize($fileatt));
        // fclose($file);
        $attachment = chunk_split(base64_encode(file_get_contents($fileatt)));
        $headers = '';
        $headers .= "MIME-Version: 1.0" . $eol;
        $headers .= "Content-Type: multipart/mixed; boundary=\"" . $separator . "\"" . $eol . $eol;
        $body = "Content-Transfer-Encoding: 7bit" . $eol;
        $body .= "This is a MIME encoded message." . $eol;
        $body .= "--" . $separator . $eol;
        $body .= "Content-Type: text/html; charset=\"UTF-8\"" . $eol;
        $body .= "Content-Transfer-Encoding: 8bit" . $eol . $eol;
        $body .= $html . $eol;
        $body .= "--" . $separator . $eol;
        $body .= "Content-Type: application/octet-stream; name=\"" . $filename . "\"" . $eol;
        $body .= "Content-Transfer-Encoding: base64" . $eol;
        $body .= "Content-Disposition: attachment" . $eol . $eol;
        $body .= $attachment . $eol;
        $html .= "--" . $separator . "--";

        $email['body'] = $html;
        $email['subject'] = $customerSubject;
        $email['mail_type'] = $siteSettings->mail_option;
        $email['to'] = $restaurant_registration->email;
        $email['from'] = $siteSettings->admin_email;
        $email['header_pdf'] = $headers;
        (new MailHelper())->sendMail($email);

        return 'Mail Send Successfully';
    }
}