<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomRestaurantVoucher;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomRestaurantVoucherController extends Controller
{
    /**
     * List restaurant_vouchers
     * GET /tiffintom/restaurant_vouchers
     */
    public function index(Request $req)
    {
        $restaurant_vouchers = TiffintomRestaurantVoucher::selectRaw("restaurantvoucher.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('id', 'DESC');

        if ($query = $req->query("restaurant_id", false)) {
            $restaurant_vouchers->where('restaurant_id', $query);
            // User wise check
            if ($req->hasHeader('business-id')) {
                $business = Business::where('restaurant_id', $restaurant_vouchers)->first();
                if (empty($business)) Utils::error('User not authenticated.', 404);
                (new ExtraController())->userCheck($req->header('business-id'), $business->id);
            }
        }
        if ($status = $req->query("status", false)) {
            $restaurant_vouchers->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $restaurant_vouchers = $restaurant_vouchers->get();
        } else {
            $restaurant_vouchers = $restaurant_vouchers->paginate($req->query('per_page', 20));
        }

        if ($restaurant_vouchers->isEmpty()) Utils::error('Voucher not found!', 404);

        return $restaurant_vouchers;
    }

    /**
     * Create restaurant_voucher.
     * POST /tiffintom/restaurant_vouchers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'voucher_code' => 'required|string',
            'device_type' => 'in:web,app,all',
            'type_offer' => 'in:single,multiple',
            'offer_mode' => 'in:price,percentage,free_delivery',
            'offer_value' => 'required|numeric|gt:0',
            'minimum_value' => 'required|numeric|gt:0',
            'multiple_value' => 'required|numeric|gt:0',
            'voucher_from' => 'required|date_format:d-m-Y',
            'voucher_to' => 'required|date_format:d-m-Y|after:voucher_from',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        // create restaurant_voucher
        $restaurant_voucher = new TiffintomRestaurantVoucher;
        $restaurant_voucher->fill($req->all());
        $restaurant_voucher->save();
        return $restaurant_voucher;
    }

    /**
     * List restaurant_vouchers
     * GET /tiffintom/restaurant_vouchers/id
     */
    public function show(Request $req, $id)
    {
        $restaurant_vouchers = TiffintomRestaurantVoucher::find($id);
        if (empty($restaurant_vouchers)) Utils::error("Voucher not found", 404);
        return $restaurant_vouchers;
    }

    /**
     * Update restaurant_voucher.
     * POST  /tiffintom/restaurant_vouchers/id
     */
    public function update(Request $req, $id)
    {
        $restaurant_voucher = TiffintomRestaurantVoucher::find($id);
        if (empty($restaurant_voucher)) Utils::error('Voucher not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $restaurant_voucher->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'voucher_code' => 'required|string',
            'device_type' => 'in:web,app,all',
            'type_offer' => 'in:single,multiple',
            'offer_mode' => 'in:price,percentage,free_delivery',
            'offer_value' => 'required|numeric|gt:0',
            'minimum_value' => 'required|numeric|gt:0',
            'multiple_value' => 'required|numeric|gt:0',
            'voucher_from' => 'required|date_format:d-m-Y',
            'voucher_to' => 'required|date_format:d-m-Y|after:voucher_from',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // update restaurant_voucher
        if ($req->has('restaurant_id')) $restaurant_voucher->restaurant_id = $req->input('restaurant_id');
        if ($req->has('day')) $restaurant_voucher->day = $req->input('day');
        if ($req->has('voucher_code')) $restaurant_voucher->voucher_code = $req->input('voucher_code');
        if ($req->has('device_type')) $restaurant_voucher->device_type = $req->input('device_type');
        if ($req->has('type_offer')) $restaurant_voucher->type_offer = $req->input('type_offer');
        if ($req->has('offer_mode')) $restaurant_voucher->offer_mode = $req->input('offer_mode');
        if ($req->has('offer_value')) $restaurant_voucher->offer_value = $req->input('offer_value');
        if ($req->has('minimum_value')) $restaurant_voucher->minimum_value = $req->input('minimum_value');
        if ($req->has('multiple_value')) $restaurant_voucher->multiple_value = $req->input('multiple_value');
        if ($req->has('voucher_from')) $restaurant_voucher->voucher_from = $req->input('voucher_from');
        if ($req->has('voucher_to')) $restaurant_voucher->voucher_to = $req->input('voucher_to');
        $restaurant_voucher->status = $req->input('status');
        $restaurant_voucher->delivery_type = $req->input('delivery_type');
        $restaurant_voucher->pickup_type = $req->input('pickup_type');
        $restaurant_voucher->dinein_type = $req->input('dinein_type');
        $restaurant_voucher->save();

        return $restaurant_voucher;
    }

    /**
     * Delete restaurant_voucher by id.
     * DELETE /tiffintom/restaurant_vouchers/{id}
     */
    public function delete(Request $req, $id)
    {
        $restaurant_voucher = TiffintomRestaurantVoucher::find($id);
        if (empty($restaurant_voucher)) Utils::error('Voucher not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $restaurant_voucher->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $restaurant_voucher->delete();
        return $restaurant_voucher;
    }

    /**
     * Enable restaurant_voucher.
     * POST /tiffintom/restaurant_vouchers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $restaurant_voucher = TiffintomRestaurantVoucher::find($id);
        if (empty($restaurant_voucher)) Utils::error('Voucher not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $restaurant_voucher->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $restaurant_voucher->status = '1';
        $restaurant_voucher->save();
        return $restaurant_voucher;
    }

    /**
     * Disable restaurant_voucher.
     * POST /tiffintom/restaurant_vouchers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $restaurant_voucher = TiffintomRestaurantVoucher::find($id);
        if (empty($restaurant_voucher)) Utils::error('Voucher not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $restaurant_voucher->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $restaurant_voucher->status = '0';
        $restaurant_voucher->save();
        return $restaurant_voucher;
    }

    /**
     * Bulk Upload restaurant_voucher.
     * POST /tiffintom/restaurant_vouchers/{id}/disable
     */
    public function bulk_restaurant_voucher(Request $req)
    {
        if ($req->has('restaurant_vouchers') && !is_array($req->input('restaurant_vouchers')))
            $req->merge(['restaurant_vouchers' => json_decode($req->input('restaurant_vouchers'))]);
        $this->validate($req, [
            'restaurant_vouchers' => 'required|array'
        ]);

        $restaurantVouchers = ($req->input('restaurant_vouchers'));

        foreach ($restaurantVouchers as $restaurantVoucher) {
            $restaurantVoucherItem = null;
            if (isset($restaurantVoucher->id)) {
                $restaurantVoucherItem = TiffintomRestaurantVoucher::find($restaurantVoucher->id);
            }
            if (empty($restaurantVoucherItem)) {
                $restaurantVoucherItem = new TiffintomRestaurantVoucher((array)$restaurantVoucher);
                $restaurantVoucherItem->save();
            } else {
                $restaurantVoucherItem->restaurant_id = $restaurantVoucher->restaurant_id;
                $restaurantVoucherItem->day = $restaurantVoucher->day;
                $restaurantVoucherItem->voucher_code = $restaurantVoucher->voucher_code;
                $restaurantVoucherItem->device_type = $restaurantVoucher->device_type;
                $restaurantVoucherItem->type_offer = $restaurantVoucher->type_offer;
                $restaurantVoucherItem->offer_mode = $restaurantVoucher->offer_mode;
                $restaurantVoucherItem->offer_value = $restaurantVoucher->offer_value;
                $restaurantVoucherItem->minimum_value = $restaurantVoucher->minimum_value;
                $restaurantVoucherItem->multiple_value = $restaurantVoucher->multiple_value;
                $restaurantVoucherItem->voucher_from = $restaurantVoucher->voucher_from;
                $restaurantVoucherItem->voucher_to = $restaurantVoucher->voucher_to;
                $restaurantVoucherItem->status = (int)$restaurantVoucher->status;
                $restaurantVoucherItem->delivery_type = $restaurantVoucher->delivery_type;
                $restaurantVoucherItem->pickup_type = $restaurantVoucher->pickup_type;
                $restaurantVoucherItem->dinein_type = $restaurantVoucher->dinein_type;
                $restaurantVoucherItem->save();
            }
        }

        return  $restaurantVouchers;
    }
}
