<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomUser;
use App\ModelsTiffintom\TiffintomReview;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomReviewController extends Controller
{

    /**
     * List review
     * GET /tiffintom/review
     */
    public function index(Request $req)
    {
        $review = TiffintomReview::selectRaw('reviews.*,users.first_name as first_name')->leftJoin('users', 'users.id', '=', 'customer_id')->orderBy('reviews.id', 'DESC');

        $customer = Auth::user();
        if ($customer) {
            // $review->where('customer_id', $customer->id);
        }

        if ($query = $req->query('restaurant_id', false)) {
            $review->where('restaurant_id', $query);
            // User wise check
            if ($req->hasHeader('business-id')) {
                $business = Business::where('restaurant_id', $query)->first();
                if (empty($business)) Utils::error('User not authenticated.', 404);
                (new ExtraController())->userCheck($req->header('business-id'), $business->id);
            }
        }

        if ($order_type = $req->query('order_type', false)) {
            $review->where('order_type', $order_type);
        }

        if ($status = $req->query('status', false)) {
            $review->where('reviews.status', $status);
        }

        if ($query = $req->query('rating', false)) {
            $review->where('rating', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $review = $review->get();
            // $review['finalReview'] = $this->convertToDecimal($reviewCount / $review['totalRating']);
        } else {
            $review = $review->paginate($req->query('per_page', 20));
        }

        if ($review->isEmpty()) Utils::error('No review found!', 404);

        return $review;
    }


    /**
     * List review-history
     * GET /tiffintom/reviews/history
     */
    public function history(Request $req)
    {
        $reviews = TiffintomReview::selectRaw("reviews.*,restaurants.restaurant_name as restaurant_name,users.first_name as customer_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->leftJoin('users', 'users.id', '=', 'customer_id')->orderBy('reviews.id', 'DESC');

        if ($query = $req->query("query", false)) {
            $reviews->whereRaw('(message LIKE ?)', ["%$query%"]);
        }

        if ($restaurant_id = $req->query("restaurant_id", false)) {
            $reviews->where('restaurant_id', $restaurant_id);
        }

        if ($customer_id = $req->query("customer_id", false)) {
            $reviews->where('customer_id', $customer_id);
        }

        if ($order_id = $req->query("order_id", false)) {
            $reviews->where('order_id', $order_id);
        }

        if ($status = $req->query("status", false)) {
            $reviews->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $reviews = $reviews->get();
        } else {
            $reviews = $reviews->paginate($req->query('per_page', 20));
        }

        if ($reviews->isEmpty()) Utils::error('Reviews not found!', 404);

        return $reviews;
    }

    /**
     * Only for partner
     * List review-home
     * GET /tiffintom/reviews/home
     */
    public function home(Request $req)
    {
        $startDate = Carbon::now()->startOfDay();
        $endDate = Carbon::now()->endOfDay();

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $lastReview = TiffintomReview::selectRaw('reviews.*,users.first_name as first_name')->leftJoin('users', 'users.id', '=', 'customer_id')->where('reviews.status', '1')->where('reviews.delete_status', 'N')->where('reviews.restaurant_id', $req->input('restaurant_id'))->orderBy('reviews.id', "DESC")->limit('4')->get();

        $newReviewCount = TiffintomReview::where('status', '1')->where('restaurant_id', $req->input('restaurant_id'))->where('created', '>', $startDate)->where('created', '<', $endDate)->where('delete_status', 'N')->orderBy('id', "DESC")->count();

        $reviewsHistory = TiffintomReview::where('status', '1')->where('restaurant_id', $req->input('restaurant_id'))->orderBy('id', "DESC")->get();

        if (!empty($reviewsHistory)) {
            $reviewCount = 0;
            $totalRating = count($reviewsHistory);
            if ($totalRating > 0) {
                foreach ($reviewsHistory as $rKey => $rValue) {
                    $reviewCount  = $rValue['rating'] + $reviewCount;
                }
                $finalReview = ($reviewCount / $totalRating);
            } else {
                $totalRating = 0;
                $finalReview = 0;
            }
        } else {
            $totalRating = 0;
            $finalReview = 0;
        }

        $response = [
            'last_reviews' => $lastReview,
            'new_review_count' => $newReviewCount,
            'total_rating' => $totalRating,
            'final_review' => $finalReview,
        ];

        return $response;
    }


    /**
     * List review-history
     * GET /tiffintom/reviews/count
     */
    public function count(Request $req)
    {
        $reviews = TiffintomReview::selectRaw('COUNT(*) as total_count,rating')->orderBy('rating', 'DESC')->groupBy("rating");

        if ($restaurant_id = $req->query("restaurant_id", false)) {
            $reviews->where('restaurant_id', $restaurant_id);
        }
        $reviews = $reviews->get();

        return $reviews;
    }

    /**
     * Create review.
     * POST /tiffintom/review
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'order_id' => 'required',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'rating' => 'integer',
            'message' => 'string',
            'responce' => 'string',
            'order_type' => 'string',
        ]);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('restaurant_id'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        // create review
        $review = new TiffintomReview;
        $review->fill($req->all());
        $review->save();

        $user = TiffintomUser::find($review->customer_id);

        $restaurant = TiffintomRestaurant::find($review->restaurant_id);

        if ($req->input('responce') == '') {
            $message      = 'You have a review from ' . $user->first_name;
            $notification['title']          = "Order Review";
            $notification['body']        = $message;

            $notificationdata['data']['title']          = "Order Review";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['order_id']  = $review->order_id;
            $notificationdata['data']['payload']        = array('ReviewDetails' => "", 'type' => "review");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            $Devices = explode(',', $restaurant->device_id);
            foreach ($Devices as $Device) {
                (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $Device, $notification, 'restaurant');
            }
        }

        //Call Socket
        (new TiffintomRestaurantController())->restaurant_pending_count($req, $review->restaurant_id);


        return $review;
    }

    /**
     * List review
     * GET /tiffintom/review/id
     */
    public function show(Request $req, $id)
    {
        $review = TiffintomReview::find($id);
        if (empty($review)) Utils::error("No review found", 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $review->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        return $review;
    }

    /**
     * Update review.
     * POST  /tiffintom/review/id
     */
    public function update(Request $req, $id)
    {
        $review = TiffintomReview::find($id);
        if (empty($review)) Utils::error('review not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $review->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'order_id' => 'required',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'rating' => 'integer',
            'message' => 'string',
            'responce' => 'string',
            'order_type' => 'string',
        ]);

        // update menu
        if ($req->has('customer_id')) $review->customer_id = $req->input('customer_id');
        if ($req->has('order_id')) $review->order_id = $req->input('order_id');
        if ($req->has('restaurant_id')) $review->restaurant_id = $req->input('restaurant_id');
        if ($req->has('rating')) $review->rating = $req->input('rating');
        if ($req->has('message')) $review->message = $req->input('message');
        if ($req->has('responce')) $review->responce = $req->input('responce');
        $review->order_type = $req->input('order_type');

        $review->save();
        return $review;
    }

    /**
     * Delete review by id.
     * DELETE /tiffintom/review/{id}
     */
    public function delete(Request $req, $id)
    {
        $review = TiffintomReview::find($id);
        if (empty($review)) Utils::error('review not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $review->restaurant_id)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $review->delete();

        return $review;
    }


    /**
     * Enable offer.
     * POST /tiffintom/offer-texts/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $review = TiffintomReview::find($id);
        if (empty($review)) Utils::error('Review not found!', 404);
        $review->status = '1';
        $review->save();
        return $review;
    }

    /**
     * Disable offer.
     * POST /tiffintom/offer-texts/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $review = TiffintomReview::find($id);
        if (empty($review)) Utils::error('Review not found!', 404);
        $review->status = '0';
        $review->save();
        return $review;
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
