<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCustomerReward;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomReward;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomRewardsController extends Controller
{

    /**
     * List rewards
     * GET /tiffintom/rewards
     */
    public function index(Request $req)
    {
        $rewards = TiffintomReward::orderBy('id', 'DESC')->first();
        if (empty($rewards)) Utils::error("No reward found", 404);
        return $rewards;
    }

    /**
     * List Points
     * GET /tiffintom/points
     */
    public function points(Request $req)
    {
        $orders = TiffintomOrder::where('status', 'Delivered')->orderBy('id', 'DESC');

        if ($query = $req->query("restaurant_id", false)) {
            $orders->where('restaurant_id', $query);
        }
        $orders = $orders->get();

        if ($orders->isEmpty()) Utils::error('orders not found!', 404);

        foreach ($orders as $order) {
            $reward = TiffintomReward::orderBy('id', 'ASC')->first();
            $restaurant = TiffintomRestaurant::find($order->restaurant_id);

            if (!empty($reward) && $restaurant->reward_option == 'Yes') {
                $getRewardFromTotal = $order->order_sub_total / $reward->reward_amount;
                $orderPoint = $getRewardFromTotal * $reward->reward_point;

                $customerReward = new TiffintomCustomerReward;
                $customerReward->order_id = $order->id;
                $customerReward->restaurant_name = $restaurant->restaurant_name;
                $customerReward->customer_id = $order->customer_id;
                $customerReward->total = $order->order_sub_total;
                $customerReward->points = $orderPoint;
                $customerReward->type = 'Earned';
                $customerReward->save();
            }
        }

        return $orders;
    }


    /**
     * List rewards
     * GET /tiffintom/rewards/id
     */
    public function show(Request $req, $id)
    {
        $rewards = TiffintomReward::orderBy('id', 'ASC')->first();
        if (empty($rewards)) Utils::error("No reward found", 404);
        // $rewards = $rewards->first();

        return $rewards;
    }

    /**
     * Update rewards.
     * POST  /tiffintom/rewards/id
     */
    public function update(Request $req, $id)
    {
        $rewards = TiffintomReward::find($id);
        if (empty($rewards)) Utils::error('rewards not found!', 404);

        $this->validate($req, [
            'order_id' => 'required|exists:tiffintom_connection.orders,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'rating' => 'integer',
            'message' => 'string',
            'responce' => 'string',
            'order_type' => 'string',
        ]);

        // update menu
        if ($req->has('customer_id')) $rewards->customer_id = $req->input('customer_id');
        if ($req->has('order_id')) $rewards->order_id = $req->input('order_id');
        if ($req->has('restaurant_id')) $rewards->restaurant_id = $req->input('restaurant_id');
        if ($req->has('rating')) $rewards->rating = $req->input('rating');
        if ($req->has('message')) $rewards->message = $req->input('message');
        if ($req->has('responce')) $rewards->responce = $req->input('responce');
        $rewards->order_type = $req->input('order_type');

        $rewards->save();
        return $rewards;
    }


    /**
     * Update rewards.
     * POST  /tiffintom/rewards/id
     */
    public function update_reward(Request $req, $id)
    {
        $rewards = TiffintomReward::find($id);
        if (empty($rewards)) Utils::error('rewards not found!', 404);

        $this->validate($req, [
            'reward_option' => 'string',
            'reward_amount' => 'string',
            'reward_point' => 'string',
            'reward_totalpoint' => 'string',
            'reward_percentage' => 'string',
            'reward_validity' => 'string',
            'value_type' => 'string',
            'miximum_value' => 'string',
            'minimum_order' => 'string',
            'redeem_order' => 'string',
        ]);

        // update menu
        if ($req->has('reward_option')) $rewards->reward_option = $req->input('reward_option');
        if ($req->has('reward_amount')) $rewards->reward_amount = $req->input('reward_amount');
        if ($req->has('reward_point')) $rewards->reward_point = $req->input('reward_point');
        if ($req->has('reward_totalpoint')) $rewards->reward_totalpoint = $req->input('reward_totalpoint');
        if ($req->has('reward_percentage')) $rewards->reward_percentage = $req->input('reward_percentage');
        if ($req->has('reward_validity')) $rewards->reward_validity = $req->input('reward_validity');
        if ($req->has('miximum_value')) $rewards->miximum_value = $req->input('miximum_value');
        if ($req->has('minimum_order')) $rewards->minimum_order = $req->input('minimum_order');
        if ($req->has('redeem_order')) $rewards->redeem_order = $req->input('redeem_order');
        $rewards->value_type = $req->input('value_type');

        $rewards->save();
        return $rewards;
    }
}
