<?php

namespace App\Http\Controllers\Tiffintom;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Helpers\Utils;
use App\ModelsTiffintom\TiffintomSMSEvent;

class TiffintomSMSEventController extends Controller
{
    public function index(Request $request)
    {
        $sms_event = TiffintomSMSEvent::latest('created');

        if ($request->query('query', false)) {
            $sms_event->where(function ($query) use ($request) {
                $query->where('package', 'like', '%'.$request->query('query').'%');
            })->orWhere(function ($query) use ($request) {
                $query->where('events', 'like', '%'.$request->query('query').'%');
            });
        }

        if ($request->query('nopaginate', false) == '1') {
            $sms_event = $sms_event->get();
        } else {
            $sms_event = $sms_event->paginate($request->query('per_page', 20));
        }

        if ($sms_event->isEmpty()) Utils::error('SMS event details not found!', 404);
        
        return $sms_event;
    }
    
    // Load all events
    public function load_events(Request $request)
    {
        $events = smsEvents();

        foreach ($events as $key => $event) {
            TiffintomSMSEvent::updateOrCreate(
                ['events' => $event], []
            );
        }
        TiffintomSMSEvent::whereNotIn('events', $events)->delete();
        $allEvents = TiffintomSMSEvent::get();
        return $allEvents;
    }

    // create new event
    public function create(Request $request)
    {
        $smsPackages = smsPackages();
        $smsEvents = smsEvents();
        $this->validate($request, [
            'package' => 'required|in:'. implode(',', $smsPackages),
            'events' => 'required|string|unique:tiffintom_connection.sms_events,events|in:'. implode(',', $smsEvents),
        ]);

        $sms_event = new TiffintomSMSEvent();
        $sms_event->fill($request->all());
        $sms_event->save();

        return $sms_event;
    }

    public function show(Request $request, $id)
    {
        $sms_event = TiffintomSMSEvent::find($id);
        if (empty($sms_event)) Utils::error('SMS event details not found!', 404);
        return $sms_event;
    }

    public function update(Request $request, $id)
    {
        $sms_event = TiffintomSMSEvent::find($id);
        if (empty($sms_event)) Utils::error('SMS event details not found!', 404);

        $smsPackages = smsPackages();
        $smsEvents = smsEvents();
        $this->validate($request, [
            'package' => 'required|in:'. implode(',', $smsPackages),
        ]);

        $sms_event->update($request->all());
    
        return $sms_event;
    }

    public function delete(Request $request, $id)
    {
        $sms_event = TiffintomSMSEvent::find($id);
        if (empty($sms_event)) Utils::error('SMS event details not found!', 404);
        $sms_event->delete();
        return $sms_event;
    }
    
}