<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSMSPort;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomSMSPortController extends Controller
{
    /**
     * List sms-ports
     * GET /tiffintom/sms-ports
     */
    public function index(Request $req)
    {
        $sms_ports = TiffintomSMSPort::orderBy('id', 'DESC');

        if ($query = $req->query("port_no", false)) {
            $sms_ports->where('port_no', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $sms_ports = $sms_ports->get();
        } else {
            $sms_ports = $sms_ports->paginate($req->query('per_page', 20));
        }

        if ($sms_ports->isEmpty()) Utils::error('sms ports not found!', 404);

        return $sms_ports;
    }


    /**
     * Create cart.
     * POST /tiffintom/sms-ports
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'port_no' => 'required|string',
        ]);

        $sms_port = new TiffintomSMSPort;
        $sms_port->fill($req->all());
        $sms_port->status = '1';
        $sms_port->save();

        return $sms_port;
    }

    /**
     * List sms-ports
     * GET /tiffintom/sms-ports/id
     */
    public function show(Request $req, $id)
    {
        $sms_port = TiffintomSMSPort::find($id);
        if (empty($sms_port)) Utils::error("sms port not found", 404);
        return $sms_port;
    }

    /**
     * Update sms-ports.
     * POST  /tiffintom/sms-ports/id
     */
    public function update(Request $req, $id)
    {
        $sms_port = TiffintomSMSPort::find($id);
        if (empty($sms_port)) Utils::error('sms port not found!', 404);

        $this->validate($req, [
            'port_no' => 'required|string',
        ]);

        // update sms port
        if ($req->has('port_no')) $sms_port->port_no = $req->input('port_no');
        $sms_port->save();

        return $sms_port;
    }

    /**
     * Delete sms-ports by id.
     * DELETE /tiffintom/sms-ports/{id}
     */
    public function delete(Request $req, $id)
    {
        $sms_port = TiffintomSMSPort::find($id);
        if (empty($sms_port)) Utils::error('sms port not found!', 404);
        $sms_port->delete();
        return $sms_port;
    }

    /**
     * Enable sms-ports.
     * POST /tiffintom/sms-ports/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $sms_port = TiffintomSMSPort::find($id);
        if (empty($sms_port)) Utils::error('sms port not found!', 404);
        $sms_port->status = '1';
        $sms_port->save();
        return $sms_port;
    }

    /**
     * Disable sms-ports.
     * POST /tiffintom/sms-ports/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $sms_port = TiffintomSMSPort::find($id);
        if (empty($sms_port)) Utils::error('sms port not found!', 404);
        $sms_port->status = '0';
        $sms_port->save();
        return $sms_port;
    }
}