<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\BusinessCardReader;
use App\CardBrand;
use App\CardBrandCommission;
use App\Country;
use App\CountryCommission;
use App\Helpers\Auth;
use App\Helpers\OtpHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposOrder;
use App\ModelsEpos\EposTable;
use App\ModelsTiffintom\TiffintomShortlnUrl;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class TiffintomShortlnUrlController extends Controller
{

    /**
     * List shortln_url
     * GET /tiffintom/payment-message
     */
    public function index(Request $req)
    {
        $shortln_url = TiffintomShortlnUrl::orderBy('id', 'DESC');

        if ($business_id = $req->query("business_id", false)) {
            $business = Business::where('id', $business_id)->first();
            // User wise check
            if ($req->hasHeader('business-id')) {
                (new ExtraController())->userCheck($req->header('business-id'), $business->id);
            }
            $shortln_url->whereRaw('(name LIKE ? )', ["%$business->name%"]);
        }

        if ($query = $req->query("query", false)) {
            $shortln_url->whereRaw('(name LIKE ? OR email LIKE ? OR phone LIKE ? OR description LIKE ? OR descriptor LIKE ? OR amount LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($query = $req->query('status')) {
            $shortln_url->where('status', $query);
        }

        if ($expired = $req->query('expired')) {
            $shortln_url->where('expired', $expired);
        } else {
            $shortln_url->where('expired', '0');
        }

        if ($qr = $req->query('qr')) {
            $shortln_url->where('qr', $qr);
        } else {
            $shortln_url->where('qr', '0');
        }

        if ($query = $req->query('from_date')) {
            $fromDate = Carbon::parse($query)->startOfDay();
            $shortln_url->where('created', '>=', $fromDate);
        }
        if ($query = $req->query('to_date')) {
            $toDate = Carbon::parse($query)->endOfDay();
            $shortln_url->where('created', '<=', $toDate);
        }

        if ($req->query('nopaginate', false) == '1') {
            $shortln_url = $shortln_url->get();
        } else {
            $shortln_url = $shortln_url->paginate($req->query('per_page', 20));
        }

        if ($shortln_url->isEmpty()) Utils::error('payment messages not found!', 404);

        return $shortln_url;
    }

    /**
     * List shortln_url
     * GET /tiffintom/payment-message
     */
    public function find(Request $req, $code)
    {
        $shortln_url = TiffintomShortlnUrl::whereRaw('(shortn LIKE ? )', ["%$code%"])->first();
        if (empty($shortln_url)) Utils::error('payment link not found!', 404);

        $business = Business::whereRaw('(name LIKE ? )', ["%$shortln_url->name%"])->first();
        if (empty($business)) Utils::error('Business not found!', 404);
        $shortln_url->business_id = $business->id;
        $shortln_url->image_url = $business->image_url;

        return $shortln_url;
    }

    /**
     * Create shortln_url.
     * POST /tiffintom/payment-message
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'order_id' => 'string',
            'email' => 'string',
            'phone' => 'string',
            'publishkey' => 'required|string',
            'secretkey' => 'required|string',
            'description' => 'required|string',
            'descriptor' => 'required|string',
            'amount' => 'required|numeric|gt:0',
            'device_id' => 'string',
            'qr' => 'boolean',
        ]);

        // create shortln_url
        $shortln_url = new TiffintomShortlnUrl;
        $shortln_url->fill($req->all());

        if ($req->input('name') != '') {
            $business = Business::whereRaw('(name LIKE ? )', ["%$req->input('name')%"])->first();
            if (!empty($business)) {
                if ($business->connect_service == 1) {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $columns = 'connect_stripe_public_key_test';
                        $columns1 = 'connect_stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $columns = 'connect_stripe_public_key_live';
                        $columns1 = 'connect_stripe_private_key_live';
                    }

                    $publishkey = SiteSettingController::findByKey($columns)->value;
                    $secretkey = SiteSettingController::findByKey($columns1)->value;

                    $shortln_url->publishkey = $publishkey;
                    $shortln_url->secretkey = $secretkey;
                } else {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $columns = 'stripe_public_key_test';
                        $columns1 = 'stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $columns = 'stripe_public_key_live';
                        $columns1 = 'stripe_private_key_live';
                    }

                    $publishkey = SiteSettingController::findByKey($columns)->value;
                    $secretkey = SiteSettingController::findByKey($columns1)->value;

                    $shortln_url->publishkey = $publishkey;
                    $shortln_url->secretkey = $secretkey;
                }
            }
        }


        // Rendom Shuffle Code Generate
        $arr = str_split('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789');
        shuffle($arr);
        $arr = array_slice($arr, 0, 9);
        $random = implode('', $arr);
        $shortln_url->shortn = $random;

        $shortln_url->save();

        $send_url = env('APP_URL') . 'epos-payment?code=' . $random . '&'; // http://192.168.1.23:8300/epos-payment?code=dZCMEwjIO&

        $message = 'This is payment request sent from ' . $req->input('name') . ', please click link below to complete your payment.' . $send_url;
        (new OtpHelper())->sendMessage($message, $req->input('phone'), 'payment_sms_link');

        return [
            'message' => 'message send successfully!!',
            'payment_link' => $send_url,
            'payment_message' => $shortln_url,
        ];
    }

    /**
     * List shortln_url
     * GET /tiffintom/payment-message
     */
    public function change_status(Request $req, $id)
    {
        $shortln_url = TiffintomShortlnUrl::find($id);
        if (empty($shortln_url)) Utils::error('payment link not found!', 404);

        $this->validate($req, [
            'status' => 'required|in:Pending,Success',
            'payment_id' => 'required|string',
        ]);

        // update shortln url
        if ($req->has('status')) $shortln_url->status = $req->input('status');
        if ($req->has('payment_id')) $shortln_url->payment_id = $req->input('payment_id');
        $shortln_url->save();

        if ($req->input('status') == 'Success') {
            $message  = 'Payment has been accepted successfully.';

            $notification['title']          = "Payment";
            $notification['body']        = $message;

            $notificationdata['data']['title']          = "Payment";
            $notificationdata['data']['message']        = $message;
            $notificationdata['data']['is_background']  = false;
            $notificationdata['data']['id']  = $shortln_url->id;
            $notificationdata['data']['status']  = $shortln_url->status;
            $notificationdata['data']['amount']  = $shortln_url->amount;
            $notificationdata['data']['phone']  = $shortln_url->phone;
            $notificationdata['data']['email']  = $shortln_url->email;
            $notificationdata['data']['order_id']  = $shortln_url->order_id;
            $notificationdata['data']['transaction_id']  = $shortln_url->payment_id;
            $notificationdata['data']['payload']        = array('PaymentDetails' => "", 'type' => "payment");
            $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');
            (new PushNotificationTiffintomHelper())->sendNotification($notificationdata, $shortln_url->device_id, $notification, 'restaurant');

            $business = Business::whereRaw('(name LIKE ? )', ["%$shortln_url->name%"])->first();
            if (empty($business)) Utils::error('Business not found!', 404);
            $business_id = $business->id;
            if (!empty($business) && $shortln_url->order_id != '') {
                if ($business->epos_service) {
                    $this->establishEposConnection($business);
                    $order = EposOrder::where('unique_id', $shortln_url->order_id)->first();
                    if (empty($order)) {
                        $order = EposOrder::find($shortln_url->order_id);
                    }
                    if (empty($order)) Utils::error('EposOrder not found!', 404);
                    $order->order_status_id = 5; // Order Complete Status
                    $order->save();

                    $table = EposTable::find($order->table_id);
                    if (empty($table)) Utils::error('EposTable not found!', 404);
                    $table->table_status_id = 1; // Table Free Status
                    $table->save();
                }
            }
        }

        return $shortln_url;
    }
    /**
     * List shortln_url
     * GET /tiffintom/payment-message/id/cancel-payment
     */
    public function change_cancel(Request $req, $id)
    {
        $shortln_url = TiffintomShortlnUrl::find($id);
        if (empty($shortln_url)) Utils::error('payment link not found!', 404);

        // cancel shortln url
        $shortln_url->cancel = '1';
        $shortln_url->save();

        return $shortln_url;
    }

    /**
     * Update shortln_url.
     * POST  /tiffintom/payment-message/id
     */
    public function resend(Request $req, $id)
    {
        $shortln_url = TiffintomShortlnUrl::find($id);
        if (empty($shortln_url)) Utils::error('payment link not found!', 404);

        //Amount Update
        if ($req->has('amount')) $shortln_url->amount = $req->input('amount');
        $shortln_url->save();

        $send_url = env('APP_URL') . 'epos-payment?code=' . $shortln_url->shortn . '&';

        // $message = 'Enter below link ' . $send_url;
        $message = 'This is payment request sent from ' . $shortln_url->name . ', please click link below to complete your payment.' . $send_url;
        (new OtpHelper())->sendMessage($message, $shortln_url->phone, 'payment_sms_link');

        return [
            'message' => 'message send successfully!!',
            'payment_link' => $send_url,
            'payment_message' => $shortln_url,
        ];
    }

    /**
     * Delete All payment-message.
     * GET All/tiffintom/payment-message/delete-all/
     */
    public function delete_previous(Request $req)
    {
        $to_date = Carbon::today()->startOfDay();
        // TiffintomShortlnUrl::where('created', '<=', $to_date)->where('status', 'Pending')->delete();
        TiffintomShortlnUrl::where('created', '<=', $to_date)->where('status', 'Pending')->update(['expired' => '1']);
        return ['message' => 'links rejected done!'];
    }

    /**
     * Create Payment Intent.
     * POST  /tiffintom/payment-message/id
     */
    public function payment_intent(Request $req)
    {

        $shortln_url = TiffintomShortlnUrl::find($req->input('id'));
        if ($shortln_url->cancel == '1') Utils::error('payment link cancel,please try another link!', 404);
        if ($req->has('customer_email')) $shortln_url->email = $req->input('customer_email');
        $shortln_url->save();

        $key =  $req->input('secretkey');
        $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        $this->validate($req, [
            'amount' => 'required|numeric|gt:0',  // multiplied by 100
            's_terminal_id' => 'string',
            'transaction_type' => 'required|in:online,merchant,connect',
            'business_id' => 'required_without_all:statement_descriptor,restaurant_id,s_terminal_id',

            'order_type' => 'required_if:transaction_type,online|in:dinein,pickup,delivery',

            'restaurant_id' => 'required_without_all:statement_descriptor,business_id,s_terminal_id',
            'statement_descriptor' => 'required_without_all:restaurant_id,business_id,s_terminal_id',

            'customer_id' => 'string',
            'customer_name' => 'string',
            'customer_email' => 'string|email|required_with:customer_name',

            'auto_confirm' => 'boolean'
        ]);

        $transaction_type = $req->input('transaction_type');
        $order_type = $req->input('order_type');
        $statement_descriptor = $req->input('descriptor');
        $statement_description = $req->input('description');
        $business_id = $req->input('business_id');
        $business = null;
        $autoConfirm = $req->input('auto_confirm', false);

        if ($req->has('business_id') && $business_id != '') {
            $business = Business::find($business_id);
        } else {
            $business = Business::where('name', $shortln_url->name)->first();
        }

        $stripe_account_id = null;
        $transfer_connect = null;
        if (!empty($business)) {
            $stripe_account_id = $business->s_account_id;
            $transfer_connect = $business->transfer_connect;
        }

        $fee = 0;
        $country_stripe_fixed = 0;
        $country_stripe_percentage = 0;
        $card_stripe_fixed = 0;
        $card_stripe_percentage = 0;

        $humanAmount = $req->input('amount') / 100;
        if ($transaction_type != null && $transaction_type != 'online' && $req->has('payment_method_id')) {
            $stripe_payment_method = $stripe->paymentMethods->retrieve($req->input('payment_method_id'));
            $card_brand = $stripe_payment_method->card->brand;
            $card_country = $stripe_payment_method->card->country;
            $country = Country::where('iso', $card_country)->first();
            if (!empty($country)) {
                $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
            }
            if (empty($countryCommission)) {
                $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($countryCommission)) {
                $country_stripe_percentage = $this->calculateFee($humanAmount, $countryCommission->stripe_percentage, 'percentage');
                $country_stripe_fixed = $countryCommission->stripe_fixed;
            }
            $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$card_brand, $card_brand])->first();
            if (!empty($cardBrand)) {
                $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
            }
            if (empty($cardBrandCommission)) {
                $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
            }
            if (!empty($cardBrandCommission)) {
                $card_stripe_percentage = $this->calculateFee($humanAmount, $cardBrandCommission->stripe_percentage, 'percentage');
                $card_stripe_fixed = $cardBrandCommission->stripe_fixed;
            }
        }

        $fee = $this->calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type);
        $fee = $fee + $card_stripe_fixed + $card_stripe_percentage + $country_stripe_fixed + $country_stripe_percentage;

        if ($req->has('currency')) {
            $currency = $req->input('currency');
        } else {
            $currency = 'GBP';
        }

        $intentParams = [
            'amount' => (int) ($humanAmount * 100),
            'currency' => $currency,
        ];
        if ($autoConfirm) {
            $intentParams['confirm'] = true;
        } else {
            $intentParams['confirmation_method'] = 'automatic';
        }
        if ($req->has('payment_method_id')) {
            $intentParams['payment_method'] = $req->input('payment_method_id');
        }
        if ($statement_descriptor != null) {
            $intentParams['statement_descriptor'] = $statement_descriptor;
            $intentParams['description'] = $statement_description;
        }
        if ($req->input('customer_id') != null) {
            $intentParams['customer'] = $req->input('customer_id');
            $intentParams['save_payment_method'] = true;
        }
        if ($req->input('customer_email') != null) {
            $customerParams['email'] = $req->input('customer_email');
            if ($req->input('customer_name') != null) {
                $customerParams['name'] = $req->input('customer_name');
            }
            $customer = $stripe->customers->create($customerParams);
            $intentParams['customer'] = $customer->id;
            $intentParams['save_payment_method'] = true;
        }
        if ($stripe_account_id != null && $transfer_connect == 'yes') {
            if ($business->business_commissions->auto_fee == 'yes') {
                $intentParams['application_fee_amount'] = $fee * 100;
            }
            $intentParams['transfer_data'] = [
                'destination' => $stripe_account_id
            ];
        }

        $intentParams['metadata'] = [
            'is_payby_link' => 1
        ];

        if ($shortln_url->order_id != '') {
            $intentParams['metadata']['order_id'] = $shortln_url->order_id;
        }

        $intent = $stripe->paymentIntents->create($intentParams);
        return ['payment_intent_id' => $intent->client_secret, 'payment_intent' => $intent, "status" => $intent->status];
    }

    public function calculateApplicationFee($humanAmount, $business, $transaction_type, $order_type)
    {
        $payment_method_fees = 0;
        $orderTypeWiseCommission = 0;
        $orderTypeWiseServiceCharge = 0;
        $orderBasedOrTransactionWiseCharge = 0;
        $cardAuthFee = 0;
        $cardProcessingFee = 0;
        $fee = 0;
        if ($transaction_type == 'online') {
            $payment_method_fees  = $this->calculateFee($humanAmount, $business->business_commissions->online_card_fee, $business->business_commissions->online_card_fee_type);
            if ($order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->dinein_commission_fee, $business->business_commissions->dinein_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->dinein_service_fee, $business->business_commissions->dinein_service_fee_type);
            } else {
                $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->online_commission_fee, $business->business_commissions->online_commission_fee_type);
                $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->online_service_fee, $business->business_commissions->online_service_fee_type);
            }
            if ($business->business_commissions->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($humanAmount, $business->business_commissions->per_order_charge, $business->business_commissions->per_order_charge_type);
            } else {
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($humanAmount, $business->business_commissions->card_charge, $business->business_commissions->card_charge_type);
            }
            $fee = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $cardAuthFee + $cardProcessingFee);
        }
        if ($transaction_type == 'merchant') {
            $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_auth_fee, $business->business_commissions->merchant_card_auth_fee_type);
            $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->merchant_card_processing_fee, $business->business_commissions->merchant_card_processing_fee_type);

            $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
        }
        if ($transaction_type == 'connect') {
            $cardAuthFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_auth_fee, $business->business_commissions->connect_card_auth_fee_type);
            $cardProcessingFee  = $this->calculateFee($humanAmount, $business->business_commissions->connect_card_processing_fee, $business->business_commissions->connect_card_processing_fee_type);

            $fee = $this->convertToDecimal($cardAuthFee + $cardProcessingFee);
        }
        return $fee;
    }

    public function calculateFee($humanAmount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($humanAmount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }
}
