<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomSiteSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomSiteSettingController extends Controller
{
    /**
     * List site-settings
     * GET /tiffintom/site-settings
     */
    public function index(Request $req)
    {
        $site_settings = TiffintomSiteSetting::select('id', 'site_name', 'site_logo', 'site_fav', 'site_currency', 'assign_status', 'assign_miles', 'facebook_login', 'google_login', 'twitter_login', 'apple_login', 'banner_status', 'charity_message', 'charity_amount', 'android_version', 'ios_version', 'dine_in', 'show_offers', 'signup_verify', 'checkout_verify', 'booking_verify', 'order_verify', 'signup_verify_type', 'order_verify_type', 'booking_verify_type', 'wallet_available')->where('id', '1')->first();

        if (empty($site_settings)) Utils::error('site setting not found!', 404);

        return $site_settings;
    }

    /**
     * List site-settings
     * GET /tiffintom/site-settings/id
     */
    public function show(Request $req, $id)
    {
        $site_settings = TiffintomSiteSetting::select('id', 'site_name', 'site_logo', 'site_fav', 'site_currency', 'assign_status', 'assign_miles', 'facebook_login', 'google_login', 'twitter_login', 'apple_login', 'banner_status', 'charity_message', 'charity_amount', 'android_version', 'ios_version', 'dine_in', 'show_offers', 'signup_verify', 'checkout_verify', 'booking_verify', 'order_verify', 'signup_verify_type', 'order_verify_type', 'booking_verify_type', 'wallet_available')->find($id);
        if (empty($site_settings)) Utils::error("site setting not found", 404);
        return $site_settings;
    }


    /**
     * List All site-settings
     * GET /tiffintom/site-settings/id/all
     */
    public function show_all(Request $req, $id)
    {
        $site_settings = TiffintomSiteSetting::find('1');
        if (empty($site_settings)) Utils::error("site setting not found", 404);
        return $site_settings;
    }

    /**
     * Update site-settings.
     * POST  /tiffintom/site-settings/id
     */
    public function update(Request $req, $id)
    {
        $site_settings = TiffintomSiteSetting::find($id);
        if (empty($site_settings)) Utils::error('site setting not found!', 404);

        $this->validate($req, [
            'user_id' => 'string',
            'site_name' => 'string',
            'image_logo' => 'image',
            'image_fav' => 'image',
            'address_mode' => 'in:Google,Normal',
            'site_mile' => 'numeric',
            'search_by' => 'in:zip,area',
            'admin_name' => 'string',
            'admin_email' => 'email',
            'contact_us_email' => 'email',
            'order_email' => 'email',
            'invoice_email' => 'email',
            'contact_phone' => 'string',
            'admin_phone' => 'string',
            'site_address' => 'string',
            'site_country' => 'string',
            'site_state' => 'string',
            'site_city' => 'string',
            'site_zip' => 'string',
            'site_currency' => 'string',
            'site_timezone' => 'string',
            'site_currency_code' => 'string',
            'stripe_mode' => 'in:Live,Test',
            'stripe_publishkey' => 'string',
            'stripe_secretkey' => 'string',
            'stripe_secretkeyTest' => 'string',
            'stripe_publishkeyTest' => 'string',
            'stripe_description' => 'string',
            'stripe_descriptor' => 'string',
            'paypal_mode' => 'in:Live,Test',
            'test_clientid' => 'string',
            'live_clientid' => 'string',
            'test_secretkey' => 'string',
            'live_secretkey' => 'string',
            'getaddress_key' => 'string',
            'google_analytics' => 'string',
            'woopra_analytics' => 'string',
            'zoopim_code' => 'string',
            'mail_option' => 'string',
            'smtp_host' => 'string',
            'smtp_port' => 'string',
            'smtp_username' => 'string',
            'smtp_password' => 'string',
            'vat_no' => 'string',
            'vat_percent' => 'numeric',
            'card_fee' => 'numeric',
            'service_charge' => 'numeric',
            'invoice_duration' => 'string',
            'offline_status' => 'string',
            'offline_notes' => 'string',
            'sms_option' => 'string',
            'sms_token' => 'string',
            'sms_id' => 'string',
            'sms_source_number' => 'string',
            'sms_username' => 'string',
            'sms_password' => 'string',
            'meta_title' => 'string',
            'meta_keywords' => 'string',
            'meta_description' => 'string',
            'multiple_language' => 'string',
            'default_language' => 'string',
            'other_language' => 'string',
            'facebook_api_id' => 'string',
            'facebook_secret_key' => 'string',
            'google_api_id' => 'string',
            'google_secret_key' => 'string',
            'twitter_key' => 'string',
            'twitter_secret_key' => 'string',
            'google_key1' => 'string',
            'google_key2' => 'string',
            'google_key3' => 'string',
            'google_key4' => 'string',
            'google_key5' => 'string',
            'mailchimp_key' => 'string',
            'mailchimp_list_id' => 'string',
            'pusher_key' => 'string',
            'pusher_secret' => 'string',
            'pusher_id' => 'string',
            'fcm_key' => 'string',
            'fcm_key1' => 'string',
            'assign_status' => 'string',
            'assign_miles' => 'string',
            'facebook_login' => 'string',
            'google_login' => 'string',
            'twitter_login' => 'string',
            'apple_login' => 'string',
            'banner_status' => 'string',
            'charity_message' => 'string',
            'charity_amount' => 'numeric',
            'android_version' => 'string',
            'ios_version' => 'string',
            'admin_complaint_all' => 'string',
            'admin_chat_all' => 'string',
            'admin_order_all' => 'string',
            'admin_pending_order' => 'string',
            'restaurant_order_all' => 'string',
            'restaurant_pending_order' => 'string',
            'complaint_all' => 'string',
            'chat_all' => 'string',
            'pending_order_time' => 'string',
            'created' => 'string',
            'updated' => 'string',
            'dine_in' => 'string',
            'signup_verify' => 'string',
            'checkout_verify' => 'string',
            'sms_gateway' => 'string',
            'click_send_username' => 'string',
            'click_send_api_key' => 'string',
            'click_send_sender_id' => 'string',
            'text_magic_username' => 'string',
            'text_magic_api_key' => 'string',
            'route_mobile_username' => 'string',
            'route_mobile_password' => 'string',
            'route_mobile_sender' => 'string',
            'show_offers' => 'boolean',
        ]);

        // update card
        if ($req->has('user_id')) $site_settings->user_id = $req->input('user_id');
        if ($req->has('site_name')) $site_settings->site_name = $req->input('site_name');
        if ($req->has('image_logo')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image_logo'), ImageHelper::$TYPE_TIFFINTOM_SITE_LOGO, 'tiffintom');
            $site_settings->site_logo = $fileName;
        }
        if ($req->has('image_fav')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image_fav'), ImageHelper::$TYPE_TIFFINTOM_SITE_FAVICON, 'tiffintom');
            $site_settings->site_fav = $fileName;
        }
        if ($req->has('address_mode')) $site_settings->address_mode = $req->input('address_mode');
        if ($req->has('site_mile')) $site_settings->site_mile = $req->input('site_mile');
        if ($req->has('search_by')) $site_settings->search_by = $req->input('search_by');
        if ($req->has('admin_name')) $site_settings->admin_name = $req->input('admin_name');
        if ($req->has('admin_email')) $site_settings->admin_email = $req->input('admin_email');
        if ($req->has('contact_us_email')) $site_settings->contact_us_email = $req->input('contact_us_email');
        if ($req->has('order_email')) $site_settings->order_email = $req->input('order_email');
        if ($req->has('invoice_email')) $site_settings->invoice_email = $req->input('invoice_email');
        if ($req->has('contact_phone')) $site_settings->contact_phone = $req->input('contact_phone');
        if ($req->has('admin_phone')) $site_settings->admin_phone = $req->input('admin_phone');
        if ($req->has('site_address')) $site_settings->site_address = $req->input('site_address');
        if ($req->has('site_country')) $site_settings->site_country = $req->input('site_country');
        if ($req->has('site_state')) $site_settings->site_state = $req->input('site_state');
        if ($req->has('site_city')) $site_settings->site_city = $req->input('site_city');
        if ($req->has('site_zip')) $site_settings->site_zip = $req->input('site_zip');
        if ($req->has('site_currency')) $site_settings->site_currency = $req->input('site_currency');
        if ($req->has('site_timezone')) $site_settings->site_timezone = $req->input('site_timezone');
        if ($req->has('site_currency_code')) $site_settings->site_currency_code = $req->input('site_currency_code');
        if ($req->has('stripe_mode')) $site_settings->stripe_mode = $req->input('stripe_mode');
        if ($req->has('stripe_publishkey')) $site_settings->stripe_publishkey = $req->input('stripe_publishkey');
        if ($req->has('stripe_secretkey')) $site_settings->stripe_secretkey = $req->input('stripe_secretkey');
        if ($req->has('stripe_secretkeyTest')) $site_settings->stripe_secretkeyTest = $req->input('stripe_secretkeyTest');
        if ($req->has('stripe_publishkeyTest')) $site_settings->stripe_publishkeyTest = $req->input('stripe_publishkeyTest');
        if ($req->has('stripe_description')) $site_settings->stripe_description = $req->input('stripe_description');
        if ($req->has('stripe_descriptor')) $site_settings->stripe_descriptor = $req->input('stripe_descriptor');
        if ($req->has('paypal_mode')) $site_settings->paypal_mode = $req->input('paypal_mode');
        if ($req->has('test_clientid')) $site_settings->test_clientid = $req->input('test_clientid');
        if ($req->has('live_clientid')) $site_settings->live_clientid = $req->input('live_clientid');
        if ($req->has('test_secretkey')) $site_settings->test_secretkey = $req->input('test_secretkey');
        if ($req->has('live_secretkey')) $site_settings->live_secretkey = $req->input('live_secretkey');
        if ($req->has('getaddress_key')) $site_settings->getaddress_key = $req->input('getaddress_key');
        if ($req->has('google_analytics')) $site_settings->google_analytics = $req->input('google_analytics');
        if ($req->has('woopra_analytics')) $site_settings->woopra_analytics = $req->input('woopra_analytics');
        if ($req->has('zoopim_code')) $site_settings->zoopim_code = $req->input('zoopim_code');
        if ($req->has('mail_option')) $site_settings->mail_option = $req->input('mail_option');
        if ($req->has('smtp_host')) $site_settings->smtp_host = $req->input('smtp_host');
        if ($req->has('smtp_port')) $site_settings->smtp_port = $req->input('smtp_port');
        if ($req->has('smtp_username')) $site_settings->smtp_username = $req->input('smtp_username');
        if ($req->has('smtp_password')) $site_settings->smtp_password = $req->input('smtp_password');
        if ($req->has('vat_no')) $site_settings->vat_no = $req->input('vat_no');
        if ($req->has('vat_percent')) $site_settings->vat_percent = $req->input('vat_percent');
        if ($req->has('card_fee')) $site_settings->card_fee = $req->input('card_fee');
        if ($req->has('service_charge')) $site_settings->service_charge = $req->input('service_charge');
        if ($req->has('invoice_duration')) $site_settings->invoice_duration = $req->input('invoice_duration');
        if ($req->has('offline_status')) $site_settings->offline_status = $req->input('offline_status');
        if ($req->has('offline_notes')) $site_settings->offline_notes = $req->input('offline_notes');
        if ($req->has('sms_option')) $site_settings->sms_option = $req->input('sms_option');
        if ($req->has('sms_token')) $site_settings->sms_token = $req->input('sms_token');
        if ($req->has('sms_id')) $site_settings->sms_id = $req->input('sms_id');
        if ($req->has('sms_source_number')) $site_settings->sms_source_number = $req->input('sms_source_number');
        if ($req->has('sms_username')) $site_settings->sms_username = $req->input('sms_username');
        if ($req->has('sms_password')) $site_settings->sms_password = $req->input('sms_password');
        if ($req->has('meta_title')) $site_settings->meta_title = $req->input('meta_title');
        if ($req->has('meta_keywords')) $site_settings->meta_keywords = $req->input('meta_keywords');
        if ($req->has('meta_description')) $site_settings->meta_description = $req->input('meta_description');
        if ($req->has('multiple_language')) $site_settings->multiple_language = $req->input('multiple_language');
        if ($req->has('default_language')) $site_settings->default_language = $req->input('default_language');
        if ($req->has('other_language')) $site_settings->other_language = $req->input('other_language');
        if ($req->has('facebook_api_id')) $site_settings->facebook_api_id = $req->input('facebook_api_id');
        if ($req->has('facebook_secret_key')) $site_settings->facebook_secret_key = $req->input('facebook_secret_key');
        if ($req->has('google_api_id')) $site_settings->google_api_id = $req->input('google_api_id');
        if ($req->has('google_secret_key')) $site_settings->google_secret_key = $req->input('google_secret_key');
        if ($req->has('twitter_key')) $site_settings->twitter_key = $req->input('twitter_key');
        if ($req->has('twitter_secret_key')) $site_settings->twitter_secret_key = $req->input('twitter_secret_key');
        if ($req->has('google_key1')) $site_settings->google_key1 = $req->input('google_key1');
        if ($req->has('google_key2')) $site_settings->google_key2 = $req->input('google_key2');
        if ($req->has('google_key3')) $site_settings->google_key3 = $req->input('google_key3');
        if ($req->has('google_key4')) $site_settings->google_key4 = $req->input('google_key4');
        if ($req->has('google_key5')) $site_settings->google_key5 = $req->input('google_key5');
        if ($req->has('mailchimp_key')) $site_settings->mailchimp_key = $req->input('mailchimp_key');
        if ($req->has('mailchimp_list_id')) $site_settings->mailchimp_list_id = $req->input('mailchimp_list_id');
        if ($req->has('pusher_key')) $site_settings->pusher_key = $req->input('pusher_key');
        if ($req->has('pusher_secret')) $site_settings->pusher_secret = $req->input('pusher_secret');
        if ($req->has('pusher_id')) $site_settings->pusher_id = $req->input('pusher_id');
        if ($req->has('fcm_key')) $site_settings->fcm_key = $req->input('fcm_key');
        if ($req->has('fcm_key1')) $site_settings->fcm_key1 = $req->input('fcm_key1');
        if ($req->has('assign_status')) $site_settings->assign_status = $req->input('assign_status');
        if ($req->has('assign_miles')) $site_settings->assign_miles = $req->input('assign_miles');
        if ($req->has('facebook_login')) $site_settings->facebook_login = $req->input('facebook_login');
        if ($req->has('google_login')) $site_settings->google_login = $req->input('google_login');
        if ($req->has('twitter_login')) $site_settings->twitter_login = $req->input('twitter_login');
        if ($req->has('apple_login')) $site_settings->apple_login = $req->input('apple_login');
        if ($req->has('banner_status')) $site_settings->banner_status = $req->input('banner_status');
        if ($req->has('charity_message')) $site_settings->charity_message = $req->input('charity_message');
        if ($req->has('charity_amount')) $site_settings->charity_amount = $req->input('charity_amount');
        if ($req->has('android_version')) $site_settings->android_version = $req->input('android_version');
        if ($req->has('ios_version')) $site_settings->ios_version = $req->input('ios_version');
        if ($req->has('admin_complaint_all')) $site_settings->admin_complaint_all = $req->input('admin_complaint_all');
        if ($req->has('admin_chat_all')) $site_settings->admin_chat_all = $req->input('admin_chat_all');
        if ($req->has('admin_order_all')) $site_settings->admin_order_all = $req->input('admin_order_all');
        if ($req->has('admin_pending_order')) $site_settings->admin_pending_order = $req->input('admin_pending_order');
        if ($req->has('restaurant_order_all')) $site_settings->restaurant_order_all = $req->input('restaurant_order_all');
        if ($req->has('restaurant_pending_order')) $site_settings->restaurant_pending_order = $req->input('restaurant_pending_order');
        if ($req->has('complaint_all')) $site_settings->complaint_all = $req->input('complaint_all');
        if ($req->has('chat_all')) $site_settings->chat_all = $req->input('chat_all');
        if ($req->has('pending_order_time')) $site_settings->pending_order_time = $req->input('pending_order_time');
        if ($req->has('dine_in')) $site_settings->dine_in = $req->input('dine_in');
        if ($req->has('signup_verify')) $site_settings->signup_verify = $req->input('signup_verify');
        if ($req->has('checkout_verify')) $site_settings->checkout_verify = $req->input('checkout_verify');
        if ($req->has('sms_gateway')) $site_settings->sms_gateway = $req->input('sms_gateway');
        if ($req->has('click_send_username')) $site_settings->click_send_username = $req->input('click_send_username');
        if ($req->has('click_send_api_key')) $site_settings->click_send_api_key = $req->input('click_send_api_key');
        if ($req->has('click_send_sender_id')) $site_settings->click_send_sender_id = $req->input('click_send_sender_id');
        if ($req->has('text_magic_username')) $site_settings->text_magic_username = $req->input('text_magic_username');
        if ($req->has('text_magic_api_key')) $site_settings->text_magic_api_key = $req->input('text_magic_api_key');
        if ($req->has('route_mobile_username')) $site_settings->route_mobile_username = $req->input('route_mobile_username');
        if ($req->has('route_mobile_password')) $site_settings->route_mobile_password = $req->input('route_mobile_password');
        if ($req->has('route_mobile_sender')) $site_settings->route_mobile_sender = $req->input('route_mobile_sender');

        $site_settings->show_offers = $req->input('show_offers');

        $site_settings->save();

        return $site_settings;
    }
}
