<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomState;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomStateController extends Controller
{
    /**
     * List states
     * GET /tiffintom/states
     */
    public function index(Request $req)
    {
        $states = TiffintomState::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $states->whereRaw('(state_name LIKE ? OR state_code LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($country_id = $req->query("country_id", false)) {
            $states->where('country_id', $country_id);
        }

        if ($status = $req->query("status", false)) {
            $states->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $states = $states->get();
        } else {
            $states = $states->paginate($req->query('per_page', 20));
        }

        if ($states->isEmpty()) Utils::error('States not found!', 404);

        return $states;
    }

    /**
     * Create state.
     * POST /tiffintom/states
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'country_id' => 'required|exists:tiffintom_connection.countries,id',
            'state_name' => 'required|string',
            'state_code' => 'required|string',
        ]);

        // create state
        $state = new TiffintomState;
        $state->fill($req->all());
        $state->status = '1';
        $state->save();
        return $state;
    }

    /**
     * List states
     * GET /tiffintom/states/id
     */
    public function show(Request $req, $id)
    {
        $states = TiffintomState::find($id);
        if (empty($states)) Utils::error("State not found", 404);
        return $states;
    }

    /**
     * Update state.
     * POST  /tiffintom/states/id
     */
    public function update(Request $req, $id)
    {
        $state = TiffintomState::find($id);
        if (empty($state)) Utils::error('State not found!', 404);

        $this->validate($req, [
            'country_id' => 'required|exists:tiffintom_connection.countries,id',
            'state_name' => 'required|string',
            'state_code' => 'required|string',
        ]);

        // update country
        if ($req->has('country_id')) $state->country_id = $req->input('country_id');
        if ($req->has('state_name')) $state->state_name = $req->input('state_name');
        if ($req->has('state_code')) $state->state_code = $req->input('state_code');
        $state->save();

        return $state;
    }

    /**
     * Delete state by id.
     * DELETE /tiffintom/states/{id}
     */
    public function delete(Request $req, $id)
    {
        $state = TiffintomState::find($id);
        if (empty($state)) Utils::error('State not found!', 404);
        $state->delete();
        return $state;
    }

    /**
     * Enable state.
     * POST /tiffintom/states/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $state = TiffintomState::find($id);
        if (empty($state)) Utils::error('State not found!', 404);
        $state->status = '1';
        $state->save();
        return $state;
    }

    /**
     * Disable state.
     * POST /tiffintom/states/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $state = TiffintomState::find($id);
        if (empty($state)) Utils::error('State not found!', 404);
        $state->status = '0';
        $state->save();
        return $state;
    }
}
