<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomStaticPage;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomStaticPageController extends Controller
{

    /**
     * List static-page
     * GET /tiffintom/static-pages
     */
    public function index(Request $req)
    {
        $static_pages = TiffintomStaticPage::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $static_pages->whereRaw('(section LIKE ? OR title LIKE ?  OR content LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($status = $req->query('status', false)) {
            $static_pages->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $static_pages = $static_pages->get();
        } else {
            $static_pages = $static_pages->paginate($req->query('per_page', 20));
        }

        if ($static_pages->isEmpty()) Utils::error('No static pages found!', 404);

        return $static_pages;
    }


    /**
     * Create static-page.
     * POST /tiffintom/static-pages
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'title' => 'required|string',
            'section' => 'string',
            'seo_url' => 'string',
            'sortorder' => 'string',
            'content' => 'required|string',
        ]);

        // create static-page
        $static_page = new TiffintomStaticPage;
        $static_page->fill($req->all());
        $static_page->status = '1';
        if ($req->has('title')) $static_page->seo_url = $this->seoUrl($static_page->title);
        $static_page->save();

        $static_page->sortorder = $static_page->id;
        $static_page->save();

        return $static_page;
    }

    /**
     * List static-pages
     * GET /tiffintom/static-pages/id
     */
    public function show(Request $req, $id)
    {
        $static_page = TiffintomStaticPage::find($id);
        if (empty($static_page)) Utils::error("Static pages not found", 404);
        return $static_page;
    }

    /**
     * Update static-page.
     * POST  /tiffintom/static-pages/id
     */
    public function update(Request $req, $id)
    {
        $static_page = TiffintomStaticPage::find($id);
        if (empty($static_page)) Utils::error('Static pages not found!', 404);

        $this->validate($req, [
            'title' => 'required|string',
            'section' => 'string',
            'seo_url' => 'string',
            'sortorder' => 'string',
            'content' => 'required|string',
        ]);

        // update static-page
        if ($req->has('title')) $static_page->title = $req->input('title');
        if ($req->has('title')) $static_page->seo_url = $this->seoUrl($static_page->title);
        if ($req->has('section')) $static_page->section = $req->input('section');
        if ($req->has('sortorder')) $static_page->sortorder = $req->input('sortorder');
        if ($req->has('content')) $static_page->content = $req->input('content');
        $static_page->save();

        return $static_page;
    }

    /**
     * Delete static-page by id.
     * DELETE /tiffintom/static-pages/{id}
     */
    public function delete(Request $req, $id)
    {
        $static_page = TiffintomStaticPage::find($id);
        if (empty($static_page)) Utils::error('Static pages not found!', 404);
        $static_page->delete();

        return $static_page;
    }

    /**
     * Enable static-page.
     * POST /tiffintom/static-pages/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $static_page = TiffintomStaticPage::find($id);
        if (empty($static_page)) Utils::error('Static pages not found!', 404);
        $static_page->status = '1';
        $static_page->save();

        return $static_page;
    }

    /**
     * Disable static-page.
     * POST /tiffintom/static-pages/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $static_page = TiffintomStaticPage::find($id);
        if (empty($static_page)) Utils::error('Static pages not found!', 404);
        $static_page->status = '0';
        $static_page->save();

        return $static_page;
    }

    //Convert to seo url name
    public function seoUrl($string)
    {
        $text = preg_replace('~[^\\pL\d]+~u', '-', $string);

        $text = trim($text, '-');

        $text = strtolower($text);

        $text = preg_replace('~[^-\w]+~', '', $text);

        if (strlen($text) > 70) {
            $text = substr($text, 0, 70);
        }

        if (empty($text)) {
            return time();
        }

        return $text;
    }
}
