<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\CardBrand;
use App\CardBrandCommission;
use App\Country;
use App\CountryCommission;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsTiffintom\TiffintomRestaurant;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomStripeCustomer;
use App\ModelsTiffintom\TiffintomUser;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class TiffintomStripeCustomerController extends Controller
{


    /**
     * List card
     * GET /tiffintom/card
     */
    public function index(Request $req)
    {
        $card = TiffintomStripeCustomer::orderBy('id', 'DESC');

        // $customer = Auth::user();
        // if ($customer) {
        //     $card->where('customer_id', $customer->id);
        // }

        if ($query = $req->query('customer_id', false)) {
            $card->where('customer_id', $query);
        }

        if ($query = $req->query('service_type', false)) {
            $card->where('service_type', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $card = $card->get();
        } else {
            $card = $card->paginate($req->query('per_page', 20));
        }

        if ($card->isEmpty()) Utils::error('No card found!', 404);

        return $card;
    }

    /**
     * Create card.
     * POST /tiffintom/card
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'customer_name' => 'required|string',
            'stripe_token_id' => 'required|string',
            'stripe_customer_id' => 'string',
            'card_id' => 'string',
            'card_number' => 'required|string',
            'card_brand' => 'string',
            'card_type' => 'string',
            'exp_month' => 'required|string',
            'exp_year' => 'required|string',
            'country' => 'string',
            'client_ip' => 'string',
            'service_type' => 'in:normal,connect',
        ]);

        $latestCard = TiffintomStripeCustomer::where('customer_id', $req->input('customer_id'))->where('service_type', $req->input('service_type'))->latest()->first();

        if ($req->input('service_type') == 'connect') {
            $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
            if ($mode == 'test') {
                $stripeKeyColoumns = 'connect_stripe_private_key_test';
            }
            if ($mode == 'live') {
                $stripeKeyColoumns = 'connect_stripe_private_key_live';
            }
            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        } else {
            $siteSetting = TiffintomSiteSetting::where('id', '1')->first();
            if ($siteSetting['stripe_mode'] == 'Test') {
                $key = $siteSetting['stripe_secretkeyTest'];
            }
            if ($siteSetting['stripe_mode'] == 'Live') {
                $key = $siteSetting['stripe_secretkey'];
            }
            $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
        }

        if (empty($latestCard)) {
            $Customer = TiffintomUser::where('id', $req->input('customer_id'))->first();
            $customerParams['email'] = $Customer['username'];
            if ($req->input('customer_name') != null) {
                $customerParams['name'] = $req->input('customer_name');
            }
            $customer = $stripe->customers->create($customerParams);

            $attachCustomer = $stripe->paymentMethods->attach(
                $req->input('stripe_token_id'),
                ['customer' => $customer->id]
            );
            $customerId = $customer->id;
        } else {
            $attachCustomer = $stripe->paymentMethods->attach(
                $req->input('stripe_token_id'),
                ['customer' => $latestCard->stripe_customer_id]
            );
            $customerId = $latestCard->stripe_customer_id;
        }
        // create card
        $card = new TiffintomStripeCustomer;
        $card->fill($req->all());
        $card->stripe_customer_id = $customerId;
        $card->save();

        return $card;
    }

    /**
     * List card
     * GET /tiffintom/card/id
     */
    public function show(Request $req, $id)
    {
        $card = TiffintomStripeCustomer::where('id', $id);
        if (empty($card)) Utils::error("No card found", 404);
        $card = $card->first();

        return $card;
    }

    /**
     * Update card.
     * POST  /tiffintom/card/id
     */
    public function update(Request $req, $id)
    {
        $card = TiffintomStripeCustomer::find($id);
        if (empty($card)) Utils::error('card not found!', 404);

        $this->validate($req, [
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'customer_name' => 'required|string',
            'stripe_token_id' => 'required|string',
            'stripe_customer_id' => 'required|string',
            'card_id' => 'string',
            'card_number' => 'required|string',
            'card_brand' => 'string',
            'card_type' => 'string',
            'exp_month' => 'required|string',
            'exp_year' => 'required|string',
            'country' => 'string',
            'client_ip' => 'string',
            'service_type' => 'in:normal,connect',
        ]);

        // update card
        if ($req->has('customer_id')) $card->customer_id = $req->input('customer_id');
        if ($req->has('customer_name')) $card->customer_name = $req->input('customer_name');
        if ($req->has('stripe_token_id')) $card->stripe_token_id = $req->input('stripe_token_id');
        if ($req->has('stripe_customer_id')) $card->stripe_customer_id = $req->input('stripe_customer_id');
        if ($req->has('card_id')) $card->card_id = $req->input('card_id');
        if ($req->has('card_number')) $card->card_number = $req->input('card_number');
        if ($req->has('card_brand')) $card->card_brand = $req->input('card_brand');
        if ($req->has('card_type')) $card->card_type = $req->input('card_type');
        if ($req->has('exp_month')) $card->exp_month = $req->input('exp_month');
        if ($req->has('exp_year')) $card->exp_year = $req->input('exp_year');
        if ($req->has('country')) $card->country = $req->input('country');
        if ($req->has('client_ip')) $card->client_ip = $req->input('client_ip');
        if ($req->has('service_type')) $card->service_type = $req->input('service_type');
        $card->save();

        return $card;
    }

    /**
     * Delete card by id.
     * DELETE /tiffintom/card/{id}
     */
    public function delete(Request $req, $id)
    {
        $card = TiffintomStripeCustomer::find($id);
        if (empty($card)) Utils::error('card not found!', 404);
        $card->delete();

        return $card;
    }

    public function payment_intent(Request $req)
    {
        $siteSetting = TiffintomSiteSetting::where('id', '1')->first();
        if ($siteSetting['stripe_mode'] == 'Test') {
            $key = $siteSetting['stripe_secretkeyTest'];
        }
        if ($siteSetting['stripe_mode'] == 'Live') {
            $key = $siteSetting['stripe_secretkey'];
        }
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        $restaurant = '';
        // if ($req->input('service_type') == 'connect') {
        $restaurant = TiffintomRestaurant::where('id', $req->input('restaurant_id'))->first();
        // }

        // if (empty($restaurant)) Utils::error('Restaurant not found!', 404);
        if (!empty($restaurant) && $restaurant['stripe_descriptor'] != null && $restaurant['stripe_description']) {
            $statement_descriptor = $restaurant['stripe_descriptor'];
            $statement_description = $restaurant['stripe_description'];
        } else {
            $statement_descriptor = $siteSetting['stripe_descriptor'];
            $statement_description = $siteSetting['stripe_description'];
        }


        if ($req->input('service_type') == 'normal') {
            if ($siteSetting['minimum_wallet'] > $req->input('amount')) {
                Utils::error('Please enter minimum £' . $siteSetting['minimum_wallet'] . ' amount.', 404);
            }
        }

        $amount = $req->input('amount') * 100;
        $humanAmount = $req->input('amount');
        // $card = TiffintomStripeCustomer::find($req->input('id'));
        // if (empty($card)) Utils::error('card not found!', 404);

        $intentParams = [
            'payment_method' => $req->input('stripe_token_id'),
            'amount' => (float) ($amount),
            'currency' => 'GBP',
            'confirmation_method' => 'automatic',
        ];

        if ($statement_descriptor != null) {
            $intentParams['statement_descriptor'] = $statement_descriptor;
            $intentParams['description'] = $statement_description;
        }

        if ($req->has('order_id')) {
            $intentParams['statement_descriptor_suffix'] = 'ORD00' . $req->input('order_id');
        }

        if (!empty($restaurant) && $req->input('service_type') == 'connect') {
            $business = Business::where('restaurant_id', $restaurant->id)->where('connect_service', '1')->whereNotNull('s_account_id')->first();
            if (!empty($business)) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                $statement_descriptor = $business->s_account_id;
                $statement_description = $business->statement_description;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
                $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                $fee = 0;
                $country_stripe_fixed = 0;
                $country_stripe_percentage = 0;
                $card_stripe_fixed = 0;
                $card_stripe_percentage = 0;

                $stripe_payment_method = $stripe->paymentMethods->retrieve($req->input('stripe_token_id'));
                $card_brand = $stripe_payment_method->card->brand;
                $card_country = $stripe_payment_method->card->country;

                $country = Country::where('iso', $card_country)->first();
                if (!empty($country)) {
                    $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                }
                if (empty($countryCommission)) {
                    $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                }
                if (!empty($countryCommission)) {
                    $country_stripe_percentage = $this->calculateFee($humanAmount, $countryCommission->stripe_percentage, 'percentage');
                    $country_stripe_fixed = $countryCommission->stripe_fixed;
                }
                $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$card_brand, $card_brand])->first();
                if (!empty($cardBrand)) {
                    $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
                }
                if (empty($cardBrandCommission)) {
                    $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
                }
                if (!empty($cardBrandCommission)) {
                    $card_stripe_percentage = $this->calculateFee($humanAmount, $cardBrandCommission->stripe_percentage, 'percentage');
                    $card_stripe_fixed = $cardBrandCommission->stripe_fixed;
                }

                $fee = $this->calculateApplicationFee($humanAmount, $business);
                $fee = $fee + $card_stripe_fixed + $card_stripe_percentage + $country_stripe_fixed + $country_stripe_percentage;

                // $intentParams['application_fee_amount'] = $fee * 100;
                if ($business->transfer_connect == 'yes') {
                    $intentParams['transfer_data'] = [
                        'destination' => $business->s_account_id
                    ];
                }
            }
        } else {
            // Default Connect Account Key
            $stripe_account_id = (new SiteSettingController())->findByKey('default_connect_account')->value;
            if ($stripe_account_id != null) {
                $intentParams['transfer_data'] = [
                    'destination' => $stripe_account_id
                ];
            }
        }

        if ($req->input('stripe_customer_id') != null) {
            $intentParams['customer'] = $req->input('stripe_customer_id');
            $intentParams['save_payment_method'] = true;
        }

        if ($req->input('stripe_customer_id') == null) {
            $Customer = TiffintomUser::where('id', $req->input('customer_id'))->first();
            $customerParams['email'] = $Customer['username'];
            if ($req->input('customer_name') != null) {
                $customerParams['name'] = $req->input('customer_name');
            }
            $customer = $stripe->customers->create($customerParams);
            $intentParams['customer'] = $customer->id;
            $intentParams['save_payment_method'] = true;
        }

        try {
            $intent = $stripe->paymentIntents->create($intentParams);
        } catch (Exception $e) {
            return Utils::error($e, 404);
        }


        return ['payment_intent_id' => $intent->client_secret, 'payment_intent' => $intent];
    }

    public function payment_intent_wihout_customer(Request $req)
    {
        $siteSetting = TiffintomSiteSetting::where('id', '1')->first();
        if ($siteSetting['stripe_mode'] == 'Test') {
            $key = $siteSetting['stripe_secretkeyTest'];
        }
        if ($siteSetting['stripe_mode'] == 'Live') {
            $key = $siteSetting['stripe_secretkey'];
        }
        $stripe =  new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

        $restaurant = '';
        $restaurant = TiffintomRestaurant::where('id', $req->input('restaurant_id'))->first();

        if (!empty($restaurant) && $restaurant['stripe_descriptor'] != null && $restaurant['stripe_description']) {
            $statement_descriptor = $restaurant['stripe_descriptor'];
            $statement_description = $restaurant['stripe_description'];
        } else {
            $statement_descriptor = $siteSetting['stripe_descriptor'];
            $statement_description = $siteSetting['stripe_description'];
        }

        $amount = $req->input('amount') * 100;
        $humanAmount = $req->input('amount');

        $intentParams = [
            'payment_method' => $req->input('stripe_token_id'),
            'amount' => (float) ($amount),
            'currency' => 'GBP',
            'confirmation_method' => 'automatic',
        ];

        if ($statement_descriptor != null) {
            $intentParams['statement_descriptor'] = $statement_descriptor;
            $intentParams['description'] = $statement_description;
        }

        if (!empty($restaurant) && $req->input('service_type') == 'connect') {
            $business = Business::where('restaurant_id', $restaurant->id)->where('connect_service', '1')->whereNotNull('s_account_id')->first();
            if (!empty($business)) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                $statement_descriptor = $business->s_account_id;
                $statement_description = $business->statement_description;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
                $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

                $fee = 0;
                $country_stripe_fixed = 0;
                $country_stripe_percentage = 0;
                $card_stripe_fixed = 0;
                $card_stripe_percentage = 0;

                $stripe_payment_method = $stripe->paymentMethods->retrieve($req->input('stripe_token_id'));
                $card_brand = $stripe_payment_method->card->brand;
                $card_country = $stripe_payment_method->card->country;

                $country = Country::where('iso', $card_country)->first();
                if (!empty($country)) {
                    $countryCommission = CountryCommission::where('country_id', $country->id)->where('business_id', $business->id)->first();
                }
                if (empty($countryCommission)) {
                    $countryCommission = CountryCommission::whereNull('country_id')->where('other', '1')->where('business_id', $business->id)->first();
                }
                if (!empty($countryCommission)) {
                    $country_stripe_percentage = $this->calculateFee($humanAmount, $countryCommission->stripe_percentage, 'percentage');
                    $country_stripe_fixed = $countryCommission->stripe_fixed;
                }
                $cardBrand = CardBrand::whereRaw('(name LIKE ? OR code LIKE ?)', [$card_brand, $card_brand])->first();
                if (!empty($cardBrand)) {
                    $cardBrandCommission = CardBrandCommission::where('card_brand_id', $cardBrand->id)->where('business_id', $business->id)->first();
                }
                if (empty($cardBrandCommission)) {
                    $cardBrandCommission = CardBrandCommission::whereNull('card_brand_id')->where('other', '1')->where('business_id', $business->id)->first();
                }
                if (!empty($cardBrandCommission)) {
                    $card_stripe_percentage = $this->calculateFee($humanAmount, $cardBrandCommission->stripe_percentage, 'percentage');
                    $card_stripe_fixed = $cardBrandCommission->stripe_fixed;
                }

                // $fee = $this->calculateApplicationFee($humanAmount, $business);
                // $fee = $fee + $card_stripe_fixed + $card_stripe_percentage + $country_stripe_fixed + $country_stripe_percentage;

                // $intentParams['application_fee_amount'] = $fee * 100;
                if ($business->transfer_connect == 'yes') {
                    $intentParams['transfer_data'] = [
                        'destination' => $business->s_account_id
                    ];
                }
            }
        }

        if ($stripe_payment_method->customer != null) {
            $intentParams['customer'] = $stripe_payment_method->customer;
        } else {
            $customerParams['name'] = $req->input('customer_name');
            if ($req->input('customer_email') != null) {
                $customerParams['email'] = $req->input('customer_email');
            }
            $customer = $stripe->customers->create($customerParams);
            $intentParams['customer'] = $customer->id;
        }
        $intentParams['save_payment_method'] = true;

        $intent = $stripe->paymentIntents->create($intentParams);
        return ['payment_intent_id' => $intent->client_secret, 'payment_intent' => $intent];
    }

    public function calculateApplicationFee($humanAmount, $business)
    {
        $onlineCardFee = 0;
        $onlineCommissionFee = 0;
        $fee = 0;
        $payment_method_fees = 0;
        $orderTypeWiseCommission = 0;
        $orderTypeWiseServiceCharge = 0;
        $orderBasedOrTransactionWiseCharge = 0;
        $cardAuthFee = 0;
        $cardProcessingFee = 0;

        // $onlineCardFee  = $this->calculateFee($humanAmount, $business->business_commissions->online_card_fee, $business->business_commissions->online_card_fee_type);
        // $onlineCommissionFee  = $this->calculateFee($humanAmount, $business->business_commissions->online_commission_fee, $business->business_commissions->online_commission_fee_type);
        // $fee = $this->convertToDecimal($onlineCardFee + $onlineCommissionFee);

        $payment_method_fees  = $this->calculateFee($humanAmount, $business->business_commissions->online_card_fee, $business->business_commissions->online_card_fee_type);

        $orderTypeWiseCommission = $this->calculateFee($humanAmount, $business->business_commissions->online_commission_fee, $business->business_commissions->online_commission_fee_type);
        $orderTypeWiseServiceCharge = $this->calculateFee($humanAmount, $business->business_commissions->online_service_fee, $business->business_commissions->online_service_fee_type);
        if ($business->business_commissions->order_charge_type == 'order_based') {
            $orderBasedOrTransactionWiseCharge = $this->calculateFee($humanAmount, $business->business_commissions->per_order_charge, $business->business_commissions->per_order_charge_type);
        } else {
            $orderBasedOrTransactionWiseCharge  = $this->calculateFee($humanAmount, $business->business_commissions->card_charge, $business->business_commissions->card_charge_type);
        }

        $fee = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $cardAuthFee + $cardProcessingFee);

        return $fee;
    }

    public function calculateFee($humanAmount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($humanAmount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
