<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCart;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomRestaurantVoucher;
use App\ModelsTiffintom\TiffintomUserVoucher;
use App\ModelsTiffintom\TiffintomSurcharge;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomSurchargeController extends Controller
{
    /**
     * List surcharges
     * GET /tiffintom/surcharges
     */
    public function index(Request $req)
    {
        $surcharges = TiffintomSurcharge::orderBy('id', 'ASC');

        if ($query = $req->query("query", false)) {
            $surcharges->whereRaw('(surcharge_name LIKE ? OR surcharge_amount LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($restaurant_id = $req->query('restaurant_id')) {
            $surcharges->where('restaurant_id', $restaurant_id);
        }

        if ($status = $req->query('status')) {
            $surcharges->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $surcharges = $surcharges->get();
        } else {
            $surcharges = $surcharges->paginate($req->query('per_page', 20));
        }

        if ($surcharges->isEmpty()) Utils::error('Surcharge not found!', 404);

        return $surcharges;
    }


    /**
     * Create surcharge.
     * POST /tiffintom/surcharges
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|string',
            'surcharge_name' => 'required|string',
            'surcharge_amount' => 'required|numeric|gt:0',
            'status' => 'numeric',
        ]);

        // create surcharge
        $surcharge = new TiffintomSurcharge;
        $surcharge->fill($req->all());
        $surcharge->save();

        return $surcharge;
    }

    /**
     * List surcharges
     * GET /tiffintom/surcharges/id
     */
    public function show(Request $req, $id)
    {
        $surcharges = TiffintomSurcharge::find($id);
        if (empty($surcharges)) Utils::error("Surcharge not found", 404);
        return $surcharges;
    }

    /**
     * Update surcharge.
     * POST  /tiffintom/surcharges/id
     */
    public function update(Request $req, $id)
    {
        $surcharge = TiffintomSurcharge::find($id);
        if (empty($surcharge)) Utils::error('Surcharge not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|string',
            'surcharge_name' => 'required|string',
            'surcharge_amount' => 'required|numeric|gt:0',
            'status' => 'numeric',
        ]);

        // update surcharge
        if ($req->has('restaurant_id')) $surcharge->restaurant_id = $req->input('restaurant_id');
        if ($req->has('surcharge_name')) $surcharge->surcharge_name = $req->input('surcharge_name');
        if ($req->has('surcharge_amount')) $surcharge->surcharge_amount = $req->input('surcharge_amount');
        if ($req->has('status')) $surcharge->status = $req->input('status');

        $surcharge->save();

        return $surcharge;
    }

    /**
     * Delete surcharge by id.
     * DELETE /tiffintom/surcharges/{id}
     */
    public function delete(Request $req, $id)
    {
        $surcharge = TiffintomSurcharge::find($id);
        if (empty($surcharge)) Utils::error('Surcharge not found!', 404);
        $surcharge->delete();

        return $surcharge;
    }
}