<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomTimeZone;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomTimeZoneController extends Controller
{
    /**
     * List time-zones
     * GET /tiffintom/time-zones
     */
    public function index(Request $req)
    {
        $timezones = TiffintomTimeZone::orderBy('id', 'DESC');

        if ($query = $req->query("query", false)) {
            $timezones->whereRaw('(timezone_name LIKE ? OR iso_code LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $timezones = $timezones->get();
        } else {
            $timezones = $timezones->paginate($req->query('per_page', 20));
        }

        if ($timezones->isEmpty()) Utils::error('time-zones not found!', 404);

        return $timezones;
    }

    /**
     * Create timezone.
     * POST /tiffintom/time-zones
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'iso_code' => 'required|string',
            'timezone_name' => 'required|string',
        ]);

        // create timezone
        $timezone = new TiffintomTimeZone;
        $timezone->fill($req->all());
        $timezone->save();

        return $timezone;
    }

    /**
     * List time-zones
     * GET /tiffintom/time-zones/id
     */
    public function show(Request $req, $id)
    {
        $timezones = TiffintomTimeZone::find($id);
        if (empty($timezones)) Utils::error("time-zone not found", 404);
        return $timezones;
    }

    /**
     * Update timezone.
     * POST  /tiffintom/time-zones/id
     */
    public function update(Request $req, $id)
    {
        $timezone = TiffintomTimeZone::find($id);
        if (empty($timezone)) Utils::error('time-zone not found!', 404);

        $this->validate($req, [
            'iso_code' => 'required|string',
            'timezone_name' => 'required|string',
        ]);

        // update country
        if ($req->has('iso_code')) $timezone->iso_code = $req->input('iso_code');
        if ($req->has('timezone_name')) $timezone->timezone_name = $req->input('timezone_name');
        $timezone->save();

        return $timezone;
    }

    /**
     * Delete timezone by id.
     * DELETE /tiffintom/time-zones/{id}
     */
    public function delete(Request $req, $id)
    {
        $timezone = TiffintomTimeZone::find($id);
        if (empty($timezone)) Utils::error('time-zone not found!', 404);
        $timezone->delete();
        return $timezone;
    }
}