<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\BusinessCardReader;
use App\CardBrand;
use App\CardBrandCommission;
use App\Country;
use App\CountryCommission;
use App\Helpers\Auth;
use App\Helpers\OtpHelper;
use App\Helpers\PushNotificationTiffintomHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposOrder;
use App\ModelsEpos\EposTable;
use App\ModelsTiffintom\TiffintomUrl;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class TiffintomUrlController extends Controller
{

    /**
     * List url
     * GET /tiffintom/urls
     */
    public function index(Request $req)
    {
        $urls = TiffintomUrl::orderBy('id', 'DESC');

        if ($query = $req->query('status')) {
            $urls->where('status', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $urls = $urls->get();
        } else {
            $urls = $urls->paginate($req->query('per_page', 20));
        }

        if ($urls->isEmpty()) Utils::error('urls not found!', 404);

        return $urls;
    }

    /**
     * Create url.
     * POST /tiffintom/urls
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'package_name' => 'required|string',
            'url' => 'required|string',
            'delete_status' => 'string',
        ]);

        // create url
        $url = new TiffintomUrl;
        $url->fill($req->all());
        $url->save();

        return $url;
    }

    /**
     * List url
     * GET /tiffintom/urls/id
     */
    public function show(Request $req, $id)
    {
        $url = TiffintomUrl::find($id);
        if (empty($url)) Utils::error("No url found", 404);
        return $url;
    }

    /**
     * Update url.
     * POST  /tiffintom/urls/id
     */
    public function update(Request $req, $id)
    {
        $url = TiffintomUrl::find($id);
        if (empty($url)) Utils::error('url not found!', 404);

        $this->validate($req, [
            'package_name' => 'required|string',
            'url' => 'required|string',
            'delete_status' => 'string',
        ]);

        // update url
        if ($req->has('package_name')) $url->package_name = $req->input('package_name');
        if ($req->has('url')) $url->url = $req->input('url');
        if ($req->has('delete_status')) $url->delete_status = $req->input('delete_status');
        $url->save();

        return $url;
    }

    /**
     * Delete url by id.
     * DELETE /tiffintom/urls/{id}
     */
    public function delete(Request $req, $id)
    {
        $url = TiffintomUrl::find($id);
        if (empty($url)) Utils::error('url not found!', 404);
        $url->delete();

        return $url;
    }
}
