<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use Lcobucci\JWT\Signer\Hmac\Sha256;
use Lcobucci\JWT\Token;
use Illuminate\Support\Facades\Crypt;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomUser;
use DateTimeImmutable;
use App\Helpers\DefaultPasswordHasher;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\Jobs\EmailSender;
use App\ModelsEpos\EposCustomer;
use App\ModelsTiffintom\TiffintomAddressBook;
use App\ModelsTiffintom\TiffintomBooking;
use App\ModelsTiffintom\TiffintomCustomerReward;
use App\ModelsTiffintom\TiffintomMessage;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomReferral;
use App\ModelsTiffintom\TiffintomReferralPending;
use App\ModelsTiffintom\TiffintomReward;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ModelsTiffintom\TiffintomStripeCustomer;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Queue;
use Illuminate\Validation\Rule;
use Lcobucci\JWT\Configuration;
use Lcobucci\JWT\Signer\Key\InMemory;
use Lcobucci\JWT\Validation\Constraint\SignedWith;
use mysqli;
use Illuminate\Support\Str;

class TiffintomUserController extends Controller
{
    /**
     * Show admin.
     * GET /api/users/me
     */
    public function index(Request $req)
    {
        $users = TiffintomUser::orderBy('created', 'DESC');

        if ($query = $req->query('query', false)) {
            $users->whereRaw('(first_name LIKE ? OR last_name LIKE ? OR phone_number LIKE ? OR username LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($role_id = $req->query("role_id", false)) {
            $users->where('role_id', '!=', '1');
            $users->where('role_id', '!=', '2');
            $users->where('role_id', '!=', '4');
            $users->where('role_id', $role_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $users = $users->get();
        } else {
            $users = $users->paginate($req->query('per_page', 20));
        }

        if ($users->isEmpty()) Utils::error('No users found!', 404);
        $users->append('orders');
        $users->append('reservations');

        return $users;
    }

    /**
     * List users.
     * GET /api/users/mini
     */
    public function mini(Request $req)
    {
        $users = TiffintomUser::orderByRaw('id DESC');
        // $users->where('role_id', '3');
        $users->where('role_id', '!=', '1');
        $users->where('role_id', '!=', '2');
        $users->where('role_id', '!=', '4');
        $users->where('status', '1');

        if ($query = $req->query("query", false)) {
            $users->whereRaw('(first_name)', ["%$query%"]);
        }

        $users = $users->get();

        if ($users->isEmpty()) Utils::error('No users found!', 404);

        return ($users);
    }


    /**
     * Subadmin Create user.
     * POST /tiffintom/users
     */
    public function subadmin_create(Request $req)
    {
        $this->validate($req, [
            'first_name' => 'string|max:255',
            'last_name' => 'string|max:255',
            'image' => 'image',
            'cover_image' => 'image',
            'phone_number' => 'string|max:255',
            'newsletter' => 'string|max:255',
            'device_type' => 'string|max:255',
            'device_id' => 'string|max:255',
            'order_place' => 'string',
            'order_accept' => 'string',
            'order_reject' => 'string',
            'order_delivered' => 'string',
            'book_table' => 'string',
            'book_status' => 'string',
            'important_update' => 'string',
            'social_id' => 'string|max:255',
            'email_verify' => 'string|max:255',
            'phone_verify' => 'string|max:255',
        ]);

        // create user
        $user = new TiffintomUser;
        $user->fill($req->all());
        $user->status = '1';
        $user->save();

        return $user;
    }

    public function show(Request $req, $id = 'me')
    {
        if ($id == 'me') $user = Auth::user();
        else $user = TiffintomUser::find($id);

        if (empty($user)) Utils::error('User not found!', 404);

        if ($user->status == 0 || $user->deleted_status == 'Y') Utils::error('Your account is disabled', 401, 401);

        if (!$user->social_id) {
            $checkMessages =  TiffintomMessage::where('status', '1')->where('delete_status', 'N')->orderByRaw('id ASC')->get();
            if (!$checkMessages->isEmpty()) {
                foreach ($checkMessages as $CheckMessage) {
                    $sub = substr($user->phone_number, 0, $CheckMessage['number_count']);
                    if ($CheckMessage['number'] == $sub) {
                        $phone_length =  Str::length($user->phone_number);
                        if ($CheckMessage['minimum_digit'] <= $phone_length && $CheckMessage['maximum_digit'] >= $phone_length) {
                            $checkMessages =  TiffintomMessage::find($CheckMessage['id']);
                            $user->contact_type = $checkMessages->device_type;
                            $user->mobile = $checkMessages->mobile;
                            // if ($user->mobile == '0') {
                            //     $user->phone_verify = '1';
                            // }
                            // $user->save();
                        }
                    }
                }
            }
        }

        if ($user->otp) {
            $user['otp_send'] = true;
        } else {
            $user['otp_send'] = false;
        }

        //Total Earn Points Sum
        $rewardPoint = TiffintomReward::orderBy('id', 'ASC')->first();
        $today = date("d-m-Y");
        $expireday = '-' . $rewardPoint['reward_validity'] . ' days';
        $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
        $startdate = date("Y-m-d 23:59:59", strtotime($today));
        $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));
        $user['total_earn_point'] = TiffintomCustomerReward::where('customer_id', $user->id)->where('created', '<=', $startdate)->where('created', '>=', $endDate)->where('status', '1')->sum('points');

        $total_orders = TiffintomOrder::where('customer_id', $user->id)->count();
        if ($total_orders > 0) {
            $user['first_user'] = 'No';
        } else {
            $user['first_user'] = 'Yes';
        }

        return $user;
    }

    /**
     * Login user.
     * POST /api/users/signup
     */
    public function signup(Request $req)
    {
        $users = TiffintomUser::where('username', $req->input('username'));
        if ($req->has('role_id')) {
            $users->where('role_id', $req->input('role_id'));
        } else {
            $users->where('role_id', '3');
        }
        $users = $users->first();
        if (!empty($users) && !$req->has('social_id')) Utils::error("The email address is alredy in use.", 404);

        $this->validate($req, [
            'first_name' => 'string',
            'last_name' => 'string',
            'phone_number' => 'string',
            'referral_code' => 'string',
            'username' => 'string',
            'password' => 'string',
        ]);

        if ($req->has('confirm_password') && $req->input('password') != $req->input('confirm_password') && !$req->has('social_id')) {
            return Utils::error('Password and confirm password not match', 400);
        }

        if (!$req->has('social_id')) {
            $validate = (new TiffintomMessageController())->validate_number($req->input('phone_number'));
        }

        if ($req->has('social_id')) {
            $user = TiffintomUser::where('social_id', $req->input('social_id'))->first();
            if (!empty($user)) {
                if ($req->has('username') && $req->input('social_type') == 'apple') {
                    $user->username = $req->input('username');
                    $user->save();
                }

                $tokens = $this->getNewTokens($user->id);
                return response()->json($user, 200, [
                    'x-token' => (string) $tokens['aToken'],
                    'x-refresh-token' => (string) $tokens['rToken'],
                ]);
            }
        }

        $user = new TiffintomUser;
        $user->fill($req->all());
        if ($req->has('role_id')) {
            $user->role_id = $req->input('role_id');
        } else {
            $user->role_id = '3';
        }
        if (!$req->has('social_id')) {
            if ($validate > 0) {
                $checkMessages =  TiffintomMessage::find($validate['id']);
                $user->contact_type = $checkMessages->device_type;
                $user->mobile = $checkMessages->mobile;
                if ($user->mobile == '0') {
                    $user->phone_verify = '1';
                }
            }
        }
        if ($req->has('password')) {
            $user->password = Hash::make($user->password);
        } else {
            $user->password = Hash::make('123123');
        }
        $user->referral_code = str_replace(' ', '', strtoupper($req->input('first_name'))) . rand(1000, 10000);
        $user->save();

        if ($req->has('referred_by')) {
            $refercode = TiffintomUser::where('referral_code', $req->input('referred_by'))->first();
            if (!empty($refercode)) {
                // Invited User
                $refferal = TiffintomReferral::orderBy('id', 'DESC')->first();
                $refer_pending = new TiffintomReferralPending;
                $refer_pending->invite_user = $refercode->id;
                $refer_pending->recieve_user = $user->id;
                $refer_pending->invite_amount = $refferal->invite_amount;
                $refer_pending->receive_amount = $refferal->receive_amount;
                $refer_pending->save();
            }
        }

        $tokens = $this->getNewTokens($user->id);

        $this->signupMail($user->id);

        $users = TiffintomUser::find($user->id);

        return response()->json($users, 200, [
            'x-token' => (string) $tokens['aToken'],
            'x-refresh-token' => (string) $tokens['rToken'],
        ]);
    }
    /**
     * Login user.
     * POST /api/users/login
     */
    public function login(Request $req)
    {
        $this->validate($req, [
            'username' => 'required|string|max:255',
            'password' => 'required|string',
        ], [
            'username.required' => 'Email address is required',
            'password.required' => 'Password is required',
        ]);

        $user = TiffintomUser::where('username', $req->input('username'));
        if ($req->has('role_id')) {
            $user->where('role_id', $req->input('role_id'));
        } else {
            $user->where('role_id', '3');
        }
        $user = $user->first();
        if (!empty($user)) {
            if ($user->social_id != '') Utils::error('This email has already been registered to log in via a social account. Please log in using the social account registered with this email.', 404);
            if (!Hash::check($req->input('password'), $user->password)) {
                return Utils::error('Incorrect email address or password.', 400);
            } else {

                if ($req->has('device_id')) {
                    $user->device_type = $req->input('device_type');
                    $user->device_id = $req->input('device_id');
                    $user->save();
                }

                $tokens = $this->getNewTokens($user->id);

                $total_orders = TiffintomOrder::where('customer_id', $user->id)->count();
                if ($total_orders > 0) {
                    $user->first_user = 'No';
                } else {
                    $user->first_user = 'Yes';
                }

                return response()->json($user, 200, [
                    'x-token' => (string) $tokens['aToken'],
                    'x-refresh-token' => (string) $tokens['rToken'],
                ]);
            }
        } else {
            return Utils::error('You are not registered with us. Please signup.', 400);
        }
        return $user;
    }

    /**
     * Login Admin.
     * POST /api/users/login-admin
     */
    public function login_admin(Request $req)
    {
        $this->validate($req, [
            'username' => 'required|string|max:255',
            'password' => 'required|string',
        ], [
            'username.required' => 'Email address is required',
            'password.required' => 'Password is required',
        ]);

        $user = TiffintomUser::where('username', $req->input('username'));
        $user->whereRaw('(role_id = ? OR role_id = ?)', ['1', '4']);
        $user = $user->first();

        if (!empty($user)) {
            if (!Hash::check($req->input('password'), $user->password)) {
                return Utils::error('Incorrect username or password.', 400);
            } else {
                $tokens = $this->getNewTokens($user->id);
                return response()->json($user, 200, [
                    'x-token' => (string) $tokens['aToken'],
                    'x-refresh-token' => (string) $tokens['rToken'],
                ]);
            }
        } else {
            return Utils::error('Your not authorize for this site.', 400);
        }

        return $user;
    }


    /**
     * Update user password.
     * POST /api/users/me/password
     */
    public function password(Request $req)
    {
        $user = Auth::user();
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'password' => 'required|string',
            'new_password' => 'required|string',
        ]);


        if (!Hash::check($req->input('password'), $user->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $user->password = Hash::make($req->input('new_password'));
            $user->save();

            $tokens = $this->getNewTokens($user->id);
            return response()->json($user, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
        return $user;
    }


    /**
     * Update user.
     * POST /api/users/{id}
     */
    public function update(Request $req, $id)
    {
        if ($id == 'me') $user = Auth::user();
        else $user = TiffintomUser::find($id);

        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'first_name' => 'string|max:255',
            'last_name' => 'string|max:255',
            'image' => 'image',
            'cover_image' => 'image',
            'phone_number' => 'string|max:255',
            'newsletter' => 'string|max:255',
            'device_type' => 'string|max:255',
            'device_id' => 'string|max:255',
            'order_place' => 'string',
            'order_accept' => 'string',
            'order_reject' => 'string',
            'order_delivered' => 'string',
            'book_table' => 'string',
            'book_status' => 'string',
            'important_update' => 'string',
            'social_id' => 'string|max:255',
            'email_verify' => 'string|max:255',
            'phone_verify' => 'string|max:255',
        ]);

        // update user
        if ($req->has('first_name')) $user->first_name = $req->input('first_name');
        if ($req->has('last_name')) $user->last_name = $req->input('last_name');
        if ($req->has('phone_number')) $user->phone_number = $req->input('phone_number');
        if ($req->has('order_place')) $user->order_place = $req->input('order_place');
        if ($req->has('order_accept')) $user->order_accept = $req->input('order_accept');
        if ($req->has('order_reject')) $user->order_reject = $req->input('order_reject');
        if ($req->has('order_delivered')) $user->order_delivered = $req->input('order_delivered');
        if ($req->has('book_table')) $user->book_table = $req->input('book_table');
        if ($req->has('book_status')) $user->book_status = $req->input('book_status');
        if ($req->has('important_update')) $user->important_update = $req->input('important_update');
        if ($req->has('phone_verify')) $user->phone_verify = $req->input('phone_verify');
        if ($req->has('device_id')) $user->device_id = $req->input('device_id');
        if ($req->has('email_verify')) $user->email_verify = $req->input('email_verify');
        if ($req->has('newsletter')) $user->newsletter = $req->input('newsletter');
        // $user->phone_verify = $req->input('phone_verify');

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_TIFFINTOM_USER, 'tiffintom');
            if ($user->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_TIFFINTOM_USER, $user->image, 'tiffintom');
            $user->image = $fileName;
        }

        if (!$user->social_id) {
            $validate = (new TiffintomMessageController())->validate_number($user->phone_number);
            if ($validate > 0) {
                $checkMessages =  TiffintomMessage::find($validate['id']);
                $user->contact_type = $checkMessages->device_type;
                $user->mobile = $checkMessages->mobile;
                if ($user->mobile == '0') {
                    $user->phone_verify = '1';
                }
            }
        }

        $user->save();

        return $user;
    }

    /**
     * Update user.
     * POST /api/users/{id}
     */
    public function verify_status(Request $req, $id)
    {
        if ($id == 'me') $user = Auth::user();
        else $user = TiffintomUser::find($id);

        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            // 'email_verify' => 'string|max:255',
            'phone_verify' => 'string|max:255',
        ]);

        // $user->email_verify = $req->input('email_verify');
        $user->phone_verify = $req->input('phone_verify');

        $user->save();

        return $user;
    }

    /**
     * Update user.
     * POST /api/users/{id}
     */
    public function remove_image(Request $req, $id)
    {
        if ($id == 'me') $user = Auth::user();
        else $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        if ($user->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_TIFFINTOM_USER, $user->image, 'tiffintom');
        $user->image = '';
        $user->save();

        return $user;
    }

    /**
     * Update Password user.
     * POST /api/users/{id}/change-password
     */
    public function chage_password(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'current_password' => 'string',
            'password' => 'required',
            'confirm_password' => 'string',
        ]);

        if ($req->has('current_password')) {
            if (!Hash::check($req->input('current_password'), $user->password)) {
                return Utils::error('Current password not matched.', 400);
            }
        }

        if ($req->has('confirm_password')) {
            if ($req->input('confirm_password') != $req->input('password')) {
                return Utils::error('Confirm password and password not matched.', 400);
            }
        }

        // update user
        if ($req->has('password')) $user->password = Hash::make($req->input('password'));
        $user->save();

        $total_orders = TiffintomOrder::where('customer_id', $user->id)->count();
        if ($total_orders > 0) {
            $user->first_user = 'No';
        } else {
            $user->first_user = 'Yes';
        }

        $tokens = $this->getNewTokens($user->id);

        return response()->json($user, 200, [
            'x-token' => (string) $tokens['aToken'],
            'x-refresh-token' => (string) $tokens['rToken'],
        ]);
        // return $user;
    }

    /**
     * Update Password user.
     * POST /api/users/{id}/change-user-password
     */
    public function chage_user_password(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'new_password' => 'required|string',
            'confirm_password' => 'required|string',
        ]);

        if ($req->input('new_password') != $req->input('confirm_password')) {
            return Utils::error('Confirm password not matched.', 400);
        }

        // update user
        if ($req->has('new_password')) $user->password = Hash::make($req->input('new_password'));
        $user->save();

        return $user;
    }


    /**
     * Enable user.
     * POST /api/users/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $user->status = 0;
        $user->save();
        return $user;
    }

    /**
     * Disable user.
     * POST /api/users/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $user->status = 1;
        $user->save();
        return $user;
    }

    /**
     * Delete user by id.
     * DELETE /api/users/{id}
     */
    public function delete(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $user->delete();
        if ($user->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_USER, $user->image);

        $tokens = $this->getNewTokens($user->id);

        // return response()->json($user, 200, [
        //     'x-token' => (string) $tokens['aToken'],
        //     'x-refresh-token' => (string) $tokens['rToken'],
        // ]);
        return $user;
    }

    /**
     * Token helper to get new tokens.
     */
    public static function getNewTokens($userId, $role = 'tiffintom_user')
    {
        $user = TiffintomUser::find($userId);
        if (empty($user)) return null;

        if ($user->status == 0 || $user->deleted_status == 'Y') Utils::error('Your account is disabled', 401, 401);

        $now   = new DateTimeImmutable();
        $aTokenExp = $now->modify('+1 minute');
        $rTokenExp = (new DateTimeImmutable())->modify('+360 hours'); // 3 days mate token valid rese

        $aTokenConfig = Auth::getConfig(env('X_TOKEN_KEY'));
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $user->password);
        $aBuilder = $aTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($aTokenExp)
            ->withClaim('id', $user->id)
            ->withClaim('role', $role);
        $aToken = $aBuilder->getToken($aTokenConfig->signer(), $aTokenConfig->signingKey())->toString();


        $rBuilder = $rTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($rTokenExp)
            ->withClaim('id', $user->id)
            ->withClaim('role', $role);
        $rToken = $rBuilder->getToken($rTokenConfig->signer(), $rTokenConfig->signingKey())->toString();

        return array('aToken' => Crypt::encrypt((string) $aToken), 'rToken' => Crypt::encrypt((string) $rToken));
    }

    public static function getConfig($key): Configuration
    {
        $config = Configuration::forSymmetricSigner(new Sha256(), InMemory::plainText($key));
        $config->setValidationConstraints(new SignedWith($config->signer(), $config->signingKey()));
        return $config;
    }

    /**
     * Token helper to get verify tokens.
     */
    public static function verifyRefreshToken(Token $rToken)
    {
        $user = TiffintomUser::find($rToken->claims()->get('id'));
        if (empty($user)) return false;
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $user->password);
        $constraints = $rTokenConfig->validationConstraints();
        $rTokenConfig->validator()->assert($rToken, ...$constraints);
        if (!$rTokenConfig->validator()->validate($rToken, ...$constraints)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * User Dashboard
     */
    public function dashboard()
    {
        $customer = Auth::user();

        $complete_orders = TiffintomOrder::where('customer_id', $customer->id)->limit('3')->orderByRaw('id DESC')->get();
        $delivered_orders = TiffintomOrder::select("orders.*")->where('orders.customer_id', $customer->id)->where('orders.status', 'Delivered')->leftJoin('reviews', 'reviews.order_id', '=', 'orders.id')->whereNull('reviews.id')->limit('3')->orderByRaw('orders.id DESC')->get();

        $bookings = TiffintomBooking::where('customer_id', $customer->id)->limit('3')->orderByRaw('id DESC')->get();
        $cards = TiffintomStripeCustomer::where('customer_id', $customer->id)->limit('3')->orderByRaw('id DESC')->get();
        $addresses = TiffintomAddressBook::where('user_id', $customer->id)->limit('3')->orderByRaw('id DESC')->get();

        //Total Earn Points Sum
        $rewardPoint = TiffintomReward::orderBy('id', 'ASC')->first();
        $today = date("d-m-Y");
        $expireday = '-' . $rewardPoint['reward_validity'] . ' days';
        $remainingDayss = date("d-m-Y", strtotime($today . $expireday));
        $startdate = date("Y-m-d 23:59:59", strtotime($today));
        $endDate = date("Y-m-d 00:00:00", strtotime($remainingDayss));
        $customerTotalPoints = TiffintomCustomerReward::where('customer_id', $customer->id)->where('created', '<=', $startdate)->where('created', '>=', $endDate)->where('status', '1')->sum('points');

        $usernamecount = 0;
        $phonecount = 0;
        $lastnamecount = 0;
        $firstnamecount = 0;
        $imagecount = 0;

        if ($customer['username'] != '') {
            if ($customer['email_verify'] == 1) {
                $usernamecount = 20;
            }
        }
        if ($customer['phone_number'] != '') {
            if ($customer['phone_verify'] == 1) {
                $phonecount = 20;
            }
        }
        if ($customer['last_name'] != '') {
            $lastnamecount = 20;
        }
        if ($customer['first_name'] != '') {
            $firstnamecount = 20;
        }
        if ($customer['image'] != '') {
            $imagecount = 20;
        }

        $allovercount = $usernamecount + $phonecount + $lastnamecount + $firstnamecount + $imagecount;

        $data = [
            'complete_order_list' => $complete_orders,
            'delivered_order_list' => $delivered_orders,
            'booking_list' => $bookings,
            'card_list' => $cards,
            'address_list' => $addresses,
            'stats' => [
                'total_earn_point' => $customerTotalPoints,
                'profile_percentage' => $allovercount,
            ]
        ];
        return $data;
    }

    /**
     * GET /tiffintom/users/send-referral
     */
    public function send_referral(Request $req)
    {

        if ($req->query('phone_number', false)) {
            $mobile = $req->query('phone_number');
        }

        $message = $req->query('message');
        (new OtpHelper())->sendMessage($message, $mobile, 'other_events');

        return ['message' => 'Link send successfully!'];
    }

    /**
     * GET /tiffintom/users/send-referral-email
     */
    public function send_referral_email(Request $req)
    {

        if ($req->query('email', false)) {
            $email = $req->query('email');
        }

        $customer = Auth::user();
        $this->referralMail($customer->id, $email);

        return ['message' => 'Email link send successfully!'];
    }

    /**
     * GET /tiffintom/users/send-otp
     */
    public function send_otp(Request $req)
    {

        $otp = rand(100000, 999999);
        $customer = Auth::user();

        $user = TiffintomUser::find($customer->id);
        $user->otp = $otp;
        $user->save();

        $message = 'Your one-time password is ' . $otp . ' please use this to verify your phone number on www.tiffintom.com.';
        (new OtpHelper())->sendMessage($message, $user->phone_number, 'otp_service');

        return ['message' => 'otp send successfully!'];
    }

    /**
     * GET /tiffintom/users/send-otp-both
     */
    public function send_otp_both(Request $req, $id)
    {
        $this->validate($req, [
            'verify_type' => 'required|in:phone,email,both',
        ]);

        $otp = rand(100000, 999999);
        $email_otp = rand(100000, 999999);
        $user = TiffintomUser::find($id);
        if ($req->input('verify_type') == 'phone' || $req->input('verify_type') == 'both') {
            $user->otp = $otp;
        }
        if ($req->input('verify_type') == 'email' || $req->input('verify_type') == 'both') {
            $user->email_otp = $email_otp;
        }
        $user->save();

        if ($req->input('verify_type') == 'phone' || $req->input('verify_type') == 'both') {
            $message = 'Your one-time password is ' . $otp . ' please use this to verify your phone number on www.tiffintom.com.';
            (new OtpHelper())->sendMessage($message, $user->phone_number, 'otp_service');
        }

        if ($req->input('verify_type') == 'email' || $req->input('verify_type') == 'both') {
            $this->otpBothMail($user->id);
        }

        return ['message' => 'otp send successfully!'];
    }

    /**
     * POST  GET /tiffintom/users/verify-otp
     */
    public function verify_otp(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'otp' => 'required',
        ]);

        if ($req->input('otp') !=  $user->otp) {
            return Utils::error('OTP not matched.please enter correct otp.', 400);
        }

        // update user
        $user->phone_verify = 1;
        $user->save();

        return $user;
    }

    /**
     * POST  GET /tiffintom/users/verify-otp
     */
    public function verify_otp_both(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'verify_type' => 'required|in:phone,email,both',
            'otp' => 'required_if:verify_type,phone,both',
            'email_otp' => 'required_if:verify_type,email,both',
        ]);

        if ($req->input('verify_type') == 'phone' || $req->input('verify_type') == 'both') {
            if ($req->input('otp') !=  $user->otp) {
                return Utils::error('Mobile OTP not matched. Please enter correct OTP.', 400);
            }
            $user->phone_verify = 1;
        }
        if ($req->input('verify_type') == 'email' || $req->input('verify_type') == 'both') {
            if ($req->input('email_otp') !=  $user->email_otp) {
                return Utils::error('Email OTP not matched. Please enter correct OTP.', 400);
            }
            $user->email_verify = 1;
        }

        // update user
        $user->save();

        return $user;
    }

    /**
     * POST  GET /tiffintom/users/verify-otp
     */
    public function email_verify_otp(Request $req, $id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'otp' => 'required',
        ]);

        if ($req->input('otp') !=  $user->otp) {
            return Utils::error('OTP not matched.please enter correct otp.', 400);
        }

        // update user
        $user->email_verify = 1;
        $user->save();

        return $user;
    }

    /**
     * POST  GET /tiffintom/users/verify-email
     */
    public function validate_email(Request $req)
    {
        if ($req->query('id', false)) {
            $id = $req->query('id');
        }

        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $otp = rand(100000, 999999);
        $user->otp = $otp;
        $user->save();

        $this->otpMail($user->id);

        return ['message' => 'mail send successfully!'];
    }

    /**
     * POST  GET /tiffintom/users/verify-email
     */
    public function verify_email(Request $req)
    {
        if ($req->query('id', false)) {
            $id = $req->query('id');
        }

        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $message = $req->query('code');
        $this->verifyMail($user->id, $message);

        return ['message' => 'mail send successfully!'];
    }

    /**
     * GET /tiffintom/users/forgot-password
     */
    public function forgot_password(Request $req)
    {
        $user = TiffintomUser::where('username', $req->input('username'))->where('deleted_status', 'N');
        if ($req->has('role_id')) {
            $user->where('role_id', $req->input('role_id'));
        } else {
            $user->where('role_id', '3');
        }
        $user = $user->first();

        if (empty($user)) Utils::error('User not found!', 404);

        if ($user->social_id != '') Utils::error('This email has already been registered to log in via a social account. Please log in using the social account registered with this email.', 404);

        $input      =   "1234567890abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
        $password   =   substr(str_shuffle($input), 0, 7);
        $user->password = Hash::make($password);
        $user->save();

        $this->forgotMail($user->id, $password);

        return $user;
    }

    /**
     * GET /tiffintom/users/forgot-password-direct
     */
    public function forgot_password_direct(Request $req)
    {
        $user = TiffintomUser::where('username', $req->input('email'))->where('deleted_status', 'N');
        if ($req->has('role_id')) {
            $user->where('role_id', $req->input('role_id'));
        } else {
            $user->where('role_id', '3');
        }
        $user = $user->first();

        if (empty($user)) Utils::error('User not found!', 404);

        if ($user->social_id != '') Utils::error('This email has already been registered to log in via a social account. Please log in using the social account registered with this email.', 404);

        $email_otp = rand(100000, 999999);
        $user->email_otp = $email_otp;
        $user->save();

        $this->forgotMailDirect($user->id);

        $message = 'Your one-time password is ' . $email_otp . ' please use this to verify your Email on Your App.';
        (new OtpHelper())->sendMessage($message, $user->phone_number, 'otp_service');

        return $user;
    }

    /**
     * List users/count
     * GET /tiffintom/users/count
     */
    public function count()
    {
        // where('role_id', '5')->
        $users = collect(TiffintomUser::where('deleted_status', 'N')->orderByRaw('id DESC')->get());
        $restaurants = collect(Business::orderByRaw('id DESC')->get());

        // Users
        $total_users = $users->where('status', '1')->count();

        // Restaurants
        $total_restaurants = $restaurants->where('disabled', '0')->count();

        $data = [
            'total_restaurants' => $total_restaurants,
            'total_users' => $total_users,
        ];
        return $data;
    }

    public function signupMail($id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($user->role_id == 5) {
            $colorCode = '#fc353a';
            $fontCode = '#FFFFFF';
            $buttonCode = '#fc353a';
            $logoUrl = env('APP_URL') . 'images/logo-gogrubz.png';
        } else {
            $colorCode = '#f1e0c6';
            $logoUrl = env('APP_URL') . 'images/logo-home.png';
            $buttonCode = '#f3cc2c';
            $fontCode = '#000000';
        }

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="' . $colorCode . '">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . $logoUrl . '">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                                  Hey ' . $user->first_name . ',<br>
                                  Thanks for registering. This is the beginning of a beautiful friendship.<br>
                                  Your account details are below – dont forget about them. They are kind of a big deal.
                             </p>
                          </td>
                       </tr>
                       <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <b>Account details</b><br>
                                  <center>Name: ' . $user->first_name . '</center><br>
                                  <center>Email: ' . $user->username . '</center>
                             </p>
                          </td>
                       </tr><br><br>
                         <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                            <center><a href="#"><button style="background-color: ' . $buttonCode . ';border: none;color: ' . $fontCode . ';padding: 15px 32px;text-align: center;text-decoration: none;display: inline-block;font-size: 20px;margin: 4px 2px;cursor: pointer;">Log In</button></a></center>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        if ($user->role_id != 5) {
            $emailinfo .= '  <tr>   
                <td bgcolor="#fff" >
                    <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                        <tr>
                            <td style="padding: 30px 30px; text-align: right;">
                                    <table align="left" style="text-align: left;">
                                        <tr>
                                            <td>
                                                   <p style="color:#000;"> Get The App : </p> 
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                        </tr>
                                        
                                    </table>
                                    <table align="right" style="text-align: right;">
                                    <tr>
                                        <td>
                                                <p style="color:#000;">Follow Us : </p>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                    </tr>
                                </table>
                            </td>
                        </tr>
                    </table>
                </td>
             </tr>';
        }
        $emailinfo .= '
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="' . $colorCode . '"><div style="color:' . $fontCode . ';font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Customer Signup',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->username,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function otpMail($id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($user->role_id == 5) {
            $colorCode = '#fc353a';
            $fontCode = '#FFFFFF';
            $buttonCode = '#fc353a';
            $logoUrl = env('APP_URL') . 'images/logo-gogrubz.png';
        } else {
            $colorCode = '#f1e0c6';
            $logoUrl = env('APP_URL') . 'images/logo-home.png';
            $buttonCode = '#f3cc2c';
            $fontCode = '#000000';
        }

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="' . $colorCode . '">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . $logoUrl . '">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $user->first_name . ',<br>
                             Your one-time password is ' . $user->otp . ' please use this to verify your Email on Your App.<br>
                             </p>
                          </td>
                       </tr>
                       <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <b>Account details</b><br>
                                  <center>Name: ' . $user->first_name . '</center><br>
                                  <center>Email: ' . $user->username . '</center>
                             </p>
                          </td>
                       </tr><br><br>
                         <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                            <center><a href="#"><button style="background-color: ' . $buttonCode . ';border: none;color: ' . $fontCode . ';padding: 15px 32px;text-align: center;text-decoration: none;display: inline-block;font-size: 20px;margin: 4px 2px;cursor: pointer;">Log In</button></a></center>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        if ($user->role_id != 5) {

            $emailinfo .= '  <tr>   
                <td bgcolor="#fff" >
                    <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                        <tr>
                            <td style="padding: 30px 30px; text-align: right;">
                                    <table align="left" style="text-align: left;">
                                        <tr>
                                            <td>
                                                   <p style="color:#000;"> Get The App : </p> 
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                        </tr>
                                        
                                    </table>
                                    <table align="right" style="text-align: right;">
                                    <tr>
                                        <td>
                                                <p style="color:#000;">Follow Us : </p>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                    </tr>
                                </table>
                            </td>
                        </tr>
                    </table>
                </td>
             </tr>';
        }
        $emailinfo .= '<tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="' . $colorCode . '"><div style="color:' . $fontCode . ';font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->username,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function otpBothMail($id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($user->role_id == 5) {
            $colorCode = '#fc353a';
            $fontCode = '#FFFFFF';
            $buttonCode = '#fc353a';
            $logoUrl = env('APP_URL') . 'images/logo-gogrubz.png';
        } else {
            $colorCode = '#f1e0c6';
            $logoUrl = env('APP_URL') . 'images/logo-home.png';
            $buttonCode = '#f3cc2c';
            $fontCode = '#000000';
        }

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="' . $colorCode . '">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . $logoUrl . '">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $user->first_name . ',<br>
                             Your one-time password is ' . $user->email_otp . ' please use this to verify your Email on Your App.<br>
                             </p>
                          </td>
                       </tr>
                       <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <b>Account details</b><br>
                                  <center>Name: ' . $user->first_name . '</center><br>
                                  <center>Email: ' . $user->username . '</center>
                             </p>
                          </td>
                       </tr><br><br>
                         <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                            <center><a href="#"><button style="background-color: ' . $buttonCode . ';border: none;color: ' . $fontCode . ';padding: 15px 32px;text-align: center;text-decoration: none;display: inline-block;font-size: 20px;margin: 4px 2px;cursor: pointer;">Log In</button></a></center>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
        </tr>';
        if ($user->role_id != 5) {

            $emailinfo .= '  <tr>   
                   <td bgcolor="#fff" >
                       <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                           <tr>
                               <td style="padding: 30px 30px; text-align: right;">
                                       <table align="left" style="text-align: left;">
                                           <tr>
                                               <td>
                                                      <p style="color:#000;"> Get The App : </p> 
                                               </td>
                                                   <td width="10">&nbsp;</td>
                                               <td>
                                                   <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                   <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                               </td>
                                                   <td width="10">&nbsp;</td>
                                               <td>
                                                   <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                   <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                               </td>
                                                   <td width="10">&nbsp;</td>
                                           </tr>
                                           
                                       </table>
                                       <table align="right" style="text-align: right;">
                                       <tr>
                                           <td>
                                                   <p style="color:#000;">Follow Us : </p>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td><td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td><td width="10">&nbsp;</td>
                                       </tr>
                                   </table>
                               </td>
                           </tr>
                       </table>
                   </td>
                </tr>';
        }
        $emailinfo .= '<tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="' . $colorCode . '"><div style="color:' . $fontCode . ';font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->username,
            'from' => $siteSettings->admin_email,
        );

        (new MailHelper())->sendMail($email);
    }

    public function forgotMail($id, $password)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($user->role_id == 5) {
            $colorCode = '#fc353a';
            $fontCode = '#FFFFFF';
            $buttonCode = '#fc353a';
            $logoUrl = env('APP_URL') . 'images/logo-gogrubz.png';
        } else {
            $colorCode = '#f1e0c6';
            $logoUrl = env('APP_URL') . 'images/logo-home.png';
            $buttonCode = '#f3cc2c';
            $fontCode = '#000000';
        }

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="' . $colorCode . '">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . $logoUrl . '">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                Hey ' . $user->first_name . ',<br>
                                Forgot your password? Dont worry – happens to the best of us.<br>
                                Simply click the button below to reset your password.<br>
                                <center><a href="#" style="background-color: ' . $buttonCode . ';border-radius: 5px;color: ' . $fontCode . ';padding: .5em;text-decoration: none;">Reset Password</a></center>
                                <br>
                                If you forgot the email you use with TIFFINTOM, its ' . $user->username . '. Your Password Is ' . $password . ' <br>
                                Hope that is sorted everything out.
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        if ($user->role_id != 5) {

            $emailinfo .= '  <tr>   
                <td bgcolor="#fff" >
                    <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                        <tr>
                            <td style="padding: 30px 30px; text-align: right;">
                                    <table align="left" style="text-align: left;">
                                        <tr>
                                            <td>
                                                   <p style="color:#000;"> Get The App : </p> 
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                        </tr>
                                        
                                    </table>
                                    <table align="right" style="text-align: right;">
                                    <tr>
                                        <td>
                                                <p style="color:#000;">Follow Us : </p>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                    </tr>
                                </table>
                            </td>
                        </tr>
                    </table>
                </td>
             </tr>';
        }
        $emailinfo .= '<tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="' . $colorCode . '"><div style="color:' . $fontCode . ';font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Forgot Password',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->username,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function forgotMailDirect($id)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#f1e0c6">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/logo-home.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $user->first_name . ',<br>
                             Your one-time password is ' . $user->email_otp . ' please use this to verify your Email on Your App.<br>
                             </p>
                          </td>
                       </tr>
                       <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <b>Account details</b><br>
                                  <center>Name: ' . $user->first_name . '</center><br>
                                  <center>Email: ' . $user->username . '</center>
                             </p>
                          </td>
                       </tr><br><br>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#f1e0c6"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Forgot Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->username,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function verifyMail($id, $link)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        if ($user->role_id == 5) {
            $colorCode = '#fc353a';
            $fontCode = '#FFFFFF';
            $buttonCode = '#fc353a';
            $logoUrl = env('APP_URL') . 'images/logo-gogrubz.png';
        } else {
            $colorCode = '#f1e0c6';
            $logoUrl = env('APP_URL') . 'images/logo-home.png';
            $buttonCode = '#f3cc2c';
            $fontCode = '#000000';
        }

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="' . $colorCode . '">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . $logoUrl . '">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                             Hey ' . $user->first_name . ',<br>
                             Please click the button below to verify your email address.<br>
                              <center><a href=' . $link . ' style="background-color: ' . $buttonCode . ';border-radius: 5px;color: ' . $fontCode . ';padding: .5em;text-decoration: none;">Verify Email Address</a> </center>												
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        if ($user->role_id != 5) {

            $emailinfo .= '  <tr>   
                <td bgcolor="#fff" >
                    <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                        <tr>
                            <td style="padding: 30px 30px; text-align: right;">
                                    <table align="left" style="text-align: left;">
                                        <tr>
                                            <td>
                                                   <p style="color:#000;"> Get The App : </p> 
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                                <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                            </td>
                                                <td width="10">&nbsp;</td>
                                        </tr>
                                        
                                    </table>
                                    <table align="right" style="text-align: right;">
                                    <tr>
                                        <td>
                                                <p style="color:#000;">Follow Us : </p>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td>
                                            <td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                        <td>
                                            <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                            <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                        </td><td width="10">&nbsp;</td>
                                    </tr>
                                </table>
                            </td>
                        </tr>
                    </table>
                </td>
             </tr>';
        }
        $emailinfo .= '<tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="' . $colorCode . '"><div style="color:' . $fontCode . ';font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->username,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function referralMail($id, $email)
    {
        $user = TiffintomUser::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#f1e0c6">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/logo-home.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                               ' . $user->referral_code . '
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#f1e0c6"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Refferal Code',
            'mail_type' => $siteSettings->mail_option,
            'to' => $email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);

        return 'Mail Send Successfully';
    }

    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }
}
