<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomUserVoucher;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomUserVoucherController extends Controller
{
    /**
     * List user_vouchers
     * GET /tiffintom/user_vouchers
     */
    public function index(Request $req)
    {
        $user_vouchers = TiffintomUserVoucher::selectRaw("uservoucher.*,restaurants.restaurant_name as restaurant_name,users.first_name as user_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->leftJoin('users', 'users.id', '=', 'uservoucher.user_id')->orderBy('uservoucher.id', 'DESC');

        if ($query = $req->query("restaurant_id", false)) {
            $user_vouchers->where('uservoucher.restaurant_id', $query);
        }
        if ($query = $req->query("user_id", false)) {
            $user_vouchers->where('uservoucher.user_id', $query);
        }
        if ($status = $req->query("status", false)) {
            $user_vouchers->where('uservoucher.status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $user_vouchers = $user_vouchers->get();
        } else {
            $user_vouchers = $user_vouchers->paginate($req->query('per_page', 20));
        }

        if ($user_vouchers->isEmpty()) Utils::error('Vouchers not found!', 404);

        return $user_vouchers;
    }

    /**
     * Create user_voucher.
     * POST /tiffintom/user_vouchers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'user_id' => 'required|exists:tiffintom_connection.users,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'voucher_code' => 'required|string',
            'device_type' => 'in:web,app,all',
            'type_offer' => 'in:single,multiple',
            'offer_mode' => 'in:price,percentage,free_delivery',
            'offer_value' => 'required|numeric|gt:0',
            'minimum_value' => 'required|numeric|gt:0',
            'multiple_value' => 'required|numeric|gt:0',
            'voucher_from' => 'required|date_format:d-m-Y',
            'voucher_to' => 'required|date_format:d-m-Y|after:voucher_from',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // create user_voucher
        $user_voucher = new TiffintomUserVoucher;
        $user_voucher->fill($req->all());
        $user_voucher->save();
        return $user_voucher;
    }

    /**
     * List user_vouchers
     * GET /tiffintom/user_vouchers/id
     */
    public function show(Request $req, $id)
    {
        $user_vouchers = TiffintomUserVoucher::find($id);
        if (empty($user_vouchers)) Utils::error("Voucher not found", 404);
        return $user_vouchers;
    }

    /**
     * Update user_voucher.
     * POST  /tiffintom/user_vouchers/id
     */
    public function update(Request $req, $id)
    {
        $user_voucher = TiffintomUserVoucher::find($id);
        if (empty($user_voucher)) Utils::error('Voucher not found!', 404);

        $this->validate($req, [
            'user_id' => 'required|exists:tiffintom_connection.users,id',
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'day' => 'required|string',
            'voucher_code' => 'required|string',
            'device_type' => 'in:web,app,all',
            'type_offer' => 'in:single,multiple',
            'offer_mode' => 'in:price,percentage,free_delivery',
            'offer_value' => 'required|numeric|gt:0',
            'minimum_value' => 'required|numeric|gt:0',
            'multiple_value' => 'required|numeric|gt:0',
            'voucher_from' => 'required|date_format:d-m-Y',
            'voucher_to' => 'required|date_format:d-m-Y|after:voucher_from',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // update user_voucher
        if ($req->has('user_id')) $user_voucher->user_id = $req->input('user_id');
        if ($req->has('restaurant_id')) $user_voucher->restaurant_id = $req->input('restaurant_id');
        if ($req->has('day')) $user_voucher->day = $req->input('day');
        if ($req->has('voucher_code')) $user_voucher->voucher_code = $req->input('voucher_code');
        if ($req->has('device_type')) $user_voucher->device_type = $req->input('device_type');
        if ($req->has('type_offer')) $user_voucher->type_offer = $req->input('type_offer');
        if ($req->has('offer_mode')) $user_voucher->offer_mode = $req->input('offer_mode');
        if ($req->has('offer_value')) $user_voucher->offer_value = $req->input('offer_value');
        if ($req->has('minimum_value')) $user_voucher->minimum_value = $req->input('minimum_value');
        if ($req->has('multiple_value')) $user_voucher->multiple_value = $req->input('multiple_value');
        if ($req->has('voucher_from')) $user_voucher->voucher_from = $req->input('voucher_from');
        if ($req->has('voucher_to')) $user_voucher->voucher_to = $req->input('voucher_to');
        $user_voucher->status = $req->input('status');
        $user_voucher->delivery_type = $req->input('delivery_type');
        $user_voucher->pickup_type = $req->input('pickup_type');
        $user_voucher->dinein_type = $req->input('dinein_type');
        $user_voucher->save();

        return $user_voucher;
    }

    /**
     * Delete user_voucher by id.
     * DELETE /tiffintom/user_vouchers/{id}
     */
    public function delete(Request $req, $id)
    {
        $user_voucher = TiffintomUserVoucher::find($id);
        if (empty($user_voucher)) Utils::error('Voucher not found!', 404);
        $user_voucher->delete();
        return $user_voucher;
    }

    /**
     * Enable user_voucher.
     * POST /tiffintom/user_vouchers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $user_voucher = TiffintomUserVoucher::find($id);
        if (empty($user_voucher)) Utils::error('Voucher not found!', 404);
        $user_voucher->status = '1';
        $user_voucher->save();
        return $user_voucher;
    }

    /**
     * Disable user_voucher.
     * POST /tiffintom/user_vouchers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $user_voucher = TiffintomUserVoucher::find($id);
        if (empty($user_voucher)) Utils::error('Voucher not found!', 404);
        $user_voucher->status = '0';
        $user_voucher->save();
        return $user_voucher;
    }
}
