<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomCart;
use App\ModelsTiffintom\TiffintomOrder;
use App\ModelsTiffintom\TiffintomRestaurantVoucher;
use App\ModelsTiffintom\TiffintomUserVoucher;
use App\ModelsTiffintom\TiffintomVoucher;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomVoucherController extends Controller
{
    /**
     * List vouchers
     * GET /tiffintom/vouchers
     */
    public function index(Request $req)
    {
        $vouchers = TiffintomVoucher::orderBy('id', 'DESC');

        if ($req->query('nopaginate', false) == '1') {
            $vouchers = $vouchers->get();
        } else {
            $vouchers = $vouchers->paginate($req->query('per_page', 20));
        }

        if ($vouchers->isEmpty()) Utils::error('Vouchers not found!', 404);

        return $vouchers;
    }

    /**
     * Create vouchers.
     * POST /tiffintom/voucherss
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'day' => 'required|string',
            'voucher_code' => 'required|string',
            'type_offer' => 'in:single,multiple',
            'offer_mode' => 'in:price,percentage,free_delivery',
            'offer_value' => 'required|numeric|gt:0',
            'minimum_value' => 'required|numeric|gt:0',
            'voucher_from' => 'required|date_format:d-m-Y',
            'voucher_to' => 'required|date_format:d-m-Y|after:voucher_from',
            'status' => 'boolean',
        ]);

        // create vouchers
        $vouchers = new TiffintomVoucher;
        $vouchers->fill($req->all());
        $vouchers->save();
        return $vouchers;
    }

    /**
     * List vouchers
     * GET /tiffintom/vouchers/id
     */
    public function show(Request $req, $id)
    {
        $vouchers = TiffintomVoucher::find($id);
        if (empty($vouchers)) Utils::error("Voucher not found", 404);
        return $vouchers;
    }

    /**
     * List vouchers
     * POST /tiffintom/vouchers/find/{code}
     */
    public function find(Request $req, $code)
    {
        $customer = Auth::user();
        $cartAmount = TiffintomCart::where('customer_id', $customer->id)->where('order_id', '0')->sum('total_price');

        if ($req->has('delivery_date')) {
            $checkdate = Carbon::parse($req->input('delivery_date'))->timestamp;
            $day_of_name = strtolower(date('l', strtotime($req->input('delivery_date'))));
        } else {
            $checkdate = Carbon::today()->timestamp;
            $day_of_name = strtolower(date('l'));
        }

        $voucher = TiffintomVoucher::whereRaw('(voucher_code LIKE ? OR voucher_code LIKE ? ) ', [$code, str_replace(" ", "", $code)])->where('status', '1')->first();
        if (!empty($voucher)) {

            if ($voucher->voucher_to != null) {
                if (Carbon::parse($voucher->voucher_from)->timestamp > $checkdate || Carbon::parse($voucher->voucher_to)->timestamp < $checkdate) {
                    Utils::error("Voucher expired", 400);
                }
            }

            if ($voucher->day != 'all' && !str_contains($voucher->day, $day_of_name))
                Utils::error("Voucher not applicable for selcted date", 400);

            if ($voucher->minimum_value >= $cartAmount && !Auth::$package_name) {
                Utils::error("Please purchase minimum " . $voucher->minimum_value, 400);
            }

            if ($voucher->type_offer == 'single') {
                $appliedVoucherOrder = TiffintomOrder::where('voucher_code', $voucher->voucher_code)->where('customer_id', $customer->id)->get();
                if (!$appliedVoucherOrder->isEmpty()) {
                    Utils::error("Voucher already used", 400);
                }
            }
        }

        if (empty($voucher)) {

            $voucher = TiffintomUserVoucher::whereRaw('(voucher_code LIKE ? OR voucher_code LIKE ? ) ', [$code, str_replace(" ", "", $code)])->where('status', '1')->where('user_id', $customer->id)->first();

            if (!empty($voucher)) {
                if ($voucher->restaurant_id != $req->input('restaurant_id') && $voucher->restaurant_id != 'all') {
                    Utils::error("Voucher not applicable for this restaurant", 400);
                }

                if ($req->input('order_type') == 'delivery') {
                    if ($voucher->delivery_type != '1') {
                        Utils::error("Voucher not applicable for this order type", 400);
                    }
                }
                if ($req->input('order_type') == 'pickup') {
                    if ($voucher->pickup_type != '1') {
                        Utils::error("Voucher not applicable for this order type", 400);
                    }
                }

                if ($voucher->voucher_to != null) {
                    if (Carbon::parse($voucher->voucher_from)->timestamp > $checkdate || Carbon::parse($voucher->voucher_to)->timestamp < $checkdate) {
                        Utils::error("Voucher expired", 400);
                    }
                }

                if ($voucher->minimum_value >= $cartAmount && !Auth::$package_name) {
                    Utils::error("Please purchase minimum " . $voucher->minimum_value, 400);
                }

                if ($voucher->type_offer == 'single') {
                    $appliedVoucherOrder = TiffintomOrder::where('voucher_code', $voucher->voucher_code)->where('customer_id', $customer->id)->get();
                    if (!$appliedVoucherOrder->isEmpty()) {
                        Utils::error("Voucher already used", 400);
                    }
                }

                if (strtolower($req->header('device-type')) == 'ios' || strtolower($req->header('device-type')) == 'android') {
                    if ($voucher->device_type == 'web') {
                        Utils::error("Voucher not applicable for this device", 400);
                    }
                } else {
                    if ($voucher->device_type == 'app') {
                        Utils::error("Voucher not applicable for this device", 400);
                    }
                }

                if ($voucher->day != 'all' && !str_contains($voucher->day, $day_of_name))
                    Utils::error("Voucher not applicable for selcted date", 400);
            }

            if (empty($voucher)) {

                $voucher = TiffintomRestaurantVoucher::whereRaw('(voucher_code LIKE ? OR voucher_code LIKE ? ) ', [$code, str_replace(" ", "", $code)])->where('status', '1')->first();

                if (!empty($voucher)) {

                    if ($voucher->restaurant_id != $req->input('restaurant_id')) {
                        Utils::error("Voucher not applicable for this restaurant", 400);
                    }

                    if ($req->input('order_type') == 'delivery') {
                        if ($voucher->delivery_type != '1') {
                            Utils::error("Voucher not applicable for this order type", 400);
                        }
                    }
                    if ($req->input('order_type') == 'pickup') {
                        if ($voucher->pickup_type != '1') {
                            Utils::error("Voucher not applicable for this order type", 400);
                        }
                    }

                    if ($voucher->voucher_to != null) {
                        if (Carbon::parse($voucher->voucher_from)->timestamp > $checkdate || Carbon::parse($voucher->voucher_to)->timestamp < $checkdate) {
                            Utils::error("Voucher expired", 400);
                        }
                    }

                    if ($voucher->minimum_value >= $cartAmount && !Auth::$package_name) {
                        Utils::error("Please purchase minimum " . $voucher->minimum_value, 400);
                    }

                    if ($voucher->type_offer == 'single') {
                        $appliedVoucherOrder = TiffintomOrder::where('voucher_code', $voucher->voucher_code)->where('customer_id', $customer->id)->get();
                        if (!$appliedVoucherOrder->isEmpty()) {
                            Utils::error("Voucher already used", 400);
                        }
                    }


                    if (strtolower($req->header('device-type')) == 'ios' || strtolower($req->header('device-type')) == 'android') {
                        if ($voucher->device_type == 'web') {
                            Utils::error("Voucher not applicable for this device", 400);
                        }
                    } else {
                        if ($voucher->device_type == 'app') {
                            Utils::error("Voucher not applicable for this device", 400);
                        }
                    }

                    if ($voucher->day != 'all' && !str_contains($voucher->day, $day_of_name))
                        Utils::error("Voucher not applicable for selcted date", 400);
                }
            }
        }

        if (empty($voucher)) Utils::error("Voucher not found", 404);

        return $voucher;
    }

    /**
     * Update voucher.
     * POST  /tiffintom/vouchers/id
     */
    public function update(Request $req, $id)
    {
        $voucher = TiffintomVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);

        $this->validate($req, [
            'day' => 'required|string',
            'voucher_code' => 'required|string',
            'type_offer' => 'in:single,multiple',
            'offer_mode' => 'in:price,percentage,free_delivery',
            'offer_value' => 'required|numeric|gt:0',
            'minimum_value' => 'required|numeric|gt:0',
            'voucher_from' => 'required|date_format:d-m-Y',
            'voucher_to' => 'required|date_format:d-m-Y|after:voucher_from',
            'status' => 'boolean',
        ]);

        // update voucher
        if ($req->has('day')) $voucher->day = $req->input('day');
        if ($req->has('voucher_code')) $voucher->voucher_code = $req->input('voucher_code');
        if ($req->has('type_offer')) $voucher->type_offer = $req->input('type_offer');
        if ($req->has('offer_mode')) $voucher->offer_mode = $req->input('offer_mode');
        if ($req->has('offer_value')) $voucher->offer_value = $req->input('offer_value');
        if ($req->has('minimum_value')) $voucher->minimum_value = $req->input('minimum_value');
        if ($req->has('voucher_from')) $voucher->voucher_from = $req->input('voucher_from');
        if ($req->has('voucher_to')) $voucher->voucher_to = $req->input('voucher_to');
        $voucher->status = $req->input('status');
        $voucher->save();

        return $voucher;
    }

    /**
     * Delete voucher by id.
     * DELETE /tiffintom/vouchers/{id}
     */
    public function delete(Request $req, $id)
    {
        $voucher = TiffintomVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);
        $voucher->delete();
        return $voucher;
    }

    /**
     * Enable voucher.
     * POST /tiffintom/vouchers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $voucher = TiffintomVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);
        $voucher->status = '1';
        $voucher->save();
        return $voucher;
    }

    /**
     * Disable voucher.
     * POST /tiffintom/vouchers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $voucher = TiffintomVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);
        $voucher->status = '0';
        $voucher->save();
        return $voucher;
    }
}
