<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsTiffintom\TiffintomUser;
use App\ModelsTiffintom\TiffintomWalletHistory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomWalletHistoryController extends Controller
{

    /**
     * List wallet-history
     * GET /tiffintom/wallet-history
     */
    public function index(Request $req)
    {
        $wallet_history = TiffintomWalletHistory::orderBy('id', 'DESC');

        // $customer = Auth::user();
        if ($customer = $req->query('customer_id', false)) {
            $wallet_history->where('customer_id', $customer);
        }

        if ($req->query('nopaginate', false) == '1') {
            $wallet_history = $wallet_history->get();
        } else {
            $wallet_history = $wallet_history->paginate($req->query('per_page', 20));
        }

        if ($wallet_history->isEmpty()) Utils::error('No wallet history found!', 404);

        return $wallet_history;
    }

    /**
     * Create wallet-history.
     * POST /tiffintom/wallet-history
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'purpose' => 'string',
            'transaction_type' => 'string',
            'transaction_details' => 'string',
            'amount' => 'string',
        ]);

        // create wallet_history
        $wallet_history = new TiffintomWalletHistory;
        $wallet_history->fill($req->all());
        $wallet_history->save();

        if ($req->input('customer_id') != '') {
            $user = TiffintomUser::find($req->input('customer_id'));
            $user->wallet_amount = $user->wallet_amount + $req->input('amount');
            $user->save();
        }
        return $wallet_history;
    }

    /**
     * List wallet-history
     * GET /tiffintom/wallet-history/id
     */
    public function show(Request $req, $id)
    {
        $wallet_history = TiffintomWalletHistory::find($id);
        if (empty($wallet_history)) Utils::error("No wallet history found", 404);
        return $wallet_history;
    }

    /**
     * Update wallet-history.
     * POST  /tiffintom/wallet-history/id
     */
    public function update(Request $req, $id)
    {
        $wallet_history = TiffintomWalletHistory::find($id);
        if (empty($wallet_history)) Utils::error('wallet history not found!', 404);

        $this->validate($req, [
            'customer_id' => 'required|exists:tiffintom_connection.users,id',
            'purpose' => 'string',
            'transaction_type' => 'string',
            'transaction_details' => 'string',
            'amount' => 'string',
        ]);

        // update menu
        if ($req->has('customer_id')) $wallet_history->customer_id = $req->input('customer_id');
        if ($req->has('purpose')) $wallet_history->purpose = $req->input('purpose');
        if ($req->has('transaction_type')) $wallet_history->transaction_type = $req->input('transaction_type');
        if ($req->has('transaction_details')) $wallet_history->transaction_details = $req->input('transaction_details');
        if ($req->has('amount')) $wallet_history->amount = $req->input('amount');

        $wallet_history->save();
        return $wallet_history;
    }

    /**
     * Delete wallet-history by id.
     * DELETE /tiffintom/wallet-history/{id}
     */
    public function delete(Request $req, $id)
    {
        $wallet_history = TiffintomWalletHistory::find($id);
        if (empty($wallet_history)) Utils::error('wallet history not found!', 404);
        $wallet_history->delete();

        return $wallet_history;
    }
}
