<?php

namespace App\Http\Controllers;

use App\Tutorial;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class TutorialController extends Controller
{
    /**
     * List tutorials.
     * GET /api/tutorials
     */
    public function index(Request $req)
    {
        $tutorials = Tutorial::orderByRaw('id ASC');

        if ($query = $req->query('query', false)) {
            $tutorials->whereRaw('(title LIKE ? OR  link LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($disabled = $req->query('disabled', false)) {
            $tutorials->where('disabled', $disabled);
        }

        if ($req->query('nopaginate', false) == '1') {
            $tutorials = $tutorials->get();
        } else {
            $tutorials = $tutorials->paginate($req->query('per_page', 20));
        }

        if ($tutorials->isEmpty()) Utils::error('No tutorials found!', 404);

        return $tutorials;
    }

    /**
     * Create tutorial.
     * POST /api/tutorials
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'title' => 'required|string',
            'link' => 'required|string',
            'description' => 'string',
        ]);

        // create tutorial
        $tutorial = new Tutorial;
        $tutorial->fill($req->all());
        $tutorial->save();

        return $tutorial;
    }

    /**
     * Show tutorial.
     * GET /api/tutorials/{id}
     */
    public function show(Request $req, $id)
    {
        $tutorial = Tutorial::find($id);
        if (empty($tutorial)) Utils::error('Tutorial not found!', 404);
        return $tutorial;
    }

    /**
     * Update tutorial.
     * POST /api/tutorials/{id}
     */
    public function update(Request $req, $id)
    {
        $tutorial = Tutorial::find($id);
        if (empty($tutorial)) Utils::error('Tutorial not found!', 404);

        $this->validate($req, [
            'title' => 'required|string',
            'link' => 'required|string',
            'description' => 'string',
        ]);

        // update tutorial
        if ($req->has('title')) $tutorial->title = $req->input('title');
        if ($req->has('link')) $tutorial->link = $req->input('link');
        if ($req->has('description')) $tutorial->description = $req->input('description');

        $tutorial->save();

        return $tutorial;
    }

    /**
     * Delete tutorial by id.
     * DELETE /api/tutorials/{id}
     */
    public function delete(Request $req, $id)
    {
        $tutorial = Tutorial::find($id);
        if (empty($tutorial)) Utils::error('Tutorial not found!', 404);
        $tutorial->delete();
        return $tutorial;
    }

    /**
     * Enable tutorial.
     * POST /epos/tutorials/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $tutorial = Tutorial::find($id);
        if (empty($tutorial)) Utils::error('Tutorial not found!', 404);
        $tutorial->disabled = 0;
        $tutorial->save();
        return $tutorial;
    }

    /**
     * Disable tutorial.
     * POST /epos/tutorials/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $tutorial = Tutorial::find($id);
        if (empty($tutorial)) Utils::error('Tutorial not found!', 404);
        $tutorial->disabled = 1;
        $tutorial->save();
        return $tutorial;
    }
}
