<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\UserContact;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class UserContactController extends Controller
{
    /**
     * List userContacts.
     * GET /api/user-contacts
     */
    public function index(Request $req)
    {
        $userContacts = UserContact::orderByRaw('user_contacts.name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $userContacts->whereRaw('(user_contacts.name LIKE ? OR number LIKE ? )', ["%$query%", "%$query%"]);
        }

        if (Auth::isUser()) {
            $userContacts->where('disabled', 0);
            $userContacts->where('user_id', Auth::id());
        } else {
            $query = $req->query('disabled');
            if ($query != null) {
                $userContacts->where('disabled', $query == 0 ? '0' : '1');
            }
            if ($query = $req->query('user_id', false)) {
                $userContacts->where('user_id', $query);
            }
        }
        if ($query = $req->query('contact_group_id', false)) {
            $contact_group_ids = explode(",", $query);
            $index = 0;
            foreach ($contact_group_ids as $contact_group_id) {
                if ($index > 0) {
                    $userContacts->whereRaw("(OR FIND_IN_SET ($contact_group_id, contact_group_ids))");
                } else {
                    $userContacts->whereRaw("(FIND_IN_SET ($contact_group_id, contact_group_ids))");
                }
                $index = +1;
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $userContacts = $userContacts->get();
        } else {
            $userContacts = $userContacts->paginate($req->query('per_page', 20));
        }

        if ($userContacts->isEmpty()) Utils::error('No userContacts found!', 404);


        return ($userContacts);
    }

    /**
     * Create userContact.
     * POST /api/user-contacts
     */
    public function create(Request $req)
    {
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'number' => 'required|string',
            'type' => 'required|in:Landline,Mobile',
            'user_id' => 'required|exists:users,id',
        ]);

        // create userContact
        $userContact = new UserContact;
        $userContact->fill($req->all());

        $userContact->save();

        return $userContact;
    }

    /**
     * Show userContact.
     * GET /api/user-contacts/{id}
     */
    public function show(Request $req, $id)
    {
        $userContact = UserContact::find($id);
        if (empty($userContact)) Utils::error('UserContact not found!', 404);
        return $userContact;
    }

    /**
     * Update userContact.
     * POST /api/user-contacts/{id}
     */
    public function update(Request $req, $id)
    {
        $userContact = UserContact::find($id);
        if (empty($userContact)) Utils::error('UserContact not found!', 404);
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'name' => 'string|max:255',
            'number' => 'string',
            'type' => 'in:Landline,Mobile',
            'user_id' => 'exists:users,id',
        ]);

        // update userContact
        if ($req->has('name')) $userContact->name = $req->input('name');
        if ($req->has('number')) $userContact->number = $req->input('number');
        if ($req->has('type')) $userContact->type = $req->input('type');
        if ($req->has('user_id')) $userContact->user_id = $req->input('user_id');
        if ($req->has('contact_group_ids')) $userContact->contact_group_ids = $req->input('contact_group_ids');

        $userContact->save();

        return $userContact;
    }


    /**
     * Delete userContact by id.
     * DELETE /api/user-contacts/{id}
     */
    public function delete(Request $req, $id)
    {
        $userContact = UserContact::find($id);
        if (empty($userContact)) Utils::error('UserContact not found!', 404);
        $userContact->delete();
        if ($userContact->image != null) ImageHelper::delete(ImageHelper::$TYPE_BRAND, $userContact->image);
        return $userContact;
    }
}
