<?php

namespace App\Http\Controllers;

use App\Exports\ResellerMultiplePayoutReportExport;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use Lcobucci\JWT\Builder;
use Lcobucci\JWT\Signer\Hmac\Sha256;
use Lcobucci\JWT\Token;
use Illuminate\Support\Facades\Crypt;
use App\User;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Tiffintom\TiffintomMessageController;
use App\ModelsTiffintom\TiffintomMessage;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\ResellerPayoutInvoice;
use App\ResellerPayoutInvoiceGroup;
use App\UserBusiness;
use Carbon\Carbon;
use DateTimeImmutable;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Lcobucci\JWT\Configuration;
use Lcobucci\JWT\Signer\Key\InMemory;
use Lcobucci\JWT\Validation\Constraint\SignedWith;
use Maatwebsite\Excel\Facades\Excel;

class UserController extends Controller
{
    /**
     * Show admin.
     * GET /api/users/me
     */
    public function index(Request $req)
    {
        $users = User::with(['bankDetails'])->latest();

        if ($query = $req->query('query', false)) {
            $users->whereRaw('(first_name LIKE ? OR contact_emails LIKE ? )', ["%$query%", "%$query%"]);
        }
        if (Auth::isUser()) {
            $users->where('disabled', 0);
        } else {
            $query = $req->query('disabled');
            if ($query != null) {
                $users->where('disabled', $query == 0 ? '0' : '1');
            }
        }
        if (Auth::isUser()) {
            if (Auth::user()->reseller) {
                $loggedInUser = Auth::user();
                $users->whereRaw('(id IN (SELECT user_id from user_businesses WHERE business_id IN (SELECT business_id from user_businesses where user_id = ?)))', [$loggedInUser->id]);
            } else {
                Utils::error('You not authorised to access!', 401, 401);
            }
        } else {
            $query = $req->query('reseller');
            if ($query != null) {
                $users->where('reseller', $query == 0 ? '0' : '1');
            }
        }

        if ($req->query('nopaginate', false) == '1') {
            $users = $users->get();
        } else {
            $users = $users->paginate($req->query('per_page', 20));
        }

        if ($users->isEmpty()) Utils::error('No users found!', 404);
        return $users;
    }

    /**
     * List users.
     * GET /api/users/mini
     */
    public function mini(Request $req)
    {
        $users = User::select('id', 'first_name', 'last_name', 'reseller', 'login_email')->orderByRaw('first_name ASC');
        $users->where('disabled', 0);
        if ($query = $req->query('query', false)) {
            $users->whereRaw('(first_name LIKE ? OR contact_emails LIKE ? )', ["%$query%", "%$query%"]);
        }
        if ($reseller = $req->query('reseller', false)) {
            $users->where('reseller', '1');
        }
        $users = $users->get();
        if ($users->isEmpty()) Utils::error('No users found!', 404);

        return ($users);
    }

    public function create(Request $req)
    {
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }
        $this->validate($req, [
            'first_name' => 'required|string|max:255',
            'last_name' => 'string|max:255',
            'image' => 'image',
            'parent_id' => 'exist:users,id',
            'dob' => 'date_format:Y-m-d',
            'contact_emails' => 'string',
            'contact_numbers' => 'string',
            'contact_type' => 'in:mobile,landline,other',
            'door_no' => 'string',
            'city' => 'string',
            'state' => 'string',
            'postcode' => 'string',
            'country_id' => 'string',
            'residential_address' => 'string',
            'login_email' => 'required|email|unique:users,login_email',
            'password' => 'required|string',
            'reseller' => 'boolean',
            // 'commission_type' => 'required_if:reseller,1|in:flat,percentage',
            // 'commission' => 'required_if:reseller,1|min:0',

            'payout_comment' => 'string',
            'payout_mode' => 'in:Auto,Manual',
            'payout_method' => 'in:Bank,Paypal,Cash',
            'archived_payout_mode' => 'in:Auto,Manual',
            'archived_payout_method' => 'in:Bank,Paypal,Cash',
        ]);

        // create user
        $user = new User;
        $user->fill($req->all());
        $user->password = Hash::make($user->password);
        if (Auth::isUser()) {
            $user->parent_id = Auth::id();
        }
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_USER);
            $user->image = $fileName;
        }
        $user->save();
        return $user;
    }

    public function direct_create(Request $req)
    {
        $this->validate($req, [
            'first_name' => 'required|string|max:255',
            'last_name' => 'string|max:255',
            'dob' => 'date_format:Y-m-d',
            'contact_emails' => 'string',
            'contact_numbers' => 'string',
            'contact_type' => 'in:mobile,landline,other',
            'door_no' => 'string',
            'city' => 'string',
            'state' => 'string',
            'postcode' => 'string',
            'country_id' => 'string',
            'residential_address' => 'string',
            // 'login_email' => 'required|email|unique:users,login_email',
            'login_email' =>  ['required', Rule::unique('users')->where(function ($query) use ($req) {
                return $query->where('login_email', $req->input('login_email'))->where('disabled', 0);
            })],
            'password' => 'required|string',
            'reseller' => 'boolean',
        ]);
        $validate = (new TiffintomMessageController())->validate_number($req->input('contact_numbers'));

        // create user
        $otp = rand(100000, 999999);

        $user = new User;
        $user->fill($req->all());
        $user->password = Hash::make($user->password);
        if ($validate > 0) {
            $checkMessages =  TiffintomMessage::find($validate['id']);
            $user->contact_type = $checkMessages->device_type;
            $user->mobile = $checkMessages->mobile;
        }
        $user->disabled = 1;
        $user->otp = $otp;
        $user->country_id = $req->input('country_id');
        $user->save();

        if ($user->mobile == '1') {
            $message = 'Your one-time password is ' . $otp . ' please use this to verify your phone number.';
            (new OtpHelper())->sendMessage($message, $req->input('contact_numbers'), 'otp_service');

            $this->otpMail($user->id);
        }

        return $user;
    }

    public function direct_resend_otp(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        if ($user->mobile == '1') {
            $message = 'Your one-time password is ' . $user->otp . ' please use this to verify your phone number.';
            (new OtpHelper())->sendMessage($message, $user->contact_numbers, 'otp_service');
            $this->otpMail($user->id);
        }

        if ($req->has('contact_numbers')) $user->contact_numbers = $req->input('contact_numbers');
        if ($req->has('contact_emails')) $user->contact_emails = $req->input('contact_emails');
        if ($req->has('login_email')) $user->login_email = $req->input('login_email');
        $user->save();

        return $user;
    }

    /**
     * OTP Match and update user.
     * POST /api/users/{id}/direct-update
     */
    public function direct_update(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'otp' => 'required|string|max:255',
        ]);

        // update user
        if ($user->otp != $req->input('otp')) {
            Utils::error('One time password not matched,please try again!', 404);
        }
        if ($req->has('otp')) $user->otp = $req->input('otp');
        $user->disabled = 0;
        $user->save();

        // return $user;
        $tokens = $this->getNewTokens($user->id);

        return response()->json($user, 200, [
            'x-token' => (string) $tokens['aToken'],
            'x-refresh-token' => (string) $tokens['rToken'],
        ]);
    }

    public function show(Request $req, $id)
    {
        if ($id == 'me') $user = Auth::user();
        else $user = User::with('bankDetails')->find($id);
        $user->append('default_bank_detail');
        $user->append('default_paypal_detail');
        if (empty($user)) Utils::error('User not found!', 404);
        return $user;
    }

    /**
     * Login admin.
     * POST /api/users/login
     */
    public function login(Request $req)
    {
        $this->validate($req, [
            'login_email' => 'required|string|max:255|exists:users',
            'password' => 'required|string',
        ]);

        $admin = User::where('login_email', $req->input('login_email'))->first();

        if (!Hash::check($req->input('password'), $admin->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $tokens = $this->getNewTokens($admin->id);

            return response()->json($admin, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Update admin password.
     * POST /api/users/me/password
     */
    public function password(Request $req)
    {
        $admin = Auth::user();
        if (empty($admin)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'password' => 'required|string',
            'new_password' => 'required|string',
        ]);


        if (!Hash::check($req->input('password'), $admin->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $admin->password = Hash::make($req->input('new_password'));
            $admin->save();

            $tokens = $this->getNewTokens($admin->id);
            return response()->json($admin, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Token helper to get new tokens.
     */
    public static function getNewTokens($userId, $role = 'user')
    {
        $user = User::find($userId);
        if (empty($user)) return null;
        // if ($user->disabled == 1) Utils::error('Your account is disabled', 401, 401);

        $now   = new DateTimeImmutable();
        $aTokenExp = $now->modify('+360 hours');
        $rTokenExp = (new DateTimeImmutable())->modify('+360 hours'); // 3 days mate token valid rese

        $aTokenConfig = Auth::getConfig(env('X_TOKEN_KEY'));
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $user->password);
        $aBuilder = $aTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($aTokenExp)
            ->withClaim('id', $user->id)
            ->withClaim('role', $role);
        $aToken = $aBuilder->getToken($aTokenConfig->signer(), $aTokenConfig->signingKey())->toString();


        $rBuilder = $rTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($rTokenExp)
            ->withClaim('id', $user->id)
            ->withClaim('role', $role);
        $rToken = $rBuilder->getToken($rTokenConfig->signer(), $rTokenConfig->signingKey())->toString();

        return array('aToken' => Crypt::encrypt((string) $aToken), 'rToken' => Crypt::encrypt((string) $rToken));
    }

    public static function getConfig($key): Configuration
    {
        $config = Configuration::forSymmetricSigner(new Sha256(), InMemory::plainText($key));
        $config->setValidationConstraints(new SignedWith($config->signer(), $config->signingKey()));
        return $config;
    }

    /**
     * Token helper to get verify tokens.
     */
    public static function verifyRefreshToken(Token $rToken)
    {
        $user = User::find($rToken->claims()->get('id'));
        if (empty($user)) return false;
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $user->password);
        $constraints = $rTokenConfig->validationConstraints();
        $rTokenConfig->validator()->assert($rToken, ...$constraints);
        if (!$rTokenConfig->validator()->validate($rToken, ...$constraints)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Update user.
     * POST /api/users/{id}
     */
    public function update(Request $req, $id)
    {
        if ($id == 'me') $user = Auth::user();
        else $user = User::find($id);

        if (empty($user)) Utils::error('User not found!', 404);
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        if (Auth::isAdmin()) {
            $this->validate($req, [
                'first_name' => 'string|max:255',
                'last_name' => 'string|max:255',
                'image' => 'image',
                'parent_id' => 'exist:users,id',
                'dob' => 'date_format:Y-m-d',
                'contact_emails' => 'string',
                'contact_numbers' => 'string',
                'contact_type' => 'in:mobile,landline,other',
                'door_no' => 'string',
                'city' => 'string',
                'state' => 'string',
                'postcode' => 'string',
                'country_id' => 'string',
                'residential_address' => 'string',
                'login_email' => ['email', Rule::unique('users', 'login_email')->ignore($user->id)],
                'password' => 'string',
                'reseller' => 'boolean',
                'payout_comment' => 'string',
                'payout_mode' => 'in:Auto,Manual',
                'payout_method' => 'in:Bank,Paypal,Cash',
                'archived_payout_mode' => 'in:Auto,Manual',
                'archived_payout_method' => 'in:Bank,Paypal,Cash',
                'allow_3rd_party_use' => 'required|boolean',
                // 'secret_key' => 'required_if:allow_3rd_party_use,1|string',
                // 'initial_vector' => 'required_if:allow_3rd_party_use,1|string',
            ]);
        } else if (Auth::isUser()) {
            $this->validate($req, [
                'first_name' => 'string|max:255',
            ]);
        }

        // update user
        if ($req->has('first_name')) $user->first_name = $req->input('first_name');
        if (Auth::isAdmin()) {
            if ($req->has('first_name')) $user->first_name = $req->input('first_name');
            if ($req->has('last_name')) $user->last_name = $req->input('last_name');
            if ($req->has('parent_id')) $user->parent_id = $req->input('parent_id');
            if ($req->has('dob')) $user->dob = $req->input('dob');
            if ($req->has('contact_emails')) $user->contact_emails = $req->input('contact_emails');
            if ($req->has('contact_numbers')) $user->contact_numbers = $req->input('contact_numbers');
            if ($req->has('contact_type')) $user->contact_type = $req->input('contact_type');
            if ($req->has('door_no')) $user->door_no = $req->input('door_no');
            if ($req->has('city')) $user->city = $req->input('city');
            if ($req->has('state')) $user->state = $req->input('state');
            if ($req->has('postcode')) $user->postcode = $req->input('postcode');
            if ($req->has('country_id')) $user->country_id = $req->input('country_id');
            if ($req->has('residential_address')) $user->residential_address = $req->input('residential_address');
            if ($req->has('login_email')) $user->login_email = $req->input('login_email');
            if ($req->has('password')) $user->password = $req->input('password');
            if ($req->has('reseller')) $user->reseller = $req->input('reseller');
            if ($req->has('commission_type')) $user->commission_type = $req->input('commission_type');

            if ($req->has('payout_mode')) $user->payout_mode = $req->input('payout_mode');
            if ($req->has('payout_method')) $user->payout_method = $req->input('payout_method');
            if ($req->has('archived_payout_mode')) $user->archived_payout_mode = $req->input('archived_payout_mode');
            if ($req->has('archived_payout_method')) $user->archived_payout_method = $req->input('archived_payout_method');
            if ($req->has('payout_comment')) $user->payout_comment = $req->input('payout_comment');

            $user->allow_3rd_party_use = $req->input('allow_3rd_party_use');

            if ($user->allow_3rd_party_use) {
                if ($user->secret_key == null || $user->initial_vector == null) {
                    $keys = $this->generateEncryptionKeys($user->password);
                    $user->secret_key = $keys['secret_key'];
                    $user->initial_vector = $keys['initial_vector'];
                }
            } else {
                $user->secret_key = null;
                $user->initial_vector = null;
            }
            // if ($user->reseller == 1) $user->commission = $req->input('commission');
            if ($req->has('disabled')) $user->disabled = $req->input('disabled');
            if ($req->has('permissions')) $user->permissions = $req->input('permissions');

            if (Auth::isAdmin()) {
                if ($req->has('password')) $user->password = Hash::make($req->input('password'));
            }
            if ($req->has('image')) {
                $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_USER);
                if ($user->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_USER, $user->image);
                $user->image = $fileName;
            }
        }
        $user->save();

        return $user;
    }

    public function generateEncryptionKeys($password)
    {
        $data = [];
        $data['secret_key'] = substr($password, 7, 32);
        $data['initial_vector'] = substr($password, -16);
        return $data;
    }

    /**
     * Update user.
     * POST /api/users/{id}/all-update
     */
    public function merchant_user_update(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }
        $this->validate($req, [
            'first_name' => 'string|max:255',
            'last_name' => 'string|max:255',
            'image' => 'image',
            'parent_id' => 'exist:users,id',
            'dob' => 'date_format:Y-m-d',
            'contact_emails' => 'string',
            'contact_numbers' => 'string',
            'contact_type' => 'string',
            'door_no' => 'string',
            'city' => 'string',
            'state' => 'string',
            'postcode' => 'string',
            'country_id' => 'string',
            'residential_address' => 'string',
            'login_email' => ['email', Rule::unique('users', 'login_email')->ignore($user->id)],
            'password' => 'string',
            'reseller' => 'boolean',
            'payout_comment' => 'string',
            'payout_mode' => 'in:Auto,Manual',
            'payout_method' => 'in:Bank,Paypal,Cash',
            'archived_payout_mode' => 'in:Auto,Manual',
            'archived_payout_method' => 'in:Bank,Paypal,Cash',
        ]);

        // update user
        if ($req->has('first_name')) $user->first_name = $req->input('first_name');
        if ($req->has('last_name')) $user->last_name = $req->input('last_name');
        if ($req->has('parent_id')) $user->parent_id = $req->input('parent_id');
        if ($req->has('dob')) $user->dob = $req->input('dob');
        if ($req->has('contact_emails')) $user->contact_emails = $req->input('contact_emails');
        if ($req->has('contact_numbers')) $user->contact_numbers = $req->input('contact_numbers');
        if ($req->has('contact_type')) $user->contact_type = $req->input('contact_type');
        if ($req->has('door_no')) $user->door_no = $req->input('door_no');
        if ($req->has('city')) $user->city = $req->input('city');
        if ($req->has('state')) $user->state = $req->input('state');
        if ($req->has('postcode')) $user->postcode = $req->input('postcode');
        if ($req->has('country_id')) $user->country_id = $req->input('country_id');
        if ($req->has('residential_address')) $user->residential_address = $req->input('residential_address');
        if ($req->has('login_email')) $user->login_email = $req->input('login_email');
        // if ($req->has('password')) $user->password = $req->input('password');
        if ($req->has('reseller')) $user->reseller = $req->input('reseller');
        if ($req->has('commission_type')) $user->commission_type = $req->input('commission_type');
        if ($req->has('payout_mode')) $user->payout_mode = $req->input('payout_mode');
        if ($req->has('payout_method')) $user->payout_method = $req->input('payout_method');
        if ($req->has('archived_payout_mode')) $user->archived_payout_mode = $req->input('archived_payout_mode');
        if ($req->has('archived_payout_method')) $user->archived_payout_method = $req->input('archived_payout_method');
        if ($req->has('payout_comment')) $user->payout_comment = $req->input('payout_comment');
        if ($req->has('disabled')) $user->disabled = $req->input('disabled');
        if ($req->has('permissions')) $user->permissions = $req->input('permissions');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_USER);
            if ($user->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_USER, $user->image);
            $user->image = $fileName;
        }
        $user->save();

        return $user;
    }

    /**
     * OTP Send user.
     * POST /api/users/{id}/send-otp-both
     */
    public function send_otp_both(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'verify_type' => 'required|string|max:255',
            'verify_field' => 'required|string|max:255',
        ]);

        $otp = rand(100000, 999999);
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $user->otp = $otp;
        $user->save();

        if ($req->input('verify_type') == 'phone') {
            if ($user->mobile == '1') {
                $message = 'Your one-time password is ' . $otp . ' please use this to verify your phone number on Ubsidi.';
                (new OtpHelper())->sendMessage($message, $req->input('verify_field'), 'otp_service');
            }
        }
        if ($req->input('verify_type') == 'email') {
            $this->verifyMail($user->id, $req->input('verify_field'));
        }

        return ['message' => 'otp send successfully!'];
    }

    /**
     * OTP Match and update user.
     * POST /api/users/{id}/verify-otp-both
     */
    public function verify_otp(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'verify_type' => 'required|string|max:255',
            'verify_field' => 'required|string|max:255',
            'otp' => 'required|string|max:6',
        ]);

        if ($req->input('otp') !=  $user->otp) {
            return Utils::error('otp not matched.please enter correct otp.', 400);
        }

        // update user
        if ($req->input('verify_type') == 'phone') {
            $user->contact_numbers = $req->input('verify_field');
        }
        if ($req->input('verify_type') == 'email') {
            $user->contact_emails = $req->input('verify_field');
        }

        $user->save();

        return $user;
    }

    /**
     * Password Check.
     * POST /api/users/{id}/password-check
     */
    public function password_check(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'password' => 'required|string',
        ]);

        if (!Hash::check($req->input('password'), $user->password)) {
            return Utils::error('Invalid password.', 400);
        }

        return ['message' => 'Password matched successfully!'];
    }

    /**
     * Update Password user.
     * POST /api/users/{id}/change-password
     */
    public function chage_password(Request $req, $id)
    {
        $user = User::find($id);

        if (empty($user)) Utils::error('User not found!', 404);

        $this->validate($req, [
            'password' => 'required',
        ]);

        // update user
        if ($req->has('password')) $user->password = Hash::make($req->input('password'));
        $user->save();

        return $user;
    }

    /**
     * POST /api/users/forget-password
     */
    public function forgetPassword(Request $req)
    {
        $user = User::where('login_email', $req->input('login_email'))->first();
        if (empty($user)) Utils::error('User not found!', 404);

        $input      =   "1234567890abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
        $password   =   substr(str_shuffle($input), 0, 10);
        $user->password = Hash::make($password);
        $user->save();

        $this->forgotMail($user->id, $password);

        return $user;
    }

    /**
     * Enable user.
     * POST /api/users/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $user->disabled = 0;
        $user->save();
        return $user;
    }

    /**
     * Disable user.
     * POST /api/users/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $user->disabled = 1;
        $user->save();
        return $user;
    }

    /**
     * Delete user by id.
     * DELETE /api/users/{id}
     */
    public function delete(Request $req, $id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error('User not found!', 404);
        $user->delete();
        if ($user->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_USER, $user->image);
        return $user;
    }

    /*
     |-------------------------------------------
     | User businesses
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/businesses
    public function businesses_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new UserBusinessController())->index($req);
    }

    // POST /api/users/{user_id}/businesses
    public function businesses_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new UserBusinessController())->create($req);
    }

    // POST /api/users/{user_id}/businesses/{id}
    public function businesses_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new UserBusinessController())->update($req, $id);
    }

    // DELETE /api/users/{user_id}/businesses/{id}
    public function businesses_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new UserBusinessController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | User Id Proofs
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/proofs
    public function proofs_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new IdProofController())->index($req);
    }

    // POST /api/users/{user_id}/proofs
    public function proofs_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new IdProofController())->create($req);
    }

    // POST /api/users/{user_id}/proofs/{id}
    public function proofs_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new IdProofController())->update($req, $id);
    }

    // DELETE /api/users/{user_id}/proofs/{id}
    public function proofs_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new IdProofController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | User bank details
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/bank-details
    public function bank_details_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new BankDetailController())->index($req);
    }

    // POST /api/users/{user_id}/bank-details
    public function bank_details_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new BankDetailController())->create($req);
    }

    // POST /api/users/{user_id}/bank-details/{id}
    public function bank_details_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new BankDetailController())->update($req, $id);
    }

    // POST /api/users/{user_id}/bank-details/{id}/default
    public function bank_details_default(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new BankDetailController())->default($req, $id);
    }
    // DELETE /api/users/{user_id}/bank-details/{id}
    public function bank_details_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new BankDetailController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | User paypal details
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/paypal-details
    public function paypal_details_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PaypalDetailController())->index($req);
    }

    // POST /api/users/{user_id}/paypal-details
    public function paypal_details_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PaypalDetailController())->create($req);
    }

    // POST /api/users/{user_id}/paypal-details/{id}
    public function paypal_details_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PaypalDetailController())->update($req, $id);
    }

    // POST /api/users/{user_id}/paypal-details/{id}/default
    public function paypal_details_default(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PaypalDetailController())->default($req, $id);
    }
    // DELETE /api/users/{user_id}/paypal-details/{id}
    public function paypal_details_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PaypalDetailController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Reseller payout-charges
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/payout-charges
    public function payout_charges_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutChargeController())->index($req);
    }

    // POST /api/users/{user_id}/payout-charges
    public function payout_charges_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutChargeController())->create($req);
    }

    // POST /api/users/{user_id}/payout-charges/{id}
    public function payout_charges_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutChargeController())->update($req, $id);
    }

    // DELETE /api/users/{user_id}/payout-charges/{id}
    public function payout_charges_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutChargeController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | User misc charges
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/misc-charges
    public function misc_charges_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerMiscChargeController())->index($req);
    }

    // POST /api/users/{user_id}/misc-charges
    public function misc_charges_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerMiscChargeController())->create($req);
    }

    // POST /api/users/{user_id}/misc-charges/{id}
    public function misc_charges_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerMiscChargeController())->update($req, $id);
    }

    // DELETE /api/users/{user_id}/misc-charges/{id}
    public function misc_charges_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerMiscChargeController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Reseller payout invoices
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/payout-invoices
    public function payout_invoice_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutInvoiceController())->index($req);
    }

    // POST /api/users/{user_id}/payout-invoices
    public function payout_invoice_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutInvoiceController())->create($req);
    }

    // GET /api/users/{user_id}/payout-invoices/{id}
    public function payout_invoice_show(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutInvoiceController())->show($req, $id);
    }

    // GET /api/users/{user_id}/payout-invoices/groups/
    public function payout_invoice_groups(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutInvoiceController())->groups($req);
    }
    // GET /api/users/{user_id}/payout-invoices/groups/{id}
    public function payout_invoice_group_show(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutInvoiceController())->group_show($req, $id);
    }

    // GET /api/users/{user_id}/payout-invoices/{id}/download
    public function payout_invoice_download(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutInvoiceController())->download($req, $id);
    }
    // DELETE /api/users/{user_id}/payout-invoices/{id}
    public function payout_invoice_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutInvoiceController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | User payout_requests
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/payout-requests
    public function payout_requests_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PayoutRequestController())->index($req);
    }
    // POST /api/users/{user_id}/payout-requests
    public function payout_requests_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PayoutRequestController())->create($req);
    }
    // POST /api/users/{user_id}/payout-requests/{id}
    public function payout_requests_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PayoutRequestController())->update($req, $id);
    }
    // DELETE /api/users/{user_id}/payout-requests/{id}
    public function payout_requests_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PayoutRequestController())->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | Reseller payout_requests
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/reseller-payout-requests
    public function reseller_payout_requests_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutRequestController)->index($req);
    }

    // POST /api/users/{user_id}/reseller-payout-requests
    public function reseller_payout_requests_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutRequestController)->create($req);
    }

    // POST /api/users/{user_id}/reseller-payout-requests
    public function reseller_payout_requests_export(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutRequestController)->export($req);
    }

    // GET /api/users/{user_id}/reseller-payout-requests
    public function reseller_payout_requests_show(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutRequestController)->show($req, $id);
    }

    // POST /api/users/{user_id}/reseller-payout-requests
    public function reseller_payout_requests_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutRequestController)->update($req, $id);
    }

    // GET /api/users/{user_id}/reseller-payout-requests
    public function reseller_payout_requests_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new ResellerPayoutRequestController)->delete($req, $id);
    }

    /*
     |-------------------------------------------
     | User passbook
     |-------------------------------------------
     */

    // GET /api/users/{user_id}/passbook
    public function passbook_index(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PassbookController())->index($req);
    }
    // POST /api/users/{user_id}/passbook
    public function passbook_create(Request $req, $user_id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PassbookController())->create($req);
    }
    // POST /api/users/{user_id}/passbook/{id}
    public function passbook_update(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PassbookController())->update($req, $id);
    }
    // DELETE /api/users/{user_id}/passbook/{id}
    public function passbook_delete(Request $req, $user_id, $id)
    {
        $req->merge(['user_id' => $user_id]);
        return (new PassbookController())->delete($req, $id);
    }

    public function otpMail($id)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $user->first_name . ',<br>
                             Your one-time password is ' . $user->otp . ' please use this to verify your Email.<br>
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->contact_emails,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function passwordMail($id, $password)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $user->first_name . ',<br>
                             This is your password: ' . $password . ', Please use this credetial at login time.<br>
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->contact_emails,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function forgotMail($id, $password)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#f1e0c6">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/logo-home.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                Hey ' . $user->first_name . ',<br>
                                Forgot your password? Dont worry – happens to the best of us.<br>
                                Simply click the button below to reset your password.<br>
                                <center><a href="#" style="background-color: #f1e0c6;border-radius: 5px;color: #000;padding: .5em;text-decoration: none;">Reset Password</a></center>
                                <br>
                                If you forgot the email you use with UBSIDI Epos, its ' . $user->username . '. Your Password Is ' . $password . ' <br>
                                Hope that is sorted everything out.
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://play.google.com/store/apps/details?id=com.tiffintom" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8" target="_blank">
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://in.linkedin.com/company/tiffin-tom-limited/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#f1e0c6"><div style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Forgot Password',
            'mail_type' => $siteSettings->mail_option,
            'to' => $user->username,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }

    public function verifyMail($id, $to_email)
    {
        $user = User::find($id);
        if (empty($user)) Utils::error("No user found", 404);

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        $emailinfo = '<table>
        <tr>
            <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                    <tr>
                        <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#239f96">
                            <div style="display:inline-block; vertical-align:middle; text-align:center">
                                <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:300px;">
                                    <tr>
                                        <td align="left" valign="top" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-white.png">
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </td>
                    </tr>';

        $emailinfo .= ' <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
           <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
              <tr>
                 <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:670px;">
                       
                        <tr>
                          <td align="left" style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                             <p style="font-size: 16px; font-weight: 400; line-height: 24px; color: #777777;">
                                
                             Hey ' . $user->first_name . ',<br>
                             Your one-time password is ' . $user->otp . ' please use this to verify your Email.<br>
                             </p>
                          </td>
                       </tr>
                    </table>
                 </td>
              </tr>
             
             </table>
        </td>
     </tr>';
        $emailinfo .= '  <tr>   
               <td bgcolor="#fff" >
                   <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                       <tr>
                           <td style="padding: 30px 30px; text-align: right;">
                                   <table align="left" style="text-align: left;">
                                       <tr>
                                           <td>
                                                  <p style="color:#000;"> Get The App : </p> 
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/playstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                           <td>
                                               <a>
                                               <img src="' . env('APP_URL') . 'images/appstore.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                           </td>
                                               <td width="10">&nbsp;</td>
                                       </tr>
                                       
                                   </table>
                                   <table align="right" style="text-align: right;">
                                   <tr>
                                       <td>
                                               <p style="color:#000;">Follow Us : </p>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/facebook.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/twitter.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td>
                                           <td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/linkedin.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                       <td>
                                           <a>
                                           <img src="' . env('APP_URL') . 'images/Instragram.png" style="margin:0; padding:0; border:none; display:block;" border="0" alt=""></a>
                                       </td><td width="10">&nbsp;</td>
                                   </tr>
                               </table>
                           </td>
                       </tr>
                   </table>
               </td>
            </tr>
            <tr>
                <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#239f96"><div style="color:#fff;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">Copyrights @ ' . date('Y') . '</div></td>
            </tr>
        </table>
        </td>
        </tr>
        </table>';

        // User side mail
        $email = array(
            'body' => $emailinfo,
            'subject' => 'Email Verification',
            'mail_type' => $siteSettings->mail_option,
            'to' => $to_email,
            'from' => $siteSettings->admin_email,
        );
        (new MailHelper())->sendMail($email);
    }
}
