<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\WeeklyCharge;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;

class WeeklyChargeController extends Controller
{
    /**
     * List weekly-charges.
     * GET /api/weekly-charges
     */
    public function index(Request $req)
    {
        $weekly_charges = WeeklyCharge::selectRaw('weekly_charges.*,businesses.name as business_name')->leftJoin('businesses', 'businesses.id', '=', 'business_id')->orderByRaw('weekly_charges.created_at DESC');

        if ($req->has('from_date')) {
            $from_date = $req->input('from_date');
            $to_date = $req->input('to_date');
            $weekly_charges = $weekly_charges->where('weekly_charges.start_date', '>=', $from_date);
            $weekly_charges = $weekly_charges->where('weekly_charges.end_date', '<=', $to_date);
        }
        if ($query = $req->query('query', false)) {
            $weekly_charges->whereRaw('(businesses.name  LIKE ? OR weekly_charges.amount = ?)', ["%$query%", "%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $weekly_charges = $weekly_charges->get();
        } else {
            $weekly_charges = $weekly_charges->paginate($req->query('per_page', 20));
        }

        if ($weekly_charges->isEmpty()) Utils::error('No weekly charge found!', 404);

        return ($weekly_charges);
    }

    /**
     * Create weekly-charges.
     * POST /api/weekly-charges
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'total' => 'numeric',
            'start_date' => 'date',
            'end_date' => 'date',
            'debited' => 'boolean',
        ]);

        // create weekly-charge
        $weekly_charge = new WeeklyCharge;
        $weekly_charge->fill($req->all());
        $weekly_charge->save();

        return $weekly_charge;
    }

    /**
     * Show weekly-charges.
     * GET /api/weekly-charges/{id}
     */
    public function show(Request $req, $id)
    {
        $weekly_charge = WeeklyCharge::find($id);
        if (empty($weekly_charge)) Utils::error('Weekly charge not found!', 404);
        return $weekly_charge;
    }

    /**
     * Update weekly-charges.
     * POST /api/weekly-charges/{id}
     */
    public function update(Request $req, $id)
    {
        $weekly_charge = WeeklyCharge::find($id);
        if (empty($weekly_charge)) Utils::error('WeeklyCharge not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'total' => 'numeric',
            'start_date' => 'date',
            'end_date' => 'date',
            'debited' => 'boolean',
        ]);

        // update weekly-charge
        if ($req->has('business_id')) $weekly_charge->business_id = $req->input('business_id');
        if ($req->has('total')) $weekly_charge->total = $req->input('total');
        if ($req->has('start_date')) $weekly_charge->start_date = $req->input('start_date');
        if ($req->has('end_date')) $weekly_charge->end_date = $req->input('end_date');
        $weekly_charge->debited = $req->input('debited');

        $weekly_charge->save();

        return $weekly_charge;
    }


    /**
     * Delete weekly-charges by id.
     * DELETE /api/weekly-charges/{id}
     */
    public function delete(Request $req, $id)
    {
        $weekly_charge = WeeklyCharge::find($id);
        if (empty($weekly_charge)) Utils::error('Weekly charge not found!', 404);
        $weekly_charge->delete();
        return $weekly_charge;
    }
}
