<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Log;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\User;

class CorsMiddleware
{
    protected $auth, $unrestrictedUrl;

    public function __construct(Auth $auth)
    {
        $this->auth = $auth;
        $this->unrestrictedUrl = [];
    }

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  \Closure $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        $corsWhitelist = [
            '*',
            'http://localhost:4200',
            'http://192.168.1.13:8300',
            'http://192.168.1.13:8089',
            'http://192.168.1.13:8090',
            'http://192.168.1.13:8091',
            'http://www.testrms.tiffintom.com',
            'https://www.testrms.tiffintom.com',
            'http://www.ubsidi.com.bd',
            'https://www.ubsidi.com.bd',
            'https://cust.smartmps.co.uk',
        ];

        if (in_array("*", $corsWhitelist)) {
            $origin = "*";
        } else {
            if (!isset($_SERVER['HTTP_ORIGIN'])) {
                $origin = "*";
            } else {
                $origin = $_SERVER['HTTP_ORIGIN'];
            }
        }


        if (in_array($origin, $corsWhitelist)) {
            $headers = [
                'Access-Control-Allow-Origin' => "$origin",
                'Access-Control-Allow-Methods' => 'POST, GET, OPTIONS, PUT, DELETE',
                'Access-Control-Allow-Credentials' => 'true',
                'Access-Control-Max-Age' => '86400',
                'Access-Control-Allow-Headers' => 'Content-Type,Authorization,X-Requested-With,x-refresh-token,x-token,business_id,business-id,package-name,restaurant-id',
                'Access-Control-Expose-Headers' => 'x-refresh-token,x-token'
            ];
        } else {
            // $HTTP_USER_AGENT = $_SERVER['HTTP_USER_AGENT'];
            // $REQUEST_URI = $_SERVER['REQUEST_URI'];
            // if (str_contains($HTTP_USER_AGENT, 'okhttp') || str_contains($HTTP_USER_AGENT, 'CFNetwork')) {
            $headers = [
                'Access-Control-Allow-Origin' => 'Invalid domain',
                'Access-Control-Allow-Methods' => 'POST, GET, OPTIONS, PUT, DELETE',
                'Access-Control-Allow-Credentials' => 'true',
                'Access-Control-Max-Age' => '86400',
                'Access-Control-Allow-Headers' => 'Content-Type,Authorization,X-Requested-With,x-refresh-token,x-token,business_id,business-id,package-name',
                'Access-Control-Expose-Headers' => 'x-refresh-token,x-token'
            ];
            // } else {
            //     Utils::error('Access Forbidden !', 403);
            // }
        }

        if ($request->isMethod('OPTIONS')) {
            return response()->json('{"method":"OPTIONS"}', 200, $headers);
        }

        $response = $next($request);
        foreach ($headers as $key => $value) {
            $response->headers->set($key, $value);
        }
        $path = $request->getPathInfo();
        $explode = explode('/', $path);
        $epos_payment = $explode[1] ?? $explode[0];
        $params = $explode[6] ?? "";
        if (!isset($params) || (isset($params) && $params != 'download')) {
            // && ($request->segment(1) != 'api' && $request->segment(2) != 'ubsidi')
            if ($epos_payment != 'online-desk-integration' && $epos_payment != 'sitemap' && $epos_payment != 'epos-payment' && $epos_payment != 'epos-payment-success' && $epos_payment != 'epos-payment-failed' && $epos_payment != 'invoice-payment' && $epos_payment != 'invoice-payment-success' && $epos_payment != 'invoice-payment-failed' && $epos_payment != 'invoice-send-recepit' && !$request->hasHeader('link') && !$request->has('format') && $epos_payment != 'stripe-custom-payment' && $epos_payment != 'create-payment-intent') {
                if ($response->getStatusCode() == 200) {
                    $data = $this->opensslEncrypt($response, $request);
                    $response->setContent($data);
                }
            }
        }

        return $response;
    }

    public function opensslEncrypt($response, $req)
    {
        $path = $req->getPathInfo();
        $explode = explode('/', $path);
        $pg = $explode[1] ?? $explode[0];
        $role = Auth::role();
        if ($role != 'admin' && $role != null && $pg == 'pg') {
            $user = Auth::user();
            if ($user->allow_3rd_party_use) {
                $mode = 'AES-256-CBC';
                $data = openssl_encrypt($response->getContent(), $mode, $user->secret_key, OPENSSL_RAW_DATA, $user->initial_vector);
                $data = base64_encode($data);
                return $data;
            }
        } elseif (isset(collect($response->headers)['x-token'][0]) && isset(collect($response->headers)['x-refresh-token'][0]) && $pg == 'pg') {
            $userDetails = json_decode($response->getContent());
            $user = User::find($userDetails->id);
            if ($user->allow_3rd_party_use) {
                $mode = 'AES-256-CBC';
                $data = openssl_encrypt($response->getContent(), $mode, $user->secret_key, OPENSSL_RAW_DATA, $user->initial_vector);
                $data = base64_encode($data);
                return $data;
            }
        }
        $mode = 'AES-256-CBC';
        $data = openssl_encrypt($response->getContent(), $mode, env('AES_KEY'), OPENSSL_RAW_DATA, env('AES_IV'));
        $data = base64_encode($data);
        $first_fifteen = 'h&EKZdsBUkRaG%3';
        $second_seven = 'xejTbcK';
        $third_eight = 'j#pEcft4';
        $fourth_six = 't^FusM';
        $fifth_seven = 'rD9$b#Y';
        $last_fifteen = 'aez964DZn%$BhF^';
        if (strlen($data) > 29) {
            $data = substr_replace($data, $last_fifteen, -3, 0);
            $data = substr_replace($data, $fifth_seven, 28, 0);
            $data = substr_replace($data, $fourth_six, 22, 0);
            $data = substr_replace($data, $third_eight, 14, 0);
            $data = substr_replace($data, $second_seven, 8, 0);
            $data = substr_replace($data, $first_fifteen, 0, 0);
        }
        return $data;
    }
}