<?php

namespace App\Http\Middleware;

use App\ActionLog;
use App\Admin;
use Closure;
use Illuminate\Http\Request;

use Lcobucci\JWT\Signer\Hmac\Sha256;
use Lcobucci\JWT\Parser;
use Lcobucci\JWT\ValidationData;
use App\Exceptions\BaseException;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\AdminController;
use App\Http\Controllers\Epos\EposUserController;
use App\Http\Controllers\Tiffintom\TiffintomUserController;
use Illuminate\Support\Facades\Crypt;
use App\Http\Controllers\UserController;
use App\Logs;
use DateTimeImmutable;
use Exception;
use Illuminate\Support\Facades\Log;
use Throwable;

class ParseToken
{
    public function handle(Request $req, Closure $next)
    {
        $res = null;
        if ($req->hasHeader('business-id')) {
            Auth::$business_id = $req->header('business-id');
            Auth::setUserConfig();
        }
        if ($req->hasHeader('package-name')) {
            Auth::$package_name = $req->header('package-name');
        }
        if ($req->hasHeader('restaurant-id')) {
            Auth::$restaurant_id = $req->header('restaurant-id');
        }
        if ($req->hasHeader('business_id')) {
            Auth::$business_id = $req->header('business_id');
            Auth::setUserConfig();
        }

        // try {
        //     if (isset($_SERVER['HTTP_USER_AGENT'])) {
        //         $HTTP_USER_AGENT = $_SERVER['HTTP_USER_AGENT'];
        //         $REQUEST_URI = $_SERVER['REQUEST_URI'];
        //         if (str_contains($REQUEST_URI, 'tiffintom') && !str_contains($REQUEST_URI, 'payment-message') && !str_contains($HTTP_USER_AGENT, 'okhttp') && !str_contains($HTTP_USER_AGENT, 'CFNetwork')) {
        //             Utils::error('Access Forbidden !', 403);
        //         } else if (str_contains($HTTP_USER_AGENT, 'okhttp')) {
        //         } else if (str_contains($HTTP_USER_AGENT, 'CFNetwork')) {
        //         } else if (str_contains($REQUEST_URI, 'api')) {
        //         } else if (str_contains($REQUEST_URI, 'epos')) {
        //         } else if (str_contains($REQUEST_URI, 'payment-message')) {
        //         } else {
        //             Utils::error('Access Forbidden !', 403);
        //         }
        //     }
        // } catch (BaseException $e) {
        //     Utils::error('Access Forbidden !', 403);
        // }

        if ($req->hasHeader('x-token')) {
            try {
                // access token
                $aConfig = Auth::getConfig(env('X_TOKEN_KEY'));
                $aToken =  $aConfig->parser()->parse(Crypt::decrypt($req->header('x-token')));
                $aConstraints = $aConfig->validationConstraints();
                if (!$aConfig->validator()->validate($aToken, ...$aConstraints)) {
                    throw new BaseException('Invalid authentication', 401, 401);
                }

                // set id and role in request
                Auth::setUser($aToken->claims()->get('role'), $aToken->claims()->get('id'));
                $rIsVerif = true;
                if ($aToken->isExpired(new DateTimeImmutable())) {
                    switch (Auth::role()) {
                        case 'guest':
                            $rIsVerif = true;
                            break;
                        case 'user':
                            $rConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . Auth::user()->password);
                            $rToken =  $rConfig->parser()->parse(Crypt::decrypt($req->header('x-refresh-token')));
                            $rIsVerif = UserController::verifyRefreshToken($rToken);
                            break;
                        case 'admin':
                            $rConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . Auth::admin()->password);
                            $rToken =  $rConfig->parser()->parse(Crypt::decrypt($req->header('x-refresh-token')));
                            $rIsVerif = AdminController::verifyRefreshToken($rToken);
                            break;
                        case 'epos_admin':
                            $rConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . Auth::user()->password);
                            $rToken =  $rConfig->parser()->parse(Crypt::decrypt($req->header('x-refresh-token')));
                            $rIsVerif = UserController::verifyRefreshToken($rToken);
                            break;

                        case 'epos_user':
                            $rConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . Auth::user()->password);
                            $rToken =  $rConfig->parser()->parse(Crypt::decrypt($req->header('x-refresh-token')));
                            $rIsVerif = EposUserController::verifyRefreshToken($rToken);
                            break;
                        case 'tiffintom_user':
                            $rConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . Auth::user()->password);
                            $rToken =  $rConfig->parser()->parse(Crypt::decrypt($req->header('x-refresh-token')));
                            $rIsVerif = TiffintomUserController::verifyRefreshToken($rToken);
                            break;
                        default:
                            $rIsVerif = false;
                            break;
                    }
                    // invalid refresh token
                    if (!$rIsVerif) throw new BaseException('Session expired. Please login again!', 401, 401);

                    if ($rToken->isExpired(new DateTimeImmutable())) {
                        throw new BaseException('Session expired. Please login again!', 401, 401);
                    }
                    // generate new tokens with refresh token
                    switch (Auth::role()) {
                        case 'user':
                            $tokens = UserController::getNewTokens($rToken->claims()->get('id'));
                            break;
                        case 'admin':
                            $tokens = AdminController::getNewTokens($rToken->claims()->get('id'));
                            break;
                        case 'epos_admin':
                            $tokens = UserController::getNewTokens($rToken->claims()->get('id'), 'epos_admin');
                            break;
                        case 'epos_user':
                            $tokens = EposUserController::getNewTokens($rToken->claims()->get('id'));
                            break;
                        case 'tiffintom_user':
                            $tokens = TiffintomUserController::getNewTokens($rToken->claims()->get('id'));
                            break;
                        default:
                            $tokens = null;
                            break;
                    }
                    if ($tokens == null) throw new \Exception();

                    $aToken = (string) $tokens['aToken'];
                    $rToken = (string) $tokens['rToken'];

                    $res = $next($req);

                    // attach new tokens to response header
                    $res->header('x-token', $aToken);
                    $res->header('x-refresh-token', $rToken);
                }
            } catch (BaseException $e) {
                throw $e;
            } catch (Throwable $th) {
                throw new BaseException("Invalid session or expired, Please login to continue", 401, 401);
            }
        }
        if (Auth::$restaurant_id != null) {
            $req->merge(['restaurant_id' => Auth::$restaurant_id]);
        }

        $path = $req->getPathInfo();
        $explod = explode('/', $path);
        $last = end($explod);
        if ($last == '' || $last == '/') {
            $last = prev($explod);
        }
        // if (
        //     $explod[1] == 'api' && !empty(Auth::admin()) && $explod[2] != 'admins' && $explod[2] != 'action-logs' && $last != 'action-logs' && ($req->header('x-token') && $req->header('x-refresh-token'))
        // ) {
        //     try {
        //         $user = Auth::admin();
        //         if (!empty($user)) {
        //             $action_log = new ActionLog();
        //             $action_log->admin_id =  $user->id;
        //             $action_log->method =  $req->getMethod();
        //             $action_log->prefix =  $explod[2];
        //             $action_log->end_point =  $last;
        //             $action_log->full_path =  $path;
        //             $action_log->request_raw =  json_encode($req->all());
        //             $action_log->save();
        //         }
        //     } catch (Exception $e) {
        //     }
        // }

        if (isset($explod[1]) && isset($explod[2]) && isset($explod[3])) {
            if (
                (($explod[1] == 'epos' && $explod[2] == 'tiffintom' && $explod[3] == 'bookings') || ($explod[1] == 'tiffintom' && $explod[2] == 'bookings' && $last == 'bookings'))  && ($req->header('x-token') && $req->header('x-refresh-token'))
            ) {
                try {
                    $user = Auth::user();

                    if (isset($_SERVER['HTTP_CLIENT_IP']))
                        $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
                    else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
                        $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
                    else if (isset($_SERVER['HTTP_X_FORWARDED']))
                        $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
                    else if (isset($_SERVER['HTTP_FORWARDED_FOR']))
                        $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
                    else if (isset($_SERVER['HTTP_FORWARDED']))
                        $ipaddress = $_SERVER['HTTP_FORWARDED'];
                    else if (isset($_SERVER['REMOTE_ADDR']))
                        $ipaddress = $_SERVER['REMOTE_ADDR'];
                    else
                        $ipaddress = '';

                    if (!empty($user) && $req->getMethod() == "DELETE") {
                        $action_log = new ActionLog();
                        $action_log->user_type =  'tiffintom';
                        $action_log->user_id =  $user->id;
                        $action_log->business_id =  Auth::$business_id;
                        $action_log->method =  $req->getMethod();
                        if ($explod[1] == 'epos' && $explod[2] == 'tiffintom' && $explod[3] == 'bookings') {
                            $action_log->prefix =  $explod[3];
                        } else {
                            $action_log->prefix =  $explod[2];
                        }
                        $action_log->end_point =  $last;
                        $action_log->full_path =  $path;
                        $action_log->request_raw =  json_encode($req->all());
                        $action_log->ip =  $ipaddress;
                        $action_log->save();
                    }
                } catch (Exception $e) {
                }
            }
        }

        if (isset($explod[1]) && isset($explod[2])) {
            if (
                (($explod[1] == 'api' && (($explod[2] == 'finance' || $explod[2] == 'finance-users'))) && $req->header('x-token') && $req->header('x-refresh-token'))
            ) {
                try {
                    $user = Auth::user();

                    if (isset($_SERVER['HTTP_CLIENT_IP']))
                        $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
                    else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
                        $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
                    else if (isset($_SERVER['HTTP_X_FORWARDED']))
                        $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
                    else if (isset($_SERVER['HTTP_FORWARDED_FOR']))
                        $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
                    else if (isset($_SERVER['HTTP_FORWARDED']))
                        $ipaddress = $_SERVER['HTTP_FORWARDED'];
                    else if (isset($_SERVER['REMOTE_ADDR']))
                        $ipaddress = $_SERVER['REMOTE_ADDR'];
                    else
                        $ipaddress = '';

                    if (!empty($user) && ($req->getMethod() == "POST" || $req->getMethod() == "DELETE")) {
                        $action_log = new ActionLog();
                        $action_log->user_type =  'admin';
                        $action_log->user_id =  $user->id;
                        $action_log->business_id =  Auth::$business_id;
                        $action_log->method =  $req->getMethod();
                        $action_log->prefix =  $explod[2];
                        $action_log->end_point =  $last;
                        $action_log->full_path =  $path;
                        $action_log->request_raw =  json_encode($req->all());
                        $action_log->ip =  $ipaddress;
                        $action_log->save();
                    }
                } catch (Exception $e) {
                }
            }
        }

        return $res != null ? $res : $next($req);
    }
}
