<?php

namespace App\Http\Middleware;

use App\Exceptions\BaseException;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\UserBusiness;
use Closure;
use Illuminate\Support\Facades\Log;

class UserHasBusinessMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        $user = Auth::user();
        if ($user) {
            $hasHeader = $request->hasHeader('business-id');
            if ($hasHeader) {
                $businessId = $request->header('business-id');
                $parameter = $request->route('business_id');
                if (!is_null($parameter)) {
                    if ($businessId != $parameter) {
                        throw new BaseException('Business id must be the same in URL Parameter and Header.', 400, 400);
                    }
                }

                $userBusiness = UserBusiness::where('user_id', $user->id)->where('business_id', $businessId)->first();
                if (!$userBusiness) {
                    throw new BaseException('User can\'t access this business.', 400, 400);
                }
                $response = $next($request);
            } else {
                throw new BaseException('Business id field is required.', 400, 400);
            }
        } else {
            throw new BaseException('User not authenticated.', 400, 400);
        }
        return $response;
    }
}
