<?php

namespace App;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class Invoice extends Model
{
    protected $fillable = [
        'invoice_type',
        'business_id',
        'currency_symbol',
        'header',
        'bill_to',
        'ship_to',
        'date',
        'sub_total',
        'grand_total',
        'tax_mode',
        'tax',
        'discount_mode',
        'discount',
        'notes',
        'terms',
        'image',
        'status',
        'payment_mode',
        'paid_amount',
        'mobile_number',
        'remark',
        'recurring',
        'recurring_days',
        'recurring_date',
        'archived',
    ];
    protected $hidden = ['updated_at', 'recurring_date'];
    protected $appends = ['image_url', 'thumb_url'];
    protected $casts = [
        'grand_total' => 'double',
        'tax' => 'double',
        'discount' => 'double',
        'percentage' => 'double',
        'paid_amount' => 'double',
        'recurring' => 'bool',
        'archived' => 'bool',
    ];

    // Attributes
    //
    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_INVOICE_LOGO, $this->image);
    }
    public function getThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_INVOICE_LOGO, $this->image);
    }
    public function getInvoiceItemsAttribute()
    {
        return InvoiceItem::where('invoice_id', $this->id)->get();
    }
    public function getInvoicePaymentsAttribute()
    {
        return InvoicePayment::where('invoice_id', $this->id)->get();
    }
    public function getBusinessAttribute()
    {
        return Business::where('id', $this->business_id)->first();
    }
}
