<?php

namespace App\Jobs;

use App\Helpers\Auth;
use App\Http\Controllers\Epos\EposPrinterModelController;
use App\ModelsEpos\EposPrinterModel;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class PrinterCreatorUpdaterJob extends Job
{
    /**
     * Run jobs
     * nohup php artisan queue:work --daemon > storage/logs/queue.log &
     * this will run all queues in background task
     * 
     */
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public $business;
    public $printer;
    public $create = true;
    public function __construct($business, $printer, $create = true)
    {
        $this->business = $business;
        $this->printer = $printer;
        $this->create = $create;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $business = $this->business;
        if ($business->epos_service == 0) return;
        if ($business->db_name == null) return;
        if ($business->database_server_id == null) return;
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) return;

        $config = app()->make('config');
        $config->set('database.connections.printer_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
        ]);
        DB::purge('printer_connection');

        $printerModel = new EposPrinterModel;
        $printerModel->setConnection("printer_connection");
        $printerModel = $printerModel->find($this->printer->id);
        if (empty($printerModel)) {
            $printerModel = new EposPrinterModel();
            $printerModel->setConnection("printer_connection");
            $printerModel->name = $this->printer->name;
            $printerModel->id = $this->printer->id;
            $printerModel->save();
        } else {
            $printerModel->setConnection("printer_connection");
            $printerModel->name = $this->printer->name;
            $printerModel->save();
        }
    }
}
