<?php

namespace App\Jobs;

use App\CaxtonPayoutRequest;
use App\CaxtonWallet;
use App\Helpers\Auth;
use App\Http\Controllers\SiteSettingController;
use App\Passbook;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class WalletTransactionCreatorJob extends Job
{
    /**
     * Run jobs
     * nohup php artisan queue:work --daemon > storage/logs/queue.log &
     * this will run all queues in background task
     * 
     */
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public $payout;
    public $create = true;
    public function __construct($payout, $create = true)
    {
        $this->payout = $payout;
        $this->create = $create;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $endPoint = 'payments/currency-bank/card-load';
        $client = new \GuzzleHttp\Client();
        $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
            'body' => '{
            "WalletId":"' . $this->payout->wallet_id . '",
            "Currency":"GBP",
            "Amount":"' . $this->payout->amount . '",
        }',
            'headers' => [
                'Authorization' => 'bearer ' . $apiKey,
                'UserApiToken' => $user_apiKey,
                'content-type' => 'application/json',
            ],
        ]);
        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            $data = json_decode($data->getBody()->getContents());
            $shop_order_id  = $data->Content->Model->ShopOrderId;
            $processEndPoint = 'payments/currency-bank/process';
            $walletDetails = $client->request('POST', env('CAXTON_BASEURL') . $processEndPoint, [
                'body' => '{
                "ShopOrderId":"' . $shop_order_id . '",
            }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($walletDetails->getStatusCode() >= 200 && $walletDetails->getStatusCode() <= 210) {
                $walletDetails = json_decode($walletDetails->getBody()->getContents());

                $bankPayoutRequest = CaxtonPayoutRequest::find($this->payout->bank_request_id);
                $bankPayoutRequest->status = 'Accepted';
                $bankPayoutRequest->save();

                $passbook = new Passbook();
                $passbook->entity = "payout_request";
                $passbook->entity_id = $bankPayoutRequest->id;
                $passbook->business_id = $bankPayoutRequest->business_id;
                $passbook->user_id = $bankPayoutRequest->user_id;
                $passbook->date = Carbon::today()->toDateString();
                $passbook->from_date = $bankPayoutRequest->created_at;
                $passbook->to_date = $bankPayoutRequest->created_at;
                $passbook->amount = -1 * $bankPayoutRequest->amount;
                $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($bankPayoutRequest->created_at)->toDateString();
                $passbook->save();
            }
        }
    }
}
