<?php

namespace App\Jobs;

use App\Business;
use App\CaxtonPayoutRequest;
use App\CaxtonWallet;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\SiteSettingController;
use App\PouchHistory;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class WalletTransferCreatorJob extends Job
{
    /**
     * Run jobs
     * nohup php artisan queue:work --daemon > storage/logs/queue.log &
     * this will run all queues in background task
     * 
     */
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public $payout;
    public $create = true;
    public function __construct($payout, $create = true)
    {
        $this->payout = $payout;
        $this->create = $create;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $apiKey = SiteSettingController::findByKey('caxton_token')->value;
        $user_apiKey = SiteSettingController::findByKey('caxton_password')->value;

        $caxtonWallet = CaxtonWallet::find($this->payout->wallet_id);
        if (empty($caxtonWallet)) Utils::error('wallet not found!', 404);

        $business = Business::find($this->payout->business_id);
        if (empty($business)) Utils::error('business not found!', 404);

        try {
            $endPoint = 'payments/currency-bank/card-load';

            $client = new \GuzzleHttp\Client();
            $data = $client->request('POST', env('CAXTON_BASEURL') . $endPoint, [
                'body' => '{
                "WalletId":"' . $caxtonWallet->wallet_id . '",
                "Currency":"GBP",
                "Amount":"' . $this->payout->amount . '",
            }',
                'headers' => [
                    'Authorization' => 'bearer ' . $apiKey,
                    'UserApiToken' => $user_apiKey,
                    'content-type' => 'application/json',
                ],
            ]);
            if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
                $data = json_decode($data->getBody()->getContents());

                $bankPayoutRequest = CaxtonPayoutRequest::find($this->payout->id);
                $bankPayoutRequest->shop_order_id = $data->Content->Model->ShopOrderId;
                $bankPayoutRequest->save();

                // sleep(5);

                $processEndPoint = 'payments/currency-bank/process';
                $walletDetails = $client->request('POST', env('CAXTON_BASEURL') . $processEndPoint, [
                    'body' => '{
                    "ShopOrderId":"' . $bankPayoutRequest->shop_order_id . '",
                }',
                    'headers' => [
                        'Authorization' => 'bearer ' . $apiKey,
                        'UserApiToken' => $user_apiKey,
                        'content-type' => 'application/json',
                    ],
                ]);

                if ($walletDetails->getStatusCode() >= 200 && $walletDetails->getStatusCode() <= 210) {

                    $statusEndPoint = 'payments/currency-bank/status?shopOrderId=' . $bankPayoutRequest->shop_order_id;
                    $statusDetails = $client->request('GET', env('CAXTON_BASEURL') . $statusEndPoint, [
                        'headers' => [
                            'Authorization' => 'bearer ' . $apiKey,
                            'UserApiToken' => $user_apiKey,
                            'content-type' => 'application/json',
                        ],
                    ]);

                    if ($statusDetails->getStatusCode() >= 200 && $statusDetails->getStatusCode() <= 210) {
                        $statussDetails = json_decode($statusDetails->getBody()->getContents());
                        $status = $statussDetails->Content->Model->PaymentStatus;

                        if ($status == 'Success') {
                            // Update Payout Request
                            $bankPayoutRequest = CaxtonPayoutRequest::find($this->payout->id);
                            $bankPayoutRequest->status = 'Accepted';
                            $bankPayoutRequest->save();

                            // Create pouch-history
                            $pouch_history = new PouchHistory();
                            $pouch_history->business_id = $business->id;
                            $pouch_history->debit_amount = $this->payout->amount;
                            $pouch_history->transaction_category = 'debit';
                            $pouch_history->description = 'Wallet Debited';
                            $pouch_history->status_description = 'wallet-transfer';
                            $pouch_history->remark = $this->payout->note;
                            $pouch_history->wallet_id = $caxtonWallet->wallet_id;
                            $pouch_history->status = 'Accepted';
                            $pouch_history->updater_id = Auth::id();
                            $pouch_history->save();

                            // Total Balance Update
                            $pouch_history->total_balance = $business->pouch_weight;
                            $pouch_history->save();
                        }

                        if ($status == 'Failed') {
                            // Update Payout Request
                            $bankPayoutRequest = CaxtonPayoutRequest::find($this->payout->id);
                            $bankPayoutRequest->status = 'Rejected';
                            $bankPayoutRequest->save();

                            // Create pouch-history
                            $pouch_history = new PouchHistory();
                            $pouch_history->business_id = $business->id;
                            $pouch_history->credit_amount = $this->payout->amount;
                            $pouch_history->transaction_category = 'credit';
                            $pouch_history->description = 'Wallet Credited';
                            $pouch_history->status_description = 'wallet-transfer';
                            $pouch_history->remark = 'redemption';
                            $pouch_history->wallet_id = $caxtonWallet->wallet_id;
                            $pouch_history->status = 'Rejected';
                            $pouch_history->save();

                            // Business Wallet Update
                            $business->pouch_weight = $business->pouch_weight + $this->payout->amount;
                            $business->save();

                            // Total Balance Update
                            $pouch_history->total_balance = $business->pouch_weight;
                            $pouch_history->save();
                        }
                    }
                }
            }
        } catch (\Exception $e) {

            // / Update Payout Request
            $bankPayoutRequest = CaxtonPayoutRequest::find($this->payout->id);
            $bankPayoutRequest->status = 'Rejected';
            $bankPayoutRequest->save();

            // Create pouch-history
            $pouch_history = new PouchHistory();
            $pouch_history->business_id = $business->id;
            $pouch_history->credit_amount = $this->payout->amount;
            $pouch_history->transaction_category = 'credit';
            $pouch_history->description = 'Wallet Credited';
            $pouch_history->status_description = 'wallet-transfer';
            $pouch_history->remark = 'redemption';
            $pouch_history->wallet_id = $caxtonWallet->wallet_id;
            $pouch_history->status = 'Rejected';
            $pouch_history->save();

            // Business Wallet Update
            $business->pouch_weight = $business->pouch_weight + $this->payout->amount;
            $business->save();

            // Total Balance Update
            $pouch_history->total_balance = $business->pouch_weight;
            $pouch_history->save();

            if ($e instanceof \GuzzleHttp\Exception\RequestException) {
                $data = json_decode($e->getResponse()->getBody());
                Utils::error($data->FirebirdApiError->ApiStatusDescription, 404);
            }
        }
    }
}
