<?php

namespace App\Jobs;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Invoice;
use App\InvoiceItem;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class invoiceCreatorJob extends Job
{
    /**
     * Run jobs
     * nohup php artisan queue:work --daemon > storage/logs/queue.log &
     * this will run all queues in background task
     * 
     */
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public $invoiceId;
    public function __construct($invoiceId)
    {
        $this->invoiceId = $invoiceId;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $oldInvoice = Invoice::find($this->invoiceId);
        if (empty($oldInvoice)) Utils::error('Invoice not found!', 404);
        $oldInvoice->append('invoice_items');
        $oldInvoice->append('invoice_payments');

        $invoice = new Invoice;
        $invoice->fill($oldInvoice);
        if ($oldInvoice->recurring) {
            $invoice->recurring_date = Carbon::today()->addDays($oldInvoice->recurring_days)->format('Y-m-d');
        }
        $invoice->save();

        $items = ($oldInvoice->invoice_items);
        foreach ($items as $item) {
            $invoiceItem = new InvoiceItem((array)$item);
            $invoiceItem->invoice_id = $invoice->id;
            $invoiceItem->save();
        }
    }
}
