<?php

namespace App\ModelsEpos;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class EposAddon extends Model
{
    protected $table = 'addons';
    protected $connection = "epos_connection";
    protected $fillable = [
        'name',
        'image',
        'parent_id',
        'price',
        'takeaway_price',
        'web_price',
        'delivery_price',
        'waiting_price',
        'sequence',
    ];
    protected $hidden = ['updated_at'];
    protected $casts = ['disabled' => 'bool', 'price' => 'double', 'takeaway_price' => 'double', 'web_price' => 'double', 'delivery_price' => 'double', 'waiting_price' => 'double'];
    protected $appends = ['title', 'key', 'children', 'image_url', 'thumb_url'];


    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_EPOS_ADDON, $this->image);
    }
    public function getThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_EPOS_ADDON, $this->image);
    }
    public function getTitleAttribute()
    {
        return $this->name;
    }
    public function getKeyAttribute()
    {
        return $this->id;
    }
    public function getChildrenAttribute()
    {
        $children = $this->hasMany('App\ModelsEpos\EposAddon', 'parent_id', 'id')->orderByRaw('sequence DESC')->get();
        if ($children->isEmpty()) {
            return null;
        }
        return $children->makeHidden(['thumb_url', 'image_url', 'created_at', 'description']);
    }
}
