<?php

namespace App\ModelsEpos;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class EposCategory extends Model
{
    protected $table = 'categories';
    protected $connection = "epos_connection";
    protected $fillable = [
        'parent_id',
        'name',
        'image',
        'description',
        'top_color',
        'bottom_color',
        'font_color',
        'sequence',
    ];
    protected $hidden = ['updated_at'];
    protected $casts = [
        'disabled' => 'bool',
        'is_dinein' => 'bool',
        'is_delivery' => 'bool',
        'is_collection' => 'bool',
        'is_banquet' => 'bool',
    ];
    protected $appends = ['title', 'key', 'children', 'image_url', 'thumb_url'];

    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_EPOS_CATEGORY, $this->image);
    }
    public function getThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_EPOS_CATEGORY, $this->image);
    }
    public function getTitleAttribute()
    {
        return $this->name;
    }
    public function getKeyAttribute()
    {
        return $this->id;
    }
    public function getChildrenAttribute()
    {
        $children = $this->hasMany('App\ModelsEpos\EposCategory', 'parent_id', 'id')->orderByRaw('sequence DESC')->get();
        if ($children->isEmpty()) {
            return null;
        }
        return $children->makeHidden(['thumb_url', 'image_url', 'created_at', 'description']);
    }
    public function getPrintBlockIdAttribute()
    {
        $printBlockAssign = $this->assign_print_block();
        if (!empty($printBlockAssign)) {
            return $printBlockAssign->print_block_id;
        } else return null;
    }
    public function getAssignPrintBlockAttribute()
    {
        $PrintBlockAssign = $this->assign_print_block();
        return $PrintBlockAssign;
    }
    public function assign_print_block()
    {
        $PrintBlockAssign = EposPrintBlockAssign::select('print_block_id')->where('category_id', $this->id)->first();
        return $PrintBlockAssign;
    }
}
