<?php

namespace App\ModelsEpos;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class EposOrder extends Model
{
    protected $table = 'orders';
    protected $connection = "epos_connection";
    protected $fillable = [
        'customer_id',
        'table_id',
        'order_type_id',
        'order_status_id',
        'preparation_location_id',
        'updater_id',
        'order_action_id',
        'sub_total',
        'tax',
        'service_charge',
        'delivery_charge',
        'gratuity',
        'discount',
        'total',
        'total_paid',
        'due_amount',
        'comment',
        'delivery_date',
        'split_type',
        'split_count',
        'device_id',
        'no_guest',
        'distance',
        'is_archived',
        'unique_id',
        'display_order_id',
        'delivery_collection_time_slot',
        'is_delivery_collection_ask',
        'is_auto_service_charge_remove',
        'created_at',
        'updated_at',
    ];
    protected $attributes = [
        'table_id' => null,
    ];
    protected $casts = [
        'is_archived' => 'bool',
        'is_delivery_collection_ask' => 'bool',
        'is_auto_service_charge_remove' => 'bool',
    ];
    // protected $appends = ['order_items','order_payments','customer'];
    // protected $hidden = ['updated_at'];

    // Attributes
    //
    public function getOrderItemsAttribute()
    {
        return $this->order_items();
    }
    public function order_items()
    {
        // return $this->hasMany(EposOrderItem::class, 'order_id', 'id')->selectRaw('order_items.*,products.name as product_name,products.short_name as short_name,order_splits.group_name as group_name')->leftJoin('products', 'products.id', '=', 'product_id')->leftJoin('order_splits', 'order_splits.id', '=', 'order_split_id');
        return EposOrderItem::selectRaw('order_items.*,products.name as product_name,products.short_name as short_name,products.short_name as product_short_name,products.misc as misc,order_splits.group_name as group_name')->leftJoin('products', 'products.id', '=', 'product_id')->leftJoin('order_splits', 'order_splits.id', '=', 'order_split_id')->where('order_items.order_id', $this->id)->get();
    }
    public function getOrderPaymentsAttribute()
    {
        return $this->order_payments();
    }
    public function order_payments()
    {
        // return $this->hasMany(EposOrderPayment::class, 'order_id', 'id')->selectRaw('order_payments.*,payment_methods.name as payment_method_name')->leftJoin('payment_methods', 'payment_methods.id', '=', 'payment_method_id');
        return EposOrderPayment::selectRaw('order_payments.*,payment_methods.name as payment_method_name,vouchers.voucher_code as voucher_code')->leftJoin('payment_methods', 'payment_methods.id', '=', 'payment_method_id')->leftJoin('vouchers', 'vouchers.id', '=', 'voucher_id')->where('order_id', $this->id)->get();
    }
    public function getOrderSplitsAttribute()
    {
        return $this->order_splits();
    }
    public function order_splits()
    {
        // return $this->hasMany(EposOrderSplit::class, 'order_id', 'id')->selectRaw("order_splits.*, (SELECT SUM(amount) from order_payments WHERE order_split_id= order_splits.id)as paid_amount");
        return EposOrderSplit::selectRaw("order_splits.*, (SELECT SUM(amount) from order_payments WHERE order_split_id= order_splits.id)as paid_amount")->where('order_id', $this->id)->get();
    }
    public function getCustomerAttribute()
    {
        return $this->customer();
    }
    public function customer()
    {
        // return $this->hasOne(EposCustomer::class, 'id', 'customer_id');
        return EposCustomer::where('id', $this->customer_id)->first();
    }
    public function getTableAttribute()
    {
        return $this->table();
    }
    public function table()
    {
        // return $this->hasOne(EposTable::class, 'id', 'table_id');
        return EposTable::selectRaw("tables.*,floors.name as floor_name")->leftJoin('floors', 'floors.id', '=', 'floor_id')->where('tables.id', $this->table_id)->first();
    }
    public function getReportProductImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_EPOS_PRODUCT, $this->report_product_image);
    }
    public function getReportProductThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_EPOS_PRODUCT, $this->report_product_image);
    }
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d\TH:i:s.000000\Z');
    }
}
