<?php

namespace App\ModelsEpos;

use Illuminate\Database\Eloquent\Model;
use PDO;

class EposOrderItem extends Model
{
    protected $table = 'order_items';
    protected $connection = "epos_connection";
    protected $fillable = [
        'order_id',
        'product_id',
        'updater_id',
        'order_split_id',
        'price',
        'addons_price',
        'ingredients_price',
        'quantity',
        'sub_total',
        'total',
        'special_instruction',
        'instruction_price',
        'sent_to_kitchen',
        'sent_to_kitchen_quantity',
        'is_delete',
        'unique_id',
        'item_unique_id',
    ];
    protected $hidden = ['updated_at'];
    protected $appends = ['product', 'order_item_addons', 'order_item_ingredients', 'misc', 'preparation_location_id'];
    protected $casts = ['sent_to_kitchen' => 'bool', 'misc' => 'bool', 'is_delete' => 'bool'];
    // Attributes
    //
    public function getOrderItemAddonsAttribute()
    {
        return $this->order_item_addons();
    }
    public function order_item_addons()
    {
        return EposOrderItemAddon::selectRaw('order_item_addons.*,addons.name as addon_name')->leftJoin('addons', 'addons.id', '=', 'addon_id')->where('order_item_id', $this->id)->get();
    }
    public function getOrderItemIngredientsAttribute()
    {
        return $this->order_item_ingredients();
    }
    public function getMiscAttribute()
    {
        $product = $this->product();
        if (!empty($product)) {
            return $product->misc;
        } else {
            return false;
        }
    }
    public function getPreparationLocationIdAttribute()
    {
        $product = $this->product();
        if (!empty($product)) {
            return $product->preparation_location_id;
        } else {
            return null;
        }
    }
    public function order_item_ingredients()
    {
        return EposOrderItemIngredient::selectRaw('order_item_ingredients.*,ingredients.name as ingredient_name')->leftJoin('product_ingredients', 'product_ingredients.id', '=', 'product_ingredient_id')->leftJoin('ingredients', 'ingredients.id', '=', 'product_ingredients.ingredient_id')->where('order_item_id', $this->id)->get();
    }
    public function getProductAttribute()
    {
        return $this->product();
    }
    public function product()
    {
        return EposProduct::where('id', $this->product_id)->first();
    }
}