<?php

namespace App\ModelsEpos;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class EposProduct extends Model
{
    protected $table = 'products';
    protected $connection = "epos_connection";
    protected $fillable = [
        'name',
        'short_name',
        'description',
        'image',
        'top_color',
        'font_color',
        'parent_id',
        'category_id',
        'preparation_location_id',
        'collection_preparation_location_id',
        'delivery_preparation_location_id',
        'banquet_preparation_location_id',
        'price',
        'takeaway_price',
        'web_price',
        'delivery_price',
        'waiting_price',
        'is_dinein',
        'is_delivery',
        'is_collection',
        'is_banquet',
        'favourite',
        'bar_favourite',
        'tax_included',
        'taxable',
        'auto_modify',
        'auto_addon',
        'stockable',
        'misc',
        'barcode',
        'sequence',
    ];
    protected $hidden = ['updated_at'];
    protected $casts = [
        'disabled' => 'bool',
        'is_dinein' => 'bool',
        'is_delivery' => 'bool',
        'is_collection' => 'bool',
        'is_banquet' => 'bool',
        'favourite' => 'bool',
        'bar_favourite' => 'bool',
        'tax_included' => 'bool',
        'taxable' => 'bool',
        'auto_modify' => 'bool',
        'auto_addon' => 'bool',
        'stockable' => 'bool',
        'misc' => 'bool',
        'price' => 'double',
        'takeaway_price' => 'double',
        'web_price' => 'double',
        'delivery_price' => 'double',
        'waiting_price' => 'double'
    ];
    protected $appends = ['image_url', 'thumb_url', 'collection_preparation_location_name', 'delivery_preparation_location_name', 'banquet_preparation_location_name'];

    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_EPOS_PRODUCT, $this->image);
    }
    public function getThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_EPOS_PRODUCT, $this->image);
    }
    public function getCollectionPreparationLocationNameAttribute()
    {
        $collection_preparation_location = EposPrepLocation::find($this->collection_preparation_location_id);
        if (!empty($collection_preparation_location)) {
            return $collection_preparation_location->name;
        } else {
            return;
        }
    }
    public function getDeliveryPreparationLocationNameAttribute()
    {
        $delivery_preparation_location = EposPrepLocation::find($this->delivery_preparation_location_id);
        if (!empty($delivery_preparation_location)) {
            return $delivery_preparation_location->name;
        } else {
            return;
        }
    }
    public function getBanquetPreparationLocationNameAttribute()
    {
        $banquet_preparation_location = EposPrepLocation::find($this->banquet_preparation_location_id);
        if (!empty($banquet_preparation_location)) {
            return $banquet_preparation_location->name;
        } else {
            return;
        }
    }



    public function product_addons()
    {
        return EposProductAddon::select('product_addons.*', 'addons.name as addon_name')->leftJoin('addons', 'addons.id', '=', 'product_addons.addon_id')->where('product_addons.disabled', 0)->where('product_id', $this->id)->orderByRaw('product_addons.sequence ASC')->get();
    }

    public function getProductIngredientsAttribute()
    {
        return $this->product_ingredients();
    }
    public function product_ingredients()
    {
        return EposProductIngredient::select('product_ingredients.*', 'ingredients.name as ingredient_name', 'ingredients.price', 'ingredients.price_without')->leftJoin('ingredients', 'ingredients.id', '=', 'product_ingredients.ingredient_id')->where('product_ingredients.disabled', 0)->where('product_id', $this->id)->orderByRaw('product_ingredients.sequence ASC')->get();
    }
}
