<?php

namespace App\ModelsEpos;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class EposProductAddon extends Model
{
    protected $table = "product_addons";
    protected $connection = "epos_connection";

    protected $fillable = [
        'product_id',
        'addon_id',
        'parent_id',
        'sequence',
        'min_quantity',
        'max_quantity',
    ];
    protected $hidden = ['password', 'updated_at'];
    protected $casts = ['disabled' => 'bool', 'sequence' => 'int', 'min_quantity' => 'int', 'max_quantity' => 'int', 'contain' => 'bool'];
    protected $appends = ['children'];

    public function getChildrenAttribute()
    {
        $children = EposProductAddon::select('product_addons.*', 'addons.name as addon_name', 'addons.price', 'addons.web_price', 'addons.takeaway_price', 'addons.delivery_price', 'addons.waiting_price', 'addons.name as addon_name')->leftJoin('addons', 'addons.id', '=', 'product_addons.addon_id')->where('product_addons.parent_id', $this->id)->get();
        // $children = $this->hasMany('App\ModelsEpos\EposProductAddon', 'parent_id', 'id')->orderByRaw('id DESC')->get();
        if ($children->isEmpty()) {
            return null;
        }
        return $children->makeHidden(['thumb_url', 'image_url', 'created_at']);
    }

    public function getAddonAttribute()
    {
        $addon = EposAddon::find($this->addon_id);
        if (!empty($addon)) {
            $addon->append('children');
        } else {
            return null;
        }
        return $addon;
    }

    public function addon()
    {
        return $this->belongsTo('App\ModelsEpos\EposAddon',  $this->addon_id);
    }

    public function subaddons()
    {
        return $this->hasMany(EposProductAddon::class, 'parent_id', 'id');
    }

    public function getSubaddonIdsAttribute()
    {
        return implode(',', EposProductAddon::where('parent_id', $this->id)->pluck('addon_id')->toArray());
    }
}
