<?php

namespace App\ModelsEpos;

use Illuminate\Database\Eloquent\Model;
use Stripe\Order;

class EposTable extends Model
{
    protected $table = 'tables';
    protected $connection = "epos_connection";
    protected $fillable = [
        'number',
        'floor_id',
        'table_status_id',
        'updater_id',
        'sequence',
        'merge_table_id',
    ];
    protected $hidden = ['updated_at'];
    protected $appends = ['table_status', 'last_order_id', 'last_order_total', 'last_order_created_at'];
    protected $casts = ['disabled' => 'bool', 'last_order_total' => 'double', 'locked' => 'bool'];

    // Attributes
    //
    public function getTableStatusAttribute()
    {
        return $this->table_status()->select('status', 'top_color', 'bottom_color')->first();
    }
    public function getLastOrderIdAttribute()
    {
        $lastOrder = $this->last_order();
        if (!empty($lastOrder)) {
            return $lastOrder->id;
        } else {
            $lastMerge = $this->last_merge_order();
            if (!empty($lastMerge)) {
                return $lastMerge->id;
            }
        }
        return null;
    }
    public function getLastOrderTotalAttribute()
    {
        $lastOrder = $this->last_order();
        if (!empty($lastOrder)) {
            return $lastOrder->total;
        }
        return 0;
    }
    public function getLastOrderCreatedAtAttribute()
    {
        $lastOrder = $this->last_order();
        if (!empty($lastOrder)) {
            return date('Y-m-d\TH:i:s.000000\Z', strtotime($lastOrder->created_at));
        }
        return null;
    }
    // Relations
    //
    public function table_status()
    {
        return $this->belongsTo('App\ModelsEpos\EposTableStatus', 'table_status_id');
    }
    public function last_order()
    {
        return EposOrder::select('id', 'total', 'created_at')->where('table_id', $this->id)->whereNotIn('order_status_id', ['5', '10'])->where('order_type_id', '1')->orderByRaw('updated_at DESC')->first();
    }
    public function last_merge_order()
    {
        return EposOrder::select('id', 'total', 'created_at')->where('table_id', $this->merge_table_id)->whereNotIn('order_status_id', ['5', '10'])->where('order_type_id', '1')->orderByRaw('updated_at DESC')->first();
    }
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d\TH:i:s.000000\Z');
    }
}