<?php

namespace App\ModelsTiffintom;

use Carbon\Carbon;
use DateInterval;
use DatePeriod;
use DateTime;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class TiffintomBooking extends Model
{
    protected $table = "bookings";
    protected $connection = "tiffintom_connection";
    protected $casts = [];
    protected $fillable = [
        'id',
        'customer_id',
        'restaurant_id',
        'booking_id',
        'customer_name',
        'guest_count',
        'booking_email',
        'booking_phone',
        'booking_instruction',
        'booking_date',
        'booking_time',
        'type',
        'status',
    ];
    protected $appends = ['restaurant', 'past_days'];

    const CREATED_AT = 'created';
    const UPDATED_AT = 'updated';

    // Attributes
    //
    public function getRestaurantAttribute()
    {
        $restaurant = TiffintomRestaurant::where('id', $this->restaurant_id)->first();

        $reviews = TiffintomReview::selectRaw("COUNT(id) as count,CAST(sum(rating) AS DECIMAL(7,2)) as total_rating")->where('restaurant_id', $restaurant->id)->where('status', '1')->first();
        if (!empty($reviews)) {
            if ($reviews->count > 0) {
                $restaurant->average_rating = $this->convertToDecimal($reviews->total_rating / $reviews->count);
                $restaurant->total_reviews = $reviews->count;
            }
        }

        return $restaurant;
    }

    public function getPastDaysAttribute()
    {
        $from_date = new DateTime(Carbon::today()->startOfDay());
        $to_date =  new DateTime($this->booking_date);
        $period = $to_date->diff($from_date);
        $days = (int)$period->format('%a');
        return $days;
    }

    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
