<?php

namespace App\ModelsTiffintom;

use App\OnlineOrder;
use Carbon\Carbon;
use DateTime;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class TiffintomOrder extends Model
{
    protected $table = "orders";
    protected $connection = "tiffintom_connection";
    protected $casts = [
        'offer_percentage' => 'double',
        'offer_amount' => 'double',
        'order_point' => 'double',
        'order_sub_total' => 'double',
        'order_grand_total' => 'double',
        'service_charge' => 'double',
        'delivery_charge' => 'double',
        'driver_tip' => 'double',
        'charity_amount' => 'double',
        'voucher_amount' => 'double',
        'surcharges' => 'json',
    ];
    protected $fillable = [
        'id',
        'restaurant_id',
        'customer_id',
        'driver_id',
        'customer_name',
        'customer_email',
        'customer_phone',
        'source_latitude',
        'source_longitude',
        'destination_latitude',
        'destination_longitude',
        'flat_no',
        'google_address',
        'address',
        'landmark',
        'state_name',
        'city_name',
        'location_name',
        'user_type',
        'order_type',
        'assoonas',
        'order_description',
        'delivery_date',
        'delivery_time',
        'delivery_time_slot',
        'delivered_time',
        'offer_percentage',
        'offer_amount',
        'day_offer',
        'tax_percentage',
        'tax_amount',
        'delivery_charge',
        'service_charge',
        'voucher_code',
        'voucher_percentage',
        'voucher_amount',
        'tip_percentage',
        'tip_amount',
        'order_sub_total',
        'order_grand_total',
        'cardfee_percentage',
        'cardfee_price',
        'payment_status',
        'payment_method',
        'card_id',
        'payment_wallet',
        'paid_full',
        'split_payment',
        'wallet_amount',
        'transaction_id',
        'reward_used',
        'distance',
        'driver_invoice',
        'driver_invoice_number',
        'driver_deliver_date',
        'driver_charge',
        'failed_reason',
        'order_point',
        'spent_point',
        'reward_offer',
        'reward_offer_percentage',
        'stripe_customerid',
        'payerID',
        'paymentToken',
        'paymentID',
        'order_proof',
        'payout_type',
        'payout_amount',
        'status',
        'type',
        'completed_time',
        'preparation',
        'dont_bell',
        'dont_cutlery',
        'is_favourite',
        'charity_amount',
        'driver_tip',
        'charity_message',
        'delivery_instruction',
        'surcharge_amount',
    ];
    protected $appends = ['past_days'];
    const CREATED_AT = 'created';
    const UPDATED_AT = 'modified';


    // Attributes
    //
    public function getCartViewAttribute()
    {
        // $carts = TiffintomCart::where('order_id', $this->id)->orderBy('id', 'DESC')->get();
        $carts = TiffintomCart::selectRaw("carts.*,print_block_assign.print_block_id as print_block_id")->leftJoin('restaurant_menus', 'restaurant_menus.id', '=', 'carts.menu_id')->leftJoin('categories', 'categories.id', '=', 'restaurant_menus.category_id')->leftJoin('print_block_assign', 'print_block_assign.category_id', '=', 'restaurant_menus.category_id')->leftJoin('print_block', 'print_block.id', '=', 'print_block_assign.print_block_id')->where('order_id', $this->id)->orderByRaw('print_block.sequence ASC')->get();

        $carts->each(function ($cart) {
            if ($cart->subaddons_name != '') {
                $s = explode("[Free]", $cart->subaddons_name);
                $t = implode('', $s);
                $st = explode("]", $t);
                $tt = implode('', $st);
                $stt = explode(": :", $tt);
                $ttt = implode(':', $stt);
                $stta = explode("£", $ttt);
                $ttta = implode(" ", $stta);
                $sttaa = explode("+", $ttta);
                $tttaa = implode(',', $sttaa);
                $sttaaa = explode("Free", $tttaa);
                $tttaaa = implode('', $sttaaa);
                $sttaaaa = explode("(", $tttaaa);
                $tttaaaa = implode('', $sttaaaa);
                $sttaaaaa = explode(")", $tttaaaa);
                $tttaaaaa = implode('', $sttaaaaa);
                $o = preg_replace('<<br>>', ',', $tttaaaaa);
                $cart->subaddons_name =  str_replace("[", " ", $o);
            }
        });
        return $carts;
    }
    public function getCardViewAttribute()
    {
        $cards = TiffintomStripeCustomer::where('id', $this->card_id)->first();
        return $cards;
    }
    public function getReviewsAttribute()
    {
        $reviews = TiffintomReview::where('order_id', $this->id)->orderBy('id', 'DESC')->first();
        return $reviews;
    }
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d\TH:i:s.000000\Z');
    }

    public function getCalculatedFeesAttribute()
    {

        return $this->calculate_order_fees($this);
    }
    public function getPastDaysAttribute()
    {
        $from_date = new DateTime(Carbon::today()->startOfDay());
        $to_date =  new DateTime($this->delivery_date);
        $period = $to_date->diff($from_date);
        $days = (int)$period->format('%a');
        return $days;
    }


    public function calculate_order_fees($orders)
    {
        $orderTypeWiseCommission = 0.00;
        $orderTypeWiseServiceCharge = 0.00;
        $orderBasedOrTransactionWiseCharge = 0.00;
        $countryWiseCommission = 0.0;
        $cardBrandWiseCommission = 0.0;
        $payment_method_fees = 0;
        $chargeBackFee = 0;
        $handlingFee = 0;
        $cardCountry = "";
        $brand = "";

        $order = OnlineOrder::where('order_id', $orders->id)->first();
        if (empty($order)) {
            $transaction['total_fee'] = 0.0;
        } else {
            $orderTypeWiseServiceCharge = $order->user_service_charge;
            // if ($order->order_type == 'dinein') {
            //     $serviceCharge = $order->dinein_service_fee;
            // }
            $orderGrandTotal = $order->grand_total;

            if ($order->payment_method == 'paypal') {
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->commissions->online_paypal_fee, $order->commissions->online_paypal_fee_type);
            }
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online') {
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->commissions->online_card_fee, $order->commissions->online_card_fee_type);
            }
            if ($order->payment_method == 'cod')
                $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->commissions->online_cod_fee, $order->commissions->online_cod_fee_type);

            if ($order->order_type == 'dinein') {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->commissions->dinein_commission_fee, $order->commissions->dinein_commission_fee_type);
            } else {
                $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->commissions->online_commission_fee, $order->commissions->online_commission_fee_type);
            }
            if ($order->order_charge_type == 'order_based') {
                $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotal, $order->commissions->per_order_charge, $order->commissions->per_order_charge_type);
            } else {
                if ($order->payment_method == 'paypal')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->commissions->paypal_charge, $order->commissions->paypal_charge_type);
                if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Credit Card')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->commissions->card_charge, $order->commissions->card_charge_type);
                if ($order->payment_method == 'cod')
                    $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->commissions->cod_charge, $order->commissions->cod_charge_type);
            }

            $transaction['order_id'] = "#ODR000" . $order->order_id;
            $transaction['restaurant_id'] = $order->restaurant_id;
            if (empty($order->stripe_txn_id)) {
                $transaction['txn_id'] = $order->online_txn_id;
            } else {
                $transaction['txn_id'] = $order->stripe_txn_id;
            }
            $transaction['order_date'] = $order->order_date;
            $transaction['order_type'] = $order->order_type;
            $transaction['order_status'] = $order->order_status;
            $transaction['payment_method'] = $order->payment_method;
            $transaction['service'] = 'Online';

            $transaction['payment_method_fees'] = $payment_method_fees;
            $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
            $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
            $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;
            $transaction['country'] = $cardCountry;
            $transaction['country_charges'] = $countryWiseCommission;
            $transaction['card_brand'] = $brand;
            $transaction['card_brand_charge'] = $cardBrandWiseCommission;
            $transaction['charge_back_fee'] = $chargeBackFee;
            $transaction['handling_fee'] = $handlingFee;

            $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);
            if ($order->order_status == 'Failed') {
                $transaction['order_grand_total'] = -$this->convertToDecimal($orderGrandTotal);
                $transaction['total_fee'] = 0.0;
            } else {
                $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotal);
            }
            if ($order->payment_method == 'cod') {
                $transaction['net_total'] = $this->convertToDecimal(0 - $transaction['total_fee']);
            } else {
                $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $transaction['total_fee']);
            }
        }

        return $transaction;
    }
    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }
    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
