<?php

namespace App\ModelsTiffintom;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class TiffintomOrderHelp extends Model
{
    protected $table = "orderhelp";
    protected $connection = "tiffintom_connection";
    protected $fillable = [
        'id',
        'customer_id',
        'restaurant_id',
        'order_id',
        'message',
        'admin',
        'status',
        'is_attach',
    ];
    protected $hidden = ['modified'];
    protected $appends = ['image_url', 'user', 'restaurant', 'unread_count_restaurant', 'unread_count_user'];
    protected $casts = [
        'admin' => 'bool',
        'status' => 'bool',
    ];

    const CREATED_AT = 'created';
    const UPDATED_AT = 'modified';

    // Attributes
    //
    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_TIFFINTOM_ORDERHELP, $this->message, env('FTP_IMAGE_PATH_TIFFINTOM'));
    }
    public function getUserAttribute()
    {
        return TiffintomUser::where('id', $this->customer_id)->first();
    }
    public function getRestaurantAttribute()
    {
        return TiffintomRestaurant::where('id', $this->restaurant_id)->first();
    }
    public function getUnreadCountRestaurantAttribute()
    {
        return TiffintomOrderHelp::where('restaurant_id', $this->restaurant_id)->where('customer_id', $this->customer_id)->where('status', '1')->where('admin', '1')->count();
    }
    public function getUnreadCountUserAttribute()
    {
        return TiffintomOrderHelp::where('restaurant_id', $this->restaurant_id)->where('customer_id', $this->customer_id)->where('status', '1')->where('admin', '0')->count();
    }
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d\TH:i:s.000000\Z');
    }
}
