<?php

namespace App\ModelsTiffintom;

use App\Helpers\ImageHelper;
use Illuminate\Database\Eloquent\Model;

class TiffintomUser extends Model
{
    protected $table = "users";
    protected $connection = "tiffintom_connection";
    protected $casts = [
        'wallet_amount' => 'double',
        'order_place' => 'integer',
        'order_accept' => 'integer',
        'order_reject' => 'integer',
        'order_delivered' => 'integer',
        'book_table' => 'integer',
        'book_status' => 'integer',
        'important_update' => 'integer',
        'email_verify' => 'bool',
        'phone_verify' => 'bool',
        'status' => 'bool',
        'mobile' => 'bool',
    ];
    protected $fillable = [
        'id',
        'role_id',
        'username',
        'password',
        'permissions',
        'sms_permissions',
        'first_name',
        'last_name',
        'phone_number',
        'address',
        'referral_code',
        'referred_by',
        'wallet_amount',
        'image',
        'cover_image',
        'newsletter',
        'device_type',
        'device_id',
        'order_place',
        'order_accept',
        'order_reject',
        'order_delivered',
        'book_table',
        'book_status',
        'important_update',
        'status',
        'deleted_status',
        'created',
        'modified',
        'social_id',
        'otp',
        'email_verify',
        'phone_verify',
        'ip_address'
    ];

    protected $hidden = ['password', 'ip_adress', 'otp', 'email_otp'];
    protected $appends = ['image_url', 'thumb_url'];
    const CREATED_AT = 'created';
    const UPDATED_AT = 'modified';

    // Attributes
    //
    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_TIFFINTOM_USER, $this->image, env('FTP_IMAGE_PATH_TIFFINTOM'));
    }
    public function getThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_TIFFINTOM_USER, $this->image, env('FTP_IMAGE_PATH_TIFFINTOM'));
    }
    public function getOrdersAttribute()
    {
        return TiffintomOrder::where('customer_id', $this->id)->count();
    }
    public function getReservationsAttribute()
    {
        return TiffintomBooking::where('customer_id', $this->id)->count();
    }
}
