<?php

namespace App;

use DateTime;
use Illuminate\Database\Eloquent\Model;

class OnlineOrder extends Model
{
    protected $fillable = [
        'order_id',
        'business_id',
        'restaurant_id',
        'order_type',
        'txn_id',
        'payment_method',
        'sub_total',
        'delivery_charge',
        'driver_tip',
        'gratuity_tip',
        'grand_total',
        'charity_amount',
        'voucher_amount',
        'user_service_charge',
        'order_status',
        'order_date',
    ];
    // protected $hidden = ['updated_at'];
    protected $appends = ['commissions'];

    public function getCommissionsAttribute()
    {
        return $this->commissions();
    }
    public function commissions()
    {
        return OnlineOrderCommission::where('online_order_id', $this->id)->first();
    }

    public function getCalculatedFeesAttribute()
    {

        return $this->calculate_order_fees($this);
    }

    public function calculate_order_fees($order)
    {
        $orderTypeWiseCommission = 0.00;
        $orderTypeWiseServiceCharge = 0.00;
        $orderBasedOrTransactionWiseCharge = 0.00;
        $countryWiseCommission = 0.0;
        $cardBrandWiseCommission = 0.0;
        $payment_method_fees = 0;
        $chargeBackFee = 0;
        $handlingFee = 0;
        $cardCountry = "";
        $brand = "";

        $orderTypeWiseServiceCharge = $order->user_service_charge;
        // if ($order->order_type == 'dinein') {
        //     $serviceCharge = $order->dinein_service_fee;
        // }
        $orderGrandTotal = $order->grand_total;

        if ($order->payment_method == 'paypal') {
            $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->commissions->online_paypal_fee, $order->commissions->online_paypal_fee_type);
        }
        if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online') {
            $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->commissions->online_card_fee, $order->commissions->online_card_fee_type);
        }
        if ($order->payment_method == 'cod')
            $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->commissions->online_cod_fee, $order->commissions->online_cod_fee_type);

        if ($order->order_type == 'dinein') {
            $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->commissions->dinein_commission_fee, $order->commissions->dinein_commission_fee_type);
        } else {
            $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->commissions->online_commission_fee, $order->commissions->online_commission_fee_type);
        }
        if ($order->order_charge_type == 'order_based') {
            $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotal, $order->commissions->per_order_charge, $order->commissions->per_order_charge_type);
        } else {
            if ($order->payment_method == 'paypal')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->commissions->paypal_charge, $order->commissions->paypal_charge_type);
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Credit Card')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->commissions->card_charge, $order->commissions->card_charge_type);
            if ($order->payment_method == 'cod')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->commissions->cod_charge, $order->commissions->cod_charge_type);
        }

        $transaction['order_id'] = "#ODR000" . $order->order_id;
        $transaction['restaurant_id'] = $order->restaurant_id;
        if (empty($order->stripe_txn_id)) {
            $transaction['txn_id'] = $order->online_txn_id;
        } else {
            $transaction['txn_id'] = $order->stripe_txn_id;
        }
        $transaction['order_date'] = $order->order_date;
        $transaction['order_type'] = $order->order_type;
        $transaction['order_status'] = $order->order_status;
        $transaction['payment_method'] = $order->payment_method;
        $transaction['service'] = 'Online';

        $transaction['payment_method_fees'] = $payment_method_fees;
        $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
        $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
        $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;
        $transaction['country'] = $cardCountry;
        $transaction['country_charges'] = $countryWiseCommission;
        $transaction['card_brand'] = $brand;
        $transaction['card_brand_charge'] = $cardBrandWiseCommission;
        $transaction['charge_back_fee'] = $chargeBackFee;
        $transaction['handling_fee'] = $handlingFee;

        $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);
        if ($order->order_status == 'Failed') {
            $transaction['order_grand_total'] = -$this->convertToDecimal($orderGrandTotal);
            $transaction['total_fee'] = 0.0;
        } else {
            $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotal);
        }
        if ($order->payment_method == 'cod') {
            $transaction['net_total'] = $this->convertToDecimal(0 - $transaction['total_fee']);
        } else {
            $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $transaction['total_fee']);
        }

        return $transaction;
    }
    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }
    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
