<?php

namespace App;

use App\Http\Controllers\SiteSettingController;
use App\ModelsConnect\ExternalAccount;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class PayoutInvoice extends Model
{
    protected $fillable = [
        'business_id',
        'gross_total',
        'gross_fee',
        'management_fee',
        'handling_fee',
        'payout_charge',
        'misc_charge',
        'monthly_fee',
        'weekly_fee',
        'total_charge_back_fee',
        'total_charge_back_transactions',
        'total_non_domestic_fee',
        'total_non_domestic_transactions',
        'total_refund_transactions',
        'net_fee',
        'net_total',
        'paid_amount',
        'transaction_type',
        'payout_method',
        'payout_mode',
        'from_date',
        'to_date',
        'status',
        'comment',
        'payout_comment',
        'payout_account',
        'payout_invoice_raw',
    ];
    // protected $hidden = ['updated_at'];
    protected $appends = ['country', 'total_paid_amount'];
    protected $casts = ['payout_account' => 'json', 'payout_invoice_raw' => 'json'];

    // Relations
    public function business()
    {
        return $this->belongsTo(Business::class, 'business_id', 'id');
    }

    // Attributes
    public function getTransactionsAttribute()
    {
        return InvoiceTransaction::where('payout_invoice_id', $this->id)->get();
    }
    public function getSurchargesAttribute()
    {
        return InvoiceSurcharge::where('payout_invoice_id', $this->id)->get();
    }
    public function getInvoicesAttribute()
    {
        if ($this->transaction_type == 'Normal') {
            return Invoice::select("invoices.*", "invoice_payments.amount as amount")->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $this->from_date)->where('date', '<=', $this->to_date)->where('business_id', $this->business_id)->where('invoice_payments.archived', '0')->where('invoices.archived', '0')->get();
        } else {
            return Invoice::select("invoices.*", "invoice_payments.amount as amount")->leftJoin('invoice_payments', 'invoice_payments.invoice_id', '=', 'invoices.id')->where('invoice_type', 'Internal')->where('date', '>=', $this->from_date)->where('date', '<=', $this->to_date)->where('business_id', $this->business_id)->where('invoice_payments.archived', '1')->where('invoices.archived', '1')->get();
        }
    }
    public function getBusinessAttribute()
    {
        return Business::where('id', $this->business_id)->first();
    }
    public function getCountryAttribute()
    {
        return $this->country();
    }
    public function country()
    {
        return Country::where('id', $this->country_id)->first();
    }
    public function getTotalPaidAmountAttribute()
    {
        $total_paid_amount = Passbook::where('business_id', $this->business_id)->where('entity_id', $this->id)->where('entity', 'payout_invoice')->where('amount', '!=', $this->net_total)->sum('amount');

        return $total_paid_amount;
    }
    public function getStatusAttribute()
    {
        $payableAmount = ($this->net_total > 0 ? -1 * ($this->net_total - (-1 * ($this->getTotalPaidAmountAttribute()))) : (-1 * ($this->net_total)) - $this->getTotalPaidAmountAttribute());
        if ($payableAmount == 0) {
            $this->status = "Settled";
            $this->save();
            return "Settled";
        }
        return "Pending";
    }
    public function getPayableTotAmountAttribute()
    {
        $externalAccount = ExternalAccount::where('business_id', $this->business_id)->where('default', 1)->orderByRaw('id DESC')->first();
        if (!empty($externalAccount)) {
            // $stringValue = 'UBSIDI LIMITED,Tiffin Tom,TIFFINTOMUK LTD,Tiffin Tom Limited,Ubsidi Ltd,TIFFINTOM LIMITED,TiffinTomuk Ltd';
            $stringValue = (new SiteSettingController())->findByKey('bank_string')->value;
            if ($externalAccount->account_holder_name == '' || stripos($stringValue, $externalAccount->account_holder_name) !== false) {
                return 1;
            } else {
                return 0;
            }
        } else {
            return 0;
        }
    }
}
