<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class PaypalPayment extends Model
{
    protected $fillable = [
        'payment_id',
        'capture_id',

        'state',

        'description',
        'custom_description',

        'sale_id',
        'sale_state',

        'transaction_amount',
        'transaction_fee',
        'refunded_amount',

        'currency',
        'payer_country',
        'email',
        'non_domestic',
        'sale_created_at',
        'refund_created_at',

    ];
    protected $hidden = ['updated_at'];
    protected $appends = ['calculated_fees'];
    protected $casts = ['non_domestic' => 'bool', 'archived' => 'bool'];
    public function getCalculatedFeesAttribute()
    {
        $order = OnlineOrder::leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id')->where('txn_id', $this->sale_id)->first();
        if (!empty($order)) {
            return $this->calculate_order_fees($this, $order);
        } else {
            return null;
        }
    }
    // Attributes
    //
    public function getOnlineOrderAttribute()
    {
        return OnlineOrder::where('txn_id', str_replace('PAYID-', '', $this->sale_id))->first();
    }
    public function calculate_order_fees($stripeTransaction, $order)
    {
        $orderTypeWiseCommission = 0.00;
        $orderTypeWiseServiceCharge = 0.00;
        $orderBasedOrTransactionWiseCharge = 0.00;
        $countryWiseCommission = 0.0;
        $cardBrandWiseCommission = 0.0;
        $payment_method_fees = 0;
        $chargeBackFee = 0;
        $handlingFee = 0;
        $cardCountry = "";
        $brand = "";
        if ($order->payment_method == 'paypal') {
            $payment_method_fees  = $this->calculateFee($order->grand_total, $order->online_paypal_fee, $order->online_paypal_fee_type);
        }
        if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online') {
            $payment_method_fees  = $this->calculateFee($order->grand_total, $order->online_card_fee, $order->online_card_fee_type);
        }
        if ($order->payment_method == 'cod')
            $payment_method_fees  = $this->calculateFee($order->grand_total, $order->online_cod_fee, $order->online_cod_fee_type);

        if ($order->order_type == 'dinein') {
            $orderTypeWiseCommission = $this->calculateFee($order->grand_total, $order->dinein_commission_fee, $order->dinein_commission_fee_type);
            $orderTypeWiseServiceCharge = $this->calculateFee($order->grand_total, $order->dinein_service_fee, $order->dinein_service_fee_type);
        } else {
            $orderTypeWiseCommission = $this->calculateFee($order->grand_total, $order->online_commission_fee, $order->online_commission_fee_type);
            $orderTypeWiseServiceCharge = $this->calculateFee($order->grand_total, $order->online_service_fee, $order->online_service_fee_type);
        }
        if ($order->order_charge_type == 'order_based') {
            $orderBasedOrTransactionWiseCharge = $this->calculateFee($order->grand_total, $order->per_order_charge, $order->per_order_charge_type);
        } else {
            if ($order->payment_method == 'paypal')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($order->grand_total, $order->paypal_charge, $order->paypal_charge_type);
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Credit Card')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($order->grand_total, $order->card_charge, $order->card_charge_type);
            if ($order->payment_method == 'cod')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($order->grand_total, $order->cod_charge, $order->cod_charge_type);
        }

        $transaction['order_id'] = "#ODR000" . $order->order_id;
        $transaction['restaurant_id'] = $order->restaurant_id;
        if (empty($order->stripe_txn_id)) {
            $transaction['txn_id'] = $order->online_txn_id;
        } else {
            $transaction['txn_id'] = $order->stripe_txn_id;
        }
        $transaction['order_date'] = $order->order_date;
        $transaction['order_type'] = $order->order_type;
        $transaction['order_status'] = $order->order_status;
        $transaction['payment_method'] = $order->payment_method;
        $transaction['service'] = 'Online';

        $transaction['payment_method_fees'] = $payment_method_fees;
        $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
        $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
        $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;
        $transaction['country'] = $cardCountry;
        $transaction['country_charges'] = $countryWiseCommission;
        $transaction['card_brand'] = $brand;
        $transaction['card_brand_charge'] = $cardBrandWiseCommission;
        $transaction['charge_back_fee'] = $chargeBackFee;
        $transaction['handling_fee'] = $handlingFee;

        $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);
        if ($order->order_status == 'Failed') {
            $transaction['order_grand_total'] = -$this->convertToDecimal($order->grand_total);
            $transaction['total_fee'] = 0.0;
        } else {
            $transaction['order_grand_total'] = $this->convertToDecimal($order->grand_total);
        }
        $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $transaction['total_fee']);
        return $transaction;
    }
    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }
    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}
