<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class ResellerMiscInvoice extends Model
{
    protected $fillable = [
        'grand_total',
        'user_id',
        'mode',
        'percentage',
        'status',
        'comment',
        'date',
    ];
    protected $hidden = ['updated_at'];
    protected $appends = ['total_paid_amount'];
    protected $casts = ['grand_total' => 'double', 'percentage' => 'double'];

    // Attributes
    //
    public function getMiscInvoiceItemsAttribute()
    {
        return ResellerMiscInvoiceItem::where('misc_invoice_id', $this->id)->get();
    }
    public function getTotalPaidAmountAttribute()
    {
        $total_paid_amount = Passbook::where('user_id', $this->user_id)->where('entity_id', $this->id)->where('entity', 'misc_invoice')->where('amount', '!=', -1 * $this->grand_total)->sum('amount');
        return $total_paid_amount;
    }
    public function getStatusAttribute()
    {
        $payableAmount = $this->grand_total  - $this->getTotalPaidAmountAttribute();
        if ($payableAmount == 0) {
            $this->status = "Paid";
            $this->save();
            return "Paid";
        }
        if ($this->getTotalPaidAmountAttribute() > 0 &&  $this->getTotalPaidAmountAttribute() < $this->grand_total) {
            $this->status = "Partially Paid";
            $this->save();
            return "Partially Paid";
        }
        return "Pending";
    }
    public function getUserAttribute()
    {
        return User::where('id', $this->user_id)->first();
    }
}
