<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class ResellerPayoutInvoice extends Model
{
    protected $fillable = [
        'business_id',
        'user_id',
        'gross_total',
        'gross_fee',
        'misc_charge',
        'total_charge_back_fee',
        'total_charge_back_transactions',
        'total_non_domestic_fee',
        'total_non_domestic_transactions',
        'total_refund_transactions',
        'net_fee',
        'net_total',
        'paid_amount',
        'transaction_type',
        'payout_method',
        'payout_mode',
        'from_date',
        'to_date',
        'status',
        'comment',
        'payout_comment',
        'payout_account',
    ];
    // protected $hidden = ['updated_at'];
    protected $appends = ['country', 'total_paid_amount'];
    protected $casts = ['payout_account' => 'json', 'payout_invoice_raw' => 'json'];


    // Attributes
    //
    public function getTransactionsAttribute()
    {
        return ResellerInvoiceTransaction::where('reseller_payout_invoice_id', $this->id)->get();
    }
    public function getBusinessAttribute()
    {
        return Business::where('id', $this->business_id)->first();
    }
    public function getResellerAttribute()
    {
        return User::where('id', $this->user_id)->first();
    }
    public function getCountryAttribute()
    {
        return $this->country();
    }
    public function country()
    {
        return Country::where('id', $this->country_id)->first();
    }

    public function getTotalPaidAmountAttribute()
    {
        $total_paid_amount = Passbook::where('user_id', $this->user_id)->where('entity_id', $this->id)->where('entity', 'reseller_payout_invoice')->where('amount', '!=', $this->net_fee)->sum('amount');

        return $total_paid_amount;
    }
    public function getStatusAttribute()
    {
        $payableAmount = ($this->net_fee > 0 ? -1 * ($this->net_fee - (-1 * ($this->getTotalPaidAmountAttribute()))) : (-1 * ($this->net_fee)) - $this->getTotalPaidAmountAttribute());
        if ($payableAmount == 0) {
            $this->status = "Settled";
            $this->save();
            return "Settled";
        }
        return "Pending";
    }
}
