<?php

namespace App;

use App\Helpers\ImageHelper;
use App\Http\Controllers\SiteSettingController;
use Illuminate\Database\Eloquent\Model;

class User extends Model
{
    protected $fillable = [
        'first_name', 'last_name', 'image', 'parent_id', 'dob', 'contact_emails', 'contact_numbers', 'contact_type', 'residential_address', 'login_email', 'password', 'reseller', 'disabled', 'commission_type', 'commission',
        'door_no',
        'city',
        'state',
        'postcode',
        'payout_comment',
        'payout_mode',
        'payout_method',
        'archived_payout_mode',
        'archived_payout_method',
        'permissions',
        'allow_3rd_party_use'
    ];
    protected $hidden = ['password', 'updated_at', 'otp', 'secret_key', 'initial_vector'];
    protected $appends = ['business_counts', 'country', 'image_url', 'thumb_url', 'balance'];
    protected $casts = ['disabled' => 'bool', 'mobile' => 'bool', 'reseller' => 'bool', 'permissions' => 'json', 'allow_3rd_party_use' => 'bool',];

    public function getImageUrlAttribute()
    {
        return ImageHelper::getServerImageUrl(ImageHelper::$TYPE_USER, $this->image);
    }
    public function getThumbUrlAttribute()
    {
        return ImageHelper::getServerThumbUrl(ImageHelper::$TYPE_USER, $this->image);
    }
    public function getBusinessCountsAttribute()
    {
        return $this->businessCounts();
    }
    public function getDefaultBankDetailAttribute()
    {
        return $this->bank_detail();
    }
    public function getDefaultPaypalDetailAttribute()
    {
        return $this->paypal_detail();
    }
    public function getBalanceAttribute()
    {
        return Passbook::where('user_id', $this->id)->sum('amount');
    }
    public function getCountryAttribute()
    {
        return $this->country();
    }
    public function getSelectedBusinessAttribute()
    {
        // $business = Business::whereRaw('id = (SELECT business_id from user_selected_businesses WHERE user_id= ? ORDER BY id DESC LIMIT 1)', [$this->id])->latest()->first();
        $business = Business::selectRaw("restaurant_id,online_service,merchant_service,id,name,type,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,online_service,country_id,supervisor_password,admin_password,s_account_id,connect_service,issuing_service,pouch_service,pouch_weight,load_wallet,load_bank")->where('epos_service', 1)->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$this->id])->latest()->first();
        if (empty($business)) {
            $business = Business::selectRaw("restaurant_id,online_service,merchant_service,id,name,type,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,online_service,country_id,supervisor_password,admin_password,s_account_id,connect_service,issuing_service,pouch_service,pouch_weight,load_wallet,load_bank")->where('merchant_service', 1)->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$this->id])->latest()->first();
            if (empty($business)) {
                $business = Business::selectRaw("restaurant_id,online_service,merchant_service,id,name,type,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,online_service,country_id,supervisor_password,admin_password,s_account_id,connect_service,issuing_service,pouch_service,pouch_weight,load_wallet,load_bank")->where('online_service', 1)->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$this->id])->latest()->first();
                if (empty($business)) {
                    $business = Business::selectRaw("restaurant_id,online_service,merchant_service,id,name,type,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,online_service,country_id,supervisor_password,admin_password,s_account_id,connect_service,issuing_service,pouch_service,pouch_weight,load_wallet,load_bank")->where('issuing_service', 1)->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$this->id])->latest()->first();
                    if (empty($business)) {
                        $business = Business::selectRaw("restaurant_id,online_service,merchant_service,id,name,type,tip,statement_description,statement_descriptor,stripe_mode,stripe_public_key_test,stripe_private_key_test,stripe_public_key_live,stripe_private_key_live,paypal_mode,paypal_public_key_test,paypal_private_key_test,paypal_public_key_live,paypal_private_key_live,epos_service,merchant_service,online_service,country_id,supervisor_password,admin_password,s_account_id,connect_service,issuing_service,pouch_service,pouch_weight,load_wallet,load_bank")->where('pouch_service', 1)->whereRaw('(id IN (SELECT business_id from user_businesses where user_id = ?))', [$this->id])->latest()->first();
                        if (empty($business)) {
                            return null;
                        }
                    }
                }
            }
        };
        $business->append('business_commissions');
        $business->makeHidden('user_counts');
        $business->append('default_bank');
        $business->append('plans');

        if ($business->s_account_id != null) {
            $connectMode = (new SiteSettingController())->findByKey('connect_stripe_mode')->value;
            if ($connectMode == 'test') {
                $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_test')->value;
                $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_test')->value;
            }
            if ($connectMode == 'live') {
                $business->connect_stripe_public_key = (new SiteSettingController())->findByKey('connect_stripe_public_key_live')->value;
                $business->connect_stripe_private_key = (new SiteSettingController())->findByKey('connect_stripe_private_key_live')->value;
            }
        }


        return $business;
    }
    // Attributes
    //
    public function businessCounts()
    {
        return UserBusiness::where('user_id', $this->id)->count();
    }
    public function bank_detail()
    {
        return BankDetail::where('id', $this->bank_detail_id)->first();
    }
    public function paypal_detail()
    {
        return PaypalDetail::where('id', $this->paypal_detail_id)->first();
    }
    public function country()
    {
        return Country::where('id', $this->country_id)->first();
    }

    public function bankDetails()
    {
        return $this->hasMany(BankDetail::class, 'user_id', 'id');
    }
}