<?php

require_once __DIR__ . '/../vendor/autoload.php';

(new Laravel\Lumen\Bootstrap\LoadEnvironmentVariables(
    dirname(__DIR__)
))->bootstrap();
date_default_timezone_set(env('APP_TIMEZONE', 'Europe/London'));
/*
|--------------------------------------------------------------------------
| Create The Application
|--------------------------------------------------------------------------
|
| Here we will load the environment and create the application instance
| that serves as the central piece of this framework. We'll use this
| application as an "IoC" container and router for this framework.
|
*/

$app = new Laravel\Lumen\Application(
    dirname(__DIR__)
);

$app->withFacades();
$app->withEloquent();
// $app->configure('cors');
$app->configure('filesystems');
$app->configure('mail');
/*
|--------------------------------------------------------------------------
| Register Container Bindings
|--------------------------------------------------------------------------
|
| Now we will register a few bindings in the service container. We will
| register the exception handler and the console kernel. You may add
| your own bindings here if you like or you can make another file.
|
*/

$app->singleton(
    Illuminate\Contracts\Debug\ExceptionHandler::class,
    App\Exceptions\Handler::class
);

$app->singleton(
    Illuminate\Contracts\Console\Kernel::class,
    App\Console\Kernel::class
);

/*
|--------------------------------------------------------------------------
| Register Middleware
|--------------------------------------------------------------------------
|
| Next, we will register the middleware with the application. These can
| be global middleware that run before and after each request into a
| route or middleware that'll be assigned to some specific routes.
|
*/

$app->middleware([
    // Spatie\Cors\Cors::class,
    App\Http\Middleware\CorsMiddleware::class,
    App\Http\Middleware\ParseToken::class,
]);

$app->routeMiddleware([
    'role' => App\Http\Middleware\Role::class,
    'outsider' => App\Http\Middleware\UserHasBusinessMiddleware::class,
]);

/*
|--------------------------------------------------------------------------
| Register Service Providers
|--------------------------------------------------------------------------
|
| Here we will register all of the application's service providers which
| are used to bind services into the container. Service providers are
| totally optional, so you are not required to uncomment this line.
|
*/

// $app->register(App\Providers\AppServiceProvider::class);
// $app->register(App\Providers\AuthServiceProvider::class);
// $app->register(App\Providers\EventServiceProvider::class);
//$app->register(Spatie\Cors\CorsServiceProvider::class);
$app->register(Maatwebsite\Excel\ExcelServiceProvider::class);
$app->register(Intervention\Image\ImageServiceProvider::class);
$app->register(Flipbox\LumenGenerator\LumenGeneratorServiceProvider::class);
$app->register(Illuminate\Mail\MailServiceProvider::class);

$app->alias('mail.manager', Illuminate\Mail\MailManager::class);
$app->alias('mail.manager', Illuminate\Contracts\Mail\Factory::class);

$app->alias('mailer', Illuminate\Mail\Mailer::class);
$app->alias('mailer', Illuminate\Contracts\Mail\Mailer::class);
$app->alias('mailer', Illuminate\Contracts\Mail\MailQueue::class);
/*
|--------------------------------------------------------------------------
| Load The Application Routes
|--------------------------------------------------------------------------
|
| Next we will include the routes file so that they can all be added to
| the application. This will provide all of the URLs the application
| can respond to, as well as the controllers that may handle them.
|
*/

$app->router->group([
    'namespace' => 'App\Http\Controllers',
], function ($router) {
    require __DIR__ . '/../routes/web.php';
});

$app->router->group([
    'namespace' => 'App\Http\Controllers',
    'prefix' => 'reseller',
], function ($router) {
    require __DIR__ . '/../routes/reseller.php';
});

$app->router->group([
    'namespace' => 'App\Http\Controllers',
    'prefix' => 'merchant',
], function ($router) {
    require __DIR__ . '/../routes/merchant.php';
});

$app->router->group([
    // 'namespace' => 'App\Http\Controllers',
    'prefix' => 'pg',
], function ($router) {
    require __DIR__ . '/../routes/third-party.php';
});

$app->router->group([
    'namespace' => 'App\Http\Controllers',
    'prefix' => 'epos/merchant',
], function ($router) {
    require __DIR__ . '/../routes/merchant.php';
});
$app->router->group([
    'namespace' => 'App\Http\Controllers\Epos',
    'prefix' => 'epos',
], function ($router) {
    require __DIR__ . '/../routes/epos.php';
});
$app->router->group([
    'namespace' => 'App\Http\Controllers\Connect',
    'prefix' => 'connect',
], function ($router) {
    require __DIR__ . '/../routes/connect.php';
});
$app->router->group([
    'namespace' => 'App\Http\Controllers\Tiffintom',
    'prefix' => 'tiffintom',
], function ($router) {
    require __DIR__ . '/../routes/tiffintom.php';
});
$app->router->group([
    'namespace' => 'App\Http\Controllers\Tiffintom',
    'prefix' => 'online-desk-integration',
], function ($router) {
    require __DIR__ . '/../routes/restaurant.php';
});
$app->router->group([
    'namespace' => 'App\Http\Controllers\Tiffintom',
    'prefix' => 'epos/tiffintom',
], function ($router) {
    require __DIR__ . '/../routes/tiffintom.php';
});
$app->router->group([
    'namespace' => 'App\Http\Controllers',
    'prefix' => 'api',
], function ($router) {
    require __DIR__ . '/../routes/api.php';
});

return $app;
