<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateProductsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('products', function (Blueprint $table) {
            $table->id();
            $table->string('name');
            $table->string('short_name')->nullable();
            $table->string('description')->nullable();
            $table->string('image')->nullable();
            $table->string('top_color')->nullable();
            $table->string('bottom_color')->nullable();

            $table->unsignedBigInteger('parent_id')->nullable();
            $table->unsignedBigInteger('category_id')->nullable();
            $table->unsignedBigInteger('preparation_location_id')->nullable();

            $table->decimal('price', 10, 2)->default(0);
            $table->decimal('takeaway_price', 10, 2)->default(0);
            $table->decimal('web_price', 10, 2)->default(0);
            $table->decimal('delivery_price', 10, 2)->default(0);
            $table->decimal('waiting_price', 10, 2)->default(0);

            $table->boolean('is_dinein')->default(1);
            $table->boolean('is_delivery')->default(1);
            $table->boolean('is_collection')->default(1);
            $table->boolean('is_banquet')->default(1);

            $table->boolean('favourite')->default(0);
            $table->boolean('bar_favourite')->default(0);
            $table->boolean('tax_included')->default(0);
            $table->boolean('taxable')->default(0);
            $table->boolean('auto_modify')->default(0);
            $table->boolean('stockable')->default(0);
            $table->boolean('misc')->default(0);
            $table->integer('sequence')->default(1);

            $table->boolean('disabled')->default(0);

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('parent_id')->references('id')->on('products')->onDelete('set null');
            $table->foreign('category_id')->references('id')->on('categories')->onDelete('set null');
            $table->foreign('preparation_location_id')->references('id')->on('preparation_locations')->onDelete('set null');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('products');
    }
}
