<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateOrdersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('orders', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('customer_id');
            $table->unsignedBigInteger('table_id')->nullable();
            $table->unsignedBigInteger('order_type_id');
            $table->unsignedBigInteger('order_status_id');
            $table->unsignedBigInteger('preparation_location_id')->nullable();
            $table->unsignedBigInteger('updater_id')->nullable();
            $table->unsignedBigInteger('order_action_id')->nullable();
            $table->decimal('sub_total', 10, 2)->default(0);
            $table->decimal('tax', 10, 2)->default(0);
            $table->decimal('service_charge', 10, 2)->default(0);
            $table->decimal('delivery_charge', 10, 2)->default(0);
            $table->decimal('gratuity', 10, 2)->default(0);
            $table->decimal('discount', 10, 2)->default(0);
            $table->decimal('total', 10, 2)->default(0);
            $table->decimal('total_paid', 10, 2)->default(0);
            $table->longText('comment')->nullable();
            $table->enum('split_type', ['order_item', 'person'])->nullable();
            $table->integer('split_count')->nullable();
            $table->unsignedBigInteger('device_id')->nullable();
            $table->string('no_guest')->nullable();
            $table->string('distance')->nullable();

            $table->timestamp('delivery_date')->nullable();

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('customer_id')->references('id')->on('customers')->onDelete('cascade');
            $table->foreign('table_id')->references('id')->on('tables')->onDelete('set null');
            $table->foreign('updater_id')->references('id')->on('users')->onDelete('set null');
            $table->foreign('preparation_location_id')->references('id')->on('preparation_locations')->onDelete('set null');
            $table->foreign('order_type_id')->references('id')->on('order_types')->onDelete('restrict');
            $table->foreign('order_action_id')->references('id')->on('order_actions')->onDelete('restrict');
            $table->foreign('order_status_id')->references('id')->on('order_statuses')->onDelete('restrict');
            $table->foreign('device_id')->references('id')->on('devices')->onDelete('set null');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('orders');
    }
}
