<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateReservationsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('reservations', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('customer_id')->nullable();
            $table->unsignedBigInteger('updater_id')->nullable();
            $table->unsignedBigInteger('table_id')->nullable();
            $table->unsignedBigInteger('reservation_status_id')->nullable();
            $table->unsignedBigInteger('reservation_confirmation_id')->nullable();
            $table->unsignedBigInteger('deposit_type_id')->nullable();
            $table->decimal('deposit_amount', 10, 2)->default(0);
            $table->integer('children')->default(0);
            $table->integer('infants')->default(0);
            $table->integer('diners')->default(0);
            $table->string('telephone')->nullable();
            $table->timestamp('reservation_date_time')->nullable();
            $table->string('special_instruction')->nullable();

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('updater_id')->references('id')->on('users')->onDelete('set null');
            $table->foreign('table_id')->references('id')->on('tables')->onDelete('set null');
            $table->foreign('reservation_status_id')->references('id')->on('reservation_statuses')->onDelete('set null');
            $table->foreign('reservation_confirmation_id')->references('id')->on('reservation_confirmations')->onDelete('set null');
            $table->foreign('deposit_type_id')->references('id')->on('deposit_types')->onDelete('set null');
            $table->foreign('customer_id')->references('id')->on('customers')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('reservations');
    }
}
