<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateUsersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('users', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->string('first_name', 255);
            $table->string('last_name', 255);
            $table->longText('image', 255)->nullable();
            $table->unsignedBigInteger('parent_id')->nullable();
            $table->date('dob')->nullable();
            $table->longText('contact_emails')->nullable();
            $table->longText('contact_numbers')->nullable();
            $table->longText('residential_address')->nullable();

            $table->string('login_email', 255);
            $table->longText('password');

            $table->boolean('reseller')->default(0);
            $table->enum('commission_type', ['flat', 'percentage'])->default('percentage');
            $table->double('commission')->default(0);

            $table->boolean('disabled')->default(0);

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('parent_id')->references('id')->on('users')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('users');
    }
}
