<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateBusinessesTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('businesses', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->longText('name');
            $table->longText('address')->nullable();
            $table->longText('email')->nullable();
            $table->longText('contact_numbers')->nullable();
            $table->date('joining_date')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->string('statement_description')->nullable();
            $table->string('statement_descriptor')->nullable();
            $table->string('online_statement_description')->nullable();
            $table->string('online_statement_descriptor')->nullable();
            $table->string('domain')->nullable();
            $table->date('domain_valid_till')->nullable();
            $table->enum('type', ['Sole trader', 'Partnership', 'Limited company', 'Not registered comapny', 'Other'])->default('other');
            $table->string('restaurant_id')->nullable();
            $table->unsignedBigInteger('country_id')->nullable();

            $table->boolean('disabled')->default(0);

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('country_id')->references('id')->on('countries')->onDelete('set null');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('businesses');
    }
}
