<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateStripeTransactionsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('stripe_transactions', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->unsignedBigInteger('business_id')->nullable();
            $table->string('txn_id')->unique();
            $table->string('payment_intent_id')->nullable();
            $table->enum('service', ['Merchant', 'Online'])->nullable();
            $table->string('object');
            $table->longText('description')->nullable();
            $table->longText('descriptor')->nullable();
            $table->integer('amount')->default(0);
            $table->integer('fee')->default(0);
            $table->integer('net')->default(0);
            $table->string('currency')->nullable();
            $table->string('reporting_category');
            $table->string('source');
            $table->string('status');
            $table->string('type');
            $table->string('card_type')->nullable();
            $table->string('card_brand')->nullable();
            $table->string('card_country')->nullable();
            $table->string('available_on');
            $table->string('created');

            $table->enum('cash_handling_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('cash_handling_fee')->default(0);

            $table->enum('management_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('management_fee')->default(0);

            $table->enum('merchant_card_processing_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('merchant_card_processing_fee')->default(0);

            $table->enum('merchant_card_auth_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('merchant_card_auth_fee')->default(0);

            $table->enum('charges_refundable', ['no', 'yes'])->default('no');

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('business_id')->references('id')->on('businesses')->onDelete('set NULL');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('stripe_transactions');
    }
}
