<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateOnlineOrdersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('online_orders', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->string('order_id')->nullable();
            $table->unsignedBigInteger('business_id')->nullable();
            $table->string('restaurant_id')->nullable();
            $table->enum('order_type', ['delivery', 'pickup', 'dinein'])->default('delivery');
            $table->string('txn_id')->nullable();
            $table->string('payment_method')->nullable();
            $table->double('sub_total')->default(0);
            $table->double('delivery_charge')->default(0);
            $table->double('driver_tip')->default(0);
            $table->double('gratuity_tip')->default(0);
            $table->double('grand_total')->default(0);
            $table->double('charity_amount')->default(0);
            $table->double('voucher_amount')->default(0);
            $table->double('user_service_charge')->default(0);
            $table->string('order_status')->nullable();
            $table->timestamp('order_date')->default(DB::raw('CURRENT_TIMESTAMP'));

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('business_id')->references('id')->on('businesses')->onDelete('set null');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('online_orders');
    }
}
