<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateOrderCommissionsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('order_commissions', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->unsignedBigInteger('online_order_id');

            $table->enum('online_commission_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('online_commission_fee')->default(0);

            $table->enum('dinein_commission_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('dinein_commission_fee')->default(0);

            $table->enum('online_cod_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('online_cod_fee')->default(0);

            $table->enum('online_paypal_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('online_paypal_fee')->default(0);

            $table->enum('online_card_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('online_card_fee')->default(0);

            // May be this is nakkami, because it occurs on end user not on restaurant
            $table->enum('online_service_fee_status', ['show', 'hide'])->default('show');
            $table->enum('online_service_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('online_service_fee')->default(0);

            // May be this is nakkami, because it occurs on end user not on restaurant
            $table->enum('dinein_service_fee_status', ['show', 'hide'])->default('show');
            $table->enum('dinein_service_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('dinein_service_fee')->default(0);

            $table->enum('order_charge_type', ['order_based', 'transaction_based'])->default('order_based');

            $table->enum('per_order_charge_type', ['flat', 'percentage'])->default('percentage');
            $table->double('per_order_charge')->default(0);

            $table->enum('cod_charge_type', ['flat', 'percentage'])->default('percentage');
            $table->double('cod_charge')->default(0);

            $table->enum('paypal_charge_type', ['flat', 'percentage'])->default('percentage');
            $table->double('paypal_charge')->default(0);

            $table->enum('card_charge_type', ['flat', 'percentage'])->default('percentage');
            $table->double('card_charge')->default(0);

            $table->enum('charges_refundable', ['no', 'yes'])->default('no');

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('online_order_id')->references('id')->on('online_orders')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('order_commissions');
    }
}
