<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreatePayoutInvoicesTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('payout_invoices', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->unsignedBigInteger('business_id');
            $table->double('gross_total')->default(0);
            $table->double('gross_fee')->default(0);
            $table->double('misc_charge')->default(0);
            $table->double('net_fee')->default(0);
            $table->double('net_total')->default(0);
            $table->double('paid_total')->default(0);
            $table->enum('transaction_type', ['Normal', 'Archived'])->default('Normal');

            $table->enum('payout_mode', ['Auto', 'Manual'])->default('Manual');
            $table->enum('payout_method', ['Bank', 'Paypal', 'Cash'])->default('Cash');

            $table->date('from_date');
            $table->date('to_date');

            $table->enum('status', ['Pending', 'Partially Settled', 'Settled']);

            $table->longText('comment')->nullable();
            $table->json('payout_account')->nullable();

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('business_id')->references('id')->on('businesses')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('payout_invoices');
    }
}
