<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateInvoiceTransactionsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('invoice_transactions', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->unsignedBigInteger('payout_invoice_id');

            $table->string('order_id')->nullable();
            $table->string('restaurant_id')->nullable();
            $table->string('txn_id')->nullable();
            $table->timestamp('order_date')->nullable();
            $table->string('order_type');
            $table->string('order_status');
            $table->string('payment_method');

            $table->string('country')->nullable();
            $table->string('card_brand')->nullable();

            $table->double('country_charges')->default(0);
            $table->double('card_brand_charge')->default(0);

            $table->double('processing_fee')->default(0);
            $table->double('auth_fee')->default(0);
            $table->double('handling_fee')->default(0);
            $table->double('management_fee')->default(0);

            $table->double('payment_method_fees')->default(0);
            $table->double('order_type_wise_commission')->default(0);
            $table->double('order_type_wise_service_charge')->default(0);
            $table->double('order_based_or_transaction_wise_charge')->default(0);

            $table->double('order_grand_total')->default(0);
            $table->double('total_fee')->default(0);
            $table->double('net_total')->default(0);

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('payout_invoice_id')->references('id')->on('payout_invoices')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('invoice_transactions');
    }
}
