<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateBusinessCardReadersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('business_card_readers', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('business_id');
            $table->unsignedBigInteger('card_reader_id')->nullable();
            $table->string('name');
            $table->enum('connectivity', ['Bluetooth', 'Wifi']);
            $table->string('ip', 15)->nullable();
            $table->string('port', 5)->nullable();
            $table->longText('merchant_key')->nullable();
            $table->longText('merchant_secret')->nullable();

            $table->boolean('disabled')->default(0);

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('business_id')->references('id')->on('businesses')->onDelete('cascade');
            $table->foreign('card_reader_id')->references('id')->on('card_readers')->onDelete('set null');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('business_card_readers');
    }
}
