<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateResellerStripeCommissionsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('reseller_stripe_commissions', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('user_id');
            $table->unsignedBigInteger('stripe_transaction_id');

            $table->enum('cash_handling_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('cash_handling_fee')->default(0);

            $table->enum('management_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('management_fee')->default(0);

            $table->enum('merchant_card_processing_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('merchant_card_processing_fee')->default(0);

            $table->enum('merchant_card_auth_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('merchant_card_auth_fee')->default(0);

            $table->enum('charge_back_fee_type', ['flat', 'percentage'])->default('percentage');
            $table->double('charge_back_fee')->default(0);

            $table->enum('charges_refundable', ['no', 'yes'])->default('no');

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('user_id')->references('id')->on('users')->onDelete('cascade');
            $table->foreign('stripe_transaction_id')->references('id')->on('stripe_transactions')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('reseller_stripe_commissions');
    }
}
