<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateIssuingCardsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('issuing_cards', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('business_id');
            $table->unsignedBigInteger('cardholder_id');

            $table->string('card_id');
            $table->string('brand');
            $table->string('cancellation_reason')->nullable();
            $table->string('currency')->nullable();
            $table->string('exp_month')->nullable();
            $table->string('exp_year')->nullable();
            $table->string('last4')->nullable();

            $table->boolean('livemode')->default(0);

            $table->string('replaced_by')->nullable();
            $table->string('replacement_for')->nullable();
            $table->string('replacement_reason')->nullable();

            $table->enum('type', ['physical', 'virtual'])->default('virtual');
            $table->enum('status', ['active', 'inactive'])->default('active');

            $table->enum('service', ['standard', 'express', 'priority'])->nullable();
            $table->enum('shipping_type', ['individual', 'bulk'])->nullable();

            $table->string('city')->nullable();
            $table->string('country')->nullable();
            $table->string('line1')->nullable();
            $table->string('postal_code')->nullable();
            $table->string('line2')->nullable();
            $table->string('state')->nullable();
            $table->string('created');

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('business_id')->references('id')->on('businesses')->onDelete('cascade');
            $table->foreign('cardholder_id')->references('id')->on('card_holders')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('issuing_cards');
    }
}
