<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateInvoicesTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('invoices', function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->enum('invoice_type', ['Internal', 'External', 'Customer'])->default('Internal');
            $table->unsignedBigInteger('business_id')->nullable();
            $table->longText('header')->nullable();
            $table->longText('bill_to')->nullable();
            $table->longText('ship_to')->nullable();
            $table->date('date');
            $table->double('sub_total')->default(0);
            $table->double('grand_total')->default(0);
            $table->enum('tax_mode', ['Percentage', 'Flat'])->default('Percentage');
            $table->double('tax')->default(0);
            $table->enum('discount_mode', ['Percentage', 'Flat'])->default('Percentage');
            $table->double('discount')->default(0);
            $table->longText('notes')->nullable();
            $table->longText('terms')->nullable();
            $table->enum('status', ['Unpaid', 'Paid', 'Cancel', 'Dispute'])->default('Unpaid');

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('business_id')->references('id')->on('businesses')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('invoices');
    }
}
