<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreateFilesTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('files', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('connect_account_id');
            $table->unsignedBigInteger('person_id')->nullable();
            $table->unsignedBigInteger('company_id')->nullable();
            $table->string('s_account_id')->nullable();
            $table->string('s_person_id')->nullable();
            $table->string('s_file_id')->nullable();

            $table->enum('proof_type', ['document_front', 'document_back', 'additional_front', 'additional_back', 'other']);
            $table->enum('purpose', ['account_requirement', 'business_icon', 'business_logo', 'customer_signature', 'dispute_evidence', 'identity_document', 'pci_document', 'tax_document_user_upload', 'additional_verification'])->default('identity_document');
            $table->string('file');

            $table->timestamp('created_at')->default(DB::raw('CURRENT_TIMESTAMP'));
            $table->timestamp('updated_at')->default(DB::raw('CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP'));

            $table->foreign('connect_account_id')->references('id')->on('connect_accounts')->onDelete('cascade');
            $table->foreign('person_id')->references('id')->on('persons')->onDelete('cascade');
            $table->foreign('company_id')->references('id')->on('companies')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('files');
    }
}
