var url_string = window.location.href;
var url = new URL(url_string);
var paramAmount = parseInt(url.searchParams.get("amount"));
var paramBusinessId = url.searchParams.get("business_id");
var locationId = url.searchParams.get("s_location_id");
var terminalId = url.searchParams.get("s_terminal_id");

var currency = "£";
var selectedCardReader;
var discoveredReaders;
var isSimulated = false;
var baseUrl = "https://testrms.tiffintom.com/public/";
if (url.hostname.includes("ubsidi")) {
    baseUrl = "https://www.ubsidi.com/api/public/"
}
// DOM Elements
buttonPayNow = document.getElementById("pay_now_button");
buttonPayNowText = document.getElementById("pay_now_button_text");
buttonResponse = document.getElementById("response_button");
buttonResponseText = document.getElementById("response_button_text");

loader = document.getElementById("loader");
amountText = document.getElementById("amount");
messageText = document.getElementById("message");

// Terminal Object
var terminal = StripeTerminal.create({
    onFetchConnectionToken: fetchConnectionToken,
    onUnexpectedReaderDisconnect: unexpectedDisconnect,
});



if (paramAmount == null || isNaN(paramAmount)) {
    messageText.innerHTML = "Invalid amount";
    messageText.classList.remove("invisible");

    buttonPayNow.innerHTML = "Pay Now";
    buttonPayNow.classList.add("disabled");

}

amountText.innerHTML = currency + " " + parseFloat(paramAmount / 100).toFixed(2);

pay_now_button_text.innerHTML = "Pay Now";
buttonPayNow.classList.remove("disabled");
buttonPayNow.click(); // <
function payNowButton() {
    loader.classList.remove('d-none');
    buttonPayNow.classList.add("disabled");

    discoverReaders();
}

function unexpectedDisconnect() {
    // In this function, your app should notify the user that the reader disconnected.
    // You can also include a way to attempt to reconnect to a reader.
    console.log("Disconnected from reader");
}

function fetchConnectionToken() {
    // Your backend should call /v1/terminal/connection_tokens and return the JSON response from Stripe
    let token = null;
    var bodyParams = {}
    bodyParams['business_id'] = paramBusinessId;
    if (locationId != null && locationId != NaN && locationId != "") {
        bodyParams['s_location_id'] = locationId;
    }

    return fetch(baseUrl + "api/pg/card-reader-connection-token", {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(bodyParams),
        })
        .then(function (response) {
            return response.json();
        })
        .then(function (data) {
            if (data.code) {
                showMessage(data.message, 'error');
                showResponseButton("Payment failed", "error");
                return null;
            }

            token = data.token;
            return token;
        });
}


function discoverReaders() {
    const config = {
        simulated: isSimulated,
        location: locationId
    };

    terminal.discoverReaders(config).then(function (discoverResult) {
        if (discoverResult.error) {
            console.log(connectResult.error);
            showMessage("Could not find card readers, Please make sure card readers are in same network", 'error');
            showResponseButton("Payment failed", "error");
        } else if (discoverResult.discoveredReaders.length === 0) {
            showMessage("No readers found", 'error');
            showResponseButton("Payment failed", "error");
        } else {
            messageText.classList.add("invisible");

            //We have found list of readers
            discoveredReaders = discoverResult.discoveredReaders;

            /**
             * check if param terminal id matches with discovered list
             * if terminal not found, then show discovered list and on click of that list item
             * connect that reader and make payment
             **/

            if (terminalId != null) {
                discoveredReaders.forEach((reader) => {
                    if (terminalId == reader.id) {
                        selectedCardReader = reader;
                    }
                });
                if (selectedCardReader == null) {
                    displayAllTerminals(discoveredReaders);
                } else {
                    connectReader(selectedCardReader);
                }
            } else {
                displayAllTerminals(discoveredReaders);
            }

        }
    });
}

function displayAllTerminals(readersList) {
    buttonPayNow.classList.add('d-none');
    var cardReadersDOM = document.getElementById('card_readers');
    var listHtml = '<div class="card">';
    listHtml = listHtml + '<div class="card-header text-center">Select Card Reader</div>';
    listHtml = listHtml + '<div class="card-body p-0">';
    listHtml = listHtml + '<ul class="list-group">';
    readersList.forEach((reader) => {
        listHtml = listHtml + '<li class="list-group-item">';
        listHtml = listHtml + '<div class="d-flex align-items-center justify-content-between ">';
        listHtml = listHtml + reader.label + ' <span class="btn-primary btn btn-sm " onclick="findAndConnectReader(\'' + reader.id + '\')">Select</span>';
        listHtml = listHtml + '</div>';
        listHtml = listHtml + '</li>';

    });

    listHtml = listHtml + '</ul>';
    listHtml = listHtml + '</div>';
    listHtml = listHtml + '</div>';
    cardReadersDOM.innerHTML = listHtml;
    document.getElementById('card_readers').classList.remove('d-none');
}

function findAndConnectReader(readerId) {

    discoveredReaders.forEach((reader) => {
        if (reader.id == readerId) {
            selectedCardReader = reader;
        }
    });
    if (selectedCardReader != null) {
        connectReader(selectedCardReader);
    } else {
        showMessage("No readers found", 'error');
        showResponseButton("Payment failed", "error");
    }
}

function connectReader(selectedReader) {
    document.getElementById('card_readers').innerHTML = "";
    document.getElementById('card_readers').classList.add('d-none');
    buttonPayNow.classList.remove('d-none');

    terminal.connectReader(selectedReader, {
        fail_if_in_use: true
    }).then(function (connectResult) {
        if (connectResult.error) {
            console.log(connectResult.error);
            showMessage("Could not find card readers, Please make sure card readers are in same network", 'error');
            showResponseButton("Payment failed", "error");
        } else {
            createPatmentIntent(paramAmount);
        }
    });
}


function createPatmentIntent(amount) {
    fetch(baseUrl + "api/pg/payment-intent", {
            method: "POST",
            headers: {
                "Content-Type": "application/json",
            },
            body: JSON.stringify({
                amount: amount,
                s_terminal_id: selectedCardReader.id,
                transaction_type: "merchant",
                business_id: paramBusinessId,
            }),
        })
        .then(function (response) {
            return response.json();
        })
        .then(function (data) {
            if (data.code) {
                showMessage(data.message, 'error');
                showResponseButton("Payment failed", "error");

                return;
            }
            showMessage("Please present the card", 'info')

            var client_secret = data.payment_intent.client_secret;
            if (isSimulated) {
                terminal.setSimulatorConfiguration({
                    testCardNumber: '4242424242424242'
                });
            }
            terminal.collectPaymentMethod(client_secret).then(function (result) {
                if (result.error) {
                    console.log(result.error);
                    showMessage(result.error.message, 'error');
                    showResponseButton("Payment failed", "error");
                } else {
                    var paymentIntent = result.paymentIntent;
                    messageText.innerHTML = "Processing payment";
                    messageText.classList.add("invisible");

                    // Placeholder for processing result.paymentIntent
                    terminal.processPayment(paymentIntent).then(function (result) {
                        if (result.error) {

                            showMessage("Error while processing payment", 'error');
                            showResponseButton("Payment failed", "error");

                        } else if (result.paymentIntent) {

                            // Capture payment intent
                            fetch(
                                    baseUrl + "api/pg/payment-intent-capture", {
                                        method: "POST",
                                        headers: {
                                            "Content-Type": "application/json",
                                        },
                                        body: JSON.stringify({
                                            payment_intent_id: paymentIntent.id,
                                            s_terminal_id: selectedCardReader.id,
                                            business_id: paramBusinessId,
                                        }),
                                    }
                                )
                                .then(function (response) {
                                    return response.json();
                                })
                                .then(function (data) {

                                    if (data.code) {
                                        showMessage(data.message, 'error');
                                        showResponseButton(data.message, "error");
                                        return;
                                    }
                                    showMessage("Payment successful", 'success', data);
                                    showResponseButton("Payment successful", "success");
                                });
                        }
                    });
                }
            });
        });
}

function showResponseButton(message, status) {
    buttonPayNow.classList.add("d-none");
    buttonResponse.classList.remove("d-none");
    buttonResponse.classList.remove('disabled');
    buttonResponseText.innerHTML = message
    if (status == 'error') {
        buttonResponse.classList.remove('btn-success');
        buttonResponse.classList.add('btn-danger');

    } else if (status == 'success') {
        buttonResponse.classList.add('btn-success');
        buttonResponse.classList.remove('btn-danger');
    }
    buttonResponse.classList.add('d-none');
}

function showMessage(message, status, data = null) {
    messageText.innerHTML = message;
    messageText.classList.remove("invisible");
    document.getElementById('action_button').classList.remove("btn-primary");
    document.getElementById('action_button').classList.remove("d-none");
    if (status == "success") {
        document.getElementById('action_button').classList.remove("btn-danger");
        document.getElementById('action_button').classList.add("btn-success");
        document.getElementById('action_button_text').innerHTML = "Close";
        document.getElementById('action_button').classList.add('disabled');
        window.location.href = "./success?status=1&payment_intent_id=" + data.id + "&amount=" + data.amount + "&client_secret=" + data.client_secret;
    } else if (status == "error") {
        document.getElementById('action_button').classList.add("btn-danger");
        document.getElementById('action_button').classList.remove("btn-success");
        document.getElementById('action_button_text').innerHTML = "Exit";
    } else {
        document.getElementById('action_button').classList.add("d-none");
    }

}

function action() {
    var action = document.getElementById('action_button_text').innerHTML;
    if (action.toLowerCase() == 'retry') {
        window.location.reload();
    } else if (action.toLowerCase() == 'exit') {
        window.location.href = "./failed?success=0" + "&amount=" + paramAmount;
    }
}

function finish() {

}
