class Ubsidi {
  constructor(ubsidiKey) {
    this.ubsidiKey = ubsidiKey;
  }

  createCardToken(cardNumber, expiryMonth, expiryYear, cvv) {
    var stripe = null;
    var xhr = new XMLHttpRequest();
    xhr.open("GET", "https://ubsidi.com/V2/public/api/ubsidi/get-key", true);
    xhr.setRequestHeader("Content-type", "application/json; charset=UTF-8");
    xhr.setRequestHeader("api-key", this.ubsidiKey);
    xhr.send();
    xhr.onload = function () {
      var response = JSON.parse(xhr.response);
      if (response.status) {
        stripe = Stripe(response.data.key, { apiVersion: "2023-10-16" });
        const token = stripe.tokens.create({
          card: {
            number: cardNumber,
            exp_month: expiryMonth,
            exp_year: expiryYear,
            cvc: cvv,
          },
        });
        console.log(token);
      } else {
        alert(response.message);
        throw new Error(response.message);
      }
    };
  }

  createPaymentSchema(amount, cardNumber, expiryMonth, expiryYear, cvv) {
    let data = {
      amount: amount,
      card_number: cardNumber,
      expiry_month: expiryMonth,
      expiry_year: expiryYear,
      cvv: cvv,
    };
    let post = JSON.stringify(data);
    const url = "";
    let xhr = new XMLHttpRequest();
    xhr.open("POST", url, true);
    xhr.setRequestHeader("Content-type", "application/json; charset=UTF-8");
    xhr.setRequestHeader("api-key", this.ubsidiKey);
    xhr.send(post);
    xhr.onload = function () {
      var response = JSON.parse(xhr.response);
      if (response.status) {
        console.log(response.data);
        return response.data;
      } else {
        console.error(response.message);
        return null;
      }
    };
  }

  confirmPaymentSchema(schemaSecret) {
    var stripe = null;
    var xhr = new XMLHttpRequest();
    xhr.open("GET", "https://ubsidi.com/V2/public/api/ubsidi/get-key", true);
    xhr.setRequestHeader("Content-type", "application/json; charset=UTF-8");
    xhr.setRequestHeader("api-key", this.ubsidiKey);
    xhr.send();
    xhr.onload = function () {
      var response = JSON.parse(xhr.response);
      if (response.status) {
        stripe = Stripe(response.data.key);
      } else {
        alert(response.message);
        throw new Error(response.message);
      }
    };

    if (stripe != null) {
      stripe.confirmCardPayment(schemaSecret).then((result) => {
        if (result.error) {
          alert(result.error.message);
        } else {
          let captureData = {
            payment_intent_id: result.paymentIntent.id,
          };
          let post = JSON.stringify(captureData);
          const url = "https://ubsidi.com/V2/public/api/ubsidi/capture-payment";
          let xhr = new XMLHttpRequest();
          xhr.open("POST", url, true);
          xhr.setRequestHeader(
            "Content-type",
            "application/json; charset=UTF-8"
          );
          xhr.setRequestHeader("api-key", ubsidiKey);
          xhr.send(post);
          xhr.onload = function () {
            var response = JSON.parse(xhr.response);
            console.log(response);
            if (response.status) {
              console.info(response.message);
            } else {
              alert(response.message);
            }
          };
        }
      });
    } else {
      console.error("Something went wrong.");
    }
  }
}
