<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta http-equiv="X-UA-Compatible" content="ie=edge">
    <title>Stripe Payment</title>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/css/bootstrap.min.css"
        integrity="sha384-T3c6CoIi6uLrA9TneNEoa7RxnatzjcDSCmG1MXxSR1GAsXEV/Dwwykc2MPK8M2HN" crossorigin="anonymous">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.5.1/css/all.min.css"
        integrity="sha512-DTOQO9RWCH3ppGqcWaEA1BIZOC6xxalwEsw9c2QQeAIftl+Vegovlnee1c9QX4TctnWMn13TZye+giMm8e2LwA=="
        crossorigin="anonymous" referrerpolicy="no-referrer" />
</head>

<body>
    <div class="card w-75 mt-5 m-auto">
        <div class="card-body">
            @if (request()->query('payment_success'))
                <div class="alert alert-success alert-dismissible fade show" role="alert">
                    <strong>Success!</strong> {{ request()->query('payment_success') }}
                    <button type="button" class="btn-close" data-bs-dismiss="alert" aria-label="Close"></button>
                </div>
            @elseif (request()->query('payment_failed'))
                <div class="alert alert-danger alert-dismissible fade show" role="alert">
                    <strong>Error!</strong> {{ request()->query('payment_failed') }}
                    <button type="button" class="btn-close" data-bs-dismiss="alert" aria-label="Close"></button>
                </div>
            @endif
            <h5 class="card-title mb-3">Card Payment</h5>
            <form id="payment_form">
                <div class="form-group">
                    <label for="amount">Amount</label>
                </div>
                <div class="input-group mb-3">
                    <div class="input-group-prepend">
                        <span class="input-group-text" id="basic-addon1">€</span>
                    </div>
                    <input type="text" class="form-control" id="amount" name="amount" value="{{ request()->query('payment_success') || request()->query('payment_failed') ? 0 : 10 }}" disabled>
                </div>
                <div class="form-group mb-3">
                    <label class="mb-2">Card Details</label>
                    <div id="card-element"></div>
                </div>
                <button type="submit" id="submit_button" class="btn btn-primary">Submit</button>
            </form>
        </div>
    </div>
    <script src="https://code.jquery.com/jquery-3.7.1.min.js"
        integrity="sha256-/JqT3SQfawRcv/BIHPThkBvs0OEvtFFmqPF/lYI/Cxo=" crossorigin="anonymous"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/js/bootstrap.bundle.min.js"
        integrity="sha384-C6RzsynM9kWDrMNeT87bh95OGNyZPhcTNXj1NW7RuBCsyN/o0jlpcV8Qyq46cDfL" crossorigin="anonymous">
    </script>
    <script src="https://js.stripe.com/v3/"></script>
    <script>
        var stripe = Stripe("{{ $key }}");
        var elements = stripe.elements();
        var cardElement = elements.create('card', {
            hidePostalCode: true
        });
        cardElement.mount('#card-element');

        function toggleButton() {
            var button = $("#submit_button");
            var html = button.html();
            if (html == 'Submit') {
                button.prop('disabled', true);
                button.html('<i class="fa-solid fa-spinner fa-spin-pulse"></i> Loading....');
            } else if (html == '<i class="fa-solid fa-spinner fa-spin-pulse"></i> Loading....') {
                button.prop('disabled', false);
                button.html('Submit');
            }
        }

        $('#payment_form').on('submit', function(e) {
            e.preventDefault();

            stripe
                .createPaymentMethod({
                    type: 'card',
                    card: cardElement
                })
                .then(function(result) {
                    if (result.error) {
                        alert(result.error.message);
                    } else {
                        $.ajax({
                            url: "{{ env('APP_ENV') == 'production' ? str_replace('http://', 'https://', url('/create-payment-intent')) : url('/create-payment-intent') }}",
                            type: 'POST',
                            data: {
                                amount: $('#amount').val(),
                                paymentIntent: result.paymentMethod.id
                            },
                            beforeSend: function() {
                                toggleButton();
                            },
                            complete: function() {
                                toggleButton();
                            },
                            success: function(response) {
                                if (response.status) {
                                    stripe.confirmCardPayment(response.data.paymentIntentSecret)
                                        .then((paymentResult) => {
                                            if (paymentResult.error) {
                                                window.location.href = window.location.href
                                                    .split('?')[0] +
                                                    '?payment_failed=' + paymentResult.error
                                                    .message;
                                            }
                                            if (paymentResult.paymentIntent.status ==
                                                'succeeded') {
                                                window.location.href = window.location.href
                                                    .split('?')[0] +
                                                    '?payment_success=Payment completed successfully.';
                                            }
                                        });
                                } else {
                                    alert(response.message);
                                }
                            },
                            error: function(response) {}
                        });
                    }
                });
        });
    </script>
</body>

</html>
