<?php

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
*/

use App\Http\Controllers\PayoutReportController;
use App\Jobs\PrintJob;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Queue;

$router->group(['prefix' => 'test/'], function () use ($router) {
    $router->post('create', ['uses' => 'TestConnectAccountController@create']);
    $router->get('create-person', ['uses' => 'TestConnectAccountController@create_person']);
    $router->get('update-person', ['uses' => 'TestConnectAccountController@update_person']);
    $router->get('persons', ['uses' => 'TestConnectAccountController@persons']);
    $router->get('upload_file', ['uses' => 'TestConnectAccountController@upload_file']);
    $router->get('validate_file', ['uses' => 'TestConnectAccountController@validate_file']);
    $router->get('external-account', ['uses' => 'TestConnectAccountController@external_account']);
    $router->get('external-accounts', ['uses' => 'TestConnectAccountController@external_accounts']);
    $router->get('delete/{id}', ['uses' => 'TestConnectAccountController@delete']);
    $router->get('enable/{id}', ['uses' => 'TestConnectAccountController@enable_payment']);
    $router->get('{id}', ['uses' => 'TestConnectAccountController@show']);
    $router->post('{account_id}', ['uses' => 'TestConnectAccountController@update']);
});

$router->group(['prefix' => 'connect-accounts'], function () use ($router) {
    $router->get('', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@index']);
    $router->post('', ['middleware' => 'role:admin,user,guest', 'uses' => 'ConnectAccountController@create']);

    $router->get('debit-recurring-fees', ['uses' => 'ConnectAccountController@debit_recurring_fees']);
    $router->get('business/{business_id}', ['middleware' => 'role:admin,epos_admin,user', 'uses' => 'ConnectAccountController@show_by_business']);
    $router->post('business/{business_id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@add_connect_id']);

    $router->post('{id}/generate', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@generate']);
    $router->post('{account_id}/bank-accounts', ['middleware' => 'role:admin,epos_admin,user', 'uses' => 'ConnectAccountController@create_bank_account']);
    $router->post('{account_id}/bank-accounts/{id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@update_bank_account']);
    $router->post('{account_id}/payout-settings', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@payout_settings']);
    $router->post('{account_id}/payout', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@payout']);
    $router->post('{account_id}/instant-payout', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@instant_payout']);
    $router->post('{account_id}/debit-single', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@debit_single_amount']);

    $router->get('{id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@show']);
    $router->post('{id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@update']);
    $router->post('{id}/enable-external-customer', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@enable_external_customer']);
    $router->post('{id}/disable-external-customer', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@disable_external_customer']);
    $router->delete('{id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@delete']);
    $router->delete('{id}/external-accounts', ['middleware' => 'role:admin,user', 'uses' => 'ConnectAccountController@delete_external']);
});
$router->group(['prefix' => 'companies'], function () use ($router) {
    $router->get('', ['middleware' => 'role:admin,user', 'uses' => 'CompanyController@index']);
    $router->post('', ['middleware' => 'role:admin,user', 'uses' => 'CompanyController@create']);
    $router->get('{id}', ['middleware' => 'role:admin,user', 'uses' => 'CompanyController@show']);
    $router->post('{id}', ['middleware' => 'role:admin,user', 'uses' => 'CompanyController@update']);
    $router->delete('{id}', ['middleware' => 'role:admin,user', 'uses' => 'CompanyController@delete']);
});
$router->group(['prefix' => 'persons'], function () use ($router) {
    $router->get('', ['middleware' => 'role:admin,user', 'uses' => 'PersonController@index']);
    $router->post('', ['middleware' => 'role:admin,user', 'uses' => 'PersonController@create']);
    $router->get('{id}', ['middleware' => 'role:admin,user', 'uses' => 'PersonController@show']);
    $router->post('{id}', ['middleware' => 'role:admin,user', 'uses' => 'PersonController@update']);
    $router->delete('{id}', ['middleware' => 'role:admin,user', 'uses' => 'PersonController@delete']);
});

$router->group(['prefix' => 'files'], function () use ($router) {
    $router->get('', ['middleware' => 'role:admin,user', 'uses' => 'ConnectFileController@index']);
    $router->post('', ['middleware' => 'role:admin,user', 'uses' => 'ConnectFileController@create']);
    $router->get('{id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectFileController@show']);
    $router->post('{id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectFileController@update']);
    $router->delete('{id}', ['middleware' => 'role:admin,user', 'uses' => 'ConnectFileController@delete']);
});
