<?php

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
*/

use App\Http\Controllers\PayoutReportController;
use App\Jobs\PrintJob;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Queue;
use Stripe\Stripe;
use Stripe\StripeClient;

$router->group(['prefix' => '/'], function () use ($router) {
    $router->get('verify', ['uses' => 'EposUserController@verify']);
    $router->get('test', ['uses' => 'EposArtisanController@test']);
    $router->get('schema', ['uses' => 'EposArtisanController@schema']);
    $router->get('load-data', ['uses' => 'EposArtisanController@load_data']);
    $router->get('refresh_schema', ['uses' => 'EposArtisanController@refresh_schema']);
    $router->delete('factory-reset', ['uses' => 'EposArtisanController@factory_reset']);
    //@TODO delete after live
    $router->get('statement', ['uses' => 'EposArtisanController@epos_statement']);
    $router->get('statement/{id}/download', ['uses' => 'EposArtisanController@payout_invoice_download']);
    //
    $router->post('create-reader', ['uses' => 'EposArtisanController@readerCreate']);
    $router->post('create-payment', ['uses' => 'EposArtisanController@paymentIntentCreate']);
});
$router->group(['prefix' => 'dashboard'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDashboardController@index']);
    $router->get('top-products', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDashboardController@top_product']);
});

$router->group(['prefix' => 'database'], function () use ($router) {
    $router->post('push', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDatabaseController@push']);
    $router->post('pull', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDatabaseController@pull']);
});

$router->group(['prefix' => '/'], function () use ($router) {
    $router->post('login', ['uses' => 'EposLoginController@login']);

    $router->get('me', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposLoginController@show']);
    $router->post('me/password', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposLoginController@password']);

    $router->get('businesses', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposLoginController@businesses_index']);
    $router->post('businesses', ['middleware' => 'role:epos_admin,guest', 'uses' => 'EposLoginController@business_create']);
    $router->get('businesses/{business_id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposLoginController@businesses_show']);
    $router->post('businesses/{business_id}/select', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposLoginController@businesses_select']);
});

$router->group(['prefix' => '/users'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@index']);
    $router->get('mini', ['uses' => 'EposUserController@mini']);
    $router->get('{id}/logout', ['uses' => 'EposUserController@logout']);

    $router->post('login', ['uses' => 'EposUserController@login']);

    $router->get('me', ['middleware' => 'role:epos_user', 'uses' => 'EposUserController@show']);
    $router->post('me/password', ['middleware' => 'role:epos_user', 'uses' => 'EposUserController@password']);

    $router->post('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@create']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@update']);
    $router->post('{id}/change-password', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@chage_password']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@disable']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserController@delete']);
});
$router->group(['prefix' => 'devices'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDeviceController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeviceController@create']);

    $router->get('validate/{device_id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeviceController@validate_device']);
    $router->post('validate/{device_id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeviceController@validate_device_token']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeviceController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDeviceController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDeviceController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDeviceController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDeviceController@disable']);
    $router->post('{id}/enable-weborder', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDeviceController@enable_weborder']);
    $router->post('{id}/enable-callerid', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposDeviceController@enable_callerid_admin']);
});
$router->group(['prefix' => 'payout-requests'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposPayoutRequestController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPayoutRequestController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPayoutRequestController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposPayoutRequestController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposPayoutRequestController@delete']);
});
$router->group(['prefix' => 'user-groups'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserGroupController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserGroupController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserGroupController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserGroupController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserGroupController@delete']);
});
$router->group(['prefix' => 'user-types'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserTypeController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserTypeController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserTypeController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserTypeController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposUserTypeController@delete']);
});

$router->group(['prefix' => 'floors'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposFloorController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposFloorController@create']);

    $router->post('floor-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposFloorController@floor_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposFloorController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposFloorController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposFloorController@delete']);
});
$router->group(['prefix' => 'tables'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposTableController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableController@create']);

    $router->post('table-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableController@table_bulk']);
    $router->post('status-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableController@status_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposTableController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableController@disable']);
});
$router->group(['prefix' => 'table-statuses'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposTableStatusController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableStatusController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableStatusController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableStatusController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposTableStatusController@delete']);
});
$router->group(['prefix' => 'categories'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposCategoryController@index']);
    $router->get('mini', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@mini']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@create']);

    $router->post('category-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@category_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCategoryController@disable']);
});
$router->group(['prefix' => 'printers'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterController@delete']);
});
$router->group(['prefix' => 'printer-models'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterModelController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterModelController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterModelController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterModelController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrinterModelController@delete']);
});
$router->group(['prefix' => 'preparation-locations'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrepLocationController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrepLocationController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrepLocationController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrepLocationController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrepLocationController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrepLocationController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrepLocationController@disable']);
});
$router->group(['prefix' => 'products'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposProductController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductController@create']);

    $router->post('product-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductController@product_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductController@disable']);
});
$router->group(['prefix' => 'addons'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@create']);

    $router->post('addon-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@addon_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposAddonController@disable']);
});
$router->group(['prefix' => 'ingredients'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@create']);

    $router->post('ingredient-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@ingredient_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposIngredientController@disable']);
});
$router->group(['prefix' => 'product-addons'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposProductAddonController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@delete']);

    $router->get('{id}/subaddons', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@subaddons']);
    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@disable']);
    $router->post('{id}/contain-enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@containsEnable']);
    $router->post('{id}/contain-disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductAddonController@containDisable']);
});
$router->group(['prefix' => 'product-ingredients'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposProductIngredientController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@create']);
    $router->post('multiple', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@create_multiple']);
    $router->post('delete-bulk', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposProductIngredientController@deleteBulk']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@disable']);
    $router->post('{id}/contain-enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@containEnable']);
    $router->post('{id}/contain-disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposProductIngredientController@containDisable']);
});
$router->group(['prefix' => 'customers'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCustomerController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCustomerController@create']);

    $router->post('delete-bulk', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposCustomerController@deleteBulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCustomerController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCustomerController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCustomerController@delete']);
});


$router->group(['prefix' => 'order-types'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderTypeController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderTypeController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderTypeController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderTypeController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderTypeController@delete']);
});
$router->group(['prefix' => 'order-actions'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderActionController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderActionController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderActionController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderActionController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderActionController@delete']);
});
$router->group(['prefix' => 'order-statuses'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderStatusController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderStatusController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderStatusController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderStatusController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderStatusController@delete']);
});


$router->group(['prefix' => 'deposit-types'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDepositTypeController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDepositTypeController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDepositTypeController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDepositTypeController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDepositTypeController@delete']);
});

$router->group(['prefix' => 'reservation-confirmations'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationConfirmationController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationConfirmationController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationConfirmationController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationConfirmationController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationConfirmationController@delete']);
});
$router->group(['prefix' => 'reservation-statuses'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationStatusController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationStatusController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationStatusController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationStatusController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationStatusController@delete']);
});
$router->group(['prefix' => 'reservation-types'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationTypeController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationTypeController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationTypeController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationTypeController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationTypeController@delete']);
});
$router->group(['prefix' => 'discounts'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposDiscountController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDiscountController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDiscountController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDiscountController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDiscountController@delete']);
});
$router->group(['prefix' => 'delivery-charges'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@create']);

    $router->post('calculate', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@delivery_charge_calculation']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposDeliveryChargeController@disable']);
});
$router->group(['prefix' => 'orders'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user', 'uses' => 'EposOrderController@index']);
    $router->get('latest', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@index_latest']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposOrderController@create']);
    $router->delete('delete-all', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@delete_all']);

    $router->post('bulk-archive', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@bulk_archive']);
    $router->post('bulk-unarchive', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@bulk_unarchive']);

    $router->post('{id}/swap-table', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@swap_table']);
    $router->post('{id}/ready-to-collect', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@ready_to_collect']);
    $router->post('{id}/on-the-way', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@on_the_way']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposOrderController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposOrderController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderController@delete']);
});
$router->group(['prefix' => 'order-items'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@delete']);
});
$router->group(['prefix' => 'reservations'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationController@create']);
    $router->delete('delete-all', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationController@delete_all']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReservationController@delete']);
});
$router->group(['prefix' => 'site-settings'], function () use ($router) {
    $router->get('', ['uses' => 'EposSettingController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSettingController@create']);

    $router->post('setting-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSettingController@setting_bulk']);

    $router->post('load-settings', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSettingController@load_settings']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSettingController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSettingController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSettingController@delete']);
});
$router->group(['prefix' => 'vouchers'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user', 'uses' => 'EposVoucherController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposVoucherController@create']);

    $router->post('find/{code}', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user', 'uses' => 'EposVoucherController@find']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposVoucherController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposVoucherController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposVoucherController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposVoucherController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposVoucherController@disable']);
});
$router->group(['prefix' => 'payment-methods'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user', 'uses' => 'EposPaymentMethodController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPaymentMethodController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPaymentMethodController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPaymentMethodController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPaymentMethodController@delete']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPaymentMethodController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPaymentMethodController@disable']);
});
$router->group(['prefix' => 'loyalty-points'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposLoyaltyPointController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposLoyaltyPointController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposLoyaltyPointController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposLoyaltyPointController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposLoyaltyPointController@delete']);
});
$router->group(['prefix' => 'order-payments'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user', 'uses' => 'EposOrderPaymentController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user,guest', 'uses' => 'EposOrderPaymentController@create']);

    $router->post('payment-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderPaymentController@payment_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderPaymentController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderPaymentController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderPaymentController@delete']);
});
$router->group(['prefix' => 'order-splits'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderSplitController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderSplitController@create']);

    $router->post('split-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderSplitController@split_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderSplitController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderSplitController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderSplitController@delete']);
});
$router->group(['prefix' => 'reports'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReportController@index']);
    $router->get('cancel-orders', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReportController@cancelList']);
    $router->get('products-wise', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReportController@productWise']);
    $router->get('online-order', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReportController@onlineOrder']);
    $router->get('products-category-wise', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposReportController@productCategoryWise']);
});
$router->group(['prefix' => 'call-logs'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCallLogController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCallLogController@create']);
    $router->delete('delete-all', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCallLogController@delete_all']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCallLogController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCallLogController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCallLogController@delete']);
});
$router->group(['prefix' => 'cashdraw-logs'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCashdrawLogController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCashdrawLogController@create']);
    $router->delete('delete-all', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCashdrawLogController@delete_all']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCashdrawLogController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCashdrawLogController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposCashdrawLogController@delete']);
});
$router->group(['prefix' => 'print-blocks'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockController@create']);

    $router->post('print-block-bulk', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockController@print_block_bulk']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockController@delete']);
});
$router->group(['prefix' => 'print-block-assigns'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockAssignController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockAssignController@create']);
    $router->post('multiple', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockAssignController@create_multiple']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockAssignController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockAssignController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintBlockAssignController@delete']);
});
$router->group(['prefix' => 'online-orders'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOnlineOrderController@index']);
    $router->get('reservation', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOnlineOrderController@reservation']);
    $router->get('fetch', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOnlineOrderController@fetch']);
    $router->get('history', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOnlineOrderController@history']);
    $router->post('archive', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOnlineOrderController@archive']);
    $router->post('unarchive', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOnlineOrderController@unarchive']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOnlineOrderController@delete']);
});

$router->group(['prefix' => 'print-settings'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintSettingController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposPrintSettingController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposPrintSettingController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposPrintSettingController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposPrintSettingController@delete']);
});

$router->group(['prefix' => 'sms-packages'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSmsPackageController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposSmsPackageController@create']);
    $router->post('send-message', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposSmsPackageController@send_message']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposSmsPackageController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposSmsPackageController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposSmsPackageController@delete']);
});

$router->group(['prefix' => 'order-logs'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user,tiffintom_user', 'uses' => 'EposOrderLogController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderLogController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderLogController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderLogController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderLogController@delete']);
});

$router->group(['prefix' => 'external-payments'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposExternalPaymentController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposExternalPaymentController@create']);
    $router->delete('delete-all', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposExternalPaymentController@delete_all']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposExternalPaymentController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposExternalPaymentController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposExternalPaymentController@delete']);
});

$router->group(['prefix' => 'deleted-items'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,epos_user', 'uses' => 'EposOrderItemController@delete']);
});

$router->get('{id}', ['middleware' => 'role:epos_admin,epos_user,user', 'uses' => 'EposLoginController@show']);