<?php

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
*/

use App\Http\Controllers\PayoutReportController;
use App\Jobs\PrintJob;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Queue;

$router->group(['prefix' => '/'], function () use ($router) {
    $router->post('login', ['uses' => 'AppMerchantController@login']);

    $router->get('businesses', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_index']);
    $router->get('businesses/{business_id}', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@businesses_show']);
    $router->get('businesses/{business_id}/dashboard', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_dashboard']);
    $router->get('businesses/{business_id}/all-transactions', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_all_transactions']);
    $router->post('businesses/{business_id}/all-transactions/archive', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_archive_all_transactions']);
    $router->post('businesses/{business_id}/all-transactions/unarchive', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_unarchive_all_transactions']);
    $router->get('businesses/{business_id}/card-transactions', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_stripe_transactions']);
    $router->get('businesses/{business_id}/report-stripe-transactions', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@businesses_report_stripe_transactions']);
    $router->get('businesses/{business_id}/dispute-all-transactions', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@businesses_dispute_all_transactions']);
    $router->get('businesses/{business_id}/card-transactions/archive', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_archived_transactions']);
    $router->get('businesses/{business_id}/card-transactions/unarchive', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_unarchived_transactions']);
    $router->post('businesses/{business_id}/card-transactions/archive', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_archive_stripe_transactions']);
    $router->post('businesses/{business_id}/card-transactions/unarchive', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_unarchive_stripe_transactions']);
    $router->post('businesses/{business_id}/card-transactions/{id}/refund', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@refund']);
    $router->post('businesses/{business_id}/card-transactions/{id}/send-receipt', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@send_receipt']);
    $router->post('businesses/{business_id}/paypal-transactions/{id}/refund', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@refund_paypal']);
    $router->post('businesses/{business_id}/card-transactions/{id}/dispute-accept', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@dispute_accept']);
    $router->post('businesses/{business_id}/card-transactions/{id}/dispute-counter', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@dispute_counter']);
    $router->post('businesses/{business_id}/report-summary', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@report_summary']);
    $router->post('businesses/{business_id}/earning-summary', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@earning_summary']);
    $router->get('businesses/{business_id}/statements', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@statements']);
    $router->get('businesses/{business_id}/statements/{statement_id}/download', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@statement_download']);
    $router->post('businesses/{business_id}/device-registrations', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@device_registrations']);
    $router->get('businesses/{business_id}/device-registrations/validate/{device_id}', ['middleware' => 'role:user,epos_admin,epos_user,guest', 'uses' => 'AppMerchantController@device_validate']);
});
