<?php

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
*/

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Queue;
use App\Helpers\Utils;

// Category
$router->group(['prefix' => 'categories'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomCategoryController@index']);
});

// Site Setting 
$router->group(['prefix' => 'site-settings'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomSiteSettingController@index']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomSiteSettingController@show']);
    $router->get('{id}/all', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomSiteSettingController@show_all']);

    $router->post('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomSiteSettingController@update']);
});

// Restaurant
$router->group(['prefix' => 'restaurants'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@index']);
    $router->get('time-slot', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@restaurant_timeslot']);
    $router->get('booking-time-slot', ['uses' => 'Tiffintom\TiffintomRestaurantController@restaurant_booking_timeslot']);
    $router->get('fetch', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@fetch']);
    $router->get('offer-all', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@restaurant_offer']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@show']);
    $router->get('{id}/all', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@show_all']);
    $router->get('{id}/counts', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@restaurant_pending_count']);
    $router->post('search', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@search']);
    $router->post('nearby', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomRestaurantController@nearby']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomRestaurantController@update']);
});

// Menu
$router->group(['prefix' => 'menu'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@create']);

    $router->get('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomMenuController@show']);
    $router->post('bulk', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@bulk_menu']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@delete']);
    $router->delete('{id}/variant', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@delete_variant']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomMenuController@disable']);
});

// Order
$router->group(['prefix' => 'orders'], function () use ($router) {
    $router->get('all-orders', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@all_orders']);
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@index']);
    $router->get('re-order/{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@reOrder']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@show']);
    $router->get('{id}/time-slots', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@order_timeslot']);

    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@create']);
    $router->post('bulk-delete', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@bulk_delete']);
    $router->post('{id}/change-status', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@changeStatus']);
    $router->post('offer-check', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@offer_check']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomOrderController@delete']);
});
// Order

//users
$router->group(['prefix' => 'users'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@index']);
    $router->get('send-referral', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@send_referral']);
    $router->get('send-referral-email', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@send_referral_email']);
    $router->get('verify-email', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@verify_email']);
    $router->get('validate-email', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@validate_email']);
    $router->get('send-otp', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@send_otp']);
    $router->post('login', ['uses' => 'Tiffintom\TiffintomUserController@login']);
    $router->post('signup', ['uses' => 'Tiffintom\TiffintomUserController@signup']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@show']);
    $router->post('forgot-password', ['uses' => 'Tiffintom\TiffintomUserController@forgot_password']);
    $router->post('forgot-password-direct', ['uses' => 'Tiffintom\TiffintomUserController@forgot_password_direct']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@update']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@delete']);
    $router->post('{id}/verify', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@verify_status']);
    $router->post('{id}/send-otp-both', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@send_otp_both']);
    $router->post('{id}/remove-image', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@remove_image']);
    $router->post('{id}/change-password', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomUserController@chage_password']);
    $router->post('{id}/change-user-password', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomUserController@chage_user_password']);
    $router->post('{id}/verify-otp', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@verify_otp']);
    $router->post('{id}/verify-otp-both', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomUserController@verify_otp_both']);
    $router->post('{id}/email-verify-otp', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@email_verify_otp']);

    $router->post('{id}/enable', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@enable']);
    $router->post('{id}/disable', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomUserController@disable']);
});

// Wallet History
$router->group(['prefix' => 'wallet-history'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomWalletHistoryController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomWalletHistoryController@create']);
});

// Review
$router->group(['prefix' => 'reviews'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomReviewController@index']);
});

// Referral
$router->group(['prefix' => 'referrals'], function () use ($router) {
    $router->get('', ['uses' => 'Tiffintom\TiffintomReferralController@index']);
    $router->get('referral-pending', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomReferralController@refferal_pending']);
    $router->get('referral-history', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomReferralController@refferal_pending_index']);
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomReferralController@create']);

    $router->get('{id}', ['uses' => 'Tiffintom\TiffintomReferralController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomReferralController@update']);
    $router->post('{id}/referral', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomReferralController@update_referral']);
    $router->delete('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomReferralController@delete']);
});

// Address-Book
$router->group(['prefix' => 'address-book'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomAddressBookController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomAddressBookController@create']);
});

// Card // Stripe Customers
$router->group(['prefix' => 'cards'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomStripeCustomerController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomStripeCustomerController@create']);
});

// Wallet History
$router->group(['prefix' => 'wallet-history'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomWalletHistoryController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomWalletHistoryController@create']);
});

// Account Help 
$router->group(['prefix' => 'account-help'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user,guest', 'uses' => 'Tiffintom\TiffintomAccountHelpController@index']);
});

// Chat
$router->group(['prefix' => 'chat'], function () use ($router) {
    $router->post('chat-general-question', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomChatController@order_general_chat']);
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomChatController@index']);
});

// App-Feedback
$router->group(['prefix' => 'app-feedback'], function () use ($router) {
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomAppFeedbackController@create']);
});
// Cart
$router->group(['prefix' => 'cart'], function () use ($router) {
    $router->post('cart-bulk', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomCartController@cart_bulk']);
});
// Checkout List 
$router->group(['prefix' => 'checkout-list'], function () use ($router) {
    $router->get('', ['uses' => 'Tiffintom\TiffintomCheckoutListController@index']);
});

// Rewards
$router->group(['prefix' => 'rewards'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomRewardsController@index']);
});

// Vouchers
$router->group(['prefix' => 'vouchers'], function () use ($router) {
    $router->post('find/{code}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomVoucherController@find']);
});

// Restaurant Booking
$router->group(['prefix' => 'bookings'], function () use ($router) {
    $router->get('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomBookingController@index']);
    $router->post('', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomBookingController@create']);
    $router->get('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomBookingController@show']);
    $router->post('{id}', ['middleware' => 'role:epos_admin,tiffintom_user,epos_user', 'uses' => 'Tiffintom\TiffintomBookingController@update']);
});

// Merchant 
$router->group(['prefix' => 'merchant'], function () use ($router) {
    $router->post('login', ['uses' => 'AppMerchantController@login']);
    $router->get('businesses', ['middleware' => 'role:user,epos_admin,epos_user', 'uses' => 'AppMerchantController@businesses_index']);
});