<?php

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
*/

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Queue;

$router->group(['prefix' => '/'], function () use ($router) {
    $router->post('login', ['uses' => 'App\Http\Controllers\AppMerchantController@login']);
    $router->get('businesses', ['middleware' => 'role:user,epos_admin,epos_user,tiffintom_user', 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_index']);
    $router->get('businesses/{business_id}', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_show']);
    $router->get('businesses/{business_id}/dashboard', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_dashboard']);
    $router->get('businesses/{business_id}/all-transactions', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_all_transactions']);
    $router->post('businesses/{business_id}/all-transactions/archive', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_archive_all_transactions']);
    $router->post('businesses/{business_id}/all-transactions/unarchive', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_unarchive_all_transactions']);
    $router->get('businesses/{business_id}/card-transactions', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_stripe_transactions']);
    $router->get('businesses/{business_id}/report-stripe-transactions', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_report_stripe_transactions']);
    $router->get('businesses/{business_id}/dispute-all-transactions', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_dispute_all_transactions']);
    $router->get('businesses/{business_id}/card-transactions/archive', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_archived_transactions']);
    $router->get('businesses/{business_id}/card-transactions/unarchive', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_unarchived_transactions']);
    $router->post('businesses/{business_id}/card-transactions/archive', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_archive_stripe_transactions']);
    $router->post('businesses/{business_id}/card-transactions/unarchive', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@businesses_unarchive_stripe_transactions']);
    $router->post('businesses/{business_id}/card-transactions/{id}/refund', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@refund']);
    $router->post('businesses/{business_id}/card-transactions/{id}/send-receipt', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@send_receipt']);
    $router->post('businesses/{business_id}/paypal-transactions/{id}/refund', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@refund_paypal']);
    $router->post('businesses/{business_id}/card-transactions/{id}/dispute-accept', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@dispute_accept']);
    $router->post('businesses/{business_id}/card-transactions/{id}/dispute-counter', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@dispute_counter']);
    $router->post('businesses/{business_id}/report-summary', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@report_summary']);
    $router->post('businesses/{business_id}/earning-summary', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@earning_summary']);
    $router->get('businesses/{business_id}/statements', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@statements']);
    $router->get('businesses/{business_id}/statements/{statement_id}/download', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@statement_download']);
    $router->post('businesses/{business_id}/device-registrations', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@device_registrations']);
    $router->get('businesses/{business_id}/device-registrations/validate/{device_id}', ['middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider'], 'uses' => 'App\Http\Controllers\AppMerchantController@device_validate']);
});

// Partner Dashboard
$router->group(['prefix' => 'dashboard', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('overview', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomDashboardController@dashboard_overview']);
    $router->get('payment-overview', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomDashboardController@payment_overview']);
});

// Restaurant
$router->group(['prefix' => 'restaurants', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@index']);
    $router->get('mini', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@mini']);
    $router->get('time-slot', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@restaurant_timeslot']);
    $router->get('booking-time-slot', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@restaurant_booking_timeslot']);
    $router->get('fetch', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@fetch']);
    $router->get('offer-all', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@restaurant_offer']);
    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@show']);
    $router->get('{id}/counts', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@restaurant_pending_count']);
    $router->post('search', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@search']);
    $router->post('nearby', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@nearby']);
    $router->post('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@update']);

    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@delete']);

    $router->post('{id}/enable', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@enable']);
    $router->post('{id}/disable', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantController@disable']);
});

// Order
$router->group(['prefix' => 'orders', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {

    $router->get('home', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@home']);
    $router->get('all-orders', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@all_orders']);
    $router->get('history', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@history']);

    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@index']);
    $router->get('re-order/{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@reOrder']);
    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@show']);
    $router->get('{id}/time-slots', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@order_timeslot']);

    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@create']);
    $router->post('bulk-delete', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@bulk_delete']);
    $router->post('{id}/change-status', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@changeStatus']);
    $router->post('{id}/time-extend', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@time_extend']);
    $router->post('{id}/rtc', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@order_rtc']);
    $router->post('offer-check', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@offer_check']);
    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderController@delete']);
});

// Booking
$router->group(['prefix' => 'bookings', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@index']);
    $router->get('home', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@home']);
    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@create']);

    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@show']);
    $router->post('bulk-delete', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@bulk_delete']);
    $router->post('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@update']);
    $router->post('{id}/change-status', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@change_status']);
    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomBookingController@delete']);
});

// Card // Stripe Customers
$router->group(['prefix' => 'cards', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomStripeCustomerController@index']);
    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomStripeCustomerController@create']);

    $router->post('payment-intent', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomStripeCustomerController@payment_intent']);
    $router->post('payment-intent-without-customer', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomStripeCustomerController@payment_intent_wihout_customer']);

    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomStripeCustomerController@show']);
    $router->post('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomStripeCustomerController@update']);
    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomStripeCustomerController@delete']);
});

// Order Help
$router->group(['prefix' => 'order-help', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderHelpController@index']);
    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderHelpController@create']);

    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderHelpController@show']);
    $router->post('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderHelpController@update']);
    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderHelpController@delete']);
    $router->delete('{id}/delete-user', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOrderHelpController@delete_user']);
});

// Offer 
$router->group(['prefix' => 'offers', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@index']);
    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@create']);

    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@show']);
    $router->post('bulk', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@bulk_offer']);
    $router->post('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@update']);
    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@delete']);

    $router->post('{id}/enable', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@enable']);
    $router->post('{id}/disable', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomOfferController@disable']);
});

// Restaurant Voucher
$router->group(['prefix' => 'restaurant-vouchers', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@index']);
    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@create']);

    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@show']);
    $router->post('bulk', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@bulk_restaurant_voucher']);
    $router->post('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@update']);
    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@delete']);

    $router->post('{id}/enable', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@enable']);
    $router->post('{id}/disable', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomRestaurantVoucherController@disable']);
});

// Epos Shortln Url Message
$router->group(['prefix' => 'payment-message', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController@index']);
    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController@create']);
    $router->post('payment-intent', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController@payment_intent']);
    $router->get('find/{code}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController@find']);
    $router->post('{id}/resend', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController@resend']);
    $router->post('{id}/change-status', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController@change_status']);
    $router->post('{id}/cancel-payment', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomShortlnUrlController@change_cancel']);
});

// Review
$router->group(['prefix' => 'reviews', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomReviewController@index']);
    $router->get('home', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomReviewController@home']);
    $router->post('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomReviewController@create']);

    $router->get('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomReviewController@show']);
    $router->post('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomReviewController@update']);
    $router->delete('{id}', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomReviewController@delete']);
});

// Payment Transaction
$router->group(['prefix' => 'payment-transactions', 'middleware' => ['role:user,epos_admin,epos_user,tiffintom_user', 'outsider']], function () use ($router) {
    $router->get('', ['uses' => 'App\Http\Controllers\Tiffintom\TiffintomPaymentTransactionController@index']);
});